#!/bin/csh

echo REMARK Extracting arguments ...

set parentName  = test.ft2
set hi          = 1000.0
set noise       = 1.0e+4
set xDelay      = 0.0;
set yDelay      = 0.0;
set xP0         = -90;
set xP1         = 0;
set yP0         = -90;
set yP1         = 0;
set sigCount    = 0 
set pCount      = 0 
set maxSig      = 5
set maxPassive  = 5
set itMax       = 75
set id          = 1
set diagFlag    = 0
set xActive     = 1
set yActive     = 1

set parentName  = (`getArgD $* -parent     $parentName`)
set hi          = (`getArgD $* -hi         $hi`) 
set noise       = (`getArgD $* -noise      $noise`)
set sigCount    = (`getArgD $* -sigCount   $sigCount`)
set pCount      = (`getArgD $* -pCount     $pCount`)
set maxSig      = (`getArgD $* -maxSig     $maxSig`)
set maxPassive  = (`getArgD $* -maxPassive $maxPassive`)
set xDelay      = (`getArgD $* -xDelay     $xDelay`)
set yDelay      = (`getArgD $* -yDelay     $yDelay`)
set xP0         = (`getArgD $* -xP0        $xP0`)
set xP1         = (`getArgD $* -xP1        $xP1`)
set yP0         = (`getArgD $* -yP0        $yP0`)
set yP1         = (`getArgD $* -yP1        $yP1`)
set diagFlag    = (`getArgD $* -diag       $diagFlag`)
set xActive     = (`getArgD $* -xActive    $xActive`)
set yActive     = (`getArgD $* -yActive    $yActive`)
set id          = (`getArgD $* -id         $id`)
set itMax       = (`getArgD $* -iter       $itMax`)

#
#------------------------------------------------------------------

echo REMARK Extracting spectral parameters ...

set expName     = acme.exp.dat
set simName     = acme.sim.dat
set difName     = acme.dif.dat
set tabName     = acme.tab

set tX  = (`getParm -in $parentName -parm NDAPOD     -dim CUR_XDIM -fmt %.0f`)
set fX  = (`getParm -in $parentName -parm NDSIZE     -dim CUR_XDIM -fmt %.0f`)
set qcX = (`getParm -in $parentName -parm NDAPODCODE -dim CUR_XDIM -fmt %.0f`)
set q1X = (`getParm -in $parentName -parm NDAPODQ1   -dim CUR_XDIM`)
set q2X = (`getParm -in $parentName -parm NDAPODQ2   -dim CUR_XDIM`)
set q3X = (`getParm -in $parentName -parm NDAPODQ3   -dim CUR_XDIM`)
set swX = (`getParm -in $parentName -parm NDSW       -dim CUR_XDIM`)
set nX  = $fX

set tY  = (`getParm -in $parentName -parm NDAPOD     -dim CUR_YDIM -fmt %.0f`)
set fY  = (`getParm -in $parentName -parm NDSIZE     -dim CUR_YDIM -fmt %.0f`)
set qcY = (`getParm -in $parentName -parm NDAPODCODE -dim CUR_YDIM -fmt %.0f`)
set q1Y = (`getParm -in $parentName -parm NDAPODQ1   -dim CUR_YDIM`)
set q2Y = (`getParm -in $parentName -parm NDAPODQ2   -dim CUR_YDIM`)
set q3Y = (`getParm -in $parentName -parm NDAPODQ3   -dim CUR_YDIM`)
set swY = (`getParm -in $parentName -parm NDSW       -dim CUR_YDIM`)
set nY  = $fY

if ($tX > $nX) then
   set $nX = $tX
endif

if ($tY > $nY) then
   set $nY = $tY
endif

@ n = 1 + $#argv - $pCount
set pInfo = ($argv[$n-$#argv])

set n     = `IMATH 36\*$sigCount`
set aInfo = ($argv[1-$n])

echo REMARK Calculating model ...

modelXY -signal -silent -id $id -tol 1e-6 -p $pInfo -a $aInfo \
  -data  $expName -macro $NMRTXT/acme2D.M -gc 1 -cf 0.68 -noise $noise     \
  -str   simName $simName difName $difName tabName $tabName                \
  -var   nX $nX tSizeX $tX fSizeX $fX swX $swX q1X $q1X q2X $q2X q3X $q3X  \
         nY $nY tSizeY $tY fSizeY $fY swY $swY q1Y $q1Y q2Y $q2Y q3Y $q3Y  \
         maxSig $maxSig maxPassive $maxPassive sigCount $sigCount          \
         xDelay $xDelay yDelay $yDelay xP0 $xP0 xP1 $xP1 yP0 $yP0 yP1 $yP1 \
         diagFlag $diagFlag xActive $xActive yActive $yActive              \
         qcX $qcX qcY $qcY -iter $itMax 

