#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help] || $ARGC == 1} \
   {
    puts "addFC.tcl: Insert or replace FC force constant values in a table:"
    puts " -in     inName   Input Table."
    puts " -out    outName  Output Table."
    puts " -fc     fc       Default Force Constant."
    puts "Set Force Constant by ATOMNAME ATOMNAME_I ATOMNAME_J etc:"
    puts " -fcInfo fcInfo   Atom Name Groups: e.g. HN,N C,N etc."
    puts " -fcList fcList   Corresponding Force Constant List."
    exit 0
   }

set inName  None
set outName None
set fcDef   1.0
set fcList  1.0
set fcInfo  None 

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -fc  fcDef

getListArgD $ARGV -fcList fcList
getListArgD $ARGV -fcInfo fcInfo 

#
#------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts "Error: no such input file $inName"
    exit 1
   }

set n  [llength $fcList]
set nn [llength $fcInfo]

if {$n != $nn} \
   {
    puts "Error: fcList/fcInfo mismatch."
   exit 1
   }

set inTab [gdbRead table -in $inName]
set vList [gdbGetInfo $inTab -parms varNames]

if {[lsearch -exact $vList FC] < 0} \
   {
    gdbCreate variable -parent $inTab -name FC -varType float -varFmt %8.4f 
   }

set aList ""

foreach i {"" "_I" "_J" "_K" "_L" "_M" "_N" "_O" "_P"} \
   {
    set vName ATOMNAME$i

    if {[lsearch -exact $vList $vName] >= 0} {lappend aList $vName}
   }

set qList ""

foreach i $fcInfo \
   {
    lappend qList [join [lsort [split $i ","]] ","]
   }

set eList [gdbGetInfo $inTab -parms entries]

if {![strcasecmp $fcInfo None]} \
   {
    foreach entry $eList {gdbSet $entry FC $fcDef}
   } \
else \
   {
    foreach entry $eList \
       {
        set aInfo ""

        foreach a $aList {lappend aInfo [gdbGet $entry $a]}

        set q   [join [lsort $aInfo] ","]
        set loc [lsearch -exact $qList $q] 

        if {$loc < 0} \
           {
            gdbSet $entry FC $fcDef
           } \
        else \
           {
            gdbSet $entry FC [lindex $fcList $loc]
           }
       }
   }

gdbWrite $inTab $outName

exit
