#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Add or Adjust a Variable in an NMRPipe format Table."
    puts " -in    inName  \[test.tab\]  Name of Input Table."
    puts " -out   outName \[stdout\]    Name of Output Table."
    puts " -var   varName \[V\]         Name of Table Variable to Create."
    puts " -val   varVal  \[0\]         Initial Value of Variable to Create."
    puts " -fmt   varFmt  \[%s\]        Format of Table Variable."
    puts " -after locVar  \[END]\       Existing Variable for Insert Location."
    puts " -cond  cond    \[1]\         Selection Condition."
    puts "Variable Type, Use Only One:"
    puts " -float                     Floating Point Number (Sets Default -fmt %f)."
    puts " -int                       Integer (Sets Default -fmt %d)."
    puts " -text                      Text String (Sets Default -fmt %s)."
    exit 1
   }

set inName   test.tab 
set outName  stdout
set locVar   END
set varType  text
set varFmt   %s
set varName  V 
set varVal   0
set cond     1
set outUnit  stdout

if {[flagLoc $ARGV -float]} \
   {
    set varType float
    set varFmt  %f
   }

if {[flagLoc $ARGV -int]} \
   {
    set varType int
    set varFmt  %d
   }

if {[flagLoc $ARGV -text]} \
   {
    set varType text 
    set varFmt  %s
   }

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -after locVar
getArgD $ARGV -var   varName
getArgD $ARGV -val   varVal 
getArgD $ARGV -fmt   varFmt

getListArgD $ARGV -cond cond

#
#----------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error finding input file $inName"
    exit 1
   }

if {[strcasecmp $outName stdout]} \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error opening output file $outName"
        exit 1
       }
   }

set t        [gdbRead table -in $inName -blank]
set text     [gdbGetInfo $t -parms text]
set vList    [gdbGetInfo $t -parms varNames]
set vListNew $vList

set wasBlank 0

while {$text} \
   {
    set str [gdbGetInfo $text -parms value]
    set key ""

    scan $str %s key

    set n [string length $key]

    if {$n} \
       {
        puts $outUnit $str
        set wasBlank 0
       } \
    else \
       {
        if {!$wasBlank} {puts $outUnit $str}
        set wasBlank 1
       }

    set text [gdbNext $text]
   }

set i 0
                                                                                
foreach v $vList \
   {
    set vID [gdbFindByName $t $v]
    set fInfo($v) [gdbGetInfo $vID -parms varFmt]
    incr i
   }

set fInfo($varName) $varFmt

if {[lsearch -exact $vList $varName] < 0} \
   {
    gdbCreate variable -parent $t -varType $varType -name $varName -varFmt $varFmt 

    set fInfo($varName) $varFmt

    if {![strcasecmp $locVar END]} \
       {
        set vListNew $vList
        lappend vListNew $varName
       } \
    else \
       {
        set loc [expr [lsearch -exact $vList $locVar] + 1]
        if {$loc < 0} {set loc 0}

        set vListNew [linsert $vList $loc $varName]
       }
   }

puts -nonewline $outUnit "VARS  "
foreach v $vListNew {puts -nonewline $outUnit " $v"}
puts $outUnit ""

puts -nonewline $outUnit "FORMAT"
foreach v $vListNew {puts -nonewline $outUnit " $fInfo($v)"}
puts $outUnit ""
puts $outUnit ""

set eList [gdbSelect -from $t -cond $cond]

foreach e $eList \
   {
    gdbSet $e $varName $varVal

    foreach v $vListNew \
       {
        set val [gdbGet $e $v]

        if {[llength $val] > 1} \
           {
            puts -nonewline $outUnit " [format $fInfo($v) \"$val\"]"
           } \
        else \
           {
            puts -nonewline $outUnit " [format $fInfo($v) $val]"
           }
       }

    puts $outUnit ""
   }

if {[strcasecmp $outUnit stdout]} {close $outUnit}

exit
