#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Print Protein Torsion Angle Information:"
    puts stderr " angles.tcl \[pdbName1 ...\]"
    exit 0
   }

foreach inName $argv \
   {
    if {![file exists $inName]} continue

    set pdbTab  [gdbRead table -in $inName -name "$inName PDB" -pdb]

    set  minRes [getMinRes $pdbTab]
    set  maxRes [getMaxRes $pdbTab]

    for {set resID $minRes} {$resID <= $maxRes} {incr resID} \
       {
        set phiPsi  [getPhiPsiAngles $resID $pdbTab]
        set omega   [getOmegaAngle   $resID $pdbTab]
        set chi     [getChiAngle     $resID $pdbTab]

        set phi [lindex $phiPsi 0]
        set psi [lindex $phiPsi 1]

        set resName [getNthResName $pdbTab $resID]
        set fmt     "%4d %4s % 9.3f % 9.3f % 9.3f % 9.3f %s"

        puts [format $fmt $resID $resName $phi $psi $omega $chi $inName]
       }

    gdbFree $pdbTab
   }

exit
