#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Append NMRPipe-Format Spectral Data to a Single File:"
 puts stderr " -in    inList   \[None\]        List of NMRPipe-Format Input Files."
 puts stderr " -out   outName  \[append.dat\]  Output, or Keyword stdout."
 puts stderr " -ndim  dimCount \[1\]           Append Mode Dimension Count, 1 or 2."
 puts stderr " -nohdr                        Write Data Only, without Header."
 puts stderr " -verb                         Verbose Mode ON."
 exit 1
}

set inList      None
set outName     append.dat
set outDimCount 1
set hdrFlag     1
set verbose     0

getListArgD $ARGV -in inList

getArgD $ARGV -out  outName
getArgD $ARGV -ndim outDimCount

if {[flagLoc $ARGV -hdr]}    {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}  {set hdrFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {$outDimCount != 1 && $outDimCount != 2} \
   {
    puts stderr "AppendNMR Error: Bad Dimension Count Mode $outDimCount."
    appendExit 1
   }

#
#------------------------------------------------------------------------------

proc appendExit { error } \
{
   if {$error} sendNullHdr
   exit $error
}

proc sendNullHdr {} \
{
   global outName outUnit hdrDoneFlag outFDATA

   if {[strcasecmp $outName stdout]} return
   if {$hdrDoneFlag} return

   puts stderr "AppendNMR Exiting with Null Header."

   vsSet $outFDATA 0.0 $FDATASIZE
   set error [catch "sysWrite $outUnit $outFDATA $FDATASIZE"]
}

set thisList    ""
set hdrDoneFlag 0
set totalSize   0
set totalVecs   0
set maxSize     0 
set inCount     0
set outUnit     1
set workBuff    0
set inFDATA     0
set outFDATA    0

set inFDATA  [sysMalloc $FDATASIZE float]
set outFDATA [sysMalloc $FDATASIZE float]

foreach inName $inList \
   {
    if {![file exists $inName]} continue
    lappend thisList $inName
    incr inCount
   }

if {!$inCount} {appendExit 1}

set i 0

foreach inName $thisList \
   {
    set sizeInfo($i) 0
    set dimCount($i) 0
    set qSize($i)    0
    set xSize($i)    0
    set ySize($i)    0
    set zSize($i)    0
    set aSize($i)    0
    set vCount($i)   0
    set fdata($i)    0
    set rdata($i)    0
    incr i
   }

set i 0

foreach inName $thisList \
   {
    if {![file exists $inName]} \
       {
        puts stderr "AppendNMR Error Finding Input $inName"
        appendExit 1
       }

    set error [rdFDATA $inName fdata($i)]

    if {$error} \
       {
        puts stderr "AppendNMR Error Reading Header in $inName"
        appendExit 1
       } 

    set sizeInfo($i) [getDataSize $fdata($i)]

    set dimCount($i) [lindex $sizeInfo($i) 1]
    set nSize($i)    [lindex $sizeInfo($i) 0]
    set qSize($i)    [lindex $sizeInfo($i) 2]
    set xSize($i)    [lindex $sizeInfo($i) 3]
    set ySize($i)    [lindex $sizeInfo($i) 4]
    set zSize($i)    [lindex $sizeInfo($i) 5]
    set aSize($i)    [lindex $sizeInfo($i) 6]
    set vCount($i)   [expr $qSize($i)*$ySize($i)*$zSize($i)*$aSize($i)]

    if {$verbose} \
       {
        set fmt " %s Points Per Vector: %5d Vectors: %5d"
        puts stderr [format $fmt $inName $xSize($i) $vCount($i)]
       }

    if {$xSize($i) > $maxSize} {set maxSize $xSize($i)}

    incr totalSize $nSize($i)
    incr totalVecs $vCount($i)

    incr i
   }

if {$verbose} \
   {
    puts stderr "AppendNMR: Max Size: $maxSize Total Size: $totalSize Vectors: $totalVecs"
   }

if {![strcasecmp $outName None]} {exit 0}

set i 0

foreach inName $thisList \
   {
    set error [catch {set rdata($i) [sysMalloc $nSize($i) float]}]

    if {$error} \
       {
        puts stderr "AppendNMR Error Allocating Data for $inName"
        appendExit 1
       }

    set error [catch {set inUnit [sysOpen $inName r]}]

    if {$error} \
       {
        puts stderr "AppendNMR Error Opening File $inName"
        appendExit 1
       }

    set error [catch {sysRead $inUnit $inFDATA $FDATASIZE]}]

    if {$error} \
       {
        puts stderr "AppendNMR Error Reading Header in $inName"
        appendExit 1
       }

    set error [catch {sysRead $inUnit $rdata($i) $nSize($i)]}]

    if {$error} \
       {
        puts stderr "AppendNMR Error Reading File $inName"
        appendExit 1
       }

    incr i
   }

set error [catch {set workBuff [sysMalloc $maxSize float]}]

if {$error} \
   {
    puts stderr "AppendNMR Error Allocating Work Buffer."
    appendExit 1
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 1
   } \
else \
   {
    set error [catch {set outUnit [sysOpen $outName w]}]
   }

if {$hdrFlag} \
   {
    set i 0

    if {$outDimCount == 1} \
       {
        setParm $fdata($i) FDDIMCOUNT 1 NULL_DIM
        setParm $fdata($i) FDQUADFLAG 1 NULL_DIM
        setParm $fdata($i) NDQUADFLAG 1 CUR_XDIM

        setParm $fdata($i) NDSIZE $totalSize CUR_XDIM
        setParm $fdata($i) NDSIZE 1          CUR_YDIM
        setParm $fdata($i) NDSIZE 1          CUR_ZDIM
        setParm $fdata($i) NDSIZE 1          CUR_ADIM
       } \
    else \
       {
        setParm $fdata($i) FDDIMCOUNT 2 NULL_DIM
        setParm $fdata($i) FDQUADFLAG 1 NULL_DIM
        setParm $fdata($i) NDQUADFLAG 1 CUR_XDIM

        setParm $fdata($i) NDSIZE $maxSize   CUR_XDIM
        setParm $fdata($i) NDSIZE $totalVecs CUR_YDIM
        setParm $fdata($i) NDAPOD $totalVecs CUR_YDIM
        setParm $fdata($i) NDSIZE 1          CUR_ZDIM
        setParm $fdata($i) NDSIZE 1          CUR_ADIM
       }

    set error [catch "sysWrite $outUnit $fdata($i) $FDATASIZE"]
    set hdrDoneFlag 1

    if {$error} \
       {
        puts stderr "AppendNMR Error Writing Header to $outName"
        appendExit 1
       }
   }

set i 0

foreach inName $thisList \
   {
    set rPtr $rdata($i)

    for {set iv 0} {$iv < $vCount($i)} {incr iv} \
       {
        if {$outDimCount == 1} \
           {
            set error [catch "sysWrite $outUnit $rPtr $xSize($i)"]
           } \
        else \
           {
            vsSet  $workBuff 0.0   $maxSize
            vvMove $workBuff $rPtr $xSize($i)

            set error [catch "sysWrite $outUnit $workBuff $maxSize"]
           }

        if {$error} \
           {
            puts stderr "AppendNMR Error Writing Data from $inName $iv to $outName"
            appendExit 1
           }

        set rPtr [incrPtr $rPtr $xSize($i)]
       }

    incr i
   }

if {[strcasecmp $outName stdout]} \
   {
    catch "sysClose $outUnit"
   }

exit 0
