#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Append One or More NMRPipe-Format Tables or PDB Files."
    puts " -in       inList  \[test.tab\]  List of Input Tables."
    puts " -out      outName \[stdout\]    Name of Output Table."
    puts " -renum    varName \[None\]      Variable to Re-Number."
    puts " -hetatm                       Interpret HETATMs as ATOMs (PDB Only)."
    puts " -keepText                     Keep Text Info from First File."
    puts "Notes:"
    puts " Input or output with .pdb extension will be treated as PDB Format."
    exit 1
   }

set inNameList test.tab 
set outName    stdout
set varName    None
set outUnit    stdout
set pdbMode    nopdb
set hetMode    nohetatm
set keepText   0

getListArgD $ARGV -in inNameList

getArgD $ARGV -out    outName
getArgD $ARGV -renum  varName

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {[flagLoc $ARGV -keepText]}   {set keepText   1}
if {[flagLoc $ARGV -nokeepText]} {set nokeepText 1}

#
#-------------------------------------------------------------------------

foreach inName $inNameList \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Error finding input file $inName"
        exit 1
       }
   }

if {[strcasecmp $outName stdout]} \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error opening output file $outName"
        exit 1
       }
   }

set n [llength $inNameList]

set vListAll ""

for {set i 0} {$i < $n} {incr i} \
   {
    set inName [lindex $inNameList $i]

    if {![strcasecmp [file extension $inName] .pdb]} \
       {
        set t($i)   [gdbRead table -in $inName -pdb -$hetMode]
        set pdbMode pdb
       } \
    else \
       {
        set t($i)   [gdbRead table -in $inName]
        set pdbMode nopdb
       }

    set eList($i) [gdbGetInfo $t($i) -parms entries]
    set vList($i) [gdbGetInfo $t($i) -parms varNames]
    set vListAll  [listUnion $vListAll $vList($i)]

    foreach v $vList($i) \
       {
        set vID [gdbFindByName $t($i) $v]
        set fInfo($v) [gdbGetInfo $vID -parms varFmt]
        set nInfo($v) [string length [format $fInfo($v) 0]]
       }
   }

if {[strcasecmp $varName None]} \
   {
    set prevMax 0

    for {set i 0} {$i < $n} {incr i} \
       {
        set thisMin [getTabMin $t($i) $varName]
        set offset  [expr ($thisMin - $prevMax) - 1]

        foreach e $eList($i) \
           {
            gdbSet $e $varName [expr [gdbGet $e $varName] - $offset]
           }

        set prevMax [getTabMax $t($i) $varName]
       }
   }

if {$keepText} \
   {
    set text [gdbGetInfo $t(0) -parms text]
     
    while {$text} \
       {
        set str [gdbGetInfo $text -parms value]
        set key ""
 
        scan $str %s key
 
        if {![strcasecmp $key VARS]}   continue
        if {![strcasecmp $key FORMAT]} continue
                                                                                
        puts $outUnit $str
                                                                                
        set text [gdbNext $text]
       }
   }

if {[strcasecmp $pdbMode pdb]} {puts $outUnit ""}
puts -nonewline $outUnit "VARS  "
foreach v $vListAll {puts -nonewline $outUnit " $v"}
puts $outUnit ""
                                                                                
puts -nonewline $outUnit "FORMAT"
foreach v $vListAll {puts -nonewline $outUnit " $fInfo($v)"}
puts $outUnit ""

if {[strcasecmp $pdbMode pdb]} {puts $outUnit ""}

for {set i 0} {$i < $n} {incr i} \
   {
    if {[strcasecmp $pdbMode pdb]} \
       {
        puts $outUnit "REMARK Table [lindex $inNameList $i]"
       }

    foreach e $eList($i) \
       {
        foreach v $vListAll \
           {
            if {[lsearch -exact $vList($i) $v] < 0} \
               {
                set val [format %$nInfo($v)s (Null)]
               } \
            else \
               {
                set val [format $fInfo($v) [gdbGet $e $v]]
               }
    
            puts -nonewline $outUnit " $val"
           }

        puts $outUnit ""
       }
   }

if {[strcasecmp $outUnit stdout]} {close $outUnit}

if {![strcasecmp [file extension $outName] .pdb]} \
   {
    set t(out) [gdbRead table -in $outName]
    gdbWrite $t(out) $outName -pdb
   }

exit
