#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
#----------------------------------------------------------
 
if {[flagLoc $ARGV -help]} \
   {
    puts "autoProc.tcl: Auto 1D/2D process and PCA for spectral series."
    puts " -list  listName \[in.list\]     List of Data Directories to Process."
    puts " -conv  convName \[fid.com\]     Name of Example Conversion Script."
    puts " -xP0   xP0      \[Auto\]        X-Axis P0 Phase (or Keyword Auto)."
    puts " -xP1   xP1      \[0\]           X-Axis P1 Phase."
    puts " -yP0   yP0      \[-90\]         Y-Axis P0 Phase."
    puts " -yP1   yP1      \[180\]         Y-Axis P1 Phase."
    puts " -def   maskVal  \[1.0\]         Default Mask Value, If Any."
    puts " -ndim  dimCount \[2\]           Dimension Count of Input Data."
    puts "Solvent Correction, Window Functions:"
    puts " -sol   solFn   \[SOL\]          Function: SOL POLY CBF None (1D Default: CBF)."
    puts " -xAPOD aFn     \[SP\]           X-Axis Window Name (1D Default: EM)."
    puts " -xQ1   aQ1     \[0.50\]         X-Axis Window Q1 (1D Default: 1.0)."
    puts " -xQ2   aQ1     \[0.98\]         X-Axis Window Q2."
    puts " -xQ3   aQ1     \[2\]            X-Axis Window Q3."
    puts " -yAPOD aFn     \[SP\]           X-Axis Window Name."
    puts " -yQ1   aQ1     \[0.50\]         X-Axis Window Q1."
    puts " -yQ2   aQ1     \[0.98\]         X-Axis Window Q2."
    puts " -yQ3   aQ1     \[1\]            X-Axis Window Q3."
    puts " -lb    lbHz    \[1.0\]          1D EM Window Line Broadening Value;"
    puts "                                 (Same as -xAPOD EM -xQ1 ...) "
    puts "1D Extract and Process Default Parameters:"
    puts " -x1    x1Loc    \[5%\]          X-Axis Extraction Origin."
    puts " -xn    xnLoc    \[95%\]         X-Axis Extraction Endpoint."
    puts " -psx1  x1Loc    \[5%\]          X-Axis AutoPhase Origin."
    puts " -psxn  xnLoc    \[95%\]         X-Axis AutoPhase Endpoint."
    puts " -post  pCmnd    \[NULL\]        Post-Process Command."
    puts "2D Extract and Phase Default Parameters:"
    puts " -psx1  x1Loc    \[10.0ppm\]     X-Axis AutoPhase Origin."
    puts " -psxn  xnLoc    \[6.0ppm\]      X-Axis AutoPhase Endpoint."
    puts " -x1    x1Loc    \[10.5ppm\]     X-Axis Extraction Origin."
    puts " -xn    xnLoc    \[5.5ppm\]      X-Axis Extraction Endpoint."
    puts " -y1    y1Loc    \[0%\]          Y-Axis Extraction Origin."
    puts " -yn    ynLoc    \[100%\]        Y-Axis Extraction Endpoint."
    puts "Output Names:"
    puts " -com   comName   \[autoProc.com\]       Name of Output Script."
    puts " -fid   fidName   \[fid/test%05d.dat\]   Name of Ouput FID."
    puts " -ft    ftName    \[ft/test%05d.dat\]    Name of Output Spectrum."
    puts " -pca   pcaName   \[pca/pca%03d.dat\]    Name of Output PCA Data;"
    puts "                                       Use 'None' for No PCA."
    puts "Baseline Correction and Linear Prediction:"
    puts " -xBase     bCmnd    Baseline Correction Command."
    puts " -xBaseFirst         Perform Baseline Correction Prior to Phasing."
    puts " -lp        lpOrd    Order for Y-Axis Forward-Backward LP;"
    puts "                     Use Zero for No LP (Default)."
    puts "Flags:"
    puts " -scale          Rescale By Spectra Max Intensity."
    puts " -noscale        Use Unscaled Intensities (Default)."
    puts " -avg            PCA Iteration Starts with Average Spectrum."
    puts " -noavg          PCA Iteration Starts with 1st Spectrum (Default)."
    puts " -mask           Create and Apply Mask (Uses mask.tab as input)."
    puts " -noexec         Create Analysis Script, Don't Execute."
    puts " -clean          Delete Existing Data."
    puts ""
    puts "Notes:"
    puts " The mask table mask.tab lists limits of regions to"
    puts " emphasize  (SCALE > 1.0) or ignore  (SCALE = 0.0)."
    puts " Region limits are given in PPM, for example:"
    puts ""
    puts "   VARS   X1 XN Y1 YN SCALE"
    puts "   FORMAT %f %f %f %f %f"
    puts ""
    puts "   8.135 7.818 129.338 126.913 0.0"
    puts "   8.512 8.105 124.730 121.578 0.0"
    exit 0
   }

set dimCount 2

getArgD $ARGV -ndim  dimCount

set listName   in.list
set convName   fid.com
set comName    autoProc.com
set convCmnd   bruker
set xBase      None
set xBaseFirst 0

if {$dimCount == 1} \
   {
    set solFn  CBF 
    set x1     5%
    set xn    95%
    set psx1   5%
    set psxn  95%
   } \
else \
   {
    set solFn     SOL
    set x1    10.5ppm
    set xn     5.5ppm
    set psx1  10.0ppm
    set psxn   6.0ppm
   }

set y1       0%
set yn     100%

set xP0    Auto 
set xP1       0 
set yP0     -90
set yP1     180

if {$dimCount == 1} \
   {
    set xAPOD   EM 
    set xQ1     1.0
    set xQ2     0.0
    set xQ3     0.0 
   } \
else \
   {
    set xAPOD   SP
    set xQ1     0.50
    set xQ2     0.98
    set xQ3     2
   }

set yAPOD   SP
set yQ1     0.50
set yQ2     0.98
set yQ3     1

set defVal  1.0
set lb      1.0

set avgFlag   0
set maskFlag  0
set scaleFlag 0
set execFlag  1
set cleanFlag 0

set specType  amide 
set postCmnd  NULL
set fidName   fid/test%05d.dat
set ftName    ft/test%05d.dat
set pcaName   pca/pca%03d.dat
set lpOrder   0

getArgD $ARGV -list  listName
getArgD $ARGV -conv  convName
getArgD $ARGV -cmnd  convCmnd
getArgD $ARGV -com   comName
getArgD $ARGV -fid   fidName
getArgD $ARGV -ft    ftName
getArgD $ARGV -pca   pcaName
getArgD $ARGV -x1    x1
getArgD $ARGV -xn    xn 
getArgD $ARGV -y1    y1
getArgD $ARGV -yn    yn 
getArgD $ARGV -psx1  psx1
getArgD $ARGV -psxn  psxn 
getArgD $ARGV -xBase xBase
getArgD $ARGV -post  postCmnd
getArgD $ARGV -xP0   xP0
getArgD $ARGV -xP1   xP1
getArgD $ARGV -yP0   yP0
getArgD $ARGV -yP1   yP1
getArgD $ARGV -def   defVal 
getArgD $ARGV -type  specType
getArgD $ARGV -lp    lpOrder
getArgD $ARGV -sol   solFn

getArgD $ARGV -lb    xQ1 
getArgD $ARGV -xAPOD xAPOD
getArgD $ARGV -xQ1   xQ1
getArgD $ARGV -xQ2   xQ2
getArgD $ARGV -xQ3   xQ3
 
getArgD $ARGV -yAPOD yAPOD
getArgD $ARGV -yQ1   yQ1
getArgD $ARGV -yQ2   yQ2
getArgD $ARGV -yQ3   yQ3
 
if {[flagLoc $ARGV -xBaseFirst]} {set xBaseFirst 1}

if {[flagLoc $ARGV -nopca]}   {set pcaName None}

if {[flagLoc $ARGV -avg]}     {set avgFlag  1}
if {[flagLoc $ARGV -noavg]}   {set avgFlag  0}

if {[flagLoc $ARGV -scale]}   {set scaleFlag 1}
if {[flagLoc $ARGV -noscale]} {set scaleFlag 0}

if {[flagLoc $ARGV -clean]}   {set cleanFlag 1}
if {[flagLoc $ARGV -noclean]} {set cleanFlag 0}

if {[flagLoc $ARGV -noexec]}  {set execFlag  0}

if {[flagLoc $ARGV -mask]}    {set maskFlag  1}

update

if {$cleanFlag} \
   {
    foreach i "$fidName $ftName $pcaName" \
       {
        catch "file delete -force [file dirname $i]"
       }
   }

#
#----------------------------------------------------------

proc strSubs { origLine pattern str offset } \
{
   set loc [lsearch -exact $origLine $pattern]

   if {$loc < 0} {return $origLine}

   set line  [string trim $origLine "\\"]

   if {[strcmp $line $origLine]} \
      {
       set addBS 1
      } \
   else \
      {
       set addBS 0
      }

   set first [expr $loc + $offset]
   set last  [expr $loc + $offset]
   set line  [lreplace $line $first $last $str]
   set n     [string length $line]
   set s     ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $line $i]

       if {![strcmp $c "\{"]} continue
       if {![strcmp $c "\}"]} continue

       append s $c
      }

   if {$addBS} {append s " \\"}
 
   return $s
}

proc initProcIO {} \
{
   global autoFlag execFlag listName inList convName convCmnd comName
   global iu ou

   if {[llength [glob -nocomplain ft/*]]} \
      {
       puts "Warning: directory ft should be empty."
       if {$execFlag} {exit 1}
      }

   if {[llength [glob -nocomplain pca/*]]} \
      {
       puts "Warning: directory pca should be empty." 
       if {$execFlag} {exit 1}
      }

   set error [catch {set iu [open $listName r]}]

   if {$error} \
      {
       puts "Error opening input directory list $listName"
       exit 1
      }

   set inList [read $iu]
   close $iu

   puts "[llength $inList] spectra"

   if {![strcasecmp $convName Auto]} \
      {
       set thisDir [lindex $inList 0]
       catch "exec $convCmnd -auto -nosleep -cd $thisDir -script autofid.com"
       set convName $thisDir/autofid.com
      }

   set error [catch {set iu [open $convName r]}]

   if {$error} \
      {
       puts "Error opening example conversion script $convName"
       exit 1
      }

   set error [catch {set ou [open $comName w 0744]}]

   if {$error} \
      {
       puts "Error opening output automation script $comName"
       exit 1
      }
}

proc insertConvCmnd {} \
{
   global fidName ftName pcaName inName listName 
   global iu ou

   puts $ou "#!/bin/csh -f"
   puts $ou ""
   puts $ou "if (!(-d [file dirname $fidName])) then"
   puts $ou "   mkdir [file dirname $fidName]"
   puts $ou "endif"
   puts $ou ""
   puts $ou "if (!(-d [file dirname $ftName])) then"
   puts $ou "   mkdir [file dirname $ftName]"
   puts $ou "endif"
   puts $ou ""
   puts $ou "if (!(-d [file dirname $pcaName])) then"
   puts $ou "   mkdir [file dirname $pcaName]"
   puts $ou "endif"
   puts $ou ""
   puts $ou "set n = 0"
   puts $ou ""

   puts $ou "foreach id (`cat $listName`)"
   puts $ou "   @ n++"
   puts $ou "   set inName = None"
   puts $ou ""
   puts $ou "   if (-e \$id/ser) then"
   puts $ou "      set inName = \$id/ser"
   puts $ou "   endif"
   puts $ou ""
   puts $ou "   if (-e \$id/ser.Z) then"
   puts $ou "      set inName = \$id/ser.Z"
   puts $ou "   endif"
   puts $ou ""
   puts $ou "   if (-e \$id/fid) then"
   puts $ou "      set inName = \$id/fid"
   puts $ou "   endif"
   puts $ou ""
   puts $ou "   if (-e \$id/fid.Z) then"
   puts $ou "      set inName = \$id/fid.Z"
   puts $ou "   endif"
   puts $ou ""
   puts $ou "   if (\$inName == None) then"
   puts $ou "      echo Error: missing input file from \$id"
   puts $ou "      exit" 
   puts $ou "   endif"
   puts $ou ""
   puts $ou "   set fidName = (`nmrPrintf $fidName \$n`)"
   puts $ou "   set ftName  = (`nmrPrintf $ftName  \$n`)"
   puts $ou ""
   puts $ou "   echo FID: \$inName \$fidName Spectrum: \$ftName"
   puts $ou ""
   
   while {[gets $iu line] >= 0} \
      {
       set key [lindex $line 0]

       if {![string length $line]} continue
       if {![string length $key]}  continue
   
       set line [strSubs $line bin2pipe  "bin2pipe  -u1 \$n -title \$id" 0]
       set line [strSubs $line bruk2pipe "bruk2pipe -u1 \$n -title \$id" 0]
       set line [strSubs $line var2pipe  "var2pipe  -u1 \$n -title \$id" 0]
   
       set line [strSubs $line "zcat"  \$inName    1]
       set line [strSubs $line "-in"   \$inName    1]
       set line [strSubs $line "-out"  \$fidName   1]

       switch -- $key \
          {
           "#!/bin/csh"   { puts $ou "" }
           "sleep"        { puts $ou "" }
           "default"      { puts $ou "   $line" }
          }
      }

   puts $ou ""
}

proc insertProcCmnd {} \
{
   global dimCount

   switch $dimCount \
      {
       1 {insert1DProcCmnd}
       2 {insert2DProcCmnd}

       default {
                puts stderr "AutoProc Error: Unsupported Dim Count $dimCount."
                exit 1
               }
      }

}

proc insert1DProcCmnd {} \
{
   global fidName ftName xBase xBaseFirst xP0 xP1 x1 xn psx1 psxn dimCount
   global solFn xAPOD xQ1 xQ2 xQ3 postCmnd iu ou

   if {![strcasecmp $fidName $ftName]} \
      {
       set ipArg "-inPlace"
      } \
   else \
      {
       set ipArg ""
      }

   puts $ou ""
   puts $ou "   nmrPipe -in \$fidName \\"

   if {[strcasecmp $solFn NULL] && [strcasecmp $solFn None]} \
      {
       if {![strcasecmp $solFn POLY]} \
          {
           puts $ou "   | nmrPipe  -fn POLY -time \\"
          } \
       else \
          {
           puts $ou "   | nmrPipe  -fn $solFn \\"
          } 
      }

   puts $ou "   | nmrPipe  -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -c 0.5 \\"
   puts $ou "   | nmrPipe  -fn ZF -auto \\"
   puts $ou "   | nmrPipe  -fn FT \\"
   puts $ou "   | nmrPipe  -fn EXT -x1 $x1 -xn $xn -sw \\"

   if {$xBaseFirst} \
      {
       puts $ou "   | nmrPipe  -fn $xBase \\"
      }

   if {![strcasecmp $xP0 Auto]} \
      {
       puts $ou "   | nmrPipe  -fn MAC -macro \$NMRTXT/aph1D.M -di -all  \\"
       puts $ou "                      -reg -str psX1 $psx1 psX2 $psxn   \\"
       puts $ou "                      -var xP1 $xP1                     \\"
      } \
   else \
      {
       puts $ou "   | nmrPipe  -fn PS -p0 $xP0 -p1 $xP1 -di \\"
      }

   if {[strcasecmp $xBase None]} \
      {
       puts $ou "   | nmrPipe  -fn $xBase \\"
      }
   
   puts $ou "   | nmrPipe  -fn $postCmnd \\"
   puts $ou "      -verb -ov -out \$ftName $ipArg"
   puts $ou "end"
   puts $ou ""
}

proc insert2DProcCmnd {} \
{
   global fidName ftName xP0 xP1 yP0 yP1 x1 xn y1 yn psx1 psxn dimCount lpOrder
   global solFn xAPOD xQ1 xQ2 xQ3 yAPOD yQ1 yQ2 yQ3 iu ou

   if {![strcasecmp $fidName $ftName]} \
      {
       set ipArg "-inPlace"
      } \
   else \
      {
       set ipArg ""
      }
  
   set yC    1.0
   set yBase "NULL             "

   if {$yP1 > 350 && $yP1 < 370} {set yBase "POLY -auto -ord 0"}
   if {$yP1 < 10.0}              {set yC    0.5}
 
   puts $ou ""
   puts $ou "   nmrPipe -in \$fidName \\"

   if {[strcasecmp $solFn NULL] && [strcasecmp $solFn None]} \
      {
       if {![strcasecmp $solFn POLY]} \
          {
           puts $ou "   | nmrPipe  -fn POLY -time \\"
          } \
       else \
          {
           puts $ou "   | nmrPipe  -fn $solFn \\"
          }
      }

   puts $ou "   | nmrPipe  -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -c 0.5 \\"
   puts $ou "   | nmrPipe  -fn ZF -auto \\"
   puts $ou "   | nmrPipe  -fn FT -verb \\"
   puts $ou "   | nmrPipe  -fn EXT -x1 $x1 -xn $xn -sw \\"

   if {![strcasecmp $xP0 Auto]} \
      {
       puts $ou "   | nmrPipe  -fn MAC -macro \$NMRTXT/aph.M -di -all    \\"
       puts $ou "                      -reg -str psX1 $psx1 psX2 $psxn   \\"
       puts $ou "                      -var xP1 $xP1                     \\"
      } \
   else \
      {
       puts $ou "   | nmrPipe  -fn PS -p0 $xP0 -p1 $xP1 -di \\"
      }
   
   puts $ou "   | nmrPipe  -fn TP \\"

   if {$lpOrder > 0} \
      {
       puts $ou "   | nmrPipe  -fn LP -fb -ord $lpOrder \\"
      }

   puts $ou "   | nmrPipe  -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -c $yC \\"
   puts $ou "   | nmrPipe  -fn ZF -auto \\"
   puts $ou "   | nmrPipe  -fn FT -verb \\"
   puts $ou "   | nmrPipe  -fn PS -p0 $yP0 -p1 $yP1 -di \\"
   puts $ou "   | nmrPipe  -fn $yBase \\"

   if {[strcasecmp $y1 "0%"] || [strcasecmp $yn "100%"]} \
      {
       puts $ou "   | nmrPipe  -fn EXT -x1 $y1 -xn $yn -sw \\"
      }

   puts $ou "   | nmrPipe  -fn TP \\"
   puts $ou "   | nmrPipe  -fn POLY -auto \\"
   puts $ou "      -verb -ov -out \$ftName $ipArg"
   puts $ou "end"
   puts $ou ""
}

proc insertPostCmnd {} \
{
   global dimCount maskFlag scaleFlag avgFlag ftName pcaName 
   global iu ou defVal

   puts $ou "foreach i (ft/*)"

   if {$dimCount == 1} \
      {
       puts $ou ""
       puts $ou "  sethdr \$i -fileCount \$n -cubeFlag 0 -pipeFlag 0 \\"
       puts $ou "    -yN    1    -zN         \$n  \\"
       puts $ou "    -yT    1    -zT         \$n  \\"
       puts $ou "    -yMode Real -zMODE    Real  \\"
       puts $ou "    -yFT   Freq -zFT      Freq  \\"
       puts $ou "    -ySW   1.0  -zSW        \$n  \\"
       puts $ou "    -yOBS  1.0  -zOBS      1.0  \\"
       puts $ou "    -yCAR  0.0  -zCAR      0.0  \\"
       puts $ou "    -yLAB  Y    -zLAB       ID  \\"
       puts $ou "    -ndim  3"
       puts $ou ""
      } \
   else \
      {
       puts $ou ""
       puts $ou "  sethdr \$i -fileCount \$n -cubeFlag 0 -pipeFlag 0 \\"
       puts $ou "    -zN         \$n  \\"
       puts $ou "    -zT         \$n  \\"
       puts $ou "    -zMODE    Real  \\"
       puts $ou "    -zFT      Freq  \\"
       puts $ou "    -zSW        \$n  \\"
       puts $ou "    -zOBS      1.0  \\"
       puts $ou "    -zCAR      0.0  \\"
       puts $ou "    -zLAB       ID  \\"
       puts $ou "    -ndim        3"
       puts $ou ""
      }

   puts $ou "end"
   puts $ou ""

   if {$maskFlag} \
      {
       puts $ou ""
       puts $ou "mask.tcl -def $defVal -noapply"
       puts $ou ""
   
       set maskArg   "-mask mask.dat"
      } \
   else \
      {
       set maskArg   "-nomask"
      }
   
   if {$scaleFlag} \
      {
       puts $ou ""
       puts $ou "scale.com"
       puts $ou ""
      }
   
   if {$avgFlag} \
      {
       set avgArg "-avg"
      } \
   else \
      {
       set avgArg "-noavg"
      }

   if {[strcasecmp $pcaName None]} \
      {   
       if {$dimCount == 1} \
          {
           puts $ou ""
           puts $ou "pcaNMR -in $ftName $avgArg $maskArg \\"
           puts $ou "       -nc 5 -pca $pcaName -load load.dat -score score.dat"
           puts $ou ""
          } \
       else \
          {
           puts $ou ""
           puts $ou "pcaNMR -in $ftName $avgArg $maskArg \\"
           puts $ou "       -nc 5 -pca $pcaName -load load.dat"
           puts $ou ""
          }

       puts $ou "sleep 10"
      }
}

initProcIO
insertConvCmnd
insertProcCmnd
insertPostCmnd

close $iu
close $ou

if {$execFlag} \
   {
    set g 64x12+15+15
    set error [catch {exec xterm -bg cyan -fg black -geom $g -e $comName}]
   }

exit 0
