#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2021.122.17.40"

if {[flagLoc $ARGV -help]} \
   {
    puts "Compute Distances Between 1D or 2D NMR Spectra in Two Classes."
    puts "Version $thisVersion:"
    puts " -class1        c1List  \[None ...\]    Spectra in Class 1."
    puts " -class2        c2List  \[None ...\]    Spectra in Class 2."
    puts " -classV        cvList  \[None ...\]    Spectra in Optional Variance Class."
#   puts " -center1       cList1  \[None ...\]    Optional Spectra for Centering Class 1."
#   puts " -center2       cList2  \[None ...\]    Optional Spectra for Centering Class 2."
    puts " -zMode         zMode   \[SqrtSumSq\]   Class Variance: Self Max Avg SqrtSumSq."
    puts " -noise1        n1List  \[Auto ...\]    Noise Estimates for Class 1 Spectra."
    puts " -noise2        n2List  \[Auto ...\]    Noise Estimates for Class 2 Spectra."
    puts " -title1        title1  \[CLASS1\]      Title for Class 1."
    puts " -title2        title2  \[CLASS2\]      Title for Class 2."
    puts " -titleV        titleV  \[CLASSV\]      Title for Optional Variance Class."
#   puts " -titleC1       titleC1 \[CLASSC1\]     Title for Optional Centering Class 1."
#   puts " -titleC2       titleC2 \[CLASSC2\]     Title for Optional Centering Class 2."
    puts " -reg           regInfo \[0% 100% ...\] Region Limits for Each Dimension."
    puts " -outDir        outDir  \[Auto\]        Output Directory."
    puts " -brief                               Brief Report Format."
    puts " -verb                                Verbose Mode ON (Default)."
    puts " -noverb                              Verbose Mode OFF."
    puts "Distance Metric Parameters:"
    puts " -confid        c       \[95.0\]        Confidence Limit, Percent."
    puts " -thresh        t       \[0.0\]         Threshold Used by Any Metric."
    puts "PCA Score Distribution Surfaces:"
    puts " -pcaVarX       pcaVarX \[PCAZ1\]       PCA Variable for X-Axis of Distribution."
    puts " -pcaVarY       pcaVarY \[PCAZ2\]       PCA Variable for Y-Axis of Distribution."
    puts " -pcaXSigmaMult m       \[1.0\]         Width Multiplier for Distribution."
    puts " -pcaYSigmaMult m       \[1.0\]         Width Multiplier for Distribution."
    puts " -pcaSigmaPow   p       \[1.0\]         Exponent for Distribution (Usually m*m)."
    puts " -pcaXMin       xMin    \[Auto\]        X-Axis Start Coord of Distribution."
    puts " -pcaXMax       xMax    \[Auto\]        X-Axis End Coord of Distribution."
    puts " -pcaYMin       yMin    \[Auto\]        Y-Axis Start Coord of Distribution."
    puts " -pcaYMax       yMax    \[Auto\]        Y-Axis End Coord of Distribution."
    puts " -pcaXMargin    xm      \[0.2\]         X-Axis Margin Size as a Fraction of Range."
    puts " -pcaYMargin    ym      \[0.2\]         Y-Axis Margin Size as a Fraction of Range."
    puts " -pcaSquare                           Use the Same Limits for the X-Axis and Y-Axis." 
    puts " -autoNeg                             Automatically Negate Component Scores as Needed."
    puts " -noautoNeg                           No Automatic Negation of Component Scores (Default)."
    puts " -nobound                             Do Not Show Surface Bounds."
    puts "Normalization Program (Terminate with --):"
    puts " -prog          nProg  \[None\]         Normalization Program (See normSpec.tcl)."
    puts " -rescale                             Rescale Final Result to Match First Spectrum."
    puts " -norescale                           Do Not Rescale Final Result (Default)."
    puts "Plotting Options:"
    puts " -axis                                Plot With Axis (Default)."
    puts " -noaxis                              Plot Without Axis."
    puts " -plotLabel     label  \[Auto\]         Label for Plot, or Keyword None Auto."
    puts " -color1        color1 \[blue\]         Color for Class 1."
    puts " -color2        color2 \[red\]          Color for Class 2."
    puts " -colorV        colorV \[gray\]         Color for Variance Class."
    puts " -pageColor     pColor \[gray90\]       Color for Plot Page Background."
    puts " -plot                                Create Hard-Copy Plot (Default)."
    puts " -noplot                              No Hard-Copy Plot."
    puts "PCA Computation (Termininate with --):"
    puts " -pcaArgs       pArgs  \[None\]         Arguments for PCA Calculation."
    exit 0
   }

set ARGV_ORIG    $ARGV
set outDir       Auto
set allDir       None

set verbose      1
set briefFlag    0
set rescaleFlag  0
set plotFlag     1
set squareFlag   0
set boundFlag    1
set zMode        SqrtSumSq
set axisArg      axis

set classList(1)  None
set classList(2)  None
set classList(V)  None
set classList(C1) None
set classList(C2) None

set plotColor(1)  blue
set plotColor(2)  red
set plotColor(V)  green
set plotColor(T)  gray

set noiseList(1)  Auto
set noiseList(2)  Auto
set noiseList(V)  Auto

set plotLabel     Auto
set title(1)      CLASS1
set title(2)      CLASS2
set title(V)      CLASSV
set title(C1)     CLASSC1
set title(C2)     CLASSC2

set regInfo      "0% 100% 0% 100% 0% 100% 0% 100%"

set pcaArgs      None
set normProg     None

set pcaDX        0.75
set pcaDY        0.75

set pcaVarX      PCAZ1
set pcaVarY      PCAZ2

set pcaXSigmaMult 1.0
set pcaYSigmaMult 1.0
set pcaSigmaPow   1.0

set pcaXMin      Auto
set pcaXMax      Auto
set pcaYMin      Auto
set pcaYMax      Auto
set pcaXMargin   0.2
set pcaYMargin   0.2
set pcaNegFlag   0

set thresh       0.0
set confidPct    95.0
set pageColor    gray90

set ARGV [getListArgDT $ARGV -pcaArgs pcaArgs]
set ARGV [getListArgDT $ARGV -prog    normProg]

getListArgD $ARGV -class1   classList(1)
getListArgD $ARGV -class2   classList(2)
getListArgD $ARGV -classV   classList(V)
getListArgD $ARGV -center1  classList(C1)
getListArgD $ARGV -center2  classList(C2)

if {![strcasecmp $classList(V) None]} {set classList(V) ""}

getListArgD $ARGV -noise1   noiseList(1)
getListArgD $ARGV -noise2   noiseList(2)
getListArgD $ARGV -noiseV   noiseList(V)

getListArgD $ARGV -title    title(All)

getListArgD $ARGV -reg      regInfo

set refName [lindex $classList(1) 0]

getArgD $ARGV -ref           refName
getArgD $ARGV -outDir        outDir
getArgD $ARGV -allDir        allDir

getArgD $ARGV -pageColor     pageColor
getArgD $ARGV -plotLabel     plotLabel
getArgD $ARGV -title1        title(1)
getArgD $ARGV -title2        title(2)
getArgD $ARGV -titleV        title(V)
getArgD $ARGV -titleC1       title(C1)
getArgD $ARGV -titleC2       title(C2)

getArgD $ARGV -color1        plotColor(1) 
getArgD $ARGV -color2        plotColor(2) 
getArgD $ARGV -colorV        plotColor(V) 
getArgD $ARGV -colorT        plotColor(T)

getArgD $ARGV -zMode         zMode
getArgD $ARGV -thresh        thresh
getArgD $ARGV -confid        confidPct

getArgD $ARGV -pcaSigmaMult  pcaXSigmaMult
getArgD $ARGV -pcaSigmaMult  pcaYSigmaMult

getArgD $ARGV -pcaXSigmaMult pcaXSigmaMult
getArgD $ARGV -pcaYSigmaMult pcaYSigmaMult

getArgD $ARGV -pcaSigmaPow   pcaSigmaPow

getArgD $ARGV -pcaDX         pcaDX
getArgD $ARGV -pcaDY         pcaDY

getArgD $ARGV -pcaVarX       pcaVarX
getArgD $ARGV -pcaVarY       pcaVarY

getArgD $ARGV -pcaXMargin    pcaXMargin
getArgD $ARGV -pcaYMargin    pcaYMargin

getArgD $ARGV -pcaXMin       pcaXMin
getArgD $ARGV -pcaXMax       pcaXMax
getArgD $ARGV -pcaYMin       pcaYMin
getArgD $ARGV -pcaYMax       pcaYMax

if {[flagLoc $ARGV -axis]}       {set axisArg axis}
if {[flagLoc $ARGV -noaxis]}     {set axisArg noaxis}

if {[flagLoc $ARGV -autoNeg]}    {set pcaNegFlag 1}
if {[flagLoc $ARGV -noautoNeg]}  {set pcaNegFlag 0}

if {[flagLoc $ARGV -pcaSquare]}   {set squareFlag 1}
if {[flagLoc $ARGV -noocaSquare]} {set squareFlag 0}

if {[flagLoc $ARGV -rescale]}    {set rescaleFlag 1}
if {[flagLoc $ARGV -norescale]}  {set rescaleFlag 0}

if {[flagLoc $ARGV -bound]}      {set boundFlag 1}
if {[flagLoc $ARGV -nobound]}    {set boundFlag 0}

if {[flagLoc $ARGV -brief]}      {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]}    {set briefFlag 0}

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

set defNoise1 [lindex $noiseList(1) end]
set defNoise2 [lindex $noiseList(2) end]
set defNoiseV [lindex $noiseList(V) end]

foreach thisName $classList(1) {lappend noiseList(1) $defNoise1}
foreach thisName $classList(2) {lappend noiseList(2) $defNoise2}
foreach thisName $classList(V) {lappend noiseList(V) $defNoiseV}

set noiseList(1) [lrange $noiseList(1) 0 [expr [llength $classList(1)] - 1]]
set noiseList(2) [lrange $noiseList(2) 0 [expr [llength $classList(2)] - 1]]
set noiseList(V) [lrange $noiseList(V) 0 [expr [llength $classList(V)] - 1]]
 
#
#----------------------------------------------------------------------------------------------------------------------

proc safeExec { cmnd } \
{
   global outDir verbose

   if {$verbose} {puts stderr [format "classDistance. Command: %-14s Log: %s Directory: %s\n" [lindex $cmnd 0] $outDir/pcaDist.log [pwd]]}

   catch "eval exec execCmnd.com $outDir/pcaDist.log $cmnd"
}

proc findSizes { thisName } \
{
   global nextROIID roiInfo outDir fdata xSize ySize n dimCount regInfo coordInfo refName verbose

   if {![file exists $thisName]} \
      {
       puts stderr "classDistance. Error Finding Input $thisName"
       exit 1
      }

   set error [rdFDATA $thisName fdata]

   if {$error} \
      {
       puts stderr "classDistance. Error Reading Header From Input $thisName"
       exit 1
      }

   set dimCount  [getParmI $fdata FDDIMCOUNT NULL_DIM]

   set xSizeFull [getParmI $fdata NDSIZE CUR_XDIM]
   set ySizeFull [getParmI $fdata NDSIZE CUR_YDIM]

   if {$dimCount > 2} {set dimCount 2}

   if {$dimCount == 2 && $ySizeFull < 2} {set dimCount 1}
   if {$dimCount == 1} {set ySizeFull 1}

   if {$dimCount == 1} \
      {
       set coordInfo "-ndim 1 -x X_AXIS [lindex $regInfo 0] [lindex $regInfo 1]"
       set regInfo   [lrange $regInfo 0 1]
      } \
   elseif {$dimCount == 2} \
      {
       set coordInfo "-ndim 2 -x X_AXIS [lindex $regInfo 0] [lindex $regInfo 1] -y Y_AXIS [lindex $regInfo 2] [lindex $regInfo 3]"
       set regInfo   [lrange $regInfo 0 3]
      }

   set regInfo [lrange $regInfo 0 [expr 2*$dimCount]]

   if {[llength $regInfo] != 2*$dimCount} \
      {
       puts stderr "classDistance. Bad Region Coordinate List: $regInfo"
       exit 1
      }

   incr nextROIID

   eval "readROI  -in  $refName        -roi $nextROIID $coordInfo"
   eval "writeROI -out $outDir/ref.dat -roi $nextROIID"

   getThisROIInfo Ref $nextROIID

   set xSize $roiInfo(Ref,xSize)
   set ySize $roiInfo(Ref,ySize)
   set n     $roiInfo(Ref,n)

   if {$verbose} \
      {
       puts stderr ""
       puts stderr "classDistance. Output Directory: $outDir"
       puts stderr "classDistance. Reference Name:   $thisName"
       puts stderr "classDistance. Dimensions:       $dimCount"

       if {$dimCount >= 1} {puts stderr "classDistance. Full X-Size:      $xSizeFull"}
       if {$dimCount >= 2} {puts stderr "classDistance. Full Y-Size:      $ySizeFull"}

       puts stderr "classDistance. Region Info:      readROI $coordInfo"

       if {$dimCount >= 1} {puts stderr "classDistance. X-Size:           $xSize"}
       if {$dimCount >= 2} {puts stderr "classDistance. Y-Size:           $ySize"}

       puts stderr "classDistance. Total Size:       $n"
       puts stderr ""
      }
}

proc readClassList { thisList className classID } \
{
   global xSize ySize roiInfo nextROIID dimCount regInfo coordInfo noiseList verbose

   set i 0

   if {$verbose} {puts stderr "classDistance. Class: $classID Class Name: $className"}

   foreach thisName $thisList \
      {
       if {![file exists $thisName]} \
          {
           puts stderr "classDistance. Error Finding Class $classID Input $thisName"
           exit 1
          }

       incr nextROIID
       incr i

       eval "readROI -in $thisName -roi $nextROIID $coordInfo"

       getThisROIInfo $i,$classID $nextROIID

       if {$verbose} \
          {
           if {$dimCount == 1} \
              {
               set fmt "classDistance.  Spectrum: %3d Class: %s X-Size: %5d Input: %s %s"
               puts stderr [format $fmt $i $classID $roiInfo($i,$classID,xSize) $thisName $roiInfo($i,$classID,title)]
              } \
           else \
              {
               set fmt "classDistance.  Spectrum: %3d Class: %s X-Size: %4d Y-Size: %4d Input: %s %s"
               puts stderr [format $fmt $i $classID $roiInfo($i,$classID,xSize) $roiInfo($i,$classID,ySize) $thisName $roiInfo($i,$classID,title)]
              }
          }
      }

   if {$verbose} {puts stderr ""}
}

proc getThisROIInfo { specID roiID } \
{
   global roiInfo dimCount

   set roiInfo($specID) [getROIInfo -roi $roiID]

   set sizeInfo [lindex $roiInfo($specID) 2]

   set roiInfo($specID,roiID) $roiID

   set roiInfo($specID,rdata) [lindex $roiInfo($specID) 0]
   set roiInfo($specID,fdata) [lindex $roiInfo($specID) 1]

   set roiInfo($specID,xSize) 1
   set roiInfo($specID,ySize) 1

   if {$dimCount >= 1} \
      {
       set roiInfo($specID,xSize) [lindex $sizeInfo 0]
      }

   if {$dimCount >= 2} \
      {
       set roiInfo($specID,ySize) [lindex $sizeInfo 1]
      }

   set roiInfo($specID,title) [getParmStr $roiInfo($specID,fdata) FDTITLE NULL_DIM]
   if {![llength $roiInfo($specID,title)] || ![string length $roiInfo($specID,title)]} {set roiInfo($specID,title) $specID}

   set roiInfo($specID,n) [expr $roiInfo($specID,xSize)*$roiInfo($specID,ySize)]
}

proc findOutDir {} \
{
   global outDir

   if {![strcasecmp $outDir Auto]} \
      {
       for {set i 1} {$i <= 999} {incr i} \
          {
           set thisDir [format pca%03d $i]
   
        if {![file isdirectory $thisDir]} \
              {
               set outDir $thisDir
               break
              }
          }
      }
   
   if {![strcasecmp $outDir Auto]} \
      {
       puts stderr "classDistance. Error Creating Auto-Mode Output Directory (pca%03d)."
       exit 1
      }
   
   catch "file delete -force $outDir"
   catch "file mkdir $outDir"

   if {![file isdirectory $outDir]} \
      {
       puts stderr "classDistance. Error Creating Output Directory $outDir"
       exit 1
      }

   catch "file mkdir $outDir/ft"
   catch "file mkdir $outDir/var"
   catch "file mkdir $outDir/pca"
}

proc saveRegions {} \
{
   global outDir classList classSize roiInfo refName xSize ySize normProg rescaleFlag thresh title noiseList verbose

   set k 0

   foreach classID {1 2 V} \
      {
       set kk 0

       for {set i 1} {$i <= $classSize($classID)} {incr i} \
          {
           incr kk
           incr k

           if {![strcasecmp $classID V]} \
              {
               set thisOutName [format $outDir/var/test%05d.dat $kk]
              } \
           else \
              {
               set thisOutName [format $outDir/ft/test%05d.dat  $k]
              }

           set thisNoise   [lindex $noiseList($classID) [expr $i - 1]]
           set thisSrcName [lindex $classList($classID) [expr $i - 1]]
           set thisInName  $thisOutName

           set cmnd "writeROI -roi $roiInfo($i,$classID,roiID) -out $thisOutName"

           if {$verbose} \
              {
               puts stderr "classDistance. Spectrum: $i Class: $classID Class Name: $title($classID) Input: $thisSrcName Title: $roiInfo($i,$classID,title)"
               puts stderr "classDistance.  Initial Output: $thisOutName" 
              }

           catch "eval $cmnd"

           if {![file exists $thisOutName]} \
              {
               puts stderr "classDistance. Error Creating Output $thisOutName"
               exit 1
              }

           if {[strcasecmp $normProg None]} \
              {
               set normName $outDir/norm.dat
               set cmnd     "normSpec.tcl -in $thisInName -out $normName -thresh $thresh -noise $thisNoise -prog $normProg"

               if {$verbose} {puts stderr "classDistance.  Normalization:  normSpec.tcl -thresh $thresh -noise $thisNoise -prog $normProg"}

               catch {file delete -force $normName}
               
               safeExec $cmnd

               if {![file exists $normName]} \
                  {
                   puts stderr "classDistance. Error Creating Normalized Output ($cmnd)."
                   exit 1
                  }

               catch "file rename -force $normName $thisOutName"
              
               set thisInName $thisOutName
              }
       
           if {$roiInfo($i,$classID,xSize) != $xSize || $roiInfo($i,$classID,ySize) != $ySize} \
              {
               set interpName  $outDir/interp.dat
               set thisRefName $outDir/ref.dat

               set cmnd       "interpNMR -in $thisInName -ref $thisRefName -out $interpName"

               if {$verbose} {puts stderr "classDistance.  Interpolation:  $cmnd"}

               catch {file delete -force $interpName}

               safeExec $cmnd

               if {![file exists $interpName]} \
                  {
                   puts stderr "classDistance. Error Creating Interpolated Output ($cmnd)."
                   exit 1
                  }

               catch "file rename -force $interpName $thisOutName"
 
               set thisInName $thisOutName 
              }

           if {$rescaleFlag} \
              {
               set scaleName   $outDir/scale.dat
               set thisRefName $outDir/ft/test00001.dat

               set cmnd       "findScale.tcl -in1 $thisRefName -in2 $thisInName -out $scaleName"

               if {$verbose} {puts stderr "classDistance.  Rescaling:      $cmnd"}

               catch {file delete -force $scaleName}

               safeExec $cmnd

               if {![file exists $scaleName]} \
                  {
                   puts stderr "classDistance. Error Creating Rescaled Output ($cmnd)."
                   exit 1
                  }

               catch "file rename -force $scaleName $thisOutName"

               set thisInName $thisOutName
              }

           if {$verbose} {puts stderr ""}
          }
      }
}

proc performPCA {} \
{
   global dimCount title outDir classList classSize pcaArgs zMode plotColor verbose

   catch "file delete -force $outDir/pca.tab"

   if {![strcasecmp $pcaArgs None]} \
      {
       set thisArgs ""
      } \
   else \
      {
       set thisArgs $pcaArgs 
      }

   if {$dimCount == 1} \
      {
       set thisScoreName score001.dat
      } \
   else \
      {
       set thisScoreName score%03d.dat
      }

   set    cmnd "pcaNMR.com"
   append cmnd " -pcaTab $outDir/pca.tab -pcaArgs -in $outDir/ft/test%05d.dat -score $outDir/$thisScoreName"
   append cmnd " -load $outDir/load.dat -pca $outDir/pca/pca%03d.dat $thisArgs -noscale -avg -nc 6 -itmin 8 -conv 1.0e-6"

   if {$verbose} {puts stderr "classDistance. Performing PCA ..."}

   safeExec $cmnd

   if {![file exists $outDir/pca.tab]} \
      {
       puts stderr "classDistance. Error Creating PCA Result $outDir/pca.tab"
       exit 1
      }

   if {$verbose} {puts stderr "classDistance. Extracting PCA Coords ..."}

   set classNames ""
   set pcaColors  ""

   for {set i 0} {$i < $classSize(1)} {incr i} \
       {
        lappend classNames $title(1)
        lappend pcaColors  $plotColor(1)
       }

   for {set i 0} {$i < $classSize(2)} {incr i} \
       {
        lappend classNames $title(2)
        lappend pcaColors  $plotColor(2) 
       }

   for {set i 0} {$i < $classSize(V)} {incr i} \
       {
        lappend classNames $title(V)
        lappend pcaColors  $plotColor(V)
       }

   catch "file delete -force $outDir/pcaCoords.tab"

   set    cmnd "pcaCoords.tcl -in $outDir/ft/test%05d.dat"

   if {$classSize(V) > 0} {append cmnd " -inV $outDir/var/test%05d.dat -classV $title(V)"}

   append cmnd " -pcaCoords $outDir/pcaCoords.tab -load $outDir/load.dat -score $outDir/$thisScoreName -title Class $title(1) and $title(2)"
   append cmnd " -classList $classNames -zMode $zMode -pcaColors $pcaColors -pcaArgs $thisArgs -noscale -avg -nc 6 -itmin 8 -conv 1.0e-6"

   safeExec $cmnd

   if {![file exists $outDir/pcaCoords.tab]} \
      {
       puts stderr "classDistance. Error Creating PCA Coords Result $outDir/pcaCoords.tab"
       exit 1
      }
}

proc computeCoordDist {} \
{
   global outDir title pcaVarX pcaVarY pcaNegFlag title classList classSize distInfo verbose

   if {$verbose} {puts stderr "classDistance. Calculating Coord Distances ..."}

   set t     [gdbRead table -in $outDir/pcaCoords.tab]
   set eList [gdbGetInfo $t -parms entries]

   set classI $title(1)
   set classJ $title(2)

   foreach e $eList \
      {
       set thisClass [gdbGet $e CLASS]

       lappend xInfo($thisClass) [gdbGet $e $pcaVarX]
       lappend yInfo($thisClass) [gdbGet $e $pcaVarY]
      } 

   if {![info exists xInfo($classI)] || ![info exists xInfo($classJ)]} \
      {
       puts "classDistance. Error Finding Info for Class $classI or $classJ in PCA results $outDir/pcaCoords.tab"
       exit 1
      }

   if {[llength $xInfo($classI)] != $classSize(1) || [llength $xInfo($classJ)] != $classSize(2)} \
      {
       puts "classDistance. Error: Count Mismatch in PCA results $outDir/pcaCoords.tab"
       exit 1
      }

   if {$pcaNegFlag} \
      {
       set avgI [getAvg $xInfo($classI)]
       set avgJ [getAvg $xInfo($classJ)]

       if {$avgI > $avgJ} \
          {
           if {$verbose} {puts stderr "classDistance. Negating X-Axis Scores."}

           foreach e $eList {gdbSet $e $pcaVarX [expr -1*[gdbGet $e $pcaVarX]]}

           set xInfo($classI) [getNeg $xInfo($classI)]
           set xInfo($classJ) [getNeg $xInfo($classJ)]
          }

       set avgI [getAvg $yInfo($classI)]
       set avgJ [getAvg $yInfo($classJ)]

       if {$avgI < $avgJ} \
          {
           if {$verbose} {puts stderr "classDistance. Negating Y-Axis Scores."}

           foreach e $eList {gdbSet $e $pcaVarY [expr -1*[gdbGet $e $pcaVarY]]}

           set yInfo($classI) [getNeg $yInfo($classI)]
           set yInfo($classJ) [getNeg $yInfo($classJ)]
          }
      }

   gdbWrite $t $outDir/pcaCoordsAdj.tab
 
   set minD 0.0

   set k 0

   for {set i 0} {$i < $classSize(1)} {incr i} \
      {
       set xI [lindex $xInfo($classI) $i]
       set yI [lindex $yInfo($classI) $i]

       for {set j 0} {$j < $classSize(2)} {incr j} \
          {
           set xJ [lindex $xInfo($classJ) $j]
           set yJ [lindex $yInfo($classJ) $j]

           set dx [expr $xI - $xJ]
           set dy [expr $yI - $yJ]

           set d  [expr sqrt( $dx*$dx + $dy*$dy )]

           if {$k == 0}    {set minD $d}
           if {$d < $minD} {set minD $d}

           incr k
          }
      }

   set dx [expr [getAvg $xInfo($classI)] - [getAvg $xInfo($classJ)]]
   set dy [expr [getAvg $yInfo($classI)] - [getAvg $yInfo($classJ)]]

   set distInfo(center)  [expr sqrt( $dx*$dx + $dy*$dy )]
   set distInfo(nearest) $minD

   set distInfo(p,center)  [expr 100.0 - 100.0*[getZProb $distInfo(center)  6.0]]
   set distInfo(p,nearest) [expr 100.0 - 100.0*[getZProb $distInfo(nearest) 6.0]]
}

proc computeOverlap {} \
{
   global outDir allDir title pcaXSigmaMult pcaYSigmaMult pcaSigmaPow pcaVarX pcaVarY confidPct nextROIID distInfo verbose
   global pcaXMargin pcaYMargin pcaXMin pcaXMax pcaYMin pcaYMax squareFlag zMode

   if {$verbose} {puts stderr "classDistance. Creating PCA Surfaces ..."}

   catch "file delete -force $outDir/surf $outDir/surfWide"

   set type       overlap
   set mode       thresh

   foreach surfDir {surf surfWide} \
      {
       set thisName1 $outDir/$surfDir/$title(1).dat
       set thisName2 $outDir/$surfDir/$title(2).dat

       set    cmnd "tab2surf.tcl -in $outDir/pcaCoordsAdj.tab -outDir $outDir/$surfDir -var $pcaVarX $pcaVarY -normMode $mode"
       append cmnd " -xm $pcaXMargin -ym $pcaYMargin -xMin $pcaXMin -xMax $pcaXMax -yMin $pcaYMin -yMax $pcaYMax"
       append cmnd " -thresh $confidPct -xsd $pcaXSigmaMult -ysd $pcaYSigmaMult -pow $pcaSigmaPow -zMode $zMode"

       if {$squareFlag} {append cmnd " -square"}

       if {[strcasecmp $allDir None]} {append cmnd " -allDir $allDir"}

       safeExec $cmnd

       if {![file exists $thisName1] || ![file exists $thisName2]} \
          {
           puts stderr "classDistance. Error Creating PCA Surfaces: $thisName1 $thisName2"
           exit 1
          }

       incr nextROIID
       set sID1 $nextROIID

       incr nextROIID
       set sID2 $nextROIID

       readROI -roi $sID1 -ndim 2 -in $thisName1 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
       readROI -roi $sID2 -ndim 2 -in $thisName2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

       set rInfo1 [getROIInfo -roi $sID1]
       set rInfo2 [getROIInfo -roi $sID2]

       if {$verbose && ![strcasecmp $surfDir surf]} \
          {
           set thisX1    [pnt2spec [lindex $rInfo1 1] CUR_XDIM 1    user]
           set thisXN    [pnt2spec [lindex $rInfo1 1] CUR_XDIM last user]
           set thisY1    [pnt2spec [lindex $rInfo1 1] CUR_YDIM 1    user]
           set thisYN    [pnt2spec [lindex $rInfo1 1] CUR_YDIM last user]

           puts -nonewline stderr "classDistance. X-Limits:"
           puts stderr [format " % 8.3f % 8.3f" $thisX1 $thisXN]

           puts -nonewline stderr "classDistance. Y-Limits:"
           puts stderr [format " % 8.3f % 8.3f" $thisY1 $thisYN]
          }

       set thisSize [getROISize $sID1]

       set sum1 [vSum [lindex $rInfo1 0] $thisSize]
       set sum2 [vSum [lindex $rInfo2 0] $thisSize]

       if {$sum1 != 0.0} {vsMult [lindex $rInfo1 0] [expr 1.0/$sum1] $thisSize}
       if {$sum2 != 0.0} {vsMult [lindex $rInfo2 0] [expr 1.0/$sum2] $thisSize}

       vvMin [lindex $rInfo1 0] [lindex $rInfo1 0] [lindex $rInfo2 0] $thisSize

       set distInfo($type) [expr 100.0*[vSum [lindex $rInfo1 0] $thisSize]]

       setParmStr [lindex $rInfo1 1] FDTITLE Overlap-$title(1)-$title(2) NULL_DIM

       set ovName $outDir/$surfDir/Ov.dat

       writeROI -roi $sID1 -out $outDir/$surfDir/Ov.dat
    
       if {![file exists $ovName]} \
          {
           puts stderr "classDistance. Error Creating PCA Overlap Surface: $ovName"
           exit 1
          }

       set thisMargin 0.5
       set type       overlapWide
       set mode       sum
      }
}

proc reportResults {} \
{
   global axisArg boundFlag outDir title plotLabel plotColor pageColor classList classSize pcaVarX pcaVarY 
   global pcaDX pcaDY confidPct distInfo briefFlag verbose
   global argv0 ARGV_ORIG

   set thisName1 $outDir/surf/$title(1).dat
   set thisName2 $outDir/surf/$title(2).dat

   set    tStr   "${confidPct}% Confidence Limit Shown.   "
   append tStr " Overlap: [format %.2f $distInfo(overlapWide)]%"
   append tStr " Min Z Distance: [format %.2f $distInfo(nearest)]"
   append tStr " P: [format %.2f $distInfo(p,nearest)]%"

   if {$boundFlag} \
      {
       set thisHi [expr 100.0 - $confidPct]
      } \
   else \
      {
       set thisHi 1000.0 
      }

   set nChar [expr 3 + [getMax "[string length $title(1)] [string length $title(2)]"]]
   set kw    [expr 0.12*$nChar]
   set kx1   [expr 7.0 - 0.12*($nChar + 2)]
  
   set    cmnd "plotSeries.tcl -out $outDir/pcaDist.ps -pageColor $pageColor -cx1 0.25 -cxn 7.0 -cy1 0.5 -cyn 7.0 -charX 0.08 -charY 0.25 -$axisArg -pcaDX $pcaDX -pcaDY $pcaDY"
   append cmnd " -kw $kw -kx1 $kx1 -ky1 0.7 -title1X 0.15 -title1Y 0.38 -title1 $tStr"

   if {[llength $classList(V)]} {append cmnd " -keyExtra $plotColor(V) $title(V)"}

   if {[strcasecmp $plotLabel None]} \
      {
       if {![strcasecmp $plotLabel Auto]} {set plotLabel [file normalize [lindex $classList(1) 0]]}
       append cmnd " -title2X 0.15 -title2Y 8.0 -title2 $plotLabel -title2Color $plotColor(T)"
      }
         
   append cmnd " -in $thisName1 $thisName2 -hi $thisHi -plev 1 -colors $plotColor(1) $plotColor(2) -pageColor $pageColor -lwPlus 2 -pcaVarX $pcaVarX -pcaVarY $pcaVarY"
   append cmnd " -bAxis bottom,units=user,lab=$pcaVarX -rAxis right,units=user,lab=$pcaVarY,labhoriz=1 -pca $outDir/pcaCoordsAdj.tab"

   safeExec $cmnd

   set    thisInfo "class1 $title(1) class2 $title(2) overlapPct $distInfo(overlapWide)"
   append thisInfo " nearest $distInfo(nearest) pnearestPct $distInfo(p,nearest) center $distInfo(center) pcenterPct $distInfo(p,center)"

   set error [catch {set outUnit [open $outDir/pcaDist.tab w]}]

   if {$error} \
      {
       puts stderr "classDistance. Error Opening Output $outDir/pcaDist.tab"
       exit 1
      }

   puts $outUnit ""

   prettyPrintCmnd $outUnit "# " "[file tail $argv0] [lrange $ARGV_ORIG 3 end]"
   
   puts $outUnit ""
   puts $outUnit "DATA DIST_INFO $thisInfo"

   catch "close $outUnit"

   catch "exec cat $outDir/pcaCoordsAdj.tab >> $outDir/pcaDist.tab"

   set n1 [string length $title(1)]
   set n2 [string length $title(2)]
   set n3 [string length $title(V)]

   if {$n2 > $n1} {set n1 $n2}
   if {$n3 > $n1} {set n1 $n3}

   if {$verbose} \
      {
       puts stderr "classDistance. Log File: $outDir/pcaDist.log"
       puts stderr "classDistance. Plot:     $outDir/pcaDist.pdf"
       puts stderr "classDistance. Results:  $outDir/pcaDist.tab"
       puts stderr ""
       puts stderr [format " Class 1:     %-${n1}s %-2d Spectra" $title(1) $classSize(1)]
       puts stderr [format " Class 2:     %-${n1}s %-2d Spectra" $title(2) $classSize(2)]

       if {[llength $classList(V)]} \
          {
           puts stderr [format " Class V:     %-${n1}s %-2d Spectra" $title(V) $classSize(V)]
          } \
       else \
          {
           puts stderr [format " Class V:     %-${n1}s" None]
          }
       
       puts stderr ""
       puts stderr [format " Components:  %s %s" $pcaVarX $pcaVarY]
       puts stderr ""
       puts stderr " Overlap Between Class Distributions: [format %.2f $distInfo(overlapWide)]%"
       puts stderr [format " Smallest Distance Between Classes:   %-8s %-8s" [format Z=%.2f $distInfo(nearest)] [format P=%.2f%% $distInfo(p,nearest)]]
       puts stderr [format " Distance Between Class Centers:      %-8s %-8s" [format Z=%.2f $distInfo(center)]  [format P=%.2f%% $distInfo(p,center)]]
       puts stderr ""
      }

   if {$briefFlag} {puts $thisInfo}
}

set coordInfo    ""
set allList      ""
set nextROIID    0

set dimCount     0
set xSize        0
set ySize        0
set n            0

foreach type {center nearest overlap overlapWide} {set distInfo($type) 0.0}

set classSize(1) [llength $classList(1)]
set classSize(2) [llength $classList(2)]
set classSize(V) [llength $classList(V)]

findOutDir
findSizes $refName

foreach classID {1 2 V} {readClassList $classList($classID) $title($classID) $classID} 

saveRegions
performPCA
computeCoordDist
computeOverlap
reportResults

exit 0
