#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Depends on definitions in NMRPipe fdatap.h

if {[flagLoc $ARGV -help]} \
   {
    puts "Compare FDATA Header Values in Two NMRPipe Format Files:"
    puts " -in1  inName1 \[None\]   Input File 1."
    puts " -in2  inName2 \[None\]   Input File 2."
    puts " -tol  tol     \[0.001\]  Comparison Tolerance for Float Data."
    exit 1
   }

set inName1 None
set inName2 None
set tol     0.001

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -tol tol

#
#------------------------------------------------------------------------------

if {![strcasecmp $inName1 None] || ![strcasecmp $inName2 None]} \
   {
    puts stderr "No Input Files Specified."
    exit 0
   }

set thisName1 [format $inName1 1 1]
set thisName2 [format $inName2 1 1]

if {![file exists $thisName1]} \
   {
    puts stderr "Error Finding NMRPipe Format Input $inName1"
    exit 1
   }

if {![file exists $thisName2]} \
   {
    puts stderr "Error Finding NMRPipe Format Input $inName2"
    exit 1
   }

set error [rdFDATA $thisName1 fdata1]

if {$error} \
   {
    puts stderr "Error Reading Header from $inName1"
    exit 1
   }

set error [rdFDATA $thisName2 fdata2]

if {$error} \
   {
    puts stderr "Error Reading Header from $inName2"
    exit 1
   }

set pInfo \
  {
   {FDMAGIC        float}
   {FDFLTFORMAT    float}
   {FDFLTORDER     float}
   {FDSIZE         int}
   {FDREALSIZE     int}
   {FDSPECNUM      int}
   {FDQUADFLAG     int}
   {FD2DPHASE      int}
   {FDTRANSPOSED   int}
   {FDDIMCOUNT     int}
   {FDNUSDIM       int}
   {FDDIMORDER     int}
   {FDDIMORDER1    int}
   {FDDIMORDER2    int}
   {FDDIMORDER3    int}
   {FDDIMORDER4    int}
   {FDPIPEFLAG     int}
   {FDCUBEFLAG     int}
   {FDPIPECOUNT    int}
   {FDSLICECOUNT   int}
   {FDSLICECOUNT0  int}
   {FDSLICECOUNT1  int}
   {FDFILECOUNT    int}
   {FDFIRSTPLANE   int}
   {FDLASTPLANE    int}
   {FDPARTITION    int}
   {FDPLANELOC     int}
   {FDTHREADCOUNT  int}
   {FDTHREADID     int}
   {FDDMXVAL       float}
   {FDDMXFLAG      int}
   {FDDELTATR      float}
   {FDMAX          float}
   {FDMIN          float}
   {FDSCALEFLAG    int}
   {FDDISPMAX      float}
   {FDDISPMIN      float}
   {FDPTHRESH      float}
   {FDNTHRESH      float}
   {FDUSER1        float}
   {FDUSER2        float}
   {FDUSER3        float}
   {FDUSER4        float}
   {FDUSER5        float}
   {FDUSER6        float}
   {FDLASTBLOCK    int}
   {FDCONTBLOCK    int}
   {FDBASEBLOCK    int}
   {FDPEAKBLOCK    int}
   {FDBMAPBLOCK    int}
   {FDHISTBLOCK    int}
   {FD1DBLOCK      int}
   {FDMONTH        int}
   {FDDAY          int}
   {FDYEAR         int}
   {FDHOURS        int}
   {FDMINS         int}
   {FDSECS         int}
   {FDMCFLAG       int}
   {FDNOISE        float}
   {FDRANK         int}
   {FDSCORE        float}
   {FDSCANS        int}
   {FDTEMPERATURE  float}
   {FDPRESSURE     float}
   {FD2DVIRGIN     int}
   {FDTAU          float}
   {FDDOMINFO      int}
   {FDMETHINFO     int}
   {FDSRCNAME      text}
   {FDUSERNAME     text}
   {FDOPERNAME     text}
   {FDTITLE        text}
   {FDCOMMENT      text}
   {FDF2LABEL      text}
   {FDF2APOD       int}
   {FDF2SW         float}
   {FDF2OBS        float}
   {FDF2OBSMID     float}
   {FDF2ORIG       float}
   {FDF2UNITS      int}
   {FDF2QUADFLAG   int}
   {FDF2FTFLAG     int}
   {FDF2AQSIGN     int}
   {FDF2CAR        float}
   {FDF2CENTER     int}
   {FDF2OFFPPM     float}
   {FDF2P0         float}
   {FDF2P1         float}
   {FDF2APODCODE   int}
   {FDF2APODQ1     float}
   {FDF2APODQ2     float}
   {FDF2APODQ3     float}
   {FDF2APODDF     float}
   {FDF2LB         float}
   {FDF2GB         float}
   {FDF2GOFF       float}
   {FDF2C1         float}
   {FDF2ZF         int}
   {FDF2X1         int}
   {FDF2XN         int}
   {FDF2FTSIZE     int}
   {FDF2TDSIZE     int}
   {FDF1LABEL      text}
   {FDF1APOD       int}
   {FDF1SW         float}
   {FDF1OBS        float}
   {FDF1OBSMID     float}
   {FDF1ORIG       float}
   {FDF1UNITS      int}
   {FDF1FTFLAG     int}
   {FDF1AQSIGN     int}
   {FDF1QUADFLAG   int}
   {FDF1CAR        float}
   {FDF1CENTER     int}
   {FDF1OFFPPM     float}
   {FDF1P0         float}
   {FDF1P1         float}
   {FDF1APODCODE   int}
   {FDF1APODQ1     float}
   {FDF1APODQ2     float}
   {FDF1APODQ3     float}
   {FDF1LB         float}
   {FDF1GB         float}
   {FDF1GOFF       float}
   {FDF1C1         float}
   {FDF1ZF         int}
   {FDF1X1         int}
   {FDF1XN         int}
   {FDF1FTSIZE     int}
   {FDF1TDSIZE     int}
   {FDF3LABEL      text}
   {FDF3APOD       int}
   {FDF3OBS        float}
   {FDF3OBSMID    float}
   {FDF3SW         float}
   {FDF3ORIG       float}
   {FDF3FTFLAG     int}
   {FDF3AQSIGN     int}
   {FDF3SIZE       int}
   {FDF3QUADFLAG   int}
   {FDF3UNITS      int}
   {FDF3P0         float}
   {FDF3P1         float}
   {FDF3CAR        float}
   {FDF3CENTER     int}
   {FDF3OFFPPM     float}
   {FDF3APODCODE   int}
   {FDF3APODQ1     float}
   {FDF3APODQ2     float}
   {FDF3APODQ3     float}
   {FDF3LB         float}
   {FDF3GB         float}
   {FDF3GOFF       float}
   {FDF3C1         float}
   {FDF3ZF         int}
   {FDF3X1         int}
   {FDF3XN         int}
   {FDF3FTSIZE     int}
   {FDF3TDSIZE     int}
   {FDF4LABEL      text}
   {FDF4APOD       int}
   {FDF4OBS        float}
   {FDF4OBSMID     float}
   {FDF4SW         float}
   {FDF4ORIG       float}
   {FDF4FTFLAG     int}
   {FDF4AQSIGN     int}
   {FDF4SIZE       int}
   {FDF4QUADFLAG   int}
   {FDF4UNITS      int}
   {FDF4P0         float}
   {FDF4P1         float}
   {FDF4CAR        float}
   {FDF4CENTER     int}
   {FDF4OFFPPM     float}
   {FDF4APODCODE   int}
   {FDF4APODQ1     float}
   {FDF4APODQ2     float}
   {FDF4APODQ3     float}
   {FDF4LB         float}
   {FDF4GB         float}
   {FDF4GOFF       float}
   {FDF4C1         float}
   {FDF4ZF         int}
   {FDF4X1         int}
   {FDF4XN         int}
   {FDF4FTSIZE     int}
   {FDF4TDSIZE     int}
   {NDSIZE         int}
   {NDAPOD         int}
   {NDSW           float}
   {NDORIG         float}
   {NDOBS          float}
   {NDOBSMID       float}
   {NDFTFLAG       int}
   {NDQUADFLAG     int}
   {NDUNITS        int}
   {NDLABEL        text}
   {NDP0           float}
   {NDP1           float}
   {NDCAR          float}
   {NDCENTER       float}
   {NDAQSIGN       int}
   {NDAPODCODE     int}
   {NDAPODQ1       float}
   {NDAPODQ2       float}
   {NDAPODQ3       float}
   {NDLB           float}
   {NDGB           float}
   {NDGOFF         float}
   {NDC1           float}
   {NDZF           int}
   {NDX1           int}
   {NDXN           int}
   {NDOFFPPM       float}
   {NDFTSIZE       int}
   {NDTDSIZE       int}
   {NDACQMETHOD    int}
   {NDFTDOMAIN     int}
  }

set fmt  "%-16s %16s %16s"
set fmt2 "%-16s %16s %16s %16.4f"

puts [format "%-16s %16s %16s %16s" Parameter: ${inName1}: ${inName2}: Difference:]

foreach pItem $pInfo \
   {
    set pName [lindex $pItem 0]
    set pType [lindex $pItem 1]

    if {[string first ND $pName] == 0} continue

    switch $pType \
       {
        int   {
               set val1 [getParmI $fdata1 $pName NULL_DIM]
               set val2 [getParmI $fdata2 $pName NULL_DIM]

               if {$val1 != $val2} {puts [format $fmt $pName $val1 $val2]}
              }

        float {
               set val1 [getParm $fdata1 $pName NULL_DIM]
               set val2 [getParm $fdata2 $pName NULL_DIM]
               set d    [expr $val1 - $val2]

               if {abs($d) > $tol} {puts [format $fmt2 $pName $val1 $val2 $d]}
              }
        text {
               set val1 [getParmStr $fdata1 $pName NULL_DIM]
               set val2 [getParmStr $fdata2 $pName NULL_DIM]

               if {[strcmp $val1 $val2]} {puts [format $fmt $pName $val1 $val2]}
              }
       }
   }

exit 0
