#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Simple Viewer for Coherent 2D Spectroscopy Data."
    puts " -in   inName   \[test.dat\]  Input 2D Data (NMRPipe-Format)."
    puts " -tab  tabName  \[test.tab\]  Input 2D Peak Table (NMRPipe-Format)."
    puts " -plev nPos     \[16\]        Positive Contour Count."
    puts " -nlev nNeg     \[16\]        Negative Contour Count."
    puts " -hi   hi       \[Auto\]      First Contour Height."
    puts " -mult mult     \[4.0\]       Multiplier for Auto Contour Mode."
    puts "Drawing Area:"
    puts " -xSize xPix    \[16000\]     Drawing Area X-Axis Size, Pixels."
    puts " -ySize yPix    \[16000\]     Drawing Area Y-Axis Size, Pixels."
   }

set inName  test.dat 
set tabName test.tab
set plev    16
set nlev    16
set hi      Auto
set mult    4.0
set xSize   16000
set ySize   16000

getArgD $ARGV -in    inName
getArgD $ARGV -tab   tabName
getArgD $ARGV -plev  plev 
getArgD $ARGV -nlev  nlev 
getArgD $ARGV -hi    hi
getArgD $ARGV -mult  mult

getArgD $ARGV -xSize xSize
getArgD $ARGV -ySize ySize

#
#--------------------------------------------------------------------

proc showPeak { showFlag tID bID str } \
{
   if {$showFlag} \
      {
       wm title . $str
       .c itemconfigure $tID -state normal
       .c itemconfigure $bID -fill white
       update
      } \
   else \
      {
       wm title . cohView.tcl
       .c itemconfigure $tID -state hidden
       .c itemconfigure $bID -fill green 
       update
      }
}

if {![file exists $inName]} \
   {
    puts "Error Finding Input Spectrum $inName"
    exit 1
   }

if {![strcasecmp $hi Auto]} \
   {
    set noise  [estSpecNoise $inName]
    set hi     [expr $noise*$mult]
   }

set wx  [winfo screenwidth  .]
set wy  [winfo screenheight .]

set xAdj  100
set yAdj  120

set sxSize [expr $wx - $xAdj] 
set sySize [expr $wy - $yAdj]

set cx1 10
set cxn [expr $xSize - $xAdj]
set cy1 10
set cyn [expr $ySize - $yAdj]

wm geom . +2+28

frame .top

scrollbar .sh -orient horiz -command ".c xview"
scrollbar .sv -orient vert  -command ".c yview"

canvas    .c -width $sxSize -height $sySize -border 0  \
             -scrollregion "0 0 $xSize $ySize"     \
             -xscrollcommand ".sh set" \
             -yscrollcommand ".sv set"

button    .b -text Quit -command exit
pack      .c  -side left -in .top -expand 1 -fill both 
pack      .sv -side left -in .top -expand 1 -fill y
pack      .top .sh .b -side top   -expand 1 -fill x

.c create pixmap 0 0 -width $xSize -height $ySize -win 1 

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1

readROI -ndim 2 -roi 1 -in $inName -x X_AXIS -y Y_AXIS  -verb

newScreen

set rInfo [getROIInfo -roi 1]
set fdata [lindex $rInfo 1]

set xSize [getParmI $fdata NDSIZE CUR_XDIM]
set ySize [getParmI $fdata NDSIZE CUR_YDIM]

set xOrig [getParm  $fdata NDORIG CUR_XDIM]
set yOrig [getParm  $fdata NDORIG CUR_YDIM]

set xSW   [getParm  $fdata NDSW   CUR_XDIM]
set ySW   [getParm  $fdata NDSW   CUR_YDIM]

set x1 $xOrig
set xn [expr $xOrig + $xSW]

set y1 $yOrig
set yn [expr $yOrig + $ySW]

wm title . "Drawing Spectrum ..."
update
 
drawROI -roi 1 -cell 1 -noxGrid -noyGrid -colorA black -colorS None \
    -bg black -color1 multi -color2 multi -plev $plev -nlev $nlev   \
    -axis Bottom,lab=X_pts,orig=$x1,end=$xn Right,lab=Y_nm,orig=$y1,end=$yn \
    -hi $hi

wm title . "Creating Peak Labels ..."
update

set xPW [expr $cxn - $cx1]
set yPW [expr $cyn - $cy1]

if {[file exists $tabName]} \
   {
    set t     [gdbRead table -in $tabName]
    set entry [gdbFirst $t]
   
    while {$entry} \
       {
        set pkID [gdbGet $entry INDEX]

        set xPts [gdbGet $entry X_AXIS]
        set yPts [gdbGet $entry Y_AXIS]

        set xPix [expr $cx1 + $xPW*($xPts - 1)/($xSize - 1)]
        set yPix [expr $cy1 + $yPW*($ySize - $yPts)/($ySize - 1)]

        set xNM  [expr $x1 + $xSW*($xPts - 1)/($xSize - 1)]
        set yNM  [expr $y1 + $ySW*($yPts - 1)/($ySize - 1)]

        set t [format "%.4f,%.4f" $xNM $yNM]
 
        set tID \
           [.c create text $xPix $yPix \
               -anchor sw -fill green -text $t -tags pklab -state hidden]

        set bID \
           [.c create rectangle \
               [expr $xPix - 3] [expr $yPix - 3] \
               [expr $xPix + 3] [expr $yPix + 3] \
               -outline green -fill green -tags pkbox -state normal]

        .c bind $bID <Any-Enter> "showPeak 1 $tID $bID $t"
        .c bind $bID <Any-Leave> "showPeak 0 $tID $bID $t"

        set entry [gdbNext $entry]
       }
   }

wm title . "Ready."
update

