#
# Procedure to build a color choice window:

proc getColorChoice args \
{
   global env glbColorChoiceInfo

   set thisArgv [concat getColorChoice $args]

   if {![info exists glbColorChoiceInfo(id)]} \
      {
       set glbColorChoiceInfo(id) 0
      }

   incr glbColorChoiceInfo(id)

   set id  $glbColorChoiceInfo(id)
   set obj colorChoice$id

   set w  .glbColorChoice$id

   toplevel $w

   wm title $w Colors

   set glbColorChoiceInfo($obj,window)     $w
   set glbColorChoiceInfo($obj,color)      blue 
   set glbColorChoiceInfo($obj,origColor)  blue 
   set glbColorChoiceInfo($obj,id)         $id
   set glbColorChoiceInfo($obj,dbName)     None
   set glbColorChoiceInfo($obj,cmnd)       glbColorChoiceProc
   set glbColorChoiceInfo($obj,title)      "Color Choice"
   set glbColorChoiceInfo($obj,applyCount) 0

   if {[info exists env(NMRTXT)]} \
      {
       set glbColorChoiceInfo($obj,dbName) $env(NMRTXT)/rgb.txt
      } 

    getArgD $thisArgv -color   glbColorChoiceInfo($obj,color)
    getArgD $thisArgv -db      glbColorChoiceInfo($obj,dbName)
    getArgD $thisArgv -title   glbColorChoiceInfo($obj,title)
    getArgD $thisArgv -command glbColorChoiceInfo($obj,cmnd)

    set glbColorChoiceInfo($obj,origColor) $glbColorChoiceInfo($obj,color)

#
# Regular and Extended Color Choices:

   set cList {   
              black  red   orange  yellow    green   blue   \
              violet cyan  magenta lightGray dimGray gray30 \
              gray20 white
             }

   set wList {
              white  black black   black     white   white
              white  black white   black     white   white
              black  black
             }

   set cList ""

   if {[file exists $glbColorChoiceInfo($obj,dbName)]} \
      {
       set error [catch {set inUnit [open $glbColorChoiceInfo($obj,dbName)]}]

       if {$error} \
          {
           showError getColorChoice \
            "Error Opening Color Database File $glbColorChoiceInfo($obj,dbName)"
          } \
       else \
          {
           set xList ""
           set wList ""

           while {[gets $inUnit line] >= 0} \
              {
               set cInfo [string trim $line]
               set key   [string index $cInfo 0]

               if {![strcmp $key "#"]} continue
               if {![string length $cInfo]} continue
               if {[llength $cInfo] < 4} continue

               set ir [lindex $cInfo 0]
               set ig [lindex $cInfo 1]
               set ib [lindex $cInfo 2]

               if {$ig > $ir} {set ir $ig}
               if {$ib > $ir} {set ir $ib}
 
               set iv [expr $ir/256.0]
 
               lappend xList [lindex $cInfo 3]

               if {$iv > 0.2} \
                  {
                   lappend wList black
                  } \
               else \
                  {
                   lappend wList white 
                  }
              }
          }
      }

#
# Build interface:

   label  $w.l -text $glbColorChoiceInfo($obj,title)

   canvas $w.c -height 80 -width 130
   entry  $w.e -textvariable glbColorChoiceInfo($obj,color)

   pack $w.l      -in $w -side top
   pack $w.e $w.c -in $w -side top -expand 1 -fill x

   frame $w.lx

   scrollbar $w.slx -orient vert -command "$w.lbx yview"

   listbox   $w.lbx -height 10 -width 22 -selectmode single \
                    -yscrollcommand "$w.slx set"

   pack $w.lbx $w.slx -in $w.lx -side left -expand 1 -fill y

   pack $w.lx -in $w -side top -expand 1 -fill both

   frame   $w.cmnd
   button  $w.apl  -text Apply  -command "glbColorApplyChoice  $obj"
   button  $w.sel  -text Select -command "glbColorAcceptChoice $obj"
   button  $w.quit -text Cancel -command "glbColorCancelChoice $obj"

   pack $w.apl $w.sel $w.quit -in $w.cmnd -side left -expand 1 -fill x
   pack $w.cmnd -in $w -side top -expand 1 -fill x

   update

   set xw   [winfo width  $w.c]
   set yw   [winfo height $w.c]

   set xMid [expr $xw/2]
   set yMid [expr $yw/2]

   set lx1 10
   set ly1 10
   set lxn [expr $xMid - 10]
   set lyn [expr $yw - 10]

   set rx1 [expr $xMid + 10] 
   set ry1 10
   set rxn [expr $xw - 10]
   set ryn [expr $yw - 10]

   set glbColorChoiceInfo($obj,box1) \
    [$w.c create rectangle $lx1 $ly1 $lxn $lyn \
          -outline white -fill blue -width 20]

   set glbColorChoiceInfo($obj,box2) \
    [$w.c create rect $rx1 $ry1 $rxn $ryn \
          -outline black -fill blue -width 20]

   $w.lbx delete 0 end
 
   set n [llength $xList] 

   set loc 0
   set j   0

   for {set i 0} {$i < $n} {incr i} \
      {
       set fg [lindex $wList $i]
       set x  [lindex $xList $i]
       set xc $x

       if {![strcasecmp $xc None]}  {set xc black}
       if {![strcasecmp $xc Multi]} {set xc black}

       set error \
        [catch "$w.c itemconfigure $glbColorChoiceInfo($obj,box1) -fill $xc"]

       if {$error} continue

       lappend cList $x

       if {![strcasecmp $x $glbColorChoiceInfo($obj,color)]} {set loc $j}

       $w.lbx insert end $x

       $w.lbx itemconfigure $j -foreground $fg -selectforeground gray10 \
                               -background $xc -selectbackground $xc
       incr j
      }

    set glbColorChoiceInfo($obj,colorList) $cList

    glbColorSelectChoice2 $w $obj $loc

    bind $w.lbx <ButtonRelease-1> "+ glbColorSelectChoice $obj %y"

    glbColorCopyColorChoice $obj

    return $w
}

proc glbColorApplyChoice { obj } \
{
   global glbColorChoiceInfo

   if {[strcasecmp $glbColorChoiceInfo($obj,cmnd) None]} \
      {
       catch "eval $glbColorChoiceInfo($obj,cmnd) $glbColorChoiceInfo($obj,color)"
      }  

   glbColorCopyColorChoice $obj

   incr glbColorChoiceInfo($obj,applyCount)
}

proc glbColorCopyColorChoice { obj } \
{
   global glbColorChoiceInfo

   clipboard clear
   clipboard append -- $glbColorChoiceInfo($obj,color)
}

proc glbColorAcceptChoice { obj } \
{
   global glbColorChoiceInfo

   glbColorApplyChoice $obj
   catch "destroy $glbColorChoiceInfo($obj,window)"
}

proc glbColorCancelChoice { obj } \
{
   global glbColorChoiceInfo

   if {$glbColorChoiceInfo($obj,applyCount)} \
      {
       set glbColorChoiceInfo($obj,color) $glbColorChoiceInfo($obj,origColor)
       glbColorApplyChoice $obj
      }

   catch "destroy $glbColorChoiceInfo($obj,window)" 
}

proc glbColorSelectChoice { obj yPix } \
{
   global glbColorChoiceInfo

   set w $glbColorChoiceInfo($obj,window)
   set j [$w.lbx curselection]

   glbColorSelectChoice2 $w $obj $j
}

proc glbColorSelectChoice2 { w obj j } \
{
   global glbColorChoiceInfo

   set x [lindex $glbColorChoiceInfo($obj,colorList) $j]
   set xc $x

   if {![strcasecmp $xc None]}  {set xc ""}
   if {![strcasecmp $xc Multi]} {set xc ""}

   set glbColorChoiceInfo($obj,color) $x
   
   $w.c itemconfigure $glbColorChoiceInfo($obj,box1) -fill $xc
   $w.c itemconfigure $glbColorChoiceInfo($obj,box2) -fill $xc

   $w.lbx see $j
}

proc glbColorChoiceProc args \
{
   puts [string trim $args "{}"]
}

