#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Create Library Table for Pool Screening Compounds:"
   echo " -dir dirList [.]    List of Compound Library Plate Directories."
   echo " -hdr                Print Header for Output (Default)." 
   echo " -nohdr              No Header for Output." 
   exit 0
endif

set hdrFlag = 1

if (`flagLoc $argv -hdr`) then
   set hdrFlag = 1
endif

if (`flagLoc $argv -nohdr`) then
   set hdrFlag = 0
endif

if ($hdrFlag != 0) then
   echo ""
   echo "#"
   echo "# Pool Screen Compound Library Compiled on `date`"
   echo ""
   echo "VARS   TITLE CMPD_ID DIR_NAME" 
   echo "FORMAT %-32s %-12s   %s"
   echo ""
endif

set dirList = (`getListArgD $argv -dir -- .`)
set origDir = (`pwd`)

set nGood   = 0
set nBad    = 0

foreach dirName ($dirList)
   cd $origDir

   if (!(-d $dirName)) then
      continue
   endif

   cd $dirName

   set thisDir = (`pwd`)
   set fCount  = (`/bin/ls | wc -w`)   

   if ($fCount == 0) then
      continue
   endif

   nmrPrintf -stderr "Compiling Data in $thisDir ...\n"

   set fList = (`/bin/ls | sort -n`)

   foreach fName ($fList) 
      if (!(-d $fName)) then
         continue
      endif

      if (!(-e $fName/test.ft1) || !(-e $fName/title)) then
         continue
      endif

      set tTitle = (`cat $fName/title | tr ' ' '_'`)
      set sTitle = (`getParm -in $fName/test.ft1 -parm FDTITLE -txt -fmt %s | tr ' ' '_'`)

      set tInfo = (`string split $tTitle "-"` None None None None None)
      set tType = $tInfo[4]
      set cID   = $tInfo[5]

      if (`string csame $tType lib`) then
         if (`string csame $tTitle $sTitle`) then
            nmrPrintf " %-32s %-12s %s\n" $tTitle $cID $thisDir/$fName
            @ nGood++
         else
            nmrPrintf -stderr " Warning: Title Mismatch in $thisDir/$fName ...\n"
            nmrPrintf -stderr "    title file: %-32s\n" $tTitle
            nmrPrintf -stderr "    test.ft1:   %-32s\n" $sTitle
            @ nBad++            
         endif
      endif
   end
end

nmrPrintf -stderr "Done Compiling Data. Good Entries: $nGood Bad Entries: $nBad\n"

exit $nBad 
