#!/bin/csh

set defValList = \
 (RMSD_BSDIF_PCT NOISE_BLANK NOISE_SCREEN NOISE_C%s \
  MAX_BSDIF RMSD_SDDIF_PCT RMSD_BDDIF_PCT SCOEF_DPCT_CN_HI \
  RMSD_C%sDIF_DMASK_EPCT RMSD_C%sDIF_CMASK_EPCT SCOEF_DPCT_C%s)

if (`flagLoc $argv -help`) then
   echo "Summarize Scores from Processed and Scored Blanks and Screen racks."
   echo " -blankDir  bRackDirs   Blanks_Rack_1_Pools"
   echo " -screenDir sRackDirs   Screen_Rack_1_Pools"
   echo " -rackID    rackIDs     1 ..."
   echo " -val       valList     PNAME Values to Extract from Score Table."
   echo " -prefix    prefixList  ext"
   echo " -verb                  Verbose Mode ON."
   echo " -noverb                Verbose Mode OFF (Default)."
   echo "Default PNAME Values:"
   echo " $defValList"
   exit 0
endif

set vFlag = 0
set eFlag = 1

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

set blankDirList  = (`getListArgD $argv -blankDir  -- Blanks_Rack_1_Pools`)
set screenDirList = (`getListArgD $argv -screenDir -- Screen_Rack_1_Pools`)
set rackIDList    = (`getListArgD $argv -rackID    -- None`)
set prefixList    = (`getListArgD $argv -prefix    -- ext`)
set valList       = (`getListArgD $argv -val       -- $defValList`)

if (`string csame $rackIDList[1] None`) then
   set rackIDList = ""
   set n = $#screenDirList
   set i = 0

   while ($i < $n)
      @ i++
      set rackIDList = ($rackIDList $i)
   end
endif

#
#------------------------------------------------------------------------------

set origDir = (`pwd`)
set nGood   = 0
set nBad    = 0
set nRack   = 0

set fullBlankDirList  = ""
set fullScreenDirList = ""

if ($#blankDirList != $#screenDirList) then
   nmrPrintf -stderr "SummarizePoolScores Error: blanks and screen rack mismatch.\n"
   exit 1
endif

if ($#rackIDList != $#screenDirList) then
   echo "SummarizePoolScores Error: screen and rack ID mismatch."
   exit 1
endif

foreach blankDir ($blankDirList)
   cd $origDir

   if (!(-d $blankDir)) then
      nmrPrintf -stderr  "SummarizePoolScores Error Finding blanks dir $blankDir\n"
      exit 1
   endif

   cd $blankDir

   set fullBlankDirList = ($fullBlankDirList `pwd`)
end

foreach screenDir ($screenDirList)
   cd $origDir

   if (!(-d $blankDir)) then
      nmrPrintf -stderr "SummarizePoolScores Error Finding screen dir $screenDir\n"
      exit 1
   endif

   cd $screenDir

   set fullScreenDirList = ($fullScreenDirList `pwd`)
end

set fmtList     = ""
set varNameList = ""

foreach val ($valList)
   set fmtList     = ($fmtList %12.4e)
   set varNameList = ($varNameList `nmrPrintf $val N`)
end

echo ""
echo "#"
echo "# ScorePool (c) 2012  F. Delaglio www.nmrscience.com"
echo ""

echo ""
echo "VARS   POOL RACK WELL COMPOUND REGION TITLE $varNameList BLANK_NAME SCREEN_NAME DIF_NAME CN_NAME STATUS" 
echo "FORMAT %6s  %2s  %2s  %6s      %-6s   %-32s $fmtList     %s         %s          %s       %s      %s"
echo ""

set poolID = 0
set i      = 1
set ii     = 0

foreach blankDir ($fullBlankDirList)
   cd $origDir
   @ ii++

   set screenDir = $fullScreenDirList[$i]

   if (!(-d $blankDir) || !(-d $screenDir)) then
      continue
   endif

   cd $screenDir
  
   set parentDir = (`pwd`)
   set fCount    = (`/bin/ls | wc -w`)

   if ($fCount == 0) then
      continue
   endif

   set fList = (`/bin/ls | sort -n`)
   set nExp  = 0

   foreach fName ($fList)
      if (!(-d $fName)) then
         continue
      endif

      if (!(-e $fName/test.ft1) || !(-e $fName/title) || !(-e $fName/pool.txt) || !(-e $fName/pool_lib.txt)) then
         continue
      endif

      if ($nExp == 0) then
         nmrPrintf -stderr "\n"
         nmrPrintf -stderr "Blanks Rack: $fullBlankDirList[$i]\n"
         nmrPrintf -stderr "Screen Rack: $fullScreenDirList[$i]\n"
         @ nRack++
      endif

      @ poolID++
      @ nExp++

      if (!(-d $blankDir/$fName)) then
         nmrPrintf -stderr "SummarizePoolScores Error Finding blank data $blankDir/$fName for screen data $parentDir/$fName\n"
         @ nBad++
         continue
      endif

      foreach prefix ($prefixList)
         if (!(-e $fName/scorePool/${prefix}_score.tab)) then
            nmrPrintf -stderr "SummarizePoolScores Error Finding score table $parentDir/$fName/scorePool/${prefix}_score.tab\n"
            @ nBad++
            continue
         endif

         set j = 1

         foreach cID (`cat $fName/pool.txt`)
            set thisValList = ""

            foreach val ($valList)
               set thisValList = ($thisValList `nmrPrintf $val $j`)
            end

            echo "P$poolID $rackIDList[$ii] $fName C$j $prefix `getScoreInfo.tcl -in $fName/scorePool/${prefix}_score.tab -data C${j}_TITLE -dataLast BLANK_NAME SCREEN_NAME DIF_NAME C${j}_NAME -val $thisValList` None"

            echo ""

            @ j++
         end

         @ nGood++
      end

      cd $parentDir
   end

   nmrPrintf -stderr "Experiments: $nExp\n" 
   @ i++
end

nmrPrintf -stderr "\n"
nmrPrintf -stderr "SummarizePoolScores: Regions: $nGood Bad Entries: $nBad\n"

exit 0
