#
# Utilities for dipolar coupling calculation.
#------------------------------------------------------------------

proc getNthTensor { dcListInfo n } \
{
   set t     ""
   set first 0

# rms chi2 n da dr nEuler eulerSet1 ... saupe1 ... saupe5 nValid

   for {set i 0} {$i < $n} {incr i} \
      {
       set nEuler [lindex $dcListInfo [expr $first + 5]]
       set last   [expr $first + 3*$nEuler + 11]
       set t      [lrange $dcListInfo $first $last]
       set first  [expr $last + 1] 
      }

    return $t
}

proc getRandDC { di da dr n } \
{
   set dList ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set xJ  [expr 2.0*[rand -gauss]]
       set yJ  [expr 2.0*[rand -gauss]]
       set zJ  [expr 2.0*[rand -gauss]]

       set rIJ [expr sqrt($xJ*$xJ + $yJ*$yJ + $zJ*$zJ)]
       if {$rIJ < 0.01} continue

       set r   [expr 1.0/$rIJ]
       set xIJ [expr $xJ*$r]
       set yIJ [expr $yJ*$r]
       set zIJ [expr $zJ*$r]

       set d [expr abs($di)*( $da*(3.0*$zIJ*$zIJ - 1.0) + 1.5*$dr*($xIJ*$xIJ - $yIJ*$yIJ) )]

       lappend dList $d
      }

   return $dList
}

proc extractSaupe args \
{
   set dInfo [lindex $args 0]

   if {[llength $args] > 1} \
      {
       set sPtr [lindex $args 1]
       upvar $sPtr s
      }

   set nn [llength $dInfo]
   set k  0

   for {set j [expr $nn - 6]} {$j < $nn - 1} {incr j} \
      {
       set s($k) [lindex $dInfo $j]
       incr k
      }

   return "$s(0) $s(1) $s(2) $s(3) $s(4)"
}

proc getSaupe { pdbSrc dSrc s1 sN sPtr } \
{
   upvar $sPtr s

   set dInfo \
    [dynSimulate -dc $dSrc -src $pdbSrc -s1 $s1 -sN $sN -a1 $s1 -euler -svd]

   extractSaupe $dInfo s

   return $dInfo
}

proc getSaupeAngle { saPtr sbPtr } \
{
   upvar $saPtr sa
   upvar $sbPtr sb

   set aXX    [expr 0.5*($sa(0) - $sa(1))]
   set aYY    [expr 0.5*($sa(0) + $sa(1))]
   set aZZ    $sa(0)
   set aXY    $sa(2)
   set aXZ    $sa(3)
   set aYZ    $sa(4)

   set aNorm \
    [expr $aXX*$aXX + $aYY*$aYY + $aZZ*$aZZ + \
          2.0*$aXY*$aXY + 2.0*$aXZ*$aXZ + 2.0*$aYZ*$aYZ]

   set bXX    [expr 0.5*($sb(0) - $sb(1))]
   set bYY    [expr 0.5*($sb(0) + $sb(1))]
   set bZZ    $sb(0)
   set bXY    $sb(2)
   set bXZ    $sb(3)
   set bYZ    $sb(4)

   set bNorm \
    [expr $bXX*$bXX + $bYY*$bYY + $bZZ*$bZZ + \
          2.0*$bXY*$bXY + 2.0*$bXZ*$bXZ + 2.0*$bYZ*$bYZ]

   set abProd \
    [expr $aXX*$bXX + $aYY*$bYY + $aZZ*$bZZ + \
          2.0*$aXY*$bXY + 2.0*$aXZ*$bXZ + 2.0*$aYZ*$bYZ]

   set q [expr $aNorm*$bNorm]

   if {$q != 0.0} \
      {
       set abProd [expr $abProd/sqrt($q)]
      } \
   else \
      {
       set abProd 1.0
      }

   set a [expr 180.0*acos($abProd)/3.14159265]

   return $a
}

proc getDCRotNearest { rxRef ryRef rzRef rx0 ry0 rz0 } \
{
    set rotList ""

    set n 0

    set t      [dynTransform -reset -rx $rx0 -ry $ry0 -rz $rz0 -verb]
    set eInfo  [dynEuler [lrange $t 0 15]]
    set eInfo1 [lrange $eInfo 1 end]
    set n      [expr $n + [lindex $eInfo 0]]

    set t      [dynTransform -rx 180 -verb]
    set eInfo  [dynEuler [lrange $t 0 15]]
    set eInfo2 [lrange $eInfo 1 end]
    set n      [expr $n + [lindex $eInfo 0]]

    set t      [dynTransform -reset -rx $rx0 -ry $ry0 -rz $rz0 -verb]
    set t      [dynTransform -ry 180 -verb]
    set eInfo  [dynEuler [lrange $t 0 15]]
    set eInfo3 [lrange $eInfo 1 end]
    set n      [expr $n + [lindex $eInfo 0]]

    set t      [dynTransform -reset -rx $rx0 -ry $ry0 -rz $rz0 -verb]
    set t      [dynTransform -rz 180 -verb]
    set eInfo  [dynEuler [lrange $t 0 15]]
    set eInfo4 [lrange $eInfo 1 end]
    set n      [expr $n + [lindex $eInfo 0]]

    set rotList "$eInfo1 $eInfo2 $eInfo3 $eInfo4"
    
    set bestRot "$rx0 $ry0 $rz0"
    set bestD   1.0e+16

    set j 0

    for {set i 0} {$i < $n} {incr i} \
       {
        set rx [lindex $rotList $j]
        incr j

        set ry [lindex $rotList $j]
        incr j

        set rz [lindex $rotList $j]
        incr j

        set dx [expr $rxRef - $rx]
        set dy [expr $ryRef - $ry]
        set dz [expr $rzRef - $rz]

        set d  [expr sqrt($dx*$dx + $dy*$dy + $dz*$dz)]

        if {!$i || $d < $bestD} \
           {
            set bestRot "$rx $ry $rz"
            set bestD   $d
           } 
       }

   return $bestRot
}

