#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
puts "[file tail $argv0]: Delete Entries in a Table."
puts " -in         inName   \[test.tab\]     Input Table."
puts " -out        outName  \[selected.tab\] Output Table for Selected Entries."
puts " -del        delName  \[deleted.tab\]  Output Table for Deleted Entries."
puts " -verb                               Verbose Mode ON (Default)."
puts " -noverb                             Verbose Mode OFF."
puts "Delete By Random Fraction:"
puts " -keep       keepFrac \[0.9\]          Fraction of Entries to Keep."
puts " -iseed      randSeed \[54321\]        Random Seed (Zero for Auto)."
puts "Delete By Specified Row Number:"
puts " -n          rowList  \[None\]         Rows to Delete, First Row is 0."
puts "Delete By Random or Specified RESID:"
puts " -selByRes                           Select by Residue ID."
puts " -noselByRes                         Select by Entry (Default)."
puts " -resID      resList  \[None\]         Optional List of RESIDs to Delete."
puts "Notes:"
puts " 1. When deleting by fraction, entries are selected such that the"
puts "    result table will never be empty, unless keepFrac is zero."
puts " 2. Use -iseed 0 to set the random seed according to the clock."
puts " 3. Residue ID is Max of RESID RESID_I RESID_J etc." 
exit 1
}

set inName      test.tab 
set inTitle     test.tab 
set outName     selected.tab
set delName     deleted.tab
set iseed       54321
set keepFrac    0.9
set verbose     1
set selByRes    0

set userResList ""
set userRowList ""
 
getArgD $ARGV -in      inName
getArgD $ARGV -in      inTitle
getArgD $ARGV -inTitle inTitle

getArgD $ARGV -out   outName
getArgD $ARGV -del   delName
getArgD $ARGV -keep  keepFrac
getArgD $ARGV -iseed iseed

getListArgD $ARGV -resID userResList
getListArgD $ARGV -n     userRowList

if {[llength $userResList]} {set selByRes 1}

if {[flagLoc $ARGV -selByRes]}     {set selByRes 1}
if {[flagLoc $ARGV -noselByRes]}   {set selByRes 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {$keepFrac < 0.0} {set keepFrac 0.0}
if {$keepFrac > 1.0} {set keepFrac 1.0}

#
#-------------------------------------------------------------------------

proc splitByResID { keepListPtr delListPtr rvList resID } \
{
   upvar $keepListPtr keepList
   upvar $delListPtr  delList

   global verbose

   set newDelList  $delList
   set newKeepList ""
   set n 0

   foreach e $keepList \
      {
       set thisID [getMax [gdbGet $e $rvList]]

       if {$thisID == $resID} \
          {
           if {!$n && $verbose} {puts stderr "Deleting RESID $resID"}
           lappend newDelList $e
           incr n
          } \
       else \
          {
           lappend newKeepList $e
          }
      }

   if {[llength $newKeepList]} \
      {
       set keepList $newKeepList
       set delList  $newDelList
      }
}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input File $inTitle"
    exit 1
   }


set t     [gdbRead table -in $inName]
set vList [gdbGetInfo $t -parms varNames]

set nIn   [gdbGetInfo $t -parms entryCount]
set nOut  [expr int($keepFrac*$nIn)] 


if {$nOut == 0 && $keepFrac != 0.0} {set nOut 1} 
if {$nIn == 0} {set nOut 0}

set nDel [expr $nIn - $nOut]

set rvList ""
set minRes 0
set maxRes 0

if {$selByRes} \
   {
    foreach i {"" _I _J _K _L _M _N} \
       {
        set v RESID$i

        if {[lsearch -exact $vList $v] >= 0} {lappend rvList $v} 
       }

    if {![llength $rvList]} \
       {
        if {$verbose} \
           {
            puts stderr "Selection by RESID: No RESID Values Found in Input."
           }
       } \
    else \
       {
        set minList ""
        set maxList ""

        foreach v $rvList \
           {
            lappend minList [getTabMin $t $v]
            lappend maxList [getTabMax $t $v]
           }
    
        set minRes [getMin $minList]
        set maxRes [getMax $maxList]

        if {$verbose} \
           {
            puts stderr "Selection by RESID ($minRes to $maxRes)."
           }
       }
   }

gdbCreate variable -parent $t -varType float -name TMP_RAND_  -varFmt %.8f
gdbCreate variable -parent $t -varType int   -name TMP_INDEX_ -varFmt %d

srand $iseed

set eList [gdbGetInfo $t -parms entries]
set i 0

if {[llength $userResList] || [llength $userRowList]} \
   {
    foreach entry $eList \
       {
        gdbSet $entry TMP_RAND_  2.0 
        gdbSet $entry TMP_INDEX_ $i

        incr i
       }
   } \
else \
   {
    foreach entry $eList \
       {
        gdbSet $entry TMP_RAND_  [rand]
        gdbSet $entry TMP_INDEX_ $i

        incr i
       }

    gdbSortTab $t TMP_RAND_
   }

set eList [gdbGetInfo $t -parms entries]
set i 0

if {$selByRes} \
   {
    set keepList $eList 
    set delList  ""

    if {![llength $rvList]} \
       {
        set resList  ""
       } \
    elseif {[llength $userResList]} \
       {
        if {$verbose} {puts stderr "Deleting by Specified RESIDs."}

        set resList  ""

        foreach resID $userResList \
           {
            if {![strcasecmp $resID None]} continue
            if {[lsearch -exact $resList $resID] >= 0} continue

            lappend resList $resID

            splitByResID keepList delList $rvList $resID
           }
       } \
    else \
       {
        if {$verbose} {puts stderr "Deleting by Random RESIDs."}

        set resList  ""

        foreach e $eList \
           {
            set resID [getMax [gdbGet $e $rvList]]
            if {[lsearch -exact $resList $resID] >= 0} continue

            lappend resList $resID

            splitByResID keepList delList $rvList $resID
            if {[llength $delList] >= $nDel} break
           }
       }

    foreach e $keepList {gdbSet $e TMP_RAND_ 2.0}
    foreach e $delList  {gdbSet $e TMP_RAND_ 0.0}
   } \
else \
   {
    if {[llength $userRowList]} \
       {
        if {$verbose} {puts stderr "Deleting by Specified Rows."}

        foreach e $eList {gdbSet $e TMP_RAND_ 2.0}

        set n [llength $eList]

        foreach row $userRowList \
           {
            if {![strcasecmp $row None]} return
            if {$row < 0 || $row > $n - 1} continue 

            set e [lindex $eList $row]

            gdbSet $e TMP_RAND_ 0.0
           }
       } \
    else \
       {
        if {$verbose} {puts stderr "Deleting by Random Fraction."}

        foreach e $eList \
           {
            if {$i < $nOut} \
               {
                gdbSet $e TMP_RAND_ 2.0
               } \
            else \
               {
                gdbSet $e TMP_RAND_ 0.0 
               }

            incr i
           }
       }
   }
 
set newT [gdbSelect -from $t -cond "TMP_RAND_ >  0.0" -new]
set newD [gdbSelect -from $t -cond "TMP_RAND_ == 0.0" -new]

gdbSortTab $newT TMP_INDEX_
gdbSortTab $newD TMP_INDEX_

# if {$selByRes} {gdbSortTab $newD [lindex $rvList 0]}

foreach v {TMP_RAND_ TMP_INDEX_} \
   {
    set vv [gdbFindByName $newT $v]
    gdbDestroy $vv

    set vv [gdbFindByName $newD $v]
    gdbDestroy $vv
   }

if {[strcasecmp $outName None]} \
   {
    gdbWrite $newT $outName
   }

if {[strcasecmp $delName None]} \
   {
    gdbWrite $newD $delName
   }

if {$verbose} \
   {
    set nK [gdbGetInfo $newT -parms entryCount]
    set nD [gdbGetInfo $newD -parms entryCount]

    puts stderr [format " Input:   %4d Entries. File: %s" $nIn $inTitle]
    puts stderr [format " Kept:    %4d Entries. File: %s" $nK $outName]
    puts stderr [format " Deleted: %4d Entries. File: %s" $nD $delName]
   }

exit 0
