
#
# Event handler for reading Delta parameters.

proc readParams_JEOL_DELTA {} \
{
   global inName outName dimCount

   clearScript

   set error [catch read_JEOL_DELTA_TCL]
}

proc updateSpecial_JEOL_DELTA {} \
{
   global dmxMode dfSize dfVal dfVal trVal FTx specialArgs commentStr

   clearScript

   if {![info exists dfSize]} {set dfSize 0}
   if {![info exists dfVal]}  {set dfVal  0.0}
   if {![info exists trVal]}  {set trVal  0.0}
   if {![info exists FTx]}    {set FTx    Time}

   set commentStr ""

   if {$dfSize && ![strcasecmp $FTx Time]} \
      {
       set commentStr \
        "# X-Axis Size after DF Adjustment: $dfSize Complex Points.\n\n"
      }

   if {$dmxMode && ![strcasecmp $FTx Time]} \
      {
       set specialArgs " \\\n  -df"
      } \
   else \
      {
       set specialArgs " \\\n  -nodf"
      }

   if {$dfVal != 0.0} \
      {
       append specialArgs " -dfVal $dfVal"
      }

   if {$trVal != 0.0} \
      {
       append specialArgs " -trVal $trVal"
      }
}

proc read_JEOL_DELTA_TCL {} \
{
   global Nx Tx MODEx SWx ORIGx OBSx CARx FTx LABx \
          Ny Ty MODEy SWy ORIGy OBSy CARy FTy LABy \
          Nz Tz MODEz SWz ORIGz OBSz CARz FTz LABz \
          Na Ta MODEa SWa ORIGa OBSa CARa FTa LABa

   global dimCount dimCountName temperature aq2D dmxMode dfSize dfVal trVal
   global inName outName specialArgs dataSource shExt

   set specialArgs ""
 
   if {![file exists $inName]} return

   set dataSource  [getDataSource $inName]

   switch $dataSource \
      {
       ZCAT \
         {
          eval [exec cat $inName | zcat | delta2pipe -showTCL -info]
         }

       GZCAT \
         {
          eval [exec cat $inName | zcat | delta2pipe -showTCL -info]
         }

       default \
         {
          eval [exec delta2pipe -in $inName -showTCL -info]
         }
      }

   updateSpecial_JEOL_DELTA

   set dimCountName $dimCount

   if {![strcasecmp $outName None]} \
      {
       if {$dimCount < 3} \
          {
           set outName test.fid
          } \
       elseif {$dimCount == 3} \
          {
           set outName fid/test%03d.fid 
          } \
       else \
          {
           set outName fid/test%02d%03d.fid 
          }
      }

   setDimCount N
}

#
# Bottom.
