#!/bin/csh

if (`flagLoc $argv -help`) then
 echo 'Fitting and Subtraction of 2D Diagonal:'
 echo ' -in  inName  [test.ft1] 2D Data Input, F2 Processed, Transposed.' 
 echo ' -out outName [diag.ft1] 2D Diagonal Output, F2 Processed, Transposed.' 
 echo ' -p0  p0      [0]        F1 Zero Order Phase.'
 echo ' -p1  p1      [0]        F1 First Order Phase.'
 echo ' -dx  dxPts   [1.0]      F1 Diagonal Position Adjustment Range, +/- Pts.'
 echo ' -off offPts  [0.0]      F1 Diagonal Position Offset, Pts.'
 echo ' -lw  lwPts   [1.0]      Initial Linewidth Estimate, Pts.'
 echo 'Uses Temporary Files: r.dat i.dat ri.dat'
 exit 0
endif

set inName  = (`getArgD $argv -in  test.ft1`)
set outName = (`getArgD $argv -out diag.ft1`)
set p0      = (`getArgD $argv -p0  0`)
set p1      = (`getArgD $argv -p1  0`)
set lw      = (`getArgD $argv -lw  1.0`)
set xRange  = (`getArgD $argv -dx  1.0`)
set xOff    = (`getArgD $argv -off 0.0`)

#
#-----------------------------------------------------------------------------

set loc   = 1
set tSize = (`getParm -in $inName -parm NDSIZE -dim CUR_XDIM -fmt %.0f`)
set fSize = (`getParm -in $inName -parm NDSIZE -dim CUR_YDIM -fmt %.0f`)

dd  if=$inName of=$outName bs=2048 count=1

while ($loc <= $fSize)
 readROI -silent -in test.ft1 -x F1 0% 100% -dy F2 $loc 0 \
         -ndim 1 -real | nmrPipe -fn ZF -out r.dat -ov

 readROI -silent -in test.ft1 -x F1 0% 100% -dy F2 $loc 0 \
         -ndim 1 -imag | nmrPipe -fn ZF \
         | nmrPipe -fn RS -rs $tSize -out i.dat -ov

 addNMR -in1 r.dat -in2 i.dat -out ri.dat -add

 modelXY -data ri.dat -macro $NMRTXT/diagFit.M -p $lw 0.0 -gc 1 -tol 1.0e-8 \
         -var tSize $tSize fSize $fSize thisP0 $p0 thisP1 $p1 \
          xRange $xRange xOff $xOff x0 $loc \
         -str outName $outName -silent -noverb

  /bin/rm r.dat i.dat ri.dat 

  @ loc++
end

echo Created synthetic diagonal interferogram diag.ft1

