#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "List Differences Between Columns in Two Related Tables or PDB Files."
    puts " -in1     \[sum.ass.tab\]     Name of Input Table 1."
    puts " -in2     \[dif.ass.tab\]     Name of Input Table 2."
    puts " -out     \[stdout\]          Name of Output."
    puts " -idVar   \[INDEX\]           Variable for Identifying Entries."
    puts " -assVar  \[ASS\]             Variable for Matching Entries."
    puts " -vList   \[X_HZ Y_HZ ...\]   List of Variables to Use."
    puts "Alternate Method of Specifying Variables:"
    puts " -xVar    \[X_HZ\]"
    puts " -yVar    \[Y_HZ\]"
    puts " -zVar    \[Z_HZ\]"
    puts " -aVar    \[A_HZ\]"
    puts "Output Mode:"
    puts " -copy    Output Columns are Copy of Input." 
    puts " -nocopy  Output Columns Report Differences Only (Default)." 
    puts "PDB Options:"
    puts " -pdb                         Input is PDB Data (Default for .pdb)."
    puts " -nopdb                       Input is NMRPipe-Format Table."
    puts " -hetatm                      Interpret HETATMs as ATOMs."
    puts "Notes:"
    puts " 1. Input files will be read as PDB format if they have"
    puts "    the .pdb file extenstion."
    puts " 2. Output files will be saved as PDB format if they have"
    puts "    the .pdb file extenstion."
    puts " 3. Default setting for PDB input is:"
    puts "    -idVar ATOMID -assVar ATOMID -vList X Y Z TEMPFACTOR"
    exit 1
   }

set inName1  sum.ass.tab
set inName2  dif.ass.tab
set outName  stdout
set hetMode  nohetatm
set warnFlag 1
set copyFlag 0

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2

set xVar    X_HZ
set yVar    Y_HZ
set zVar    Z_HZ
set aVar    A_HZ
 
set idVar   INDEX
set assVar  ASS

if {![strcasecmp [file extension $inName1] .pdb] ||
     ![strcasecmp [file extension $inName2] .pdb]} \
   {
    set xVar    X
    set yVar    Y
    set zVar    Z
    set aVar    TEMPFACTOR

    set idVar   ATOMID 
    set assVar  ATOMID 
   }

getArgD $ARGV -out outName

getArgD $ARGV -xVar   xVar
getArgD $ARGV -yVar   yVar
getArgD $ARGV -zVar   zVar
getArgD $ARGV -aVar   aVar

set vList "$xVar $yVar $zVar aVar"

getListArgD $ARGV -vList vList

getArgD $ARGV -idVar  idVar
getArgD $ARGV -assVar assVar

if {[flagLoc $ARGV -copy]}      {set copyFlag 1}
if {[flagLoc $ARGV -nocopy]}    {set copyFlag 0}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

if {[flagLoc $ARGV -warn]}      {set warnFlag 1}
if {[flagLoc $ARGV -nowarn]}    {set warnFlag 0}

#
#------------------------------------------------------------------------

if {![strcasecmp [file extension $inName1] .pdb]} \
   {
    set tab1 [gdbRead table -in $inName1 -pdb -$hetMode]
   } \
else \
   {
    set tab1 [gdbRead table -in $inName1]
   }

if {![strcasecmp [file extension $inName2] .pdb]} \
   {
    set tab2 [gdbRead table -in $inName2 -pdb -$hetMode]
   } \
else \
   {
    set tab2 [gdbRead table -in $inName2]
   }

set eListI [gdbGetInfo $tab1 -parms entries]
set vListI [gdbGetInfo $tab1 -parms varNames]
set vListJ [gdbGetInfo $tab2 -parms varNames]

set qList ""

if {[lsearch -exact $vListI $assVar] < 0} \
   {
    puts stderr "Error: Missing Assignmemt Variable $assVar"
    exit 1
   }

foreach v $vList \
   {
    if {[lsearch -exact $vListI $v] >= 0 && [lsearch -exact $vListJ $v] >= 0} \
       {
        lappend qList $v
       }
   }

set aID     [gdbFindByName $tab1 $assVar]
set assType [gdbGetInfo $aID -parms varType]
set assFmt  [gdbGetInfo $aID -parms varFmt]

if {$copyFlag} \
   {
    set dList ""

    set mCount 0
    set rCount 0
    set zCount 0
 
    foreach entryI $eListI \
       {
        set ass [gdbGet $entryI $assVar]

        if {![strcasecmp $assType text]} \
           {
            if {![strcasecmp $ass None]} continue
            set cond "!strcasecmp( $assVar, '$ass' )"
           } \
        else \
           {
            set cond "$assVar == $ass"
           }

        set eListJ [gdbSelect -from $tab2 -cond $cond]

        set nJ  [llength $eListJ]

        if {$nJ == 0} {incr zCount}
        if {$nJ > 1}  {incr mCount}
 
        foreach entryJ $eListJ \
           {
            if {[lsearch -exact $dList] < 0} \
               {
                lappend dList $entryJ 
               } \
            else \
               {
                incr rCount
               }

            set idJ [gdbGet $entryJ $idVar]

            foreach v $qList \
               {
                set xI [gdbGet $entryI $v]
                set xJ [gdbGet $entryJ $v]

                gdbSet $entryI $v [expr $xI - $xJ]
               }
           }
       }

    if {$mCount || $rCount || $zCount} \
       {
        if {$warnFlag} \
           {
            puts stderr "In-Place Warnings:"
            puts stderr " Entries with No Matches:       $zCount"
            puts stderr " Entries with Multiple Matches: $mCount"
            puts stderr " Entries with Reused Matches:   $rCount"
           }
       }

    if {![strcasecmp [file extension $outName] .pdb]} \
       {
        gdbWrite $tab1 $outName -pdb -$hetMode
       } \
    else \
       {
        gdbWrite $tab1 $outName
       }
   } \
else \
   {
    if {[strcasecmp $outName stdout] && [strcasecmp $outName None]} \
       {
        set error [catch {set outUnit [open $outName w]}]

        if {$error} \
           {
            puts stderr "Error Opening Output File $outName"
            exit 1
           }
       } \
    else \
       {
        set outUnit stdout
       }

    puts -nonewline $outUnit "VARS   ASS INDEX_I INDEX_J"
    foreach v $qList {puts -nonewline $outUnit " D$v"}
    puts $outUnit ""

    puts -nonewline $outUnit "FORMAT $assFmt %4d     %4d    "
    foreach v $qList {puts -nonewline $outUnit " %9.4f"}
    puts $outUnit ""
    puts $outUnit ""
 
    foreach entryI $eListI \
       {
        set ass [gdbGet $entryI $assVar]
    
        if {![strcasecmp $assType text]} \
           {
            if {![strcasecmp $ass None]} continue
            set cond "!strcasecmp( $assVar, '$ass' )"
           } \
        else \
           {
            set cond "$assVar == $ass"
           }
 
        set eListJ [gdbSelect -from $tab2 -cond $cond]

        set idI [gdbGet $entryI $idVar]

        foreach entryJ $eListJ \
           {
            set idJ [gdbGet $entryJ $idVar]

            puts -nonewline $outUnit [format " $assFmt %4d %4d" $ass $idI $idJ]

            foreach v $qList \
               {
                set xI [gdbGet $entryI $v]
                set xJ [gdbGet $entryJ $v]
                set dX [expr $xI - $xJ]

                puts -nonewline $outUnit [format " % 9.4f" $dX]
               }

             puts $outUnit ""
           }
       }

    if {[strcasecmp $outUnit stdout]} {close $outUnit}
   }

exit 0

