#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc next1D {} \
{
   global nameLoc inList inPath

   incr nameLoc

   if {$nameLoc > [llength $inList] - 1} \
      {
       set nameLoc 0
      }

   set inPath  [lindex $inList $nameLoc]

   read1D
   draw1D
}

proc prev1D {} \
{
   global nameLoc inList inPath

   incr nameLoc -1

   if {$nameLoc < 0} \
      {
       set nameLoc [expr [llength $inList] - 1]
      }

   set inPath  [lindex $inList $nameLoc]

   read1D
   draw1D
}

proc read1D {} \
{
   global inPath inName minVal maxVal

   if {![file exists $inPath/$inName]} \
      {
       showError draw1D "Error opening $inPath/$inName"
       return 1
      }

   wm title  . "Working ..."
   update

   readROI -ndim 1 -roi 1 -in $inPath/$inName -x X_AXIS 
   readPk

   set roiInfo [getROIInfo -roi 1]
   set rdata   [lindex $roiInfo 0]
   set size    [lindex [lindex $roiInfo 2] 0]

   set v [vMax $rdata $size]

   set minVal [expr -0.05*abs($v)]
   set maxVal [expr  1.05*abs($v)]

   return 0
}

proc draw1D {} \
{
   global minVal maxVal inPath inName

   drawSettings -reset

   drawROI -roi 1 -zero -fhi $minVal -lhi $maxVal -cell 1 -xGrid -yGrid \
       -bg black -axis Bottom

   drawPk

   wm title  . $inPath/$inName
   update
}

proc readPk {} \
{
   global inPath inTab tabName

   if {![strcasecmp $tabName None]}     {return 0}
   if {![file exists $inPath/$tabName]} {return 0}

   wm title  . "Working ..."

   if {$inTab} \
      {
       gdbDestroy $inTab
       set inTab  0
      }

   set inTab [gdbRead table -in $inPath/$tabName]

   return 0
}

proc drawPk {} \
{
   global minVal maxVal inTab cy1 cyn

   if {!$inTab} return

   set entry [gdbFirst $inTab]

   set xPrev   1000.0
   set yPrev 100000.0

   while {$entry} \
      {
       set xPPM  [gdbGet $entry X_PPM]
       set hi    [gdbGet $entry HEIGHT]

       set y1 [hi2pix $hi]
       set y1 [expr $y1 - 4]

       if {abs($xPrev - $xPPM) > 0.5 || abs($yPrev - $y1) > 30} \
          {
           set yn [expr $y1 - 16]
          } \
       else \
          {
           set yn [expr $yPrev - 16]
          }

       if {$yn >= $cy1 && $yn <= $cyn} \
          {
           drawLine -cell 1 -roiRef 1 \
              -x ${xPPM}ppm ${xPPM}ppm -y $y1 $yn -fg green
          }

       set yn [expr $yn - 4]

       if {$yn >= $cy1 && $yn <= $cyn} \
          {
           drawText -cell 1 -roiRef 1 \
              -xOff -2 -x ${xPPM}ppm -y $yn -bg black -fg green -text $xPPM
          }

       set xPrev $xPPM
       set yPrev $yn

       set entry [gdbNext $entry]
      }
}

proc hi2pix { hi } \
{
   global maxVal minVal cy1 cyn

   set range [expr $maxVal - $minVal]
   set n     [expr $cyn - $cy1]
   set pix   [expr int($cyn - $n*($hi - $minVal)/$range)]

   return $pix
}

proc initWin1D {} \
{
   global iyPrev xSize ySize cx1 cxn cy1 cyn

   set xSize  15000
   set ySize  600
   set iyPrev 300

   set cx1 20
   set cxn 14900
   set cy1 20
   set cyn 540

   wm geometry . =850x650+20+20
   update

   scrollbar .s -orient horiz -command ".c xview"

   canvas .c -width $xSize -height $ySize -border 0 \
      -xscrollcommand ".s set" -scrollregion "0 0 $xSize $ySize"

   frame  .f

   button .next -text Next     -command next1D 
   button .prev -text Previous -command prev1D 

   button .quit -text Quit     -command "exit 0"

   pack   .next .prev .quit -in .f -side left -expand 1 -fill x
   pack   .c .s .f -side top -expand 1 -fill x

   .c create pixmap 0 0 -width $xSize -height $ySize -win 1 

   defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -win 1
}

proc scale1D { iy } \
{
   global minVal maxVal iyPrev

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal/0.9]
       set maxVal [expr $maxVal/0.9]
      } \
   else \
      {
       set minVal [expr $minVal*0.9]
       set maxVal [expr $maxVal*0.9]
      }

   set iyPrev $iy

   draw1D
}

proc move1D { iy } \
{
   global minVal maxVal iyPrev

   set range [expr $maxVal - $minVal]

   if {$iy > $iyPrev} \
      {
       set minVal [expr $minVal + $range*0.05]
       set maxVal [expr $maxVal + $range*0.05]
      } \
   else \
      {
       set minVal [expr $minVal - $range*0.05]
       set maxVal [expr $maxVal - $range*0.05]
      }

   set iyPrev $iy

   draw1D
}

if {[flagLoc $ARGV -help]} \
   {
    puts "draw1D.tcl: Draw Related 1D Spectra in a List of Directories."
    puts " -dir dirList  \[./\]       List of Input Directories."
    puts " -in  inName   \[test.ft1\] Input Spectrum Name."
    puts " -tab tabName  \[test.tab\] Input Peak Table Name, or 'None'."
    exit 0
   }

set nameLoc 0
set inList  ./ 
set inName  test.dat
set tabName test.tab
set inTab   0

getListArgD $ARGV -dir inList
getArgD     $ARGV -in  inName
getArgD     $ARGV -tab tabName

set inPath [lindex $inList 0]

wm title  . $inPath/$inName

initWin1D

set error [read1D]
if {$error} {exit 0}

set error [readPk]
if {$error} {exit 0}


bind .c <B1-Motion> "scale1D %y"
bind .c <B2-Motion> "move1D  %y"

newScreen
draw1D
