#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set objInfo(dyn,firstTime) 1

proc initForce {} \
{
   global objInfo

   initForceVars
   initForceSettings
   initForceArgs
   initForceVecs
   initForceTables
   initCmndPanel
   initForceDisplay
   initForceGraph
   initMolDisplay

   set objInfo(dyn,firstTime) 0
}

proc initForceVars {} \
{
   global objInfo

   srand

   set objInfo(Force,terms) \
    { \
     RMSD    Sum                      \
     Torsion Angle  Bond Rama   Align \
     VDW     Radius NOE  J      D     \
     CS      HBond \
    }

   set objInfo(Force,colors) \
    { \
     red     cyan                         \
     orange  magenta  blue   wheat white  \
     pink    brown    orchid gray  yellow \
     green   navy
    }

   set n [llength $objInfo(Force,colors)] 
   set i 0

   foreach term $objInfo(Force,terms) \
      {
       set j [expr $i % $n]
       set objInfo($term,color)         [lindex $objInfo(Force,colors) $j]
       set objInfo($term,name)          None
       set objInfo($term,forceConstant) 0.0
       set objInfo($term,force)         0.0
       set objInfo($term,tab)           0
       set objInfo($term,src)           0

       incr i
      }
 
   set objInfo(RMSD,forceConstant)     0.0
   set objInfo(Sum,forceConstant)      1.0

   set objInfo(Ref,name)     [testForceFile ref.pdb]
   set objInfo(PDB,name)     [testForceFile dyn.pdb]
   set objInfo(Align,name)   [testForceFile dc.search.tab]
   set objInfo(NOE,name)     [testForceFile noeObs.tab]
   set objInfo(CS,name)      [testForceFile csObs.tab]
   set objInfo(D,name)       [testForceFile dObs.tab]
   set objInfo(J,name)       [testForceFile jObs.tab]
   set objInfo(HBond,name)   [testForceFile hObs.tab]
   set objInfo(Torsion,name) [testForceFile tObs.tab]

   set objInfo(Ref,res1)     0
   set objInfo(Ref,resN)     0 
   set objInfo(Ref,src)      0 
   set objInfo(Ref,tab)      0 

   set objInfo(PDB,res1)     0 
   set objInfo(PDB,resN)     0 
   set objInfo(PDB,src)      0 
   set objInfo(PDB,tab)      0 

   set objInfo(dyn,maxTrials)       3000 
   set objInfo(dyn,updateCount)     10
   set objInfo(dyn,rasmolFlag)      0

   set objInfo(dyn,protcol)         gradAngles
   set objInfo(dyn,startProc)       dynStart 
   set objInfo(dyn,autoStart)       0

   if {[lsearch -exact [winfo interps] rasmol] >= 0} \
      {
       set objInfo(dyn,rasmolFlag) 1
      }

   set objInfo(Da,target)        ""
   set objInfo(Dr,target)        ""

   set objInfo(Da,current)       ""
   set objInfo(Dr,current)       ""
   set objInfo(Da,forceConstant) ""
   set objInfo(Dr,forceConstant) ""

   set objInfo(VDW,contactDist)     1.45 
   set objInfo(Align,excludeCount)  0
   set objInfo(Align,segCount)      0
   set objInfo(Grad,step)           0.75

   set objInfo(dyn,dPhiList) {-25 -10 -5 -3 -1 0 1 3 5 10 25}
   set objInfo(dyn,dPsiList) {-25 -10 -5 -3 -1 0 1 3 5 10 25}

   set objInfo(dyn,interuptFlag)  0
   set objInfo(dyn,reportCount)   0
   set objInfo(dyn,trial)         0

   set objInfo(SA,saTrial)        0
   set objInfo(SA,tTrial)         0

   set objInfo(dyn,resOffset)     0
   set objInfo(Ref,resID)         0
   set objInfo(PDB,resID)         0
}

proc initForceSettings {} \
{
   global objInfo

   catch {source dynInit.tcl}

   initDaDr
}

proc initForceArgs {} \
{
   global objInfo ARGC ARGV

   if {$ARGC > 1} \
      {
       set objInfo(Ref,res1)      [lindex $ARGV 1]
       set objInfo(Ref,resN)      [lindex $ARGV 2]
       set objInfo(PDB,res1)      [lindex $ARGV 1]
       set objInfo(PDB,resN)      [lindex $ARGV 2]
       set objInfo(dyn,maxTrials) [lindex $ARGV 3]
      }
}

proc initDaDr {} \
{
   global objInfo

   set objInfo(Da,current)       ""
   set objInfo(Dr,current)       ""
   set objInfo(Da,forceConstant) ""
   set objInfo(Dr,forceConstant) ""

   set n [llength $objInfo(D,name)]

   for {set i 0} {$i < $n} {incr i} \
      {
       set Da [lindex $objInfo(Da,target) $i]
       set Dr [lindex $objInfo(Da,target) $i]

       lappend objInfo(Da,current) 0.0
       lappend objInfo(Dr,current) 0.0

       lappend objInfo(Da,forceConstant) [expr 0.50/abs($Da)]
       lappend objInfo(Dr,forceConstant) [expr 0.25/abs($Dr)]
      }
}

proc testForceFile { tabName } \
{
   global objInfo

   if {[file exists $tabName]} {return $tabName}

   return None
}

proc initForceTables {} \
{
   global objInfo

   if {$objInfo(dyn,firstTime)} \
      {
       dynInit

       dynSetChainMode -ignore
       dynSetSegMode -ignore

       gdbInit
       gdbCreate dBase
      }

   set tInfo "pdb pdb noe cs"
   set i     0

   foreach term "Ref PDB NOE CS" \
      {
       set  t [lindex $tInfo $i]
       incr i

       if {$objInfo($term,src)} \
          {
           dynFree $objInfo($term,src)
           set objInfo($term,src) 0
          }

       if {[file exists $objInfo($term,name)]} \
          {
           if {"$t" == "pdb"} \
              {
               set objInfo($term,src) [dynRead -pdb -in $objInfo($term,name)]
              } \
           else \
              {
               set objInfo($term,src) \
                [dynRead -$t -in $objInfo($term,name) -src $objInfo(PDB,src)]
              }
          }
      }

   foreach src $objInfo(D,src) \
      {
       if {$src} {dynFree $src}
      }

   set objInfo(D,src) ""

   foreach dName $objInfo(D,name) \
      {
       if {[file exists $dName]} \
          {
           lappend objInfo(D,src) \
            [dynRead -dc -in $dName -src $objInfo(PDB,src)]
          }
      }

   if {$objInfo(Align,tab)} \
      {
       gdbDestroy $objInfo(Align,tab)
       set objInfo(Align,tab) 0
      }

   if {"$objInfo(Align,name)" != "None"} \
      {
       set objInfo(Align,tab) [gdbRead table -in $objInfo(Align,name)]
      }

   initAlignForce
}

proc initForceVecs {} \
{
   global VecTrial objInfo

   catch {vector destroy VecTrial}

   vector create VecTrial(1)

   foreach term $objInfo(Force,terms) \
      {
       global Vec$term

       catch {eval [format "vector destroy Vec%s"    $term]}
       catch {eval [format "vector create  Vec%s(1)" $term]}
   
       set objInfo($term,force)        0.0
       set objInfo($term,initialForce) 0.0
      }
}

proc initCmndPanel {} \
{
   global objInfo env

   if {!$objInfo(dyn,firstTime)} return

   set w .t

   if {[winfo exists $w]} return

   toplevel $w 

   wm title $w "DYNAMO 2010.160.15.01"
   wm geom  $w -10+50

   catch {image delete dynImage}

   image create photo dynImage -file $env(DYNAMO_DIR)/image/dynamo.gif

   button $w.dynLogo -relief sunken -image dynImage -command "showHelp logo"

   pack $w.dynLogo -in $w -side top -anchor center

   foreach term $objInfo(Force,terms) \
      {
       if {"$term" == "RMSD"} continue
       if {"$term" == "Sum"}  continue

       frame $w.f$term

       label $w.lab$term -text $term: -width 8 -anchor w

       entry $w.fnam$term -textvariable objInfo($term,name) -width 12 

       button $w.bnam$term -command "setForceName $term" \
        -relief raised -bitmap @$env(NMRTXT)/caret.map -fg darkGreen

       entry $w.fcon$term -textvariable objInfo($term,forceConstant) -width 6

       pack $w.lab$term \
            $w.fnam$term $w.fcon$term $w.bnam$term \
            -in $w.f$term -side left -expand 1 -fill x

       pack $w.f$term -in $w -side top -expand 1 -fill x
      }
}

proc setForceConstant { term } \
{
}

proc setForceName { term } \
{
   global objInfo

   set types \
      {
       { {Table}      {.tab} }
       { {PDB}        {.pdb} }
       { {dat}        {.dat .DAT} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
       { {All Files}  *      }
      }

   set inName [tk_getOpenFile -title "DYNAMO $term Input" -filetypes $types]

   if {"$inName" == ""} return

   set objInfo($term,name) $inName
}

proc initForceDisplay {} \
{
   global VecTrial objInfo

   if {!$objInfo(dyn,firstTime)} return

   set obj dyn

   update

   set objInfo(screen,xSize) [winfo screenwidth  .]
   set objInfo(screen,ySize) [winfo screenheight .]

   set objInfo($obj,window) .
   set objInfo($obj,graph)  .g

   if {[winfo exists .g]} return

   if {$objInfo(screen,ySize) == 768} \
      {
       set objInfo($obj,xSize)  700 
       set objInfo($obj,ySize)  500 
      } \
   else \
      {
       set objInfo($obj,xSize)  800 
       set objInfo($obj,ySize)  500 
      }

   wm geom $objInfo(dyn,window) +10+10

   graph   $objInfo($obj,graph) \
           -height $objInfo($obj,ySize) -width $objInfo($obj,xSize) \
           -plotbackground black -title Forces

   frame  .f

   button .update -text Update -command "getForceStatus 0" 
   button .start  -text Start  -command "dynStart" 
   button .stop   -text Stop   -command "dynStop" 
   button .reset  -text Reset  -command "dynReset" 
   button .quit   -text Quit   -command "dynQuit"

   pack   .update .start .stop .reset .quit \
          -in .f -side left -expand 1 -fill x 

   pack   .g -side top
   pack   .f -side top -expand 1 -fill x 

   update
}

proc initForceGraph {} \
{
   global objInfo

   set obj dyn

   foreach term $objInfo(Force,terms) \
      {
       if {"$term" == "RMSD" || $objInfo($term,forceConstant) != 0.0} \
          {
           global Vec$term

           catch {$objInfo($obj,graph) element delete $term}

           $objInfo($obj,graph) element create $term \
              -xdata VecTrial -ydata Vec$term -fill white -pixels 2 \
              -color $objInfo($term,color)
          }
      }

   update
}

proc initMolDisplay {} \
{
   global objInfo

   if {!$objInfo(dyn,firstTime)}  return
   if {!$objInfo(dyn,rasmolFlag)} return

#  exec /u/delaglio/bin/linux/rasmol &
#  exec sleep 3

#  send rasmol wireframe off
#  send rasmol ribbons   on
   send rasmol zoom      150
   send rasmol set fontsize 16
}

proc showHelp { obj } \
{
}

proc getForce {} \
{
   global objInfo

   set objInfo(Sum,force) 0.0

   foreach term $objInfo(Force,terms) \
      {
       set objInfo($term,force) [get${term}Force]

       set objInfo(Sum,force) \
        [expr $objInfo(Sum,force) + \
              $objInfo($term,forceConstant)*$objInfo($term,force)]
      }

   set objInfo(Sum,force) \
    [expr $objInfo(Sum,forceConstant)*$objInfo(Sum,force)]

   return $objInfo(Sum,force)
}

proc getForceReport {} \
{
   global VecTrial objInfo

   incr objInfo(dyn,reportCount)

   foreach term $objInfo(Force,terms) \
      {
       if {$objInfo(dyn,reportCount) == 1} \
          {
           set objInfo($term,initialForce) $objInfo($term,force)
          }

       if {"$term" == "RMSD" || $objInfo($term,forceConstant) != 0.0} \
          {
           append str [format " $term: %.4f" $objInfo($term,force)]
          }
      }

   if {$objInfo(D,forceConstant) != 0.0} \
      { 
       set n [llength $objInfo(D,name)]

       for {set i 0} {$i < $n} {incr i} \
          {
           set fmt " Dar($i): %.3e %.3e"

           set thisDa [lindex $objInfo(Da,current) $i]
           set thisDr [lindex $objInfo(Dr,current) $i]

           append str [format $fmt $thisDa $thisDr]
          }
      }

   if {$objInfo(dyn,reportCount) == 1} \
      {
       set index 0
      } \
   else \
      {
       set index ++end
      }

   set VecTrial($index) $objInfo(dyn,reportCount)

###   puts -nonewline stderr " $objInfo(dyn,trial)"

   foreach term $objInfo(Force,terms) \
      {
       global Vec$term

       if {$objInfo($term,initialForce) != 0.0} \
          {
           set f [expr $objInfo($term,force)/$objInfo($term,initialForce)]
          } \
       else \
          {
           set f $objInfo($term,force) 
          }

###       puts -nonewline stderr " $term $f"

       eval [format "set Vec%s($index) $f" $term]
      }

###   puts stderr ""

   update

   return $str
}

proc getForceStatus { verbose } \
{
   global objInfo

   set obj RMSD

   set aInfo [dynAlign -refSrc $objInfo(Ref,src) -aSrc $objInfo(PDB,src) \
                       -r1 $objInfo(Ref,res1) -rN $objInfo(Ref,resN)     \
                       -a1 $objInfo(PDB,res1) -atom CA -verb]

   set objInfo($obj,force) [lindex $aInfo end]

   set aRMS [format %.3f $objInfo($obj,force)]
   set fmt  "%3d %8.2f %8.2f  %8.2f %8.2f"

   set objInfo(Ref,resID) $objInfo(Ref,res1)

   while {$objInfo(Ref,resID) <= $objInfo(Ref,resN)} \
      {
       set objInfo(PDB,resID) \
        [expr $objInfo(PDB,res1) + $objInfo(Ref,resID) - $objInfo(Ref,res1)]

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $objInfo(PDB,resID)]

       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]

       set aInfo [dynGetPhiPsi -src $objInfo(Ref,src) -res $objInfo(Ref,resID)]

       set phiRef   [lindex $aInfo 0]
       set psiRef   [lindex $aInfo 1]

       if {$verbose} \
          {
           puts [format $fmt $objInfo(Ref,resID) $phiRef $psiRef $phi $psi]
          }

       incr objInfo(Ref,resID)
      }

   dynWrite -pdb -src $objInfo(PDB,src) -out dyn.pdb

   wm title $objInfo(dyn,window) \
    "Trial: $objInfo(dyn,trial)  Current RMSD: ${aRMS}A"

   showMol

   if {$verbose} {puts "Final RMSD: ${aRMS}A"}
}

proc showMol {} \
{
   global objInfo

   if {!$objInfo(dyn,rasmolFlag)} return

   send rasmol zap 
   send rasmol load        $objInfo(PDB,name)
#  send rasmol wireframe   off
#  send rasmol backbone    on
}

proc getSumForce {} \
{
   global objInfo

   set obj Sum
   return $objInfo($obj,force)
}

proc getRMSDForce {} \
{
   global objInfo

   set obj RMSD
   return $objInfo($obj,force)
}

proc getBondForce {} \
{
   global objInfo

   set obj Bond
   return $objInfo($obj,force)
}

proc getHBondForce {} \
{
   global objInfo

   set obj Bond
   return $objInfo($obj,force)
}

proc getTorsionForce {} \
{
   global objInfo

   set obj Torsion 
   return $objInfo($obj,force)
}

proc getNOEForce {} \
{
   global objInfo

    global objInfo

    set obj NOE 

    if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

    set noeInfo \
     [dynSimulate -noe $objInfo(NOE,src) -src $objInfo(PDB,src)]

    set noeForce 1.0
    set noeRMS   [lindex $noeInfo 1]
    set noeN     [lindex $noeInfo 2]

    if {$noeN != 0.0} {set noeForce [expr $noeRMS/$noeN]}

    set noeForce $noeRMS

    return $noeForce
}

proc getJForce {} \
{
   global objInfo

   set obj J
   return $objInfo($obj,force)
}

proc getRadiusForce {} \
{
   global objInfo

   set obj Radius
   return $objInfo($obj,force)
}

proc getAngleForce {} \
{
   global objInfo

   set obj Angle 
   return $objInfo($obj,force)
}

proc initAlignForce {} \
{
   global objInfo env

   set obj Align
   set objInfo($obj,segCount) 0

   set pdbDir $env(PDBH_DIR)

   if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

   set resID $objInfo(Ref,res1)

   while {$resID <= $objInfo(Ref,resN)} \
      {
       set  objInfo($obj,segLoc,$resID) ""
       incr resID
      }

   set entry [gdbFirst $objInfo(Align,tab)]

   while {$entry} \
      {
       set aName   [gdbGet $entry PDB_NAME]
       set r1      [gdbGet $entry D_RES1]
       set rN      [gdbGet $entry D_RESN]
       set res1    [gdbGet $entry RES1]
       set resN    [gdbGet $entry RESN]

       set i       $objInfo($obj,segCount)

       for {set resID $r1} {$resID <= $rN} {incr resID} \
          {
           lappend objInfo($obj,segLoc,$resID) $i
          }

       set aSrc  \
        [dynRead -pdb -in $pdbDir/$aName]

       set aInfo \
        [dynAlign -refSrc $objInfo(PDB,src) -aSrc $aSrc \
                  -r1 $r1 -rN $rN -a1 $res1]

       set objInfo($obj,segInfo,$i) "$aSrc $aName $r1 $rN $res1 $resN"
       set objInfo($obj,force,$i)   [lindex $aInfo end]

       puts "Align $aName $objInfo($obj,segInfo,$i) $objInfo($obj,force,$i)"

       set  entry [gdbNext $entry]

       incr objInfo($obj,segCount)
      }
}

proc getAlignForce {} \
{
   global objInfo

   set obj Align

   if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}
   if {$objInfo($obj,segCount) == 0}        {return 0.0}

   set sum 0.0

   if {$objInfo(dyn,trial)} \
      {
       set resIDRef $objInfo(Ref,resID)

       foreach i $objInfo($obj,segLoc,$resIDRef) \
          {
           set info $objInfo($obj,segInfo,$i)
 
           set aSrc [lindex $info 0]
           set r1   [lindex $info 2]
           set rN   [lindex $info 3]
           set res1 [lindex $info 4]
           set resN [lindex $info 5]
     
           set aInfo \
            [dynAlign -refSrc $objInfo(PDB,src) -aSrc $aSrc \
                      -r1 $r1 -rN $rN -a1 $res1]

           set objInfo($obj,force,$i) [lindex $aInfo end]
          }
      }

    for {set i 0} {$i < $objInfo($obj,segCount)} {incr i} \
       {
        set sum [expr $sum + $objInfo($obj,force,$i)]
       }

    set sum [expr $sum/double($objInfo($obj,segCount))]

    return $sum
}

proc getVDWForce {} \
{
   global objInfo

   set obj VDW

   if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

   set  r1 [expr $objInfo(PDB,res1) + $objInfo(Ref,res1) - 1]
   set  rN [expr $objInfo(PDB,res1) + $objInfo(Ref,resN) - 1]

   dynSelect -clear -src $objInfo(PDB,src)

   set aList "HA* HB* HN O"

   set count \
       [dynSelect -src $objInfo(PDB,src) -r1 $r1 -rN $rN -glob -atom $aList]

   set cInfo [dynGetDist -src $objInfo(PDB,src) -contact -verb \
                         -dist $objInfo(VDW,contactDist)]

   set sum 0.0

   foreach contact $cInfo \
      {
       set dev [lindex $contact 0]
       set d   [expr $objInfo($obj,contactDist) - $dev]
   
       set sum [expr $sum + $d*$d] 
      }

   return $sum
}

proc getDForce {} \
{
    global objInfo

    set obj D
    set n   [llength $objInfo(D,name)]

    if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

    set objInfo(Da,current) ""
    set objInfo(Dr,current) ""
    set sum 0.0

    for {set i 0} {$i < $n} {incr i} \
       {
        set dSrc [lindex $objInfo(D,src)     $i]
        set Da   [lindex $objInfo(Da,target) $i]
        set Dr   [lindex $objInfo(Da,target) $i]

        set dInfo \
         [dynSimulate -dc $dSrc -src $objInfo(PDB,src) \
                      -s1 $objInfo(saupe,res1) -sN $objInfo(saupe,resN) \
                      -a1 $objInfo(saupe,res1) -euler]

        set nn [llength $dInfo]
        set k  0

        for {set j [expr $nn - 6]} {$j < $nn - 1} {incr j} \
           {
            set s($k) [lindex $dInfo $j]
            incr k
           }

        set dInfo \
         [dynSimulate -dc $dSrc -src $objInfo(PDB,src) \
                      -saupe $s(0) $s(1) $s(2) $s(3) $s(4) \
                      -s1 $objInfo(Ref,res1) -sN $objInfo(Ref,resN) \
                      -a1 $objInfo(PDB,res1) -euler]

         set dRMS   [lindex $dInfo 0]
         set thisDa [lindex $dInfo 3]
         set thisDr [lindex $dInfo 4]
 
         set DaFC   [lindex $objInfo(Da,forceConstant) $i]
         set DrFC   [lindex $objInfo(Dr,forceConstant) $i]

         lappend objInfo(Da,current) $thisDa
         lappend objInfo(Dr,current) $thisDr

         set sum \
          [expr $sum + $dRMS + $DaFC*abs($thisDa-$Da) + $DrFC*abs($thisDr-$Dr)]
        }

    return $sum 
}

proc getCSForce {} \
{
    global objInfo 

    set obj CS 

    if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

    set csInfo \
     [dynSimulate -cs $objInfo(CS,src) -src $objInfo(PDB,src)   \
                  -s1 $objInfo(Ref,res1) -sN $objInfo(Ref,resN) \
                  -a1 $objInfo(PDB,res1)]

    set csForce 1.0
    set csRMS   [lindex $csInfo 1]
    set csN     [lindex $csInfo 2]

    if {$csN != 0.0} {set csForce [expr $csRMS/$csN]}

    return $csForce
}

proc getRamaForce {} \
{
    global objInfo

    set obj Rama

    if {$objInfo($obj,forceConstant) == 0.0} {return 0.0}

    set sInfo \
     [dynSurface  -refSrc $objInfo(Ref,src) -aSrc $objInfo(PDB,src) \
                  -r1 $objInfo(Ref,res1) -rN $objInfo(Ref,resN)     \
                  -a1 $objInfo(PDB,res1)]

    set sForce [lindex $sInfo 0]

    return $sForce
}

proc getPhiPsiByGrad { gradInfo pairCount } \
{
   set n [expr 2*$pairCount - 1]
   set x ""
   set y ""

   foreach gItem $gradInfo \
      {
       lappend y      [lindex $gItem end]
       set     tList  [lrange $gItem 0 $n]
 
       foreach theta $tList \
          {
           lappend x [expr $theta*$theta]
           lappend x $theta
          }

       lappend x 1.0
      }

   set coef  [svdLinSol -x $x -y $y]
   set tList ""
   set i     0

   for {set j 0} {$j < 2*$pairCount} {incr j} \
      {
       set  a [lindex $coef $i]
       incr i

       set  b [lindex $coef $i]
       incr i
   
       set dTheta 0.0

       if {$a != 0.0} {set dTheta [expr -$b/(2.0*$a)]}

       lappend tList $dTheta
      }

   return $tList
}

proc saTest { eOld eNew t } \
{
   if {$eNew < $eOld} {return 1}

   set de     [expr 100.0*($eNew - $eOld)]
   set status 0
 
   catch {set status [expr [rand] < exp( -$de/$t )]}

   return $status
}

proc saAngles {} \
{
   global objInfo 

   set objInfo(SA,t) $objInfo(SA,tInit)

   set resCount [llength $objInfo(dyn,allowedList)]
   set newForce [getForce]

   while {$objInfo(SA,tTrial) < $objInfo(SA,tCount)} \
      {
       set objInfo(SA,saTrial) 1

       while {$objInfo(SA,saTrial) < $objInfo(SA,saCount)} \
          {
           if {$objInfo(dyn,trial) >= $objInfo(dyn,maxTrials)} return

           dynRestore $objInfo(PDB,src)

           if {$objInfo(dyn,interuptFlag)} break

           if {$objInfo(dyn,trial) == 1 || 
                !($objInfo(dyn,trial) % $objInfo(dyn,updateCount))} \
              {
               getForceStatus 0
              }

           set resLoc             [expr int([rand]*$resCount)]
           set objInfo(Ref,resID) [lindex $objInfo(dyn,allowedList) $resLoc]

           set loc \
            [lsearch -exact $objInfo(dyn,excludeList) $objInfo(Ref,resID)]

           if {loc >= 0} continue

           set objInfo(dyn,resOffset) \
            [expr $objInfo(Ref,resID) - $objInfo(Ref,res1)]

           set objInfo(PDB,resID)     \
            [expr $objInfo(PDB,res1) + $objInfo(dyn,resOffset)]

           set aInfo \
            [dynGetPhiPsi -src $objInfo(PDB,src) -res $objInfo(PDB,resID)]

           set phi [lindex $aInfo 0]
           set psi [lindex $aInfo 1]

           if {$phi > 999 || $psi > 999} continue

           set bestForce $newForce
           set bestPhi   $phi
           set bestPsi   $psi
           set accepted  0

           set dPhi [expr 50.0*[rand] - 25.0]
           set dPsi [expr 50.0*[rand] - 25.0]
 
           set newPsi [expr $psi + $dPsi]
           set newPhi [expr $phi + $dPhi]
   
           dynSetPhiPsi -store -src $objInfo(PDB,src) \
                        -res $objInfo(PDB,resID) -phi $newPhi -psi $newPsi
           
           set newForce [getForce]

           if {[saTest $bestForce $newForce $objInfo(SA,t)]} \
              {
               set bestForce $newForce
               set bestPhi   $newPhi
               set bestPsi   $newPsi
               set accepted  1
              }

           dynSetPhiPsi -store -src $objInfo(PDB,src) -res $objInfo(PDB,resID) \
                        -phi $bestPhi -psi $bestPsi

           set newForce [getForce]
           set str      [getForceReport]

           set dPhi [expr $phi - $bestPhi]
           set dPsi [expr $psi - $bestPsi]

           if {$accepted} \
              {
               puts \
                "Trial: $objInfo(dyn,trial) SA-Trial: $objInfo(SA,tTrial) \
                 T-Trial: $objInfo(SA,saTrial) T: $objInfo(SA,t) \ 
                 ResID: $objInfo(Ref,resID) DPhi: $dPhi DPsi: $dPsi $str"
              }

           incr objInfo(SA,saTrial) 
           incr objInfo(dyn,trial) 
          }

       set objInfo(SA,t) [expr (1.0 - $objInfo(SA,tFactor))*$objInfo(SA,t)]

       incr objInfo(SA,tTrial) 
      }
}

proc gradAngles {} \
{
   global objInfo 

   set resCount [llength $objInfo(dyn,allowedList)]
   set newForce [getForce]

   while {$objInfo(dyn,trial) < $objInfo(dyn,maxTrials)} \
      {
       incr objInfo(dyn,trial) 

       dynRestore $objInfo(PDB,src)

       if {$objInfo(dyn,interuptFlag)} break

       if {$objInfo(dyn,trial) == 1 || 
            !($objInfo(dyn,trial) % $objInfo(dyn,updateCount))} \
          {
           getForceStatus 0
          }

       set resLoc             [expr int([rand]*$resCount)]
       set objInfo(Ref,resID) [lindex $objInfo(dyn,allowedList) $resLoc]

       set objInfo(dyn,resOffset) \
        [expr $objInfo(Ref,resID) - $objInfo(Ref,res1)]

       set objInfo(PDB,resID)     \
        [expr $objInfo(PDB,res1) + $objInfo(dyn,resOffset)]

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $objInfo(PDB,resID)]

       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set bestForce $newForce
       set bestPhi   $phi
       set bestPsi   $psi
  
       set gradInfo  ""
 
       foreach dPhi {-5 0 5} \
          {
           foreach dPsi {-5 0 5} \
              {
               set newPsi [expr $psi + $dPsi]
               set newPhi [expr $phi + $dPhi]
   
               dynSetPhiPsi -store -src $objInfo(PDB,src) \
                            -res $objInfo(PDB,resID) -phi $newPhi -psi $newPsi
           
               set newForce [getForce]
  
               if {$newForce < $bestForce} \
                  {
                   set bestForce $newForce
                   set bestPhi   $newPhi
                   set bestPsi   $newPsi
                  }

               lappend gradInfo "$dPhi $dPsi $newForce"
              }
          }

       set gInfo    [getPhiPsiByGrad $gradInfo 1]

       set newPhi   [expr $phi + [expr $objInfo(Grad,step)*[lindex $gInfo 0]]]
       set newPsi   [expr $psi + [expr $objInfo(Grad,step)*[lindex $gInfo 1]]]
  
       dynSetPhiPsi -store -src $objInfo(PDB,src) \
                    -res $objInfo(PDB,resID) -phi $newPhi -psi $newPsi

       set newForce [getForce]
  
       if {abs($phi-$newPhi) < 25 && \
            abs($psi-$newPsi) < 25 && \
             $newForce < $bestForce} \
          {
           set bestForce $newForce
           set bestPhi   $newPhi
           set bestPsi   $newPsi
          }
       
       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $objInfo(PDB,resID) \
                    -phi $bestPhi -psi $bestPsi

       set dPhi [expr $phi - $bestPhi]
       set dPsi [expr $psi - $bestPsi]

       set newForce [getForce]
       set str      [getForceReport]

       set  fmt "%4d %s ResID: %3d dPhi: % 3.0f dPsi: % 3.0f"

       puts \
        [format $fmt $objInfo(dyn,trial) $str $objInfo(Ref,resID) $dPhi $dPsi]
      }
}

proc sweepAngles {} \
{
   global objInfo 

   set origForce [getForce]

   while {$objInfo(dyn,trial) < $objInfo(dyn,maxTrials)} \
      {
       incr objInfo(dyn,trial) 

       dynRestore $objInfo(PDB,src)

       if {$objInfo(dyn,interuptFlag)} break

       if {$objInfo(dyn,trial) == 1 || 
            !($objInfo(dyn,trial) % $objInfo(dyn,updateCount))} \
          {
           getForceStatus 0
          }

       set objInfo(dyn,resOffset) \
        [expr int([rand]*($objInfo(Ref,resN) - $objInfo(Ref,res1)))]

       set objInfo(Ref,resID)     \
        [expr $objInfo(Ref,res1) + $objInfo(dyn,resOffset)]

       set objInfo(PDB,resID)     \
        [expr $objInfo(PDB,res1) + $objInfo(dyn,resOffset)]

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $objInfo(PDB,resID)]

       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set bestForce $origForce
       set bestPhi   $phi
       set bestPsi   $psi
   
       foreach dPhi $objInfo(dyn,dPhiList) \
          {
           foreach dPsi $objInfo(dyn,dPsiList) \
              {
               set newPsi [expr $psi + $dPsi]
               set newPhi [expr $phi + $dPhi]
   
               dynSetPhiPsi -store -src $objInfo(PDB,src) \
                            -res $objInfo(PDB,resID) -phi $newPhi -psi $newPsi
           
               set newForce [getForce]
    
               if {$newForce < $bestForce} \
                  {
                   set bestForce $newForce
                   set bestPhi   $newPhi
                   set bestPsi   $newPsi
                  }
              }
          }
   
       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $objInfo(PDB,resID) \
                    -phi $bestPhi -psi $bestPsi

       set dPhi [expr $phi - $bestPhi]
       set dPsi [expr $psi - $bestPsi]

       set newForce [getForce]
       set str      [getForceReport]

       set  fmt "%4d %s ResID: %3d dPhi: % 3.0f dPsi: % 3.0f"

       puts \
        [format $fmt $objInfo(dyn,trial) $str $objInfo(Ref,resID) $dPhi $dPsi]
      }
}

proc sweepAngles2 {} \
{
   global objInfo 

   set origForce [getForce]

   while {$objInfo(dyn,trial) < $objInfo(dyn,maxTrials)} \
      {
       incr objInfo(dyn,trial) 

       dynRestore $objInfo(PDB,src)

       if {$objInfo(dyn,interuptFlag)} break

       if {$objInfo(dyn,trial) == 1 || 
            !($objInfo(dyn,trial) % $objInfo(dyn,updateCount))} \
          {
           getForceStatus 0
          }

       set objInfo(dyn,resOffset) \
        [expr 1 + int([rand]*(-1 + $objInfo(Ref,resN) - $objInfo(Ref,res1)))]

       set objInfo(Ref,resID)     \
        [expr $objInfo(Ref,res1) + $objInfo(dyn,resOffset)]

       set objInfo(PDB,resID)     \
        [expr $objInfo(PDB,res1) + $objInfo(dyn,resOffset)]

       set resID  $objInfo(PDB,resID)
       set resIDM [expr $resID - 1]

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $resID]

       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $resIDM]

       set phiM  [lindex $aInfo 0]
       set psiM  [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set bestForce $origForce
       set bestPhi   $phi
       set bestPsi   $psi
       set bestPhiM  $phiM
       set bestPsiM  $psiM
   
       foreach dPhiM {-5 -1 0 1 5} \
          {
           foreach dPsiM {-5 -1 0 1 5} \
              {
       foreach dPhi {-5 -1 0 1 5} \
          {
           foreach dPsi {-5 -1 0 1 5} \
              {
               set newPsi  [expr $psi  + $dPsi]
               set newPhi  [expr $phi  + $dPhi]
               set newPsiM [expr $psiM + $dPsiM]
               set newPhiM [expr $phiM + $dPhiM]
   
               dynSetPhiPsi -store -src $objInfo(PDB,src) \
                            -res $resID -phi $newPhi -psi $newPsi

               dynSetPhiPsi -store -src $objInfo(PDB,src) \
                            -res $resIDM -phi $newPhiM -psi $newPsiM
           
               set newForce [getForce]
    
               if {$newForce < $bestForce} \
                  {
                   set bestForce $newForce
                   set bestPhi   $newPhi
                   set bestPsi   $newPsi
                   set bestPhiM  $newPhiM
                   set bestPsiM  $newPsiM
                  }
              }
          }
              }
          }
   
       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $resID \
                    -phi $bestPhi -psi $bestPsi

       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $resIDM \
                    -phi $bestPhiM -psi $bestPsiM

       set dPhi  [expr $phi - $bestPhi]
       set dPsi  [expr $psi - $bestPsi]
       set dPhiM [expr $phiM - $bestPhiM]
       set dPsiM [expr $psiM - $bestPsiM]

       set newForce [getForce]
       set str      [getForceReport]

       set  fmt \
        "%4d %s ResID: %3d dPhi: % .0f dPsi: % .0f dPhi: % .0f dPsi: % .0f"

       puts \
        [format $fmt $objInfo(dyn,trial) \
                     $str $objInfo(Ref,resID) $dPhi $dPsi $dPhiM $dPsiM]
      }
}

proc hybridAngles {} \
{
   global objInfo 

   set origForce [getForce]

   while {$objInfo(dyn,trial) < $objInfo(dyn,maxTrials)} \
      {
       incr objInfo(dyn,trial) 

       dynRestore $objInfo(PDB,src)

       if {$objInfo(dyn,interuptFlag)} break

       if {$objInfo(dyn,trial) == 1 || 
            !($objInfo(dyn,trial) % $objInfo(dyn,updateCount))} \
          {
           getForceStatus 0
          }

       set objInfo(dyn,resOffset) \
        [expr 1 + int([rand]*(-1 + $objInfo(Ref,resN) - $objInfo(Ref,res1)))]

       set objInfo(Ref,resID)     \
        [expr $objInfo(Ref,res1) + $objInfo(dyn,resOffset)]

       set objInfo(PDB,resID)     \
        [expr $objInfo(PDB,res1) + $objInfo(dyn,resOffset)]

       set resID  $objInfo(PDB,resID)
       set resIDM [expr $resID - 1]

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $resID]

       set phi   [lindex $aInfo 0]
       set psi   [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set aInfo [dynGetPhiPsi -src $objInfo(PDB,src) -res $resIDM]

       set phiM  [lindex $aInfo 0]
       set psiM  [lindex $aInfo 1]

       if {$phi > 999 || $psi > 999} continue

       set bestForce $origForce
       set bestPhi   $phi
       set bestPsi   $psi
       set bestPhiM  $phiM
       set bestPsiM  $psiM
   
       foreach dPhiM {-180 -10 0 10 180} \
          {
           foreach dPsiM {-180 -10 0 10 180} \
              {
               set gradInfo  ""

               foreach dPhi {-5 0 5} \
                  {
                   foreach dPsi {-5 0 5} \
                      {
                       set newPsi  [expr $psi  + $dPsi]
                       set newPhi  [expr $phi  + $dPhi]
                       set newPsiM [expr $psiM + $dPsiM]
                       set newPhiM [expr $phiM + $dPhiM]
   
                       dynSetPhiPsi -store -src $objInfo(PDB,src) \
                                    -res $resID -phi $newPhi -psi $newPsi

                       dynSetPhiPsi -store -src $objInfo(PDB,src) \
                                    -res $resIDM -phi $newPhiM -psi $newPsiM
           
                       set newForce [getForce]
    
                       if {$newForce < $bestForce} \
                          {
                           set bestForce $newForce
                           set bestPhi   $newPhi
                           set bestPsi   $newPsi
                           set bestPhiM  $newPhiM
                           set bestPsiM  $newPsiM
                          }

                       lappend gradInfo "$dPhi $dPsi $newForce"
                      }
                  }

               set gInfo    [getPhiPsiByGrad $gradInfo 1]

               set newPhi   [expr $phi + [lindex $gInfo 0]]
               set newPsi   [expr $psi + [lindex $gInfo 1]]
 
               dynSetPhiPsi -store -src $objInfo(PDB,src) \
                            -res $resID -phi $newPhi -psi $newPsi

               set newForce [getForce]
 
               if {$newForce < $bestForce} \
                  {
                   set bestForce $newForce
                   set bestPhi   $newPhi
                   set bestPsi   $newPsi
                   set bestPhiM  $newPhiM
                   set bestPsiM  $newPsiM
                  }
              }
          }
   
       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $resID \
                    -phi $bestPhi -psi $bestPsi

       dynSetPhiPsi -store -src $objInfo(PDB,src) -res $resIDM \
                    -phi $bestPhiM -psi $bestPsiM

       set dPhi  [expr $phi - $bestPhi]
       set dPsi  [expr $psi - $bestPsi]
       set dPhiM [expr $phiM - $bestPhiM]
       set dPsiM [expr $psiM - $bestPsiM]

       set newForce [getForce]
       set str      [getForceReport]

       set  fmt \
        "%4d %s ResID: %3d dPhiM: % .0f dPsiM: % .0f dPhi: % .0f dPsi: % .0f"

       puts \
        [format $fmt $objInfo(dyn,trial) \
                     $str $objInfo(Ref,resID) $dPhiM $dPsiM $dPhi $dPsi]
      }
}

proc dynReset {} \
{
   initForce
}

proc dynStop {} \
{
   global objInfo

   set objInfo(dyn,interuptFlag) 1
}

proc dynQuit {} \
{
   getForceStatus 0
   exit
}

proc dynStart {} \
{
   global objInfo

   if {![info exists objInfo(dyn,allowedList)]} \
      {
       for {set id $objInfo(Ref,res1)} {$id <= $objInfo(Ref,resN)} {incr id} \
           {
            lappend objInfo(dyn,allowedList) $id
           }
      }

   if {![info exists objInfo(dyn,excludeList)]} \
      {
       set objInfo(dyn,excludeList) ""
      }

   if {![info exists objInfo(saupe,res1)]} \
      {
       set objInfo(saupe,res1) $objInfo(Ref,res1)
      }

   if {![info exists objInfo(saupe,resN)]} \
      {
       set objInfo(saupe,resN) $objInfo(Ref,resN)
      }

   if {![info exists objInfo(saupe,a1)]} \
      {
       set objInfo(saupe,res1) $objInfo(PDB,res1)
      }

   if {![info exists objInfo(saupe,aN)]} \
      {
       set objInfo(saupe,aN) $objInfo(PDB,resN)
      }

   showMol

   set objInfo(dyn,interuptFlag) 0

   if {"$objInfo(dyn,startProc)" != "dynStart"} \
      {
       $objInfo(dyn,startProc)
      } \
   else \
      {
       $objInfo(dyn,protocol)
      }
}

initForce

if {$objInfo(dyn,autoStart)} dynStart
