#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Find the Medoid Spectrum (Here, by Smallest Average RMS) in a Group:"
    puts " -in      inList   \[ft/*\]  List of Input Spectra."
    puts " -verb                     Verbose Mode ON."
    puts "Region of Interest (readROI Options) Should be Last on Command Line:"
    puts " -regInfo roiInfo  \[Auto]"
    exit 0
   }

set verbose 0

set inList   [lsort -dictionary [glob -nocomplain ft/*]]
set regInfo Auto

getListArgD $ARGV -in inList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

set loc [flagLoc $ARGV -regInfo]

if {$loc} {set regInfo [lrange $ARGV [expr $loc + 1] end]}

if {![strcasecmp $regInfo Auto]} {set regInfo ""}

#
#-----------------------------------------------------------------------------------------------

set nameSize 0

foreach inName $inList \
   {
    if {[string length $inName] > $nameSize} {set nameSize [string length $inName]}

    if {![file exists $inName]} \
       {
        if {$verbose} \
           {
            puts "FindMedoid. Error Finding Input Spectrum $inName"
            exit
           } \
        else \
           {
            puts -nonewline "error finding $inName"
           }

        exit 1
       }
   }

set fmt " %3d. Spectrum I: %-${nameSize}s J: %-${nameSize}s Scale: % .4e RMS: %.4e"

set n [llength $inList]
set k 1

if {$n == 0} {exit 0}

if {$n == 1} \
   {
    set bestSpec [lindex $inList 0]
   } \
else \
   {
    for {set i 0} {$i < $n - 1} {incr i} \
       {
        set inNameI [lindex $inList $i]

        for {set j [expr $i + 1]} {$j < $n} {incr j} \
           {
            set inNameJ [lindex $inList $j]

            set sInfo [eval exec findScale.tcl -in1 $inNameI -in2 $inNameJ -rms $regInfo]
            set scale [getNamedItem scale $sInfo]
            set rms   [getNamedItem rms   $sInfo]

            if {$verbose} {puts [format $fmt $k $inNameI $inNameJ $scale $rms]}

            lappend rmsInfo($inNameI) $rms
            lappend rmsInfo($inNameJ) $rms

            lappend nameInfo($inNameI) $inNameJ
            lappend nameInfo($inNameJ) $inNameI

            incr k
           }
       }

    if {$verbose} {puts ""} 

    set fmt " %3d. Spectrum: %-${nameSize}s Average RMS: %.4e N: %3d"

    set bestSpec [lindex $inList 0]
    set bestRMS  0.0

    set k 1

    foreach inNameI $inList \
       {
        set rms [getAvg $rmsInfo($inNameI)]

        if {$verbose} {puts [format $fmt $k $inNameI $rms [llength $rmsInfo($inNameI)]]}

        if {$k == 1 || $rms < $bestRMS} \
           {
            set bestSpec $inNameI
            set bestRMS  $rms
           }

        incr k
       }
   }

if {$verbose} \
   {
    puts ""
    puts "Medoid Spectrum: $bestSpec"
   } \
else \
   {
    puts -nonewline $bestSpec
   }

exit 0
