#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "List Sequence Location Probabilties for Groups of Residues (2022.164.11.43):"
    puts " -in     resProbName \[resProb.tab\]  Residue Type Probability Table."
    puts " -id     idList      \[None\]         List of Sequential Residue IDs to Test."
    puts " -best   bestCount   \[All\]          Number of Best Locations to Report, or Keyword All."
    puts " -nosort                              Suppress Sorting by Probabilty (Sets -best All)."
    exit 1
   }

set inName    resProb.tab
set idList    None 
set bestCount All 
set sortFlag  1

getArgD $ARGV -in   inName
getArgD $ARGV -best bestCount

getListArgD $ARGV -id idList

if {![strcasecmp $idList None]} {set idList ""}

if {[flagLoc $ARGV -nosort]} \
   {
    set sortFlag  0
    set bestCount All
   }

#
#--------------------------------------------------------------------------------------

initResNames

if {![file exists $inName]} \
   {
    puts stderr "findResLoc. Error finding input file $inName"
    exit 1
   }

set r1 0
set rN 0

set t       [gdbRead table -in $inName]
set eList   [gdbGetInfo $t -parms entries]
set seqList [getSeqList $t r1 rN]
set n       [llength $idList]

if {![strcasecmp $bestCount All]} {set bestCount n}

set resCount [expr 1 + $rN - $r1]

foreach resName $seqList \
   {
    set resNameU [string toupper $resName]

    if {![info exists countInfo($resNameU)]} {set countInfo($resNameU) 0}
    incr countInfo($resNameU)
   }
 
foreach e $eList \
   {
    set id      [gdbGet $e INDEX]
    set resName [gdbGet $e RESNAME]
    set rank    [gdbGet $e RANK]
    set p       [gdbGet $e P]

    set pInfo($id,$resName)    $p
    set rankInfo($id,$resName) $rank
   }

set missCount 0
set pList ""

for {set resID $r1} {$resID <= [expr 1 + $rN - $n]} {incr resID} \
   {
    set p 1.0

    for {set off 0} {$off < $n} {incr off} \
       {
        set resName  [lindex $seqList [expr $resID + $off - $r1]]
        set resNameU [string toupper $resName]

        set id       [lindex $idList $off]
       
        if {[info exists pInfo($id,$resName)]} \
           {
            set thisP $pInfo($id,$resName)
            if {!$countInfo($resNameU)} {set thisP 0.0}
           } \
        else \
           {
            set thisP [expr double($countInfo($resNameU)/$resCount]
            incr missCount
           }

        set p [expr $p*$thisP]
       }

    lappend pList $p
   }

if {$sortFlag} \
   {
    set locList [lreverse [getVecSortLoc $pList]]
   } \
else \
   {
    set locList [getIota 1 1 [expr 1 + $rN - ($r1 + $n - 1)]]
   }

set sum [getSum $pList]

set thisCount 0

foreach loc $locList \
   {
    incr thisCount

    set resID [expr $loc + $r1 - 1]

    set rankList ""
    set resList  ""
    set p 1.0

    for {set off 0} {$off < $n} {incr off} \
       {
        set resName [lindex $seqList [expr $resID + $off - $r1]]

        lappend resList $resName

        set id [lindex $idList $off]
       
        if {[info exists pInfo($id,$resName)]} \
           {
            set p [expr $p*$pInfo($id,$resName)] 
            lappend rankList $rankInfo($id,$resName)
           } \
        else \
           {
            lappend rankList 0
           } 
       }

    puts -nonewline [format " %4d %.5f %s" $resID [safeDiv $p $sum] $resList]

    foreach rank $rankList {puts -nonewline [format " %2d" $rank]}

    puts ""

    if {$thisCount >= $bestCount} break
   }

exit 0
