#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Find Time-Domain Noise Level:"
   echo " -in  tdName  [fid/test%03d.fid] Time-Domain Input."
   echo " -tmp tmpDir  [tmpnoise]         Temporary Directory."
   echo " -reg regList [None]             Noise Region (x1 xn y1 ...) or Keyword None."
   echo "Processing Options (Should Be Last on Command Line):"
   echo " -procArgs argList [None]        Options for basicFT3.com etc."
   exit 0
endif

set thisArgv = ($argv)
set procArgs = ""

set loc = (`flagLoc $argv -procArgs`)
set n   = $#argv

if ($loc) then
   @ loc++
   set procArgs = ($argv[$loc-$n])
   set thisArgv = ($argv[1-$loc])
endif

set inName = (`getArgD $thisArgv -in    fid/test%03d.fid`)
set tmpDir = (`getArgD $thisArgv -tmp   tmpnoise`)
set iseed  = (`getArgD $thisArgv -iseed 7827313`)

set regList = (`getListArgD $thisArgv -reg -- None`) 

#
#-----------------------------------------------------------------------------------

set thisName  = (`nmrPrintf $inName 1 1`)
set specNoise = None

if (!(-e $thisName)) then
   nmrPrintf -stderr "%s\n" "FindTDNoise Error Finding Time-Domain Input $inName"
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -dim NULL_DIM -fmt %.0f`)

set origDir = (`pwd`)

/bin/rm -rf $tmpDir

mkdir $tmpDir

if ($dimCount == 3) then
   set tdNoiseName = noisefid/test%03d.fid
   set ftNoiseName = noiseft/test%03d.dat
   set tdName      = fid/test%03d.dat
   set ftName      = ft/test%03d.dat

   xyz2pipe -in $inName | pipe2xyz -out $tmpDir/$tdName
 
   xyz2pipe -in $inName \
   | nmrPipe -fn SET -c 0.0 \
   | nmrPipe -fn ADD -g 1.0 -iseed $iseed\
   | pipe2xyz -out $tmpDir/$tdNoiseName
else
   set tdNoiseName = noisefid.fid
   set ftNoiseName = noiseft.dat
   set tdName      = fid.dat 
   set ftName      = ft.dat

   /bin/cp $inName $tmpDir/$tdName

   nmrPipe -in $inName \
   | nmrPipe -fn SET -c 0.0 \
   | nmrPipe -fn ADD -g 1.0 -iseed $iseed\
     -out $tmpDir/$tdNoiseName -ov
endif   

cd $tmpDir

basicFT${dimCount}.com -in $tdNoiseName -out $ftNoiseName $procArgs >& noiseproc.log
basicFT${dimCount}.com -in $tdName      -out $ftName      $procArgs >& noiseproc.log

set tdNoiseSyn = (`specStat.com -brief -in $tdNoiseName -stat vRMS`)
set ftNoiseSyn = (`specStat.com -brief -in $ftNoiseName -stat vEstNoise`)
set specNoise  = (`specStat.com -brief -in $ftName      -stat vEstNoise`)

set noiseRatio = (`MATH "div( $tdNoiseSyn, $ftNoiseSyn )"`)
set tdNoise    = (`MATH "$noiseRatio*$specNoise"`)

set regNoise   = None 
set tdNoiseReg = None

if (`string csame $regList None` == 0) then
   set n = (`IMATH "2*$dimCount"`)

   if ($n != $#regList) then
      nmrPrintf -stderr "%s\n" "FindTDNoise Error: -reg needs $n values for ${dimCount}D data."
      exit 1
   endif

   set thisList = ($regList 0% 100% 0% 100% 0% 100% 0% 100%)

   set x1 = $thisList[1]
   set xn = $thisList[2]

   set y1 = $thisList[3]
   set yn = $thisList[4]

   set z1 = $thisList[5]
   set zn = $thisList[6]

   set a1 = $thisList[7]
   set an = $thisList[8]

   set regNoise   = (`specStat.com -brief -in $ftName -stat vRMS -x1 $x1 -xn $xn -y1 $y1 -yn $yn -z1 $z1 -zn $zn -a1 $a1 -an $an`)
   set tdNoiseReg = (`MATH "$noiseRatio*$regNoise"`)
endif

nmrPrintf "%s" "tdNoise $tdNoise tdNoiseReg $tdNoiseReg specNoise $specNoise regNoise $regNoise tdSyntheticNoise $tdNoiseSyn ftSyntheticNoise $ftNoiseSyn noiseRatio $noiseRatio"

cd $origDir

/bin/rm -rf $tmpDir

exit 0
