#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Fit evolutions from a "series" table to exp(t2)cos(j).

set specList    [lsort -dictionary [glob -nocomplain ft/*]]
set inName      nlin.tab
set pInit0      {1.0 0.1  1.0}
set pInit1      {1.0 0.1 30.0}
set gc          20
set noiseMinPct 3.0

#
#----------------------------------------------------------------

proc modelProc {} \
{
   global xyCount xList pList yModel qList dList nList eList
   global amp t2Sec jHz dAmp dT2Sec dJHz currentRMS reportFlag

   set amp   [lindex $pList 0]
   set t2Sec [lindex $pList 1]
   set jHz   [lindex $pList 2]

   set PI    3.14159265
   set t2Sec [expr abs($t2Sec)]
   set jHz   [expr abs($jHz)]

   set resList ""
   set chiList ""
   set i       0

   foreach t $xList \
      {
       set y     [expr $amp*exp( -1.0*$t/$t2Sec )*cos( $PI*$jHz*$t )]

       set yObs  [lindex $qList $i]
       set sigma [lindex $nList $i]
       set d     [expr $yObs - $y]

       lappend resList $d
       lappend chiList [expr $d/$sigma] 
       lappend yModel $y

       incr i
      }

   set currentRMS  [getRMS $resList]

   if {$reportFlag} \
      {
       set dAmp   [lindex $eList 0]
       set dT2Sec [lindex $eList 1]
       set dJHz   [lindex $eList 2]
      }
}

proc getZ_A { entry specCount } \
{
   set z ""

   for {set i 0} {$i < $specCount} {incr i} \
      {
       lappend z [gdbGet $entry Z_A$i]
      }

   return $z
}

wm iconify .
 
set specCount [llength $specList]
set inTab     [gdbRead table -in $inName]
set eList     [gdbGetInfo $inTab -parms entries]

set noiseList ""
set tauList   ""

foreach specName $specList \
   {
    set noise [estSpecNoise $specName]
    set tau   [getTau $specName]

    lappend tauList   $tau
    lappend noiseList $noise

    puts stderr [format "%s: Noise: %.4e Tau: %.5f" $specName $noise $tau]
   } 

foreach entry $eList \
   {
    set pkID [gdbGet $entry INDEX]
    set hi   [gdbGet $entry HEIGHT]
    set y    [getZ_A $entry $specCount]

    set thisNoise ""
    set posCount  0

    foreach noise $noiseList \
       {
        set q [expr $noise/$hi]

        if {$q < $noiseMinPct/100.0} {set q [expr $noiseMinPct/100.0]} 

        lappend thisNoise $q
       }

    foreach val $y {if {$val > 0.0} {incr posCount}}

    if {$posCount == $specCount} \
       {
        set p $pInit0 
       } \
    else \
       {
        set p $pInit1 
       }

    set result \
     [modelXY -x $tauList -y $y -p $p \
              -macro modelProc -noise $thisNoise -gc $gc -silent -noverb]

    set fmt  \
     "Peak: %3d RMS: %.4e Amp: % .4f T2 Sec: %.4f +/- %.4f J: %8.3f +/- %.3f"

    puts [format $fmt $pkID $currentRMS $amp $t2Sec $dT2Sec $jHz $dJHz]

    for {set i 0} {$i < $xyCount} {incr i} \
       {
        set vObs   [lindex $dList $i]
        set vCalc  [lindex $yModel $i]
        set vSigma [lindex $thisNoise $i]

        puts [format "  % .5f % .5f % .5f" $vObs $vCalc $vSigma]
       }
   }

exit
