
#
# Modules for manipulating PDB data.

proc initResNames {} \
{
    global resNames resTypes initResNameFlag

    set initResNameFlag  1

    set resNames(exists) 1
    set resTypes(exists) 1

    set resNames(ALA) A
    set resNames(ARG) R
    set resNames(ASN) N
    set resNames(ASP) D
    set resNames(CYS) C
    set resNames(cys) c
    set resNames(GLN) Q
    set resNames(GLU) E
    set resNames(GLY) G
    set resNames(HIS) H
    set resNames(his) h
    set resNames(HIH) #
    set resNames(ILE) I
    set resNames(LEU) L
    set resNames(LYS) K
    set resNames(MET) M
    set resNames(PHE) F
    set resNames(PRO) P
    set resNames(SER) S
    set resNames(THR) T
    set resNames(TRP) W
    set resNames(TYR) Y
    set resNames(VAL) V

    set resNames(NH3) <
    set resNames(COO) >

    set resTypes(A) ALA
    set resTypes(R) ARG
    set resTypes(N) ASN
    set resTypes(D) ASP
    set resTypes(c) cys 
    set resTypes(C) CYS
    set resTypes(Q) GLN
    set resTypes(E) GLU
    set resTypes(G) GLY
    set resTypes(H) HIS
    set resTypes(h) his 
    set resTypes(#) HIH
    set resTypes(I) ILE
    set resTypes(L) LEU
    set resTypes(K) LYS
    set resTypes(M) MET
    set resTypes(F) PHE
    set resTypes(P) PRO
    set resTypes(S) SER
    set resTypes(T) THR
    set resTypes(W) TRP
    set resTypes(Y) TYR
    set resTypes(V) VAL

    set resTypes(U) URA
    set resTypes(<) NH3
    set resTypes(>) COO
}

#
# Initialize info for chi angle determination.

proc initChiInfo {} \
{
    global chiInfo

    set chiInfo(exists) 1

    set chiInfo(ALA) "N CA CB HB1"
    set chiInfo(ARG) "N CA CB CG"
    set chiInfo(ASN) "N CA CB CG"
    set chiInfo(ASP) "N CA CB CG"
    set chiInfo(CYS) "N CA CB SG"
    set chiInfo(cys) "N CA CB SG"
    set chiInfo(GLN) "N CA CB CG"
    set chiInfo(GLU) "N CA CB CG"
    set chiInfo(GLY) ""
    set chiInfo(HIS) "N CA CB CG"
    set chiInfo(HIH) "N CA CB CG"
    set chiInfo(ILE) "N CA CB CG1"
    set chiInfo(LEU) "N CA CB CG"
    set chiInfo(LYS) "N CA CB CG"
    set chiInfo(MET) "N CA CB CG"
    set chiInfo(PHE) "N CA CB CG"
    set chiInfo(PRO) "N CA CB CG"
    set chiInfo(SER) "N CA CB CG"
    set chiInfo(THR) "N CA CB OG1"
    set chiInfo(TRP) "N CA CB CG"
    set chiInfo(TYR) "N CA CB CG"
    set chiInfo(VAL) "N CA CB CG1"
}

#
# Get 3-letter code from 1-letter or 3-letter resName.

proc getResName3 args \
{
   global resTypes resNames

   set unkRes  "?"
   set resName1 [lindex $args 0]

   if {[info exists resNames($resName1)]} {return $resName1}

   if {[llength $args] > 1} {set unkRes [lindex $args 1]}

   if {[info exists resTypes($resName1)]} \
      {
       return $resTypes($resName1)
      }

   return $unkRes 
}

#
# Get 1-letter code from 1-letter or 3-letter resName.

proc getResName1 args \
{
   global resNames resTypes

   set unkRes  "?"
   set resName3  [lindex $args 0]
   set resName3U [string toupper $resName3]

   if {[info exists resTypes($resName3)]}  {return $resName3}

   if {[llength $args] > 1} {set unkRes [lindex $args 1]}
   
   if {[info exists resNames($resName3)]}  {return $resNames($resName3)}
   if {[info exists resNames($resName3U)]} {return $resNames($resName3U)}

   return $unkRes 
}

#
# Return three-letter DNA/RNA code if possible.

proc getNAResName3 { resNameN } \
{
   switch -exact [string toupper $resNameN] \
      {
       A {return ADE}
       C {return CYT}
       G {return GUA}
       T {return THY}
       U {return URI}
       RADE {return ADE}
       RCYT {return CYT}
       RGUA {return GUA}
       RURI {return URI}
       DADE {return ADE}
       DCYT {return CYT}
       DGUA {return GUA}
       DTHY {return THY}
      }

   return $resNameN
}

#
# Return 1 if table has SEQRES info.
# Return 2 if table also has DBREF info.
# Return 0 otherwise

proc hasSEQRES { pdbTab } \
{
    set hasSR  0
    set hasDB  0

    set tPtr [gdbGetInfo $pdbTab -parms text]
    if {![llength $tPtr]} {return $status}

    while {$tPtr} \
       {
        set line [gdbGetInfo $tPtr -parms value]
        set s    [string toupper $line]

        set key None

        scan $s %s key

        if {![strcasecmp $key SEQRES]} \
           {
            set hasSR 1
           } \
        elseif {![strcasecmp $key DBREF]} \
           {
            set hasDB 1
           }

        set tPtr [gdbNext $tPtr]
       }

    if {$hasSR} \
       {
        if {$hasDB} {return 2} 
        return 1
       }

    return 0 
}

proc getSEQRES args \
{
    set pdbTab [lindex $args 0]

    if {[llength $args] > 1} {upvar [lindex $args 1] r1}
    if {[llength $args] > 2} {upvar [lindex $args 2] rN}

    set firstRes None
    set lastRes  None
    set sList    ""

    set r1       1
    set rN       0
    set nRes     0

    set tPtr [gdbGetInfo $pdbTab -parms text]
    if {![llength $tPtr]} {return $sList}

    while {$tPtr} \
       {
        set line [gdbGetInfo $tPtr -parms value]
        set s    [string toupper $line]

        set key None

        scan $s %s key

        if {![strcasecmp $key SEQRES]} \
           {
            set nRes [string trim [string range $line 13 16]]
            set str  [string trim [string range $line 19 69]]

            foreach s $str {lappend sList [getResName1 $s]}
           } \
        elseif {![strcasecmp $key DBREF] && ![strcasecmp $firstRes None]} \
           {
            set firstRes [string trim [string range $line 14 17]]
            set lastRes  [string trim [string range $line 20 23]]
           }

        set tPtr [gdbNext $tPtr]
       }

   set nS [llength $sList]

   if {!$nS} {return $sList}

   if {![strcasecmp $firstRes None]} \
      {
       set r1 1
       set rN [expr $r1 + $nS - 1]
      } \
   else \
      {
       if {$nS == [expr 1 + $lastRes - $firstRes]} \
          {
           set r1 $firstRes
           set rN $lastRes
          } \
       else \
          {
           set r1 $firstRes 
           set rN [expr $r1 + $nS - 1]
          }
      }

   return $sList
}

#
# Extract protein sequence from DATA statements of a table;
# data are returned as an array.
 
proc getSequence { tab resArrayPtr resCountPtr {seqID None} } \
{
    upvar $resArrayPtr resArray
    upvar $resCountPtr resCount

    if {![strcasecmp $seqID None]} \
       {
        set prefix ""
       } \
    else \
       {
        set prefix "$seqID,"
       }

    set resCount 0

    set tPtr [gdbGetInfo $tab -parms text]
    if {![llength $tPtr]} return

    while {$tPtr} \
       {
        set line [gdbGetInfo $tPtr -parms value]
        set s    [string toupper $line]

        scan $s %s key1

        if {![strcmp $key1 DATA]} {scan $s "%s %s" key1 key2}

        if {![strcmp $key1 DATA] && ![strcmp $key2 SEQUENCE]} \
           {
            set seqText [lrange $line 2 end]
            set count   [string length $seqText]

            for {set i 0} {$i < $count} {incr i} \
               {
                set c [string index $seqText $i] 
           
                if {![strcmp $c "/"]} continue
 
                if {"$c" >= "!" && "$c" <= "~"} \
                   {
                    incr resCount
                    set  resArray($prefix$resCount) $c
                   }
               }
           }

        set tPtr [gdbNext $tPtr]
       }
}

#
# Read generic protein chemical shift information.

proc getRefCS { refName refInfoPtr } \
{
   upvar $refInfoPtr refInfo

   global initResNameFlag env 

   if {![info exists $initResNameFlag]} {initResNames}

   if {![strcasecmp $refName Auto]} \
      {
       if {![info exists env(NMRTXT)]} {return 1} 
       set refName $env(NMRTXT)/refCS.tab
      }

   if {![file exists $refName]} {return 2}

   set t     [gdbRead table -in $refName]
   set eList [gdbGetInfo $t -parms entries]

   set vInfo {ATOMTYPE COUNT MIN_PPM MAX_PPM AVG_PPM STDDEV_PPM}
   set rInfo {atomType count minVal  maxVal  avg     dev}
   set n     [llength $rInfo]

   foreach e $eList \
      {
       set resName  [gdbGet $e RESNAME]
       set resType  [gdbGet $e RESTYPE]
       set atomName [gdbGet $e ATOMNAME]

       for {set i 0} {$i < $n} {incr i} \
          {
           set vName [lindex $vInfo $i]
           set rName [lindex $rInfo $i]
           set val   [gdbGet $e $vName]

           set refInfo($resName,$atomName,$rName) $val 
           set refInfo($resType,$atomName,$rName) $val 
          }       
      }

   gdbFree $t

   return 0
}

#
# Return 1 if a shift is inside the reference range, 0 if outside, -1 if it is unknown.

proc refCSTest { resName atomName shift refInfoPtr } \
{
   upvar $refInfoPtr refInfo

   set refInfo(minVal) 0.0
   set refInfo(maxVal) 0.0

   if {![info exists refInfo($resName,$atomName,count)]} \
      {
       set resName  [string toupper $resName]
       set atomName [string toupper $atomName] 

       if {![info exists refInfo($resName,$atomName,count)]} {return -1} 
      }

   set refInfo(minVal) $refInfo($resName,$atomName,minVal)
   set refInfo(maxVal) $refInfo($resName,$atomName,maxVal)

   if {$shift < $refInfo($resName,$atomName,minVal) || 
        $shift > $refInfo($resName,$atomName,maxVal)} {return 0}

   return 1 
} 

#
# Return 1 if table has DATA SEQUENCE info.
# Return 2 if table also has DATA FIRST_RESID info.
# Return 0 otherwise

proc hasSeqData { t } \
{
    set tPtr [gdbGetInfo $t -parms text]
    if {![llength $tPtr]} {return 0}

    set hasFirst 0
    set hasSeq   0

    while {$tPtr} \
       {
        set line [gdbGetInfo $tPtr -parms value]
        set s    [string toupper $line]

        set key1 None
        set key2 None

        scan $s %s key1

        if {![strcmp $key1 DATA]} {scan $s "%s %s" key1 key2}

        if {![strcmp $key1 DATA] && ![strcmp $key2 FIRST_RESID]} \
           {
            set hasFirst 1
           } \
        elseif {![strcmp $key1 DATA] && ![strcmp $key2 SEQUENCE]} \
           {
            set hasSeq   1
           } 

        set tPtr [gdbNext $tPtr]
       }

   if {$hasSeq} \
      {
       if {$hasFirst} {return 2}
       return 1
      }

   return 0 
}

#
# Extract protein sequence data from table or PDB.
# Use DATA SEQUENCE info if possible.
# Use RESID/RESNAME info otherwise.
# Complete with DBREF/SEQRES info if possible.

proc getTabSeqList args \
{
   set t       [lindex $args 0]
   set r1Ptr   [lindex $args 1]
   set rNPtr   [lindex $args 2]
   set modePtr [lindex $args 3]

   upvar $r1Ptr   r1
   upvar $rNPtr   rN
   upvar $modePtr mode

   set seqResFlag 0

   if {[flagLoc $args -seqres]}   {set seqResFlag 1}
   if {[flagLoc $args -noseqres]} {set seqResFlag 0}

   set mode       None

   set seqResList ""
   set seqList    ""
   set resList    ""

   set r1      0
   set rN      0

   set thisR1  0
   set thisRN  0

   if {[hasSeqData $t]} \
      {
       set seqList [getSeqList $t thisR1 thisRN]

       set r1   $thisR1
       set rN   $thisRN
       set mode data

       return $seqList
      }

   if {$seqResFlag} \
      {
       if {[hasSEQRES $t]} \
          {
           set seqResList [getSEQRES $t thisR1 thisRN]
    
           set r1   $thisR1
           set rN   $thisRN
           set mode seqres
          }
      }

   set vList  [gdbGetInfo $t -parms varNames]
   set pList  ""

   foreach prefix {"" _I _J _K _L} \
      {
       set vRESID   RESID$prefix
       set vRESNAME RESNAME$prefix

       if {[lsearch -exact $vList $vRESID]   < 0} continue
       if {[lsearch -exact $vList $vRESNAME] < 0} continue

       lappend pList $prefix
      }

   if {[llength $pList]} \
      {
       set eList [gdbGetInfo $t -parms entries]

       foreach e $eList \
          {
           foreach prefix $pList \
              {
               set resID   [gdbGet $e RESID$prefix]
               set resName [getResName1 [gdbGet $e RESNAME$prefix]]

               lappend thisSeqInfo($resID) $resName
               lappend resList $resID
              }
          }
      } \
   else \
      {
       return $seqResList
      }

   set mode resname

   if {[llength $seqResList]} \
      {
       set mode merge

       lappend resList $r1
       lappend resList $rN

       set i 0

       for {set resID $r1} {$resID <= $rN} {incr resID} \
          {
           lappend thisSeqInfo($resID) [lindex $seqResList $i]
           incr i
          }
      }

   set r1 [getMin $resList]
   set rN [getMax $resList] 

   for {set resID $r1} {$resID <= $rN} {incr resID} \
      {
       if {[info exists thisSeqInfo($resID)]} \
          {
           set uList [listUniq $thisSeqInfo($resID)]

           if {[llength $uList] != 1} \
              {
               lappend seqList X
              } \
           else \
              {
               lappend seqList [lindex $uList 0]
              }
          } \
       else \
          {
           lappend seqList X
          }
      }

   return $seqList
}

#
# Extract protein sequence from DATA statements of a table;
# data are returned as a list.

proc getSeqList args \
{
    set assTab [lindex $args 0]

    if {[llength $args] > 1} {upvar [lindex $args 1] r1}
    if {[llength $args] > 2} {upvar [lindex $args 2] rN}

    set cisFlag 0

    if {[flagLoc $args -cis]} {set cisFlag 1}

    set sList  ""
    set r1     1
    set rN     0
    set nRes   0

    set tPtr [gdbGetInfo $assTab -parms text]
    if {![llength $tPtr]} {return $sList}

    while {$tPtr} \
       {
        set line [gdbGetInfo $tPtr -parms value]
        set s    [string toupper $line]

        set key1 None
        set key2 None

        scan $s %s key1

        if {![strcmp $key1 DATA]} {scan $s "%s %s" key1 key2}

        if {![strcmp $key1 DATA] && ![strcmp $key2 FIRST_RESID]} \
           {
            set r1 [lindex $line 2]
           } \
        elseif {![strcmp $key1 DATA] && ![strcmp $key2 SEQUENCE]} \
           {
            set seqText [lrange $line 2 end]
            set count   [string length $seqText]

            for {set i 0} {$i < $count} {incr i} \
               {
                set c [string index $seqText $i]

                if {![strcmp $c "/"]} \
                   {
                    if {$cisFlag} {lappend sList $c}
                   } \
                elseif {"$c" >= "!" && "$c" <= "~"} \
                   {
                    lappend sList $c
                    incr nRes
                   }
               }
           }

        set tPtr [gdbNext $tPtr]
       }

    set rN [expr $r1 + $nRes - 1]

    return $sList
}

proc getPDBSeqList args \
{
   global resNames initResNameFlag

   set pdbTab [lindex $args 0]

   if {[llength $args] > 1} {upvar [lindex $args 1] r1}
   if {[llength $args] > 2} {upvar [lindex $args 2] rN}

   set r1 0
   set rN 0

   if {![info exists initResNameFlag]} initResNames

   set r1 [getMinRes $pdbTab]
   set rN [getMaxRes $pdbTab]

   set disuList [getPDBDisuList $pdbTab]
   set sList ""

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       if {[lsearch -exact $disuList $res] < 0} \
          {
           set resType [getNthResName $pdbTab $res]
           set resName [getResName1 $resType]
          } \
       else \
          {
           set resType cys
           set resName c
          }

       lappend sList $resName
      }

   return $sList
}

#
# Get the chain ID or segment ID, adjusting for missing values:

proc getChainID { e defChain } \
{
   set chain [gdbGet $e CHAINID]

   if {![strcasecmp $chain "(Null)"]} {return $defChain}
   if {![strcasecmp $chain "None"]}   {return $defChain}
   if {![llength $chain]} {return $defChain}

   return $chain
}

proc getSegID { e defSeg } \
{
   set seg [gdbGet $e SEGID]

   if {![strcasecmp $seg "(Null)"]} {return $defSeg}
   if {![strcasecmp $seg "None"]}   {return $defSeg}
   if {![llength $seg]} {return $defSeg}

   return $seg
}

#
# Return the number of HETATM entries.

proc getHETATMCount { tab } \
{
    set eList [gdbGetInfo $tab -parms entries]
    set n 0

    foreach e $eList \
       {
        set type [gdbGetInfo $e -parms type]
        if {![strcasecmp $type HETATM]} {incr n}
       }

   return $n
}

#
# Center the given PDB data.

proc centerPDB { tab } \
{
   set eList [gdbGetInfo $tab -parms entries]
   set n     [llength $eList]

   set cx 0.0
   set cy 0.0
   set cz 0.0

   if {!$n} {return "0.0 0.0 0.0"}

   foreach e $eList \
      {
       set cx [expr $cx + [gdbGet $e X]]
       set cy [expr $cy + [gdbGet $e Y]]
       set cz [expr $cz + [gdbGet $e Z]]
      }

   set cx [expr $cx/double($n)]
   set cy [expr $cy/double($n)]
   set cz [expr $cz/double($n)]
   
   foreach e $eList \
      {
       set x [expr [gdbGet $e X] - $cx]
       set y [expr [gdbGet $e Y] - $cy]
       set z [expr [gdbGet $e Z] - $cz]

       gdbSet $e X $x
       gdbSet $e Y $y
       gdbSet $e Z $z
      }
    
   return "$cx $cy $cz"
}
 
#
# Find lowest RESID in a given table.

proc getMinRes { assTab } \
{
    set var     [gdbFindByName $assTab RESID]
    set resList [gdbGetInfo $var -parms values]
    set minRes  0

    if {[llength $resList] > 0} \
       { 
        set minRes [lindex $resList 0]
       }

    foreach i $resList \
       {
        if {$i < $minRes} {set minRes $i}
       }

    return $minRes
}

#
# Find highest RESID in a given table.

proc getMaxRes { assTab } \
{
    set var     [gdbFindByName $assTab RESID]
    set resList [gdbGetInfo $var -parms values]
    set maxRes  0

    if {[llength $resList] > 0} \
       { 
        set maxRes [lindex $resList 0]
       }

    foreach i $resList \
       {
        if {$i > $maxRes} {set maxRes $i}
       }

    return $maxRes
}

#
# Find given shift assignment.

proc getAssShift { assTab resID atomName } \
{
   set cond  "RESID == $resID && !strcasecmp( ATOMNAME, \"$atomName\" )"
   set eList [gdbSelect -from $assTab -cond $cond]
   set s     9999.0

   if {[llength $eList]} \
      {
       set entry [lindex $eList 0]
       set s     [gdbGet $entry SHIFT]
      }

    return $s
}

#
# Match a protein subsequence, allows for "*".

proc matchSequence { seq resArrayPtr resCountPtr offset {seqID None} } \
{
    upvar $resArrayPtr resArray
    upvar $resCountPtr resCount

    set mList   ""
    set count   [string length $seq]
    set length  [expr 1 + $resCount - $count]

    if {![strcasecmp $seqID None]} \
       {
        set prefix ""
       } \
    else \
       {
        set prefix "$seqID,"
       }

    for {set i $offset} {$i <= $length} {incr i} \
       {
        set isMatch 1

        for {set j 0} {$j < $count} {incr j} \
           {
            set c1 [string index $seq $j]
            set c2 $resArray($prefix[expr $i + $j])

            if {[strcmp $c1 *] && [strcmp $c1 $c2]} \
               {
                set isMatch 0
                break
               }
           } 

        if {$isMatch} {lappend mList $i}
       }

    return $mList
}

#
# Get backbone angles for resID.

proc getPhiPsiAngles { resID pdbTab } \
{
   set aInfo [getAngles [expr $resID - 1] $pdbTab]

   return $aInfo
}

#
# Get omega angles for resID.

proc getOmegaAngle { resID pdbTab } \
{
   set i 1

   foreach atomInfo {{CA -1} {C -1} {N 0} {CA 0}} \
      {
       set atom     [lindex $atomInfo 0]
       set offset   [lindex $atomInfo 1]
       set thisRes  [expr $resID + $offset]

       set cond "RESID == $thisRes && !strcmp( ATOMNAME, \"$atom\" )"
       set entry [gdbSelect -from $pdbTab -cond $cond]

       if {[llength $entry] != 1} {return 9999.0}

       set e($i) $entry

       incr i
      }

   set omega [pdbAngle $e(1) $e(2) $e(3) $e(4)]

   while {$omega >  180.0} {set omega [expr $omega - 360.0]}
   while {$omega < -180.0} {set omega [expr $omega + 360.0]}

   return $omega
}

#
# Get chi angles for resID.

proc getChiAngle { resID pdbTab } \
{
   global chiInfo

   if {![info exists chiInfo(exists)]} {initChiInfo}

   set resName [getNthResName $pdbTab $resID]

   if {![info exists chiInfo($resName)]} {return 9999.0}
   if {![llength $chiInfo($resName)]}    {return 9999.0}

   set i 1

   foreach atom $chiInfo($resName) \
      {
       set cond "RESID == $resID && !strcmp( ATOMNAME, \"$atom\" )"
       set entry [gdbSelect -from $pdbTab -cond $cond]

       if {[llength $entry] != 1} {return 9999.0}

       set e($i) $entry

       incr i
      }

   set chi [pdbAngle $e(1) $e(2) $e(3) $e(4)]

   return $chi
}

#
# Get the backbone angles associated with a given TRIPLET!.

proc getAngles { resID pdbTab } \
{
   set phi 9999
   set psi 9999

   set r1  ?
   set r2  ?
   set r3  ?

   set  cond  "RESID == $resID && !strcmp( ATOMNAME, 'C' )"
   set  prevC [gdbSelect -from $pdbTab -cond $cond]
   incr resID

   if {[llength $prevC] == 1} \
      {
       set r1 [gdbGet $prevC RESNAME]
      }

   foreach i {N C CA} \
      {
       set cond "RESID == $resID && !strcmp( ATOMNAME, \"$i\" )"
       set $i   [gdbSelect -from $pdbTab -cond $cond]
      }

   if {[llength $N] == 1} \
      {
       set r2 [gdbGet $N RESNAME]
      }

   incr resID
   set  cond  "RESID == $resID && !strcmp( ATOMNAME, 'N' )"
   set  nextN [gdbSelect -from $pdbTab -cond $cond]

   if {[llength $nextN] == 1} \
      {
       set r3 [gdbGet $nextN RESNAME]
      }

   if {[llength "$prevC $N $CA $C"] == 4} \
      {
       set phi [pdbAngle $prevC $N $CA $C]
      }

   if {[llength "$N $CA $C $nextN"] == 4} \
      {
       set psi [pdbAngle $N $CA $C $nextN]
      }

   set rr [list $r1 $r2 $r3]

   return [list $phi $psi $rr]
}

#
# Return the given atom's entry.

proc getAtomEntry { pdbTab resID atomName } \
{
   set cond  "RESID == $resID && !strcmp( ATOMNAME, \"$atomName\" )"
   set entry [gdbSelect -from $pdbTab -cond $cond]

   if {[llength $entry] == 1} \
      {
       return $entry
      }

   return 0
}

#
# Center PDB coords:

proc pdbCenter { pdbTab } \
{
   set eList [gdbTabInfo $pdbTab -parms entries]
   set n     [llength $eList]

   if {!$n} return

   set x0 0.0
   set y0 0.0
   set z0 0.0

   foreach e $eList \
      {
       set x0 [expr $x0 + [gdbGet $e X]]
       set y0 [expr $z0 + [gdbGet $e Y]]
       set z0 [expr $z0 + [gdbGet $e Z]]
      }

   set xAvg [expr -1.0*$x0/double($n)]
   set yAvg [expr -1.0*$y0/double($n)]
   set zAvg [expr -1.0*$z0/double($n)]

   pdbTranslate $pdbTab $xAvg $yAvg $zAvg
}

proc pdbTranslate { pdbTab dx dy dz } \
{
   set eList [gdbTabInfo $pdbTab -parms entries]
   set n     [llength $eList]

   if {!$n} return

   foreach e $eList \
      {
       set x0 [expr $dx + [gdbGet $e X]]
       set y0 [expr $dy + [gdbGet $e Y]]
       set z0 [expr $dz + [gdbGet $e Z]]

       gdbSet $e X $x0
       gdbSet $e Y $y0
       gdbSet $e Z $z0
      }
}

proc pdbRotate { pdbTab rx ry rz } \
{
   set eList [gdbTabInfo $pdbTab -parms entries]
   set n     [llength $eList]

   if {!$n} return

   set r [dynRotate -verb -rx $rx -ry $ry -rz $zr]

   set c00 [lindex $r  0]
   set c01 [lindex $r  1]
   set c02 [lindex $r  2]
   set c03 [lindex $r  3]

   set c10 [lindex $r  4]
   set c11 [lindex $r  5]
   set c12 [lindex $r  6]
   set c13 [lindex $r  7]

   set c20 [lindex $r  8]
   set c21 [lindex $r  9]
   set c22 [lindex $r 10]
   set c23 [lindex $r 11]

   foreach e $eList \
      {
       set x [gdbGet $e X]
       set y [gdbGet $e Y]
       set z [gdbGet $e Z]

       gdbSet $e X [expr $c00*$x + $c01*$y + $c02*$z + $c03]
       gdbSet $e Y [expr $c10*$x + $c11*$y + $c12*$z + $c13]
       gdbSet $e Z [expr $c20*$x + $c21*$y + $c22*$z + $c23]
      }
}

#
# Return the given atom's entries, accounting for # and *.

proc getAtomEntries { pdbTab resID atomName } \
{
   set cond     "RESID == $resID && atomMatch( ATOMNAME, \"$atomName\" )"
   set atomList [gdbSelect -from $pdbTab -cond $cond]

   return $atomList
}

#
# Return the RESNAME of the given resID

proc getNthResName { pdbTab resID } \
{
   set cond  "RESID == $resID"
   set eList [gdbSelect -from $pdbTab -cond $cond]

   if {[llength $eList]} \
      {
       set entry   [lindex $eList 0]
       set resName [gdbGet $entry RESNAME]

       return $resName
      }

   return "?"
}

proc dynGetPDBSeqList args \
{
   global resNames initResNameFlag

   set pdbSrc [lindex $args 0]

   if {[llength $args] > 1} {upvar [lindex $args 1] r1}
   if {[llength $args] > 2} {upvar [lindex $args 2] rN}

   set r1 0
   set rN 0

   if {![info exists initResNameFlag]} initResNames

   set pInfo [dynGetInfo -src $pdbSrc]
   set r1    [lindex $pInfo 0]
   set rN    [lindex $pInfo 1]

   set disuList [dynGetPDBDisuList $pdbSrc] 
   set sList ""

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       if {[lsearch -exact $disuList $res] < 0} \
          {
           set resName3 [dynGetInfo -src $pdbSrc -res $res -resName]
           set resName  [getResName1 $resName3]
          } \
       else \
          {
           set resName3 cys
           set resName  c
          }

       lappend sList $resName
      }

   return $sList
}

proc dynGetPDBSeqStr args \
{
   global resNames initResNameFlag

   set pdbSrc    0
   set chainName None

   if {[llength $args] > 0} {set pdbSrc    [lindex $args 0]}
   if {[llength $args] > 1} {set chainName [lindex $args 1]}

   if {![info exists initResNameFlag]} initResNames

   set disuList [dynGetPDBDisuList $pdbSrc] 
   set str ""

   set pInfo  [dynGetInfo -src $pdbSrc]
   set minRes [lindex $pInfo 0]
   set maxRes [lindex $pInfo 1]

   append str "\n"
   append str "DATA FIRST_RESID $minRes\n"
   append str "\n"

   set j 0

   for {set res $minRes} {$res <= $maxRes} {incr res} \
      {
       if {[lsearch -exact $disuList $res] < 0} \
          {
           if {![strcasecmp $chainName None]} \
              {
               set resName3 [dynGetInfo -src $pdbSrc -res $res -resName]
              } \
           else \
              {
               set resName3 [dynGetInfo -src $pdbSrc -res $res -resName -chain $chainName]
              }

           set resName  [getResName1 $resName3]
          } \
       else \
          {
           set resName3 cys
           set resName  c
          }

       if {!$j} \
          {
           append str "DATA SEQUENCE "
          }

       append str $resName
       incr j

       if {$j == 50} \
          {
           append str "\n"
           set j 0
          } \
       elseif {!($j % 10)} \
          {
           append str " "
          }
      }

   append str "\n"

   return $str
}

proc getPDBSeqStr { pdbTab } \
{
   global resNames initResNameFlag

   if {![info exists initResNameFlag]} initResNames

   set str ""

   set minRes   [getMinRes $pdbTab]
   set maxRes   [getMaxRes $pdbTab]
   set disuList [getPDBDisuList $pdbTab]

   append str "\n"
   append str "DATA FIRST_RESID $minRes\n"
   append str "\n"

   set j 0

   for {set res $minRes} {$res <= $maxRes} {incr res} \
      {
       if {[lsearch -exact $disuList $res] < 0} \
          {
           set resType [getNthResName $pdbTab $res]
           set resName [getResName1 $resType]
          } \
       else \
          {
           set resType cys
           set resName c
          }

       if {!$j} \
          {
           append str "DATA SEQUENCE "
          }

       append str $resName
       incr j

       if {$j == 50} \
          {
           append str "\n"
           set j 0
          } \
       elseif {!($j % 10)} \
          {
           append str " "
          }
      }

   append str "\n"

   return $str
}

proc getPDBDisuList { pdbTab } \
{
   set disuDist 3.0
   set disuList ""

   set vList [gdbGetInfo $pdbTab -parms varNames]

   if {[lsearch -exact $vList X] < 0} {return $disuList}
   if {[lsearch -exact $vList Y] < 0} {return $disuList}
   if {[lsearch -exact $vList Z] < 0} {return $disuList}

   set cond  "!strcmp( RESNAME, 'CYS' ) && !strcmp( ATOMNAME, 'SG' )"
   set eList [gdbSelect -from $pdbTab -cond $cond]
   set n     [llength $eList]

   for {set i 0} {$i < [expr $n - 1]} {incr i} \
      {
       set entryI [lindex $eList $i]

       for {set j [expr $i + 1]} {$j < $n} {incr j} \
          {
           set entryJ [lindex $eList $j]
           set d      [pdbDist $entryI $entryJ]

           if {$d < $disuDist} \
              {
               set resIDI [gdbGet $entryI RESID]
               set resIDJ [gdbGet $entryJ RESID]
   
               if {abs($resIDI - $resIDJ) < 4} continue

               lappend disuList $resIDI
               lappend disuList $resIDJ
              }
          }
      }

   return $disuList
}

proc dynGetPDBDisuList { pdbSrc } \
{
   set disuDist 3.0
   set disuList ""

   set iList [dynSelect -list -src $pdbSrc -resName CYS -atomName SG -res *]
   set n     [llength $iList]

   for {set i 0} {$i < [expr $n - 1]} {incr i} \
      {
       set entryI [lindex $iList $i]

       for {set j [expr $i + 1]} {$j < $n} {incr j} \
          {
           set entryJ [lindex $iList $j]
           set d      [dynGetDist -src $pdbSrc -atomI $entryI -atomJ $entryJ]

           if {$d < $disuDist} \
              {
               set iInfo [dynGetCoord -src $pdbSrc -atomEntry $entryI -verb 1]
               set jInfo [dynGetCoord -src $pdbSrc -atomEntry $entryJ -verb 1]

               set resIDI [lindex $iInfo 4]
               set resIDJ [lindex $jInfo 4]
   
               if {abs($resIDI - $resIDJ) < 4} continue

               lappend disuList $resIDI
               lappend disuList $resIDJ
              }
          }
      }

   return $disuList
}

proc getSeqListSeqStr { seqList minRes maxRes } \
{
   append str "\n"
   append str "DATA FIRST_RESID $minRes\n"
   append str "\n"

   set j 0
   set i 0

   for {set res $minRes} {$res <= $maxRes} {incr res} \
      {
       set  resName [lindex $seqList $i]
       incr i

       if {!$j} {append str "DATA SEQUENCE "}

       append str $resName
       incr j

       if {$j == 50} \
          {
           append str "\n"
           set j 0
          } \
       elseif {!($j % 10)} \
          {
           append str " "
          }
      }

   append str "\n"

   return $str
}

proc getSeqListSeqTxt { seqList minRes maxRes prefix } \
{
   set str ""

   set j 0
   set i 0

   for {set res $minRes} {$res <= $maxRes} {incr res} \
      {
       set  resName [lindex $seqList $i]
       incr i

       if {!$j} {append str $prefix} 

       append str $resName
       incr j

       if {$j == 50} \
          {
           append str "\n"
           set j 0
          } \
       elseif {!($j % 10)} \
          {
           append str " "
          }
      }

   append str "\n"

   return $str
}

proc cntSeqIdentity { sA sB bestOffPtr } \
{
   upvar $bestOffPtr bestOff

   set nA [string length $sA]
   set nB [string length $sB]

   set maxMatches  0
   set bestOff    -1

   for {set off [expr 1 - $nA]} {$off < $nB} {incr off} \
      {
       set n 0

       for {set locA 0} {$locA < $nA} {incr locA} \
          {
           set locB [expr $locA + $off]
           if {$locB < 0} {continue}

           set cA [string index $sA $locA]
           set cB [string index $sB $locB]

           if {![strcmp $cA $cB]} {incr n}
          }

       if {$n > $maxMatches} \
          {
           set maxMatches $n
           set bestOff  $off
          }
      }

   return $maxMatches
}

proc pdbNAAngles { pdbTab res } \
{
   set aInfo [getNAAngles $pdbTab $res PDB]

   return $aInfo
}

proc dynNAAngles { pdbSrc res } \
{
   set aInfo [getNAAngles $pdbSrc $res DYNAMO]

   return $aInfo
}

proc getNAAngles { pdbPtr res mode } \
{
   set naInfo {
               {alpha   *               {-1 0 0 0}  {O3' P O5' C5'}}
               {beta    *               {0 0 0 0}   {P O5' C5' C4'}}
               {gamma   *               {0 0 0 0}   {O5' C5' C4' C3'}}
               {delta   *               {0 0 0 0}   {C5' C4' C3' O3'}}
               {epsilon *               {0 0 0 1}   {C4' C3' O3' P}}
               {zeta    *               {0 0 1 1}   {C3' O3' P O5'}}
               {chi   {ADE RADE DADE A} {0 0 0 0}   {C4 C1 N9 C4}}
               {chi   {GUA RGUA DGUA G} {0 0 0 0}   {C4 C1 N9 C4}}
               {chi   {CYT RCYT DCYT C} {0 0 0 0}   {C4 C1 N1 C2}}
               {chi   {THY DTHY T}      {0 0 0 0}   {C4 C1 N1 C2}}
               {chi   {URI RURI U}      {0 0 0 0}   {C4 C1 N1 C2}}
               {v0      *               {0 0 0 0}   {C4' O4' C1' C2'}}
               {v1      *               {0 0 0 0}   {O4' C1' C2' C3'}}
               {v2      *               {0 0 0 0}   {C1' C2' C3' C4'}}
               {v3      *               {0 0 0 0}   {C2' C3' C4' O4'}}
               {v4      *               {0 0 0 0}   {C3' C4' O4' C1'}}
              }

   foreach item $naInfo \
      {
       set angName [lindex $item 0]
       set rList   [lindex $item 1]
       set oList   [lindex $item 2]
       set aList   [lindex $item 3]

       if {[strcmp $rList "*"]} \
          {
           if {![strcasecmp $mode PDB]} \
              {
               set resName [string toupper [getNthResName $pdbPtr $res]]
              } \
           else \
              {
               set resName \
                [string toupper [dynGetInfo -src $pdbPtr -res $res -resName]]
              }

           if {[lsearch -exact $rList $resName] < 0} continue
          }

       set r1 [expr $res + [lindex $oList 0]]
       set r2 [expr $res + [lindex $oList 1]]
       set r3 [expr $res + [lindex $oList 2]]
       set r4 [expr $res + [lindex $oList 3]]

       set a1 [lindex $aList 0]
       set a2 [lindex $aList 1]
       set a3 [lindex $aList 2]
       set a4 [lindex $aList 3]

       if {![strcasecmp $mode PDB]} \
          {
           set c1 "RESID == $r1 && !strcasecmp( ATOMNAME, '$a1' )"
           set e1 [gdbSelect -from $pdbPtr -cond $c1]

           set c2 "RESID == $r2 && !strcasecmp( ATOMNAME, '$a2' )"
           set e2 [gdbSelect -from $pdbPtr -cond $c2]

           set c3 "RESID == $r3 && !strcasecmp( ATOMNAME, '$a3' )"
           set e3 [gdbSelect -from $pdbPtr -cond $c3]

           set c4 "RESID == $r4 && !strcasecmp( ATOMNAME, '$a4' )"
           set e4 [gdbSelect -from $pdbPtr -cond $c4]
          } \
       else \
          {
           set e1 [dynSelect -src $pdbPtr -list -res $r1 -atomName $a1]
           set e2 [dynSelect -src $pdbPtr -list -res $r2 -atomName $a2]
           set e3 [dynSelect -src $pdbPtr -list -res $r3 -atomName $a3]
           set e4 [dynSelect -src $pdbPtr -list -res $r4 -atomName $a4]
          }

       if {[llength $e1] != 1 || [llength $e2] != 1 || \
           [llength $e3] != 1 || [llength $e4] != 1} continue

       if {![strcasecmp $mode PDB]} \
          {
           set angle [pdbAngle $e1 $e2 $e3 $e4]
          } \
       else \
          {
           set angle [dynGetTorsion \
            -src $pdbPtr -atomI $e1 -atomJ $e2 -atomK $e3 -atomL $e4]
          }    

       if {$angle > 999} continue

       lappend aInfo $angName,$angle
      }

   return $aInfo
}

proc isAmino20 { resName } \
{
   set aa3List {
                ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE
                LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL
               }

   if {[lsearch -exact $aa3List [string toupper $resName]] >= 0} {return 1}

   return 0
}

proc isDNARNA { resName } \
{
   set na3List {ADE CYT GUA THY URI DADE DGYT DGUA DTHY RADE RCYT RGUA RURI}

   if {[lsearch -exact $na3List [string toupper $resName]] >= 0} {return 1}

   return 0
}

proc getPDBType { pdbTab } \
{
   set aaCount 0
   set naCount 0
   set oCount  0

   set aa3List {
                ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE  
                LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL  
               }

   set na3List  {ADE CYT GUA THY URI DADE DGYT DGUA DTHY RADE RCYT RGUA RURI}
   set na1List  {A C G T U}
   set skipList {ANI XYZ X Y Z XY XZ YZ}

   set aaCount   0
   set naCount   0
   set oCount    0
   set skipCount 0

   set resList  [string toupper [getTabUniqVals $pdbTab RESNAME]]
   set resCount 0

   foreach resName $resList \
      {
       if {[lsearch -exact $skipList $resName] >= 0} \
          {
           incr skipCount
           continue
          }

       incr resCount

       if {[lsearch -exact $aa3List $resName] >= 0} \
          {
           incr aaCount
          } \
       elseif {[lsearch -exact $na3List $resName] >= 0} \
          {
           incr naCount
          } \
       elseif {[lsearch -exact $na1List $resName] >= 0} \
          {
           incr naCount
          } \
       else \
          {
           incr oCount
          }
      }

   if {$aaCount == 0 && $naCount == 0} \
      {
       return "other $aaCount $naCount $oCount $skipCount"
      }

   if {$aaCount == $resCount} \
      {
       return "protein $aaCount $naCount $oCount $skipCount"
      }

   if {$naCount == $resCount} \
      {
       return "na $aaCount $naCount $oCount $skipCount"
      }

   if {$aaCount && $naCount && !$oCount} \
      {
       return "protein_na $aaCount $naCount $oCount $skipCount"
      }

   if {$aaCount && !$naCount && $oCount} \
      {
       return "protein_other $aaCount $naCount $oCount $skipCount"
      }

   if {!$aaCount && $naCount && $oCount} \
      {
       return "na_other $aaCount $naCount $oCount $skipCount"
      } 

   if {$aaCount && $naCount && $oCount} \
      {
       return "protein_na_other $aaCount $naCount $oCount $skipCount"
      }
   
   return "other $aaCount $naCount $oCount $skipCount"
}

proc probePDBSrc { pdbSrc r1 rN } \
{
   set aa3Count 0
   set aa1Count 0
   set na3Count 0
   set na1Count 0
   set oCount   0

   set aa3List {
                ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE  
                LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL  
               }

   set na3List {ADE CYT GUA THY URI DADE DGYT DGUA DTHY RADE RCYT RGUA RURI}
   set aa1List {A R N D C Q E G H I L K M F P S T W Y V}
   set na1List {A C G T U}

   set skipList {ANI XYZ X Y Z XY XZ YZ}

   for {set resID $r1} {$resID <= $rN} {incr resID} \
      {
       set resName [dynGetInfo -src $pdbSrc -res $resID -resName]
       set resName [string toupper $resName] 
       set n 0

       if {[lsearch -exact $skipList $resName] >= 0} continue
 
       if {[lsearch -exact $aa3List $resName] >= 0} \
          {
           incr aa3Count
           incr n
          } \
       elseif {[lsearch -exact $na3List $resName] >= 0} \
          {
           incr na3Count
           incr n
          }

       if {[lsearch -exact $aa1List $resName] >= 0} \
          {
           incr aa1Count
           incr n
          }

        if {[lsearch -exact $na1List $resName] >= 0} \
          {
           incr na1Count
           incr n
          }

       if {!$n} {incr oCount}
      }

   if {$aa3Count && !$na3Count} {return protein}
   if {!$aa3Count && $na3Count} {return na}
   if {!$aa3Count && $na1Count} {return na}
   if {$aa3Count && $na3Count}  {return complex}

   return other 
}

proc getPDBModelCount { inName } \
{
   if {![file exists $inName]} {return -1}

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return -1}

   set mCount 0
   set aCount 0

   while {[gets $inUnit line] >= 0} \
      {
       if {[llength $line] != 2} continue

       set key [lindex $line 0]
       set val [lindex $line 1]

       if {![strcasecmp $key ATOM] || ![strcasecmp $key HETATM]} \
          {
           incr aCount
           continue
          }

       if {[strcasecmp $key MODEL]} continue
       if {![string is integer $val]} continue

       incr mCount
      }

   if {$mCount == 0 && $aCount > 0} {set mCount 1}

   catch "close $inUnit"

   return $mCount
}

proc pdbNeedHTest { pdbTab thresh } \
{
   set eList [gdbGetInfo $pdbTab -parms entries]
   set pct   0.0
   set nA    0
   set nH    0

   foreach e $eList \
      {
       set atom [gdbGet $e ATOMNAME]
       set res  [gdbGet $e RESNAME]

       if {![strcasecmp $res HOH]} continue
       if {![strcasecmp $res H2O]} continue
       if {![strcasecmp $res WAT]} continue

       set n   [string length $atom]
       set isH 0
       incr nA

       for {set i 0} {$i < $n} {incr i} \
          {
           set c [string index $atom $i]
           if {[lsearch -exact "0 1 2 3 4 5 6 7 8 9 0" $c] >= 0} continue
           if {![strcasecmp $c H]} {set isH 1}
           break
          }

       if {$isH} {incr nH}
      }

   if {$nA} {set pct [expr 100.0*$nH/$nA]}

   if {$pct < $thresh} {return 1}

   return 0
}

proc createResSelectBox args \
{
   set ARGV [concat createResSelectBox $args]

   set skipList   ""
   set seqList    ""
   set r1         1
   set rN         1
   set cmnd       list  
   set w          ""
   set methylFlag 0
   set configFlag 0
   set createFlag 0
   set raiseFlag  0

   set hiliteList None

   set abgColor dimGray
   set afgColor white
   set bgColor  lightGray
   set fgColor  black

   getArgD $ARGV -skip    skipList
   getArgD $ARGV -seq     seqList
   getArgD $ARGV -r1      r1
   getArgD $ARGV -rN      rN

   getArgD $ARGV -command cmnd
   getArgD $ARGV -fg      fgColor
   getArgD $ARGV -bg      bgColor
   getArgD $ARGV -abg     abgColor
   getArgD $ARGV -afg     abgColor
   getArgD $ARGV -win     w

   getListArgD $ARGV -hilite hiliteList

   if {[flagLoc $ARGV -methyl]}   {set methylFlag 1}
   if {[flagLoc $ARGV -nomethyl]} {set methylFlag 0}

   if {[flagLoc $ARGV -config]}   {set configFlag 1}
   if {[flagLoc $ARGV -noconfig]} {set configFlag 0}

   if {[flagLoc $ARGV -raise]}   {set raiseFlag 1}
   if {[flagLoc $ARGV -noraise]} {set raiseFlag 0}

   if {[string length $w]} \
      {  
       set wName $w
      } \
   else \
      {
       set wName .
      }

   if {![winfo exists $w.close]} {set createFlag 1}

   if {![winfo exists $wName]} \
      {
       toplevel $wName
       wm geom $wName +20+40
      } \
   else \
      {
       if {$raiseFlag} \
          {
           wm deiconify $wName
           update
           raise $wName
           update
          }
      }

   set r1  [expr int($r1)]
   set rN  [expr int($rN)]
   set off [expr ($r1 - 1) % 10]

   set fList ""

   set nB  0
   set nR  0
   set nF  0

   for {set resID [expr $r1 - $off]} {$resID <= $rN} {incr resID} \
      {
       set abg $abgColor
       set afg $afgColor
       set bg  $bgColor
       set fg  $fgColor

       if {![expr ($resID - 1) % 10]} \
          {
           set frameName $w.row$nF
           lappend fList $frameName

           if {$createFlag} {frame $frameName}

           incr nF
          }

       if {$resID < $r1 || $resID > $rN} \
          {
           set bName $w.blank$nB

           if {$createFlag} {label $bName -relief flat}

           incr nB 
          } \
       else \
          {
           set resName [lindex $seqList $nR]
           set bName   $w.resOffset$nR

           if {$methylFlag} \
              {
               if {[lsearch -exact "I L V" $resName] < 0} \
                  {
                   if {[lsearch -exact $hiliteList $resID] < 0} \
                      {
                       set abg blue
                       set afg white
                       set bg  cyan
                       set fg  black
                      } \
                   else \
                      {
                       set abg white 
                       set afg blue 
                       set bg  black 
                       set fg  cyan 
                      }
                  } \
               else \
                  {
                   if {[lsearch -exact $hiliteList $resID] < 0} \
                      {
                       set abg red
                       set afg white
                       set bg  pink 
                       set fg  black
                      } \
                   else \
                      {
                       set abg white
                       set afg red 
                       set bg  black 
                       set fg  pink 
                      }
                  }
              }

           if {[lsearch -exact $skipList $resID] >= 0} \
              {
               set abg gray80 
               set afg gray50 
               set bg  gray80
               set fg  gray50
              }

           if {$createFlag} \
              {
               button $w.resOffset$nR -text $resID$resName \
                      -command "$cmnd $bName $resID" -width 4 -padx 2 -pady 1 \
                      -fg $fg -bg $bg -activeforeground $afg \
                      -activebackground $abg
              } \
           else \
              {
               $w.resOffset$nR configure \
                -fg $fg -bg $bg -activeforeground $afg -activebackground $abg
              }

           incr nR 
          }

       pack $bName -in $frameName -side left
      }

   if {$createFlag} \
      {   
       foreach f $fList {pack $f -in $wName -side top -anchor w}

       button $w.close -text Close -command "lower $wName"

       pack $w.close -side top -fill x -expand 1 -pady 2
      }

   update
}

#
# Bottom
