#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#    This TCL code was contributed by Peter Howe.

set auto_path "[split $env(TCLPATH) :] $auto_path"

#
# Event handler for reading GE parameters.

proc rdGEPar { inname parmPtr } {
   upvar $parmPtr var

   foreach dim_number { 0 1 2 3 } {
       lappend varnames "spec_width$dim_number"
       lappend varnames "dim${dim_number}_freq"
       lappend varnames "dim$dim_number"
   }
       lappend varnames "nb"
       lappend varnames "nc"
       lappend varnames "nd"

   set keepers $varnames
   set parfile [open $inname r]
   while {[gets  $parfile s] != -1} {
    set par [lindex $s 0]
    if {[lsearch $keepers $par]>=0} {
        set value [lindex $s 1]
#        puts stdout "$par $value"
        set var($par) $value
        set locpar [ lsearch -exact $keepers $par ]
        while { $locpar > -1 } {
            set keepers [ lreplace $keepers $locpar $locpar ]
            set locpar [ lsearch -exact $keepers $par ]
        }
    }
}

}

proc readParams_GE_EXPORT {} \
{
   global Tx Ty Tz Ta dimCount dimCountName
   global inName specialArgs dataSource

   clearScript

   set dataSource  [getDataSource $inName]
   set specialArgs "-noaswap"

   getGE_ND $inName

#
# Update the number of dimensions:

    if {![string compare $dimCountName N]} \
       {
        if {$Ty < 2 && $dimCount > 1} \
           {
            set dimCount 1
            setDimCount  N
           } \
        elseif {$Tz < 2 && $dimCount > 2} \
           {
            set dimCount 2
            setDimCount  N
           } \
        elseif {$Ta < 2 && $dimCount > 3} \
           {
            set dimCount 3
            setDimCount  N
           }
       }
}

#
# get parameters from a GE file.

proc getGE_ND { fidName } \
{
   global Nx Tx MODEx SWx OBSx CARx FTx LABx \
          Ny Ty MODEy SWy OBSy CARy FTy LABy \
          Nz Tz MODEz SWz OBSz CARz FTz LABz \
          Na Ta MODEa SWa OBSa CARa FTa LABa

   global aqORD dimCount dimCountName temperature carrierInfo
   global tcl_precision old_tcl_precision

#
# Initialize:

   set fidBytes -1
   set pFile    procpar
   set nmrPath  [tkFileFn dirname $fidName] 
   set aqORD    0

#
# Set the precision for the PPM expressions, enable restore of precision later:

   if {[info exists tcl_precision]} {set old_tcl_precision $tcl_precision}

   set shutdown \
     {
      if {[info exists old_tcl_precision]} \
         {
          set tcl_precision $old_tcl_precision
         }
     }

   set tcl_precision 17
#
# Test for the existence of the parameter and data files.
# Read their contents into arrays.

set pFile [ tk_getOpenFile \
	-title "Select parameter file" -initialdir $nmrPath \
	-filetypes "{par {*.par}}" ]

   if {[tkFileFn exists $pFile]} {
       rdGEPar $pFile pList 
       set pList(np) $pList(dim0)
       if { $pList(nb) > 1 } { set pList(ni) $pList(nb) }
       if { $pList(dim2) > 1 } { set pList(ni2) $pList(dim2) }
       if { $pList(dim3) > 1 } { set pList(ni3) $pList(dim3) }
      } \
   else \
      {
       puts stderr "Error: Unable to find file $nmrPath/$pFile"
       eval $shutdown
       return 1
      }

#
# Set the dimension count if needed:

   if {![string compare $dimCountName N]} \
      {
       if {[info exists pList(np)]}  {set dimCount 1}
       if {[info exists pList(ni)]}  {set dimCount 2}
       if {[info exists pList(ni2)]} {set dimCount 3}
       if {[info exists pList(ni3)]} {set dimCount 4}

       setDimCount $dimCountName
      }

#
# Set the axis labels and acquisition order:

   if {![info exists pList(array)]} {set pList(array) None}

   switch $dimCount \
      {
       1 {
          set LABx  X 
          set LABy  Y
          set LABz  Z
          set LABa  A
          set aqORD 0
         }

       2 {
          set LABx  F2 
          set LABy  F1 
          set LABz  Z
          set LABa  A
          set aqORD 0
         }

       3 {
          set LABx  F3
          set LABy  F1
          set LABz  F2 
          set LABa  A
          set aqORD 0

          if {![string compare $pList(array) "phase,phase2"]} \
             { 
              set aqORD 1
             }
         }

       4 {
          set LABx F4
          set LABy F1
          set LABz F2
          set LABa F3 
          set aqORD 0

          if {![string compare $pList(array) "phase,phase2,phase3"]} \
             {
              set aqORD 1
             }
         }

       default {
               }
      }

   set MODEx Complex
   set MODEy Complex
   set MODEz Complex
   set MODEa Complex

   catch {set Nx    [ expr $pList(dim0) * 2] }
   catch {set Tx    [expr $Nx / 2]}
   catch {set SWx   $pList(spec_width0)}
   catch {set OBSx  $pList(dim0_freq)}
   catch {set LABx  $pList(tn)}

   catch {set Ny    $pList(nb)}
   catch {set Ty    [expr $Ny / 2]}
   catch {set SWy   $pList(spec_width1)}
   catch {set OBSy  $pList(dim1_freq)}
   catch {set LABy  $pList(dn)}
  
   catch {set Nz    $pList(dim2)}
   catch {set Tz    [expr $Nz / 2]}
   catch {set SWz   $pList(spec_width2)}
   catch {set OBSz  $pList(dim2_freq)}
   catch {set LABz  $pList(dn)}

   catch {set Na    $pList(dim3)}
   catch {set Ta    [expr $Ny / 2]}
   catch {set SWa   $pList(spec_width3)}
   catch {set OBSa  $pList(dim3_freq)}
   catch {set LABa  $pList(dn)}

#
# Update the menus associated with the ambiguous values:

   set frqList   {sfrq dfrq dfrq1 dfrq2 dfrq3 dfrq4 dfrq5 dfrq6}
   set nucList   {tn   dn   dn1   dn2   dn3   dn4   dn5   dn6}
   set offList   {tof  dof  dof1  dof2  dof3  dof4  dof5  dof6}

   set swList  {}
   set obsList {}
   set carList {}

   if {![info exists pList(temp)]} {set pList(temp)   0.0}

   foreach i {sw sw1 sw2 sw3 sw4 sw5} \
      {
       if {[info exists pList($i)]} \
          {
           if {$pList($i) != 0.0} \
              {
               set     line [list "[format %9.3f $pList($i)] $i" $pList($i)]
               lappend swList $line
              }
          }
      }

   foreach i $nucList \
      {
       if {![info exists pList($i)]} {set pList($i) NONE}
      }

   foreach i $offList \
      {
       if {![info exists pList($i)]} {set pList($i) 0.0}
      }

   set k 0

   foreach i $frqList \
      {
       if {[info exists pList($i)]} \
          {
           set j [lindex $nucList $k]

           if {$pList($i) != 0.0} \
              {
               set     val  $pList($i)
               set     line [list "[format %9.3f $val] ($i) $pList($j)" $val]
               lappend obsList $line
              }
          }

       incr k
      }

   foreach i $frqList \
      {
       if {![info exists pList($i)]} {set pList($i) 500.00}
      }

#
# Update the carrier positions:

   set updateCar 0

   if {"$temperature" == "From File"} \
      {
       set temperature [expr $pList(temp) + 273.0]
      }

   if {"$pList(tn)" == "H1"} {set updateCar 1}

   if {"$CARx" == "H2O"} \
      {
       set h2oPPM  [getH2Oppm $temperature]
       set rName   H2O
      } \
   else \
      {
       set h2oPPM  $CARx
       set rName   User
      }

#   set h2oOBS  $pList(sfrq)
   set h2oOBS  $pList(dim0_freq)
   set $pList(tn) "H1"

   set line    [list "    H2O (sfrq) $pList(tn)" H2O]
   lappend     carList $line

   set car     [format %7.3f $h2oPPM]
   set line    [list "$car (sfrq) $pList(tn)/$rName" $car]
   lappend     carList $line

   set k       0

   foreach i $frqList \
      {
       set j  [lindex $nucList $k]
       set g  [getGamma $pList($j)]

       set val $pList($i)

       if {$g != 0.0 && $val != 0.0} \
          {
           set car [expr (1.0e+6*$val-((($h2oOBS*1.0e+6)-($h2oOBS*$h2oPPM))*$g))/$val]
           set car [format %7.3f $car]

           if {$car > 0.0} \
              {
               set line [list "$car ($i) $pList($j)" $car]
               lappend  carList $line

               set carrierInfo($i) [string trim $car]

               if {$k == 0} {set CARx $car}
               if {$k == 1} {set CARy $car}
               if {$k == 3} {set CARz $car}
               if {$k == 4} {set CARa $car}
              }
          }

       incr k
      }

   if {!$updateCar} {set carList {}}

   foreach i {x y z a} \
      {
       if {[llength $swList]  > 0} {updateGenMenu SW  $i $swList}
       if {[llength $obsList] > 0} {updateGenMenu OBS $i $obsList}
       if {[llength $carList] > 0} {updateGenMenu CAR $i $carList}
      }

   showPoorEntries {CARx CARy CARz CARa OBSy OBSz OBSa}

#
# Adjust precision of calculated parameters:

   set SWx  [format "%.3f" $SWx]
   set SWy  [format "%.3f" $SWy]
   set SWz  [format "%.3f" $SWz]

   if { [string compare $CARx "H2O"] != 0 } { 
       set CARx [format "%.3f" $CARx] }

   set CARy [format "%.3f" $CARy]
   set CARz [format "%.3f" $CARz]

   set OBSx [format "%.3f" $OBSx]
   set OBSy [format "%.3f" $OBSy]
   set OBSz [format "%.3f" $OBSz]

   set temperature [format "%.3f" $temperature]

   eval $shutdown

   return 0
}

#
# Bottom.
