#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Extract Number of Bruker NUS Samples Measured:"
   echo " -parm   parmName  [acqus2]   Bruker Parameter File."
   echo " -sample schedName [nuslist]  NUS Schedule."
   echo " -rev                         Reverse Order of Columns in Schedule."
   echo " -verb                        Verbose Mode ON."
   exit 0
endif

set parmName = (`getArgD $argv -parm   acqu2s`)
set nusName  = (`getArgD $argv -sample nuslist`)

set sampleCount  = 0
set nusDimCount  = 2
set sampleFactor = 2
set td           = 0

if (-e $nusName) then
   if (`flagLoc $argv -rev`) then
      set nusInfo = (`getNUSInfo.tcl -sample $nusName -rev` sampleCount 0)
   else
      set nusInfo = (`getNUSInfo.tcl -sample $nusName` sampleCount 0)
   endif

   set sampleCount = (`getNamedItem sampleCount $nusInfo`)
   set nusDimCount = (`getNamedItem nusDimCount $nusInfo`)

   if (`string same $nusDimCount 3`) then
      set sampleFactor = 4
   endif

   if (`string same $nusDimCount 4`) then
      set sampleFactor = 8
   endif
endif

if (-e $parmName) then
   set tdInfo = (`jcampInfo.tcl -in $parmName -parm TD` TD MISSING MISSING)
   set td     = $tdInfo[3]

   if (!(`string isint $td`)) then
      set td = 0
   endif
endif

set validSamples = $sampleCount
set statusStr       = "unknown"

if (`string cmp $sampleCount 0` && `string cmp $td 0`) then
   set validSamples = (`IMATH "integer($td/$sampleFactor)"`)

   if (`IMATH "$sampleCount == $validSamples"`) then
      set statusStr = "complete"
   else
      set statusStr = "partial"
   endif
endif

if (`flagLoc $argv -verb`) then
   echo "validSamples $validSamples sampleCount $sampleCount TD $td nusDimCount $nusDimCount status $statusStr"
else
   echo $validSamples
endif

exit 0
