#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract One or More Columns from Text Input:"
    puts " -in   inName   \[stdin\]   Input File, or Keyword stdin."
    puts " -out  outName  \[stdout\]  Output File, or Keyword stdout stderr."
    puts " -col  colList  \[0\]       List of Column Index Numbers, First is 0."
    puts " -fmt  fmtList  \[%s ...\]  Corresponding Output Formats."
    puts " -skip                    Skip # and // comments (Default)."
    puts " -noskip                  No Skipping Comments."
    exit 0
   }

set inName   stdin
set outName  stdout
set skipFlag 1
set colList  0
set fmtList  %s

getArgD $ARGV -in inName
getArgD $ARGV -out outName

getListArgD $ARGV -col colList
getListArgD $ARGV -fmt fmtList

if {[flagLoc $ARGV -skip]}   {set skipFlag 1}
if {[flagLoc $ARGV -noskip]} {set skipFlag 0}

set nC [llength $colList]
set nF [llength $fmtList]
set n  [expr $nC - $nF]

for {set i 0} {$i < $n} {incr i} {lappend fmtList %s}

#
#-----------------------------------------------------------------------------------

set inUnit  stdin
set outUnit stdout

if {![strcasecmp $inName stdin]} \
   {
    set inUnit stdin
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts stderr "GetCol Error Finding Input File $inName"
        exit 1
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "GetCol Error Opening Input File $inName"
        exit 1
       }
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "GetCol Error Opening Output File $outName"
        exit 1
       }
   }

while {[gets $inUnit line] >= 0} \
   {
    set loc [string first "#" $line]

    if {$loc >= 0} \
       {
        set line [string range $line 0 [expr $loc - 1]]
       } 

    set loc [string first "//" $line]
   
    if {$loc >= 0} \
       {
        set line [string range $line 0 [expr $loc - 1]]
       } 

    set str [string trim $line]
    set n   [llength $str]

    if {!$n} continue

    set i 0

    foreach col $colList \
       {
        if {$i} {puts -nonewline $outUnit " "}
        puts -nonewline $outUnit [format [lindex $fmtList $i] [lindex $line $col]]
        incr i
       }

    puts $outUnit ""
   }

if {[strcasecmp $outName stdout] && [strcasecmp $outName stderr]} {catch "close $outUnit"}
if {[strcasecmp $inName stdin]} {catch "close $inUnit"}

exit 0
