#!/bin/csh -f

#
# Find likely NMRPipe files (2019.128.15.08) Returns:
#  templateName zPlane aPlane fileName

if (`flagLoc $argv -help`) then
   echo ""
   exit 0
endif

#
# Use explicit input name if given:

if ($#argv == 1) then
   if (`isFlag $argv[1]`) then
      set tName = None
   else 
      set tName = $argv[1]
   endif
else 
   set tName = (`getArgD $argv -in None`)
endif

if (`string casecmp $tName None`) then
   set thisName = (`nmrPrintf $tName 1 1`)

   if (-e $thisName) then
      set iInfo = (`showhdr -test $tName` valid 0 input $tName 1 1 $thisName)

      if ($iInfo[2] == 0) then
         echo $tName 1 1 $thisName
      else
         echo $iInfo[4-7]
      endif
   else
      echo $tName 1 1 $thisName
   endif

   exit 0
endif

#
# Find likely NMRPipe input file and plane.

foreach ext (ft4 ft3 ft2 ft1 fid dat DAT)
   foreach d (smile rist ist rist_nuszf ist_nuszf hmsist hms_ist hmsist_nuszf mem ml maxent nesta nesta_nuszf pcalp lp ft dat pca res . ft4 ft3 ft2 ft1 data fid)
      if ("$d" == ".") then
         foreach fName (smile rist ist mem ml pcalp lp spec test pca res)
            set tName = ${fName}.$ext

           if (-e $tName) then
               set iInfo = (`showhdr -test $tName` valid 0 input $tName 1 1 $tName)

               if ($iInfo[2] == 0) then
                  continue
               endif

               echo $iInfo[4-7] 
               exit 0
           endif
         end
      endif

      if (-d $d) then
         foreach fmt (%04d%04d. %03d%03d. %02d%03d. %05d. %04d. %03d. %02d. .)
            set thisName = (`nmrPrintf $fmt 1 1`)
            set fList    = (`find $d -maxdepth 1 -name \*${thisName}$ext`)

            if ($#fList == 0) then
               continue
            endif

            if ("$fmt" == ".") then
               set tName    = $fList[1]
               set thisName = $tName
               set nextName = $tName
            else
               set fInfo = (`echo $fList[1] | tr '[0-9]' ' '`)

               if ($#fInfo == 0) then
                  continue
               endif

               set tName    = "$fInfo[1]$fmt$ext"
               set thisName = (`nmrPrintf $tName 1 1`) 
               set nextName = (`nmrPrintf $tName 2 1`) 
            endif

            if ((-e $thisName) && (-e $nextName)) then
               set iInfo = (`showhdr -test $tName` valid 0 input $tName 1 1 $thisName)

               if ($iInfo[2] == 0) then
                  continue
               endif

               echo $iInfo[4-7] 
               exit 0
            endif
         end
      endif
   end
end

echo test.ft2 1 1 test.ft2
