#!/bin/csh

if (`flagLoc $argv -help`) then
   nmrPrintf -stderr "Utility to Extract Options for nmrDraw -resample ...\n"
   exit 0
endif

set resArgs = ""

if (`flagLoc $argv -resample` == 0) then
   echo $resArgs 
   exit 0
endif

set fidName    = (`getArgD $argv -fid    None`)
set nusFIDName = (`getArgD $argv -nusFID None`)
set maskName   = (`getArgD $argv -mask   None`)
set schedName  = (`getArgD $argv -sample None`)

if (`string csame $fidName None`) then
   echo $resArgs 
   exit 0
endif

set thisFIDName = (`nmrPrintf $fidName 1 1`)

if (!(-e $thisFIDName)) then
   nmrPrintf -stderr "NMRDraw NUS Resampling Mode: Error Finding Time-Domain Input $fidName\n\n"
   exit 0
endif

@ fmtCount = (`echo $fidName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if (`string csame $schedName None`) then
   set schedName = "nuslist_us"
endif

if (!(-e $schedName)) then
   nmrPrintf -stderr "\nNMRDraw NUS Resampling Mode: Creating Initial NUS Schedule $schedName\n"
   fakeNUS.com -nus 1.0 -in $fidName > $schedName
endif

if (`string csame $nusFIDName None`) then
   if ($fmtCount == 1) then
      set nusFIDName = nus/test%03d.fid
   else if ($fmtCount == 2) then
      set nusFIDName = nus/test%02d%03d.fid
   else
      set nusFIDName = nus.fid
   endif
endif

if (`string csame $maskName None`) then
   if ($fmtCount == 1) then
      set maskName = mask/test%03d.fid
   else if ($fmtCount == 2) then
      set maskName = mask/test%02d%03d.fid
   else
      set maskName = mask.fid
   endif
endif

if (`flagLoc $argv -nus` == 0) then
   set resArgs = "$resArgs -nus"
endif

if (`flagLoc $argv -mask` == 0) then
   set resArgs = "$resArgs -mask $maskName"
endif

if (`flagLoc $argv -nusFID` == 0) then
   set resArgs = "$resArgs -nusFID $nusFIDName"
endif

if (`flagLoc $argv -sample` == 0) then
   set resArgs = "$resArgs -sample $schedName"
endif

set thisMaskName   = (`nmrPrintf $maskName   1 1`)
set thisNUSFIDName = (`nmrPrintf $nusFIDName 1 1`)

if (!(-e $thisMaskName) || !(-e $thisNUSFIDName)) then
   nmrPrintf -stderr "\nNMRDraw NUS Resampling Mode. Creating Initial Mask and Resampled FID ...\n"
   nmrPrintf -stderr "\nNMRDraw NUS Resampling Mode. Log File: resample.log\n"

   resampleNMR.com -in $fidName -out $nusFIDName -mask $maskName -schedule $schedName >& resample.log
endif

echo $resArgs

exit 0
