#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Find RGB and HSV Histograms for an Image:"
    puts " -in inName \[image.pbm\]  Input Image (GIF or PBM Format)."
    puts " -nH nH     \[256\]        Histogram Size."
    puts " -verb                     Verbose Mode ON (Default)."
    puts " -noverb                   Verbose Mode OFF."
    puts " -hdr                      Include Header in Output."
    puts " -nohdr                    No Header in Output (Default)."
    exit 1
   }

set inName    image.pbm
set nH        256 

set satThresh 0.00
set valThresh 0.00

set verbose   1
set hdrFlag   0

getArgD $ARGV -in inName
getArgD $ARGV -nH nH

getArgD $ARGV -satThresh satThresh
getArgD $ARGV -valThresh valThresh

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -hdr]}    {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}  {set hdrFlag 0}

if {$nH < 2} {exit 0}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Image $inName"
    exit 1
   }

set error 0

set imageName roiImage

catch {image delete $imageName}
image create photo $imageName -file $inName

set xSize     [image width  $imageName]
set ySize     [image height $imageName]
set n         [expr $xSize*$ySize]

puts -nonewline stderr "$inName $xSize $ySize $n"

for {set i 0} {$i <= $nH} {incr i} \
   {
    set rInfo($i) 0.0
    set gInfo($i) 0.0
    set bInfo($i) 0.0
    set hInfo($i) 0.0
   }

set nL [expr $nH - 1]

for {set iy 0} {$iy < $ySize} {incr iy} \
   {
    for {set ix 0} {$ix < $xSize} {incr ix} \
       {
        set cInfo [$imageName get $ix $iy]

        set ir [lindex $cInfo 0]
        set ig [lindex $cInfo 1]
        set ib [lindex $cInfo 2]

        set locR [expr int($ir*$nL/255.0)]
        set locG [expr int($ig*$nL/255.0)]
        set locB [expr int($ib*$nL/255.0)]

        if {$locR > $nL} {set locR $nL}
        if {$locG > $nL} {set locG $nL}
        if {$locB > $nL} {set locB $nL}

        set rInfo($locR) [expr $rInfo($locR) + $ir/255.0]
        set gInfo($locG) [expr $gInfo($locG) + $ig/255.0]
        set bInfo($locB) [expr $bInfo($locB) + $ib/255.0]

        set hsvInfo [rgb2hsv $ir $ig $ib]

        set hue     [lindex $hsvInfo 0]
        set sat     [lindex $hsvInfo 1]
        set val     [lindex $hsvInfo 2]

        if {$sat < $satThresh} {set sat 0.0}
        if {$val < $valThresh} {set val 0.0}

        set locH    [expr int($hue*$nL)]
        set locS    [expr int($sat*$nL)]
        set locV    [expr int($val*$nL)]

        if {$locH > $nL} {set locH $nL}
        if {$locS > $nL} {set locS $nL}
        if {$locV > $nL} {set locV $nL}

        set hInfo($locH) [expr $hInfo($locH) + $val*$sat]
       } 
   }

set sum    0.0
set maxVal 0.0

for {set i 0} {$i < $nH} {incr i} {set sum [expr $sum + $hInfo($i)]} 

for {set i 0} {$i < $nH} {incr i} \
   {
    set rgbIndex [expr int(255.0*$i/double($nL))]
    set hsvIndex [expr $i/double($nL)]

    if {$rgbIndex > $nL} {set rgbIndex $nL}
    if {$hsvIndex > 1.0} {set hsvIndex 1.0}

    set thisVal [safeDiv $hInfo($i) $sum]

    if {$thisVal > $maxVal} {set maxVal $thisVal}

    puts [format " %5.3f %e" $hsvIndex [safeDiv $hInfo($i) $sum]]
   }

puts stderr " Max $maxVal"

exit

#
# Bottom.
