#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initIPRArgs {} \
{
   global progName plotName inDir testList verbose ARGC ARGV
   global layoutInfo groupInfo groupList layoutRows layoutCols

   set verbose   1
   set progName  InstallPack 
   set plotName  nmrReport.ps
   set inDir     History
   set testList  *

# Layout for report:
#  testName row column groupName

   set layoutInfo {
                   {H1SN    1 1 1H}    {H1lshp_spin 1 2 1H}    {H1lshp_ns 1 3 1H}  {H1pw90  1 4 1H}
                   {C13SN   2 1 13C}   {C13SN_ETB   2 2 13C}   {C13res    2 3 13C} {C13pw90 2 4 13C}
                   {N15SN   3 1 15N}   {N15pw90     3 2 15N}   {F19SN     3 3 19F} {F19pw90 3 4 19F}  
                   {P31SN   4 1 31P}   {P31pw90     4 2 31P}
                   {Z0drift 5 1 Field} {GPROFILE    5 2 Field} {GREC      5 3 Field}       
                  }

# Layout within a group:
#  firstRow lastRow firstColumn lastColumn

   set groupList {1H 13C 15N 19F Field}

   set groupInfo(1H)    {1 1 1 4}
   set groupInfo(13C)   {2 2 1 4}
   set groupInfo(15N)   {3 3 1 2}
   set groupInfo(19F)   {3 3 3 4}
   set groupInfo(Field) {4 4 1 3}

   set layoutRows 5
   set layoutCols 4

   if {[flagLoc $ARGV -help]} \
      {
       puts "Generate a Printable Report for $progName:"
       puts " -progName progName \[InstallPack\]  Program Name (Used for Messages)."
       puts " -plotName plotName \[nmrReport.ps\] Name for PostScript Output."
       puts " -inDir    dirName  \[History\]      Directory of Test Results."
       puts " -testList tList    \[*\]            List of Test Results to Show."
       puts " -verb                             Verbose Mode ON (Default)."
       puts " -noverb                           Verbose Mode OFF."
       exit 0
      }

   getArgD $ARGV -plotName plotName
   getArgD $ARGV -progName progName
   getArgD $ARGV -inDir    inDir

   getListArgD $ARGV -testList testList 

   if {[flagLoc $ARGV -verb]}   {ser verbose 1}
   if {[flagLoc $ARGV -noverb]} {ser verbose 0}

   set thisList ""

   if {![strcmp $testList "*"]} \
      {
       set thisList [glob -nocomplain $inDir/*]
      } \
   else \
      {
       foreach testName $testList \
          {
           if {[file exists $inDir/$testName]} {lappend thisList $inDir/$testName} 
          }
      }

   if {![llength $thisList]} \
      {
       puts stderr "$progName Report: No Tests Found."
       exit 1
      }

   set testList [lsort $thisList]

   if {$verbose} \
      {
       puts stderr "$progName Report. [llength $testList] Tests:"

       set i 1

       foreach t $testList \
          {
           puts stderr [format " %2d. %s" $i [file tail $t]]
           incr i
          }      
      }
}

proc initIPRPlot {} \
{
   global plotName paperType plotOrient pageColor verbose
   global pageLeft pageRight pageTop pageBot c1Width c1Height c2Width c2Height
   global leftMargin rightMargin topMargin botMargin 

   set paperType    A

   set pageLeft     0.5
   set pageRight    8.0
   set pageTop      0.5
   set pageBot     10.5

   set leftMargin   0.6
   set rightMargin  0.2
   set topMargin    1.5
   set botMargin    0.2

   set c1Width   0.10
   set c1Height  0.13
   set c2Width   [expr 2.0*$c1Width]
   set c2Height  [expr 2.0*$c1Height]

   set plotOrient portrait
   set pageColor  gray90

   plotOpen -color -plotName $plotName \
            -c1Width ${c1Width}in -c1Height ${c1Height}in \
            -c2Width ${c2Width}in -c2Height ${c2Height}in \
            -paper $paperType -$plotOrient

#  drawRec -x ${pageLeft}in ${pageRight}in -y ${pageTop}in ${pageBot}in -fg $pageColor
}

proc getLayoutInfo { tName } \
{
   global layoutInfo groupInfo 

   foreach i $layoutInfo \
      {
       set thisName [lindex $i 0]
       if {![strcasecmp $tName $thisName]} {return $i}
      }

   return ""
}

proc getGroupLimits { groupName gx1Ptr gxnPtr gy1Ptr gynPtr } \
{
   global groupInfo

   upvar $gx1Ptr gx1
   upvar $gxnPtr gxn
   upvar $gy1Ptr gy1
   upvar $gynPtr gyn

   set gx1 0.0
   set gn1 0.0
   set gy1 0.0
   set gy1 0.0

   if {![info exists groupInfo($groupName)]} return

   set firstRow [lindex $groupInfo($groupName) 0]
   set lastRow  [lindex $groupInfo($groupName) 1]
   set firstCol [lindex $groupInfo($groupName) 2]
   set lastCol  [lindex $groupInfo($groupName) 3]

   set xList    ""
   set yList    ""

   for {set row $firstRow} {$row <= $lastRow} {incr row} \
      {
       for {set col $firstCol} {$col <= $lastCol} {incr col} \
          {
           getItemLimits $row $col px1 pxn py1 pyn

           lappend xList $px1 $pxn
           lappend yList $py1 $pyn
          }
      }

   set gx1 [getMin $xList]
   set gxn [getMax $xList]
   set gy1 [getMax $yList]
   set gyn [getMin $yList]
}

proc getItemLimits { row col gx1Ptr gxnPtr gy1Ptr gynPtr } \
{
   global layoutInfo groupInfo groupList layoutRows layoutCols
   global pageLeft pageRight pageTop pageBot c1Width c1Height c2Width c2Height
   global leftMargin rightMargin topMargin botMargin 
   global pageLeft pageRight pageTop pageBot

   upvar $gx1Ptr gx1
   upvar $gxnPtr gxn
   upvar $gy1Ptr gy1
   upvar $gynPtr gyn

   set xPad [expr 4*$c2Width]
   set yPad [expr 4*$c2Height]

   set px1 [expr $pageLeft  + $leftMargin]
   set pxn [expr $pageRight - $rightMargin]
   set py1 [expr $pageBot   - $botMargin]
   set pyn [expr $pageTop   + $topMargin]

   set pxw [expr $pxn - $px1]
   set pyw [expr $py1 - $pyn]

   set gxw [expr ($pxw - $xPad*($layoutCols - 1))/$layoutCols] 
   set gyw [expr ($pyw - $yPad*($layoutRows - 1))/$layoutRows] 

   set gx1 [expr $px1 + ($col - 1)*$gxw]
   set gxn [expr $gx1 + $gxw]

   set gyn [expr $pyn + ($row - 1)*$gyw]
   set gy1 [expr $gyn + $gyw]
}

proc iprOutlineGroup { groupName } \
{
   global c1Width c1Height c2Width c2Height

   getGroupLimits $groupName gx1 gxn gy1 gyn

   drawRec -x ${gx1}in ${gxn}in -y ${gy1}in ${gyn}in -fg gray90
   drawBox -x ${gx1}in ${gxn}in -y ${gy1}in ${gyn}in -fg gray50

   drawText -x [expr $gx1 + 0.5*$c2Width]in -y [expr $gyn + 1.25*$c2Height]in -fg blue -big -text $groupName 
}

proc iprPlotAll {} \
{
   global plotName paperType plotOrient pageColor progName verbose
   global testList layoutInfo groupInfo groupList layoutRows layoutCols
   global pageLeft pageRight pageTop pageBot c1Width c1Height c2Width c2Height
   global leftMargin rightMargin topMargin botMargin 
   global pageLeft pageRight pageTop pageBot

   foreach groupName $groupList \
      {
       iprOutlineGroup $groupName
      }

   foreach tPath $testList \
      {
       set tName [file tail $tPath]

       if {![file exists $tPath]} \
          {
           puts stderr "$progName Error: Missing File $tPath"
           continue
          }

       set tInfo [getLayoutInfo $tName]
       if {![llength $tInfo]} continue

       set t [gdbRead table -in $tPath]

       iprPlot $t $tName $tInfo
      }
}

proc iprPlot { t tName tInfo } \
{
   global layoutInfo groupInfo groupList layoutRows layoutCols
   global c1Width c1Height

   set testName  [lindex $tInfo 0]
   set row       [lindex $tInfo 1]
   set col       [lindex $tInfo 2]
   set groupName [lindex $tInfo 3]

   getItemLimits $row $col gx1 gxn gy1 gyn

   drawRec  -x ${gx1}in ${gxn}in -y ${gy1}in ${gyn}in -fg cyan
   drawText -x [expr $gx1 + 0.5*$c1Width]in -y [expr $gyn + 1.25*$c1Height]in -fg black -text $testName 
}

initIPRArgs
initIPRPlot
iprPlotAll

exit 0
