#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

wm iconify .

if {[flagLoc $ARGV -help]} \
   {
    puts "Project Assignments onto IP/AP Peak Tables:"
    puts " -specName1  specName1  \[sum.ft2\]      Input Spectrum 1 (ASS-J/2)."
    puts " -specName2  specName2  \[dif.ft2\]      Input Spectrum 2 (ASS+J/2)."
    puts " -inName1    tabName1   \[sum.tab\]      Input Peak Table 1."
    puts " -inName2    tabName2   \[dif.tab\]      Input Peak Table 2."
    puts " -outName1   tabName1   \[sum.ass.tab\]  Output Peak Table 1."
    puts " -outName2   tabName2   \[dif.ass.tab\]  Output Peak Table 2."
    puts " -assName    assTabName \[ass.tab\]      Input Assignment Table."
    puts " -ndim       dimCount   \[Auto\]         Dimension Count."
    puts " -single                               Single-Spectrum Mode ON."
    puts "Auto-Assign at Start-Up:"
    puts " -auto                  Auto Assign Peaks at Start (Default)."
    puts " -noauto                No Auto Assign Peaks at Start."
    puts " -nosort                No Sort of Assignments at Start."
    puts " -replace               Replace All Existing Assignments (Default)."
    puts " -noreplace             Keep Any Existing Assignments."
    puts " -exit                  Exit After Auto Assign."
    puts "Multiplet Description Via Common Offset:"
    puts " -jx jxHz     \[0.0\]  Expected X-Axis Coupling, Hz"
    puts " -jy jyHz     \[90.0\] Expected X-Axis Coupling, Hz"
    puts " -jz jzHz     \[0.0\]  Expected Z-Axis Coupling, Hz"
    puts "Multiplet Description Via Individual Peaks:"
    puts " -jInfo jList \[None\] Offsets for Each Peak, Hz" 
    puts "Assignment Mapping Parameters:"
    puts " -xw xwPPM \[0.1\]  X-Axis Region Size for Assignment Mapping, PPM." 
    puts " -yw ywPPM \[1.0\]  Y-Axis Region Size for Assignment Mapping, PPM." 
    puts " -zw zwPPM \[1.0\]  Z-Axis Region Size for Assignment Mapping, PPM." 
    puts " -pk pkN   \[1\]    Expected Peak Count in a Spectrum per Assignment."
    puts "Drawing Parameters:"
    puts " -xv     xvPPM \[0.15\]    X-Axis Region Size for Display, PPM."
    puts " -yv     yvPPM \[2.50\]    Y-Axis Region Size for Display, PPM."
    puts " -zv     zvPts \[0\]       Z-Axis Projection Size for Display, Pts."
    puts " -hi1    hi1   \[Auto\]    Contour Height for Spectrum 1."
    puts " -hi2    hi2   \[Auto\]    Contour Height for Spectrum 2."
    puts " -mult   mult  \[7.0\]     Factor for Auto Contour Height."
    puts "Spectrum Colors:"
    puts " -color1      c    \[cyan\]    Color for Spectrum 1."
    puts " -color2      c    \[white\]   Color for Spectrum 2."
    puts " -posColor1   c    \[cyan\]    Color for Spectrum 1 (Positive Data)."
    puts " -negColor1   c    \[cyan\]    Color for Spectrum 1 (Negative Data)."
    puts " -posColor2   c    \[white\]   Color for Spectrum 2 (Positive Data)."
    puts " -negColor2   c    \[white\]   Color for Spectrum 2 (Negative Data)."
    puts "Label Colors:"
    puts " -pkOnColor1  c    \[green\]   Color for Current Peaks, Table 1."
    puts " -pkOffColor1 c    \[gray50\]  Color for Other Peaks, Table 1." 
    puts " -pkOnColor2  c    \[green\]   Color for Current Peaks, Table 2."
    puts " -pkOffColor2 c    \[gray50\]  Color for Other Peaks, Table 2."
    puts " -assOnColor  c    \[red\]     Color for Other Assignments."
    puts " -assOffColor c    \[yellow4\] Color for Current Assignment."
    puts "Spectrum Axis Parameters:"
    puts " -xName  xName \[Auto\]   X-Axis Name in Spectrum." 
    puts " -yName  yName \[Auto\]   Y-Axis Name in Spectrun." 
    puts " -zName  zName \[Auto\]   Z-Axis Name in Spectrum." 
    puts "Peak Table Parameters:"
    puts " -xVar   xVar  \[Auto\]   X-Axis Name in Peak Table." 
    puts " -yVar   yVar  \[Auto\]   Y-Axis Name in Peak Table." 
    puts " -zVar   zVar  \[Auto\]   Z-Axis Name in Peak Table." 
    puts "Assignment Table Parameters:"
    puts " -xAss   xAss  \[Auto\]   X-Axis Name in Assignment Table." 
    puts " -yAss   yAss  \[Auto\]   Y-Axis Name in Assignment Table." 
    puts " -zAss   zAss  \[Auto\]   Z-Axis Name in Assignment Table." 
    puts " -assVar assV  \[ASS\]    Assignment Name in Assignment Table." 
    puts "Assignment Table PPM Offsets:"
    puts " -xOff   xOff  \[0.0\]    X-Axis Offset for Assignment, PPM." 
    puts " -yOff   yOff  \[0.0\]    Y-Axis Offset for Assignment, PPM." 
    puts " -zOff   zOff  \[0.0\]    Z-Axis Offset for Assignment, PPM." 
    exit
   }

setFonts

set specName1   sum.ft2
set specName2   dif.ft2
set inName1     sum.tab
set inName2     dif.tab
set outName1    sum.ass.tab
set outName2    dif.ass.tab
set assTabName  ass.tab
set dimCount    Auto

set peaksPerAss 1 
 
set hi1         Auto 
set hi2         Auto 
set mult        7.0
set factor      1.3
set plev        10
set nlev        10

set posColor1   cyan
set negColor1   cyan
set posColor2   white
set negColor2   white

set pkOffColor1 gray50
set pkOnColor1  green
set pkOffColor2 gray50 
set pkOnColor2  green
set assOnColor  red
set assOffColor yellow4 

set jxHz       0.0
set jyHz      90.0
set jzHz       0.0
set jInfo     None

set xwPPM      0.1
set ywPPM      1.0
set zwPPM      1.0

set xViewPPM   0.15
set yViewPPM   2.50
set zViewPPM   0.60 

set xName     Auto 
set yName     Auto 
set zName     Auto 

set xVarName  Auto 
set yVarName  Auto
set zVarName  Auto

set xAssName  Auto 
set yAssName  Auto
set zAssName  Auto

set xOffset   0.0
set yOffset   0.0
set zOffset   0.0

set assVar       ASS
set autoFlag     1
set exitFlag     0
set sortFlag     1
set replaceFlag  1

set pkLabel1Flag  1
set pkLabel2Flag  1

set drawSpec1Flag 1
set drawSpec2Flag 1

set singleFlag    0

if {[flagLoc $ARGV -single]} \
   {
    set singleFlag    1

    set specName2     None
    set inName2       None

    set pkLabel2Flag  0
    set drawSpec2Flag 0
   }

getArgD $ARGV -specName  specName1
getArgD $ARGV -inName    inName1
getArgD $ARGV -outName   outName1

getArgD $ARGV -specName1 specName1
getArgD $ARGV -specName2 specName2

if {![strcasecmp $specName2 None]} {set inName2 None}

getArgD $ARGV -inName1   inName1
getArgD $ARGV -inName2   inName2

getArgD $ARGV -outName1  outName1
getArgD $ARGV -outName2  outName2

getArgD $ARGV -assName   assTabName

getArgD $ARGV -pk peaksPerAss

getArgD $ARGV -hi   hi1
getArgD $ARGV -hi   hi2
getArgD $ARGV -hi1  hi1
getArgD $ARGV -hi2  hi2
getArgD $ARGV -plev plev
getArgD $ARGV -nlev nlev
getArgD $ARGV -mult mult
getArgD $ARGV -ndim dimCount

getArgD $ARGV -color1    posColor1
getArgD $ARGV -color1    negColor1
getArgD $ARGV -posColor  posColor1
getArgD $ARGV -negColor  negColor1
getArgD $ARGV -posColor1 posColor1
getArgD $ARGV -negColor1 negColor1

getArgD $ARGV -color2    posColor2
getArgD $ARGV -color2    negColor2
getArgD $ARGV -posColor1 posColor2
getArgD $ARGV -negColor1 negColor2

getArgD $ARGV -pkOnColor1  pkOnColor1
getArgD $ARGV -pkOffColor1 pkOffColor1
getArgD $ARGV -pkOnColor2  pkOnColor2
getArgD $ARGV -pkOffColor2 pkOffColor2

getArgD $ARGV -assOnColor  assOnColor 
getArgD $ARGV -assOffColor assOffColor

getArgD $ARGV -jx jxHz
getArgD $ARGV -jy jyHz
getArgD $ARGV -jz jzHz

getListArgD $ARGV -jInfo jInfo

getArgD $ARGV -xw xwPPM
getArgD $ARGV -yw ywPPM
getArgD $ARGV -zw zwPPM

getArgD $ARGV -xv xViewPPM
getArgD $ARGV -yv yViewPPM
getArgD $ARGV -zv zViewPPM

getArgD $ARGV -xName  xName
getArgD $ARGV -yName  yName
getArgD $ARGV -zName  zName

getArgD $ARGV -xVar   xVarName
getArgD $ARGV -yVar   yVarName
getArgD $ARGV -zVar   zVarName

getArgD $ARGV -xAss   xAssName
getArgD $ARGV -yAss   yAssName
getArgD $ARGV -zAss   zAssName
getArgD $ARGV -assVar assVar

getArgD $ARGV -xOff   xOffset
getArgD $ARGV -yOff   yOffset
getArgD $ARGV -zOff   zOffset

if {[flagLoc $ARGV -sort]}      {set sortFlag 1}
if {[flagLoc $ARGV -nosort]}    {set sortFlag 0}

if {[flagLoc $ARGV -exit]}      {set exitFlag 1}
if {[flagLoc $ARGV -noexit]}    {set exitFlag 0}

if {[flagLoc $ARGV -auto]}      {set autoFlag 1}
if {[flagLoc $ARGV -noauto]}    {set autoFlag 0}

if {[flagLoc $ARGV -replace]}   {set replaceFlag 1}
if {[flagLoc $ARGV -noreplace]} {set replaceFlag 0}

#
#-----------------------------------------------------------------------------

proc initAssOffset {} \
{
   global assEList xOffset yOffset zOffset dimCount
   global xAssName yAssName zAssName

   if {$xOffset != 0.0} \
      {
       foreach entry $assEList \
          {
           set xLoc [gdbGet $entry $xAssName]
           set xLoc [expr $xLoc + $xOffset]
           gdbSet $entry $xAssName $xLoc
          }
      }

   if {$yOffset != 0.0} \
      {
       foreach entry $assEList \
          {
           set yLoc [gdbGet $entry $yAssName]
           set yLoc [expr $yLoc + $yOffset]
           gdbSet $entry $yAssName $yLoc
          }
      }

   if {$zOffset != 0.0 && $dimCount > 2} \
      {
       foreach entry $assEList \
          {
           set zLoc [gdbGet $entry $zAssName]
           set zLoc [expr $zLoc + $zOffset]
           gdbSet $entry $zAssName $zLoc
          }
      }
}

proc adjPeakAss { pk ass } \
{
    global saveNeeded

    incr saveNeeded

    gdbSet $pk ASS $ass

    buildAssList
    drawIPAPLabels
}
 
proc selectNewAss { loc } \
{
   global assLoc assCount

   busy hold .t

   set w .assList

   if {$loc < 0} {set loc [expr $assCount - 1]}
   if {$loc > [expr $assCount - 1]} {set loc 0}

   set assLoc $loc

   $w.assListBox selection clear 0 end
   $w.assListBox selection set $assLoc

   update

   set curView [$w.assListBox yview]
   set bbox    [$w.assListBox bbox $assLoc]

   if {[llength $curView] && [llength $bbox]} \
      {
       set x0 [lindex $bbox 0]
       set y0 [lindex $bbox 1]
       set xw [lindex $bbox 2]
       set yw [lindex $bbox 3]

       set y1 [lindex $curView 0]
       set yn [lindex $curView 1]

       if {$yn < $y0 || $y1 > $y0 + $yw} \
          {
           $w.assListBox see $assLoc
          }
      } \
   else \
      {
       $w.assListBox see $assLoc
      }

   busy release .t

   selectAss 
}

proc selectAss {} \
{
   global assLoc

   set w .assList
 
   set loc [lindex [$w.assListBox curselection] 0]

   if {![llength $loc]} return

   set assLoc $loc 

   readRegion
   drawScreen
   update
}

proc drawScreen {} \
{
   drawIPAPRegion
   drawIPAPLabels
}

proc drawIPAPLabels {} \
{
   global pkTab1 pkTab2 pkLabel1Flag pkLabel2Flag
   global pkOffColor1 pkOnColor1 pkOffColor2 pkOnColor2

   drawIPAPAssLabels
   drawIPAPPeakLabels pk1 $pkTab1 $pkLabel1Flag $pkOffColor1 $pkOnColor1
   drawIPAPPeakLabels pk2 $pkTab2 $pkLabel2Flag $pkOffColor2 $pkOnColor2 
}

proc drawIPAPAssLabels {} \
{
   global xViewPPM yViewPPM zViewPPM xAssName yAssName zAssName assVar
   global assLabelFlag assOnColor assOffColor dimCount
   global assInfo assLoc assTab 

   set w .t

   $w.c delete ass

   if {!$assLabelFlag} return

   set ass $assInfo($assLoc)

   set cond "" 

   append cond "near( $xAssName, $assInfo($ass,x), $xViewPPM ) && "
   append cond "near( $yAssName, $assInfo($ass,y), $yViewPPM ) && "

   if {$dimCount > 2} \
      {
       append cond "near( $zAssName, $assInfo($ass,z), $zViewPPM ) && "
      }

   append cond "strcasecmp( $assVar, 'None' )"

   set eList [gdbSelect -from $assTab -cond $cond]

   foreach entry $eList \
      {
       set x [gdbGet $entry $xAssName]ppm
       set y [gdbGet $entry $yAssName]ppm

       set thisAss [gdbGet $entry $assVar]

       if {[strcmp $ass $thisAss]} \
          {
           set thisColor $assOffColor 
          } \
       else \
          {
           set thisColor $assOnColor
          }

       set xPix [spec2pix -cell 100 -roi 101 -x $x]
       set yPix [spec2pix -cell 100 -roi 101 -y $y]

       createBoxText $w.c $xPix $yPix ass $thisColor black $thisAss tID bID

       $w.c bind $tID <ButtonRelease-1> "+ selectNewAss $assInfo($thisAss,loc)"
       $w.c bind $tID <ButtonRelease-2> "+ hideAss $tID $bID"
      }
}

proc drawIPAPPeakLabels { tag pkTab labelFlag offColor onColor } \
{
   global xViewPPM yViewPPM zViewPPM xVarName yVarName zVarName
   global assInfo assLoc assTab dimCount

   set w .t

   $w.c delete $tag 

   if {!$labelFlag} return
   if {!$pkTab}     return

   set ass $assInfo($assLoc)

   set cond "" 

   append cond "near( $xVarName, $assInfo($ass,x), $xViewPPM ) && "
   append cond "near( $yVarName, $assInfo($ass,y), $yViewPPM )"

   if {$dimCount > 2} \
      {
       append cond " && near( $zVarName, $assInfo($ass,z), $zViewPPM )"
      }

   set eList [gdbSelect -from $pkTab -cond $cond]

   foreach entry $eList \
      {
       set x  [gdbGet $entry $xVarName]ppm
       set y  [gdbGet $entry $yVarName]ppm
       set id [gdbGet $entry INDEX]

       set thisAss [gdbGet $entry ASS]

       if {[strcmp $ass $thisAss]} \
          {
           set thisColor $offColor 
          } \
       else \
          {
           set thisColor $onColor 
          }

       set str "$id=$thisAss"

       set xPix [spec2pix -cell 100 -roi 101 -x $x]
       set yPix [spec2pix -cell 100 -roi 101 -y $y]

       createBoxText $w.c $xPix $yPix $tag $thisColor black $str tID bID

       $w.c bind $tID <ButtonRelease-1> "+ adjPeakAss $entry $ass"
       $w.c bind $tID <ButtonRelease-2> "+ hideAss $tID $bID"
       $w.c bind $tID <ButtonRelease-3> "+ adjPeakAss $entry None"
      }
}

proc createBoxText { c x0 y0 t fg bg str tPtr bPtr } \
{
   global stdfont

   upvar $tPtr tID
   upvar $bPtr bID

   set tID \
    [$c create text $x0 $y0 -tags $t -fill $fg -font $stdfont -text $str]

   set bInfo [$c bbox $tID]

   set x1  [lindex $bInfo 0]
   set y1  [lindex $bInfo 1]
   set xN  [lindex $bInfo 2]
   set yN  [lindex $bInfo 3]

   set x1  [expr $x1 - 1]
   set yN  [expr $yN - 1]

   set bID [$c create rectangle $x1 $y1 $xN $yN -tags $t -outline $fg -fill $bg]

   $c raise $tID

   return $tID
}

proc assResize {} \
{
   global xWinSize yWinSize

   set w .t

   bind $w.c <Configure> ""
   update

   set xWinSize [winfo width  $w]
   set yWinSize [winfo height $w]

   $w.c delete pixmap -tags win2D
   $w.c create pixmap 0 0 -width $xWinSize -height $yWinSize -win 1 -tags win2D

   drawScreen

   bind $w.c <Configure> assResize
}

proc drawIPAPRegion {} \
{
   global assInfo assLoc hi1 hi2 plev nlev zName zViewPPM
   global drawSpec1Flag drawSpec2Flag cellX cellY dimCount
   global posColor1 posColor2 negColor1 negColor2

   newScreen
   drawSettings -reset

   defineCell -cell 100 -x 2% $cellX -y 2% $cellY -win 1

   drawRec -cell 1   -x 0% 100% -y 0% 100% -fg dimGray
   drawRec -cell 100 -x 0% 100% -y 0% 100% -fg black

   set ass $assInfo($assLoc)
 
   set xPPM $assInfo($ass,x)ppm
   set yPPM $assInfo($ass,y)ppm

   drawLine -cell 100 -roiRef 101 -x $xPPM $xPPM -y 0%    100%  -fg dimGray
   drawLine -cell 100 -roiRef 101 -x 0%    100%  -y $yPPM $yPPM -fg dimGray

   if {$drawSpec1Flag} \
      {
       drawROI -cell 100 -roi 101 -hi $hi1 -axis Bottom Right -bg None \
               -color1 $posColor1 -color2 $negColor1 -plev $plev -nlev $nlev
      }

   if {$drawSpec2Flag} \
      {
       drawROI -cell 100 -roi 102 -hi $hi2 -bg None \
               -color1 $posColor2 -color2 $negColor2 -plev $plev -nlev $nlev
      }

   if {$dimCount > 2} \
      {
       set str \
        [format "%s %.2f +/- %.2fppm" $zName $assInfo($ass,z) $zViewPPM]

       drawText -cell 100 -x 1.5c1 -y 1.5c1 -fg yellow -text $str
      }
}

proc readRegion {} \
{
   global xName yName zName xViewPPM yViewPPM zViewPPM
   global assInfo assLoc specName1 specName2 dimCount
   global drawSpec1Flag drawSpec2Flag

   set ass $assInfo($assLoc)

   set xA ""
   set yA ""
   set zA ""

   if {$dimCount > 0} {set xA "-dx $xName $assInfo($ass,x)ppm ${xViewPPM}ppm"}
   if {$dimCount > 1} {set yA "-dy $yName $assInfo($ass,y)ppm ${yViewPPM}ppm"}
   if {$dimCount > 2} {set zA "-dz $zName $assInfo($ass,z)ppm ${zViewPPM}ppm"}

   if {[strcasecmp $specName1 None]} \
      {
       eval readROI -ndim 2 -roi 101 -in $specName1 $xA $yA $zA -center X Y
      } \
   else \
      {
       set drawSpec1Flag 0
      }

   if {[strcasecmp $specName2 None]} \
      {
       eval readROI -ndim 2 -roi 102 -in $specName2 $xA $yA $zA -center X Y
      } \
   else \
      {
       set drawSpec2Flag 0
      }
}
 
proc quitAss {} \
{
   saveAss
   okExit exit
}

proc autoQuitAss {} \
{
   global pkTab1 pkTab2 outName1 outName2

   if {$pkTab1 && [strcasecmp $outName1 None]} {gdbWrite $pkTab1 $outName1}
   if {$pkTab2 && [strcasecmp $outName2 None]} {gdbWrite $pkTab2 $outName2}

   exit 0
}

proc saveAss {} \
{
   global pkTab1 pkTab2 outName1 outName2 saveNeeded

   set w .t

   set types \
      {
       { {All Files}  *      }
       { {tab}        {.tab} }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {dat}        {.dat .DAT} }
       { {seq}        {.seq} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
      }

   set saveRemaining 0

   if {$pkTab1} {incr saveRemaining}
   if {$pkTab2} {incr saveRemaining}

   set thisName \
     [tk_getSaveFile -title "Assignment Output 1" -parent $w \
       -filetypes $types -initialfile $outName1]

   if {[string length $thisName]} \
      {
       gdbWrite $pkTab1 $thisName
       incr saveRemaining -1
      }

   if {$pkTab2} \
      {
       set thisName \
         [tk_getSaveFile -title "Assignment Output 2" -parent $w \
           -filetypes $types -initialfile $outName2]

       if {[string length $thisName]} \
          {
           gdbWrite $pkTab2 $thisName
           incr saveRemaining -1
          }
      }

   if {$saveRemaining <= 0} {set saveNeeded 0}

   updateAssTitle
}

proc nextAss {} \
{
   global assLoc

   selectNewAss [expr $assLoc + 1]
}

proc prevAss {} \
{
   global assLoc

   selectNewAss [expr $assLoc - 1]
}

proc resetAss {} \
{
   global pkTab1 pkTab2 assLoc assInfo saveNeeded

   set w .t

   incr saveNeeded

   busy hold $w

   set ass $assInfo($assLoc)

   foreach pkTab "$pkTab1 $pkTab2" \
      { 
       if {!$pkTab} continue

       set eList [gdbSelect -from $pkTab -cond "!strcasecmp( ASS, \"$ass\" )"]
       foreach entry $eList {gdbSet $entry ASS None}
      }

   busy release $w

   buildAssList
   drawIPAPLabels
}

proc hideAss { tID bID } \
{
   set w .t

   $w.c raise $bID win2D
   $w.c raise $tID $bID

   update
}

proc goodAss {} \
{
}

proc badAss {} \
{
}
 
proc aSort { a1 a2 d } \
{
   global assVar

   set n1 [string length $a1]
   set n2 [string length $a2]

   set s1 [string range $a1 0 [expr $n1 - 2]]
   set s2 [string range $a2 0 [expr $n2 - 2]]

   if {$s1 < $s2} {return -1}
   if {$s1 > $s2} {return  1}

   return 0
}

proc pSort { i1 i2 } \
{
   set s1 [lindex $i1 0]
   set s2 [lindex $i2 0]

   if {$s1 < $s2} {return -1}
   if {$s1 > $s2} {return  1}

   return 0
}

proc findNearPeak { pkTab xPPM yPPM zPPM } \
{
    global xwPPM ywPPM zwPPM xVarName yVarName zVarName xSW ySW zSW
    global peaksPerAss dimCount

    if {!$pkTab} {return ""}

    set cond \
     "near( $xVarName, $xPPM, $xwPPM ) && near( $yVarName, $yPPM, $ywPPM )"

    if {$dimCount > 2} \
       {
        append cond " && near( $zVarName, $zPPM, $zwPPM )"
       }

    set eList [gdbSelect -from $pkTab -cond $cond]
    set n     [llength $eList]

    set sList ""
    set pList ""
    set bList ""

    set sx 1.0
    set sy [expr $xSW/$ySW]

    if {$dimCount > 2} \
       {
        set sz [expr $xSW/$zSW]
       } \
    else \
       {
        set sz 0.0
       }
 
    foreach entry $eList \
       {
        set x [ipapGet X $entry $xVarName]
        set y [ipapGet Y $entry $yVarName]
        set z [ipapGet Z $entry $zVarName]

        set d [expr pow(($sx*($x-$xPPM)),2) + \
                    pow(($sy*($y-$yPPM)),2) + \
                    pow(($sz*($z-$zPPM)),2)]

        set d [expr pow($d,0.5)]

        lappend pList "$d $entry"

        set entry [gdbNext $entry]
       }

    set sList [lsort -command pSort $pList]

    for {set i 0} {$i < $peaksPerAss && $i < $n} {incr i} \
       {
        lappend bList [lindex [lindex $sList $i] 1] 
       }

    return $bList
}

proc updateAssTitle {} \
{
   global saveNeeded

   set w .assList

   wm title $w "Changes: $saveNeeded"
   update
}

proc buildAssList {} \
{
   global assInfo assEList xAssName yAssName zAssName assVar pkTab1 pkTab2
   global firstTime singleFlag 

   busy hold .t

   set w .assList

   set activeList [$w.assListBox curselection]
   set activeView [$w.assListBox yview]

   $w.assListBox delete 0 end

   set loc 0

   foreach entry $assEList \
      {
       set ass [gdbGet $entry $assVar]

       if {$firstTime} \
          { 
           set assX [ipapGet X $entry $xAssName]
           set assY [ipapGet Y $entry $yAssName]
           set assZ [ipapGet Z $entry $zAssName]

           set assInfo($ass,status) None
           set assInfo($ass,entry)  $entry
           set assInfo($ass,loc)    $loc
           set assInfo($ass,x)      $assX
           set assInfo($ass,y)      $assY
           set assInfo($ass,z)      $assZ
           set assInfo($loc)        $ass
          }

       set cond    "!strcmp( $assVar, \"$ass\" )"
       set topList ""
       set botList ""

       if {$pkTab1} {set topList [gdbSelect -from $pkTab1 -cond $cond]}
       if {$pkTab2} {set botList [gdbSelect -from $pkTab2 -cond $cond]}
   
       set topIDList "" 
       set botIDList "" 
   
       foreach pk $topList {lappend topIDList [gdbGet $pk INDEX]}
       foreach pk $botList {lappend botIDList [gdbGet $pk INDEX]}
   
       set assInfo($ass,topList) $topList
       set assInfo($ass,botList) $botList
 
       if {$singleFlag} \
          {
           set str [format "%7s %8s" $ass \{$topIDList\}]
          } \
       else \
          { 
           set str [format "%7s %8s %8s" $ass \{$topIDList\} \{$botIDList\}]
          }
   
       $w.assListBox insert end $str

       incr loc 
      }

   $w.assListBox yview moveto [lindex $activeView 0]

   foreach loc $activeList {$w.assListBox selection set $loc}

   busy release .t

   update

   updateAssTitle

   set firstTime 0
}

proc getAssInt { ass } \
{
   set error 0
   set i ""
   set n 0

   foreach c [expand $ass] \
      {
       set loc [string first $c "-0123456789"]
       if {$loc < 0} continue

       if {!$loc} \
          {
           if {$n} {set error 1}
          }

       append i $c
       incr n
      }

   if {!$n || $error} {set i $ass}

   return $i
}

proc assSortProc { ass1 ass2 } \
{
   set v1 [getAssInt $ass1]
   set v2 [getAssInt $ass2]

   if {$v1 > $v2} {return  1}
   if {$v1 < $v2} {return -1}

   return 0
}

#
#---------------------------------------------------------------

set firstTime     1
set saveNeeded    0
set assLabelFlag  1

set assWidth  0
set assHeight 0

set assEList  ""
set assTab    0
set pkTab1    0
set pkTab2    0

if {[file exists $inName1]} {set pkTab1 [gdbRead table -in $inName1]}
if {[file exists $inName2]} {set pkTab2 [gdbRead table -in $inName2]}

if {!$pkTab1} \
   {
    puts "Error Finding Peak Table $inName1"
    exit 1
   }

if {![file exists $assTabName]} \
   {
    puts "Error Finding Assignment Table $assTabName"
    exit 1
   }

if {![file exists [format $specName1 1 1]]} \
   {
    puts "Error Finding Input Spectrum $specName1"
    exit 1
   }

if {[strcasecmp $specName2 None] && ![file exists [format $specName2 1 1]]} \
   {
    puts "Error Finding Input Spectrum $specName2"
    exit 1
   }

set assTab [gdbRead table -in $assTabName]
set assVID [gdbFindByName $assTab $assVar]

if {$sortFlag} {gdbSortTab $assTab $assVar -proc assSortProc}

set assEList [gdbSelect -from $assTab -cond "strcasecmp( $assVar, 'None' )"]

set assCount [llength $assEList]
set assLoc 0
set fdata  0

set error    [rdFDATA $specName1 fdata]

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM] 
   }

if {$dimCount == 2} \
   {
    set zName    None
    set zVarName None
    set zAssName None
   }

if {![strcasecmp $xName Auto]} \
   {
    set xName [getDirectDim $fdata]
    puts "Auto Mode Direct Dimension:  $xName"
   }

if {![strcasecmp $yName Auto]} \
   {
    set yName [get15NDim $fdata]
    puts "Auto Mode 15N Dimension:     $yName"
   }

if {![strcasecmp $zName Auto]} \
   {
    set zName [getIndirectDim $fdata]
    puts "Auto Mode 3rd Dimension:     $zName"
   }

if {![strcasecmp $xAssName Auto]} \
   {
    set xAssName [getHNTabVar $assTab]
    puts "Auto Mode HN Ass Variable:   $xAssName"
   }

if {![strcasecmp $yAssName Auto]} \
   {
    set yAssName [get15NTabVar $assTab]
    puts "Auto Mode 15N Ass Variable:  $yAssName"
   }

if {![strcasecmp $zAssName Auto]} \
   {
    set zAssName [getIndirectTabVar $assTab]
    puts "Auto Mode 3rd Ass Variable:  $zAssName"
   }

if {![strcasecmp $xVarName Auto]} \
   {
    set xVarName [getHNTabVar $pkTab1]
    puts "Auto Mode HN Peak Variable:  $xVarName"
   }

if {![strcasecmp $yVarName Auto]} \
   {
    set yVarName [get15NTabVar $pkTab1]
    puts "Auto Mode 15N Peak Variable: $yVarName"
   }

if {![strcasecmp $zVarName Auto]} \
   {
    set zVarName [getIndirectTabVar $pkTab1]
    puts "Auto Mode 3rd Peak Variable: $zVarName"
   }

if {![strcasecmp $hi1 Auto]} \
   {
    set noise [estSpecNoise $specName1]
    set hi1   [expr $noise*$mult]

    if {$hi1 == 0.0} {set hi1 1.0e+6}

    puts "Auto Mode Contour Height 1:  [format %.2e $hi1]"
   }

if {![strcasecmp $hi2 Auto]} \
   {
    set noise [estSpecNoise $specName2]
    set hi2   [expr $noise*$mult]

    if {$hi2 == 0.0} {set hi2 1.0e+6}

    puts "Auto Mode Contour Height 2:  [format %.2e $hi2]"
   }

#
#---------------------------------------------------------------------------

set xDimName [getDimKeyword $xName $fdata]
set yDimName [getDimKeyword $yName $fdata]

if {$dimCount == 2} \
   {
    set zDimName $yDimName
   } \
else \
   {
    set zDimName [getDimKeyword $zName $fdata]
   }

set xSize  [getParm $fdata NDSIZE $xDimName]
set xSWHz  [getParm $fdata NDSW   $xDimName]
set xOBS   [getParm $fdata NDOBS  $xDimName]

set ySize  [getParm $fdata NDSIZE $yDimName]
set ySWHz  [getParm $fdata NDSW   $yDimName]
set yOBS   [getParm $fdata NDOBS  $yDimName]

set zSize  [getParm $fdata NDSIZE $zDimName]
set zSWHz  [getParm $fdata NDSW   $zDimName]
set zOBS   [getParm $fdata NDOBS  $zDimName]

if {$xOBS == 0.0} {set xOBS 1.0}
if {$yOBS == 0.0} {set yOBS 1.0}
if {$zOBS == 0.0} {set zOBS 1.0}

set xSW [expr $xSWHz/$xOBS]
set ySW [expr $ySWHz/$yOBS]
set zSW [expr $zSWHz/$zOBS]

set jx [expr $jxHz/$xOBS]
set jy [expr $jyHz/$yOBS]
set jz [expr $jzHz/$zOBS]

set jOffsetList ""
set jInfo [lindex $jInfo 0]

if {![strcasecmp $jInfo None]} \
   {

    if {$singleFlag} \
       {
        for {set i 0} {$i < $peaksPerAss} {incr i} \
           {
            lappend jOffsetList "[expr 0.5*$jx] [expr 0.5*$jy] [expr 0.5*$jz]"
            incr i

            if {$i >= $peaksPerAss} break

            lappend jOffsetList \
             "[expr -0.5*$jx] [expr -0.5*$jy] [expr -0.5*$jz]"
           }

       } \
    else \
       {
        for {set i 0} {$i < $peaksPerAss} {incr i} \
           {
            lappend jOffsetList \
             "[expr -0.5*$jx] [expr -0.5*$jy] [expr -0.5*$jz]"
           }

        for {set i 0} {$i < $peaksPerAss} {incr i} \
           {
            lappend jOffsetList "[expr 0.5*$jx] [expr 0.5*$jy] [expr 0.5*$jz]"
           }
       } 
   } \
else \
   {
    set peaksPerAss [llength $jInfo]
 
    foreach j $jInfo \
       {
        set thisJ ""

        for {set i 0} {$i < $dimCount} {incr i} \
           {
            set jVal [lindex $j $i]
            set jObs [lindex "$xOBS $yOBS $zOBS" $i]

            lappend thisJ [expr $jVal/$jObs] 
           }

        lappend jOffsetList $thisJ
       }
   }

#
#---------------------------------------------------------------------------

proc initFoldInfo {} \
{
   foldYAss
   foldZAss
}

proc foldYAss {} \
{
   global fdata dimCount yDimName yAssName ySW ySize assEList assVar

   set yFirst [pnt2spec $fdata $yDimName 1      ppm]
   set yLast  [pnt2spec $fdata $yDimName $ySize ppm]

   set fmt "Folding Assignments ($yAssName) into Range %.2f to %.2f"

   puts [format $fmt $yLast $yFirst]

   foreach entry $assEList \
      {
       set yLoc [gdbGet $entry $yAssName]
       set ass  [gdbGet $entry $assVar]

      if {$yLoc > $yFirst || $yLoc < $yLast} \
         {
          set yNew $yLoc
   
          while {$yNew > $yFirst} {set yNew [expr $yNew - $ySW]}
          while {$yNew < $yLast}  {set yNew [expr $yNew + $ySW]}

          puts "Folded: $ass $yLoc -> $yNew"

          gdbSet $entry $yAssName $yNew
         }
     }
}

proc foldZAss {} \
{
   global fdata dimCount zDimName zAssName zSW zSize assEList assVar

   if {$dimCount < 3} return

   set zFirst [pnt2spec $fdata $zDimName 1      ppm]
   set zLast  [pnt2spec $fdata $zDimName $zSize ppm]

   set fmt "Folding Assignments ($zAssName) into Range %.2f to %.2f"

   puts [format $fmt $zLast $zFirst]

   foreach entry $assEList \
      {
       set zLoc [gdbGet $entry $zAssName]
       set ass  [gdbGet $entry $assVar]

      if {$zLoc > $zFirst || $zLoc < $zLast} \
         {
          set zNew $zLoc
   
          while {$zNew > $zFirst} {set zNew [expr $zNew - $zSW]}
          while {$zNew < $zLast}  {set zNew [expr $zNew + $zSW]}

          puts "Folded: $ass $zLoc -> $zNew"

          gdbSet $entry $zAssName $zNew
         }
     }
}

proc setHi { hiPtr } \
{
   upvar $hiPtr hi

   drawIPAPRegion
}

proc incrHi { hiPtr } \
{
   upvar $hiPtr hi
   global factor

   if {$factor != 0.0} {set hi [expr $hi*$factor]}
}

proc decrHi { hiPtr } \
{
   upvar $hiPtr hi
   global factor

   if {$factor != 0.0} {set hi [expr $hi/$factor]}
}

proc initSpecWin {} \
{
   global assLabelFlag pkLabel1Flag pkLabel2Flag drawSpec1Flag drawSpec2Flag 
   global hi1 hi2 assHeight assWidth cellX cellY
   global singleFlag

   set wmStr [string tolower [winfo server .]]

   if {[string first "silicon graphics" $wmStr] >= 0} \
      {
       set cellX 88%
       set cellY 89%

       set dx 60
       set dy 40
       set bx 15
       set by 35
      } \
   else \
      {
       set cellX 88%
       set cellY 91%

       set dx 40
       set dy 80
       set bx 8 
       set by 32 
      }

   set xw [winfo screenwidth  .]
   set yw [winfo screenheight .]

   set xWinSize [expr $xw - $assWidth - $dx] 
   set yWinSize [expr $yw - 120]

   set w .t

   toplevel $w 
   wm title $w {IPAP (C) F. Delaglio 2007.068.09.07 [Select] [Lower] [Reset]}

   frame $w.cmnd -width $xWinSize

   if {$singleFlag} \
      {
       set str1 "" 
       set str2 "" 
      } \
   else \
      {
       set str1 "1" 
       set str2 "2" 
      }

   label $w.sl1 -text "Spec:"

   checkbutton $w.spec1 \
    -text $str1 -anchor e -selectcolor green \
    -onvalue 1 -offvalue 0 -command drawIPAPRegion -variable drawSpec1Flag

   checkbutton $w.spec2 \
    -text $str2 -anchor w -width 2 -selectcolor green \
    -onvalue 1 -offvalue 0 -command drawIPAPRegion -variable drawSpec2Flag

   label $w.sl2 -text "Peak:"

   checkbutton $w.peak1 \
    -text $str1 -anchor e -selectcolor green -padx 1 \
    -onvalue 1 -offvalue 0 -command drawIPAPLabels -variable pkLabel1Flag 

   checkbutton $w.peak2 \
    -text $str2 -anchor w -width 2 -selectcolor green \
    -onvalue 1 -offvalue 0 -command drawIPAPLabels -variable pkLabel2Flag

   label $w.al -text "Ass:"

   checkbutton $w.ass \
    -text "" -anchor w -width 2 -selectcolor green \
    -onvalue 1 -offvalue 0 -command drawIPAPLabels -variable assLabelFlag

   if {$singleFlag} \
      {
       pack $w.sl1 $w.spec1 $w.sl2 $w.peak1 $w.al $w.ass -in $w.cmnd -side left
      } \
   else \
      {
       pack $w.sl1 $w.spec1 $w.spec2 $w.sl2 $w.peak1 $w.peak2 $w.al $w.ass \
            -in $w.cmnd -side left
      }

   label  $w.lab1   -text "First:"
   entry  $w.first1 -textvariable hi1 -width 11
   button $w.incr1  -text "+" -padx 1 -pady 1 -command "incrHi hi1"
   button $w.decr1  -text "-" -padx 1 -pady 1 -command "decrHi hi1"
   bind   $w.first1 <Return> "+ setHi hi1"

   button $w.draw -text Draw -command drawIPAPRegion

   label  $w.lab2   -text "First:"
   entry  $w.first2 -textvariable hi2 -width 11
   button $w.incr2  -text "+" -padx 1 -pady 1 -command "incrHi hi2"
   button $w.decr2  -text "-" -padx 1 -pady 1 -command "decrHi hi2"
   bind   $w.first2 <Return> "+ setHi hi2"

   pack $w.draw -in $w.cmnd -side right

   if {$singleFlag} \
      {
       pack $w.decr1 $w.incr1 $w.first1 $w.lab1 -in $w.cmnd -side right
      } \
   else \
      {
       pack $w.decr2 $w.incr2 $w.first2 $w.lab2 -in $w.cmnd -side right
       pack $w.decr1 $w.incr1 $w.first1 $w.lab1 -in $w.cmnd -side right
      }

   canvas   $w.c -width $xWinSize -height $yWinSize -border 0
   pack     $w.cmnd -in $w -side top -expand 1 -fill x
   pack     $w.c    -in $w -side top -expand 1 -fill both

   update

   wm geom  $w +$bx+$by

   update

   .t.c create pixmap 0 0 -width $xWinSize -height $yWinSize -win 1 -tags win2D

   update

   bind .t.c <Configure> assResize
}

proc initAssWin {} \
{
   global assLoc assWidth assHeight typingfont

   set w .assList

   toplevel $w

   wm title $w Assignments
   wm geom  $w -15+45

   frame $w.top
   frame $w.bot

   listbox $w.assListBox -width 32 -height 40     \
           -yscrollcommand "$w.assListScroll set" \
           -font $typingfont -selectmode single

   scrollbar $w.assListScroll -command "$w.assListBox yview"
   pack      $w.assListBox    -side left -expand 1 -fill both -in $w.top
   pack      $w.assListScroll -side left -fill y -in $w.top

   button $w.next  -text Next  -command nextAss  -padx 4
   button $w.prev  -text Prev  -command prevAss  -padx 4
   button $w.reset -text Reset -command resetAss -padx 4
   button $w.save  -text Save  -command saveAss  -padx 4
   button $w.quit  -text Quit  -command quitAss  -padx 4

   pack $w.next $w.prev $w.reset $w.save $w.quit -in $w.bot \
        -expand 1 -side left -fill x 

   pack $w.top $w.bot -in $w -side top -expand 1 -fill both

   bind $w.assListBox <ButtonRelease-1> "+ selectAss"

   $w.assListBox selection set $assLoc

   update

   set assWidth  [winfo width  $w]
   set assHeight [winfo height $w]
}

proc ipapGet { axis entry var } \
{
   global dimCount

   if {$dimCount < 3 && ![strcasecmp $axis Z]} {return 0.0}

   return [gdbGet $entry $var]
}

proc initAssInfo {} \
{
   global singleFlag

   if {$singleFlag} \
      {
       initAssInfo1
      } \
   else \
      {
       initAssInfo2
      }
}

proc initAssInfo1 {} \
{
   global autoFlag xAssName yAssName zAssName assVar xVarName yVarName zVarName
   global replaceFlag jOffsetList pkTab1 pkTab2 assEList saveNeeded dimCount
   global singleFlag peaksPerAss

   if {!$autoFlag} return

   set lab "Peak:"
 
   foreach entry $assEList \
      {
       set assX [ipapGet X $entry $xAssName]
       set assY [ipapGet Y $entry $yAssName]
       set assZ [ipapGet Z $entry $zAssName]

       set ass  [gdbGet $entry $assVar]

       foreach jItem $jOffsetList \
          {
           set jx [lindex $jItem 0]
           set jy [lindex $jItem 0]
           set jz [lindex $jItem 0]

           set topX  [expr $assX + 0.5*$jx]
           set topY  [expr $assY + 0.5*$jy]
           set topZ  [expr $assZ + 0.5*$jz]

           set topList [findNearPeak $pkTab1 $topX $topY $topZ]

           set fmt \
            "%8s %6.2f %6.2f Target: %6.2f %6.2f $lab    %4d %6.2f %6.2f %s"

           foreach pk $topList \
              {
               incr saveNeeded

               set pkX   [ipapGet X $pk $xVarName]
               set pkY   [ipapGet Y $pk $yVarName]
               set pkZ   [ipapGet Z $pk $zVarName]

               set pkAss [gdbGet $pk ASS]
               set id    [gdbGet $pk INDEX]

               if {[strcasecmp $pkAss None] && !$replaceFlag} \
                  {
                   puts [format $fmt \
                    $pkAss $assX $assY $topX $topY $id $pkX $pkY Old]
                  } \
               else \
                  {
                   puts \
                    [format $fmt $ass   \
                      $assX $assY $topX $topY $id $pkX $pkY New]

                   gdbSet $pk ASS $ass
                  }
              }
          }
      }
}

proc initAssInfo2 {} \
{
   global autoFlag xAssName yAssName zAssName assVar xVarName yVarName zVarName
   global replaceFlag jx jy jz pkTab1 pkTab2 assEList saveNeeded dimCount
   global singleFlag

   if {!$autoFlag} return

   if {$singleFlag} \
      {
       set lab "Peak:"
      } \
   else \
      {
       set lab "Top:"
      }
 
   foreach entry $assEList \
      {
       set assX [ipapGet X $entry $xAssName]
       set assY [ipapGet Y $entry $yAssName]
       set assZ [ipapGet Z $entry $zAssName]

       set ass  [gdbGet $entry $assVar]

       set topX  [expr $assX - 0.5*$jx]
       set topY  [expr $assY - 0.5*$jy]
       set topZ  [expr $assZ - 0.5*$jz]

       set botX  [expr $assX + 0.5*$jx]
       set botY  [expr $assY + 0.5*$jy]
       set botZ  [expr $assZ + 0.5*$jz]

       if {$singleFlag} \
          {
           set topList [findNearPeak $pkTab1 $topX $topY $topZ]
           set botList [findNearPeak $pkTab1 $botX $botY $topZ]
          } \
       else \
          {
           set topList [findNearPeak $pkTab1 $topX $topY $topZ]
           set botList [findNearPeak $pkTab2 $botX $botY $topZ]
          }

       set fmt "%8s %6.2f %6.2f Target: %6.2f %6.2f $lab    %4d %6.2f %6.2f %s"

       foreach pk $topList \
          {
           incr saveNeeded

           set pkX   [ipapGet X $pk $xVarName]
           set pkY   [ipapGet Y $pk $yVarName]
           set pkZ   [ipapGet Z $pk $zVarName]

           set pkAss [gdbGet $pk ASS]
           set id    [gdbGet $pk INDEX]

           if {[strcasecmp $pkAss None] && !$replaceFlag} \
              {
               puts \
                [format $fmt $pkAss $assX $assY $topX $topY $id $pkX $pkY Old]
              } \
           else \
              {
               puts \
                [format $fmt $ass   $assX $assY $topX $topY $id $pkX $pkY New]

               gdbSet $pk ASS $ass
              }
          }

       set fmt "%8s %6.2f %6.2f Target: %6.2f %6.2f Bottom: %4d %6.2f %6.2f %s"

       foreach pk $botList \
          {
           incr saveNeeded

           set pkX   [ipapGet X $pk $xVarName]
           set pkY   [ipapGet Y $pk $yVarName]
           set pkZ   [ipapGet Z $pk $zVarName]

           set pkAss [gdbGet $pk ASS]
           set id    [gdbGet $pk INDEX]

           if {[strcasecmp $pkAss None] && !$replaceFlag} \
              {
               puts \
                [format $fmt $pkAss $assX $assY $botX $botY $id $pkX $pkY Old]
              } \
           else \
              {
               puts \
                [format $fmt $ass   $assX $assY $botX $botY $id $pkX $pkY New]

               gdbSet $pk ASS $ass
              }
          }
      }
}

initAssOffset
initAssWin
initSpecWin
initFoldInfo
initAssInfo
buildAssList

selectNewAss $assLoc

if {$exitFlag} {autoQuitAss} 
