#!/bin/csh

set thisVersion = "2024.061.14.37"

if (`flagLoc $argv -help`) then
   echo "ist2D.com $thisVersion"
   echo "Perform IST Recontruction of 2D Non-Uniformly Sampled (NUS) Data:"
   echo " -in       inName   [Auto]     Expanded Phase-Senstitive 2D Time-Domain Input."
   echo " -mask     maskName [mask.fid] Corresponding 2D Time-Domain Mask Input, or Auto."
   echo " -out      outName  [ist.ft2]  IST Spectrum Output."
   echo " -ft1      ft1Name  [None]     Optional IST Interferogram Output (Sets -noapod)."
   echo " -title    title    [None]     Title for Output."
   echo " -pseudo3D                     Process Data as Pseudo-3D 2D Series."
   echo " -nus                          Input is Non-Uniformly Sampled (Default)."
   echo " -us                           Input is Conventional Uniformly Sampled (US)."
   echo "IST Parameters:"
   echo " -istMaxRes r [Auto]  Target Max in Residual, as % of Original Max (Or Auto)."
   echo " -istTMult  t [0.70]  Fraction for Threshold at Each Iteration"
   echo " -istCMult  c [0.30]  Fraction for Reducing Data Above Threshold." 
   echo " -istIter   n [2048]  Maximum Allowed Iterations."
   echo " -mult      m [1.0]   Multiplication Factor for -istMaxRes Auto."
   echo " -retain              Retain Original Time-Domain Data Points (Sets -noapod)."
   echo " -noretain            Replace Original Time-Domain Data Points (Default)."
   echo " -rist                Same as -retain Option."
   echo " -pos                 Use All-Positive IST (No Negative Signals)."
   echo "Processing Parameters (Use -x... -y...):"
   echo " -xSOL    sName  [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD   aName  [SP]         Window Function Name."
   echo " -xQ1     q1     [0.50]       Window Parameter Q1 (SP -off)."
   echo " -xQ2     q2     [0.95]       Window Parameter Q2 (SP -end)."
   echo " -xQ3     q3     [2]          Window Parameter Q3 (SP -pow, Default -yQ3 is 1)."
   echo " -xELB    elb    [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB    glb    [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF   goff   [0.0]        Gaussian Center Position, 0.0 to 1.0."
   echo " -xC1     c      [Auto]       First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0     p0     [0.0]        Zero-Order Phase Correction."
   echo " -xP1     p1     [0.0]        First Order Phase Correction."
   echo " -xZFARG  zfArgs [zf=1,auto]  ZF (Zero Fill) Options."
   echo " -xFTARG  ftArg  [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1  x1     [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN  xn     [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -xPREFT  fnArgs [NULL]       Function to Execute Before Any Other Processing."
   echo " -xPOSTFT fnArgs [NULL]       Function to Execute After FT and Phase Correction."
   echo " -scaleTo maxVal [None]       Scale to Given Max Value, or None for No Scaling."
   echo " -noapod                      No Apodization for Indirect Dimensions During IST."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo "Auto Phasing Options (Currently Only For X-Axis Zero Order Phase, -xP0 Auto):"
   echo " -apOrd   ord    [0]          Auto-Phase Baseline Order (-1 for No Correction)."
   echo " -apArgs  apArgs [None]       Other Options for basicAutoPhase.com Utility."
   echo "Processing Parameters for X-Axis Only:"
   echo " -xBASEARG_INIT bArgs [NULL]  Baseline Correction Before Region Extraction."
   echo " -xP0EXT        p0Ext [0.0]   Zero-Order Phase After Region Extraction."
   echo " -xP1EXT        p1Ext [0.0]   First Order Phase After Region Extraction."
   echo "NUS Zero-Fill Mode (Time-Domain Extrapolation by IST):"
   echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
   echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
   echo " -yNUSZF  zfArgs  [zf=1]      Amount of Y-Axis NUS Zero-Fill Extrapolation."
   echo " -istName outName [ist.ft2]   Overrides -out Setting."
   echo "Baseline Correction Function Name and Options (Use -x... -y...):"
   echo " -xBASEARG_FT1    [NULL]      Interferogram Baseline Correction (X-Axis Only)."
   echo " -xBASEARG        [NULL]      Spectrum Baseline Correction."
   echo " -xBASEARG_FINAL  [NULL]      Baseline Correction for Final IST Result."
   echo "Baseline Correction Applied to Final Residual:"
   echo " -rBASEARG     [POLY,auto,ord=0] Residual Baseline Correction."
   echo "Other Options:"
   echo "  -noclean     Do Not Delete Intermediate IST Results."
   echo "  -progress    Display Progress Bar Graphic."
   echo ""
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 -xBASEARG and -xBASEARG_FINAL are expanded"
   echo "    into NMRPipe processing function names and command line arguments."
   echo "    Options should contain no spaces. Flags are separated by commas, and"
   echo "    values are given using = signs. For example, solvent filter:"
   echo ""
   echo "      -xSOL     NULL            Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL             Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time       Becomes: nmrPipe -fn POLY -time"
   echo ""
   echo "    For baseline correction:"
   echo ""
   echo "      -xBASEARG NULL            Becomes: nmrPipe -fn NULL (No Correction)"
   echo "      -xBASEARG POLY,auto,ord=0 Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG zf=2,auto       Becomes: nmrPipe -fn ZF -zf 2 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo "      -xFTARG neg,alt         Becomes: nmrPipe -fn FT -neg -alt"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise."
   echo ""
   echo " 4. For applying NUS Zero Fill to conventional data, use -nusZF -mask Auto to"
   echo "    automatically create mask data via the nusZF.com script."
   echo ""
   echo " 5. If the X-Axis label of the input spectrum starts with 'HN', default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG_FINAL POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo "    Or when -us or -nusZF -mask Auto is used for conventional data:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 6. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "       -xSOL NULL -xBASEARG NULL -xBASEARG_FINAL NULL"
   echo ""
   echo " 7. Commonly, only region of interest, FT, and phase correction options need"
   echo "    to be provided. Time-domain input and mask can be identified automatically,"
   echo "    and the convergence parameter will be set automatically. Include -nonusZF"
   echo "    to suppress extrapolation during reconstruction if desired."
   echo ""
   echo "    Example Usage:"
   echo ""
   echo "       ist2D.com -xP0 -16 -xP1   0 -xEXTX1 2.6ppm -xEXTXN -0.8ppm \"
   echo "                 -yP0 -13 -yP1 180 -yZFARG zf=2,auto -yFTARG alt"
   echo ""
   echo " 8. To stop an IST calculation in progress:"
   echo ""
   echo "       touch ist.stop"
   echo ""
   echo " 9. See utility nusExpand.tcl for expanding time-domain NUS data and"
   echo "    creating time-domain NUS schedule mask."
   echo ""
   echo "10. Auto phasing is available only for zero-order phase correction in the"
   echo "    direct-detected dimension (-xP0 Auto). The auto-phasing, which is performed"
   echo "    via the utility basicAutoPhase.com, might not be ideal for all cases. Auto"
   echo "    phasing parameters for this utility can be adjusted via the -apArgs option."
   echo "    For example, to use the Y-Axis region 120.0 ppm to 114.0ppm for evaluating"
   echo "    phase correction:"
   echo ""
   echo "      ist2D.com \"
   echo "       -xP0 Auto -xP1 0 -xEXTX1 10.6ppm -xEXTXN 5.5ppm \"
   echo "       -apArgs apy1=120.0ppm,apyn=114.0ppm"
   echo ""
   echo "    See 'basicAutoPhase.com -help' for more."

   exit 0
endif

set vFlag        = (`getLogArgD $argv -verb       -noverb   verb noverb`)
set pseudo3DFlag = (`getLogArgD $argv -nopseudo3D -pseudo3D 0    1`)
set ssNMRFlag    = (`getLogArgD $argv -nossNMR    -ssNMR    0    1`)
set retainFlag   = (`getLogArgD $argv -noretain   -retain   0    1`)
set posFlag      = (`getLogArgD $argv -nopos      -pos      0    1`)

if (`flagLoc $argv -rist`) then
   set retainFlag = 1
endif

@ listFlag  = 0
@ apodFlag  = 1
@ skipCount = 0
@ nusFlag   = 1

set inName  = (`getArgD $argv -in Auto`)

if (`string same $vFlag verb`) then
   echo ""
   echo "ist2D.com Version:    $thisVersion"
endif

if (`string csame $inName Auto`) then
   set inName = test.fid

   if ($pseudo3DFlag) then
      set fList  = (nuszf.fid test.fid spec.fid fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid)
   else
      set fList  = (nuszf.fid test.fid spec.fid nuszf/test001.fid nuszf/spec001.fid nuszf/test0001.fid nuszf/spec0001.fid fid/test001.fid data/test001.fid fid/spec001.fid data/spec001.fid fid/test0001.fid data/test0001.fid)
   endif

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

if ($retainFlag) then
   set istPrefix = ist
else 
   set istPrefix = ist
endif

if ($pseudo3DFlag) then
   set thisMaskName = mask/test%03d.fid
   set thisOutName  = $istPrefix/test%03d.ft2
else
   set thisMaskName = mask.fid
   set thisOutName  = ${istPrefix}.ft2
endif

set maskName   = (`getArgD $argv -mask     $thisMaskName`)
set outName    = (`getArgD $argv -out      $thisOutName`)
set outName    = (`getArgD $argv -istName  $outName`)
set iftName    = (`getArgD $argv -ft1      None`)
set titStr     = (`getArgD $argv -title    None`)
set finalName  = $outName

set thisDir = (`string path $outName`)

if (!(-d $thisDir)) then
   if (`string same $vFlag verb`) then
      echo "ist2D.com New Directory: $thisDir"
      echo ""
   endif

   mkdir $thisDir
endif

set origInName   = $inName
set origMaskName = $maskName
set origIFTName  = $iftName

set tMult        = (`getArgD $argv -istTMult  0.70`)
set cMult        = (`getArgD $argv -istCMult  0.30`)
set istIter      = (`getArgD $argv -istIter   2048`)
set istMaxRes    = (`getArgD $argv -istMaxRes Auto`)
set mult         = (`getArgD $argv -mult      1.0`)
set scaleToVal   = (`getArgD $argv -scaleTo   None`)

set zPlane       = (`getArgD $argv -zPlane    1`)
set aPlane       = (`getArgD $argv -aPlane    1`)

set istSaveFlag  = (`getLogArgD $argv -noistSave  -istSave  0 1`)
set istCleanFlag = (`getLogArgD $argv -clean      -noclean  1 0`)
set nusZFFlag    = (`getLogArgD $argv -nusZF      -nonusZF  1 0`)
set apodDF       = ""

if (`string csame $maskName Auto`) then
   @ nusFlag = 0
endif

if (`flagLoc $argv -nus`) then
   @ nusFlag = 1
endif

if (`flagLoc $argv -us`) then
   @ nusFlag = 0
endif

if ($retainFlag) then
   @ apodFlag = 0
endif

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if (`string csame $iftName None`) then
   @ skipCount++
else
   @ apodFlag = 0
endif

if (`flagLoc $argv -apod`) then
   @ apodFlag = 1
endif

if (`flagLoc $argv -noapod`) then
   @ apodFlag = 0
endif

if (`string same $vFlag verb`) then
   if ($listFlag) then
      @ skipCount++
   else
      echo "ist2D.com Input:      $inName"
      echo "ist2D.com Output:     $outName"

      if (`string csame $istMaxRes Auto`) then
         echo "ist2D.com Parameters: istTMult: $tMult istCMult: $cMult istMaxRes: Auto" 
      else
         echo "ist2D.com Parameters: istTMult: $tMult istCMult: $cMult istMaxRes:" `nmrPrintf %.4f $istMaxRes`
      endif

      echo ""
   endif
endif

set thisPID      = $$
set progressFlag = 0
set sendName     = nmrProgress.tcl

if (`flagLoc $argv -progress`) then
   set progressFlag = 1
   set sendName     = nmrProgress.tcl
endif

if (`flagLoc $argv -noprogress`) then
   set progressFlag = 0
   set sendName     = None
endif

if ($progressFlag != 0) then
   nmrProgress.com -stop -noprogress
   nmrProgress.tcl &
   sleep 3
endif

set thisName = (`nmrPrintf $origInName 1 1`)

if (!(-e $thisName)) then
   echo "ist2D.com Notice: Missing Time-Domain Input $inName"
   exit 1
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL           = "SOL"
   set uxEXTX1         = "3%"
   set uxEXTXN         = "47%"

   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "POLY,auto"

   if ($nusFlag == 0) then
      set uxBASEARG       = "POLY,auto"
      set uxBASEARG_FINAL = "NULL"
   endif
else
   set uxSOL           = "NULL"
   set uxEXTX1         = "0%"
   set uxEXTXN         = "100%"
   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "NULL"
endif

if ($ssNMRFlag) then
   set uxSOL           = "NULL"
   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "NULL"
endif

set apArgs  = (`getArgD $argv -apArgs  None`)
set apOrd   = (`getArgD $argv -apOrd   0`)
set xSOL    = (`getArgD $argv -xSOL    $uxSOL`)
set xAPOD   = (`getArgD $argv -xAPOD   SP`)
set xQ1     = (`getArgD $argv -xQ1     0.5`)
set xQ2     = (`getArgD $argv -xQ2     0.95`)
set xQ3     = (`getArgD $argv -xQ3     2`)
set xELB    = (`getArgD $argv -xELB    0.0`)
set xGLB    = (`getArgD $argv -xGLB    0.0`)
set xGOFF   = (`getArgD $argv -xGOFF   0.0`)
set xC1     = (`getArgD $argv -xC1     Auto`)
set xZFARG  = (`getArgD $argv -xZFARG  zf=1,auto`)
set xFTARG  = (`getArgD $argv -xFTARG  None`)
set xP0     = (`getArgD $argv -xP0     0.0`)
set xP1     = (`getArgD $argv -xP1     0.0`)
set xP0EXT  = (`getArgD $argv -xP0EXT  0.0`)
set xP1EXT  = (`getArgD $argv -xP1EXT  0.0`)
set xEXTX1  = (`getArgD $argv -xEXTX1  $uxEXTX1`)
set xEXTXN  = (`getArgD $argv -xEXTXN  $uxEXTXN`)
set xPREFT  = (`getArgD $argv -xPREFT  NULL`)
set xPOSTFT = (`getArgD $argv -xPOSTFT NULL`)

if (`string same $xP0EXT 0.0` && `string same $xP1EXT 0.0`) then
   @ usePSEXTFlag = 0
else
   @ usePSEXTFlag = 1
endif

set xBASEARG_INIT  = (`getArgD $argv -xBASEARG_INIT  NULL`)
set xBASEARG_FT1   = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG       = (`getArgD $argv -xBASEARG       $uxBASEARG`)
set xBASEARG_FINAL = (`getArgD $argv -xBASEARG_FINAL $uxBASEARG_FINAL`)

set rBASEARG       = (`getArgD $argv -rBASEARG       POLY,auto,ord=0`)

if (`string csame $xZFARG None`) then
   set xZFARG = "zf=0"
endif

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set yAPOD   = (`getArgD $argv -yAPOD   SP`)
set yQ1     = (`getArgD $argv -yQ1     0.50`)
set yQ2     = (`getArgD $argv -yQ2     0.95`)
set yQ3     = (`getArgD $argv -yQ3     1`)
set yELB    = (`getArgD $argv -yELB    0.0`)
set yGLB    = (`getArgD $argv -yGLB    0.0`)
set yGOFF   = (`getArgD $argv -yGOFF   0.0`)
set yC1     = (`getArgD $argv -yC1     Auto`)
set yZFARG  = (`getArgD $argv -yZFARG  zf=1,auto`)
set yFTARG  = (`getArgD $argv -yFTARG  None`)
set yP0     = (`getArgD $argv -yP0     0.0`)
set yP1     = (`getArgD $argv -yP1     0.0`)
set yPREFT  = (`getArgD $argv -yPREFT  NULL`)
set yPOSTFT = (`getArgD $argv -yPOSTFT NULL`)

set yNUSZF         = (`getArgD $argv -yNUSZF         zf=1`)
set yBASEARG       = (`getArgD $argv -yBASEARG       NULL`)
set yBASEARG_FINAL = (`getArgD $argv -yBASEARG_FINAL NULL`)

if (`string csame $yZFARG None`) then
   set yZFARG = "zf=0"
endif

if (`string csame $yNUSZF None`) then
   set yNUSZF = "zf=0"
endif

if (`string csame $yC1 Auto`) then
   set v = (`nmrPrintf "%f" $yP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif
endif

set yQ1Final       = $yQ1
set yQ2Final       = $yQ2
set yQ3Final       = $yQ3
set yELBFinal      = $yELB
set yGLBFinal      = $yGLB
set yGOFFFinal     = $yGOFF

if (!($apodFlag)) then
   set yAPOD = SP
   set yQ1   = 0.5
   set yQ2   = 0.5
   set yQ3   = 1
   set yELB  = 0.0
   set yGLB  = 0.0
   set yGOFF = 0.0
endif

if (`string csame $xP0 Auto`) then
   if (`string csame $apArgs None`) then
      set apArgInfo = ""
   else
      set apArgInfo = (`expandArg $apArgs`)
   endif

   if (`string csame $zPlane 1` == 0) then
      set apArgInfo = "$apArgInfo -apzPlane $zPlane"
   endif

   if (`string csame $aPlane 1` == 0) then
      set apArgInfo = "$apArgInfo -apaPlane $aPlane"
   endif

   if (`string csame $apOrd None`) then
      set apOrd = -1
   endif

   if (`string same $vFlag verb`) then
      echo "ist2D.com X-Axis Auto-Phase ..."
   endif

   set xP0 = (`basicAutoPhase.com -in $origInName $apArgInfo -apxP1 $xP1 -apyP0 $yP0 -apyP1 $yP1 -apyFTARG $yFTARG -apOrd $apOrd`)

   if (`string same $vFlag verb`) then
      nmrPrintf "ist2D.com xP0: %.1f\n\n" $xP0
   endif
endif

set multDir  = ${istPrefix}mult
set specDir  = ${istPrefix}spec

set ft1Name     = ${istPrefix}_ft.ft1
set istMask     = ${istPrefix}_mask.ft1
set sumName     = ${istPrefix}_sum.ft2
set resName     = ${istPrefix}_res.ft2
set invName     = ${istPrefix}_inv.ft1
set tmpNName    = ${istPrefix}_tmp.dat
set tmp0Name    = ${istPrefix}_tmp.fid
set tmp1Name    = ${istPrefix}_tmp.ft1
set tmp2Name    = ${istPrefix}_tmp.ft2
set tmp3Name    = ${istPrefix}_pseudo3D_tmp.ft2
set tpName      = ${istPrefix}_tmp_tp.ft2
set zfName      = ${istPrefix}_nuszf.fid
set zfMaskName  = ${istPrefix}_nuszf_mask.fid

if ($retainFlag) then
   if (`string csame $iftName None`) then
      set iftName = ${istPrefix}_tmp_ift.ft1
   endif
endif

if ($nusZFFlag) then
   foreach f ($zfName $zfMaskName)
      if (-e $f) then
         /bin/rm -f $i
      endif
   end

   if (`string same $vFlag verb`) then
      echo "IST NUS Zero-Fill Extrapolation: Creating Zero Fill and Mask Data ..."
   endif

   if ($pseudo3DFlag) then
      nusZF.com -in $inName -out $zfName -mask $zfMaskName -sample None \
                -xNUSZF None -yNUSZF $yNUSZF -$vFlag -pseudo3D
   else
      if (`string csame $maskName Auto`) then
         nusZF.com -in $inName -out $zfName -mask $zfMaskName -sample None \
                   -xNUSZF None -yNUSZF $yNUSZF -$vFlag
      else
         nusZF.com -in $inName -out $zfName -mask None -sample None \
                   -xNUSZF None -yNUSZF $yNUSZF -$vFlag

         nusZF.com -in $maskName -out $zfMaskName -mask None -sample None \
                   -xNUSZF None -yNUSZF $yNUSZF -$vFlag
      endif
   endif

   if (`string same $vFlag verb`) then
      echo ""
   endif

   set inName   = $zfName
   set maskName = $zfMaskName
endif

set thisName = (`nmrPrintf $maskName   1 1`)

if (!(-e $thisName)) then
   echo "ist2D.com Notice: Missing Mask Input $maskName"
   exit 1
endif

@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount != 0) then
   xyz2pipe -in $inName -out $tmp0Name 
   set inName  = $tmp0Name
   set outName = $tmp3Name
endif

@ fmtCount = (`echo $outName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount != 0) then
   set outName = $tmp3Name
endif

if (-d $multDir) then
   /bin/rm -rf $multDir
endif

if (-d $specDir) then
   /bin/rm -rf $specDir 
endif

foreach f ($tmpNName $tmp1Name $tmp2Name $iftName $ft1Name $sumName $invName $outName $finalName)
   if (-e $f) then
      /bin/rm -f $f
   endif 
end

if ($istIter < 1) then
   exit 0
endif

if (`string same $vFlag verb`) then
   echo "Preparing Interferogram $ft1Name from $inName ..."
endif

mkdir $multDir
mkdir $specDir

nmrPipe -in $inName \
| nmrPipe -fn `expandArg0 $xPREFT` \
| nmrPipe -fn `expandArg0 $xSOL` \
| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn FT `expandArg $xFTARG` \
| nmrPipe -fn PS -p0 $xP0 -p1 $xP1 \
| nmrPipe -fn `expandArg0 $xBASEARG_INIT` \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn PS -p0 $xP0EXT -p1 $xP1EXT -di \
| nmrPipe -fn `expandArg0 $xBASEARG_FT1` \
| nmrPipe -fn `expandArg0 $xPOSTFT` \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $yPREFT` \
| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \
| nmrPipe -fn ZF `expandArg $yZFARG` \
| nmrPipe -fn FT `expandArg $yFTARG` \
| nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
| nmrPipe -fn `expandArg0 $yBASEARG` \
| nmrPipe -fn `expandArg0 $yPOSTFT` \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $xBASEARG` \
| nmrPipe -fn TP \
| nmrPipe -fn HT -auto -nozf \
| nmrPipe -fn PS -inv -hdr \
| nmrPipe -fn FT -inv \
| nmrPipe -fn EXT -time -sw \
   -out $ft1Name -ov

if (`string same $vFlag verb`) then
   echo "Preparing Interferogram Mask $istMask from $maskName ..."
endif

@ fmtCount = (`echo $maskName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount == 0) then
   set inProc = nmrPipe
else
   set inProc = xyz2pipe
endif

$inProc -in $maskName \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn SET -locC 1 \
| nmrPipe -fn FT -null -di \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn TP \
  -out $istMask -ov

set thisInName = $ft1Name

@ i  = 1
@ ii = 1

if (`string same $vFlag verb`) then
   echo ""
endif

if ($progressFlag != 0) then
   set thisMsg = "nmrProgress -loc 0 -n $istIter -pct 0.0 -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

if (-e ist.stop) then
   /bin/rm -f ist.stop
endif

@ convFlag = 0

set yZFInfo = (`expandArg $yZFARG`)

while( $i <= $istIter )
   if ($istSaveFlag) then
      @ ii = $i
   else
      @ ii = 1
   endif
 
   addNMR -in1 $thisInName -in2 $istMask -out $tmp1Name -mul

   nmrPipe -in $tmp1Name \
   | nmrPipe -fn ZF $yZFInfo \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -hdr -di \
      -out $tmp2Name -ov

   if (`string csame $istMaxRes Auto`) then
      if ($listFlag) then
         @ skipCount++
      else
         nmrPipe -in $tmp2Name -fn TP | nmrPipe -fn `expandArg0 $xBASEARG_FINAL` -out $tpName -ov

         set istMaxRes = (`specStat.com -in $tpName -stat istMaxRes -mult $mult -brief`)

         if (`string same $vFlag verb`) then
            echo "ist2D.com istMaxRes Auto Estimate:" `nmrPrintf %.4f $istMaxRes`
            echo ""
         endif
      endif
   endif

#  set thresh  = (`scale2D -brief -abs $tmp2Name`)

   set minVal  = (`getParm -in $tmp2Name -parm FDMIN`)
   set maxVal  = (`getParm -in $tmp2Name -parm FDMAX`)
   set thresh  = (`EMATH "fMax(abs($minVal),abs($maxVal))"`)

   if ($i == 1) then
      set firstThresh = $thresh
   endif

   set thisThresh = (`MATH "100.0*div($thresh,$firstThresh)"`)

   if (`string same $vFlag verb`) then
      nmrPrintf " %4d. Threshold: %7.3f%%\n" $i $thisThresh
   endif

   set thresh   = (`MATH "$thresh*$tMult"`)
   set multName = (`nmrPrintf $multDir/spec%03d.ft2 $ii`)
   set specName = (`nmrPrintf $specDir/spec%03d.ft2 $ii`)

   nmrPipe -in $tmp2Name -out $multName -ov -fn MULT -c $cMult -thresh  $thresh

   if ($posFlag) then
      nmrPipe -in $tmp2Name -fn MULT -c $cMult -thresh -$thresh | nmrPipe -fn CLIP -below 0.0 -out $specName -ov
   else
      nmrPipe -in $tmp2Name -fn MULT -c $cMult -thresh -$thresh -out $specName -ov
   endif

   if ($i == 1) then
      nmrPipe -in $specName -out $sumName -fn SET -c 0.0 -ov
   endif

   addNMR -in1 $sumName -in2 $specName -out $tmp2Name -add
   /bin/mv $tmp2Name $sumName

   nmrPipe -in $multName \
   | nmrPipe -fn HT -auto -nozf \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -sw \
     -out $invName -ov

   if ($progressFlag != 0) then
      set progPct = (`MATH "100.0 - 100.0*div($thisThresh-$istMaxRes, 100.0-$istMaxRes)"`)
      set thisMsg = "nmrProgress -loc $i -pct `nmrPrintf %.3f $progPct` -n $istIter -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
      nmrSend -norecv -send $sendName -msg "$thisMsg"
   endif

   set thisInName = $invName 
   @ i++

   if (`IMATH "$thisThresh < $istMaxRes"`) then
      @ convFlag = 1
      break
   endif 

  if (-e ist.stop) then
     if (`string same $vFlag verb`) then
        echo "Found File ist.stop ... stopping IST Calculation."
     endif

     /bin/rm -f ist.stop
     @ convFlag = 1
     break
  endif
end

if ($progressFlag != 0) then
   if (!($convFlag)) then
      set statusStr = "Iteration Count Exceeded."
   else
      set statusStr = "None"
   endif

   set thisMsg = "nmrProgress -loc $istIter -n $istIter -pct 100.0 -fromID 1 -pid $thisPID -title IST Convergence -axis None -status $statusStr"

   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

if (`string same $vFlag verb`) then
   echo ""
endif

nmrPipe -in $multName -out $resName -ov -fn `expandArg0 $rBASEARG`
addNMR  -in1 $sumName -in2 $resName -out $outName -add

nmrPipe -inPlace -in $resName -out $resName -fn TP -ov
nmrPipe -inPlace -in $sumName -out $sumName -fn TP -ov

nmrPipe -in $ft1Name -fn MULT -c $yC1 -xn 1 -inv -out $ft1Name -inPlace -$vFlag

if (`string csame $iftName None`) then
   @ skipCount++
else
   @ fmtCount = (`echo $iftName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

   if ($fmtCount == 0) then
      set outProc = nmrPipe
   else
      set outProc = pipe2xyz
   endif

  if (`string same $vFlag verb`) then
     echo ""
     echo "Generating Interferogram $iftName from $outName ..."
   endif

   if ($retainFlag) then
      nmrPipe -in $outName \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | nmrPipe -out $tmp1Name -ov

      if (`string same $vFlag verb`) then
         echo "Restoring Original Time Domain Points from $ft1Name to $iftName ..."
      endif

      addNMR -in1 $ft1Name  -in2 $istMask  -out $ft1Name  -mask
      addNMR -in1 $tmp1Name -in2 $istMask  -out $tmpNName -imask
      addNMR -in1 $ft1Name  -in2 $tmpNName -out $tmpNName -add

      nmrPipe -in $tmpNName | $outProc -out $iftName -ov
   else
      nmrPipe -in $outName \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | $outProc -out $iftName -ov
   endif
endif

@ fmtCount = (`echo $iftName   | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount == 0) then
   set inProc = nmrPipe
else
   set inProc = xyz2pipe
endif

@ fmtCount = (`echo $finalName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

if ($fmtCount == 0) then
   set outProc = nmrPipe
else
   set outProc = pipe2xyz
endif

if ($retainFlag) then
   if (`string same $vFlag verb`) then
      echo "Transforming $iftName to Generate $finalName ..."
      echo ""
   endif

   $inProc -in $iftName \
   | nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1Final -q2 $yQ2Final -q3 $yQ3Final -elb $yELBFinal -glb $yGLBFinal -goff $yGOFFFinal -c $yC1 \
   | nmrPipe -fn ZF `expandArg $yZFARG` \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di -noup \
   | nmrPipe -fn `expandArg0 $yBASEARG_FINAL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG_FINAL` -$vFlag \
   | nmrPipe -fn `expandArg0 $xBASEARG_FINAL` \
   | $outProc -out $finalName -ov
else
   if (`string same $outName $finalName`) then
      set ipArg = "-inPlace"
   else
      set ipArg = ""
   endif
   
   nmrPipe -in $outName \
   | nmrPipe -fn `expandArg0 $yBASEARG_FINAL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG_FINAL` -$vFlag \
   | nmrPipe -fn `expandArg0 $xBASEARG_FINAL` \
   | $outProc -out $finalName -ov $ipArg
endif

if ($pseudo3DFlag) then
   if (`string same $vFlag verb`) then
      echo ""
   endif

   copyTau.com $origInName $finalName -$vFlag -all
endif

if (`string csame $scaleToVal None`) then
   @ skipCount++
else
   if (`string same $vFlag verb`) then
      echo ""
      echo "Scaling $finalName to Max $scaleToVal ..."
   endif

   rescale.com -in $finalName -out $finalName -max $scaleToVal
endif

if (`string casecmp $titStr None`) then
   sethdr.com $finalName -title $titStr
endif
   
series.com $multDir/*.ft2
series.com $specDir/*.ft2

if ($istCleanFlag) then
   set fList = (nuslist_zf nuszf.fid nuszf_mask.fid sum.ft2 res.ft2 inv.ft1 tmp.dat tmp.fid tmp.ft1 tmp_tp.ft2 pseudo3D_tmp.ft2 mask.ft1 ft.ft1 tmp_ift.ft1)

   foreach f ($fList)
      if (-e ${istPrefix}_$f) then
         /bin/rm -f ${istPrefix}_$f 
      endif
   end

   set dList = ($multDir $specDir)

   foreach d ($dList)
      if (-d $d) then
         /bin/rm -rf $d
      endif
   end
endif

exit 0
