#!/bin/csh

set thisVersion = "2020.023.18.46"

if (`flagLoc $argv -help`) then
   echo "ist4D.com $thisVersion"
   echo "Perform IST Recontruction of 4D Non-Uniformly Sampled (NUS) Data:"
   echo " -in   inName    [Auto]              Expanded Time-Domain Input."
   echo " -mask maskName  [mask/mask%04d.fid] Corresponding Time-Domain Mask Input."
   echo " -out  outName   [ist/test%04d.ft4]  IST Spectrum Output."
   echo " -ft1  ft1Name   [None]              IST Interferogram Output (Sets -noapod)."
   echo " -nus                                Input is NUS (Default)."
   echo " -us                                 Input is Uniformly Sampled (US)."
   echo " -yz                                 Exchange Y-Axis and Z-Axis of Result."
   echo " -ya                                 Exchange Y-Axis and A-Axis of Result."
   echo "IST Parameters:"
   echo " -istMaxRes r [Auto]  Target Max in Residual, as % of Original Max (Or Auto)."
   echo " -istTMult  t [0.70]  Fraction for Threshold at Each Iteration"
   echo " -istCMult  c [0.30]  Fraction for Reducing Data Above Threshold." 
   echo " -istIter   n [2048]  Maximum Allowed Iterations."
   echo " -mult      m [1.0]   Multiplication Factor for -istMaxRes Auto."
   echo " -retain              Retain Original Time-Domain Data Points (Sets -noapod)."
   echo " -noretain            Replace Original Time-Domain Data Points (Default)."
   echo " -rist                Same as -retain Option."
   echo " -pos                 Use All-Positive IST (No Negative Signals)."
   echo "Noise Estimate (Used Only for Statistic Reports):"
   echo " -noise     s [None]  Estimate of Noise in Spectrum."
   echo "Processing Parameters (Use -x... -y... -z... -a...):"
   echo " -xSOL   sName   [NULL]       Solvent Function Name and Options (X-Axis Only)."
   echo " -xAPOD  aName   [SP]         Window Function Name."
   echo " -xQ1    q1      [0.50]       Window Parameter Q1."
   echo " -xQ2    q2      [0.95]       Window Parameter Q2."
   echo " -xQ3    q3      [2]          Window Parameter Q3 (Default for -yQ3/-zQ3 is 1)."
   echo " -xELB   elb     [0.0]        Exponential Line Broaden, Hz."
   echo " -xGLB   glb     [0.0]        Gaussian Line Broaden, Hz."
   echo " -xGOFF  goff    [0.0]        Gaussian Center, 0.0 to 1.0."
   echo " -xC1    c       [Auto]       First-Point Time-Domain Scale (0.5 1.0 Auto)."
   echo " -xP0    p0      [0.0]        Zero-Order Phase Correction."
   echo " -xP1    p1      [0.0]        First Order Phase Correction."
   echo " -xZFARG zfArgs  [zf=1,auto]  ZF (Zero Fill) Option."
   echo " -xFTARG ftArg   [None]       FT (Fourier Transform) Options (None neg alt)."
   echo " -xEXTX1 x1      [0%]         Extract Range Start (X-Axis Only) (% Hz ppm pts)."
   echo " -xEXTXN xn      [100%]       Extract Range End   (X-Axis Only) (% Hz ppm pts)."
   echo " -apodDF                      Adjust Window Function for Digital Oversampling."
   echo " -noapod                      No Apodization for Indirect Dimensions."
   echo " -ssNMR                       Set Defaults for Solid-State NMR Data."
   echo "Processing Parameters for X-Axis Only:"
   echo " -xBASEARG_INIT bArgs [NULL]  Baseline Correction Before Region Extraction."
   echo " -xP0EXT        p0Ext [0.0]   Zero-Order Phase After Region Extraction."
   echo " -xP1EXT        p1Ext [0.0]   First Order Phase After Region Extraction."
   echo "NUS Zero-Fill Mode (Time-Domain Extrapolation by IST):"
   echo " -nusZF                       NUS Zero-Fill Extrapolation ON (Default)."
   echo " -nonusZF                     NUS Zero-Fill Extrapolation OFF."
   echo " -yNUSZF  zfArgs  [zf=1]      Amount of Y-Axis NUS Zero-Fill Extrapolation."
   echo " -zNUSZF  zfArgs  [zf=1]      Amount of Z-Axis NUS Zero-Fill Extrapolation."
   echo " -aNUSZF  zfArgs  [zf=1]      Amount of A-Axis NUS Zero-Fill Extrapolation."
   echo " -istName outName [ist/test%04d.ft4]  Overrides -out Setting."
   echo "Baseline Correction Function Name and Options (Use -x... -y... -z...):"
   echo " -xBASEARG_FT1    [NULL]      Interferogram Baseline Correction (X-Axis Only)."
   echo " -xBASEARG        [NULL]      Spectrum Baseline Correction."
   echo " -xBASEARG_FINAL  [NULL]      Baseline Correction for Final IST Result."
   echo "Baseline Correction Applied to Final Residual:"
   echo " -rBASEARG        [POLY,auto,ord=0] Residual Baseline Correction."
   echo "Other Options:"
   echo " -clean           Delete Intermediate Results (Default)."
   echo " -noclean         Retain Intermediate Results."
   echi " -istSave         Save Separate Results for Each Iteration."
   echo " -progress        Display Progress Bar Graphic."
   echo " -yz              Move Z-Axis to Y-Axis of Result."
   echo " -ya              Move A-Axis to Y-Axis of Result."
   echo ""
   echo "Notes:"
   echo " 1. Options -xSOL -xBASEARG_FT1 -xBASEARG and -xBASEARG_FINAL are expanded"
   echo "    into NMRPipe processing function names and command line arguments."
   echo "    Options should contain no spaces. Flags are separated by commas, and"
   echo "    values are given using = signs. For example, solvent filter:"
   echo ""
   echo "      -xSOL     NULL            Becomes: nmrPipe -fn NULL (No Solvent Filter)"
   echo "      -xSOL     SOL             Becomes: nmrPipe -fn SOL"
   echo "      -xSOL     POLY,time       Becomes: nmrPipe -fn POLY -time"
   echo ""
   echo "    For baseline correction:"
   echo ""
   echo "      -xBASEARG NULL            Becomes: nmrPipe -fn NULL (No Correction)"
   echo "      -xBASEARG POLY,auto,ord=0 Becomes: nmrPipe -fn POLY -auto -ord 0"
   echo ""
   echo ""
   echo " 2. Options -xZFARG -xFTARG are expanded to command line arguments for ZF"
   echo "    and FT, for example:"
   echo ""
   echo "      -xZFARG zf=2,auto       Becomes: nmrPipe -fn ZF -zf 2 -auto"
   echo "      -xFTARG neg             Becomes: nmrPipe -fn FT -neg"
   echo "      -xFTARG neg,alt         Becomes: nmrPipe -fn FT -neg -alt"
   echo ""
   echo " 3. When first-point time-domain scaling is set to Auto (-xC1 Auto etc),"
   echo "    the scaling value will be set to 0.5 if the first-order phase correction"
   echo "    for that dimension is 0+/-10 degrees, or to 1.0 otherwise."
   echo ""
   echo " 4. For applying NUS Zero Fill to conventional data, use -nusZF -mask Auto to" 
   echo "    automatically create mask data via the nusZF.com script."
   echo ""
   echo " 5. If the X-Axis label of the input spectrum starts with 'HN', default"
   echo "    parameters will be adjusted as follows:"
   echo ""
   echo "       -xSOL SOL -xBASEARG_FINAL POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo "    Or when -nusZF -mask Auto is used for conventional data:"
   echo ""
   echo "       -xSOL SOL -xBASEARG POLY,auto -xEXTX1 3% -xEXTXN 47%"
   echo ""
   echo " 6. If the -ssNMR flag is used, default parameters will be:"
   echo ""
   echo "       -xSOL NULL -xBASEARG NULL -xBASEARG_FINAL NULL"
   echo ""
   echo " 7. Commonly, only region of interest, FT, and phase correction options need"
   echo "    to be provided, and the convergence parameter can be set automatically."
   echo "    Include -nusZF if extrapolation during reconstruction is desired. Example:"
   echo ""
   echo "    ist4D.com -xP0 -75 -xP1   0 -xEXTX1 10.4ppm -xEXTXN 5.4ppm \"
   echo "              -yP0 -90 -yP1 180 -zFTARG alt -aFTARG alt -nusZF"
   echo ""
   echo " 8. To stop an IST calculation in progress:"
   echo ""
   echo "       touch ist.stop"
   echo ""
   echo " 9. See utility nusExpand.tcl for expanding time-domain NUS data and"
   echo "    creating time-domain NUS schedule mask."
   exit 0
endif

set vFlag        = (`getLogArgD $argv -verb      -noverb  verb noverb`)
set ssNMRFlag    = (`getLogArgD $argv -nossNMR   -ssNMR   0    1`)
set retainFlag   = (`getLogArgD $argv -noretain  -retain  0    1`)
set posFlag      = (`getLogArgD $argv -nopos     -pos     0    1`)

if (`flagLoc $argv -rist`) then
   set retainFlag = 1
endif

@ listFlag = 0
@ nusFlag  = 1
@ apodFlag = 1
@ yzaFlag  = 0

set inName = (`getArgD $argv -in Auto`)

if (`string same $vFlag verb`) then
   echo ""
   echo "ist4D.com Version:    $thisVersion"
endif

if (`string csame $inName Auto`) then
   set inName = test.fid
   set fList  = (nuszf.fid nuszf/test%03d.fid nuszf/spec%03d.fid nuszf/test%04d.fid nuszf/spec%04d.fid test.fid spec.fid fid/test%03d.fid data/test%03d.fid fid/spec%03d.fid data/spec%03d.fid fid/test%04d.fid data/test%04d.fid)

   foreach i ($fList)
      set thisName = (`nmrPrintf $i 1 1`)

      if (-e $thisName) then
         set inName = $i
         break
      endif
   end
endif

set maskName   = (`getArgD $argv -mask    mask/test%04d.fid`)
set outName    = (`getArgD $argv -out     ist/test%04d.ft4`)
set outName    = (`getArgD $argv -istName $outName`)
set iftName    = (`getArgD $argv -ft1     None`)

set origInName  = $inName
set origIFTName = $iftName

set istPrefix = ist
set tMult     = (`getArgD $argv -istTMult  0.70`)
set cMult     = (`getArgD $argv -istCMult  0.30`)
set istIter   = (`getArgD $argv -istIter   1024`)
set istMaxRes = (`getArgD $argv -istMaxRes Auto`)
set mult      = (`getArgD $argv -mult      1.0`)
set noise     = (`getArgD $argv -noise     None`)
set ft1Noise  = (`getArgD $argv -ft1Noise  None`)

set istSaveFlag  = (`getLogArgD $argv -noistSave -istSave 0 1`)
set istCleanFlag = (`getLogArgD $argv -clean     -noclean 1 0`)
set nusZFFlag    = (`getLogArgD $argv -nusZF     -nonusZF 1 0`)
set apodDF       = ""

if (`string csame $maskName Auto`) then
   @ nusFlag = 0
endif

if (`flagLoc $argv -nus`) then
   @ nusFlag = 1
endif

if (`flagLoc $argv -us`) then
   @ nusFlag = 0
endif

if ($retainFlag) then
   @ apodFlag = 0
endif

if (`string csame $iftName None`) then
   @ skipCount++
else
   @ apodFlag = 0
endif

if (`flagLoc $argv -apod`) then
   @ apodFlag = 1
endif

if (`flagLoc $argv -noapod`) then
   @ apodFlag = 0
endif

if (`flagLoc $argv -apodDF`) then
   set apodDF = "-df"
endif

if (`flagLoc $argv -noapodDF`) then
   set apodDF = ""
endif

if (`flagLoc $argv -yz`) then
   set yzaFlag = 1
endif

if (`flagLoc $argv -ya`) then
   set yzaFlag = 2
endif

if (`flagLoc $argv -noyz`) then
   @ yzaFlag = 0
endif

if (`flagLoc $argv -noya`) then
   @ yzaFlag = 0
endif

if (`string same $vFlag verb`) then
   if ($listFlag) then
   else
      echo "ist4D.com Input:      $inName"
      echo "ist4D.com Output:     $outName"

      if (`string csame $istMaxRes Auto`) then
         echo "ist4D.com Parameters: istTMult: $tMult istCMult: $cMult istMaxRes: Auto"
      else
         echo "ist4D.com Parameters: istTMult: $tMult istCMult: $cMult istMaxRes:" `nmrPrintf %.4f $istMaxRes`
      endif

      echo ""
   endif
endif

set thisPID      = $$
set progressFlag = 0
set sendName     = nmrProgress.tcl

if (`flagLoc $argv -progress`) then
   set progressFlag = 1
   set sendName     = nmrProgress.tcl
endif

if (`flagLoc $argv -noprogress`) then
   set progressFlag = 0
   set sendName     = None
endif

if ($progressFlag != 0) then
   nmrProgress.com -stop -noprogress
   nmrProgress.tcl &
   sleep 3
endif

set thisName = (`nmrPrintf $inName 1 1`)

if (!(-e $thisName)) then
   echo "ist4D.com Notice: Missing Time-Domain Input $inName"
   exit 1
endif

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   set uxSOL           = "SOL"
   set uxEXTX1         = "3%"
   set uxEXTXN         = "47%"

   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "POLY,auto"

   if ($nusFlag) then
      set uxBASEARG       = "POLY,auto"
      set uxBASEARG_FINAL = "NULL"
   endif
else
   set uxSOL           = "NULL"
   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "NULL"
   set uxEXTX1         = "0%"
   set uxEXTXN         = "100%"
endif

if ($ssNMRFlag) then
   set uxSOL           = "NULL"
   set uxBASEARG       = "NULL"
   set uxBASEARG_FINAL = "NULL"
endif

set xSOL   = (`getArgD $argv -xSOL   $uxSOL`)
set xAPOD  = (`getArgD $argv -xAPOD  SP`)
set xQ1    = (`getArgD $argv -xQ1    0.5`)
set xQ2    = (`getArgD $argv -xQ2    0.95`)
set xQ3    = (`getArgD $argv -xQ3    2`)
set xELB   = (`getArgD $argv -xELB   0.0`)
set xGLB   = (`getArgD $argv -xGLB   0.0`)
set xGOFF  = (`getArgD $argv -xGOFF  0.0`)
set xC1    = (`getArgD $argv -xC1    Auto`)
set xZFARG = (`getArgD $argv -xZFARG zf=1,auto`)
set xFTARG = (`getArgD $argv -xFTARG None`)
set xP0    = (`getArgD $argv -xP0    0.0`)
set xP1    = (`getArgD $argv -xP1    0.0`)
set xP0EXT = (`getArgD $argv -xP0EXT 0.0`)
set xP1EXT = (`getArgD $argv -xP1EXT 0.0`)
set xEXTX1 = (`getArgD $argv -xEXTX1 $uxEXTX1`)
set xEXTXN = (`getArgD $argv -xEXTXN $uxEXTXN`)

if (`string same $xP0EXT 0.0` && `string same $xP1EXT 0.0`) then
   @ usePSEXTFlag = 0
else
   @ usePSEXTFlag = 1
endif

set xBASEARG_INIT  = (`getArgD $argv -xBASEARG_INIT  NULL`)
set xBASEARG_FT1   = (`getArgD $argv -xBASEARG_FT1   NULL`)
set xBASEARG       = (`getArgD $argv -xBASEARG       $uxBASEARG`)
set xBASEARG_FINAL = (`getArgD $argv -xBASEARG_FINAL $uxBASEARG_FINAL`)

set rBASEARG       = (`getArgD $argv -rBASEARG       POLY,auto,ord=0`)

if (`string csame $xZFARG None`) then
   set xZFARG = "zf=0"
endif

if (`string csame $xC1 Auto`) then
   set v = (`nmrPrintf "%f" $xP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif
endif

set yAPOD  = (`getArgD $argv -yAPOD  SP`)
set yQ1    = (`getArgD $argv -yQ1    0.50`)
set yQ2    = (`getArgD $argv -yQ2    0.95`)
set yQ3    = (`getArgD $argv -yQ3    1`)
set yELB   = (`getArgD $argv -yELB   0.0`)
set yGLB   = (`getArgD $argv -yGLB   0.0`)
set yGOFF  = (`getArgD $argv -yGOFF  0.0`)
set yC1    = (`getArgD $argv -yC1    Auto`)
set yZFARG = (`getArgD $argv -yZFARG zf=1,auto`)
set yFTARG = (`getArgD $argv -yFTARG None`)
set yP0    = (`getArgD $argv -yP0    0.0`)
set yP1    = (`getArgD $argv -yP1    0.0`)

set yNUSZF         = (`getArgD $argv -yNUSZF         zf=1`)
set yBASEARG       = (`getArgD $argv -yBASEARG       NULL`)
set yBASEARG_FINAL = (`getArgD $argv -yBASEARG_FINAL NULL`)

if (`string csame $yZFARG None`) then
   set yZFARG = "zf=0"
endif

if (`string csame $yNUSZF None`) then
   set yNUSZF = "zf=0"
endif

if (`string csame $yC1 Auto`) then
   set v = (`nmrPrintf "%f" $yP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif
endif

set zAPOD  = (`getArgD $argv -zAPOD  SP`)
set zQ1    = (`getArgD $argv -zQ1    0.50`)
set zQ2    = (`getArgD $argv -zQ2    0.95`)
set zQ3    = (`getArgD $argv -zQ3    1`)
set zELB   = (`getArgD $argv -zELB   0.0`)
set zGLB   = (`getArgD $argv -zGLB   0.0`)
set zGOFF  = (`getArgD $argv -zGOFF  0.0`)
set zC1    = (`getArgD $argv -zC1    Auto`)
set zZFARG = (`getArgD $argv -zZFARG zf=1,auto`)
set zFTARG = (`getArgD $argv -zFTARG None`)
set zP0    = (`getArgD $argv -zP0    0.0`)
set zP1    = (`getArgD $argv -zP1    0.0`)

set zNUSZF         = (`getArgD $argv -zNUSZF         zf=1`)
set zBASEARG       = (`getArgD $argv -zBASEARG       NULL`)
set zBASEARG_FINAL = (`getArgD $argv -zBASEARG_FINAL NULL`)

if (`string csame $zZFARG None`) then
   set zZFARG = "zf=0"
endif

if (`string csame $zNUSZF None`) then
   set zNUSZF = "zf=0"
endif

if (`string csame $zC1 Auto`) then
   set v = (`nmrPrintf "%f" $zP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set zC1 = 1.0
   else
      set zC1 = 0.5
   endif
endif

set aAPOD  = (`getArgD $argv -aAPOD  SP`)
set aQ1    = (`getArgD $argv -aQ1    0.50`)
set aQ2    = (`getArgD $argv -aQ2    0.95`)
set aQ3    = (`getArgD $argv -aQ3    1`)
set aELB   = (`getArgD $argv -aELB   0.0`)
set aGLB   = (`getArgD $argv -aGLB   0.0`)
set aGOFF  = (`getArgD $argv -aGOFF  0.0`)
set aC1    = (`getArgD $argv -aC1    Auto`)
set aZFARG = (`getArgD $argv -aZFARG zf=1,auto`)
set aFTARG = (`getArgD $argv -aFTARG None`)
set aP0    = (`getArgD $argv -aP0    0.0`)
set aP1    = (`getArgD $argv -aP1    0.0`)

set aNUSZF         = (`getArgD $argv -aNUSZF         zf=1`)
set aBASEARG       = (`getArgD $argv -aBASEARG       NULL`)
set aBASEARG_FINAL = (`getArgD $argv -aBASEARG_FINAL NULL`)

if (`string csame $aZFARG None`) then
   set aZFARG = "zf=0"
endif

if (`string csame $aNUSZF None`) then
   set aNUSZF = "zf=0"
endif

if (`string csame $aC1 Auto`) then
   set v = (`nmrPrintf "%f" $aP1`)
   @ i   = (`IMATH "abs($v) < 10.0"`)

   if (!($i)) then
      set aC1 = 1.0
   else
      set aC1 = 0.5
   endif
endif

set yQ1Final       = $yQ1
set yQ2Final       = $yQ2
set yQ3Final       = $yQ3
set yELBFinal      = $yELB
set yGLBFinal      = $yGLB
set yGOFFFinal     = $yGOFF

set zQ1Final       = $zQ1
set zQ2Final       = $zQ2
set zQ3Final       = $zQ3
set zELBFinal      = $zELB
set zGLBFinal      = $zGLB
set zGOFFFinal     = $zGOFF

set aQ1Final       = $aQ1
set aQ2Final       = $aQ2
set aQ3Final       = $aQ3
set aELBFinal      = $aELB
set aGLBFinal      = $aGLB
set aGOFFFinal     = $aGOFF

if (!($apodFlag)) then
   set yAPOD = SP
   set yQ1   = 0.5
   set yQ2   = 0.5
   set yQ3   = 1
   set yELB  = 0.0
   set yGLB  = 0.0
   set yGOFF = 0.0

   set zAPOD = SP
   set zQ1   = 0.5
   set zQ2   = 0.5
   set zQ3   = 1
   set zELB  = 0.0
   set zGLB  = 0.0
   set zGOFF = 0.0

   set aAPOD = SP 
   set aQ1   = 0.5
   set aQ2   = 0.5
   set aQ3   = 1
   set aELB  = 0.0
   set aGLB  = 0.0
   set aGOFF = 0.0
endif

set multDir   = ${istPrefix}mult
set specDir   = ${istPrefix}spec
set resDir    = ${istPrefix}res
set tmpDir    = ${istPrefix}tmp

set ft1Name     = ${istPrefix}_ft1.ft1
set istName     = ${istPrefix}_ft4.ft4
set istMask     = ${istPrefix}_mask.ft1
set sumName     = ${istPrefix}_sum.ft4
set resName     = ${istPrefix}_res.ft4
set invName     = ${istPrefix}_inv.ft1
set tmp1Name    = ${istPrefix}_tmp.ft1
set tmp4Name    = ${istPrefix}_tmp.ft4
set zfName      = ${istPrefix}_nuszf.fid
set zfMaskName  = ${istPrefix}_nuszf_mask.fid
set zfSchedName = ${istPrefix}_nuslist_zf

if ($retainFlag) then
   if (`string csame $iftName None`) then
      set iftName = ${istPrefix}_tmp_ift.ft1
   endif
endif

if ($nusZFFlag) then
   foreach f ($zfName $zfMaskName $zfSchedName)
      if (-e $f) then
         /bin/rm -f $i
      endif
   end

   if (`string same $vFlag verb`) then
      echo "NUS Zero-Fill Extrapolation: Creating ZF File $zfName and Mask File $zfMaskName ..."
   endif

   if (`string csame $maskName Auto`) then
      nusZF.com -in $inName -out $zfName -mask $zfMaskName -sample None \
                -xNUSZF None -yNUSZF $yNUSZF -zNUSZF $zNUSZF -aNUSZF $aNUSZF -$vFlag
   else
      nusZF.com -in $inName -out $zfName -mask None -sample None \
                -xNUSZF None -yNUSZF $yNUSZF -zNUSZF $zNUSZF -aNUSZF $aNUSZF -$vFlag

      nusZF.com -in $maskName -out $zfMaskName -mask None -sample None \
                -xNUSZF None -yNUSZF $yNUSZF -zNUSZF $zNUSZF -aNUSZF $aNUSZF -$vFlag
   endif

   if (`string same $vFlag verb`) then
      echo ""
   endif

   set inName   = $zfName
   set maskName = $zfMaskName
endif

set thisName = (`nmrPrintf $maskName 1 1`)

if (!(-e $thisName)) then
   echo "ist4D.com Notice: Missing Mask Input $maskName"
   exit 1
endif

foreach d ($multDir $specDir $resDir $tmpDir)
   if (-d $d) then
      /bin/rm -rf $d
   endif

   mkdir $d
end

foreach f ($tmp1Name $ft1Name $sumName $invName $istName $iftName)
   if (-e $f) then
      /bin/rm -f $f
   endif 
end

if ($istIter < 1) then
   exit 0
endif

if (`string same $vFlag verb`) then
   echo "Preparing Spectrum $tmp4Name from $inName ..."
endif

xyz2pipe -in $inName \
| nmrPipe -fn `expandArg0 $xSOL` \
| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 $apodDF \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn FT `expandArg $xFTARG` \
| nmrPipe -fn PS -p0 $xP0 -p1 $xP1 \
| nmrPipe -fn `expandArg0 $xBASEARG_INIT` \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| nmrPipe -fn PS -p0 $xP0EXT -p1 $xP1EXT -di \
| nmrPipe -fn `expandArg0 $xBASEARG_FT1` \
| nmrPipe -fn TP \
| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \
| nmrPipe -fn ZF `expandArg $yZFARG` \
| nmrPipe -fn FT `expandArg $yFTARG` \
| nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
| nmrPipe -fn `expandArg0 $yBASEARG` \
| nmrPipe -fn TP \
| nmrPipe -fn `expandArg0 $xBASEARG` \
| nmrPipe -fn ZTP \
| nmrPipe -fn APOD -qName $zAPOD -q1 $zQ1 -q2 $zQ2 -q3 $zQ3 -elb $zELB -glb $zGLB -goff $zGOFF -c $zC1 \
| nmrPipe -fn ZF `expandArg $zZFARG` \
| nmrPipe -fn FT `expandArg $zFTARG` \
| nmrPipe -fn PS -p0 $zP0 -p1 $zP1 -di \
| nmrPipe -fn `expandArg0 $zBASEARG` \
| nmrPipe -fn ZTP \
| pipe2xyz -out $tmpDir/test%05d.ft3 -a -mem 64

xyz2pipe -in $tmpDir/test%05d.ft3 -x \
| nmrPipe -fn ZTP \
| nmrPipe -fn APOD -qName $aAPOD -q1 $aQ1 -q2 $aQ2 -q3 $aQ3 -elb $aELB -glb $aGLB -goff $aGOFF -c $aC1 \
| nmrPipe -fn ZF `expandArg $aZFARG` \
| nmrPipe -fn FT `expandArg $aFTARG` \
| nmrPipe -fn PS -p0 $aP0 -p1 $aP1 -di \
| nmrPipe -fn `expandArg0 $aBASEARG` \
| nmrPipe -fn ZTP \
   -out $tmp4Name -ov

/bin/rm -rf $tmpDir

if (`string same $vFlag verb`) then
   echo "Preparing Interferogram $ft1Name from $tmp4Name ..."
endif

nmrPipe -in $tmp4Name \
| nmrPipe -fn ZTP \
| nmrPipe -fn HT -auto -nozf \
| nmrPipe -fn PS -inv -hdr \
| nmrPipe -fn FT -inv \
| nmrPipe -fn EXT -time -sw \
| nmrPipe -fn ZTP \
| nmrPipe -fn HT -auto -nozf \
| nmrPipe -fn PS -inv -hdr \
| nmrPipe -fn FT -inv \
| nmrPipe -fn EXT -time -sw -ad \
| nmrPipe -fn TP \
| nmrPipe -fn HT -auto -nozf \
| nmrPipe -fn PS -inv -hdr \
| nmrPipe -fn FT -inv \
| nmrPipe -fn EXT -time -sw \
| nmrPipe -fn TP \
  -out $ft1Name -ov

if (`string same $vFlag verb`) then
   echo "Preparing Interferogram Mask $istMask from $maskName ..."
endif

xyz2pipe -in $maskName \
| nmrPipe -fn ZF `expandArg $xZFARG` \
| nmrPipe -fn SET -locC 1 \
| nmrPipe -fn FT -null -di \
| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \
| pipe2xyz -out $tmpDir/test%05d.ft1 -a -mem 64

xyz2pipe -in $tmpDir/test%05d.ft1 -x \
| nmrPipe -out $istMask -ov

/bin/rm -rf $tmpDir

set thisInName = $ft1Name

@ i  = 1
@ ii = 1

if (`string same $vFlag verb`) then
   echo ""
endif

if ($progressFlag != 0) then
   set thisMsg = "nmrProgress -loc 0 -n $istIter -pct 0.0 -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

if (-e ist.stop) then
   /bin/rm -f ist.stop
endif

@ convFlag = 0

set yZFInfo = (`expandArg $yZFARG`)
set zZFInfo = (`expandArg $zZFARG`)
set aZFInfo = (`expandArg $aZFARG`)

while( $i <= $istIter )
   if ($istSaveFlag) then
      @ ii = $i
   else
      @ ii = 1
   endif

   addPipe.tcl -in1 $thisInName -in2 $istMask -out $tmp1Name -mult

   set tmp2Name = (`nmrPrintf $resDir/spec%05d.ft4 $ii`)

   nmrPipe -in $tmp1Name \
   | nmrPipe -fn ZF $yZFInfo \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -hdr -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZF $zZFInfo \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -hdr -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn ZF $aZFInfo \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -hdr -di \
   | nmrPipe -fn ZTP -minMax \
      -out $tmp2Name -ov

#  set thresh = (`scale2D -brief -abs $tmp2Name`)

   set minVal  = (`getParm -in $tmp2Name -parm FDMIN`)
   set maxVal  = (`getParm -in $tmp2Name -parm FDMAX`)
   set thresh  = (`EMATH "fMax(abs($minVal),abs($maxVal))"`)

   if (`string csame $istMaxRes Auto`) then
      if ($listFlag) then
      else
         set noiseInfo = (`showApod -in $tmp2Name -noverb` 1.0 1.0)
         set istMaxRes = (`MATH "3.0*100.0*$mult*$noiseInfo[1]/$thresh"`)

         if (`string same $vFlag verb`) then
            echo "ist4D.com istMaxRes Auto Estimate:" `nmrPrintf %.4f $istMaxRes`
            echo ""
         endif
      endif
   endif

   if ($i == 1) then
      set firstThresh = $thresh
   endif

   set thisThresh = (`MATH "100.0*div($thresh,$firstThresh)"`)

   if (`string same $vFlag verb`) then
      nmrPrintf " %4d. Threshold: %7.3f%%" $i `MATH "100.0*div($thresh,$firstThresh)"`

      if (`string casecmp $noise None`) then
         nmrPrintf "   Max/Noise: %7.1f" `MATH "div($thresh,$noise)"`
      endif
   
      echo ""
   endif
 
   set thresh   = (`MATH "$thresh*$tMult"`)
   set multName = (`nmrPrintf $multDir/spec%05d.ft4 $ii`)
   set specName = (`nmrPrintf $specDir/spec%05d.ft4 $ii`)

   nmrPipe -in $tmp2Name -fn MULT -c $cMult -thresh  $thresh -out $multName -ov 

   if ($posFlag) then
      nmrPipe -in $tmp2Name -fn MULT -c $cMult -thresh -$thresh | nmrPipe -fn CLIP -below 0.0 -out $specName -ov
   else
      nmrPipe -in $tmp2Name -fn MULT -c $cMult -thresh -$thresh -out $specName -ov 
   endif

   if ($i == 1) then
      nmrPipe -in $specName -out $sumName -fn SET -c 0.0 -ov
   endif

   addPipe.tcl -in1 $sumName -in2 $specName -out $tmp2Name -add

   if ($istSaveFlag) then
      /bin/cp $tmp2Name $sumName
   else
      /bin/mv $tmp2Name $sumName
   endif

   nmrPipe -in $multName \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn HT -auto -nozf \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -sw \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn HT -auto -nozf \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -sw -ad \
   | nmrPipe -fn TP \
   | nmrPipe -fn HT -auto -nozf \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -sw \
   | nmrPipe -fn TP \
     -out $invName -ov

   if ($progressFlag != 0) then
      set progPct = (`MATH "100.0 - 100.0*div($thisThresh-$istMaxRes, 100.0-$istMaxRes)"`)
      set thisMsg = "nmrProgress -loc $i -pct `nmrPrintf %.3f $progPct` -n $istIter -fromID 1 -pid $thisPID -title IST Convergence (steps get smaller) -axis None"
      nmrSend -norecv -send $sendName -msg "$thisMsg"
   endif

   set thisInName = $invName 
   @ i++

  if (`IMATH "$thisThresh < $istMaxRes"`) then
      @ convFlag = 1
      break
   endif

  if (-e ist.stop) then
     if (`string same $vFlag verb`) then
        echo "Found File ist.stop ... stopping IST Calculation."
     endif

     /bin/rm -f ist.stop
     @ convFlag = 1
     break
  endif
end

if ($progressFlag != 0) then
   if (!($convFlag)) then
      set statusStr = "Iteration Count Exceeded."
   else
      set statusStr = "None"
   endif

   set thisMsg = "nmrProgress -loc $istIter -n $istIter -pct 100.0 -fromID 1 -pid $thisPID -title IST Convergence -axis None -status $statusStr"

   nmrSend -norecv -send $sendName -msg "$thisMsg"
endif

if (`string same $vFlag verb`) then
   echo ""
endif

nmrPipe -in $multName -out $resName -ov -fn `expandArg0 $rBASEARG` 
addPipe.tcl -in1 $sumName -in2 $resName -out $istName -add

nmrPipe -in $ft1Name \
| nmrPipe -fn MULT -c $yC1 -xn 1 -inv \
| nmrPipe -fn TP \
| nmrPipe -fn MULT -c $zC1 -xn 1 -inv \
| nmrPipe -fn TP \
| nmrPipe -fn ZTP \
| nmrPipe -fn MULT -c $aC1 -xn 1 -inv \
| nmrPipe -fn ZTP \
   -out $ft1Name -inPlace

/bin/rm -rf $tmpDir

if (`string csame $iftName None`) then
   @ skipCount++
else
   @ fmtCount = (`echo $iftName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

   if ($fmtCount == 0) then
      set inProc  = nmrPipe
      set outProc = nmrPipe
   else
      set inProc  = xyz2pipe
      set outProc = pipe2xyz
   endif


   if (`string same $vFlag verb`) then
     echo ""
     echo "Generating Interferogram $iftName from $istName ..."
   endif

   if ($retainFlag) then
      nmrPipe -in $istName \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr -ad \
      | nmrPipe -fn TP \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | nmrPipe -fn ZTP \
      | nmrPipe -out $tmp1Name -ov

      if (`string same $vFlag verb`) then
         echo "Restoring Original Time Domain Points from $ft1Name to $iftName ..."
      endif

      addNMR -in1 $ft1Name  -in2 $istMask  -out $ft1Name  -mask
      addNMR -in1 $tmp1Name -in2 $istMask  -out $tmp1Name -imask
      addNMR -in1 $ft1Name  -in2 $tmp1Name -out $tmp1Name -add

      nmrPipe -in $tmp1Name | $outProc -out $iftName -ov
   else
     nmrPipe -in $istName \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr -ad \
      | nmrPipe -fn TP \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | nmrPipe -fn TP \
      | nmrPipe -fn ZTP \
      | nmrPipe -fn HT -auto -nozf \
      | nmrPipe -fn PS -inv -hdr \
      | nmrPipe -fn FT -inv \
      | nmrPipe -fn ZF -inv \
      | nmrPipe -fn APOD -inv -hdr \
      | nmrPipe -fn ZTP \
      | $outProc -out $iftName -ov
   endif
endif

if ($yzaFlag == 0) then
   set outDim = x
else if ($yzaFlag == 1) then
   set outDim = y
else
   set outDim = a
endif

if ($retainFlag) then
   if (`string same $vFlag verb`) then
      echo "Transforming $iftName to Generate $outName ..."
      echo ""
   endif

   $inProc -in $iftName \
   | nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1Final -q2 $yQ2Final -q3 $yQ3Final -elb $yELBFinal -glb $yGLBFinal -goff $yGOFFFinal -c $yC1 \
   | nmrPipe -fn ZF `expandArg $yZFARG` \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
   | nmrPipe -fn `expandArg0 $yBASEARG_FINAL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn APOD -qName $zAPOD -q1 $zQ1Final -q2 $zQ2Final -q3 $zQ3Final -elb $zELBFinal -glb $zGLBFinal -goff $zGOFFFinal -c $zC1 \
   | nmrPipe -fn ZF `expandArg $zZFARG` \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $zP0 -p1 $zP1 -di \
   | nmrPipe -fn `expandArg0 $zBASEARG_FINAL` \
   | nmrPipe -fn TP -nohdr \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn APOD -qName $aAPOD -q1 $aQ1Final -q2 $aQ2Final -q3 $aQ3Final -elb $aELBFinal -glb $aGLBFinal -goff $aGOFFFinal -c $aC1 \
   | nmrPipe -fn ZF `expandArg $aZFARG` \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $aP0 -p1 $aP1 -di \
   | nmrPipe -fn `expandArg0 $aBASEARG_FINAL` \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $tmpDir/test%05d.ft4 -$outDim -ov
else
   if (`string same $vFlag verb`) then
      echo "Applying Final Baseline Corrections ..."
   endif

   nmrPipe -in $istName \
   | nmrPipe -fn `expandArg0 $yBASEARG_FINAL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $zBASEARG_FINAL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn `expandArg0 $aBASEARG_FINAL` \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $tmpDir/test%05d.ft4 -$outDim -ov
endif

if ($yzaFlag == 0) then
   xyz2pipe -in $tmpDir/test%05d.ft4 -a \
   | pipe2xyz -out $outName
else if ($yzaFlag == 1) then
   xyz2pipe -in $tmpDir/test%05d.ft4 -a \
   | pipe2xyz -out $outName
else
   xyz2pipe -in $tmpDir/test%05d.ft4 -x \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $outName
endif

if ($istCleanFlag) then
   set fList = (ist_nuslist_zf ist_nuszf.fid ist_nuszf_mask.fid ist_sum.ft4 ist_tmp.ft1 ist_tmp.ft2 ist_tmp.ft3 ist_tmp.ft4 ist_res.ft4 ist_inv.ft1 ist_mask.ft1 ist_ft1.ft1 ist_ft4.ft4 istspec istmult istres isttmp)

   foreach f ($fList)
      if (-e $f) then
         /bin/rm -rf $f
      endif
   end
endif

exit 0
