#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

set inName  None
set pdbName None
set segName None

#
# HNHA, Phi coupling

set aK  7.09
set bK -1.42  
set cK  1.55 
set pK   -60 

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert GDB J Input to DYNAMO Format."
    puts " -in  inName    GDB Input (RESID/J Information Only)."
    puts " -pdb pdbName   Corresponding PDB (for Sequence and Atom Names)."
    puts " -seg segName   Default Seg Name."
    puts " -aK  A         Karplus A Parameter."
    puts " -bK  B         Karplus B Parameter."
    puts " -cK  C         Karplus C Parameter."
    puts " -pK  Phase     Karplus Phase Parameter, Degrees."
    puts ""
    puts "Example Karplus Curve Values:"
    puts ""
    puts "  J-    Backbone  A     B    C    Phase"
    puts " Type    Angle"
    puts ""
    puts " coha     phi     3.72 -2.18 1.28 120" 
    puts " cocb     phi     1.74 -0.57 0.25 240" 
    puts " coco     phi     1.36 -0.93 0.60   0"
    puts " hncb     phi     3.06 -0.74 0.13  60"
    puts " hnco     phi     4.29 -1.01 0.00 180"
    puts " hnha     phi     7.09 -1.42 1.55 -60"
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName

getArgD $ARGV -aK  aK
getArgD $ARGV -bK  bK
getArgD $ARGV -cK  cK
getArgD $ARGV -pK  pK

#
#--------------------------------------------------------------

proc printJAss { resID assID groupID jVal } \
{
   global pdbTab segName
   global aK bK cK pK

#
# C(i-1) N(i) CA(i) C(i)

   set prevRes  [expr $resID - 1]
   set resName  [getNthResName $pdbTab $resID]
   set prevName [getNthResName $pdbTab $prevRes]
 
   set resIDI    $prevRes
   set atomNameI C
   set segNameI  $segName 
   set resNameI  $prevName

   set resIDJ    $resID
   set atomNameJ N 
   set segNameJ  $segName
   set resNameJ  $resName

   set resIDK    $resID 
   set atomNameK CA
   set segNameK  $segName
   set resNameK  $resName

   set resIDL    $resID 
   set atomNameL C
   set segNameL  $segName
   set resNameL  $resName

   puts -nonewline [format "%4d %2d" $assID $groupID]
   puts -nonewline [format " %3d %-4s %-4s" $resIDI $resNameI $atomNameI]
   puts -nonewline [format " %-4s" $segNameI]
   puts -nonewline [format " %3d %-4s %-4s" $resIDJ $resNameJ $atomNameJ]
   puts -nonewline [format " %-4s" $segNameJ]
   puts -nonewline [format " %3d %-4s %-4s" $resIDK $resNameK $atomNameK]
   puts -nonewline [format " %-4s" $segNameK]
   puts -nonewline [format " %3d %-4s %-4s" $resIDL $resNameL $atomNameL]
   puts -nonewline [format " %-4s" $segNameL]

   set fmt " % 9.5f % 9.5f % 9.5f % 9.5f % 7.2f %.1f %.1f %.1f"

   puts [format $fmt $aK $bK $cK $pK $jVal 1.0 1.0 1.0]
}

proc printJHdr {} \
{
   puts -nonewline "VARS INDEX GROUP"
   puts -nonewline " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
   puts -nonewline " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
   puts -nonewline " RESID_K RESNAME_K ATOMNAME_K SEGNAME_K"
   puts -nonewline " RESID_L RESNAME_L ATOMNAME_L SEGNAME_L"
   puts -nonewline " A B C PHASE J FC W S"
   puts ""

   puts -nonewline "FORMAT %3d %3d"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %9.5f %9.5f %9.5f %9.5f %9.3f %.2f %.2f %.2f"
   puts ""
   puts ""
}

proc parseJ {} \
{
   global jTab

   set eList     [gdbGetInfo $jTab -parms entries]
   set itemCount [llength $eList]
   set assID     1
   set groupID   1

   foreach entry $eList \
      {
       wm title . "Interpreting $assID/$itemCount"
       update

       set resID [gdbGet $entry RESID]
       set jVal  [gdbGet $entry J]

       printJAss $resID $assID $groupID $jVal

       incr assID
      }
}

showDynLogo .

wm title . "Reading $pdbName"
update

set pdbTab [gdbRead table -in $pdbName -pdb]

wm title . "Reading $inName"
update

set jTab   [gdbRead table -in $inName]

printJHdr

parseJ

exit
