#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

set inName  None
set pdbName None
set segName None

set aK  1.74
set bK -0.57
set cK  0.25
set pK   240

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert XPLOR J Input to DYNAMO Format."
    puts " -in  inName    XPLOR Input (J Information Only)."
    puts " -pdb pdbName   Corresponding PDB (for Sequence and Atom Names)."
    puts " -seg segName   Default Seg Name."
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName

#
#--------------------------------------------------------------

proc parseJAss { assID line } \
{
   parseXPLORAss $assID $line 4 assList tail

   set assI  [lindex $assList 0]
   set assJ  [lindex $assList 1]
   set assK  [lindex $assList 2]
   set assL  [lindex $assList 3]

   set d0    [lindex $tail 0]

   set assText $assI,$assJ,$assK,$assL
   set pList   [permList $assText]
   set groupID 1
 
   foreach p $pList \
      {
       printJAss $assID $groupID $p $d0
       incr groupID
      }
}

proc printJAss { assID groupID p d } \
{
   global seqInfo
   global aK bK cK pK

   set str [string toupper [split $p "~,"]]

   set resIDI    [lindex $str 0]
   set atomNameI [lindex $str 1]
   set segNameI  [lindex $str 2]
   set resNameI  $seqInfo($resIDI,$segNameI)
   set atomListI [expandXPLORAtom $resIDI $atomNameI $segNameI]

   set resIDJ    [lindex $str 3]
   set atomNameJ [lindex $str 4]
   set segNameJ  [lindex $str 5]
   set resNameJ  $seqInfo($resIDJ,$segNameJ)
   set atomListJ [expandXPLORAtom $resIDJ $atomNameJ $segNameJ]

   set resIDK    [lindex $str 6]
   set atomNameK [lindex $str 7]
   set segNameK  [lindex $str 8]
   set resNameK  $seqInfo($resIDK,$segNameK)
   set atomListK [expandXPLORAtom $resIDK $atomNameK $segNameK]

   set resIDL    [lindex $str 9]
   set atomNameL [lindex $str 10]
   set segNameL  [lindex $str 11]
   set resNameL  $seqInfo($resIDL,$segNameL)
   set atomListL [expandXPLORAtom $resIDL $atomNameL $segNameL]

   foreach atomI $atomListI \
      {
   foreach atomJ $atomListJ \
      {
   foreach atomK $atomListK \
      {
   foreach atomL $atomListL \
      {
       puts -nonewline [format "%4d %2d" $assID $groupID]
       puts -nonewline [format " %3d %-4s %-4s" $resIDI $resNameI $atomI]
       puts -nonewline [format " %-4s" $segNameI]
       puts -nonewline [format " %3d %-4s %-4s" $resIDJ $resNameJ $atomJ]
       puts -nonewline [format " %-4s" $segNameJ]
       puts -nonewline [format " %3d %-4s %-4s" $resIDK $resNameK $atomK]
       puts -nonewline [format " %-4s" $segNameK]
       puts -nonewline [format " %3d %-4s %-4s" $resIDL $resNameL $atomL]
       puts -nonewline [format " %-4s" $segNameL]

       set fmt " % 9.5f % 9.5f % 9.5f % 9.5f % 7.2f %.1f %.1f %.1f"

       puts [format $fmt $aK $bK $cK $pK $d 1.0 1.0 1.0]
      }
      }
      }
      }
}

proc printJHdr {} \
{
   puts -nonewline "VARS INDEX GROUP"
   puts -nonewline " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
   puts -nonewline " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
   puts -nonewline " RESID_K RESNAME_K ATOMNAME_K SEGNAME_K"
   puts -nonewline " RESID_L RESNAME_L ATOMNAME_L SEGNAME_L"
   puts -nonewline " A B C PHASE J FC W S"
   puts ""

   puts -nonewline "FORMAT %3d %3d"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %5d %4s %4s %4s"
   puts -nonewline " %9.5f %9.5f %9.5f %9.5f %9.3f %.2f %.2f %.2f"
   puts ""
   puts ""
}

proc parseJ {} \
{
   global inUnit
   global aK bK cK pK

   set itemCount           0
   set assInfo($itemCount) ""

   while {[gets $inUnit line] >= 0} \
      {
       set keyword [lindex $line 0]

       if {"$keyword" == "assign"} \
          {
           set line [lrange $line 1 end]
           set line [stripXPLORComment $line]

           incr itemCount

           wm title . "Reading $itemCount"
           update
          } \
       elseif {"$keyword" == "coefficients"} \
          {
           set aK [lindex $line 1]
           set bK [lindex $line 2]
           set cK [lindex $line 3]
           set pK [lindex $line 4]

           set line ""
          }

       set kInfo($itemCount) "$aK $bK $cK $pK"

       append assInfo($itemCount) " $line"
      }

   for {set assID 1} {$assID <= $itemCount} {incr assID} \
      {
       wm title . "Interpreting $assID/$itemCount"
       update

       set aK [lindex $kInfo($assID) 0]
       set bK [lindex $kInfo($assID) 1]
       set cK [lindex $kInfo($assID) 2]
       set pK [lindex $kInfo($assID) 3]

       set result ""
       set error  0

       set error [catch {parseJAss $assID $assInfo($assID)}]

       if {$error} \
          {
           puts stderr "DYNAMO Warning: Error interpreting Restraint $assID:"
           puts stderr "  Error Code: $error $result"
           puts stderr "  Line: $assInfo($assID)"
          }
      }
}


if {![file exists $inName]} \
   {
    puts stderr "Error finding input restraint file $inName"
    exit 1
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding input PDB file $pdbName"
    exit 1
   }

dynInit -fast

showDynLogo .

wm title . "Reading $pdbName"
update

set inUnit [open $inName r]

getXPLORPDBInfo $pdbName


wm title . "Reading $inName"
update

printJHdr

parseJ

exit
