#!/bin/csh

if (`flagLoc $argv -help`) then
   echo 'Create NMRPipe 2D Data from a Series of Related 1D NMRPipe Files:'
   echo ' -in  fileList  [ft/*]        List of 1D NMRPipe-Format Input Files.'
   echo ' -out outName   [series.dat]  NMRPipe-format 2D Output.'
   exit 0
endif


#
# Extract argments:

if (-d ft) then
   set inDef = (ft/*)
else
   set inDef = None
endif

set inList  = (`getListArgD $argv -in -- $inDef`)
set outName = (`getArgD     $argv -out   series.dat`)

#
# Test for missing input:

set yN = $#inList

if ($yN == 0) then
   echo No 1D input files specified, exiting.
   exit 0
endif

if ($inList[1] == "None") then
   echo No 1D input files found, exiting.
   exit 0
endif

foreach i ($inList)
   if (!(-e $i)) then
      echo Error finding 1D input file $i, exiting.
      exit 0
   endif
end

#
# Use header and data from first 1D file:
 
echo $inList[1]
cp   $inList[1] $outName

#
# Catenate data from remainder of 1D files:

foreach i ($inList[2-])
 echo $i
 nmrPipe -in $i -outHdr None >> $outName   
end

#
# Adjust header of result:

sethdr $outName -pipeFlag 0 -cubeFlag 0 -fileCount 1 \
    -yN         $yN  \
    -yT         $yN  \
    -yMODE     Real  \
    -yFT       Time  \
    -ySW        $yN  \
    -yOBS       1.0  \
    -yCAR       0.0  \
    -yLAB       TAU  \
    -ndim         2

echo Created $outName
