#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "joinPDB.tcl: join PDB files using MODEL/ENDMDL statements:"
    puts " -in    inPDBName   \[ref.pdb\]  Input PDB List."
    puts " -out   outPDBName  \[stdout\]   Output PDB Name."
    puts "Options:"
    puts " -remark                       Include REMARK statements (Default)."
    puts " -noremark                     Skip REMARK statements."
    puts " -end                          Add END statement (Default)."
    puts " -noend                        No END statement."
    puts " -verb                         Verbose Mode ON."
    puts "Join PDB files as multiple chains:"
    puts " -multiChain"
    exit 1
   }

set inList     ref.pdb
set outName    stdout
set outUnit    stdout
set remFlag    1
set endFlag    1
set chainFlag  0
set verbose    0
 
getListArgD $ARGV -in  inList
getArgD     $ARGV -out outName

if {[flagLoc $ARGV -multiChain]}    {set chainFlag 1}
if {[flagLoc $ARGV -nomultichain]}  {set chainFlag 0}

if {[flagLoc $ARGV -remark]}   {set remFlag 1}
if {[flagLoc $ARGV -noremark]} {set remFlag 0}

if {[flagLoc $ARGV -end]}    {set endFlag 1}
if {[flagLoc $ARGV -noend]}  {set endFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#---------------------------------------------------------------------------

#
#       iiiii          crrrr    xxxxxxxxyyyyyyyyzzzzzzzz                  ssssee
# 01234567890123456789012345678901234567890123456789012345678901234567890123456789
# ATOM      4  HT3 MET     1      28.153  24.620   3.231  1.00  0.00      1UBQ
# ATOM      5  CA  MET     1      26.281  25.430   2.826  1.00 10.38      1UBQ


proc cleanLine { line } \
{
    global chainFlag chainName atomID

    if {!$chainFlag} {return $line}

    if {$atomID > 99999} {set atomID 1}

    set str $line
    set n   [string length $str]

    while {$n < 79} \
       {
        append str " "
        incr n
       }

   set ii [string range $str 76 77]

   if {[string is integer $ii]} \
      {
       set str [string range $str 0 75]
      }

   set str [string replace $str 72 75 "    "]
   set str [string replace $str 21 21 $chainName]
   set str [string replace $str  6 10 [format "%5d" $atomID]]

   return $str
}

set cList {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z}

if {![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "JoinPDB Error Opening Output PDB $outName"
        exit 1
       }
   }

dynInit -fast

set atomID 0
set n  0
set nL 0
set id 0

foreach inName $inList \
   {
    if {$verbose} {puts stderr $inName}

    if {![file exists $inName]} \
       {
        puts stderr "JoinPDB Error Finding Input PDB $inName"
        continue
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "JoinPDB Error Opening Input PDB $inName"
        continue
       }

    set chainName [lindex $cList $id]
    if {![llength $chainName]} {set chainName 0}

    incr id

    if {!$nL} \
       {
        puts $outUnit "REMARK DYNAMO $dynamoVersion Rev $dynamoRevision"
        incr nL
       }

# 1 -  6       Record name    "MODEL "
# 11 - 14      Integer        Model serial number
#                         "12345678901234"

    if {!$chainFlag} \
       {
        puts $outUnit [format "MODEL     %4d" $id]
        incr nL
       }

    while {[gets $inUnit line] >= 0} \
       {
        scan $line %s key

        switch -- $key \
           {
            REMARK  { 
                     if {$remFlag} \
                        {
                         if {!$nL} {puts $outUnit "REMARK DYNAMO $dynamoVersion Rev $dynamoRevision"}
                         puts $outUnit $line
                         incr nL
                        }
                    }

            ATOM -
            HETATM  { 
                     incr atomID

                     if {!$nL} {puts $outUnit "REMARK DYNAMO $dynamoVersion Rev $dynamoRevision"}
                     puts $outUnit [cleanLine $line]
                     incr nL
                     incr n
                    }

            END -
            MODEL -
            ENDMDL  { continue }

            default { 
                     if {!$nL} {puts $outUnit "REMARK DYNAMO $dynamoVersion Rev $dynamoRevision"}
                     puts $outUnit $line 
                     incr nL
                    }
           }
       }

    if {!$chainFlag} {puts $outUnit "ENDMDL"}
   }

if {$n && $endFlag} {puts $outUnit "END"}

if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit
