
#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Join Two or More Tables:"
    puts " -in    inList  \[test.tab\]  List of Input Tables to Join."
    puts " -out   outName \[join.tab\]  Output Table."
    puts " -renum                     Renumber INDEX of Result if it Exists (Default)."
    puts " -norenum                   Do Not Renumber INDEX of Result."
    puts " -verb                      Verbose Mode ON (Default)."
    puts " -noverb                    Verbose Mode OFF."
    exit 0
   }

set inList    test.tab
set outName   join.tab

set renumFlag 1
set verbose   1

getListArgD $ARGV -in  inList
getArgD     $ARGV -out outName

if {[flagLoc $ARGV -renum]}   {set renumFlag 1}
if {[flagLoc $ARGV -norenum]} {set renumFlag 0}

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

set i 0

set nn [string length $outName] 

foreach inName $inList \
   {
    set n [string length $inName]
    if {$n > $nn} {set nn $n}
   }

foreach inName $inList \
   {
    incr i

    if {![file exists $inName]} \
       {
        puts "JoinTab Error Finding Input Table $inName"
        exit 1
       }

    set tabInfo($i) [gdbRead table -in $inName]
    set thisCount   [gdbGetInfo $tabInfo($i) -parms entryCount]

    if {$verbose} \
       {
        if {$i == 1} {puts ""}
        puts [format "Input:   %${nn}s  Entries: %5d" $inName $thisCount]
       }

    if {$i > 1} {gdbAppend -dest $tabInfo(1) -src $tabInfo($i)}
   }

set totalCount [gdbGetInfo $tabInfo(1) -parms entryCount]
set varList    [gdbGetInfo $tabInfo(1) -parms varNames]

set i 0

foreach inName $inList \
   {
    incr i

    set thisCount [gdbGetInfo $tabInfo($i) -parms entryCount]

    if {$verbose} \
       {
        if {$i == 1} \
           {
            gdbAddNote $tabInfo(1) ""
            gdbAddNote $tabInfo(1) "#"
            gdbAddNote $tabInfo(1) "# $totalCount Entries Created by Joining Tables:"
           }

        gdbAddNote $tabInfo(1) [format "#  Input:  %${nn}s  Entries: %5d" $inName $thisCount]
       }
   }

if {[lsearch -exact $varList INDEX] < 0} {set renumFlag 0}

if {$renumFlag} \
   {
    if {$verbose} \
       {
        puts ""
        puts "Renumbering INDEX ..."
        puts ""
       }

    set eList [gdbGetInfo $tabInfo(1) -parms entries]
    set i     0

    foreach e $eList \
       {
        incr i
        gdbSet $e INDEX $i
       }
   }

if {$verbose} {puts [format "Output: %${nn}s   Entries: %5d" $outName $totalCount]}

gdbWrite $tabInfo(1) $outName

exit 0
