#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {$SYS_BYTESWAP} \
   {
    set swapFlag 0
   } \
else \
   {
    set swapFlag 1
   }

set inName   None
set outName  stdout
set verbose  1
set thisList [glob -nocomplain *.1d *.2d]

if {[llength $thisList]} {set inName [lindex $thisList 0]}

set inStr  \[$inName\]
set outStr \[$outName\]

set nsIn  [string length $inStr]
set nsOut [string length $outStr]

if {$nsIn > $nsOut} \
   {
    set ns $nsIn
   } \
else \
   {
    set ns $nsOut
   }

puts $ns
 
set inStr    [format "%-${ns}s"   \[$inName\]]
set outStr   [format "%-${ns}s"   \[$outName\]]
set blankStr [format "%-${ns}s  " ""] 

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Extract Header Information from Magritek Spectral Data:"
    puts stderr " -in     inName  $inStr    Name of Magritek-format Input File, or Keyword stdin."
    puts stderr " -out    outName $outStr    Name of NMRPipe-format Output File, or Keyword stdout."
    puts stderr " -verb           $blankStr  Verbose Mode ON (Default)."
    puts stderr " -noverb         $blankStr  Verbose Mode OFF."
    puts stderr "Byte Swap Options:"

    if {$SYS_BYTESWAP} \
       {
        puts stderr " -swap     Do Four-Byte Swap."
        puts stderr " -noswap   Suppress Four-Byte Swap."
        puts stderr " -aswap    Suppress Four-Byte Swap (Default. Meaning is System-dependant)."
        puts stderr " -noaswap  Do Four-Byte Swap (Meaning is System-dependant)."
       } \
    else \
       {
        puts stderr " -swap     Do Four-Byte Swap."
        puts stderr " -noswap   Suppress Four-Byte Swap."
        puts stderr " -noaswap  Suppress Four-Byte Swap (Meaning is System-dependant)."
        puts stderr " -aswap    Do Four-Byte Swap (Default. Meaning is System-dependant)."
       }


    puts stderr "Header Options:"
    buildHdr -help
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -swap]}   {set swapFlag 1}
if {[flagLoc $ARGV -noswap]} {set swapFlag 0}

if {$SYS_BYTESWAP} \
   {
    if {[flagLoc $ARGV -aswap]}   {set swapFlag 0}
    if {[flagLoc $ARGV -noaswap]} {set swapFlag 1}
   } \
else \
   {
    if {[flagLoc $ARGV -aswap]}   {set swapFlag 1}
    if {[flagLoc $ARGV -noaswap]} {set swapFlag 0}
   }

set hdrArgs ""
set n [llength $argv]

for {set i 0} {$i < $n} {incr i} \
   {
    set thisArg [lindex $argv $i]

    if {[lsearch "-in -out" $thisArg] >= 0} \
       {
        incr i
        continue
       } \
    elseif {[lsearch "-verb -noverb -swap -noswap -aswap -noaswap -tk -notk --" $thisArg] >= 0} \
       {
        continue
       } \
    else \
       {
        lappend hdrArgs $thisArg
       } 
   }

if {![strcasecmp $inName None]} {exit 0}

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Magritek Input $inName"
    exit 1
   }

if {![strcasecmp $inName stdin]} \
   {
    set inUnit 0
   } \
else \
   {
    set error [catch {set inUnit [sysOpen $inName r]}]

    if {$error} \
       {
        puts stderr "Error Opening Magritek Input $inName"
        exit 1
       }
   }

set parmNames    {owner version format dataType xSize ySize zSize aSize}

set dataCodeInfo {500 501 502 503 504}
set dataTypeInfo {r   r,i d   x,y x,r,i}

set hdrSize      [llength $parmNames]
set hdrInfo      [sysMalloc $hdrSize int]

set error [catch {sysRead $inUnit $hdrInfo $hdrSize}]

if {$error} \
   {
    puts stderr "Error Reading Magritek Header from $inName"
    exit 1
   }

puts stderr [format "%-9s %s" File: $inName]
puts stderr [format "%-9s %s" Size: [file size $inName]]

for {set i 0} {$i < $hdrSize} {incr i} \
   {
    set p [lindex $parmNames $i]
    set magriInfo($p) [getPtrVal $hdrInfo $i]

    puts [format "%-9s %s" ${p}: $magriInfo($p)]
   }

if {$magriInfo(dataType) == 500} \
   {
    set inSize     [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)]
    set outSize    $inSize
    set inType     float
    set quadType   Real
    set qSize      1
    set formatName r
    set wordSize   4
   } \
elseif {$magriInfo(dataType) == 501} \
   {
    set inSize     [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)*2]
    set outSize    $inSize
    set inType     float
    set quadType   Complex
    set qSize      2
    set formatName r,i
    set wordSize   4
   } \
elseif {$magriInfo(dataType) == 502} \
   {
    set inSize     [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)]
    set outSize    $inSize
    set quadType   Real
    set inType     double
    set qSize      1
    set formatName d
    set wordSize   8
   } \
elseif {$magriInfo(dataType) == 503} \
   {
    set inSize     [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)*2]
    set outSize    $inSize
    set inType     float
    set quadType   Real
    set qSize      1
    set formatName x,y 
    set wordSize   4
   } \
elseif {$magriInfo(dataType) == 504} \
   {
    set inSize     [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)*3]
    set outSize    [expr $magriInfo(xSize)*$magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)*2]
    set inType     float
    set formatName x,r,i
    set quadType   Complex
    set qSize      2
    set wordSize   4
   } \
else \
   {
    puts stderr "Error: Unknown Type $magriInfo(dataType)"
    exit 1
   }

if {$magriInfo(aSize) > 1} \
   {
    set dimCount 4
   } \
elseif {$magriInfo(zSize) > 1} \
   {
    set dimCount 3
   } \
elseif {$magriInfo(ySize) > 1} \
   {
    set dimCount 2
   } \
else \
   {
    set dimCount 1
   }

for {set i 0} {$i < $dimCount} {incr i} \
   {
    set dimName [lindex "x y z a" $i]

    if {![strcasecmp $dimName x]} \
       {
        if {[lsearch -exact $hdrArgs -${dimName}N] < 0}  {lappend hdrArgs -${dimName}N    [expr $qSize*$magriInfo(${dimName}Size)]}
        if {[lsearch -exact $hdrArgs -${dimName}T] < 0}  {lappend hdrArgs -${dimName}T    $magriInfo(${dimName}Size)}
       } \
    else \
       {
        if {[lsearch -exact $hdrArgs -${dimName}N] < 0}  {lappend hdrArgs -${dimName}N    $magriInfo(${dimName}Size)}
        if {[lsearch -exact $hdrArgs -${dimName}T] < 0}  {lappend hdrArgs -${dimName}T    [expr $magriInfo(${dimName}Size)/2]}
       }

    if {[lsearch -exact $hdrArgs -${dimName}LAB]  < 0}   {lappend hdrArgs -${dimName}LAB  [string toupper $dimName]}
    if {[lsearch -exact $hdrArgs -${dimName}SW]   < 0}   {lappend hdrArgs -${dimName}SW   [format %.f1 $magriInfo(${dimName}Size)]}
    if {[lsearch -exact $hdrArgs -${dimName}MODE] < 0}   {lappend hdrArgs -${dimName}MODE Complex}
    if {[lsearch -exact $hdrArgs -${dimName}CAR]  < 0}   {lappend hdrArgs -${dimName}CAR  0.0}
    if {[lsearch -exact $hdrArgs -${dimName}OBS]  < 0}   {lappend hdrArgs -${dimName}OBS  60.0}
   }

set fmt "Format:  %s %s %s-byte %s words Dimensions: %s Quad: %s"

puts stderr ""
puts stderr "Header: $hdrArgs"
puts stderr [format $fmt $formatName $inSize $wordSize $inType $dimCount $quadType]

if {![strcasecmp $outName None]} \
   {
    catch "sysClose $inUnit"
    exit 0
   }

set error [catch {set inBuff [sysMalloc $inSize $inType]}]

if {$error} \
   {
    puts stderr "Error Allocating Input Memory."
    exit 1
   }

set error [catch {set outBuff [sysMalloc $outSize float]}]

if {$error} \
   {
    puts stderr "Error Allocating Output Memory."
    exit 1
   }

set error [catch {set fdata [sysMalloc $FDATASIZE float]}]

if {$error} \
   {
    puts stderr "Error Allocating Header Memory."
    exit 1
   }

set error [catch {sysRead $inUnit $inBuff $inSize}]

if {$error} \
   {
    puts stderr "Error Reading Magritek Data from $inName"
    exit 1
   }

if {$swapFlag} {vByteSwap $inBuff $inSize $inType}

eval "buildHdr $fdata $hdrArgs -ndim $dimCount"

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 1
   } \
else \
   {
    set error [catch {set outUnit [sysOpen $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output File $outName"
        exit 1
       }
   }

set error [catch {sysWrite $outUnit $fdata $FDATASIZE}]

if {$error} \
   {
    puts stderr "Error Writing Header to $outName"
    exit 1
   }

if {$magriInfo(dataType) == 500} \
   {
    vvCopy $outBuff $inBuff $inSize
   } \
elseif {$magriInfo(dataType) == 501 || $magriInfo(dataType) == 503} \
   {
    set vCount [expr $magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)]

    set srcLoc 0
    set destR  0
    set destI  $magriInfo(xSize)
    set jump   [expr 2*$magriInfo(xSize)]

    for {set iv 0} {$iv < $vCount} {incr iv} \
       {
        for {set ix 0} {$ix < $magriInfo(xSize)} {incr ix} \
           {
            setPtrVal $outBuff [expr $destR + $ix] [getPtrVal $inBuff $srcLoc]
            incr srcLoc

            setPtrVal $outBuff [expr $destI + $ix] [getPtrVal $inBuff $srcLoc]
            incr srcLoc
           }

        incr destR $jump
        incr destI $jump
       }
   } \
elseif {$magriInfo(dataType) == 502} \
   {
    for {set i 0} {$i < $inSize} {incr i} {setPtrVal $outBuff $i [getPtrVal $inBuff $i]}
   } \
elseif {$magriInfo(dataType) == 504} \
   {
    set vCount [expr $magriInfo(ySize)*$magriInfo(zSize)*$magriInfo(aSize)]

    set srcLoc 0
    set destR  $magriInfo(xSize) 
    set destI  [expr 2*$magriInfo(xSize)]
    set jump   [expr 3*$magriInfo(xSize)]

    for {set iv 0} {$iv < $vCount} {incr iv} \
       {
        for {set ix 0} {$ix < $magriInfo(xSize)} {incr ix} \
           {
            incr srcLoc

            puts "setPtrVal $outBuff [expr $destR + $ix] [getPtrVal $inBuff $srcLoc]"
            setPtrVal $outBuff [expr $destR + $ix] [getPtrVal $inBuff $srcLoc]
            incr srcLoc

            setPtrVal $outBuff [expr $destI + $ix] [getPtrVal $inBuff $srcLoc]
            incr srcLoc
           }

        incr destR $jump
        incr destI $jump
       }
   } 

set error [catch {sysWrite $outUnit $outBuff $outSize}]

if {$error} \
   {
    puts stderr "Error Writing Data to $outName"
    exit 1
   }

if {[strcasecmp $inName stdin]} \
   {
    catch "sysClose $inUnit"
   }
 
if {[strcasecmp $outName stdout] && [strcasecmp $outName None]} \
   {
    catch "sysClose $outUnit"
   }

exit 0
