
#
# Utilities for analysis of MFR results.

proc mfr2DaDr { mfrTab alpha dcNMin } \
{
   set tList "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"
   set eList [gdbGetInfo $mfrTab -parms entries]
   set vList [gdbGetInfo $mfrTab -parms varNames]

   gdbAddNote $mfrTab "#"
   gdbAddNote $mfrTab "# Tensor Estimates, DaDr Method:"

   set dadrInfo ""

   foreach t $tList \
      {
       if {[lsearch -exact $vList ${t}_DA] < 0} break;

       set daList  ""
       set drList  ""
       set rhList  ""
       set wList   ""

       set nP 0.0
       set nN 0.0

       set n 0

       foreach entry $eList \
          {
           set da    [gdbGet $entry ${t}_DA]
           set dr    [gdbGet $entry ${t}_DR]
           set avg   [gdbGet $entry GROUP_AVG]
           set thisN [gdbGet $entry DC_NMIN]

           if {$da == 0.0} continue
           if {$thisN < $dcNMin} continue

           set w [expr exp(-$alpha*$avg)]

           if {$da > 0.0} \
              {
               set nP [expr $nP + $w]
              } \
           else \
              {
               set nN [expr $nN + $w]
              }

           set da  [expr abs($da)]
           set dr  [expr abs($dr)]
           set rh  [expr $dr/$da]

           lappend daList $da
           lappend drList $dr
           lappend rhList $rh
           lappend wList  $w

           incr n
          }

       set daAvg [getWAvg $daList $wList]
       set drAvg [getWAvg $drList $wList]
       set rhAvg [getWAvg $rhList $wList]

       set ddaList ""
       set ddrList ""
       set drhList ""

       for {set i 0} {$i < $n} {incr i} \
          {
           set da  [lindex $daList $i]
           set dr  [lindex $drList $i]
           set rh  [lindex $rhList $i]
           set w   [lindex $wList  $i]

           set da  [expr $da - $daAvg]
           set dr  [expr $dr - $drAvg]
           set rh  [expr $rh - $rhAvg]

           lappend ddaList [expr $w*$da*$da]
           lappend ddrList [expr $w*$dr*$dr]
           lappend drhList [expr $w*$rh*$rh]
          }

       set daRMS [getWAvg $ddaList $wList]
       set drRMS [getWAvg $ddrList $wList]
       set rhRMS [getWAvg $drhList $wList]

       set daRMS [expr sqrt($daRMS)]
       set drRMS [expr sqrt($drRMS)]
       set rhRMS [expr sqrt($rhRMS)]

       if {$daAvg == 0.0 || $drAvg == 0.0} \
          {
           set daPct 0.0
           set drPct 0.0
          } \
       else \
          {
           set daPct [expr 100.0*$daRMS/$daAvg]
           set drPct [expr 100.0*$drRMS/$drAvg]
          }

       if {$nN > $nP} \
          {
           set daAvg [expr -1.0*$daAvg]
           set drAvg [expr -1.0*$drAvg]
          }

       set fmt \
        "%s %s Da: % .4e +/- %4.1f%% Dr: % .4e +/- %4.1f%% Rh: %.3f +/- %.3f"

       puts [format $fmt Tensor $t $daAvg $daPct $drAvg $drPct $rhAvg $rhRMS]
 
       lappend dadrInfo "dadr $t $daAvg $drAvg $daRMS $drRMS"

       set fmt "DATA TENSOR_DA %s % .4e +/- %4.1f%%"
       set str [format $fmt $t $daAvg $daPct]
       gdbAddNote $mfrTab $str

       set fmt "DATA TENSOR_DR %s % .4e +/- %4.1f%%"
       set str [format $fmt $t $drAvg $drPct]
       gdbAddNote $mfrTab $str
      }

   return $dadrInfo
}

proc mfr2Dot { mfrTab alpha dcNMin } \
{
   set tList "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"
   set eList [gdbGetInfo $mfrTab -parms entries]
   set vList [gdbGetInfo $mfrTab -parms varNames]

   set dotInfo ""

   gdbAddNote $mfrTab "#" 
   puts ""

   foreach t [lrange $tList 1 end] \
      {
       if {[lsearch -exact $vList ${t}_DA] < 0} break;

       set dotList ""
       set wList   ""

       set nP 0.0
       set nN 0.0

       set n 0

       foreach entry $eList \
          {
           set da    [gdbGet $entry ${t}_DA]
           set dot   [gdbGet $entry ${t}_DOT]
           set avg   [gdbGet $entry GROUP_AVG]
           set thisN [gdbGet $entry DC_NMIN]

           if {$da == 0.0} continue
           if {$thisN < $dcNMin} continue

           set w [expr exp(-$alpha*$avg)]

           if {$dot > 0.0} \
              {
               set nP [expr $nP + $w]
              } \
           else \
              {
               set nN [expr $nN + $w]
              }

           lappend dotList [expr abs($dot)] 
           lappend wList   $w

           incr n
          }

       set aAvg [getWAvg $dotList $wList]
       set ddotList ""

       for {set i 0} {$i < $n} {incr i} \
          {
           set dot [lindex $dotList $i]
           set w   [lindex $wList $i]
           set d   [expr $dot - $aAvg]

           lappend ddotList [expr $w*$d*$d]

           incr i
          }

       set aRMS [getWAvg $ddotList $wList]
       set aRMS [expr sqrt($aRMS)]

       if {$nN > $nP} {set aAvg [expr -1.0*$aAvg]}

       set aHi  [expr $aAvg + $aRMS]
       set aLo  [expr $aAvg - $aRMS]

       while {$aAvg < -1.0} {set aAvg [expr $aAvg + 2.0]}
       while {$aAvg >  1.0} {set aAvg [expr $aAvg - 2.0]}

       while {$aHi  < -1.0} {set aHi  [expr $aHi  + 2.0]}
       while {$aHi  >  1.0} {set aHi  [expr $aHi  - 2.0]}

       while {$aLo  < -1.0} {set aLo  [expr $aLo  + 2.0]}
       while {$aLo  >  1.0} {set aLo  [expr $aLo  - 2.0]}

       set e1 [catch "set aAvg [expr 180.0*acos($aAvg)/3.14159265]"]
       set e2 [catch "set aHi  [expr 180.0*acos($aHi)/3.14159265]"]
       set e3 [catch "set aLo  [expr 180.0*acos($aLo)/3.14159265]"]

       if {$e1 || $e2 || $e3} \
          {
           set aAvg 0.0
           set aHi  0.0
           set aLo  0.0
          }

       set aRMS [expr $aHi - $aLo]

       while {$aRMS < -180.0} {set aRMS [expr $aRMS + 360.0]}
       while {$aRMS >  180.0} {set aRMS [expr $aRMS - 360.0]}

       set aRMS [expr 0.5*abs($aRMS)]

       set fmt "Tensor Angle A%s: % 7.2f +/- %5.2f"
       puts [format $fmt $t $aAvg $aRMS]

       set fmt "DATA TENSOR_ANGLE A %s % 7.2f +/- %5.2f"
       set str [format $fmt $t $aAvg $aRMS]
       gdbAddNote $mfrTab $str

       lappend dotInfo "dot $t $aAvg $aRMS"
      }

   return $dotInfo
}

proc mfr2DaDrXYZ { mfrTab alpha dcNMin } \
{
   set aList "A B C D E F G H I J K L M N O P Q R S T U V W X Y Z"
   set tList ""

   set eList  [gdbGetInfo $mfrTab -parms entries]
   set vList  [gdbGetInfo $mfrTab -parms varNames]

   set dadrInfo ""

   gdbAddNote $mfrTab "#"
   gdbAddNote $mfrTab "# Tensor Estimates, XYZ Method:"

   foreach t $aList \
      {
       if {[lsearch -exact $vList ${t}_DA] < 0} break;
       lappend tList $t
      }

   set tCount [llength $tList]
 
   foreach t $tList \
      {
       set daList  ""
       set drList  ""
       set rhList  ""

       set ssList  ""
       set xxList  ""
       set yyList  ""
       set zzList  ""
       set wList   ""

       set n 0

       foreach entry $eList \
          {
           set da    [gdbGet $entry ${t}_DA]
           set dr    [gdbGet $entry ${t}_DR]
           set avg   [gdbGet $entry GROUP_AVG]
           set thisV [gdbGet $entry DC_VALID]
           set thisN [gdbGet $entry DC_NMIN]
           set dcQ   [gdbGet $entry DC_Q]
           set dcQ   1.0

           set xx    [gdbGet $entry ${t}_XX]
           set yy    [gdbGet $entry ${t}_YY]
           set zz    [gdbGet $entry ${t}_ZZ]

           if {$da == 0.0} continue
           if {$thisN < $dcNMin} continue

           set w  [expr exp(-$alpha*$avg)]
           set rh [expr $dr/$da]
           set cz [expr 2.0*$da]
           set cy [expr -1.0*$da*(1.0 + 1.5*$rh)]
           set cx [expr -1.0*$da*(1.0 - 1.5*$rh)]
           set nn [expr $thisV - 5*$tCount]
 
           set cxyzList [lsort -real "$cx $cy $cz"]

           set cx [lindex $cxyzList 2]
           set cy [lindex $cxyzList 1]
           set cz [lindex $cxyzList 0]

           if {$nn > 0} {lappend ssList [expr $dcQ/(21585.20*sqrt($nn))]}

           lappend daList [expr abs($da)]
           lappend drList [expr abs($dr)]
           lappend rhList [expr abs($rh)]

           lappend xxList $cx
           lappend yyList $cy
           lappend zzList $cz 
           lappend wList  $w

           incr n
          }

       if {!$n} continue

       set xxAvg [getWAvg $xxList $wList]
       set yyAvg [getWAvg $yyList $wList]
       set zzAvg [getWAvg $zzList $wList]

       set dxxList ""
       set dyyList ""
       set dzzList ""
       set i 0

       for {set i 0} {$i < $n} {incr i} \
          {
           set dx [expr [lindex $xxList $i] - $xxAvg]
           set dy [expr [lindex $yyList $i] - $yyAvg]
           set dz [expr [lindex $zzList $i] - $zzAvg]
           set w  [lindex $wList $i]

           lappend dxxList [expr $dx*$dx]
           lappend dyyList [expr $dy*$dy]
           lappend dzzList [expr $dz*$dz]
          }

       set ssRMS [getWAvg $ssList $wList]

       set xxRMS [expr sqrt($ssRMS)/sqrt($n)]
       set yyRMS [expr sqrt($ssRMS)/sqrt($n)]
       set zzRMS [expr sqrt($ssRMS)/sqrt($n)]

       if {abs($zzAvg) > abs($xxAvg)} \
          {
           set daAvg [expr 0.5*$zzAvg]
           set daRMS [expr 0.5*$zzRMS]
           set drAvg [expr ($yyAvg - $xxAvg)/3.0]
           set drRMS [expr sqrt(pow($yyRMS,2) + pow($xxRMS,2))/3.0]
          } \
       else \
          {
           set daAvg [expr 0.5*$xxAvg]
           set daRMS [expr 0.5*$xxRMS]
           set drAvg [expr ($yyAvg - $zzAvg)/3.0]
           set drRMS [expr sqrt(pow($yyRMS,2) + pow($zzRMS,2))/3.0]
          }

       if {$daAvg == 0.0 || $drAvg == 0.0} \
          {
           set rhAvg 0.0
           set rhRMS 0.0
           set daPct 0.0
           set drPct 0.0
          } \
       else \
          {
           set rhAvg [expr $drAvg/$daAvg]
           set rhRMS [expr sqrt(pow($drRMS/$drAvg,2) + pow($daRMS/$daAvg,2))]
           set daPct [expr 100.0*abs($daRMS/$daAvg)]
           set drPct [expr 100.0*abs($drRMS/$drAvg)]
          }

       set fmt \
        "%s %s Da: % .4e +/- %4.1f%% Dr: % .4e +/- %4.1f%% Rh: %.3f +/- %.3f"

       puts [format $fmt Tensor $t $daAvg $daPct $drAvg $drPct $rhAvg $rhRMS]

       set fmt "DATA TENSOR_DA %s % .4e +/- %4.1f%%"
       set str [format $fmt $t $daAvg $daPct]
       gdbAddNote $mfrTab $str

       set fmt "DATA TENSOR_DR %s % .4e +/- %4.1f%%"
       set str [format $fmt $t $drAvg $drPct]
       gdbAddNote $mfrTab $str
 
       lappend dadrInfo "dadr $t $daAvg $drAvg $daRMS $drRMS"
      }

   return $dadrInfo
}

proc uSortMFR { p1 p2 } \
{
   global pdbHistCountInfo

   if {$pdbHistCountInfo($p1) < $pdbHistCountInfo($p2)} {return  1}
   if {$pdbHistCountInfo($p1) > $pdbHistCountInfo($p2)} {return -1}

   return 0
}
 
proc mfrPDBHist { mfrTab nn } \
{
   global pdbHistCountInfo

   set eList [gdbGetInfo $mfrTab -parms entries]

   set report  ""
   set pdbList ""
   set nList   ""

   foreach e $eList \
      {
       set pdbName [file root [file tail [gdbGet $e PDB_NAME]]]

       if {[lsearch -exact $pdbList $pdbName] < 0} \
          {
           lappend pdbList $pdbName
           set pdbHistCountInfo($pdbName) 0
          }
      
       incr pdbHistCountInfo($pdbName) 
      }

   foreach pdbName $pdbList \
      {
       set n $pdbHistCountInfo($pdbName)

       if {[lsearch -exact $nList $n] < 0} \
          {
           lappend nList $n
           set pdbUseInfo($n) ""
          }

       lappend pdbUseInfo($n) $pdbName
      }

   set n [llength $pdbList]

   gdbAddNote $mfrTab "#"
   gdbAddNote $mfrTab "# Fragment Source Analysis, $n PDB Files Used:"
   gdbAddNote $mfrTab "#"
 
   foreach pdbName [lrange [lsort -command uSortMFR $pdbList] 0 [expr $nn-1]] \
      {
       set fmt " %4d Fragments from PDB %s"
       set str [format $fmt $pdbHistCountInfo($pdbName) $pdbName]
       gdbAddNote $mfrTab "# $str"
       lappend report $str
      }

   gdbAddNote $mfrTab "#"

   foreach n [lsort -integer $nList] \
      {
       set fmt " %4d PDBs Contributed: %4d Fragments"
       set str [format $fmt [llength $pdbUseInfo($n)] $n]
       gdbAddNote $mfrTab "# $str"
       lappend report $str
      }

   return $report
}
