#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Initialize for mini plots.

proc initMiniPlot args \
{
    global lastPlotFlag plotCount cellCount paperType plotOrient plotBG
    global c1Width c1Height c2Width c2Height mpPrefix mpListFlag mpOpenProc mpCloseProc
    global px1 pxn py1 pyn bx1 bxn by1 byn bdx bdy bwx bwy 

    set plotOrient   landscape
    set plotBG       lightGray
    set paperType    A
    set lastPlotFlag 0
    set mpListFlag   0
    set mpPrefix     roi
    set mpOpenProc   None
    set mpCloseProc  None

    set c1Width  0.10in 
    set c1Height 0.14in
    set c2Width  0.10in 
    set c2Height 0.14in

    set ArgList [concat initMiniPlot $args]

    if {[flagLoc $ArgList -portrait]}  {set plotOrient portrait}
    if {[flagLoc $ArgList -landscape]} {set plotOrient landscape}

    if {[flagLoc $ArgList -list]}      {set mpListFlag 1}
    if {[flagLoc $ArgList -nolist]}    {set mpListFlag 0}

    getArgD $ArgList -paper paperType

    switch [string toupper $paperType] \
       {
        A  {
            set pxn 10.0
            set pyn  8.0
           }

        B  {
            set pxn 16.0
            set pyn 10.5
           }

        C  {
            set pxn 21.0
            set pyn 16.5
           }

        A4 {
            set pxn 10.60
            set pyn  7.75
           }
  
        A3 {
            set pxn 15.5
            set pyn 11.1
           }

        A2 {
            set pxn 22.3
            set pyn 16.0
           }

        default {
                 set pxn 10.0
                 set pyn  8.0
                }
       }

    if {![strcasecmp $plotOrient portrait]} \
       {
        set tmp $pxn
        set pxn $pyn
        set pyn $tmp
       }

    set px1  0.5
    set py1  0.5

    set bwx  1.3
    set bwy  1.3
    set bdx  0.25
    set bdy  0.25

    getArgD $ArgList -px1 px1
    getArgD $ArgList -py1 py1
    getArgD $ArgList -pxn pxn
    getArgD $ArgList -pyn pyn

    getArgD $ArgList -bwx bwx 
    getArgD $ArgList -bwy bwy 
    getArgD $ArgList -bdx bdx 
    getArgD $ArgList -bdy bdy 

    getArgD $ArgList -c1Height c1Height
    getArgD $ArgList -c1Width  c1Width

    getArgD $ArgList -c2Height c2Height
    getArgD $ArgList -c2Width  c2Width

    getArgD $ArgList -bg     plotBG
    getArgD $ArgList -prefix mpPrefix

    getArgD $ArgList -openProc  mpOpenProc
    getArgD $ArgList -closeProc mpCloseProc

    set plotCount 0
    set cellCount 0
}

#
# Set up for the next little plot box.

proc nextMiniPlot {} \
{
    global px1 pxn py1 pyn bx1 bxn by1 byn bdx bdy bwx bwy 
    global plotCount cellCount paperType plotOrient plotBG
    global mpOpenProc mpCloseProc mpListFlag mpPrefix miniPlotInfo lastPlotFlag
    global c1Width c1Height c2Width c2Height

    set lastPlotFlag 0
    set newPlot      0

    if {$plotCount == 0} \
       {
        set newPlot 1
       } \
    else \
       {
        set bx1 [expr $bxn + $bdx]
        set bxn [expr $bx1 + $bwx]

        if {$bxn > $pxn} \
           {
            set bx1  $px1
            set bxn  [expr $bx1 + $bwx]
            set by1  [expr $byn + $bdy]
            set byn  [expr $by1 + $bwy]
           }

        if {$byn > $pyn} {set newPlot 1}
       }

    if {$newPlot} \
       {
        if {$plotCount != 0} \
           {
            if {[strcasecmp $mpCloseProc None]} {catch $mpCloseProc}
            plotClose
           }

        set  cellCount 0
        incr plotCount

        set bx1  $px1
        set bxn  [expr $bx1 + $bwx]
        set by1  $py1 
        set byn  [expr $by1 + $bwy] 

        if {$mpListFlag} {puts $mpPrefix${plotCount}.ps}

        plotOpen -color -plotName $mpPrefix${plotCount}.ps \
                 -c1Width $c1Width -c1Height $c1Height     \
                 -c2Width $c2Width -c2Height $c2Height     \
                 -paper $paperType -$plotOrient

        if {[strcasecmp $plotBG None]} {drawRec -x 0% 100% -y 0% 100% -fg $plotBG} 
        if {[strcasecmp $mpOpenProc None]} {catch $mpOpenProc} 
       }

    incr cellCount

    drawSettings -reset

    defineCell -cell $cellCount   \
       -x [format %sin $bx1] [format %sin $bxn] \
       -y [format %sin $by1] [format %sin $byn]

    set miniPlotInfo($cellCount) "$bx1 $by1 $bxn $byn"

    set bx1Next [expr $bxn + $bdx]
    set bxnNext [expr $bx1Next + $bwx]
    set by1Next $by1
    set bynNext $byn

    if {$bxnNext > $pxn} \
       {
        set by1Next [expr $bynNext + $bdy]
        set bynNext [expr $by1Next + $bwy]
       }

    if {$bynNext > $pyn} {set lastPlotFlag 1}

    return $cellCount
}
