#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert a Series of Related Peak Tables to a Series Table."
    puts " -in    inName  \[tab/test%03d.tab\]  Template of Input Tables."
    puts " -out   outName \[series.tab\]        Name of Output Table."
    exit 1
   }

set inName   tab/test%03d.tab
set outName  series.tab

getArgD $ARGV -in  inName
getArgD $ARGV -out outName

#
#---------------------------------------------------------------------------

set tabCount 0
set dimCount 0

for {set i 1} {$i < 2048} {incr i} \
   {
    set thisName($i) [format $inName $i]

    if {![file exists $thisName($i)]} break 

    set inTab($i) [gdbRead table -in $thisName($i)]

    set tabCount $i
   }

if {!$tabCount} \
   {
    puts "Error Finding Input Files for $inName"
    exit 1
   }

set vList   [gdbGetInfo $inTab(1) -parms varNames]
set dimList {X Y Z A B C}

foreach dim $dimList \
   {
    if {[lsearch -exact $vList ${dim}_AXIS] < 0} break
    incr dimCount
   }

set prefix  [lindex {X Y Z A B C} $dimCount]
set refList ""
set idList  ""

for {set i 1} {$i <= $tabCount} {incr i} \
   {
    set j [expr $i - 1]

    set vName ${prefix}_A$j

    gdbCreate variable -parent $inTab(1) -name $vName \
              -varFmt  %7.4f -varType float

    set eList($i) [gdbGetInfo $inTab($i) -parms entries]
    set n($i)     [llength $eList($i)] 

    if {$i == 1} \
       {
        foreach e $eList($i) \
           {
            set thisHi [gdbGet $e HEIGHT]
            if {$thisHi == 0.0} {set thisHi 1.0}
            
            lappend refList $thisHi
            lappend idList  [gdbGet $e INDEX]
           }
       }

    if {$n(1) != $n($i)} \
       {
        puts "Error: Entry Count Mismatch."
        puts " $inName(1): $n(1).  $inName($i): $n($i)"
        exit 1
       }

    set k 0

    foreach e $eList($i) \
       {
        set thisHi [gdbGet $e HEIGHT]
        set thisID [gdbGet $e INDEX]

        set refHi    [lindex $refList  $k]
        set refID    [lindex $idList   $k]
        set refEntry [lindex $eList(1) $k]

        if {$refID != $thisID} \
           {
            puts "Error: INDEX Mismatch, Entry [expr $k + 1]."
            puts " $thisName(1): $refID  $thisName($i): $thisID."
            exit 1
           }

        gdbSet $refEntry $vName [expr $thisHi/$refHi]

        incr k
       }
   }

gdbAddNote $inTab(1) "REMARK Converted to Series Format from $inName"
gdbWrite   $inTab(1) $outName

exit 0
