#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert Named Torsion Input to DYNAMO Format."
    puts " -in    inName  \[torsion.txt\]  Torsion Names/Values (See Below)."
    puts " -out   outName \[stdout\]       Output Torsion Table."
    puts " -pdb   pdbName \[ref.pdb\]      Corresponding PDB (for Atom Names)."
    puts " -seg   segName \[SEG1\]         Segment Name."
    puts " -fc    fc      \[1.0\]          Force Constant."
    puts "Optional Adjustment of Torsion Range:" 
    puts " -da    daDeg   \[0.0\]          +/- Degrees for Angular Range."
    puts " -a     aScale  \[1.0\]          Angular Range Scale Factor."
    puts "Input Line Formats:"
    puts " torsionName resID torsionValue" 
    puts " torsionName resID torsionValue torsionRange" 
    puts " torsionName resID unusedValue  torsionLo torsionHi" 
    puts "Recognized Torsion Names for Proteins:"
    puts " phi psi alpha omega chi1 ..."
    puts "Recognized Torsion Names for DNA/RNA:"
    puts " alpha beta gamma delta epsilon zeta chi v0 v1 v2 v3 v4"
    exit 0
   }

set pdbName ref.pdb
set inName  torsion.txt 
set outName stdout
set outUnit stdout
set segName SEG1 
set range   0.0 
set aScale  1.0
set fc      1.0
set w       1.0
set s       1.0
set assID   1
set groupID 1

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName
getArgD $ARGV -da  range
getArgD $ARGV -fc  fc
getArgD $ARGV -id  assID 
getArgD $ARGV -a   aScale

set tNameInfo \
   {
    {protein phi     *  {-1 0 0 0}  {C N CA C}}
    {protein psi     *  { 0 0 0 1}  {N CA C N}}
    {protein alpha   *  {-1 0 1 2}  {CA CA CA CA}}
    {protein omega   *  {-1 -1 0 0} {CA C N CA}}

    {na      alpha   *  {-1 0 0 0}  {O3' P O5' C5'}}
    {na      beta    *  {0 0 0 0}   {P O5' C5' C4'}}
    {na      gamma   *  {0 0 0 0}   {O5' C5' C4' C3'}}
    {na      delta   *  {0 0 0 0}   {C5' C4' C3' O3'}}
    {na      epsilon *  {0 0 0 1}   {C4' C3' O3' P}}
    {na      zeta    *  {0 0 1 1}   {C3' O3' P O5'}}
    {na      chi    ADE {0 0 0 0}   {C4 C1 N9 C4}}
    {na      chi    GUA {0 0 0 0}   {C4 C1 N9 C4}}
    {na      chi    CYT {0 0 0 0}   {C4 C1 N1 C2}}
    {na      chi    THY {0 0 0 0}   {C4 C1 N1 C2}}
    {na      chi    URI {0 0 0 0}   {C4 C1 N1 C2}}
    {na      v0      *  {0 0 0 0}   {C4' O4' C1' C2'}}
    {na      v1      *  {0 0 0 0}   {O4' C1' C2' C3'}}
    {na      v2      *  {0 0 0 0}   {C1' C2' C3' C4'}}
    {na      v3      *  {0 0 0 0}   {C2' C3' C4' O4'}}
    {na      v4      *  {0 0 0 0}   {C3' C4' O4' C1'}}

    {protein chi1   ALA {0 0 0 0}   {N CA CB HB1}}
    {protein chi1   ARG {0 0 0 0}   {N CA CB CG}}
    {protein chi2   ARG {0 0 0 0}   {CA CB CG CD}}
    {protein chi3   ARG {0 0 0 0}   {CB CG CD NE}}
    {protein chi4   ARG {0 0 0 0}   {CG CD NE CZ}}
    {protein chi5   ARG {0 0 0 0}   {CD NE CZ NH1}}
    {protein chi6   ARG {0 0 0 0}   {NE CZ NH1 HH11}}
    {protein chi7   ARG {0 0 0 0}   {NE CZ NH2 HH21}}
    {protein chi1   ASN {0 0 0 0}   {N CA CB CG}}
    {protein chi2   ASN {0 0 0 0}   {CA CB CG OD1}}
    {protein chi3   ASN {0 0 0 0}   {CB CG ND2 HD21}}
    {protein chi1   ASP {0 0 0 0}   {N CA CB CG}}
    {protein chi2   ASP {0 0 0 0}   {CA CB CG OD1}}
    {protein chi1   CYS {0 0 0 0}   {N CA CB SG}}
    {protein chi2   CYS {0 0 0 0}   {CA CB SG HG}}
    {protein chi1   GLN {0 0 0 0}   {N CA CB CG}}
    {protein chi2   GLN {0 0 0 0}   {CA CB CG CD}}
    {protein chi3   GLN {0 0 0 0}   {CB CG CD OE1}}
    {protein chi4   GLN {0 0 0 0}   {CG CD NE2 HE21}}
    {protein chi1   GLU {0 0 0 0}   {N CA CB CG}}
    {protein chi2   GLU {0 0 0 0}   {CA CB CG CD}}
    {protein chi3   GLU {0 0 0 0}   {CB CG CD OE1}}
    {protein chi1   HIS {0 0 0 0}   {N CA CB CG}}
    {protein chi2   HIS {0 0 0 0}   {CA CB CG ND1}}
    {protein chi1   ILE {0 0 0 0}   {N CA CB CG1}}
    {protein chi2   ILE {0 0 0 0}   {CA CB CG1 CD}}
    {protein chi3   ILE {0 0 0 0}   {CB CG1 CD HD1}}
    {protein chi1   LEU {0 0 0 0}   {N CA CB CG}}
    {protein chi2   LEU {0 0 0 0}   {CA CB CG CD1}}
    {protein chi3   LEU {0 0 0 0}   {CB CG CD1 HD11}}
    {protein chi4   LEU {0 0 0 0}   {CB CG CD2 HD21}}
    {protein chi1   LYS {0 0 0 0}   {N CA CB CG}}
    {protein chi2   LYS {0 0 0 0}   {CA CB CG CD}}
    {protein chi3   LYS {0 0 0 0}   {CB CG CD CE}}
    {protein chi4   LYS {0 0 0 0}   {CG CD CE NZ}}
    {protein chi5   LYS {0 0 0 0}   {CD CE NZ HZ1}}
    {protein chi1   MET {0 0 0 0}   {N CA CB CG}}
    {protein chi2   MET {0 0 0 0}   {CA CB CG SD}}
    {protein chi3   MET {0 0 0 0}   {CB CG SD CE}}
    {protein chi4   MET {0 0 0 0}   {CG SD CE HE1}}
    {protein chi1   PHE {0 0 0 0}   {N CA CB CG}}
    {protein chi2   PHE {0 0 0 0}   {CA CB CG CD1}}
    {protein chi1   PRO {0 0 0 0}   {N CA CB CG}}
    {protein chi2   PRO {0 0 0 0}   {CA CB CG CD}}
    {protein chi3   PRO {0 0 0 0}   {CB CG CD N}}
    {protein chi3   PRO {0 0 0 0}   {CG CD N CA}}
    {protein chi1   SER {0 0 0 0}   {N CA CB OG}}
    {protein chi2   SER {0 0 0 0}   {CA CB OG HG}}
    {protein chi1   THR {0 0 0 0}   {N CA CB OG1}}
    {protein chi2   THR {0 0 0 0}   {CA CB OG1 HG1}}
    {protein chi3   THR {0 0 0 0}   {CA CB CG2 HG21}}
    {protein chi1   TRP {0 0 0 0}   {N CA CB CG}}
    {protein chi2   TRP {0 0 0 0}   {CA CB CG CD1}}
    {protein chi1   TYR {0 0 0 0}   {N CA CB CG}}
    {protein chi2   TYR {0 0 0 0}   {CA CB CG CD1}}
    {protein chi1   VAL {0 0 0 0}   {N CA CB CG1}}
    {protein chi2   VAL {0 0 0 0}   {CA CB CG1 HG11}}
    {protein chi3   VAL {0 0 0 0}   {CA CB CG2 HG21}}
   }

#
#------------------------------------------------------------------------

proc showTorsion { assID groupID d dLo dHi fc w s } \
{
   global resIDI resNameI atomI segNameI
   global resIDJ resNameJ atomJ segNameJ
   global resIDK resNameK atomK segNameK
   global resIDL resNameL atomL segNameL

   global outUnit

   puts -nonewline $outUnit [format "%4d %2d" $assID $groupID]
   puts -nonewline $outUnit [format " %3d %-4s %-4s" $resIDI $resNameI $atomI]
   puts -nonewline $outUnit [format " %-4s" $segNameI]
   puts -nonewline $outUnit [format " %3d %-4s %-4s" $resIDJ $resNameJ $atomJ]
   puts -nonewline $outUnit [format " %-4s" $segNameJ]
   puts -nonewline $outUnit [format " %3d %-4s %-4s" $resIDK $resNameK $atomK]
   puts -nonewline $outUnit [format " %-4s" $segNameK]
   puts -nonewline $outUnit [format " %3d %-4s %-4s" $resIDL $resNameL $atomL]
   puts -nonewline $outUnit [format " %-4s" $segNameL]

   puts $outUnit \
    [format " % 7.2f % 7.2f % 7.2f %.3f %.1f %.1f" $d $dLo $dHi $fc $w $s]
}

proc showTorsionHdr {} \
{
   global outUnit

   puts -nonewline $outUnit "VARS INDEX GROUP"
   puts -nonewline $outUnit " RESID_I RESNAME_I ATOMNAME_I SEGNAME_I"
   puts -nonewline $outUnit " RESID_J RESNAME_J ATOMNAME_J SEGNAME_J"
   puts -nonewline $outUnit " RESID_K RESNAME_K ATOMNAME_K SEGNAME_K"
   puts -nonewline $outUnit " RESID_L RESNAME_L ATOMNAME_L SEGNAME_L"
   puts -nonewline $outUnit " ANGLE ANGLE_LO ANGLE_HI FC W S"
   puts $outUnit ""

   puts -nonewline $outUnit "FORMAT %3d %3d"
   puts -nonewline $outUnit " %5d %4s %4s %4s"
   puts -nonewline $outUnit " %5d %4s %4s %4s"
   puts -nonewline $outUnit " %5d %4s %4s %4s"
   puts -nonewline $outUnit " %5d %4s %4s %4s"
   puts -nonewline $outUnit " %9.3f %9.3f %9.3f %.3f %.2f %.2f"
   puts $outUnit ""
   puts $outUnit ""
}

proc findTorsionInfo { resID tName } \
{
   global pdbInfo tNameInfo

   global resIDI resNameI atomI segNameI
   global resIDJ resNameJ atomJ segNameJ
   global resIDK resNameK atomK segNameK
   global resIDL resNameL atomL segNameL

   foreach tItem $tNameInfo \
      {
       set type     [lindex $tItem 0]
       set thisName [lindex $tItem 1]
       set resName  [lindex $tItem 2]
       set oList    [lindex $tItem 3]
       set aList    [lindex $tItem 4]

       if {[strcasecmp $thisName $tName]} continue
       if {[strcasecmp $type $pdbInfo($resID,type)]} continue
       
       if {[strcmp $resName "*"] && \
           [strcasecmp $resName $pdbInfo($resID,resName)]} continue
     
       set resIDI [expr $resID + [lindex $oList 0]] 
       set resIDJ [expr $resID + [lindex $oList 1]] 
       set resIDK [expr $resID + [lindex $oList 2]] 
       set resIDL [expr $resID + [lindex $oList 3]] 

       if {![info exists pdbInfo($resIDI,resName)]} {return ""}
       if {![info exists pdbInfo($resIDJ,resName)]} {return ""}
       if {![info exists pdbInfo($resIDK,resName)]} {return ""}
       if {![info exists pdbInfo($resIDL,resName)]} {return ""}

       set atomI [lindex $aList 0]
       set atomJ [lindex $aList 1]
       set atomK [lindex $aList 2]
       set atomL [lindex $aList 3]

       set resNameI $pdbInfo($resIDI,resName)
       set resNameJ $pdbInfo($resIDJ,resName)
       set resNameK $pdbInfo($resIDK,resName)
       set resNameL $pdbInfo($resIDL,resName)

       set segNameI $pdbInfo($resIDI,segName)
       set segNameJ $pdbInfo($resIDJ,segName)
       set segNameK $pdbInfo($resIDK,segName)
       set segNameL $pdbInfo($resIDL,segName)

       if {[strcasecmp $segNameI $segNameJ]} {return ""}
       if {[strcasecmp $segNameI $segNameK]} {return ""}
       if {[strcasecmp $segNameI $segNameL]} {return ""}

       if {[lsearch -exact $pdbInfo($resIDI,atomList) $atomI] < 0} {return ""}
       if {[lsearch -exact $pdbInfo($resIDJ,atomList) $atomJ] < 0} {return ""}
       if {[lsearch -exact $pdbInfo($resIDL,atomList) $atomK] < 0} {return ""}
       if {[lsearch -exact $pdbInfo($resIDL,atomList) $atomL] < 0} {return ""}

       return $tItem
      }

   return ""
}

proc parseTorsionNames {} \
{
   global inUnit assID groupID range fc w s aScale

   set count 0
   set str ""

   while {[gets $inUnit line] >= 0} \
      {
       set n [llength $line]
       incr count

       if {!$n} continue

       scan $line "%s" str

       set c [string index $str 0]
 
       if {![strcmp $c "#"]} continue

       switch $n \
          {
           3 {
              set tName [lindex $line 0]
              set resID [lindex $line 1]
              set d     [lindex $line 2]
              set dLo   [expr $d - $range]
              set dHi   [expr $d + $range]
             }

           4 {
              set tName [lindex $line 0]
              set resID [lindex $line 1]
              set d     [lindex $line 2]
              set dd    [lindex $line 3]
              set dLo   [expr $d - $aScale*$dd]
              set dHi   [expr $d + $aScale*$dd]
             }

           5 {
              set tName [lindex $line 0]
              set resID [lindex $line 1]
              set dLo   [lindex $line 3]
              set dHi   [lindex $line 4]
              set d     [getAvgAngle "$dLo $dHi"]
             }

           default {set tName None}
          }

       if {![strcmp $tName None]} \
          {
           puts stderr "Error In Torsion Input, Line $count:"
           puts stderr " $line"
           continue
          }

       set tInfo [findTorsionInfo $resID $tName]

       if {![llength $tInfo]} \
          {
           puts stderr "Notice: No Data for Torsion Input, Line $count:"
           puts stderr " $line"
           continue
          }

       showTorsion $assID $groupID $d $dLo $dHi $fc $w $s

       incr assID 
      }
}

proc initTorsionPDB {} \
{
   global pdbName pdbInfo segName

   set pdbTab [gdbRead table -in $pdbName -pdb]
   set eList  [gdbGetInfo $pdbTab -parms entries]

   set pdbInfo(resList) ""

   foreach entry $eList \
      {
       set resID    [gdbGet $entry RESID]
       set resName  [gdbGet $entry RESNAME]
       set atomName [gdbGet $entry ATOMNAME]
       set segID    [gdbGet $entry SEGID]

       if {![string length $segID]}       {set segID $segName}
       if {![strcasecmp $segID "(Null)"]} {set segID $segName}

       if {[lsearch -exact $pdbInfo(resList) $resID] < 0} \
          {
           lappend pdbInfo(resList) $resID
           set pdbInfo($resID,atomList) "" 
           set pdbInfo($resID,segList)  "" 
          }

       if {[lsearch -exact $pdbInfo($resID,segList)] < 0} \
          {
           lappend pdbInfo($resID,segList) $segID
          }

       lappend pdbInfo($resID,atomList) $atomName

       set pdbInfo($resID,$atomName,resName) $resName
       set pdbInfo($resID,resName) $resName
       set pdbInfo($resID,segName) $segID
      }

   foreach resID $pdbInfo(resList) \
      {
       if {[info exists pdbInfo($resID,C,resName)] &&
           [info exists pdbInfo($resID,N,resName)] &&
           [info exists pdbInfo($resID,CA,resName)]} \
          {
           set pdbInfo($resID,type) protein
          } \
       else \
          {
           set pdbInfo($resID,type) na 
          }
      }
}

if {![strcasecmp $inName None] || ![file exists $inName]} \
   {
    set inUnit stdin
   } \
else \
   {
    set inUnit [open $inName r]
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding input PDB file $pdbName"
    exit 1
   }

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output File."
        exit 1
       }
   }

initTorsionPDB
showTorsionHdr
parseTorsionNames

if {[strcasecmp $inUnit stdin]} {catch "close $inUnit"}

if {[strcasecmp $outUnit stdout] && ![strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }

exit 0
