#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[info exists env(NMRTXT)]} \
   {
    set tStr " ($env(NMRTXT)/rgb.txt)."
   } \
else \
   {
    set tStr "."
   }

if {[file exists ft/spec001.ft3]} \
   {
    set inName ft/spec%03d.ft3
   } \
else \
   {
    set inName ft/test%03d.ft3
   }

if {[flagLoc $ARGV -help]} \
{
 puts "Draw a Simple Static 3D Cube Display of NMRPipe-Format 3D Data:"
 puts " -in   inName   \[$inName\]  3D Spectrum to Draw (Default can change)."
 puts " -plev plev     \[1\]                Number of Positive Contours."
 puts " -nlev nlev     \[1\]                Number of Negative Contours."
 puts " -lw2d lw2d     \[1\]                Contour Line Width, Pixels."
 puts " -hi   hi       \[Auto\]             Contour Height (value or %.)"
 puts " -mult mult     \[7.0\]              Factor for Auto Contour Height."
 puts " -axis                             Draw Axis Labels (Default)."
 puts " -noaxis                           No Axis Labels."
 puts "Spectral Orientation for Drawing:"
 puts " -xName xName   \[H_AXIS\]      Spectral Axis Name for Plot X-Axis."
 puts " -yName yName   \[V_AXIS\]      Spectral Axis Name for Plot Y-Axis."
 puts " -zName zName   \[Z_AXIS\]      Spectral Axis Name for Plot Z-Axis."
 puts " -aName aName   \[A_AXIS\]      Spectral Axis Name, 4D Data Only."
 puts "Spectral Limits (Use -x -y -z -a with Units % Hz ppm pts):"
 puts " -ix1   ix1     \[0%\]          Starting Coord of Range to Plot."
 puts " -ixn   ixn     \[100%\]        Ending Coord of Range to Plot."
 puts "Cursor Planes:"
 puts " -ix    xPlane  \[1\]           Position of YZ Cursor Plane."  
 puts " -iy    yPlane  \[1\]           Position of XZ Cursor Plane."  
 puts " -iz    zPlane  \[1\]           Position of XY Cursor Plane."  
 puts "Drawing Sizes:"
 puts " -cxSize cxSize \[400\]         Plot X-Axis Size, Pixels."
 puts " -cySize cySize \[400\]         Plot Y-Axis Size, Pixels."
 puts " -czSize czSize \[Auto\]        Plot Final Z-Axis Offset, Pixels."
 puts "Window Position on Screen:"
 puts " -xOff   xOff   \[16\]          Window Horizontal Offset, Pixels."
 puts " -yOff   yOff   \[32\]          Window Vertical Offset, Pixels."
 puts "Spectrum Colors:"
 puts " -colors   cList  \[red ...\]   List of Colors for Spectral Drawing."
 puts " -posColor pColor \[None\]      Fixed Color for Positive Contours."
 puts " -negColor nColor \[None\]      Fixed Color for Negative Contours."
 puts "Drawing Colors:"
 puts " -fgColor  fg      \[white\]     Forground Color (for Box Outlines)."
 puts " -bgColor  bg      \[black\]     Spectrum Background Color."
 puts " -winColor wbg     \[dimGray\]   Window Background Color."
 puts "Cube Cursors:"
 puts " -cursor                      Show Cursor Planes on Startup."
 puts " -nocursor                    Hide Cursor Planes on Starup (Default)."
 puts " -fillColor fColor  \[gray\]    Fill Color for Cursor Plane."
 puts " -xyColor   xyfg    \[blue\]    Color for XY Cursor Plane."
 puts " -xzColor   xzfg    \[green\]   Color for XZ Cursor Plane."
 puts " -yzColor   yzfg    \[red\]     Color for YZ Cursor Plane."
 puts " -xyStipple xyXBM   \[default\] Stipple for XY Cursor Plane."
 puts " -xzStipple xzXBM   \[default\] Stipple for XZ Cursor Plane."
 puts " -yzStipple yzXBM   \[default\] Stipple for YZ Cursor Plane."
 puts "Notes:"
 puts " 1. Contour height can be specified as a value, or by a"
 puts "    percentage ending with the '%' character, or by keywords"
 puts "    'Auto' or 'Full'."
 puts " 2. Option -cubeHi can be used as an alternative to the -hi option."
 puts " 3. Use '-colors multi' for spectral colors selected according"
 puts "    to intensity rather than Z-Axis position."
 puts " 4. See file 'rgb.txt' in the NMRPipe 'nmrtxt' directory for valid"
 puts "    colors$tStr"
 puts " 5. The graphic produced is a simple stack of fixed-sized 2D"
 puts "    plots with a linear offset."
 exit 0
}

set hi      Auto 

set tMargin 16 
set bMargin 76 
set lMargin 26  
set rMargin 90

set mult     7.0
set plev     1
set nlev     1
set lw2d     1

set axisFlag 1

set cxSize   360
set cySize   360
set czSize   Auto

set xOff     16
set yOff     32

set bgColor   black
set fgColor   white
set winColor  dimGray
set xyColor   blue 
set xzColor   green 
set yzColor   red
set xyStipple default
set xzStipple default
set yzStipple default
set posColor  None
set negColor  None
set fillColor gray

set colorA   white
set colorS   black

set xName    H_AXIS
set yName    V_AXIS
set zName    Z_AXIS
set aName    A_AXIS

set cursorFlag 0
set xPlane     1
set yPlane     1
set zPlane     1
set aPlane     1

set ix1      0%
set ixn      100%
set iy1      0%
set iyn      100%
set iz1      0%
set izn      100%
set ia1      0%
set ian      100% 

set colorList {red orange yellow green blue cyan orchid}

getArgD $ARGV -in      inName

if {[flagLoc $ARGV -cubeHi]} \
   {
    getArgD $ARGV -cubeHi hi
   } \
else \
   {
    getArgD $ARGV -hi     hi
   }

foreach d {x y z a} \
   {
    getArgD $ARGV  -${d}Name  ${d}Name
    getArgD $ARGV  -i${d}1    i${d}1
    getArgD $ARGV  -i${d}n    i${d}n
    getArgD $ARGV  -i${d}     ${d}Plane
   }

getArgD $ARGV -tMargin tMargin
getArgD $ARGV -bMargin bMargin
getArgD $ARGV -rMargin rMargin
getArgD $ARGV -lMargin lMargin

getArgD $ARGV -mult    mult
getArgD $ARGV -plev    plev
getArgD $ARGV -nlev    plev
getArgD $ARGV -lw2d    lw2d

getArgD $ARGV -cxSize  cxSize
getArgD $ARGV -cySize  cySize
getArgD $ARGV -czSize  czSize

getArgD $ARGV -xOff    xOff
getArgD $ARGV -yOff    yOff

getArgD $ARGV  -bgColor  bgColor
getArgD $ARGV  -fgColor  fgColor
getArgD $ARGV  -winColor winColor
getArgD $ARGV  -posColor posColor
getArgD $ARGV  -negColor negColor
getArgD $ARGV  -colorA   colorA
getArgD $ARGV  -colorS   colorS

getArgD $ARGV  -xyColor  xyColor
getArgD $ARGV  -xzColor  xzColor
getArgD $ARGV  -yzColor  yzColor

getArgD $ARGV  -xyStipple xyStipple
getArgD $ARGV  -xzStipple xzStipple
getArgD $ARGV  -yzStipple yzStipple
getArgD $ARGV  -fillColor fillColor

getListArgD $ARGV -colors colorList

if {[flagLoc $ARGV -axis]}   {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]} {set axisFlag 0}

if {[flagLoc $ARGV -cursor]}   {set cursorFlag 1}
if {[flagLoc $ARGV -nocursor]} {set cursorFlag 0}

proc drawCube {} \
{
   global rMargin lMargin tMargin bMargin wxSize wySize cxSize cySize czSize
   global ix1 ixn iy1 iyn iz1 izn ia1 ian kz1 kzn nZ planeCount dimCount
   global winColor fgColor bgColor colorA colorS posColor negColor nC colorList 
   global fillColor xyColor xzColor yzColor xyStipple xzStipple yzStipple env
   global xPlane yPlane zPlane aPlane cursorInfo cursorFlag axisFlag
   global xName yName zName aName inName hi plev nlev lw2d

   if {[info exists env(NMRTXT)]} \
      {
       if {![strcasecmp $xyStipple default]} \
          {
           if {[file exists $env(NMRTXT)/gray1A.xbm]} \
              {
               set xyStipple @$env(NMRTXT)/gray1A.xbm
              }
          }

       if {![strcasecmp $xzStipple default]} \
          {
           if {[file exists $env(NMRTXT)/gray1B.xbm]} \
              {
               set xzStipple @$env(NMRTXT)/gray1B.xbm 
              }
          }

       if {![strcasecmp $yzStipple default]} \
          {
           if {[file exists $env(NMRTXT)/gray1C.xbm]} \
              {
               set yzStipple @$env(NMRTXT)/gray1C.xbm 
              }
          }
      }

   if {![strcasecmp $xyStipple default]} {set xyStipple gray12} 
   if {![strcasecmp $xzStipple default]} {set xzStipple gray12} 
   if {![strcasecmp $yzStipple default]} {set yzStipple gray12} 

   if {![strcasecmp $xyStipple None]} {set xyStipple ""} 
   if {![strcasecmp $xzStipple None]} {set xzStipple ""} 
   if {![strcasecmp $yzStipple None]} {set yzStipple ""} 

   if {![strcasecmp $fillColor None]} {set fillColor ""} 

   set cxn [expr 1 + $wxSize - $rMargin]
   set cx1 [expr 1 + $cxn - $cxSize]

   set cyn $tMargin
   set cy1 [expr $cyn + $cySize - 1]

   set iz 1

   set xD  [expr int(double($czSize)*($planeCount - $iz)/$nZ)]
   set yD  [expr int(double($czSize)*($planeCount - $iz)/$nZ)]

   set tx1 [expr $cx1 - $xD]
   set txn [expr $cxn - $xD]

   set ty1 [expr $cy1 + $yD]
   set tyn [expr $cyn + $yD]

   defineCell -win 1 -cell 2 -x $txn $cxn -y $ty1 $cy1 

   drawRec -win 1 -x 0% 100% -y 0% 100% -fg $winColor

   if {[strcasecmp $winColor $bgColor]} \
      {
       for {set iz $czSize} {$iz >= 1} {incr iz -1} \
          {
           set xD  [expr $czSize - $iz]
           set yD  [expr $czSize - $iz]

           set qx1 [expr $cx1 - $xD]
           set qxn [expr $cxn - $xD]

           set qy1 [expr $cy1 + $yD]
           set qyn [expr $cyn + $yD]
   
           drawRec -win 1 -x $qx1 $qxn -y $qy1 $qyn -fg $bgColor
          }
      }

   drawBox  -win 1 -x $cx1 $cxn -y $cy1 $cyn -fg $fgColor
   drawLine -win 1 -x $tx1 $cx1 -y $ty1 $cy1 -fg $fgColor 
   drawLine -win 1 -x $txn $cxn -y $ty1 $cy1 -fg $fgColor 

   for {set iz $kzn} {$iz >= $kz1} {incr iz -1} \
      {
       set xD  [expr int(double($czSize)*($planeCount - $iz)/$nZ)]
       set yD  [expr int(double($czSize)*($planeCount - $iz)/$nZ)]
       set ic  [expr int(double($nC)*($planeCount - $iz)/$nZ)]
   
       if {$ic > $nC - 1} {set ic [expr $nC - 1]}

       set color1 [lindex $colorList $ic]
       set color2 [lindex $colorList $ic]

       if {[strcasecmp $posColor None]} {set color1 $posColor}
       if {[strcasecmp $negColor None]} {set color2 $negColor}

       set tx1 [expr $cx1 - $xD]
       set txn [expr $cxn - $xD]

       set ty1 [expr $cy1 + $yD]
       set tyn [expr $cyn + $yD]

       if {$dimCount == 4} \
          {
           readROI -ndim 2 -roi 1 -in $inName \
                   -x $xName $ix1 $ixn -y $yName $iy1 $iyn \
                   -dz $zName $iz 0 -a $aName $ia1 $ian
          } \
       else \
          {
           readROI -ndim 2 -roi 1 -in $inName \
                   -x  $xName $ix1 $ixn -y $yName $iy1 $iyn \
                   -dz $zName $iz 0
          } 

       defineCell -win 1 -cell 1 -x $tx1 $txn -y $ty1 $tyn
   
       drawROI    -win 1 -cell 1 -roi 1 -bg None -fg None -nobox \
                  -hi $hi -color1 $color1 -color2 $color2        \
                  -plev $plev -nlev $nlev -lw2d $lw2d 
 
       if {$iz == $kzn && $axisFlag} \
          {
           drawROI -axisOnly -axis right -win 1 -cell 1 -roi 1 -nobox \
                   -colorA $colorA -colorS $colorS
          }

      if {$iz == $kz1 && $axisFlag} \
         {
          drawROI -axisOnly -axis bottom -win 1 -cell 1 -roi 1 -nobox \
                  -colorA $colorA -colorS $colorS
         } 

       update
      }

   if {$planeCount > 1 && $dimCount > 2} \
      {
       if {$dimCount == 4} \
          {
           readROI -ndim 2 -roi 1 -in $inName \
                   -x $xName $ix1 $ixn -y $zName $kz1 $kzn -da $aName 1 0
          } \
       else \
          {
           readROI -ndim 2 -roi 1 -in $inName \
                   -x $xName $ix1 $ixn -y $zName $kz1 $kzn
          }

       if {$axisFlag} \
          {
           drawROI -axisOnly -axisTilt -axis right -win 1 -cell 2 -roi 1 -nobox \
                   -colorA $colorA -colorS $colorS
          }
      }

   drawLine -win 1 -x $tx1 $cx1 -y $tyn $cyn -fg $fgColor
   drawLine -win 1 -x $txn $cxn -y $tyn $cyn -fg $fgColor
   drawBox  -win 1 -x $tx1 $txn -y $ty1 $tyn -fg $fgColor

   set cursorInfo(xy,first) "$tx1 $ty1 $txn $tyn"
   set cursorInfo(xy,last)  "$cx1 $cy1 $cxn $cyn"

   set cursorInfo(xz,first) "$tx1 $ty1 $cx1 $cy1 $cxn $cy1 $txn $ty1"
   set cursorInfo(xz,last)  "$tx1 $tyn $cx1 $cyn $cxn $cyn $txn $tyn"

   set cursorInfo(yz,first) "$tx1 $ty1 $tx1 $tyn $cx1 $cyn $cx1 $cy1"
   set cursorInfo(yz,last)  "$txn $ty1 $txn $tyn $cxn $cyn $cxn $cy1"

   set s hidden
   if {$cursorFlag} {set s normal}

   if {[strcasecmp $xyColor None]} \
      {
       .c create rectangle [getCursorCoords xy $zPlane] \
          -fill $fillColor -stipple $xyStipple -outline $xyColor  \
          -tags "xy cursorPlane" -state $s 
      }

   if {[strcasecmp $xzColor None]} \
      {
       .c create polygon [getCursorCoords xz $yPlane]  \
          -fill $fillColor -stipple $xzStipple -outline $xzColor \
          -tags "xz cursorPlane" -state $s 
      }

   if {[strcasecmp $yzColor None]} \
      {
       .c create polygon [getCursorCoords yz $xPlane]  \
          -fill $fillColor -stipple $yzStipple -outline $yzColor \
          -tags "yz cursorPlane" -state $s 
      }
 
   update

   bind . <KeyPress> "+ parseKey .c %K %x %y"
}

proc getCursorCoords { mode loc } \
{
   global cursorInfo xSize ySize zSize aSize

   set size 0

   switch $mode \
      {
       xy {
           set size $zSize
          }

       xz {
           set size $ySize
          }

       yz {
           set size $xSize
          }
      }

   if {$size < 2} return

   if {$loc < 1} {set loc 1}
   if {$loc > $size} {set loc $size}
   
   set f [expr double($loc - 1)/($size - 1)] 
   set n [llength $cursorInfo($mode,first)]

   set cList ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set x1 [lindex $cursorInfo($mode,first) $i]
       set xn [lindex $cursorInfo($mode,last)  $i]
       set dx [expr $xn - $x1]

       lappend cList [expr int(0.5 + $x1 + $f*$dx)]
      }

   return $cList
}

proc parseKey { w kStr xPix yPix } \
{
   switch $kStr \
      {
       1 {toggleCubeCursor xy} 
       2 {toggleCubeCursor xz} 
       3 {toggleCubeCursor yz} 
       t {toggleCubeCursor cursorPlane}
      }
}

proc showCubeCursor { mode } \
{
   set w .c
   $w itemconfig $mode -state normal 
   update
}

proc hideCubeCursor { mode } \
{
   set w .c
   $w itemconfig $mode -state hidden 
   update
}

proc toggleCubeCursor { mode } \
{
   set sList ""
   set w .c

   if {![strcasecmp $mode cursorPlane]} \
      {
       foreach i {xy xz yz} {lappend sList [$w itemcget $i -state]}
      } \
   else \
      {
       lappend sList [$w itemcget $mode -state]
      }

   if {[lsearch -exact [string tolower $sList] hidden] >= 0} \
      {
       $w itemconfig $mode -state normal 
      } \
   else \
      {
       $w itemconfig $mode -state hidden 
      }

   update
}

proc setCubeCursor { dim loc } \
{
   global xSize ySize zSize aSize xPlane yPlane zPlane aPlane

puts "get event setCubeCursor $dim $loc"

   set size 0
   set mode None

   if {$loc < 1} {set loc 1}

   switch [string tolower $dim] \
      {
       x {
          set size $xSize
          if {$loc > $size} {set loc $size}

          set xPlane $size
          set mode   yz
         }

       y {
          set size $ySize
          if {$loc > $size} {set loc $size}

          set yPlane $size
          set mode xz
         } 

       z {
          set size $zSize
          if {$loc > $size} {set loc $size}

          set zPlane $size
          set mode xy
         }
      }

   if {!$size} return

   .c coord $mode [getCursorCoords $mode $loc]
   update
}

#
#---------------------------------------------------------------------------

wm geom . +$xOff+$yOff

#
# Check input:

set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "Error Finding 3D Drawing Input $inName"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from 3D Input $inName"
    exit 1
   }

set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

if {$dimCount < 2} \
   {
    puts stderr "3D Drawing Error: Bad number of dimensions."
    exit 1
   }

#
# Identify axis names: 

set xDim [getDimKeyword $xName $fdata]
set yDim [getDimKeyword $yName $fdata]

set xLab [getParmStr $fdata NDLABEL $xDim]
set yLab [getParmStr $fdata NDLABEL $yDim]

if {$dimCount > 3} \
   {
    set zDim [getDimKeyword $zName $fdata]
    set aDim A_AXIS 

    set zLab [getParmStr $fdata NDLABEL $zDim]
    set aLab A_AXIS
   } \
else \
   {
    set zDim [getDimKeyword $zName $fdata]
    set aDim [getDimKeyword $aName $fdata]

    set zLab [getParmStr $fdata NDLABEL $zDim]
    set aLab [getParmStr $fdata NDLABEL $aDim]
   }

#
# Find first and last plane:

set xSize [getParmI $fdata NDSIZE $xDim]
set ySize [getParmI $fdata NDSIZE $yDim]
set zSize [getParmI $fdata NDSIZE $zDim]

set kz1   [expr int([spec2pnt $fdata $zName $iz1])]
set kzn   [expr int([spec2pnt $fdata $zName $izn])]

if {$kz1 < 1} {set kz1 1}
if {$kz1 > $zSize} {set kz1 $zSize}

if {$kzn < 1} {set kzn 1}
if {$kzn > $zSize} {set kzn $zSize}

if {$kz1 > $kzn} \
   {
    set tmp $kz1
    set kz1 $kzn
    set kzn $tmp
   }

#
# Create the graphics area:

if {$dimCount == 4} \
   {
    wm title . \
     "$inName Z-Axis $zLab Planes $kz1 to $kzn A-Axis $aLab $ia1 to $ian"
   } \
else \
   {
    wm title . "$inName Z-Axis $zLab Planes $kz1 to $kzn"
   }

if {![strcasecmp $czSize Auto]} \
   {
    set czSize [expr int(0.25*($cxSize + $cySize))]
   }

set wxSize [expr $bMargin + $lMargin + $cxSize + $czSize]
set wySize [expr $tMargin + $bMargin + $cySize + $czSize]

canvas .c -width $wxSize -height $wySize -border 0
pack   .c -in . -side top -expand 1 -fill both

button .q -text Quit -command exit
pack   .q -in . -side top -expand 1 -fill x

update

.c create pixmap 0 0 -width $wxSize -height $wySize -win 1

#
# Calculate the contour drawing height as needed:

set hiInfo [getSpecContHi $inName $hi $plev $nlev $mult]
set hi     [lindex $hiInfo 0]

puts $hiInfo

#
# Draw the planes:

set planeCount [expr 1 + $kzn - $kz1]

if {$kz1 == $kzn} \
   {
    set nZ 1
   } \
else \
   {
    set nZ [expr $kzn - $kz1]
   }

set nC [llength $colorList]

if {!$nC} \
   {
    set colorList white
    set nC 1
   }

drawCube

puts "NMRCube Done."
