#!/bin/csh 

#
# Example usage:
#
#   if (-e /u/delaglio/com/nmrInit.com) then
#      source /u/delaglio/com/nmrInit.com
#   endif
#   
#   if (!($?PATHFIXED)) then
#      set path = (`uniqenv.com "$path" " "`)
#      setenv PATHFIXED TRUE
#   endif

if ($?NMRPATHSET && $?LD_LIBRARY_PATH) then
   exit 0
endif

#
# I/O Optimization: these settings may improve NMRPipe pipeline performance: 

setenv NMR_IO_TIMEOUT 0
setenv NMR_IO_SELECT  0
setenv NMR_AUTOSWAP   1

#
# NMRPipe Plugins:

setenv NMR_PLUGIN_FN    SMILE:FLATT
setenv NMR_PLUGIN_EXE   nusPipe:nusPipe
setenv NMR_PLUGIN_INFO  "-nDim -sample sName ... MD NUS Reconstruction":"-bw ... 1D FLATT Baseline Correction"

#
# Binary type:

set    noEcho = 1
setenv NMRBASE  /u/delaglio

if (-e ${NMRBASE}/com/binval.com) then
   source ${NMRBASE}/com/binval.com
   setenv BINVAL $binType
else
   setenv BINVAL unknown
endif

setenv NMR_TCLTK8 TRUE 

if ($BINTYPE == linux) then
   setenv BINVAL linux
endif

if ($BINTYPE == linux9) then
   setenv BINVAL linux9
endif

if ($BINTYPE == linux212_64) then
   setenv BINVAL linux212_64
endif

if ($BINTYPE == mac11) then
   setenv BINVAL mac11 
endif

if ($BINTYPE == mac11_64) then
   setenv BINVAL mac11_64
endif

if ($BINTYPE == mac_arm64) then
   setenv BINVAL mac_arm64
endif

if (-e /usr/local/lib/linuxver.7) then
   setenv BINVAL     linux
   unsetenv NMR_TCLTK8 TRUE 
endif

unset noEcho

#
# Environment definitions; note special case for NMRBIN, for NIH NMR lab only.
# Usual definition should be: setenv NMRBIN ${NMRBASE}/bin/${BINVAL}

if ($BINVAL == sun) then
   setenv NMRBIN ${NMRBASE}/bin
   setenv NMRLIB ${NMRBASE}/local/${BINVAL}/lib
   setenv AUXLIB ${NMRBASE}/xview_src/${BINVAL}/openwin/lib
else
   setenv NMRBIN ${NMRBASE}/bin/${BINVAL}
   setenv NMRLIB ${NMRBASE}/local/${BINVAL}/lib
   setenv AUXLIB ${NMRBASE}/xview_src/${BINVAL}/openwin/lib
endif

setenv NMR_CHECKARGS   ALL
setenv NMR_FSCHECK     NO
setenv NMRBINTYPE      $BINVAL
setenv NMRTXT          ${NMRBASE}/nmrtxt
setenv TCLPATH         ${NMRBASE}/com
setenv TALOS_DIR       ${NMRBASE}/talos
setenv TALOSP_DIR      ${NMRBASE}/talosplus
setenv TALOSN_DIR      ${NMRBASE}/talosn
setenv SPARTAP_DIR     ${NMRBASE}/spartaplus
setenv PROMEGA_DIR     ${NMRBASE}/promega

if ($?NMR_TCLTK8) then
   setenv TCL_LIBRARY     ${NMRBASE}/nmrtcl/tcl8.4
   setenv TK_LIBRARY      ${NMRBASE}/nmrtcl/tk8.4
   setenv BLT_LIBRARY     ${NMRBASE}/nmrtcl/blt2.4z
   setenv NMRPIPE_TCL_LIB ${NMRBASE}/nmrtcl/tcl8.4
   setenv NMRPIPE_TK_LIB  ${NMRBASE}/nmrtcl/tk8.4
   setenv NMRPIPE_BLT_LIB ${NMRBASE}/nmrtcl/blt2.4z
else
   setenv TCL_LIBRARY     ${NMRBASE}/nmrtcl/tcl7.6
   setenv TK_LIBRARY      ${NMRBASE}/nmrtcl/tk4.2
   setenv BLT_LIBRARY     ${NMRBASE}/nmrtcl/blt2.4
   setenv NMRPIPE_TCL_LIB ${NMRBASE}/nmrtcl/tcl7.6
   setenv NMRPIPE_TK_LIB  ${NMRBASE}/nmrtcl/tk4.2
   setenv NMRPIPE_BLT_LIB ${NMRBASE}/nmrtcl/blt2.4
endif

#
# Useful aliases:

alias u       'chmod a+x *.com *.tcl'

#set   history = 256
#alias h      'history'
#alias rm     'rm -i'
#alias cd     'cd \!*; set prompt = "$cwd% "'
#alias dirs   'ls -l | grep -E -e ^d'
#alias links  'ls -l | grep -E -e ^l'
#alias fgrep  'grep -F'
#alias egrep  'grep -E'

#
# Paths for manuals, executables, and fonts:

if ($?MANPATH) then
   setenv MANPATH \
   ${NMRBASE}/man:/usr/man:/usr/share/man:/usr/share/catman:/X11/usr/man:/usr/local/man:${MANPATH}
else
   setenv MANPATH \
   ${NMRBASE}/man:/usr/man:/usr/share/man:/usr/share/catman:/X11/usr/man:/usr/local/man
endif

set path = \
 (. ${NMRBIN} ${TCLPATH} ${NMRBASE}/bin /usr/X11R6/bin /usr/X11/bin $path)

if ($BINVAL =~ sgi* && -e ${NMRBASE}/com/font.com) then
   source ${NMRBASE}/com/font.com
endif

#
# XView Graphics library definitions:

if (!($?LD_LIBRARY_PATH)) then
   setenv LD_LIBRARY_PATH ${NMRLIB}:${AUXLIB}
else
   setenv LD_LIBRARY_PATH ${NMRLIB}:${LD_LIBRARY_PATH}:${AUXLIB}
endif

if (!($?DYLD_LIBRARY_PATH)) then
   setenv DYLD_LIBRARY_PATH ${NMRLIB}:${AUXLIB}
else
   setenv DYLD_LIBRARY_PATH ${NMRLIB}:${DYLD_LIBRARY_PATH}:${AUXLIB}
endif

if (!($?SHLIB_PATH)) then
   setenv SHLIB_PATH      ${NMRLIB}:${AUXLIB}
else
   setenv SHLIB_PATH      ${NMRLIB}:${SHLIB_PATH}:${AUXLIB}
endif

if (!($?OPENWINHOME)) then
   setenv OPENWINHOME /usr/openwin
endif

#
# Exit:

setenv NMRPATHSET TRUE
