#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#--------------------------------------------------------------

set inName  None
set pdbName None
set segName None

set boundMin 1.8 
set boundMax None

set boundLo  None 
set boundHi  None

set resetCount 0
set clipCount  0

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Convert XPLOR NOE Input to DYNAMO Format."
    puts " -in  inName          XPLOR Input (NOE Information Only)."
    puts " -pdb pdbName         Corresponding PDB (for Sequence, Atom Names)."
    puts " -seg segName         Default Seg Name."
    puts "Clipping of Distance Bounds: (Use 'None' to Retain Exact Distance):"
    puts " -min minDist  \[1.8\]  Lower Allowed Distance."
    puts " -max maxDist  \[None\] Upper Allowed Distance."
    puts "Reset of Distance Bounds: (Use 'None' to Retain Exact Distance):"
    puts " -lo  loDist   \[None\] Lower Distance for All Cases."
    puts " -hi  hiDist   \[None\] Upper Distance for All Cases."
    exit 0
   }

getArgD $ARGV -in  inName
getArgD $ARGV -pdb pdbName
getArgD $ARGV -seg segName
getArgD $ARGV -min boundMin
getArgD $ARGV -max boundMax
getArgD $ARGV -lo  boundLo
getArgD $ARGV -hi  boundHi

#
#--------------------------------------------------------------

proc parseNOEAss { assID line } \
{
   global boundMin boundMax boundHi boundLo resetCount clipCount

   parseXPLORAss $assID $line 2 assList tail

   set leftAss  [lindex $assList 0]
   set rightAss [lindex $assList 1]

   set d0 [lindex $tail 0]
   set d1 [lindex $tail 1]
   set d2 [lindex $tail 2]

   set dHi [expr $d0 + $d2]
   set dLo [expr $d0 - $d1]

   if {[strcasecmp $boundLo None]} \
      {
       set dLo $boundLo
       incr resetCount
      } 

   if {[strcasecmp $boundHi None]} \
      {
       set dHi $boundHi
       incr resetCount
      } 

   if {[strcasecmp $boundMin None]} \
      {
       if {$dLo < $boundMin} \
          {
           set dLo $boundMin
           incr clipCount
          }

       if {$dHi < $boundMin} \
          {
           set dHi $boundMin
           incr clipCount
          }
      } 

   if {[strcasecmp $boundMax None]} \
      {
       if {$dLo > $boundMax} \
          {
           set dLo $boundMax
           incr clipCount
          }

       if {$dHi > $boundMax} \
          {
           set dHi $boundMax
           incr clipCount
          }
      } 

   set assText $leftAss,$rightAss
   set pList   [permList $assText]
   set groupID 1
 
   foreach p $pList \
      {
       printNOEAss $assID $groupID $p $dLo $dHi
       incr groupID
      }
}

proc printNOEAss { assID groupID p dLo dHi } \
{
   global seqInfo segName

   set str [string toupper [split $p "~,"]]

   set resIDI    [lindex $str 0]
   set atomNameI [lindex $str 1]
   set segNameI  [lindex $str 2]
   set resNameI  $seqInfo($resIDI,$segNameI)
   set atomListI [expandXPLORAtom $resIDI $atomNameI $segNameI]

   set resIDJ    [lindex $str 3]
   set atomNameJ [lindex $str 4]
   set segNameJ  [lindex $str 5]
   set resNameJ  $seqInfo($resIDJ,$segNameJ)
   set atomListJ [expandXPLORAtom $resIDJ $atomNameJ $segNameJ]

   foreach atomI $atomListI \
      {
       foreach atomJ $atomListJ \
          {
           puts -nonewline [format "%4d %2d" $assID $groupID]
           puts -nonewline [format "  %3d %-4s %-4s" $resIDI $resNameI $atomI]
           puts -nonewline [format " %-4s" $segNameI]
           puts -nonewline [format "  %3d %-4s %-4s" $resIDJ $resNameJ $atomJ]
           puts -nonewline [format " %-4s" $segNameJ]
           puts [format "  %7.3f %7.3f %.1f %.1f %.1f"  $dLo $dHi 1.0 1.0 1.0]
          }
      }
}

proc printNOEHdr {} \
{
   puts "VARS INDEX GROUP RESID_I RESNAME_I ATOMNAME_I SEGNAME_I RESID_J RESNAME_J ATOMNAME_J SEGNAME_J D_LO D_HI FC W S"

   puts "FORMAT %4d %3d  %5d %6s %6s %4s  %5d %6s %6s %4s  %9.3f %9.3f %.2f %.2f %.2f"

   puts ""
}

proc parseNOE {} \
{
   global inUnit 

   set itemCount           0
   set assInfo($itemCount) ""

   while {[gets $inUnit line] >= 0} \
      {
       set keyword [lindex $line 0]

       if {"$keyword" == "assign"} \
          {
           set line [lrange $line 1 end]
           set line [stripXPLORComment $line]

           incr itemCount

           wm title . "Reading $itemCount"
           update
          }

       append assInfo($itemCount) " $line"
      }

   for {set assID 1} {$assID <= $itemCount} {incr assID} \
      {
       wm title . "Interpreting $assID/$itemCount"
       update

       set result ""
       set error  0

       set error [catch {parseNOEAss $assID $assInfo($assID)} result]

       if {$error} \
          {
           puts stderr "DYNAMO Warning: Error interpreting Restraint $assID:"
           puts stderr "  Error Code: $error $result"
           puts stderr "  Line: $assInfo($assID)"
          }
      }
}

if {![file exists $inName]} \
   {
    puts stderr "Error finding input restraint file $inName"
    exit 1
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error finding input PDB file $pdbName"
    exit 1
   }

dynInit -fast

showDynLogo .

wm title . "Reading $pdbName"
update

set inUnit [open $inName r]

getXPLORPDBInfo $pdbName

wm title . "Reading $inName"
update

printNOEHdr
parseNOE

puts stderr "NOE Number of Distance Bounds Reset:   $resetCount"
puts stderr "NOE Number of Distance Bounds Clipped: $clipCount"

exit
