#!/bin/csh

if ($#argv == 0) then
   echo "2D Magnitude J-Resolved Conversion and Processing,"
   echo "with Solvent Filter, Manual Tilting, Center Offset Adjustment,"
   echo "and Symmetrization."
   echo "
   echo " $0 inName tiltValue centerAdj xSize ySize hdrBytes wordSize"
   echo "Example:"
   echo " $0 P03584 -0.05 1.0 1024 128 48132 4"
   echo "Notes:"
   echo " Default values are:"
   echo " tilt=0.0 adj=0.0 xSize=1024 ySize=128 hdrBytes=48132 ws=4"
   exit
endif

set x1ppm = 3.2ppm
set xnppm = 1.0ppm

set inRoot = $1

if ($#argv > 1) then
   set tiltVal = $2
else
   set tiltVal = 0.0
endif

if ($#argv > 2) then
   set adj = $3
else
   set adj = 0.0
endif

if ($#argv > 3) then
   set xSize = $4
else
   set xSize = 1024
endif

if ($#argv > 4) then
   set ySize = $5
else
   set ySize = 128 
endif

set fileInfo = (`/bin/ls -l $inRoot`)
set fileSize = $fileInfo[5]

echo "True file size: $fileSize"

@ oldSize16 = 48132 + 2 * $xSize * $ySize
@ oldSize32 = 48132 + 4 * $xSize * $ySize
@ newSize16 = 44080 + 2 * $xSize * $ySize
@ newSize32 = 48176  + 4 * $xSize * $ySize

set defOffset = 0

if ($newSize16 == $fileSize) then
    set defOffset = 44080
endif

if ($newSize32 == $fileSize) then
   set defOffset = 48176
endif

if ($defOffset == 0) then
   set defOffset = 48132
endif

if ($oldSize16 == $fileSize || $newSize16 == $fileSize) then
   set defWordSize = 2
else
   set defWordSize = 4
endif

if ($oldSize16 == $fileSize || $newSize16 == $fileSize) then
   set defWordSize = 2
else
   set defWordSize = 4
endif

echo "Header Size: $defOffset Bytes Per Word: $defWordSize"

if ($#argv > 5) then
   set offset = $6
else
   set offset = $defOffset 
endif

if ($#argv > 6) then
   set wordSize = $7
else
   set wordSize = $defWordSize
endif

@ xSize2 = $xSize / 2

if ($wordSize == 4) then 
   bin2pipe -in $inRoot -bo $offset -i2r -ri -neg  \
     -xN            $xSize  -yN            $ySize  \
     -xT           $xSize2  -yT            $ySize  \
     -xMODE        Complex  -yMODE           Real  \
     -xSW         1000.000  -ySW          100.000  \
     -xOBS         63.8600  -yOBS          63.860  \
     -xCAR           4.700  -yCAR           0.000  \
     -xLAB               H  -yLAB               J  \
     -ndim               2  -aq2D       Magnitude  \
     -out $inRoot.fid -verb -ov
else
   @ offset *= 2

   byteAdjust -in $inRoot -is 65536 \
   | bin2pipe -bo $offset -i2r -ri -neg  \
     -xN            $xSize  -yN            $ySize  \
     -xT           $xSize2  -yT            $ySize  \
     -xMODE        Complex  -yMODE           Real  \
     -xSW         1000.000  -ySW          100.000  \
     -xOBS         63.8600  -yOBS          63.860  \
     -xCAR           4.700  -yCAR           0.000  \
     -xLAB               H  -yLAB               J  \
     -ndim               2  -aq2D       Magnitude  \
     -out $inRoot.fid -verb -ov
endif

echo "Created converted result $inRoot.fid"

nmrPipe -in $inRoot.fid \
| nmrPipe -fn SP -off .5 -pow 2   \
| nmrPipe -fn ZF -auto            \
| nmrPipe -fn FT -verb            \
| nmrPipe -fn TP                  \
| nmrPipe -fn SP -off .5 -pow 2   \
| nmrPipe -fn ZF -auto            \
| nmrPipe -fn FT -verb -neg       \
| nmrPipe -fn MC                  \
| nmrPipe -fn TP                  \
   -out $inRoot.ft2 -ov

echo "Created processed result $inRoot.ft2"

set maxInfo = \
 (`M $NMRTXT/maxLoc.M -str name $inRoot.ft2 x1ppm $x1ppm xnppm $xnppm -quit`)

set ySW     = (`getParm -in $inRoot.ft2 -parm NDSW   -dim CUR_YDIM`)
set ySize   = (`getParm -in $inRoot.ft2 -parm NDSIZE -dim CUR_YDIM`)
set maxLoc  = $maxInfo[3]
set mid     = $maxInfo[5]
set shift   = $maxInfo[7]

set mid   = (`MATH $mid   + $adj`)
set shift = (`MATH $shift - $adj`)

echo $maxInfo
echo "Max for J=0 at point $maxLoc, offset of $shift, ySW is $ySW"

nmrPipe -in $inRoot.fid \
| nmrPipe -fn SOL                 \
| nmrPipe -fn SP -off .5 -pow 2   \
| nmrPipe -fn ZF -auto            \
| nmrPipe -fn FT -verb            \
| nmrPipe -fn TP                  \
| nmrPipe -fn SP -off .5 -pow 2   \
| nmrPipe -fn ZF -auto            \
| nmrPipe -fn FT -verb -neg       \
| nmrPipe -fn MC                  \
| nmrPipe -fn TP                  \
   -out $inRoot.sol.ft2 -ov

echo "Created solvent-suppressed processed result $inRoot.sol.ft2"

nmrPipe -in $inRoot.sol.ft2 \
| nmrPipe -fn MAC -macro $NMRTXT/notilt.M -noRd -noWr    \
                  -var mid $maxLoc tilt $tiltVal         \
| nmrPipe -fn TP                  \
| nmrPipe -fn CS -ls $shift       \
| nmrPipe -fn TP                  \
   -out $inRoot.tilt.ft2 -ov

echo "Created shifted, tilted result $inRoot.tilt.ft2"

nmrPipe -in $inRoot.tilt.ft2 \
| nmrPipe -fn TP                                        \
| nmrPipe -fn MAC -macro $NMRTXT/sym.M -var mid $mid    \
| nmrPipe -fn TP                                        \
  -out $inRoot.sym.ft2 -verb -ov

echo "Created symmetrized result $inRoot.sym.ft2"
