#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepare an NMRPipe-Format NUS Schedule Simulation:"
    puts " -in     inName   \[fid/test%03d.fid\]  Full-Schedule Input Data."
    puts " -nus    outName  \[projYZ.dat\]        NMRPipe-format NUS Output."
    puts " -frac   f        \[0.25\]              Fraction of Data to Keep."
    puts " -ndim   ndim     \[2\]                 Number of Dimensions for Schedule."
    puts " -xDecay xD       \[1.0\]               Decay Fraction at End of X-Axis Schedule."
    puts " -yDecay yD       \[1.0\]               Decay Fraction at End of Y-Axis Schedule."
    puts " -iter   itMax    \[262144\]            Maximum Allowed Iterations."
    puts " -iseed  iseed    \[654321\]            Random Number Seed, 0 for Auto."
    puts "Notes:"
    puts " 1. Output is an NMRPipe-Format File of Ones and Zeros."
    puts " 2. Currently Supports Schedule for 3D FID Only."
    exit 0
   }

set inName   fid/test%03d.fid
set outName  profYZ.dat
set tmpName  nustmp.dat
set xAxis    Y_AXIS
set yAxis    Z_AXIS
set zAxis    A_AXIS
set iseed    654321
set frac     1.0
set dimCount 2
set maxIter  1

set xDecay 1.0
set yDecay 1.0
set zDecay 1.0

getArgD $ARGV -ndim dimCount

if {$dimCount < 1} {set dimCount 1}
if {$dimCount > 3} {set dimCount 3}

for {set i 0} {$i < $dimCount} {incr i} \
    {
     set frac    [expr $frac/2.0]
     set maxIter [expr 512*$maxIter]
    }

getArgD $ARGV -in  inName
getArgD $ARGV -nus outName
getArgD $ARGV -tmp tmpName

getArgD $ARGV -xDecay xDecay
getArgD $ARGV -yDecay yDecay
getArgD $ARGV -zDecay zDecay

getArgD $ARGV -xAxis  Y_AXIS
getArgD $ARGV -yAxis  Z_AXIS
getArgD $ARGV -zAxis  A_AXIS

getArgD $ARGV -frac   frac
getArgD $ARGV -iter   maxIter
getArgD $ARGV -iseed  iseed 

srand $iseed

if {$frac < 0.0} {set frac 0.0}
if {$frac > 1.0} {set frac 1.0}

if {$xDecay > 1.0}    {set xDecay 1.0}
if {$xDecay < 1.0e-6} {set xDecay 1.0e-6}

if {$yDecay > 1.0}    {set yDecay 1.0}
if {$yDecay < 1.0e-6} {set yDecay 1.0e-6}

if {$zDecay > 1.0}    {set zDecay 1.0}
if {$zDecay < 1.0e-6} {set zDecay 1.0e-6}

#
#-----------------------------------------------------------------------------

set thisName [format $inName 1]

if {![file exists $thisName]} \
   {
    puts stderr "NUS Schedule Error Finding Input Data $thisName"
    exit 1
   }

readROI -in   $inName -ndim 2 -x $xAxis 0% 100% -y $yAxis 0% 100% -roi 2
writeROI -out $tmpName -roi 2 -ov

exec nmrPipe -in $tmpName -fn EXT -left \
     | nmrPipe -fn TP | nmrPipe -fn EXT -left | nmrPipe -fn TP \
     | nmrPipe -fn SET -c 1.0 -out $outName -ov

readROI  -in $outName -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -roi 1
writeROI -out $outName -roi 1 -ov

set rInfo    [getROIInfo -roi 1]
set rdata    [lindex $rInfo 0]
set fdata    [lindex $rInfo 1]
set sizeInfo [lindex $rInfo 2]
set xSize    [lindex $sizeInfo 0]
set ySize    [lindex $sizeInfo 1] 
set zSize    1

for {set xLoc 0} {$xLoc < $xSize} {incr xLoc} {set xTally($xLoc) 0}
for {set yLoc 0} {$yLoc < $ySize} {incr yLoc} {set yTally($yLoc) 0}
for {set zLoc 0} {$zLoc < $zSize} {incr zLoc} {set zTally($zLoc) 0}

set xLab     [getParmStr $fdata NDLABEL CUR_XDIM]
set yLab     [getParmStr $fdata NDLABEL CUR_YDIM]
set zLab     None

set nTotal   [expr $xSize*$ySize*$zSize]
set nTarget  [expr int($frac*$nTotal)]
set nCurrent $nTotal
set nFail    0

puts "$inName $xLab $xSize Points $yLab $ySize Points"
puts "Total Size: $nCurrent Target Size: $nTarget"

set xSize1 [expr $xSize - 1]
set ySize1 [expr $ySize - 1]

set xA 0.0
set yA 0.0
set zA 0.0

if {$xSize1 > 0} {set xA [expr log($xDecay)/$xSize1]}
if {$ySize1 > 0} {set yA [expr log($yDecay)/$ySize1]}

while {$nCurrent > $nTarget} \
   {
    if {$nFail > $maxIter} \
       {
        puts stderr "NUS Schedule Maximum Iteration Count Reached."
        break
       }

    set xLoc [expr int($xSize*[rand])]
    set yLoc [expr int($ySize*[rand])]

    if {$xLoc < 0} {set xLoc 0}
    if {$xLoc > $xSize1} {set xLoc $xSize1}

    if {$yLoc < 0} {set yLoc 0}
    if {$yLoc > $ySize1} {set yLoc $ySize1}

    if {$xLoc == 0 && $yLoc == 0 || $xLoc == $xSize1 && $yLoc == $ySize1} \
       {
        incr nFail
        continue
       }

    set t [expr 0.5*exp($xA*($xSize1 - $xLoc))*exp($yA*($ySize1 - $yLoc))]
    set r [rand]

    if {$r > $t} \
       {
        incr nFail
       } \
    else \
       {
        set loc [expr $xLoc + $xSize*$yLoc]
        set val [getPtrVal $rdata $loc]

        if {$val > 0.5} \
           {
            setPtrVal $rdata $loc 0.0
            incr nCurrent -1
            set nFail 0
           } \
        else \
           {
            incr nFail
           }
       }
   }

set loc 0

for {set yLoc 0} {$yLoc < $ySize} {incr yLoc} \
   {
    for {set xLoc 0} {$xLoc < $xSize} {incr xLoc} \
       {
        set val [getPtrVal $rdata $loc]

        if {$val > 0.5} \
           {
            incr xTally($xLoc)
            incr yTally($yLoc)
           }

        incr loc
       }
   }

set nxMin $xTally(0)
set nxMax $xTally(0)

set nyMin $yTally(0)
set nyMax $yTally(0)

for {set xLoc 0} {$xLoc < $xSize} {incr xLoc} \
   {
    if {$xTally($xLoc) < $nxMin} {set nxMin $xTally($xLoc)}
    if {$xTally($xLoc) > $nxMax} {set nxMax $xTally($xLoc)}
   }

for {set yLoc 0} {$yLoc < $ySize} {incr yLoc} \
   {
    if {$yTally($yLoc) < $nyMin} {set nyMin $yTally($yLoc)}
    if {$yTally($yLoc) > $nyMax} {set nyMax $yTally($yLoc)}
   }

puts "Target Size: $nTarget Actual Size: $nCurrent"
puts " X Samples Min: $nxMin Max: $nxMax"
puts " Y Samples Min: $nyMin Max: $nyMax"

writeROI -roi 1 -out $outName -ov


exit 0
