#!/bin/sh 
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@" 

set auto_path "[split $env(TCLPATH) :] $auto_path" 
set ARGV      [concat $argv0 $argv] 
set ARGC       [llength $ARGV] 

if {[flagLoc $ARGV -help]} \
   {
    puts "Find Optimal Orientation for Two Domains Using Dipolar Couplings."
    puts " -ref  refPDBName \[ref.pdb\]     Reference (Fixed) PDB Input File."
    puts " -pdb  rotPDBName \[rot.pdb\]     PDB Input File To Be Rotated."
    puts " -dc   dcNameList \[dObs*.tab\]   DC Input Tables."
    puts " -tab  tabName    \[orient.tab\]  Output Table of Orientation Maxima."
    puts " -fHi             \[1.0\]         First Contour Height."
    puts " -lHi             \[10.0\]        Last Contour Height."
    puts " -save                          Save Surfaces surf*/r%03d.dat"
    puts " -nosave                        No Surface File Output (Default)."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts " -ignore                        Ignore Seg/Chain Information."
    puts " -exit                          Exit After Grid Search is Complete."
    puts "PDB Output Templates:"
    puts " -or   orPDBName  \[orient%d.pdb\]  Output of Rotated PDB."
    puts " -out  outName    \[complex%d.pdb\] Output of Rotated PDB and Ref."
    puts " -n    bestCount  \[2\]             Number of Orientations to Save."
    puts "Rotation Grid Search Parameters, Degrees:"
    puts " -rx1    rx1       \[-180.0\]   X-Axis Starting Rotation." 
    puts " -rxn    rxn        \[180.0\]   X-Axis Ending Rotation."
    puts " -rxStep rxStep       \[5.0\]   X-Axis Rotation Step."
    puts " -ry1    ry1          \[0.0\]   Y-Axis Starting Rotation." 
    puts " -ryn    ryn        \[180.0\]   Y-Axis Ending Rotation."
    puts " -ryStep ryStep       \[5.0\]   Y-Axis Rotation Step."
    puts " -rz1    rz1          \[0.0\]   Z-Axis Starting Rotation." 
    puts " -rxn    rzn        \[180.0\]   Z-Axis Ending Rotation."
    puts " -rzStep rzStep       \[5.0\]   Z-Axis Rotation Step."
#   puts " -refine                      Refine First Result by 1 Degree Steps."
    exit
   }

set refName    ref.pdb
set pdbName    rot.pdb
set orName     orient%d.pdb
set outName    complex%d.pdb
set tabName    orient.tab
set trialCount 1
set bestCount  2
set verbose    1
set ignoreFlag 0
set saveFlag   0
set exitFlag   0

set rx1     -180
set rxn      180
set rxStep     5

set ry1        0
set ryn      180
set ryStep     5

set rz1     -180
set rzn      180
set rzStep     5
set fHi      1.0
set lHi     10.0

set dcNameList [glob -nocomplain dObs*.tab]

getArgD $ARGV -ref    refName
getArgD $ARGV -pdb    pdbName
getArgD $ARGV -out    outName
getArgD $ARGV -tab    tabName
getArgD $ARGV -or     orName
getArgD $ARGV -n      bestCount

getArgD $ARGV -rx1    rx1
getArgD $ARGV -rxn    rxn
getArgD $ARGV -rxStep rxStep

getArgD $ARGV -ry1    ry1
getArgD $ARGV -ryn    ryn
getArgD $ARGV -ryStep ryStep

getArgD $ARGV -rz1    rz1
getArgD $ARGV -rzn    rzn
getArgD $ARGV -rzStep rzStep
 
getArgD $ARGV -fHi    fHi 
getArgD $ARGV -lHi    lHi

getListArgD $ARGV -in dcNameList
getListArgD $ARGV -dc dcNameList

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -save]}   {set saveFlag 1}
if {[flagLoc $ARGV -nosave]} {set saveFlag 0}

if {[flagLoc $ARGV -ignore]} {set ignoreFlag 1}
if {[flagLoc $ARGV -refine]} {set trialCount 2}

if {[flagLoc $ARGV -exit]}   {set exitFlag 1}

#
#--------------------------------------------------------------------

if {![file exists $refName]} \
   {
    puts stderr "Error Finding PDB Reference Input $refName"
    exit 1
   }

if {![file exists $pdbName]} \
   {
    puts stderr "Error Finding PDB Input $pdbName"
    exit 1
   }

foreach dcName $dcNameList \
   {
    if {![file exists $dcName]} \
       {
        puts stderr "Error Finding DC Input $dcName"
        exit 1
       }
   }

#
#------------------------------------------------------------------

proc initGridWin {} \
{
   global dx dy cxAdj cyAdj cx1 cxn cy1 cyn cellHi cellWide
   global winXSize winYSize drawCount

   wm geom . +40+40

   set drawCount 0

   set cellHi    600
   set cellWide  600
   
   set dx 40
   set dy 20

   set cxAdj 100
   set cyAdj  80

   set cx1 $dx 
   set cy1 $dy
   set cxn [expr $cx1 + $cellWide] 
   set cyn [expr $cy1 + $cellHi] 

   set winXSize [expr $cxn + $cxAdj]
   set winYSize [expr $cyn + $cyAdj]
 
   canvas .c -width $winXSize -height $winYSize -border 0
   button .b -text Quit -command exit
   pack   .c -side top
   pack   .b -side top -expand 1 -fill x

   .c create pixmap 0 0 -width $winXSize -height $winYSize -win 1

   defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1

   clearGrid
}

proc clearGrid {} \
{
   global winXSize winYSize drawCount

   drawRec -x 0 $winXSize -y 0 $winYSize -fg dimGray 
   drawRec -cell 1 -x 0% 100% -y 0% 100% -fg black

   set drawCount 0

   update
}

proc saveGrid {} \
{
   global plane trial trialCount gridSize saveFlag fdata FDATASIZE

   if {!$saveFlag} return

   if {$trialCount == 1} \
      {
       set surfDir surf
      } \
   else \
      {
       set surfDir surf$trial
      }

   if {![file exists $surfDir]} {catch "file mkdir $surfDir"}

   set thisName [format "$surfDir/r%03d.dat" $plane]
   set outUnit  [sysOpen $thisName w]

   if {$outUnit < 0} \
      {
       puts stderr "Error Opening Surface Output File $thisName"
       return 1
      }

   set rInfo [getROIInfo -roi 2]
   set pdata [lindex $rInfo 0]

   sysWrite $outUnit $fdata $FDATASIZE
   sysWrite $outUnit $pdata $gridSize
   
   sysClose $outUnit
  
   return 0 
}

proc drawGrid {} \
{
   global rx1 rxn ry1 ryn rz rz1 rzn gridSize fHi lHi dRMSMin plane drawCount

   readROI -inROI 1 -roi 2 -ndim 2 -x RX -y RY -dz RZ $plane 0

   set rInfo [getROIInfo -roi 2]
   set pdata [lindex $rInfo 0]

   set minVal [vMin $pdata $gridSize]
   set maxVal [vMax $pdata $gridSize]

   set f [expr ($rz - $rz1)/($rzn - $rz1)]

   set fmt \
    "RZ %.1f of %.1f to %.1f DC RMS Min: %.3f Max %.3f Best Possible: %.3f Hz"

   wm title . [format $fmt $rz $rz1 $rzn $minVal $maxVal $dRMSMin]

# -fhi $hiF*$dRMSMin -lhi [expr 0.5*$maxVal]

   if {!$drawCount} \
      {
       drawROI -roi 2 -cell i -bg Black -fhi $fHi -lhi $lHi \
               -pcc 16 -plev 16 -mult 1 -pHue1 0.0 -pHue2 0.7 -fg white \
               -axis Bottom,lab=RX,step=30,orig=$rx1,end=$rxn \
                     Right,lab=RY,step=30,orig=$ry1,end=$ryn -xGrid -yGrid 
      } \
   else \
      {
       drawROI -roi 2 -cell i -bg None -fhi $fHi -lhi $lHi \
               -pcc 16 -plev 16 -mult 1 -pHue1 0.0 -pHue2 0.7 -fg white
      }

   incr drawCount

   update
}

proc createGrid {} \
{
   global fdata rdata rPtr xSize ySize zSize gridSize totalSize FDATASIZE
   global rx1 rxn rxStep ry1 ryn ryStep rz1 rzn rzStep

   set xSize 0
   set ySize 0
   set zSize 0

   for {set rx $rx1} {$rx <= $rxn} {set rx [expr $rx + $rxStep]} {incr xSize} 
   for {set ry $ry1} {$ry <= $ryn} {set ry [expr $ry + $ryStep]} {incr ySize} 
   for {set rz $rz1} {$rz <= $rzn} {set rz [expr $rz + $rzStep]} {incr zSize} 

   set totalSize  [expr $xSize*$ySize*$zSize]
   set gridSize   [expr $xSize*$ySize]

   set error [catch {set fdata [sysMalloc $FDATASIZE float]}]

   if {$error} \
      {
       puts stderr ""
       puts stderr "Error allocating grid header."
       exit 1
      }

   set error [catch {set rdata [sysMalloc $totalSize float]}]

   if {$error} \
      {
       puts stderr ""
       puts stderr "Error allocating grid data."
       exit 1
      }

   vsSet $rdata 0.0 $totalSize

   set xw [expr double($rxn - $rx1)*($xSize + 1)/$xSize]
   set yw [expr double($ryn - $ry1)*($ySize + 1)/$ySize]
   set zw [expr double($rzn - $rz1)*($zSize + 1)/$zSize]

   set x0 [expr 0.5*($rx1 + $rxn) + ($rxn - $rx1)/$xSize]
   set y0 [expr 0.5*($ry1 + $ryn) + ($ryn - $ry1)/$ySize]
   set z0 [expr 0.5*($rz1 + $rzn) + ($rzn - $rz1)/$zSize]

   buildHdr $fdata \
     -xN      $xSize  -yN      $ySize  -zN      $zSize \
     -xT      $xSize  -yT      $ySize  -zT      $zSize \
     -xMODE   Real    -yMODE   Real    -zMode   Real   \
     -xSW     $xw     -ySW     $yw     -zSW     $zw    \
     -xOBS    1.0000  -yOBS    1.0000  -zOBS    1.0000 \
     -xCAR    $x0     -yCAR    $y0     -zCAR    z0     \
     -xFT     Freq    -yFT     Freq    -zFT     Freq   \
     -xLAB    RX      -yLAB    RY      -zLAB    RZ     \
     -ndim    3       -aq2D    Image -pipeFlag 1 -cubeFlag 0 -fileCount 1

#  set outUnit [sysOpen tmpsurf.dat w]
#
#  sysWrite $outUnit $fdata $FDATASIZE
#  sysWrite $outUnit $rdata $totalSize
#
#  readROI -roi 1 -in tmpsurf.dat -x RX -y RY -z RZ -ndim 3

   readROI -roi 1 -inPtr $rdata -inHdr $fdata -x RX -y RY -z RZ -ndim 3

   sysFree $rdata $totalSize

   set rInfo [getROIInfo -roi 1]
   set rdata [lindex $rInfo 0]

   set rPtr $rdata
}

proc searchGrid {} \
{
   global rPtr rx1 rxn rxStep ry1 ryn ryStep rx ry rz
   global refSrc pdbSrc dSrcN plane

   for {set ry $ry1} {$ry <= $ryn} {set ry [expr $ry + $ryStep]} \
      {
       for {set rx $rx1} {$rx <= $rxn} {set rx [expr $rx + $rxStep]} \
           {
            dynRestore -src $pdbSrc
            dynRotate  -src $pdbSrc -rx $rx -ry $ry -rz $rz -all

            set dInfoN [dynSimulate -dcList $dSrcN -src $pdbSrc -keepTensor]
            set n      [llength $dInfoN]
            set dRMS   [lindex $dInfoN [expr $n - 3]]

            setPtrVal *$rPtr $dRMS

            set rPtr [incrPtr $rPtr]
           }
      }
}

proc doGridSearch {} \
{ 
   global rx ry rz rx1 rxn rxStep ry1 ryn ryStep rz1 rzn rzStep pdbSrc 
   global plane trial trialCount

   dynStore -src $pdbSrc

   initGridWin

   for {set trial 1} {$trial <= $trialCount} {incr trial} \
      {
       set fmt   "Grid Search Steps: %4.1f %4.1f %4.1f"
       set plane 0

       puts [format $fmt $rxStep $ryStep $rzStep]

       createGrid
       clearGrid

       for {set rz $rz1} {$rz <= $rzn} {set rz [expr $rz + $rzStep]} \
           {
            set p [expr 100.0*($rz - $rz1)/($rzn - $rz1)]
            incr  plane

            puts -nonewline [format " %5.1f%%\015" $p]
            flush stdout

            searchGrid
            drawGrid
            saveGrid
           }

        findOrientMaxima

        if {$trial == $trialCount} break

        set rx1 [expr int($rx - 2*$rxStep - 0.5)]
        set rxn [expr int($rx + 2*$rxStep + 0.5)]

        set ry1 [expr int($ry - 2*$ryStep - 0.5)]
        set ryn [expr int($ry + 2*$ryStep + 0.5)]

        set rz1 [expr int($rz - 2*$rzStep - 0.5)]
        set rzn [expr int($rz + 2*$rzStep + 0.5)]

        set rxStep 1.0
        set ryStep 1.0
        set rzStep 1.0
       }
}

proc extractEntryRot { entry } \
{
   global xSize ySize zSize rx1 rxn ry1 ryn rz1 rzn rx ry rz plane

   set px [gdbGet $entry X_AXIS]
   set py [gdbGet $entry Y_AXIS]
   set pz [gdbGet $entry Z_AXIS]

   while {$px > $xSize} {set px [expr $px - $xSize]}
   while {$py > $ySize} {set py [expr $py - $ySize]}
   while {$pz > $zSize} {set pz [expr $pz - $zSize]}

   set rx [expr $rx1 + ($rxn - $rx1)*($px - 1.0)/($xSize - 1)]
   set ry [expr $ry1 + ($ryn - $ry1)*($py - 1.0)/($ySize - 1)]
   set rz [expr $rz1 + ($rzn - $rz1)*($pz - 1.0)/($zSize - 1)]

   set plane [expr int($pz + 0.5)]
}

proc findOrientMaxima {} \
{
   global rx ry rz rx1 rxn rxStep ry1 ryn ryStep rz1 rzn rzStep
   global rdata xSize ySize zSize totalSize tabName dRMSMin

   set thisMax [vMax $rdata $totalSize]

   vsSub $rdata $thisMax $totalSize

   readROI -ndim 3 -roi 3 -inROI 1 \
           -x X_AXIS 1 [expr $xSize + 2] \
           -y Y_AXIS 1 [expr $ySize + 2] \
           -z Z_AXIS 1 [expr $zSize + 2]

   vsAdd $rdata $thisMax $totalSize

   pkFindROI -roi 3 \
      -sigma 0.1 -pChi 0 -plus 0.1 -minus -0.1 \
      -dx        1     1     1 \
      -idx       1     1     1 \
      -out $tabName -verb

   set sTab  [gdbRead table -in $tabName]
   set vID   [gdbFindByName $sTab HEIGHT]

   gdbSort $vID

   set eList [gdbGetInfo $sTab -parms entries]

   set error [catch {set outUnit [open $tabName w]}]

   if {$error} \
      {
       puts stderr "Error Opening Output Table $tabName"
       return 1
      }

   puts $outUnit "DATA RX $rx1 $rxn $rxStep $xSize"
   puts $outUnit "DATA RY $ry1 $ryn $ryStep $ySize"
   puts $outUnit "DATA RZ $rz1 $rzn $rzStep $ySize"
   puts $outUnit ""
   puts $outUnit "REMARK Best Possible DC RMS: [format %.3f $dRMSMin] Hz"
   puts $outUnit ""
   puts $outUnit "VARS    INDEX RX    RY    RZ    D_RMS"
   puts $outUnit "FORMAT  %2d   %8.2f %8.2f %8.2f %8.4f"
   puts $outUnit ""

   set fmt1 " %2d % 8.2f % 8.2f % 8.2f  %8.4f"
   set fmt2 " %2d. Rotations: % 8.2f % 8.2f % 8.2f DC RMS: %8.3f Hz"

   set bestD $thisMax
   set id 0 

   puts stderr ""

   foreach entry $eList \
      {
       extractEntryRot $entry

       incr id

       gdbSet $entry INDEX $id

       set dRMS [gdbGet $entry HEIGHT]
       set dRMS [expr $dRMS + $thisMax]

       puts $outUnit [format $fmt1 $id $rx $ry $rz $dRMS]
       puts stderr   [format $fmt2 $id $rx $ry $rz $dRMS]
      }

   close $outUnit

   puts stderr ""
   puts stderr "Created $tabName"

   return 0
}

proc initPDB {} \
{
   global refName pdbName refSrc pdbSrc minRes maxRes ignoreFlag

   dynInit -fast

   if {$ignoreFlag} \
      {
       dynSetChainMode -ignore
       dynSetSegMode   -ignore
      }

   set refSrc [dynRead -pdb -in $refName]
   set pdbSrc [dynRead -pdb -in $pdbName]

   set pInfo  [dynGetInfo -src $pdbSrc -limits]
   set minRes [lindex $pInfo 0]
   set maxRes [lindex $pInfo 1]
}

proc initTensor {} \
{
   global refSrc pdbSrc dcNameList dSrcList dSrcN dRMSMin tCount

   set dSrcList [dynRead -dc -in $dcNameList -list]
   set dSrcN    [lindex $dSrcList end]


   set dInfoN  [dynSimulate -dcList $dSrcN -src $pdbSrc -svd]
   set n       [llength $dInfoN]
   set dRMSMin [lindex $dInfoN [expr $n - 3]]

   set dInfoN  [dynSimulate -dcList $dSrcN -src $refSrc -svd]
   set tCount  [expr [llength $dSrcList] - 1]

   for {set n 1} {$n <= $tCount} {incr n} \
      {
       set dSrc [lindex $dSrcList [expr $n - 1]]

       set tInfo($n) [getNthTensor $dInfoN $n]
       set sInfo($n) [extractSaupe $tInfo($n)]

       dynSetTensor -dc $dSrc -saupe $sInfo($n)
      }
}

proc saveOrient {} \
{
   global rx ry rz pdbSrc refSrc orName outName tabName bestCount

   dynTranslate -src $refSrc -center -all

   set pInfo  [dynGetInfo -src $refSrc -limits]
   set aCount [lindex $pInfo 2]

   set pInfo [dynGetInfo -src $refSrc -extent -all]
   set xn    [lindex $pInfo 1]

   dynTranslate -src $refSrc -tx [expr -1.0*($xn + 1)] -all

   set sTab  [gdbRead table -in $tabName]
   set eList [gdbGetInfo $sTab -parms entries]

   for {set i 1} {$i <= $bestCount} {incr i} \
      {
       set entry [lindex $eList [expr $i - 1]]

       set thisOrName  [format $orName  $i]
       set thisOutName [format $outName $i]

       set rx   [gdbGet $entry RX]
       set ry   [gdbGet $entry RY]
       set rz   [gdbGet $entry RZ]
       set dRMS [gdbGet $entry D_RMS]

       dynRestore   -src $pdbSrc
       dynRotate    -src $pdbSrc -rx $rx -ry $ry -rz $rz -all
       dynTranslate -src $pdbSrc -center -all

       set fmt "REMARK RX: %.2f RY: %.2f RZ: %.2f DC RMS: %.3f Hz"
       set rem [format $fmt $rx $ry $rz $dRMS]

       set pInfo [dynGetInfo -src $pdbSrc -extent -all]
       set x1    [lindex $pInfo 0]

       dynTranslate -src $pdbSrc -tx [expr 1.0 - $x1] -all

       dynWrite -pdb -src $refSrc -out $thisOrName -renumber -rem $rem
       catch "exec fgrep -v END $thisOrName > $thisOutName"

       dynWrite -pdb -src $pdbSrc -out $thisOrName -renumber -off $aCount
       catch "exec cat $thisOrName >> $thisOutName"

       dynWrite -pdb -src $pdbSrc -out $thisOrName -renumber -rem $rem

       puts stderr "Created $thisOrName and $thisOutName"
      }
}

#
#------------------------------------------------------------------------------

initPDB
initTensor
doGridSearch
saveOrient

if {$exitFlag} exit
