#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "4D HN-Methyl NOE Viewer:"
    puts " -dir  \[../\]    Root Data Directory."
    puts " -xAdj \[12\]     Adjustment for Window Width, Pixels."
    puts " -yAdj \[65\]     Adjustment for Window Height, Pixels."
    exit 1
   }

set d              ../ 

getArgD $ARGV -dir d

set inName(hn4d)   $d/hn4dnoe/ft/test%02d%03d.ft4
set inName(hn_n_c) $d/hcn/ft/test%03d.ft3
set inName(h_c_hn) $d/hchn/ft/test%03d.ft3

set outName(hn4d)    hnhcAss.txt
set outName(assInfo) assInfo.txt

set tabName(hnAss)   $d/hn4dnoe/hnAss.tab
set tabName(h_c_hn)  hchn.tab

set varName(hnAss,INDEX)   RESID
set varName(hnAss,X_PPM)   HN_PPM
set varName(hnAss,Y_PPM)   N_PPM 

# HN N HC CH3

set varName(hn4d,X_AXIS)   X_AXIS
set varName(hn4d,Y_AXIS)   Y_AXIS
set varName(hn4d,Z_AXIS)   Z_AXIS
set varName(hn4d,A_AXIS)   A_AXIS

# HA CH3 HN

set varName(h_c_hn,INDEX)  INDEX
set varName(h_c_hn,X_PPM)  X_PPM
set varName(h_c_hn,Y_PPM)  Y_PPM 
set varName(h_c_hn,Z_PPM)  Z_PPM 

set varName(h_c_hn,X_AXIS)  X_AXIS
set varName(h_c_hn,Y_AXIS)  Y_AXIS
set varName(h_c_hn,Z_AXIS)  Z_AXIS

# HN C N
 
set varName(hn_n_c,X_AXIS) X_AXIS
set varName(hn_n_c,Y_AXIS) Z_AXIS
set varName(hn_n_c,Z_AXIS) Y_AXIS

set hi(hn)                 3.0e+4
set hi(hc)                 5.0e+4
set hi(h_c_hn)             2.0e+5
set hi(hn_n_c)             1.8e+6
set hi(factor)             1.3

# < set ppmOffset(hn4d,HN)     0.038
# > set ppmOffset(hn4d,HN)     0.029

# < set ppmOffset(h_c_hn,HN)   0.038
# > set ppmOffset(h_c_hn,HN)   0.029

# < set ppmOffset(hn_n_c,HN)   0.025
# > set ppmOffset(hn_n_c,HN)   0.019

set ppmOffset(hn4d,HN)     0.029
set ppmOffset(hn4d,N)     -0.512
set ppmOffset(hn4d,HC)     0.00
set ppmOffset(hn4d,CH3)    0.00

set planeOffset(hn4d,HN)   0
set planeOffset(hn4d,N)    0
set planeOffset(hn4d,HC)   0
set planeOffset(hn4d,CH3)  0

set ppmOffset(h_c_hn,HN)   0.029
set ppmOffset(h_c_hn,N)   -0.512
set ppmOffset(h_c_hn,HC)   0.00
set ppmOffset(h_c_hn,CH3)  0.00
set planeOffset(h_c_hn,HN) 0

set ppmOffset(hn_n_c,HN)   0.019
set ppmOffset(hn_n_c,N)   -0.512
set ppmOffset(hn_n_c,HC)   0.00
set ppmOffset(hn_n_c,CH3)  0.00
set planeOffset(hn_n_c,N)  0

set zoomInfo(hn,zoom)      0.2
set zoomInfo(n,zoom)       1.2
set zoomInfo(hc,zoom)      0.2
set zoomInfo(ch3,zoom)     1.2

set xAxisAdj    60
set yAxisAdj    80

set cellX       12
set cellY       18

set xAdj        12
set yAdj        56 

set topX        1
set topY        1

getArgD $ARGV -xAxisAdj xAxisAdj
getArgD $ARGV -yAxisAdj xAxisAdj

getArgD $ARGV -cellX    cellX
getArgD $ARGV -cellY    cellY

getArgD $ARGV -xAdj     xAdj
getArgD $ARGV -yAdj     yAdj

getArgD $ARGV -topX     topX
getArgD $ARGV -topY     topY

#
#----------------------------------------------------------------------------

foreach i {ass none hc hnhc hnCur hnhcCur} \
   {
    set boxFlag($i)      1
    set boxFlag($i,prev) 1
   }

set zoomInfo(cell)  None
set zoomInfo(x1Pix) 0 
set zoomInfo(xnPix) 0 
set zoomInfo(y1Pix) 0 
set zoomInfo(ynPix) 0 

foreach i {hn hc hn_n_c h_c_hn} \
   {
    set zoomInfo($i,x1)   0%
    set zoomInfo($i,xn) 100%
    set zoomInfo($i,y1)   0%
    set zoomInfo($i,yn) 100%

    set hi($i,default) $hi($i)
   }
 
set cursorFlag      1
set firstTime       1
set blankCount      0

set seqList     ""
set skipList    ""
set r1           0
set rN           0

set eID(hn)     -666
set eID(hc)     -666
set eID(hnAss)  -666
set eID(h_c_hn) -666
set eID(hn_n_c) -666

set eMax(hn4d)   0
set eMax(hn_n_c) 0
set eMax(h_c_hn) 0
set eMax(hnAss)  0

set shift(ass,hn)  None
set shift(ass,n)   None
set shift(ass,hc)  None
set shift(ass,ch3) None

set assInfo(hc)     ""
set assInfo(hn)     ""
set assInfo(note)   ""
set assInfo(idList) ""

set assInfo(saveNeeded) 0
set assInfo(save,resID) -666 
set assInfo(save,chID)  -666 

set hiInfo(hn4d)    0.0
set hiInfo(hn)      0.0
set hiInfo(h_c_hn)  0.0

set wx [winfo screenwidth  .]
set wy [winfo screenheight .]

set wxSize [expr $wx - $xAdj]
set wySize [expr $wy - $yAdj]

foreach i {hn hc h_c_hn hn_n_c} \
   {
    set roiInfo($i) ""
    set roiInfo($i,roiID) 0
   } 

setFonts

set w .t

toplevel $w

wm geom $w ${wxSize}x${wySize}+$topX+$topY

update

set xSize  [expr $wxSize - 1]
set ySize  [expr $wySize - $yAdj]
set cx1    10
set cy1    10
set dx     [expr ($xSize - 2*$yAxisAdj - $cx1)/2]
set dy     [expr ($ySize - 2*$xAxisAdj - $cy1)/2]


proc getBlank { w n } \
{
   global blankCount

   incr blankCount
   set  lab $w.blank[format %05d $blankCount]

   label $lab -text " " -width $n
}

proc getFold { type ppmLoc } \
{
   global zoomInfo varName

   set i hn4d

   switch -exact $type \
      {
       HN  {
            if {$ppmLoc > $zoomInfo($i,sx1)} {set ppmLoc $zoomInfo($i,sx1)}
            if {$ppmLoc < $zoomInfo($i,sxn)} {set ppmLoc $zoomInfo($i,sxn)}
           }

       N   {
            while {$ppmLoc > $zoomInfo($i,sy1)} \
               {
                set ppmLoc [expr $ppmLoc - $zoomInfo($i,swY)]
               }

            while {$ppmLoc < $zoomInfo($i,syn)} \
               {
                set ppmLoc [expr $ppmLoc + $zoomInfo($i,swY)]
               }
           }

       HC  {
            while {$ppmLoc > $zoomInfo($i,sz1)} \
               {
                set ppmLoc [expr $ppmLoc - $zoomInfo($i,swZ)]
               }

            while {$ppmLoc < $zoomInfo($i,szn)} \
               {
                set ppmLoc [expr $ppmLoc + $zoomInfo($i,swZ)]
               }
           }

       CH3 {
            while {$ppmLoc > $zoomInfo($i,sa1)} \
               {
                set ppmLoc [expr $ppmLoc - $zoomInfo($i,swA)]
               }

            while {$ppmLoc < $zoomInfo($i,san)} \
               {
                set ppmLoc [expr $ppmLoc + $zoomInfo($i,swA)]
               }
           }
      }

   return $ppmLoc
}

proc getSpecLimits { i } \
{
  global zoomInfo inName varName

  set fd 0

  set thisName [format $inName($i) 1 1]

  set error [rdFDATA $thisName fd]

  if {$error} \
     {
      puts "Error Reading Header in $thisName"
      exit 1
     }

  set dimCount [getParmI $fd FDDIMCOUNT NULL_DIM]

  set zoomInfo($i,fdata)    $fd
  set zoomInfo($i,dimCount) $dimCount

  set zoomInfo($i,xSize) [getParmI $fd NDSIZE $varName($i,X_AXIS)]
  set zoomInfo($i,sx1)   [pnt2spec $fd $varName($i,X_AXIS) 1 ppm] 
  set zoomInfo($i,sxn)   [pnt2spec $fd $varName($i,X_AXIS) $zoomInfo($i,xSize) ppm]
  set zoomInfo($i,swX)   [expr $zoomInfo($i,sx1) - $zoomInfo($i,sxn)]

  if {$dimCount == 1} return

  set zoomInfo($i,ySize) [getParmI $fd NDSIZE $varName($i,Y_AXIS)]
  set zoomInfo($i,sy1)   [pnt2spec $fd $varName($i,Y_AXIS) 1 ppm] 
  set zoomInfo($i,syn)   [pnt2spec $fd $varName($i,Y_AXIS) $zoomInfo($i,ySize) ppm] 
  set zoomInfo($i,swY)   [expr $zoomInfo($i,sy1) - $zoomInfo($i,syn)]

  if {$dimCount == 2} return

  set zoomInfo($i,zSize) [getParmI $fd NDSIZE $varName($i,Z_AXIS)]
  set zoomInfo($i,sz1)   [pnt2spec $fd $varName($i,Z_AXIS) 1 ppm] 
  set zoomInfo($i,szn)   [pnt2spec $fd $varName($i,Z_AXIS) $zoomInfo($i,zSize) ppm] 
  set zoomInfo($i,swZ)   [expr $zoomInfo($i,sz1) - $zoomInfo($i,szn)]

  if {$dimCount == 3} return

  set zoomInfo($i,aSize) [getParmI $fd NDSIZE $varName($i,A_AXIS)]
  set zoomInfo($i,sa1)   [pnt2spec $fd $varName($i,A_AXIS) 1 ppm] 
  set zoomInfo($i,san)   [pnt2spec $fd $varName($i,A_AXIS) $zoomInfo($i,aSize) ppm]
  set zoomInfo($i,swA)   [expr $zoomInfo($i,sa1) - $zoomInfo($i,san)]
}

proc drawCurrentPlane { i } \
{
   global eLoc eList eID varName

   set e       [lindex $eList($i) $eLoc($i)]

   set eID($i) [gdbGet $e $varName($i,INDEX)]

   set xPPM [gdbGet $e $varName($i,X_PPM)]
   set yPPM [gdbGet $e $varName($i,Y_PPM)]

   drawPlaneByPPM $i $xPPM $yPPM
}

proc createHNLabels {} \
{
   global tabName inTab eList eCount eLoc eID varName labInfo r1 rN resNames1
   global boxFlag ppmOffset roiInfo

   set i hnAss
   set w .t

   if {![llength $roiInfo(hc)]} return

   if {![info exists labInfo(hn,drawBoxes)]} \
      {
       set labInfo(hn,drawBoxes) 1
      }

   if {!$labInfo(hn,drawBoxes)} return

   set labInfo(hn,drawBoxes) 0

   $w.c delete assBox noneBox assText

   set xOff $ppmOffset(hn4d,HN)
   set yOff $ppmOffset(hn4d,N)

   foreach e $eList($i) \
      { 
       set xPPM [getFold HN [expr $xOff + [gdbGet $e $varName(hnAss,X_PPM)]]]
       set yPPM [getFold N  [expr $yOff + [gdbGet $e $varName(hnAss,Y_PPM)]]]

       if {![zoomIsInside hc $xPPM $yPPM]} continue
 
       set resID   [gdbGet $e RESID]
       set resName $resNames1($resID)
       set s       hidden

       if {[badAss $resID $resName]} \
          {
           set ass   ${resID}X
           set t     noneBox 
           set color red

           if {$boxFlag(none)} {set s normal}
          } \
       else \
          {
           set ass   $resID$resName
           set t     assBox 
           set color green 

           if {$boxFlag(ass)} {set s normal}
          }

       set ix0 [expr int( 0.5 + [spec2pix -roi 2 -cell 2 -x ${xPPM}ppm])]
       set iy0 [expr int( 0.5 + [spec2pix -roi 2 -cell 2 -y ${yPPM}ppm])]

       set ix1 [expr $ix0 - 2]
       set ixn [expr $ix0 + 2]
       set iy1 [expr $iy0 - 2]
       set iyn [expr $iy0 + 2]

       set tID [$w.c create text $ixn $iy1 -anchor sw \
                   -fill white -text $ass -tags assText -state hidden]

       set bID [$w.c create rectangle $ix1 $iy1 $ixn $iyn \
                   -fill $color -outline white -tags $t -state $s]

       $w.c bind $bID <Any-Enter> "+ showAssText 1 $bID $tID"
       $w.c bind $bID <Any-Leave> "+ showAssText 0 $bID $tID"

       $w.c bind $bID <ButtonRelease-1>       "selectByEntry $i $e"
       $w.c bind $bID <ButtonRelease-3>       "appendHN $e"
       $w.c bind $bID <Shift-ButtonRelease-3> "assignHC $e"
      }

   $w.c raise noneBox
   $w.c raise assBox

   update
}

proc createHCLabels {} \
{
   global tabName inTab eList eCount eLoc eID varName roiInfo
   global boxFlag ppmOffset eMax assInfo varName

   set i h_c_hn
   set w .t

   if {![llength $roiInfo($i)]} return

   set xOff $ppmOffset($i,HC)
   set yOff $ppmOffset($i,CH3)

   $w.c delete hcBox  hcText

   set v     $varName($i,Z_PPM)
   set hnPPM $roiInfo(h_c_hn,zPPM)

   set eMax($i)   0
   set maxVal   0.0

   foreach j $assInfo(idList) \
      {
       set iList [split $j ,]
       set resID [lindex $iList 0]
       set chID  [lindex $iList 1]

       if {$resID != $eID(hnAss)} continue

       set thisEList [gdbSelect -from $inTab($i) -cond "$varName($i,INDEX) == $chID"]

       if {![llength $thisEList]} continue
    
       set eMax($i) [lindex $thisEList 0]
       break
      }

   if {!$eMax($i)} \
      {
       set thisEList [gdbSelect -from $inTab($i) -cond "abs( $v - $hnPPM ) < 0.02"]

       if {[llength $thisEList]} \
          {
           foreach e $thisEList \
              {
               set val [gdbGet $e HEIGHT]

               if {abs($val) >= abs($maxVal)} \
                  {
                   set eMax($i) $e
                   set maxVal   $val
                  }
              }
          }
      }

   set thisEList [gdbSelect -from $inTab($i) -cond "abs( $v - $hnPPM ) < 0.04"]

   foreach e $thisEList \
      { 
       set xPPM [format %.3f [expr $xOff + [gdbGet $e $varName($i,X_PPM)]]]
       set yPPM [format %.3f [expr $yOff + [gdbGet $e $varName($i,Y_PPM)]]]

       if {![zoomIsInside h_c_hn $xPPM $yPPM]} continue

       set id   [gdbGet $e $varName($i,INDEX)]
       set val  [gdbGet $e HEIGHT]

       if {!$eMax($i) && (abs($val) >= abs($maxVal))} \
          {
           set eMax($i) $e
           set maxVal   $val 
          }

       if {![strcasecmp [gdbGet $e ASS] inserted]} \
          {
           set color darkOrchid
          } \
       else \
          {
           set color yellow
          }

       if {$boxFlag(hc)} \
          {
           set s normal 
          } \
       else \
          {
           set s hidden
          }

       set ix0 [expr int( 0.5 + [spec2pix -roi 3 -cell 3 -x ${xPPM}ppm])]
       set iy0 [expr int( 0.5 + [spec2pix -roi 3 -cell 3 -y ${yPPM}ppm])]

       set ix1 [expr $ix0 - 2]
       set ixn [expr $ix0 + 2]
       set iy1 [expr $iy0 - 2]
       set iyn [expr $iy0 + 2]

       set tID [$w.c create text $ixn $iy1 -anchor sw \
                   -text "$id ${xPPM}ppm ${yPPM}ppm [format %.1e $val]" \
                   -fill white -tags hcText -state hidden]

       set bID [$w.c create rectangle $ix1 $iy1 $ixn $iyn \
                   -fill $color -outline white -tags hcBox -state $s]

       $w.c bind $bID <Any-Enter> "+ showAssText 1 $bID $tID"
       $w.c bind $bID <Any-Leave> "+ showAssText 0 $bID $tID"

       $w.c bind $bID <ButtonRelease-1> "selectByEntry $i $e"
      }

   $w.c raise hcBox
}

proc createHNHCLabels {} \
{
   global assInfo ppmOffset boxFlag resNames1 eID

   set rList ""
   set w .t

   $w.c delete hnhcBox hnCurBox hnhcCurBox hnhcText hnCurText

   set e   0
   set eCH 0
   set eHN 0
 
   foreach j $assInfo(idList) \
      {
       set iList [split $j ,]
       set resID [lindex $iList 0]
       set chID  [lindex $iList 1]
      
       if {$resID == $eID(hnAss)} \
          {
           if {$boxFlag(hnhcCur) || $boxFlag(hnhc)} \
              {
               set s normal
              } \
           else \
              {
               set s hidden
              }

           set color white 
           set t     hnhcCurBox
          } \
       else \
          {
           if {$boxFlag(hnhc)} \
              {
               set s normal
              } \
           else \
              {
               set s hidden
              }

           set color magenta 
           set t     hnhcBox
          }

       set xyInfo [getXYShift h_c_hn $chID e]

       set xPPM [getFold HC  [expr [lindex $xyInfo 0] + $ppmOffset(hn4d,HC)]]
       set yPPM [getFold CH3 [expr [lindex $xyInfo 1] + $ppmOffset(hn4d,CH3)]]

       if {![zoomIsInside hn $xPPM $yPPM]} continue

       set ix0 [expr int( 0.5 + [spec2pix -roi 1 -cell 1 -x ${xPPM}ppm])]
       set iy0 [expr int( 0.5 + [spec2pix -roi 1 -cell 1 -y ${yPPM}ppm])]

       set ix1 [expr $ix0 - 2]
       set ixn [expr $ix0 + 2]
       set iy1 [expr $iy0 - 2]
       set iyn [expr $iy0 + 2]

       set ass "$resID$resNames1($resID) CH $chID HN $assInfo($resID,$chID,hn)"

       set tID [$w.c create text $ixn $iy1 -anchor sw \
                   -fill white -text $ass -tags hnhcText -state hidden]

       set bID [$w.c create rectangle $ix1 $iy1 $ixn $iyn \
                   -fill $color -outline white -tags $t  -state $s]

       if {$resID == $eID(hnAss)} {$w.c raise $bID}
 
       $w.c bind $bID <Any-Enter> "+ showAssText 1 $bID $tID"
       $w.c bind $bID <Any-Leave> "+ showAssText 0 $bID $tID"

       $w.c bind $bID <ButtonRelease-1> "selectByEntryAndResID h_c_hn $e $resID"

       if {$chID == $eID(h_c_hn)} \
          {
           set thisResList [getAssResIDList $assInfo($resID,$chID,hn)]
           set loc 0

           foreach thisResID $thisResList \
              { 
               set xyInfo [getXYShift hnAss $thisResID e]

               set xPPM   [expr [lindex $xyInfo 0] + $ppmOffset(hn4d,HN)]
               set yPPM   [expr [lindex $xyInfo 1] + $ppmOffset(hn4d,N)]

               if {![zoomIsInside hc $xPPM $yPPM]} continue

               set ix0 \
                 [expr int( 0.5 + [spec2pix -roi 2 -cell 2 -x ${xPPM}ppm])]

               set iy0 \
                [expr int( 0.5 + [spec2pix -roi 2 -cell 2 -y ${yPPM}ppm])]
    
               set ix1 [expr $ix0 - 6]
               set ixn [expr $ix0 + 6]
               set iy1 [expr $iy0 - 6]
               set iyn [expr $iy0 + 6]
   
               set ass [lindex $assInfo($resID,$chID,hn) $loc]
 
               set bID [$w.c create oval $ix1 $iy1 $ixn $iyn \
                       -fill "" -outline magenta -tags hnCurBox  -state $s]

#              set tID [$w.c create text $ixn $iy1 -anchor sw \
#                      -fill white -text $ass -tags hnCurText -state hidden]

#              $w.c bind $bID <Any-Enter> "+ showAssText 1 $bID $tID"
#              $w.c bind $bID <Any-Leave> "+ showAssText 0 $bID $tID"

               incr loc
              } 
          }
      }

   $w.c raise hnhcBox
   $w.c raise hnhcCurBox

   update
}

proc getAssResIDList { assList } \
{
    set resList ""

    foreach aInfo $assList \
       {
        set ass   [lindex [split $aInfo =] 0]
        if {![strcasecmp $ass None]} continue

        set nn    [string length $ass]
        set resID ""

        for {set i 0} {$i < $nn} {incr i} \
           {
            set c [string index $ass $i]
            if {[string first $c "0123456789"] >= 0} {append resID $c}
           }

        lappend resList $resID
       }

   return $resList
}

proc cursorIsInside { i orient ppmLoc } \
{
   global zoomInfo

   if {![strcasecmp $orient vert]} \
      {
       set q1 x1
       set qn xn
      } \
   else \
     {
       set q1 y1
       set qn yn
     }

   if {![strcmp $zoomInfo($i,$q1) 0%] && ![strcmp $zoomInfo($i,$qn) 100%]} \
      {
       return 1
      }

   set xPPM [lindex [split $ppmLoc ppm] 0]

   set x1 [lindex [split $zoomInfo($i,$q1) ppm] 0]
   if {$xPPM > $x1} {return 0}

   set xn [lindex [split $zoomInfo($i,$qn) ppm] 0]
   if {$xPPM < $xn} {return 0}

   return 1
}

proc zoomIsInside { i xPPM yPPM } \
{
   global zoomInfo

   if {![strcmp $zoomInfo($i,x1) 0%] && ![strcmp $zoomInfo($i,xn) 100%] \
       && ![strcmp $zoomInfo($i,y1) 0%] && ![strcmp $zoomInfo($i,yn) 100%]} \
      {
       return 1
      }

   set x1 [lindex [split $zoomInfo($i,x1) ppm] 0]
   if {$xPPM > $x1} {return 0}

   set xn [lindex [split $zoomInfo($i,xn) ppm] 0]
   if {$xPPM < $xn} {return 0}

   set y1 [lindex [split $zoomInfo($i,y1) ppm] 0]
   if {$yPPM > $y1} {return 0}

   set yn [lindex [split $zoomInfo($i,yn) ppm] 0]
   if {$yPPM < $yn} {return 0}

   return 1
}

proc drawPlaneByPPM { i xPPM yPPM } \
{
   global inName hi firstTime labInfo ppmOffset roiInfo seqList r1 rN eID
   global hiInfo zoomInfo

   set w .t
 
   catch "busy hold $w"
   catch "busy hold .selectResID"
   update

   puts "drawPlaneByPPM $i $xPPM $yPPM"

   zoomOFF

   if {![strcmp $i hn] || ![strcmp $i hnAss]} \
      {
       set sx [format %.3f [expr $ppmOffset(hn4d,HN) + $xPPM]]
       set sy [format %.3f [expr $ppmOffset(hn4d,N)  + $yPPM]]

       if {$eID(hnAss) < $r1 || $eID(hnAss) > $rN} \
          {
           set ass "No ASS"
          } \
       else \
          {
           set ass $eID(hnAss)[lindex $seqList [expr $eID(hnAss) - $r1]]
          }

       set labInfo(hn,str) "4DHNHC HN=${sx}ppm N=${sy}ppm $ass"

       showStatus "Reading $labInfo(hn,str)."

       readROI -ndim 2 -roi 1 -in $inName(hn4d) \
               -x Z_AXIS $zoomInfo(hn,x1) $zoomInfo(hn,xn) \
               -y A_AXIS $zoomInfo(hn,y1) $zoomInfo(hn,yn) \
               -dz X_AXIS ${sx}ppm 0 -da Y_AXIS ${sy}ppm 0 -verb

       showStatus "Drawing $labInfo(hn,str)."

       clearCell hn

       showStatus Reading

       drawROI -roi 1 -cell 1 -noxGrid -noyGrid \
               -color1 multi -color2 multi -hi $hi(hn) \
               -bg black -axis Bottom Right -colorA dimGray -colorS white

       set roiInfo(hn,roiID) 1
       set roiInfo(hn)       [getROIInfo -roi 1]

       set roiInfo(hn,zPPM) $sx
       set roiInfo(hn,aPPM) $sy

       set sx [format %.3f [expr $ppmOffset(h_c_hn,HN) + $xPPM]]
       set sy [format %.3f [expr $ppmOffset(hn_n_c,N)  + $yPPM]]

       set labInfo(h_c_hn,str) "3D H_C_HN HN=${sx}ppm"

       showStatus "Reading $labInfo(h_c_hn,str)."

       readROI -ndim 2 -roi 3 -in $inName(h_c_hn) \
               -x X_AXIS $zoomInfo(h_c_hn,x1) $zoomInfo(h_c_hn,xn) \
               -y Y_AXIS $zoomInfo(h_c_hn,y1) $zoomInfo(h_c_hn,yn) \
               -dz Z_AXIS ${sx}ppm 0 -verb

       showStatus "Drawing $labInfo(h_c_hn,str)."

       clearCell h_c_hn

       drawROI -roi 3 -cell 3 -noxGrid -noyGrid \
               -color1 multi -color2 multi -hi $hi(h_c_hn) \
               -bg black -axis Bottom Right -colorA dimGray -colorS white

       set roiInfo(h_c_hn,roiID) 3
       set roiInfo(h_c_hn) [getROIInfo -roi 3]
       set roiInfo(h_c_hn,zPPM) $sx

       set labInfo(hn_n_c,str) "3D HN_N_C N=${sy}"

       showStatus "Reading $labInfo(hn_n_c,str)."

       readROI -ndim 2 -roi 4 -in $inName(hn_n_c) \
               -x X_AXIS $zoomInfo(hn_n_c,x1) $zoomInfo(hn_n_c,xn) \
               -y Z_AXIS $zoomInfo(hn_n_c,y1) $zoomInfo(hn_n_c,yn) \
               -dz Y_AXIS ${sy}ppm 0 -verb

       showStatus "Drawing $labInfo(hn_n_c,str)."

       clearCell hn_n_c

       drawROI -roi 4 -cell 4 -noxGrid -noyGrid \
               -color1 multi -color2 multi -hi $hi(hn_n_c) \
               -bg black -axis Bottom Right -colorA dimGray -colorS white

       set roiInfo(hn_n_c,roiID) 4
       set roiInfo(hn_n_c) [getROIInfo -roi 4]
       set roiInfo(hn_n_c,zPPM) $sy

       showStatus "Updating Residue Selection."

       resSelectUpdate
      } \
   elseif {![strcmp $i hc] || ![strcmp $i h_c_hn]} \
      {
       set sx [format %.3f [expr $ppmOffset(hn4d,HC)  + $xPPM]]
       set sy [format %.3f [expr $ppmOffset(hn4d,CH3) + $yPPM]]

       set labInfo(hc,str) \
        "4DHNHC H=${sx}ppm C=${sy}ppm Hi [format %.2e $hiInfo(hn4d)]"

       showStatus "Reading $labInfo(hc,str)."

       readROI -ndim 2 -roi 2 -in $inName(hn4d) \
               -x X_AXIS $zoomInfo(hc,x1) $zoomInfo(hc,xn) \
               -y Y_AXIS $zoomInfo(hc,y1) $zoomInfo(hc,yn) \
               -dz Z_AXIS ${sx}ppm 0 -da A_AXIS ${sy}ppm 0 -verb

       showStatus "Drawing $labInfo(hc,str)."

       clearCell hc

       drawROI -roi 2 -cell 2 -noxGrid -noyGrid \
               -color1 multi -color2 multi -hi $hi(hc) \
               -bg black -axis Bottom Right -colorA dimGray -colorS white


       set roiInfo(hc,roiID) 2
       set roiInfo(hc)       [getROIInfo -roi 2]

       set roiInfo(hc,zPPM), $sx
       set roiInfo(hc,aPPM), $sy

       showStatus "Updating HN Labels."
       createHNLabels 

       set firstTime 0
      } \
   elseif {![strcmp $i hn_n_c]} \
      {
      } 

   if {!$firstTime} \
      {
       showStatus "Updating Cursors."
       updateCursors
      }

   showStatus "Updating HC Labels."
   createHCLabels 

   showStatus "Updating HNHC Labels."
   createHNHCLabels

   catch "busy release $w"
   catch "busy release .selectResID"

   showStatus "Ready."
   update
}

proc updateCursors {} \
{
    global shift cellInfo roiInfo cursor ppmOffset hiInfo labInfo

    set w .t

    if {![strcasecmp $shift(ass,n) None]} \
       {
        $w.c itemconfigure $cursor(hc,h)  -state hidden
       } \
    else \
       {
        set s [expr $ppmOffset(hn4d,N) + $shift(ass,n)]ppm
     
        if {[cursorIsInside hc horiz $s]} \
           { 
            set iy0 [spec2pix -roi 2 -cell 2 -y $s]

            set ix1 [lindex $cellInfo(hc) 0]
            set ixn [lindex $cellInfo(hc) 1]

            $w.c coords        $cursor(hc,h) $ix1 $iy0 $ixn $iy0
            $w.c itemconfigure $cursor(hc,h)  -state normal 
           } \
        else \
           {
            $w.c itemconfigure $cursor(hc,h)  -state hidden 
           }
       }

    if {![strcasecmp $shift(ass,hn) None]} \
       {
        $w.c itemconfigure $cursor(hc,v)     -state hidden 
        $w.c itemconfigure $cursor(hn_n_c,v) -state hidden 
       } \
    else \
       {
        set s [expr $ppmOffset(hn4d,HN) + $shift(ass,hn)]ppm

        if {[cursorIsInside hc vert $s]} \
           {
            set ix0 [spec2pix -roi 2 -cell 2 -x $s]

            set iy1 [lindex $cellInfo(hc) 2]
            set iyn [lindex $cellInfo(hc) 3]

            $w.c coords        $cursor(hc,v) $ix0 $iy1 $ix0 $iyn
            $w.c itemconfigure $cursor(hc,v) -state normal 
           } \
        else \
           {
            $w.c itemconfigure $cursor(hc,v) -state hidden 
           }

        set s [expr $ppmOffset(hn_n_c,HN) + $shift(ass,hn)]ppm

        if {[cursorIsInside hn_n_c vert $s]} \
           {
            set ix0 [spec2pix -roi 4 -cell 4 -x $s]

            set iy1 [lindex $cellInfo(hn_n_c) 2]
            set iyn [lindex $cellInfo(hn_n_c) 3]

            $w.c coords        $cursor(hn_n_c,v) $ix0 $iy1 $ix0 $iyn
            $w.c itemconfigure $cursor(hn_n_c,v) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(hn_n_c,v) -state hidden 
           }
       }

   if {[strcasecmp $shift(ass,n) None] && [strcasecmp $shift(ass,hn) None]} \
      {
       set sx [expr $ppmOffset(hn4d,HN) + $shift(ass,hn)]ppm
       set sy [expr $ppmOffset(hn4d,N) + $shift(ass,n)]ppm

       readROI -roi 12 -inROI 2 -dx X_AXIS $sx 1 -dy Y_AXIS $sy 1 -ndim 2

       set maxVal [getROIStat 12 vMax]
       set minVal [getROIStat 12 vMin]

       if {abs($minVal) > abs($maxVal)} \
          {
           set hiInfo(hn4d) $minVal
          } \
       else \
          {
           set hiInfo(hn4d) $maxVal
          }
      }

   if {![strcasecmp $shift(ass,hc) None]} \
       {
        $w.c itemconfigure $cursor(hn,v)     -state hidden
        $w.c itemconfigure $cursor(h_c_hn,v) -state hidden

        set roiInfo(h_c_hn,xPPM) None 
        set roiInfo(h_c_hn,yPPM) None 
       } \
    else \
       {
        set xPPM [expr $ppmOffset(hn4d,HC) + $shift(ass,hc)]

        if {[cursorIsInside hn vert $xPPM]} \
           {
            set ix0  [spec2pix -roi 1 -cell 1 -x ${xPPM}ppm]

            set iy1 [lindex $cellInfo(hn) 2]
            set iyn [lindex $cellInfo(hn) 3]

            $w.c coords        $cursor(hn,v) $ix0 $iy1 $ix0 $iyn
            $w.c itemconfigure $cursor(hn,v) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(hn,v) -state hidden 
           }

        set xPPM [expr $ppmOffset(h_c_hn,HC) + $shift(ass,hc)]
        set roiInfo(h_c_hn,xPPM) $xPPM 

        if {[cursorIsInside h_c_hn vert $xPPM]} \
           { 
            set ix0  [spec2pix -roi 3 -cell 3 -x ${xPPM}ppm]
            set iy1  [lindex $cellInfo(h_c_hn) 2]
            set iyn  [lindex $cellInfo(h_c_hn) 3]

            $w.c coords        $cursor(h_c_hn,v) $ix0 $iy1 $ix0 $iyn
            $w.c itemconfigure $cursor(h_c_hn,v) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(h_c_hn,v) -state hidden 
           }
       }

   if {![strcasecmp $shift(ass,ch3) None]} \
       {
        $w.c itemconfigure $cursor(hn,h)     -state hidden
        $w.c itemconfigure $cursor(h_c_hn,h) -state hidden
        $w.c itemconfigure $cursor(hn_n_c,h) -state hidden
       } \
    else \
       {
        set yPPM [expr $ppmOffset(hn4d,CH3) + $shift(ass,ch3)]

        if {[cursorIsInside hn horiz $yPPM]} \
           {
            set iy0  [spec2pix -roi 1 -cell 1 -y ${yPPM}ppm]
            set ix1  [lindex $cellInfo(hn) 0]
            set ixn  [lindex $cellInfo(hn) 1]

            $w.c coords        $cursor(hn,h) $ix1 $iy0 $ixn $iy0
            $w.c itemconfigure $cursor(hn,h) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(hn,h) -state hidden 
           }

        set yPPM [expr $ppmOffset(h_c_hn,CH3) + $shift(ass,ch3)]
        set roiInfo(h_c_hn,yPPM) $yPPM 

        if {[cursorIsInside h_c_hn horiz $yPPM]} \
           { 
            set iy0  [spec2pix -roi 3 -cell 3 -y ${yPPM}ppm]

            set ix1 [lindex $cellInfo(h_c_hn) 0]
            set ixn [lindex $cellInfo(h_c_hn) 1]

            $w.c coords        $cursor(h_c_hn,h) $ix1 $iy0 $ixn $iy0
            $w.c itemconfigure $cursor(h_c_hn,h) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(h_c_hn,h) -state hidden 
           }

        set yPPM [expr $ppmOffset(hn_n_c,CH3) + $shift(ass,ch3)]

        if {[cursorIsInside hn_n_c horiz $yPPM]} \
           {
            set iy0  [spec2pix -roi 4 -cell 4 -y ${yPPM}ppm]

            set ix1 [lindex $cellInfo(hn_n_c) 0]
            set ixn [lindex $cellInfo(hn_n_c) 1]

            $w.c coords        $cursor(hn_n_c,h) $ix1 $iy0 $ixn $iy0
            $w.c itemconfigure $cursor(hn_n_c,h) -state normal
           } \
        else \
           {
            $w.c itemconfigure $cursor(hn_n_c,h) -state hidden 
           }
       }

   $w.c itemconfigure $labInfo(hn,text)     -text $labInfo(hn,str)
   $w.c itemconfigure $labInfo(hn_n_c,text) -text $labInfo(hn_n_c,str)

   $w.c itemconfigure $labInfo(h_c_hn,text) \
      -text "$labInfo(h_c_hn,str) Hi [format %.2e $hiInfo(h_c_hn)]"

   $w.c itemconfigure $labInfo(hc,text) \
      -text "$labInfo(hc,str) Hi [format %.2e $hiInfo(hn4d)]"
}

proc incrResID { jump } \
{
   global eMax assInfo

   nextID hnAss $jump
   selectByEntry h_c_hn $eMax(h_c_hn)
}

proc selectByEntryAndResID { i e resID } \
{
   global eID

   set eID(hnAss) $resID
 
   selectByResID
   selectByEntry $i $e
}

proc selectByEntry { i e } \
{
   global eLoc eID eList eCount varName shift

   if {!$e} return

   for {set loc 0} {$loc < $eCount($i)} {incr loc} \
       {
        set thisE [lindex $eList($i) $loc]

        if {$thisE == $e} \
           {
            set eLoc($i) $loc

            nextID $i 0
            updateAssInfo
            return
           }
      }
}

proc appendHN { e } \
{
   global assInfo eID resNames1 hiInfo

   if {!$e} return

   set resID   [gdbGet $e RESID]
   set resName $resNames1($resID)
   set ass     $resID$resName
   set chID    $eID(h_c_hn) 

   updateHNHi $e

   set resList [getAssResIDList $assInfo(hn)]
 
   if {[lsearch -exact $resList $resID] < 0} \
      {
       set aList [concat $ass=[format %.2e $hiInfo(hn)] $assInfo(hn)]
       set assInfo(hn) $aList

       set assInfo(saveNeeded) 1
       set assInfo(save,resID) $resID
       set assInfo(save,chID)  $chID

       updateSaveNeeded
      }
}

proc updateHNHi { e } \
{
   global ppmOffset hiInfo varName

   set xPPM [gdbGet $e $varName(hnAss,X_PPM)]
   set yPPM [gdbGet $e $varName(hnAss,Y_PPM)]

   set sx [expr $ppmOffset(hn4d,HN) + $xPPM]ppm
   set sy [expr $ppmOffset(hn4d,N)  + $yPPM]ppm

   readROI -roi 12 -inROI 2 -dx X_AXIS $sx 1 -dy Y_AXIS $sy 1 -ndim 2

   set maxVal [getROIStat 12 vMax]
   set minVal [getROIStat 12 vMin]

   if {abs($minVal) > abs($maxVal)} \
      {
       set hiInfo(hn) $minVal
      } \
   else \
      {
       set hiInfo(hn) $maxVal
      }
}

proc assignHC { e } \
{
   global assInfo eID resNames1 hiInfo

   if {!$e} return

   set resID   [gdbGet $e RESID]
   set resName $resNames1($resID)
   set ass     $resID$resName
   set chID    $eID(h_c_hn) 

   updateHNHi $e

   set assInfo($resID,$chID,hc) $ass
   set assInfo(hc) $ass=[format %.2e $hiInfo(hn)]

   set assInfo(saveNeeded) 1
   set assInfo(save,resID) $resID
   set assInfo(save,chID)  $chID

   updateSaveNeeded
}

proc selectByResID {} \
{
   global eLoc eID eList eCount varName eMax
  
   set i hnAss
 
   for {set loc 0} {$loc < $eCount($i)} {incr loc} \
      {
       set id [gdbGet [lindex $eList($i) $loc] $varName($i,INDEX)]

       if {$id == $eID($i)} \
          {
           set eLoc($i) $loc
           nextID $i 0
           selectByEntry h_c_hn $eMax(h_c_hn)
           updateAssInfo
           return
          }
      }
}

proc updateAssInfo {} \
{
   global assInfo eList eLoc eID

   set assInfo(hn)   ""
   set assInfo(hc)   ""
   set assInfo(note) ""

   set e [lindex $eList(hnAss) $eLoc(hnAss)]

   set resID [gdbGet $e RESID]
   set chID  $eID(h_c_hn)

   if {[info exists assInfo($resID,$chID,hn)]} \
      {
       set assInfo(hn) $assInfo($resID,$chID,hn)
      }

   if {[info exists assInfo($resID,$chID,hc)]} \
      {
       set assInfo(hc) $assInfo($resID,$chID,hc)
      }

   if {[info exists assInfo($resID,$chID,note)]} \
      {
       set assInfo(note) $assInfo($resID,$chID,note)
      }  

   set assInfo(saveNeeded) 0
   set assInfo(save,resID) $resID 
   set assInfo(save,chID)  $chID 

   showStatus "Updating HC Labels."
   createHCLabels

   showStatus "Ready."
   
   updateSaveNeeded

}

proc nextID { i jump } \
{
   global eList eLoc eCount shift varName assInfo hiInfo

   incr eLoc($i) $jump

   if {$eLoc($i) < 0} {set eLoc($i) [expr $eCount($i) - 1]}
   if {$eLoc($i) > $eCount($i) - 1} {set eLoc($i) 0}

   set e [lindex $eList($i) $eLoc($i)]

   if {![strcmp $i hnAss]} \
      {
       set shift(ass,hn) [gdbGet $e $varName($i,X_PPM)]
       set shift(ass,n)  [gdbGet $e $varName($i,Y_PPM)]
      } \
   else \
      {
       set shift(ass,hc)  [gdbGet $e $varName($i,X_PPM)]
       set shift(ass,ch3) [gdbGet $e $varName($i,Y_PPM)]

       if {![strcasecmp $i h_c_hn]} \
          {
           set assInfo(chID)  [gdbGet $e $varName($i,INDEX)]
           set hiInfo(h_c_hn) [gdbGet $e HEIGHT]
          }
      }

   drawCurrentPlane $i
}

proc selectThisPPM { type } \
{
    global shift eID

    if {![strcasecmp $type HN]} \
       {
        set eID(hnAss)    -666
        drawPlaneByPPM hn $shift(ass,hn) $shift(ass,n) 
       } \
    elseif {![strcasecmp $type N]} \
       {
        set eID(hnAss)    -666
        drawPlaneByPPM hn $shift(ass,hn) $shift(ass,n) 
       } \
    elseif {![strcasecmp $type HC]} \
       {
        set eID(h_c_hn)   -666
        drawPlaneByPPM hc $shift(ass,hc) $shift(ass,ch3) 
       } \
    elseif {![strcasecmp $type CH3]} \
       {
        set eID(h_c_hn)   -666
        drawPlaneByPPM hc $shift(ass,hc) $shift(ass,ch3) 
       }
}

proc findCellByPix { xPix yPix }  \
{
   global cellInfo 

   foreach i {hn hc h_c_hn hn_n_c} \
      {
       set cx1 [lindex $cellInfo($i) 0]
       set cxn [lindex $cellInfo($i) 1]
       set cy1 [lindex $cellInfo($i) 2]
       set cyn [lindex $cellInfo($i) 3]

       if {$xPix < $cx1 || $xPix > $cxn} continue
       if {$yPix < $cy1 || $yPix > $cyn} continue

       return $i
      }

   return None 
}

proc selectByPPM { xPix yPix } \
{
   global cellInfo shift eID hiInfo ppmOffset

   set i [findCellByPix $xPix $yPix]
   if {![strcasecmp $i None]} return

   set id [lindex $cellInfo($i) end]

   set xPPM \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 -x $xPix -units ppm]]

   set yPPM \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 -y $yPix -units ppm]]

   if {![strcasecmp $i hn]} \
      {
       set shift(ass,hc)  [expr $xPPM - $ppmOffset(hn4d,HC)] 
       set shift(ass,ch3) [expr $yPPM - $ppmOffset(hn4d,CH3)]

       drawPlaneByPPM hc $xPPM $yPPM
      } \
   elseif {![strcasecmp $i h_c_hn]} \
      {
       set shift(ass,hc)  [expr $xPPM - $ppmOffset(h_c_hn,HC)]
       set shift(ass,ch3) [expr $yPPM - $ppmOffset(h_c_hn,CH3)]
       set eID(h_c_hn)    -666
 
       readROI -roi 13 -inROI 3 -ndim 2 \
               -dx X_AXIS ${xPPM}ppm 1 -dy Y_AXIS ${yPPM}ppm 1 

       set maxVal [getROIStat 13 vMax]
       set minVal [getROIStat 13 vMin]

       if {abs($minVal) > abs($maxVal)} \
          {
           set hiInfo(h_c_hn) $minVal
          } \
       else \
          {
           set hiInfo(h_c_hn) $maxVal
          }

       drawPlaneByPPM hc $xPPM $yPPM
      } \
   elseif {![strcasecmp $i hc]} \
      {
       set shift(ass,hn) [expr $xPPM - $ppmOffset(hn4d,HN)]
       set shift(ass,n)  [expr $yPPM - $ppmOffset(hn4d,N)]
       set eID(hnAss)    -666

       drawPlaneByPPM hn $shift(ass,hn) $shift(ass,n)
      } \
   elseif {![strcasecmp $i hn_n_c]} \
      {
       set shift(ass,ch3)  [expr $yPPM - $ppmOffset(hn_n_c,CH3)]

       set xPPM [expr $shift(ass,hc) + ppmOffset(h_c_hn,HC)]
       drawPlaneByPPM hc $xPPM $yPPM
      }
}

proc initTabInfo {} \
{ 
   global tabName inTab eList eCount eLoc eID varName seqList skipList r1 rN
   global resNames1 resNames3 assInfo outName

   foreach i {hnAss h_c_hn} \
      {
       set inTab($i)  [gdbRead table -in $tabName($i)]
       set eList($i)  [gdbGetInfo $inTab($i) -parms entries]
       set eCount($i) [llength $eList($i)]
       set eLoc($i)   0

       set eID($i) \
        [gdbGet [lindex $eList($i) $eLoc($i)] $varName($i,INDEX)]
      }

   set seqList  [getSeqList $inTab(hnAss) r1 rN]
   set skipList ""
   set resList  ""

   foreach e $eList(hnAss) \
      {
       set resID    [gdbGet $e RESID]
       set resName3 [gdbGet $e RESNAME]

       lappend resList $resID

       if {![strcasecmp $resName3 None]} \
          {
           set resNames1($resID) X
           set resNames3($resID) X
          } \
       else \
          {
           set resNames1($resID) [getResName1 $resName3]
           set resNames3($resID) $resName3
          }
      }

   for {set resID  $r1} {$resID <= $rN} {incr resID} \
      {
       if {[lsearch -exact $resList $resID] < 0} {lappend skipList $resID} 
      }

   if {[file exists $outName(assInfo)]} \
      {
       puts "Restoring assInfo Array ..."

       set nn [loadArray $outName(assInfo) assInfo]
      }

}

proc resSelect {} \
{
   global skipList seqList r1 rN eID

   set w .t

   if {![llength $seqList]} return

   if {[winfo exists .selectResID]} \
      {
       wm deiconify .selectResID
       raise .selectResID
       update

       set sList [wm stackorder .]
       set sLoc  [lsearch -exact $sList .selectResID]
       set wLoc  [lsearch -exact $sList $w]

       if {$sLoc <= $wLoc} {destroy .selectResID}
      }

   createResSelectBox -win .selectResID -hilite $eID(hnAss) \
                      -skip $skipList -seq $seqList -r1 $r1 -rN $rN -methyl \
                      -command selectResID -raise
  
   
   update
}

proc resSelectUpdate {} \
{
   global skipList seqList r1 rN eID

   if {![llength $seqList]} return

   createResSelectBox -config -hilite $eID(hnAss) -win .selectResID \
                      -skip $skipList -seq $seqList -r1 $r1 -rN $rN -methyl \
                      -command selectResID 
}

proc selectResID { w resID } \
{
   global eID skipList

   if {[lsearch -exact $skipList $resID] >= 0} return

   set eID(hnAss) $resID

   selectByResID
}

proc badAss { resID resName } \
{
   global r1 rN

   if {$resID < $r1 || $resID > $rN} {return 1}

   if {![strcasecmp $resName None]}  {return 1}
   if {![strcasecmp $resName XXX]}   {return 1}
   if {![strcasecmp $resName X]}     {return 1}
   if {![strcasecmp $resName *]}     {return 1}

   return 0
}

proc toggleCursors { } \
{
   global cursorFlag

   set w .t

   if {$cursorFlag} \
      {
       $w.c itemconfigure cursor -state normal
      } \
   else \
      {
       $w.c itemconfigure cursor -state hidden
      }

   update
}

proc toggleBoxes { t } \
{
   global boxFlag 

   set w .t

   if {[info exists boxFlag($t)]} \
      {
       set thisFlag $boxFlag($t) 
      } \
   else \
      {
       set thisFlag 0
      }

   if {$thisFlag} \
      {
       $w.c itemconfigure ${t}Box -state normal
      } \
   else \
      {
       $w.c itemconfigure ${t}Box -state hidden
      }

   update
}

proc parseKey { kSym xPix yPix } \
{
   global boxFlag cursorFlag hi

   set i [findCellByPix $xPix $yPix]
   if {![strcasecmp $i None]} return

   switch -exact $kSym \
      {
       b {
          if {$boxFlag(ass) || $boxFlag(none) || \
                  $boxFlag(hc) || $boxFlag(hnhc) || $boxFlag(hnhcCur)} \
             {
              foreach b {hc ass none hnhc hnhcCur} \
                 {
                  set boxFlag($b,prev) $boxFlag($b) 
                  set boxFlag($b) 0
                 }
             } \
          else \
             {
              foreach b {hc ass none hnhc hnhcCur} \
                 {
                  set boxFlag($b) $boxFlag($b,prev) 
                 }
             }

          foreach b {hc ass none hnhc hnhcCur} {toggleBoxes $b} 
         }

       c {
          if {$cursorFlag} \
             {
              set cursorFlag 0
             } \
          else \
             {
              set cursorFlag 1
             }

          toggleCursors
         }

       z {
          autoZoom
         }

       f {
          full2D
         }

       minus {adjCont $i [expr 1.0/$hi(factor)]} 

       equal -
       plus  {adjCont $i [expr $hi(factor)]}
      }
}

proc adjCont { i f } \
{
   global hi

   if {![info exists hi($i)]} return

   set hi($i) [expr $hi($i)*$f]

   drawCurrentPlane h_c_hn
   drawCurrentPlane hnAss
}

proc showAssText { showFlag bID tID } \
{
   set w .t

   if {$showFlag} \
      {
       $w.c itemconfigure $tID -state normal
      } \
   else \
      {
       $w.c itemconfigure $tID -state hidden
      }
}

proc insertHC {} \
{
   global hiInfo roiInfo tabName inTab eList eCount eLoc eID varName

   set i h_c_hn

   if {![strcasecmp $roiInfo($i,xPPM) None]} return
   if {![strcasecmp $roiInfo($i,yPPM) None]} return
   if {![strcasecmp $roiInfo($i,zPPM) None]} return

   set vList [gdbGetInfo $inTab($i) -parms varNames]
   set  id   [getTabMax $inTab($i) INDEX]
   incr id

   set eSrc  [lindex $eList($i) 0]
   set eDest [gdbCreate entry -parent $inTab($i)]

   foreach v $vList {gdbSet $eDest $v [gdbGet $eSrc $v]}

   gdbSet $eDest INDEX $id
   gdbSet $eDest ASS   inserted

   gdbSet $eDest $varName($i,X_PPM) $roiInfo($i,xPPM)
   gdbSet $eDest $varName($i,Y_PPM) $roiInfo($i,yPPM)
   gdbSet $eDest $varName($i,Z_PPM) $roiInfo($i,zPPM)
  
   gdbSet $eDest HEIGHT $hiInfo(h_c_hn) 

   gdbWrite $inTab($i) $tabName($i)
 
   gdbFree $inTab($i)

   set inTab($i)  [gdbRead table -in $tabName($i)]
   set eList($i)  [gdbGetInfo $inTab($i) -parms entries]
   set eCount($i) [llength $eList($i)]
   set eLoc($i)   [expr $eCount($i) - 1] 

   set e          [lindex $eList($i) $eLoc($i)]
   set eID($i)    [gdbGet $e INDEX]

   selectByEntry $i $e 
}

proc recordAss {} \
{
   global eID

   set resID $eID(hnAss)
   set chID  $eID(h_c_hn)

   recordAss2 $resID $chID
}

proc recordAss2 { resID chID } \
{
   global eID eList assInfo tabName resNames1 resNames3 outName 

   if {![info exists resNames1($resID)]} return

   if {[lsearch -exact $assInfo(idList) $resID,$chID] < 0} \
      {
       lappend assInfo(idList) $resID,$chID
      }

   set resName $resNames1($resID)
   set ass     $resID$resName

   if {![llength $assInfo(hn)]}   {set assInfo(hn) None}
   if {![llength $assInfo(hc)]}   {set assInfo(hc) None}
   if {![llength $assInfo(note)]} {set assInfo(note) None}

   set assInfo($resID,$chID,hn)   $assInfo(hn)
   set assInfo($resID,$chID,hc)   $assInfo(hc)
   set assInfo($resID,$chID,note) $assInfo(note)

   recordAss3 $resID $chID
}

proc recordAss3 { resID chID } \
{
   global eID eList assInfo tabName resNames1 resNames3 outName 

   showStatus "Updating Assignments."

   set error [catch {set outUnit [open $outName(hn4d) w]}]

   if {$error} \
      {
       showError p4d "Error Writing Assignments to File $outName(hn4d)"
       showStatus "Assignment Write Error."
       return
      }

   set assInfo(saveNeeded) 0
   set assInfo(save,resID) $resID 
   set assInfo(save,chID)  $chID

   set n   0
   set eHN 0
   set eCH 0

   foreach i $assInfo(idList) \
      {
       set iList [split $i ,]
       set resID [lindex $iList 0]
       set chID  [lindex $iList 1]

       incr n

       puts $outUnit "Assignment: $n Residue: $resID $resNames3($resID) [getXYShift hnAss $resID eHN] CH ID: $chID [getXYShift h_c_hn $chID eCH] Related HN: $assInfo($i,hn) Suggested Assignment: $assInfo($i,hc) Notes: $assInfo($i,note)"
      }

   close $outUnit

   showStatus "Storing Assignments."

   set nn [storeArray $outName(assInfo) assInfo]
   updateSaveNeeded

   showStatus "Updating HNHC Labels."
   createHNHCLabels

   showStatus "Ready."
}

proc clearAss {} \
{
   global assInfo eID

   set loc \
    [tk_dialog .clearDialog Clear \
      "Clear HN/HC Assignment?" question 1 Clear Cancel]

   if {$loc} return

   set assInfo(hc)   ""
   set assInfo(hn)   ""
   set assInfo(note) ""

   set resID $eID(hnAss)
   set chID  $eID(h_c_hn)

   set assInfo($resID,$chID,hn)   $assInfo(hn)
   set assInfo($resID,$chID,hc)   $assInfo(hc)
   set assInfo($resID,$chID,note) $assInfo(note)

   set aList ""

   foreach a $assInfo(idList) \
      {
       if {[strcmp $a $resID,$chID]} {lappend aList $a}
      } 

   set assInfo(idList) $aList

   recordAss3 $resID $chID

   showStatus "Updating HNHC Labels."
   createHNHCLabels

   showStatus "Ready."
}

proc getXYShift { i id ePtr } \
{
   global inTab varName

   upvar $ePtr e

   set sList [gdbSelect -from $inTab($i) -cond "$varName($i,INDEX) == $id"]
   set n [llength $sList]
   set e 0

   if {!$n} {return "9999.000 9999.000 0 0"}

   if {$n != 1} \
      {
       showError p4d \
        "Error: ID $resID is listed more than once in $tabName($i)"
      }

   set e   [lindex $sList 0]
   set fmt " %9.3f %9.3f"

   set str \
    [format $fmt [gdbGet $e $varName($i,X_PPM)] [gdbGet $e $varName($i,Y_PPM)]]
 
   if {![strcasecmp $i h_c_hn]} {append str [format " % .2e" [gdbGet $e HEIGHT]]}

   return $str 
}

proc updateAssEntry {} \
{
   global assInfo

   set assInfo(saveNeeded) 1
   updateSaveNeeded
}

proc updateSaveNeeded {} \
{
   global assInfo

   set w .t

   if {$assInfo(saveNeeded)} \
      {
       $w.recordAss configure -foreground red
      } \
   else \
      {
       $w.recordAss configure -foreground black 
      }
}

proc full2D {} \
{
    global zoomInfo labInfo hi

    foreach i {hn hc hn_n_c h_c_hn} \
       {
        set zoomInfo($i,x1)   0%
        set zoomInfo($i,xn) 100%
        set zoomInfo($i,y1)   0%
        set zoomInfo($i,yn) 100%
 
        set hi($i) $hi($i,default)
       }

    set labInfo(hn,drawBoxes) 1

    drawCurrentPlane hnAss
    drawCurrentPlane h_c_hn
}

proc zoomOFF {} \
{
   global zoomInfo

   set w .t

   set zoomInfo(cell)  None

   set zoomInfo(x1Pix) 0
   set zoomInfo(xnPix) 0
   set zoomInfo(y1Pix) 0
   set zoomInfo(ynPix) 0

   $w.c delete zoomBox
}

proc zoomON { xPix yPix } \
{
   global zoomInfo

   set w .t

   zoomOFF

   set zoomInfo(cell) [findCellByPix $xPix $yPix]
   if {![strcasecmp $zoomInfo(cell) None]} return

   set zoomInfo(x1Pix) $xPix 
   set zoomInfo(xnPix) $xPix 
   set zoomInfo(y1Pix) $yPix 
   set zoomInfo(ynPix) $yPix 
 
   $w.c create rectangle $zoomInfo(x1Pix) $zoomInfo(y1Pix) \
                       $zoomInfo(xnPix) $zoomInfo(ynPix) \
                       -outline white -tags zoomBox
}

proc trackZoom { xPix yPix } \
{
   global zoomInfo

   set w .t

   if {![strcasecmp $zoomInfo(cell) None]} return

   set zoomInfo(xnPix) $xPix 
   set zoomInfo(ynPix) $yPix 

    if {$zoomInfo(x1Pix) > $zoomInfo(xnPix)} \
       {
        set itemp $zoomInfo(x1Pix)
        set zoomInfo(x1Pix) $zoomInfo(xnPix)
        set zoomInfo(xnPix) $itemp
       }

    if {$zoomInfo(y1Pix) > $zoomInfo(ynPix)} \
       {
        set itemp $zoomInfo(y1Pix)
        set zoomInfo(y1Pix) $zoomInfo(ynPix)
        set zoomInfo(ynPix) $itemp
       }
 
   $w.c coords zoomBox $zoomInfo(x1Pix) $zoomInfo(y1Pix) \
                     $zoomInfo(xnPix) $zoomInfo(ynPix) 
}

proc zoom2D { xPix yPix } \
{
    global zoomInfo roiInfo labInfo

    set w .t

    $w.c delete zoomBox

    set cell $zoomInfo(cell)
    set id   $roiInfo($cell,roiID)

    set zoomInfo(cell) None

    if {$zoomInfo(x1Pix) > $zoomInfo(xnPix)} \
       {
        set itemp $zoomInfo(x1Pix)
        set zoomInfo(x1Pix) $$zoomInfo(xnPix)
        set zoomInfo(xnPix) $itemp
       }

    if {$zoomInfo(y1Pix) > $zoomInfo(ynPix)} \
       {
        set itemp $zoomInfo(y1Pix)
        set zoomInfo(y1Pix) $$zoomInfo(ynPix)
        set zoomInfo(ynPix) $itemp
       }

   set zoomInfo($cell,x1) \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 \
            -x $zoomInfo(x1Pix) -units ppm]]ppm

   set zoomInfo($cell,xn) \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 \
            -x $zoomInfo(xnPix) -units ppm]]ppm

   set zoomInfo($cell,y1) \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 \
            -y $zoomInfo(ynPix) -units ppm]]ppm

   set zoomInfo($cell,yn) \
    [format %.3f [pix2spec -cell $id -roi $id -win 1 \
            -y $zoomInfo(y1Pix) -units ppm]]ppm

    set labInfo(hn,drawBoxes) 1

    drawCurrentPlane hnAss
    drawCurrentPlane h_c_hn
}

proc autoZoom {} \
{
   global zoomInfo roiInfo labInfo shift ppmOffset

    foreach i {hn n hc ch3} \
       {
        if {![strcasecmp $shift(ass,$i) None]} return 
       }

    set hnPos   [expr $ppmOffset(hn4d,HN)  + $shift(ass,hn)]
    set nPos    [expr $ppmOffset(hn4d,N)   + $shift(ass,n)]
    set hcPos   [expr $ppmOffset(hn4d,HC)  + $shift(ass,hc)]
    set ch3Pos  [expr $ppmOffset(hn4d,CH3) + $shift(ass,ch3)]

    set zoomInfo(hc,x1) [expr $hnPos + $zoomInfo(hn,zoom)]ppm
    set zoomInfo(hc,xn) [expr $hnPos - $zoomInfo(hn,zoom)]ppm
    set zoomInfo(hc,y1) [expr $nPos  + $zoomInfo(n,zoom)]ppm
    set zoomInfo(hc,yn) [expr $nPos  - $zoomInfo(n,zoom)]ppm

    set zoomInfo(hn,x1) [expr $hcPos  + $zoomInfo(hc,zoom)]ppm
    set zoomInfo(hn,xn) [expr $hcPos  - $zoomInfo(hc,zoom)]ppm
    set zoomInfo(hn,y1) [expr $ch3Pos + $zoomInfo(ch3,zoom)]ppm
    set zoomInfo(hn,yn) [expr $ch3Pos - $zoomInfo(ch3,zoom)]ppm

    set hcPos   [expr $ppmOffset(h_c_hn,HC)  + $shift(ass,hc)]
    set ch3Pos  [expr $ppmOffset(h_c_hn,CH3) + $shift(ass,ch3)]

    set zoomInfo(h_c_hn,x1) [expr $hcPos  + $zoomInfo(hc,zoom)]ppm
    set zoomInfo(h_c_hn,xn) [expr $hcPos  - $zoomInfo(hc,zoom)]ppm
    set zoomInfo(h_c_hn,y1) [expr $ch3Pos + $zoomInfo(ch3,zoom)]ppm
    set zoomInfo(h_c_hn,yn) [expr $ch3Pos - $zoomInfo(ch3,zoom)]ppm

    set hnPos   [expr $ppmOffset(hn_n_c,HN)  + $shift(ass,hn)]
    set ch3Pos  [expr $ppmOffset(hn_n_c,CH3) + $shift(ass,ch3)]

    set zoomInfo(hn_n_c,x1) [expr $hnPos  + $zoomInfo(hn,zoom)]ppm
    set zoomInfo(hn_n_c,xn) [expr $hnPos  - $zoomInfo(hn,zoom)]ppm
    set zoomInfo(hn_n_c,y1) [expr $ch3Pos + $zoomInfo(ch3,zoom)]ppm
    set zoomInfo(hn_n_c,yn) [expr $ch3Pos - $zoomInfo(ch3,zoom)]ppm

    set labInfo(hn,drawBoxes) 1

    drawCurrentPlane hnAss
    drawCurrentPlane h_c_hn
}

proc clearCell { i } \
{
   global cellInfo

   drawRec -noupdate -fg gray30 \
           -x $cellInfo($i,cx1) $cellInfo($i,cxn) \
           -y $cellInfo($i,cy1) $cellInfo($i,cyn)
}

proc showHelp {} \
{
   set tInfo \
    {
     ""
     " +----------------------+  +----------------------+"
     " |     4D HC Window     |  |    4D HN Window      |"
     " |   HC x CH3 at HN,N   |  |  HN x N at HC,CH3    |"
     " |                      |  |                      |"
     " | Boxes show HC,CH3    |  |  Boxes show known    |"
     " | positions that have  |  |  HN/N assignments;   |"
     " | been associated with |  |  assignments are     |"
     " | specific residues.   |  |  fixed.              |"
     " +----------------------+  +----------------------+"
     ""
     " +----------------------+  +----------------------+"
     " |    3D HC Window      |  |    3D HN/C Window    |"
     " |   HC x CH3 at HN     |  |     HN x CH3 at N    |"
     " |                      |  |                      |"
     " | Boxes show peaks in  |  |                      |"
     " | 3D peak table; New   |  |                      |"
     " | peaks can be added.  |  |                      |"
     " +----------------------+  +----------------------+"
     ""
     " 2D ZOOM DISPLAY and CONTOURS:"
     "  Use CTRL-LEFT to Zoom -- press and HOLD the keyboard"
     "  CTRL key while dragging with the LEFT mouse button to"
     "  a select a zoom region in any of the four windows."
     "  When you release the mouse button, the selected region"
     "  will be expanded. Be sure to continue pressing the"
     "  keyboard CTRL key until the mouse button is released."
     ""
     "  Move the mouse pointer to one of the four windows,"
     "  and use the keyboard - (MINUS) key to reduce the"
     "  the contour heights in that window. Use the + (PLUS)"
     "  or = (EQUAL) key to increase contour levels."
     ""
     "  Use the keyboard Z key to automatically adjust all"
     "  zoom regions around the current HN,N,HC,CH3 coords."
     ""
     "  Use the keyboard F key to return to the full"
     "  view in all windows simultaneously. This will also"
     "  reset all contour heights to their default values."
     ""
     "BOX LABEL AND CURSOR DISPLAY:"
     "  Use the keyboard B key to toggle box display on and off."
     "  Use the keyboard C key to toggle cursor display on and off."
     ""
     "NAVIGATION:"
     "  Clicking on a box label with the LEFT mouse button will"
     "  will adjust the views in the other windows accordingly." 
     "  It will also update information in the command panel."
     ""
     "  Clicking on any spectral position with SHIFT-LEFT (keyboard"
     "  SHIFT key and mouse LEFT button) will move the views and"
     "  cursors in the other windows to the selected chemical"
     "  shift position."
     ""
     "ASSIGNMENT:"
     " LEFT click on a peak label box in the 3D HC WINDOW."
     " Then use the RECORD HNHC button in the command panel"
     " to associate this HC,CH3 postion as an NOE to the"
     " current HN,N residue assignment."
     ""
     " RIGHT click on an assignment box in the 4D HN WINDOW"
     " to add the given residue to the list of HN,N assignments"
     " that seem to have an NOE to the current HC,CH3."
     ""
     " SHIFT-RIGHT click on an assignment box in the 4D HN WINDOW"
     " to record a specific residue assignment for the current HC,CH3"
    }

   catch "destroy .helpWin"

   set h .helpWin

   toplevel $h
   frame    $h.top

   text $h.tHelpNote \
    -yscrollcommand "$h.sHelpNote set" -relief flat -borderwidth 3 \
    -height 24 -width 80 -bg green -fg black

   scrollbar $h.sHelpNote -command "$h.tHelpNote yview" -orient vertical

   button $h.sHelpClose -text Close -command "destroy $h"

   pack $h.tHelpNote  -in $h.top -side left -fill both -expand 1 -padx 8 -pady 8
   pack $h.sHelpNote  -in $h.top -side left -fill y    -expand 1

   pack $h.top        -in $h -side top -fill both -expand 1
   pack $h.sHelpClose -in $h -side top -fill x    -expand 1

   foreach t $tInfo {$h.tHelpNote insert end "$t\n"}

   update
}

proc showStatus { s } \
{
   global w labInfo

   $w.c itemconfigure $labInfo(status,text) -text $s
   update
}

initResNames
initTabInfo

frame  $w.cmndA
frame  $w.cmndB

canvas $w.c -width $xSize -height $ySize -border 0 -closeenough 4

label  $w.plabRes -text "ResID:"
entry  $w.pentRes -width 5 -textvariable eID(hnAss)
button $w.nextRes -text Next -command "incrResID  1"
button $w.prevRes -text Prev -command "incrResID -1"
button $w.resSel  -bitmap @$env(NMRTXT)/caret.map -command resSelect

label  $w.slabHN -text "HN:"
entry  $w.sentHN -width 7 -textvariable shift(ass,hn)

label  $w.slabN  -text "N:"
entry  $w.sentN  -width 7 -textvariable shift(ass,n)

label  $w.slabHC -text "HC:"
entry  $w.sentHC -width 7 -textvariable shift(ass,hc)

label  $w.slabCH3 -text "CH3:"
entry  $w.sentCH3 -width 7 -textvariable shift(ass,ch3)

label  $w.slabCHID -text "CH ID:"
label  $w.sentCHID -width 5 -textvariable eID(h_c_hn)

label  $w.alabHC  -text "HC ASS:"
entry  $w.aentHC  -width 7 -textvariable   assInfo(hc)

label  $w.alabRel  -text "Related HN:"
entry  $w.aentRel  -width 16 -textvariable assInfo(hn)

label  $w.alabNote -text "Note:"
entry  $w.aentNote -width 36 -textvariable assInfo(note)

checkbutton $w.chAssBox -text "HN ASS" -anchor e \
             -onvalue 1 -offvalue 0 \
            -selectcolor green -command "toggleBoxes ass" \
            -variable boxFlag(ass)

checkbutton $w.chNoneBox -text "HN NONE" -anchor e \
            -onvalue 1 -offvalue 0 \
            -selectcolor red -command "toggleBoxes none" \
            -variable boxFlag(none)

checkbutton $w.chHCBox -text "HC" -anchor e -onvalue 1 -offvalue 0 \
            -selectcolor yellow -command "toggleBoxes hc"        \
            -variable boxFlag(hc)

checkbutton $w.chHNHCAllBox -text "HNHC" -anchor e -onvalue 1 \
            -offvalue 0 -selectcolor magenta \
            -command "toggleBoxes hnhc"      \
            -variable boxFlag(hnhc)

checkbutton $w.chHNHCCurBox -text "HNHC Current" -anchor e -onvalue 1 \
            -offvalue 0 -selectcolor white \
            -command "toggleBoxes hnhcCur" \
            -variable boxFlag(hnhcCur)

checkbutton $w.chHNRelBox -text "HN Related" -anchor e \
            -onvalue 1 -offvalue 0 \
            -selectcolor magenta -command "toggleBoxes hnCur" \
            -variable boxFlag(hnCur)

checkbutton $w.chCursor -text "Cursors" -anchor e -onvalue 1 -offvalue 0 \
            -selectcolor pink -command "toggleCursors" -variable cursorFlag

button $w.insertHC  -text "Insert HC"   -command insertHC 
button $w.recordAss -text "Record HNHC" -command recordAss 
button $w.clearAss  -text "Clear HNHC"  -command clearAss 

button $w.help     -text "Help" -command showHelp 
button $w.quit     -text "Exit" -command exit
 
pack   $w.plabRes $w.pentRes $w.resSel $w.nextRes $w.prevRes \
       -in $w.cmndA -side left

pack   [getBlank $w 1] -in $w.cmndA -side left
pack   $w.slabHN $w.sentHN -in $w.cmndA -side left
pack   [getBlank $w 1] -in $w.cmndA -side left
pack   $w.slabN  $w.sentN  -in $w.cmndA -side left
pack   [getBlank $w 1] -in $w.cmndA -side left
pack   $w.slabHC $w.sentHC -in $w.cmndA -side left
pack   [getBlank $w 1] -in $w.cmndA -side left

pack   $w.slabCH3 $w.sentCH3 [getBlank $w 1] $w.slabCHID $w.sentCHID \
       -in $w.cmndA -side left

pack   [getBlank $w 1] $w.insertHC [getBlank $w 1] \
       $w.chAssBox $w.chNoneBox $w.chHCBox $w.chHNHCAllBox $w.chHNHCCurBox $w.chHNRelBox \
       $w.chCursor -in $w.cmndA -side left

pack   $w.recordAss $w.clearAss [getBlank $w 2]  \
       $w.alabHC $w.aentHC [getBlank $w 1] $w.alabRel $w.aentRel [getBlank $w 1] \
       $w.alabNote $w.aentNote -in $w.cmndB -side left

pack   $w.help -in $w.cmndA -side right
pack   $w.quit -in $w.cmndB -side right

pack   $w.c    -in $w -side top -expand 1 -fill both
pack   $w.cmndA $w.cmndB -in $w -side top -expand 1 -fill x

bind $w.pentRes <Return> "+ selectByResID"

bind $w.sentHN  <Return> "+ selectThisPPM hn"
bind $w.sentN   <Return> "+ selectThisPPM n"
bind $w.sentHC  <Return> "+ selectThisPPM hc"
bind $w.sentCH3 <Return> "+ selectThisPPM ch3"

bind $w.c <Shift-ButtonRelease-1>   "selectByPPM %x %y"
bind $w.c <Control-ButtonPress-1>   "zoomON %x %y"
bind $w.c <Control-B1-Motion>       "trackZoom %x %y"
bind $w.c <Control-ButtonRelease-1> "zoom2D %x %y"
bind $w.c <Any-Enter>               "focus $w.c"

bind $w <KeyPress>   "+ parseKey %K %x %y"

bind $w.aentHC   <KeyPress> "+ updateAssEntry"
bind $w.aentRel  <KeyPress> "+ updateAssEntry"
bind $w.aentNote <KeyPress> "+ updateAssEntry"
 
update
 
$w.c create pixmap 0 0 -width $xSize -height $ySize -win 1

set cxn [expr $cx1 + $dx]
set cyn [expr $cy1 + $dy]
set cx0 [expr $cx1 + $cellX]
set cy0 [expr $cy1 + $cellY]

set cellInfo(hn,cx1)  1
set cellInfo(hn,cxn)  [expr $cxn + $yAxisAdj - 1]
set cellInfo(hn,cy1)  1
set cellInfo(hn,cyn)  [expr $cyn + $xAxisAdj - 1]  

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 1 -verb -win 1

set cursor(hn,v) \
 [$w.c create line $cx1 $cy1 $cx1 $cyn -fill pink -tags "cursor hn_v"]

set cursor(hn,h) \
 [$w.c create line $cx1 $cy1 $cxn $cy1 -fill pink -tags "cursor hn_h"]

set labInfo(hn,text)  \
 [$w.c create text $cx0 $cy0 -anchor nw -fill white -font $stdfont -text hn]

set cellInfo(hn) "$cx1 $cxn $cy1 $cyn 1"

set cx1 [expr $cxn + $yAxisAdj]
set cxn [expr $cx1 + $dx]
set cx0 [expr $cx1 + $cellX]
set cy0 [expr $cy1 + $cellY]

set cellInfo(hc,cx1)  [expr $cx1 - 1] 
set cellInfo(hc,cxn)  [expr $xSize - 1]
set cellInfo(hc,cy1)  1
set cellInfo(hc,cyn)  [expr $cyn + $xAxisAdj - 1]

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 2 -verb -win 1

set cursor(hc,v) \
 [$w.c create line $cx1 $cy1 $cx1 $cyn -fill green -tags "cursor hc_v"]

set cursor(hc,h) \
 [$w.c create line $cx1 $cy1 $cxn $cy1 -fill green -tags "cursor hc_h"]

set labInfo(hc,text)  \
 [$w.c create text $cx0 $cy0 -anchor nw -fill white -font $stdfont -text hc]

set cellInfo(hc) "$cx1 $cxn $cy1 $cyn 2"

set cx1 10
set cxn [expr $cx1 + $dx]
set cy1 [expr $cyn + $xAxisAdj]
set cyn [expr $cy1 + $dy]
set cx0 [expr $cx1 + $cellX]
set cy0 [expr $cy1 + $cellY]

set cellInfo(h_c_hn,cx1)  1
set cellInfo(h_c_hn,cxn)  [expr $cxn + $yAxisAdj - 1]
set cellInfo(h_c_hn,cy1)  [expr $cy1 - 1] 
set cellInfo(h_c_hn,cyn)  [expr $ySize - 1]

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 3 -verb -win 1

set cursor(h_c_hn,v) \
 [$w.c create line $cx1 $cy1 $cx1 $cyn -fill pink -tags "cursor h_c_hn_v"]

set cursor(h_c_hn,h) \
 [$w.c create line $cx1 $cy1 $cxn $cy1 -fill pink -tags "cursor h_c_hn_h"]

set labInfo(h_c_hn,text)  \
 [$w.c create text $cx0 $cy0 -anchor nw -fill white -font $stdfont -text h_c_hn]

set cellInfo(h_c_hn) "$cx1 $cxn $cy1 $cyn 3"

set cx1 [expr $cxn + $yAxisAdj]
set cxn [expr $cx1 + $dx]
set cx0 [expr $cx1 + $cellX]
set cy0 [expr $cy1 + $cellY]

set cellInfo(hn_n_c,cx1)  [expr $cx1 - 1]
set cellInfo(hn_n_c,cxn)  [expr $xSize - 1]
set cellInfo(hn_n_c,cy1)  [expr $cy1 - 1] 
set cellInfo(hn_n_c,cyn)  [expr $ySize - 1]

defineCell -x $cx1 $cxn -y $cy1 $cyn -cell 4 -verb -win 1

drawRec -x 0% 100% -y 0% 100% -fg gray30

set cursor(hn_n_c,v) \
 [$w.c create line $cx1 $cy1 $cx1 $cyn -fill green -tags "cursor hn_n_c_v"]

set cursor(hn_n_c,h) \
 [$w.c create line $cx1 $cy1 $cxn $cy1 -fill pink -tags "cursor hn_n_c_h"]

set labInfo(hn_n_c,text)  \
 [$w.c create text $cx0 $cy0 -anchor nw -fill white -font $stdfont -text hn_n_c]

set cellInfo(hn_n_c) "$cx1 $cxn $cy1 $cyn 4"

set cx1 $cellX
set cy1 [expr $cyn + 2*$cellY]

set labInfo(status,text) \
   [$w.c create text $cx1 $cy1 -anchor nw -fill white -font $stdfont -text "Configuring."]

resSelect

getSpecLimits hn4d
getSpecLimits h_c_hn
getSpecLimits hn_n_c

toggleCursors

selectByEntry hnAss  [lindex $eList(hnAss) 0]
selectByEntry h_c_hn $eMax(h_c_hn)

focus $w.c

wm title $w \
 "Ctrl-LEFT: Drag to Zoom | LEFT: Select Box | Shift-LEFT: Select Position | RIGHT: Related HN | Shift-RIGHT CH3 Assign | b Toggle Boxes | c Toggle Cursors | z Auto Zoom | f Full View | + - Adjust Contours"

