#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Crude Auto-Conversion of Paravision 1D Data:"
   echo ' -dir  dir     [.]              Data Directory.'
   echo ' -in   inName  [$dir/fid]       Input Data Name.'
   echo ' -parm pName   [$dir/acqp]      Input Parameter Name.'
   echo ' -fid  fidName [$dir/test.fid]  NMRPipe FID Output Name.'
   echo ' -out  outName [$dir/test.ft1]  NMRPipe FT Output Name.'
   echo ' -xN   nTot    [xN]             Total Number of Points Real+Imag.'
   echo ' -xT   nApod   [xN/2]           Number of Valid Complex Points.'
   echo ' -xSW  sw      [SW_h]           Spectral Width, Hz.'
   echo ' -xOBS obs     [SFO1]           Observe Frequency, MHz.'
   echo ' -xCAR car     [4.668]          Center Position, PPM.'
   echo ' -xLAB lab     [1H]             Axis Label.'
   echo 'Processing:'
   echo ' -lb   lb      [5.0]            Line Broaden, Hz.'
   echo ' -p0   p0      [0.0]            Zero-Order Phase Correction.'
   exit 0
endif

set dDir    = (`getArgD $argv -dir   .`)
set inName  = (`getArgD $argv -in    $dDir/fid`)
set pName   = (`getArgD $argv -parm  $dDir/acqp`)
set fidName = (`getArgD $argv -fid   $dDir/test.fid`)
set outName = (`getArgD $argv -out   $dDir/test.ft1`)

if (!(-e $inName)) then
   echo Error finding input data $inName
   exit 1
endif

if (!(-e $pName)) then
   echo Error finding input data $pName
   exit 1
endif

set sInfo = (`/bin/ls -l $inName`)
set xN    = (`MATH "$sInfo[5] / 4"`)
set xT    = (`MATH "$xN / 2"`)

foreach i (DECIM DSPFVS SW_h SFO1)
   set v  = (`fgrep '##$'$i $pName | tr '=' ' '` 0 0)
   set $i = $v[2]

   echo $i = $v[2]
end

set xN   = (`getArgD $argv -xN   $xN`)
set xT   = (`getArgD $argv -xT   $xT`)
set xSW  = (`getArgD $argv -xSW  $SW_h`)
set xCAR = (`getArgD $argv -xCAR 4.668`)
set xOBS = (`getArgD $argv -xOBS $SFO1`)
set xLAB = (`getArgD $argv -xLAB 1H`)

set lb   = (`getArgD $argv -lb   5.0`)
set p0   = (`getArgD $argv -p0   0.0`)

bruk2pipe -in $inName \
  -bad 0.0 -aswap -AMX -decim $DECIM -dspfvs $DSPFVS -grpdly -1  \
  -xN               $xN  \
  -xT               $xT  \
  -xMODE        Complex  \
  -xSW             $xSW  \
  -xOBS           $xOBS  \
  -xCAR           $xCAR  \
  -xLAB           $xLAB  \
  -ndim               1  \
  -out $fidName -verb -ov

echo Created $fidName
echo ""

nmrPipe -in $fidName \
| nmrPipe -fn EM -lb $lb -c 0.5    \
| nmrPipe -fn ZF -auto             \
| nmrPipe -fn FT                   \
| nmrPipe -fn PS -p0 $p0 -p1 0 -di \
   -out $outName -verb -ov

echo Created $outName
