#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

set thisVersion "2022.172.12.57"

if {[flagLoc $ARGV -help]} \
   {
    puts "Create Coord Table from pcaNMR Results (Version $thisVersion):"
    puts " -in         inList     \[None\]          Input Spectra (List or Template)."
    puts " -inV        inVList    \[None\]          Optional Variance Spectra (List or Template)."
    puts " -load       loadName   \[load.dat\]      Loading File."
    puts " -score      scoreName  \[score03d.dat\]  Score Template."
    puts " -pcaCoords  coordName  \[pcaCoords.tab\] Output File."
    puts " -classList  classList  \[CLASS1 ...\]    Class Names of PCA Results."
    puts " -classV     classV     \[_VAR_\]         Name of Variance Spectra Class, if Any."
    puts " -valList    valList    \[1.0 ...\]       Corresponding Values to Include, if Any."
    puts " -title      title      \[PCA\]           Title for Output File."
    puts " -zMode      zMode      \[SqrtSumSq\]     Normalization for Z-Score: Self Max Avg SqrtSumSq"
    puts " -extraCount extraCount \[0\]             Number of Additional Spectra Included with Previous PCA."
    puts "Arguments Used for pcaNMR Application (Should be Last on Command Line):"
    puts " -pcaArgs ..."
    exit 0
   }

set dirName   .
set title     PCA

getArgD $ARGV -dir dirName

set coordName $dirName/pcaCoords.tab
set scoreName $dirName/score%03d.dat
set loadName  $dirName/load.dat

set pcaColors  blue
set classList  CLASS1
set zMode      SqrtSumSq 
set thisList   "None" 
set thisListV  "None" 
set valList    "1.0"
set inList     ""
set inListV    ""
set classV     _VAR_
set rdata      0
set fdata      0
set extraCount 0

getArgD $ARGV -pcaCoords  coordName
getArgD $ARGV -load       loadName
getArgD $ARGV -score      scoreName
getArgD $ARGV -classV     classV
getArgD $ARGV -zMode      zMode
getArgD $ARGV -extraCount extraCount 

getListArgD $ARGV -in        thisList
getListArgD $ARGV -inV       thisListV
getListArgD $ARGV -pcaColors pcaColors
getListArgD $ARGV -classList classList
getListArgD $ARGV -valList   valList
getListArgD $ARGV -title     title

if {![strcasecmp $thisList  None]} {set thisList  ""}
if {![strcasecmp $thisListV None]} {set thisListV ""}

#
#---------------------------------------------------------------------------------------

set error [catch {set outUnit [open $coordName w]}]
if {$error} {exit 0}

if {![file exists $loadName]} \
   {
    puts $outUnit "# Error Finding PCA File $loadName"
    catch "close $outUnit"
    exit 0
   }

set thisName [lindex $thisListV 0]
set fmtCount [getFmtCount $thisName]
set n        [llength $thisListV]

if {$n == 1 && $fmtCount == 1} \
   {
    for {set i 1} {$i <= 100000} {incr i} \
       {
        set inName [format $thisName $i]
        if {![file exists $inName]} break

        lappend inListV $inName
       }
   } \
else \
   {
    foreach inName $thisList \
       {
        if {![file exists $inName]} continue
        lappend inListV $thisList    
       }
   } 

set thisName [lindex $thisList 0]
set fmtCount [getFmtCount $thisName]
set n        [llength $thisList]

if {$n == 1 && $fmtCount == 1} \
   {
    for {set i 1} {$i <= 100000} {incr i} \
       {
        set inName [format $thisName $i]
        if {![file exists $inName]} break

        lappend inList $inName
       }
   } \
else \
   {
    foreach inName $thisList \
       {
        if {![file exists $inName]} continue
        lappend inList $thisList
       }
   }

set inCount   [llength $inList]
set thisClass [lindex $classList end]
set thisVal   [lindex $valList end]

for {set i 0} {$i < $inCount} {incr i} \
   {
    lappend classList $thisClass
    lappend valList   $thisVal
   }

for {set i 0} {$i < $inCount} {incr i} {lappend classList $thisClass}

set classList   [lrange $classList 0 [expr $inCount - 1]]
set valList     [lrange $valList   0 [expr $inCount - 1]]

set uniqClasses [listUniq $classList]

readROI -roi 1 -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -in $loadName

set rInfo [getROIInfo -roi 1]
set rdata [lindex $rInfo 0]
set nI    [lindex [lindex $rInfo 2] 0]
set nC    [lindex [lindex $rInfo 2] 1]

if {$inCount != $nI} \
   {
    puts $outUnit "# Error: PCA Count Mismatch. File Count: $inCount  $loadName Size: $nI $nC."
    catch "close $outUnit"
    exit 0
   }

set ii 0

foreach inName $inList \
   {
    lappend pcaColors blue

    set error [rdFDATA $inName fdata]
    
    if {$error} \
       {
        puts $outUnit "# Error Reading Header in PCA Input $inName"
        catch "close $outUnit"
        exit 0
       }

    incr ii

    set tInfo($ii) [getParmStr $fdata FDTITLE NULL_DIM]
    if {![llength $tInfo($ii)]} {set tInfo($ii) $inName}

    if {$ii == 1} \
       {
        set xSize    [getParmI $fdata NDSIZE     CUR_XDIM]
        set ySize    [getParmI $fdata NDSIZE     CUR_YDIM]
        set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

        if {$dimCount == 1 || $ySize == 1} \
           {
            set dimCount 1
            set ySize    1
           } \
        else \
           {
            set dimCount 2
           }
       }

    set pcaInfo($ii) ""

    catch "sysFree $fdata"
   }

# set pcaColors [lrange $pcaColors 0 [expr [llength $inList] - 1]]

set refClass  [lindex $classList 0]

for {set ii 1} {$ii <= $nI} {incr ii} {set pcaInfo($ii)  ""}

for {set ic 1} {$ic <= $nC} {incr ic} \
    {
     foreach thisClass [concat $uniqClasses ALL] {set classInfo($thisClass,$ic) ""}

     set compInfo($ic) ""
    }

set k 0

for {set ic 1} {$ic <= $nC} {incr ic} \
   {
    for {set ii 1} {$ii <= $nI} {incr ii} \
       {
        set thisClass [lindex $classList [expr $ii - 1]]
        set val       [getPtrVal $rdata $k]

        lappend classInfo($thisClass,$ic) $val
        lappend classInfo(ALL,$ic)        $val

        lappend compInfo($ic) $val
        lappend pcaInfo($ii)  $val

        incr k           
       }
   }

if {[llength $inListV]} \
   {
    set thisClass $classV
    set maxSize   0

    lappend uniqClasses $thisClass

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        set thisID [expr $ic + 100]

        if {$dimCount == 1} \
           {
            readROI -roi $thisID -ndim 1 -x X_AXIS 0% 100% -dy Y_AXIS $ic 0 -in [format $scoreName 1]

            set thisInfo    [getROIInfo -roi $thisID]
            set rdataV($ic) [lindex $thisInfo 0]
            set fdataV($ic) [lindex $thisInfo 1]
            set xSizeV($ic) [lindex [lindex $thisInfo 2] 0]
            set ySizeV($ic) 1
            set nV($ic)     $xSizeV($ic)
           } \
        else \
           {
            readROI -roi $thisID -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -in [format $scoreName $ic]

            set thisInfo    [getROIInfo -roi $thisID]
            set rdataV($ic) [lindex $thisInfo 0]
            set fdataV($ic) [lindex $thisInfo 1]
            set xSizeV($ic) [lindex [lindex $thisInfo 2] 0]
            set ySizeV($ic) [lindex [lindex $thisInfo 2] 1]
            set nV($ic)     [expr $xSizeV($ic)*$ySizeV($ic)]
           }

        if {$nV($ic) > $maxSize} {set maxSize $nV($ic)}
       }

    set rdataQ [sysMalloc $maxSize float]
    set thisID 2
    set iv     0

    foreach thisName $inListV \
       {
        set ic 0

        incr iv
        set  ii [expr $iv + [llength $inList]]

        if {$dimCount == 1} \
           {
            readROI -roi $thisID -ndim 1 -x X_AXIS 0% 100% -in $thisName

            set thisInfo    [getROIInfo -roi $thisID]
            set rdataV($ic) [lindex $thisInfo 0]
            set fdataV($ic) [lindex $thisInfo 1]
            set xSizeV($ic) [lindex [lindex $thisInfo 2] 0]
            set ySizeV($ic) 1
            set nV($ic)     $xSizeV($ic)
           } \
        else \
           {
            readROI -roi $thisID -ndim 2 -x X_AXIS 0% 100% -in $thisName

            set thisInfo    [getROIInfo -roi $thisID]
            set rdataV($ic) [lindex $thisInfo 0]
            set fdataV($ic) [lindex $thisInfo 1]
            set xSizeV($ic) [lindex [lindex $thisInfo 2] 0]
            set ySizeV($ic) [lindex [lindex $thisInfo 2] 1]
            set nV($ic)     [expr $xSizeV($ic)*$ySizeV($ic)]
           }

        set tInfo($ii) [getParmStr $fdataV($ic) FDTITLE NULL_DIM]
        if {![llength $tInfo($ii)]} {set tInfo($ii) $thisName}

        for {set ic 1} {$ic <= $nC} {incr ic} \
            {
             if {$xSizeV($ic) != $xSizeV(0) || $ySizeV($ic) != $ySizeV(0)} \
                {
                 puts $outUnit "# Error: mismatched size $thisName $xSizeV(0) $ySizeV(0) score size $xSizeV($ic) $ySizeV($ic)"
                 catch "close $outUnit"
                 exit 0
                }

             set val [vvGetScale $rdataV(0) $rdataV($ic) $nV($ic)]

             set scaleInfo($iv,$ic) $val

             lappend classInfo($thisClass,$ic) $val
             lappend classInfo(ALL,$ic)        $val

             lappend compInfo($ic) $val
             lappend pcaInfo($ii)  $val

             vsMult $rdataQ $rdataV($ic) $scaleInfo($iv,$ic) $nV(0)
             vvSub  $rdataV(0) $rdataQ $nV(0)
            }
       }
   }

for {set ic 1} {$ic <= $nC} {incr ic} \
   {
    set sigmaInfo($ic) ""
    set zInfo($ic)     ""

    set devList ""

    foreach thisClass $uniqClasses \
       {
        set devInfo($thisClass,$ic)   [getStdDev $classInfo($thisClass,$ic)]
        set avgInfo($thisClass,$ic)   [getAvg    $classInfo($thisClass,$ic)]
        set minInfo($thisClass,$ic)   [getMin    $classInfo($thisClass,$ic)]
        set maxInfo($thisClass,$ic)   [getMax    $classInfo($thisClass,$ic)]

        lappend sigmaInfo($ic) $devInfo($thisClass,$ic)
       }

    set thisClass ALL

    set devInfo($thisClass,$ic) [getStdDev $classInfo($thisClass,$ic)]
    set avgInfo($thisClass,$ic) [getAvg    $classInfo($thisClass,$ic)]
    set minInfo($thisClass,$ic) [getMin    $classInfo($thisClass,$ic)]
    set maxInfo($thisClass,$ic) [getMax    $classInfo($thisClass,$ic)]

    set rangeInfo($ic)          [getRange $compInfo($ic)]

    if {![strcasecmp $zMode Max]} \
       {
        set zInfo($ic) [getMax $sigmaInfo($ic)]
       } \
    elseif {![strcasecmp $zMode Avg]} \
       {
        set zInfo($ic) [getAvg $sigmaInfo($ic)]
       } \
    elseif {![strcasecmp $zMode Self]} \
       {
        set zInfo($ic) [getMin $sigmaInfo($ic)]
       } \
    else \
       {
        set zInfo($ic) [getSqrtSumSq $sigmaInfo($ic)]
        set thisN      [llength $sigmaInfo($ic)]

        if {$thisN > 2} {set zInfo($ic) [expr $zInfo($ic)/double($thisN - 1)]}
       }
   } 

set nStr 4

foreach thisClass $uniqClasses \
   {
    if {[string length $thisClass] > $nStr} \
       {
        set nStr [expr [string length $thisClass] + 1]
       }
   }

puts $outUnit ""
puts $outUnit "DATA TITLE     $title"
puts $outUnit "DATA PCA_EXTRA $extraCount"
puts $outUnit ""

puts -nonewline $outUnit "DATA CLASSES        "

foreach thisClass $uniqClasses {puts -nonewline $outUnit " $thisClass"}
puts $outUnit ""

puts $outUnit "DATA REFERENCE_CLASS $refClass"

if {[llength $inListV]} \
   {
    puts $outUnit "DATA VARIANCE_CLASS  $classV"
   } \
else \
   {
    puts $outUnit "DATA VARIANCE_CLASS  None"
   }

if {$dimCount == 1} \
   {
    puts $outUnit "DATA FIRST_SPECTRUM  [lindex $inList 0] DIMCOUNT $dimCount SIZE $xSize"
   } \
else \
   {
    puts $outUnit "DATA FIRST_SPECTRUM  [lindex $inList 0] DIMCOUNT $dimCount SIZE $xSize $ySize"
   }

puts $outUnit "DATA COMPONENT_COUNT $nC"
puts $outUnit ""

foreach thisClass "$uniqClasses ALL" \
   {
    set fmt "DATA MIN        CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         puts -nonewline $outUnit [format " % .6e" $minInfo($thisClass,$ic)]
        }

    puts $outUnit ""

    set fmt "DATA MAX        CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         puts -nonewline $outUnit [format " % .6e" $maxInfo($thisClass,$ic)]
        }

    puts $outUnit ""

    set fmt "DATA AVG        CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         puts -nonewline $outUnit [format " % .6e" $avgInfo($thisClass,$ic)]
        }

    puts $outUnit ""

    set fmt "DATA RANGE      CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         puts -nonewline $outUnit [format " % .6e" [expr $maxInfo($thisClass,$ic) - $minInfo($thisClass,$ic)]]
        }

    puts $outUnit ""

    set fmt "DATA STDDEV     CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         puts -nonewline $outUnit [format " % .6e" $devInfo($thisClass,$ic)]
        }

    puts $outUnit ""

    set fmt "DATA STDDEV_PCT CLASS %-${nStr}s COUNT %2d"

    puts -nonewline $outUnit [format $fmt $thisClass [llength $classInfo($thisClass,1)]]

    for {set ic 1} {$ic <= $nC} {incr ic} \
        {
         if {$minInfo($thisClass,$ic) == $maxInfo($thisClass,$ic)} \
            {
             set p 0.0
            } \
         else \
            {
             set p [expr 100.0*$devInfo($thisClass,$ic)/($maxInfo($thisClass,$ic) - $minInfo($thisClass,$ic))]
            }

         puts -nonewline $outUnit [format " %13.2f" $p]
        }

    puts $outUnit ""
    puts $outUnit ""
   }

puts -nonewline $outUnit [format "DATA ZMODE      %-${nStr}s               " $zMode]

for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit [format " % .6e" $zInfo($ic)]}

puts $outUnit ""

puts -nonewline $outUnit [format "DATA ZMODE_PCT  %-${nStr}s               " $zMode]

set thisClass ALL

for {set ic 1} {$ic <= $nC} {incr ic} \
   {
    if {$minInfo($thisClass,$ic) == $maxInfo($thisClass,$ic)} \
       {
        set p 0.0
       } \
    else \
       {
        set p [expr 100.0*$zInfo($ic)/($maxInfo($thisClass,$ic) - $minInfo($thisClass,$ic))]
       }

    puts -nonewline $outUnit [format " %13.2f" $p]
   }

puts $outUnit ""
puts $outUnit ""

puts -nonewline $outUnit "VARS   INDEX FILE_NAME TITLE CLASS VALUE COLOR"

for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit [format " PCA%d "    $ic]}
for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit [format " PCANORM%d" $ic]}
for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit [format " PCAZ%d"    $ic]}

puts $outUnit ""

puts -nonewline $outUnit "FORMAT %5d   %s        %-26s %-12s %e    %-10s"

for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit " %.6e "}
for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit " %.6e    "}
for {set ic 1} {$ic <= $nC} {incr ic} {puts -nonewline $outUnit " %.6e "}

puts $outUnit ""
puts $outUnit ""

for {set ii 1} {$ii <= $nI} {incr ii} \
   {
    set thisName  [lindex $inList    [expr $ii - 1]]
    set thisClass [lindex $classList [expr $ii - 1]]
    set thisVal   [lindex $valList   [expr $ii - 1]]
    set thisColor [lindex $pcaColors [expr $ii - 1]]

    puts -nonewline $outUnit [format " %5d %s %-26s %-12s % e %-10s" $ii $thisName $tInfo($ii) $thisClass $thisVal $thisColor]

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        set sVal [lindex $pcaInfo($ii) [expr $ic - 1]]

        puts -nonewline $outUnit [format " % .6e" $sVal]
       }

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        set sVal [safeDiv [lindex $pcaInfo($ii) [expr $ic - 1]] $rangeInfo($ic)]

        puts -nonewline $outUnit [format " % .6e" $sVal]
       }

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        if {![strcasecmp $zMode Self]} \
           {
            set dev $devInfo($thisClass,$ic)
           } \
        else \
           {
            set dev $zInfo($ic)
           }

        set sVal [lindex $pcaInfo($ii) [expr $ic - 1]]
        set dVal [expr $sVal - $avgInfo($refClass,$ic)]
        set zVal [safeDiv $dVal $dev]

        puts -nonewline $outUnit [format " % .6e" $zVal]
       }

    puts $outUnit ""
   }

set thisClass $classV
set iv 0

foreach thisName $inListV \
   {
    incr iv
    set  ii [expr $iv + [llength $inList]]

    set thisVal   1.0
    set thisColor [lindex $pcaColors [expr $ii - 1]]

    if {![llength $thisColor]} {set thisColor gray}

    puts -nonewline $outUnit [format " %5d %s %-26s %-12s % e %-10s" $ii $thisName $tInfo($ii) $thisClass $thisVal $thisColor]

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        set sVal [lindex $pcaInfo($ii) [expr $ic - 1]]

        puts -nonewline $outUnit [format " % .6e" $sVal]
       }

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        set sVal [safeDiv [lindex $pcaInfo($ii) [expr $ic - 1]] $rangeInfo($ic)]

        puts -nonewline $outUnit [format " % .6e" $sVal]
       }

    for {set ic 1} {$ic <= $nC} {incr ic} \
       {
        if {![strcasecmp $zMode Self]} \
           {
            set dev $devInfo($thisClass,$ic)
           } \
        else \
           {
            set dev $zInfo($ic)
           }

        set sVal [lindex $pcaInfo($ii) [expr $ic - 1]]
        set zVal [safeDiv [expr $sVal - $avgInfo($refClass,$ic)] $dev]

        puts -nonewline $outUnit [format " % .6e" $zVal]
       }

    puts $outUnit ""
   }

exit 0
