#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

proc initPCAViewVars {} \
{
   global objInfo

   gdbInit
   gdbCreate dBase

   setFonts

   set objInfo(prog,name) "PCAView 2010.160.15.01 (c) Frank Delaglio"

   set objInfo(load,name)    load.dat
   set objInfo(load,id)      1
   set objInfo(load,idH)     2
   set objInfo(load,idV)     3
   set objInfo(load,rdataH)  0
   set objInfo(load,rdataV)  0
   set objInfo(load,hLoc)    1
   set objInfo(load,vLoc)    2
   set objInfo(load,bgColor) black 
   set objInfo(load,fgColor) yellow 
   set objInfo(load,count)   0

   set objInfo(pca,name)     pca/pca%03d.dat
   set objInfo(pca,labMode)  title
   set objInfo(pca,id)       4
   set objInfo(pca,rdata)    0
   set objInfo(pca,loc)      0
   set objInfo(pca,bgColor)  black
   set objInfo(pca,fgColor)  yellow
   set objInfo(pca,count)    0

   set objInfo(pca,ix1)      0
   set objInfo(pca,ixn)      0
   set objInfo(pca,iy1)      0
   set objInfo(pca,iyn)      0

   set objInfo(pca,zx1)      0
   set objInfo(pca,zxn)      0
   set objInfo(pca,zy1)      0
   set objInfo(pca,zyn)      0

   set objInfo(spec,name)     ft/test%05d.dat
   set objInfo(spec,id)       5
   set objInfo(spec,rdata)    0
   set objInfo(spec,loc)      1
   set objInfo(spec,prevLoc)  0 
   set objInfo(spec,locList)  1
   set objInfo(spec,expID)    0
   set objInfo(spec,count)    0
   set objInfo(spec,hi)       0.0
   set objInfo(spec,factor)   1.3
   set objInfo(spec,hAxis)    None 
   set objInfo(spec,vAxis)    None 

   set objInfo(spec,x1)       0 
   set objInfo(spec,xn)       0 
   set objInfo(spec,y1)       0 
   set objInfo(spec,yn)       0

   set objInfo(spec,zx1)      0 
   set objInfo(spec,zxn)      0 
   set objInfo(spec,zy1)      0 
   set objInfo(spec,zyn)      0

   set objInfo(spec,ix1)      0
   set objInfo(spec,ixn)      0
   set objInfo(spec,iy1)      0
   set objInfo(spec,iyn)      0

   set objInfo(spec,x1Pix)    0
   set objInfo(spec,y1Pix)    0 
   set objInfo(spec,xnPix)    0 
   set objInfo(spec,ynPix)    0

   set objInfo(spec,bgColor)  black 
   set objInfo(spec,fgColor)  cyan

   set objInfo(color,inName)  color.tab
   set objInfo(color,outName) color.new.tab
   set objInfo(color,tab)     0
   set objInfo(color,zoom)    white

   set objInfo(color,nameList) {
                                black  red   orange  yellow    green   blue   \
                                violet cyan  magenta lightGray dimGray gray30 \
                                gray20 white
                               }
}

proc initPCAViewArgs {} \
{
   global objInfo ARGC ARGV

   if {[flagLoc $ARGV -help]} \
      {
       puts "pcaView.tcl: Multivariate Spectral Viewer:"
       puts " -in   tabName   \[color.tab\]        Spectral Color Class Input."
       puts " -out  tabName   \[color.new.tab\]    Spectral Color Class Output."
       puts " -spec specName  \[ft/test%05d.dat\]  Spectral Data Input."
       puts " -pca  pcaName   \[pca/pca%03d.dat\]  PCA Spectrum Input."
       puts " -load loadName  \[load.dat\]         PCA Load Vector Input."
       puts " -hi   hiVal     \[0.0\]              First Contour Height."
       puts "Label Modes:"
       puts " -title                    Label by Title in Header (Default)."
       puts " -tail                     Label by Tail of File Name."
       puts " -full                     Label by Full File Name."
       puts " -num                      Label by File Number."
       exit 0
      }

   if {[flagLoc $ARGV -title]} {set objInfo(pca,labMode) title}
   if {[flagLoc $ARGV -tail]}  {set objInfo(pca,labMode) tail}
   if {[flagLoc $ARGV -full]}  {set objInfo(pca,labMode) full}
   if {[flagLoc $ARGV -num]}   {set objInfo(pca,labMode) num}

   getArgD $ARGV -in   objInfo(color,inName)
   getArgD $ARGV -out  objInfo(color,outName)

   getArgD $ARGV -spec objInfo(spec,name)
   getArgD $ARGV -hi   objInfo(spec,hi)
   getArgD $ARGV -load objInfo(load,name)
   getArgD $ARGV -pca  objInfo(pca,name)
}

proc initSpecData {} \
{
   global objInfo

   set obj spec

   set thisName [format $objInfo(spec,name) 1]
   set error    [rdFDATA $thisName fdata]

   if {$error} \
      {
       showError pcaView.tcl "Error reading file $thisName"
       exit
      }

   set objInfo($obj,fdata)  $fdata
   set objInfo($obj,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]

   if {$objInfo($obj,tpFlag)} \
      {
       set objInfo($obj,hAxis)  Y_AXIS 
       set objInfo($obj,vAxis)  X_AXIS
      } \
   else \
      {
       set objInfo($obj,hAxis)  X_AXIS 
       set objInfo($obj,vAxis)  Y_AXIS
      }

   readROI -in $thisName -ndim 2 -roi $objInfo($obj,id) \
           -x $objInfo($obj,hAxis) -y $objInfo($obj,vAxis)

   set roiInfo [getROIInfo -roi $objInfo($obj,id)]
   set xSize   [lindex [lindex $roiInfo 2] 0]
   set ySize   [lindex [lindex $roiInfo 2] 1]
   set n       [expr $xSize*$ySize]

   set objInfo($obj,x1)  1
   set objInfo($obj,xn)  $xSize 
   set objInfo($obj,y1)  1
   set objInfo($obj,yn)  $ySize 

   set objInfo($obj,zx1)  1
   set objInfo($obj,zxn)  $xSize 
   set objInfo($obj,zy1)  1
   set objInfo($obj,zyn)  $ySize 

   set objInfo($obj,rdata) [lindex $roiInfo 0]
   set objInfo($obj,fdata) [lindex $roiInfo 1]

#   set noise               [vEstNoise $objInfo($obj,rdata) $n]
#
#   if {$noise != 0.0 && $objInfo($obj,hi) == 0.0} \
#      { 
#       set objInfo($obj,hi) [expr 6.0*[vEstNoise $objInfo($obj,rdata) $n]]
#      } \
#   else \
#      {
#       set minVal [vMin $objInfo($obj,rdata) $n]
#       set maxVal [vMax $objInfo($obj,rdata) $n]
#
#       set objInfo($obj,hi) [expr ($maxVal - $minVal)/20.0]
#      }

   if {$objInfo($obj,hi) == 0.0} \
      {
       set minVal [vMin $objInfo($obj,rdata) $n]
       set maxVal [vMax $objInfo($obj,rdata) $n]

       set objInfo($obj,hi) [getFirstLevel 10 10 $minVal $maxVal 1.3]
      }
}

proc initExpID {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       set thisName [format $objInfo(spec,name) $i]

       readROI -in $thisName -ndim 1 -roi $objInfo(spec,id) \
               -x $objInfo(spec,hAxis) -dy $objInfo(spec,vAxis) 1 0

       set roiInfo [getROIInfo -roi $objInfo(spec,id)]
       set fdata   [lindex $roiInfo 1]

       switch $objInfo(pca,labMode) \
          {
           title {
                  set objInfo(expID,$i) [getParmStr $fdata FDTITLE NULL_DIM]
                 }

           full {
                 set objInfo(expID,$i) $thisName
                }

           tail {
                 set objInfo(expID,$i) [file root [file tail $thisName]]
                }

           num  {
                 set objInfo(expID,$i) $i 
                }

           default {
                    set objInfo(expID,$i) $i

                    catch {set objInfo(expID,$i) \
                            [expr int([getParm $fdata FDUSER1 NULL_DIM])]}
                   }
          }
      }

   readSpec
}

proc initPCAViewWin {} \
{
   global objInfo env

   set w .

   wm title $w $objInfo(prog,name)
   wm geom  $w +10+10

   frame .locList
   frame .top 
   frame .left 
   frame .right
   frame .bottom
   frame .cmnd

   set objInfo(load,xSize) 440
   set objInfo(load,ySize) 100

   set objInfo(pca,canvas) .pca
   set objInfo(pca,xSize)  440
   set objInfo(pca,ySize)  440
   set objInfo(pca,ix1)    0 
   set objInfo(pca,iy1)    0 
   set objInfo(pca,ixn)    [expr $objInfo(pca,xSize) - 1]
   set objInfo(pca,iyn)    [expr $objInfo(pca,ySize) - 1] 

   set objInfo(spec,canvas) .spec
   set objInfo(spec,xSize)  640
   set objInfo(spec,ySize)  660
   set objInfo(spec,ix1)    10
   set objInfo(spec,iy1)    10 
   set objInfo(spec,ixn)    540
   set objInfo(spec,iyn)    580

   canvas .loadH \
     -width $objInfo(load,xSize) -height $objInfo(load,ySize) \
     -bg    $objInfo(load,bgColor) -closeenough 10 -border 0

   canvas .loadV \
     -width $objInfo(load,xSize) -height $objInfo(load,ySize) \
     -bg    $objInfo(load,bgColor) -closeenough 10 -border 0

   bind .loadH <ButtonRelease-1> "selectLoc %x %y"
   bind .loadH <Any-Enter>       "setTitle loadH"
   bind .loadH <Any-Leave>       "setTitle title"

   bind .loadV <ButtonRelease-1> "selectLoc %x %y"
   bind .loadV <Any-Enter>       "setTitle loadV"
   bind .loadV <Any-Leave>       "setTitle title"

   canvas .pca \
     -width $objInfo(pca,xSize) -height $objInfo(pca,ySize)   \
     -bg $objInfo(pca,bgColor) -closeenough 10 -border 0

   bind .pca <Any-Enter>       "setTitle pca"
   bind .pca <Any-Leave>       "setTitle title"

   canvas .spec \
     -width $objInfo(spec,xSize) -height $objInfo(spec,ySize)  \
     -closeenough 10 -border 0

   bind .spec <Any-Enter>       "setTitle spec"
   bind .spec <Any-Leave>       "setTitle title"

   button .next      -text "Next"           -command nextSpec
   button .prev      -text "Prev"           -command prevSpec
   button .drawSpec  -text "Draw Spectrum"  -command drawSpec
   button .fullSpec  -text "Full Spectrum"  -command fullSpec
   button .clearSpec -text "Clear Spectrum" -command resetSpec
   button .drawPCA   -text "Draw PCA"       -command drawScatter
   button .fullPCA   -text "Full PCA"       -command fullScatter
   button .clearPCA  -text "Clear PCA"      -command clearScatter
   button .save      -text "Save"           -command writeClassify 
   button .quit      -text "Quit"           -command quitPCAView

   label .lspec -text "Spectrum:" 
   entry .espec -textvariable objInfo(spec,loc)   -width 4

   label .lid -text " ID:" 
   entry .eid -textvariable objInfo(spec,expID)   -width 5

   label .lpcax -text " PCA X:"
   entry .epcax  -textvariable objInfo(load,hLoc) -width 2

   menubutton .bpcax -bitmap @$env(NMRTXT)/caret.map -relief raised \
              -menu .bpcax.menu

   label .lpcay -text " PCA Y:"
   entry .epcay  -textvariable objInfo(load,vLoc) -width 2

   menubutton .bpcay -bitmap @$env(NMRTXT)/caret.map -relief raised \
              -menu .bpcay.menu

   label .lhi  -text " Hi:"
   entry .ehi  -textvariable objInfo(spec,hi)     -width 9 

   button .incr  -text "+" -padx 1 -pady 1 -command incrHi
   button .decr  -text "-" -padx 1 -pady 1 -command decrHi

   label .lcolor -text " Color:"
   entry .ecolor -width 9 -textvariable objInfo(pca,fgColor)

   menubutton .bcolor -bitmap @$env(NMRTXT)/caret.map -relief raised \
              -menu .bcolor.menu

   menu .bpcax.menu
   menu .bpcay.menu
   menu .bcolor.menu

   for {set i 1} {$i <= $objInfo(pca,count)} {incr i} \
      {
       .bpcax.menu add command -label $i \
          -command "selectLoad hLoc $i"

       .bpcay.menu add command -label $i \
          -command "selectLoad vLoc $i"
      }

   foreach c $objInfo(color,nameList) \
      {
       .bcolor.menu add command -label $c \
          -command "set objInfo(pca,fgColor) $c"
      }

   label .locs  -text "Spectra:"

   pack .locs -in .locList -side left -anchor w

   for {set i 1} {$i <= 20} {incr i} \
      {
       label .loc$i  -text " " -width 8
       pack  .loc$i -in .locList -side left -anchor w
      }

   bind .espec <Return> "chooseSpec   append"
   bind .eid   <Return> "chooseSpecID append"

   bind .epcax <Return> choosePCA
   bind .epcay <Return> choosePCA

   pack .loadH .loadV .pca -in .left -side top -padx 10 -pady 10
   pack .spec .bottom -in .right -side top 

   pack .next .prev .drawSpec .fullSpec .clearSpec \
        .drawPCA .fullPCA .clearPCA .save .quit \
        -in .cmnd -side left -expand 1 -fill x

   pack .lspec .espec .lid .eid \
        .lpcax .epcax .bpcax .lpcay .epcay .bpcay           \
        .lhi .ehi .incr .decr .lcolor .ecolor .bcolor       \
        -in .bottom -side left -anchor w

   pack .left .right -in .top -side left
   pack .locList -in . -side top -expand 1 -fill x -pady 2
   pack .locList .top  -in . -side top
   pack .cmnd -in . -side top -expand 1 -fill x

   update

   .loadH create pixmap 0 0 \
       -width $objInfo(load,xSize) -height $objInfo(load,ySize) \
       -win $objInfo(load,idH) -tags loadH

   defineCell -cell $objInfo(load,idH) -win $objInfo(load,idH) \
      -x 0% 100% -y 0% 100%

   .loadV create pixmap 0 0 \
       -width $objInfo(load,xSize) -height $objInfo(load,ySize) \
       -win $objInfo(load,idV) -tags loadV

   defineCell -cell $objInfo(load,idV) -win $objInfo(load,idV) \
      -x 0% 100% -y 0% 100%

   set obj spec

   .spec create pixmap 0 0 \
     -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
     -win $objInfo($obj,id) -tags spec

   defineCell -cell $objInfo($obj,id) -win $objInfo($obj,id)   \
      -x $objInfo($obj,ix1) $objInfo($obj,ixn)                 \
      -y $objInfo($obj,iy1) $objInfo($obj,iyn)

   bind $objInfo($obj,canvas) <ButtonPress-3>   "zoomON     $obj %x %y"
   bind $objInfo($obj,canvas) <ButtonRelease-3> "zoomSpec   $obj %x %y"
   bind $objInfo($obj,canvas) <B3-Motion>       "trackZoom  $obj %x %y"

   set obj pca

   bind $objInfo($obj,canvas) <ButtonPress-2>   "zoomON           $obj %x %y"
   bind $objInfo($obj,canvas) <ButtonRelease-2> "classifyScatter  $obj %x %y"
   bind $objInfo($obj,canvas) <B2-Motion>       "trackZoom        $obj %x %y"

   bind $objInfo($obj,canvas) <ButtonPress-3>   "zoomON       $obj %x %y"
   bind $objInfo($obj,canvas) <ButtonRelease-3> "zoomScatter  $obj %x %y"
   bind $objInfo($obj,canvas) <B3-Motion>       "trackZoom    $obj %x %y"
}

proc setTitle { mode } \
{
   global objInfo

   set str $objInfo(prog,name)

   switch $mode \
      {
       loadH {
              append str " -- PCA $objInfo(load,hLoc)"
              append str " \[Click Left Button to Select Spectrum\]"
             }

       loadV {
              append str " -- PCA $objInfo(load,vLoc):"
              append str " \[Click Left Button to Select Spectrum\]"
             }

       pca   {
              append str " -- PCA $objInfo(load,hLoc) vs $objInfo(load,vLoc):"

              append str " \[Click Left Button to Select Spectrum\]"

              append str \
               " \[Drag Middle Button to Colorize Region]"

              append str " \[Drag Right Button to Zoom\]"
             }

       spec  {
              append str " -- 2D Spectral Display:"
              append str " \[Drag Right Button to Zoom\]"
             }

       default {}
      } 

   wm title . $str

   update
}

proc incrHi {} \
{
   global objInfo

   set objInfo(spec,hi) [expr $objInfo(spec,hi)*$objInfo(spec,factor)]
}

proc decrHi {} \
{
   global objInfo

   set objInfo(spec,hi) [expr $objInfo(spec,hi)/$objInfo(spec,factor)]
}

proc zoomON { obj xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo($obj,ix1) || $xPix > $objInfo($obj,ixn)} return
   if {$yPix < $objInfo($obj,iy1) || $yPix > $objInfo($obj,iyn)} return

   set objInfo($obj,x1Pix) $xPix
   set objInfo($obj,y1Pix) $yPix
   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) create rectangle \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix) \
      -outline $objInfo(color,zoom) -tags zoom

   update
}

proc trackZoom { obj xPix yPix } \
{
   global objInfo

   if {$xPix < $objInfo($obj,ix1) || $xPix > $objInfo($obj,ixn)} return
   if {$yPix < $objInfo($obj,iy1) || $yPix > $objInfo($obj,iyn)} return

   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) coords zoom \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix)

   update
}

proc fullScatter {} \
{
   global objInfo

   set obj pca

   set objInfo($obj,zx1) $objInfo(load,minH)
   set objInfo($obj,zxn) $objInfo(load,maxH)
   set objInfo($obj,zy1) $objInfo(load,minV)
   set objInfo($obj,zyn) $objInfo(load,maxV)

   clearScatter
   drawScatter
   drawCursor
}

proc zoomScatter { obj xPix yPix } \
{
   global objInfo

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp               $objInfo($obj,x1Pix)
       set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp               $objInfo($obj,y1Pix)
       set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set objInfo($obj,ynPix) $itemp
      }

   getPCAXY $obj $objInfo($obj,x1Pix) $objInfo($obj,ynPix) zx1 zy1
   getPCAXY $obj $objInfo($obj,xnPix) $objInfo($obj,y1Pix) zxn zyn

   clearScatter

   set objInfo($obj,zx1) $zx1
   set objInfo($obj,zxn) $zxn
   set objInfo($obj,zy1) $zy1
   set objInfo($obj,zyn) $zyn

   drawScatter
   drawCursor
}

proc classifyScatter { obj xPix yPix } \
{
   global objInfo

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp                $objInfo($obj,x1Pix)
       set $objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set $objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp                $objInfo($obj,y1Pix)
       set $objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set $objInfo($obj,ynPix) $itemp
      }

   clearScatter

   getPCAXY $obj $objInfo($obj,x1Pix) $objInfo($obj,ynPix) zx1 zy1
   getPCAXY $obj $objInfo($obj,xnPix) $objInfo($obj,y1Pix) zxn zyn

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       set xVal [getPtrVal $objInfo(load,rdataH) [expr $i - 1]]
       set yVal [getPtrVal $objInfo(load,rdataV) [expr $i - 1]]

       if {$xVal < $zx1} continue
       if {$xVal > $zxn} continue
       if {$yVal < $zy1} continue
       if {$yVal > $zyn} continue

       set objInfo(color,$i) $objInfo(pca,fgColor) 
      }

   drawScatter
   drawCursor
}

proc fullSpec {} \
{
   global objInfo

   set obj spec

   set objInfo($obj,zx1) $objInfo($obj,x1)
   set objInfo($obj,zxn) $objInfo($obj,xn)
   set objInfo($obj,zy1) $objInfo($obj,y1)
   set objInfo($obj,zyn) $objInfo($obj,yn)

   drawAllSpec
}

proc zoomSpec { obj xPix yPix } \
{
   global objInfo

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp               $objInfo($obj,x1Pix)
       set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp               $objInfo($obj,y1Pix)
       set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set objInfo($obj,ynPix) $itemp
      }

   getDataXY $obj $objInfo($obj,x1Pix) $objInfo($obj,ynPix) zx1 zy1
   getDataXY $obj $objInfo($obj,xnPix) $objInfo($obj,y1Pix) zxn zyn

   set objInfo($obj,zx1) $zx1
   set objInfo($obj,zxn) $zxn
   set objInfo($obj,zy1) $zy1
   set objInfo($obj,zyn) $zyn

   drawAllSpec
}

proc getDataXY { obj xPix yPix ixPtr iyPtr } \
{
   global objInfo

   upvar $ixPtr ix
   upvar $iyPtr iy

   set xw [expr double(1 + $objInfo($obj,ixn) - $objInfo($obj,ix1))]
   set yw [expr double(1 + $objInfo($obj,iyn) - $objInfo($obj,iy1))]

   set dx [expr 1 + $objInfo($obj,zxn) - $objInfo($obj,zx1)]
   set dy [expr 1 + $objInfo($obj,zyn) - $objInfo($obj,zy1)]

   if {!$xw} {set xw 1}
   if {!$yw} {set yw 1}

   set ix [expr int($objInfo($obj,zx1)+$dx*($xPix - $objInfo($obj,ix1))/$xw)]
   set iy [expr int($objInfo($obj,zy1)+$dy*($objInfo($obj,iyn) - $yPix)/$yw)]

   if {$ix < $objInfo($obj,zx1)} {set ix $objInfo($obj,zx1)}
   if {$ix > $objInfo($obj,zxn)} {set ix $objInfo($obj,zxn)}

   if {$iy < $objInfo($obj,zy1)} {set iy $objInfo($obj,zy1)}
   if {$iy > $objInfo($obj,zyn)} {set iy $objInfo($obj,zyn)}
}

proc getPCAXY { obj xPix yPix ixPtr iyPtr } \
{
   global objInfo

   upvar $ixPtr ix
   upvar $iyPtr iy

   set xw [expr double(1 + $objInfo($obj,ixn) - $objInfo($obj,ix1))]
   set yw [expr double(1 + $objInfo($obj,iyn) - $objInfo($obj,iy1))]

   set dx [expr $objInfo($obj,zxn) - $objInfo($obj,zx1)]
   set dy [expr $objInfo($obj,zyn) - $objInfo($obj,zy1)]

   if {!$xw} {set xw 1.0}
   if {!$yw} {set yw 1.0}

   set ix [expr $objInfo($obj,zx1) + $dx*($xPix - $objInfo($obj,ix1))/$xw]
   set iy [expr $objInfo($obj,zy1) + $dy*($objInfo($obj,iyn) - $yPix)/$yw]

   if {$ix < $objInfo($obj,zx1)} {set ix $objInfo($obj,zx1)}
   if {$ix > $objInfo($obj,zxn)} {set ix $objInfo($obj,zxn)}

   if {$iy < $objInfo($obj,zy1)} {set iy $objInfo($obj,zy1)}
   if {$iy > $objInfo($obj,zyn)} {set iy $objInfo($obj,zyn)}
}

proc nextSpec {} \
{
   global objInfo

   incr objInfo(spec,loc)

   if {$objInfo(spec,loc) > $objInfo(spec,count)} \
      {
       set objInfo(spec,loc) 1
      }

   lappend objInfo(spec,locList) $objInfo(spec,loc)

   readSpec
   drawSpec
}

proc prevSpec {} \
{
   global objInfo

   incr objInfo(spec,loc) -1

   if {$objInfo(spec,loc) < 1} \
      {
       set objInfo(spec,loc) $objInfo(spec,count) 
      }

   lappend objInfo(spec,locList) $objInfo(spec,loc)

   readSpec
   drawSpec
}

proc chooseSpecID { mode } \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       if {$objInfo(spec,expID) == $objInfo(expID,$i)} \
          {
           set objInfo(spec,loc) $i
           break
          }
      }

   if {"$mode" == "append"} \
      {
       lappend objInfo(spec,locList) $objInfo(spec,loc)
      }

   readSpec
   drawSpec
}

proc chooseSpec { mode } \
{
   global objInfo

   if {$objInfo(spec,loc) < 1} \
      {
       set objInfo(spec,loc) $objInfo(spec,count)
      }

   if {$objInfo(spec,loc) > $objInfo(spec,count)} \
      {
       set objInfo(spec,loc) 1
      }

   readSpec

   if {"$mode" == "append"} \
      {
       lappend objInfo(spec,locList) $objInfo(spec,loc)
       drawSpec
      } \
   else \
      {
       drawThisSpec
      }
}

proc choosePCA {} \
{
   clearPCA
   clearScatter

   readPCA
   drawPCA
   drawCursor
   drawScatter
}

proc resetSpec {} \
{
   global objInfo

   set objInfo(spec,locList) $objInfo(spec,loc)

   clearSpec
   showSpecLocs
}

proc clearSpec {} \
{
   global objInfo

   newScreen -win $objInfo(spec,id)

   drawRec -cell $objInfo(spec,id) -fg $objInfo(spec,bgColor) \
           -x 0% 100% -y 0% 100%

   update
}

proc quitPCAView {} \
{
   global objInfo

   set error [writeClassify]

   if {$error} return

   exit
}

proc readClassify {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       set objInfo(color,$i) $objInfo(pca,fgColor)
      }

   if {![file exists $objInfo(color,inName)]} return

   set objInfo(color,tab) [gdbRead table -in $objInfo(color,inName)]

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       set cond  "INDEX == $i"
       set entry [gdbSelect -from $objInfo(color,tab) -M -inPlace -cond $cond]

       if {[llength $entry] == 1} \
          {
           set objInfo(color,$i) [gdbGet $entry COLOR]
          }
      }
}

proc writeClassify {} \
{
   global objInfo

   set types \
      {
       { {tab}        {.tab} }
       { {Text}       {.txt} }
       { {TCL}        {.tcl} }
       { {dat}        {.dat .DAT} }
       { {ser}        {ser*} }
       { {fid}        {fid*} }
       { {seq}        {.seq} }
       { {C-Shell}    {.com .csh} }
       { {K-Shell}    {.ksh} }
       { {Shell}      {.sh}  }
       { {All Files}  *      }
      }

   set thisName \
    [tk_getSaveFile -title "Color Classification Output" \
                    -filetypes $types -initialfile $objInfo(color,outName)]

   if {![string length $thisName]} {return 0}

   set objInfo(color,outName) $thisName

   set error [catch {set outUnit [open $objInfo(color,outName) w]}]

   if {$error} \
      {
       showError pcaView.tcl "Error opening $objInfo(color,outName)"
       return 1
      }

   puts $outUnit "VARS   INDEX COLOR"
   puts $outUnit "FORMAT %5d   %s"
   puts $outUnit ""

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       puts $outUnit [format " %5d %s" $i $objInfo(color,$i)]
      }

   close $outUnit

   set txt \
    "Saved $objInfo(spec,count) classifications in file $objInfo(color,outName)"

   tk_dialog .saveDialog Save $txt info 0 Continue

   return 0
}

proc readPCA {} \
{
   global objInfo

   readROI -in $objInfo(load,name) -ndim 2 -roi $objInfo(load,id)

   set roiInfo  [getROIInfo -roi $objInfo(load,id)]
   set sizeList [lindex $roiInfo 2]

   set objInfo(spec,count) [lindex $sizeList 0]
   set objInfo(pca,count)  [lindex $sizeList 1]

   if {$objInfo(load,hLoc) < 1} {set objInfo(load,hLoc) 1}
   if {$objInfo(load,vLoc) < 1} {set objInfo(load,vLoc) 1}

   if {$objInfo(load,hLoc) > $objInfo(pca,count)} \
      {
       set objInfo(load,hLoc) $objInfo(pca,count)
      }

   if {$objInfo(load,vLoc) > $objInfo(pca,count)} \
      {
       set objInfo(load,vLoc) $objInfo(pca,count)
      }

   readROI -in $objInfo(load,name) -ndim 1 -roi $objInfo(load,idH) \
           -x ID -dy Y_AXIS $objInfo(load,hLoc) 0

   readROI -in $objInfo(load,name) -ndim 1 -roi $objInfo(load,idV) \
           -x ID -dy Y_AXIS $objInfo(load,vLoc) 0

   set roiInfo [getROIInfo -roi $objInfo(load,idH)]

   set objInfo(load,rdataH) [lindex $roiInfo 0]
   set objInfo(load,minH)   [vMin $objInfo(load,rdataH) $objInfo(spec,count)]
   set objInfo(load,maxH)   [vMax $objInfo(load,rdataH) $objInfo(spec,count)]

   set range  [expr $objInfo(load,maxH) - $objInfo(load,minH)]

   set objInfo(load,minH) [expr $objInfo(load,minH) - 0.05*$range]
   set objInfo(load,maxH) [expr $objInfo(load,maxH) + 0.05*$range]

   set roiInfo [getROIInfo -roi $objInfo(load,idV)]

   set objInfo(load,rdataV) [lindex $roiInfo 0]
   set objInfo(load,minV)   [vMin $objInfo(load,rdataV) $objInfo(spec,count)]
   set objInfo(load,maxV)   [vMax $objInfo(load,rdataV) $objInfo(spec,count)]

   set range  [expr $objInfo(load,maxV) - $objInfo(load,minV)]

   set objInfo(load,minV) [expr $objInfo(load,minV) - 0.05*$range]
   set objInfo(load,maxV) [expr $objInfo(load,maxV) + 0.05*$range]

   set objInfo(pca,zx1) $objInfo(load,minH)
   set objInfo(pca,zxn) $objInfo(load,maxH)
   set objInfo(pca,zy1) $objInfo(load,minV)
   set objInfo(pca,zyn) $objInfo(load,maxV)
}

proc readSpec {} \
{
   global objInfo

   set thisName [format $objInfo(spec,name) $objInfo(spec,loc)]

   readROI -in $thisName -ndim 2 -roi $objInfo(spec,id) \
            -x $objInfo(spec,hAxis)  $objInfo(spec,zx1) $objInfo(spec,zxn) \
            -y $objInfo(spec,vAxis)  $objInfo(spec,zy1) $objInfo(spec,zyn)

   set roiInfo [getROIInfo -roi $objInfo(spec,id)]
   set fdata   [lindex $roiInfo 1]

   switch $objInfo(pca,labMode) \
      {
       title {
              set objInfo(spec,expID) [getParmStr $fdata FDTITLE NULL_DIM]
             }

       full {
             set objInfo(spec,expID) $thisName
            }

       tail {
             set objInfo(spec,expID) [file root [file tail $thisName]]
            }

       default {
                set objInfo(spec,expID) $objInfo(spec,loc) 

                catch {set objInfo(spec,expID) \
                        [expr int([getParm $fdata FDUSER1 NULL_DIM])]}
               }
      }
}

proc clearScatter {} \
{
   global objInfo

   set obj pca

   $objInfo($obj,canvas) delete label cursorH cursorV
}

proc drawCursor {} \
{
   global objInfo

   set cx [expr $objInfo(pca,xSize)/($objInfo(pca,zxn) - $objInfo(pca,zx1))]
   set cy [expr $objInfo(pca,ySize)/($objInfo(pca,zyn) - $objInfo(pca,zy1))]

   set i  [lindex $objInfo(spec,locList) end]

   set xVal [getPtrVal $objInfo(load,rdataH) [expr $i - 1]]
   set yVal [getPtrVal $objInfo(load,rdataV) [expr $i - 1]]

   set xLoc [expr $cx*($xVal - $objInfo(pca,zx1))]
   set yLoc [expr $cy*($objInfo(pca,zyn) - $yVal)]

   if {$xVal >= $objInfo(pca,zx1) && $xVal <= $objInfo(pca,zxn)} \
      {
       .pca coords $objInfo(cursorV,id) \
             $xLoc 0 $xLoc [expr $objInfo(pca,ySize) - 1]
      }

   if {$yVal >= $objInfo(pca,zy1) && $yVal <= $objInfo(pca,zyn)} \
      {
       .pca coords $objInfo(cursorH,id) \
             0 $yLoc [expr $objInfo(pca,xSize) - 1] $yLoc
      }
}

proc drawScatter {} \
{
   global objInfo stdfont

   set cx [expr $objInfo(pca,xSize)/($objInfo(pca,zxn) - $objInfo(pca,zx1))]
   set cy [expr $objInfo(pca,ySize)/($objInfo(pca,zyn) - $objInfo(pca,zy1))]

   set objInfo(cursorH,id) \
       [.pca create line 0 0 [expr $objInfo(pca,xSize) - 1] 0 \
             -fill dimGray -tags cursorH]

   set objInfo(cursorV,id) \
       [.pca create line 0 0 0 [expr $objInfo(pca,ySize) - 1] \
             -fill dimGray -tags cursorV]

   for {set i 1} {$i <= $objInfo(spec,count)} {incr i} \
      {
       set xVal [getPtrVal $objInfo(load,rdataH) [expr $i - 1]]
       set yVal [getPtrVal $objInfo(load,rdataV) [expr $i - 1]]

       if {$xVal < $objInfo(pca,zx1)} continue
       if {$xVal > $objInfo(pca,zxn)} continue
       if {$yVal < $objInfo(pca,zy1)} continue
       if {$yVal > $objInfo(pca,zyn)} continue

       set xLoc [expr $cx*($xVal - $objInfo(pca,zx1))]
       set yLoc [expr $cy*($objInfo(pca,zyn) - $yVal)]       

       set objInfo(tID,$i) \
        [.pca create text $xLoc $yLoc -fill $objInfo(color,$i) -anchor c \
            -font $stdfont -text $objInfo(expID,$i) -tags "$i label"]

       .pca bind $objInfo(tID,$i) <Any-Enter>       "+ showThisID   $i"
       .pca bind $objInfo(tID,$i) <Any-Leave>       "+ unshowThisID $i"
       .pca bind $objInfo(tID,$i) <ButtonRelease-1> "+ drawThisID   $i"
      }

   update
}

proc showThisID { loc } \
{
   global objInfo 

   .pca itemconfigure $objInfo(tID,$loc) -fill white
   .pca raise $objInfo(tID,$loc)

   update
}

proc unshowThisID { loc } \
{
   global objInfo 

   .pca itemconfigure $objInfo(tID,$loc) -fill $objInfo(color,$loc) 
   .pca raise $objInfo(tID,$loc)

   update
}

proc drawThisID { loc } \
{
   global objInfo

   set objInfo(spec,loc) $loc

   chooseSpec append 
}

proc selectLoad { axis loc } \
{
   global objInfo

   set objInfo(load,$axis) $loc

   choosePCA
}

proc selectLoc { xPix yPix } \
{
   global objInfo

   set objInfo(spec,loc) \
    [expr int(0.5+$objInfo(spec,count)*$xPix/double($objInfo(load,xSize)))]

   chooseSpec append 
}

proc clearPCA {} \
{
   global objInfo

   drawRec -fg $objInfo(load,bgColor) -cell $objInfo(load,idH) \
           -x 0% 100% -y 0% 100% -noupdate

   drawRec -fg $objInfo(load,bgColor) -cell $objInfo(load,idV) \
           -x 0% 100% -y 0% 100% -noupdate 

   update
}

proc drawPCA {} \
{
   global objInfo

   set x1 [expr 100.0*($objInfo(spec,loc) - 1)/double($objInfo(spec,count)-1)]
   set xn [expr 100.0*($objInfo(spec,loc) - 0)/double($objInfo(spec,count)-1)]

   drawRec -fg $objInfo(load,bgColor) -cell $objInfo(load,idH) \
           -x 0% 100% -y 0% 100% -noupdate 

   drawRec -fg dimGray -roi $objInfo(load,idH) -cell $objInfo(load,idH) \
           -x $x1% $xn% -y 0% 100% -noupdate 

   drawROI -roi $objInfo(load,idH) -cell $objInfo(load,idH) -nobox \
           -zero -hi Auto -bg None -fg $objInfo(load,fgColor)

   drawRec -fg $objInfo(load,bgColor) -cell $objInfo(load,idV) \
           -x 0% 100% -y 0% 100% -noupdate 

   drawRec -fg dimGray -roi $objInfo(load,idV)  -cell  $objInfo(load,idV) \
           -x $x1% $xn% -y 0% 100% -noupdate 

   drawROI -roi $objInfo(load,idV) -cell $objInfo(load,idV) -nobox \
           -zero -hi Auto -bg  -fg $objInfo(load,fgColor)

   update
}

proc drawAllSpec {} \
{
   global objInfo

   busy hold .

   clearSpec

   foreach loc $objInfo(spec,locList) \
      {
       set objInfo(spec,loc) $loc
       chooseSpec noAppend
      }

   showSpecLocs

   busy release .
}

proc drawSpec {} \
{
   busy hold .

   drawThisSpec
   showSpecLocs

   drawPCA
   drawCursor

   busy release .
}

proc showSpecLocs {} \
{
   global objInfo

   set n [llength $objInfo(spec,locList)]

   for {set i 1} {$i <= 20} {incr i} \
      {
       if {$i > $n} \
          {
           .loc$i configure -text " " -bg black -fg black
          } \
       else \
          {
           set loc [lindex $objInfo(spec,locList) [expr $i-1]]

           .loc$i configure -text "$objInfo(expID,$loc)" \
             -bg black -fg $objInfo(color,$loc) 
          }
      }

   update
}

proc drawThisSpec {} \
{
   global objInfo

   set roiInfo [getROIInfo -roi $objInfo(spec,id)]
   set xSize   [lindex [lindex $roiInfo 2] 0]
   set ySize   [lindex [lindex $roiInfo 2] 1]

   set i       $objInfo(spec,loc)

   if {$ySize == 1} \
      {
       drawROI -roi $objInfo(spec,id) -cell $objInfo(spec,id) \
               -hi Auto -axis Bottom -fg  $objInfo(color,$i)
      } \
   else \
      {
       drawROI -roi $objInfo(spec,id) -cell $objInfo(spec,id) \
               -hi $objInfo(spec,hi) -axis Bottom Right       \
               -color1 $objInfo(color,$i) -color2 $objInfo(color,$i)
      }

   update
}

initPCAViewVars 
initPCAViewArgs
initSpecData

readPCA
readSpec
readClassify
initExpID

initPCAViewWin

clearScatter
clearSpec

drawScatter
drawSpec
