#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Compute Difference Between Two PDB Files:"
    puts " -ref    refName \[ref.pdb\]    Reference PDB Input." 
    puts " -in     inName  \[test.pdb\]   Input PDB."
    puts " -out    outName \[dif.pdb\]    Output, Reference - Input PDB."
    puts " -tmp    tmpName \[align.pdb\]  Aligned Input."
    puts " -thresh thresh  \[0.15\]       Maximum Atom Displacement, A."
    puts " -heavy                       Compare Heavy Atoms Only."
    puts " -align                       Align PDB Coordinates."
    puts " -addH                        Evaluate Results of Adding Protons."
    puts " -verb                        Verbose Mode ON (Default)."
    puts " -noverb                      Verbose Mode OFF."
    puts "Multi-Chain Mode:"
    puts " -refVar refVar \[CHAINID\]   Chain Information from Reference."
    puts " -inVar  inVar  \[CHAINID\]   Chain Information from Input."
    puts " -multi                     Use Multi-Chain Mode."
    exit 0
   }

set refName   ref.pdb
set refTitile ref.pdb

set inName    test.pdb
set inTitle   test.pdb

set refVar     CHAINID
set inVar      CHAINID
set heavyFlag  0
set alignFlag  0
set multiFlag  0
set addHFlag   0
set thresh     0.15
set verbose    1
set error      0

set outName dif.pdb
set tmpName align.pdb

getArgD $ARGV -ref      refName
getArgD $ARGV -ref      refTitle
getArgD $ARGV -refTitle refTitle

getArgD $ARGV -in       inName
getArgD $ARGV -in       inTitle
getArgD $ARGV -inTitle  inTitle

getArgD $ARGV -out      outName
getArgD $ARGV -tmp      tmpName
getArgD $ARGV -thresh   thresh

getArgD $ARGV -refVar   refVar
getArgD $ARGV -inVar    inVar

if {[flagLoc $ARGV -addH]} \
   {
    set heavyFlag 1
    set alignFlag 1
    set addHFlag  1
   }

if {[flagLoc $ARGV -heavy]}    {set heavyFlag 1}
if {[flagLoc $ARGV -noheavy]}  {set heavyFlag 0}

if {[flagLoc $ARGV -center]}   {set alignFlag 1}
if {[flagLoc $ARGV -nocenter]} {set alignFlag 0}

if {[flagLoc $ARGV -align]}    {set alignFlag 1}
if {[flagLoc $ARGV -noalign]}  {set alignFlag 0}

if {[flagLoc $ARGV -multi]}    {set multiFlag 1}
if {[flagLoc $ARGV -nomulti]}  {set multiFlag 0}

if {[flagLoc $ARGV -verb]}     {set verb 1}
if {[flagLoc $ARGV -noverb]}   {set verb 0}

#
#--------------------------------------------------------------------------------------------------------------

proc getPDBChainList { t var } \
{
   global multiFlag

   if {!$multiFlag} {return A}

   set eList [gdbGetInfo $t -parms entries]
   set cList ""

   foreach e $eList \
      {
       set chain [gdbGet $e $var]
       if {[lsearch -exact $cList $chain] < 0} {lappend cList $chain}
      }

   return $cList
}

proc getPDBResLimit { mode tab var chain } \
{
   global multiFlag

   if {$multiFlag} \
      {
       set eList [gdbSelect -from $tab -cond "!strcasecmp( $var, '$chain' )"]
      } \
   else \
      {
       set eList [gdbGetInfo $tab -parms entries]
      }

   set res [getEListStat $mode $eList RESID]

   return $res
}

proc isProton { entry } \
{
   set atomName [string toupper [gdbGet $entry ATOMNAME]]

   set key [string index $atomName 0]
   if {![strcasecmp $key H]} {return 1} 
   if {[lsearch -exact "0 1 2 3 4 5 6 7 8 9" $key] < 0} {return 0}

   set key [string index $atomName 1]
   if {![strcasecmp $key H]} {return 1} 
   if {[lsearch -exact "0 1 2 3 4 5 6 7 8 9" $key] < 0} {return 0}

   set key [string index $atomName 2]
   if {![strcasecmp $key H]} {return 1} 

   return 0
}

proc isSkipAtom { entry } \
{
   global minResI maxResI refVar multiFlag

   set aa3List \
    {
     ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE
     LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL
    }

   if {$multiFlag} \
      {
       set chain [gdbGet $entry $refVar] 
      } \
   else \
      {
       set chain A
      }

   if {![info exists maxResI($chain)]} {return 0}

   set atomName [string toupper [gdbGet $entry ATOMNAME]]
   set resID    [gdbGet $entry RESID]
   set resName  [string toupper [gdbGet $entry RESNAME]]
   set atomName [string toupper [gdbGet $entry ATOMNAME]]

   if {$resID != $maxResI($chain)} {return 0}
   if {[lsearch -exact $aa3List $resName] < 0} {return 0}
   if {[lsearch -exact "O OXT OT1 OT2" $atomName] < 0} {return 0}

   return 1
}

proc findThisAtom { entry pdbTab } \
{
   global multiFlag refVar inVar

   set aa3List \
    {
     ALA  ARG  ASN  ASP  CYS  GLN  GLU  GLY  HIS  ILE
     LEU  LYS  MET  PHE  PRO  SER  THR  TRP  TYR  VAL
    }

   set atomName [string toupper [gdbGet $entry ATOMNAME]]
   set resName  [string toupper [gdbGet $entry RESNAME]]
   set resID    [gdbGet $entry RESID]

   set cond "RESID == $resID && atomMatch( ATOMNAME, \"$atomName\" )"

   if {![strcasecmp $resName ILE] && (![strcasecmp $atomName CD] || ![strcasecmp $atomName CD1])} \
      {
       set cond "RESID == $resID && atomMatch( ATOMNAME, \"CD*\" )"
      } \
   elseif {(![strcasecmp $atomName H] || ![strcasecmp $atomName HN]) && [lsearch -exact $aa3List $resName] >= 0} \
      {
       set cond "RESID == $resID && (atomMatch( ATOMNAME, \"H\" ) || atomMatch( ATOMNAME, \"HN\" ))"
      }

   if {$multiFlag} \
      {
       set chain [gdbGet $entry $refVar]
       append cond " && !strcasecmp( $inVar, \"$chain\" )" 
      }

   set atomList [gdbSelect -from $pdbTab -cond $cond]

   return $atomList
}

if {$verbose && $addHFlag} \
   {
    puts ""
    puts "Evaluation of Proton Addition by Simulated Annealing"
    puts "Input: $refTitle"
    puts ""
   }

if {![file exists $refName]} \
   {
    puts stderr "Error Finding Reference PDB File $refTitle"
    exit 1
   }

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input PDB File $inTitle"
    exit 1
   }

if {$multiFlag} \
   {
    set multiArg "multi"
   } \
else \
   {
    set multiArg "nomulti"
   }

if {$alignFlag} \
   {
    catch "file delete $tmpName"
    catch "exec ov.tcl -ref $refName -in $inName -align $tmpName -heavy -noverb -$multiArg"

    if {![file exists $tmpName]} \
       {
        puts stderr "Error Creating Aligned Input From PDB File $inTitle"
        exit 1
       }

    set refTab   [gdbRead table -in $refName -pdb]
    set inTab    [gdbRead table -in $tmpName -pdb]
   } \
else \
   {
    set refTab   [gdbRead table -in $refName -pdb]
    set inTab    [gdbRead table -in $inName  -pdb]
   }

set chainListI [getPDBChainList $refTab $refVar]
set chainListJ [getPDBChainList $inTab  $inVar]

foreach chain $chainListI \
   { 
    set minResI($chain) [getPDBResLimit getMin $refTab $refVar $chain]
    set maxResI($chain) [getPDBResLimit getMax $refTab $refVar $chain]
   }

foreach chain $chainListJ \
   { 
    set minResJ($chain) [getPDBResLimit getMin $inTab $inVar $chain]
    set maxResJ($chain) [getPDBResLimit getMax $inTab $inVar $chain]
   }

set eListI [gdbGetInfo $refTab -parms entries]
set hListI ""

foreach eI $eListI \
   {
    if {[isProton $eI]} {lappend hListI $eI}
   }

set nA [llength $eListI]
set nH [llength $hListI]

if {$verbose} \
   {
    if {$multiFlag} \
       {
        if {$addHFlag} \
           {
            puts " Input:  $nH Protons, $nA Atoms Total."
           } \
        else \
           {
            puts ""
            puts " Reference: [format %-14s $refTitle] $nH Protons, $nA Atoms Total."
           }

        foreach chain $chainListI \
           {
            puts "  Chain $chain Residues $minResI($chain) to $maxResI($chain)"
           }

        puts ""
       } \
    else \
       {
        set chain [lindex $chainListI 0]

        if {$addHFlag} \
           {
            puts " Input:  Residues $minResI($chain) to $maxResI($chain), $nH Protons, $nA Atoms Total."
           } \
        else \
           {
            puts ""
            puts " Reference: [format %-14s $refTitle] Residues $minResI($chain) to $maxResI($chain), $nH Protons, $nA Atoms Total."
           }
       }
   }

if {$heavyFlag} {gdbFree $hListI}

set eListJ [gdbGetInfo $inTab -parms entries]
set hListJ ""

foreach eJ $eListJ \
   {
    if {[isProton $eJ]} {lappend hListJ $eJ}
   }
 
set nA [llength $eListJ]
set nH [llength $hListJ]

if {$verbose} \
   {
   if {$multiFlag} \
       {
        if {$addHFlag} \
           {
            puts " Result: $nH Protons, $nA Atoms Total."
           } \
        else \
           {
            puts ""
            puts " Input:     [format %-14s $refTitle] $nH Protons, $nA Atoms Total."
           }

        foreach chain $chainListI \
           {
            puts "  Chain $chain Residues $minResJ($chain) to $maxResJ($chain)"
           }
       } \
    else \
       {
        set chain [lindex $chainListI 0]

        if {$addHFlag} \
           {
            puts " Result: Residues $minResJ($chain) to $maxResJ($chain), $nH Protons, $nA Atoms Total."
           } \
        else \
           {
            puts " Input:     [format %-14s $inTitle] Residues $minResJ($chain) to $maxResJ($chain), $nH Protons, $nA Atoms Total."
           }
       }

    puts ""
   }

if {$heavyFlag} {gdbFree $hListJ}

set missList ""
set skipList ""
set dupList  ""
set badList  ""
set rList    ""

set refList [gdbGetInfo $refTab -parms entries]

foreach eI $refList \
   {
    set atomList [findThisAtom $eI $inTab] 
    set n [llength $atomList]

    if {!$n} \
       {
        if {[isSkipAtom $eI]} \
           {
            lappend skipList $eI
           } \
        else \
           {
            lappend missList $eI
           }

        continue
       } \
    elseif {$n > 1} \
       { 
        lappend dupList $eI
       }

    set eJ [lindex $atomList 0]

    set xI [gdbGet $eI X]
    set yI [gdbGet $eI Y]
    set zI [gdbGet $eI Z]

    set xJ [gdbGet $eJ X]
    set yJ [gdbGet $eJ Y]
    set zJ [gdbGet $eJ Z]

    set dx [expr $xI - $xJ]
    set dy [expr $yI - $yJ]
    set dz [expr $zI - $zJ]

    gdbSet $eI X $dx
    gdbSet $eI Y $dy
    gdbSet $eI Z $dz

    set r  [expr sqrt( $dx*$dx + $dy*$dy + $dz*$dz )]

    if {$r > $thresh} \
       {
        lappend badList $eI,$r
       }

    lappend rList $r
   }

set nM [llength $missList]
set nS [llength $skipList]
set nD [llength $dupList]
set nB [llength $badList]
set nR [llength $rList]

if {$verbose && $nM} \
   {
    puts " Atoms in Reference Missing in Input:"

    foreach eI $missList \
       {
        set fmt "  %3d %4s %4s"
        if {$verbose} {puts [format $fmt [gdbGet $eI RESID] [gdbGet $eI RESNAME] [gdbGet $eI ATOMNAME]]}
       }
   }

if {$verbose && $nM} \
   {
    puts " Atoms in Reference Ignored:"

    foreach eI $skipList \
       {
        set fmt "  %3d %4s %4s"
        if {$verbose} {puts [format $fmt [gdbGet $eI RESID] [gdbGet $eI RESNAME] [gdbGet $eI ATOMNAME]]}
       }
   }

if {$verbose && $nD} \
   {
    puts " Redundant Atoms in Input:"

    foreach eI $dupList \
       {
        set fmt "  %3d %4s %4s"
        if {$verbose} {puts [format $fmt [gdbGet $eI RESID] [gdbGet $eI RESNAME] [gdbGet $eI ATOMNAME]]}
       }
   }


if {$verbose && $nB} \
   {
    puts " Atoms With Displacements Greater Than $thresh A:"

    foreach b $badList \
       {
        set bList [split $b ","]
        set eI    [lindex $bList 0]
        set r     [lindex $bList 1]

        set fmt "  %3d %4s %4s   %5.3f A"
        if {$verbose} {puts [format $fmt [gdbGet $eI RESID] [gdbGet $eI RESNAME] [gdbGet $eI ATOMNAME] $r]}
       }
   }

if {$nM || $nD || $nB} \
   {
    if {$verbose} {puts ""}
   }
 
gdbFree $missList
gdbFree $skipList

set eListI [gdbGetInfo $refTab -parms entries]
set id 0

foreach eI $eListI \
   {
    incr id
    gdbSet $eI ATOMID $id  
   }

if {$verbose} \
   {
    puts [format "Atoms Matched:   %5d"   $nR]
    puts [format "Missing Atoms:   %5d"   $nM]
    puts [format "Ignored Atoms:   %5d"   $nS]
    puts [format "Duplicate Atoms: %5d"   $nD]
    puts [format "Over Threshold:  %5d"   $nB]
    puts ""
    puts [format "Displacement Threshold: %5.3f A" $thresh]
    puts [format "Largest Displacement:   %5.3f A" [getMax $rList]]
    puts [format "RMS Displacement:       %5.3f A" [getRMS $rList]]
   } \
else \
   {
    puts "$nB [format %.3f [getMax $rList]] [format %.3f [getRMS $rList]]"
   }

gdbWrite $refTab $outName -pdb

if {$verbose && $addHFlag} \
   {
    puts ""
   }

if {$addHFlag && $nB} {set error 1}

exit $error 
