#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# Show Help Text:

if {$ARGC < 3 || [flagLoc $ARGV -help]} \
   {
    puts "Convert Complex NMRPipe FID to 2D PE-COSY FID:"
    puts " -in  inName     \[test.fid\]     Nx2 Complex FID Input."
    puts " -out outName2D  \[peCOSY.fid\]   2D Output File."
    puts " -xSize xSize    \[0\]            Final X-Axis Size (Zero for Auto)"
    puts " -ySize ySize    \[0\]            Final Y-Axis Size (Zero for Auto)"
    puts "Notes:"
    puts " 1. Sizes are in complex points."
    puts " 2. Auto sizes are half of original 1D complex size."
    exit 1
   }

#
# Extract Command line:

set inName  test.fid 
set outName peCOSY.fid 

set inUnit  0
set outUnit 1

set xT      0
set yT      0

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -xSize xT
getArgD $ARGV -ySize yT

#
# Prepare input file:

if {[strcasecmp $inName None]} {set inUnit [sysOpen $inName r]}

if {$inUnit < 0} \
   {
    puts "Error opening input file."
    exit 1
   }

#
# Prepare output file:

if {[strcasecmp $outName None]} {set outUnit [sysOpen $outName w]}

if {$outUnit < 0} \
   {
    puts "Error opening output file."
    exit 1
   }

#
# Read input header, create output header:

set fdataOut  [sysMalloc $FDATASIZE float]

set error [rdFDATAU $inUnit fdata]

#
# Find values for output header:

set xnSize [getParm $fdata NDSIZE CUR_XDIM]
set xtSize [getParm $fdata NDAPOD CUR_XDIM]
set xSW    [getParm $fdata NDSW   CUR_XDIM]
set xOBS   [getParm $fdata NDOBS  CUR_XDIM]
set xCAR   [getParm $fdata NDCAR  CUR_XDIM]

set ySW  $xSW
set yOBS $xOBS
set yCAR $xCAR

if {!$xT} {set xT [expr int($xtSize/2)]}
if {!$yT} {set yT [expr int($xtSize/2)]}

set xN [expr 2*$xT]
set yN [expr 2*$yT]

#
# Update and write output header:

buildHdr $fdataOut \
  -xN         $xN   -yN        $yN  \
  -xT         $xT   -yT        $yT  \
  -xMODE  Complex   -yMODE Complex  \
  -xSW       $xSW   -ySW      $ySW  \
  -xOBS     $xOBS   -yOBS    $yOBS  \
  -xCAR     $xCAR   -yCAR    $yCAR  \
  -xLAB        F2   -yLAB       F1  \
  -ndim         2   -aq2D   States

set error [catch {sysWrite $outUnit $fdataOut $FDATASIZE}]

if {$error} \
   {
    puts "Error writing header data."
    exit $error
   }

#
# Allocate input buffers:

set buffSize [expr $xnSize*2]

set rdataR [sysMalloc $buffSize float]
set idataR [sysMalloc $buffSize float]
set rdataI [sysMalloc $buffSize float]
set idataI [sysMalloc $buffSize float]

vsSet $rdataR 0.0 $buffSize
vsSet $idataR 0.0 $buffSize
vsSet $rdataI 0.0 $buffSize
vsSet $idataI 0.0 $buffSize

#
# Read the four channels:

sysRead $inUnit $rdataR $xnSize
sysRead $inUnit $idataR $xnSize

vvMove $idataI $rdataR $xnSize
vvMove $rdataI $idataR $xnSize

vNeg $idataI $xnSize

#
# Write the 2D data:

set rPtrR $rdataR
set iPtrR $idataR
set rPtrI $rdataI
set iPtrI $idataI

for {set iy 0} {$iy < $yT} {incr iy} \
   {
    sysWrite $outUnit $rPtrR $xT 
    sysWrite $outUnit $iPtrR $xT 
    sysWrite $outUnit $rPtrI $xT 
    sysWrite $outUnit $iPtrI $xT 

    set rPtrR [incrPtr $rPtrR]
    set iPtrR [incrPtr $iPtrR]
    set rPtrI [incrPtr $rPtrI]
    set iPtrI [incrPtr $iPtrI]
   }

exit
