#!/bin/csh

if ($#argv != 2) then
   echo "Convert NMRPipe format data to SPIFF format:"
   echo "Usage:"
   echo "  pipe2spiff.com inDirName  spiffName    (Assumes ir%03d.dat)"
   echo "  pipe2spiff.com inTemplate spiffName"
   echo "  pipe2spiff.com inName     spiffName    (Assumes 2D image input)"
   echo "Examples:"
   echo "  pipe2spiff.com sili2_ft sili2.spc"
   echo "  pipe2spiff.com sili2_ft/test%03d.dat sili2.spc"
   echo "  pipe2spiff.com dist001.dat dist001.spc"
   exit 0
endif

if (-d $1) then
   echo "Input is specified as a directory ..."

   if (!(-e $1/ir001.dat)) then
      echo "Error: the expected file $1/ir001.dat was not found."
      exit 1
   endif

   xyz2pipe -in $1/ir%03d.dat | pipe2spiff -out $2 -verb -ov
   exit 0
endif

if (-e $1) then
   echo "Input is specified as a file ..."
   
   pipe2spiff -in $1 -out $2 -verb -ov
   exit 0
endif

set a = (`echo $1 | fgrep "%03d"`)

if ($a == $1) then
   echo "Input is specified as a template ..."

   set a = (`nmrPrintf $1 1`)

   if (!(-e $a)) then
      echo "Error: the expected file $a was not found."
      exit 1
   endif

   xyz2pipe -in $1 | pipe2spiff -out $2 -verb -ov
   exit 0
endif

