#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc showViewerHelp args \
{
   puts "Plot Geometry and Details:"
   puts " -sxw       sxw    \[500\]      Initial Contour Plot X-Size, Pixels."
   puts " -syw       syw    \[500\]      Initial Contour Plot Y-Size, Pixels."
   puts " -charX     cx     \[10\]       Character Width, Pixels." 
   puts " -charY     cy     \[16\]       Character Height, Pixels." 
   puts " -axis                        Draw Axis (Default)."
   puts " -noaxis                      No Axis."
   puts " -grid                        Draw Grid."
   puts " -nogrid                      No Grid (Default)."
   puts "Plot Colors:"
   puts " -bgColor     bg   \[black\]    Plot Background Color."
   puts " -winColor    wc   \[gray30\]   Window Background Color."
   puts " -gridColor   gc   \[dimGray\]  Plot Grid Color."
   puts " -boxColor    bc   \[yellow\]   Plot Box Color."
   puts " -axisColor   ac   \[yellow\]   Axis Color."
   puts " -shadowColor sc   \[black\]    Axis Shadow Color."

   return 0
}

proc getViewerArgs { obj } \
{ 
   global objInfo ARGV

   set thisARGV $ARGV

   set objInfo($obj,canvas)     .canvas$obj
   set objInfo($obj,pixmap)     pixmap$obj 
   set objInfo($obj,toplevel)   .
   set objInfo($obj,win)        1

   set objInfo($obj,inList)     ""
   set objInfo($obj,specCount)  0

   set objInfo($obj,tabList)    ""
   set objInfo($obj,tabCount)   0

   set objInfo($obj,hi)         5%

   set objInfo($obj,specX1)     0%
   set objInfo($obj,specXN)     100%

   set objInfo($obj,specY1)     0%
   set objInfo($obj,specYN)     100%

   set objInfo($obj,specZ1)     0%
   set objInfo($obj,specZN)     100%

   set objInfo($obj,specA1)     0%
   set objInfo($obj,specAN)     100%

   set objInfo($obj,gridFlag)   0
   set objInfo($obj,axisFlag)   1

   set objInfo($obj,sxw) 400
   set objInfo($obj,syw) 400

   set objInfo($obj,charX) 10
   set objInfo($obj,charY) 16

   set objInfo($obj,leftMargin)   10
   set objInfo($obj,rightMargin)  80
   set objInfo($obj,topMargin)    10
   set objInfo($obj,bottomMargin) 80

   set objInfo($obj,bgColor)      black
   set objInfo($obj,winColor)     gray30
   set objInfo($obj,gridColor)    dimGray
   set objInfo($obj,axisColor)    yellow
   set objInfo($obj,fgColor)      yellow
   set objInfo($obj,shadowColor)  black

   set objInfo($obj,posColorA)    green 
   set objInfo($obj,negColorA)    yellow 
   set objInfo($obj,posColorB)    darkOrchid 
   set objInfo($obj,negColorB)    magenta 

   getArgD $thisARGV -toplevel        objInfo($obj,toplevel)
   getArgD $thisARGV -canvas          objInfo($obj,canvas)
   getArgD $thisARGV -win             objInfo($obj,win)
   getArgD $thisARGV -pixmap          objInfo($obj,pixmap)

   getArgD $thisARGV -leftMargin      objInfo($obj,leftMargin)
   getArgD $thisARGV -rightMargin     objInfo($obj,rightMargin)
   getArgD $thisARGV -bottomMargin    objInfo($obj,bottomMargin)
   getArgD $thisARGV -topMargin       objInfo($obj,topMargin)

   getArgD $thisARGV -bgColor         objInfo($obj,bgColor)
   getArgD $thisARGV -winColor        objInfo($obj,winColor)
   getArgD $thisARGV -gridColor       objInfo($obj,gridColor)
   getArgD $thisARGV -boxColor        objInfo($obj,fgColor)
   getArgD $thisARGV -axisColor       objInfo($obj,axisColor)
   getArgD $thisARGV -shadowColor     objInfo($obj,shadowColor)

   getArgD $thisARGV -x1 objInfo($obj,specX1)
   getArgD $thisARGV -xn objInfo($obj,specXN)

   getArgD $thisARGV -y1 objInfo($obj,specY1)
   getArgD $thisARGV -yn objInfo($obj,specYN)

   getArgD $thisARGV -z1 objInfo($obj,specZ1)
   getArgD $thisARGV -zn objInfo($obj,specZN)

   getArgD $thisARGV -a1 objInfo($obj,specA1)
   getArgD $thisARGV -an objInfo($obj,specAN)

   if {[flagLoc $thisARGV -grid]}     {set objInfo($obj,gridFlag) 1}
   if {[flagLoc $thisARGV -nogrid]}   {set objInfo($obj,gridFlag) 0}
   
   if {[flagLoc $thisARGV -axis]}     {set objInfo($obj,axisFlag) 1}
   if {[flagLoc $thisARGV -noaxis]}   {set objInfo($obj,axisFlag) 0}

   getListArgD $thisARGV -in          objInfo($obj,inList)
   getListArgD $thisARGV -tab         objInfo($obj,tabList)

   getArgD     $thisARGV -hi          objInfo($obj,hi)

   return 0
}

proc setViewerCanvas { obj } \
{
   global objInfo 

   update

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,toplevel)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,toplevel)]

   $objInfo($obj,canvas) configure -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize)

   update

   set objInfo($obj,sxw) \
    [expr ($objInfo($obj,winXSize) - (2*$objInfo($obj,charX) + $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)))/3]

   set objInfo($obj,syw) \
    [expr $objInfo($obj,winYSize) - ($objInfo($obj,topMargin) + $objInfo($obj,bottomMargin))]

   set objInfo($obj,cx1) $objInfo($obj,leftMargin)
   set objInfo($obj,cxn) [expr $objInfo($obj,cx1) + $objInfo($obj,sxw)] 
   set objInfo($obj,cy1) $objInfo($obj,topMargin)
   set objInfo($obj,cyn) [expr $objInfo($obj,cy1) + $objInfo($obj,syw)]

   set objInfo($obj,cx1A) [expr $objInfo($obj,cxn)  + $objInfo($obj,charX)]
   set objInfo($obj,cxnA) [expr $objInfo($obj,cx1A) + $objInfo($obj,sxw)] 
   set objInfo($obj,cy1A) $objInfo($obj,cy1) 
   set objInfo($obj,cynA) $objInfo($obj,cyn)

   set objInfo($obj,cx1B) [expr $objInfo($obj,cxnA)  + $objInfo($obj,charX)]
   set objInfo($obj,cxnB) [expr $objInfo($obj,cx1B) + $objInfo($obj,sxw)]
   set objInfo($obj,cy1B) $objInfo($obj,cy1)
   set objInfo($obj,cynB) $objInfo($obj,cyn)

   $objInfo($obj,canvas) delete pixmap -tags $objInfo($obj,pixmap)

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) \
      -win $objInfo($obj,win) -tags $objInfo($obj,pixmap)

   defineCell -x $objInfo($obj,cx1)  $objInfo($obj,cxn)  -y $objInfo($obj,cy1)  $objInfo($obj,cyn)  -cell 1 -win $objInfo($obj,win)
   defineCell -x $objInfo($obj,cx1A) $objInfo($obj,cxnA) -y $objInfo($obj,cy1A) $objInfo($obj,cynA) -cell 2 -win $objInfo($obj,win)
   defineCell -x $objInfo($obj,cx1B) $objInfo($obj,cxnB) -y $objInfo($obj,cy1B) $objInfo($obj,cynB) -cell 3 -win $objInfo($obj,win)
 
   update

   bind $objInfo($obj,canvas) <Configure> "resizeViewer $obj %w %h %T"

   return 0
}

proc resizeViewer args \
{
   global objInfo

   set obj [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   setViewerCanvas   $obj
   drawViewerGraphic $obj
}

proc getViewerTab { obj } \
{
   global objInfo tabInfo

   set objInfo($obj,tabCount) 0
   set i 0

   set xList ""
   set yList ""
   set nList ""

   foreach tabName $objInfo($obj,tabList) \
      {
       if {![file exists $tabName]} \
          {
           puts stderr "Viewer Error Finding Input Table $tabName"
           exit 1
          }

       incr objInfo($obj,tabCount)
       incr i

       catch "unset tInfo"

       set tInfo(lineCount) 0

       set lineCount [getTextLines $tabName tInfo]

       lappend nList $lineCount

       if {$lineCount < 1} \
          {
           puts stderr "Viewer Error Reading Data from Input Table $tabName"
           exit 1
          }

       for {set j 0} {$j < $lineCount} {incr j} \
          {
           set tabInfo($i,INDEX)  [lindex $tInfo($j) 0]
           set tabInfo($i,X_PPM)  [lindex $tInfo($j) 1]
           set tabInfo($i,Y_PPM)  [lindex $tInfo($j) 2]
           set tabInfo($i,HEIGHT) [lindex $tInfo($j) end]

           lappend xList $tabInfo($i,X_PPM)
           lappend yList $tabInfo($i,Y_PPM)
          }

       set tabInfo($i,pkCount) $lineCount
      }

   set tabInfo(xMinPPM) [getMin $xList]
   set tabInfo(xMaxPPM) [getMax $xList]
   set tabInfo(yMinPPM) [getMin $yList]
   set tabInfo(yMaxPPM) [getMax $yList]

   if {![strcasecmp $objInfo($obj,specX1) Auto]} \
      {
       set objInfo($obj,specX1) [expr $tabInfo(xMaxPPM) + 0.05*($tabInfo(xMaxPPM) - $tabInfo(xMinPPM))]ppm
      }

   if {![strcasecmp $objInfo($obj,specXN) Auto]} \
      { 
       set objInfo($obj,specXN) [expr $tabInfo(xMinPPM) - 0.05*($tabInfo(xMaxPPM) - $tabInfo(xMinPPM))]ppm
      } 

   if {![strcasecmp $objInfo($obj,specY1) Auto]} \
      { 
       set objInfo($obj,specY1) [expr $tabInfo(yMaxPPM) + 0.05*($tabInfo(yMaxPPM) - $tabInfo(yMinPPM))]ppm
      } 

   if {![strcasecmp $objInfo($obj,specYN) Auto]} \
      {
       set objInfo($obj,specYN) [expr $tabInfo(yMinPPM) - 0.05*($tabInfo(yMaxPPM) - $tabInfo(yMinPPM))]ppm
      }

   set tabInfo(pkCount) [getMax $nList]

   return 0
}

proc getViewerData { obj } \
{
   global objInfo 

   if {![llength $objInfo($obj,inList)]} \
      {
       puts stderr "Viewer Error: No Input Spectra Specified."
       exit 1
      }

   if {![strcasecmp $objInfo($obj,specX1) Auto]} {set objInfo($obj,specX1) 0%}
   if {![strcasecmp $objInfo($obj,specXN) Auto]} {set objInfo($obj,specXN) 100%}

   if {![strcasecmp $objInfo($obj,specY1) Auto]} {set objInfo($obj,specY1) 0%}
   if {![strcasecmp $objInfo($obj,specYN) Auto]} {set objInfo($obj,specYN) 100%}

   set objInfo($obj,specCount) 0
   set roiID 0

   foreach inName $objInfo($obj,inList) \
      {
       if {![file exists $inName]} \
          {
           puts stderr "Viewer Error Finding Input Spectrum $inName"
           exit 1
          }

       incr roiID
       incr objInfo($obj,specCount)

       set objInfo($obj,sizeInfo,$roiID)   [getSpecSizeInfo $inName]
       set objInfo($obj,dimCount,$roiID)   [lindex $objInfo($obj,sizeInfo,$roiID) 1]
       set objInfo($obj,pkTab,$roiID)      [lindex $objInfo($obj,sizeInfo,$roiID) 1]

       if {$objInfo($obj,dimCount,$roiID) == 1} \
          {
           readROI -ndim 1 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN)
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 2} \
          {
           readROI -ndim 2 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN)
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 3} \
          {
           readROI -ndim 2 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN) \
                   -z Z_AXIS 0%                    100%
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 4} \
          {
           readROI -ndim 2 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN) \
                   -z 0%                           100%                  \
                   -a 0%                           100%
          }

       set rInfo [getROIInfo -roi $roiID]

       set objInfo($obj,rdata,$roiID) [lindex $rInfo 0]
       set objInfo($obj,fdata,$roiID) [lindex $rInfo 1]

       set objInfo($obj,title,$roiID) [getParmStr $objInfo($obj,fdata,$roiID) FDTITLE NULL_DIM]
       if {![string length $objInfo($obj,title,$roiID)]} {set objInfo($obj,title,$roiID) None}
      }

   return 0
}

proc drawViewerGraphic { obj } \
{
   global objInfo

   set objInfo($obj,bgColor)      black
   set objInfo($obj,winColor)     gray30
   set objInfo($obj,gridColor)    dimGray
   set objInfo($obj,axisColor)    yellow
   set objInfo($obj,fgColor)      yellow
   set objInfo($obj,shadowColor)  black

   drawRec -fg $objInfo($obj,winColor) -x 0 $objInfo($obj,winXSize) -y 0 $objInfo($obj,winYSize) -win $objInfo($obj,win)

   drawROI -roi 1 -cell 1 -axis Bottom -hi $objInfo($obj,hi) -fg $objInfo($obj,fgColor) -bg $objInfo($obj,bgColor) \
           -colorG $objInfo($obj,gridColor) -colorA $objInfo($obj,axisColor) -colorS $objInfo($obj,shadowColor)    \
           -color1 $objInfo($obj,posColorA) -color2 $objInfo($obj,negColorA)

   drawROI -roi 2 -cell 1 -hi $objInfo($obj,hi) -fg $objInfo($obj,fgColor) -bg None \
           -colorG $objInfo($obj,gridColor) -colorA $objInfo($obj,axisColor) -colorS $objInfo($obj,shadowColor) \
           -color1 $objInfo($obj,posColorB) -color2 $objInfo($obj,negColorB)

   drawROI -roi 1 -cell 2 -axis Bottom -hi $objInfo($obj,hi) -fg $objInfo($obj,fgColor) -bg $objInfo($obj,bgColor) \
           -colorG $objInfo($obj,gridColor) -colorA $objInfo($obj,axisColor) -colorS $objInfo($obj,shadowColor)


   drawROI -roi 2 -cell 3 -axis Bottom Right -hi $objInfo($obj,hi) -fg $objInfo($obj,fgColor) -bg $objInfo($obj,bgColor) \
           -colorG $objInfo($obj,gridColor) -colorA $objInfo($obj,axisColor) -colorS $objInfo($obj,shadowColor)
}

proc waitForAlignment { obj } \
{
}

#
#-----------------------------------------------------------------------

set obj pkAlignView1

if {[flagLoc $ARGV -help]} \
   {
    puts "Viewer for Peak Table Matching:"
    puts " -in        inName1  inName2  \[None\]  Input Spectrum Pair."
    puts " -tab       tabName1 tabName2 \[None\]  Input Peak List Pair."
    puts " -hi        hi                \[5%\]    Contour Drawing Height."
    puts " -dir       dirName           \[match\] Input Directory of Alignment Data."
    puts "Spectral Region (-x -y -z -a):"
    puts " -x1        specX1 \[0%\]       Spectral Region Start."
    puts " -xn        specXN \[100%\]     Spectral Region End."
    showViewerHelp $obj
    exit 0
   }

wm geom . +20+20

if {[getViewerArgs $obj]} {exit 1}
if {[getViewerTab  $obj]} {exit 1}
if {[getViewerData $obj]} {exit 1}

if {$objInfo($obj,specCount) != 2} \
   {
    puts stderr "Viewer Error: Two Input Spectra are Required."
    exit 1
   }

if {$objInfo($obj,tabCount) != 2} \
   {
    puts stderr "Viewer Warning: Two Input Peak Tables are Required."
   }

set objInfo($obj,winXSize) \
 [expr 3*$objInfo($obj,sxw) + 2*$objInfo($obj,charX) + $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]

set objInfo($obj,winYSize) \
 [expr $objInfo($obj,syw) + $objInfo($obj,charY) + $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

canvas $objInfo($obj,canvas) -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) -border 0
pack   $objInfo($obj,canvas) -side top -expand 1 -fill both

update

setViewerCanvas   $obj
drawViewerGraphic $obj
waitForAlignment  $obj

