#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
  {
   puts "Detect Peaks in a 4D Spectrum Over One or More Regions:"
   puts " -in   inName   \[ft/test%03d.ft4\] Input Spectrum."
   puts " -out  tabName  \[test.tab\]        Peak Table Output."
   puts " -tmp  tmpName  \[tmp.tab\]         Temporary Output."
   puts " -reg  regList  \[0% 100% ...\]     List of Analysis Region Limits."
   puts " -verb                             Verbose Mode ON (Default)."
   puts " -noverb                           Verbose Mode OFF."
   puts "Noise Analysis:"
   puts " -noiseX1 nx1   \[None\]    X-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseXN nxn   \[None\]    X-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noiseY1 ny1   \[None\]    Y-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseYN nyn   \[None\]    Y-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noiseZ1 nz1   \[None\]    Z-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseZN nzn   \[None\]    Z-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noiseA1 na1   \[None\]    A-Axis Start of Signal-Free Region (pts ppm %)."
   puts " -noiseAN nan   \[None\]    A-Axis End of Signal-Free Region (pts ppm %)."
   puts " -noise   sigma \[Auto\]    Noise Value."
   puts "Peak Detection Parameters:"
   puts " -maxVal maxVal \[Auto\]    Maximum Value in Spectrum."
   puts " -thresh t      \[Auto\]    Peak Detection Height Threshold."
   puts " -mult   m      \[6.0\]     Multiplication Factor for -thresh Auto."
   puts " -ddx    ddx    \[Auto\]    X-Axis +/- Width of Detection Region, Pts."
   puts " -ddy    ddy    \[Auto\]    Y-Axis +/- Width of Detection Region, Pts."
   puts " -ddz    ddz    \[Auto\]    Z-Axis +/- Width of Detection Region, Pts."
   puts " -dda    dda    \[Auto\]    A-Axis +/- Width of Detection Region, Pts."
   puts " -idx    idx    \[Auto\]    X-Axis +/- Width of Interpolation Region, Pts."
   puts " -idy    idy    \[Auto\]    Y-Axis +/- Width of Interpolation Region, Pts."
   puts " -idz    idz    \[Auto\]    Z-Axis +/- Width of Interpolation Region, Pts."
   puts " -ida    ida    \[Auto\]    A-Axis +/- Width of Interpolation Region, Pts."
   puts " -tolx   tolx   \[Auto\]    Maximum Allowable X-Axis Position Change, +/- Pts."
   puts " -toly   toly   \[Auto\]    Maximum Allowable Y-Axis Position Change, +/- Pts."
   puts " -tolz   tolz   \[Auto\]    Maximum Allowable Z-Axis Position Change, +/- Pts."
   puts " -tola   tola   \[Auto\]    Maximum Allowable A-Axis Position Change, +/- Pts."
   puts " -edge                    Expand Analysis Region to Detect Peaks at Edges."
   puts "Options for nmrWish pkFindROI Command (Should be Last on Command Line):"
   puts " -pkArgs args   \[None\]    Arguments for pkFindROI."
   puts "Notes:"
   puts " 1. Limits for -reg are specified in sets of 8 values x1 xn y1 yn z1 zn a1 an."
   puts " 2. The -ddx -dd... options specify how many points are considered when testing"
   puts "    whether a given point could be a peak because it is larger than all of its"
   puts "    neighboring points."
   puts " 3. If no noise region is specified via -noiseX1 -noiseXN ... and"
   puts "    no noise value is specified via the -noise option, the noise will be"
   puts "    estimated automatically from the distribution of spectral intensities."
   puts " 4. If -thresh Auto is used, the threshold will be set to the noise value"
   puts "    times the multiplication factor (-mult)."
   puts " 5. If a percent value is used for -thresh (such as -thresh 10%) then"
   puts "    the detection threshold will be set as a percentage of the highest"
   puts "    value in the entire spectrum."
   puts " 6. 4D Peak detection can be memory and resource intensive. These steps might"
   puts "    improve performance:"
   puts ""
   puts "       o Turn off mask analysis (peak overlap analysis) via -pkArgs -nomask."
   puts ""
   puts "       o Use -maxVal ... to avoid reading the entire spectrum in order to"
   puts "         to determine the maximum."
   puts ""
   puts "       o Use -noise ... to explicitly specify noise rather then determining"
   puts "         it automatically."
   puts ""
   puts "       o Perform the detection over multiple smaller regions via -reg ..."
   exit 1
  }

set inName     ft/test%03d.ft4
set tabName    test.tab
set tmpName    tmp.tab
set pkArgs     None

set noiseRMS   Auto
set maxVal     Auto
set thresh     Auto

set noiseX1    None
set noiseXN    None
set noiseY1    None
set noiseYN    None
set noiseZ1    None
set noiseZN    None
set noiseA1    None
set noiseAN    None

set threshMult 6.0

set regList    "0% 100% 0% 100% 0% 100% 0% 100%"

set ddx        Auto 
set ddy        Auto 
set ddz        Auto 
set dda        Auto 

set idx        Auto 
set idy        Auto 
set idz        Auto 
set ida        Auto 

set tolx       Auto 
set toly       Auto 
set tolz       Auto 
set tola       Auto 

set verbose    1
set edgeFlag   0

set loc [flagLoc $ARGV -pkArgs]

if {$loc > 0} \
   {
    set pkArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV   [lrange $ARGV 0 [expr $loc - 1]]
   }

if {![strcasecmp $pkArgs None]} {set pkArgs ""}

getListArgD $ARGV -reg regList

getArgD $ARGV -in  inName
getArgD $ARGV -out tabName
getArgD $ARGV -tab tabName

getArgD $ARGV -ddx  ddx
getArgD $ARGV -ddy  ddy
getArgD $ARGV -ddz  ddz
getArgD $ARGV -dda  dda

getArgD $ARGV -idx  idx
getArgD $ARGV -idy  idy
getArgD $ARGV -idz  idz
getArgD $ARGV -ida  ida

getArgD $ARGV -tolx tolx
getArgD $ARGV -toly toly
getArgD $ARGV -tolz tolz
getArgD $ARGV -tola tola

getArgD $ARGV -noiseX1 noiseX1
getArgD $ARGV -noiseXN noiseXN

getArgD $ARGV -noiseY1 noiseY1
getArgD $ARGV -noiseYN noiseYN

getArgD $ARGV -noiseZ1 noiseZ1
getArgD $ARGV -noiseZN noiseZN

getArgD $ARGV -noiseA1 noiseA1
getArgD $ARGV -noiseAN noiseAN

getArgD $ARGV -noise  noiseRMS
getArgD $ARGV -maxVal maxVal
getArgD $ARGV -thresh thresh
getArgD $ARGV -mult   threshMult 

if {[flagLoc $ARGV -edge]}    {set edgeFlag 1}
if {[flagLoc $ARGV -noedge]}  {set edgeFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

if {$verbose} \
   {
    set vArg "verb"
   } \
else \
   {
    set vArg "noverb"
   }

#
#--------------------------------------------------------------------------

set fdata 0

set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "4D Peak Detection. Error Finding Input Spectrum $inName"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {$error} \
   {
    puts stderr "4D Peak Detection. Error Reading Header from $thisName"
    exit 1
   }

set xtSize [getParmI $fdata NDTDSIZE CUR_XDIM]
set ytSize [getParmI $fdata NDTDSIZE CUR_YDIM]
set ztSize [getParmI $fdata NDTDSIZE CUR_ZDIM]
set atSize [getParmI $fdata NDTDSIZE CUR_ADIM]

set xfSize [getParmI $fdata NDFTSIZE CUR_XDIM]
set yfSize [getParmI $fdata NDFTSIZE CUR_YDIM]
set zfSize [getParmI $fdata NDFTSIZE CUR_ZDIM]
set afSize [getParmI $fdata NDFTSIZE CUR_ADIM]

if {$xtSize > 0 && $xfSize > 0} \
   {
    set dxDef   [expr int(sqrt( 0.5*double($xfSize)/double($xtSize) ))]
    set xTolDef [expr 0.5*double($xfSize)/double($xtSize)]

    if {$dxDef < 1}     {set dxDef   1}
    if {$xTolDef < 2.0} {set xTolDef 2.0}
   } \
else \
   {
    set dxDef   1
    set xTolDef 2.0
   }

if {$ytSize > 0 && $yfSize > 0} \
   {
    set dyDef   [expr int(sqrt( 0.5*double($yfSize)/double($ytSize) ))]
    set yTolDef [expr 0.5*double($yfSize)/double($ytSize)]

    if {$dyDef < 1}     {set dyDef   1}
    if {$yTolDef < 2.0} {set yTolDef 2.0}
   } \
else \
   {
    set dyDef   1
    set yTolDef 2.0
   }

if {$ztSize > 0 && $zfSize > 0} \
   {
    set dzDef   [expr int(sqrt( 0.5*double($zfSize)/double($ztSize) ))]
    set zTolDef [expr 0.5*double($zfSize)/double($ztSize)]

    if {$dzDef < 1}     {set dzDef   1}
    if {$zTolDef < 2.0} {set zTolDef 2.0}
   } \
else \
   {
    set dzDef   1
    set zTolDef 2.0
   }

if {$atSize > 0 && $afSize > 0} \
   {
    set daDef   [expr int(sqrt( 0.5*double($afSize)/double($atSize) ))]
    set aTolDef [expr 0.5*double($afSize)/double($atSize)]

    if {$daDef < 1}     {set daDef   1}
    if {$aTolDef < 2.0} {set aTolDef 2.0}
   } \
else \
   {
    set daDef   1
    set aTolDef 2.0
   }

set n [llength $regList]

if {$n % 8} \
   {
    puts stderr "4D Peak Detection. Error: Region List -reg Must Have a Multiple of Eight Values."
    puts stderr $regList
    exit 1
   }

if {![strcasecmp $ddx  Auto]} {set ddx  $dxDef}
if {![strcasecmp $idx  Auto]} {set idx  $dxDef}
if {![strcasecmp $tolx Auto]} {set tolx $xTolDef}

if {![strcasecmp $ddy  Auto]} {set ddy  $dyDef}
if {![strcasecmp $idy  Auto]} {set idy  $dyDef}
if {![strcasecmp $toly Auto]} {set toly $yTolDef}

if {![strcasecmp $ddz  Auto]} {set ddz  $dzDef}
if {![strcasecmp $idz  Auto]} {set idz  $dzDef}
if {![strcasecmp $tolz Auto]} {set tolz $zTolDef}

if {![strcasecmp $dda  Auto]} {set dda  $daDef}
if {![strcasecmp $ida  Auto]} {set ida  $daDef}
if {![strcasecmp $tola Auto]} {set tola $aTolDef}

#
# Noise estimate:

if {[lsearch -exact [string tolower "$noiseX1 $noiseXN $noiseY1 $noiseYN $noiseZ1 $noiseZN $noiseA1 $noiseAN"] none] >= 0} \
   {
    if {![strcasecmp $noiseRMS Auto]} \
       {
        set noiseRMS [estSpecNoiseND $inName]
        set noiseStr " (Auto)"
       } \
    else \
       {
        set noiseStr " (User-Supplied)"
       }
   } \
else \
   {
    readROI -roi 1 -in $inName -ndim 3 -x X_AXIS $noiseX1 $noiseXN -y Y_AXIS $noiseY1 $noiseYN -z Z_AXIS $noiseZ1 $noiseZN -a A_AXIS $noiseA1 $noiseAN

    set noiseStr " (RMS of X: $noiseX1 to $noiseXN Y: $noiseY1 to $noiseYN Z: $noiseZ1 to $noiseZN A: $noiseA1 $noiseAN)"
    set noiseRMS [getROIStat 1 vRMS]
   }

#
# Detection threshold:

if {![strcasecmp $maxVal Auto]} \
   {
    set maxList ""

    for {set ia 1} {$ia <= $afSize} {incr ia} \
       {
        readROI -roi 1 -in $inName -ndim 3 -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100% -da A_AXIS $ia 0
        lappend maxList [getROIStat 1 vMaxAbs]
       }

    set maxVal [getMax $maxList]
   }

set threshStr ""

if {![strcasecmp $thresh Auto]} \
   {
    set threshStr " ($threshMult*Noise RMS)"
    set thresh    [expr $threshMult*$noiseRMS]
   }

if {![strcmp [string index $thresh end] "%"]} \
   {
    set threshStr " ($thresh of Max Value)"
    set pct       [string trim $thresh "%"]
    set thresh    [expr $maxVal*$pct/100.0]
   }

if {$verbose} \
   {
    puts "4D Peak Detection. Input:              $inName"
   
    if {![strcasecmp $maxVal None]} \
       {
        puts "4D Peak Detection. Max Value:          Not Determined."
       } \
    else \
       { 
        puts "4D Peak Detection. Max Value:          [format %.3e $maxVal]"
       }

    puts "4D Peak Detection. Noise RMS:          [format %.3e $noiseRMS]$noiseStr"
    puts "4D Peak Detection. Threshold:          [format %.3e $thresh]$threshStr"
    puts "4D Peak Detection. X-Axis Detection:   ddx $ddx idx $idx tolx $tolx"
    puts "4D Peak Detection. Y-Axis Detection:   ddy $ddy idy $idy toly $toly"
    puts "4D Peak Detection. Z-Axis Detection:   ddz $ddz idz $idz tolz $tolz"
    puts "4D Peak Detection. A-Axis Detection:   dda $dda ida $ida tola $tola"
    puts ""
   }

#
# Peak detection for each specified analysis region:

set regCount [expr $n/8]
set loc      0

for {set i 1} {$i <= $regCount} {incr i} \
   {
    if {$regCount == 1} \
       {
        set thisName $tabName
       } \
    else \
       {
        set thisName $tmpName
       }

    set rx1 [lindex $regList $loc]
    incr loc

    set rxn [lindex $regList $loc]
    incr loc

    set ry1 [lindex $regList $loc]
    incr loc

    set ryn [lindex $regList $loc]
    incr loc

    set rz1 [lindex $regList $loc]
    incr loc

    set rzn [lindex $regList $loc]
    incr loc

    set ra1 [lindex $regList $loc]
    incr loc

    set ran [lindex $regList $loc]
    incr loc

    set kx1 [expr int(0.5 + [spec2pnt $fdata CUR_XDIM $rx1])]
    set kxn [expr int(0.5 + [spec2pnt $fdata CUR_XDIM $rxn])]

    set ky1 [expr int([spec2pnt $fdata CUR_YDIM $ry1])]
    set kyn [expr int([spec2pnt $fdata CUR_YDIM $ryn])]

    set kz1 [expr int([spec2pnt $fdata CUR_ZDIM $rz1])]
    set kzn [expr int([spec2pnt $fdata CUR_ZDIM $rzn])]

    set ka1 [expr int([spec2pnt $fdata CUR_ADIM $ra1])]
    set kan [expr int([spec2pnt $fdata CUR_ADIM $ran])]

    if {$edgeFlag} \
       {
        if {$kyn == $yfSize} \
           {
            set ky1 [expr $ky1 - $ddy]
            set kyn [expr $kyn + 2*$ddy]
           } \
        else \
           {
            set ky1 [expr $ky1 - $ddy]
            set kyn [expr $kyn + $ddy]
           }

        if {$kzn == $zfSize} \
           {
            set kz1 [expr $kz1 - $ddz]
            set kzn [expr $kzn + 2*$ddz]
           } \
        else \
           {
            set kz1 [expr $kz1 - $ddz]
            set kzn [expr $kzn + $ddz]
           }

        if {$kan == $afSize} \
           {
            set ka1 [expr $ka1 - $dda]
            set kan [expr $kan + 2*$dda]
           } \
        else \
           {
            set ka1 [expr $ka1 - $dda]
            set kan [expr $kan + $dda]
           }

        if {$ky1 < 1} {set ky1 1}
        if {$kz1 < 1} {set kz1 1}
        if {$ka1 < 1} {set ka1 1}
       }

    readROI -roi 1 -ndim 4 -in $inName -x X_AXIS $kx1 $kxn -y Y_AXIS $ky1 $kyn -z Z_AXIS $kz1 $kzn -a A_AXIS $ka1 $kan

    if {$verbose} \
       {
        set localMin [getROIStat 1 vMin]
        set localMax [getROIStat 1 vMax]
       }

    eval pkFindROI $pkArgs -roi 1 \
         -sigma $noiseRMS -pChi 1.0e-04 -plus $thresh -minus -$thresh \
         -dx      $ddx  $ddy  $ddz  $dda  \
         -idx     $idx  $idy  $idz  $ida  \
         -tol     $tolx $toly $tolz $tola \
         -hiAdj   1.2   1.8   1.8   1.8   \
         -lw      15.0  5.0   5.0   5.0   \
         -parent -noreject -mask -out $thisName -$vArg

    set tabInfo($i)   [gdbRead table -in $thisName]
    set thisCount($i) [gdbGetInfo $tabInfo($i) -parms entryCount]

    if {$verbose} \
       {
        if {$edgeFlag} \
           {
            set eStr " Edge-Adjusted"
           } \
        else \
           {
            set eStr ""
           }

        puts "4D Peak Detection. Region $i Limits X:  $rx1 $rxn"
        puts "4D Peak Detection. Region $i Limits Y:  $ry1 $ryn$eStr"
        puts "4D Peak Detection. Region $i Limits Z:  $rz1 $rzn$eStr"
        puts "4D Peak Detection. Region $i Limits A:  $ra1 $ran$eStr"
        puts "4D Peak Detection. Region $i Min:      [format %.3e $localMin]"
        puts "4D Peak Detection. Region $i Max:       [format %.3e $localMax]"
        puts "4D Peak Detection. Region $i Peaks:     $thisCount($i)"
       }
   }

#
# Join peak tables if multiple regions were analyzed:

if {$regCount > 1} \
   {
    for {set i 2} {$i <= $regCount} {incr i} \
       {
        if {!$thisCount($i)} continue
        gdbAppend -dest $tabInfo(1) -src $tabInfo($i)        
       }
   }

set totalCount [gdbGetInfo $tabInfo(1) -parms entryCount]

if {$verbose} \
   {
    puts "4D Peak Detection. Total Peaks:        $totalCount"
    puts "4D Peak Detection. Output:             $tabName"
   }

gdbWrite $tabInfo(1) $tabName

#
# Renumber peaks if multiple tables were joined:

if {$regCount > 1} \
   {
    set t     [gdbRead table -in $tabName]
    set eList [gdbGetInfo $t -parms entries]

    set n     [llength $eList]
    set i     0

    foreach e $eList \
       {
        incr i
        gdbSet $e INDEX $i
       }

    gdbWrite $t $tabName
   }

if {$regCount > 1} {catch "file delete -force $tmpName"}

exit 0
