#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts " *** This Script is Under Construction, Not Implemented Yet *** "
    puts "Project Existing Assignments onto a Peak Table:"
    puts " -assName    assTabName \[ass.tab\]   Input Assignment Table."
    puts " -inName     inTabName  \[test.tab\]  Input Peak Table."
    puts " -outName    outTabName \[new.tab\]   Output, All Peaks."
    puts " -usedName   usedName   \[None\]      Output, Assigned Peaks Only."
    puts " -unusedName unusedName \[None\]      Output, UnAssigned Peaks Only."
    puts " -ndim       dimCount   \[2\]         Assignment Dimension Count."
    puts "Assignment Mapping Parameters:"
    puts " -xTol   xtPPM \[0.03\]   X-Axis +/- Tolerance, PPM." 
    puts " -yTol   ytPPM \[0.15\]   Y-Axis +/- Tolerance, PPM." 
    puts " -zTol   ztPPM \[0.25\]   Z-Axis +/- Tolerance, PPM." 
    puts "Peak Table Parameters:"
    puts " -xVar   xVar  \[Auto\]   X-Axis Name in Peak Table." 
    puts " -yVar   yVar  \[Auto\]   Y-Axis Name in Peak Table." 
    puts " -zVar   zVar  \[None\]   Z-Axis Name in Peak Table." 
    puts "Assignment Table Parameters:"
    puts " -xAss   xAss  \[Auto\]   X-Axis Name in Assignment Table." 
    puts " -yAss   yAss  \[Auto\]   Y-Axis Name in Assignment Table." 
    puts " -zAss   zAss  \[None\]   Z-Axis Name in Assignment Table." 
    puts " -assVar assV  \[ASS\]    Assignment Name in Assignment Table." 
    puts "Assignment Table PPM Offsets:"
    puts " -xOff   xOff  \[0.0\]    X-Axis Offset added to Assignment, PPM." 
    puts " -yOff   yOff  \[0.0\]    Y-Axis Offset added to Assignment, PPM." 
    puts " -zOff   zOff  \[0.0\]    Z-Axis Offset added to Assignment, PPM." 
    puts "Information for Folding via Parent Spectrum:"
    puts " -parent sName \[None\]   Spectrum Corresponding to Input Peak Table."
    puts " -xName  xName \[Auto\]   X-Axis Name of Spectrum."
    puts " -yName  yName \[Auto\]   Y-Axis Name of Spectrum."
    puts " -zName  zName \[Auto\]   Z-Axis Name of Spectrum."
    puts "Information for Folding via Spectral Limits (also -y.. -z..):"
    puts " -x1     x1PPM \[None\]   Peak Table X-Axis Folding Limit Start, PPM."
    puts " -xn     xnPPM \[None\]   Peak Table X-Axis Folding Limit End, PPM."
    puts "Scale Factors for Computing Closest Assignment:"
    puts " -xs     xS    \[Auto\]   Scale Factor for X-Axis."
    puts " -ys     yS    \[Auto\]   Scale Factor for Y-Axis."
    puts " -zs     zS    \[Auto\]   Scale Factor for Z-Axis."
    puts "Assignment Modes:"
    puts " -replace               Replace Existing Assignments (Default)."
    puts " -noreplace             Keep Existing Assignments."
    puts " -closest               Keep Only Closest Assignment (Default)."
    puts " -all                   Keep All Assignments Within Tolerance."
    puts " *** This Script is Under Construction, Not Implemented Yet *** "
    exit
   }

set assTabName  ass.tab
set inName      test.tab
set outName     new.tab
set usedName    None
set unusedName  None
set parentName  None
set dimCount    2

set xTol      0.03
set yTol      0.15
set zTol      0.25

set xVarName  Auto 
set yVarName  Auto
set zVarName  Auto

set xAssName  Auto 
set yAssName  Auto
set zAssName  Auto

set xName     Auto 
set yName     Auto 
set zName     Auto 

set x1        None
set xn        None
set y1        None
set yn        None
set z1        None
set zn        None

set xs        Auto
set ys        Auto 
set zs        Auto 

set xOffset   0.0
set yOffset   0.0
set zOffset   0.0

set peaksPerAss Auto 

set assVar       ASS

set replaceFlag  1
set closestFlag  1

#
#-----------------------------------------------------------------------------

getArgD $ARGV -ndim        dimCount

getArgD $ARGV -assName     assTabName
getArgD $ARGV -inName      inName
getArgD $ARGV -outName     outName
getArgD $ARGV -usedName    usedName
getArgD $ARGV -unusedName  unusedName
getArgD $ARGV -parent      parentName

getArgD $ARGV -pk     peaksPerAss

getArgD $ARGV -xTol   xTol
getArgD $ARGV -yTol   yTol 
getArgD $ARGV -zTol   zTol

getArgD $ARGV -xName  xName
getArgD $ARGV -yName  yName
getArgD $ARGV -zName  zName

getArgD $ARGV -xVar   xVarName
getArgD $ARGV -yVar   yVarName
getArgD $ARGV -zVar   zVarName

getArgD $ARGV -xAss   xAssName
getArgD $ARGV -yAss   yAssName
getArgD $ARGV -zAss   zAssName
getArgD $ARGV -assVar assVar

getArgD $ARGV -yOff   yOffset
getArgD $ARGV -xOff   xOffset
getArgD $ARGV -zOff   zOffset

getArgD $ARGV -x1     x1
getArgD $ARGV -xn     xn
getArgD $ARGV -y1     y1
getArgD $ARGV -yn     yn
getArgD $ARGV -z1     z1
getArgD $ARGV -zn     zn

getArgD $ARGV -xs     xs
getArgD $ARGV -ys     ys
getArgD $ARGV -zs     zs

if {[flagLoc $ARGV -replace]}   {set replaceFlag 1}
if {[flagLoc $ARGV -noreplace]} {set replaceFlag 0}

if {[flagLoc $ARGV -closest]}   {set closestFlag 1}
if {[flagLoc $ARGV -all]}       {set closestFlag 0}

#
#--------------------------------------------------------------------------

proc initAssOffset {} \
{
   global assEList xOffset yOffset zOffset dimCount
   global xAssName yAssName zAssName

   if {$dimCount > 0 && $xOffset != 0.0} \
      {
       foreach eAss $assEList \
          {
           set xLoc [gdbGet $eAss $xAssName]
           if {[isUndefShift $xLoc]} continue

           set xLoc [expr $xLoc + $xOffset]
           gdbSet $eAss $xAssName $xLoc
          }
      }

   if {$dimCount > 1 && $yOffset != 0.0} \
      {
       foreach eAss $assEList \
          {
           set yLoc [gdbGet $eAss $yAssName]
           if {[isUndefShift $yLoc]} continue

           set yLoc [expr $yLoc + $yOffset]
           gdbSet $eAss $yAssName $yLoc
          }
      }

   if {$dimCount > 2 && $zOffset != 0.0} \
      {
       foreach eAss $assEList \
          {
           set zLoc [gdbGet $eAss $zAssName]
           if {[isUndefShift $zLoc]} continue

           set zLoc [expr $zLoc + $zOffset]
           gdbSet $eAss $zAssName $zLoc
          }
      }
}

proc assIsNear { eAss ePk } \
{
   global xVarName yVarName zVarName xAssName yAssName zAssName dimCount
   global xTol yTol zTol x1 xn y1 yn z1 zn xFold yFold zFold

   set xFold 0
   set yFold 0
   set zFold 0

   set xNear 0
   set yNear 0
   set zNear 0

   if {$dimCount > 0} \
      {
       set xLocAss [gdbGet $eAss $xAssName]
       set xLocPk  [gdbGet $ePk  $xVarName]

       if {[isUndefShift $xLocAss]} {return 0}
       if {[isUndefShift $xLocPk]}  {return 0}

       set dx [expr $xLocAss - $xLocPk]
       
       if {abs($dx) <= $xTol} {set xNear 1}
      }

   if {!$xNear} {return 0}
}

proc isUndefShift { s } \
{
   if {abs($s) > 600} {return 1}
   return 0
}

proc isUndefAss { a } \
{
   set t [string tolower $a]

   if {[lsearch -exact "none null (none) (null)" $t] < 0} {return 0}

   return 1
}

proc dSort { i1 i2 } \
{
   set s1 [lindex $i1 0]
   set s2 [lindex $i2 0]

   if {$s1 < $s2} {return -1}
   if {$s1 > $s2} {return  1}

   return 0
}

#
#--------------------------------------------------------------------------

set error  0
set fdata  0

set xSW    0.0
set ySW    0.0
set zSW    0.0

#
# Input Files:

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Peak Table $inName"
    exit 1
   }

if {![file exists $assTabName]} \
   {
    puts stderr "Error Finding Input Assignment Table $assTabName"
    exit 1
   }

#
# Table contents:

set assTab     [gdbRead table -in $assTabName]
set pkTab      [gdbRead table -in $inName]

set assVarList [gdbGetInfo $assTab -parms varNames]
set assEList   [gdbGetInfo $assTab -parms entries]

set pkVarList  [gdbGetInfo $pkTab  -parms varNames]
set pkEList    [gdbGetInfo $pkTab  -parms entries]

if {$dimCount < 2} \
   {
    set yName    None
    set yVarName None
    set yAssName None
   }

if {$dimCount < 3} \
   {
    set zName    None
    set zVarName None
    set zAssName None
   }

#
# Test dimension count of peak table:

set ndim 0

foreach i {X_PPM Y_PPM Z_PPM A_PPM} \
   {
    if {[lsearch -exact $pkVarList $i] < 0} break
    incr ndim
   }

if {$ndim < $dimCount} \
   {
    puts stderr "Error: Peak Table has only $ndim dimensions."
    exit 1
   }

#
# Extract folding parameters from spectrum if given:

if {[strcasecmp $parentName None]} \
   {
    if {![file exists $parentName]} \
       {
        puts stderr "Error Finding Input Spectrum $parentName"
        exit 1
       }

     set error  [rdFDATA $parentName fdata]
     set ndim   [getParmI $fdata FDDIMCOUNT NULL_DIM] 

     if {$ndim < $dimCount} \
        {
         puts stderr "Error: Spectrum has only $ndim dimensions."
         exit 1
        }

     if {![strcasecmp $xName Auto]} \
        {
         set xName [getDirectDim $fdata]
         puts "Auto Mode Direct Dimension for Folding:  $xName"
        }

     if {![strcasecmp $yName Auto]} \
        {
         set yName [get15NDim $fdata]
         puts "Auto Mode 15N Dimension for Folding:     $yName"
        }

     if {![strcasecmp $zName Auto]} \
        {
         set zName [getIndirectDim $fdata]
         puts "Auto Mode 3rd Dimension for Folding:     $zName"
        }

     if {$ndim > 0 && ([strcasecmp $x1 None] || [strcasecmp $xn None])} \
        {
         set xDimName [getDimKeyword $xName $fdata]
         set xSize    [getParm $fdata NDSIZE $xDimName]
         set nx1      [getParm $fdata NDX1   $xDimName]
         set nxn      [getParm $fdata NDXN   $xDimName]

         if {$nx1 || $nxn} \
            {
             puts "Extracted Data. No X-Axis folding parameters will be used."
            } \
         else \
            {
             if {[strcasecmp $x1 None]} \
                {
                 set x1 [pnt2spec $fdata $xDimName 1 ppm]
                 puts   [format "Found Folding Limit X1: % 8.3ppm" $x1]
                }

             if {[strcasecmp $xn None]} \
                {
                 set xn [pnt2spec $fdata $xDimName $xSize ppm]
                 puts   [format "Found Folding Limit XN: % 8.3ppm" $xn]
                }
            }
        }

     if {$ndim > 1 && ([strcasecmp $y1 None] || [strcasecmp $yn None])} \
        {
         set yDimName [getDimKeyword $yName $fdata]
         set ySize    [getParm $fdata NDSIZE $yDimName]
         set ny1      [getParm $fdata NDX1   $yDimName]
         set nyn      [getParm $fdata NDXN   $yDimName]

         if {$ny1 || $nyn} \
            {
             puts "Extracted Data. No Y-Axis folding parameters will be used."
            } \
         else \
            {
             if {[strcasecmp $y1 None]} \
                {
                 set y1 [pnt2spec $fdata $yDimName 1 ppm]
                 puts   [format "Found Folding Limit Y1: % 8.3ppm" $y1]
                }

             if {[strcasecmp $yn None]} \
                {
                 set yn [pnt2spec $fdata $yDimName $ySize ppm]
                 puts   [format "Found Folding Limit YN: % 8.3ppm" $yn]
                }
            }
        }

     if {$ndim > 2 && ([strcasecmp $z1 None] || [strcasecmp $zn None])} \
        {
         set zDimName [getDimKeyword $zName $fdata]
         set zSize    [getParm $fdata NDSIZE $zDimName]
         set nz1      [getParm $fdata NDX1   $zDimName]
         set nzn      [getParm $fdata NDXN   $zDimName]

         if {$nz1 || $nzn} \
            {
             puts "Extracted Data. No Z-Axis folding parameters will be used."
            } \
         else \
            {
             if {[strcasecmp $z1 None]} \
                {
                 set z1 [pnt2spec $fdata $zDimName 1 ppm]
                 puts   [format "Found Folding Limit Z1: % 8.3ppm" $z1]
                }

             if {[strcasecmp $zn None]} \
                {
                 set zn [pnt2spec $fdata $zDimName $zSize ppm]
                 puts   [format "Found Folding Limit ZN: % 8.3ppm" $zn]
                }
            }
        }
    }

#
# Folding widths and shift scale factors:

if {[strcasecmp $x1 None] && [strcasecmp $xn None]} \
   {
    set xSW [expr abs($x1 - $xn)]
   }

if {[strcasecmp $y1 None] && [strcasecmp $yn None]} \
   {
    set ySW [expr abs($y1 - $yn)]
   }

if {[strcasecmp $x1 None] && [strcasecmp $xn None]} \
   {
    set zSW [expr abs($z1 - $zn)]
   }

if {![strcasecmp $xs Auto]} \
   {
    if {$xSW == 0.0} \
       {
        set xs 1.0
       } \
    else \
       {
        set xs [expr 1.0/$xSW]
       }
   }

if {![strcasecmp $ys Auto]} \
   {
    if {$ySW == 0.0} \
       {
        set ys 1.0
       } \
    else \
       {
        set ys [expr 1.0/$ySW]
       }
   }

if {![strcasecmp $zs Auto]} \
   {
    if {$zSW == 0.0} \
       {
        set zs 1.0
       } \
    else \
       {
        set zs [expr 1.0/$zSW]
       }
   }

#
# Variable names from assignment table:

if {$dimCount > 0 && ![strcasecmp $xAssName Auto]} \
   {
    set xAssName [getHNTabVar $assTab]
    puts "Auto Mode HN Ass Variable:   $xAssName"
   }

if {$dimCount > 1 && ![strcasecmp $yAssName Auto]} \
   {
    set yAssName [get15NTabVar $assTab]
    puts "Auto Mode 15N Ass Variable:  $yAssName"
   }

if {$dimCount > 2 && ![strcasecmp $zAssName Auto]} \
   {
    set zAssName [getIndirectTabVar $assTab]
    puts "Auto Mode 3rd Ass Variable:  $zAssName"
   }

#
# Variable names from peak table:

if {$dimCount > 0 && ![strcasecmp $xVarName Auto]} \
   {
    set xVarName [getHNTabVar $pkTab]
    puts "Auto Mode HN Peak Variable:  $xVarName"
   }

if {$dimCount > 1 && ![strcasecmp $yVarName Auto]} \
   {
    set yVarName [get15NTabVar $pkTab]
    puts "Auto Mode 15N Peak Variable: $yVarName"
   }

if {$dimCount > 2 && ![strcasecmp $zVarName Auto]} \
   {
    set zVarName [getIndirectTabVar $pkTab]
    puts "Auto Mode 3rd Peak Variable: $zVarName"
   }

#
#-----------------------------------------------------------------------------

proc pSort { i1 i2 } \
{
   set s1 [lindex $i1 0]
   set s2 [lindex $i2 0]

   if {$s1 < $s2} {return -1}
   if {$s1 > $s2} {return  1}

   return 0
}

