#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set assName ass.tab
set inName  master.tab
set outName assigned.tab

set assVar(X)   HN
set assVar(Y)   N
set assVar(ASS) ASS

set pkVar(X)    X_PPM
set pkVar(Y)    Y_PPM
set pkVar(ASS)  ASS
set pkVar(ID)   INDEX

set tol(X)      0.02
set tol(Y)      0.20

set maxDist(X)  0.08
set maxDist(Y)  0.80

set offset(X)   0.0
set offset(Y)   0.0

set w(X)        1.0
set w(Y)        0.25

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -ass assName

getArgD $ARGV -xPkVar    pkVar(X)
getArgD $ARGV -yPkVar    pkVar(Y)
getArgD $ARGV -idVar     pkVar(ID)
getArgD $ARGV -pkAssVar  pkVar(ASS)

getArgD $ARGV -xAssVar   assVar(X)
getArgD $ARGV -yAssVar   assVar(Y)
getArgD $ARGV -assAssVar assVar(ASS)

getArgD $ARGV -xOff      offset(X)
getArgD $ARGV -yOff      offset(Y)

getArgD $ARGV -xW        w(X)
getArgD $ARGV -yW        w(Y)

getArgD $ARGV -xTol      tol(X)
getArgD $ARGV -yTol      tol(Y)

getArgD $ARGV -dxMax     maxDist(X)
getArgD $ARGV -dyMax     maxDist(Y)

if {[flagLoc $ARGV -help]} \
   {
    puts "Map 2D Assignments onto a Peak Table (2023.090.16.20):"
    puts " -in        inName     NMRPipe-format 2D Peak Table Input.   \[$inName\]"
    puts " -ass       assName    Assignment Table Input.               \[$assName\]"
    puts " -out       outName    Assigned Peak Table Output.           \[$outName\]"
    puts " -xTol      xTol       X-Axis Position Tolerance, ppm.       \[$tol(X)\]"
    puts " -yTol      yTol       Y-Axis Position Tolerance, ppm.       \[$tol(Y)\]"
    puts " -dxMax     dxMax      X-Axis Max Assignment Distance, ppm.  \[$maxDist(X)\]"
    puts " -dyMax     dyMax      Y-Axis Max Assignment Distance, ppm.  \[$maxDist(Y)\]"
    puts " -xW        xW         X-Axis ppm Weight Factor.             \[$w(X)\]"
    puts " -yW        yW         Y-Axis ppm Weight Factor.             \[$w(Y)\]"
    puts " -xOff      xOff       X-Axis Assignment Offset to Add, ppm. \[$offset(X)\]"
    puts " -yOff      yOff       Y-Axis Assignment Offset to Add, ppm. \[$offset(Y)\]"
    puts "Assignemnt Table Variables:"
    puts " -xAssVar   xAssVar    Assignment Table X-Axis ppm Variable. \[$assVar(X)\]"
    puts " -yAssVar   yAssVar    Assignment Table Y-Axis ppm Variable. \[$assVar(Y)\]"
    puts " -assAssVar assAssVar  Assignment Table Assignment Variable. \[$assVar(ASS)\]"
    puts "Peak Table Variables:"
    puts " -xPkVar    xPkVar     Peak Table X-Axis ppm Variable.       \[$pkVar(X)\]"
    puts " -yPkVar    yPkVar     Peak Table Y-Axis ppm Variable.       \[$pkVar(Y)\]"
    puts " -idVar     idVar      Peak Table Index Variable.            \[$pkVar(ID)\]"
    puts " -pkAssVar  pkAssVar   Peak Table Assignment Variable.       \[$pkVar(ASS)\]"
    exit 0
   }

proc listAsses { assList } \
{
   global tol pkVar assVar assTab pkTab

   foreach assInfo $assList \
      {
       foreach thisAss [split $assInfo ","] \
          {
           set assList [gdbSelect -from $assTab -cond "!strcasecmp( $assVar(ASS), '$thisAss' )"]

           foreach e $assList \
              {
               puts -nonewline [format "  %4s %8.4f %8.4f"  $thisAss [gdbGet $e $assVar(X)] [gdbGet $e $assVar(Y)]]
              }

           puts ""
          }
      }
}

proc listPeaks { pkList } \
{
   global tol pkVar assVar assTab pkTab

   foreach pkInfo $pkList \
      {
       foreach thisID [split $pkInfo ","] \
          {
           set eList [gdbSelect -from $pkTab -cond "$pkVar(ID) == $thisID"]

           foreach e $eList \
              {
               puts -nonewline [format "  %4s %8.4f %8.4f %s"  $thisID [gdbGet $e $pkVar(X)] [gdbGet $e $pkVar(Y)] [gdbGet $e $pkVar(ASS)]]
              }

           puts ""
          }
      }
}

proc getNearestPk { x0 y0 } \
{
   global eListPk pkVar w maxDist

   set dMin  1.0e+16
   set eBest None

   foreach e $eListPk \
      {
       set x1 [gdbGet $e $pkVar(X)]
       set y1 [gdbGet $e $pkVar(Y)]

       if {abs($x1 - $x0) > $maxDist(X)} continue
       if {abs($y1 - $y0) > $maxDist(Y)} continue

       set dx [expr $w(X)*($x1 - $x0)]
       set dy [expr $w(Y)*($y1 - $y0)]

       set d  [expr sqrt($dx*$dx + $dy*$dy)]

       if {$d < $dMin} \
          {
           set dMin  $d
           set eBest $e
          }
      }

   return $eBest
}

proc getNearestAss { x0 y0 } \
{
   global eListAss assVar w

   set dMin  1.0e+16
   set eBest None

   foreach e $eListAss \
      {
       set x1 [gdbGet $e $assVar(X)]
       set y1 [gdbGet $e $assVar(Y)]

       if {abs($x1 - $x0) > $maxDist(X)} continue
       if {abs($y1 - $y0) > $maxDist(Y)} continue

       set dx [expr $w(X)*($x1 - $x0)]
       set dy [expr $w(Y)*($y1 - $y0)]

       set d  [expr sqrt($dx*$dx + $dy*$dy)]

       if {$d < $dMin} \
          {
           set dMin  $d
           set eBest $e
          }
      }

   return $eBest
}

proc getNearPk { x0 y0 } \
{
   global tol pkVar assVar assTab pkTab eListPk

   set cond "near( $pkVar(X), $x0, $tol(X) ) && near( $pkVar(Y), $y0, $tol(Y) )"

   set thisList [gdbSelect -from $pkTab -cond $cond] 

   if {![llength $thisList]} {set thisList [getNearestPk $x0 $y0]}

   return $thisList
}

proc getNearAss { x0 y0 } \
{
   global tol pkVar assVar assTab pkTab

   set cond "near( $assVar(X), $x0, $tol(X) ) && near( $assVar(Y), $y0, $tol(Y) )"

   set thisList [gdbSelect -from $assTab -cond $cond]

   if {![llength $thisList]} {set thisList [getNearestAss $x0 $y0]}

   return $thisList
}
 
set assTab   [gdbRead table -in $assName]
set pkTab    [gdbRead table -in $inName]

set dListAss [gdbSelect -from $assTab -cond "$assVar(X) > 666 || $assVar(Y) > 666"]
set dListPk  [gdbSelect -from $pkTab  -cond "$pkVar(X)  > 666 || $pkVar(Y)  > 666"]

gdbDestroy $dListAss
gdbDestroy $dListPk

set eListAss [gdbGetInfo $assTab -parms entries]
set eListPk  [gdbGetInfo $pkTab  -parms entries]

set assList(AllNames)   ""
set assList(Redundant)  ""
set assList(Overlapped) ""
set assList(Single)     ""
set assList(Multiple)   ""
set assList(None)       ""

set pkList(Assigned)    ""
set pkList(Unassigned)  ""
set pkList(Single)      ""
set pkList(Multiple)    ""

foreach e $eListAss \
   {
    set assX [gdbGet $e $assVar(X)]
    set assY [gdbGet $e $assVar(Y)]
    set ass  [gdbGet $e $assVar(ASS)]

    set assX [expr $assX + $offset(X)]
    set assY [expr $assY + $offset(Y)]

    gdbSet $e $assVar(X) $assX
    gdbSet $e $assVar(Y) $assY

    if {[lsearch -exact $assList(AllNames) $ass] < 0} \
       {
        lappend assList(AllNames) $ass
       } \
    else \
       {
        if {[lsearch -exact $assList(Redundant) $ass] < 0}  {lappend assList(Redundant) $ass} 
       }

    set thisList [getNearAss $assX $assY]

    if {[llength $thisList] > 1} \
       {
        set ovInfo [join [getEListVals $thisList $assVar(ASS)] ","]

        if {[lsearch -exact $assList(Overlapped) $ovInfo] < 0} {lappend assList(Overlapped) $ovInfo}
       }
   }

foreach eAss $eListAss \
   {
    set assX [gdbGet $eAss $assVar(X)]
    set assY [gdbGet $eAss $assVar(Y)]
    set ass  [gdbGet $eAss $assVar(ASS)]

    puts [format "Assignment: %4s X: %8.4f Y: %8.4f" $ass $assX $assY]

    set thisPkList [getNearPk $assX $assY]

    set n [llength $thisPkList]

    if {$n == 0 || ![strcasecmp $thisPkList None]} \
       {
        lappend assList(None) $ass
        puts " No Matches."
       } \
    else \
       {
        foreach ePk $thisPkList \
           {
            set pkID  [gdbGet $ePk $pkVar(ID)] 
            set pkAss [gdbGet $ePk $pkVar(ASS)] 

            puts [format " Peak: %4d X: %8.4f Y: %8.4f" $pkID  [gdbGet $ePk $pkVar(X)] [gdbGet $ePk $pkVar(Y)]]

            if {![strcasecmp $pkAss None]} \
               {
                gdbSet $ePk ASS $ass
               } \
            else \
               {
                gdbSet $ePk ASS ${pkAss},$ass
               }

            if {$n == 1} \
               {
                lappend assList(Single)  $ass
               } \
            else \
               {
                lappend assList(Mutiple) $ass
               }
           } 
       }
   }

foreach ePk $eListPk \
   {
    set pkID  [gdbGet $ePk $pkVar(ID)]
    set pkAss [gdbGet $ePk $pkVar(ASS)]

    if {![strcasecmp $pkAss None]} \
       {
        lappend pkList(Unassigned) $pkID
       } \
    else \
       {
        if {[llength [split $pkAss ","]] == 1} \
           {
            lappend pkList(Single)  $pkID
           } \
        else \
           {
            lappend pkList(Multiple) $pkID
           }
       }
   }

puts ""
puts "Assignment Input:"
puts [format " Total Assignments in Input:      %4d"       [llength $eListAss]]
puts [format " Deleted Assignments in Input:    %4d"       [llength $dListAss]]

puts ""
puts [format " Redundant Assignments in Input:  %4d (%s)"  [llength $assList(Redundant)]  $assList(Redundant)]
listAsses $assList(Redundant)

puts ""
puts [format " Assignment Overlaps in Input:    %4d (%s)"  [llength $assList(Overlapped)] $assList(Overlapped)]
listAsses $assList(Overlapped)

puts ""
puts "Assignments:"
puts [format " Matching Single Peaks:           %4d"       [llength $assList(Single)]]
puts [format " Matching Multiple Peaks:         %4d (%s)"  [llength $assList(Multiple)]   $assList(Multiple)]
listAsses $assList(Multiple)

puts ""
puts [format " Matching No Peaks:               %4d (%s)"  [llength $assList(None)]       $assList(None)]
listAsses $assList(None)

puts ""
puts "Peaks:"
puts [format " Total Peaks in Input:            %4d"       [llength $eListPk]]
puts [format " Deleted Peaks in Input:          %4d"       [llength $dListPk]]
puts [format " Peaks With Single Assignments:   %4d"       [llength $pkList(Single)]]

puts ""
puts [format " Peaks With Multiple Assignments: %4d (%s)"  [llength $pkList(Multiple)]   $pkList(Multiple)]
listPeaks $pkList(Multiple)

puts ""
puts [format " Peaks with No Assignments:       %4d (%s)"  [llength $pkList(Unassigned)] $pkList(Unassigned)]
listPeaks $pkList(Unassigned)

set dxAvg 0.0
set dyAvg 0.0

foreach eAss $eListAss \
   {
    set assX [gdbGet $eAss $assVar(X)]
    set assY [gdbGet $eAss $assVar(Y)]
    set ass  [gdbGet $eAss $assVar(ASS)]

#   puts [format "Assignment: %8s X: %8.4f Y: %8.4f" $ass $assX $assY]

    set ePk [getNearestPk $assX $assY]

    if {[strcasecmp $ePk None]} \
       {
        set pkID  [gdbGet $ePk $pkVar(ID)]
        set pkAss [gdbGet $ePk $pkVar(ASS)]

        set pkX [gdbGet $ePk $pkVar(X)]
        set pkY [gdbGet $ePk $pkVar(Y)]

        set dxAvg [expr $dxAvg + ($pkX - $assX)]
        set dyAvg [expr $dyAvg + ($pkY - $assY)]

#       puts [format "Nearest Peak: %6d X: %8.4f Y: %8.4f DX:  %8.4f DY: %8.4f\n" $pkID  [gdbGet $ePk $pkVar(X)] [gdbGet $ePk $pkVar(Y)] [expr $pkX - $assX] [expr $pkY - $assY]]
       }
   }

puts ""
puts [format "Avg DX: %8.4f Avg DY: %8.4f" [expr $dxAvg/[llength $eListAss]] [expr $dyAvg/[llength $eListAss]]]
gdbWrite $pkTab $outName

exit 0
