#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set assName ass.tab
set inName  master.tab
set outName assigned.tab

set assVar(X)   HN
set assVar(Y)   N
set assVar(Z)   C_PREV
set assVar(ASS) ASS

set pkVar(X)    X_PPM
set pkVar(Y)    Y_PPM
set pkVar(Z)    Z_PPM
set pkVar(ASS)  ASS
set pkVar(ID)   INDEX

set tol(X)      0.02
set tol(Y)      0.20
set tol(Z)      0.20

set offset(X)   0.0
set offset(Y)   0.0
set offset(Z)   0.0

set w(X)        1.0
set w(Y)        0.25
set w(Z)        0.25

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -ass assName

getArgD $ARGV -xPkVar    pkVar(X)
getArgD $ARGV -yPkVar    pkVar(Y)
getArgD $ARGV -zPkVar    pkVar(Z)
getArgD $ARGV -idVar     pkVar(ID)
getArgD $ARGV -pkAssVar  pkVar(ASS)

getArgD $ARGV -xAssVar   assVar(X)
getArgD $ARGV -yAssVar   assVar(Y)
getArgD $ARGV -zAssVar   assVar(Z)
getArgD $ARGV -assAssVar assVar(ASS)

getArgD $ARGV -xOff      offset(X)
getArgD $ARGV -yOff      offset(Y)
getArgD $ARGV -zOff      offset(Z)

getArgD $ARGV -xW        w(X)
getArgD $ARGV -yW        w(Y)
getArgD $ARGV -zW        w(Z)

getArgD $ARGV -xTol      tol(X)
getArgD $ARGV -yTol      tol(Y)
getArgD $ARGV -zTol      tol(Z)

if {[flagLoc $ARGV -help]} \
   {
    puts "Map 3D Assignments onto a Peak Table (2023.090.16.20):"
    puts " -in        inName     NMRPipe-format 3D Peak Table Input.   \[$inName\]"
    puts " -ass       assName    Assignment Table Input.               \[$assName\]"
    puts " -out       outName    Assigned Peak Table Output.           \[$outName\]"
    puts " -xTol      xTol       X-Axis Position Tolerance, ppm.       \[$tol(X)\]"
    puts " -yTol      yTol       Y-Axis Position Tolerance, ppm.       \[$tol(Y)\]"
    puts " -zTol      zTol       Z-Axis Position Tolerance, ppm.       \[$tol(Z)\]"
    puts " -xW        xW         X-Axis ppm Weight Factor.             \[$w(X)\]"
    puts " -yW        yW         Y-Axis ppm Weight Factor.             \[$w(Y)\]"
    puts " -zW        zW         Z-Axis ppm Weight Factor.             \[$w(Z)\]"
    puts " -xOff      xOff       X-Axis Assignment Offset to Add, ppm. \[$offset(X)\]"
    puts " -yOff      yOff       Y-Axis Assignment Offset to Add, ppm. \[$offset(Y)\]"
    puts " -zOff      zOff       Z-Axis Assignment Offset to Add, ppm. \[$offset(Z)\]"
    puts "Assignemnt Table Variables:"
    puts " -xAssVar   xAssVar    Assignment Table X-Axis ppm Variable. \[$assVar(X)\]"
    puts " -yAssVar   yAssVar    Assignment Table Y-Axis ppm Variable. \[$assVar(Y)\]"
    puts " -zAssVar   zAssVar    Assignment Table Z-Axis ppm Variable. \[$assVar(Z)\]"
    puts " -assAssVar assAssVar  Assignment Table Assignment Variable. \[$assVar(ASS)\]"
    puts "Peak Table Variables:"
    puts " -xPkVar    xPkVar     Peak Table X-Axis ppm Variable.       \[$pkVar(X)\]"
    puts " -yPkVar    yPkVar     Peak Table Y-Axis ppm Variable.       \[$pkVar(Y)\]"
    puts " -zPkVar    zPkVar     Peak Table Z-Axis ppm Variable.       \[$pkVar(Z)\]"
    puts " -idVar     idVar      Peak Table Index Variable.            \[$pkVar(ID)\]"
    puts " -pkAssVar  pkAssVar   Peak Table Assignment Variable.       \[$pkVar(ASS)\]"
    exit 0
   }

proc listAsses { assList } \
{
   global tol pkVar assVar assTab pkTab

   foreach assInfo $assList \
      {
       foreach thisAss [split $assInfo ","] \
          {
           set eList [gdbSelect -from $assTab -cond "!strcasecmp( $assVar(ASS), '$thisAss' )"]

           foreach e $eList \
              {
               puts -nonewline [format "  %4s %8.4f %8.4f %8.4f"  $thisAss [gdbGet $e $assVar(X)] [gdbGet $e $assVar(Y)] [gdbGet $e $assVar(Z)]]
              }

           puts ""
          }
      }
}

proc listPeaks { pkList } \
{
   global tol pkVar assVar assTab pkTab

   foreach pkInfo $pkList \
      {
       foreach thisID [split $pkInfo ","] \
          {
           set eList [gdbSelect -from $pkTab -cond "$pkVar(ID) == $thisID"]

           foreach e $eList \
              {
               puts -nonewline [format "  %4s %8.4f %8.4f %8.4f %s"  $thisID [gdbGet $e $pkVar(X)] [gdbGet $e $pkVar(Y)] [gdbGet $e $pkVar(Z)] [gdbGet $e $pkVar(ASS)]]
              }

           puts ""
          }
      }
}

proc getNearestPk { x0 y0 z0 } \
{
   global eListPk pkVar w

   set dMin  1.0e+16
   set eBest None

   foreach e $eListPk \
      {
       set x1 [gdbGet $e $pkVar(X)]
       set y1 [gdbGet $e $pkVar(Y)]
       set z1 [gdbGet $e $pkVar(Z)]

       set dx [expr $w(X)*($x1 - $x0)]
       set dy [expr $w(Y)*($y1 - $y0)]
       set dz [expr $w(Z)*($z1 - $z0)]

       set d  [expr sqrt($dx*$dx + $dy*$dy + $dz*$dz)]

       if {$d < $dMin} \
          {
           set dMin  $d
           set eBest $e
          }
      }

   return $eBest
}

proc getNearestAss { x0 y0 z0 } \
{
   global eListAss assVar w

   set dMin  1.0e+16
   set eBest None

   foreach e $eListAss \
      {
       set x1 [gdbGet $e $assVar(X)]
       set y1 [gdbGet $e $assVar(Y)]
       set z1 [gdbGet $e $assVar(Z)]

       set dx [expr $w(X)*($x1 - $x0)]
       set dy [expr $w(Y)*($y1 - $y0)]
       set dz [expr $w(Z)*($z1 - $z0)]

       set d  [expr sqrt($dx*$dx + $dy*$dy + $dx*$dz)]

       if {$d < $dMin} \
          {
           set dMin  $d
           set eBest $e
          }
      }

   return $eBest
}

proc getNearPk { x0 y0 z0 } \
{
   global tol pkVar assVar assTab pkTab eListPk

   set cond "near( $pkVar(X), $x0, $tol(X) ) && near( $pkVar(Y), $y0, $tol(Y) ) && near( $pkVar(Z), $z0, $tol(Z) )"

   set thisList [gdbSelect -from $pkTab -cond $cond] 

   if {![llength $thisList]} {set thisList [getNearestPk $x0 $y0 $z0]}

   return $thisList
}

proc getNearAss { x0 y0 z0 } \
{
   global tol pkVar assVar assTab pkTab

   set cond "near( $assVar(X), $x0, $tol(X) ) && near( $assVar(Y), $y0, $tol(Y) ) && near( $assVar(Z), $z0, $tol(Z) )"

   set thisList [gdbSelect -from $assTab -cond $cond]

   if {![llength $thisList]} {set thisList [getNearestAss $x0 $y0 $z0]}

   return $thisList
}
 
set assTab   [gdbRead table -in $assName]
set pkTab    [gdbRead table -in $inName]

set dListAss [gdbSelect -from $assTab -cond "$assVar(X) > 666 || $assVar(Y) > 666 || $assVar(Z) > 666"]
set dListPk  [gdbSelect -from $pkTab  -cond "$pkVar(X)  > 666 || $pkVar(Y)  > 666 || $pkVar(Z)  > 666"]

gdbDestroy $dListAss
gdbDestroy $dListPk

set eListAss [gdbGetInfo $assTab -parms entries]
set eListPk  [gdbGetInfo $pkTab  -parms entries]

set assList(AllNames)   ""
set assList(Redundant)  ""
set assList(Overlapped) ""
set assList(Single)     ""
set assList(Multiple)   ""
set assList(None)       ""

set pkList(Assigned)    ""
set pkList(Unassigned)  ""
set pkList(Single)      ""
set pkList(Multiple)    ""

foreach e $eListAss \
   {
    set assX [gdbGet $e $assVar(X)]
    set assY [gdbGet $e $assVar(Y)]
    set assZ [gdbGet $e $assVar(Z)]
    set ass  [gdbGet $e $assVar(ASS)]

    set assX [expr $assX + $offset(X)]
    set assY [expr $assY + $offset(Y)]
    set assZ [expr $assZ + $offset(Z)]

    gdbSet $e $assVar(X) $assX
    gdbSet $e $assVar(Y) $assY
    gdbSet $e $assVar(Z) $assZ

    if {[lsearch -exact $assList(AllNames) $ass] < 0} \
       {
        lappend assList(AllNames) $ass
       } \
    else \
       {
        if {[lsearch -exact $assList(Redundant) $ass] < 0}  {lappend assList(Redundant) $ass} 
       }

    set thisList [getNearAss $assX $assY $assZ]

    if {[llength $thisList] > 1} \
       {
        set ovInfo [join [getEListVals $thisList $assVar(ASS)] ","]

        if {[lsearch -exact $assList(Overlapped) $ovInfo] < 0} {lappend assList(Overlapped) $ovInfo}
       }
   }

foreach eAss $eListAss \
   {
    set assX [gdbGet $eAss $assVar(X)]
    set assY [gdbGet $eAss $assVar(Y)]
    set assZ [gdbGet $eAss $assVar(Z)]
    set ass  [gdbGet $eAss $assVar(ASS)]

    puts [format "Assignment: %4s X: %8.4f Y: %8.4f Z: %8.4f" $ass $assX $assY $assZ]

    set thisPkList [getNearPk $assX $assY $assZ]

    set n [llength $thisPkList]

    if {$n} \
       {
        foreach ePk $thisPkList \
           {
            set pkID  [gdbGet $ePk $pkVar(ID)] 
            set pkAss [gdbGet $ePk $pkVar(ASS)] 

            puts [format " Peak: %4d X: %8.4f Y: %8.4f Z: %8.4f" $pkID  [gdbGet $ePk $pkVar(X)] [gdbGet $ePk $pkVar(Y)] [gdbGet $ePk $pkVar(Z)]]

            if {![strcasecmp $pkAss None]} \
               {
                gdbSet $ePk ASS $ass
               } \
            else \
               {
                gdbSet $ePk ASS ${pkAss},$ass
               }

            if {$n == 1} \
               {
                lappend assList(Single)  $ass
               } \
            else \
               {
                lappend assList(Mutiple) $ass
               }
           } 
       } \
    else \
       {
        lappend assList(None) $ass
        puts " No Matches."
       } 
   }

foreach ePk $eListPk \
   {
    set pkID  [gdbGet $ePk $pkVar(ID)]
    set pkAss [gdbGet $ePk $pkVar(ASS)]

    if {![strcasecmp $pkAss None]} \
       {
        lappend pkList(Unassigned) $pkID
       } \
    else \
       {
        if {[llength [split $pkAss ","]] == 1} \
           {
            lappend pkList(Single)  $pkID
           } \
        else \
           {
            lappend pkList(Multiple) $pkID
           }
       }
   }

puts ""
puts "Assignment Input:"
puts [format " Total Assignments in Input:      %4d"       [llength $eListAss]]
puts [format " Deleted Assignments in Input:    %4d"       [llength $dListAss]]

puts ""
puts [format " Redundant Assignments in Input:  %4d (%s)"  [llength $assList(Redundant)]  $assList(Redundant)]
listAsses $assList(Redundant)

puts ""
puts [format " Assignment Overlaps in Input:    %4d (%s)"  [llength $assList(Overlapped)] $assList(Overlapped)]
listAsses $assList(Overlapped)

puts ""
puts "Assignments:"
puts [format " Matching Single Peaks:           %4d"       [llength $assList(Single)]]
puts [format " Matching Multiple Peaks:         %4d (%s)"  [llength $assList(Multiple)]   $assList(Multiple)]
listAsses $assList(Multiple)

puts ""
puts [format " Matching No Peaks:               %4d (%s)"  [llength $assList(None)]       $assList(None)]
listAsses $assList(None)

puts ""
puts "Peaks:"
puts [format " Total Peaks in Input:            %4d"       [llength $eListPk]]
puts [format " Deleted Peaks in Input:          %4d"       [llength $dListPk]]
puts [format " Peaks With Single Assignments:   %4d"       [llength $pkList(Single)]]

puts ""
puts [format " Peaks With Multiple Assignments: %4d (%s)"  [llength $pkList(Multiple)]   $pkList(Multiple)]
listPeaks $pkList(Multiple)

puts ""
puts [format " Peaks with No Assignments:       %4d (%s)"  [llength $pkList(Unassigned)] $pkList(Unassigned)]
listPeaks $pkList(Unassigned)

set dxList ""
set dyList ""
set dzList ""

foreach eAss $eListAss \
   {
    set assX [gdbGet $eAss $assVar(X)]
    set assY [gdbGet $eAss $assVar(Y)]
    set assZ [gdbGet $eAss $assVar(Z)]
    set ass  [gdbGet $eAss $assVar(ASS)]

#   puts [format "Assignment: %8s X: %8.4f Y: %8.4f Z: %8.4f" $ass $assX $assY $assZ]

    set ePk [getNearestPk $assX $assY $assZ]

    if {[strcasecmp $ePk None]} \
       {
        set pkID  [gdbGet $ePk $pkVar(ID)]
        set pkAss [gdbGet $ePk $pkVar(ASS)]

        set pkX [gdbGet $ePk $pkVar(X)]
        set pkY [gdbGet $ePk $pkVar(Y)]
        set pkZ [gdbGet $ePk $pkVar(Z)]

        lappend dxList [expr $pkX - $assX]
        lappend dyList [expr $pkY - $assY]
        lappend dzList [expr $pkZ - $assZ]

#       puts [format "Nearest Peak: %6d X: %8.4f Y: %8.4f Z: %8.4f DX:  %8.4f DY: %8.4f DZ: %8.4f\n" $pkID  [gdbGet $ePk $pkVar(X)] [gdbGet $ePk $pkVar(Y)] [gdbGet $ePk $pkVar(Z)] [expr $pkX - $assX] [expr $pkY - $assY] [expr $pkZ - $assZ]]
       }
   }

puts ""
puts [format "DX RMS: %8.4f DY RMS: %8.4f DZ RMS: %8.4f N: %d" [getRMS $dxList] [getRMS $dyList] [getRMS $dzList] [llength $dxList]]

gdbWrite $pkTab $outName

exit 0
