#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 [lrange $argv 2 end]]
set ARGC      [llength $ARGV]

if {$ARGC < 3 || [flagLoc $ARGV -help]} \
   {
    puts "Update X_USER ... Position Values in a Peak Table:"
    puts " $argv0 \[-xFmt xFmt ...\] tabName spectrumName"
    exit 0
   }

set tabName  [lindex $ARGV end-1]
set specName [lindex $ARGV end]
set thisName [format $specName 1 1]
set fdata    0

set fmtInfo(X) %.3e
set fmtInfo(Y) %.3e
set fmtInfo(Z) %.3e
set fmtInfo(A) %.3e

getArgD $ARGV -xFmt fmtInfo(X) 
getArgD $ARGV -yFmt fmtInfo(Y)
getArgD $ARGV -zFmt fmtInfo(Z)
getArgD $ARGV -aFmt fmtInfo(A)


if {![file exists $tabName]}  {exit 0}
if {![file exists $thisName]} {exit 0}

set error [rdFDATA $thisName fdata]
if {$error} {exit 0}

set t        [gdbRead table -in $tabName]
set vList    [gdbGetInfo $t -parms varNames]
set adjCount 0

puts $vList

foreach dim {X Y Z A} \
   {
    if {[lsearch -exact $vList ${dim}_AXIS] < 0} break

    if {[lsearch -exact $vList ${dim}_USER] < 0} \
       {
        gdbCreate variable -parent $t -varType float -name ${dim}_USER -varFmt $fmtInfo($dim)
       }

    incr adjCount
   }

if {!$adjCount} {exit 0}

set eList [gdbGetInfo $t -parms entries]

foreach dim {X Y Z A} \
   {
    if {[lsearch -exact $vList ${dim}_AXIS] < 0} break

    set sw   [getParm $fdata NDSW   CUR_${dim}DIM]
    set car  [getParm $fdata NDCAR  CUR_${dim}DIM]
    set orig [getParm $fdata NDORIG CUR_${dim}DIM]
    set size [getParm $fdata NDSIZE CUR_${dim}DIM]

    set n [expr $size - 1]
    if {$n < 2} {set n 1}

    if {$car < $orig} \
       {
        set s1 [expr $orig - $sw]
        set sN $orig
       } \
    else \
       {
        set s1 [expr $orig + $sw]
        set sN $orig
       }

    set uw [expr $sN - $s1]

    foreach e $eList \
      {
       set p [gdbGet $e ${dim}_AXIS]
       set u [expr $s1 + $uw*($p - 1.0)/$n]

       gdbSet $e ${dim}_USER $u
      }    
   }

gdbWrite $t $tabName

exit
