#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"

#
# Initialize database.

proc setUpDBase {} \
{
   global dBase assTab varNames

   gdbInit

   set assVars {
                {INDEX    int   %5d}
                {HN       float %8.3f}
                {N        float %8.3f}
                {MATCH    text  %s}
                {HEIGHT   float %e}
                {ID       int   %4d}
                {EXP_NAME text  %-12s} 
                {EXP_PPM  float %8.3f}
                {ASS      text  %-12s}
               }

   set dBase    [gdbCreate dBase -name Valpha]
   set assTab   [gdbCreate table -name Assignments]
   set varNames ""

   foreach i $assVars \
      {
       gdbCreate variable -parent $assTab \
          -name [lindex $i 0] -varType [lindex $i 1] -varFmt [lindex $i 2]

       lappend varNames [lindex $i 0]
      }
}

#
# Set adjustable parameters.

proc setUpVars {} \
{
   global fileList nameList idList nList expCount groupCount
   global n15First n15Last swList idList zFirst zLast zTol zSW zNeg
   global maxTol posList dHN dN limitCB limitST


   set maxTol  0.220
   set dHN     0.015
   set dN      0.100

   set limitCB 41.0
   set limitST 52.0

   set groupCount  0

   set nameList(1) HNCO
   set fileList(1) ../hnco/hnco.tab
   set n15First(1) 132.194
   set n15Last(1)  104.067
   set n15Size(1)      128
   set swList(1)    28.349
   set posList(1)        1
   set zFirst(1)   179.653
   set zLast(1)    171.436
   set zSW(1)        8.282
   set zTol(1)       0.080
   set zNeg(1)           1 
   set nList(1)          1

   set nameList(2) CBCANH 
   set fileList(2) ../cbcanh/cbcanh.tab
   set n15First(2) 135.006
   set n15Last(2)  101.300
   set n15Size(2)      128
   set swList(2)    33.972
   set posList(2)        0
   set zFirst(2)    72.295 
   set zLast(2)     20.116
   set zSW(2)       52.591
   set zTol(2)       0.220
   set zNeg(2)           1 
   set nList(2)          4

   set nameList(3) CBCA(CO)NH
   set fileList(3) ../cbcaconh/cbcaconh.tab
   set n15First(3) 130.004
   set n15Last(3)  106.223
   set n15Size(3)      128
   set swList(3)    23.968
   set posList(3)        1
   set zFirst(3)    65.721  
   set zLast(3)     26.587 
   set zSW(3)       39.443 
   set zTol(3)       0.200
   set zNeg(3)           0
   set nList(3)          2
  
   set nameList(4) HNCA
   set fileList(4) ../hnca/hnca.tab
   set n15First(4) 132.293
   set n15Last(4)  103.970
   set n15Size(4)      128
   set swList(4)    28.546
   set posList(4)        1
   set zFirst(4)    69.805 
   set zLast(4)     42.411 
   set zSW(4)       27.610 
   set zTol(4)       0.200
   set zNeg(4)           0
   set nList(4)          2

   set expCount          4

   for {set i 1} {$i <= $expCount} {incr i} \
      {
       set idList($nameList($i)) $i

       set n15Last($i) \
          [expr $n15Last($i) - ($n15First($i)-$n15Last($i))/$n15Size($i)]
      }
}

#
# Return the N best peaks from an entry list:

proc pkCompare {pk1 pk2} \
{
   set v1 [gdbGet $pk1 HEIGHT]
   set v2 [gdbGet $pk2 HEIGHT]
   set d  [expr abs($v1) - abs($v2)]

   if {$d < 0} {return -1}
   if {$d > 0} {return  1}

   return 0
}

proc getBestPeaks {hnLoc n15Loc n pkList} \
{
   set newList [lsort -decreasing -command pkCompare $pkList]
   return [lrange $newList 0 [expr $n - 1]] 
}

#
# Delete a list item

proc delNthItem { eList loc } \
{
    set n [llength $eList]
    set aList ""

    for {set i 0} {$i < $n} {incr i} \
       {
        if {$i != $loc} {lappend aList [lindex $eList $i]}
       }

    return $aList
}

#
# Count the inconsistancies in a proposed assignment.

proc testPerm { assList shiftList tolList } \
{
   global limitST limitCB

   set n [llength $assList]

   set badCount 0
   set count    0

   for {set i 0} {$i < $n} {incr i} \
      {
       set strI   [lindex $assList $i]
       set zTolI  [lindex $tolList $i]
       set shiftI [lindex $shiftList $i]

       if {"$strI" == "CA(i)" || "$strI" == "CA(i-1)"} \
          {
           if {$shiftI < $limitCB} \
              {
               incr badCount
              }
          }
 
       for {set j [expr $i+1]} {$j < $n} {incr j} \
           {
            set strJ   [lindex $assList $j]
            set zTolJ  [lindex $tolList $j]
            set shiftJ [lindex $shiftList $j]

            if {$zTolI > $zTolJ} {set zTolJ $zTolI}

            if {![string compare $strI $strJ]} \
               {
                if {abs($shiftI - $shiftJ) > $zTolJ} \
                   {
                    incr badCount
                   }
               } \
            else \
               {
                switch $strI$strJ \
                   {
                    CA(i)CB(i) -
                    CA(i-1)CB(i-1) \
                       {
                        if {$shiftI > $limitST && $shiftJ > $limitST} \
                           {
                            if {$shiftI > $shiftJ} {incr badCount}
                           } \
                        else \
                           {
                            if {$shiftI < $shiftJ} {incr badCount}
                           }
                       }

                    CB(i)CA(i) -
                    CB(i-1)CA(i-1) \
                       {
                        if {$shiftI > $limitST && $shiftJ > $limitST} \
                           {
                            if {$shiftI < $shiftJ} {incr badCount}
                           } \
                        else \
                           {
                            if {$shiftI > $shiftJ} {incr badCount}
                           }
                       }

                    default {incr count}
                   }
               }
           }
      }

    return $badCount
}
 
#
# Read the 2D HN/N peak table and the related 3D peak tables.

proc readTables {} \
{
   global fileList tabList hnTab nameList expCount

   set hnTab [gdbRead table -in ../hn/hn.tab -name HN -verb]

   for {set i 1} {$i <= $expCount} {incr i} \
       {
        set tabList($i) \
           [gdbRead table -in $fileList($i) -name $nameList($i) -verb]
       }
}

#
# Associated 3D peaks with corresponding 2D HN/N peaks.

proc buildAssign {} \
{
   global fileList tabList nameList nList varNames expCount groupCount
   global n15First n15Last n15Size swList
   global hnTab assTab
   global dHN dN

#
# For each 2D peak:
#  Find its HN/N coord.
#  For each 3D spectrum:
#   Find the equivalent HN/N coord after folding.
#   Find the list of peaks in that spectrum at the given HN/N coord.

   set entry      [gdbGetInfo $hnTab -parms entry]
   set entryCount 0

   while {$entry} \
      {
       set id     [gdbGet $entry INDEX]
       set hnPPM  [gdbGet $entry X_PPM]
       set n15PPM [gdbGet $entry Y_PPM]

       set count 0

       for {set i 1} {$i <= $expCount} {incr i} \
          {
           set isBound 0

           if {abs($n15PPM-$n15First($i)) < $dN || \
                 abs($n15PPM-$n15Last($i)) < $dN} \
              {
               puts -nonewline "*** Notice: group $id is near $nameList($i)"
               puts " 15N fold boundary $n15Last($i)ppm."
               set isBound 1
              }

           set tab $tabList($i)

           if {$n15PPM > $n15First($i)} \
              {
               set foldPPM [expr $n15PPM - $swList($i)]
               puts -nonewline "Folding group $id $nameList($i)"
               puts " 15N $n15PPM -> $foldPPM, limit $n15First($i)"
              } \
           elseif {$n15PPM < $n15Last($i)} \
              {
               set foldPPM [expr $n15PPM + $swList($i)]
               puts -nonewline "Folding group $id $nameList($i)"
               puts " 15N $n15PPM -> $foldPPM, limit $n15Last($i)"
              } \
           else \
              {
               set foldPPM $n15PPM
              }

           set cond  "near(Y_PPM,$hnPPM,$dHN) && near(Z_PPM,$foldPPM,$dN)"
           set fList [gdbSelect -from $tab -cond $cond]

           if {$isBound} \
              {
               set cond  "near(Y_PPM,$hnPPM,$dHN) && near(Z_PPM,$n15PPM,$dN)"
               set uList [gdbSelect -from $tab -cond $cond]

               if {[llength $uList] > [llength $fList]} \
                  {
                   set foldPPM $n15PPM
                   set fList $oList
                  }
              }

           set eList [getBestPeaks $hnPPM $foldPPM $nList($i) $fList]
           set ass   [getDefaultAss $nameList($i)] 

           foreach pk $eList \
              {
               set j [gdbCreate entry -parent $assTab]

               incr entryCount
               incr count

               set  xPPM  [gdbGet $pk Y_PPM]
               set  yPPM  [gdbGet $pk Z_PPM]
               set  zPPM  [gdbGet $pk X_PPM]
               set  hi    [gdbGet $pk HEIGHT]
               set  match "0,0"

               set  vals \
                  "$id $xPPM $yPPM $match $hi $count $nameList($i) $zPPM $ass"

               gdbSet $j $varNames $vals
              }
          }
 
       set entry [gdbNext $entry]
      }       

#
# Remove bad residue systems:
 
   set entry      [gdbGetInfo $hnTab -parms entry]
   set groupCount 0

   while {$entry} \
      {
       set id     [gdbGet $entry INDEX]
       set eList  [gdbSelect -from $assTab -cond "INDEX == $id"]

       if {[llength $eList] < $expCount + 1} \
          {
           puts "Deleting group $id."
           gdbDestroy $eList
          } \
       else \
          {
           incr groupCount
          }

       set entry  [gdbNext $entry]
      } 
}

#
# Set the default assignments for an unassigned peak.

proc getDefaultAss { eName } \
{
   switch $eName \
      {
       HNCO       {return "CO(i-1)"}
       HNCA       {return "CA(i-1)|CA(i)"}
       CBCA(CO)NH {return "CA(i-1)|CB(i-1)"}
       CBCANH     {return "CA(i-1)|CA(i)|CB(i-1)|CB(i)"}
       default    {return "None"}
      }
}

#
# Decide if two entries have related possible assignments.

proc assRelated { pk1 pk2 } \
{
    set ass1 [gdbGet $pk1 ASS]
    set ass2 [gdbGet $pk2 ASS]

    if {![strcasecmp $ass1 None] || ![strcasecmp $ass2 None]} {return 0}

    set atoms1 [split $ass1 "|"]
    set atoms2 [split $ass2 "|"]

    foreach i $atoms1 \
       {
        lappend aList1 [lindex [split $i "(),"] 0] 
       }

    foreach j $atoms2 \
       {
        lappend aList2 [lindex [split $j "(),"] 0]
       }

    foreach i $aList1 \
       {
        foreach j $aList2 \
           {
            if {![string compare $i $j]} {return 1}
           }
       } 

    return 0
}

#
# Set atom types as possible.

proc buildTypes {} \
{
   global fileList tabList nameList idList nList varNames expCount
   global dHN dN limitCB limitST 
   global zFirst zLast zSW zNeg
   global hnTab assTab

   set count 0

   for {set i 1} {$i <= $expCount} {incr i} \
      {
       switch $nameList($i) \
          {
           CBCANH \
              {
               set entry [gdbGetInfo $hnTab -parms entry]

               while {$entry} \
                  {
                   set id     [gdbGet  $entry INDEX]
                   set entry  [gdbNext $entry]
                   set pCount 0
                   set nCount 0
 
                   set    cond "" 
                   append cond "INDEX == $id"
                   append cond " && !strcmp( EXP_NAME, '$nameList($i)' )"

                   set eList [gdbSelect -from $assTab -cond $cond]

                   foreach pk1 $eList \
                      {
                       set shift1 [gdbGet $pk1 EXP_PPM]
                       set hi     [gdbGet $pk1 HEIGHT]

                       if {$hi > 0.0} \
                          {
                           incr pCount
                          } \
                       else \
                          {
                           incr nCount
                          }

                       if {$shift1 <= $limitCB} \
                          {
                           gdbSet $pk1 ASS "CB(i-1)|CB(i)"
                          }
                      }

                   if {$pCount == 2 && $nCount == 2} \
                      {
                       foreach pk1 $eList \
                          {
                           set hi [gdbGet $pk1 HEIGHT]

                           if {$hi > 0.0} \
                              {
                               gdbSet $pk1 ASS "CA(i-1)|CA(i)"
                              } \
                           else \
                              {
                               gdbSet $pk1 ASS "CB(i-1)|CB(i)"
                              }
                          }
                      } \
                   elseif {($pCount+$nCount == 4) && $zNeg($i)} \
                      {
                       puts "*** Notice: group $id may be misfolded."
                      }
                  }
              }

           CBCA(CO)NH \
              {
               set entry [gdbGetInfo $hnTab -parms entry]

               while {$entry} \
                  {
                   set id    [gdbGet $entry INDEX]
                   set entry [gdbNext $entry]
 
                   set    cond ""
                   append cond "INDEX == $id"
                   append cond " && !strcmp( EXP_NAME, '$nameList($i)' )"

                   set eList [gdbSelect -from $assTab -cond $cond]

                   if {[llength $eList] != 2} \
                      {
                       foreach pk1 $eList \
                          {
                           set shift1 [gdbGet $pk1 EXP_PPM]

                           if {$shift1 <= $limitCB} \
                              {
                               gdbSet $pk1 ASS "CB(i-1)"
                              }
                          }
                      } \
                   else \
                      {
                       set pk1 [lindex $eList 0]
                       set pk2 [lindex $eList 1]

                       set shift1 [gdbGet $pk1 EXP_PPM]
                       set shift2 [gdbGet $pk2 EXP_PPM]

                       if {$shift1 > $limitST && $shift2 > $limitST} \
                          {
                           puts "Group $id, used SER/THR rule."

                           if {$shift1 > $shift2} \
                              {
                               gdbSet $pk1 ASS "CB(i-1)"
                               gdbSet $pk2 ASS "CA(i-1)"
                              } \
                           else \
                              {
                               gdbSet $pk1 ASS "CA(i-1)"
                               gdbSet $pk2 ASS "CB(i-1)"
                              }
                          } \
                       else \
                          {
                           if {$shift1 > $shift2} \
                              {
                               gdbSet $pk1 ASS "CA(i-1)"
                               gdbSet $pk2 ASS "CB(i-1)"
                              } \
                           else \
                              {
                               gdbSet $pk1 ASS "CB(i-1)"
                               gdbSet $pk2 ASS "CA(i-1)"
                              }
                          }
                      }
                  }
              }

           default {incr count}
          }
      }
}

#
# Adjust for possible folding in groups of related peaks.

proc fixFold {} \
{
   global fileList tabList nameList idList nList varNames expCount
   global zFirst zLast zSW zTol zNeg posList
   global hnTab assTab
   global dHN dN

   set entry [gdbGetInfo $hnTab -parms entry]

   while {$entry} \
      {
       set id     [gdbGet $entry INDEX]
       set eList  [gdbSelect -from $assTab -cond "INDEX == $id"] 

       foreach i $eList \
          {
           set shift1  [gdbGet $i EXP_PPM]
           set name    [gdbGet $i EXP_NAME]
           set n       [gdbGet $i ID]
           set loc     $idList($name)
           set zTolI   $zTol($loc)
           set isMatch 0

           foreach offset "0.0 $zSW($loc) -$zSW($loc)" \
              {
               set thisShift [expr $shift1 + $offset]

               foreach j $eList \
                  {
                   set zTolJ $zTol($loc)

                   if {$zTolI > $zTolJ} {set zTolJ $zTolI}

                   if {$i != $j && [assRelated $i $j]} \
                      {
                       set shift2 [gdbGet $j EXP_PPM]

                       if {abs($thisShift - $shift2) <= $zTolJ} \
                          {
                           set isMatch 1
                           break
                          }
                      }
                  }
           
               if {$isMatch} {break}
              }

           if {$isMatch} \
              {
               if {$offset != 0.0} \
                  {
                   puts "Fold $name $id from $shift1 to $thisShift (match)"
                   gdbSet $i EXP_PPM $thisShift

                   if {$zNeg($loc)} \
                      {
                       set hi [gdbGet $i HEIGHT]
                       gdbSet $i HEIGHT [expr -($hi)]
                      }
                  }
              } \
           else \
              {
               set hi [gdbGet $i HEIGHT]

               if {($hi < 0.0) && $zNeg($loc) && $posList($loc)} \
                  {
                   if {$shift1 > 0.5*($zFirst($loc) + $zLast($loc))} \
                      {
                       set thisShift [expr $shift1 - $zSW($loc)]
                      } \
                   else \
                      {
                       set thisShift [expr $shift1 + $zSW($loc)]
                      }

                   puts "Fold $name $id from $shift1 to $thisShift (sign)"
                   gdbSet $i EXP_PPM $thisShift
                  }
              }
          }

       set entry [gdbNext $entry]
      }
}

#
# Find all shift alignments.

proc findMatches {} \
{
   global assTab zTol idList maxTol

   set i     [gdbGetInfo $assTab -parms entry]
   set count [gdbGetInfo $assTab -parms entryCount]
   set n     0

   while {$i} \
      {
       incr n
       puts -nonewline "$n of $count\015"
       flush stdout

       set shift1  [gdbGet $i EXP_PPM]
       set name    [gdbGet $i EXP_NAME]
       set loc     $idList($name)
       set zTolI   $zTol($loc)
       set mList   ""

       set cond  "near( EXP_PPM, $shift1, $maxTol )"
       set jList [gdbSelect -from $assTab -cond $cond]

       set j [gdbGetInfo $assTab -parms entry]

       foreach j $jList \
          {
           if {$i != $j && [assRelated $i $j]} \
              {
               set shift2 [gdbGet $j EXP_PPM]
               set zTolJ  $zTol($loc)

               if {$zTolI > $zTolJ} {set zTolJ $zTolI}

               if {abs($shift1 - $shift2) <= $zTolJ} \
                  {
                   lappend mList \
                      [join [gdbGet $j "INDEX ID"] ","]
                  }
              }
          }

       if {[llength $mList] > 0} \
          {
           gdbSet $i MATCH [join $mList "|"] 
          }

       set i [gdbNext $i]
      }

   puts ""
}

#
# matchTypes: adjust atom type assignments by simple matches.

proc matchTypes {} \
{
   global idList zTol assTab hnTab

   set entry [gdbGetInfo $hnTab -parms entry]

while {$entry} \
  {
   set id     [gdbGet $entry INDEX]
   set eList  [gdbSelect -from $assTab -cond "INDEX == $id"]
   set n      [llength $eList]

   for {set pass 1} {$pass <= [expr 2*$n]} {incr pass} \
       {
        if {$n < 2} break

        set count  0

        for {set i 0} {$i < $n} {incr i} \
           {
            set pkI [lindex $eList $i]

            for {set j 0} {$j < $n} {incr j} \
               {
                if {$i == $j} break

                set pkJ     [lindex $eList $j]

                set ass     [gdbGet $pkI ASS]
                set assI    [split $ass "|"]
                set cntI    [llength $assI]

                if {$cntI != 1} break

                set ass     [gdbGet $pkJ ASS]
                set assJ    [split $ass "|"]
                set cntJ    [llength $assJ]

                set shiftI  [gdbGet $pkI EXP_PPM]
                set nameI   [gdbGet $pkI EXP_NAME]
                set idI     [gdbGet $pkI ID]
                set tolI    $zTol($idList($nameI))
 
                set shiftJ  [gdbGet $pkJ EXP_PPM]
                set nameJ   [gdbGet $pkJ EXP_NAME]
                set idJ     [gdbGet $pkJ ID]
                set tolJ    $zTol($idList($nameJ))

                if {$tolJ < $tolI} {set tolJ $tolI}

                set d      [expr abs($shiftI - $shiftJ)]
                set loc    [lsearch -exact $assJ $assI]

                if {$d < $tolJ} \
                   {
                    if {($loc >= 0) && ($cntJ > 1)} \
                       {
                        puts -nonewline "*** Set match,"
                        puts " $assI group $id, peak $idJ, pass $pass."

                        gdbSet $pkJ ASS $assI
                        incr count
                       } \
                    elseif {($cntJ == 1) && [strcasecmp $assI $assJ]} \
                       {
                        puts -nonewline "*** Degeneracy,"
                        puts " ($assI)/($assJ) group $id."
                       }
                   } \
                elseif {$d > 2.0*$tolJ} \
                   {
                    if {![string compare $assI $assJ]} \
                       {
                        puts -nonewline "*** Conflict,"
                        puts " ($assI)/($assJ) group $id."
                       } \
                    elseif {($loc >= 0) && ($cntJ > 1)} \
                       {
                        puts -nonewline "*** Removed match,"
                        puts " $assI group $id, peak $idJ, pass $pass."

                        set newAss [delNthItem $assJ $loc]
                        incr count

                        gdbSet $pkJ ASS [join $newAss "|"]
                       }
                   }
               }
           }

       if {$count == 0} break
      }

   set entry [gdbNext $entry]
  }

   puts ""
}

#
# refineTypes: refine atom type assignments by testing permutations.

proc refineTypes {} \
{
   global idList zTol assTab hnTab

   set entry [gdbGetInfo $hnTab -parms entry]

   while {$entry} \
      {
       set id     [gdbGet $entry INDEX]
       set eList  [gdbSelect -from $assTab -cond "INDEX == $id"]

       if {[llength $eList] > 1} \
          {
           set assList   ""
           set shiftList ""
           set tolList   ""
           set goodList  ""

           foreach i $eList \
              {
               set shift  [gdbGet $i EXP_PPM]
               set name   [gdbGet $i EXP_NAME]
               set ass    [gdbGet $i ASS]

               lappend shiftList $shift
               lappend assList   $ass
               lappend tolList   $zTol($idList($name))
              }

           set pList [permList [join $assList ","]]

           set goodCount 0

           foreach i $pList \
              {
               set badCount [testPerm [split $i ","] $shiftList $tolList]

               if {$badCount == 0} \
                  {
                   lappend goodList $i
                   incr goodCount
                  }
              }

           if {$goodCount == 1} \
              {
               set assList [split [lindex $goodList 0] ","]
               set n  0

               foreach i $eList \
                  {
                   gdbSet $i ASS [lindex $assList $n]
                   incr n
                  }

               puts -nonewline "Adjusted group $id.\015"
               flush stdout
              } \
          }

       set entry [gdbNext $entry]
      }

   puts ""
}

#
# Main:

setUpVars
setUpDBase
readTables

puts "Initial groupings ..."
buildAssign
puts "... created $groupCount initial groups."

puts "Folding ..."
fixFold

puts "Initial Atom Types ..."
buildTypes

puts "Peak alignments ..."
findMatches

puts "Match Atom Types ..."
matchTypes

puts "Refine Atom Types ..."
refineTypes

puts "Writing results to ass.tab ..."
#gdbDestroy [gdbFindByName $assTab MATCH]
gdbWrite $assTab ass.tab

gdbCheck -verb

exit

--------------794B15FB59E2--


