#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Plot 2D Spectrum with 1D Spectra or Projections:"
    puts " -in    inName   \[test.ft2\]  Input 2D Spectrum."
    puts " -out   outName  \[roi.ps\]    Output PostScript Plot File."
    puts " -xProj xPName   \[Proj\]      Optional 1D Spectrum for X-Axis."
    puts " -yProj yPName   \[Proj\]      Optional 1D Spectrum for Y-Axis."
    puts " -hi    hi       \[Auto\]      First Contour Height."
    puts " -fhi   fhi      \[Auto\]      Min 1D Height."
    puts " -lhi   lhi      \[Auto\]      Max 1D Height."
    puts " -x1    x1       \[First\]     X-Axis Start Coord PPM."
    puts " -xn    xn       \[Last\]      X-Axis End Coord PPM."
    puts " -y1    y1       \[First\]     Y-Axis Start Coord PPM."
    puts " -yn    yn       \[Last\]      Y-Axis End Coord PPM."
    puts " -verb                       Verbose Mode ON (Default)."
    puts " -noverb                     Verbose Mode OFF."
    puts "Optional Peak Labels:"
    puts " -tab   tabName  \[None\]      Peak Table of PPM Coords."
    puts " -xVar  xVar     \[Auto\]      Name of X-Axis PPM Variable in Table."
    puts " -yVar  yVar     \[Auto\]      Name of X-Axis PPM Variable in Table."
    puts " -lab   lVar     \[INDEX\]     Name of Label Variable in Table."
    exit 1
   }

set inName  test.ft2
set outName roi.ps
set xProj   Proj 
set yProj   Proj 
set hi      Auto
set fhi     Auto
set lhi     Auto
set x1      First
set xn      Last
set y1      First
set yn      Last
set plev    20
set nlev    20
set verbose 1

set tabName None
set labVar  INDEX
set xVar    Auto
set yVar    Auto

getArgD $ARGV -in     inName
getArgD $ARGV -out    outName
getArgD $ARGV -hi     hi
getArgD $ARGV -fhi    fhi
getArgD $ARGV -lhi    lhi
getArgD $ARGV -xProj  xProj
getArgD $ARGV -yProj  yProj
getArgD $ARGV -x1     x1
getArgD $ARGV -xn     xn
getArgD $ARGV -y1     y1
getArgD $ARGV -yn     yn
getArgD $ARGV -plev   plev
getArgD $ARGV -nlev   nlev

getArgD $ARGV -tab  tabName
getArgD $ARGV -lab  labVar
getArgD $ARGV -xVar xVar 
getArgD $ARGV -yVar yVar 

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#-------------------------------------------------------------

#
# Determine horizontal and vertical axis parameters: 

if {![file exists $inName]} \
   {
    puts stderr "Error finding input spectrum $inName"
    exit 1
   }

rdFDATA $inName fdata

set tpFlag [getParmI $fdata FDTRANSPOSED NULL_DIM]
set xSize  [getParmI $fdata NDSIZE CUR_XDIM]
set ySize  [getParmI $fdata NDSIZE CUR_YDIM]

if {$tpFlag} \
   {
    set xName Y_AXIS
    set yName X_AXIS

    if {![strcasecmp $xVar Auto]} {set xVar Y_PPM}
    if {![strcasecmp $yVar Auto]} {set yVar X_PPM}

    if {![strcasecmp $x1 First]} {set x1 [pnt2spec $fdata CUR_YDIM 1      ppm]}
    if {![strcasecmp $xn Last]}  {set xn [pnt2spec $fdata CUR_YDIM $ySize ppm]}
    if {![strcasecmp $y1 First]} {set y1 [pnt2spec $fdata CUR_XDIM 1      ppm]}
    if {![strcasecmp $yn Last]}  {set yn [pnt2spec $fdata CUR_XDIM $xSize ppm]}
   } \
else \
   {
    set xName X_AXIS
    set yName Y_AXIS

    if {![strcasecmp $xVar Auto]} {set xVar X_PPM}
    if {![strcasecmp $yVar Auto]} {set yVar Y_PPM}

    if {![strcasecmp $x1 First]} {set x1 [pnt2spec $fdata CUR_XDIM 1      ppm]}
    if {![strcasecmp $xn Last]}  {set xn [pnt2spec $fdata CUR_XDIM $xSize ppm]}
    if {![strcasecmp $y1 First]} {set y1 [pnt2spec $fdata CUR_YDIM 1      ppm]}
    if {![strcasecmp $yn Last]}  {set yn [pnt2spec $fdata CUR_YDIM $ySize ppm]}
   }

if {$verbose} \
   {
    puts "Reading Region of Interest from file $inName:"
    puts [format " %-8s %8.3fppm %8.3fppm" $xVar $x1 $xn]
    puts [format " %-8s %8.3fppm %8.3fppm" $yVar $y1 $yn]
    puts "Output PostScript File: $outName"
   }

#
# Read spectrum and 1D/projections:

readROI -roi 1 -ndim 2 -in $inName \
        -x $xName ${x1}ppm ${xn}ppm -y $yName ${y1}ppm ${yn}ppm

if {![strcasecmp $xProj Proj]} \
   {
    readROI -roi 2 -ndim 1 -in $inName \
            -x $xName ${x1}ppm ${xn}ppm -y $yName ${y1}ppm ${yn}ppm -proj
   } \
else \
   {
    if {![file exists $xProj]} \
       {
        puts stderr "Error finding X-Axis 1D $xProj"
        exit 1
       }

    readROI -roi 2 -ndim 1 -in $xProj -x X_AXIS ${x1}ppm ${xn}ppm
   }

if {![strcasecmp $yProj Proj]} \
   {
    readROI -roi 3 -ndim 1 -in $inName \
            -x $yName ${y1}ppm ${yn}ppm -y $xName ${x1}ppm ${xn}ppm -proj
   } \
else \
   {
    if {![file exists $yProj]} \
       {
        puts stderr "Error finding Y-Axis 1D $yProj"
        exit 1
       }

    readROI -roi 3 -ndim 1 -in $yProj -x X_AXIS ${y1}ppm ${yn}ppm
   }

#
# Draw 2D spectrum and 1D:

plotOpen -landscape -color -plotName $outName

drawRec -x 0% 100% -y 0% 100% -fg lightGray

defineCell -cell 1 -x 1.50in 9.0in -y 1.50in 7.50in 
defineCell -cell 2 -x 1.50in 9.0in -y 0.25in 1.40in
defineCell -cell 3 -x 0.25in 1.4in -y 1.50in 7.50in

drawROI -color1 black -color2 red -plev 20 -nlev 20 \
        -roi 1 -cell 1 -axis Bottom Right -xGrid -yGrid -bg white -hi $hi 

drawROI -roi 2 -cell 2 -zero -fhi $fhi -lhi $lhi -top  -bg white
drawROI -roi 3 -cell 3 -zero -fhi $fhi -lhi $lhi -left -bg white

#
# Determine peak label parameters:

set peakList ""

if {[strcasecmp $tabName None]} \
   {
    if {![file exists $tabName]} \
       {
        puts stderr "Error finding peak table $tabName"
       }

    set pkTab [gdbRead table -in $tabName]
    set cond  "$xVar >= $xn && $xVar <= $x1 && $yVar >= $yn && $yVar <= $y1"
    set peakList [gdbSelect -from $pkTab -cond $cond]
   }
 
#
# Draw peak labels:

foreach peak $peakList \
   {
    set xPPM [gdbGet $peak $xVar]ppm
    set yPPM [gdbGet $peak $yVar]ppm
    set id   [gdbGet $peak $labVar]

    drawText -cell 1 -roiRef 1 -x $xPPM -y $yPPM \
             -box -fg blue -bg white -text $id
   }

plotClose
exit
