#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts " -in1    inName1 \[A.dat\]  Input Name for Spectrum 1."
    puts " -in2    inName2 \[B.dat\]  Input Name for Spectrum 2."
    puts " -out    outName \[roi.ps\] PostScript Output."
    puts "Contour Settings:"
    puts " -hi1    hi1     \[Auto\]   First Contour Height for Spectrum 1."
    puts " -hi2    hi2     \[Auto\]   First Contour Height for Spectrum 2."
    puts " -hi     hi      \[Auto\]   Sets both -hi1 and -hi2 Values."
    puts " -plev   np      \[20\]     Number of Positive Contours."
    puts " -nlev   nn      \[20\]     Number of Negative Contours."
    puts " -mult   c       \[7.0\]    Factor for Auto-Estimate of Heights."
    puts " -factor f       \[1.3\]    Factor for Contour Spacing."
    puts "Contour Heights for Equal-Spaced Contours (-mult 1.0):"
    puts " -fhi1   hi1     \[Auto\]   First Contour Height (Same as -hi1)."
    puts " -fhi2   hi2     \[Auto\]   First Contour Height (Same as -hi2)."
    puts " -fhi    hi      \[Auto\]   Sets both -fhi1 and -fhi2 (Same as -hi)."
    puts " -lhi1   lhi1    \[Auto\]   Last Contour Height for Spectrum 1."
    puts " -lhi2   lhi2    \[Auto\]   Last Contour Height for Spectrum 2."
    puts " -lhi    lhi     \[Auto\]   Sets both -lhi1 and -lhi2 Values."
    puts "Contour Colors:"
    puts " -pcolor1 pc1    \[blue\]   Positive Contour Color for Spectrum 1."
    puts " -ncolor1 nc1    \[cyan\]   Negative Contour Color for Spectrum 1."
    puts " -pcolor2 pc2    \[red\]    Positive Contour Color for Spectrum 2."
    puts " -ncolor2 nc2    \[pink\]   Negative Contour Color for Spectrum 2."
    puts "Spectral Region to Plot:"
    puts " -sx1     sx1    \[0%\]     X-Axis First Coord in Spectrum."
    puts " -sxn     sxn    \[100%\]   X-Axis Last Coord in Spectrum."
    puts " -sy1     sy1    \[0%\]     Y-Axis First Coord in Spectrum."
    puts " -sxn     syn    \[100%\]   Y-Axis Last Coord in Spectrum."
    puts "Graphics Region on Page:"
    puts " -px1     px1    \[0.5in\]  X-Axis First Coord on Page."
    puts " -pxn     pxn    \[7.5in\]  X-Axis Last Coord on Page."
    puts " -py1     py1    \[0.5in\]  Y-Axis First Coord on Page."
    puts " -pyn     pyn    \[7.5in\]  Y-Axis Last Coord on Page."
    puts "Axis Settings (orig= end= step= units= tic= lab= fmt=):"
    puts " -top     tStr   \[None\]   Specification for Top Axis, if Any."
    puts " -bottom  bStr   \[Bottom\] Specification for Bottom Axis, if Any."
    puts " -left    lStr   \[None\]   Specification for Left Axis, if Any."
    puts " -right   rStr   \[Right\]  Specification for Right Axis, if Any."
    puts "Orientation:"
    puts " -portrait                Portrait Mode (Default)."
    puts " -landscape               Landscape Mode."
    puts "Options for Multiple Plotting:"
    puts " -append                  Append to Existing Plot File."
    puts " -close                   Finalize and Close Plot File (Default)."
    puts " -noclose                 Do Not Finalize Plot File."
    puts "Extra NMRWish Command Options, Conclude Each with -- After Arguments:"
    puts " -plotArgs  argList   Arguments for plotOpen Command." 
    puts " -draw1Args argList   Arguments for drawROI Command, Spectrum 1." 
    puts " -draw2Args argList   Arguments for drawROI Command, Spectrum 2." 
    puts " -drawArgs  argList   Sets Both -draw1Args and -draw2Args Options."
    exit 0
   }

set inName1  A.dat
set inName2  B.dat
set outName  roi.ps

set plotArgs    "" 
set draw1Args   ""
set draw2Args   ""

set closeFlag   1
set appendFlag  0

set lhi1     Auto
set lhi2     Auto
set hi1      Auto
set hi2      Auto
set mult     7.0
set factor   1.3

set aT       None
set aB       Bottom
set aL       None
set aR       Right

set pColor1  blue
set nColor1  cyan 

set pColor2  red
set nColor2  pink 

set pLev     20
set nLev     20

set sx1        0%
set sxn      100% 
set sy1        0% 
set syn      100%

set px1 0.50in
set pxn 7.50in
set py1 0.50in
set pyn 7.50in

set portraitFlag 1

set ARGV [getListArgDT $ARGV -plotArgs  plotArgs]
set ARGV [getListArgDT $ARGV -drawArgs  draw1Args]
set ARGV [getListArgDT $ARGV -drawArgs  draw2Args]
set ARGV [getListArgDT $ARGV -draw1Args draw1Args]
set ARGV [getListArgDT $ARGV -draw2Args draw2Args]
set ARGC [llength $ARGV]

if {[flagLoc $ARGV -append]}   {set appendFlag 1}
if {[flagLoc $ARGV -noappend]} {set appendFlag 0}

if {[flagLoc $ARGV -close]}    {set closeFlag  1}
if {[flagLoc $ARGV -noclose]}  {set closeFlag  0}

getArgD $ARGV -in1 inName1
getArgD $ARGV -in2 inName2
getArgD $ARGV -out outName

getArgD $ARGV -hi  hi1
getArgD $ARGV -hi  hi2 
getArgD $ARGV -hi1 hi1
getArgD $ARGV -hi2 hi2 

getArgD $ARGV -fhi  hi1
getArgD $ARGV -fhi  hi2 
getArgD $ARGV -fhi1 hi1
getArgD $ARGV -fhi2 hi2 

getArgD $ARGV -lhi  lhi1
getArgD $ARGV -lhi  lhi2
getArgD $ARGV -lhi1 lhi1
getArgD $ARGV -lhi2 lhi2

getArgD $ARGV -top    aT
getArgD $ARGV -bottom aB
getArgD $ARGV -left   aL
getArgD $ARGV -right  aR

getArgD $ARGV -mult   mult
getArgD $ARGV -factor factor

getArgD $ARGV -plev   pLev
getArgD $ARGV -nlev   nLev

getArgD $ARGV -pcolor1 pColor1
getArgD $ARGV -ncolor1 nColor1

getArgD $ARGV -pcolor2 pColor2
getArgD $ARGV -ncolor2 nColor2

getArgD $ARGV -sx1 sx1
getArgD $ARGV -sxn sxn
getArgD $ARGV -sy1 sy1
getArgD $ARGV -syn syn

getArgD $ARGV -px1 px1
getArgD $ARGV -pxn pxn
getArgD $ARGV -py1 py1
getArgD $ARGV -pyn pyn

if {[flagLoc $ARGV -portrait]}  {set portraitFlag 1}
if {[flagLoc $ARGV -landscape]} {set portraitFlag 0}

#
#---------------------------------------------------------------------

if {[strcasecmp $inName1 None]} \
   {
    if {![file exists $inName1]} \
       {
        puts stderr "Error Finding Spectrum Input 1: $inName1"
        exit 1
       }

    set error [rdFDATA $inName1 fdata1]

    if {$error} \
       {
        puts stderr "Error Reading Header in Spectrum Input 1: $inName1"
        exit 1
       }

    if {![strcasecmp $hi1 Auto]} \
       {
        set noise [estSpecNoise $inName1]
        set hi1   [expr $noise*$mult]

        if {$hi1 == 0.0} {set hi1 1.0e+6}
    
        if {![strcasecmp $hi2 Auto]} {set hi2 $hi1}
       }

    set tpFlag [getParmI $fdata1 FDTRANSPOSED NULL_DIM]

    if {$tpFlag} \
       {
        readROI -roi 1 -x Y_AXIS $sy1 $syn -y X_AXIS $sx1 $sxn -in $inName1
       } \
    else \
       {
        readROI -roi 1 -x X_AXIS $sx1 $sxn -y Y_AXIS $sy1 $syn -in $inName1
       }

    if {![strcasecmp $lhi1 Auto]} {set lhi1 [getROIStat 1 vMax]}

    set rInfo [getROIInfo -roi 1]
    set fdata [lindex $rInfo 1]
    set xSize [lindex [lindex $rInfo 2] 0]
    set ySize [lindex [lindex $rInfo 2] 1]
    
    set rx1 [pnt2spec $fdata X_AXIS 1      ppm]ppm
    set rxn [pnt2spec $fdata X_AXIS $xSize ppm]ppm
    set ry1 [pnt2spec $fdata Y_AXIS 1      ppm]ppm
    set ryn [pnt2spec $fdata Y_AXIS $ySize ppm]ppm
   }

if {[strcasecmp $inName2 None]} \
   {
    set error [rdFDATA $inName2 fdata2]

    if {![file exists $inName2]} \
       {
        puts stderr "Error Finding Spectrum Input 2: $inName2"
        exit 1
       }

    if {$error} \
       {
        puts stderr "Error Reading Header in Spectrum Input 2: $inName2"
        exit 1
       }

    if {![strcasecmp $hi2 Auto]} \
       {
        set noise [estSpecNoise $inName2]
        set hi2   [expr $noise*$mult]
    
        if {$hi2 == 0.0} {set hi2 1.0e+6}
       }

    set tpFlag [getParmI $fdata2 FDTRANSPOSED NULL_DIM]

    if {$tpFlag} \
       {
        readROI -roi 2 -x Y_AXIS $ry1 $ryn -y X_AXIS $rx1 $rxn -in $inName2
       } \
    else \
       {
        readROI -roi 2 -x X_AXIS $rx1 $rxn -y Y_AXIS $ry1 $ryn -in $inName2
       }

    if {![strcasecmp $lhi2 Auto]} {set lhi2 [getROIStat 2 vMax]}
   }

#
#----------------------------------------------------------------------------

if {$portraitFlag} \
   {
    eval "plotOpen $plotArgs -portrait  -color -plotName $outName"
   } \
else \
   {
    eval "plotOpen $plotArgs -landscape -color -plotName $outName"
   }

defineCell -cell 1 -x $px1 $pxn -y $py1 $pyn

set    draw1Cmnd "drawROI $draw1Args -roi 1 -cell 1"
append draw1Cmnd " -mult $factor -plev $pLev -nlev $nLev"
append draw1Cmnd " -color1 $pColor1 -color2 $nColor1 -axis $aT $aB $aL $aR"

set    draw2Cmnd "drawROI $draw1Args -roi 2 -cell 1"
append draw2Cmnd " -mult $factor -plev $pLev -nlev $nLev"
append draw2Cmnd " -color1 $pColor2 -color2 $nColor2"

if {$factor == 1.0} \
   {
    append draw1Cmnd " -fhi $hi1 -lhi $lhi1"
    append draw1Cmnd " -fhi $hi2 -lhi $lhi2"
   } \
else \
   {
    append draw1Cmnd " -hi $hi1"
    append draw2Cmnd " -hi $hi2"
   }

if {[strcasecmp $inName1 None]} {eval $draw1Cmnd} 
if {[strcasecmp $inName2 None]} {eval $draw2Cmnd} 

if {$closeFlag} plotClose

exit
