#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#
# Initialize:

set inName    ft/test%03d.ft3
set hi        2.0e+5

if {[flagLoc $ARGV -help]} \
   {
    puts "Hard Copy of planes in a 3D:"
    puts "  -in     inName       Spectral data file template."
    puts "  -hi     contHi       First Contour height."
    exit
   }

getArgD $ARGV -in  inName
getArgD $ARGV -hi  hi

#
# Open the first file plane and extract the header:

set thisName [format $inName 1]
set inUnit   [sysOpen $thisName r]

if {$inUnit < 0} \
   {
    puts "Error opening data file $thisName"
    exit 1
   }

set error [rdFDATAU $inUnit $fdata]

#
# For each plane in the Z-Axis:
#  Plot the plane.

if {![file isdirectory ps]} {file mkdir ps}

set zSize  [getParm $fdata NDSIZE       CUR_ZDIM]
set tpFlag [getParm $fdata FDTRANSPOSED NULL_DIM]
set zLab   [getParmStr $fdata NDLABEL CUR_ZDIM]

for {set iz 1} {$iz <= $zSize} {incr iz} \
   {
    set thisName [format $inName $iz]
    set psName   [format ps/plot%03d.ps $iz]
    set specVal  [pnt2spec $fdata CUR_ZDIM $iz ppm]

    puts "Preparing $psName from file $thisName."

    plotOpen -landscape -gray -plotName $psName -paper A4

    drawRec -x 0% 100% -y 0% 100% -fg lightGray
    defineCell -cell 1 -x 0.75cm 25.25cm -y 0.75cm 18.25cm

    drawText -cell 1 -x 0% -y 0% -yOff -0.5 \
       -text $thisName $zLab $specVal ppm

    drawSettings -reset

    set thisName [format $inName $iz]

    if {$tpFlag} \
       { 
        readROI -roi 1 -in $thisName -x Y_AXIS -y X_AXIS -ndim 2
       } \
    else \
       {
        readROI -roi 1 -in $thisName -x X_AXIS -y Y_AXIS -ndim 2
       }

    drawROI -cell 1 -roi 1 -xGrid -yGrid -axis Bottom Right \
       -bg white -hi $hi 

    plotClose
   }

exit
