#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Set default values:

set inName    /spunky5/liz/sry/wt/4dncnoe/ft/nc_noe%02d%03d.DAT
set tabName   /spunky1/liz/sry/wt/shifts/hsqc.tab
set hi        3.0e+5

set xName    HN
set yName    15N 
set zName    13C 
set aName    H(13C) 

set xVarName Y_PPM
set yVarName X_PPM

#
# Plot region specification.
# Use "cm", paperType "A4", etc. for metric specifications:

set px1       0.25in
set pxn       9.00in
set py1       0.25in
set pyn       7.50in

set paperType A

#
#----------------------------------------------------------------

if {[flagLoc $ARGV -help]} \
   {
    puts "Plane plots of a single 4D spectrum."
    puts "  -in     inName       Spectral data file template."
    puts "  -tab    tabName      2D Coordinate table for plane locations."
    puts "  -hi     contHi       First Contour height."
    puts "Strip Axis Names:"
    puts "  -xName  xName   \[HN\]     First axis name of plane 2D coord." 
    puts "  -yName  yName   \[N\]      Second axis name of plane 2D coord."
    puts "  -zName  zName   \[C\]      Axis name of plane X dimension." 
    puts "  -aName  zName   \[H\]      Axis name of plane Y dimension." 
    puts "Coordinate Table Names:"
    puts "  -xVar   xVar    \[X_PPM\]  Variable name of first 2D PPM coord."
    puts "  -yVar   yVar    \[Y_PPM\]  Variable name of second 2D PPM coord."
    exit
   }

getArgD $ARGV -in      inName
getArgD $ARGV -tab     tabName
getArgD $ARGV -hi      hi
 
getArgD $ARGV -xName   xName 
getArgD $ARGV -yName   yName 
getArgD $ARGV -zName   zName 
getArgD $ARGV -aName   aName 

getArgD $ARGV -xVar    xVarName 
getArgD $ARGV -yVar    yVarName 

#
# Initialize database engine.
# Read the 2D peak table.
 
gdbInit
gdbCreate dBase -name NMR

set pageCount  0

set hnTab [gdbRead table -in $tabName -name 2D -verb]
set entry [gdbFirst $hnTab]

#
# For each 2D entry:
#   Extract the 2D coords.
#   Open new plot.
#   Extract and plot the plane.

while {$entry} \
   {
    set xPPM [gdbGet $entry $xVarName]ppm
    set yPPM [gdbGet $entry $yVarName]ppm
    set ass  [gdbGet $entry ASS] 

    incr pageCount
    set  plotName [format roi%03d.ps $pageCount]
    puts "Page $pageCount:"

    plotOpen -color -landscape -plotName $plotName -paper $paperType
    drawRec -x 0% 100% -y 0% 100% -fg lightGray 

    puts " [format %10s $xPPM] [format %10s $yPPM] ..."

    defineCell -x $px1 $pxn -y $py1 $pyn -cell 1

    readROI -in $inName -roi 1 -ndim 2 \
       -x $zName -y $aName -dz $xName $xPPM 1 -da $yName $yPPM 1

    drawROI -roi 1 -cell 1 -bg white -hi $hi \
       -xGrid -yGrid -axis Bottom Right -color1 black -color2 red

    drawText -cell 1 -x 0% -y 0% -yOff -0.5 \
       -text $xName $xPPM, $yName $yPPM $ass

    plotClose

    set entry [gdbNext $entry]
   }

exit
