#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2021.023.14.30"

if {[flagLoc $ARGV -help]} \
   {
    puts "Plot a 1D or 2D Spectral Series (Version $thisVersion):"
    puts " -in        inList  \[test.ft2 ...\] List of Input Spectra."
    puts " -out       outName \[roi.ps\]       Output PostScript Plot File."
    puts " -tmp       tmpName \[tmp.ps\]       Temporary PostScript Output File."
    puts " -hi        hi      \[Auto ...\]     First Contour Heights (2D Data)."
    puts " -fhi       fhi     \[Auto ...\]     Min Heights (1D Data or 2D Data -mult 1.0)."
    puts " -lhi       lhi     \[Auto ...\]     Max Heights (1D Data or 2D Data -mult 1.0)."
    puts " -colors    cList   \[multi ...\]    Colors for Spectral Drawing."
    puts " -posColors cList   \[multi ...\]    Colors for Positive Contours (2D Data)."
    puts " -negColors cList   \[multi ...\]    Colors for Negative Contours (2D Data)."
    puts " -bgScript  bgName  \[None\]         NMRWish Script for Background Drawing."
    puts " -fgScript  fgName  \[None\]         NMRWish Script for Foreground Drawing."
    puts " -rev                              Plot Spectra Last to First Listed." 
    puts " -norev                            Plot Spectra First to Last Listed (Default)." 
    puts " -pdf                              Convert PostScript to PDF (Requires ps2pdf)."
    puts " -nopdf                            No PDF Conversion."
    puts " -key                              Draw Key (Default)."
    puts " -nokey                            Do Not Draw Key."
    puts " -condense                         Condense Final PostScript Output (Default)."
    puts " -nocondense                       Do Not Condense Final PostScript Output."
    puts " -clean                            Delete Temporary and Log Files (Default)."
    puts " -noclean                          Do Not Delete Temporary and Log Files."
    puts " -noclose                          Do Not Close Plot (plotClose)."
    puts " -verb                             Verbose Mode ON (Default)."
    puts " -noverb                           Verbose Mode OFF."
    puts "Spectral Region Coordinates, Units ppm % pts (-x.. -y..):"
    puts " -x1  x1           \[0%\]            X-Axis Spectral Region Start."
    puts " -xn  xn           \[100%\]          X-Axis Spectral Region End."
    puts "Plot Position on Page:"
    puts " -plotUnits uStr   \[in\]            Units for Plot Coords."
    puts " -cx1       plotX1 \[0.25\]          X-Axis Plot Region Start."
    puts " -cxn       plotXN \[9.25\]          X-Axis Plot Region End."
    puts " -cy1       plotY1 \[0.25\]          Y-Axis Plot Region Start."
    puts " -cyn       plotYN \[7.25\]          Y-Axis Plot Region End."
    puts "Plot Key:"
    puts " -keyExtra  kText  \[None\]          Additional Key Color/Text Pairs."
    puts " -kx1       keyX1  \[0.4\]           X-Axis Plot Key Start."
    puts " -ky1       keyY1  \[0.4\]           Y-Axis Plot Key Start."
    puts " -kw        keyW   \[Auto\]          Width of Key in Plot Units."
    puts " -charX     charX  \[0.12\]          Character Width for Key Text."
    puts " -charY     charY  \[0.26\]          Character Height for Key Text."
    puts " -landscape                        Use Landscape Orientation (Default)."
    puts " -portrait                         Use Portrait Orientation."
    puts " -outside                          Place Key Outside of Plot."
    puts " -inside                           Place Key Inside of Plot (Default)."
    puts "Plot Colors (Color Name, Hex Code, or Keyword None):"
    puts " -pageColor pColor    \[gray90\]     Page Color."
    puts " -bgColor   bgColor   \[white\]      Spectral Plot Background Color."
    puts " -keyColor  keyColor  \[white\]      Key Background Color."
    puts " -bordColor bordColor \[black\]      Spectral Plot Border Color."
    puts " -axisColor axisColor \[black\]      Spectral Plot Axis Color."
    puts " -labColor  labColor  \[black\]      Peak Label Text Color."
    puts " -fillColor fillColor \[white\]      Peak Label Fill Color."
    puts " -lineColor lineColor \[gray90\]     Zero and Peak Label Line Color (1D Data)."
    puts "1D Drawing Parameters:"
    puts " -lw1d                             Line Drawing Width, 1D Spectra."
    puts " -zero                             Draw Zero Level (1D Data)."
    puts " -nozero                           No Not Draw Zero Level."
    puts "2D Drawing Parameters:"
    puts " -plev       nPos     \[12\]         Number of Positive Contours."
    puts " -nlev       nNeg     \[12\]         Number of Negative Contours."
    puts " -mult       cMult    \[1.3\]        Contour Level Multiplier."
    puts " -lwPlus     lwPlus   \[1.0\]        Line Drawing Width, Postive Contours."
    puts " -lwMinus    lwMinus  \[1.0\]        Line Drawing Width, Negative Contours."
    puts " -cont                             Draw Contour Plots for 2D Data (Default)."
    puts " -pix                              Draw Pixmap Images for 2D Data."
    puts "Plot Titles (Also: -title1 ... -title1X ...):"
    puts " -title      title    \[None\]       Plot Title."
    puts " -titleX     xLoc     \[4.0\]        X-Axis Location of Title, in Plot Units."
    puts " -titleY     yLoc     \[0.5\]        Y-Axis Location of Title, in Plot Units."
    puts " -titleColor tColor   \[black\]      Title Color."
    puts " -titleSmall                       Use Small Title Text (Default)."
    puts " -titleBig                         Use Large Title Text."
    puts "Optional Peak Labels:"
    puts " -tab        tabName  \[None\]       Peak Table of PPM Coords."
    puts " -xVar       xVar     \[X_PPM\]      X-Axis PPM Variable in Table."
    puts " -yVar       yVar     \[Y_PPM\]      Y-Axis PPM Variable in Table (2D Data)."
    puts " -lab        lVar     \[INDEX\]      Label Variable in Table."
    puts " -box                              Draw Boxes Around Peak Labels (Default)."
    puts " -nobox                            No Boxes Around Peak Labels."
    puts "PCA Information:"
    puts " -pca       pcaTab \[None\]          PCA Table (Such as pca001/pcaCoords.tab)."
    puts " -pcaVarX   varX   \[PCAZ1\]         Variable with X-Axis Score Values."
    puts " -pcaVarY   varY   \[PCAZ2\]         Variable with Y-Axis Score Values."
    puts "Additional Arguments for nmrWish Plotting Commands: (Terminate with --):"
    puts " -argsPlot  args   \[None\]          Arguments for Initialization (plotOpen)."
    puts " -args1D    args   \[None\]          Arguments for 1D Plotting (drawROI)."  
    puts " -args2D    args   \[None\]          Arguments for 2D Plotting (drawROI)."  
    puts "Axis Drawing:"
    puts " -axis                             Draw Axis (Default)."
    puts " -noaxis                           Do Not Draw Axis."
    puts " -tAxis     tArgs  \[None\]          Parameters for Top Axis."
    puts " -bAxis     bArgs  \[Bottom\]        Parameters for Bottom Axis."
    puts " -rAxis     rArgs  \[Right\]         Parameters for Right Axis."
    puts " -lAxis     lArgs  \[None\]          Parameters for Left Axis."
    puts " -c1Width   c1Wide \[0.12in\]        Axis Text Character Width."
    puts " -c1Height  c1Hi   \[0.18in\]        Axis Text Character Height."
    puts " -c2Width   c2Wide \[0.16in\]        Axis Label Character Width."
    puts " -c2Height  c2Hi   \[0.24in\]        Axis Label Character Height."
    puts " -lwtic     lw     \[1.0\]           Axis Tic Linewidth, Printer's Points."
    puts "Axis Parameters (For -bAxis -rAxis ...):"
    puts "  units=       Axis units keyword (PPM, etc)."
    puts "  orig=        Axis origin value."
    puts "  end=         Axis endpoint value."
    puts "  step=        Axis step value."
    puts "  tic=         Axis tic count."
    puts "  fmt=         Axis print format (%f etc)."
    puts "  bold=        Axis text is drawn in bold font."
    puts "  lab=         Axis label."
    puts "  labhoriz=    Axis label is drawn horizontally (For: -lAxis -rAxis)."
    puts "Axis Parameter Example:"
    puts "  plotSeries.tcl -rAxis Right,labhoriz=1,step=2.0"
    exit 1
   }

set verbose    0

set inList     test.ft2
set colorList1 multi 
set colorList2 multi 
set hiList1    Auto
set hiList2    Auto
set bgScript   None
set fgScript   None
set keyExtra   None

set pcaName    None
set pcaVarX    PCAZ1
set pcaVarY    PCAZ2
set pcaDX      0.75
set pcaDY      0.75

set glbHi1     None
set glbHi2     None

set args1D     None
set args2D     None 
set argsPlot   None

set axisFlag   1
set boxFlag    1
set keyFlag    1
set condFlag   1
set cleanFlag  1
set insideFlag 1
set revFlag    0
set zeroFlag   1
set glbHiFlag  0
set closeFlag  1

set topAxisInfo   None
set botAxisInfo   Bottom
set leftAxisInfo  None
set rightAxisInfo Right

set pdfFlag       [cmndExists ps2pdf] 

set ARGV [getListArgDT $ARGV -argsPlot argsPlot]
set ARGV [getListArgDT $ARGV -args1D   args1D]
set ARGV [getListArgDT $ARGV -args2D   args2D]
set ARGC [llength $ARGV]

getListArgD $ARGV -keyExtra keyExtra

if {![strcasecmp $keyExtra None]} {set keyExtra ""}

if {[flagLoc $ARGV -inside]}     {set insideFlag 1}
if {[flagLoc $ARGV -outside]}    {set insideFlag 0}

if {[flagLoc $ARGV -noinside]}   {set insideFlag 0}
if {[flagLoc $ARGV -nooutside]}  {set insideFlag 1}

if {[flagLoc $ARGV -close]}      {set closeFlag 1}
if {[flagLoc $ARGV -noclose]}    {set closeFlag 0}

set outName  roi.ps
set tmpName  tmp.ps
set x1       0%
set xn       100% 
set y1       0% 
set yn       100%

set charX    0.12
set charY    0.26

set cx1      0.2
set cxn      9.0
set cy1      0.2
set cyn      7.0

getArgD $ARGV -cx1 cx1
getArgD $ARGV -cxn cxn
getArgD $ARGV -cy1 cy1
getArgD $ARGV -cyn cyn

if {$insideFlag} \
   {
    set kx1  0.40
    set ky1  0.40
   } \
else \
   {
    set kx1  $cx1
    set ky1  $cy1 
   }

set keyWidth Auto
set contMode cont
set plev     12
set nlev     12 
set mult     1.3

set tabName  None
set labVar   INDEX
set xVar     X_PPM 
set yVar     Y_PPM 
set pkOrient h

set pHue2   0.5
set pSat2   1.0
set pVal2   0.8

set pHue1   0.7
set pSat1   1.0
set pVal1   1.0

set nHue2   0.0
set nSat2   1.0
set nVal2   0.8

set nHue1   0.2
set nSat1   1.0
set nVal1   1.0

set c1Width  0.20in
set c1Height 0.25in
set c2Width  0.24in
set c2Height 0.30in

set lwPlus   1.0
set lwMinus  1.0

set lwTic    1.0
set lw1d     1.0

set bgColor     white 
set keyColor    white 
set bordColor   black
set axisColor   black
set labColor    black
set fillColor   white
set lineColor   gray90
set pageColor   gray90

set plotOrient landscape
set plotUnits  in

set fdata      0
set thisFDATA  0
set pkTab      0

if {[flagLoc $ARGV -landscape]}  {set plotOrient landscape}
if {[flagLoc $ARGV -portrait]}   {set plotOrient portrait}

if {[flagLoc $ARGV -pkVert]}     {set pkOrient v}
if {[flagLoc $ARGV -pkHoriz]}    {set pkOrient h}

if {[flagLoc $ARGV -rev]}        {set revFlag 1}
if {[flagLoc $ARGV -norev]}      {set revFlag 0}

if {[flagLoc $ARGV -axis]}       {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]}     {set axisFlag 0}

if {[flagLoc $ARGV -key]}        {set keyFlag 1}
if {[flagLoc $ARGV -nokey]}      {set keyFlag 0}

if {[flagLoc $ARGV -zero]}       {set zeroFlag 1}
if {[flagLoc $ARGV -nozero]}     {set zeroFlag 0}

if {[flagLoc $ARGV -box]}        {set boxFlag 1}
if {[flagLoc $ARGV -nobox]}      {set boxFlag 0}

if {[flagLoc $ARGV -cont]}       {set contMode cont}
if {[flagLoc $ARGV -pix]}        {set contMode pix}

if {[flagLoc $ARGV -glbHi]}      {set glbHiFlag 1}
if {[flagLoc $ARGV -noglbHi]}    {set glbHiFlag 0}

if {[flagLoc $ARGV -pdf]}        {set pdfFlag 1}
if {[flagLoc $ARGV -nopdf]}      {set pdfFlag 0}

if {[flagLoc $ARGV -condense]}   {set condFlag 1}
if {[flagLoc $ARGV -nocondense]} {set condFlag 0}

if {[flagLoc $ARGV -clean]}      {set cleanFlag 1}
if {[flagLoc $ARGV -noclean]}    {set cleanFlag 0}

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

if {!$keyFlag} {set keyWidth 0.0} 

if {!$axisFlag} \
   {
    set botAxisInfo   None
    set topAxisInfo   None
    set leftAxisInfo  None
    set rightAxisInfo None
   }

getListArgD $ARGV -in        inList
getListArgD $ARGV -colors    colorList1
getListArgD $ARGV -posColors colorList1
getListArgD $ARGV -hi        hiList1
getListArgD $ARGV -fhi       hiList1

set colorList2 $colorList1
set hiList2    $hiList1

getListArgD $ARGV -negColors colorList2
getListArgD $ARGV -lhi       hiList2

set hi  [lindex $hiList1 0]
set fhi [lindex $hiList1 0]
set lhi [lindex $hiList2 0]

getArgD $ARGV -out    outName
getArgD $ARGV -x1     x1
getArgD $ARGV -xn     xn
getArgD $ARGV -y1     y1
getArgD $ARGV -yn     yn

getArgD $ARGV -bgScript  bgScript
getArgD $ARGV -fgScript  fgScript
getArgD $ARGV -plotUnits in
getArgD $ARGV -charX     charX
getArgD $ARGV -charY     charY

getArgD $ARGV -lwPlus  lwPlus
getArgD $ARGV -lwMinus lwMinus
getArgD $ARGV -lw1d    lw1d

getArgD $ARGV -kx1    kx1
getArgD $ARGV -kxn    kxn
getArgD $ARGV -ky1    ky1
getArgD $ARGV -kw     keyWidth

getArgD $ARGV -plev   plev
getArgD $ARGV -nlev   nlev
getArgD $ARGV -mult   mult

getArgD $ARGV -pca     pcaName
getArgD $ARGV -pcaVarX pcaVarX
getArgD $ARGV -pcaVarY pcaVarY
getArgD $ARGV -pcaDX   pcaDX
getArgD $ARGV -pcaDY   pcaDY

getArgD $ARGV -tab    tabName
getArgD $ARGV -lab    labVar
getArgD $ARGV -xVar   xVar 
getArgD $ARGV -yVar   yVar 

getArgD $ARGV -tAxis  topAxisInfo
getArgD $ARGV -bAxis  botAxisInfo
getArgD $ARGV -lAxis  leftAxisInfo
getArgD $ARGV -rAxis  rightAxisInfo

getArgD $ARGV -c1Width   c1Width
getArgD $ARGV -c1Height  c1Height
getArgD $ARGV -c2Width   c2Width
getArgD $ARGV -c2Height  c2Height

getArgD $ARGV -lwtic     lwTic

getArgD $ARGV -bgColor     bgColor
getArgD $ARGV -keyColor    keyColor
getArgD $ARGV -pageColor   pageColor
getArgD $ARGV -axisColor   axisColor
getArgD $ARGV -lineColor   lineColor
getArgD $ARGV -bordColor   bordColor
getArgD $ARGV -labColor    labColor
getArgD $ARGV -fillColor   fillColor

if {[flagLoc $ARGV -revColors]} \
   {
    rSwap pHue1 pHue2 
    rSwap pSat1 pSat2 
    rSwap pVal1 pVal2 

    rSwap nHue1 nHue2 
    rSwap nSat1 nSat2 
    rSwap nVal1 nVal2 
   }

getArgD $ARGV -pHue1 pHue1
getArgD $ARGV -pSat1 pSat1
getArgD $ARGV -pVal1 pVal1

getArgD $ARGV -pHue2 pHue2
getArgD $ARGV -pSat2 pSat2
getArgD $ARGV -pVal2 pVal2

getArgD $ARGV -nHue1 nHue1
getArgD $ARGV -nSat1 nSat1
getArgD $ARGV -nVal1 nVal1

getArgD $ARGV -nHue2 nHue2
getArgD $ARGV -nSat2 nSat2
getArgD $ARGV -nVal2 nVal2

#
#-------------------------------------------------------------

proc evalCmnd { cmnd } \
{
   global verbose

   if {$verbose} {puts "  $cmnd"}
   set error [catch "eval $cmnd"]

   return $error
}

proc updatePlotHiVals { fhiPtr lhiPtr thisDimCount } \
{
   global mult glbHi1 glbHi2 minVal maxVal maxAbsVal glbHiFlag

   upvar $fhiPtr fhi
   upvar $lhiPtr lhi

   if {$glbHiFlag} \
      {
       if {[strcasecmp $glbHi1 None]} {set fhi $glbHi1}
       if {[strcasecmp $glbHi2 None]} {set lhi $glbHi2}
      }

   if {$thisDimCount == 1} \
      {
       if {![strcasecmp $fhi Auto]} {set fhi [expr $minVal - 0.05*$maxAbsVal]}
       if {![strcasecmp $lhi Auto]} {set lhi [expr $maxVal + 0.05*$maxAbsVal]}
      } \
   elseif {$mult == 1.0} \
      {
       if {![strcasecmp $fhi Auto]} {set fhi $minVal}
       if {![strcasecmp $lhi Auto]} {set lhi $maxVal}
      } \
   else \
      {
       if {![strcasecmp $fhi Auto]} {set fhi [expr 6.0*[getROIStat 1 vEstNoise]]}
       if {![strcasecmp $lhi Auto]} {set lhi $maxVal}
      }

   if {[string first % $fhi] >= 0} {set fhi [expr 0.01*[getHiVal $fhi]*$maxAbsVal]}
   if {[string first % $lhi] >= 0} {set lhi [expr 0.01*[getHiVal $lhi]*$maxAbsVal]}

   if {$glbHiFlag && [strcasecmp $glbHi1 None]} {set glbHi1 $fhi}
   if {$glbHiFlag && [strcasecmp $glbHi2 None]} {set glbHi2 $lhi}
}

set n [llength $inList]

set thisColor1 [lindex $colorList1 end]
set thisColor2 [lindex $colorList2 end]
set thisHi1    [lindex $hiList1    end]
set thisHi2    [lindex $hiList2    end]

for {set i 0} {$i < $n} {incr i} \
   {
    lappend colorList1 $thisColor1
    lappend colorList2 $thisColor2
    lappend hiList1    $thisHi1
    lappend hiList2    $thisHi2
   }

set thisName [lindex $inList 0]

if {![file exists $thisName]} \
   {
    puts stderr "Error finding input spectrum $thisName"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {$error} \
   {
    puts stderr "Error Reading Header in $thisName"
    exit 1
   }

set tpFlag   [getParmI $fdata FDTRANSPOSED NULL_DIM]
set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]

if {$dimCount < 2} {set tpFlag 0}

if {$tpFlag} \
   {
    set xName Y_AXIS
    set yName X_AXIS

    if {![strcasecmp $xVar Auto]} {set xVar Y_PPM}
    if {![strcasecmp $yVar Auto]} {set yVar X_PPM}
   } \
else \
   {
    set xName X_AXIS
    set yName Y_AXIS

    if {![strcasecmp $xVar Auto]} {set xVar X_PPM}
    if {![strcasecmp $yVar Auto]} {set yVar Y_PPM}
   }

set inCount   [llength $inList]
set titleSize 0

set dimCountList ""
set titleList    ""
set minList      ""
set maxList      ""

foreach thisName $inList \
   {
    if {![file exists $thisName]} \
       {
        puts stderr "Error finding input spectrum $thisName"
        exit 1
       }

    set error [rdFDATA $thisName thisFDATA]
    
    if {$error} \
       {
        puts stderr "Error Reading Header in $thisName"
        exit 1
       }

    set thisDimCount [getParmI $thisFDATA FDDIMCOUNT NULL_DIM]

    lappend dimCountList $thisDimCount

    set thisTitle [getParmStr $thisFDATA FDTITLE NULL_DIM]
    if {![llength $thisTitle]} {set thisTitle $thisName}

    set nS [string length $thisTitle]
    if {$nS > $titleSize} {set titleSize $nS}

    lappend titleList $thisTitle

    if {$thisDimCount == 1} \
       {
        evalCmnd "readROI -roi 1 -ndim 1 -in $thisName -x $xName $x1 $xn"
       } \
    else \
       {
        evalCmnd "readROI -roi 1 -ndim 2 -in $thisName -x $xName $x1 $xn -y $yName $y1 $yn"
       }

    set rdata [lindex [getROIInfo -roi 1] 0]
    set n     [getROISize 1]

    lappend minList [vMin $rdata $n]
    lappend maxList [vMax $rdata $n]

    catch "sysFree $thisFDATA"
    set thisFDATA 0
   }

set minDimCount [getMin $dimCountList]
set maxDimCount [getMax $dimCountList]

set minVal      [getMin $minList]
set maxVal      [getMax $maxList]

set maxAbsVal   [getMaxAbs "$minVal $maxVal"]

if {![strcasecmp $argsPlot None]} {set argsPlot ""}

set argsChars "-c1Width $c1Width -c1Height $c1Height -c2Width $c2Width -c2Height $c2Height"

if {$condFlag} \
   {
    evalCmnd "plotOpen -$plotOrient -color -plotName $tmpName $argsPlot $argsChars"
   } \
else \
   {
    evalCmnd "plotOpen -$plotOrient -color -plotName $outName $argsPlot $argsChars"
   }

set boxAdj 3.2

if {$titleSize > 20} {set boxAdj 2.2}
if {$titleSize > 25} {set boxAdj 1.2}
if {$titleSize > 30} {set boxAdj 0.0}

if {![strcasecmp $keyWidth Auto]} \
   {
    set keyWidth [expr ($boxAdj + $titleSize)*$charX]
    if {$verbose} {puts [format "# Setting Key Width: %.3f in" $keyWidth]}
   }

set nExtra   [llength $keyExtra]
set keyCount $inCount

if {$nExtra} \
   {
    if {$nExtra % 2} \
       {
        puts stderr "Error: -keyExtra Option Requires an Even Number Color and Title Values."
       }

    incr keyCount [expr $nExtra/2]
   }

set kxn [expr $kx1 + $keyWidth]

if {$insideFlag} \
   {
    set kyn   [expr $ky1 + (0.5 + $keyCount)*$charY]
    set kyMax [expr $kyn + (1.6 + $keyCount)*$charY]

    if {$kyn > $cyn - $charY} \
       {
        set kyn   [expr $cyn - $charY]
        set kyMax $kyn
       }
   } \
else \
   {
    set cx1   [expr $cx1 + $keyWidth + 1.2*$charX]
    set kyn   $cyn
    set kyMax $kyn
   }

evalCmnd "defineCell -cell 1 -x $cx1$plotUnits $cxn$plotUnits -y $cy1$plotUnits $cyn$plotUnits"

if {[strcasecmp $pageColor None]} {drawRec -x 0% 100% -y 0% 100% -fg $pageColor}
if {[strcasecmp $bgColor   None]} {drawRec -x 0% 100% -y 0% 100% -fg $bgColor -cell 1}

if {[strcasecmp $bgScript None]} \
   {
    if {![file exists $bgScript]} \
       {
        puts stderr "Error Finding Script $bgScript for Drawing Background."
       } \
    else \
       {
        evalCmnd "source $bgScript"
       }
   } 

if {$revFlag} \
   {
    set iList [getIota [expr $inCount - 1] -1 $inCount]
   } \
else \
   {
    set iList [getIota 0 1 $inCount]
   }

set nAll 0
set n1D  0

if {![strcasecmp $topAxisInfo   None]} {set topAxisInfo   ""}
if {![strcasecmp $botAxisInfo   None]} {set botAxisInfo   ""}
if {![strcasecmp $leftAxisInfo  None]} {set leftAxisInfo  ""}
if {![strcasecmp $rightAxisInfo None]} {set rightAxisInfo ""}

if {![llength $topAxisInfo] && ![llength $botAxisInfo] && ![llength $leftAxisInfo] && ![llength $rightAxisInfo]} {set axisFlag 0}

foreach i $iList \
   {
    set  thisName [lindex $inList $i]
    incr nAll 

    if {![file exists $thisName]} \
       {
        puts stderr "Error Finding Input Spectrum $thisName"
        exit 1
       }

    set thisDimCount [lindex $dimCountList $i]

    if {$verbose} {puts "# Spectrum [expr $i + 1]: $thisTitle"} 

    if {$thisDimCount == 1} \
       {
        incr n1D

        set fhi       [lindex $hiList1    $i]
        set lhi       [lindex $hiList2    $i]
        set thisColor [lindex $colorList1 $i]

        if {![strcasecmp $thisColor multi]} {set thisColor blue}

        evalCmnd "readROI -roi 1 -ndim 1 -in $thisName -x $xName $x1 $xn"
       
        updatePlotHiVals fhi lhi $thisDimCount

        set cmnd "drawROI -roi 1 -cell 1 -nobox"

        if {[strcasecmp $args1D None]} {append cmnd " $args1D"}

        if {!$i && $axisFlag} \
           {
            append cmnd " -axis" 

            if {[llength $botAxisInfo]}   {append cmnd " $botAxisInfo"}
            if {[llength $topAxisInfo]}   {append cmnd " $topAxisInfo"}
            if {[llength $leftAxisInfo]}  {append cmnd " $leftAxisInfo"}
            if {[llength $rightAxisInfo]} {append cmnd " $rightAxisInfo"}
           }

        append cmnd " -lwtic $lwTic -lw1d $lw1d -fhi $fhi -lhi $lhi -fg $thisColor -colorA $axisColor -colorS None" 

        if {$zeroFlag && $n1D == 1} {append cmnd " -zero -colorG $lineColor"}

        evalCmnd $cmnd
       } \
    else \
       {
        set fhi        [lindex $hiList1    $i]
        set lhi        [lindex $hiList2    $i]
        set thisColor1 [lindex $colorList1 $i]
        set thisColor2 [lindex $colorList2 $i]

        evalCmnd "readROI -roi 1 -ndim 2 -in $thisName -x $xName $x1 $xn -y $yName $y1 $yn"

        updatePlotHiVals fhi lhi $thisDimCount

        set cmnd "drawROI -roi 1 -cell 1 -nobox -lwPlus $lwPlus -lwMinus $lwMinus" 

        if {[strcasecmp $args2D None]} {append cmnd " $args2D"}

        if {!$i && $axisFlag} \
           {
            append cmnd " -axis"

            if {[llength $botAxisInfo]}   {append cmnd " $botAxisInfo"}
            if {[llength $topAxisInfo]}   {append cmnd " $topAxisInfo"}
            if {[llength $leftAxisInfo]}  {append cmnd " $leftAxisInfo"}
            if {[llength $rightAxisInfo]} {append cmnd " $rightAxisInfo"}
           }

        if {$mult == 1.0} \
           {
            append cmnd " -color1 $thisColor1 -color2 $thisColor2 -mult $mult -plev $plev -nlev $nlev -fhi $fhi -lhi $lhi"
           } \
        else \
           {
            append cmnd " -color1 $thisColor1 -color2 $thisColor2 -mult $mult -plev $plev -nlev $nlev -hi $fhi" 
           }

        if {![strcasecmp $thisColor1 multi]} \
           {
            append cmnd " -pcc $plev -pHue1 $pHue1 -pSat1 $pSat1 -pVal1 $pVal1 -pHue2 $pHue2 -pSat2 $pSat2 -pVal2 $pVal2"
           }

        if {![strcasecmp $thisColor2 multi]} \
           {
            append cmnd " -ncc $nlev -nHue1 $nHue1 -nSat1 $nSat1 -nVal1 $nVal1 -nHue2 $nHue2 -nSat2 $nSat2 -nVal2 $nVal2"
           }

        if {![strcasecmp $contMode pix]} {append cmnd " -pix -bg $bgColor"}

        append cmnd " -lwtic $lwTic -colorA $axisColor -colorS None"

        evalCmnd $cmnd
       }

    if {[strcasecmp $bordColor None] && $nAll == $inCount} \
       {
        if {$verbose} {puts ""}
        evalCmnd "drawBox -cell 1 -x 0% 100% -y 0% 100% -fg $bordColor"
       }

    if {$verbose} {puts ""}
   }

#
# Peak labels:

set pkList ""

if {[strcasecmp $tabName None]} \
   {
    if {![file exists $tabName]} \
       {
        puts stderr "Error Finding Peak Table $tabName"
        set tabName None
       }
   }

if {[strcasecmp $tabName None]} \
   {
    set pkTab    [gdbRead table -in $tabName]
    set thisName [lindex $inList 0]

    set error [rdFDATA $thisName thisFDATA]

    if {$error} \
       {
        puts stderr "Error Reading Header in $thisName"
        exit 1
       }

    set thisDimCount [getParmI $thisFDATA FDDIMCOUNT NULL_DIM]

    catch "sysFree $thisFDATA"
    set thisFDATA 0

    if {$thisDimCount == 1} \
       {
        readROI -roi 1 -ndim 1 -in $thisName -x $xName $x1 $xn 

        set thisFDATA [lindex [getROIInfo -roi 1] 1]
        set xSize     [lindex [lindex [getROIInfo -roi 1] 2] 0]

        set sx1 [pnt2spec $thisFDATA $xName 1      ppm]
        set sxn [pnt2spec $thisFDATA $xName $xSize ppm]
        set fhi [lindex $hiList1 0]
        set lhi [lindex $hiList2 0]

        set pkList [gdbSelect -from $pkTab -cond "$xVar <= $sx1 && $xVar >= $sxn"]

        if {$verbose} {puts "# Peak List: [llength $pkList] Peaks."}

        foreach peak $pkList \
           {
            set xPPM   [gdbGet $peak $xVar]ppm
            set thisHi [gdbGet $peak HEIGHT]
            set id     [gdbGet $peak $labVar]
           }
       } \
    else \
       {
        readROI -roi 1 -ndim 2 -in $thisName -x $xName $x1 $xn -y $yName $y1 $yn
    
        set thisFDATA [lindex [getROIInfo -roi 1] 1]
        set xSize     [lindex [lindex [getROIInfo -roi 1] 2] 0]
        set ySize     [lindex [lindex [getROIInfo -roi 1] 2] 1]
    
        set sx1 [pnt2spec $thisFDATA $xName 1      ppm]
        set sxn [pnt2spec $thisFDATA $xName $xSize ppm]
        set sy1 [pnt2spec $thisFDATA $yName 1      ppm]
        set syn [pnt2spec $thisFDATA $yName $ySize ppm]

        set pkList [gdbSelect -from $pkTab -cond "$xVar <= $sx1 && $xVar >= $sxn && $yVar <= $sy1 && $yVar >= $syn"]

        if {$verbose} {puts "# Peak List: [llength $pkList] Peaks."}

        foreach peak $pkList \
           {
            set xPPM [gdbGet $peak $xVar]ppm
            set yPPM [gdbGet $peak $yVar]ppm
            set id   [gdbGet $peak $labVar]

            set cmnd "drawText -cell 1 -roiRef 1 -x $xPPM -y $yPPM -fg $labColor"

            if {$boxFlag} {append cmnd " -box -bg $fillColor"}

            append cmnd " -text $id"

            evalCmnd $cmnd
           }

        if {$verbose} {puts ""}
       }
   }

if {[strcasecmp $pcaName None]} \
   {
    set pcaTab   [gdbRead table -in $pcaName]
    set thisName [lindex $inList 0]

    set error [rdFDATA $thisName thisFDATA]

    if {$error} \
       {
        puts stderr "Error Reading Header in $thisName"
        exit 1
       }

    set thisDimCount [getParmI $thisFDATA FDDIMCOUNT NULL_DIM]

    catch "sysFree $thisFDATA"
    set thisFDATA 0

    if {$thisDimCount == 1} \
       {
        readROI -roi 1 -ndim 1 -in $thisName -x $xName $x1 $xn 

        set thisFDATA [lindex [getROIInfo -roi 1] 1]
        set xSize     [lindex [lindex [getROIInfo -roi 1] 2] 0]

        set sxn [pnt2spec $thisFDATA $xName 1      user]
        set sx1 [pnt2spec $thisFDATA $xName $xSize user]
        set fhi [lindex $hiList1 0]
        set lhi [lindex $hiList2 0]

        set objList [lreverse [gdbSelect -from $pcaTab -cond "$pcaVarX <= $sx1 && $pcaVarX >= $sxn"]]

        if {$verbose} {puts "# PCA Table: [llength $objList] Objects."}

        foreach obj $objList \
           {
            set xPos  [gdbGet $obj $pcaVarX]user
            set id    [gdbGet $obj TITLE]
           }
       } \
    else \
       {
        readROI -roi 1 -ndim 2 -in $thisName -x $xName $x1 $xn -y $yName $y1 $yn
    
        set thisFDATA [lindex [getROIInfo -roi 1] 1]
        set xSize     [lindex [lindex [getROIInfo -roi 1] 2] 0]
        set ySize     [lindex [lindex [getROIInfo -roi 1] 2] 1]
    
        set sx1 [pnt2spec $thisFDATA $xName 1      user]
        set sxn [pnt2spec $thisFDATA $xName $xSize user]
        set sy1 [pnt2spec $thisFDATA $yName 1      user]
        set syn [pnt2spec $thisFDATA $yName $ySize user]

        set objList [lreverse [gdbSelect -from $pcaTab -cond "$pcaVarX >= $sx1 && $pcaVarX <= $sxn && $pcaVarY >= $sy1 && $pcaVarY <= $syn"]]

        if {$verbose} {puts "# PCA Table: [llength $objList] Objects."}

        foreach obj $objList \
           {
            set xPos      [gdbGet $obj $pcaVarX]
            set yPos      [gdbGet $obj $pcaVarY]
            set thisColor [gdbGet $obj COLOR]
            set id        [gdbGet $obj TITLE]

            set xPct [expr 100.0*($xPos - $sx1)/($sxn - $sx1)]
            set yPct [expr 100.0*($syn - $yPos)/($syn - $sy1)]

            drawRec -fg $thisColor -cell 1 -roiRef 1 -x [expr $xPct-$pcaDX]% [expr $xPct+$pcaDX]% -y [expr $yPct-$pcaDY]% [expr $yPct+$pcaDX]%
            drawBox -fg $labColor  -cell 1 -roiRef 1 -x [expr $xPct-$pcaDX]% [expr $xPct+$pcaDX]% -y [expr $yPct-$pcaDY]% [expr $yPct+$pcaDX]%
           }

        if {$verbose} {puts ""}
       }
   }

if {$keyFlag} \
   {
    if {[strcasecmp $keyColor  None]} {evalCmnd "drawRec -fg $keyColor  -x $kx1$plotUnits $kxn$plotUnits -y $ky1$plotUnits $kyn$plotUnits"}
    if {[strcasecmp $bordColor None]} {evalCmnd "drawBox -fg $bordColor -x $kx1$plotUnits $kxn$plotUnits -y $ky1$plotUnits $kyn$plotUnits"}

    set tx1 [expr $kx1 + $charX]
    set ty1 [expr $ky1 + $charY]

    set overflowFlag 0

    set ii 0

    for {set i 0} {$i < $keyCount} {incr i} \
       {
        set rx1 [expr $tx1 + 0.10*$charX]
        set rxn [expr $tx1 + 1.50*$charX]
        set ry1 [expr $ty1 - 0.75*$charY]
        set ryn [expr $ty1 + 0.15*$charY]

        if {$i < $inCount} \
           {
            set thisColor [lindex $colorList1 $i]
            set t         [lindex $titleList  $i]
           } \
        else \
           {
            set  thisColor [lindex $keyExtra $ii]
            incr ii

            set  t         [lindex $keyExtra $ii]
            incr ii
           }

        if {![strcasecmp $thisColor multi]} {set thisColor blue}

        if {$ryn > $kyMax - 2.0*$charY} \
           {
            if {$overflowFlag} \
               {
                if {$i < $keyCount - 1} continue
               } \
            else \
               {
                if {$i != $keyCount - 1} \
                   {
                    set overflowFlag 1
                    set thisColor    None
                    set t            "   ...   "
                   }
               }
           }

        if {[strcasecmp $thisColor None]} {evalCmnd "drawRec -fg $thisColor -x $rx1$plotUnits $rxn$plotUnits -y $ry1$plotUnits $ryn$plotUnits"}

        evalCmnd "drawText -x [expr $tx1 + 1.9*$charX]$plotUnits -y $ty1$plotUnits -fg $labColor -text $t" 

        set ty1 [expr $ty1 + $charY]
       }
   }

for {set i 0} {$i <= 999} {incr i} \
   {
    if {$i} \
       {
        set flagText "-title$i"
       } \
    else \
       {
        set flagText "-title"
       }

    if {$i > 0 && ![flagLoc $ARGV ${flagText}]} break

    set titleText    None
    set titleX       4.0
    set titleY       0.5
    set titleColor   black 
    set titleBigFlag 0

    getListArgD $ARGV ${flagText}   titleText

    if {![strcasecmp $titleText None]} continue

    getArgD $ARGV  ${flagText}X     titleX
    getArgD $ARGV  ${flagText}Y     titleY
    getArgD $ARGV  ${flagText}Color titleColor

    if {[flagLoc $ARGV ${flagText}Big]}   {set titleBigFlag 1}
    if {[flagLoc $ARGV ${flagText}Small]} {set titleBigFlag 0}

    if {$titleBigFlag} \
       {
        drawText -big   -fg $titleColor -x ${titleX}$plotUnits -y ${titleY}$plotUnits -text [string trim $titleText "{}"]
       } \
    else \
       {
        drawText -small -fg $titleColor -x ${titleX}$plotUnits -y ${titleY}$plotUnits -text [string trim $titleText "{}"]
       }
   }

if {[strcasecmp $fgScript None]} \
   {
    if {![file exists $fgScript]} \
       {
        puts stderr "Error Finding Script $fgScript for Drawing Foreground."
       } \
    else \
       {
        evalCmnd "source $fgScript"
       }
   } 

if {$closeFlag} {evalCmnd "plotClose"}

if {$verbose} {puts ""}

if {$condFlag} \
   {
    set rootName [file root $outName]
    set logName  ${rootName}_condense.log

    evalCmnd "file delete -force $outName $logName"

    set cmnd "exec condensePlot.com $tmpName $outName $logName"

    set error [evalCmnd $cmnd]

    if {$error || ![file exists $outName]} \
       {
        puts stderr "Error Creating Condensed PostScript $outName from $tmpName Logfile: $logName" 
       } \
    else \
       {
        if {$cleanFlag} {evalCmnd "file delete -force $tmpName $logName"}
        if {$verbose}   {puts "# Created Condensed PostScript $outName from $tmpName"}
       }

    if {$verbose} {puts ""}
   }

if {$pdfFlag} \
   {
    set rootName [file root $outName]
    set pdfName  ${rootName}.pdf
    set logName  ${rootName}_plot.log

    evalCmnd "file delete -force $pdfName"

    set cmnd "exec stdPDF.com $outName $pdfName $logName"

    set error [evalCmnd $cmnd]

    if {$error || ![file exists $pdfName]} \
       {
        puts stderr "Warning: Error Creating PDF Result $pdfName Logfile: $logName"
       } \
    else \
       {
        if {$cleanFlag} {evalCmnd "file delete -force $logName"}
        if {$verbose}   {puts "# Created PDF $pdfName"}
       }

    if {$verbose} {puts ""}
   }

exit
