#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc postItInit {} \
{
   global fgColor bgColor rows cols xOff yOff dx dy xStep yStep
   global pageCount pageID pageList fileList

   set pageCount 0
   set pageID    0
   set pageList  ""
   set fileList  ""

   set bgColor   yellow
   set fgColor   black

   set rows      16
   set cols      32

   set xOff      100
   set yOff      100

   set dx        8
   set dy        26

   set xStep     16 
   set yStep     8
}

proc postItDir {} \
{
   set error 0

   if {![file exists ~/.postIt]} \
      {
       set error [catch {file mkdir ~/.postIt}]

       if {$error} \
          {
           set msg   "Error creating directory ~/.postIt"
           tk_dialog .errDialog Error $msg error 0 Continue
          }
      } \
   elseif {![file isdirectory ~/.postIt]} \
      {
       set error 1
       set msg   "Error: file ~/.postIt is not a directory."
       tk_dialog .errDialog Error $msg error 0 Continue
      }

   return $error
}

proc postItArgs {} \
{
   global fgColor bgColor rows cols 
   global ARGC ARGV

   if {[flagLoc $ARGV -help]} \
      {
       puts " -bg   bgColor  [yellow]  Notepaper Color."
       puts " -fg   fgColor  [black]   Text Color."
       puts " -rows nRows    [16]      Note Height, Characters."
       puts " -cols nCols    [32]      Note Width,  Characters."
       exit
      }

   getArgD $ARGV -fg    fgColor
   getArgD $ARGV -bg    bgColor
   getArgD $ARGV -xSize xSize
   getArgD $ARGV -ySize ySize
}

proc postItFrames {} \
{
   global dx dy 

   button .bNew     -text New    -command newPostIt     -width 5
   button .bToggle  -text Hide   -command togglePostIt  -width 5
   button .bSave    -text Save   -command savePostIt    -width 5
   button .bDelete  -text Delete -command deletePostIt  -width 5
   button .bClear   -text Reset  -command clearPostIt   -width 5
   button .bQuit    -text Quit   -command quitPostIt    -width 5

   bind .bNew <ButtonRelease-2> pastePostIt
   pack .bNew .bToggle .bSave .bDelete .bClear .bQuit -in . -side left

   wm title    . "PostIt for [exec whoami]"
   wm geometry . +$dx+$dx
   update

   after 600000 savePostIt
}

proc pastePostIt {} \
{
   set w   [getNewPostIt]
   set str ""

   set str [selection get]
   $w.text insert end $str

   savePostIt
}

proc clearPostIt {} \
{
   global pageCount

   set txt "Delete all $pageCount postIts?"

   if {![tk_dialog .clearDialog Clear $txt question 1 Delete Cancel]} \
      {
       hidePostIts
       deleteAllPostIts
      } 
}

proc deletePostIt {} \
{
   global pageList titleList

   if {[winfo exists .tDel]} return

   savePostIt

   toplevel .tDel

   set j 0

   foreach i $pageList \
      {
       set title [lindex $titleList $j]

       button .tDel.b$j -text "[expr $j+1]. $title" \
          -command "deleteOnePostIt $j"
    
       pack .tDel.b$j -side top

       incr j
      }

   button .tDel.done -text Done -command deleteDone
   pack   .tDel.done -side top

   wm title .tDel 
}

proc deleteDone {} \
{
   destroy .tDel

   hidePostIts
   showPostIts    
}

proc deleteOnePostIt { j } \
{
   global fileList

   set txt "Delete postIt [expr $j+1]?"

   if {![tk_dialog .delDialog Delete $txt question 1 Delete Cancel]} \
      {
       set fName [lindex $fileList $j]
       catch [file delete $fName]

       .tDel.b$j configure -state disabled
      }
}

proc deleteAllPostIts {} \
{
   set fList [glob -nocomplain ~/.postIt/*]

   foreach fName $fList \
      {
       set error [catch {file delete $fName}]

       if {$error} {puts "Error deleting file $fName"}
      }
}

proc autoSavePostIt {} \
{
   set cmnd [.bToggle cget -text]

   if {"$cmnd" == "Hide"} \
      {
       savePostIt
      }

   after 600000 savePostIt
}

proc togglePostIt {} \
{
   set cmnd [.bToggle cget -text]

   if {"$cmnd" == "Hide"} \
      {
       hidePostIts
       .bToggle configure -text Show
      } \
   else \
      {
       showPostIts
       .bToggle configure -text Hide 
      }
}

proc hidePostIts {} \
{
   global pageList pageCount

   savePostIt

   foreach i $pageList \
      {
       destroy $i
      }

   set pageList   ""
   set pageCount  0 
}

proc showPostIts {} \
{
   global pageCount pageList fileList titleList

   hidePostIts

   set fList    [glob -nocomplain ~/.postIt/*]

   set titleList ""
   set fileList  ""

   foreach fName $fList \
      {
       set w   [getNewPostIt]
       set str ""

       set error [catch {set inUnit [open $fName r]}]

       if {$error} \
          {
           puts "Error opening file $fName to read PostIt text."
          } \
       else \
          {
           set str [read $inUnit]
           $w.text insert end $str
           close   $inUnit

           set     title [split [file tail $fName] _]
           lappend titleList $title
           lappend fileList  $fName

           wm title $w "$pageCount. $title"
          }
      }
}

proc savePostIt {} \
{
   global pageList

   foreach w $pageList \
      {
       set info [lrange [wm title $w] 1 end]
       set fName ~/.postIt/[join $info _]

       set error [catch {set outUnit [open $fName w]}]

       if {$error} \
          {
           puts "Error opening file $fName to write PostIt text."
          } \
       else \
          {
           scan [$w.text index end] %d lineCount

           for {set i 1} {$i < $lineCount} {incr i} \
              {
               $w.text mark set first $i.0
               set  str [$w.text get first "first lineend"]
               puts $outUnit $str
              }
          }
      }
}

proc quitPostIt {} \
{
   savePostIt
   exit
}

proc newPostIt {} \
{
   getNewPostIt
   savePostIt
}

proc getNewPostIt {} \
{
    global fgColor bgColor rows cols dx dy xStep yStep
    global pageCount pageID pageList

    incr pageCount
    incr pageID

    toplevel .t$pageID

    lappend pageList .t$pageID

    text .t$pageID.text \
       -relief raised -yscrollcommand ".t$pageID.scrollY set" \
       -fg $fgColor -bg $bgColor -width $cols -height $rows

    scrollbar .t$pageID.scrollY -command \
      ".t$pageID.text yview" -relief sunken

    pack .t$pageID.scrollY -side right -fill y
    pack  .t$pageID.text   -side left -expand 1 -fill both

    set stamp [exec date]
    set x0    $dx
    set y0    $dy
    set xn    [expr 2*$dx + 2*$dx*$pageCount + [winfo width .]]
    set yn    [expr $dy*$pageCount]

    wm title  .t$pageID "$pageCount. [lrange $stamp 0 3]"

    while {$x0 < $xn || $y0 < $yn} \
       {
        wm geometry .t$pageID +$x0+$y0
        update

        if {$x0 <= $xn} {incr x0 $xStep}
        if {$y0 <= $yn} {incr y0 $yStep}

        if {$x0 > $xn}  {set x0 $xn}
        if {$y0 > $yn}  {set y0 $yn}
       }

   return .t$pageID
}

#
# Main:

postItInit
postItArgs
postItFrames
postItDir
showPostIts
