#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {$ARGC == 1} \
   {
    puts stderr "Extract PPM position error estimates from peak tables:"
    puts stderr " ppmErr.tcl tabName1 \[tabName2 ...\]"
    exit 1
   }

foreach tabName $argv \
   {
    set t     [gdbRead table -in $tabName]
    set vList [gdbGetInfo $t -parms varNames]
    set eList [gdbGetInfo $t -parms entries]

    getKeyData $t pkInfo

    foreach dim {X Y Z A} \
       {
        if {[lsearch -exact $vList ${dim}_AXIS] < 0} continue

        if {![info exists pkInfo(${dim}_AXIS)]} \
           {
            puts stderr "Error extracting ${dim}_AXIS data from table $tabName"
            continue
           }

        set axisInfo [lindex $pkInfo(${dim}_AXIS) 0]
        set errList  "" 

        set axisName [lindex $axisInfo 0]
        set ix1      [lindex $axisInfo 1]
        set ixN      [lindex $axisInfo 2]


        scan [lindex $axisInfo 3] "%f%s" sx1 sUnits
        scan [lindex $axisInfo 4] "%f%s" sxN sUnits

        if {$ix1 == $ixN} \
           {
            set r 0.0
           } \
        else \
           {
            set r [expr ($sx1 - $sxN)/(1 + $ixN - $ix1)]
           }

        foreach entry $eList \
           {
            set ePts [gdbGet $entry D$dim]
            set ePPM [expr abs($ePts*$r)]

            lappend errList $ePPM
           }

        puts "$tabName $axisName PPM Error:"

        foreach stat {Min Max Avg Median StdDev} \
           {
            puts [format "  %8s %.5f"  $stat: [get$stat $errList]]
           }
       }

    unset pkInfo

    gdbFree $t
   }

exit
