#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2021.235.21.09"

if {[flagLoc $ARGV -help]} \
   {
    puts "Peak Table from Chemical Shift Prediction, Under Development (Version $thisVersion)."
    exit 0
   }

#
#-------------------------------------------------------------------------------------------------------------------------------------

proc extractScriptCmnd { inName cmndList } \
{
   set cmnd ""

   set error [catch {set inUnit [open $inName r]}]
   if {$error} {return $cmnd}

   while {[gets $inUnit lineOrig] >= 0} \
      {
       set line [string trim $lineOrig]
       set key  [lindex $line 0]

       if {[lsearch -exact $cmndList $key] < 0} continue

       set cmnd  [string trim $line "\\"]
       set lastC [string index $line end]

       while {![strcmp $lastC "\\"]} \
          {
           if {[gets $inUnit lineOrig] < 0} break 
          
           set line [string trim $lineOrig]

           append cmnd " " 
           append cmnd [string trim $line "\\"]

           set lastC   [string index $lineOrig end]
          }

       catch "close $inUnit"

       return $cmnd
      }

   catch "close $inUnit"

   return $cmnd
}

proc getValInRange { minVal maxVal } \
{
   if {$minVal == $maxVal} {return $minVal}

   set mid [expr 0.5*($maxVal + $minVal)]
   set val [expr $mid + 0.35*($maxVal - $minVal)*[rand -gauss]]

   if {$val < $minVal} {set val $minVal}
   if {$val > $maxVal} {set val $maxVal}

   return $val
}

set pseudo3DFlag   0

if {[flagLoc $ARGV -pseudo3D]}   {set pseudo3DFlag 1}
if {[flagLoc $ARGV -nopseudo3D]} {set pseudo3DFlag 0}

set fidScriptName  fid.com
set simScriptName  simTD.com
set ftScriptName   ft_sim.com

if {$pseudo3DFlag} \
   {
    set simFIDName     sim/test%03d.fid
    set simFTName      sim/test%03d.ft2 
    set fidName        fid/test%03d.fid 

    set zeroFIDDir     tmpZeroFID
    set zeroFTDir      tmpZeroFT
    set zeroFIDName    $zeroFIDDir/test%03d.fid
    set zeroFTName     $zeroFTDir/test%03d.ft2
   } \
else \
   {
    set simFIDName     sim.fid
    set simFTName      sim.ft2
    set fidName        test.fid

    set zeroFIDDir     None
    set zeroFTDir      None 
    set zeroFIDName    tmpZero.fid
    set zeroFTName     tmpZero.ft2
   }

set predTabName    pred/4xcl_hsqc.tab
set pkTabName      peak.tab

set xLWScale       0.5
set yLWScale       0.5
set hiScale        1.0
set useHiFlag      0
set nusZFFlag      0

set grpdly         0
set dfVal          0.0

set xP0            0.0
set yP0            0.0

set xP1            0.0
set yP1            0.0

set hiLimitMin     1.0 
set hiLimitMax     10.0 

set xJMin          7.0
set xJMax         15.0

set yJMin         -0.5
set yJMax          0.5
 
set xP0OffMin     -8.0
set xP0OffMax      8.0

set yP0OffMin     -8.0
set yP0OffMax      8.0

set p0MinH2O      -6.0
set p0MaxH2O      -3.0

set dxMinH2O      -2.0
set dxMaxH2O      -1.5

set lwH2O          2.0

set xwOffMin       0.0
set xwOffMax       0.0

set ywOffMin       0.0
set ywOffMax       0.0

set dxRand         0.0
set dyRand         0.0
set keepFrac       1.0

set filterDX       1.5
set filterDY       1.5
set filterHi       3.0

set noisePct      25.0

set xJNames        "J_HNHA_HZ J_HNCB_HZ J_CAHN_HZ"
set yJNames        "J_NCG_HZ"

set zeroScriptName tmpZero.com

set iseed          37238547

getArgD $ARGV -in        predTabName
getArgD $ARGV -ref       fidName
getArgD $ARGV -out       pkTabName
getArgD $ARGV -fidScript fidScriptName
getArgD $ARGV -simScript simScriptName
getArgD $ARGV -ftScript  ftScriptName
getArgD $ARGV -iseed     iseed
getArgD $ARGV -fidName   simFIDName 
getArgD $ARGV -ftName    simFTName 

getListArgD $ARGV -xJNames xJNames
getListArgD $ARGV -yJNames yJNames

if {![strcasecmp $xJNames None]} {set xJNames ""}
if {![strcasecmp $yJNames None]} {set xJNames ""}

if {[flagLoc $ARGV -useHi]}   {set useHiFlag 1}
if {[flagLoc $ARGV -nouseHi]} {set useHiFlag 0}

if {[flagLoc $ARGV -nusZF]}   {set nusZFFlag 1}
if {[flagLoc $ARGV -nonusZF]} {set nusZFFlag 0}


set tStr [file root [file tail $predTabName]]

getArgD $ARGV -title tStr

set vArgList \
 {dxRand dyRand keepFrac \
  xP0 yP0 xP0OffMin xP0OffMax yP0OffMin yP0OffMax \
  xwOffMin xwOffMax ywOffMin ywOffMax xLWScale yLWScale \
  hiScale noisePct xJMin xJMax yJMin yJMax} 

foreach v $vArgList {getArgD $ARGV -$v $v}

if {![file exists $predTabName]} \
   {
    puts "pred2spec. Error Finding Input Prediction Table $predTabName"
    exit 1
   }

set fidCmnd "None"

if {[strcasecmp $fidScriptName None]} \
   {
    if {![file exists $fidScriptName]} \
       {
        puts "pred2spec. Error Finding Input Script $fidScriptName"
        exit 1
       }

    set fidCmnd [extractScriptCmnd $fidScriptName "bruk2pipe var2pipe"]

    if {![llength $fidCmnd]} \
       {
        puts "pred2spec. Error Finding Command in Input Script $fidScriptName"
        exit 1
       }
   }

getArgD $fidCmnd -grpdly grpdly
getArgD $fidCmnd -df     dfVal

if {$nusZFFlag} \
   {
    set xN [nextPower2 [expr 4*[getNamedItem -xT $fidCmnd]]]
    set yN [nextPower2 [expr 4*[getNamedItem -yT $fidCmnd]]]
    set zN 1
   } \
else \
   {
    set xN [nextPower2 [expr 2*[getNamedItem -xT $fidCmnd]]]
    set yN [nextPower2 [expr 2*[getNamedItem -yT $fidCmnd]]]
    set zN 1
   }

if {$pseudo3DFlag} {set zN [getNamedItem -${zName}N $fidCmnd]}

getArgD $ARGV -xN xN
getArgD $ARGV -yN yN
getArgD $ARGV -zN zN

srand $iseed

#
#---------------------------------------------------------------------------------------------------------------------------

set t     [gdbRead table -in $predTabName]
set eList [gdbGetInfo $t -parms entries]

if {$useHiFlag} \
   {
    set maxHi 1.0
   } \
else \
   {
    set maxHi [getTabStat getMaxAbs $t HEIGHT]
   }

set noiseVal [expr $noisePct*$maxHi/100.0]

if {$pseudo3DFlag} \
   {
    catch "file delete -force $zeroScriptName $zeroFIDDir $zeroFTDir"
   } \
else \
   {
    catch "file delete -force $zeroScriptName $zeroFIDName $zeroFTName"
   }

set error [catch {set outUnit [open $zeroScriptName w 0744]}]

if {$error} \
   {
    puts "pred2spec. Error Creating Script $zeroScriptName"
    exit 1
   }

if {$pseudo3DFlag} \
   {
    puts $outUnit "#!/bin/csh"
    puts $outUnit ""
    puts $outUnit "simTimeND -ndim 3 -in None -rms 0.0 -scale 1.0 -nots \\"
    puts $outUnit [format " -xN     %10s  -yN     %10s  -zN    %10s   \\" $xN $yN $zN]
    puts $outUnit [format " -xT     %10s  -yT     %10s  -zT    %10s   \\" [getNamedItem -xT   $fidCmnd] [getNamedItem -${yName}T   $fidCmnd] [getNamedItem -${zName}T   $fidCmnd]]
    puts $outUnit [format " -xMODE  %10s  -yMODE  %10s  -zMODE %10s   \\" Complex Complex Real]
    puts $outUnit [format " -xSW    %10s  -ySW    %10s  -zSW   %10s   \\" [getNamedItem -xSW  $fidCmnd] [getNamedItem -${yName}SW  $fidCmnd] [getNamedItem -${zName}SW  $fidCmnd]]
    puts $outUnit [format " -xOBS   %10s  -yOBS   %10s  -zOBS  %10s   \\" [getNamedItem -xOBS $fidCmnd] [getNamedItem -${yName}OBS $fidCmnd] [getNamedItem -${zName}OBS $fidCmnd]]
    puts $outUnit [format " -xCAR   %10s  -yCAR   %10s  -zCAR  %10s   \\" [getNamedItem -xCAR $fidCmnd] [getNamedItem -${yName}CAR $fidCmnd] [getNamedItem -${zName}CAR $fidCmnd]]
    puts $outUnit [format " -xLAB   %10s  -yLAB   %10s  -zLAB  %10s   \\" [getNamedItem -xLAB $fidCmnd] [getNamedItem -${yName}LAB $fidCmnd] [getNamedItem -${zName}LAB $fidCmnd]]
    puts $outUnit [format " -xP0    %10s  -yP0    %10.1f  -zP0   %10.1f \\" $xP0 $yP0 0.0]
    puts $outUnit [format " -xP1    %10s  -yP1    %10.1f  -zP1   %10.1f \\" $xP1 $yP1 0.0]
    puts $outUnit " -aq2D States -out $zeroFIDName -noverb -ov"
    puts $outUnit ""
    puts $outUnit "xyz2pipe -in $zeroFIDName \\"
    puts $outUnit "| nmrPipe -fn SP -off 0.5 -end 0.95 -c 0.5 \\"
    puts $outUnit "| nmrPipe -fn ZF -size $xN \\"
    puts $outUnit "| nmrPipe -fn FT -di \\"
    puts $outUnit "| nmrPipe -fn TP \\"
    puts $outUnit "| nmrPipe -fn SP -off 0.5 -end 0.95 -c 0.5 \\"
    puts $outUnit "| nmrPipe -fn ZF -size $yN \\"
    puts $outUnit "| nmrPipe -fn FT -di \\"
    puts $outUnit "| nmrPipe -fn TP \\"
    puts $outUnit "| pipe2xyz -out $zeroFTName -ov"
    puts $outUnit ""
   } \
else \
   {
    puts $outUnit "#!/bin/csh"
    puts $outUnit ""
    puts $outUnit "simTimeND -ndim 2 -in None -rms 0.0 -scale 1.0 -nots \\"
    puts $outUnit [format " -xN     %10s  -yN     %10s \\" $xN $yN]
    puts $outUnit [format " -xT     %10s  -yT     %10s \\" [getNamedItem -xT   $fidCmnd] [getNamedItem -yT   $fidCmnd]]
    puts $outUnit [format " -xMODE  %10s  -yMODE  %10s \\" Complex Complex]
    puts $outUnit [format " -xSW    %10s  -ySW    %10s \\" [getNamedItem -xSW  $fidCmnd] [getNamedItem -ySW  $fidCmnd]]
    puts $outUnit [format " -xOBS   %10s  -yOBS   %10s \\" [getNamedItem -xOBS $fidCmnd] [getNamedItem -yOBS $fidCmnd]]
    puts $outUnit [format " -xCAR   %10s  -yCAR   %10s \\" [getNamedItem -xCAR $fidCmnd] [getNamedItem -yCAR $fidCmnd]]
    puts $outUnit [format " -xLAB   %10s  -yLAB   %10s \\" [getNamedItem -xLAB $fidCmnd] [getNamedItem -yLAB $fidCmnd]]
    puts $outUnit [format " -xP0    %10s  -yP0    %10.1f \\" $xP0 $yP0]
    puts $outUnit [format " -xP1    %10s  -yP1    %10.1f \\" $xP1 $yP1]
    puts $outUnit " -aq2D States -out $zeroFIDName -noverb -ov"
    puts $outUnit ""
    puts $outUnit "nmrPipe -in $zeroFIDName \\"
    puts $outUnit "| nmrPipe -fn SP -off 0.5 -end 0.95 -c 0.5 \\"
    puts $outUnit "| nmrPipe -fn ZF -size $xN \\"
    puts $outUnit "| nmrPipe -fn FT -di \\"
    puts $outUnit "| nmrPipe -fn TP \\"
    puts $outUnit "| nmrPipe -fn SP -off 0.5 -end 0.95 -c 0.5 \\"
    puts $outUnit "| nmrPipe -fn ZF -size $yN \\"
    puts $outUnit "| nmrPipe -fn FT -di \\"
    puts $outUnit "| nmrPipe -fn TP \\"
    puts $outUnit "  -out $zeroFTName -ov"
    puts $outUnit ""
   }

catch "close $outUnit"
catch "exec $zeroScriptName"

if {![file exists $zeroFTName]} \
   {
    puts "pred2spec. Error Creating Empty Spectrum $zeroFTName"
    exit 1
   }

set error [rdFDATA $zeroFIDName fdataT]

if {$error} \
   {
    puts "pred2spec. Error Reading Header in Empty Time-Domain Data $zeroFIDName"
    exit 1
   }

set error [rdFDATA $zeroFTName fdataF]

if {$error} \
   {
    puts "pred2spec. Error Reading Header in Empty Spectrum $zeroFTName"
    exit 1
   }

set error [catch {set outUnit [open $pkTabName w]}]

if {$error} \
   {
    puts "pred2spec. Error Creating Peak Table $pkTabName"
    exit 1
   }

# Input:  INDEX X_PPM Y_PPM XW_HZ YW_HZ J_HNHA_HZ J_HNCB_HZ J_CAHN_HZ J_NCG_NZ HEIGHT

set xJCount [llength $xJNames]
set yJCount [llength $yJNames]

puts -nonewline $outUnit "VARS   INDEX X_AXIS Y_AXIS X_PPM Y_PPM XW    YW    XW_HZ YW_HZ X_P0  Y_P0  X1  X3  Y1  Y3 "

for {set i 1} {$i <= ($xJCount + 1)} {incr i} {puts -nonewline $outUnit " X_COSJ$i"}
for {set i 1} {$i <= ($yJCount + 1)} {incr i} {puts -nonewline $outUnit " Y_COSJ$i"}

if {$pseudo3DFlag} \
   {
    for {set i 0} {$i < $zN} {incr i} {puts -nonewline $outUnit " Z_A$i"}
   }

puts $outUnit " HEIGHT ASS"

puts -nonewline $outUnit "FORMAT %5d   %8.3f  %8.3f  %8.3f %8.3f %8.3f %8.3f %8.3f %8.3f %6.1f %6.1f %5d %5d %5d %5d"

for {set i 1} {$i <= ($xJCount + 1)} {incr i} {puts -nonewline $outUnit " %8.3f  "}
for {set i 1} {$i <= ($yJCount + 1)} {incr i} {puts -nonewline $outUnit " %8.3f  "}

if {$pseudo3DFlag} \
   {
    for {set i 0} {$i < $zN} {incr i} {puts -nonewline $outUnit " %e"}
   } 

puts $outUnit " %e     %-8s"
puts $outUnit ""

set hiSum 0.0

foreach e $eList \
   {
    if {[rand] > $keepFrac} continue

    set xPPM [expr 2.0*$dxRand*([rand] - 0.5) + [gdbGet $e X_PPM]]
    set yPPM [expr 2.0*$dyRand*([rand] - 0.5) + [gdbGet $e Y_PPM]]

    if {$useHiFlag} \
       {
        set thisHi [gdbGet $e HEIGHT]
       } \
    else \
       {
        set thisHi [getValInRange $hiLimitMin $hiLimitMax] 
       }

    set hiSum  [expr $hiSum + $thisHi]

    set xPts [spec2pnt $fdataF X_AXIS ${xPPM}ppm]
    set yPts [spec2pnt $fdataF Y_AXIS ${yPPM}ppm]

    set xwHz [expr $xLWScale*[gdbGet $e XW_HZ] + [getValInRange $xwOffMin $xwOffMax]]
    set ywHz [expr $yLWScale*[gdbGet $e YW_HZ] + [getValInRange $ywOffMin $ywOffMax]]

    set xwPts [spec2pntW $fdataF X_AXIS ${xwHz}Hz]
    set ywPts [spec2pntW $fdataF Y_AXIS ${ywHz}Hz]

    if {$xwPts < 0.0} {set xwPts 0.0}
    if {$ywPts < 0.0} {set ywPts 0.0}

    set xP0Adj [getValInRange $xP0OffMin $xP0OffMax]
    set yP0Adj [getValInRange $yP0OffMin $yP0OffMax]

    puts -nonewline $outUnit " [format %5d   [gdbGet $e INDEX]]"
    puts -nonewline $outUnit " [format %8.3f $xPts]"
    puts -nonewline $outUnit " [format %8.3f $yPts]"
    puts -nonewline $outUnit " [format %8.3f $xPPM]"
    puts -nonewline $outUnit " [format %8.3f $yPPM]"
    puts -nonewline $outUnit " [format %8.3f $xwPts]"
    puts -nonewline $outUnit " [format %8.3f $ywPts]"
    puts -nonewline $outUnit " [format %8.3f $xwHz]"
    puts -nonewline $outUnit " [format %8.3f $ywHz]"
    puts -nonewline $outUnit " [format %6.1f $xP0Adj]"
    puts -nonewline $outUnit " [format %6.1f $yP0Adj]"
    puts -nonewline $outUnit " [format %5d   [expr int($xPts - 2.0*$xwPts)]]"
    puts -nonewline $outUnit " [format %5d   [expr int($xPts + 2.0*$xwPts)]]"
    puts -nonewline $outUnit " [format %5d   [expr int($yPts - 2.0*$ywPts)]]"
    puts -nonewline $outUnit " [format %5d   [expr int($yPts + 2.0*$ywPts)]]"

    for {set i 0} {$i <= $xJCount} {incr i} \
       {
        if {$i == $xJCount} \
           {
            set jHz [getValInRange $xJMin $xJMax]
           } \
        else \
           {
            set jHz [gdbGet $e [lindex $xJNames $i]]
           }

        set jPts 0.0

        if {$jHz < 666.0} {set jPts [spec2pntW $fdataT X_AXIS ${jHz}Hz]}
 
        puts -nonewline $outUnit " [format %8.3f $jPts]"
       }

    for {set i 0} {$i <= $yJCount} {incr i} \
       {
        if {$i == $yJCount} \
           {
            set jHz [getValInRange $yJMin $yJMax]
           } \
        else \
           {
            set jHz [gdbGet $e [lindex $yJNames $i]]
           }

        set jPts 0.0

        if {$jHz < 666.0} {set jPts [spec2pntW $fdataT Y_AXIS ${jHz}Hz]}

        puts -nonewline $outUnit " [format %8.3f $jPts]"
       }

     if {$pseudo3DFlag} \
        {
         for {set i 0} {$i < $zN} {incr i} {puts -nonewline $outUnit [format " % e" [gdbGet $e Z_A$i]]}
        }

    puts -nonewline $outUnit " [format %e [gdbGet $e HEIGHT]] [format %-8s [gdbGet $e ASS]]"

    puts $outUnit ""
   }

catch "close $outUnit"

set pkTab [gdbRead table -in $pkTabName] 
set eList [gdbGetInfo $pkTab -parms entries]
set vList [gdbGetInfo $pkTab -parms varNames]
set dList ""

set pkMaxVal [getTabStat getMaxAbs $pkTab HEIGHT]

foreach eI $eList \
   {
    if {[gdbGet $eI INDEX] < 0} continue

    set thisHi [safeDiv [gdbGet $eI HEIGHT] $pkMaxVal]

    if {100.0*$thisHi < $filterHi} {gdbSet $eI INDEX -1}

    foreach eJ $eList \
       {
        if {[gdbGet $eJ INDEX] < 0} continue

        set thisXLoc [gdbGet $eI X_AXIS]
        set thisYLoc [gdbGet $eI Y_AXIS]

        if {[strcasecmp $eI $eJ]} \
           {
            if {abs($thisXLoc - [gdbGet $eJ X_AXIS]) < $filterDX && abs($thisYLoc - [gdbGet $eJ Y_AXIS]) < $filterDY} \
               {
                gdbSet $eJ INDEX -1
               }
           }
       }
   }

foreach e $eList \
   {
    if {[gdbGet $e INDEX] < 0} {lappend dList $e}
   }

gdbFree $dList

set eList   [gdbGetInfo $pkTab -parms entries]
set pkCount [llength $eList]

for {set i 1} {$i <= $pkCount} {incr i} {gdbSet [lindex $eList [expr $i - 1]] INDEX $i}

gdbWrite $pkTab $pkTabName
 
set error [catch {set outUnit [open $simScriptName w 0744]}]

if {$error} \
   {
    puts "pred2spec. Error Creating Time-Domain Simulation Script $simScriptName"
    exit 1
   }

catch "file delete $simFIDName"

set thisXMin [expr (1 + $xN/2) + $dxMinH2O]
set thisXMax [expr (1 + $xN/2) + $dxMaxH2O]

if {$pseudo3DFlag} \
   {
    puts $outUnit "#!/bin/csh"
    puts $outUnit ""
    puts $outUnit "set inName  = (\`getArgD \$argv -in    $pkTabName\`)"
    puts $outUnit "set fidName = (\`getArgD \$argv -fid   $fidName\`)"
    puts $outUnit "set outName = (\`getArgD \$argv -out   $simFIDName\`)"
    puts $outUnit "set tStr    = (\`getArgD \$argv -title $tStr\`)"
    puts $outUnit "set scale   = (\`getArgD \$argv -scale 1.0`)"
    puts $outUnit ""
    puts $outUnit "simTimeND -noverb -ndim 3 -aq2D States -in \$inName -rms $noiseVal -scale \$scale -nots \\"
    puts $outUnit [format " -xN     %10s  -yN     %10s  -zN    %10s   \\" $xN $yN $zN]
    puts $outUnit [format " -xT     %10s  -yT     %10s  -zT    %10s   \\" [getNamedItem -xT   $fidCmnd] [getNamedItem -${yName}T   $fidCmnd] [getNamedItem -${zName}T   $fidCmnd]]
    puts $outUnit [format " -xMODE  %10s  -yMODE  %10s  -zMODE %10s   \\" Complex Complex Real]
    puts $outUnit [format " -xSW    %10s  -ySW    %10s  -zSW   %10s   \\" [getNamedItem -xSW  $fidCmnd] [getNamedItem -${yName}SW  $fidCmnd] [getNamedItem -${zName}SW  $fidCmnd]]
    puts $outUnit [format " -xOBS   %10s  -yOBS   %10s  -zOBS  %10s   \\" [getNamedItem -xOBS $fidCmnd] [getNamedItem -${yName}OBS $fidCmnd] [getNamedItem -${zName}OBS $fidCmnd]]
    puts $outUnit [format " -xCAR   %10s  -yCAR   %10s  -zCAR  %10s   \\" [getNamedItem -xCAR $fidCmnd] [getNamedItem -${yName}CAR $fidCmnd] [getNamedItem -${zName}CAR $fidCmnd]]
    puts $outUnit [format " -xLAB   %10s  -yLAB   %10s  -zLAB  %10s   \\" [getNamedItem -xLAB $fidCmnd] [getNamedItem -${yName}LAB $fidCmnd] [getNamedItem -${zName}LAB $fidCmnd]]
    puts $outUnit [format " -xP0    %10s  -yP0    %10.1f  -zP0   %10.1f \\" $xP0 $yP0 0.0]
    puts $outUnit [format " -xP1    %10s  -yP1    %10.1f  -zP1   %10.1f \\" $xP1 $yP1 0.0]
    puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/simH2O.M -var iseed $iseed fSize $xN p0Min $p0MinH2O p0Max $p0MaxH2O hi [format %.3e $hiSum] \\"
    puts $outUnit "                                               xMinPts [format %.2f $thisXMin] xMaxPts [format %.2f $thisXMax] lwPts $lwH2O \\"
    puts $outUnit "| nmrPipe -fn DMX -df $dfVal -hdr -out \$outName -noverb -ov"
    puts $outUnit ""
    puts $outUnit "copyTau.com \$fidName \$outName -all"
    puts $outUnit ""
   } \
else \
   {
    puts $outUnit "#!/bin/csh"
    puts $outUnit ""
    puts $outUnit "set inName  = (\`getArgD \$argv -in    $pkTabName\`)"
    puts $outUnit "set outName = (\`getArgD \$argv -out   $simFIDName\`)"
    puts $outUnit "set tStr    = (\`getArgD \$argv -title $tStr\`)"
    puts $outUnit "set scale   = (\`getArgD \$argv -scale 1.0`)"
    puts $outUnit ""
    puts $outUnit "simTimeND -noverb -ndim 2 -aq2D States -in \$inName -rms $noiseVal -scale \$scale -nots \\"
    puts $outUnit [format " -xN     %10s  -yN     %10s \\" $xN $yN]
    puts $outUnit [format " -xT     %10s  -yT     %10s \\" [getNamedItem -xT   $fidCmnd] [getNamedItem -yT   $fidCmnd]]
    puts $outUnit [format " -xMODE  %10s  -yMODE  %10s \\" Complex Complex]
    puts $outUnit [format " -xSW    %10s  -ySW    %10s \\" [getNamedItem -xSW  $fidCmnd] [getNamedItem -ySW  $fidCmnd]]
    puts $outUnit [format " -xOBS   %10s  -yOBS   %10s \\" [getNamedItem -xOBS $fidCmnd] [getNamedItem -yOBS $fidCmnd]]
    puts $outUnit [format " -xCAR   %10s  -yCAR   %10s \\" [getNamedItem -xCAR $fidCmnd] [getNamedItem -yCAR $fidCmnd]]
    puts $outUnit [format " -xLAB   %10s  -yLAB   %10s \\" [getNamedItem -xLAB $fidCmnd] [getNamedItem -yLAB $fidCmnd]]
    puts $outUnit [format " -xP0    %10s  -yP0    %10.1f \\" $xP0 $yP0]
    puts $outUnit [format " -xP1    %10s  -yP1    %10.1f \\" $xP1 $yP1] 
    puts $outUnit "| nmrPipe -fn MAC -macro \$NMRTXT/simH2O.M -var iseed $iseed fSize $xN p0Min $p0MinH2O p0Max $p0MaxH2O hi [format %.3e $hiSum] \\"
    puts $outUnit "                                               xMinPts [format %.2f $thisXMin] xMaxPts [format %.2f $thisXMax] lwPts $lwH2O \\" 
    puts $outUnit "| nmrPipe -fn DMX -df $dfVal -hdr -out \$outName -noverb -ov"
    puts $outUnit ""
    puts $outUnit "sethdr \$outName -title \$tStr"
    puts $outUnit ""
   }

catch "close $outUnit"
catch "exec $simScriptName"

if {[strcasecmp $simFTName None]} \
   {
    set error [catch {set outUnit [open $ftScriptName w 0744]}]

    if {$error} \
       {
        puts "pred2spec. Error Creating Simulation Transform Script $ftScriptName"
        exit 1
       }

    catch "file delete $simFTName"

    puts $outUnit "#!/bin/csh"
    puts $outUnit ""
    puts $outUnit "set inName  = (\`getArgD \$argv -in    $simFIDName\`)"
    puts $outUnit "set outName = (\`getArgD \$argv -out   $simFTName\`)"
    puts $outUnit "set tStr    = (\`getArgD \$argv -title $tStr\`)"
    puts $outUnit ""

    if {$pseudo3DFlag} \
       {
        set cmnd "basicFT2.com -pseudo3D -in \$inName -noverb"
       } \
    else \
       {
        set cmnd "basicFT2.com -in \$inName -noverb"
       }

    append cmnd " -xEXTX1 10.3ppm -xEXTXN 6.0ppm -xSOL SOL -xZFARG size=$xN -xP0 $xP0 -xP1 $xP0"
    append cmnd " -yZFARG size=$yN -yP0 $yP0 -yP1 $yP1 -out \$outName"

    if {$nusZFFlag} {append cmnd " -nusZF -smile"}

    prettyPrintCmnd $outUnit "" $cmnd

    puts $outUnit ""
    puts $outUnit "sethdr $simFTName -title \$tStr"

    catch "close $outUnit"
    catch "exec $ftScriptName"
   }

exit 0
