#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

wm iconify .

proc initDBVars {} \
{
   global objInfo specInfo stripInfo

   set obj db

   set objInfo($obj,typeName)  expTypes.txt
   set objInfo($obj,wName)     expWeights.tab
   set objInfo($obj,inName)    specInfo.tab
   set objInfo($obj,seqName)   seqInfo.tab
   set objInfo($obj,joinName)  join.tab
   set objInfo($obj,shiftName) shifts.tab
   set objInfo($obj,baseDir)   ../
   set objInfo($obj,dbDir)     db
   set objInfo($obj,csDir)     cs 
   set objInfo($obj,verbose)   0
   set objInfo($obj,roi)       0

   set objInfo($obj,multiAss) \
    "h*(*) hb(i) hb(i-1) hg(i) he(i) cd(i) cg(i) ce(i)"

   set specInfo(xw)  0.12ppm
   set specInfo(yw)  0.80ppm
   set specInfo(zw)  1

   set specInfo(xw)  1
   set specInfo(yw)  1
   set specInfo(zw)  1

   set objInfo($obj,joinW)   0.1
   set objInfo($obj,joinW)   0.01
   set objInfo($obj,pThresh) 2.0
   set objInfo($obj,hiRatio) 1.5

   set stripInfo(idList)  ""
   set stripInfo(badList) ""
}

proc getDBArgs {} \
{
   global objInfo specInfo ARGV

   set obj db
 
   if {[flagLoc $ARGV -help]} \
      {
       puts "Build NMRPrime Strip Database From Input Peak Tables and Spectra."
       puts " -in   iTab   \[specInfo.tab\]   Lists Input Spectra and Tables." 
       puts " -seq  sTab   \[seqInfo.tab\]    Lists Input Residue Sequence."
       puts " -type tTab   \[expTypes.txt\]   Experiment Type Definitions."
       puts " -w    wTab   \[expWeights.tab\] Experiment Sequential Weights."
       puts " -dir  bDir   \[../\]            Spectral Base-Level Directory."
       puts " -verb                         Verbose Mode ON."
       puts " -noverb                       Verbose Mode OFF."
       puts "Strip Parameters:"
       puts " -xw   xw     \[0.12ppm\]        X-Axis +/- Width."
       puts " -yw   yw     \[0.80ppm\]        Y-Axis +/- Width."
       puts " -zw   zw     \[1\]              Z-Axis +/- Width."
       puts "Weighting Factors in Scores:"
       puts " -jW     jW   \[0.1\]            Sequential Connection Weight." 
       puts " -thresh pT   \[2.0\]            Permutation Score Threshold." 
       puts "Output Files:"
       puts " -join  jName \[join.tab\]       Sequential Connection Scores."
       puts " -db    dbDir \[db\]             Output Database Directory."
       exit 1
      }
 
   getArgD $ARGV -in     objInfo($obj,inName)
   getArgD $ARGV -seq    objInfo($obj,seqName)
   getArgD $ARGV -dir    objInfo($obj,baseDir)
   getArgD $ARGV -type   objInfo($obj,typeName)
   getArgD $ARGV -w      objInfo($obj,wName)
 
   getArgD $ARGV -xw     specInfo($obj,xw)
   getArgD $ARGV -yw     specInfo($obj,yw)
   getArgD $ARGV -zw     specInfo($obj,xz)

   getArgD $ARGV -jW     objInfo($obj,joinW)
   getArgD $ARGV -thresh objInfo($obj,pThresh)

   getArgD $ARGV -db     objInfo($obj,dbDir)
   getArgD $ARGV -join   objInfo($obj,joinName)

   if {[flagLoc $ARGV -verb]}   {set objInfo($obj,verbose) 1}
   if {[flagLoc $ARGV -noverb]} {set objInfo($obj,verbose) 0}
}

proc getExpTypes {} \
{
   global objInfo expInfo

   set obj db

   if {![file exists $objInfo($obj,typeName)]} \
      {
       puts stderr "Error Finding Experiment Type Table $objInfo($obj,typeName)"
       exit 1
      }

   set error [catch {set inUnit [open $objInfo($obj,typeName) r]}]

   if {$error} \
      {
       puts stderr "Error Opening Experiment Type Table $objInfo($obj,typeName)"
       exit 1
      }

   set expInfo(expTypes) ""
   set id 0

   while {[gets $inUnit line] >= 0} \
      {
       set str [string trim $line]
       set c   [string index $str 0]

       if {![string length $str]} continue
       if {![strcmp $c "#"]}      continue

       incr id

       set expType  [lindex $str 0]
       set posCount [lindex $str 1]
       set negCount [lindex $str 2]

       set assTypes [lrange $str 3 end]
      
       lappend expInfo(expTypes) $expType 

       set expInfo($expType,assTypes) $assTypes
       set expInfo($expType,id) $id

       set expInfo($expType,posCount) $posCount 
       set expInfo($expType,negCount) $negCount 
       set expInfo($expType,posTypes) ""
       set expInfo($expType,negTypes) ""
       set expInfo($expType,types)    ""

       foreach dim {x y z a} \
          {
           set expInfo($expType,type,$dim) ""
          }

       foreach assType $assTypes \
          {
           set c [string index $assType 0]

           set assInfo [split $assType "/"]
           set i 0

           foreach assItem $assInfo \
              {
               set assStr [split $assItem "<>"]
               set assI   [string trim [lindex $assStr 0] "+-"]
               set dim    [lindex {x y z a} $i]

               set n      [llength $assStr]

               if {[lsearch -exact $expInfo($expType,type,$dim) $assI] < 0} \
                  {
                   lappend expInfo($expType,type,$dim) $assI
                  }

               incr i
              }

           foreach dim {x y z a} \
              {
               set expInfo($expType,type,$dim) \
                [join $expInfo($expType,type,$dim) "|"] 
              }

           set assItem [lindex $assInfo end]
           set assStr  [split $assItem "<>"]
           set assI    [string trim [lindex $assStr 0] "+-"]
           set n       [llength $assStr]

           if {![strcmp $c "-"]} \
              {
               lappend expInfo($expType,negTypes) $assI
              } \
           else \
              {
               lappend expInfo($expType,posTypes) $assI
              }

           if {$n > 1} \
              {
               set assJ [lindex $assStr 1]

               set fmt "Created Peak Height Rule for %-12s %s %s %s"

               if {[string first ">" $assItem] >= 0} \
                  {
                   set assIJ $assI|$assJ
                   set expInfo($expType,hiInfo,$assIJ) gt
                   puts [format $fmt $expType: $assI > $assJ]

                   set assIJ $assJ|$assI
                   set expInfo($expType,hiInfo,$assIJ) lt
#                  puts [format $fmt $expType: $assJ < $assI]
                  } \
               else \
                  {
                   set assIJ $assI|$assJ 
                   set expInfo($expType,hiInfo,$assIJ) lt 
                   puts [format $fmt $expType: $assI < $assJ]

                   set assIJ $assJ|$assI
                   set expInfo($expType,hiInfo,$assIJ) gt
#                  puts [format $fmt $expType: $assJ > $assI]
                  }
              }

           lappend expInfo($expType,types) $assI
          }

       foreach assI $expInfo($expType,types) \
          {
           foreach assJ $expInfo($expType,types) \
              {
               set assIJ $assI|$assJ

               if {![info exists expInfo($expType,hiInfo,$assIJ)]} \
                  {
                   set expInfo($expType,hiInfo,$assIJ) none
                  }
              }
          }

       set expInfo($expType,posTypes) [join $expInfo($expType,posTypes) "|"] 
       set expInfo($expType,negTypes) [join $expInfo($expType,negTypes) "|"] 
      }

   foreach eI $expInfo(expTypes) \
      {
       foreach eJ $expInfo(expTypes) \
          {
           set expInfo($eI,$eJ,sIJ) 0.0
           set expInfo($eI,$eJ,sJI) 0.0
           set expInfo($eI,$eJ,nIJ) 0 
           set expInfo($eI,$eJ,nJI) 0 
          }
      }

   close $inUnit
}

proc getExpWeights {} \
{
   global objInfo expInfo 

   set obj db

   if {![file exists $objInfo($obj,wName)]} \
      {
       puts stderr "Error Finding Experiment Weight Table $objInfo($obj,wName)"
       exit 1
      }

   set wTab  [gdbRead table -in $objInfo($obj,wName)]
   set eList [gdbGetInfo $wTab -parms entries]

   foreach entry $eList \
      {
       set eI [gdbGet $entry EXP_TYPEI]
       set eJ [gdbGet $entry EXP_TYPEJ]

       set expInfo($eI,$eJ,sIJ) [gdbGet $entry SCORE_IJ] 
       set expInfo($eI,$eJ,sJI) [gdbGet $entry SCORE_JI]
       set expInfo($eI,$eJ,nIJ) [gdbGet $entry COUNT_IJ]
       set expInfo($eI,$eJ,nJI) [gdbGet $entry COUNT_JI]
      }

   gdbFree $wTab
}

proc getSpecLimits {} \
{
   global objInfo expInfo specInfo

   set obj db

   puts ""

   set expTypes [lrange $specInfo(expTypes) 1 end]
   set ref      [lindex $specInfo(expTypes) 0]

   set specInfo(related) ""

   foreach eI $specInfo(expTypes) \
      {
       set specInfo($eI,related) ""
      }

   foreach eI $expTypes \
      {
       foreach eJ $expTypes \
          {
           if {$expInfo($eI,$eJ,sIJ) != 0.0 || $expInfo($eI,$eJ,sJI) != 0.0} \
              {
               lappend specInfo($eI,related) $eJ
               lappend specInfo($eJ,related) $eI
              }
          }
      }

   foreach eI $expTypes \
      {
       set rList $specInfo($eI,related)

       foreach eJ $expTypes \
          {
           if {[llength [listIntersect $rList $specInfo($eJ,related)]]} \
              {
               set rList [listUnion $rList $specInfo($eJ,related)]
              }
          }

       set specInfo($eI,related) $rList
      }

   foreach eI $expTypes \
     {
      set specInfo($eI,z,minPPM) $specInfo($eI,z,lastPPM)
      set specInfo($eI,z,maxPPM) $specInfo($eI,z,firstPPM)

      foreach eJ $specInfo($eI,related) \
         {
          if {$specInfo($eJ,z,lastPPM) < $specInfo($eI,z,minPPM)} \
             {
              set specInfo($eI,z,minPPM) $specInfo($eJ,z,lastPPM)
             }

          if {$specInfo($eJ,z,firstPPM) > $specInfo($eI,z,maxPPM)} \
             {
              set specInfo($eI,z,maxPPM) $specInfo($eJ,z,firstPPM)
             }
         }

       set fmt " %-6s % 8.3f % 8.2f ppm."

       puts "Relations to $eI: $specInfo($eI,related)"

       puts [format $fmt $specInfo($eI,z,name): \
                         $specInfo($eI,z,minPPM)  \
                         $specInfo($eI,z,maxPPM)]
     }

   set n 0
   set i 0

   foreach eI $expTypes \
      {
       set groupInfo($i) [listUnion $specInfo($eI,related) $eI]

       incr i
       incr n
      }

   set adjCount $n

   while {$adjCount} \
      {
       set adjCount 0

       for {set i 0} {$i < $n - 1} {incr i} \
          {
           if {![llength $groupInfo($i)]} continue

           for {set j [expr $i + 1]} {$j < $n} {incr j} \
              {
               if {![llength $groupInfo($j)]} continue

               if {[llength [listIntersect $groupInfo($i) $groupInfo($j)]]} \
                  {
                   set groupInfo($i) [listUnion $groupInfo($i) $groupInfo($j)]
                   set groupInfo($j) ""

                   incr adjCount
                  }
              }
          }
      }

   for {set i 0} {$i < $n} {incr i} \
       {
        if {![llength $groupInfo($i)]} continue

        lappend specInfo(related) $groupInfo($i)
       }

   puts "Groupings of Related Spectra:"
   puts " $specInfo(related)"
}

proc getSpecStrips {} \
{
   global objInfo specInfo stripInfo pkInfo roiInfo

   set obj db

   puts ""

   set expTypes [lrange $specInfo(expTypes) 1 end]
   set ref      [lindex $specInfo(expTypes) 0]
   set roi      $objInfo($obj,roi) 

   set xSizeMax 0
   set ySizeMax 0

   foreach expType $specInfo(expTypes) \
      {
       foreach id $stripInfo(idList) \
          {
           set roiInfo($expType,$id,roi) 0
          }
      }

   foreach expType $expTypes \
     {
      puts "Reading Strips for: $expType"

      set n 0
 
      foreach id $stripInfo(idList) \
         {
          if {$pkInfo($ref,$id,x) > 999 || $pkInfo($ref,$id,y) > 999} \
             {
              continue
             }

          set x0 [getShift $ref $id x]ppm
          set z0 [getShift $ref $id y]ppm

          set y1 $specInfo($expType,z,maxPPM)ppm
          set yn $specInfo($expType,z,minPPM)ppm

          incr roi
          incr n

          set roiInfo($expType,$id,roi) $roi

          readROI -in $specInfo($expType,specName) -roi $roi -ndim 2 \
                  -dx $specInfo($expType,x,name) $x0 $specInfo(xw) \
                  -y  $specInfo($expType,z,name) $y1 $yn \
                  -dz $specInfo($expType,y,name) $z0 $specInfo(zw) \
                  -w  $specInfo($expType,x,name) -1 \
                      $specInfo($expType,z,name) -1 \
                      -center X

          set rInfo [getROIInfo -roi $roi] 
          set sList [lindex $rInfo 2]

          set roiInfo($expType,$id,rdata) [lindex $rInfo 0]
          set roiInfo($expType,$id,fdata) [lindex $rInfo 1]

          set roiInfo($expType,$id,xSize) [lindex $sList 0] 
          set roiInfo($expType,$id,ySize) [lindex $sList 1] 

          if {$xSizeMax < $roiInfo($expType,$id,xSize)} \
             {
              set xSizeMax $roiInfo($expType,$id,xSize)
             }

          if {$ySizeMax < $roiInfo($expType,$id,ySize)} \
             {
              set ySizeMax $roiInfo($expType,$id,ySize)
             }
         }
     }

   if {[strcmp $specInfo(xw) 1]} \
      {
       set xZF [nextPower2 $xSizeMax]
      } \
   else \
      {
       set xZF $xSizeMax
      }

   set yZF [nextPower2 $ySizeMax]

   set stripInfo(xZF) $xZF
   set stripInfo(yZF) $yZF

   puts ""
   puts [format "Strip Size X: %4d  Interpolated: %4d" $xSizeMax $xZF]
   puts [format "Strip Size Y: %4d  Interpolated: %4d" $ySizeMax $yZF]
   puts ""

   set nn [expr $xZF*$yZF]
   set nS [llength $expTypes]

   foreach expType $expTypes \
     {
      puts "Interpolating Strips for: $expType"
      set n 0

      foreach id $stripInfo(idList) \
         {
          if {!$roiInfo($expType,$id,roi)} continue

          set error \
            [catch {set roiInfo($expType,$id,pdata) \
                     [vResize -src $roiInfo($expType,$id,rdata) -ndim 2 \
                              -xSize $roiInfo($expType,$id,xSize) \
                              -ySize $roiInfo($expType,$id,ySize) \
                              -xZF $xZF -yZF $yZF]}]

          if {$error} \
             {
              puts stderr "Error Creating Interpolated Strips."
              return 1
             }

          freeROI -roi $roiInfo($expType,$id,roi)

          set roiInfo($expType,$id,roi) -1
         }
     }

   set roiInfo(workI) [sysMalloc $nn float]
   set roiInfo(workJ) [sysMalloc $nn float]
}

proc freeSpecStrips {} \
{
   global objInfo specInfo stripInfo roiInfo

   set obj db

   puts ""

   set expTypes [lrange $specInfo(expTypes) 1 end]

   puts "Freeing Strip Memory."
   puts ""

   sysFree $roiInfo(workI)
   sysFree $roiInfo(workJ)

   foreach expType $expTypes \
     {
      foreach id $stripInfo(idList) \
         {
          if {!$roiInfo($expType,$id,roi)} continue

          sysFree $roiInfo($expType,$id,pdata)
         }
     }
}

proc initScoreInfo {} \
{
   global objInfo specInfo stripInfo seqInfo joinInfo

   set obj db

   puts ""

   set expTypes [lrange $specInfo(expTypes) 1 end]

   foreach idI $stripInfo(idList) \
      {
       foreach idJ $stripInfo(idList) \
          {
           set joinInfo($idI,$idJ,surf) 0.0
           set joinInfo($idI,$idJ,pix)  0.0
           set joinInfo($idI,$idJ,pk)   0.0
          }
      }
}

proc getPeakJoinScore {} \
{
   global objInfo expInfo specInfo stripInfo joinInfo pkInfo 

   set obj db

   set expTypes [lrange $specInfo(expTypes) 1 end]

   puts ""

   set sumS 0.0
   set nS   0

   foreach idI $stripInfo(idList) \
       {
        puts "Calculating Peak Scores for Strip $idI:"

        set bestScore 0.0
        set bestID None

        foreach idJ $stripInfo(idList) \
           {
            if {$idI == $idJ} continue

            set joinInfo($idI,$idJ,pk) 0.0

            foreach eI $expTypes \
               {
                set pkListI $stripInfo($eI,$idI,pkList) 
                if {![llength $pkListI]} continue

                foreach eJ $expTypes \
                   {
                    if {$expInfo($eI,$eJ,sIJ) == 0.0} continue

                    set pkListJ $stripInfo($eJ,$idJ,pkList)
                    if {![llength $pkListJ]} continue

                    if {$specInfo($eI,z,tol) > $specInfo($eJ,z,tol)} \
                       {
                        set tol $specInfo($eI,z,tol)
                       } \
                    else \
                       {
                        set tol $specInfo($eJ,z,tol)
                       }

                    foreach pkI $pkListI \
                      {
                       set zI [getShift $eI $pkI z]

                       foreach pkJ $pkListJ \
                          {
                           set zJ [getShift $eJ $pkJ z]

                           if {abs($zI - $zJ) > $tol} continue

                           set joinInfo($idI,$idJ,pk) \
                            [expr $joinInfo($idI,$idJ,pk) + \
                                  $expInfo($eI,$eJ,sIJ)]
                          }
                      }
                  }
              }

           if {$joinInfo($idI,$idJ,pk) > $bestScore} \
              {
               set bestScore $joinInfo($idI,$idJ,pk)
               set bestID $idJ
              }

           if {$joinInfo($idI,$idJ,pk) > 0.0} \
              {
               set  sumS [expr $sumS + $joinInfo($idI,$idJ,pk)]
               incr nS
              }
          }

       if {![strcasecmp $bestID None]} \
          {
           lappend stripInfo(badList) $idI
           puts "  None"
          } \
       else \
          {
           puts [format "  %-4d %7.3f" $bestID $bestScore]
          }
      }

   if {!$nS} return
   if {$sumS == 0.0} return

   set sumS [expr $sumS/$nS]

   foreach idI $stripInfo(idList) \
      {
       foreach idJ $stripInfo(idList) \
          {
           if {$idI == $idJ} continue

           set joinInfo($idI,$idJ,pk) [expr 100.0*$joinInfo($idI,$idJ,pk)/$sumS]
          }
     }
}

proc getStripJoinScore {} \
{
   global objInfo expInfo specInfo stripInfo joinInfo roiInfo

   set obj db

   puts ""

   set expTypes [lrange $specInfo(expTypes) 1 end]

   foreach eI $expTypes \
      {
       foreach eJ $expTypes \
          {
           foreach idI $stripInfo(idList) \
              {
               foreach idJ $stripInfo(idList) \
                  {
                   set surfInfo($eI,$idI,$eJ,$idJ) 0.0
                   set pixInfo($eI,$idI,$eJ,$idJ)  0.0
                  }
              }
          }
      }

   set nn [expr $stripInfo(xZF)*$stripInfo(yZF)]
   set nS [llength $expTypes]

   set workI $roiInfo(workI)
   set workJ $roiInfo(workJ)
 
   foreach eI $expTypes \
      {
       foreach eJ $expTypes \
          {
           if {$expInfo($eI,$eJ,sIJ) == 0.0} continue

           puts "Calculating Strip Products for $eI and $eJ"

           foreach idI $stripInfo(idList) \
              {
               if {!$roiInfo($eI,$idI,roi)} continue

               foreach idJ $stripInfo(idList) \
                  {
                   if {!$roiInfo($eJ,$idJ,roi)} continue

                   vvMove $workI $roiInfo($eI,$idI,pdata) $nn
                   vAbs $workI $nn
                   set s [div0 [expr 10.0*$nS] [vSum $workI $nn]]
                   vsMult $workI $s $nn

                   vvMove $workJ $roiInfo($eJ,$idJ,pdata) $nn
                   vAbs $workJ $nn
                   set s [div0 [expr 10.0*$nS] [vSum $workJ $nn]]
                   vsMult $workJ $s $nn

                   set surfInfo($eI,$idI,$eJ,$idJ) [vvDot $workI $workJ $nn]

                   vvMove $workI $roiInfo($eI,$idI,pdata) $nn
                   vBinarize $workI $stripInfo($eI,$idI,hi) $nn
                   vAbs $workI $nn
                   set s [div0 [expr 10.0*$nS] [vSum $workI $nn]]
                   vsMult $workI $s $nn

                   vvMove $workJ $roiInfo($eJ,$idJ,pdata) $nn
                   vBinarize $workJ $stripInfo($eJ,$idJ,hi) $nn
                   vAbs $workJ $nn
                   set s [div0 [expr 10.0*$nS] [vSum $workJ $nn]]
                   vsMult $workJ $s $nn

                   set pixInfo($eI,$idI,$eJ,$idJ) [vvDot $workI $workJ $nn]
                  }
              }
          }
      }

   set nN [llength $stripInfo(idList)]

   foreach idI $stripInfo(idList) \
      {
       set sumPix 0.0
       set nPix 0

       foreach idJ $stripInfo(idList) \
          {
           if {$idI == $idJ} continue

           set sSurf 0.0
           set sPix  0.0
           set nS 0

           foreach eI $expTypes \
              {
               foreach eJ $expTypes \
                  {
                   set w $expInfo($eI,$eJ,sIJ)

                   if {$w == 0.0} continue

                   set sSurf [expr $sSurf + $w*$surfInfo($eI,$idI,$eJ,$idJ)]
                   set sPix  [expr $sPix  + $w*$pixInfo($eI,$idI,$eJ,$idJ)]

                   incr nS
                  }
              }

           if {!$nS} continue

           set sSurf  [expr $sSurf/$nS]
           set sPix   [expr $sPix/$nS]

           set sumPix [expr $sumPix + $sPix]
           incr nPix

           set joinInfo($idI,$idJ,surf) $sSurf
           set joinInfo($idI,$idJ,pix)  $sPix
          }

       if {$sumPix == 0.0} continue
       if {!$nPix} continue

       set sumPix [expr $sumPix/$nPix]

       foreach idJ $stripInfo(idList) \
          {
           if {$idI == $idJ} continue
          
           set sPix  [expr 100.0*$joinInfo($idI,$idJ,pix)/$sumPix] 
           set sSurf $joinInfo($idI,$idJ,surf)

           set joinInfo($idI,$idJ,surf) $sSurf
           set joinInfo($idI,$idJ,pix)  $sPix
          }
      }
}

proc saveShifts {} \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   puts ""
   puts "Saving Chemical Shifts."

   set error [catch {set outUnit [open $objInfo($obj,shiftName) w]}]

   if {$error} \
      {
       puts stderr \
        "Error Opening Chemical Shift Output Table $objInfo($obj,shiftName)"

       set outUnit stdout
      }

   set seqStr [getSeqListSeqStr $seqInfo(seqList) $seqInfo(r1) $seqInfo(rN)]

   puts $outUnit $seqStr
   puts $outUnit ""
   puts $outUnit "VARS   INDEX ATOMNAME SHIFT SIGMA COUNT"
   puts $outUnit "FORMAT %5d   %-16s    %9.3f %9.3f %3d"
   puts $outUnit ""

   foreach id $stripInfo(idList) \
      {
       set assList   ""
       set ambigList ""

       catch "unset shiftInfo"
       catch "unset shiftInfo"

       foreach expType $specInfo(expTypes) \
          {
           foreach pkID $stripInfo($expType,$id,pkList) \
              {
               for {set i 0} {$i < $specInfo($expType,dimCount)} {incr i} \
                  {
                   set dim [lindex {x y z a} $i]
                   set ass [getAssStr $expType $pkID $dim]
                   set s   [getShift  $expType $pkID $dim]

                   if {$s > 999 || [string first (*) $ass] >= 0} continue

                   if {![strcasecmp $ass None] || [string first | $ass] >= 0} \
                      {
                       lappend ambigList $pkID@$expType@$dim@$ass@$s
                      } \
                   else \
                      {
                       if {[lsearch -exact $assList $ass] < 0} \
                          {
                           lappend assList $ass
                          }

                       lappend shiftInfo($ass,tol) $specInfo($expType,$dim,tol)
                       lappend shiftInfo($ass,ass) $pkID@$expType@$dim
                       lappend shiftInfo($ass,s)   $s
                      }
                  }
              }
         }

      set fmt " %5d %-16s % 8.3f % 8.3f %3d"

      foreach ass $assList \
         {
          set n      [llength   $shiftInfo($ass,s)]
          set s      [getAvg    $shiftInfo($ass,s)]
          set dev    [getStdDev $shiftInfo($ass,s)]
          set tol    [getMax    $shiftInfo($ass,tol)]

          set assStr [join $shiftInfo($ass,ass) ","]

          if {$n < 3} {set dev $tol}

          puts $outUnit [format $fmt $id $ass $s $dev $n]
         }

      foreach assStr $ambigList \
         {
          set pList   [split $assStr "@"]

          set pkID    [lindex $pList 0]
          set expType [lindex $pList 1]
          set dim     [lindex $pList 2]
          set ass     [lindex $pList 3]
          set s       [lindex $pList 4]

          set n   1
          set dev $specInfo($expType,$dim,tol)

          puts $outUnit [format $fmt $id $ass $s $dev $n]
         }
     }

   close $outUnit
}

proc gScoreSort { locI locJ } \
{
   global glbScoreList

   set sI [lindex $glbScoreList $locI]
   set sJ [lindex $glbScoreList $locJ]

   if {$sI < $sJ} {return  1}
   if {$sI > $sJ} {return -1}

   return 0
}

proc getScoreRanks { sList } \
{
   global glbScoreList

   set glbScoreList $sList

   set rankList ""
   set locList  ""
   set loc 0 

   foreach s $sList \
      {
       lappend locList $loc
       incr loc 
      }

   set locList [lsort -command gScoreSort $locList] 
   set loc 0

   foreach s $sList \
      {
       set rank [expr 1 + [lsearch -exact $locList $loc]]
       lappend rankList $rank
       incr loc
      }

   return $rankList
}

proc saveJoinScore {} \
{
   global objInfo specInfo stripInfo joinInfo

   set obj db

   set expTypes [lrange $specInfo(expTypes) 1 end]

   puts ""
   puts "Finding Strip Sequential Connectivity Scores."

   set error [catch {set outUnit [open $objInfo($obj,joinName) w]}]

   if {$error} \
      {
       puts stderr \
        "Error Opening Sequential Score Table $objInfo($obj,joinName)"

       set outUnit stdout
      }

   puts $outUnit \
    "VARS   INDEX_I INDEX_J PEAK_SCORE STRIP_SCORE SCORE RANK STATUS"

   puts $outUnit \
    "FORMAT %4d     %4d     %7.3f      %7.3f       %7.5f %4d  %s"

   puts $outUnit ""

   set fmt " %4d %4d %8.2f %7.2f %7.5f %4d %s"

   set sMax 0.0

   foreach idI $stripInfo(idList) \
      {
       if {[lsearch -exact $stripInfo(badList) $idI] >= 0} continue

       foreach idJ $stripInfo(idList) \
          {
           if {[lsearch -exact $stripInfo(badList) $idJ] >= 0} continue
           if {$idI == $idJ} continue

           set s \
            [expr $joinInfo($idI,$idJ,pk) + \
                  $objInfo($obj,joinW)*$joinInfo($idI,$idJ,pix)]

           if {$s > $sMax} {set sMax $s}
          }
      }

   if {$sMax == 0.0} {set sMax 1.0}

   foreach idI $stripInfo(idList) \
      {
       if {[lsearch -exact $stripInfo(badList) $idI] >= 0} continue
       set sList ""

       foreach idJ $stripInfo(idList) \
          {
           if {[lsearch -exact $stripInfo(badList) $idJ] >= 0} continue
           if {$idI == $idJ} continue

           set s \
            [expr $joinInfo($idI,$idJ,pk) + \
                  $objInfo($obj,joinW)*$joinInfo($idI,$idJ,pix)]

           set s [expr $s/$sMax]

           lappend sList $s
          }

       set rankList [getScoreRanks $sList]
       set i 0

       foreach idJ $stripInfo(idList) \
          {
           if {[lsearch -exact $stripInfo(badList) $idJ] >= 0} continue
           if {$idI == $idJ} continue

           set s \
            [expr $joinInfo($idI,$idJ,pk) + \
                  $objInfo($obj,joinW)*$joinInfo($idI,$idJ,pix)]

           set s    [expr $s/$sMax]
           set rank [lindex $rankList $i]
         
           if {$rank == 1} \
              {
               set status Best
              } \
           else \
              {
               set status Other 
              }

           puts $outUnit \
            [format $fmt $idI $idJ \
              $joinInfo($idI,$idJ,pk) $joinInfo($idI,$idJ,pix) $s $rank $status] 
           incr i
          }
      }

   if {[strcmp $outUnit stdout]} {close $outUnit}
}

proc getSeqInfo {} \
{
   global objInfo seqInfo

   set obj db

   if {![file exists $objInfo($obj,seqName)]} \
      {
       puts stderr "Error Finding Sequence Info Table $objInfo($obj,seqInfo)"
       exit 1
      }

   set sTab [gdbRead table -in $objInfo($obj,seqName)]
   set r1   0
   set rN   0

   set seqList [getSeqList $sTab r1 rN]

   set seqInfo(seqList) $seqList
   set seqInfo(r1)      $r1
   set seqInfo(rN)      $rN

   puts ""
   puts "Residues $r1 to $rN:" 
   set i 1

   foreach res $seqList \
      {
       if {$i == 1} {puts -nonewline " "}
 
       puts -nonewline $res

       if {$i == 50} \
          {
           set i 0
           puts ""
          } \
       elseif {!($i % 10)} \
          {
           puts -nonewline " "
          }

       incr i
      }

   if {$i != 1} {puts ""} 

   for {set res $r1} {$res <= $rN} {incr res} \
      {
       set seqInfo($res,resName)  [lindex $seqList [expr $res - $r1]]
       set seqInfo($res,resName3) [getResName3 $seqInfo($res,resName)]

       set seqInfo($res,phi)    0.0
       set seqInfo($res,psi)    0.0
       set seqInfo($res,dphi) 180.0
       set seqInfo($res,dpsi) 180.0
      }

   gdbFree $sTab
}

proc getSpecInfo {} \
{
   global objInfo expInfo specInfo

   set obj db

   if {![file exists $objInfo($obj,inName)]} \
      {
       puts stderr "Error Opening Spectrum Info Table $objInfo($obj,inName)"
       exit 1
      }

   set specInfo(expTypes) ""
   set id 0

   set sTab  [gdbRead table -in $objInfo($obj,inName)]
   set eList [gdbGetInfo $sTab -parms entries]
 
   foreach entry $eList \
      {
       set expType [gdbGet $entry EXP_TYPE] 

       if {![info exists expInfo($expType,id)]} \
          {
           puts stderr "Error: Unknown Experiment Type $expType."
           exit 1
          }

       incr id

       lappend specInfo(expTypes) $expType

       set d [addBaseDir $objInfo($obj,baseDir) [gdbGet $entry SPEC_DIR]]
 
       set specInfo($expType,specName) [addBaseDir $d [gdbGet $entry SPEC_NAME]]
       set specInfo($expType,pkName)   [addBaseDir $d [gdbGet $entry TAB_NAME]]

       foreach dim {x y z} \
          {
           set axis [string toupper $dim]

           set specInfo($expType,$dim,tol) [gdbGet $entry ${axis}_TOL]
           set specInfo($expType,$dim,off) [gdbGet $entry ${axis}_OFF]

           if {$specInfo($expType,$dim,tol) <= 0.0} \
              {
               set specInfo($expType,$dim,tol) 1.0 
              } 
          }

       set specInfo($expType,hi) [gdbGet $entry HEIGHT]
       set specInfo($expType,id) $id
      }

   gdbFree $sTab
}

proc getSpecParms {} \
{
   global objInfo expInfo specInfo

   set obj db

   puts ""

   foreach expType $specInfo(expTypes) \
      {
      set error [rdFDATA $specInfo($expType,specName) fdata]

       if {$error} \
          {
           puts stderr "Error Reading Header in $specInfo($expType,specName)"
           exit 1
          }

       set specInfo($expType,dimCount) [getParmI $fdata FDDIMCOUNT NULL_DIM]

       set specInfo($expType,x,name)  [getDirectDim   $fdata]
       set specInfo($expType,y,name)  [get15NDim      $fdata]
       set specInfo($expType,z,name)  [getIndirectDim $fdata]
       set specInfo($expType,a,name)  None

       for {set i 0} {$i < $specInfo($expType,dimCount)} {incr i} \
          {
           set dim [lindex {x y z a} $i] 

           set dimName  [getDimKeyword $specInfo($expType,$dim,name) $fdata]

           set specInfo($expType,$dim,dimKeyword) $dimName

           set swHz [getParm $fdata NDSW  $dimName]
           set obs  [getParm $fdata NDOBS $dimName]
           set p1   [getParm $fdata NDP1  $dimName]

           set specInfo($expType,$dim,size)   [getParmI $fdata NDSIZE $dimName]
           set specInfo($expType,$dim,swPPM)  [div0 $swHz $obs]

           set d  [expr 1.0/$specInfo($expType,$dim,size)]
           set a1 [expr 1 + $d]
           set an [expr $specInfo($expType,$dim,size) - $d]

           set specInfo($expType,$dim,firstPPM) \
            [pnt2spec $fdata $dimName $a1 ppm]

           set specInfo($expType,$dim,lastPPM) \
            [pnt2spec $fdata $dimName $an ppm]

           if {$p1 > 170.0 && $p1 < 190.0} \
              {
               set specInfo($expType,$dim,foldSign) -1
              } \
           else \
              {
               set specInfo($expType,$dim,foldSign)  1
              }

           set specInfo($expType,$dim,dimName) $dimName

           set fmt " %-10s %-6s Limits: % 8.3f % 8.3f ppm."

           puts [format $fmt $expType \
                             $specInfo($expType,$dim,name) \
                             $specInfo($expType,$dim,lastPPM) \
                             $specInfo($expType,$dim,firstPPM)]
          }

       sysFree $fdata
      }
}

proc getPeakTabs {} \
{
   global objInfo expInfo specInfo pkInfo stripInfo

   set ref [lindex $specInfo(expTypes) 0]

   set pkInfo(pkCount) 0

   foreach expType $specInfo(expTypes) \
      {
       if {![file exists $specInfo($expType,pkName)]} \
          {
           puts stderr "Error Finding Table $specInfo($expType,pkName)"
           exit 1 
          }

       puts ""

       puts -nonewline "Peak Table [file tail $specInfo($expType,pkName)]"
       flush stdout

       set pkTab [gdbRead table -in $specInfo($expType,pkName)]

       if {![strcmp $ref $expType]} {getKeyData $pkTab kInfo}

       set pkInfo($expType,x,name)  [getHNTabVar       $pkTab]
       set pkInfo($expType,y,name)  [get15NTabVar      $pkTab]
       set pkInfo($expType,z,name)  [getIndirectTabVar $pkTab]
       set pkInfo($expType,a,name)  None

       puts " [gdbGetInfo $pkTab -parms entryCount] Entries:"

       set fmt " %-16s %-6s Limits: %8.3f %8.3f ppm."

       for {set i 0} {$i < $specInfo($expType,dimCount)} {incr i} \
          {
           set dim [lindex {x y z a} $i]

           set firstPPM \
            [getTabStatU getMin $pkTab $pkInfo($expType,$dim,name) 999]

           set lastPPM \
            [getTabStatU getMax $pkTab $pkInfo($expType,$dim,name) 999]

           puts [format $fmt \
                        "$expType $specInfo($expType,$dim,name):" \
                        $pkInfo($expType,$dim,name) $lastPPM $firstPPM]
         }

       set entry [gdbFirst $pkTab]

       set pkInfo($expType,idList) ""

       while {$entry} \
          {
           set pkID [gdbGet $entry INDEX]

           lappend pkInfo($expType,idList) $pkID
 
           set pkInfo($expType,$pkID,hi) [gdbGet $entry HEIGHT]

           for {set i 0} {$i < $specInfo($expType,dimCount)} {incr i} \
               {
                set dim  [lindex {x y z a} $i]
                set axis $pkInfo($expType,$dim,name)

                set pkInfo($expType,$pkID,$dim) [gdbGet $entry $axis]
               }

           set pkInfo($expType,$pkID,type) \
            [concat $expInfo($expType,posTypes) $expInfo($expType,negTypes)]

           set pkInfo($expType,$pkID,x,off) $specInfo($expType,x,off)
           set pkInfo($expType,$pkID,y,off) $specInfo($expType,y,off)
           set pkInfo($expType,$pkID,z,off) $specInfo($expType,z,off)
           set pkInfo($expType,$pkID,used)  ""

           incr pkInfo(pkCount)

           set entry [gdbNext $entry]
          }

       gdbFree $pkTab
      }
}

proc groupPeaksByStrip {} \
{
   global objInfo expInfo specInfo pkInfo stripInfo 

   set obj db

   set expTypes [lrange $specInfo(expTypes) 1 end]
   set ref      [lindex $specInfo(expTypes) 0]

   puts ""
   puts "Grouping Peaks by Strip."

   set stripInfo(idList) $pkInfo($ref,idList)

   foreach expType $specInfo(expTypes) \
      {
       foreach id $stripInfo(idList) \
          {
           set stripInfo($expType,$id,hi) $specInfo($expType,hi)
          }
      }

   foreach id $stripInfo(idList) \
      {
       set stripInfo($ref,$id,pkList) $id

       set xPPM [getShift $ref $id x]
       set yPPM [getShift $ref $id y]
       set nn   0
 
       if {$objInfo($obj,verbose)} {puts "Strip $id:"}

       foreach expType $expTypes \
          {
           set n [getPeaksInStrip $ref $id $xPPM $yPPM $expType]

           if {$objInfo($obj,verbose)} \
              {
               if {$n == 1} \
                  {
                   puts  [format " %-14s %3d Peak."  $expType: $n]
                  } \
               else \
                  {
                   puts  [format " %-14s %3d Peaks." $expType: $n]
                  }
              }

           incr nn $n
          }

       if {$objInfo($obj,verbose)} \
          {
           puts [format " %-14s %3d Peaks." Total: $nn]
          }
      }

   if {$objInfo($obj,verbose)} {puts ""}
 
   set nn 0

   foreach expType $expTypes \
      {
       foreach pkID $pkInfo($expType,idList) \
          {
           if {[llength $pkInfo($expType,$pkID,used)]} {incr nn}
          }
      }

   puts " Total Peaks: $pkInfo(pkCount) Peaks Used: $nn"
}

proc getPeaksInStrip { ref id xOrig yOrig expType } \
{
   global objInfo expInfo specInfo pkInfo stripInfo

   set obj db

   set stripInfo($expType,$id,foldCount) 0 
   set stripInfo($expType,$id,foldSign)  1

   set stripInfo($expType,$id,allPeaks)  ""
   set stripInfo($expType,$id,nearPeaks) ""
   set stripInfo($expType,$id,farPeaks)  ""
   set stripInfo($expType,$id,pkList)    ""

   if {$xOrig > 999 || $yOrig > 999} {return 0}

   set xPPM $xOrig
   set yPPM $yOrig
   set yAlt None 

   set yTol [expr 2.0*$specInfo($expType,y,tol)]

   set foldCount 0
   set foldSign  1

   while {$yPPM > $specInfo($expType,y,firstPPM)} \
      {   
       set yPPM [expr $yPPM - $specInfo($expType,y,swPPM)]

       incr foldCount -1
       set foldSign  [expr $foldSign*$specInfo($expType,y,foldSign)]
      }

   while {$yPPM < $specInfo($expType,y,lastPPM)} \
      {
       set yPPM [expr $yPPM + $specInfo($expType,y,swPPM)]

       incr foldCount 1
       set foldSign   [expr $foldSign*$specInfo($expType,y,foldSign)]
      }

   if {$objInfo($obj,verbose) && $foldCount} \
      {
       set fmt "Folding(%+d): %-10s Before: %7.3f %8.3f After: %7.3f %8.3f ppm."
       puts [format $fmt $foldCount $expType $xOrig $yOrig $xPPM $yPPM]
      }

   if {abs($yPPM - $specInfo($expType,y,firstPPM)) < $yTol} \
      {
       if {$objInfo($obj,verbose)} \
          {
           set fmt \
            "Edge:        %-10s Peak:   %7.3f %8.3f Limit: %7.3f %8.3f ppm."

           puts [format $fmt $expType $xPPM $yPPM \
                        $specInfo($expType,y,firstPPM) \
                        $specInfo($expType,y,lastPPM)]
          }
 
       set yAlt [expr $yPPM - $specInfo($expType,y,swPPM)]
      }

   if {abs($yPPM - $specInfo($expType,y,lastPPM)) < $yTol} \
      {
       if {$objInfo($obj,verbose)} \
          {
           set fmt \
            "Edge:        %-10s Peak:   %7.3f %8.3f Limit: %7.3f %8.3f ppm."

           puts [format $fmt $expType $xPPM $yPPM \
                        $specInfo($expType,y,firstPPM) \
                        $specInfo($expType,y,lastPPM)]
          } 

       set yAlt [expr $yPPM + $specInfo($expType,y,swPPM)]
      }

   if {![strcasecmp $yAlt None]} \
      {
       getPeaksNearXY $expType $id $xPPM $yPPM
      } \
   else \
      {
       getPeaksNearXY $expType $id $xPPM $yPPM
       getPeaksNearXY $expType $id $xPPM $yAlt
      }

   set stripInfo($expType,$id,foldCount) $foldCount
   set stripInfo($expType,$id,foldSign)  $foldSign

   return [llength $stripInfo($expType,$id,allPeaks)]
}

proc adjustGroupAmbig {} \
{
   global specInfo expInfo pkInfo stripInfo

   set expTypes [lrange $specInfo(expTypes) 1 end]
   set ref      [lindex $specInfo(expTypes) 0]
   set nn 0

   puts ""
   puts "Adjusting Ambiguities Between Strips."

   foreach expType $expTypes \
      {
       foreach pkID $pkInfo($expType,idList) \
          {
           set n [llength $pkInfo($expType,$pkID,used)]

           if {$n < 2} continue
       
           incr nn
   
           set xTol $specInfo($expType,x,tol)
           set yTol $specInfo($expType,y,tol)

           set bestD  1.0e+16
           set bestID 0

           set x [getShift $expType $pkID x]
           set y [getShift $expType $pkID y]

           if {$x > 999 || $y > 999} continue

           foreach id $pkInfo($expType,$pkID,used) \
              {
               set xPPM [getShift $ref $id x]  
               set yPPM [getShift $ref $id y]  

               if {$xPPM > 999 || $yPPM > 999} continue

               set dx [expr ($x - $xPPM)/$xTol] 
               set dy [expr ($y - $yPPM)/$yTol] 
               set d  [expr $dx*$dx + $dy*$dy] 

               if {$d < $bestD} \
                  {
                   set bestD  $d
                   set bestID $id
                  }
              }

           if {!$bestID} continue

           foreach id $pkInfo($expType,$pkID,used) \
              {
               if {$id == $bestID} continue

               removeItem stripInfo($expType,$id,nearPeaks) $id
               removeItem stripInfo($expType,$id,farPeaks)  $id
               removeItem stripInfo($expType,$id,pkList)    $id
              }

           set pkInfo($expType,$pkID,used) $bestID
          }
      }

   puts "Adjusted $nn Ambiguities."
}

proc getPeaksNearXY { expType id xPPM yPPM } \
{
   global specInfo pkInfo stripInfo

   set xTol $specInfo($expType,x,tol)
   set yTol $specInfo($expType,y,tol)

   foreach pkID $pkInfo($expType,idList) \
      {
       set x [getShift $expType $pkID x]
       set y [getShift $expType $pkID y]

       if {$x > 999 || $y > 999} continue

       if {abs($xPPM - $x) < $xTol && abs($yPPM - $y) < $yTol} \
          {
           lappend stripInfo($expType,$id,allPeaks)  $pkID 
           lappend stripInfo($expType,$id,nearPeaks) $pkID 

           lappend pkInfo($expType,$pkID,used) $id
          } \
       elseif {abs($xPPM - $x) < 2.0*$xTol && abs($yPPM - $y) < 2.0*$yTol} \
          {
           lappend stripInfo($expType,$id,allPeaks)  $pkID 
           lappend stripInfo($expType,$id,farPeaks)  $pkID 

           lappend pkInfo($expType,$pkID,used) $id
          }
      }
}

proc getIdealPeaks {} \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   set expTypes [lrange $specInfo(expTypes) 1 end]

   puts ""
   puts "Setting Initial Peak Types."

   foreach id $stripInfo(idList) \
      {
       if {$objInfo($obj,verbose)} \
          {
           puts ""
           puts "Setting Initial Type Assignments for $id:"
          }

       foreach expType $expTypes \
          {
           set stripInfo($expType,$id,pkList) ""
 
           if {!$expInfo($expType,negCount)} \
              {
               set pList1 [getHighestPeaks $expType $id nearPeaks]
               set pList2 [getHighestPeaks $expType $id farPeaks]

               set nList1 ""
               set nList2 ""
              } \
           else \
              {
               if {$stripInfo($expType,$id,foldSign) < 0} \
                  {
                   set nList1 [getHighestPosPeaks $expType $id nearPeaks]
                   set nList2 [getHighestPosPeaks $expType $id farPeaks]

                   set pList1 [getHighestNegPeaks $expType $id nearPeaks]
                   set pList2 [getHighestNegPeaks $expType $id farPeaks]
                  } \
               else \
                  {
                   set pList1 [getHighestPosPeaks $expType $id nearPeaks]
                   set pList2 [getHighestPosPeaks $expType $id farPeaks]

                   set nList1 [getHighestNegPeaks $expType $id nearPeaks]
                   set nList2 [getHighestNegPeaks $expType $id farPeaks]
                  }
              }

           set pLoc [expr $expInfo($expType,posCount) - 1]
           set nLoc [expr $expInfo($expType,posCount) - 1]

           set posList [lrange $pList1 0 $pLoc]
           set negList [lrange $nList1 0 $nLoc]

           set pN [llength $posList]
           set nN [llength $negList]

           if {$pN < $expInfo($expType,posCount)} \
              {
               set pLoc    [expr $expInfo($expType,posCount) - $pN - 1]
               set posList [concat $posList [lrange $pList2 0 $pLoc]]
              }

           if {$nN < $expInfo($expType,negCount)} \
              {
               set nLoc    [expr $expInfo($expType,negCount) - $nN - 1]
               set negList [concat $negList [lrange $nList2 0 $nLoc]]
              }

           foreach pkID $posList \
              {
               set pkInfo($expType,$pkID,type) $expInfo($expType,posTypes)
               lappend stripInfo($expType,$id,pkList) $pkID
              }

           foreach pkID $negList \
              {
               set pkInfo($expType,$pkID,type) $expInfo($expType,negTypes)
               lappend stripInfo($expType,$id,pkList) $pkID
              }

           foreach pkID $stripInfo($expType,$id,pkList) \
              {
               set nA [applyShiftRules $id $expType $pkID]
              }

           foreach pkIDI $stripInfo($expType,$id,pkList) \
              {
               set assIJ   $pkInfo($expType,$pkIDI,type)
               set assInfo [split $assIJ "|"]
               set nA      [llength $assInfo]

               if {$nA != 2} continue

               if {![strcasecmp $expInfo($expType,hiInfo,$assIJ) none]} \
                  {
                   continue 
                  }

               set pkIDJ [findAssMatchIntra $id $expType $pkIDI]
               if {$pkIDJ < 0} continue

               set cmp $expInfo($expType,hiInfo,$assIJ)
               set hiI $pkInfo($expType,$pkIDI,hi) 
               set hiJ $pkInfo($expType,$pkIDJ,hi) 

               set newAssI [lindex $assInfo 0]
               set newAssJ [lindex $assInfo 1]

               if {![strcasecmp $cmp gt]} \
                  {
                   if {abs($hiI) < $objInfo($obj,hiRatio)*abs($hiJ)} continue

                   if {[assContradict $id $expType $pkIDI $newAssI]} continue
                   if {[assContradict $id $expType $pkIDJ $newAssJ]} continue

                   set pkInfo($expType,$pkIDI,type) $newAssI 
                   set pkInfo($expType,$pkIDJ,type) $newAssJ
                  } \
               else \
                  {
                   if {abs($hiI) > $objInfo($obj,hiRatio)*abs($hiJ)} continue

                   if {[assContradict $id $expType $pkIDI $newAssI]} continue
                   if {[assContradict $id $expType $pkIDJ $newAssJ]} continue

                   set pkInfo($expType,$pkIDI,type) $newAssI 
                   set pkInfo($expType,$pkIDJ,type) $newAssJ
                  }

               if {$objInfo($obj,verbose)} \
                  {
                   set fmt " Applied %s Height Rule: %s %s %s" 
                   puts [format $fmt $cmp $id $expType $assIJ]
                  }
              }

           if {$objInfo($obj,verbose)} \
              {
               set fmt "  %-12s Strip %3d Peak %4d % .2e %8.3f type: %s"

               foreach pkID $stripInfo($expType,$id,pkList) \
                  {
                   puts \
                     [format $fmt $expType $id $pkID \
                             $pkInfo($expType,$pkID,hi) \
                             $pkInfo($expType,$pkID,z)  \
                             $pkInfo($expType,$pkID,type)]
                  }
              }
          }

       set nR [removeAssMatches $id]
      }
}

proc findAssMatches { id expTypeI pkIDJ } \
{
    global stripInfo pkInfo specInfo

    set expTypes [lrange $specInfo(expTypes) 1 end]
    set assStrI  [lsort [split $pkInfo($expTypeI,$pkIDI,type) "|"]]
    set pkListJ  ""

    foreach expTypeJ $expTypes \
       {
        foreach pkIDJ $stripInfo($expTypeJ,$id,pkList) \
           {
            if {![strcasecmp $expTypeI,$pkIDI $expTypeJ,$pkIDJ]} continue

            set assStrJ [lsort [split $pkInfo($expTypeJ,$pkIDJ,type) "|"]]

            if {![strcasecmp $assStrI $assStrJ]} \
               {
                lappend pkListJ $expTypeJ,$pkIDJ
               }
           }
       }

    return $pkListJ 
}

proc findAssMatchIntra { id expType pkIDI } \
{
    global stripInfo pkInfo

    set assStrI [lsort [split $pkInfo($expType,$pkIDI,type) "|"]]

    foreach pkIDJ $stripInfo($expType,$id,pkList) \
       {
        if {$pkIDI == $pkIDJ} continue

        set assStrJ [lsort [split $pkInfo($expType,$pkIDJ,type) "|"]]

        if {![strcasecmp $assStrI $assStrJ]} {return $pkIDJ}
       }

    return -1
}

proc applyShiftRules { id expType pkID } \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   set assList [split $pkInfo($expType,$pkID,type) "|"]
   set newList ""

   set assOrig $pkInfo($expType,$pkID,type)
   set s       $pkInfo($expType,$pkID,z)

   set adjCount 0


   foreach ass $assList \
      {
       set a [string tolower [lindex [split $ass "()"] 0]]
       set n 0

       switch -exact $a \
          {
           ha {
               if {$s < 3.0 || $s > 5.9} {incr n}
              }

           ca {
               if {$s < 40.0} {incr n}
              }
          }

       if {$n} \
          {
           incr adjCount
          } \
       else \
          {
           lappend newList $ass
          }
      }

   if {[llength $newList]} \
      {
       set pkInfo($expType,$pkID,type) [join $newList "|"]
      } \
   else \
      {
       set pkInfo($expType,$pkID,type) None 
      }

   return $adjCount
}

proc getShiftPerm {} \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   puts ""
   puts "Refining Tentative Chemical Shift Type Assignments."

   foreach id $stripInfo(idList) \
      {
       if {$objInfo($obj,verbose)} \
          {
           puts "Reducing shifts assignments for $id:"
          }

       foreach expTypes $specInfo(related) \
          {
           set pkList ""

           foreach expType $expTypes \
              {
               foreach pkID $stripInfo($expType,$id,pkList) \
                  {
                   lappend pkList $expType,$pkID

                   set \
                    pkInfo($expType,$pkID,origType) $pkInfo($expType,$pkID,type)
                  } 
              }

           set pkCount [llength $pkList]
           set n 0

           for {set i 1} {$i < $pkList} {incr i} \
               {
                set n [collapseShifts $id $pkList]
                if {$n < 1} break
               }

           if {$n >= 0} continue

           if {$objInfo($obj,verbose)} \
              {
               puts "Reverting to previous assignments."
              }

           foreach pk $pkList {set pkInfo($pk,type) $pkInfo($pk,origType)} 
          }
      }
}

proc assContradict { id expTypeI pkIDI newAss } \
{
    global objInfo expInfo specInfo seqInfo pkInfo stripInfo

    set obj db

    set sI [getShift $expTypeI $pkIDI z]
    if {$sI > 999} {return 0}

    set expTypes [lrange $specInfo(expTypes) 1 end]
    set tolI $specInfo($expTypeI,z,tol)

    foreach expTypeJ $expTypes \
       {
        foreach pkIDJ $stripInfo($expTypeJ,$id,pkList) \
           {
            set assStrJ [lsort [split $pkInfo($expTypeJ,$pkIDJ,type) "|"]]

            if {[strcasecmp $newAss $assStrJ]} continue

            set sJ [getShift $expTypeJ $pkIDJ z]
            if {$sJ > 999} {return 0}

            set tolJ $specInfo($expTypeJ,z,tol)
            set tol  [getMax "$tolI $$tolJ"]

            if {abs($sI - $sJ) > $objInfo($obj,pThresh)*$tol} \
               {
                if {$objInfo($obj,verbose)} \
                   {
                    puts "Avoiding Contradiction:"
                    puts " Proposed: $expTypeI $pkIDI $newAss  $sI"
                    puts " Existing: $expTypeJ $pkIDJ $assStrJ $sJ"
                   }

                return 1
               }
           }
       }

   return 0
}

proc collapseShifts { id pkList } \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   set nBad 0
   set n    0

   foreach pkI $pkList \
      {
       set assStrI  $pkInfo($pkI,type)
       set assListI [split $assStrI "|"]
       set nI       [llength $assListI]

       if {![strcasecmp $assStrI None]} continue

       set listI    [split $pkI ","]
       set expTypeI [lindex $listI 0]
       set pkIDI    [lindex $listI 1]
       set tolI     $specInfo($expTypeI,z,tol)
       set sI       [getShift $expTypeI $pkIDI z]

       if {$sI > 999 || [string first (*) $assStrI] >= 0} continue

       foreach pkJ $pkList \
          {
           set assStrJ  $pkInfo($pkJ,type)
           set assListJ [split $assStrJ "|"]
           set nJ       [llength $assListJ]

           if {![strcasecmp $pkI $pkJ]} continue
           if {![strcasecmp $assStrJ None]} continue

           set assListIJ [listIntersect $assListI $assListJ]
           set nIJ       [llength $assListIJ]

           if {!$nIJ} continue
 
           set listJ    [split $pkJ ","]
           set expTypeJ [lindex $listJ 0]
           set pkIDJ    [lindex $listJ 1]
           set tolJ     $specInfo($expTypeJ,z,tol)
           set sJ       [getShift $expTypeJ $pkIDJ z]

           if {$sJ > 999 || [string first (*) $assStrJ] >= 0} continue

           if {$tolI > $tolJ} \
              {
               set tol $tolI
              } \
           else \
              {
               set tol $tolJ
              }

           if {$nI == 1 && $nJ == 1 && ![strcasecmp $assListI $assListJ]} \
              {
               if {[lsearch -exact $objInfo($obj,multiAss) $assListI] >= 0} \
                  {
                   continue
                  }

               if {[lsearch -exact $objInfo($obj,multiAss) $assListJ] >= 0} \
                  {
                   continue
                  }

               if {abs($sI - $sJ) > $objInfo($obj,pThresh)*$tol} \
                  {
                   if {$objInfo($obj,verbose)} \
                      {
                       set fmt " Contradiction: %s %s %.3f %s and %s %s %.3f %s"

                       puts [format $fmt $expTypeI $pkIDI $sI $assListI \
                                         $expTypeJ $pkIDJ $sJ $assListJ]

                       incr nBad
                      }
                  }

               continue
              }

           set newAssI $assListI
           set newAssJ $assListJ

           if {abs($sI - $sJ) > $objInfo($obj,pThresh)*$tol} \
              {
               if {$objInfo($obj,verbose)} \
                  {
                   set fmt " Removing %.3f %s from %.3f %s"
                   puts [format $fmt $sJ $assListJ $sI $assListI]
                  }

               set newAssI [listDisjoint $assListI $assListJ]
               set newAssJ [listDisjoint $assListJ $assListI]
              } \
           elseif {abs($sI - $sJ) < $tol} \
              {
               if {$objInfo($obj,verbose)} \
                  {
                   set fmt " Joining %.3f %s with %.3f %s"
                   puts [format $fmt $sI $assListI $sJ $assListJ]
                  }

               set newAssI [listIntersect $assListI $assListJ]
               set newAssJ [listIntersect $assListI $assListJ]
              }

           if {![sameList $assListI $newAssI]} \
              {
               set nI [llength $newAssI]
               if {!$nI} continue

               if {$objInfo($obj,verbose)} \
                  {
                   set fmt " Old: %s %s %.3f %s New: %s"
                   puts [format $fmt $expTypeI $pkIDI $sI $assListI $newAssI]
                  }

               set pkInfo($pkI,type) [join $newAssI "|"]
               set assListI $newAssI
               incr n
              }

           if {![sameList $assListJ $newAssJ]} \
              {
               set nJ [llength $newAssJ]
               if {!$nJ} continue

               if {$objInfo($obj,verbose)} \
                  {
                   set fmt " Old: %s %s %.3f %s New: %s"
                   puts [format $fmt $expTypeJ $pkIDJ $sJ $assListJ $newAssJ]
                  }

               set pkInfo($pkJ,type) [join $newAssJ "|"]
               incr n
              }
          }
      }

   if {$nBad} {set n -1}

   return $n
}

proc getAssPerm {} \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   puts ""
   puts "Finding Tentative Chemical Shift Type Assignments."

   foreach id $stripInfo(idList) \
      {
       puts "Scoring permutations for $id:"

       foreach expTypes $specInfo(related) \
          {
           set assList ""

           foreach expType $expTypes \
              {
               foreach pkID $stripInfo($expType,$id,pkList) \
                  {
                   if {![strcasecmp $pkInfo($expType,$pkID,type) None]} continue

                   set assStr ""

                   foreach ass [split $pkInfo($expType,$pkID,type) "|"] \
                      {
                       if {[string first "*" $ass] >= 0} continue

                       lappend assStr \
                        $expType@$pkID@$ass=$pkInfo($expType,$pkID,z)
                      }               

                   if {![llength $assStr]} continue

                   lappend assList [join $assStr "|"]
                  }
              }

           puts " $expTypes:"

           if {![llength $assList]} continue

           set assStr   [join $assList ,]
           set permInfo [permList $assStr]

           catch "unset sArray"
           set n 0

           foreach p $permInfo \
              {
               set sItem [scoreAssPerm $p]
               set score [lindex $sItem 0]

               if {$score < $objInfo($obj,pThresh)} \
                  {
                   set sArray($n) $sItem
                   incr n
                  }
              }
           
           setAssPerm sArray $id $n 
          }
      }
}

proc removeAssMatches { id } \
{
   global objInfo expInfo specInfo seqInfo pkInfo stripInfo

   set obj db

   set nn 0

   foreach expTypes $specInfo(related) \
      {
       set ii 0

       foreach expTypeI $expTypes \
          {
           incr ii
           set iii 0

           set tolI $specInfo($expTypeI,z,tol)

           foreach pkIDI $stripInfo($expTypeI,$id,pkList) \
              {
               incr iii

               set assI $pkInfo($expTypeI,$pkIDI,type)

               if {![strcasecmp $assI None]} continue
               if {[string first "|" $assI] >= 0} continue

               set pI [getShift $expTypeI $pkIDI z]
               set jj 0

               foreach expTypeJ $expTypes \
                  {
                   incr jj
                   set jjj 0

                   set tolJ $specInfo($expTypeJ,z,tol)
                   set tol  [getMax "$tolI $tolJ"]

                   foreach pkIDJ $stripInfo($expTypeJ,$id,pkList) \
                      {
                       incr jjj
                   
                       if {$ii == $jj && $iii == $jjj} continue
    
                       set assJ $pkInfo($expTypeJ,$pkIDJ,type)

                       set assListJ [split $assJ "|"]
                       set nJ       [llength $assListJ]

                       if {$nJ < 2} continue
                       if {[lsearch -exact $assListJ $assI] < 0} continue

                       set pJ [getShift $expTypeJ $pkIDJ z]
 
                       if {$pI > 999 || $pJ > 999} continue
 
                       if {abs($pI - $pJ) <= $objInfo($obj,pThresh)*$tol} \
                          {
                           continue 
                          }

                       removeItem assListJ $assI 
                       incr nn

                       set pkInfo($expTypeJ,$pkIDJ,type) [join $assListJ "|"]

                       if {$objInfo($obj,verbose)} \
                          {
                           set fmt "%s %s %s"
                           set str "Removed $assI from $id:"
                           puts [format $fmt $str $expTypeJ $assJ]
                          }
                      }
                  }
              }
          }
      }

   return $nn
}

proc setAssPerm { sArrayPtr id nS } \
{
   global pkInfo

   upvar $sArrayPtr sArray

   set idList ""

   for {set loc 0} {$loc < $nS} {incr loc} \
      {
       set score [lindex $sArray($loc) 0]
       set pList [lrange $sArray($loc) 1 end]

       foreach pItem $pList \
          {
           set ass   [lindex $pItem 0]
           set sList [lindex $pItem 1]
           set eList [lindex $pItem 2]
           set iList [lindex $pItem 3]

           set n [llength $sList]

           for {set i 0} {$i < $n} {incr i} \
              {
               set s       [lindex $sList $i]
               set expType [lindex $eList $i]
               set pkID    [lindex $iList $i]

               set idStr $expType,$pkID
    
               if {[lsearch -exact $idList $idStr] < 0} {lappend idList $idStr}

               lappend aInfo($idStr) $ass
               lappend sInfo($idStr) $s
              }
          }
      }

   foreach idStr $idList \
      {
       set uList   [listUniq $aInfo($idStr)]
       set item    [split $idStr ","]
       set expType [lindex $item 0]
       set pkID    [lindex $item 1]

       if {[llength $uList] > 1} \
          {
           puts " Ambiguous Permutation for $expType:"
           puts " $aInfo($idStr)"
           puts " $sInfo($idStr)"
           continue
          }

       set ass [lindex $aInfo($idStr) 0]

       set pkInfo($expType,$pkID,type) $ass

       set fmt " %-12s %3d % .2e %8.3f type: %s"
   
       puts [format $fmt $expType $id \
                         $pkInfo($expType,$pkID,hi) \
                         $pkInfo($expType,$pkID,z)  \
                         $pkInfo($expType,$pkID,type)]
      }
}

proc assParenConvert { a } \
{
   set n [string length $a]
   set s ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $a $i]

       if {![strcmp $c "("]} \
          {
           append s "<"
          } \
       elseif {![strcmp $c ")"]} \
          {
           append s ">"
          } \
       else \
          {
           append s $c
          }
      }

   return $s
}

proc assParenRevert { a } \
{
   set n [string length $a]
   set s ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set c [string index $a $i]

       if {![strcmp $c "<"]} \
          {
           append s "("
          } \
       elseif {![strcmp $c ">"]} \
          {
           append s ")"
          } \
       else \
          {
           append s $c
          }
      }

   return $s
}

proc scoreAssPerm { pp } \
{
   global specInfo

   set pList [split $pp ","]

   set assList ""
   set score   0.0

   foreach pInfo $pList \
      {
       set p [split $pInfo "@="]

       set expType [lindex $p 0]
       set pkID    [lindex $p 1]
       set ass     [assParenConvert [lindex $p 2]]
       set shift   [lindex $p 3]

       set tol     $specInfo($expType,z,tol)
       set off     $specInfo($expType,z,off)
 
       if {[lsearch -exact $assList $ass] < 0} {lappend assList $ass}

       lappend sInfo($ass) [expr $shift + $off]
       lappend aInfo($ass) $pkID
       lappend tInfo($ass) $tol
       lappend eInfo($ass) $expType
      }

   set n 0
 
   foreach ass $assList \
      {
       set minVal [getMin $sInfo($ass)]
       set maxVal [getMax $sInfo($ass)]
       set tol    [getMax $tInfo($ass)]
       set s      [expr ($maxVal - $minVal)/$tol]
       set score  [expr $score + $s*$s]

       if {[llength $sInfo($ass)]} {incr n}
      }

   if {$n} {set score [expr sqrt($score/$n)]}

   set sItem $score

   foreach ass $assList \
      {
       set assStr [assParenRevert $ass]
   
       lappend assStr $sInfo($ass)
       lappend assStr $eInfo($ass) 
       lappend assStr $aInfo($ass) 

       lappend sItem $assStr
      }

   return $sItem
}

proc getHighestPosPeaks { expType id listType } \
{
   global stripInfo pkInfo curType

   set pkList ""

   foreach pkID $stripInfo($expType,$id,$listType) \
      {
       if {$pkInfo($expType,$pkID,hi) > 0.0} {lappend pkList $pkID}
      }

   set curType $expType
   set pkList  [lsort -command pkSort $pkList]

   return $pkList
}

proc getHighestNegPeaks { expType id listType } \
{
   global stripInfo pkInfo curType

   set pkList ""

   foreach pkID $stripInfo($expType,$id,$listType) \
      {
       if {$pkInfo($expType,$pkID,hi) < 0.0} {lappend pkList $pkID}
      }

   set curType $expType
   set pkList  [lsort -command pkSort $pkList]

   return $pkList
}

proc getHighestPeaks { expType id listType } \
{
   global stripInfo pkInfo curType 

   set curType $expType
   set pkList  [lsort -command pkSort $stripInfo($expType,$id,$listType)]

   return $pkList
}

proc getAssStr { expType pkID dim } \
{
   global pkInfo expInfo 

   if {![strcasecmp $dim x] || ![strcasecmp $dim y]} \
      {
       set ass [string toupper $expInfo($expType,type,$dim)]
       return $ass
      }

   set ass [string toupper $pkInfo($expType,$pkID,type)]

   if {[string first "|" $ass] >= 0} {return $ass}

   set loc (I)

   set j [string last $loc $ass]
   set n [string length $ass]

   if {$j > 0 && $j == $n - [string length $loc]} \
      {
       set ass [string range $ass 0 [expr $j - 1]]
      }

   return $ass
}

proc getShift { expID pkID axis } \
{
   global pkInfo

   set off $pkInfo($expID,$pkID,$axis,off)
   set pos $pkInfo($expID,$pkID,$axis)

   set s [addPkOff $off $pos]

   return $s
}

proc addPkOff { off pos } \
{
   if {$off > 999 || $pos > 999} {return 99999.0}

   return [expr $off + $pos]
}

proc pkSort { pkID1 pkID2 } \
{
   global pkInfo curType

   set d \
    [expr abs($pkInfo($curType,$pkID1,hi)) - abs($pkInfo($curType,$pkID2,hi))]

   if {$d < 0.0} {return  1}
   if {$d > 0.0} {return -1}

   return 0
}
 
proc addBaseDir { baseDir fileName } \
{
   set c [string index $fileName 0]

   if {![strcmp $c "/"]} {return $fileName}

   set d [string trimright $baseDir "/"]
 
   return $d/$fileName
}

proc removeItem { iListPtr item } \
{
   upvar $iListPtr iList

   set newList ""

   foreach i $iList \
      {
       if {[strcmp $i $item]} {lappend newList $i} 
      }

   set iList $newList

   return $newList 
}

proc div0 { a b } \
{
   if {$b == 0.0} {return 0.0}

   return [expr $a/$b] 
}

#
#----------------------------------------------------------------------------

set glbArrays \
 {expInfo specInfo seqInfo pkInfo stripInfo joinInfo}

set obj db

initDBVars
getDBArgs
getExpTypes
getExpWeights
getSpecInfo
getSpecParms
getSpecLimits
getPeakTabs
getSeqInfo

groupPeaksByStrip
adjustGroupAmbig
getIdealPeaks

initScoreInfo

getAssPerm
getShiftPerm
saveShifts

getPeakJoinScore

getSpecStrips
getStripJoinScore

saveJoinScore

storeArrayDB $objInfo($obj,dbDir) $glbArrays

exit
