#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
#-----------------------------------------------------

set inName   None
set outName  stdout
set fontName Courier
set orient   landscape
set tray     1
set paper    B
set lines    66

set step   0.15
set top    0.25
set left   0.25
set cx     0.10
set cy     0.14

#-----------------------------------------------------

if {[flagLoc $ARGV -help]} \
   {
    puts "Print a text file on large paper, via PostScript."
    puts " -in    inName   \[None\]     Input Text File."
    puts " -font  fontName \[Courier\]  PostScript Font Name."
    puts " -cx    cWide    \[0.10\]     Font Width, Inches."
    puts " -cy    cHigh    \[0.14\]     Font Height, Inches."
    puts " -paper pType    \[B\]        Paper Type."
    puts " -tray  tray     \[1\]        Paper Tray Number."
    puts " -top   tMargin  \[0.25\]     Top Margin, Inches."
    puts " -left  lMargin  \[0.25\]     Left Margin, Inches."
    puts " -step  lSpace   \[0.15\]     Line Spacing, Inches."
    puts " -lines lCount   \[66\]       Line Count."
    puts " -landscape                 Landscape Mode (Default)."
    puts " -portrait                  Portrait Mode."
    exit
   }

getArgD $ARGV -in    inName
getArgD $ARGV -font  fontName
getArgD $ARGV -tray  tray
getArgD $ARGV -paper B

getArgD $ARGV -top   top
getArgD $ARGV -left  left
getArgD $ARGV -step  step

getArgD $ARGV -cx    cx
getArgD $ARGV -cy    cy

if {[flagLoc $ARGV -landscape]} {set orient landscape}
if {[flagLoc $ARGV -portrait]}  {set orient portrait}

set cw [format "%sin" $cx]
set ch [format "%sin" $cy]

#-----------------------------------------------------

if {"$inName" != "None"} \
   {
    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts "Error opening input file $inName"
        exit
       }
   } \
else \
   {
    set inUnit stdin
   }

#-----------------------------------------------------

set x0    $left
set y0    $top
set count 0

while {[gets $inUnit line] >= 0} \
   {
    if {$count == 0} \
       {
        plotOpen -$orient -paper $paper -plotFont $fontName \
                 -tray $tray -plotName $outName -c1Width $cw -c1Height $ch
       }

    set xLoc [format "%sin" $x0]
    set yLoc [format "%sin" $y0]

    drawText -x $xLoc -y $yLoc -text $line

    set y0 [expr $y0 + $step]

    incr count

    if {$count >= $lines} \
       {
        plotClose
        set y0    $top
        set count 0
       }
   }

plotClose
exit
