#!/bin/csh

if ($#argv != 2) then
   echo "Fourier Transform an IR spectral image in the NMRPipe format."
   echo "Usage:"
   echo "  procIR.com rawTemplate ftTemplate"
   echo "Examples:"
   echo "  procIR.com raw/soap%03d.dat ft/soap%03d.dat"
   echo "  procIR.com raw/air%03d.dat  ft/air%03d.dat"
   exit 0
endif

echo "Finding signal maximum in $1 ..."

set loc = (`maxPlane.com $1`)
echo "... maximum at plane $loc."

@ loc = 2 * $loc

echo "Processing $1 ..."

xyz2pipe -in $1 -z                \
| nmrPipe -fn CBF                 \
| nmrPipe -fn ZF -zf 0 -auto      \
| nmrPipe -fn TRI -loc $loc       \
| nmrPipe -fn FT  -real -verb     \
| nmrPipe -fn MC                  \
| pipe2xyz -z -out $2

echo "... created transformed result $2"
