#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: Create 1D projections from 2D data."
    puts " -in       inName   \[test.ft2\]  Name of 2D Input."
    puts " -outDir   outDir   \[.\]         Output Directory."
    puts " -xOutName xOutName \[Auto\]      Name of 1D X-Axis Output."
    puts " -yOutName yOutName \[Auto\]      Name of 1D Y-Axis Output." 
    puts " -skyline                       Skyline Projection (Default)."
    puts " -sum                           Summation projection."
    puts " -abs                           Use Absolute Value."
    puts " -axis                          Auto Names Use Axis Names X Y."
    puts " -noaxis                        Auto Names Use Axis Labels (Default)."
    puts " -verb                          Verbose Mode ON (Default)."
    puts " -noverb                        Verbose Mode OFF."
    puts "Spectral Region (Use Prefix -x -y):"
    puts " -x1  x1 \[0%\]                   X-Axis Region Start."
    puts " -xn  xn \[100%\]                 X-Axis Region End."
    exit 1
   }

set inName      test.ft2
set outDir      .
set xOutName    Auto
set yOutName    Auto

set projMode    proj
set absFlag     0
set axisFlag    0
set verbose     1

set ix1         0%
set ixn         100%

set iy1         0%
set iyn         100%

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

if {[flagLoc $ARGV -axis]}   {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]} {set axisFlag 0}

if {$axisFlag} \
   {
    set xOutName projX.dat
    set yOutName projY.dat
   }

getArgD $ARGV -in  inName
getArgD $ARGV -out outDir

getArgD $ARGV -xOutName xOutName
getArgD $ARGV -yOutName yOutName

getArgD $ARGV -x1 ix1
getArgD $ARGV -xn ixn

getArgD $ARGV -y1 iy1
getArgD $ARGV -yn iyn


if {[flagLoc $ARGV -skyline]} {set projMode proj}
if {[flagLoc $ARGV -sum]}     {set projMode sum}
if {[flagLoc $ARGV -abs]}     {set absFlag  1}

if {$absFlag} \
   {
    set absArg abs
   } \
else \
   {
    set absArg noabs
   }

#
#--------------------------------------------------------------------------

set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "Proj2D Error Finding Input $inName"
    exit 1
   }

#
# Extract header:

set error [rdFDATA $inName fdata]

if {$error} \
   {
    puts stderr "Proj2D Error Reading header from $inName"
    exit 1
   }

#
# Prepare output directory if needed:

if {![strcasecmp $outDir "."] || ![strcasecmp $outDir "./"]} \
   {
    set outPrefix ""
   } \
else \
   {
    if {![file exists $outDir]} {catch "file mkdir $outDir"}

    if {![file isdirectory $outDir]} \
       {
        puts stderr "Proj3D Error Finding or Creating Output Directory $outDir"
        exit 1
       }

    set outPrefix $outDir/
   }


#
# Find axis labels:

set tpFlag [getParmI $fdata FDTRANSPOSED 0]

if {$tpFlag} \
   {
    set xName [getParmStr $fdata NDLABEL CUR_YDIM]
    set yName [getParmStr $fdata NDLABEL CUR_XDIM]

    set it1 $ix1
    set itn $ixn

    set ix1 $iy1
    set ixn $iyn

    set iy1 $it1
    set iyn $itn

    set xAlt  Y
    set yAlt  X
   } \
else \
   {
    set xName [getParmStr $fdata NDLABEL CUR_XDIM]
    set yName [getParmStr $fdata NDLABEL CUR_YDIM]

    set xAlt  X
    set yAlt  Y
   }

#
# Adjust axis labels if any are identical:

if {![strcmp $xName $yName]} {set xName $xAlt}

#
# Read the projections regions.
# Write the projections.

if {$verbose} {puts "Reading Projection: $xName"}

readROI -roi 1 -in $inName -ndim 1 -$projMode -$absArg -x $xName $ix1 $ixn -y $yName $iy1 $iyn

if {$verbose} {puts "Reading Projection: $yName"}

readROI -roi 2 -in $inName -ndim 1 -$projMode -$absArg -x $yName $iy1 $iyn -y $xName $ix1 $ixn

if {$absFlag} \
   {
    foreach id {1 2} \
       {
        set roiInfo [getROIInfo -roi $id]
        set rdata   [lindex $roiInfo 0]
        set sList   [lindex $roiInfo 2]
        set nx      [lindex $sList   0]

        vAbs $rdata $nx
       }
   }

if {![strcasecmp $xOutName Auto]} \
   {
    set thisName $outPrefix$xName.dat
   } \
else \
   {
    set thisName $outPrefix$xOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 1 -out $thisName -ov 

if {![strcasecmp $yOutName Auto]} \
   {
    set thisName $outPrefix$yName.dat
   } \
else \
   {
    set thisName $outPrefix$yOutName
   }

if {$verbose} {puts "Writing Projection: $thisName"}

writeROI -roi 2 -out $thisName -ov

exit 0
