#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "2D X-Axis P0 Phase Correction Estimate."
    puts " -in  inName   \[test.fid\]      2D FID Input."
    puts " -com comName  \[ps2DLoop.com\]  2D Processing Script."
    puts " -out outName  \[tmp.ft2\]       2D Spectrum Temporary Output."
    puts " -ps  psName   \[psLoop.txt\]    Phase Correction Temporary Output."
    puts " -xSol xSol    \[NULL\]          X-Axis Solvent Filter."
    puts " -x1  x1       \[5%\]            X-Axis Extract Region Start."
    puts " -xn  xn       \[45%\]           X-Axis Extract Region Start."
    puts " -y1  y1       \[0%\]            Y-Axis Extract Region Start."
    puts " -yn  yn       \[100%\]          Y-Axis Extract Region Start."
    puts " -yP0 yP0      \[0\]             Y-Axis P0."
    puts " -yP1 yP1      \[0\]             Y-Axis P1."
    exit 0
   }

set inName  test.fid 
set outName tmp.ft2
set psName  psLoop.txt
set comName ps2DLoop.com
set xSol    NULL

set x1     5%
set xn     45%
set y1     0%
set yn     100%

set yP0    0
set yP1    0

getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -ps   psName
getArgD $ARGV -com  comName

getArgD $ARGV -xSol xSol
getArgD $ARGV -x1   x1
getArgD $ARGV -xn   xn
getArgD $ARGV -y1   y1
getArgD $ARGV -yn   yn
getArgD $ARGV -yP0  yP0
getArgD $ARGV -yP1  yP1

#
# ps2DLoop.tcl: find the best phase value for the phase-response integral.

proc psResponse { xList amp p0 } \
{
   set zList ""

   foreach x $xList {lappend zList [expr $amp*cos(($x - $p0)*3.14159/180.0)]}

   return $zList
}

set    cmnd "exec $comName -in $inName -out $outName"
append cmnd " -xSol $xSol -x1 $x1 -xn $xn"
append cmnd " -y1 $y1 -yn $yn -yP0 $yP0 -yP1 $yP1 > $psName"

catch "file delete $psName"
catch "eval $cmnd"

if {![file exists $psName]} \
   {
    puts stderr "Error Creating Phase Response Output $psName"
    exit 1
   }

set error [catch {set inUnit [open $psName r]}]

if {$error} \
   {
    puts stderr "Error Opening Phase Response Output $psName"
    exit 1
   }

set sList [read $inUnit]

close $inUnit

set nn    [llength $sList]
set xList ""
set yList ""

set i 0

while {$i < $nn} \
   {
    lappend xList [lindex $sList $i]
    incr i

    lappend yList [lindex $sList $i]
    incr i
   }

set amp     [getMax $yList]
set bestRMS [getRMS $yList]
set bestP0  0

for {set xP0 0} {$xP0 < 360} {incr xP0} \
   {
    set zList [psResponse $xList $amp $xP0]
    set rms   [getVsStat getRMS $yList $zList]

    if {$rms < $bestRMS} \
       {
        set bestRMS $rms
        set bestP0 $xP0
       }
   }

puts $bestP0

exit 0
