#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Generate a List of Random Numbers:"
    puts " -n            \[100\]      Number of Values to Generate."
    puts " -r     R      \[1.0\]      Scale Factor."
    puts " -iseed iseed  \[767537\]   Random Number Seed, 0 for Auto."
    puts "Random Number Mode (Use Only One):"
    puts " -uniform                 Uniformly Distributed 0 to r (Default)."
    puts " -gauss                   Gaussian Distributed, Standard Deviation r."
    puts "Boolean Mode (Outputs Ones and Zeros):"
    puts " -bool                    Use Boolean Mode."
    puts " -xMin  xMin   \[0.0\]      Start Range for Boolean Test."
    puts " -xMax  xMax   \[0.5\]      End Range for Boolean Test."
    puts " -first                   Force First Output Value to One."
    puts " -last                    Force Last Output Value to One."
    puts "Notes:"
    puts " 1. An exact fraction of ones will be returned for the special case:"
    puts "" 
    puts "       randList.tcl -uniform -r 1.0 -bool -xMin 0.0 -xMax ..."
    puts "" 
    puts "    Since the default settings are: -uniform -r 1.0 -xMin 0.0"
    puts "    the following will all return 25 ones and 75 zeros"
    puts "" 
    puts "       randList.tcl -n 100 -bool -xMax 0.25"
    puts "       randList.tcl -n 100 -bool -xMax 0.25 -first"
    puts "       randList.tcl -n 100 -bool -xMax 0.25 -last"
    puts "       randList.tcl -n 100 -bool -xMax 0.25 -first -last"
    exit 0
   }

set iseed     767537 
set n         100
set xMin      0.0 
set xMax      0.5 
set r         1.0
set gaussFlag 0
set boolFlag  0
set firstFlag 0
set lastFlag  0

getArgD $ARGV -r      r 
getArgD $ARGV -n      n
getArgD $ARGV -iseed  iseed 
getArgD $ARGV -xMin   xMin
getArgD $ARGV -xMax   xMax

if {[flagLoc $ARGV -uniform]} {set gaussFlag 0}
if {[flagLoc $ARGV -gauss]}   {set gaussFlag 1}

if {[flagLoc $ARGV -first]}   {set firstFlag 1}
if {[flagLoc $ARGV -nofirst]} {set firstFlag 0}

if {[flagLoc $ARGV -last]}    {set lastFlag 1}
if {[flagLoc $ARGV -nolast]}  {set lastFlag 0}

if {[flagLoc $ARGV -bool]}    {set boolFlag 1}
if {[flagLoc $ARGV -nobool]}  {set boolFlag 0}

if {$xMin > $xMax} {rSwap xMin xMax} 

srand $iseed

#
#-----------------------------------------------------------------------------

proc pSort { s1 s2 } \
{
   set v1 [lindex $s1 0]
   set v2 [lindex $s2 0]

   if {$v1 > $v2} {return  1}
   if {$v1 < $v2} {return -1}

   return 0
}

if {$boolFlag} \
   {
    if {!$gaussFlag && $r == 1.0 && $xMin == 0.0 && $xMax <= 1.0} \
       {
        set minCount 0

        if {$firstFlag} {incr minCount}
        if {$lastFlag}  {incr minCount}

        set count [expr int( 0.5 + $xMax*$n )]

        if {$count < $minCount} {set count $minCount}
        if {$count > $n} {set count $n}

        set xList ""
        set iList ""

        for {set i 0} {$i < $n} {incr i} \
           {
            if {$firstFlag && !$i} {continue}
            if {$lastFlag  && $i == $n - 1} {continue}

            lappend xList "[expr $r*[rand]] $i"
           }

        set xList [lsort -command pSort $xList]

        if {$firstFlag} {lappend iList 0}
        if {$lastFlag}  {lappend iList [expr $n - 1]}

        foreach x $xList {lappend iList [lindex $x 1]}

        for {set i 0} {$i < $n} {incr i} \
           {
            set loc [lsearch -exact $iList $i]

            if {$loc < $count} \
               {
                puts "1"
               } \
            else \
               {
                puts "0"
               }
           }
       } \
    else \
       {
        for {set i 0} {$i < $n} {incr i} \
           {
            if {$gaussFlag} \
               {
                set x [expr $r*[rand -gauss]]
               } \
            else \
               {
                set x [expr $r*[rand]]
               }

            if {$x >= $xMin && $x <= $xMax} \
               {
                puts "1"
               } \
            else \
               {
                puts "0"
               }
           }
       }
   } \
else \
   {
    for {set i 0} {$i < $n} {incr i} \
       {
        if {$gaussFlag} \
           {
            puts [format " %10.6f"  [expr $r*[rand -gauss]]]
           } \
        else \
           {
            puts [format " %10.6f"  [expr $r*[rand]]]
           }
       }
   }

exit
