#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Rotate PDB Structure onto One or More DC Alignment Frames."
    puts "Input Files:"
    puts " -pdb pdbName   \[ref.pdb\]           PDB Input File"
    puts " -in  dcInList  \[dCalc\[A-Z\].tab\]    DC Input Tables"
    puts "PDB Output Files:"
    puts " -out outName \[rot.pdb\]   Structures Rotated Onto Alignment Frames."
    puts " -rel relName \[rel.pdb\]   Relative Alignment Frames on Structure."
    puts " -tmp tmpName \[tmp.pdb\]   Temporary Output Name."
    puts "Other Options:"
    puts " -chain  chain \[None\]  New Chain Name for Output."
    puts " -seg    seg   \[None\]  New Segment Name for Output."
    puts " -scale  s     \[1.0\]   Tensor Size Scale Factor."
    puts " -tx     tx    \[0.0\]   Additional X-Axis Translation to Append."
    puts " -ty     ty    \[0.0\]   Additional Y-Axis Translation to Append."
    puts " -tz     tz    \[0.0\]   Additional Z-Axis Translation to Append."
    puts " -rx     rx    \[0.0\]   Additional X-Axis Rotation to Append."
    puts " -ry     ry    \[0.0\]   Additional Y-Axis Rotation to Append."
    puts " -rz     rz    \[0.0\]   Additional Z-Axis Rotation to Append."
    puts " -off    off   \[0\]     RESID Offset for Added Atoms, or Keyword Auto."
    puts " -step   step  \[1\]     RESID Step for Added Atoms."
    puts " -axis                 Insert Axis Information (Default)."
    puts " -noaxis               No Axis Information."
    puts " -halo                 Insert Halo Information."
    puts " -nohalo               No Halo Information (Default)."
    puts " -ignore               Ignore Segment Name Info."
    puts " -verb                 Verbose Mode ON (Default)."
    puts " -noverb               Verbose Mode OFF."
    puts "Notes:"
    puts " DC input tables must have DATA for tensor EULER_ANGLES and S_XYZ."
    exit
   }

set dcList     [glob -nocomplain dCalc\[A-Z\].tab]
set pdbName    ref.pdb
set outName    rot.pdb
set relName    rel.pdb
set tmpName    tmp.pdb
set resIDStep  1
set resIDOff   0
set verbose    1

set uChainName None
set uSegName   None

set utx        0.0
set uty        0.0
set utz        0.0

set urx        0.0
set ury        0.0
set urz        0.0

set tScale     1.0
set ignoreFlag 0
set axisFlag   1
set haloFlag   0
 
getArgD $ARGV -pdb   pdbName
getArgD $ARGV -out   outName
getArgD $ARGV -rel   relName
getArgD $ARGV -tmp   tmpName
getArgD $ARGV -scale tScale

getArgD $ARGV -chain uChainName
getArgD $ARGV -seg   uSegName

getArgD $ARGV -tx    utx
getArgD $ARGV -ty    uty
getArgD $ARGV -tz    utz

getArgD $ARGV -rx    urx
getArgD $ARGV -ry    ury
getArgD $ARGV -rz    urz

getArgD $ARGV -step  resIDStep
getArgD $ARGV -off   resIDOff

getListArgD $ARGV -in dcList

if {[flagLoc $ARGV -verb]}     {set verbose 1}
if {[flagLoc $ARGV -noverb]}   {set verbose 0}

if {[flagLoc $ARGV -axis]}     {set axisFlag 1}
if {[flagLoc $ARGV -noaxis]}   {set axisFlag 0}

if {[flagLoc $ARGV -halo]}     {set haloFlag 1}
if {[flagLoc $ARGV -nohalo]}   {set haloFlag 0}

if {[flagLoc $ARGV -ignore]}   {set ignoreFlag 1}
if {[flagLoc $ARGV -noignore]} {set ignoreFlag 0}

#
#------------------------------------------------------------------

proc addTensorFrame { pdbSrc thisName frame sXYZ } \
{
   global axisFlag haloFlag tScale resIDOff resIDStep

   set error [catch {set outUnit [open $thisName a]}]

   set resID 0

   if {$error} \
      {
       puts stderr "RotDC Error opening PDB file $thisName"
       return
      }

   set pInfo [dynGetInfo -src $pdbSrc -extent -all]
   set i 0

   foreach v {x1 xn y1 yn z1 zn x0 y0 z0} \
      {
       set $v [lindex $pInfo $i]
       incr i
      }

   set xs [expr abs([lindex $sXYZ 0])]
   set ys [expr abs([lindex $sXYZ 1])]
   set zs [expr abs([lindex $sXYZ 2])]

   set xr [expr 0.5*(3.6 + $xn - $x1)]
   set yr [expr 0.5*(3.6 + $yn - $y1)]
   set zr [expr 0.5*(3.6 + $zn - $z1)]

   if {$xs == 0.0} \
      {
       set rr 1.0
      } \
   else \
      {
       set rr [expr 1.0/$zs]
      }

   set xq [expr $x0 + 1.0 + $tScale*1.2*$xr]
   set yq [expr $y0 + 1.0 + $tScale*1.2*$yr]
   set zq [expr $z0 + 1.0 + $tScale*1.2*$zr]

   set mr [getMax "$xr $yr $zr"]

   set xe [expr $tScale*1.5*$rr*$mr*$xs]
   set ye [expr $tScale*1.5*$rr*$mr*$ys]
   set ze [expr $tScale*1.5*$rr*$mr*$zs]

   set xq [expr $x0 + 1.0 + $xe]
   set yq [expr $y0 + 1.0 + $ye]
   set zq [expr $z0 + 1.0 + $ze]

   set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                         -resName XYZ -atomName CXYZ -chain $frame \
                         -x $x0 -y $y0 -z $z0]

   if {$axisFlag} \
      {
       puts $outUnit $str

       set resID 1

       for {set xc [expr $x0]} {$xc < $xq} {set xc [expr $xc+1.0]} \
          {
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName X -atomName CX -chain $frame \
                                 -x $xc -y $y0 -z $z0]

           puts $outUnit $str
           incr resID $resIDStep
          }

       set resID 1

       for {set yc [expr $y0]} {$yc < $yq} {set yc [expr $yc+1.0]} \
          {
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName Y -atomName CY -chain $frame \
                                 -x $x0 -y $yc -z $z0]

           puts $outUnit $str
           incr resID $resIDStep
          }

       set resID 1

       for {set zc [expr $z0]} {$zc < $zq} {set zc [expr $zc+1.0]} \
          {
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName Z -atomName CZ -chain $frame \
                                 -x $x0 -y $y0 -z $zc]

           puts $outUnit $str
           incr resID $resIDStep
          }
      }

   if {$haloFlag} \
      {
       set resID 1

       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set x [expr $x0 + $xe*cos(3.14159*$t/180.0)]
           set y [expr $y0 + $ye*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName XY -atomName EXY -chain $frame \
                                 -x $x -y $y -z $z0]

           puts $outUnit $str
           incr resID $resIDStep
          }

       set resID 1
    
       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set x [expr $x0 + $xe*cos(3.14159*$t/180.0)]
           set z [expr $z0 + $ze*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName XZ -atomName EXZ -chain $frame \
                                 -x $x -y $y0 -z $z]

           puts $outUnit $str
           incr resID $resIDStep
          }

       set resID 1
    
       for {set t 0} {$t <= 360} {incr t 1} \
          {
           set y [expr $x0 + $ye*cos(3.14159*$t/180.0)]
           set z [expr $z0 + $ze*sin(3.14159*$t/180.0)]
    
           set str [dynGetPDBStr -resID [expr $resID + $resIDOff] \
                                 -resName YZ -atomName EYZ -chain $frame \
                                 -x $x0 -y $y -z $z]
    
           puts $outUnit $str
           incr resID $resIDStep
          }
      }

   close $outUnit
}

#
#------------------------------------------------------------------

set segList    {A B C D E F G}
set frameList  {Q R S T U V W}
set eulerList  {{0.0 0.0 0.0}}

dynInit -fast

if {$ignoreFlag} \
   {
    dynSetChainMode -ignore
    dynSetSegMode   -ignore
   }

catch "file delete $outName"
catch "file delete $relName"

set refSrc [dynRead -pdb -in $pdbName -hetatm]

dynTranslate -src $refSrc -center -all
dynWrite     -src $refSrc -out $relName

if {![strcasecmp $resIDOff Auto]} \
   {
    set pInfo    [dynGetInfo -src $refSrc]
    set resIDOff [expr 1 + [lindex $pInfo 1]]   
   }

set i 0

foreach dcName $dcList \
   {
    set pdbSrc [dynRead -pdb  -in $pdbName -hetatm]
    set dcTab  [gdbRead table -in $dcName]

    set seg    [lindex $segList   $i]
    set frame  [lindex $frameList $i]

    getKeyData $dcTab dInfo

    if {![info exists dInfo(EULER_ANGLES)]} \
       {
        puts stderr "Notice: No EULER_ANGLES given in table $dcName" 
       } \
    else \
       {
        set eulerList $dInfo(EULER_ANGLES)
       }

    if {![info exists dInfo(S_XYZ)]} \
       {
        puts stderr "Notice: No S_XYZ values given in table $dcName"
       } \
    else \
       {
        set sXYZ [lindex $dInfo(S_XYZ) 0]
       }

    set e [lindex $eulerList 0]

    set psi   [lindex $e 0]
    set theta [lindex $e 1]
    set phi   [lindex $e 2]

    if {$verbose} \
       {
        puts stderr [format "%s % 8.2f % 8.2f % 8.2f" $dcName $psi $theta $phi]
       }

    dynSetChain -chain $seg -src $pdbSrc

    dynRotate -src $pdbSrc -rx $psi -ry $theta -rz $phi -center -all
    dynWrite  -pdb -src $pdbSrc -out $tmpName

    catch "exec cat $tmpName | fgrep -v END >> $outName"
    catch "file delete $tmpName"

    addTensorFrame $pdbSrc $tmpName $frame $sXYZ

    dynFree $pdbSrc

    set psi   [expr -1*$psi]
    set theta [expr -1*$theta]
    set phi   [expr -1*$phi]

    set pdbSrc [dynRead -pdb -in $tmpName -hetatm]

    dynRotate -src $pdbSrc -rx 0.0  -ry 0.0    -rz $phi -all
    dynRotate -src $pdbSrc -rx 0.0  -ry $theta -rz 0.0  -all
    dynRotate -src $pdbSrc -rx $psi -ry 0.0    -rz 0.0  -all

    dynWrite -pdb -src $pdbSrc -out $tmpName

    catch "exec cat $tmpName | fgrep -v END >> $relName"
    catch "file delete $tmpName"

    dynFree $pdbSrc
    gdbFree $dcTab 

    unset dInfo

    incr i
   }

set pdbSrc [dynRead -pdb -in $outName -hetatm]

addTensorFrame $pdbSrc $outName O "1.0 1.0 1.0" 

dynFree $pdbSrc

set pdbSrc [dynRead -pdb -in $outName -hetatm]
set relSrc [dynRead -pdb -in $relName -hetatm]

dynRotate -src $pdbSrc -rx $urx -ry $ury -rz $urz -all
dynRotate -src $relSrc -rx $urx -ry $ury -rz $urz -all

dynTranslate -src $pdbSrc -tx $utx -ty $uty -tz $utz -all
dynTranslate -src $relSrc -tx $utx -ty $uty -tz $utz -all

if {[strcasecmp $uChainName None]} \
   {
    dynSetChain -chain $uChainName -src $pdbSrc
    dynSetChain -chain $uChainName -src $relSrc
   }

if {[strcasecmp $uSegName None]} \
   {
    dynSetSeg -seg $uSegName -src $pdbSrc
    dynSetSeg -seg $uSegName -src $relSrc
   }

dynWrite -pdb -src $pdbSrc -out $outName -renumber
dynWrite -pdb -src $relSrc -out $relName -renumber

catch "file delete $tmpName"

exit
