#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]


if {[flagLoc $ARGV -help]} \
   {
    puts "Summarize Existing Pool Scores (as produced by scorePool.com):"
    puts " -in  inName   \[scorePool/ext_score.tab\]  Input Table of PNAME and VAL values."
    puts "- dir dirName  \[scorePool\]                Directory Containing Pool Data."
    puts " -nc  nc       \[Auto\]                     Number of Compounds in Pool."
    puts "Analysis of Decomposition Results:"
    puts " -bCoef  bcList   \[1.0 ...\]   Blank Decomposition Coefficients."
    puts " -sCoef  scList   \[1.0 ...\]   Screen Decomposition Coefficients."
    puts " -minPct minPct   \[1.0\]       Minimum Significant Pct Change in Coefficients."
    exit 1
   }

set inName  scorePool/ext_score.tab 
set dirName scorePool
set minPct  1.0

set nc      Auto
set bCoef   ""
set sCoef   ""

getArgD $ARGV -in     inName
getArgD $ARGV -dir    dirName
getArgD $ARGV -minPct minPct
getArgD $ARGV -nc     nc

if {![strcasecmp $inName None] || ![file exists $inName]} \
   {
    puts stderr "ScorePool Summary Error Finding Input Table: $inName "
    exit 1
   }

set t [gdbRead table -in $inName]

getKeyData $t dInfo 

if {![strcasecmp $nc Auto]} \
   {
    if {[info exists dInfo(NC)]} \
       {
        set nc [lindex $dInfo(NC) 0]
       } \
    else \
       {
        puts stderr "ScorePool Summary Error Finding Number of Compounds (DATA NC)."
        exit 1
       }
   }

for {set i 0} {$i < $nc} {incr i} \
   {
    lappend bCoef 1.0
    lappend sCoef 1.0
   }

getListArgD $ARGV -bCoef bCoef
getListArgD $ARGV -sCoef sCoef

if {[llength $bCoef] != $nc || [llength $sCoef] != $nc} \
   {
    puts stderr "ScorePool Summary Error: Coefficient Count Mismatch."
    exit 1
   }

#
#-----------------------------------------------------------------------------------------------

proc tallyParm { t parm } \
{
   global nc

   set valList ""

   for {set i 1} {$i <= $nc} {incr i} \
      {
       set thisParm [format $parm $i]

       set eList [gdbSelect -from $t -cond "!strcasecmp( PNAME, '$thisParm' )"]
       if {![llength $eList]} continue

       set e [lindex $eList 0]
       lappend valList [gdbGet $e VAL]
      }

   if {![llength $valList]} {set valList 0.0}

   return $valList
}

set parmList \
{
 MIN_C%s          MAX_C%s          RMS_C%s                NOISE_C%s              NONE
 MIN_C%sDIF       MAX_C%sDIF       NONE RMSD_C%sDIF       RMSD_C%sDIF_NORM       RMSD_C%sDIF_PCT       RMSD_C%sDIF_EPCT       NONE DOT_C%sDIF       NONE NOISE_C%sDIF       NONE
 MIN_C%sDIF_SMASK MAX_C%sDIF_SMASK NONE RMSD_C%sDIF_SMASK RMSD_C%sDIF_SMASK_NORM RMSD_C%sDIF_SMASK_PCT RMSD_C%sDIF_SMASK_EPCT NONE DOT_C%sDIF_SMASK NONE NOISE_C%sDIF_SMASK NONE
 MIN_C%sDIF_CMASK MAX_C%sDIF_CMASK NONE RMSD_C%sDIF_CMASK RMSD_C%sDIF_CMASK_NORM RMSD_C%sDIF_CMASK_PCT RMSD_C%sDIF_CMASK_EPCT NONE DOT_C%sDIF_CMASK NONE NOISE_C%sDIF_CMASK NONE
}

foreach parm $parmList \
   {
    if {![strcasecmp $parm None]} \
       {
        puts ""
        continue
       }

    set valList  [tallyParm $t $parm]
    set thisParm [format $parm N]

    if {[string first PCT $thisParm] < 0} \
       {
        set fmt "%-26s % .4e"
       } \
    else \
       {
        set fmt "%-26s %11.4f"
       }

    puts [format $fmt ${thisParm}_LO [getMin $valList]]
    puts [format $fmt ${thisParm}_HI [getMax $valList]]
   }

set bCoefPctList ""
set sCoefPctList ""
set dCoefPctList ""

set bpList ""
set spList ""
set dpList ""

set bSum [getSum $bCoef]
if {$bSum == 0.0} {set bSum 1.0}

set sSum [getSum $sCoef]
if {$sSum == 0.0} {set sSum 1.0}

for {set i 0} {$i < $nc} {incr i} \
   {
    set b  [lindex $bCoef $i]
    set bp [expr 100.0*$b/$bSum]

    set s  [lindex $sCoef $i]
    set sp [expr 100.0*$s/$sSum]

    lappend bpList $bp
    lappend spList $sp

    set dp [expr $bp - $sp]

    if {abs($dp) <= $minPct} {set dp 0.0}
 
    if {abs($bp) < $minPct} \
       { 
        lappend dpList 0.0
       } \
    else \
       {
        lappend dpList [expr 100.0*$dp/$bp] 
       }

    puts [format "%-26s % .4e" BCOEF_C[expr $i + 1] $b]
   }

puts ""

for {set i 0} {$i < $nc} {incr i} \
   { 
    set s [lindex $sCoef $i]
    puts [format "%-26s % .4e" SCOEF_C[expr $i + 1] $s]
   }

puts ""

for {set i 0} {$i < $nc} {incr i} \
   {
    set bp [lindex $bpList $i]
    puts [format "%-26s %11.4f" BCOEF_PCT_C[expr $i + 1] $bp]
    lappend bCoefPctList $bp
   }

puts ""

for {set i 0} {$i < $nc} {incr i} \
   {
    set sp [lindex $spList $i]
    puts [format "%-26s %11.4f" SCOEF_PCT_C[expr $i + 1] $sp]
    lappend sCoefPctList $sp
   }

puts ""

for {set i 0} {$i < $nc} {incr i} \
   {
    set dp [lindex $dpList $i]
    puts [format "%-26s %11.4f" SCOEF_DPCT_C[expr $i + 1] $dp]
    lappend dCoefPctList $dp
   }

puts ""
puts [format "%-26s %11.4f" BCOEF_PCT_CN_LO  [getMin $bCoefPctList]]
puts [format "%-26s %11.4f" BCOEF_PCT_CN_HI  [getMax $bCoefPctList]]
puts ""
puts [format "%-26s %11.4f" SCOEF_PCT_CN_LO  [getMin $sCoefPctList]]
puts [format "%-26s %11.4f" SCOEF_PCT_CN_HI  [getMax $sCoefPctList]]
puts ""
puts [format "%-26s %11.4f" SCOEF_DPCT_CN_LO [getMin $dCoefPctList]]
puts [format "%-26s %11.4f" SCOEF_DPCT_CN_HI [getMax $dCoefPctList]]

exit 0
