#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -geom +265+170 -- "$@"

#
# screenViewN (c) 2012  F. Delaglio www.nmrscience.com

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts " -in        scoreName   \[score.tab\]"
    puts " -out       hitName     \[hits.tab\]"
    puts " -prefix    pList       \[ext\]"
    puts " -nc        nc          \[Auto\]"
    puts " -viewer    vName       \[showTab.tcl\]"
    puts " -lwdraw    lwdraw      \[1\]"
    puts " -qLimHit   qLimHit     \[5.0\]"
    puts " -qLimTroub qLimTroub   \[8.0\]"
    puts " -scoreReg  scoreReg    \[aro\]"
#   puts " -qVar     qVar        \[RMSD_BSDIF_PCT\]"
#   puts " -s1Var    s1Var       \[RMSD_CNDIF_DMASK_EPCT\]"
#   puts " -s2Var    s2Var       \[SCOEF_DPCT_CN\]"
#   puts " -t1Var    t1Var       \[RMSD_BDDIF_PCT\]"
#   puts " -t2Var    t2Var       \[RMSD_SDDIF_PCT\]"
#   puts " -s1Tit    s1Title     \[RMSD_CNDIF_DMASK_EPCT\]"
#   puts " -s2Tit    s2Title     \[SCOEF_DPCT_CN\]"
#   puts " -sigmaVar sigmaVar    \[NOISE_BLANK\]"
    exit 0
   }

wm geom . +10+30

set scoreName  score.tab
set hitName    hits.tab
set prefixList ext
set viewerName showTab.tcl

set scoreTab  0
set scoreTabS 0
set absFlag   1
set lwdraw    1
set eList     ""
set eCount    0
set nc        Auto 

set s1Var     RMSD_CNDIF_DMASK_EPCT
set s2Var     SCOEF_DPCT_CN

set s1Var     RMSD_CNDIF_DMASK_EPCT
set s2Var     RMSD_CNDIF_CMASK_EPCT
set s1Tit     "% Match to Difference"
set s2Tit     "% Match to Compound"
set t1Var     RMSD_BDDIF_PCT 
set t2Var     RMSD_SDDIF_PCT

set qVar      RMSD_BSDIF_PCT 
set sigmaVar  NOISE_BLANK

set qLimHit   5.0
set qLimTroub 8.0
set scoreReg  aro

getArgD $ARGV -nc        nc
getArgD $ARGV -in        scoreName
getArgD $ARGV -out       hitName
getArgD $ARGV -viewer    viewerName
getArgD $ARGV -lwdraw    lwdraw

getArgD $ARGV -sigmaVar  sigmaVar
getArgD $ARGV -s1Var     s1Var
getArgD $ARGV -s2Var     s2Var
getArgD $ARGV -t1Var     t1Var
getArgD $ARGV -t2Var     t2Var
getArgD $ARGV -s1Tit     s1Tit
getArgD $ARGV -s2Tit     s2Tit

getArgD $ARGV -qVar      qVar
getArgD $ARGV -qLimHit   qLimHit
getArgD $ARGV -qLimTroub qLimTroub 
getArgD $ARGV -scoreReg  scoreReg

getListArgD $ARGV -prefix prefixList

#
# Hit and trouble conditions:

set regCond   "strmatch( REGION, '$scoreReg' )"
set hitCond   "$regCond && ($t1Var < 40 && $t2Var < 40 && $qVar > $qLimHit && $s1Var >= 40 && $s2Var >= 40)"
set troubCond "$regCond && ($qVar > $qLimTroub || ($qVar > $qLimTroub && ($t1Var >= 40 || $t2Var >= 40)))"

#
#------------------------------------------------------------------------------------------
 
set currentPoolID 1
set currentCID    0
set maxPoolID     0

set nameList        ""
set poolValList     ""
set compoundValList ""
set currentFileList ""

setFonts

proc tabEvent args \
{
   global currentPoolID currentCID

   puts $args

   set var [lindex $args end-1]
   if {[strcasecmp $var POOL]} return

   set pTxt [lindex $args end]
   set currentPoolID [getTrailingInt $pTxt]

   getBestCompound

   drawPool $currentPoolID

   return 0 
}

proc initScoreTab {} \
{
   global scoreName scoreTab scoreTabS nc hitInfo eList qVar

   if {![file exists $scoreName]} \
      {
       puts "ScreenView Error Finding Score Table $scoreName"
       exit 1
      }

   set scoreTabS [gdbRead table -in $scoreName]

   gdbSortTab $scoreTabS $qVar -down
   gdbWrite   $scoreTabS sort.tab

   set scoreTab [gdbRead table -in $scoreName]
   set eList    [gdbGetInfo $scoreTab -parms entries]
   set eCount   [llength $eList]

   if {!$eCount} \
      {
       puts "ScreenView Error: Empty Score Table $scoreName"
       exit 1
      }

   if {![strcasecmp $nc Auto]} \
      {
       set cList ""

       foreach e $eList \
          {
           set c [gdbGet $e COMPOUND]
           if {[lsearch -exact $cList $c] < 0} {lappend cList $c}
          }

       set n [llength $cList]

       if {$n == 5 || $n == 10} \
          {
           set nc $n
          } \
       else \
          {
           puts "ScreenView Error: Bad Count for Compounds per Pool: $nc."
           exit 1 
          }
      }
}

proc initScoreWin {} \
{
   global nameList eList hitInfo locInfo aIDInfo tagInfo sigMax scoreTab scoreName hitName nc
   global poolList compoundList t1Var t2Var s1Var s2Var s1Tit s2Tit qVar sigmaVar
   global poolValList compoundValList regCond hitCond troubCond maxPoolID
   global boldfont bigfont smallfont

# MAX_BSDIF
# RMSD_BSDIF_PCT
# SCOEF_DPCT_CN_HI
# SCOEF_DPCT_CN
# RMSD_CNDIF_DMASK_EPCT RMSD_CNDIF_CMASK_EPCT

   set nameList [getTabVals $scoreTab TITLE]
   set regList  [getTabVals $scoreTab REGION]
   set xList    [getTabVals $scoreTab $s1Var]
   set yList    [getTabVals $scoreTab $s2Var]
   set sigList  [getTabVals $scoreTab $sigmaVar]
   set sigMax   [getMax $sigList]

   set poolValList     [getTabVals $scoreTab POOL]
   set compoundValList [getTabVals $scoreTab COMPOUND]
   set rackIDList      [getTabVals $scoreTab RACK]
   set wellList        [getTabVals $scoreTab WELL]

   if {$sigMax <= 0.0} {set sigMax 1.0}

   set poolList     ""
   set compoundList ""

   .g configure -font $boldfont

   .g axis configure x -title $s1Tit -tickfont $boldfont -titlefont $boldfont
   .g axis configure y -title $s2Tit -tickfont $boldfont -titlefont $boldfont

   .g legend configure -hide 1
   .g element create score -xdata $xList -ydata $yList -pixels 1 -linewidth 0 \
                           -outline dimGray -fill dimGray 

   set hList     [gdbSelect -from $scoreTab -cond $hitCond]
   set rList     [gdbSelect -from $scoreTab -cond $regCond]
   set tt        [gdbSelect -from $scoreTab -new -cond $hitCond]

   set xxList [getTabVals $tt $s1Var] 
   set yyList [getTabVals $tt $s2Var]

   foreach e $hList {gdbSet $e STATUS Hit}

   set maxPoolID 0

   foreach e $rList \
      {
       set poolID [getTrailingInt [gdbGet $e POOL]]
       set cID    [getTrailingInt [gdbGet $e COMPOUND]]
       set well   [gdbGet $e WELL]
       set rackID [gdbGet $e RACK]

       if {$poolID > $maxPoolID} {set maxPoolID $poolID}

       set hitInfo($poolID,$cID,entry) $e
       set hitInfo(currentPool,$cID) 0
       set hitInfo($poolID,$cID)     0 
       set hitInfo($poolID,badFlag)  0

       if {![strcasecmp [gdbGet $e STATUS] Hit]} \
          {
           puts [format "Hit: Well %2d Rack %d Pool %3d Compound %2d" $well $rackID $poolID $cID]
           set hitInfo($poolID,$cID) 1
          }
      }

   gdbSortTab $tt $qVar -down
   gdbAddNote $tt "REMARK Screening Hits $hitCond"
   gdbWrite   $tt $hitName

   set thisList  [gdbSelect -from $scoreTab -cond $troubCond]
   set troubList ""

   foreach e $thisList \
      {
       set poolID [getTrailingInt [gdbGet $e POOL]] 

       if {![poolIsHit $poolID]} \
          {
           set hitInfo($poolID,badFlag) 1
           lappend troubList $e
          }
      }

   .g element create score2 \
      -xdata $xxList -ydata $yyList -pixels 2 -linewidth 0 \
      -outline white -fill white
             
   .g axis configure x -min 0 -max [expr 1.1*[getMax $xList]] 
   .g axis configure y -min 0 -max [expr 1.1*[getMax $yList]] 
   
   set cList {blue yellow orange green orchid cyan magenta pink darkorchid brown}

   set nC    [llength $cList]
   set i 0

   foreach n $nameList \
     {
      set cx  [lindex $xList $i]
      set cy  [lindex $yList $i]
      set reg [lindex $regList $i]

      set nInfo [split $n "-"]

      set poolID     [getTrailingInt [lindex $poolValList     $i]]
      set compoundID [getTrailingInt [lindex $compoundValList $i]]
      set rackID     [lindex $rackIDList $i]
      set well       [lindex $wellList   $i]

      set aIDInfo($poolID,$compoundID) [lindex $nInfo end]

      if {[lsearch -exact $poolList $poolID] < 0} \
         {
          lappend poolList $poolID
         }

      if {[lsearch -exact $compoundList $compoundID] < 0} \
         {
          lappend compoundList $compoundID
         }

# %%% tagInfo

      set locInfo($poolID,$compoundID,$reg) $i

      set color [lindex $cList [expr ($compoundID - 1) % $nC]]
      set txt   $n,$reg
      set txt   $compoundID,$reg

      set tID [.g marker create text -coord "$cx $cy" \
                  -foreground $color -background ""   \
                  -anchor s -text $txt -hide 1]

      lappend tagInfo($poolID,tag)  $tID

      set tagInfo($poolID,rack) $rackID
      set tagInfo($poolID,well) $well

      incr i
     }

   for {set poolID 1} {$poolID <= $maxPoolID} {incr poolID} \
      {
       if {[info exists tagInfo($poolID,tag)]} \
          {
           .pText insert end [format " Well %-2s Rack %s Pool %-3s" $tagInfo($poolID,well) $tagInfo($poolID,rack) $poolID]
          } \
       else \
          {
           .pText insert end "Missing"
          }
      }

   updatePoolColor

   .g element bind score <ButtonRelease-1> "+ drawCompound %x %y" 

   bind .g <ButtonRelease-1> "+ drawCompound %x %y"
}

proc updatePoolColor {} \
{
   global maxPoolID hitInfo

   for {set poolID 1} {$poolID <= $maxPoolID} {incr poolID} \
      {
       if {[poolIsHit $poolID]} \
          {
           .pText itemconfigure [expr $poolID - 1] \
                  -background forestgreen -selectbackground green \
                  -foreground black       -selectforeground black
          } \
       elseif {$hitInfo($poolID,badFlag)} \
          {
           .pText itemconfigure [expr $poolID - 1] \
                  -background white     -selectbackground gray80 \
                  -foreground firebrick -selectforeground firebrick 
          } \
       else \
          {
           .pText itemconfigure [expr $poolID - 1] \
                  -background white     -selectbackground gray80 \
                  -foreground black     -selectforeground black
          }
      }

   update
}

proc poolIsHit { poolID } \
{
   global hitInfo nc

   for {set cID 1} {$cID <= $nc} {incr cID} \
      {
       if {![info exists hitInfo($poolID,$cID)]} continue
       if {$hitInfo($poolID,$cID)} {return 1}
      }

   return 0
}

proc getTrailingInt { s } \
{
   set i [string range $s 1 end]
   if {[string is integer $i]} {return $i}

   return 0
}

proc updateScatterplot {} \
{
   global scoreTab s1Var s2Var qVar hitCond viewerName

   set c   "!strcasecmp( STATUS, 'Hit' )"
   set tt  [gdbSelect -from $scoreTab -cond $c -new]

   gdbSortTab $tt $qVar -down
   gdbAddNote $tt "REMARK Screening Hits $hitCond"
   gdbWrite   $tt hits.tab

   set xxList [getTabVals $tt $s1Var]
   set yyList [getTabVals $tt $s2Var]

   .g element configure score2 -xdata $xxList -ydata $yyList

   catch "send $viewerName tabViewReloadRecent"

   gdbFree $tt
}

proc drawCompound { xPix yPix } \
{
   global showInfo nameList poolValList compoundValList currentPoolID currentCID nc

   set status [.g element closest $xPix $yPix nearInfo -halo 20 score]
   if {!$status} return

   clearShowInfo

   set loc $nearInfo(index)

   if {$loc < 0 || $loc > [llength $nameList] - 1} \
      {
       puts stderr "Bad Item Location $loc"
       return
      }

   set name  [lindex $nameList $nearInfo(index)]
   set nInfo [split $name "-"]

   set currentPoolID [getTrailingInt [lindex $poolValList     $nearInfo(index)]]
   set currentCID    [getTrailingInt [lindex $compoundValList $nearInfo(index)]]

   set showInfo($currentCID) 1

   drawPool $currentPoolID
}

proc drawPool { poolID } \
{
    global showInfo hitInfo currentPoolID currentCID eList x1 xn sigMax absFlag nc lwdraw
    global currentPrefix currentFileList poolList compoundList locInfo aIDInfo tagInfo
    global t1Var t2Var s1Var s2Var sigmaVar qVar

    global cxSize cySize cxSizeS cySizeS charX charY

    if {![llength $poolList] || ![llength $compoundList]} return
    if {[lsearch -exact $poolList $currentPoolID] < 0} return 

    set currentFileList ""

   .pText selection clear 0 end
   .pText selection set [expr $poolID - 1]
   .pText see [expr $poolID - 1]

   foreach thisID $poolList \
      {
       if {![info exists tagInfo($thisID,tag)]} continue
       foreach tID $tagInfo($thisID,tag) {.g marker configure $tID -hide 1}
      }
   
   if {[info exists tagInfo($poolID,tag)]} \
      {
       foreach tID $tagInfo($poolID,tag) {.g marker configure $tID -hide 0}
      }

   drawRec -win 1 -x 0 $cxSize  -y 0 $cySize  -fg [. cget -background] 
   drawRec -win 2 -x 0 $cxSizeS -y 0 $cySizeS -fg [. cget -background] 
 
   set cList {blue yellow orange green orchid cyan magenta pink darkorchid brown}

   set ii 0

   set currentPoolID $poolID

   for {set cID 1} {$cID <= $nc} {incr cID} \
      {
       if {[info exists hitInfo($poolID,$cID)]} \
          {
           set hitInfo(currentPool,$cID) $hitInfo($poolID,$cID)
          } \
       else \
          {
           set hitInfo(currentPool,$cID) 0
          }
      }

   set refName    None
   set diffName   None
   set screenName None

   set sigma    0.0
   set q        0.0
   set troubStr ""

   set s1 0.0
   set s2 0.0
   set nn 0

   set reg $currentPrefix

   foreach id $compoundList \
      {
       set id100 [expr $id + 100]

       if {![info exists locInfo($currentPoolID,$id,$reg)]} \
          {
           puts stderr "Missing Pool $currentPoolID Compound $id Region $reg"
           continue
          }

       set loc $locInfo($currentPoolID,$id,$reg)
       set e   [lindex $eList $loc]

       set thisSpecName [gdbGet $e CN_NAME]
       set thisScrName  [gdbGet $e SCREEN_NAME]
       set thisRefName  [gdbGet $e BLANK_NAME]
       set thisDifName  [gdbGet $e DIF_NAME]
       set aID          None

       if {[info exists aIDInfo($currentPoolID,$id)]} \
          {
           set aID $aIDInfo($currentPoolID,$id)
          }

       if {![strcasecmp $refName None] && [file exists $thisRefName]} \
          {
           set refName    $thisRefName
           set diffName   $thisDifName
           set screenName $thisScrName
          }

       if {!$nn} \
          {
           lappend currentFileList $thisRefName
           lappend currentFileList $thisScrName
           lappend currentFileList $thisDifName
          }

       set axisArg None

       if {!(($ii + 1) % 5)} {set axisArg Bottom}

       incr nn

       set color [lindex $cList [expr $id - 1]]

       readROI -roi $id100 -ndim 1 -x X_AXIS 0% 100% -in $thisSpecName

       drawROI -roi $id100 -cell $id100 -win 1 -zero -fhi -10% -lhi 110% \
               -fg $color -bg black -lw1d 3 -axis $axisArg -colorA black -colorS None -lwdraw $lwdraw 

       drawText -cell $id100 -win 1 -big -large -x 1% -y 10% -fg white \
                -text "Pool $currentPoolID Compound $id $aID"

       lappend currentFileList $thisSpecName

       incr ii
      }

   set sP ""

   for {set tmpID 1} {$tmpID <= $nc} {incr tmpID} \
      {
       if {[info exists locInfo($currentPoolID,$tmpID,$reg)]} \
          {
           set e [lindex $eList $locInfo($currentPoolID,$tmpID,$reg)]

           if {[llength $e]} \
              {
               set sigma    [format %.2f%% [expr 100.0*[gdbGet $e $sigmaVar]/$sigMax]]
               set sigma    [format %.2f [gdbGet $e $sigmaVar]]
               set troubStr [format "Blank Deco %.1f Screen Deco %.1f" [gdbGet $e $t1Var] [gdbGet $e $t2Var]]
               set q        [format %.2f [gdbGet $e $qVar]]
               break
              }
          }
      }

   set cList {gray red white}
   set nList "$refName $screenName $diffName"
   set tList "Blank Screen Difference"

   set ii 0

   foreach id {6 7 8} \
      {
       set color [lindex $cList $ii]
       set name  [lindex $nList $ii]
       set t     [lindex $tList $ii]

       readROI -roi $id -ndim 1 -x X_AXIS 0% 100% -in $name
       incr ii
      }

   set specMinVal [getROIStat 6 vMin]
   set specMaxVal [getROIStat 6 vMax]

   set drawMaxVal [expr  1.1*$specMaxVal]
   set drawMinVal [expr -0.1*$specMaxVal]
 
   set ii 0

   foreach id {6 7 8} \
      {
       set color   [lindex $cList $ii]
       set name    [lindex $nList $ii]
       set t       [lindex $tList $ii]
       set axisArg None
       set aStr    ""

       if {$id == 8} {set axisArg Bottom}

       if {$absFlag} \
          {
           drawROI -roi $id -cell $id -win 2 -zero -fhi $drawMinVal -lhi $drawMaxVal \
                   -fg $color -bg black -lw1d 3 -axis $axisArg -colorA black -colorS None -lwdraw $lwdraw 

           if {$id == 8} {set aStr " Abs Scale"}
          } \
       else \
          {
           drawROI -roi $id -cell $id -win 2 -zero -fhi -15% -lhi 110% \
                   -fg $color -bg black -lw1d 3 -axis $axisArg -colorA black -colorS None -lwdraw lwdraw 

           if {$id == 8} {set aStr " Rel Scale"}
          }

       set thisColor white

       if {$id == 8} \
          {
           append t " Q $q $troubStr"
           if {$hitInfo($currentPoolID,badFlag)} {set thisColor red}
          }

       set well   None
       set rackID None

       if {[info exists tagInfo($currentPoolID,well)]} {set well   $tagInfo($currentPoolID,well)}
       if {[info exists tagInfo($currentPoolID,rack)]} {set rackID $tagInfo($currentPoolID,rack)}

       drawText -cell $id -win 2 -big -large -x 1% -y 10% -fg $thisColor \
                -text "Well $well Rack $rackID Pool $currentPoolID $t$aStr"

       incr ii
      }


   set nShow [getShowCount]
   if {!$nShow} return

   set labX 1
   set nH   0

   for {set iii $nc} {$iii >= 1} {incr iii -1} \
      {
       if {!$showInfo($iii)} continue
  
       incr nH
 
       set labY [expr 10 + 10*$nH]
       
       set currentCID $iii

       if {[info exists locInfo($currentPoolID,$currentCID,$reg)]} \
          {
           set e [lindex $eList $locInfo($currentPoolID,$currentCID,$reg)]
    
           if {[llength $e]} \
              {
               set s1    [format %.1f [gdbGet $e $s1Var]]
               set s2    [format %.1f [gdbGet $e $s2Var]]
               set t1    [format %.1f [gdbGet $e $t1Var]]
               set t2    [format %.1f [gdbGet $e $t2Var]]
               set sigma [format %.2f [gdbGet $e $sigmaVar]]
               set q     [format %.2f [gdbGet $e $qVar]]
              }
          }

       set cList {blue yellow orange green orchid cyan magenta pink darkorchid brown}

       set nC    [llength $cList]

       set id    $currentCID
       set ii    [expr $id - 1]
       set id100 [expr $id + 100]

       set aID ""

       if {[info exists aIDInfo($currentPoolID,$id)]} \
          {
           set aID $aIDInfo($currentPoolID,$id)
          }

       set color [lindex $cList $ii]

       drawROI -roi $id100 -cell 8 -win 2 -nozero -fhi -10% -lhi 110% \
               -fg $color -bg none -lw1d 3

       drawText -cell 8 -win 2 -big -large -x ${labX}% -y ${labY}% -fg $color \
                -text "Compound $currentCID $aID Score: $s1 $s2"
      }
}

proc getBestCompound {} \
{
   global showInfo hitInfo compoundList eList locInfo currentPoolID currentCID currentPrefix nc
   global s1Var s2Var

   clearShowInfo

   set bestScore 0.0
   set bestCID   0

   set reg $currentPrefix

   foreach id $compoundList \
      {
       if {![info exists locInfo($currentPoolID,$id,$reg)]} continue
    
       set e [lindex $eList $locInfo($currentPoolID,$id,$reg)]

       if {[llength $e]} \
          {
           set s1    [format %.1f [gdbGet $e $s1Var]]
           set s2    [format %.1f [gdbGet $e $s2Var]]
           set score [expr 2.0*$s1 + $s2]

           if {$score > $bestScore} \
              {
               set bestScore $score
               set bestCID $id
              }
          }
      }

   set currentCID $bestCID

   for {set cID 1} {$cID <= $nc} {incr cID} \
      {
       if {[info exists hitInfo($currentPoolID,$cID)]} \
          {
           set showInfo($cID) $hitInfo($currentPoolID,$cID) 
          } \
       else \
          {
           set showInfo($cID) 0
          }
      }
}

proc getShowCount {} \
{
   global showInfo nc

   set n 0

   for {set iii 1} {$iii <= $nc} {incr iii} \
      {
       if {$showInfo($iii)} {incr n}
      }

   return $n
}

proc clearShowInfo {} \
{
   global showInfo nc

   for {set iii 1} {$iii <= $nc} {incr iii} {set showInfo($iii) 0}
}

proc drawOff { xPix yPix } \
{
  global currentCellID currentPoolID

  set currentCellID 0

  clearShowInfo
  drawPool $currentPoolID
}
  
proc drawToggle { xPix yPix } \
{
   global showInfo cellInfo currentCID currentPoolID nc

   clearShowInfo

   set currentCID 0

   for {set id 1} {$id <= $nc} {incr id} \
      {
       set cList $cellInfo($id,coords)

       if {$xPix >= [lindex $cList 0] && $xPix <= [lindex $cList 1] \
            && $yPix > [lindex $cList 2] && $yPix < [lindex $cList 3]} \
           {
            set currentCID $id
            break
           }
      }

   set showInfo($currentCID) 1

   drawPool $currentPoolID
}

proc parseKey { c xPix yPix } \
{
   global currentCID currentPoolID currentPrefix prefixList currentFileList poolList compoundList absFlag

   set nC      [llength $compoundList]
   set nP      [llength $poolList]
   set thisDir None

   if {[llength $currentFileList]} \
      {
       set thisName [lindex $currentFileList 0]
       if {[file exists $thisName]} {set thisDir [file dirname $thisName]}
      } 

   if {![strcasecmp $c X]} \
      {
       catch "exec show1D.com $currentPoolID [llength $compoundList] $thisDir $currentPrefix $currentFileList &"
      }

   if {![strcasecmp $c O]} \
      {
       if {$currentCID > 0} \
          {
           set currentCID 0
          } \
       else \
          {
           getBestCompound
          }

       drawPool $currentPoolID
      }

   if {![strcasecmp $c A]} \
      {
       set absFlag 1
       drawPool $currentPoolID
      }

   if {![strcasecmp $c R]} \
      {
       set absFlag 0
       drawPool $currentPoolID
      }

   if {![strcasecmp $c B]} \
      {
       getBestCompound
       drawPool $currentPoolID
      }

   if {![strcasecmp $c N]} \
      {
       incr currentPoolID
       if {$currentPoolID > $nP} {set currentPoolID 1}

       getBestCompound
       drawPool $currentPoolID
      }

   if {![strcasecmp $c P]} \
      {
       incr currentPoolID -1
       if {$currentPoolID < 1} {set currentPoolID $nP}

       getBestCompound
       drawPool $currentPoolID
      }

   if {![strcasecmp $c C]} \
      {
       incr currentCID 
       if {$currentCID > $nC} {set currentCID 0}
       drawPool $currentPoolID
      }

   if {![strcasecmp $c V]} \
      {
       incr currentCID -1
       if {$currentCID < 0} {set currentCID $nC}
       drawPool $currentPoolID
      }

   if {![strcasecmp $c Left]} \
      {
       set n [llength $prefixList]

       set loc [lsearch -exact $prefixList $currentPrefix]
       if {$loc < 0} {set loc 0}

       incr loc -1
       if {$loc < 0} {set loc [expr $n - 1]}

       set currentPrefix [lindex $prefixList $loc]
 
       drawPool $currentPoolID
      }

   if {![strcasecmp $c Right]} \
      {
       set n [llength $prefixList]

       set loc [lsearch -exact $prefixList $currentPrefix]
       if {$loc < 0} {set loc 0}

       incr loc  1
       if {$loc > $n - 1} {set loc 0}

       set currentPrefix [lindex $prefixList $loc]

       drawPool $currentPoolID
      }
}

proc selectPool { } \
{
   global currentPoolID

   set loc [lindex [.pText curselection] 0]
   if {![llength $loc]} return

   set currentPoolID [expr $loc + 1]
 
   getBestCompound
   drawPool $currentPoolID
}

proc toggleHit { iii } \
{
   global eList currentCID currentPoolID hitInfo showInfo nc

   if {$iii < 1 || $iii > $nc} return

   set showInfo($iii) $hitInfo(currentPool,$iii)

   set hitInfo($currentPoolID,$iii) $hitInfo(currentPool,$iii)

   if {[info exists hitInfo($currentPoolID,$iii,entry)]} \
      {
       foreach e $hitInfo($currentPoolID,$iii,entry) \
          {
           if {$hitInfo(currentPool,$iii)} \
              {
               gdbSet $e STATUS Hit 
              } \
           else \
              {
               gdbSet $e STATUS None 
              }
          }
      }

   drawPool $currentPoolID
   updateScatterplot
   updatePoolColor
}

proc toggleCompound { iii } \
{
   global currentCID currentPoolID showInfo nc

   if {$iii < 1 || $iii > $nc} return

   if {$showInfo($iii)} \
      {
       set currentCID $iii
      } \
   else \
      {
       set currentCID 0 
      }


   drawPool $currentPoolID   
}

initScoreTab
clearShowInfo

set masterColorList {blue yellow orange green orchid cyan magenta pink darkorchid brown}

set currentPoolID 1
set currentCID    0

set nameList        ""
set poolValList     ""
set compoundValList ""
set currentFileList ""
set currentPrefix   [lindex $prefixList 0] 

set charX 10
set charY 16

set xw [winfo screenwidth  .]
set yw [winfo screenheight .]

set cxSize [expr int( 0.44*$xw )]
set cySize [expr int( 0.93*$yw )]

set cxSizeS [expr int( 0.44*$xw )]
set cySizeS [expr int( 0.62*$cySize + 2.5*$charY )]

set cxSizeG [expr int( 0.870*$cxSize )]
set cySizeG [expr int( 0.375*$cySize - 2.0*$charY )]

set thisBG [. cget -background]

frame .left  -background $thisBG -border 0 -relief flat
frame .right -background $thisBG -border 0 -relief flat
frame .pList -background $thisBG -border 0 -relief flat

canvas .cLeft -width $cxSize -height $cySize -border 0
pack   .cLeft -in .left -side top -padx 0 -pady 0

canvas .cRight -width $cxSizeS -height $cySizeS -border 0
pack   .cRight -in .right -side top -padx 0 -pady 0 

graph .g -width $cxSizeG -height $cySizeG -plotbackground black -background $thisBG \
         -topmargin 1 -rightmargin 1 -leftmargin [expr 6*$charX] -border 0

pack  .g -in .right -side top -anchor w -padx [expr 2*$charX] -pady 2 

listbox   .pText  -width 20 -yscrollcommand ".spText set" -selectmode single
scrollbar .spText -command ".pText yview" -orient vertical

pack .pText -in .pList -side left -fill both -expand 1 -padx 0 -pady 0
pack .spText -in .pList -side right -fill y -padx 0 -pady 0
pack .left .right -in . -side left -padx 0 -pady 0
pack .pList -in . -side left -fill y -padx 0 -pady 0
update

bind .pText <ButtonRelease-1> "+ selectPool"

.cLeft  create pixmap 0 0 -width $cxSize  -height $cySize  -win 1
.cRight create pixmap 0 0 -width $cxSizeS -height $cySizeS -win 2

set cxw [expr int(0.95*$cxSize)]
set cyw 180 
set cyw 176 

set dx  40
set dy  40
set dy  10

set cx1 40
set cx1 10
set cxn [expr $cx1 + $cxw]
set cy1 10
set cyn [expr $cy1 + $cyw]

if {$nc <= 5} \
   {
    for {set id 1} {$id <= $nc} {incr id} \
       {
        set id100 [expr $id + 100]
    
        defineCell -cell $id100 -x $cx1 $cxn -y $cy1 $cyn -win 1

        set cellInfo($id,coords) "$cx1 $cxn $cy1 $cyn"
        set cellInfo($id,win)    1

        drawRec -cell $id100 -win 1 -fg black -x 0% 100% -y 0% 100%
        drawBox -cell $id100 -win 1 -fg white -x 0% 100% -y 0% 100%

        set cy1 [expr $cyn + $dy]
        set cyn [expr $cy1 + $cyw]
       }
   } \
else \
   {
    set ccx1 $cx1
    set ccxn [expr int($ccx1 + $cxw/2.0 - 4)]

    for {set id 1} {$id <= $nc/2} {incr id} \
       {
        set id100 [expr $id + 100]
    
        defineCell -cell $id100 -x $ccx1 $ccxn -y $cy1 $cyn -win 1

        checkbutton .cLeft.hit$id -text "Hit" -variable hitInfo(currentPool,$id) \
                    -command "toggleHit $id" -selectcolor [lindex $masterColorList [expr $id - 1]]

        .cLeft create window [expr $ccxn - 5] [expr $cy1 + 5] -anchor ne -window .cLeft.hit$id -tag hit$id

        set cellInfo($id,coords) "$ccx1 $ccxn $cy1 $cyn"
        set cellInfo($id,win)    1

        drawRec -cell $id100 -win 1 -fg black -x 0% 100% -y 0% 100%
        drawBox -cell $id100 -win 1 -fg white -x 0% 100% -y 0% 100%

        set cy1 [expr $cyn + $dy]
        set cyn [expr $cy1 + $cyw]
       }

    set ccx1 [expr $ccxn + 8]
    set ccxn $cxn
    set cy1 10
    set cyn [expr $cy1 + $cyw]

    for {set id [expr 1 + $nc/2]} {$id <= $nc} {incr id} \
       {
        set id100 [expr $id + 100]
    
        defineCell -cell $id100 -x $ccx1 $ccxn -y $cy1 $cyn -win 1

        checkbutton .cLeft.hit$id -text "Hit" -variable hitInfo(currentPool,$id) \
                    -command "toggleHit $id" -selectcolor [lindex $masterColorList [expr $id - 1]]

        .cLeft create window [expr $ccxn - 5] [expr $cy1 + 5] -anchor ne -window .cLeft.hit$id -tag hit$id

        set cellInfo($id,coords) "$ccx1 $ccxn $cy1 $cyn"
        set cellInfo($id,win)    1

        drawRec -cell $id100 -win 1 -fg black -x 0% 100% -y 0% 100%
        drawBox -cell $id100 -win 1 -fg white -x 0% 100% -y 0% 100%

        set cy1 [expr $cyn + $dy]
        set cyn [expr $cy1 + $cyw]
       }
   }

set cy1 10
set cyn [expr $cy1 + $cyw]

foreach id {6 7 8} \
   {
    defineCell -cell $id -x $cx1 $cxn -y $cy1 $cyn -win 2
    drawRec -cell $id -win 2 -fg black -x 0% 100% -y 0% 100%

    set cy1 [expr $cyn + $dy]
    set cyn [expr $cy1 + $cyw]

    if {$id == 7} \
       {
        frame .cRight.panel 

        .cRight create window $cx1 [expr $cy1 + 2] -window .cRight.panel \
                -anchor nw -width [expr $cxn - $cx1] -height [expr 2*$charY - 2] 

        for {set iii 1} {$iii <= $nc} {incr iii} \
            {
             set showInfo($iii) 0

             checkbutton .cRight.sel$iii -text "$iii   " -variable showInfo($iii) \
                         -command "toggleCompound $iii" -selectcolor [lindex $masterColorList [expr $iii - 1]]

             pack .cRight.sel$iii -in .cRight.panel -side left 
            }

        incr cy1 [expr 2*$charY]
        incr cyn [expr 2*$charY]
       }
   }

update 

bind .cLeft  <ButtonRelease-1> "+ drawToggle %x %y"
bind .cRight <ButtonRelease-1> "+ drawOff    %x %y"

initScoreWin

set currentPoolID 1

getBestCompound
drawPool $currentPoolID 

bind . <KeyPress> "+ parseKey %K %x %y" 

set msg "screenViewN (c) 2012 F. Delaglio www.nmrscience.com  N-Next Pool / P-Prev Pool / C-Next Compound / V-Prev Compound / A-Abs Scale / R-Rel Scale / Left-Prev Region / Right-Next Region / X-Explore Pool"

puts $msg

wm title . $msg 

update

