#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set thisVersion "2023.115.22.23"

#
# NMRPipe Super-excellent Strip viewer by big fd.
# (c) Frank Delaglio 2016-2024.

setFonts

if {[flagLoc $ARGV -help]} \
   {
    puts "NMRPipe Strip Plot Manipulation for One or More 2D-4D Spectra:"
    puts "  -tab    tabName \[hn.tab\]              2D Table for strip locations."
    puts "  -in     inName  \[ft/test%03d.ft3 ...\] 3D Spectral data template."
    puts "Contour Drawing:"
    puts "  -hi     contHi  \[Auto ...\]    First contour height."
    puts "  -plev   nPos    \[10   ...\]    Number of Positive Contours."
    puts "  -nlev   nNeg    \[10   ...\]    Number of Negative Contours."
    puts "  -mult   mult    \[2.0  ...\]    Factor for Auto contour height."
    puts "Axis Names of Spectra:"
    puts "  -xName  xName   \[Auto ...\]    First Axis Name   (2D-4D) (Example: HN)."
    puts "  -yName  yName   \[Auto ...\]    Second Axis Name  (3D-4D) (Example: N)."
    puts "  -zName  zName   \[Auto ...\]    Third Axis Name   (3D-4D) (Example: CACB)."
    puts "  -aName  aName   \[Auto ...\]    Fourth Axis Name  (4D)."
    puts "  -sType  sType   \[Auto ...\]    Strip Type Keyword: strip 1d plane"
    puts "Strip Coordinate Table Variable Names and Parameters:"
    puts "  -xVar   xVar    \[Auto\]        Variable name of first PPM coord  (2D-4D)."
    puts "  -yVar   yVar    \[Auto\]        Variable name of second PPM coord (3D-4D)."
    puts "  -zVar   zVar    \[Auto\]        Variable name of third  PPM coord (4D)."
    puts "  -idVar  idVar   \[INDEX\]       Variable Name for Unique ID."
    puts "  -lab    lVar    \[INDEX\]       Variable name for Strip Label."
    puts "  -xOff   xOff    \[0.0 ...\]     X Offset to Apply, PPM (2D-4D)."
    puts "  -yOff   yOff    \[0.0 ...\]     Y Offset to Apply, PPM (3D)."
    puts "  -zOff   zOff    \[0.0 ...\]     Z Offset to Apply, PPM (4D)."
    puts "Peak Tables for Strip Spectra:"
    puts "  -pk     pkName  \[None ...\]    Peak Table Data for Strip Spectra."
    puts "  -pkXVar xVar    \[Auto ...\]    Peak Table X-Axis PPM Variable Name."
    puts "  -pkYVar yVar    \[Auto ...\]    Peak Table Y-Axis PPM Variable Name."
    puts "  -pkZVar zVar    \[Auto ...\]    Peak Table Z-Axis PPM Variable Name (3D-4D)."
    puts "  -pkAVar zVar    \[Auto ...\]    Peak Table A-Axis PPM Variable Name (4D)."
    puts "  -assVar aVar    \[ASS ...\]     Peak Table Assignment Variable Name."
    puts "  -xTol   xTol    \[0.03 ...\]    X-Axis Comparison Tolerance, PPM."
    puts "  -yTol   yTol    \[0.15 ...\]    Y-Axis Comparison Tolerance, PPM."
    puts "  -zTol   zTol    \[Auto ...\]    Z-Axis Comparison Tolerance, PPM (3D-4D)."
    puts "  -aTol   aTol    \[Auto ...\]    A-Axis Comparison Tolerance, PPM (4D)."
    puts "Analysis Features:"
    puts "  -seq    seqName \[None\]        Residue Sequence Table."
    puts "  -join   jName   \[None\]        Strip Join Score Table."
    puts "  -cs     csName  \[None\]        Reference Chemical Shift (CS) Table."
    puts "  -csVar  csVar   \[SHIFT\]       Reference CS Table Variable Name."
    puts "Command Panel:"
    puts "  -cmnd                         Show Command Panel Window."
    puts "  -snap                         Show Snapshot Commands."
    puts "Strip Spectral Region Widths:"
    puts "  -xw       xw    \[0.12ppm ...\] X-Axis +/- Width For view1."
    puts "  -yw       yw    \[0.80ppm ...\] Y-Axis +/- Width For view2 (2D-4D)."
    puts "  -zw       zw    \[1 ...\]       Z-Axis +/- Width For Strip Projection (3D-4D)."
    puts "  -aw       aw    \[0 ...\]       A-Axis +/- Width For Plane Projection (4D)."
    puts "  -useWW                        Apply Lorenztian Weighting Windows (Default)."
    puts "  -nouseWW                      Do Not Apply Weighting Windows."
    puts "Strip Region Limits, ppm, Hz, pts or %:"
    puts "  -z1     zStart  \[0% ...\]      Strip Starting Coord."
    puts "  -zn     zEnd    \[100% ...\]    Strip Ending Coord."
    puts "  -zOff   zOff    \[0.0 ...\]     Z Offset to Apply, PPM."
    puts "  -a1     aStart  \[0% ...\]      Strip Starting Coord."
    puts "  -an     aEnd    \[100% ...\]    Strip Ending Coord."
    puts "  -zOff   aOff    \[0.0 ...\]     A Offset to Apply, PPM."
    puts "Grid Drawing:"
    puts "  -xGrid                        Draw X-Axis Grid Line (Default)."
    puts "  -noxGrid                      No X-Axis Grid Line."
    puts "  -yGrid                        Draw Y-Axis Grid Lines (Default)."
    puts "  -noyGrid                      No Y-Axis Grid Lines."
    puts "Single/Pairwise Strip Plotting"
    puts "  -view1        Plot First (X-Axis) View For Each 2D Coord (Default)."
    puts "  -view2        Plot Second (Y-Axis) View For Each 2D Coord (3D)."
    puts "  -pair         Plot Both Complementary Views (same as -view1 -view2) (3D)."
    puts "Spectrum Colors:"
    puts " -colorP  pC       \[Multi ...\]   Positive Contour Color."
    puts " -colorN  nC       \[Multi ...\]   Negative Contour Color."
    puts "Positive Level HSV Color Vector (-colorP Multi):"
    puts " -pHue1 hue        \[0.5\]         First Hue (0-1)."
    puts " -pHueN hue        \[0.7\]         Last Hue  (0-1)."
    puts " -pSat1 saturation \[0.8\]         First Saturation (0-1)."
    puts " -pSatN saturation \[1.0\]         Last Saturation (0-1)."
    puts " -pVal1 value      \[1.0\]         First Value (0-1)."
    puts " -pValN value      \[1.0\]         Last Value  (0-1)."
    puts " -pcc   pcCount    \[10\]          Positive Color Count."
    puts "Negative Level HSV Color Vector: (-colorN Multi):"
    puts " -nHue1 hue        \[0.0\]         First Hue (0-1)."
    puts " -nHueN hue        \[0.2\]         Last Hue (0-1)."
    puts " -nSat1 saturation \[1.0\]         First Saturation (0-1)."
    puts " -nSatN saturation \[1.0\]         Last Saturation (0-1)."
    puts " -nVal1 value      \[0.8\]         First Value (0-1)."
    puts " -nValN value      \[1.0\]         Last Value (0-1)."
    puts " -ncc   ncCount    \[10\]          Negative Color Count."
    puts "Program Colors:"
    puts " -fg      fgC   \[yellow ...\]  Strip Foreground (Border) Color."
    puts " -bg      bgC   \[black\]       Strip Background Color."
    puts " -colorF  fC    \[gray20 ...\]  Folded Background Color (Or \"Blank\")."
    puts " -colorA  aC    \[white\]       Axis and Info Text Color."
    puts " -colorH  aH    \[white\]       Hilight Color for Active Items."
    puts " -colorG  gC    \[dimGray\]     Strip Grid Color."
    puts " -colorW  wC    \[gray20\]      Viewer Window Color."
    puts " -colorL  lC    \[white ...\]   Label Color."
    puts " -colorC  cC    \[Default ...\] Cursor Colors."
    puts " -colorQ  cQ    \[white ...\]   Cursor Label Colors."
    puts "Strip Graphic Region Widths:"
    puts "  -dx     xwPix  \[40\]      Strip Width, Pixels."
    puts "  -qx1    xwPix  \[3\]       Spacing Between Views,   Pixels."
    puts "  -qx2    xwPix  \[6\]       Spacing Between Spectra, Pixels."
    puts "  -qx3    xwPix  \[9\]       Spacing Between Panels,  Pixels."
    puts "  -ySize  ywPix  \[Auto\]    Height of Graphics Area, Pixels."
    puts "1D and Plane Graphic Region Sizes:"
    puts "  -opx    xwPix  \[200\]     1D Plot Region Width, Pixels."
    puts "  -dpx    xwPix  \[200\]     Plane Width, Pixels."
    puts "  -dpy    ywPix  \[200\]     Plane Width, Pixels."
    puts "Window and Text Geometry:"
    puts "  -winName wName \[None\]    Window Name."
    puts "  -font    fName \[stdfont\] Font Name (Default: $stdfont)."
    puts "  -g       geom  \[Auto\]    Initial Window Geometry."
    puts "  -xAdjW   xAdj  \[Auto\]    X Adjustment for Window Size, Pixels (Also -xAdj)."
    puts "  -yAdjW   yAdj  \[Auto\]    Y Adjustment for Window Size, Pixels (Also -yAdj)."
    puts "  -xOffW   xOff  \[Auto\]    Y Offset for Window Position, Pixels."
    puts "  -yOffW   yOff  \[Auto\]    Y Offset for Window Position, Pixels."
    puts "  -cAdj    cAdj  \[220\]     X Adjustment for Command Panel Window."
    puts "  -cx      cx    \[12\]      X Character Size, Pixels."
    puts "  -cy      cy    \[15\]      Y Character Size, Pixels."
    puts "  -mac                     Force Adjustments for Mac OS X Dock."
    puts "  -nomac                   No Adjustments for Mac OS X Dock."
    puts "Special Options:"
    puts "  -history hName \[None\]    File for Recording Strip Order Adjustments."
    puts "  -cursors nC    \[0 ...\]   Number of Horizontal Tracking Cursors."
    puts "  -shade                   Shade Backgrounds for Folded Data."
    puts "  -noshade                 No Shading of Backgrounds for Folded Data."
    puts "  -link                    Communicate to Other Scroll Applictions."
    exit 1 
   }

set verbose       0
set inNameList    ft/test%03d.ft3
set outName(def)  None
set pkName(def)   None

set scrollFont    stdfont

set hi(def)       Auto 
set mult(def)     2.0
set pLev(def)     10
set nLev(def)     10

set stripType(def) strip
set title(def)     None
set xName(def)     Auto 
set yName(def)     Auto 
set zName(def)     Auto 
set aName(def)     Auto 

set pkXVar(def)  Auto
set pkYVar(def)  Auto
set pkZVar(def)  Auto
set assVar(def)  ASS

set xOff(def)    0.00
set yOff(def)    0.00
set zOff(def)    0.00
set aOff(def)    0.00

set xTol(def)    0.03
set yTol(def)    0.15
set zTol(def)    Auto
set zTol(def)    Auto

set xw(def)      0.12ppm
set yw(def)      0.80ppm
set zw(def)      1
set aw(def)      0

set xww         -1
set yww         -1
set zww         -1
set aww         -1

set z1(def)      0%
set zn(def)      100%
set a1(def)      0%
set an(def)      100%

set tabName   hn.tab
set histName  None
set seqName   None
set joinName  None
set csName    None
set csVarName SHIFT

set xVarName Auto 
set yVarName Auto
set zVarName Auto
set idVar    INDEX
set labelVar INDEX 

set dx       40
set opx      200 
set dpx      200 
set dpy      200 
set dy       Auto
set cx       12
set cy       15 
set qx1      3 
set qx2      6 
set qx3      9 
set xAdj     Auto 
set yAdj     Auto 
set cAdj     220
set wySize   Auto 
set wySizeP  Auto 
set geomOffX Auto
set geomOffY Auto 
set geom     Auto

set plotFlag       0

set plotPageCount  0
set plotStripCount 0
set plotGroupCount 0

set plotName       None
set plotOrient     landscape
set plotPaper      A
set plotTray       0

set sdx          5.0
set sdy         85.0
set sgap         1.0
set ggap         1.5

set fgColor(def)  yellow
set lColor(def)   white
set pColor(def)   Multi
set nColor(def)   Multi
set cColor(def)   Default 
set qColor(def)   white 
set fColor(def)   gray20

set bgColor   black
set aColor    white
set hColor    white
set wColor    gray20 
set gColor    dimGray 
set sColor    dimGray
set bColor    None
set xColor    green
set rColor    red

set eColorDef lightGray
set eColorSet yellow

set defCursor top_left_arrow
set noCursor  X_cursor
set hCursor   sb_h_double_arrow
set vCursor   sb_v_double_arrow

set xGrid     xGrid
set yGrid     yGrid

set shadeFoldFlag 0
set stripsPerSpec 1
set view1Flag     0
set view2Flag     0
set linkFlag      0
set cmndFlag      0
set snapFlag      0
set quitFlag      0

set dragID       -1
set dragX1       -1
set dragXN       -1
set dragY1       -1
set dragYN       -1

set currentCursor None
set dataValid     0
set hUnit         0
set assCount      0
set eCount        0
set panelCount    0
set cellCount     0
set roiCount      0
set maxDimCount   0

set cursorInfo(totalCount) 0
set cursorInfo(countList)  ""

set panelList  ""
set cellList   ""
set prevList   ""
set viewList   ""
set eList      ""

set joinTab   0
set coordTab  0

set nullRes  -1
set r1        0
set rN        0
set firstRes  0
set lastRes   0
set resCount  0
set seqList   ""
set resList   ""

set histHdr   ""
set histInfo  ""

set snapDir   snap
set imageID   0

set pkInfo(tabCount) 0

set colorList \
 {
  black red pink firebrick orange yellow green forestGreen brown
  cyan blue darkblue violet darkOrchid magenta 
  lightGray dimGray gray30  gray20 white
 }

set posColorCount 10
set negColorCount 10

set pHue1 0.5
set pSat1 0.8
set pVal1 1.0

set pHueN 0.7
set pSatN 1.0
set pValN 1.0

set nHue1 0.0
set nSat1 1.0
set nVal1 0.8

set nHueN 0.2
set nSatN 1.0
set nValN 1.0

set winName None

#
#-----------------------------------------------------------------------------

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

getArgD $ARGV -g     geom
getArgD $ARGV -xAdj  xAdj
getArgD $ARGV -yAdj  yAdj
getArgD $ARGV -xAdjW xAdj
getArgD $ARGV -yAdjW yAdj
getArgD $ARGV -xOffW geomOffX 
getArgD $ARGV -yOffW geomOffY
getArgD $ARGV -cAdj  cAdj
getArgD $ARGV -cx    cx 
getArgD $ARGV -cy    cy

getArgD $ARGV -winName winName
getArgD $ARGV -font    scrollFont

if {![strcasecmp $scrollFont stdfont]} {set scrollFont $stdfont}
if {![strcasecmp $winName    None]}    {set winName ""}

set adjInfo "10 68 5 8"
set macFlag 0

if {[info exists env(NMRBINTYPE)]} \
   {
    if {[string first "mac" [string tolower $env(NMRBINTYPE)]] >= 0} {set macFlag 1}
   }

if {[flagLoc $ARGV -mac]}   {set macFlag 1}
if {[flagLoc $ARGV -nomac]} {set macFlag 0}

if {$macFlag} \
   {
    catch {set macInfo [exec getAppleGeom.tcl -brief]}

    if {![info exists macInfo]}  {set macInfo "80 0 0 0"}
    if {[llength $macInfo] == 4} {set adjInfo $macInfo}

    puts "Using Mac OS X Desktop Dock Adjustments: $macInfo"
   }

if {[strcasecmp $geom Auto]} \
   {
    wm geometry . $geom 
    . configure -cursor $defCursor
    update
   }

if {![strcasecmp $xAdj Auto]} {set xAdj [lindex $adjInfo 0]}
if {![strcasecmp $yAdj Auto]} {set yAdj [expr 1 + $cy + [lindex $adjInfo 1]]}

if {![strcasecmp $geomOffX Auto]} {set geomOffX [lindex $adjInfo 2]}
if {![strcasecmp $geomOffY Auto]} {set geomOffY [lindex $adjInfo 3]}

getListArgD $ARGV -in inNameList

foreach inName $inNameList \
   {
    if {![file exists [format $inName 1 1]]} \
       {
        puts "Error Finding Input 3D Spectrum $inName"
        exit 1
       }
   }

set specCount [llength $inNameList]

getArgDN $ARGV -pk      pkName  $specCount
getArgDN $ARGV -out     outName $specCount

if {[flagLoc $ARGV -stripHi]} \
   {
    getArgDN $ARGV -stripHi hi $specCount
   } \
else \
   {
    getArgDN $ARGV -hi      hi $specCount
   }

getArgDN $ARGV -mult    mult    $specCount
getArgDN $ARGV -plev    pLev    $specCount
getArgDN $ARGV -nlev    nLev    $specCount

getArgDN $ARGV -sType   stripType $specCount
getArgDN $ARGV -title   title     $specCount
getArgDN $ARGV -xName   xName     $specCount
getArgDN $ARGV -yName   yName     $specCount
getArgDN $ARGV -zName   zName     $specCount
getArgDN $ARGV -aName   aName     $specCount

getArgDN $ARGV -xOff    xOff    $specCount
getArgDN $ARGV -yOff    yOff    $specCount
getArgDN $ARGV -zOff    zOff    $specCount
getArgDN $ARGV -zOff    aOff    $specCount

getArgDN $ARGV -xTol    xTol    $specCount
getArgDN $ARGV -yTol    yTol    $specCount
getArgDN $ARGV -zTol    zTol    $specCount
getArgDN $ARGV -aTol    aTol    $specCount

getArgDN $ARGV -pkXVar  pkXVar  $specCount
getArgDN $ARGV -pkYVar  pkYVar  $specCount
getArgDN $ARGV -pkZVar  pkZVar  $specCount
getArgDN $ARGV -assVar  assVar  $specCount

getArgDN $ARGV -xw      xw      $specCount
getArgDN $ARGV -yw      yw      $specCount
getArgDN $ARGV -zw      zw      $specCount
getArgDN $ARGV -aw      aw      $specCount

getArgDN $ARGV -z1      z1      $specCount
getArgDN $ARGV -zn      zn      $specCount
getArgDN $ARGV -a1      a1      $specCount
getArgDN $ARGV -an      an      $specCount

if {[flagLoc $ARGV -useWW]} \
   {
    set xww -1
    set yww -1
    set zww -1
    set aww -1
   }

if {[flagLoc $ARGV -nouseWW]} \
   {
    set xww  0 
    set yww  0
    set zww  0
    set aww  0
   }

getArgD $ARGV -xww     xww 
getArgD $ARGV -yww     yww
getArgD $ARGV -zww     zww
getArgD $ARGV -aww     aww

getArgD $ARGV -history histName
getArgD $ARGV -tab     tabName
getArgD $ARGV -seq     seqName
getArgD $ARGV -join    joinName
getArgD $ARGV -cs      csName
getArgD $ARGV -csVar   csVarName

getArgD $ARGV -xVar    xVarName
getArgD $ARGV -yVar    yVarName
getArgD $ARGV -zVar    zVarName
getArgD $ARGV -idVar   idVar
getArgD $ARGV -lab     labelVar

getArgD $ARGV -dx      dx
getArgD $ARGV -opx     opx
getArgD $ARGV -pdx     pdx
getArgD $ARGV -pdy     pdy
getArgD $ARGV -qx1     qx1 
getArgD $ARGV -qx2     qx2 
getArgD $ARGV -qx3     qx3 
getArgD $ARGV -ySize   wySize
getArgD $ARGV -ySizeP  wySizeP

getListArgD $ARGV -colorList colorNames

getArgDN $ARGV -fg     fgColor $specCount
getArgDN $ARGV -colorP pColor  $specCount
getArgDN $ARGV -colorN nColor  $specCount
getArgDN $ARGV -colorL lColor  $specCount
getArgDN $ARGV -colorF fColor  $specCount

getArgD $ARGV -bg      bgColor
getArgD $ARGV -colorA  aColor
getArgD $ARGV -colorH  hColor
getArgD $ARGV -colorG  gColor
getArgD $ARGV -colorB  bColor
getArgD $ARGV -colorS  sColor
getArgD $ARGV -colorW  wColor

getArgD $ARGV -pHue1   pHue1
getArgD $ARGV -pSat1   pSat1
getArgD $ARGV -pVal1   pVal1
getArgD $ARGV -pHueN   pHueN
getArgD $ARGV -pSatN   pSatN
getArgD $ARGV -pValN   pValN

getArgD $ARGV -nHue1   nHue1
getArgD $ARGV -nSat1   nSat1
getArgD $ARGV -nVal1   nVal1
getArgD $ARGV -nHueN   nHueN
getArgD $ARGV -nSatN   nSatN
getArgD $ARGV -nValN   nValN

getArgD $ARGV -sdx     sdx
getArgD $ARGV -sdy     sdy
getArgD $ARGV -sgap    sgap
getArgD $ARGV -ggap    ggap

getArgD $ARGV -plotPaper  plotPaper
getArgD $ARGV -plotTray   plotTray

if {[flagLoc $ARGV -portrait]}  {set plotOrient portrait}
if {[flagLoc $ARGV -landscape]} {set plotOrient landscape}

getListArgD $ARGV -cursors cursorInfo(countList) 

getArgDN $ARGV -colorC cColor $specCount
getArgDN $ARGV -colorQ qColor $specCount

if {[flagLoc $ARGV -pair]} \
   {
    set view1Flag 1
    set view2Flag 1
   }

if {[flagLoc $ARGV -nopair]} \
   {
    set view1Flag 1
    set view2Flag 0
   }

if {[flagLoc $ARGV -view1]} {set view1Flag 1}
if {[flagLoc $ARGV -view2]} {set view2Flag 1}

if {!$view1Flag && !$view2Flag} {set view1Flag 1}

if {$view1Flag} {lappend viewList 1}
if {$view2Flag} {lappend viewList 2}

set stripsPerSpec [llength $viewList]

if {[flagLoc $ARGV -snap]}     {set snapFlag      1}
if {[flagLoc $ARGV -nosnap]}   {set snapFlag      0}

if {$snapFlag} {set cmndFlag 1}

if {[flagLoc $ARGV -cmnd]}     {set cmndFlag      1}
if {[flagLoc $ARGV -nocmnd]}   {set cmndFlag      0}

if {[flagLoc $ARGV -shade]}    {set shadeFoldFlag 1}
if {[flagLoc $ARGV -noshade]}  {set shadeFoldFlag 0}

if {[flagLoc $ARGV -link]}     {set linkFlag 1}
if {[flagLoc $ARGV -nolink]}   {set linkFlag 0}

if {[flagLoc $ARGV -xGrid]}    {set xGrid xGrid}
if {[flagLoc $ARGV -noxGrid]}  {set xGrid noxGrid}

if {[flagLoc $ARGV -yGrid]}    {set yGrid yGrid}
if {[flagLoc $ARGV -noyGrid]}  {set yGrid noyGrid}

if {[flagLoc $ARGV -plot]}     {set plotFlag 1}
if {[flagLoc $ARGV -noplot]}   {set plotFlag 0}

if {[flagLoc $ARGV -quit]}     {set quitFlag 1}

#
#------------------------------------------------------------------------------

proc evalCmnd args \
{
   global verbose

   if {$verbose} \
      {
       puts $args
       puts ""
      }

   eval $args
}

proc initCoordTab {} \
{
   global tabName coordTab eList eCount panelCount resCount specCount
   global slotInfo panelInfo cellInfo roiInfo panelList prevList viewList 
   global xVarName yVarName zVarName idVar labelVar dimCount maxDimCount

   set error 0

   if {![file exists $tabName]} \
      {
       puts "Error Finding Input Coord Table $tabName"
       exit 1
      }

   puts "Strip Coordinate Table $tabName:"

   set coordTab [gdbRead table -in $tabName]
   set eList    [gdbGetInfo $coordTab -parms entries]
   set vList    [gdbGetInfo $coordTab -parms varNames]

   set eCount [llength $eList]
   set panelCount $resCount

   if {$eCount > $panelCount} {set panelCount $eCount}

   if {$maxDimCount == 1} \
      {
       set xVarName None
       set yVarName None
      } \
   elseif {$maxDimCount == 2} \
      {
       if {![strcasecmp $xVarName Auto]} \
          {
           set yVarName None

           if {[lsearch -exact $vList X_PPM] >= 0} \
              {
               set xVarName X_PPM
              } \
           elseif {[lsearch -exact $vList H_PPM] >= 0} \
              {
               set xVarName H_PPM
              }
          }
      } \
   elseif {$maxDimCount == 3} \
      {
       if {![strcasecmp $xVarName Auto]} {set xVarName [getHNTabVar  $coordTab]}
       if {![strcasecmp $yVarName Auto]} {set yVarName [get15NTabVar $coordTab]}

       if {![strcasecmp $xVarName None] || ![strcasecmp $yVarName None]} \
          {
           set xVarName X_PPM
           set yVarName Y_PPM
          }
      } \
   else \
      {
       if {![strcasecmp $xVarName Auto]} {set xVarName [getHNTabVar  $coordTab]}
       if {![strcasecmp $yVarName Auto]} {set yVarName [get15NTabVar $coordTab]}

       if {![strcasecmp $xVarName None] || ![strcasecmp $yVarName None]} \
          {
           set xVarName X_PPM
           set yVarName Y_PPM
          }
      }

   puts " Coord Table First  Variable Name: $xVarName" 
   puts " Coord Table Second Variable Name: $yVarName" 

   if {[strcasecmp $idVar None]} \
      {
       if {[lsearch -exact $vList $idVar] < 0} {set idVar None}
      }

   if {[strcasecmp $labelVar None]} \
      {
       if {[lsearch -exact $vList $labelVar] < 0} {set labelVar None}
      }

   set panelList ""
   set prevList  ""
   set idList    ""

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        if {$panelID <= $eCount} \
           {
            set entry  [lindex $eList [expr $panelID - 1]]
            set thisID [gdbGetDef $entry $idVar $panelID]
           } \
        else \
           {
            set entry  0
            set thisID [getUnusedID $idList]
           }

        if {[lsearch -exact $idList $thisID] < 0} \
           {
            lappend idList $thisID
           } \
        else \
           {
            puts stderr "Warning: ID $thisID is used more than once."
            incr error
           }

        set panelInfo($panelID,label) [gdbGetDef $entry $labelVar $thisID]
        set panelInfo($panelID,xPPM)  [gdbGetDef $entry $xVarName 9999.0]
        set panelInfo($panelID,yPPM)  [gdbGetDef $entry $yVarName 9999.0] 
        set panelInfo($panelID,zPPM)  [gdbGetDef $entry $zVarName 9999.0] 
        set panelInfo($panelID,entry) $entry 
        set panelInfo($panelID,assID) $thisID

        set panelInfo($panelID,cellList)   ""
        set panelInfo($panelID,xLimits)    ""
        set panelInfo($panelID,yLimits)    ""
        set panelInfo($panelID,x1)         0 
        set panelInfo($panelID,xn)         0
        set panelInfo($panelID,y1)         0 
        set panelInfo($panelID,yn)         0
        set panelInfo($panelID,fixed)      0
        set panelInfo($panelID,text)       None
        set panelInfo($panelID,selMarker)  None 
        set panelInfo($panelID,next)      -1
        set panelInfo($panelID,prev)      -1
        set panelInfo($panelID,joinList)  ""

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            set panelInfo($panelID,$specID,xText)    None 
            set panelInfo($panelID,$specID,yText)    None 
            set panelInfo($panelID,$specID,eText)    None 
            set panelInfo($panelID,$specID,setCount) 0

            foreach viewID $viewList \
               {
                set cellInfo($panelID,$specID,$viewID,plotCellID) 0
                set cellInfo($panelID,$specID,$viewID,cellID)     0
                set roiInfo($panelID,$specID,$viewID,roiID)       0
               }
           }

        lappend panelList $panelID
        lappend prevList  -1
       }
 
   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
       {
        set slotInfo($slotID,anchor)     None 
        set slotInfo($slotID,vanchor)    None 
        set slotInfo($slotID,resText)    None 
        set slotInfo($slotID,joinMarker) None

        set slotInfo($slotID,x1)      0 
        set slotInfo($slotID,xn)      0 
        set slotInfo($slotID,y1)      0 
        set slotInfo($slotID,yn)      0 
       }

   if {$error} {showError Scroll "Errors Found in Strip Location Table Info."}
}

proc initSeqInfo {} \
{ 
   global seqName r1 rN firstRes lastRes resCount seqList

   if {![strcasecmp $seqName None]} return

   if {![file exists $seqName]} \
      {
       puts "Error Finding Sequence Table Input File $seqName"
       exit 1
      }

   initResNames

   set sTab     [gdbRead table -in $seqName]
   set seqList  [getSeqList $sTab firstRes lastRes]
   set resCount [llength $seqList]

   set r1 $firstRes
   set rN $lastRes

   gdbFree $sTab
}

proc initResInfo {} \
{
   global nullRes firstRes lastRes resCount seqList resList r1 rN panelCount

   set resList ""

   if {$resCount} \
      {
       set r1 $firstRes 

       for {set resID $firstRes} {$resID <= $lastRes} {incr resID} \
          {
           lappend resList [getResName3 [lindex $seqList [expr $resID - $firstRes]]] 
          }
      } \
   else \
      {
       set r1 1
      }

   set nullRes [expr $r1 - 1]
   set rN      [expr $r1 + $panelCount - 1]
   set n       [expr $panelCount - $resCount]

   if {0 < $nullRes} {set nullRes 0}

   for {set i 1} {$i <= $n} {incr i} {lappend resList x} 
}

proc initJoinTab {} \
{
   global joinName joinTab joinInfo panelInfo panelCount

   if {![strcasecmp $joinName None]} return 
  
   if {![file exists $joinName]} \
      {
       puts stderr "Error Finding Join Score Table $joinName"
       return 1
      }

   set joinTab [gdbRead table -in $joinName]

   gdbSortTab $joinTab RANK -up

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        set panelInfo($panelID,joinList) ""

        set c      "INDEX_I == $panelInfo($panelID,assID)"
        set jeList [gdbSelect -from $joinTab -cond $c]

        foreach je $jeList \
           {
            set joinID [findPanelIDByAssID [gdbGet $je INDEX_J]]
            if {!$joinID} continue

            lappend panelInfo($panelID,joinList) $joinID
           }

#       puts "Panel $panelID: [lrange $panelInfo($panelID,joinList) 0 9]"
       }

   return 0
}

proc initData {} \
{
   global inNameList specCount fdata xName yName zName aName hi mult pLev nLev
   global yFirst yLast zFirst zLast aFirst aList xDim yDim zDim xTol yTol zTol aTol
   global xSize ySize zSize aSize validInfo xw yw zw aw xwPPM ywPPM zwPPM awPPM
   global maxDimCount dimCount tpFlag

   puts ""
 
   set minVal 0.0
   set maxVal 0.0
   set noise  0.0

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set fdata($specID)    0

       set dimCount($specID) 0
       set tpFlag($specID)   0

       set xSize($specID)    1
       set ySize($specID)    1
       set zSize($specID)    1
       set aSize($specID)    1

       set yFirst($specID)  0%
       set yLast($specID) 100%
       set zFirst($specID)  0%
       set zLast($specID) 100%
       set aFirst($specID)  0%
       set aLast($specID) 100%

       set validInfo(assID)           0

       set validInfo($specID,roiID)   0
       set validInfo($specID,cellID)  0

       set validInfo($specID,roiXID)  0
       set validInfo($specID,cellXID) 0

       set validInfo($specID,roiYID)  0
       set validInfo($specID,cellYID) 0

       set xDim($specID) CUR_XDIM 
       set yDim($specID) CUR_YDIM
       set zDim($specID) CUR_ZDIM 
       set aDim($specID) CUR_ADIM 
      }

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set inName [lindex $inNameList [expr $specID - 1]]

       puts "Spectrum $specID: $inName"

       set error [rdFDATA $inName fdata($specID)]

       if {$error} \
          {
           puts "Error Reading Header in $inName"
           exit 1
          }

       set thisSpecType      [getSpecType $fdata($specID)]
       set dimCount($specID) [getParmI $fdata($specID) FDDIMCOUNT   NULL_DIM]
       set tpFlag($specID)   [getParmI $fdata($specID) FDTRANSPOSED NULL_DIM]

       if {$dimCount($specID) == 1} \
          {
           if {![strcasecmp $xName($specID) Auto]} \
              {
               set xDim($specID)  CUR_XDIM
               set xName($specID) [getParmStr $fdata($specID) NDLABEL $xDim($specID)]
              }

           set xSize($specID) [getParmI $fdata($specID) NDSIZE $xDim($specID)]
          }

       if {$dimCount($specID) == 2} \
          {
           if {![strcasecmp $xName($specID) Auto]} \
              {
               if {$tpFlag($specID)} \
                  {
                   set xDim($specID) CUR_YDIM
                  } \
               else \
                  {
                   set xDim($specID) CUR_YDIM
                  }

               set xName($specID) [getParmStr $fdata($specID) NDLABEL $xDim($specID)]
              }

           if {![strcasecmp $yName($specID) Auto]} \
              {
               set xStr [getDimKeyword $xName($specID) $fdata($specID)]

               if {![strcasecmp $xStr CUR_XDIM] || ![strcasecmp $xStr X_AXIS]} \
                  {
                   set yName($specID) CUR_YDIM
                  } \
               else \
                  {
                   set yName($specID) CUR_XDIM
                  }

               set yName($specID) [getParmStr $fdata($specID) NDLABEL $yDim($specID)]
              }

           set xSize($specID) [getParmI $fdata($specID) NDSIZE $xDim($specID)]
           set ySize($specID) [getParmI $fdata($specID) NDSIZE $yDim($specID)]

           if {$ySize($specID) == 1} {set dimCount($specID) 1}
          }

       if {$dimCount($specID) == 3} \
          {
           if {![strcasecmp $xName($specID) Auto]} \
              {
               set xName($specID) [getDirectDim $fdata($specID)]
              }

           if {![strcasecmp $thisSpecType HC]} \
              {
               if {![strcasecmp $yName($specID) Auto]} \
                  {
                   set yName($specID) [get13CDim $fdata($specID)]
                  }

               if {![strcasecmp $zName($specID) Auto]} \
                  {
                   set zName($specID) [getIndirectDim $fdata($specID)]
                  }
              } \
           else \
              {
               if {![strcasecmp $yName($specID) Auto]} \
                  {
                   set yName($specID) [get15NDim $fdata($specID)]
                  }

               if {![strcasecmp $zName($specID) Auto]} \
                  {
                   set zName($specID) [getIndirectDim $fdata($specID)]
                  }
              } 

           set xDim($specID)  [getDimKeyword $xName($specID) $fdata($specID)]
           set yDim($specID)  [getDimKeyword $yName($specID) $fdata($specID)]
           set zDim($specID)  [getDimKeyword $zName($specID) $fdata($specID)]

           set xSize($specID) [getParmI $fdata($specID) NDSIZE $xDim($specID)]
           set ySize($specID) [getParmI $fdata($specID) NDSIZE $yDim($specID)]
           set zSize($specID) [getParmI $fdata($specID) NDSIZE $zDim($specID)]

           if {$ySize($specID) == 1 && $zSize($specID) == 1} \
              {
               set dimCount($specID) 1
              } \
           elseif {$ySize($specID) > 1 && $zSize($specID) == 1} \
              {
               set dimCount($specID) 2
              }
          }

       if {$dimCount($specID) > 3} \
          {
           if {![strcasecmp $xName($specID) Auto]} \
              {
               set xName($specID) [getDirectDim $fdata($specID)]
              }

           if {![strcasecmp $yName($specID) Auto]} \
              {
               set yName($specID) [get15NDim $fdata($specID)]
              }

           if {![strcasecmp $zName($specID) Auto]} \
              {
               set zName($specID) [getIndirectDim $fdata($specID)]
              }

           if {![strcasecmp $aName($specID) Auto]} \
              {
               set aName($specID) [getFourthDim $fdata($specID)]
              }

           set xDim($specID)  [getDimKeyword $xName($specID) $fdata($specID)]
           set yDim($specID)  [getDimKeyword $yName($specID) $fdata($specID)]
           set zDim($specID)  [getDimKeyword $zName($specID) $fdata($specID)]
           set aDim($specID)  [getDimKeyword $aName($specID) $fdata($specID)]

           set xSize($specID) [getParmI $fdata($specID) NDSIZE $xDim($specID)]
           set ySize($specID) [getParmI $fdata($specID) NDSIZE $yDim($specID)]
           set zSize($specID) [getParmI $fdata($specID) NDSIZE $zDim($specID)]
           set aSize($specID) [getParmI $fdata($specID) NDSIZE $aDim($specID)]

           if {$ySize($specID) == 1 && $zSize($specID) == 1 && $aSize($specID) == 1} \
              {
               set dimCount($specID) 1
              } \
           elseif {$ySize($specID) > 1 && $zSize($specID) == 1 && $aSize($specID) == 1} \
              {
               set dimCount($specID) 2
              } \
           elseif {$ySize($specID) > 1 && $zSize($specID) > 1 && $aSize($specID) == 1} \
              {
               set dimCount($specID) 3
              }
          }

       if {$xSize($specID) < 1} {set xSize($specID) 1}
       if {$ySize($specID) < 1} {set ySize($specID) 1}
       if {$zSize($specID) < 1} {set zSize($specID) 1}
       if {$aSize($specID) < 1} {set aSize($specID) 1}

       set fmt " %s: %s %-6s Size: %4d Limits: % 8.3fppm % 8.3fppm Tol: %5.3fppm"

       if {$dimCount($specID) > 0} \
          {
           set pxw            [spec2pntW $fdata($specID) $xDim($specID) $xw($specID)]
           set xwPPM($specID) [pnt2specW $fdata($specID) $xDim($specID) $pxw ppm]

           set xFirst($specID) \
            [pnt2spec $fdata($specID) $xDim($specID) 1 ppm]

           set xLast($specID)  \
            [pnt2spec $fdata($specID) $xDim($specID) $xSize($specID) ppm]

           if {![strcasecmp $xTol($specID) Auto]} \
              {
               set xTol($specID) [getPPMTol $xDim($specID) $fdata($specID)]
              }

           puts [format $fmt X $xDim($specID) $xName($specID) $xSize($specID) $xFirst($specID) $xLast($specID) $xTol($specID)] 
          }

       if {$dimCount($specID) > 1} \
          {
           set pyw             [spec2pntW $fdata($specID) $yDim($specID) $yw($specID)]
           set ywPPM($specID)  [pnt2specW $fdata($specID) $yDim($specID) $pyw ppm]

           set yFirst($specID) [pnt2spec $fdata($specID) $yDim($specID) 1 ppm]
           set yLast($specID)  [pnt2spec $fdata($specID) $yDim($specID) $ySize($specID) ppm]

           if {![strcasecmp $yTol($specID) Auto]} \
              {
               set yTol($specID) [getPPMTol $yDim($specID) $fdata($specID)]
              }

           puts [format $fmt Y $yDim($specID) $yName($specID) $ySize($specID) $yFirst($specID) $yLast($specID) $yTol($specID)] 
          }

       if {$dimCount($specID) > 2} \
          {
           set pzw             [spec2pntW $fdata($specID) $zDim($specID) $zw($specID)]
           set zwPPM($specID)  [pnt2specW $fdata($specID) $zDim($specID) $pzw ppm]

           set zFirst($specID) [pnt2spec $fdata($specID) $zDim($specID) 1 ppm]
           set zLast($specID)  [pnt2spec $fdata($specID) $zDim($specID) $zSize($specID) ppm]

           if {![strcasecmp $zTol($specID) Auto]} \
              {
               set zTol($specID) [getPPMTol $zDim($specID) $fdata($specID)]
              }

           puts [format $fmt Z $zDim($specID) $zName($specID) $zSize($specID) $zFirst($specID) $zLast($specID) $zTol($specID)] 
          }

       if {$dimCount($specID) > 3} \
          {
           set paw             [spec2pntW $fdata($specID) $aDim($specID) $aw($specID)]
           set awPPM($specID)  [pnt2specW $fdata($specID) $aDim($specID) $paw ppm]

           set aFirst($specID) [pnt2spec $fdata($specID) $aDim($specID) 1 ppm]
           set aLast($specID)  [pnt2spec $fdata($specID) $aDim($specID) $aSize($specID) ppm]

           if {![strcasecmp $aTol($specID) Auto]} \
              {
               set aTol($specID) [getPPMTol $aDim($specID) $fdata($specID)]
              }

           puts [format $fmt A $aDim($specID) $aName($specID) $aSize($specID) $aFirst($specID) $aLast($specID) $aTol($specID)] 
          }

      getThisDataHi $specID

      puts ""
     }

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       if {$dimCount($specID) > $maxDimCount} {set maxDimCount $dimCount($specID)}
      }
}

proc getThisDataHi { specID } \
{
   global hi mult inNameList pLev nLev

   set inName [lindex $inNameList [expr $specID - 1]]

   if {![strcasecmp $hi($specID) Auto]} \
      {
       set noise       [estSpecNoise $inName]
       set hi($specID) [expr $noise*$mult($specID)]

       if {$hi($specID) == 0.0} {set hi($specID) 1.0e+6}

       set hi($specID) [format %.3e $hi($specID)]

       puts "   Auto Mode Contour Height:     $hi($specID)"
      } \
   elseif {![strcasecmp $hi($specID) Full]} \
      {
       set range       [getSpecRange $inName minVal maxVal]
       set hi($specID) [getFirstLevel $pLev($specID) $nLev($specID) $minVal $maxVal 1.3]

       if {$hi($specID) == 0.0} {set hi($specID) 1.0e+6}

       set hi($specID) [format %.3e $hi($specID)]
       set str [format "%.2e (%.2e to %.2e)" $hi($specID) $minVal $maxVal]

       puts "   Full Mode Contour Height:     $str"
      } \
   elseif {[string last @ $hi($specID)] > 0} \
      {
       set pct [string trim $hi($specID) "@ "]

       set hiInfo      [getSpecContHi $inName $hi($specID) $pLev($specID) $nLev($specID) 7.0]
       set hi($specID) [lindex $hiInfo 0]

       set str [format "%.2e at %s%%" $hi($specID) $pct]

       puts "   Fraction Mode Contour Height: $str"
      } \
   else \
      {
       if {[string first % $hi($specID)] < 0} \
          {
           set hi($specID) [format %.3e $hi($specID)]
          }
      }

   return $hi($specID)
}

proc initPkTab {} \
{
   global pkInfo pkName specCount pkXVar pkYVar pkZVar

   set pkInfo(tabCount)  0
   set pkInfo(showShift) 1
   set pkInfo(showAss)   1
   set pkInfo(showLine)  0

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set pkInfo($specID,tab) 0

       if {![strcasecmp $pkName($specID) None]} continue

       if {![file exists $pkName($specID)]} \
          {
           puts "Error Finding Peak Table $pkName($specID)."
           exit 1
          }

       set pkInfo($specID,tab) [gdbRead table -in $pkName($specID)]

       if {![strcasecmp $pkXVar($specID) Auto]} \
          {
           set pkXVar($specID) [getHNTabVar $pkInfo($specID,tab)]
           puts "Auto Mode 3D HN Table Variable:     $pkXVar($specID)"
          }

       if {![strcasecmp $pkYVar($specID) Auto]} \
          {
           set pkYVar($specID) [get15NTabVar $pkInfo($specID,tab)]
           puts "Auto Mode 3D 15N Table Variable:    $pkYVar($specID)"
          }

       if {![strcasecmp $pkZVar($specID) Auto]} \
          {
           set pkZVar($specID) [getIndirectTabVar $pkInfo($specID,tab)]
           puts "Auto Mode 3D Z-Axis Table Variable: $pkZVar($specID)"
          }

       gdbSortTab $pkInfo($specID,tab) HEIGHT -down -abs

       incr pkInfo(tabCount)
      }
}

proc cScrollProc args \
{
   eval ".s set $args"
}

proc sScrollProc args \
{
   global thisInterp interpList linkFlag wxSize cx cursorInfo

   eval ".c xview $args"

   set xInfo [.c xview] 

   for {set n 1} {$n <= $cursorInfo(totalCount)} {incr n} \
      {
       set tID $cursorInfo($n,tID)
       set tx  $cursorInfo($n,tx)

       set cInfo [.c coords $tID]
 
       set thisX [expr $tx + int(double($wxSize)*[lindex $xInfo 0])] 
       set thisY [lindex $cInfo 1]

       .c coords $tID $thisX $thisY 
      }

   if {$linkFlag} \
      {
       foreach i $interpList \
          {
           set i [string trim $i "{}"]
           set error [catch {send $i eval ".c xview $args"}]
           if {$error} {updateLink $thisInterp}
          }
      }

   update
} 

proc updateLink args \
{
   global linkFlag thisInterp interpList 

   if {!$linkFlag} return
 
   set rootInterp [lindex $thisInterp 0]
   set interpList ""
   set sender     ""

   foreach i $args {lappend sender $i}

   set sender [string trim $sender "{}"]

   puts "Receiving Update ($thisInterp) Sender: $sender"

   foreach i [winfo interps] \
      {
       set i [string trim $i "{}"]

       puts "Testing $i"

       if {[strcmp $i $thisInterp] && [string first $rootInterp $i] >= 0} \
          {
           puts "Linking to $i"

           lappend interpList $i

           if {![strcmp $sender $thisInterp]} \
              {
               puts "Sending Update. Destination: $i"
               send $i updateLink $thisInterp
              }
          }
      }

   updateTitle
}

proc updateTitle {} \
{
   global thisVersion thisInterp interpList panelCount specCount inName stripsPerSpec
   global currentInfo linkFlag winName

   set str "Strip Panels: $panelCount Spectra: $specCount Views: $stripsPerSpec."

   append str " $thisInterp $thisVersion Ready."

   if {$linkFlag} \
      {
       if {[llength $interpList]} \
          {
           append str " Linked to: $interpList"
          } \
       else \
          {
           append str " Linked to: None"
          }
      } 

   wm title . "$winName $str"

   selectPanelID $currentInfo(panelID)

   update
}

proc findPanelIDByXPix { xPix } \
{
   global panelInfo panelList

   foreach panelID $panelList \
      {
       if {$xPix < $panelInfo($panelID,x1)} continue
       if {$xPix > $panelInfo($panelID,xn)} continue

       return $panelID
      }

    return -1
}

proc findCellIDByPix { xPix yPix } \
{
    global y1 yn panelList panelInfo cellInfo

    if {$yPix < $y1 || $yPix > $yn} {return -1}

    foreach panelID $panelList \
       {
        if {$xPix < $panelInfo($panelID,x1)} continue
        if {$xPix > $panelInfo($panelID,xn)} continue

        set x0 [expr $xPix - $panelInfo($panelID,x1)]

        foreach cellID $panelInfo($panelID,cellList) \
           {
            if {$x0 < $cellInfo($cellID,x1)} continue
            if {$x0 > $cellInfo($cellID,xn)} continue

            return $cellID
           }
       }

    return -1
}

proc findSlotID { panelID } \
{
   global panelList

   set loc [lsearch -exact $panelList $panelID]
   if {$loc < 0} {return -1}

   return [expr $loc + 1]
}

proc findPanelIDLoc { panelID } \
{
   global panelList

   set loc [lsearch -exact $panelList $panelID]
   return $loc
}

proc findPanelIDByAssID { assID } \
{
   global panelCount panelInfo

   for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
      {
       if {![strcmp $panelInfo($panelID,assID) $assID]} {return $panelID}
      }

   return 0 
}

proc findSlotIDByXPix { xPix } \
{
   global slotInfo panelCount dataValid

   if {!$dataValid} {return -1}

   for {set slotID 1} {$slotID <= $panelCount} {incr $slotID} \
      {
       if {$xPix >= $slotInfo($slotID,x1) && $xPix <= $slotInfo($slotID,xn)} \
          {
           return $slotID
          }
      }

   return -1
}

proc getFmt { val } \
{
   if {abs($val) < 10.0}  {return %.3f}
   if {abs($val) < 100.0} {return %.2f}

   return %.1f
}

proc adjustPPM { ppmLoc off } \
{
   if {$ppmLoc < 999.0} {set ppmLoc [expr $ppmLoc + $off]}

   return $ppmLoc
}

proc findPeakGroups {} \
{
    global panelInfo panelCount specCount xOff yOff yFirst yLast 
    global xVarName yVarName xw yw zw xwPPM ywPPM zwPPM title
    global pkXVar pkYVar pkZVar pkInfo xTol yTol zTol

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        set assID $panelInfo($panelID,assID)
        set xBase $panelInfo($panelID,xPPM)
        set yBase $panelInfo($panelID,yPPM)

        if {$pkInfo(tabCount)} \
           {
            if {$xBase < 999.0 && $yBase < 999.0} \
               {
                set fmt "Panel: %3d ID: %3d Location: %6.3f %7.3f ppm"
                puts [format $fmt $panelID $assID $xBase $yBase]
               }
           }

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            set pkInfo($panelID,$specID,all)      ""
            set pkInfo($panelID,$specID,near)     ""
            set pkInfo($panelID,$specID,unique)   ""
            set pkInfo($panelID,$specID,nearest)  ""

            if {$xBase >= 999.0 || $yBase >= 999.0} continue
            if {!$pkInfo($specID,tab)} continue

            set xVal [adjustPPM $xBase $xOff($specID)]
            set yVal [adjustPPM $yBase $yOff($specID)]

            set ySW  [expr $yFirst($specID) - $yLast($specID)]

            while {$yVal > $yFirst($specID)} {set yVal [expr $yVal - $ySW]}
            while {$yVal < $yLast($specID)}  {set yVal [expr $yVal + $ySW]}

            set    c "near( $pkXVar($specID), $xVal, $xwPPM($specID) ) && "
            append c "near( $pkYVar($specID), $yVal, $ywPPM($specID) )"

            set pkInfo($panelID,$specID,all) \
             [gdbSelect -from $pkInfo($specID,tab) -cond $c]

            set    c "near( $pkXVar($specID), $xVal, 2.0*$xTol($specID) ) && "
            append c "near( $pkYVar($specID), $yVal, 2.0*$yTol($specID) )"

            set pkInfo($panelID,$specID,near) \
             [gdbSelect -from $pkInfo($specID,tab) -cond $c]

            set    c "near( $pkXVar($specID), $xVal, $xTol($specID) ) && "
            append c "near( $pkYVar($specID), $yVal, $yTol($specID) )"

            set pkInfo($panelID,$specID,nearest) \
             [gdbSelect -from $pkInfo($specID,tab) -cond $c]
           }

        for {set specIDI 1} {$specIDI <= $specCount} {incr specIDI} \
           {
            foreach eI $pkInfo($panelID,$specIDI,near) \
               {
                set zI [gdbGet $eI $pkZVar($specIDI)]
                set nn 0

                for {set specIDJ 1} {$specIDJ <= $specCount} {incr specIDJ} \
                   {
                    set tol [getMax "$zTol($specIDI) $zTol($specIDJ)"]

                    foreach eJ $pkInfo($panelID,$specIDJ,near) \
                       {
                        if {$eI == $eJ} continue

                        set zJ [gdbGet $eJ $pkZVar($specIDJ)]

                        if {abs($zI - $zJ) <= $tol} \
                           {
                            incr nn
                            break
                           }
                       }

                    if {$nn} break
                   }

                if {!$nn} \
                   {
                    lappend pkInfo($panelID,$specIDI,unique) $eI
                   }
               }

            set n1 [llength $pkInfo($panelID,$specIDI,all)]
            set n2 [llength $pkInfo($panelID,$specIDI,near)]
            set n3 [llength $pkInfo($panelID,$specIDI,nearest)]
            set n4 [llength $pkInfo($panelID,$specIDI,unique)]

            if {!$n1} continue

            set fmt "  %10s Peaks All: %3d Near: %3d Nearest: %3d Unique: %3d"
            puts [format $fmt $title($specIDI) $n1 $n2 $n3 $n4]
           }

       }
}

proc readCurrent {} \
{
   global currentInfo viewList panelInfo panelCount specCount r1 rN y1 yn dx dy bColor

   set panelID $currentInfo(panelID)
   set slotID  $currentInfo(slotID)
   set assID   $currentInfo(assID)
   set specID  $currentInfo(specID)
   set resID   $currentInfo(resID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount
   iClipC resID    $r1 $rN


   foreach viewID $viewList \
      {
#      puts "Reading Strip for Panel $panelID Spectrum $specID View $viewID"
       readThisData $panelID $specID $viewID
      }
}

proc readData {} \
{
    global panelCount specCount viewList dataValid winName

    set dataValid  0

    busy hold .

    set id 0

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        if {!($panelID % 10) || $panelID == 1 || $panelID == $panelCount} \
           {
            wm title . "$winName Reading Strips for 2D Position $panelID of $panelCount."
            update
           }

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            foreach viewID $viewList \
               {
                readThisData $panelID $specID $viewID
               }
           }
       }

   busy release .
   update
}

proc readThisData { panelID specID viewID } \
{
    global panelInfo panelCount roiInfo roiCount viewList dataValid verbose
    global xw yw zw xww yww zww aww z1 zn specCount dimCount xOff yOff
    global inNameList xName yName zName aName xVarName yVarName 

    if {$verbose} \
       {
        set vArg verb
       } \
    else \
       {
        set vArg noverb
       }

    if {$panelID < 1 || $panelID > $panelCount} return
    if {$specID  < 1 || $specID  > $specCount}  return
    if {$viewID  < 1 || $viewID  > 2} return

    if {$panelInfo($panelID,xPPM) >= 999.0} return
#   if {$panelInfo($panelID,yPPM) >= 999.0} return

    set xVal [adjustPPM $panelInfo($panelID,xPPM) [getCurrentVal xOff $specID $panelID]]
    set yVal [adjustPPM $panelInfo($panelID,yPPM) [getCurrentVal yOff $specID $panelID]]

    set xPPM ${xVal}ppm
    set yPPM ${yVal}ppm

    set inName [lindex $inNameList [expr $specID - 1]]

    if {$roiInfo($panelID,$specID,$viewID,roiID)} \
       {
        set roiID $roiInfo($panelID,$specID,$viewID,roiID)
       } \
    else \
       {
        incr roiCount

        set roiID $roiCount
        set roiInfo($panelID,$specID,$viewID,roiID) $roiID
       }

# %%%  check -w

    set thisZ1 [getCurrentVal z1 $specID $panelID] 
    set thisZN [getCurrentVal zn $specID $panelID] 

    if {$dimCount($specID) == 1} \
       {
       } \
    elseif {$dimCount($specID) == 2} \
       {
        evalCmnd readROI -in $inName -roi $roiID -ndim 2 \
                         -w  $xName($specID) $xww -center X \
                         -dx $xName($specID) $xPPM [getCurrentVal xw $specID $panelID] \
                         -y  $yName($specID) $thisZ1 $thisZN -$vArg

       } \
    elseif {$dimCount($specID) == 3} \
       {
        if {$viewID == 1} \
           {
            evalCmnd readROI -in $inName -roi $roiID -ndim 2 \
                             -w  $xName($specID) $xww $yName($specID) $yww -center X \
                             -dx $xName($specID) $xPPM [getCurrentVal xw $specID $panelID] \
                             -dz $yName($specID) $yPPM [getCurrentVal zw $specID $panelID] \
                             -y  $zName($specID) $thisZ1 $thisZN -$vArg
           } \
        else \
           {
            evalCmnd readROI -in $inName -roi $roiID -ndim 2 \
                             -w  $xName($specID) $xww $yName($specID) $yww -center X \
                             -dz $xName($specID) $xPPM [getCurrentVal zw $specID $panelID] \
                             -dx $yName($specID) $yPPM [getCurrentVal yw $specID $panelID] \
                             -y  $zName($specID) $thisZ1 $thisZN -$vArg
           }
       } \
    else \
       {
        evalCmnd readROI -in $inName -roi $roiID -ndim 2 \
                         -w  $zName($specID) $zww(def) $aName($specID) $aww(def) \
                         -dz $xName($specID) $xPPM 0 -da $yName($specID) $yPPM 0 \
                         -x  $zName($specID) 0% 100% -y  $aName($specID) 0% 100% -$vArg
       }
}

proc createSlots {} \
{
   global slotInfo panelCount wxSizeP wySizeP

   set mapX1 0
   set mapY1 0

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
       {
        set mapXN [expr int($mapX1 + $wxSizeP - 1)]
        set mapYN [expr int($mapY1 + $wySizeP - 1)]

        set slotInfo($slotID,x1)   $mapX1
        set slotInfo($slotID,xn)   $mapXN
        set slotInfo($slotID,y1)   $mapY1
        set slotInfo($slotID,yn)   $mapYN

        set mapX1 [expr $mapXN + 1]
       }
}

proc createPanels {} \
{
    global panelCount specCount viewList dataValid
    global wxSize wySize wxSizeP wySizeP wxSizeMax wColor 
    global dx dy qx1 qx2 qx3 cx cy y1 yn dataValid
    global slotInfo panelInfo cellInfo cellList cellCount

    set dataValid 0

    busy hold .

    set cellIDOffset [expr 1000 + 4*$panelCount*$specCount]

    set slotID 0
    set error  0

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        incr slotID

        set mapX1 $slotInfo($slotID,x1)
        set mapXN $slotInfo($slotID,xn)
        set mapY1 $slotInfo($slotID,y1)
        set mapYN $slotInfo($slotID,yn)

        set mapXN [expr int($mapX1 + $wxSizeP - 1)]
        set mapYN [expr int($mapY1 + $wySizeP - 1)]

        set pID [.c create pixmap $mapX1 $mapY1 -width $wxSizeP -height $wySizeP -win $panelID -tags panel$panelID]

        .c create rectangle $mapX1 $mapY1 $mapXN $mapYN -outline $wColor -fill $wColor -tag flushWin

        drawRec -x 0 $wxSizeP -y 0 $wySize -fg $wColor -win $panelID

        update

        set xyList [.c coords $pID]

        set panelInfo($panelID,cellList) ""
        set panelInfo($panelID,xLimits)  ""
        set panelInfo($panelID,yLimits)  ""

        set panelInfo($panelID,pID)  $pID

        set panelInfo($panelID,x0) [expr int([lindex $xyList 0])]
        set panelInfo($panelID,y0) [expr int([lindex $xyList 1])]

        set panelInfo($panelID,x1) $mapX1
        set panelInfo($panelID,xn) $mapXN
        set panelInfo($panelID,y1) $mapY1
        set panelInfo($panelID,yn) $mapYN

        set panelInfo($panelID,width)  $wxSizeP
        set panelInfo($panelID,height) $wySizeP

        if {$panelInfo($panelID,x0) - $mapX1 != 0.0} {incr error}
        if {$panelInfo($panelID,y0) - $mapY1 != 0.0} {incr error}

        set x1 $qx2 
        set xn [expr $x1 + $dx]

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            foreach viewID $viewList \
               {
                if {$cellInfo($panelID,$specID,$viewID,cellID)} \
                   {
                    set cellID     cellInfo($panelID,$specID,$viewID,cellID)
                    set plotCellID cellInfo($panelID,$specID,$viewID,plotCellID)
                   } \
                else \
                   {
                    incr cellCount
                    set  cellID $cellCount
                    set  cellInfo($panelID,$specID,$viewID,cellID) $cellID

                   set  plotCellID [expr $cellID + $cellIDOffset]
                   set  cellInfo($panelID,$specID,$viewID,plotCellID) $plotCellID
                   }

                defineCell -x $x1 $xn -y $y1 $yn -cell $cellID -win $panelID

                lappend panelInfo($panelID,cellList) $cellID
                lappend panelInfo($panelID,xLimits)  $x1 $xn
                lappend panelInfo($panelID,yLimits)  $y1 $yn

                lappend cellList $cellID
                lappend cellList $plotCellID

                set cellInfo($cellID,panelID) $panelID
                set cellInfo($cellID,specID)  $specID
                set cellInfo($cellID,viewID)  $viewID
                set cellInfo($cellID,winID)   $panelID

                set cellInfo($cellID,x1)      $x1
                set cellInfo($cellID,xn)      $xn
                set cellInfo($cellID,y1)      $y1
                set cellInfo($cellID,yn)      $yn

                set x1 [expr $xn + $qx1]
                set xn [expr $x1 + $dx]
               }

            set x1 [expr $x1 + $qx2]
            set xn [expr $x1 + $dx]
           }

        incr mapX1 $wxSizeP
       }

    .c delete flushWin

    busy release .
    update

   if {$error} \
      {
       showError Scroll "Warning: Change in requested panel positions."
      }
}

proc initSelectors {} \
{
   global slotInfo panelInfo panelCount resList r1 rN
   global wySize sColor aColor scrollFont y1 yn cx cy hColor

   set lw       [expr $cy/3]
   set qy       [expr $yn + 7*$cy/3 + $cy + $lw + 2]
   set qy1      [expr $qy - $lw + 1]
   set qyn      [expr $qy1 + 2*$lw - 1]
   set qym      [expr ($qy1 + $qyn)/2]
   set qya      [expr $wySize - $lw/2]
   set qyt      [expr $wySize + $lw/2 - 3]

   set prevXN  -1
   set panelID 0

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       incr panelID

       set  labState normal

       set ix1 $slotInfo($slotID,x1)
       set ixn $slotInfo($slotID,xn)
      
       if {$ixn - $ix1 > 4*$cx} \
          {
           set ix1 [expr int(($ix1 + 2*$cx) - $cx/4)]
           set ixn [expr int(($ixn - 2*$cx) - $cx/4)]
          }

       set vx  [expr $ix1 + 2*$cx/3 - 1]
       set ax  [expr $ix1 + 4*$cx/3 - 1]
       set ayn [expr $wySize - 4]
       set ay1 [expr $ayn - 2*$lw - 1]

       if {$panelID <= $panelCount} \
          {
           set sID \
            [.c create rectangle $ix1 $qy1 $ixn $qyn -state $labState \
                -tags "selPanelID sel$panelID panel$panelID" \
                -fill $sColor -outline $hColor]

           set panelInfo($panelID,selMarker)  $sID

           .c bind $sID <ButtonRelease-1> "+ selectPanelID $panelID"
          }

       set resID   [expr $slotID + $r1 - 1]
       set resName [lindex $resList [expr $resID - $r1]]
       set str     "$resID $resName"

       if {$panelInfo($panelID,fixed)} \
          {
           set vState normal
          } \
       else \
          {
           set vState hidden
          }

       set aID [.c create rectangle $ix1 $ay1 $ax $ayn -tags anc$slotID \
                   -state $labState -fill $sColor -outline $hColor] 

       set vID [.c create line $vx [expr $qyn + 1] $vx $ay1 \
                   -tags vanc$slotID -fill $sColor -width 3 -state $vState]

       set rID [.c create text [expr $ix1 + 2*$cx] $qyt \
                   -fill $aColor -font $scrollFont -anchor sw -state normal \
                   -tags "resID resLabel$resID slotLabel$slotID" -text $str]

       set slotInfo($slotID,anchor)     $aID
       set slotInfo($slotID,vanchor)    $vID
       set slotInfo($slotID,resText)    $rID
       set slotInfo($slotID,joinMarker) None

       .c bind $rID <ButtonRelease-1> "+ selectSlotID $slotID"
       .c bind $aID <ButtonRelease-1> "+ toggleAnchor $slotID"

       if {$prevXN >= 0} \
          {
           set thisX1 [expr int($prevXN + 0.333*(1 + $ix1 - $prevXN))]
           set thisXN [expr int($prevXN + 0.666*(1 + $ix1 - $prevXN))]
           set thisID [expr $slotID - 1]

           set lID [.c create line [expr $prevXN + 1] $qym $ix1 $qym \
                       -fill $sColor -width 3 -state hidden \
                       -tags join$slotID]

           set bID [.c create rectangle $thisX1 $qy1 $thisXN $qyn \
                       -fill $sColor -outline $hColor -state $labState \
                       -tags join$slotID]

           set slotInfo($slotID,joinMarker) $lID

           .c bind $bID <ButtonRelease-1> "+ toggleJoin $slotID"
          }

       set prevXN $ixn
      }

   selectPanelID 1
   selectSpecID  1 1
   selectSlotID  1
}

proc selectSlotID { slotID } \
{
   global r1 rN currentInfo rColor aColor slotInfo panelCount

   if {$slotID < 1} {set slotID $panelCount}
   if {$slotID > $panelCount} {set slotID 1}

   set resID [expr $slotID + $r1 - 1]

   setCurrent slotID val $slotID
   setCurrent resID  val $resID

   hiliteResID $resID
}

proc hiliteResID { resID } \
{
   global r1 rN currentInfo rColor aColor slotInfo panelCount

   .c itemconfigure resID -fill $aColor
   .c itemconfigure resLabel$resID -fill $rColor
   update
}

proc selectSpecID { panelID specID } \
{
   global aColor slotInfo currentInfo panelCount specCount

   if {$panelID < 1 || $panelID > $panelCount} return
   if {$specID  < 1 || $specID  > $specCount}  return

   setCurrent specID val $specID

   selectPanelID $panelID
}

proc hiliteSpecID { panelID specID } \
{
   global panelCount specCount panelInfo xColor

   if {$panelID < 1 || $panelID > $panelCount} return
   if {$specID  < 1 || $specID  > $specCount}  return

   set sID $panelInfo($panelID,$specID,xText)
   if {[strcasecmp $sID None]} {.c itemconfigure $sID -fill $xColor}

   set sID $panelInfo($panelID,$specID,yText)
   if {[strcasecmp $sID None]} {.c itemconfigure $sID -fill $xColor}
}

proc selectPanelID { panelID } \
{
   global panelInfo currentInfo panelCount sColor hColor aColor

   if {$panelID < 1 || $panelID > $panelCount} return

   set thisID $panelInfo($panelID,assID)

   setCurrent panelID val $panelID
   setCurrent assID   val $thisID

   hilitePanelID $panelID
}

proc hilitePanelID { panelID } \
{
   global panelInfo currentInfo panelCount sColor hColor aColor

   .c itemconfigure selPanelID -fill $sColor -outline $hColor 
   .c itemconfigure selSpecID  -fill $aColor

   hiliteSel    $panelID
   hiliteSpecID $panelID $currentInfo(specID)

   update
}

proc hiliteSel { panelID } \
{
   global xColor sColor aColor hColor currentInfo panelInfo panelCount 

   if {$panelID < 1 || $panelID > $panelCount} return

   set id $panelInfo($panelID,selMarker)

   if {[strcasecmp $id None]} \
      {
       .c itemconfigure $id -fill $hColor -outline $hColor
      }

   set id $panelInfo($panelID,text)

   if {[strcasecmp $id None]} \
      {
       .c itemconfigure $id -fill $xColor 
      }
}

proc isAnchored { panelIDList } \
{
   global panelInfo panelCount

   foreach panelID $panelIDList \
      {
       if {$panelID < 1 || $panelID > $panelCount} continue 

       if {$panelInfo($panelID,fixed)} {return 1}

       set jList [findJoinList $panelID]

       foreach thisID $jList \
          {
           if {$panelInfo($thisID,fixed)} {return 1}
          }
      }

   return 0
}

proc toggleAnchor { slotID } \
{
   toggleAnchor2 $slotID
   storeHistory
}

proc toggleAnchor2 { slotID } \
{
   global panelList panelInfo panelCount sColor

   if {$slotID < 1 || $slotID > $panelCount} return

   set panelID [lindex $panelList [expr $slotID - 1]]

   if {$panelInfo($panelID,fixed)} \
      {
       setPanelAnchor $panelID 0
      } \
   else \
      {
       setPanelAnchor $panelID 1
      }
}

proc setPanelAnchor { panelID fixed } \
{
   global panelInfo panelCount

   if {$panelID < 1 || $panelID > $panelCount} return

   set jList [findJoinList $panelID]

   foreach thisID $jList \
      {
       set panelInfo($thisID,fixed) $fixed 
       set slotID [findSlotID $thisID]      
 
       refreshThisAnchor $slotID
      }

   update
}

proc refreshThisAnchor { slotID } \
{
   global panelInfo slotInfo panelCount panelList 

   if {$slotID < 1 || $slotID > $panelCount} return
   if {![strcasecmp $slotInfo($slotID,vanchor) None]} return 
   
   set panelID [lindex $panelList [expr $slotID - 1]]

   if {$panelInfo($panelID,fixed)} \
      {
       set labState normal
      } \
   else \
      {
       set labState hidden 
      }

   .c itemconfigure $slotInfo($slotID,vanchor) -state $labState
}

proc toggleJoin { slotID } \
{
   toggleJoin2 $slotID
   storeHistory
}

proc toggleJoin2 { slotID } \
{
   global panelInfo panelList panelCount

   if {$slotID < 2 || $slotID > $panelCount} return

   set thisLoc [expr $slotID - 1]
   set panelID [lindex $panelList $thisLoc]

   if {![info exists panelInfo($panelID,assID)]} return

   if {$panelInfo($panelID,prev) > 0} \
      {
       set prevPanelID $panelInfo($panelID,prev)

       set panelInfo($panelID,prev)     -1
       set panelInfo($prevPanelID,next) -1
      } \
   else \
      {
       set prevLoc [expr $thisLoc - 1]
       set prevPanelID [lindex $panelList $prevLoc]

       set panelInfo($panelID,prev)     $prevPanelID
       set panelInfo($prevPanelID,next) $panelID

#      puts stderr "Joining $panelInfo($prevPanelID,assID) with $panelInfo($panelID,assID)"
      }

   refreshJoin $slotID 

   if {[isAnchored $panelID]} {setPanelAnchor $panelID 1}

   update
}

proc refreshJoin { slotID } \
{
   global slotInfo panelInfo panelList panelCount

   if {$slotID < 1 || $slotID > $panelCount} return 

   set panelID [lindex $panelList [expr $slotID - 1]]
   set lineID  $slotInfo($slotID,joinMarker)

   if {![strcasecmp $lineID None]} return

   if {$panelInfo($panelID,prev) > 0} \
      {
       set labState normal
      } \
   else \
      {
       set labState hidden 
      }

   .c itemconfigure $lineID -state $labState 
}
 
proc initStripDisplay {} \
{
    global y1 yn q1 qn dx viewList dataValid wxSize wySize bColor
    global panelList panelInfo panelCount specCount winName

    set dataValid  0

    busy hold .

    wm title . "$winName Drawing Strips ..."
    update

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            if {!($panelID % 10) || $panelID == 1 || $panelID == $panelCount} \
               {
                wm title . "$winName Drawing Strips for Entry $panelID of $panelCount."
                update
               }

            foreach viewID $viewList \
               {
                drawThisData  $panelID $specID $viewID
                drawDataPeaks $panelID $specID $viewID

                hiliteSel $panelID

                .c xview moveto 0
               }
           }

        if {[strcasecmp $bColor None]} \
           {
            set gx1 [lindex $panelInfo($panelID,xLimits) 0]
            set gxn [lindex $panelInfo($panelID,xLimits) end]

            drawLine -x [expr $gx1 + $dx/4] [expr $gxn - $dx/4] -y $yn $yn -fg $bColor -win $panelID
           }
       }

   set dataValid 1

   updateTitle

   busy release .
   update
}
 
proc plotAllStrips {} \
{
    global y1 yn q1 qn dx viewList dataValid wxSize wySize bColor
    global plotPageCount plotGroupCount plotStripCount plotOrient plotPaper
    global panelList panelInfo panelCount specCount winName

    set plotPageCount  0
    set plotGroupCount 0
    set plotStripCount 0

    busy hold .

    wm title . "$winName Generating Strip Hardcopy ..."
    update

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            if {!($panelID % 10) || $panelID == 1 || $panelID == $panelCount} \
               {
                wm title . "$winName Plotting Strips for Entry $panelID of $panelCount."
                update
               }

            foreach viewID $viewList \
               {
                plotThisData  $panelID $specID $viewID
                plotDataPeaks $panelID $specID $viewID

                hiliteSel $panelID

                .c xview moveto 0
               }
           }
       }

   set dataValid 1

   updateTitle

   busy release .
   update
}

proc drawCurrent {} \
{
   global currentInfo viewList panelInfo panelCount specCount r1 rN y1 yn dx dy bColor

   hiliteCurrent

   set panelID $currentInfo(panelID)
   set slotID  $currentInfo(slotID)
   set assID   $currentInfo(assID)
   set specID  $currentInfo(specID)
   set resID   $currentInfo(resID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount
   iClipC resID    $r1 $rN

#  puts "Drawing Strips for Panel $panelID Spectrum $specID"

   foreach viewID $viewList \
      {
       drawThisData $panelID $specID $viewID
      }

   if {[strcasecmp $bColor None]} \
      {
       set gx1 [lindex $panelInfo($panelID,xLimits) 0]
       set gxn [lindex $panelInfo($panelID,xLimits) end]

       drawLine -x [expr $gx1 + $dx/4] [expr $gxn - $dx/4] -y $yn $yn -fg $bColor -win $panelID
      }

   hiliteSel     $panelID
   hiliteSpecID  $panelID $specID
   hilitePanelID $panelID
}

proc updatePeakLabels {} \
{
    global panelInfo panelCount specCount viewList
    global dataValid winName

    if {!$dataValid} return

    busy hold .

    wm title . "$winName Drawing Peaks ..."
    update

    set prevPanel 0

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        set drawID 0

        clearPeaks $panelID

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            foreach viewID $viewList \
               {
                incr drawID

                if {$panelID != $prevPanel} \
                   {
                    if {!($panelID % 10) || \
                         $panelID == 1 || $panelID == $panelCount} \
                       {
                        wm title . "$winName Drawing Peaks for 2D Position $panelID of $panelCount."

                        set prevPanel $panelID
                        update
                       }
                   }

                drawDataPeaks $panelID $specID $viewID
               }
           }
       }

   updateTitle

   busy release .
   update
}

proc clearPeaks { panelID } \
{
   catch {.c delete pkLab$panelID pkLine$panelID}
}

proc clearLabels { panelID } \
{
   global panelInfo specCount

   catch {.c delete assLab$panelID stripLab$panelID axisLab$panelID}

   set panelInfo($panelID,text) None

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set panelInfo($panelID,$specID,xText) None
       set panelInfo($panelID,$specID,yText) None
       set panelInfo($panelID,$specID,eText) None
      }
}

proc drawDataPeaks { panelID specID viewID } \
{
   global pkInfo pkXVar pkYVar pkZVar assVar aColor dx dy cx cy qx1 scrollFont
   global slotInfo cellInfo roiInfo viewList
 
   if {![info exists cellInfo($panelID,$specID,$viewID,cellID)]} \
      {
#      puts "Peak: No Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set cellID $cellInfo($panelID,$specID,$viewID,cellID)

   if {!$cellID} \
      {
#      puts "Peak: Null Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   if {![info exists roiInfo($panelID,$specID,$viewID,roiID)]} \
      {
#      puts "Peak: No ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set roiID $roiInfo($panelID,$specID,$viewID,roiID)

   if {!$roiID} \
      {
#      puts "Peak: Null ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set nV [llength $viewList]

   if {$pkInfo(showShift) || $pkInfo(showAss)} \
      {
       set labState normal
      } \
   else \
      {
       set labState hidden
      }

   if {$pkInfo(showLine)} \
      {
       set lineState normal
      } \
   else \
      {
       set lineState hidden
      }

   foreach e $pkInfo($panelID,$specID,near) \
      {
       set xPPM [gdbGet $e $pkXVar($specID)]
       set yPPM [gdbGet $e $pkYVar($specID)]
       set zPPM [gdbGet $e $pkZVar($specID)]
       set ass  [gdbGet $e $assVar($specID)]

       if {$pkInfo(showShift)} \
          {
           if {$pkInfo(showAss) && [strcasecmp $ass None]} \
              {
               set str "$zPPM=$ass"
              } \
           else \
              {
               set str $zPPM
              }  
          } \
       else \
          {
           if {$pkInfo(showAss) && [strcasecmp $ass None]} \
              {
               set str "$ass"
              } \
           else \
              {
               set str "" 
              }
          }

       if {$viewID == 1} \
          {
           set xPix [spec2pix -roi $roiID -cell $cellID -x ${xPPM}ppm -nofold]

           set yPix [spec2pix -roi $roiID -cell $cellID -y ${zPPM}ppm -nofold]
          } \
       else \
          {
           set xPix [spec2pix -roi $roiID -cell $cellID -x ${yPPM}ppm -nofold]

           set yPix [spec2pix -roi $roiID -cell $cellID -y ${zPPM}ppm -nofold]
          }

       set xInfo [lindex [getCellInfo -cell $cellID] 1]
       set x1Pix [expr [lindex $xInfo 0] + 1]
       set xnPix [expr [lindex $xInfo 1] - 1]

       set xPix  [expr int($xPix  + $slotInfo($panelID,x1))]
       set x1Pix [expr int($x1Pix + $slotInfo($panelID,x1))]
       set xnPix [expr int($xnPix + $slotInfo($panelID,x1))]

       set yPix  [expr int($yPix  + $slotInfo($panelID,y1))]
       
       if {$nV == 1} \
          {
           set tID \
            [.c create text $xPix [expr $yPix - $cy/2]    \
                -fill $aColor -anchor s -font $scrollFont    \
                -tags "pkLab pkLab$panelID panel$panelID" \
                -text $str -state $labState] 
          } \
       elseif {$nV == 2 && $viewID == 1} \
          {
           set tID \
            [.c create text [expr $xnPix + $qx1] [expr $yPix - $cy/2] \
                -fill $aColor -anchor s -font $scrollFont -text $str     \
                -tags "pkLab pkLab$panelID panel$panelID" -state $labState]
          }

       set lID \
        [.c create line $x1Pix $yPix [expr $x1Pix + $dx/4] $yPix \
            -tags "pkLine pkLine$panelID panel$panelID" -fill $aColor \
            -state $lineState]

       set rID \
        [.c create line [expr $xnPix - $dx/4] $yPix $xnPix $yPix \
            -tags "pkLine pkLine$panelID panel$panelID" -fill $aColor \
            -state $lineState]
      }
}

proc plotThisData { panelID specID viewID } \
{
   global pHue1 pSat1 pVal1 pHueN pSatN pValN nHue1 nSat1 nVal1 nHueN nSatN nValN posColorCount negColorCount
   global xGrid yGrid pLev nLev hi viewList cx cy scrollFont wxSizeP wySizeP dataValid
   global pColor nColor bgColor fgColor aColor gColor lColor wColor eColorSet 
   global outName xName yName xVarName yVarName xOff yOff labelVar title
   global validInfo slotInfo panelInfo cellInfo roiInfo dimCount

   global plotFlag plotName plotPageCount plotStripCount plotGroupCount plotOrient plotPaper plotTray sdx sdy sgap ggap

   if {![info exists cellInfo($panelID,$specID,$viewID,plotCellID)]} \
      {
       puts "Plot: No Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   if {![info exists roiInfo($panelID,$specID,$viewID,roiID)]} \
      {  
       puts "Plot: No ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set cellID $cellInfo($panelID,$specID,$viewID,plotCellID)
   set roiID  $roiInfo($panelID,$specID,$viewID,roiID)

   if {!$cellID} \
      {
       puts "Plot: Null Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   if {!$roiID} \
      {
       puts "Plot: Null ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set plotX1 [expr 0.5*$sdx + ($sdx + $sgap)*$plotStripCount]
   set plotXN [expr $plotX1 + $sdx]

   if {$plotXN > 100.0 - 0.5*$sdx} \
      {
       set plotStripCount 0

       set plotX1 [expr 0.5*$sdx + ($sdx + $sgap)*$plotStripCount]
       set plotXN [expr $plotX1 + $sdx] 
      }
   
   if {$plotStripCount == 0} \
      {
       incr plotPageCount

       if {[strcasecmp $plotName None]} \
          {
           puts plotClose
           plotClose

           set plotName None
          }

       set plotName [format roi%03d.ps $plotPageCount]

       puts "plotOpen -color -plotName $plotName -paper $plotPaper -$plotOrient -tray $plotTray"

       plotOpen -color -plotName $plotName -paper $plotPaper -$plotOrient -tray $plotTray
      }

   incr plotStripCount
   
   defineCell -x ${plotX1}% ${plotXN}% -y 5% [expr 5.0 + $sdy]% -cell $cellID

   set firstView [lindex $viewList 0]
   set nV        [llength $viewList]
   set entry     $panelInfo($panelID,entry)

   set cInfo [getCellInfo -cell $cellID]
   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set xMid  [expr 0.5*([lindex $xInfo 0] + [lindex $xInfo 1])]
   set yTop  [expr [lindex $yInfo 0] - 2]
   set yBot  [expr [lindex $yInfo 1] + 7*$cy/3]

   set xMid [expr int($xMid + $slotInfo($panelID,x1))]

   set thisPosColor    [getCurrentVal pColor $specID $panelID]
   set thisNegColor    [getCurrentVal nColor $specID $panelID]

   set thisPosColorArg ""
   set thisNegColorArg ""

   if {![strcasecmp $thisPosColor Multi]} \
      {
       set thisPosColorArg "-pHueN $pHue1 -pSatN $pSat1 -pValN $pVal1 -pHueN $pHue1 -pSatN $pSat1 -pValN $pVal1 -pcc $posColorCount"
      }

   if {![strcasecmp $thisNegColor Multi]} \
      {
       set thisNegColorArg "-nHue1 $nHue1 -nSat1 $nSat1 -nVal1 $nVal1 -nHueN $nHueN -nSatN $nSatN -nValN $nValN -ncc $negColorCount"
      }

   if {$viewID == 1} \
      {   
       if {$roiID} \
          {
           set xVal [adjustPPM $panelInfo($panelID,xPPM) $xOff($specID)]
           set fmt  [getFmt $xVal]

           plotShadeFold $roiID $cellID $specID

           if {$dimCount($specID) == 4} \
              {
               set ticInfo None
              } \
           else \
              {
               set ticInfo Bot,fmt=$fmt,lab=None
              }

           eval drawROI -roi $roiID -cell $cellID \
            -color1 $thisPosColor -color2 $thisNegColor \
            -plev   [getCurrentVal pLev   $specID $panelID] \
            -nlev   [getCurrentVal nLev   $specID $panelID] \
            -hi     [getCurrentVal hi     $specID $panelID] \
            -tic $ticInfo -$xGrid -$yGrid -nobox $thisPosColorArg $thisNegColorArg -lwPlus 1 -lwMinus 1 -lwTic 1 -lw2D 1

           if {$dimCount($specID) == 4} \
              {
               set xVal [format %.3f [adjustPPM $panelInfo($panelID,xPPM) [getCurrentVal xOff $specID $panelID]]]
               set yVal [format %.3f [adjustPPM $panelInfo($panelID,yPPM) [getCurrentVal yOff $specID $panelID]]]

               set xPPM ${xVal}ppm
               set yPPM ${yVal}ppm

#              set tID \
#               [.c create text $xMid $yBot -fill $aColor -font $scrollFont \
#                   -anchor n -text "$xName($specID)=$xPPM $yName($specID)=$yPPM" -state $labState \
#                   -tags "selSpecID axisLab$panelID panel$panelID"]
              } \
           else \
              {
#              set tID \
#               [.c create text $xMid $yBot -fill $aColor -font $scrollFont \
#                   -anchor n -text $xName($specID) -state $labState        \
#                   -tags "selSpecID axisLab$panelID panel$panelID"]
              }

           plotBlankFold $roiID $cellID $specID
          }

        drawBox -cell $cellID -x 0% 100% -y 0% 100% -fg $fgColor($specID)
       }

   if {$viewID == 2} \
      {   
       if {$roiID} \
          {
           set yVal [adjustPPM $panelInfo($panelID,yPPM) $yOff($specID)]
           set fmt  [getFmt $yVal]

#          plotShadeFold $roiID $cellID $specID

           eval drawROI -roi $roiID -cell $cellID \
            -color1 $thisPosColor -color2 $thisNegColor \
            -plev   [getCurrentVal pLev   $specID $panelID] \
            -nlev   [getCurrentVal nLev   $specID $panelID] \
            -hi     [getCurrentVal hi     $specID $panelID] \
            -$xGrid -$yGrid -nobox -tic Bot,fmt=$fmt,lab=None $thisPosColorArg $thisNegColorArg -lwPlus 1 -lwMinus 1 -lwTic 1 -lw2D 1

#          plotBlankFold $roiID $cellID $specID
          } 

       drawBox -cell $cellID -x 0% 100% -y 0% 100% -fg $fgColor($specID)
      }

   if {[strcasecmp $labelVar None]} \
      {
       if {$viewID == $firstView && $specID == 1} \
          {
#          set slID \
#           [.c create text $xMid $yTop -fill $aColor \
#               -state $labState -font $scrollFont -anchor s \
#               -tags "assIDLab$panelID panel$panelID"       \
#               -text $panelInfo($panelID,label)]
          }
      }

   if {[strcasecmp $title($specID) None]} \
      {
       if {$viewID == $firstView} \
          {
           if {$specID*$nV % 2} \
              {
#              set stID \
#               [.c create text $xMid [expr $yTop - 2*$cy] \
#                   -fill $lColor($specID) -anchor s -text $title($specID)   \
#                   -tags "stripLab$panelID panel$panelID" -font $scrollFont \
#                   -state $labState]
              } \
           else \
              {
#              set stID \
#               [.c create text $xMid [expr $yTop - $cy] \
#                   -fill $lColor($specID) -anchor s -text $title($specID)   \
#                   -tags "stripLab$panelID panel$panelID" -font $scrollFont \
#                   -state $labState]
              }
          }
     }

   if {!($cellID % 10)} update
}

proc plotShadeFold args \
{
}

proc plotBlankFold args \
{
}

proc plotDataPeaks args \
{
}

proc drawThisData { panelID specID viewID } \
{
   global xGrid yGrid pLev nLev hi viewList cx cy scrollFont wxSizeP wySizeP dataValid
   global outName xName yName xVarName yVarName xOff yOff labelVar title
   global pColor nColor bgColor fgColor aColor gColor lColor wColor eColorSet 
   global validInfo slotInfo panelInfo cellInfo roiInfo dimCount
   global pHue1 pSat1 pVal1 pHueN pSatN pValN nHue1 nSat1 nVal1 nHueN nSatN nValN posColorCount negColorCount

   if {![info exists cellInfo($panelID,$specID,$viewID,cellID)]} \
      {
#      puts "Draw: No Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set cellID $cellInfo($panelID,$specID,$viewID,cellID)

   if {!$cellID} \
      {
#      puts "Draw: Null Cell for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   if {![info exists roiInfo($panelID,$specID,$viewID,roiID)]} \
      {  
#      puts "Draw: No ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set roiID $roiInfo($panelID,$specID,$viewID,roiID)

   if {!$roiID} \
      {
#      puts "Draw: Null ROI for Panel $panelID Spectrum $specID View $viewID."
       return
      }

   set firstView [lindex $viewList 0]
   set nV        [llength $viewList]
   set entry     $panelInfo($panelID,entry)

   set cInfo [getCellInfo -cell $cellID]
   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set xMid  [expr 0.5*([lindex $xInfo 0] + [lindex $xInfo 1])]
   set yTop  [expr [lindex $yInfo 0] - 2]
   set yBot  [expr [lindex $yInfo 1] + 7*$cy/3]

   set xMid [expr int($xMid + $slotInfo($panelID,x1))]

   set labState normal

   if {$dataValid} \
      {
       drawRec -cell $cellID -win $panelID -x  0% 100% -y 0  $wySizeP -fg $wColor
       drawRec -cell $cellID -win $panelID -x  0% 100% -y 0% 100%     -fg $bgColor 
      }

   if {$panelInfo($panelID,$specID,setCount)} \
      {
       set thisAColor $eColorSet
      } \
   else \
      {
       set thisAColor $aColor
      }

   set thisPosColor    [getCurrentVal pColor $specID $panelID]
   set thisNegColor    [getCurrentVal nColor $specID $panelID]

   set thisPosColorArg ""
   set thisNegColorArg ""

   if {![strcasecmp $thisPosColor Multi]} \
      {
       set thisPosColorArg "-pHue1 $pHue1 -pSat1 $pSat1 -pVal1 $pVal1 -pHueN $pHueN -pSatN $pSatN -pValN $pValN -pcc $posColorCount"
      }

   if {![strcasecmp $thisNegColor Multi]} \
      {
       set thisNegColorArg "-nHue1 $nHue1 -nSat1 $nSat1 -nVal1 $nVal1 -nHueN $nHueN -nSatN $nSatN -nValN $nValN -ncc $negColorCount"
      }

   if {$viewID == 1} \
      {   
       if {$roiID} \
          {
           if {[strcasecmp $outName($specID) None]} \
              {
               writeROI -roi $roiID -out [format $outName($specID) $cellID] -ov
              }

           set xVal [adjustPPM $panelInfo($panelID,xPPM) $xOff($specID)]
           set fmt  [getFmt $xVal]

           drawShadeFold $roiID $cellID $specID

           if {$dimCount($specID) == 4} \
              {
               set ticInfo None
              } \
           else \
              {
               set ticInfo Bot,fmt=$fmt,lab=None
              }

           eval drawROI -roi $roiID -cell $cellID -win $panelID \
            -bg None -fg $fgColor($specID) -colorA $thisAColor -colorG $gColor -color1 $thisPosColor -color2 $thisNegColor \
            -plev   [getCurrentVal pLev   $specID $panelID] \
            -nlev   [getCurrentVal nLev   $specID $panelID] \
            -hi     [getCurrentVal hi     $specID $panelID] \
            -tic $ticInfo -$xGrid -$yGrid -nobox $thisPosColorArg $thisNegColorArg

           if {$dimCount($specID) == 4} \
              {
               set xVal [format %.3f [adjustPPM $panelInfo($panelID,xPPM) [getCurrentVal xOff $specID $panelID]]]
               set yVal [format %.3f [adjustPPM $panelInfo($panelID,yPPM) [getCurrentVal yOff $specID $panelID]]]

               set xPPM ${xVal}ppm
               set yPPM ${yVal}ppm

               set tID \
                [.c create text $xMid $yBot -fill $aColor -font $scrollFont \
                    -anchor n -text "$xName($specID)=$xPPM $yName($specID)=$yPPM" -state $labState \
                    -tags "selSpecID axisLab$panelID panel$panelID"]
              } \
           else \
              {
               set tID \
                [.c create text $xMid $yBot -fill $aColor -font $scrollFont \
                    -anchor n -text $xName($specID) -state $labState        \
                    -tags "selSpecID axisLab$panelID panel$panelID"]
              }

           set panelInfo($panelID,$specID,xText) $tID

           .c bind $tID <ButtonRelease-1> "+ selectSpecID $panelID $specID"

           drawBlankFold $roiID $cellID $specID

           if {!$validInfo(assID)} \
              {
               set validInfo(assID) $panelInfo($panelID,assID)
              }

           set validInfo($specID,roiID)   $roiID
           set validInfo($specID,cellID)  $cellID

           set validInfo($specID,roiXID)  $roiID
           set validInfo($specID,cellXID) $cellID
          }

        drawBox -cell $cellID -win $panelID -x 0% 100% -y 0% 100% -fg $fgColor($specID)
       }

   if {$viewID == 2} \
      {   
       if {$roiID} \
          {
           if {[strcasecmp $outName($specID) None]} \
              {
               writeROI -roi $roiID -out [format $outName($specID) $cellID] -ov
              }

           set yVal [adjustPPM $panelInfo($panelID,yPPM) $yOff($specID)]
           set fmt  [getFmt $yVal]

           drawShadeFold $roiID $cellID $specID

           eval drawROI -roi $roiID -cell $cellID \
            -bg None -fg $fgColor($specID) -colorA $thisAColor -colorG $gColor -color1 $thisPosColor -color2 $thisNegColor \
            -plev   [getCurrentVal pLev   $specID $panelID] \
            -nlev   [getCurrentVal nLev   $specID $panelID] \
            -hi     [getCurrentVal hi     $specID $panelID] \
            -$xGrid -$yGrid -nobox -tic Bot,fmt=$fmt,lab=None $thisPosColorArg $thisNegColorArg

           set tID \
            [.c create text $xMid $yBot -fill $aColor -font $scrollFont \
                -state $labState -anchor n -text $yName($specID)        \
                -tags "selSpecID axisLab$panelID panel$panelID"]

           set panelInfo($panelID,$specID,yText) $tID

           .c bind $tID <ButtonRelease-1> "+ selectSpecID $panelID $specID"

           drawBlankFold $roiID $cellID $specID

           set validInfo($specID,roiID)   $roiID
           set validInfo($specID,cellID)  $cellID

           set validInfo($specID,roiYID)  $roiID
           set validInfo($specID,cellYID) $cellID
          } 

       drawBox -cell $cellID -x 0% 100% -y 0% 100% -fg $fgColor($specID)
      }

   if {[strcasecmp $labelVar None]} \
      {
       if {$viewID == $firstView && $specID == 1} \
          {
           set slID \
            [.c create text $xMid $yTop -fill $aColor \
                -state $labState -font $scrollFont -anchor s \
                -tags "assIDLab$panelID panel$panelID"       \
                -text $panelInfo($panelID,label)]

           set panelInfo($panelID,text) $slID

           .c bind $slID <ButtonRelease-1> "+ selectPanelID $panelID"
          }
      }

   if {[strcasecmp $title($specID) None]} \
      {
       if {$viewID == $firstView} \
          {
           if {$specID*$nV % 2} \
              {
               set stID \
                [.c create text $xMid [expr $yTop - 2*$cy] \
                    -fill $lColor($specID) -anchor s -text $title($specID)   \
                    -tags "stripLab$panelID panel$panelID" -font $scrollFont \
                    -state $labState]
              } \
           else \
              {
               set stID \
                [.c create text $xMid [expr $yTop - $cy] \
                    -fill $lColor($specID) -anchor s -text $title($specID)   \
                    -tags "stripLab$panelID panel$panelID" -font $scrollFont \
                    -state $labState]
              }

           set slotInfo($panelID,$specID,eText) $stID

           .c bind $stID <ButtonRelease-1> "+ selectSpecID $panelID $specID"
          }
     }

   if {!($cellID % 10)} update
}

proc updateRefInfo {} \
{
    global roiInfo viewList refInfo specCount panelInfo panelCount

    set cellID 0

    for {set specID 1} {$specID <= $specCount} {incr specID} \
       {
        set refInfo($specID,roiID)  0 
        set refInfo($specID,cellID) 0
       }

    for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
       {
        set doneFlag 1

        for {set specID 1} {$specID <= $specCount} {incr specID} \
           {
            if {!$refInfo($specID,roiID)} {set doneFlag 0}

            foreach viewID $viewList \
               {
                set  roiID $roiInfo($panelID,$specID,$viewID,roiID)
                incr cellID

                if {$roiID} \
                   {
                    set refInfo($specID,roiID)  $roiID
                    set refInfo($specID,cellID) $cellID
                   }
               }
           }

        if {$doneFlag} break
       }
}

proc updateShiftInfo {} \
{
   global q1 qn y1 yn refInfo specCount zFirst zLast

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set roiID  $refInfo($specID,roiID)
       set cellID $refInfo($specID,cellID)

       if {$roiID} \
          {
           set q1($specID) [spec2pix -roi $roiID -cell $cellID -y $zLast($specID)ppm  -nofold]
           set qn($specID) [spec2pix -roi $roiID -cell $cellID -y $zFirst($specID)ppm -nofold]
    
           set q1($specID) [expr int($q1($specID))]
           set qn($specID) [expr int($qn($specID))]
          } \
       else \
          {
           set q1($specID) $y1
           set qn($specID) $yn
          }
      }
}

proc drawShadeFold { roiID cellID specID } \
{
   global bgColor fColor shadeFoldFlag y1 yn q1 qn

   if {!$shadeFoldFlag} \
      {
       drawRec -roiRef $roiID -cell $cellID -x  0% 100% -y 0% 100% -fg $bgColor
       return
      }

   if {![strcasecmp $fColor($specID) blank]} \
      {
       drawRec -roiRef $roiID -cell $cellID -x  0% 100% -y 0% 100% -fg $bgColor
       return
      }

   drawRec -y 0% 100% -roiRef $roiID -cell $cellID -fg $fColor($specID) -x 0% 100% 
   drawRec -y $q1($specID) $qn($specID) -roiRef $roiID -cell $cellID -fg $bgColor -x  0% 100% 
}

proc drawBlankFold { roiID cellID specID } \
{
   global bgColor fColor shadeFoldFlag y1 yn q1 qn

   if {!$shadeFoldFlag} return 
   if {[strcasecmp $fColor($specID) blank]} return

   if {$q1($specID) > $y1} \
      {
       drawRec -roiRef $roiID -cell $cellID -fg $bgColor -x 0% 100% -y $y1 $q1($specID)
      }

   if {$qn($specID) < $yn} \
      {
       drawRec -roiRef $roiID -cell $cellID -fg $bgColor -x 0% 100% -y $qn($specID) $yn
      }
}

proc dragON { sxPix syPix b } \
{
   global dragX1 dragXN dragY1 dragYN dragID dataValid y1 yn
   global cellInfo panelInfo noCursor hCursor

   set dragID -1

   if {!$dataValid} return

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   if {$yPix < $y1 || $yPix > $yn} return

   set cellID  [findCellIDByPix $xPix $yPix]
   if {$cellID < 1} return

   set panelID $cellInfo($cellID,panelID)

   set dragID $panelID

   set xJump  [expr $cellInfo($cellID,xn) - $cellInfo($cellID,x1)]
   set yJump  [expr $cellInfo($cellID,yn) - $cellInfo($cellID,y1)]

   set dragX1 [expr $cellInfo($cellID,x1) + $panelInfo($panelID,x1) - 1]
   set dragXN [expr $dragX1 + $xJump]

   set dragY1 [expr $cellInfo($cellID,y1) + $panelInfo($panelID,y1) - 1]
   set dragYN [expr $dragY1 + $yJump] 

   if {$panelInfo($panelID,fixed)} \
      {
       setCanvasCursor $noCursor 
       set thisColor red
      } \
   else \
      {
       setCanvasCursor $hCursor
       set thisColor green 
      }

   .c create rectangle \
    $dragX1 $dragY1 $dragXN $dragYN -outline $thisColor -tags dragBox

   update
}

proc dragTrack { sxPix syPix b } \
{
   global dragID dragX1 dragXN dragY1 dragYN wxSize dataValid
   global panelInfo panelCount specCount stripsPerSpec

   set panelID $dragID

   if {!$dataValid || $panelID < 1} return
   if {$panelInfo($panelID,fixed)} return

   set xInfo   [.c xview]
   set vx1     [expr int(double($wxSize)*[lindex $xInfo 0])]
   set vxn     [expr int(double($wxSize)*[lindex $xInfo 1])]

   set xJump   [expr $dragXN - $dragX1]
   set xMid    [.c canvasx $sxPix]

   set dragX1 [expr $xMid - $xJump/2]
   set dragXN [expr $dragX1 + $xJump]

   .c coords dragBox $dragX1 $dragY1 $dragXN $dragYN

   if {$dragX1 < $vx1} \
      {
       set f   [expr double($vx1 - $xJump*$specCount*$stripsPerSpec)/$wxSize]
       set f   [expr double($vx1 - $xJump*$stripsPerSpec)/$wxSize]
       set vfw [expr [lindex $xInfo 1] - [lindex $xInfo 0]]

       if {$f < 0.0} {set f 0.0}
       if {$f > 1.0 - $vfw} {set f [expr 1.0 - $vfw]}

       .c xview moveto $f
      } \
   elseif {$dragXN > $vxn} \
      {
       set f   [expr double($vx1 + $xJump*$specCount*$stripsPerSpec)/$wxSize]
       set f   [expr double($vx1 + $xJump*$stripsPerSpec)/$wxSize]
       set vfw [expr [lindex $xInfo 1] - [lindex $xInfo 0]]

       if {$f < 0.0} {set f 0.0}
       if {$f > 1.0 - $vfw} {set f [expr 1.0 - $vfw]}

       .c xview moveto $f
      }

   update
}

proc dragOFF {sxPix syPix b} \
{
   global dragX1 dragXN dragY1 dragYN dragID dx
   global panelInfo panelCount dataValid defCursor

   setCanvasCursor $defCursor

   set panelID $dragID

   if {!$dataValid || $panelID < 1} return

   if {$panelInfo($panelID,fixed)} \
      {
       .c delete dragBox
       update
       return
      }

   set xJump   [expr $dragXN - $dragX1]
   set xMid    [.c canvasx $sxPix]

   set dragX1 [expr $xMid - $xJump/2]
   set dragXN [expr $dragX1 + $xJump]

   .c delete dragBox

   update

   set newID [findPanelIDByXPix $xMid]
   if {$newID < 1} return

   moveStrips $panelID $newID
}

proc swapStrips { side mode } \
{
   global currentInfo panelInfo panelList panelCount

   set panelID $currentInfo(panelID)
   set slotID  $currentInfo(slotID)

#  puts stderr "Attempting Swap. Panel: $panelID Slot: $slotID"

   if {$panelID < 1 || $panelID > $panelCount} return
   if {$slotID < 1 || $slotID > $panelCount}   return 

   set panelLoc [findPanelIDLoc $panelID]
   set adjLoc   $panelLoc

   if {![strcasecmp $side right]} \
      {
       cIncr adjLoc  1 0 [expr $panelCount - 1]
      } \
   else \
      {
       cIncr adjLoc -1 0 [expr $panelCount - 1]
      }

   set adjID [lindex $panelList $adjLoc]

   if {$panelInfo($adjID,fixed)} \
      {
#      puts stderr "Adjacent [string totitle $side] Side is Anchored. No Move Performed."
       return 0
      }

   if {$panelInfo($adjID,next) > 0 || $panelInfo($adjID,prev) > 0} \
      {
#      puts stderr "Adjacent [string totitle $side] Side is Joined. No Move Performed."
       return 0
      }

   set newSlotID 0

   for {set i 0} {$i <= $panelCount} {incr i} \
      {
       if {![strcasecmp $mode incr]} \
          {
           cIncr slotID  1 1 $panelCount
          } \
       else \
          {
           cIncr slotID -1 1 $panelCount
          }

       set slotLoc [expr $slotID - 1]

       if {$slotLoc == $panelLoc - 1} continue
       if {$slotLoc == $panelLoc}     continue
       if {$slotLoc == $panelLoc + 1} continue

       set thisID [lindex $panelList $slotLoc]

       if {$panelInfo($thisID,next) > 0} continue
       if {$panelInfo($thisID,prev) > 0} continue 
       if {$panelInfo($thisID,fixed)}    continue

       set newSlotID $slotID

       break
      }

   if {!$newSlotID} \
      {
#      puts stderr "No Free Locations. No Move Performed."
       return 0
      }

   set slotID $newSlotID
   set nextID [lindex $panelList [expr $slotID - 1]]

   moveStrips $nextID $adjID

   selectSlotID $slotID

   return 1
}

proc moveStrips { srcPanelID destPanelID } \
{
   global panelList panelCount

#  puts stderr "Attempting Move. Source Panel: $srcPanelID Dest Panel: $destPanelID."

   if {$srcPanelID < 1 || $destPanelID < 1} {return 0}
   if {$srcPanelID == $destPanelID} {return 0}

   set srcIDList  [findJoinList $srcPanelID]

#  puts stderr " Source Panel List: $srcIDList"

   if {![llength $srcIDList]} \
      {
#      puts stderr "Empty Source Region. No Move Performed."
       return 0
      }

   if {[isAnchored $srcIDList]} \
      {
#      puts stderr "Source Region is Anchored. No Move Performed."
       return 0
      }

   set n [llength $srcIDList]

   set srcLoc   [findPanelIDLoc $srcPanelID]
   set destLoc  [findPanelIDLoc $destPanelID]
   set srcLoc1  [findPanelIDLoc [lindex $srcIDList 0]]
   set destLoc1 [expr $destLoc + $srcLoc1 - $srcLoc]

#  puts stderr " Source Loc  selected $srcLoc first $srcLoc1."
#  puts stderr " Dest   Loc  selected $destLoc first $destLoc1."

   if {$srcLoc1 == $destLoc1} {return 0}

   if {$destLoc1 < 0 || $destLoc1 > $panelCount - $n} \
      {
#      puts stderr "Destination Region is Out of Bounds. No Move Performed."
       return 0
      }

   set srcIDList  [lrange $panelList $srcLoc1  [expr $srcLoc1  + $n - 1]]
   set destIDList [lrange $panelList $destLoc1 [expr $destLoc1 + $n - 1]]

   if {![llength $destIDList]} \
      {
#      puts stderr "Empty Destination Region. No Move Performed."
       return 0
      }

   if {[isAnchored $destIDList]} \
      {
#      puts stderr "Destination Region is Anchored. No Move Performed."
       return 0
      }

   set uList [listIntersect $srcIDList $destIDList]

   if {![llength $uList]} \
      {
#      puts stderr "Swapping $destIDList at $destLoc with $srcIDList from $srcLoc."

       set newList $panelList
       set srcLoc  $srcLoc1
       set destLoc $destLoc1

       for {set i 0} {$i < $n} {incr i} \
          {
           set srcVal  [lindex $srcIDList  $i]
           set destVal [lindex $destIDList $i]

           set thisList [lreplace $newList  $destLoc $destLoc $srcVal]
           set newList  [lreplace $thisList $srcLoc  $srcLoc  $destVal]

           incr srcLoc
           incr destLoc
          }
      } \
   else \
      {
#      puts stderr "Moving Overlap $destIDList at $destLoc with $srcIDList from $srcLoc."

       if {$destLoc1 < $srcLoc1} \
          {
           set newList $panelList
           set srcLoc  $srcLoc1

           for {set i 0} {$i < $srcLoc1 - $destLoc1} {incr i} \
              {
               set loc [expr $srcLoc - 1]
               set id  [lindex $newList $loc]

               set thisList [lreplace $newList $loc $loc] 
               set newList  [linsert  $thisList [expr $loc + $n] $id]

               incr srcLoc -1
              }
          } \
       else \
          {
           set newList $panelList
           set srcLoc  $srcLoc1

           for {set i 0} {$i < $destLoc1 - $srcLoc1} {incr i} \
              {
               set loc [expr $srcLoc + $n]
               set id  [lindex $newList $loc]

               set thisList [lreplace $newList $loc $loc]
               set newList  [linsert  $thisList $srcLoc $id]

               incr srcLoc
              }
          }
      }

   set error [testJoinInfo $newList]

   if {$error} \
      {
       showError Scroll "Errors Found in Proposed Strip Order. No Move Performed. Error: $error."
      } \
   else \
      {
       set panelList $newList
      }

   storeHistory
   reorderDisplay
}

proc reorderDisplay {} \
{
    global slotInfo panelInfo panelList prevList panelCount

    set slotID 1
    set i 0

    foreach panelID $panelList \
       {
        set prevID  [lindex $prevList $i]

        if {$panelID != $prevID} \
           {
            set xJump [expr $slotInfo($slotID,x1) - $panelInfo($panelID,x1)] 
            set yJump [expr $slotInfo($slotID,y1) - $panelInfo($panelID,y1)] 

            .c move panel$panelID $xJump $yJump

            update
            set xyList [.c coords $panelInfo($panelID,pID)]

            set panelInfo($panelID,x0) [expr int([lindex $xyList 0])]
            set panelInfo($panelID,y0) [expr int([lindex $xyList 1])]

            set panelInfo($panelID,x1) $slotInfo($slotID,x1)
            set panelInfo($panelID,xn) $slotInfo($slotID,xn)
            set panelInfo($panelID,y1) $slotInfo($slotID,y1)
            set panelInfo($panelID,yn) $slotInfo($slotID,yn)
           }

        incr slotID 
        incr i
       }

   set prevList $panelList

   refreshAnchorLines
   refreshJoinLines
}

proc testJoinInfo { testList } \
{
   global panelInfo panelList panelCount

   set error 0

   set origList  $panelList
   set panelList $testList

   set nn [llength $panelList]

   if {$nn != $panelCount} \
      {
       puts stderr "Error in Join Test Panel Count. Current: $nn Correct: $panelCount."
       set error [expr $error | 1]
      }

   for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
     {
      set thisLoc [findPanelIDLoc $panelID]

      if {$thisLoc < 0} \
         {
          puts stderr "Error in Join Test Finding PanelID: $panelID"
          set error [expr $error | 2]
         }

      set prevID $panelInfo($panelID,prev)
      set nextID $panelInfo($panelID,next)

      if {$prevID > 0} \
         {
          set prevLoc [findPanelIDLoc $prevID]
          set thisID  $panelInfo($prevID,next)
         
          if {$panelID != $thisID} \
             {
              puts stderr "Bad Join Pointer Prev->Next:"
              puts stderr " Panel: $panelID Prev: $prevID Prev->Next: $thisID"
              set error [expr $error | 4]
             }

          if {$prevLoc < 0} \
             {
              puts stderr "Error in Join Test Finding Panel $panelID Prev: $prevID"

              set error [expr $error | 8]
             }

          if {$thisLoc >= 0 && $prevLoc >= 0 && $thisLoc != $prevLoc + 1} \
             {
              puts stderr "Error Finding Adjacent Join Prev:"
              puts stderr " Panel $panelID at $thisLoc, Prev $prevID at $prevLoc"

              set error [expr $error | 16]
             }
         }

      if {$nextID > 0} \
         {
          set nextLoc [findPanelIDLoc $nextID]
          set thisID  $panelInfo($nextID,prev)

          if {$panelID != $thisID} \
             {
              puts stderr "Bad Join Pointer Next->Prev" 
              puts stderr " Panel: $panelID Next: $nextID Next->Prev: $thisID"
              set error [expr $error | 32]
             }

          if {$nextLoc < 0} \
             {
              puts stderr "Error in Join Test Finding Panel $panelID Next: $nextID"

              set error [expr $error | 64]
             }

          if {$thisLoc >= 0 && $nextLoc >= 0 && $thisLoc != $nextLoc - 1} \
             {
              puts stderr "Error Finding Adjacent Join Next:"
              puts stderr " Panel $panelID at $thisLoc, Next $nextID at $nextLoc"

              set error [expr $error | 128]
             }
         } 
     }

   set panelList $origList

   return $error 
}

proc findJoinList { panelID } \
{
   global panelInfo panelList panelCount

   set jList ""
   set error 0
   set n     0

   while {[info exists panelInfo($panelID,assID)]} \
      {
       set prevID $panelInfo($panelID,prev)
       if {$prevID < 1 || $prevID == $panelID || $prevID > $panelCount} break
 
       set  panelID $prevID 
       incr n

       if {$n > $panelCount + 1} break 
      }

   set panelLoc [lsearch -exact $panelList $panelID]
   set n 0

   while {[info exists panelInfo($panelID,assID)]} \
      {
       if {[lsearch -exact $jList $panelID] < 0} \
          {
           lappend jList $panelID
          } \
       else \
          {
           set error 1
           break
          }

       set nextID $panelInfo($panelID,next)
       if {$nextID < 1} break

       if {$nextID > $panelCount} \
          {
           set error 2
           break
          }

       set nextLoc [lsearch -exact $panelList $nextID]

       if {$panelLoc != $nextLoc - 1} \
          {
           set error 3
           break
          }

       set  panelID  $nextID
       set  panelLoc $nextLoc
       incr n

       if {$n > $panelCount + 1} \
          {
           set error 4
           break
          }
      }

   if {$error} {showError Scroll "Errors Found in Join Info. Error: $error"}

   return $jList
}

proc initHistory {} \
{
   global panelInfo panelList panelCount assCount
   global histName histHdr histInfo hUnit

   if {![strcasecmp $histName None]} {return 0}

   set error [catch {set hUnit [open $histName a+]}]

   if {$error} \
      {
       showError Scroll "Error Opening History File $histName"
       set hUnit 0
       return
      }

   set newList  $panelList
   set histInfo ""
   set histHdr  ""

   seek $hUnit 0

   while {1} \
      {
       set thisList [getNextHistory]
       if {[llength $histHdr] < 2} break

       set newList $thisList
       incr assCount
      }
 
   set error [fixHistory $newList]

   if {$error} \
      {
       showError Scroll "Warning: Mismatched History File $histName"
      }

   set prevID  -1
   set nextID  -1

   set jCount   0
   set aCount   0

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       set i [expr $slotID - 1]

       set hList   [lindex $histInfo $i] 
       set panelID [lindex $panelList $i]
       set thisID  [lindex $hList 0]

       set prevID  -1
       set nextID  -1

       if {$i > 0} {set prevID [lindex $panelList [expr $i - 1]]}
       if {$i < $panelCount - 1} {set nextID [lindex $panelList [expr $i + 1]]}

       if {$panelID != $thisID} continue 

       set thisPrev   [getAssignedVal $hList prev]
       set thisNext   [getAssignedVal $hList next]

       if {![llength $thisPrev]}   {set thisPrev   -1}
       if {![llength $thisNext]}   {set thisNext   -1}

       if {$thisPrev > 0} \
          {
           if {$thisPrev == $prevID} \
              {
               set panelInfo($panelID,prev)  $thisPrev
               set panelInfo($thisPrev,next) $panelID
  
               puts -nonewline stderr "Restoring: Joining (Prev)"
               puts stderr " $panelInfo($thisPrev,assID) with $panelInfo($panelID,assID)"
              } \
           else \
              {
               incr jCount
              }
          }

       if {$thisNext > 0} \
          {
           if {$thisNext == $nextID} \
              {
               set panelInfo($panelID,next)  $thisNext
               set panelInfo($thisNext,prev) $panelID

               puts -nonewline stderr "Restoring: Joining (Next)"
               puts stderr " $panelInfo($panelID,assID) with $panelInfo($thisNext,assID)"
              } \
           else \
              {
               incr jCount
              }
          }
      }

   refreshJoinLines

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       set i [expr $slotID - 1]

       set hList   [lindex $histInfo $i]
       set panelID [lindex $panelList $i]
       set thisID  [lindex $hList 0]

       if {$panelID != $thisID} \
          {
#          puts "skip anchor slot $slotID"
           continue
          }

       set thisAnchor [getAssignedVal $hList anchor]
       if {![llength $thisAnchor]} {set thisAnchor -1}

       if {$thisAnchor > 0} \
          {
           if {$thisAnchor == $slotID} \
              {
               puts -nonewline stderr "Restoring: Setting Anchor "
               puts stderr " for $panelInfo($panelID,assID) in slot $slotID" 

               setPanelAnchor $panelID 1
              } \
           else \
              {
               incr aCount
              }
          }
      }

   set error [testJoinInfo $panelList]

   if {$error || $jCount || $aCount} \
      {
       set eStr "Error: $error. Join Errors: $jCount Anchor Errors: $aCount."
       showError Scroll "Errors in History File. $eStr"
      }

   update

   storeHistory 
}

proc fixHistory { newList } \
{
   global panelList panelCount

   set error 0
   set thisList ""

   if {[llength $newList] != $panelCount} {set error [expr $error | 1]}

   for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
      {
       if {[lsearch -exact $newList $panelID] < 0} {set error [expr $error | 2]}
      }

   foreach panelID $newList \
      {
       if {$panelID < 1 || $panelID > $panelCount} continue
       if {[lsearch -exact $thisList $panelID] < 0} {lappend thisList $panelID}
      }

   for {set panelID 1} {$panelID <= $panelCount} {incr panelID} \
      {
       if {[lsearch -exact $thisList $panelID] < 0} {lappend thisList $panelID}
      }

   set panelList $thisList

   return $error
}

proc getNextHistory {} \
{
   global histHdr histInfo hUnit

   set thisList ""
   set histHdr  ""
   set assFlag  0

   while {[gets $hUnit line] >= 0} \
      {
       scan $line "%s" key

       set c [string index $key 0]

       if {![strcmp $c "#"]} \
          {
           continue
          } \
       elseif {![strcasecmp $key ass]} \
          {
           set histHdr [string trim $line]
          } \
       elseif {![strcasecmp $key start]} \
          {
           set histInfo ""
           set assFlag  1
          } \
       elseif {![strcasecmp $key end]} \
          {
           return $thisList
          } \
       elseif {$assFlag} \
          {
           set str [string trim $line]
           if {![llength $str]} continue

           foreach s $str \
              {
               set ss [split $s ","]

               lappend thisList [lindex $ss 0]
               lappend histInfo $ss
              }
          }
      }

   return $thisList 
}

proc getAssignedVal { aList varName } \
{
    set val ""

    foreach a $aList \
       {
        if {[string first ${varName}= $a] == 0} \
           {
            set val [lindex [split $a "="] 1]
           }
       }

    return $val
}

proc storeHistory {} \
{
   global hUnit panelInfo panelList panelCount assCount

   if {![strcmp $hUnit 0]} return

   seek $hUnit 0 end

   incr assCount

   puts stderr "Recording assignment $assCount"

   puts $hUnit "ass $assCount timestamp [lindex [sysTime] 0] count $panelCount" 
   puts $hUnit "start"

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       set panelID  [lindex $panelList [expr $slotID - 1]]
       set anchorID -1

       set fmt " %3d,id=%s,xPPM=%.3f,yPPM=%.3f,next=%d,prev=%d,anchor=%d"

       if {$panelInfo($panelID,fixed)} {set anchorID $slotID}

       puts $hUnit \
        [format $fmt $panelID $panelInfo($panelID,assID) \
                $panelInfo($panelID,xPPM) $panelInfo($panelID,yPPM) \
                $panelInfo($panelID,next) $panelInfo($panelID,prev) \
                $anchorID]
      } 

   puts  $hUnit "end"
   flush $hUnit
}

proc getSpecID { specType } \
{
   global title specCount

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       if {![strcasecmp $specType $title($specID)]} {return $specID}
      }

   return -1
}

proc initCursors {} \
{
   global cursorInfo specCount cColor qColor pColor nColor

   set cursorInfo(totalCount) 0 

   set nn [llength $cursorInfo(countList)]

   if {$nn > $specCount} \
      {
       puts stderr "Notice: unused arguments for -cursors option."
      }

   if {$nn == 0} \
      {
       set cursorType 0
      } \
   elseif {$nn == 1} \
      {
       while {$specCount > 0 && [llength $cursorInfo(countList)] < $specCount} \
          {
           lappend cursorInfo(countList) 0
          }

       set defCColor white
       set defQColor white

       set cursorType All 
      } \
   elseif {$nn >= $specCount} \
      {
       set cursorType 0
      } \
   elseif {$nn < $specCount} \
      {
       puts stderr "Error: expected $specName arguments for -cursors option."
       exit 1
      } \

   foreach i $cursorInfo(countList) {incr cursorInfo(totalCount) $i}

   if {$cursorInfo(totalCount) < 1} return

   set n 0

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set cColorList [split $cColor($specID) ","]
       set qColorList [split $qColor($specID) ","]

       set cN [llength $cColorList]
       set qN [llength $cColorList]

       if {[strcasecmp $cursorType All]} \
          {
           set defCColor $pColor($specID)
           set defQColor $pColor($specID)

           set cursorType $specID
          }

       set thisCount [lindex $cursorInfo(countList) [expr $specID - 1]]

       for {set i 0} {$i < $thisCount} {incr i} \
          {
           set thisCColor [lindex $cColorList [expr $i % $cN]]
           set thisQColor [lindex $qColorList [expr $i % $qN]]
           incr n

           if {![strcasecmp $thisCColor Default]} {set thisCColor $defCColor}
           if {![strcasecmp $thisQColor Default]} {set thisCColor $defQColor}

           if {![strcasecmp $thisCColor Multi]} {set thisCColor $defCColor}
           if {![strcasecmp $thisQColor Multi]} {set thisCColor $defQColor}

           addCursor $specID $thisCColor $thisQColor $cursorType $i $n
          }
      } 
}

proc addCursor { thisSpecID thisCColor thisQColor cursorType cursorLoc n } \
{
   global wxSize wySize cursorInfo cx cy y1 yn scrollFont specCount 
   global defCursor 

   if {![strcasecmp $thisCColor Multi]} {set thisCColor blue}

   set iy [expr $yn - ($yn - $y1)/(2 + $n)]

   update

   set cID   [.c create line 1 $iy [expr $wxSize-1] $iy -fill $thisCColor -tags cursor]

   set xInfo [.c xview]
   set thisX [expr $cx + int(double($wxSize)*[lindex $xInfo 0])] 
   set thisY [expr $iy - $cy/4]

   set cursorInfo($n,showbar)  1
   set cursorInfo($n,showtext) 1

   set cursorInfo($n,showpix)  0
   set cursorInfo($n,pix,val)  $iy
   set cursorInfo($n,type)     $cursorType
   set cursorInfo($n,loc)      $cursorLoc
   set cursorInfo($n,cID)      $cID
   set cursorInfo($n,tx)       $cx
   set cursorInfo($n,cColor)   $thisCColor
   set cursorInfo($n,qColor)   $thisQColor

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set cursorInfo($n,show$specID) 0
       set cursorInfo($n,$specID,val) 0.0
      }

   set cursorInfo($n,show$thisSpecID) 1

   set tID \
    [.c create text $thisX $thisY -fill $thisQColor -anchor sw \
        -font $scrollFont -text [getCursorLabel $n] -tags cursorLab]

   set cursorInfo($n,tID) $tID

   .c bind $cID <B2-Motion>       "+ moveCursor $n %x %y 0"
   .c bind $tID <B2-Motion>       "+ moveCursor $n %x %y 1"

   .c bind $cID <ButtonRelease-2> "+ setCanvasCursor $defCursor"
   .c bind $tID <ButtonRelease-2> "+ setCanvasCursor $defCursor"

   .c bind $tID <Shift-B2-Motion> "+ moveCursorText $n %x %y"
}

proc moveCursorText { n sxPix syPix } \
{
   global cx wxSize cursorInfo vCursor

   setCanvasCursor $vCursor

   set ix  [.c canvasx $sxPix]
   set iy  [.c canvasy $syPix]

   set tID $cursorInfo($n,tID)

   set cInfo [.c coords $tID]

   set thisX $ix
   set thisY [lindex $cInfo 1]

   if {$thisX < $cx} {set thisX $cx}
   if {$thisX > $wxSize - $cx} {set thisX [expr $wxSize - $cx]}

   .c coords $tID $thisX $thisY

   set xInfo [.c xview]

   set cursorInfo($n,tx) [expr $thisX - int(double($wxSize)*[lindex $xInfo 0])]

   update
}

proc setCursorLoc { n specID } \
{
   global cursorInfo validInfo

   if {![strcasecmp $specID pix]} \
      {
       set iy $cursorInfo($n,pix,val)
      } \
   else \
      {
       set zPPM $cursorInfo($n,$specID,val)

       set iy \
        [spec2pix -roi  $validInfo($specID,roiID) \
                  -cell $validInfo($specID,cellID) -y ${zPPM}ppm -nofold]
      }

   moveCursor2 $n $iy
}

proc moveCursor { n sxPix syPix textFlag } \
{
   global cx cy y1 yn vCursor

   setCanvasCursor $vCursor

   set ix [.c canvasx $sxPix]
   set iy [.c canvasy $syPix]

   if {$textFlag} {set iy [expr $iy + $cy/4]}

   if {$iy < $y1 || $iy > $yn} return

   moveCursor2 $n $iy
}

proc moveCursor2 { n iy } \
{
   global wxSize cx cy y1 yn cursorInfo

   if {$iy < $y1} {set iy $y1}
   if {$iy > $yn} {set iy $yn}

   set cInfo [.c coords $cursorInfo($n,tID)]

   set thisX [lindex $cInfo 0] 
   set thisY [expr $iy - $cy/4]

   .c coords $cursorInfo($n,cID) 1 $iy [expr $wxSize - 1] $iy
   .c coords $cursorInfo($n,tID) $thisX $thisY 

   set cursorInfo($n,pix,val) $iy

   updateCursorText $n
}

proc updateCursorText { n } \
{
   global cursorInfo


   .c itemconfigure $cursorInfo($n,tID) -text [getCursorLabel $n]

   update
}

proc getCursorLabel { n } \
{
   global specCount refInfo title cursorInfo

   set iy [expr int($cursorInfo($n,pix,val))]

   set str ""
   set i 0
   set j 0

   if {$cursorInfo($n,showpix)} \
      {
       append str "Pix=$iy"
       incr i
      }

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       set roiID  $refInfo($specID,roiID)
       set cellID $refInfo($specID,roiID)

       set cursorInfo($n,$specID,val) \
        [format "%.3f" [pix2spec -cell $cellID -roi $roiID -y $iy -units ppm]]

       if {$cursorInfo($n,show$specID)} \
          {
           if {$i} {append str " "}

           append str "$title($specID)=$cursorInfo($n,$specID,val)"

           incr i
           incr j
          }
      }

   if {$j} {append str " ppm"}

   return $str
}

proc initBind {} \
{
   bind .c  <ButtonPress-1>    "dragON     %x %y B1"
   bind .c  <B1-Motion>        "dragTrack  %x %y B1"
   bind .c  <ButtonRelease-1>  "dragOFF    %x %y B1"
   bind .   <KeyPress>         "+ parseKey %K %x %y"

   bind .c  <Button-4>         "setCurrent hi incr"
   bind .c  <Button-5>         "setCurrent hi decr"
}

proc initWin {} \
{
   global xAdj yAdj cAdj cx cy dx dy y1 yn qx1 qx2 qx3
   global panelCount specCount stripsPerSpec 
   global wxSize wySize wxSizeP wySizeP wxSizeFull wxSizeMax
   global thisInterp interpList linkFlag winName
   global geom geomOffX geomOffY cmndFlag wColor bgColor 

   if {$cmndFlag} \
      {
       set tAdj [expr $xAdj + $cAdj]
      } \
   else \
      {
       set tAdj $xAdj
      }

   set wx [expr int([winfo screenwidth  .] - ($tAdj + 5))]
   set wy [expr int([winfo screenheight .] - ($yAdj + 5))]

   if {![strcasecmp $geom Auto]} \
      {
# was 25
       wm geom . =${wx}x${wy}+${geomOffX}+${geomOffY}
      }

   if {![strcasecmp $wySize Auto]} \
      {
       if {$cmndFlag} \
          {
           set wySize [expr int([winfo screenheight .] - ($yAdj + 2*$cy))]
          } \
       else \
          {
           set wySize [expr int([winfo screenheight .] - ($yAdj + 4*$cy))]
          }
      }

   set wxSizeMax  [expr 32400 - $wx]
   set wxSizeP    [expr $specCount*($stripsPerSpec*($dx + $qx1) + $qx2) + $qx3]
   set wxSizeFull [expr $panelCount*$wxSizeP]

   set wxSize $wxSizeFull

   if {$wxSize > $wxSizeMax} \
      {
       puts "Crossing 16-bit window size limit: $wxSizeMax $wxSize."
      }

   if {$wySize < 10*$cy} \
      {
       set y1 [expr  int(5.0*$wySize/100.0)]
       set yn [expr  int(90.0*$wySize/100.0)]
      } \
   else \
      {
       set y1 [expr int(3.25*$cy)]
       set yn [expr int($wySize - (12*$cy/2))]
      }

   if {![strcasecmp $wySizeP Auto]} {set wySizeP [expr $wySize - 1]}

   set dy [expr 1 + $yn - $y1]

   set fmt "Geom: %dx%d Canvas: %dx%d Strip: %dx%d xAdj: %d yAdj: %d xOffW: $geomOffX yOffW: $geomOffY"

   puts ""
   puts [format $fmt $wx $wy $wxSize $wySize $dx $dy $xAdj $yAdj]

   set thisInterp [string trim [winfo name .] "{}"]
   set interpList ""

   scrollbar .s -orient horiz -command sScrollProc -repeatdelay 2000

   canvas .c -width $wxSize -height $wySize -border 0 -closeenough 4 \
          -xscrollcommand cScrollProc -scrollregion "0 0 $wxSize $wySize" \
          -bg $bgColor

   pack .c .s -side top -expand 1 -fill x

   if {!$cmndFlag} \
      {
       button .b -text Quit -command exit
       pack   .b -side top -expand 1 -fill x
      }

   if {!$cmndFlag} return

   toplevel .p

   set pwx [expr $cAdj - 5]

   wm title .p "$winName Scroll"
   wm geom  .p =${pwx}x${wy}-$cx+5 

   update
}

proc initCmndPanel {} \
{
   global cmndFlag cursorInfo pkInfo winInfo specCount title env
   global currentInfo aColor cx cy colorList snapFlag

   if {!$cmndFlag} return

   set usePixLab 0

#
# Snapshot:

   if {$snapFlag} \
      {
       button .p.snap -bitmap @$env(NMRTXT)/cameraIcon.xbm -command captureScreen
       button .p.anim -text   Animate                      -command captureAnimation
       pack   .p.snap .p.anim -in .p -side top -fill x
      }

#
# Cursor locations:

   frame .p.space0 -width 1 -height 4
   pack  .p.space0 -in .p -side top -pady 0

   for {set n 1} {$n <= $cursorInfo(totalCount)} {incr n} \
      {
       if {![strcasecmp $cursorInfo($n,type) All]} \
          {
           set str " Cursor $n"
          } \
       else \
          {
           set specID $cursorInfo($n,type)
           set loc    [expr $cursorInfo($n,loc) + 1]

           set str " $title($specID) $loc" 
          }

       frame .p.cursor$n -padx [expr $cx/6] 
 
       label .p.cursor${n}Lab -text $str -width 14 -anchor w 

       checkbutton .p.cursor${n}Bar -selectcolor $cursorInfo($n,cColor) \
        -text "Bar" -variable cursorInfo($n,showbar) \
        -width 0 -command "toggleCursorBar $n" -anchor w

       checkbutton .p.cursor${n}Txt -selectcolor $cursorInfo($n,cColor) \
        -text "Text" -variable cursorInfo($n,showtext) \
        -command "toggleCursorText $n text" -anchor w
     
       pack .p.cursor${n}Lab .p.cursor${n}Bar .p.cursor${n}Txt \
            -in .p.cursor${n} -side left 

       if {$usePixLab} \
          {
           checkbutton .p.cursor${n}TxtPix \
             -width 12 -selectcolor $cursorInfo($n,cColor)  \
             -text "Pixel" -variable cursorInfo($n,showpix) \
             -command "toggleCursorText $n pix" -anchor w

           entry .p.cursor${n}EntryPix -width 8 \
                 -textvariable cursorInfo($n,pix,val)

           bind .p.cursor${n}EntryPix <Return> "+ setCursorLoc $n pix"
 
           pack .p.cursor${n}TxtPix .p.cursor${n}EntryPix \
                -in .p.cursor${n} -side left 

          }

       pack .p.cursor${n} -in .p -side top
       update
      }

   frame .p.cursor${n}Space1 -width 1 -height 8
   pack  .p.cursor${n}Space1 -in .p -side top -pady 0

   if {$cursorInfo(totalCount) > 0} \
      {
       set n 1

       for {set specID 1} {$specID <= $specCount} {incr specID} \
          {
           frame .p.cursor${n}Frame$specID

           label  .p.cursor${n}TxtSpec${specID}Space -width 3 -text " "

           checkbutton .p.cursor${n}TxtSpec$specID \
            -width 12 -selectcolor $cursorInfo($n,cColor) \
            -text "$title($specID)" -variable cursorInfo($n,show$specID) \
            -command "toggleCursorText $n $specID" -anchor w

           entry .p.cursor${n}EntrySpec$specID -width 8 \
                 -textvariable cursorInfo($n,$specID,val)

           bind .p.cursor${n}EntrySpec$specID <Return> \
                "+ setCursorLoc $n $specID"

           pack .p.cursor${n}TxtSpec${specID}Space \
                .p.cursor${n}TxtSpec$specID .p.cursor${n}EntrySpec$specID -in \
                .p.cursor${n}Frame$specID -side left

           pack .p.cursor${n}Frame$specID \
                -in .p -side top -fill x
          }
      }

   frame .p.cursor${n}Space2 -width 1 -height 8
   pack  .p.cursor${n}Space2 -in .p -side top -pady 0

#
# Peak Label Information:

   frame .p.peak -padx [expr $cx/4] 

   label .p.peakSpace -width 2 -text " "

   checkbutton .p.peakLab  -selectcolor $aColor \
     -text "Peak PPM" -variable pkInfo(showShift) \
     -command "updatePeakLabels" -anchor w

   checkbutton .p.peakAss  -selectcolor $aColor \
     -text "Ass" -variable pkInfo(showAss) \
     -command "updatePeakLabels" -anchor w

   checkbutton .p.peakLine -selectcolor $aColor \
     -text "Lines" -variable pkInfo(showLine) \
     -command "refreshPeakLines" -anchor w

   pack .p.peakSpace .p.peakLab .p.peakAss .p.peakLine -in .p.peak -side left 
   pack .p.peak -in .p -side top -fill x

   frame .p.cursor${n}Space3 -width 1 -height 1
   pack .p.cursor${n}Space3 -in .p -side top -pady 0

#
# Spectrum Panel and Slot Selection:

   frame  .p.spec

   frame  .p.specGroup
   
   label  .p.groupPPMLab  -text "" -width 24

   label  .p.groupIDLab   -text "ID"
   entry  .p.groupIDEntry -textvariable currentInfo(assID) -width 6

   button .p.groupIDIncr  -command "setCurrent assID incr" \
                          -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.groupIDDecr  -command "setCurrent assID decr" \
                          -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map 

   label  .p.resIDLab   -text "Res"
   entry  .p.resIDEntry -textvariable currentInfo(resID) -width 5

   button .p.resIDIncr  -command "setCurrent resID incr" \
                        -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.resIDDecr  -command "setCurrent resID decr" \
                        -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map

   pack   .p.groupIDLab .p.groupIDEntry .p.groupIDIncr .p.groupIDDecr \
          .p.resIDLab   .p.resIDEntry   .p.resIDIncr   .p.resIDDecr   \
          -in .p.specGroup -side left

   pack .p.groupPPMLab .p.specGroup -in .p.spec -side top -anchor center

   bind .p.groupIDEntry <Return> "+ setCurrent assID"

#
# Spectrum Type Selection:

   frame .p.specType

   menubutton .p.typeMenu -menu .p.typeMenu.menu \
              -relief raised -bitmap @$env(NMRTXT)/caret.map

   menu .p.typeMenu.menu -tearoff False

   for {set specID 1} {$specID <= $specCount} {incr specID} \
      {
       .p.typeMenu.menu add command -label $title($specID) \
        -command "setCurrent specType val $title($specID)"
      }

   label  .p.specTypeLab   -text "Type"
   entry  .p.specTypeEntry -textvariable currentInfo(specType) -width 14

   button .p.specTypeIncr  -command "setCurrent specType incr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.specTypeDecr  -command "setCurrent specType decr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map

   pack   .p.specTypeLab .p.specTypeEntry .p.specTypeIncr .p.specTypeDecr \
          .p.typeMenu -in .p.specType -side left  

   pack   .p.specType -in .p.spec -side top -anchor center

   bind .p.specTypeEntry   <Return> "+ setCurrent specType"

#
# Spectrum Contour Height, Count, and Colors:
 
   frame  .p.space1 -width 1 -height 1 
   pack   .p.space1 -in .p.spec -side top -fill x

   frame  .p.specHi
   frame  .p.specP
   frame  .p.specN

   label  .p.specHiLab   -text "Hi" 
   entry  .p.specHiEntry -textvariable currentInfo(hi) -width 11 

   set winInfo(hi) .p.specHiEntry

   button .p.specHiIncr  -command "setCurrent hi incr" \
                         -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.specHiDecr  -command "setCurrent hi decr" \
                         -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map

   pack   .p.specHiLab .p.specHiEntry .p.specHiIncr .p.specHiDecr \
          -in .p.specHi -side left

   pack   .p.specHi -in .p.spec -side top -anchor center

   frame  .p.space2 -width 1 -height 6 
   pack   .p.space2 -in .p.spec -side top -fill x

   label  .p.specPLevLab -text "PLev" -width 4
   label  .p.specNLevLab -text "NLev" -width 4

   entry  .p.specPLevEntry -textvariable currentInfo(pLev) -width 2
   entry  .p.specNLevEntry -textvariable currentInfo(nLev) -width 2

   set winInfo(pLev) .p.specPLevEntry
   set winInfo(nLev) .p.specNLevEntry

   button .p.specPLevIncr  -command "setCurrent pLev incr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.specPLevDecr  -command "setCurrent pLev decr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map

   button .p.specNLevIncr  -command "setCurrent nLev incr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/incr.map

   button .p.specNLevDecr  -command "setCurrent nLev decr" \
                           -padx 0 -pady 0 -bitmap @$env(NMRTXT)/decr.map

   menubutton .p.pColorMenu -menu .p.pColorMenu.menu \
              -relief raised -bitmap @$env(NMRTXT)/caret.map

   menubutton .p.nColorMenu -menu .p.nColorMenu.menu \
              -relief raised -bitmap @$env(NMRTXT)/caret.map

   menu .p.pColorMenu.menu -tearoff False
   menu .p.nColorMenu.menu -tearoff False

   foreach c $colorList \
      {
       .p.pColorMenu.menu add command -label $c \
        -command "setCurrent pColor val $c"

       .p.nColorMenu.menu add command -label $c \
        -command "setCurrent nColor val $c"
      }

   label .p.specPColorLab -text "Color" -width 5
   label .p.specNColorLab -text "Color" -width 5

   entry .p.specPColorEntry -textvariable currentInfo(pColor) -width 9 
   entry .p.specNColorEntry -textvariable currentInfo(nColor) -width 9 

   set winInfo(pColor) .p.specPColorEntry
   set winInfo(nColor) .p.specNColorEntry

   pack .p.specPLevLab .p.specPLevEntry .p.specPLevIncr .p.specPLevDecr \
        .p.specPColorLab .p.specPColorEntry .p.pColorMenu \
        -in .p.specP -side left  

   pack .p.specNLevLab .p.specNLevEntry .p.specNLevIncr .p.specNLevDecr \
        .p.specNColorLab .p.specNColorEntry .p.nColorMenu \
        -in .p.specN -side left  

   pack .p.specP .p.specN -in .p.spec -side top -anchor center

   bind .p.specHiEntry     <Return> "+ setCurrent hi"

   bind .p.specPLevEntry   <Return> "+ setCurrent pLev"
   bind .p.specNLevEntry   <Return> "+ setCurrent nLev"

   bind .p.specPColorEntry <Return> "+ setCurrent pColor"
   bind .p.specNColorEntry <Return> "+ setCurrent nColor"

#
# Strip widths and offsets:

   frame .p.space3 -width 1 -height 6 
   pack  .p.space3 -in .p.spec -side top -fill x

   frame .p.specX
   frame .p.specY
   frame .p.specZ
   frame .p.specZLim

   label .p.specXWLab -text "XW" -width 2
   label .p.specYWLab -text "YW" -width 2
   label .p.specZWLab -text "ZW" -width 2

   entry .p.specXWEntry -textvariable currentInfo(xw) -width 9
   entry .p.specYWEntry -textvariable currentInfo(yw) -width 9
   entry .p.specZWEntry -textvariable currentInfo(zw) -width 9

   set winInfo(xw) .p.specXWEntry
   set winInfo(yw) .p.specYWEntry
   set winInfo(zw) .p.specZWEntry

   label .p.specXOffLab -text "XOff" -width 5 -anchor e
   label .p.specYOffLab -text "YOff" -width 5 -anchor e 
   label .p.specZOffLab -text "ZOff" -width 5 -anchor e

   entry .p.specXOffEntry -textvariable currentInfo(xOff) -width 8
   entry .p.specYOffEntry -textvariable currentInfo(yOff) -width 8
   entry .p.specZOffEntry -textvariable currentInfo(zOff) -width 8

   label .p.specZ1Lab -text "Z1" -width 2 -anchor e
   label .p.specZNLab -text "ZN" -width 3 -anchor e

   entry .p.specZ1Entry -textvariable currentInfo(z1) -width 11 
   entry .p.specZNEntry -textvariable currentInfo(zn) -width 11 

   set winInfo(xOff) .p.specXOffEntry
   set winInfo(yOff) .p.specYOffEntry
   set winInfo(zOff) .p.specZOffEntry

   set winInfo(z1) .p.specZ1Entry
   set winInfo(zn) .p.specZNEntry

   pack  .p.specXWLab .p.specXWEntry .p.specXOffLab .p.specXOffEntry \
         -in .p.specX -side left

   pack  .p.specYWLab .p.specYWEntry .p.specYOffLab .p.specYOffEntry \
         -in .p.specY -side left

   pack  .p.specZWLab .p.specZWEntry .p.specZOffLab .p.specZOffEntry \
         -in .p.specZ -side left

   pack  .p.specZ1Lab .p.specZ1Entry .p.specZNLab .p.specZNEntry \
         -in .p.specZLim -side left 

   pack .p.specX .p.specY .p.specZ -in .p.spec -side top -anchor center

   frame .p.space4 -width 1 -height 6 
   pack  .p.space4 -in .p.spec -side top -fill x

   pack .p.specZLim -in .p.spec -side top -anchor center

   frame .p.space5 -width 1 -height 6 
   pack  .p.space5 -in .p.spec -side top -fill x

   button .p.reset -text "Reset" -command "resetCurrent"
   pack   .p.reset -in .p.spec -side top

   bind .p.specXWEntry     <Return> "+ setCurrent xw"
   bind .p.specYWEntry     <Return> "+ setCurrent yw"
   bind .p.specZWEntry     <Return> "+ setCurrent zw"

   bind .p.specXOffEntry   <Return> "+ setCurrent xOff"
   bind .p.specYOffEntry   <Return> "+ setCurrent yOff"
   bind .p.specZOffEntry   <Return> "+ setCurrent zOff"
 
   bind .p.specZ1Entry     <Return> "+ setCurrent z1"
   bind .p.specZNEntry     <Return> "+ setCurrent zn"

   pack .p.spec -in .p -side top -fill x

#
# Misc:

   button .p.quit -text Quit -command exit
   pack   .p.quit -in .p -side bottom -fill x
   update

   set eColorDef [option get .p.specXWEntry background Background]
}

proc resetCurrent {} \
{
   global winInfo eColorDef eColorSet currentInfo panelInfo panelCount specCount dataValid
   global hi pLev nLev pColor nColor xw yw zw xOff yOff zOff z1 zn

   if {!$dataValid} return

   set panelID $currentInfo(panelID)
   set specID  $currentInfo(specID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount

   set panelInfo($panelID,$specID,setCount) 0

   foreach v "hi pLev nLev pColor nColor xw yw zw xOff yOff zOff z1 zn" \
      {
       if {![info exists ${v}($specID)]}   continue

       eval "set currentInfo($v)        \$${v}($specID)"
       eval "set ${v}($panelID,$specID) \$${v}($specID)"
     }

   readCurrent
   drawCurrent
}

proc hiliteCurrent {} \
{
   global winInfo eColorDef eColorSet currentInfo panelInfo panelCount specCount dataValid
   global hi pLev nLev pColor nColor xw yw zw xOff yOff zOff z1 zn

   if {!$dataValid} return

   set panelID $currentInfo(panelID)
   set specID  $currentInfo(specID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount

   set panelInfo($panelID,$specID,setCount) 0
 
   foreach v "hi pLev nLev pColor nColor xw yw zw xOff yOff zOff z1 zn" \
      {
       if {![info exists winInfo($v)]} continue
       if {![winfo exists $winInfo($v)]} continue

       $winInfo($v) configure -background $eColorDef

       if {![info exists currentInfo($v)]} continue
       if {![info exists ${v}($specID)]}   continue

       set val [getCurrentVal $v $specID $panelID]

       eval "set defVal \$${v}($specID)"

       if {[strcasecmp $val $defVal]} \
          {
           $winInfo($v) configure -background $eColorSet
           incr panelInfo($panelID,$specID,setCount)
          }
      }
  
   update 
}

proc setCurrent args \
{
   global title hi pLev nLev pColor nColor xw yw zw xOff yOff zOff z1 zn
   global r1 rN nullRes currentInfo panelInfo specCount panelCount 

   set key [lindex $args 0]
   set n   [llength $args]

   set incrFlag 0
   set decrFlag 0
   set valFlag  0

   if {[lsearch -exact $args incr] >= 0} {set incrFlag 1}
   if {[lsearch -exact $args decr] >= 0} {set decrFlag 1}

   if {[lsearch -exact $args val]  >= 0} \
      {
       set valFlag 1
       set currentInfo($key) [lindex $args end]
      }

   set panelID $currentInfo(panelID)
   set specID  $currentInfo(specID)
   set resID   $currentInfo(resID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount
   iClipC resID    $r1 $rN

   switch $key \
      {
       panelID {
                if {$incrFlag} \
                   {
                    cIncr currentInfo(panelID)  1 1 $panelCount
                   } \
                elseif {$decrFlag} \
                   {
                    cIncr currentInfo(panelID) -1 1 $panelCount
                   }

                set loc $currentInfo(panelID)
                set currentInfo(assID) $panelInfo($loc,assID)

                updateCurrentInfo
                updatePanelShiftLab $currentInfo(panelID)

                hilitePanelID $currentInfo(panelID)
                hiliteCurrent
               }
 
       slotID  {
                if {$incrFlag} \
                   {
                    cIncr currentInfo(slotID)  1 1 $panelCount
                   } \
                elseif {$decrFlag} \
                   {
                    cIncr currentInfo(slotID) -1 1 $panelCount
                   } \
                else \
                   {
                    cIncr currentInfo(slotID) 0  1 $panelCount
                   }

                updateCurrentInfo

                set currentInfo(resID) [expr $currentInfo(slotID) + $r1 - 1]

                hiliteResID $currentInfo(resID)
                hiliteCurrent
               }

       resID  {
               if {$incrFlag} \
                  {
                   cIncr currentInfo(resID)  1 $r1 $rN
                  } \
               elseif {$decrFlag} \
                  {
                   cIncr currentInfo(resID) -1 $r1 $rN
                  } \
               else \
                  {
                   cIncr currentInfo(resID)  0 $r1 $rN
                  }

               updateCurrentInfo

               set slotID [expr 1 + $currentInfo(resID) - $r1]

               if {$slotID > 0 && $slotID <= $panelCount} \
                  {
                   set currentInfo(slotID) $slotID
                  }

               hiliteResID $currentInfo(resID)
               hiliteCurrent
              }

       assID   {
                if {$incrFlag} \
                   {
                    cIncr currentInfo(panelID)  1 1 $panelCount

                    set loc $currentInfo(panelID)
                    set currentInfo(assID) $panelInfo($loc,assID)
                   } \
                elseif {$decrFlag} \
                   {
                    cIncr currentInfo(panelID) -1 1 $panelCount

                    set loc $currentInfo(panelID)
                    set currentInfo(assID) $panelInfo($loc,assID)
                   } \
                else \
                   {
                    set thisPanelID [findPanelIDByAssID $currentInfo(assID)] 

                    if {$thisPanelID} \
                       {
                        set currentInfo(panelID) $thisPanelID
                       } \
                    else \
                       {
                        set loc $currentInfo(panelID)
                        set currentInfo(assID) $panelInfo($loc,assID) 
                       }
                   } 

                updateCurrentInfo 
                updatePanelShiftLab $currentInfo(panelID)

                hilitePanelID $currentInfo(panelID)
                hiliteCurrent
               }

       specID   {
                 if {$incrFlag} \
                    {
                     cIncr currentInfo(specID)  1 1 $specCount
                    } \
                 elseif {$decrFlag} \
                    {
                     cIncr currentInfo(specID) -1 1 $specCount
                    } \

                 updateCurrentInfo 

                 hilitePanelID $currentInfo(panelID)
                 hiliteCurrent
                }
 
       specType {
                 set currentInfo(specID) [getSpecID $currentInfo(specType)]

                 if {$incrFlag} \
                    {
                     cIncr currentInfo(specID)  1 1 $specCount
                    } \
                 elseif {$decrFlag} \
                    {
                     cIncr currentInfo(specID) -1 1 $specCount
                    } \

                 updateCurrentInfo 

                 hilitePanelID $currentInfo(panelID)
                 hiliteCurrent
                }

       hi {
           if {$incrFlag} \
              {
               set currentInfo(hi) [incrHiVal $currentInfo(hi) 1.3]
              } \
           elseif {$decrFlag} \
              {
               set currentInfo(hi) [decrHiVal $currentInfo(hi) 1.3]
              }

           updateSpecificInfo hi
           drawCurrent
          }

       pLev {
             if {$incrFlag} \
                {
                 cIncr currentInfo(pLev)  1 0 128
                } \
             else \
                {
                 cIncr currentInfo(pLev) -1 0 128
                }

             updateSpecificInfo pLev 
             drawCurrent
            }

       nLev {
             if {$incrFlag} \
                {
                 cIncr currentInfo(nLev)  1 0 128
                } \
             else \
                {
                 cIncr currentInfo(nLev) -1 0 128
                }

             updateSpecificInfo nLev 
             drawCurrent
            } 

       pColor {
               updateSpecificInfo pColor 
               drawCurrent
              }

       nColor {
               updateSpecificInfo nColor 
               drawCurrent
              }

       xw {
           if {$incrFlag} \
              {
              } \
           elseif {$decrFlag} \
              {
              }

           updateSpecificInfo xw 
           readCurrent
           drawCurrent
          }

       yw {
           if {$incrFlag} \
              {
              } \
           elseif {$decrFlag} \
              {
              } 

           updateSpecificInfo yw 
           readCurrent
           drawCurrent
          }

       zw {
           if {$incrFlag} \
              {
              } \
           elseif {$decrFlag} \
              {
              } 

           updateSpecificInfo zw 
           readCurrent
           drawCurrent
          }

       z1 {
           updateSpecificInfo z1 
           readCurrent
           drawCurrent
          }

       zn {
           updateSpecificInfo zn 
           readCurrent
           drawCurrent
          }

       xOff {
             if {$incrFlag} \
                {
                 set currentInfo(xOff) [expr $currentInfo(xOff) + 0.01]
                } \
             elseif {$decrFlag} \
                {
                 set currentInfo(xOff) [expr $currentInfo(xOff) - 0.01]
                } 

             updateSpecificInfo xOff 
             readCurrent
             drawCurrent
            }

       yOff {
             if {$incrFlag} \
                {
                 set currentInfo(yOff) [expr $currentInfo(yOff) + 0.02]
                } \
             elseif {$decrFlag} \
                {
                 set currentInfo(yOff) [expr $currentInfo(yOff) - 0.02]
                } 

             updateSpecificInfo yOff
             readCurrent
             drawCurrent
            }

       zOff {
             if {$incrFlag} \
                {
                 set currentInfo(yOff) [expr $currentInfo(yOff) + 0.02]
                } \
             elseif {$decrFlag} \
                {
                 set currentInfo(yOff) [expr $currentInfo(yOff) + 0.02]
                } 

             updateSpecificInfo zOff
             readCurrent
             drawCurrent
            }
      }
}

proc updatePanelShiftLab { panelID } \
{
   global panelInfo

   if {![winfo exists .p.groupPPMLab]} return

   set t ""

   if {[info exists panelInfo($panelID,assID)]} \
      {
       set xVal $panelInfo($panelID,xPPM)
       set yVal $panelInfo($panelID,yPPM)

       if {$xVal >= 999.0} \
          {
           set xVal None
          } \
       else \
          {
           append xVal ppm
          }

       if {$yVal >= 999.0} \
          {
           set yVal None
          } \
       else \
          {
           append yVal ppm
          }

       set t    "HN: $xVal N: $yVal"
      }

   .p.groupPPMLab configure -text $t 
}

proc refreshPeakLabels {} \
{
   global pkInfo

   if {$pkInfo(showShift)} \
      {
       .c itemconfigure pkLab -state normal 
      } \
   else \
      {
       .c itemconfigure pkLab -state hidden
      }

   update
}

proc refreshPeakLines {} \
{
   global pkInfo 

   if {$pkInfo(showLine)} \
      {
       .c itemconfigure pkLine -state normal
      } \
   else \
      {
       .c itemconfigure pkLine -state hidden
      }

   update
}

proc refreshJoinLines {} \
{
   global panelCount

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       refreshJoin $slotID
      }

   update
}

proc refreshAnchorLines {} \
{
   global panelCount 

   for {set slotID 1} {$slotID <= $panelCount} {incr slotID} \
      {
       refreshThisAnchor $slotID
      }

   update
}

proc toggleCursorBar { n } \
{
   global cursorInfo 

   if {$cursorInfo($n,showbar)} \
      {
       .c itemconfigure $cursorInfo($n,cID) -state normal 
      } \
   else \
      {
       .c itemconfigure $cursorInfo($n,cID) -state hidden 
       .c itemconfigure $cursorInfo($n,tID) -state hidden 
      }

   update
}

proc toggleCursorText { n type } \
{
   global cursorInfo specCount

   if {[strcmp $type text]} \
      {
       set nn 0

       if {[strcmp $type pix]} \
          {
           if {$cursorInfo($n,showpix)} {incr nn}
          }

       for {set specID 1} {$specID <= $specCount} {incr specID} \
          {
           if {[strcmp $type $specID]} \
              {
               if {$cursorInfo($n,show$specID)} {incr nn}
              }
          }

       if {!$nn} \
          {
           if {$cursorInfo($n,show$type)} \
              {
               set cursorInfo($n,showbar)  1
               set cursorInfo($n,showtext) 1
        
               toggleCursorBar  $n
              }
          }
      }

   updateCursorText $n

   if {$cursorInfo($n,showtext)} \
      {
       .c itemconfigure $cursorInfo($n,tID) -state normal 
      } \
   else \
      {
       .c itemconfigure $cursorInfo($n,tID) -state hidden 
      }

   update
}

proc findJoin {} \
{
   global panelList panelInfo panelCount currentInfo

   set slotID  $currentInfo(slotID)
   set panelID $currentInfo(panelID)

   if {$panelID < 1 || $panelID > $panelCount} return

   set newList [lrange $panelList 0 [expr $slotID - 1]]

   foreach thisID $panelInfo($panelID,joinList) \
      {
       if {[lsearch -exact $newList $thisID] < 0} {lappend newList $thisID}
      }

   for {set thisID 1} {$thisID <= $panelCount} {incr thisID} \
      {
       if {[lsearch -exact $newList $thisID] < 0} {lappend newList $thisID}
      }

   set panelList $newList

   reorderDisplay
}

proc setCanvasCursor { cName } \
{
   global currentCursor

   if {![strcasecmp $currentCursor $cName]} return

   catch ".c configure -cursor $cName"

   set currentCursor $cName
   update
}

proc captureAnimation {} \
{
   set step 0.0001

   set prevView -1

   for {set off 0.0} {$off < 1.0} {set off [expr $off + $step]} \
      {
       .c xview moveto $off
       set thisView [lindex [.c xview] 0]
       if {![strcmp $thisView $prevView]} continue

       update

       for {set ii 0} {$ii < 100000} {incr ii} {set c [expr cos($ii)]}
       set prevView $thisView

#      captureScreen
      }
}

proc captureScreen { } \
{
   global snapDir imageID 

   update

   if {![file exists $snapDir]} {catch "file mkdir $snapDir"}

   set im [image create photo]
   winop snap .c $im

   set n [llength [glob -nocomplain $snapDir/*.gif]]

   for {set i 1} {$i <= $n + 1} {incr i} \
      {
       set outName [format $snapDir/im%05d.gif $i]
       if {[file exists $outName]} continue

       set imageID $i
       break
      }

   set outName [format $snapDir/im%05d.gif $imageID]

   $im write -format gif $outName

   image delete $im
}

proc updateSpecificInfo { aPtr } \
{
   global $aPtr

   global currentInfo specCount panelCount r1 rN

   if {![info exists currentInfo($aPtr)]} \
      {
       puts stderr "No info value for $aPtr"
       return
      }

   set val     $currentInfo($aPtr)
   set panelID $currentInfo(panelID)
   set slotID  $currentInfo(slotID)
   set assID   $currentInfo(assID)
   set specID  $currentInfo(specID)
   set resID   $currentInfo(resID)

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount
   iClipC resID    $r1 $rN

   eval "set ${aPtr}($panelID,$specID) $val" 

   return ${aPtr}($panelID,$specID)
}

proc updateCurrentInfo args \
{
   global title hi pLev nLev pColor nColor xw yw zw z1 zn xOff yOff zOff 
   global currentInfo specCount panelCount r1 rN

   set argv [concat updateCurrentInfo $args]

   if {[flagLoc $argv -init]} \
      {
       set currentInfo(panelID)  1
       set currentInfo(slotID)   1
       set currentInfo(assID)    1
       set currentInfo(specID)   1
       set currentInfo(resID)    $r1
      }

   set panelID $currentInfo(panelID)
   set slotID  $currentInfo(slotID)
   set assID   $currentInfo(assID)
   set specID  $currentInfo(specID)
   set resID   $currentInfo(resID)

   getArgD $argv -panelID  panelID
   getArgD $argv -slotID   slotID
   getArgD $argv -specID   specID
   getArgD $argv -assID    assID
   getArgD $argv -resID    resID

   iClipC panelID  1   $panelCount
   iClipC specID   1   $specCount
   iClipC resID    $r1 $rN

   set currentInfo(panelID)  $panelID
   set currentInfo(slotID)   $slotID
   set currentInfo(assID)    $assID
   set currentInfo(specID)   $specID 
   set currentInfo(resID)    $resID
   set currentInfo(specType) $title($specID)

   set currentInfo(hi)       [getCurrentVal hi     $specID $panelID]
   set currentInfo(pLev)     [getCurrentVal pLev   $specID $panelID]
   set currentInfo(nLev)     [getCurrentVal nLev   $specID $panelID]
   set currentInfo(pColor)   [getCurrentVal pColor $specID $panelID]
   set currentInfo(nColor)   [getCurrentVal nColor $specID $panelID]
   set currentInfo(xw)       [getCurrentVal xw     $specID $panelID]
   set currentInfo(yw)       [getCurrentVal yw     $specID $panelID]
   set currentInfo(zw)       [getCurrentVal zw     $specID $panelID]
   set currentInfo(xOff)     [getCurrentVal xOff   $specID $panelID]
   set currentInfo(yOff)     [getCurrentVal yOff   $specID $panelID]
   set currentInfo(zOff)     [getCurrentVal zOff   $specID $panelID]
   set currentInfo(z1)       [getCurrentVal z1     $specID $panelID]
   set currentInfo(zn)       [getCurrentVal zn     $specID $panelID]
}

proc getUnusedID { idList } \
{
   set n [llength $idList]
   if {!$n} {return 1}

   for {set i [expr $n + 1]} {$i <= 2*$n + 2} {incr i} \
      {
       if {[lsearch -exact $idList $i] < 0} {return $i}
      }

   return -1
}

proc getCurrentVal { aPtr specID panelID } \
{
   global $aPtr

   set val 0

   set iFlag [eval "info exists ${aPtr}($panelID,$specID)"]

   if {$iFlag} \
      {
       eval "set val \$${aPtr}($panelID,$specID)"
       if {[info exists val]} {return $val}
      }

   set iFlag [eval "info exists ${aPtr}($specID)"]

   if {$iFlag} \
      {
       eval "set val \$${aPtr}($specID)"
       if {[info exists val]} {return $val}
      }

   return 0
}

proc getNextCellID {} \
{
   global cellList

   set i [expr 1 + [llength $cellList]]

   while {$i < 100000} \
      {
       if {[lsearch -exact $cellList $i] < 0} \
          {
           lappend cellList $i
           return $i
          }

       incr i
      }

   puts "Cell Allocation Failure."
   
   return 100001
}

proc parseKey { kSym xPix yPix } \
{
   global panelInfo

   puts stderr "Key $kSym at $xPix $yPix."

   switch -exact $kSym \
      {
       comma        { }
       period       { }

       bracketleft  { swapStrips right decr }
       bracketright { swapStrips right incr }

       semicolon    { swapStrips left decr }
       apostrophe   { swapStrips left incr }

       minus        { }
       equal        { }

       a {}

       j {findJoin}
      }
}

initSeqInfo
initData
initCoordTab

initJoinTab
initResInfo

initPkTab
initWin

updateCurrentInfo -init

createSlots
createPanels

readData
updateRefInfo
updateShiftInfo
findPeakGroups

initStripDisplay
initSelectors

initHistory
reorderDisplay

initBind
initCursors
initCmndPanel
 
updateLink $thisInterp
hiliteCurrent

if {$plotFlag} plotAllStrips

if {$quitFlag} exit
