#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# Build start-up script for NMRPipe Super-excellent Strip viewer.

if {[flagLoc $ARGV -help]} \
{
    exit 1 
}

proc scrollInitInitVars {} \
{
   global objInfo

   set objInfo(stripLoc,specName) None
   set objInfo(stripLoc,pkName)    None
}

proc scrollInitInitArgs {} \
{
   global objInfo ARGC ARGV

   set objInfo(idList) \
    {locSpec hncoSpec  hncaSpec hnhaSpec   hnnSpec     hacaSpec hhSpec} 

   set objInfo(showList) \
    {locSpec hncoSpec  hncaSpec hnhaSpec hhSpec} 

   set objInfo(classList) \
    {HSQC    HNCO      HNCA     HNHA       HNN         HACA     HOMO}

   set countList \
    {2       1         4        3          2           2         3}

   set pList  \
    {multi   green     multi    yellow     orchid      firebrick white}

   set nList  \
    {multi   green4    multi    yellow4    darkOrchid  magenta   gray10}

   set x1List \
    {Auto    Auto      Auto     Auto       Auto        Auto      Auto}

   set xnList \
    {Auto    Auto      Auto     Auto       Auto        Auto      Auto}

   set hiList \
    {1.5@    1.5@      1.5@     1.5@       1.5@        1.5@}

   set tList  {
               "Strip Locations" 
               "HN CO Spectra"  
               "HN CA and CB Spectra" 
               "HN 1H Spectra"  
               "HN 15N Spectra"
               "HC Spectra" 
               "HH Spectra" 
              } 

   set loc 0

   foreach id $objInfo(idList) \
      {
       set n [lindex $countList $loc]

       set objInfo($id,title)    [lindex $tList  $loc]
       set objInfo($id,posColor) [lindex $pList  $loc]
       set objInfo($id,negColor) [lindex $nList  $loc]
       set objInfo($id,specX1)   [lindex $x1List $loc]
       set objInfo($id,specXN)   [lindex $xnList $loc]
       set objInfo($id,count)    $n

       for {set i 1} {$i <= $n} {incr i} \
          {
           set objInfo($id$i,specName) Auto
           set objInfo($id$i,tabName)  Auto
          } 

       incr loc
      }
}

proc scrollInitInitWin {} \
{
   global objInfo wishFlag env

   if {!$wishFlag} return

   foreach id $objInfo(idList) \
      {
       frame .${id}Frame -border 3 -relief groove
      }

   foreach id $objInfo(idList) \
      {
       if {[lsearch -exact $objInfo(showList) $id] < 0} continue

       label .${id}FrameTitle -text $objInfo($id,title) -anchor w
       pack  .${id}FrameTitle -in .${id}Frame -side top -expand 1 -fill x

       for {set i 1} {$i <= $objInfo($id,count)} {incr i} \
          {
           frame  .${id}Frame$i

           label  .${id}SpecLab$i  -text "Spectrum:" -width 10 -anchor e
           entry  .${id}SpecEnt$i -width 12 -textvariable objInfo($id$i,specName)

           button .${id}SpecBut$i -command "setSpecName $id $i" \
                  -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green

           label  .${id}PeakLab$i -text "Peak Table:" -width 12 -anchor e
           entry  .${id}PeakEnt$i -width 12 -textvariable objInfo($id$i,pkName)

           button .${id}PeakBut$i -command "setPeakName $id $i" \
                  -relief raised -bitmap @$env(NMRTXT)/caret.map -fg green

           label .${id}Space$i   -width 1 -text " "
           label .${id}SpecTit$i -text " " -width 14 -anchor w -background white
           label .${id}SpaceN$i  -width 1 -text " "

           pack .${id}SpecLab$i .${id}SpecEnt$i .${id}SpecBut$i \
                .${id}PeakLab$i .${id}PeakEnt$i .${id}PeakBut$i \
                .${id}Space$i   .${id}SpecTit$i .${id}SpaceN$i  \
                -in .${id}Frame$i -side left

           pack .${id}Frame$i -in .${id}Frame -side top -expand 1 -fill x

           set objInfo($id$i,titLab) .${id}SpecTit$i
          }

       frame  .${id}FrameColor

       label  .${id}PColorLab -text "Pos Color:" -width 10 -anchor e
       entry  .${id}PColorEnt -width 12 -textvariable objInfo($id,posColor)

       button .${id}PColorBut -command "setSpecColor $id posColor" \
              -relief raised -bitmap @$env(NMRTXT)/caret.map -fg blue

       label  .${id}NColorLab -text "Neg Color:" -width 12 -anchor e
       entry  .${id}NColorEnt -width 12 -textvariable objInfo($id,negColor)

       button .${id}NColorBut -command "setSpecColor $id negColor" \
              -relief raised -bitmap @$env(NMRTXT)/caret.map -fg blue 

       label .${id}SpaceColor -width 1 -text " "

       pack .${id}PColorLab .${id}PColorEnt .${id}PColorBut \
            .${id}NColorLab .${id}NColorEnt .${id}NColorBut \
            .${id}SpaceColor -in .${id}FrameColor -side left

       pack  .${id}FrameColor -in .${id}Frame -side top -expand 1 -fill x


       frame  .${id}FrameLim

       label  .${id}x1Lab -text "X1:" -width 8 -anchor e
       entry  .${id}x1Ent -width 12 -textvariable objInfo($id,specX1)

       label  .${id}xnLab -text "XN:" -width 14 -anchor e
       entry  .${id}xnEnt -width 12 -textvariable objInfo($id,specXN)

       label .${id}SpaceLim -width 1 -text " "

       pack .${id}x1Lab .${id}x1Ent  .${id}xnLab .${id}xnEnt  \
            .${id}SpaceLim -in .${id}FrameLim -side left

       pack  .${id}FrameLim -in .${id}Frame -side top -expand 1 -fill x

       frame .${id}SpaceN -height 4
       pack  .${id}SpaceN -in .${id}Frame   -side top -expand 1 -fill x
       
       update
      }

   frame .space1 -height 4
   frame .cmnd  
   frame .spaceN -height 4
 
   pack  .space1 -in . -side top
   
   foreach id $objInfo(idList) \
      {
       pack .${id}Frame -in . -side top -expand 1 -fill both
       update
      }

   button .scan  -text Scan    -command scanScroll
   button .exec  -text Execute -command execScroll
   button .reset -text Reset   -command resetScroll
   button .quit  -text Quit    -command quitScroll

   pack  .spaceN -in . -side top

   pack .scan .exec .reset .quit -in .cmnd -side left -expand 1 -fill x
   pack .cmnd -in . -side top -expand 1 -fill x

   update
}

proc scanScroll {} \
{
   global objInfo
}

proc quitScroll {} \
{
   global objInfo

   saveScroll
   exit 0
}

proc saveScroll {} \
{
}

proc resetScroll {} \
{
   global objInfo
}

proc setSpecName { id i } \
{
   global objInfo

    set fileTypes \
       {
        { {ft3 Files}        {.ft3} }
        { {ft  Files}        {.ft}  }   
        { {ft1 Files}        {.ft1} }
        { {ft2 Files}        {.ft2} }
        { {ft4 Files}        {.ft4} }
        { {fid Files}        {.fid} }
        { {dat Files}        {.dat} }
        { {DAT Files}        {.DAT} }
        { {All Files}        *      }
       }

   set objInfo($id$i,specName) \
     [tk_getOpenFile -title "Spectrum $id" -filetypes $fileTypes]

   if {![llength $objInfo($id$i,specName)]} {set objInfo($id$i,specName) None}
}

proc setPeakName { id i } \
{
   global objInfo
           
    set fileTypes \
       {
        { {All Files}        *      }
        { {tab Files}        {.tab} }
       }
 
   set objInfo($id$i,pkName) \
     [tk_getOpenFile -title "Peak Table $id" -filetypes $fileTypes]
 
   if {![llength $objInfo($id$i,pkName)]} {set objInfo($id$i,pkName) None}
}

proc setSpecColor { id mode } \
{
   global objInfo

   if {[info exists objInfo($id,$mode,colorWindow)]} \
      {
       if {[winfo exists $objInfo($id,$mode,colorWindow)]} \
          {
           catch "raise $objInfo($id,$mode,colorWindow)"
           return
          }
      }

   set objInfo($id,$mode,colorWindow) \
    [getColorChoice -color $objInfo($id,$mode) -command "specColorChoiceProc $id $mode"]
}

proc specColorChoiceProc { id mode color } \
{
   global objInfo

   set objInfo($id,$mode) $color
}

proc scrollInitScan {} \
{
   global objInfo
}

proc scrollInitCreateScript {} \
{
   global objInfo 
}

scrollInitInitVars
scrollInitInitArgs
scrollInitInitWin


