#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Simple General-Purpose 3D  Strip Drawing via 2D Projection."
   echo " -dir       dList  [* ...]            Directories to Search."
   echo " -excl      eList  [None ...]         Directories to Exclude."
   echo " -specName  sName  [test%03d.ft3 ...] Default Spectrum Names."
   echo " -defHi     hi     [10%]              Default Contour Height."
   echo " -pkHi      hi     [Full]             Height for 2D Peak Detection of Projection."
   echo " -nus                                 Assume Non-Uniformly Sampled Data."
   echo " -assume                              Assume All Requested Directories Exist."
   echo " -list                                Print Commands Instead of Executing Them."
   echo "Arguments for scroll.tcl (Should be Last on Command Line):"
   echo " -scrollArgs args  [None]             Arguments for scroll.tcl."
   exit 0
endif

set scrollArgs = ""

@ nusFlag    = 0
@ listFlag   = 0
@ assumeFlag = 0

if (-e fid.com) then
   set sInfo = (`grep -E -e -sample fid.com | wc -l`)

   if ($sInfo == 0) then
      @ nusFlag = 0
   else
      @ nusFlag = 1
   endif
endif

if (`flagLoc $argv -nus`) then
   @ nusFlag = 1
endif

if (`flagLoc $argv -nonus`) then
   @ nusFlag = 0
endif

set thisPosColors = (red orange yellow green blue indigo violet pink magenta purple darkOrchid tan darkGreen chocolate cornflowerbluet gray brown azure coral)
set thisNegColors = (indigo magenta purple darkOrchid tan darkGreen chocolate cornflowerbluet gray brown azure coral blue red yellow green orange pink violet)
set thisNameList  = (test%03d.ft3 spec%03d.ft3 test%04d.ft3 spec%04d.ft3)
set thisDirList   = ""

foreach d (*)
   if (-d $d) then
      if (`string csame $d ft`) then
         continue
      endif

      set thisDirList = ($thisDirList $d)
   endif
end

if ($nusFlag == 0) then
   set thisDirList = (ft $thisDirList)
endif

set dirList   = (`getListArgD $argv -dir     -- $thisDirList`)
set defNames  = (`getListArgD $argv -defName -- $thisNameList`)
set exclList  = (`getListArgD $argv -excl    -- None`)

set posColors = (`getListArgD $argv -posColors -- $thisPosColors`)
set negColors = (`getListArgD $argv -negColors -- $thisNegColors`)

set defHi     = (`getArgD $argv -defHi 10%`)
set pkHi      = (`getArgD $argv -pkHi  Full`)

if (`flagLoc $argv -assume`) then
   @ assumeFlag = 1
endif

if (`flagLoc $argv -noassume`) then
   @ assumeFlag = 0
endif

if (`flagLoc $argv -list`) then
   @ listFlag = 1
endif

set loc = (`flagLoc $argv -scrollArgs`)

if ($loc) then
   @ loc++
   set scrollArgs = ($argv[$loc-])
endif

set inList     = ""
set hiList     = ""
set pColorList = ""
set nColorList = ""
set titList    = ""

@ posColorCount = $#posColors
@ negColorCount = $#negColors

@ n = 0

foreach thisDir ($dirList)
   set loc = (`string lsearch $exclList $thisDir`)

   if ($loc != 0) then
      continue
   endif

   foreach thisName ($defNames)
      set testName = (`nmrPrintf $thisDir/$thisName 1`)

      if ($assumeFlag == 0 && !(-e $testName)) then
         continue
      endif

      @ n++

      set pLoc = (`string hash 1 $posColorCount $testName`)
      set nLoc = (`string hash 1 $negColorCount $testName`)

      if ($n == 1) then
         set pColorList = ($pColorList multi)
         set nColorList = ($nColorList multi)
      else
         set pColorList = ($pColorList $posColors[$pLoc])
         set nColorList = ($nColorList $negColors[$nLoc])
      endif

      set inList  = ($inList $thisDir/$thisName)
      set titList = ($titList $thisDir)
      set hiList  = ($hiList  $defHi)
   end
end

if (`string length $inList` == 0) then
   if ($listFlag) then
      nmrPrintf "scrollMaster: No Spectra Found.\n"
   else
      nmrPrintf -stderr "scrollMaster: No Spectra Found.\n"
   endif

   exit 0
endif

if ($listFlag) then
   set inCount = $#inList
   set fmtStr  = ""
   set i       = 1

   foreach inName ($inList)
      if ($i == 1) then
         set fmtStr = "%-`string length $inName`s"
      else if ($i == $inCount) then
         set fmtStr = "$fmtStr %s"
      else
         set fmtStr = "$fmtStr %-`string length $inName`s"
      endif

      @ i++
   end

   nmrPrintf "set inListFull = ($fmtStr)\n" $inList
   nmrPrintf "set tListFull  = ($fmtStr)\n" $titList
   nmrPrintf "set pListFull  = ($fmtStr)\n" $pColorList
   nmrPrintf "set nListFull  = ($fmtStr)\n" $nColorList
   nmrPrintf "set hiListFull = ($fmtStr)\n" $hiList

   echo 'set pkHi       =' $pkHi
   echo ''
   echo 'set inList = ""'
   echo 'set tList  = ""'
   echo 'set pList  = ""'
   echo 'set nList  = ""'
   echo 'set hiList = ""'
   echo ''
   echo '@ i = 0'
   echo '@ n = 0'
   echo ''
   echo 'foreach inName ($inListFull)'
   echo '   @ i++'
   echo ''
   echo '   set thisName = (`nmrPrintf $inName 1`)'
   echo ''
   echo '   if (-e $thisName) then'
   echo '      set inList = ($inList $inListFull[$i])'
   echo '      set tList  = ($tList  $tListFull[$i])'
   echo '      set nList  = ($nList  $nListFull[$i])'
   echo '      set pList  = ($pList  $pListFull[$i])'
   echo '      set hiList = ($hiList $hiListFull[$i])'
   echo '      @ n++'
   echo '   endif'
   echo 'end'
   echo ''
   echo 'if ($n == 0) then'
   echo '   echo "Strip Plot Error: No Spectra Found."'
   echo '   exit 1'
   echo 'endif'
   echo ''
   echo 'proj3D.tcl -in $inList[1]'
   echo 'peakHN.tcl -in $inList[1] -out hn.proj.tab -proj -hi $pkHi'
   echo ''

   prettyPrintCmnd.tcl -nocomment \
    scroll.tcl $scrollArgs -tab hn.proj.tab -pair \
               -noyGrid -colorB dimGray -colorW black -fg black \
               -in '$inList' -hi '$hiList' -colorP '$pList' -colorN '$nList' -title '$tList'
else
   proj3D.tcl -in $inList[1]

   peakHN.tcl -in $inList[1] -out hn.proj.tab -proj -hi $pkHi 

   echo "Spectra: $inList"
   echo "Heights: $hiList"

   scroll.tcl $* -tab hn.proj.tab -pair \
              -noyGrid -colorB dimGray -colorW black -fg black \
              -in $inList -hi $hiList -colorP $pColorList -colorN $nColorList -title $titList
endif
