#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inName    ft/test%03d.ft3
set tabName   hn.tab
set prevName  None
set hnTab     0

set hi        Auto 

set mult      2.0
set plev      10
set nlev      10
set seqFlag   0

set xGridArg -xGrid
set yGridArg -yGrid

set xName    Auto 
set yName    Auto 
set zName    Auto 

set xVarName Auto 
set yVarName Auto

set labelVar INDEX

#
#---------------------------------------------------------------

if {[flagLoc $ARGV -help]} \
   {
    puts "On-Screen Strip plots of a single 3D spectrum."
    puts "  -in     inName  \[ft/test%03d.ft3\] 3D Spectral data file template."
    puts "  -tab    tabName \[hn.tab\]          Initial 2D Location Table."
    puts "  -hi     contHi  \[Auto\]            First contour height."
    puts "  -plev   nPos    \[10\]              Number of Positive Contours."
    puts "  -nlev   nNeg    \[10\]              Number of Negative Contours."
    puts "  -mult   mult    \[2.0\]             Factor for Auto contour height."
    puts "Strip Axis Names:"
    puts "  -xName  xName   \[Auto\]   First axis name of strip 2D coord."
    puts "  -yName  yName   \[Auto\]   Second axis name of strip 2D coord."
    puts "  -zName  zName   \[Auto\]   Axis name of strip dimension."
    puts "Coordinate Table Names:"
    puts "  -xVar   xVar    \[Auto\]   Variable name of first 2D PPM coord."
    puts "  -yVar   yVar    \[Auto\]   Variable name of second 2D PPM coord."
    puts "Strip Label:"
    puts "  -lab    lVar    \[INDEX\]  Variable name for Strip Label."
    exit
   }

getArgD $ARGV -in      inName
getArgD $ARGV -tab     tabName
getArgD $ARGV -hi      hi
getArgD $ARGV -mult    mult 
getArgD $ARGV -plev    plev
getArgD $ARGV -nlev    nlev 

getArgD $ARGV -xName   xName
getArgD $ARGV -yName   yName
getArgD $ARGV -zName   zName

getArgD $ARGV -xVar    xVarName
getArgD $ARGV -yVar    yVarName

getArgD $ARGV -lab     labelVar

if {[flagLoc $ARGV -seq]}   {set seqFlag 1}
if {[flagLoc $ARGV -noseq]} {set seqFlag 0}
 
if {[flagLoc $ARGV -grid]} \
   {
    set xGridArg -xGrid
    set yGridArg -yGrid
   }

if {[flagLoc $ARGV -nogrid]} \
   {
    set xGridArg -noxGrid
    set yGridArg -noyGrid
   }

#
#------------------------------------------------------------------------

proc initScrollParams {} \
{
   global inName hnTab xName yName zName xVarName yVarName hi noise mult
   global cellInfo dragX1 dragXN dragY1 dragYN dragLoc 

   set fdata 0

   set cellInfo ""
   set dragLoc  -1
   set dragX1   0
   set dragXN   0
   set dragY1   0
   set dragYN   0

   set error [rdFDATA $inName fdata]

   if {![strcasecmp $xName Auto]} \
      {
       set xName [getDirectDim $fdata]
       puts "Auto Mode Direct Dimension:   $xName"
      }

   if {![strcasecmp $yName Auto]} \
      {
       set yName [get15NDim $fdata]
       puts "Auto Mode 15N Dimension:      $yName"
      }

   if {![strcasecmp $zName Auto]} \
      {
       set zName [getIndirectDim $fdata]
       puts "Auto Mode Strip Dimension:    $zName"
      }

   if {![strcasecmp $xVarName Auto]} \
      {
       set xVarName [getHNTabVar $hnTab]
       puts "Auto Mode HN Table Variable:  $xVarName"
      }

   if {![strcasecmp $yVarName Auto]} \
      {
       set yVarName [get15NTabVar $hnTab]
       puts "Auto Mode 15N Table Variable: $yVarName"
      }

   if {![strcasecmp $hi Auto]} \
      {
       set noise [estSpecNoise $inName]
       set hi    [expr $noise*$mult]

       if {$hi == 0.0} {set hi 1.0e+6}

       puts "Auto Mode Contour Height:     [format %.2e $hi]"
      }
}

proc initScrollWin {} \
{
   global dx qx xSize ySize hnTab dragY1 dragYN

   set count [gdbGetInfo $hnTab -parms entryCount]

   set sWide [winfo screenwidth  .]
   set sHigh [winfo screenheight .]
   set sHigh [expr $sHigh/2]

   set wWide [expr $sWide - 8]
   set wHigh [expr $sHigh - 25]

   set dx 40
   set dy 80
   set qy 20
   set qx 10

   wm geom  . ${wWide}x${wHigh}+0-$qy
   
   set ySize [expr $sHigh - $dy] 
   set xSize [expr (2+$count)*($dx + $qx) + 2*$qx]

   set dragY1 $qy
   set dragYN [expr $ySize - $qy]

   scrollbar .s -orient horiz -command ".c xview"

   canvas .c -width $xSize -height $ySize -border 0 \
          -xscrollcommand ".s set" -scrollregion "0 0 $xSize $ySize"
   
   button .b -text Quit -command exit
   pack   .c .s .b -side top -expand 1 -fill x

   .c create pixmap 0 0 -width $xSize -height $ySize -win 1

   update

   drawRec -x 0 $xSize -y 0 $ySize -fg gray20 

   bind .c  <ButtonPress-1>    "dragON     %x %y B1"
   bind .c  <B1-Motion>        "dragTrack  %x %y B1"
   bind .c  <ButtonRelease-1>  "dragOFF    %x %y B1"
}

proc dragON { sxPix syPix b } \
{
   global cellInfo eList dragX1 dragXN dragY1 dragYN dragLoc

   set dragLoc -1

   if {![llength $cellInfo]} return

   set xPix [.c canvasx $sxPix]
   set yPix [.c canvasy $syPix]

   set loc  [findCellByXPix $xPix]
   if {$loc < 0} return

   set c [lindex $cellInfo $loc]

   set dragX1 [expr [lindex $c 1] + 1]
   set dragXN [expr [lindex $c 2] - 1]

   .c create rectangle $dragX1 $dragY1 $dragXN $dragYN \
                       -outline green -tags dragBox

   set dragLoc $loc

   update
}

proc dragTrack { sxPix syPix b } \
{
   global dragX1 dragXN dragY1 dragYN dx

   set xMid   [.c canvasx $sxPix]
   set dragX1 [expr $xMid + 1 - $dx/2]
   set dragXN [expr $xMid + $dx/2 - 1]
                                                                                
   .c coords dragBox $dragX1 $dragY1 $dragXN $dragYN
                                                                                
   update
}

proc dragOFF {sxPix syPix b} \
{
   global cellInfo eList dragX1 dragXN dragY1 dragYN dx dragLoc

   set xMid   [.c canvasx $sxPix]
   set dragX1 [expr $xMid - $dx/2]
   set dragXN [expr $xMid + $dx/2]
                                                                                              
   .c delete dragBox 
   update
                                                                                              
   set loc  [findCellByXPix $xMid]
   if {$loc < 0} return

puts "swap $loc $dragLoc"

   set cI [lindex $cellInfo $dragLoc]
   set cJ [lindex $cellInfo $loc]

   lset cellInfo $dragLoc $cJ
   lset cellInfo $loc     $cI

   set eList ""

   foreach c $cellInfo {lappend eList [lindex $c 0]}

   drawScrollCells
}

proc findCellByXPix { xPix } \
{
   global cellInfo

   set loc 0

   foreach c $cellInfo \
      {
       set x1 [lindex $c 1]
       set xn [lindex $c 2]

       if {$xPix >= $x1 && $xPix <= $xn} {return $loc}

       incr loc
      }

   return -1
}

proc initScrollCells {} \
{
   global dx qx stripCount hnTab seqFlag eList cellInfo

   set stripCount 0

   set x1 20
   set xn [expr $x1 + $dx]
   set i  0

   set cellInfo ""

   foreach entry $eList \
      {
       incr stripCount

       defineCell -x $x1 $xn -y 5% 95% -cell $stripCount -win 1

       lappend cellInfo "$entry $x1 $xn $stripCount 0 0"

       if {!($stripCount % 10)} update

       if {$seqFlag} \
          {
           defineCell -x $x1 $xn -y 95% 100% -cell [expr $stripCount + 1000]
          }

       set x1 [expr $xn + $qx]
       set xn [expr $x1 + $dx]
      }
}

proc drawScrollCells {} \
{
   global inName xVarName yVarName labelVar xName yName zName stripCount
   global xGridArg yGridArg nlev plev hi roiInfo seqFlag eList


   set stripCount 0

   foreach entry $eList \
      {
       incr stripCount

       set xVal [gdbGet $entry $xVarName]
       set yVal [gdbGet $entry $yVarName]

       set xPPM  ${xVal}ppm
       set yPPM  ${yVal}ppm
       set str   [gdbGet $entry $labelVar]
       set id    [gdbGet $entry INDEX]
       set color [gdbGet $entry COLOR]
 
       drawRec -cell $stripCount -x 0% 100% -y -10% 110% -fg gray20

       drawText -cell $stripCount -x 10% -y -1% -fg $color -text $str
 
       if {$xVal < 999.0 && $yVal < 999.0} \
          {
           if {![info exists roiInfo($id)]} \
              {
               set roiInfo($id) $stripCount

               readROI -in $inName -roi $roiInfo($id) -w $xName -1 $yName -1 \
                       -ndim 2 -dx $xName $xPPM 0.12ppm -y $zName \
                       -dz $yName $yPPM 1 -center X
              }

           drawROI -roi $roiInfo($id) -cell $stripCount -hi $hi -plev $plev \
                   -nlev $nlev $xGridArg $yGridArg -fg $color -bg black
          }

       if {$seqFlag} \
          {
           set ass  [gdbGet $entry ASS]
           set xOff [expr 75 - 25*[string length $ass]]%

           drawText -cell $stripCount -x $xOff -y 105% -fg $color -text $ass
          }

       if {!($stripCount % 10)} update
      }

   update
}

proc readScrollTab { thisName } \
{
   global hnTab eList prevName

   catch "file delete $prevName"

   gdbFree $hnTab
 
   set hnTab [gdbRead table -in $thisName]
   set eList [gdbGetInfo $hnTab -parms entries]

   set prevName $thisName
}

proc updateScrollCells { thisName } \
{
   readScrollTab $thisName
   drawScrollCells
}
 
readScrollTab $tabName

initScrollParams
initScrollWin
initScrollCells
drawScrollCells

wm title . "Done: $stripCount strips. Use Left Mouse Button to Drag Strips"

raise .

update
