#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

#
# To do:
#  adjust exec process loop, 2D-ND cases
#  check for send timeout 

proc initSelVars {} \
{
   global selInfo fontInfo glbBinType

   setBinType

   set fontDY [lindex $fontInfo end]

   set selInfo(scanInfo)     ""
   set selInfo(eNameList)    ""
   set selInfo(idList)       ""
   set selInfo(idCount)      0
   set selInfo(selCount)     0

   set selInfo(fileList)     ""
   set selInfo(sortMode)     None

   set selInfo(mode,Bruker)  1
   set selInfo(mode,Varian)  1
   set selInfo(mode,JEOL)    1
   set selInfo(mode,Delta)   1
   set selInfo(mode,nmr)     1

   set selInfo(mode,2D)      0
   set selInfo(mode,3D)      0
   set selInfo(mode,4D)      0
   set selInfo(mode,Old)     0
   set selInfo(mode,New)     0

   if {![strcmp $glbBinType mac]} \
      {
       set selInfo(c,height)   281
       set selInfo(c,width)    825 
       set selInfo(lb,height)    4
       set selInfo(lb,width)    82 
       set selInfo(char,width)  10 
       set selInfo(char,height) 16
      } \
   elseif {$fontDY == 12} \
      {
       set selInfo(c,height)   281
       set selInfo(c,width)    825
       set selInfo(lb,height)    4
       set selInfo(lb,width)   118 
       set selInfo(char,width)  10 
       set selInfo(char,height) 16
      } \
   else \
      {
       set selInfo(c,height)   281
       set selInfo(c,width)    823
       set selInfo(lb,height)    4
       set selInfo(lb,width)    92
       set selInfo(char,width)  10
       set selInfo(char,height) 16
      }

   set selInfo(im,height)   32
   set selInfo(im,width)   128

   set selInfo(oldFiles,ND) "fid.com ft1.com ft2.com xz.com xyz.com"
   set selInfo(oldFiles,1D) "test.dat test.ft1"
   set selInfo(oldFiles,2D) "test.ft2"
   set selInfo(oldFiles,3D) "ft/test001.ft3"
   set selInfo(oldFiles,4D) "ft/test01001.ft4"

#  fid.com ft.com ft1.com ft2.com xz.com xyz.com

   set selInfo(cleanFiles)  \
    {
     ezProc.log pk.tcl psLoop.txt
     test.fid test.dat test.ft1 test.ft2 xz001.fid
     aux.tab test.tab test.ps aux.ft1 aux.ft2 aux.dat
     axt.tab axt.dat axt.ft1 axt.ft2 refPPM.tab pkROI_1d.dat
    }

   set selInfo(cleanDirs) {fid ft dat pca}

   set selInfo(procCmnd,1D)  "ezProc1D.tcl"
   set selInfo(procCmnd,2D)  None
   set selInfo(procCmnd,3D)  None
   set selInfo(procCmnd,4D)  None
   set selInfo(procCmnd,ND)  None
   set selInfo(procCmnd,com) None

   set selInfo(procCmnd,args,1D)  ""
   set selInfo(procCmnd,args,2D)  ""
   set selInfo(procCmnd,args,3D)  ""
   set selInfo(procCmnd,args,4D)  ""
   set selInfo(procCmnd,args,ND)  ""
   set selInfo(procCmnd,args,com) ""

   set selInfo(dirName)     [file normalize .]
   set selInfo(listName)    "in.list"
   set selInfo(logName)     "selNMR.log"
   set selInfo(autoFlag)    0
   set selInfo(fastFlag)    0
   set selInfo(exitFlag)    0
}

proc initSelArgs {} \
{
 global selInfo ARGC ARGV

if {[flagLoc $ARGV -help]} \
{
puts "Scan for NMR Data Directories, Send Info to ezProc1D.tcl or Other Script:"
puts " -dir        dirList        \[.\]                 List of Directories to Scan."
puts " -old1D      oldFileList    \[test.dat test.ft1\] Files that indicate already-processed ('old') data." 
puts " -proc1D     proc1DCmndList \[ezProc1D.tcl]"
puts "Execution Modes:"
puts " -fast   Scan for Data without Determining Dimension Count etc."
puts " -auto   Scan for Data Automatically on start-up, Process All."
puts " -exit   Exit After Processing."
puts "Parameters for Clean Option:"
puts " -cleanFiles cleanFileList  \[test.fid test.dat ...\]"
puts " -cleanDirs  cleanDirList   \[fid ft dat pca\]        Subdirectories to Delete."
puts "Arguments for Processing Module ezProc1D.tcl;"
puts "Use -args1DEnd to terminate if needed:"
puts " -args1D  proc1DArgs ... -args1DEnd"
puts "Alternative Processing via Script:"
puts " -com     comName \[None\]        Script Name."
puts " -log     logName \[selNMR.log\]  Log File for Script."
puts " -argscom comArgs ... -argscomEnd Optional Arguments for Script."
puts "Notes:"
puts " 1. By default, processing is performed by ezProc1D.tcl;"
puts "    Use exProc1D.tcl -help for more info."
puts " 2. If the -com argument is specified, processing will be"
puts "    performed by the given script, that will be invoked once"
puts "    for every NMR data directory selected."
puts " 3. If used, the -com processing script will be invoked"
puts "    with the NMR data directory as the first argument, followed"
puts "    by any arguments specified via the -argscom option."
puts "Processing Module Examples:"
puts " selNMR.tcl -args1D -zf 2"
puts " selNMR.tcl -args1D -zf SIZE=8000 -args1DEnd -auto"
puts " selNMR.tcl -args1D -fnF SIGN -left \\'\\|\\' nmrPipe -fn SMO"
exit 0
}

   if {[flagLoc $ARGV -fast]}   {set selInfo(fastFlag) 1}
   if {[flagLoc $ARGV -nofast]} {set selInfo(fastFlag) 0}

   if {[flagLoc $ARGV -auto]}   {set selInfo(autoFlag) 1}
   if {[flagLoc $ARGV -noauto]} {set selInfo(autoFlag) 0}

   if {[flagLoc $ARGV -exit]}   {set selInfo(exitFlag) 1}
   if {[flagLoc $ARGV -noexit]} {set selInfo(exitFlag) 0}

   getListArgD $ARGV -cleanFiles  selInfo(cleanFiles)
   getListArgD $ARGV -cleanDirs   selInfo(cleanDirs)
   getListArgD $ARGV -dir         selInfo(dirName)
   getListArgD $ARGV -log         selInfo(logName)

   getListArgD $ARGV -com         selInfo(procCmnd,com)

   if {[strcasecmp $selInfo(procCmnd,com) None]} \
      {
       foreach i {ND 1D 2D 3D 4D} {set selInfo(procCmnd,$i) None}
      }

   foreach i {ND 1D 2D 3D 4D com} \
      {
       getListArgD $ARGV -old$i  selInfo(oldFiles,$i)
       getArgD $ARGV -proc$i selInfo(procCmnd,$i)

       set str [string trim $selInfo(procCmnd,$i) "{}"]
       set $selInfo(procCmnd,$i) $str

       set loc1 [flagLoc $ARGV -args$i]
       set locN [flagLoc $ARGV -args${i}End]

       if {$loc1 > 0} \
          {
           incr loc1

           if {$locN > $loc1} \
              {
               incr locN -1
              } \
           else \
              {
               set locN end
              }
 
           set selInfo(procCmnd,args,$i) [lrange $ARGV $loc1 $locN]
          }
      }

   return

   if {![flagLoc $ARGV -exp]} return

   set selInfo(expLabels)   ""
   set selInfo(expPatterns) ""
  
   for {set loc 1} {$loc < $ARGC} {incr loc} \
      {
       if {[strcasecmp [lindex $ARGV $loc] "-exp"]} continue 

       set nextLoc [nextFlag $ARGV $loc]

       if {$nextLoc != $loc + 3} \
          {
           puts stderr "Error with Experiment Argument Item $loc."
           continue
          }

       lappend selInfo(expLabels)   [lindex $ARGV [expr $loc + 1]]
       lappend selInfo(expPatterns) [lindex $ARGV [expr $loc + 2]]
      }
}

proc initSelWin {} \
{
   global selInfo env fixedfont


   setFonts
   initSelIM

   frame .path 
   frame .tool 
   frame .list

   frame .left 
   frame .left1 -relief groove -border 3
   frame .left2 -relief groove -border 3
   frame .right 
   frame .top
   frame .bottom

   wm geom  . +120+64

   label .dirLab -text "Path:"

   entry .dirEntry -width 50 -relief sunken -textvariable selInfo(dirName) 

   button .dirButton -relief raised -fg green \
    -bitmap @$env(NMRTXT)/caret.map -command setSelPath

   button .scanButton -text Scan -command scanSel 

   checkbutton .autoButton -text "Auto Mode" \
     -variable selInfo(autoFlag) -selectcolor blue -command toggleAutoMode

   pack .autoButton -in .path -side left -padx 20 -pady 2
   pack .dirLab .dirEntry .dirButton -in .path -side left -pady 2
   pack .scanButton -in .path -side left -padx 20 -pady 2

   set bInfo {Bruker Varian JEOL 1D 2D 3D "Old Only" "New Only"} 

   foreach j $bInfo \
      {
       set i [lindex $j 0]

       if {![info exists selInfo(mode,$i)]} {set selInfo(mode,$i) 1}

       checkbutton .sel$i -selectcolor blue -anchor w -width 13 \
        -text $j -variable selInfo(mode,$i) -command "toggleSelMode $i" 

       pack .sel$i -in .left1 -side top -expand 1 -fill x
      }

   updateExpNameChoices

   pack .left1 .left2 -in .left -side top -pady 1

   button .selAdd  -padx 4 \
    -text "Add Selection to List"      -command "selCmnd Add"

   button .selRem  -padx 4 \
    -text "Remove Selection from List" -command "selCmnd Remove"

   button .selAll  -padx 4 \
    -text "Select All"                 -command "selCmnd All"

   button .selNone -padx 4 \
    -text "Select None"                -command "selCmnd None"

   button .selInv  -padx 4 \
    -text "Invert"                     -command "selCmnd Invert"

   button .selClr  -padx 4 \
    -text "Clear List"                 -command "selCmnd Clear"

   button .selProc -padx 4 \
    -text "Process List"               -command "selCmnd Process"

   button .selCln  -padx 4 \
    -text "Clean List"                 -command "selCmnd Clean"

   button .selSave -padx 4 \
    -text "Save List"                  -command "selCmnd Save"

   button .selQuit -padx 4 \
    -text "Quit"                       -command "selCmnd Quit"

   pack .selAdd  -in .bottom -side left -expand 1 -fill x
   pack .selRem  -in .bottom -side left -expand 1 -fill x
   pack .selAll  -in .bottom -side left -expand 1 -fill x
   pack .selNone -in .bottom -side left -expand 1 -fill x
   pack .selInv  -in .bottom -side left -expand 1 -fill x
   pack .selClr  -in .bottom -side left -expand 1 -fill x
   pack .selProc -in .bottom -side left -expand 1 -fill x
   pack .selCln  -in .bottom -side left -expand 1 -fill x
   pack .selSave -in .bottom -side left -expand 1 -fill x
   pack .selQuit -in .bottom -side left -expand 1 -fill x

   scrollbar .sc -orient vert -command ".c yview"

   canvas .hdr -width $selInfo(c,width) -relief flat -border 0 \
          -height [expr $selInfo(char,height) + 4] -highlightthickness 0

   canvas .c -width $selInfo(c,width) -height $selInfo(c,height) \
             -scrollregion "0 0 $selInfo(c,width) $selInfo(c,height)" \
             -yscrollcommand ".sc set" -relief sunken -border 3

   scrollbar .sl -orient vert -command ".lb yview"

   listbox .lb -font $fixedfont -selectmode single \
           -width $selInfo(lb,width) -height $selInfo(lb,height) \
           -yscrollcommand ".sl set"

   pack .sl -in .list -side right -expand 1 -fill y 
   pack .lb -in .list -side left  -expand 1 -fill both

#  pack .path -in .      -side top 
   pack .path -in .right -side top 

   pack .tool -in . -side top -expand 1 -fill x

   pack .sc -in .top -side right -expand 1 -fill y
   pack .c  -in .top -side left  -expand 1 -fill both

   pack .hdr    -in .right -side top  -expand 1 -fill x 
   pack .top    -in .right -side top  -expand 1 -fill both -pady 8

#  pack .list   -in .      -side top  -expand 1 -fill both -pady 4 
   pack .list   -in .right -side top  -expand 1 -fill both -pady 4 

   pack .bottom -in .      -side top  -expand 1 -fill x

   pack .left   -in .tool -side left -expand 1 -fill x -padx 4
   pack .right  -in .tool -side left -expand 1 -fill x

   update
}

proc updateExpNameChoices {} \
{
   global selInfo

   set n [llength $selInfo(eNameList)]

   for {set i 0} {$i < 12} {incr i} \
      {
       if {$i >= $n} \
          {
           set j "Experiment [expr $i + 1]"
           set selInfo(mode,exp$i) 0
          } \
       else \
          {
           set j [lindex $selInfo(eNameList) $i]
           set selInfo(mode,exp$i) 1
          }

       if {![winfo exists .exp$i]} \
          {
           checkbutton .exp$i -selectcolor purple -anchor w -width 13 \
            -variable selInfo(mode,exp$i) -command "toggleExpMode $i"

           pack .exp$i -in .left2 -side top -expand 1 -fill x
          }

       .exp$i configure -text $j
      }

   update
}

proc hiliteLineItem { id msg strOrig mode } \
{
   global selInfo

   set str [string trim $strOrig "{}"]
   set s   ""

   if {![strcasecmp $mode ON]} \
      {
       set s "$msg: $str"

       if {$selInfo($id,selected)} \
          {
           append s " (Left-Click to De-Select this entry)"
          } \
       else \
          {
           append s " (Left-Click to Select this entry)"
          }

       wm title . $s 
       update
      } \
   else \
      {
       showDefTitle
      }
}

proc showDefTitle {} \
{
   global selInfo

   set s "selNMR 2022.151.13.01: Use 'Scan' to Search for NMR Data."

   if {$selInfo(idCount)} \
      {
       set n [expr $selInfo(idCount) - $selInfo(selCount)]

       append s " Selected (White): $selInfo(selCount)"
       append s " Not Selected (Gray): $n"
       append s " Total: $selInfo(idCount)."
      }

   wm title . $s 
   update
}

proc setSelPath {} \
{
   global selInfo

   set newDir \
    [tk_chooseDirectory -mustexist true -title "Directory to Scan" \
                        -initialdir $selInfo(dirName)]

   if {![string length $newDir]}    return
   if {![file isdirectory $newDir]} return

   set selInfo(dirName) $newDir
}

proc initSelIM {} \
{
   global env selInfo

   foreach type {bruker varian jeol delta nmr} \
      {
       set selInfo(im,$type)    None
       set selInfo(imOff,$type) None

       set imType $type

       if {![strcasecmp $type varian]} {set imType agilent}

       set imFile $env(NMRTXT)/${imType}Logo.gif

       if {[file exists $imFile]} \
          {
           image create photo ${type}LogoOn -file $imFile -format gif
           set selInfo(im,height)  [image height ${type}LogoOn]
           set selInfo(im,width)   [image width  ${type}LogoOn]
           set selInfo(im,$type)   ${type}LogoOn
          }

       set imFile $env(NMRTXT)/${imType}LogoOff.gif

       if {[file exists $imFile]} \
          {
           image create photo ${type}LogoOff -file $imFile -format gif
           set selInfo(imOff,$type) ${type}LogoOff
          }
      }
}

proc addImageItem { msg type id } \
{
   global ix1 iy1 selInfo

   if {[info exists selInfo(im,$type)]} \
      {
       if {[strcasecmp $selInfo(im,$type) None]} \
          {
           set iID \
            [.c create image $ix1 $iy1 -image $selInfo(im,$type) -anchor nw \
                -tags "scanItem line$id img$id"]

           set selInfo($id,im) $selInfo(im,$type)

           .c bind $iID <Any-Enter>       "+ hiliteLineItem $id $msg $type ON"
           .c bind $iID <Any-Leave>       "+ hiliteLineItem $id $msg $type OFF"

           .c bind $iID <ButtonRelease-1> "+ toggleLineItem $id"
          }
      }

   if {[info exists selInfo(imOff,$type)]} \
      {
       if {[strcasecmp $selInfo(imOff,$type) None]} \
          {
           set selInfo($id,imOff) $selInfo(imOff,$type)
          }
      }
}

proc addTextItem { msg str n id } \
{
   global ix1 ixn iy1 iyn selInfo stdfont boldfont

   set ixn [expr $ix1 + $n*$selInfo(char,width)]

   set rID \
    [.c create rectangle $ix1 $iy1 $ixn $iyn -fill white -outline white \
        -tags "scanItem line$id rec$id"]
 
   set tID \
    [.c create text [expr $ix1 + 3] [expr $iy1 + 3] -fill black -anchor nw \
        -font $stdfont -text $str -tags "scanItem line$id text$id"]

   scan $str %s strFirst
 
   .c bind $tID <Any-Enter>       "+ hiliteLineItem $id $msg $strFirst ON"
   .c bind $rID <Any-Enter>       "+ hiliteLineItem $id $msg $strFirst ON"
   .c bind $rID <Any-Leave>       "+ hiliteLineItem $id $msg $strFirst OFF"

   .c bind $tID <ButtonRelease-1> "+ toggleLineItem $id"
   .c bind $rID <ButtonRelease-1> "+ toggleLineItem $id"

   set ix1 [expr $ixn + 4]
}

proc addHdrItem { str mode n } \
{
   global ix1 ixn iy1 iyn selInfo boldfont

   if {![info exists selInfo(sort,$mode)]} {set selInfo(sort,$mode) 0}

   set ixn [expr $ix1 + $n*$selInfo(char,width)]

   set tID \
    [.hdr create text [expr $ix1 + 3] [expr $iy1 + 3] -fill black -anchor nw \
          -font $boldfont -text $str -tags "hdr $mode"]

   .hdr bind $tID <Any-Enter>       "+ toggleHdr $mode blue"
   .hdr bind $tID <Any-Leave>       "+ toggleHdr $mode black"
   .hdr bind $tID <ButtonRelease-1> "+ sortLineItem $mode"

   set ix1 [expr $ixn + 4]
}

proc toggleHdr { mode color } \
{
   .hdr itemconfigure $mode -fill $color
   update
}

proc toggleLineItem { id } \
{
   global selInfo

   if {$selInfo($id,selected)} \
      {
       set selInfo($id,selected) 0
      } \
   else \
      {
       set selInfo($id,selected) 1
      }

   updateLineItem $id
   tallySelItems
}

proc updateLineItem { id } \
{
   global selInfo

   if {$selInfo($id,selected)} \
      {
       .c itemconfigure rec$id  -fill white -outline white
       .c itemconfigure text$id -fill black

       if {[strcasecmp $selInfo($id,im) None]} \
          {
           .c itemconfigure img$id -image $selInfo($id,im)
          }
      } \
   else \
      {
       .c itemconfigure rec$id  -fill gray60 -outline gray60
       .c itemconfigure text$id -fill gray50

       if {[strcasecmp $selInfo($id,imOff) None]} \
          {
           .c itemconfigure img$id -image $selInfo($id,imOff)
          }
      }

   update
}

proc liSortProc { idI idJ } \
{
   global selInfo

   set mode $selInfo(sortMode)

   if {![strcasecmp $mode None]} {return 0}

   set vI $selInfo($idI,$mode) 
   set vJ $selInfo($idJ,$mode) 

   set i  [string compare $vI $vJ]

   if {$selInfo(sort,$mode)} {set i [expr -($i)]}

   return $i
}
 
proc sortLineItem { mode } \
{
   global selInfo

   set selInfo(sortMode) $mode 

   set newList [lsort -command liSortProc $selInfo(idList)]

   set selInfo(idList) $newList

   createLineItems   

   if {$selInfo(sort,$mode)} \
      {
       set selInfo(sort,$mode) 0
      } \
   else \
      {
       set selInfo(sort,$mode) 1
      }
}

proc scanSel {} \
{
   global selInfo
 
   clearLineItems

   set selInfo(scanInfo)  ""
   set selInfo(idList)    ""
   set selInfo(eNameList) ""
   set selInfo(idCount)   0
   set selInfo(selCount)  0

   if {$selInfo(fastFlag)} \
      {
       wm  title . "selNMR Scanning in Fast Mode, please wait ..."
       set fastArg "-fast"
      } \
   else \
      {
       wm  title . "selNMR Scanning, please wait ..."
       set fastArg "-nofast"
      }

   busy hold  .
   update

   set selInfo(scanInfo) [exec scanNMR.tcl -dir $selInfo(dirName) $fastArg]

   wm   title . "selNMR Done scanning."
   busy release  .
   update

   set n  [llength $selInfo(scanInfo)]   
   set dy [expr $selInfo(im,height) + 3]
   set hi [expr $n*$dy]

   if {$hi < $selInfo(c,height)} {set hi $selInfo(c,height)}

   .c configure -scrollregion "0 0 $selInfo(c,width) $hi"

   extractLineItems
   testLineItems
   createLineItems

   update

   if {$selInfo(autoFlag)} \
      {
       selCmnd Add
       update

       selCmnd Process
       update

       if {$selInfo(exitFlag)} exitSel 
      }
}

proc clearLineItems {} \
{
   global selInfo

   catch {.hdr delete hdr}
   catch {.c   delete scanItem}

   update
}

proc extractLineItems {} \
{
   global selInfo

   set id 0

   foreach s $selInfo(scanInfo) \
      {
       incr id

       lappend selInfo(idList) $id

       set selInfo($id,im)       None
       set selInfo($id,imOff)    None
       set selInfo($id,str)      $s
       set selInfo($id,id)       [lindex $s 0]
       set selInfo($id,path)     [string trim [lindex $s 1] \"\{\}]
       set selInfo($id,dir)      [string trim [lindex $s 2] \"\{\}]
       set selInfo($id,type)     [lindex $s 3]
       set selInfo($id,ndim)     [lindex $s 4]
       set selInfo($id,nx)       [lindex $s 5]
       set selInfo($id,ny)       [lindex $s 6]
       set selInfo($id,nz)       [lindex $s 7]
       set selInfo($id,na)       [lindex $s 8]
       set selInfo($id,xLab)     [lindex $s 9]
       set selInfo($id,yLab)     [lindex $s 10]
       set selInfo($id,zLab)     [lindex $s 11]
       set selInfo($id,aLab)     [lindex $s 12]
       set selInfo($id,xSW)      [safeFltFormat %.0f [lindex $s 13]]
       set selInfo($id,ySW)      [safeFltFormat %.0f [lindex $s 14]]
       set selInfo($id,zSW)      [safeFltFormat %.0f [lindex $s 15]]
       set selInfo($id,aSW)      [safeFltFormat %.0f [lindex $s 16]]
       set selInfo($id,eName)    [lindex $s 17]
       set selInfo($id,status)   [lindex $s 18]
       set selInfo($id,selected) 1

       set selInfo($id,eText)    "$selInfo($id,eName)\n$selInfo($id,xLab) $selInfo($id,xSW) Hz"

       if {$selInfo($id,ndim) >= 2} {append selInfo($id,eText) ", $selInfo($id,yLab) $selInfo($id,ySW) Hz"}
       if {$selInfo($id,ndim) >= 3} {append selInfo($id,eText) ", $selInfo($id,zLab) $selInfo($id,zSW) Hz"}
       if {$selInfo($id,ndim) >= 4} {append selInfo($id,eText) ", $selInfo($id,aLab) $selInfo($id,aSW) Hz"}

       if {![strmatch [string toupper $selInfo($id,xLab)] *H*]} \
          {
           append selInfo($id,eName) "-$selInfo($id,xLab)"
          }

       set ndStr   $selInfo($id,ndim)D

       set sizeStr $selInfo($id,nx)

       if {$selInfo($id,ndim) > 1} {append sizeStr "x$selInfo($id,ny)"}
       if {$selInfo($id,ndim) > 2} {append sizeStr "x$selInfo($id,nz)"}
       if {$selInfo($id,ndim) > 3} {append sizeStr "x$selInfo($id,na)"}

       set selInfo($id,size) $sizeStr

       set eName [string toupper $selInfo($id,eName)]
       set eName $selInfo($id,eName)

       if {[lsearch -exact $selInfo(eNameList) $eName] < 0} \
          {
           lappend selInfo(eNameList) $eName
          }
      }

   set selInfo(eNameList) [lsort $selInfo(eNameList)]

   set selInfo(selCount) $id
   set selInfo(idCount)  $id

   updateExpNameChoices
}

proc createLineItems {} \
{
   global selInfo ix1 ixn iy1 iyn

   clearLineItems

   set ix1 4
   set iy1 4

   addHdrItem ND         ndim    2
   addHdrItem Status     status  5
   addHdrItem Size       size    9 
   addHdrItem Experiment eName  16
   addHdrItem Directory  dir    35
   addHdrItem Type       type   12

   set iy1 1

   foreach id $selInfo(idList) \
      {
       set ix1 4
       set iyn [expr $iy1 + $selInfo(im,height)]

       addTextItem  Dimensions $selInfo($id,ndim)D   2 $id
       addTextItem  Status     $selInfo($id,status)  5 $id
       addTextItem  Size       $selInfo($id,size)    9 $id
       addTextItem  Experiment $selInfo($id,eText)  16 $id
       addTextItem  Directory  $selInfo($id,dir)    35 $id

       addImageItem Type       $selInfo($id,type)      $id

       set iy1 [expr $iyn + 3]

       updateLineItem $id
      } 

   update
}

proc refreshLineItems {} \
{
   global selInfo

   tallySelItems

   foreach id $selInfo(idList) \
      {
       updateLineItem $id
      }
}

proc testLineItems {} \
{
   global selInfo

   foreach id $selInfo(idList) \
      {
       testLineItem $id
      }

   tallySelItems
}

proc tallySelItems {} \
{
   global selInfo

   set selInfo(selCount) 0

   foreach id $selInfo(idList) \
      {
       if {$selInfo($id,selected)} {incr selInfo(selCount)}
      } 

   showDefTitle
}

proc selCmnd { cmnd } \
{
   switch [string tolower $cmnd] \
      {
       all     {resetSel All} 
       none    {resetSel None}
       invert  {resetSel Invert}
       add     {addToList}
       remove  {removeFromList} 
       clean   {cleanList} 
       clear   {clearList} 
       save    {saveList} 
       process {procList} 
       quit    {exitSel}
      }
}

proc exitSel {} \
{
   global selInfo

   foreach i {1D 2D 3D 4D ND} \
      {
       if {![strcasecmp $selInfo(procCmnd,$i) None]} continue
       set s [lindex $selInfo(procCmnd,$i) 0]
       catch "send -async $s ezProc${i}Cmnd exit"
      }

   exit
}

proc saveList {} \
{
   global selInfo

   if {![llength $selInfo(fileList)]} return
 
   set types \
      {
       { {All Files}  *      }
       { {Text}       {.txt} }
       { {tab}        {.tab} }
       { {List}       {.list} }
      }

   set outName \
    [tk_getSaveFile -title "File List Output" -filetypes $types \
                    -initialfile $selInfo(listName)]

   if {![string length $outName]} return

   set error [catch {set outUnit [open $outName w]}]

   if {$error} \
      {
       showError selNMR "Error opening file list output $outName"
      }

   foreach f $selInfo(fileList) {puts $outUnit $f}

   set selInfo(listName) $outName

   close $outUnit
}

proc cleanList {} \
{
   global selInfo

   if {![llength $selInfo(fileList)]} return

   set msg "Clean: Delete processed data and scripts from directories in list?"

   if {![tk_dialog .selCleanDiag Clean $msg info 0 Cancel Clean]} return 

   foreach f $selInfo(fileList) \
      {
       foreach ff $selInfo(cleanFiles) \
          {
           if {[file exists $f/$ff]} \
              {
               puts $f/$ff
               catch {file delete $f/$ff}
              }
          }

       foreach ff $selInfo(cleanDirs) \
          {
           if {[file isdirectory $f/$ff]} \
              {        
               puts $f/$ff
               catch {file delete $f/$ff}
              }
          }

      }

   scanSel
}

proc addToList {} \
{
   global selInfo

   foreach id $selInfo(idList) \
      {
       if {!$selInfo($id,selected)} continue

       set thisName [file normalize $selInfo($id,path)/$selInfo($id,dir)]

       if {[lsearch -exact $selInfo(fileList) $thisName] < 0} \
          {
           lappend selInfo(fileList) $thisName
          }
      }

   .lb delete 0 end

   foreach thisName $selInfo(fileList) \
      {
       .lb insert end $thisName
      }

   .lb see end

   update
}

proc removeFromList {} \
{
   global selInfo

   set thisList "" 
   set newList  "" 

   foreach id $selInfo(idList) \
      {
       if {!$selInfo($id,selected)} continue
       lappend thisList [file normalize $selInfo($id,path)/$selInfo($id,dir)]
      }

   foreach thisName $selInfo(fileList) \
      {
       if {[lsearch -exact $thisList $thisName] >= 0} continue
       lappend newList $thisName
      }

   .lb delete 0 end

   set selInfo(fileList) $newList

   foreach thisName $selInfo(fileList) \
      {
       .lb insert end $thisName
      }

   .lb see end

   update
}

proc clearList {} \
{
   global selInfo

   .lb delete 0 end
   set selInfo(fileList) "" 
   update
}

proc procList {} \
{
   global selInfo

   if {![llength $selInfo(fileList)]} {return 0}

   if {[strcasecmp $selInfo(procCmnd,com) None]} \
      {
       set n [llength $selInfo(fileList)]
       set i 0

       foreach thisName $selInfo(fileList) \
          {
           incr i
           wm title . "$i of $n: $thisName"
           update

           catch "exec selNMRExec.com $selInfo(procCmnd,com) $thisName $selInfo(logName) $selInfo(procCmnd,args,com)"
          }

       showDefTitle

       return 0
      }

   if {[strcasecmp $selInfo(procCmnd,1D) None]} \
      {
       send ezProc1D.tcl ezProc1DCmnd reset

       foreach thisName $selInfo(fileList) \
          {
           send ezProc1D.tcl ezProc1DCmnd \"append $thisName\"
           puts $thisName
          }

       send ezProc1D.tcl ezProc1DCmnd process
       send ezProc1D.tcl ezProc1DCmnd reset
      }
 
   return 0
}

proc resetSel { mode } \
{
   global selInfo

   switch $mode \
      {
       All {
            foreach id $selInfo(idList) \
               {
                set selInfo($id,selected) 1
                updateLineItem $id
               }
           }

       None {
             foreach id $selInfo(idList) \
                {
                 set selInfo($id,selected) 0
                 updateLineItem $id
                }
           }

       Invert {
               foreach id $selInfo(idList) \
                  {
                   if {$selInfo($id,selected)} \
                      {
                       set selInfo($id,selected) 0
                      } \
                   else \
                      {
                       set selInfo($id,selected) 1
                      }

                   updateLineItem $id
                  }
              }
      }

   refreshLineItems
}

proc toggleExpMode { loc } \
{
   global selInfo

   testLineItems
   tallySelItems
   refreshLineItems
}

proc toggleAutoMode {} \
{
}

proc toggleSelMode { mode } \
{
   global selInfo

   switch $mode \
      {
       Bruker { }
       Varian { }
       JEOL   { }
       1D     { }
       2D     { }
       3D     { }
       4D     { }

       New  {set selInfo(mode,Old) 0}
       Old  {set selInfo(mode,New) 0}
      }  

   testLineItems
   tallySelItems
   refreshLineItems
}

proc testLineItem { id } \
{
   global selInfo

   if {![strcasecmp $selInfo($id,type) Bruker]} \
      {
       if {$selInfo(mode,Bruker)} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }
      }

   if {![strcasecmp $selInfo($id,type) Varian]} \
      {
       if {$selInfo(mode,Varian)} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }  
      }

   if {![strcasecmp $selInfo($id,type) JEOL] \
         || ![strcasecmp $selInfo($id,type) delta]} \
      {
       if {$selInfo(mode,JEOL)} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }  
      }

   set j $selInfo($id,ndim)D

   if {[info exists selInfo(mode,$j)]} \
      { 
       if {$selInfo(mode,$j)} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }
      } \
   else \
      {
       set selInfo($id,selected) 0
       return
      }

   if {$selInfo(mode,Old)} \
      {
       if {![strcasecmp $selInfo($id,status) Old]} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }
      }

   if {$selInfo(mode,New)} \
      {
       if {![strcasecmp $selInfo($id,status) New]} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
           return
          }
      }

   set j [string toupper $selInfo($id,eName)]
   set j $selInfo($id,eName)

   set i [lsearch -exact $selInfo(eNameList) $j]

   if {$i >= 0} \
      {
       if {$selInfo(mode,exp$i)} \
          {
           set selInfo($id,selected) 1
          } \
       else \
          {
           set selInfo($id,selected) 0
          }
      }
}

proc initProcCmnds {} \
{
   global selInfo 

   if {[strcasecmp $selInfo(procCmnd,com) None]} return 

   foreach i {1D 2D 3D 4D ND} \
      {
       if {![strcasecmp $selInfo(procCmnd,$i) None]} continue

       puts "$selInfo(procCmnd,$i) -noproc $selInfo(procCmnd,args,$i) &"

       catch [eval "exec $selInfo(procCmnd,$i) -noproc $selInfo(procCmnd,args,$i) &"]
      }
}

set fontInfo [testFonts -init]

initSelVars
initSelArgs
initSelWin

testFonts -done

initProcCmnds
showDefTitle

update

if {$selInfo(autoFlag)} \
   {
    catch "exec sleep 3"
    scanSel
   }
