#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract Peaks From a Peak Table According to Height:"
    puts " -in   inName  \[test.tab\] Input Table." 
    puts " -out  outName \[stdout\]   Output Table." 
    puts " -n    n       \[1\]        Number of Peaks to Select."
    puts " -abs                     Select According to Absolute Value of Height."
    puts " -max                     Select Largest Peaks (Default)."
    puts " -min                     Select Smallest Peaks."
    exit 0
   }

set inName test.tab
set outName stdout
set n       1

set maxFlag 1
set absFlag 0

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -n   n

if {[flagLoc $ARGV -max]}   {set maxFlag 1}
if {[flagLoc $ARGV -min]}   {set maxFlag 0}

if {[flagLoc $ARGV -abs]}   {set absFlag 1}
if {[flagLoc $ARGV -noabs]} {set absFlag 0}

#
#----------------------------------------------------------------------

proc pkSortProc { pk1 pk2 } \
{
   global maxFlag absFlag

   set v1 [gdbGet $pk1 HEIGHT]
   set v2 [gdbGet $pk2 HEIGHT]

   if {$absFlag} \
      {
       set d [expr abs($v1) - abs($v2)]
      } \
   else \
      {
       set d [expr $v1 - $v2]
      }

   if {$maxFlag} \
      {
       if {$d < 0} {return -1}
       if {$d > 0} {return  1}
      } \
   else \
      {
       if {$d < 0} {return  1}
       if {$d > 0} {return -1}
      }

   return 0
}

set outUnit stdout

if {![file exists $inName]} \
   {
    puts stderr "selectPk. Error Finding Input Table $inName"
    exit 1
   }

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "selectPk. Error Opening Output File $outName"
        exit 1
       }
   }

set t       [gdbRead table -in $inName]
set pkList  [gdbSelect -from $t -parms entries]

set newList [lsort -decreasing -command pkSortProc $pkList]
set delList [lrange $newList $n end]

gdbDestroy $delList

gdbWrite $t $outName

exit 0
