#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Extract Selected Data From Table:"
    puts " -in   inName  \[test.tab\] Input Table." 
    puts " -out  outName \[stdout\]   Output Table." 
    puts " -var  vList   \[*\]        Variables to Extract, Use * in Patterns."
    puts " -cond cond    \[1\]        Selection Condition."
    puts " -text                    Keep REMARK and DATA Information."
    puts " -verb                    Verbose Mode ON."
    puts "Table Options:"
    puts " -hdr                     Keep VARS and FORMAT Header Information."
    puts " -nohdr                   No VARS and FORMAT Header (Default)."
    puts "PDB Options:"
    puts " -pdb          Input/Output is PDB Data (Default for .pdb)."
    puts " -nopdb        Input is NMRPipe-Format Table."
    puts " -hetatm       Interpret HETATMs as ATOMs."
    puts " -renumber     Renumber ATOMID Before Writing Result."
    exit 1 
   }

set inName  test.tab
set inTitle test.tab

set outName stdout
set outUnit stdout

set vList   * 
set cond    1

set renumFlag 0
set verbose   0

set hdrMode nohdr
set pdbMode nopdb
set outMode nopdb
set hetMode nohetatm
set txtMode notext

getArgD $ARGV -in      inName
getArgD $ARGV -in      inTitle
getArgD $ARGV -inTitle inTitle
getArgD $ARGV -out     outName

getListArgD $ARGV -show vList
getListArgD $ARGV -var  vList
getListArgD $ARGV -cond cond

set origVList $vList

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -text]}     {set txtMode text}
if {[flagLoc $ARGV -notext]}   {set txtMode notext}

if {[flagLoc $ARGV -hdr]}      {set hdrMode hdr}
if {[flagLoc $ARGV -nohdr]}    {set hdrMode nohdr}

if {[flagLoc $ARGV -pdb]}      {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}    {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}   {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]} {set hetMode nohetatm}

if {[flagLoc $ARGV -renumber]}   {set renumFlag 1}
if {[flagLoc $ARGV -norenumber]} {set renumFlag 0}

if {[flagLoc $ARGV -verb]}       {set verbose 1}
if {[flagLoc $ARGV -noverb]}     {set verbose 0}

#
#----------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input Table $inName"
    exit 1
   }

if {![strcasecmp $outName None] || ![strcasecmp $outName stdout]} \
   {
    set outUnit stdout
   } \
elseif {![strcasecmp $outName stderr]} \
   {
    set outUnit stderr
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "Error Opening Output File."
        exit 1
       }
   }

set cond   [string trim $cond "{}"]
set tOrig  [gdbRead table -in $inName -$pdbMode -$hetMode]
set vOrig  [gdbGetInfo $tOrig -parms varNames]
set tNew   [gdbSelect -new -from $tOrig -cond $cond]

if {![strcmp $vList "*"]} {set vList $vOrig}

set eCount [gdbGetInfo $tOrig -parms entryCount]
set sCount [gdbGetInfo $tNew  -parms entryCount]

set keepList [listMatch $vOrig $vList]

foreach v $vOrig \
   {
    if {[lsearch -exact $keepList $v] < 0} \
       {
        set vPtr [gdbFindByName $tNew $v]
        gdbFree $vPtr
       }
   }

set vNew [gdbGetInfo $tNew -parms varNames]

if {$verbose} \
   {
    puts stderr ""
    puts stderr "Input:            $inTitle"
    puts stderr "Input Variables:  [llength $vOrig]"
    puts stderr "Input Entries:    $eCount"
    puts stderr "Condition:        $cond"
    puts stderr "Output:           $outName"
    puts stderr "Output Variables: [llength $vNew]"
    puts stderr "Output Entries:   $sCount"
   }

if {![strcasecmp $pdbMode pdb]} \
   {
    if {$renumFlag} \
       {
        set eList [gdbGetInfo $tNew -parms entries]
        set id    0

        foreach e $eList \
           {
            incr id
            gdbSet $e ATOMID $id
           }
       }
    }

if {![strcasecmp $pdbMode pdb] && ![strcasecmp $origVList "*"]} \
   {
    if {![strcasecmp [file extension $outName] .pdb]} {set outMode pdb} 

    if {![strcasecmp $outName stdout]} {set outMode pdb} 
    if {![strcasecmp $outName stderr]} {set outMode pdb} 
   }

gdbWrite $tNew $outName -$outMode -$hetMode -$txtMode -$hdrMode

exit 0
