#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Prepare Titles to Process Spectra for Pool Screening Compound Library:"
    puts " -rackID  rackID   \[1\]               Rack ID to Use."
    puts " -cIDTab  tabInfo  \[id/plate%d.tab\]  File Name of Compound ID Table." 
    puts " -rackDir rackInfo \[Rack_%d\]         Corresponding Rack Directory."
    puts " -jobID   jobID    \[sysTime %Y%m%d\]  Job ID for Titles."
    puts " -force                              Replace existing title files."
    puts " -verb                               Verbose Mode ON."
    puts " -noverb                             Verbose Mode OFF (Default)."
    puts "Notes:"
    puts " 1. Each rack directory is expected to have subdirectories 1 up to 96." 
    puts ""
    puts " 2. The Compound ID Tables should have the following format:"
    puts ""
    puts "   VARS    EXPERIMENT CMPD_ID"
    puts "   FORMAT  %11s       %10s"
    puts ""
    puts "     1  1948979#1"
    puts "     2  1375692#1"
    puts "     3    11994#2"
    puts "     4  2031826#1"
    exit 0
   }

set rackID 1

getArgD $ARGV -rackID rackID

set tabName   id/plate$rackID.tab
set rackDir   Rack_$rackID
set jobID     [sysTime %Y%m%d]
set forceFlag 0
set verbose   0

getArgD $ARGV -cIDTab  tabName
getArgD $ARGV -rackDir rackDir

getArgD $ARGV -jobID jobID

if {[flagLoc $ARGV -force]}   {set forceFlag 1}
if {[flagLoc $ARGV -noforce]} {set forceFlag 0}

if {[flagLoc $ARGV -verb]}    {set verbose 1}
if {[flagLoc $ARGV -noverb]}  {set verbose 0}

#
#------------------------------------------------------------------------------------------

if {![file exists $tabName]} \
   {
    puts "Compound Title Error Finding Compound ID File $tabName"
    exit 1
   }

if {![file isdirectory $rackDir]} \
   {
    puts "Compound Title Error Finding Rack Directory $rackDir"
    exit 1
   }

set t     [gdbRead table -in $tabName]
set eList [gdbGetInfo $t -parms entries]
set vList [gdbGetInfo $t -parms varNames]

if {[lsearch -exact $vList EXPERIMENT] < 0} \
   {
    puts "Compound Title Error: Table $tabName is missing EXPERIMENT column."
    exit 1
   }

if {[lsearch -exact $vList CMPD_ID] < 0} \
   {
    puts "Compound Title Error: Table $tabName is missing CMPD_ID column."
    exit 1
   }

set nMiss 0
set nGood 0

foreach e $eList \
   {
    set cID     [join [split [gdbGet $e CMPD_ID] "#"] "_"]
    set expName [gdbGet $e EXPERIMENT]
    set eID     [file root $expName]

    if {![file isdirectory $rackDir/$expName]} \
       {
        if {$verbose} \
           {
            puts "Compound Title Warning: Missing Spectral Data $rackDir/$expName $cID"
           }

        incr nMiss
        continue
       }

    if {$verbose} \
       {
        puts "$rackDir/$expName Job $jobID Rack $rackID Exp $eID Library CID $cID"
       }

    if {!$forceFlag} \
       {
        if {[file exists $rackDir/$expName/title]} \
           {
            puts "Compound Title Notice: Existing title file for Data $rackDir/$expName $cID"
            puts "   Use -force flag to replace existing title files."
            continue
           }
       }

    set error [catch {set outUnit [open $rackDir/$expName/title w]}]

    if {$error} \
       {
        puts "Compound Title Error Opening File $rackDir/$expName/title"
        puts "Check file permissions and try again."
        exit 1
       }

    puts $outUnit "${jobID}-r${rackID}-e${eID}-lib-$cID"

    catch "close $outUnit"
    incr nGood
   }

if {$nMiss} \
   {
    puts "Compound Title Warning: $nMiss Entries in $tabName are missing spectral data."
   }

puts "[format %-16s $tabName] [format %-12s $rackDir] Job: $jobID Rack: [format %3d $rackID] Experiments: $nGood"
exit 0
