#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Set Values for One Column in an NMRPipe-Format Table:"
    puts stderr " -in   inName   \[test.tab\] Input Table."
    puts stderr " -out  outName  \[new.tab\]  Output Table."
    puts stderr " -var  varName  \[VAR\]      Column Variable to Set."
    puts stderr " -val  valList  \[None\]     List of Values for the Column."
    puts stderr " -row  firstRow \[0\]        Starting Row Number, First Row is 0."
    puts stderr " -verb                     Verbose Mode ON (Default)."
    puts stderr " -noverb                   Verbose Mode OFF."
    puts stderr "Parameters for Columns That Don't Already Exist:"
    puts stderr " -fmt  varFmt   \[%s\]       Format for Column."
    puts stderr " -type varType  \[Auto\]     Variable Type for the Column (Auto text int float)."
    puts stderr " -def  defVal   \[0\]        Default Value for Column Entries."
    puts stderr "PDB Options:"
    puts stderr " -pdb                      Input is PDB Data (Default for .pdb)."
    puts stderr " -nopdb                    Input is NMRPipe-Format Table."
    puts stderr " -hetatm                   Interpret HETATMs as ATOMs."
    exit 1
   }

set inName   test.tab
set outName  new.tab
set valList  None
set varName  VAR
set varType  Auto
set varFmt   "%s"
set firstRow 0
set defVal   0

set pdbMode nopdb
set hetMode nohetatm

set verbose 1

getArgD $ARGV -in   inName
getArgD $ARGV -var  varName
getArgD $ARGV -fmt  varFmt
getArgD $ARGV -type varType
getArgD $ARGV -row  firstRow
getArgD $ARGV -def  defVal

getListArgD $ARGV -val valList

if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

if {[flagLoc $ARGV -verb]}      {set verbose 1}
if {[flagLoc $ARGV -noverb]}    {set verbose 0}

if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

#
#------------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts stderr "Error Finding Input File $inName"
    exit 1
   }

set inTab  [gdbRead table -in $inName -$pdbMode -$hetMode]
set vList  [gdbGetInfo $inTab -parms varNames]
set eList  [gdbGetInfo $inTab -parms entries]

set eCount   [llength $eList]
set valCount [llength $valList]

if {$verbose} {puts stderr "setTabCol. Input: $inName Entries: $eCount Variable: $varName First Row: $firstRow"}

if {[lsearch -exact $vList $varName] < 0} \
   {
    if {![strcasecmp $varType Auto]} \
       {
        set c [string index $varFmt end]

        if {![strcasecmp $c e] || ![strcasecmp $c f] || ![strcasecmp $c g]} \
           {
            set varType float
           } \
        elseif {![strcasecmp $c d] || ![strcasecmp $c x]} \
           {
            set varType int
           } \
        else \
           {
            set varType text 
           }
       }

    gdbCreate variable -parent $inTab -name $varName -varType $varType -varFmt $varFmt

    set vList  [gdbGetInfo $inTab -parms varNames]
    set eList  [gdbGetInfo $inTab -parms entries]
    set eCount [llength $eList]

    foreach e $eList {gdbSet $e $varName $defVal}

    if {$verbose} {puts stderr "setTabCol. New Variable. Type: $varType Format: $varFmt Default Value: $defVal"}
   }

set usedCount 0

set j $firstRow

for {set i 0} {$i < $valCount} {incr i} \
   {
    set val [lindex $valList $i]

    if {$j >= 0 && $j < $eCount} \
       {
        gdbSet [lindex $eList $j] $varName $val
        incr usedCount
       }

    incr j
   }

if {$verbose} {puts stderr "setTabCol. Output: $outName Values Specified: $valCount Used: $usedCount"}

gdbWrite $inTab $outName -$hetMode -$pdbMode

exit 0
