#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

puts stderr "shift2tab.tcl < pippShiftTable > talosShiftTable"
if {[flagLoc $ARGV -help]} {exit 0}

#
# Convert PIPP format shift table to TALOS assignment table.

#
# Initialize:

set atomList \
 "HN HA* HB* HC* HD* HE* HG* HZ* CA* CB* CD* CE* CG* CZ C N ND* NE* NH"

set inUnit stdin

set resID    -1
set spinID   -1
set resType  None
set resName  None
set prevLine ""
set fmt      "%4d %s %s %10s  % 9.3f"

set uCount    0
set lineCount 0

initResNames

proc showAtom { key thisLine } \
{
   global atomList atomName shift resID resType resName lineCount uCount fmt

   set line $thisLine

   if {[string first "*" $key] >= 0} \
      {
       puts stderr "Line $lineCount unused: $line"
       incr uCount
       return
      }

   foreach atom $atomList \
      {
       if {[string match $atom $key]} \
          {
           set line [lrange $line 0 1]
          }
      }

   if {[llength $line] == 2} \
      {
       set atomName [lindex $line 0]
       set shift    [lindex $line 1]

       if {"$atomName" == "HETEROGENEITY"} \
          {
           incr uCount
          } \
       else \
          {
           set str ""

           set error [catch \
              {set str [format $fmt $resID $resType $resName $atomName $shift]}]

           puts $str
          }
      } \
   else \
      {
       puts stderr "Line $lineCount unused: $line"
       incr uCount
      }
}

#
# Output table header:

puts "VARS    RESID RESTYPE RESNAME ATOMNAME SHIFT"
puts "FORMAT  %4d   %s      %s      %s       %9.3f"
puts ""

#
# Read and convert the shift information:

while {[gets $inUnit line] >= 0} \
   {
    incr lineCount

    if {[llength $prevLine]} {set line "$prevLine $line"}

    set key [lindex $line 0]
    set n   [llength $line]
    set c1  [string index $key 0]
    set cn  [string range $line end end]

    if {!$n}           continue
    if {"$c1" == "#"}  continue

    if {"$cn" == "\\"} \
       {
        append prevLine " $line" 
        continue 
       } \
    else \
       {
        set prevLine ""
       }

    switch $key \
       {
        RES_ID   {
                  set resID [lindex $line 1]
                 }

        RES_TYPE {
                  set resType [lindex $line 1]
                  set resName [getResName1 $resType]
                 }

        SEQUENCE         {incr uCount}
        SHIFT_FL_FRMT    {incr uCount}
        FIRST_RES_IN_SEQ {incr uCount}
        BEGIN_RES_DEF    {incr uCount}

        PREV_TYPE        {set prevType [lrange $line 1 end]}
        SPIN_SYSTEM_ID   {set spinID   [lindex $line 1]}

        END_RES_DEF      {
                          set resID -1
                          incr uCount
                         }

        default          {
                          if {$resID > 0} \
                             {
                              showAtom $key $line
                             }
                         }
       }
    
   }

exit
