#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Display H-H Planes from 4D Spectrum:"
    exit 0
   }

set xSize   [expr int(0.9*[winfo screenwidth .])]
set ySize   [expr int(0.45*$xSize)]

set charX    12
set charY    16

set tabName     None
set inName4D    ft/test%04d.ft4
set inName2D    proj.dat

set hi(2D)      Auto
set hi(4D)      Auto

set xName       Auto
set yName       Auto
set zName       Auto
set aName       Auto

set roiID(2D)     1
set roiID(4D)     2 
set roiID(4DFull) 0

set cellID(2D)    1
set cellID(4D)    2

set eList         ""
set pkTab         0
set rdCount       0

set dz            0
set da            0

getArgD $ARGV -in    inName4D
getArgD $ARGV -plane inName2D 
getArgD $ARGV -tab   tabName

getArgD $ARGV -hi    hi(2D)
getArgD $ARGV -hi    hi(4D)
getArgD $ARGV -hi2D  hi(2D)
getArgD $ARGV -hi4D  hi(4D)

getArgD $ARGV -xSize xSize
getArgD $ARGV -ySize ySize

getArgD $ARGV -charX charX
getArgD $ARGV -charY charY

getArgD $ARGV -xName xName 
getArgD $ARGV -yName yName 

getArgD $ARGV -dz    dz
getArgD $ARGV -da    da

if {[flagLoc $ARGV -readAll]}   {set roi(4DFull) 3}
if {[flagLoc $ARGV -noreadAll]} {set roi(4DFull) 0}

#
#---------------------------------------------------------------------------------

proc drawPeaks { } \
{
   global eList 

   foreach eI $eList \
      {
       set xI  [gdbGet $eI X_PPM]
       set yI  [gdbGet $eI Y_PPM]
       set id  [gdbGet $eI INDEX]
       set ass [gdbGet $eI ASS]
   
       drawText -cell 1 -roiRef 1 -x ${xI}ppm -y ${yI}ppm -fg -yellow -text ${id}=${ass}
      }
}

proc setThisHi { mode xPix yPix } \
{
   global hi cx1 cxn cy1 cyn

   if {$xPix >= $cx1(2D) && $xPix <= $cxn(2D) && $yPix >= $cy1(2D) && $yPix <= $cyn(2D)} \
      {
       set type 2D
      } \
   elseif {$xPix >= $cx1(4D) && $xPix <= $cxn(4D) && $yPix >= $cy1(4D) && $yPix <= $cyn(4D)} \
      {
       set type 4D
      } \
   else \
      {
       return
      }

   if {![info exists hi($type)]} return

   if {![strcasecmp $mode up]} \
      {
       set hi($type) [incrHiVal $hi($type) 1.3]
      } \
   elseif {![strcasecmp $mode down]} \
      {
       set hi($type) [decrHiVal $hi($type) 1.3]
      }

   if {![strcasecmp $type 2D]} {draw2D}
   if {![strcasecmp $type 4D]} {draw4D}
}

proc drawAssGrid {} \
{
   global px1 pxn py1 pyn roiID cellID

   drawRec -cell $cellID(4D) -x 0% 100% -y 0% 100% -fg black

   drawLine -cell $cellID(4D) -roiRef $roiID(4D) -x $px1 $px1 -y 0% 100% -fg gray10
   drawLine -cell $cellID(4D) -roiRef $roiID(4D) -x $pxn $pxn -y 0% 100% -fg gray10 

   drawLine -cell $cellID(4D) -roiRef $roiID(4D) -x 0% 100% -y $px1 $px1 -fg gray10 
   drawLine -cell $cellID(4D) -roiRef $roiID(4D) -x 0% 100% -y $pxn $pxn -fg gray10 
}

proc draw2D {} \
{
   global hi cellID roiID xSize ySize cx1 cxn 

   drawRec -x 1 $cxn(2D) -y 1 $ySize -win 1 -fg dimGray

   drawROI -roi $roiID(2D) -cell $cellID(2D) -noxGrid -noyGrid -fg gray -bg black -colorS None -colorA gray \
           -color1 multi -color2 multi -axis Bottom Right -hi $hi(2D)

   drawPeaks
}

proc draw4D {} \
{
   global hi cellID roiID xSize ySize charX charY cx1 cxn  rdCount

   drawRec -x [expr $cxn(2D) + 7*$charX + 1] $xSize -y 1 $ySize -win 1 -fg dimGray

   if {!$rdCount} return

   drawAssGrid

   drawROI -roi $roiID(4D) -cell $cellID(4D) -noxGrid -noyGrid -fg gray -bg None -colorS None -colorA gray \
           -color1 multi -color2 multi -axis Bottom Right -hi $hi(4D)
}

proc rdPeakTab {} \
{
   global tabName pkTab eList

   if {[strcasecmp $tabName None]} \
      {
       if {![file exists $tabName]} \
          {
           puts "Error Finding 2D PPM Table $tabName"
           exit 1
          }
 
       set pkTab [gdbRead table -in $tabName]
       set eList [gdbGetInfo $pkTab -parms entries]

       puts "Peak Table $tabName [llength $eList] peaks."
      }
}

proc setWinSize {} \
{
   global xSize ySize charX charY cWide cHigh cx1 cxn cy1 cyn qx1 qy1 qxn qyn

   set cWide [expr ($xSize - (2*6*$charX + $charX))/2]
   set cHigh [expr $ySize - (4*$charY + $charY)]

   puts "setWinSize $cWide $cHigh"

   set cx1(2D) $charX
   set cxn(2D) [expr $cx1(2D) + $cWide] 
   set cy1(2D) $charY 
   set cyn(2D) [expr $cy1(2D) + $cHigh] 

   set cx1(4D) [expr $cxn(2D) + 7*$charX]
   set cxn(4D) [expr $cx1(4D) + $cWide]
   set cy1(4D) $charY
   set cyn(4D) [expr $cy1(4D) + $cHigh]

   set qx1 [expr $cx1(2D) + $cWide/4]
   set qxn [expr $qx1 + $cWide/2]

   set qy1 [expr $cyn(2D) - $cHigh/4]
   set qyn [expr $qy1 - $cHigh/2]
}

proc resizeView args \
{
   global xSize ySize cx1 cxn cy1 cyn cellID

   puts "resizeview $args"

   bind .c <Configure> ""

   update

   set xSize [winfo width  .]
   set ySize [winfo height .]

   setWinSize

   catch ".c delete pixmap -tags pixmap"

   .c create pixmap 0 0 -width $xSize -height $ySize -win 1 -tags pixmap

   defineCell -x $cx1(2D) $cxn(2D) -y $cy1(2D) $cyn(2D) -cell $cellID(2D) -win 1
   defineCell -x $cx1(4D) $cxn(4D) -y $cy1(4D) $cyn(4D) -cell $cellID(4D) -win 1

   bind .c <Configure> "resizeView resize %w %h %T"
   update

   drawRec -x 1 $xSize -y 1 $ySize -win 1 -fg dimGray
  
   draw2D
   draw4D 
}

proc update4DLoc {} \
{
    global qx1 qxn qy1 qyn fdata zLoc aLoc xName yName zName aName px1 pxn py1 pyn dz da inName4D rdCount fdata roiID cellID

    if {![strcmp $fdata 0]} return

    set px1 [pix2spec -cell $cellID(2D) -win 1 -roi $roiID(2D) -x $qx1 -units ppm]ppm
    set pxn [pix2spec -cell $cellID(2D) -win 1 -roi $roiID(2D) -x $qxn -units ppm]ppm

    set py1 [pix2spec -cell $cellID(2D) -win 1 -roi $roiID(2D) -y $qy1 -units ppm]ppm
    set pyn [pix2spec -cell $cellID(2D) -win 1 -roi $roiID(2D) -y $qyn -units ppm]ppm

    set zLoc [expr int(0.5 + [spec2pnt $fdata CUR_ZDIM $py1])]
    set aLoc [expr int(0.5 + [spec2pnt $fdata CUR_ADIM $pyn])]

    puts "4D Loc $px1 $py1 $pxn $pyn"

    puts "readROI -ndim 2 -in $inName4D -roi $roiID(4D) -x $xName 0% 100% -y $yName 0% 100% -dz $zName $zLoc $dz -da $aName $aLoc $da"

    readROI -ndim 2 -in $inName4D -roi $roiID(4D) -x $xName 0% 100% -y $yName 0% 100% -dz $zName $zLoc $dz -da $aName $aLoc $da

    incr rdCount
}

proc adjView { xPix yPix } \
{
   global cx1 cxn cy1 cyn cellID roiID rdCount qx1 qy1 qxn qyn charX charY

   puts "adjView $xPix $yPix Current: $qx1 $qxn $qy1 $qyn"

   set dx1 [expr abs($qx1 - $xPix)]
   set dy1 [expr abs($qy1 - $yPix)]
   set d1  [expr sqrt($dx1*$dx1 + $dy1*$dy1)]

   set dxn [expr abs($qxn - $xPix)]
   set dyn [expr abs($qyn - $yPix)]
   set dn  [expr sqrt($dxn*$dxn + $dyn*$dyn)]

   if {$d1 > 2*$charY && $dn > 2*$charY} return

   if {$d1 < $dn} \
      {
       set qx1 $xPix
       set qy1 $yPix
      } \
   else \
      {
       set qxn $xPix
       set qyn $yPix
      }

   .c coords cursor $qx1 $qy1 $qxn $qyn

   update4DLoc
   draw4D
}

proc parseKey args \
{
   puts "parsekey $args"
}

set fdata 0
set zLoc  1
set aLoc  1

setWinSize

if {![file exists $inName2D]} \
   {
    puts "Error Finding 2D Spectrum $inName2D"
    exit 1
   }

set thisName [format $inName4D 1 1]

if {![file exists $thisName]} \
   {
    puts "Error Finding 4D Spectrum $inName4D"
    exit 1
   }

set error [rdFDATA $thisName fdata]

if {![strcasecmp $xName Auto]} {set xName [getParmStr $fdata NDLABEL CUR_XDIM]}
if {![strcasecmp $yName Auto]} {set yName [getParmStr $fdata NDLABEL CUR_YDIM]}
if {![strcasecmp $zName Auto]} {set zName [getParmStr $fdata NDLABEL CUR_ZDIM]}
if {![strcasecmp $aName Auto]} {set aName [getParmStr $fdata NDLABEL CUR_ADIM]}

puts "4D Spectrum $inName4D $xName $yName $zName $aName"

set hi(2D) [exec getSpecHi.tcl -in $inName2D -hi $hi(2D) -brief]
set hi(4D) [exec getSpecHi.tcl -in $inName4D -hi $hi(4D) -brief]

puts [format "2D Contour Height: %.3e 4D Contour Height: %.3e" $hi(2D) $hi(4D)]

readROI -roi $roiID(2D) -in $inName2D -ndim 2 -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

frame  .cmnd

canvas .c -width $xSize -height $ySize -border 0 -closeenough $charY 

pack   .cmnd -in . -side top -fill x
pack   .c    -in . -side top

resizeView init

bind .c <B1-Motion>   "adjView %x %y"
bind .c <Button-4>    "setThisHi up   %x %y"
bind .c <Button-5>    "setThisHi down %x %y"

bind .  <KeyPress>    "+ parseKey %K %x %y"

.c create line $qx1 $qy1 $qxn $qyn -width 3 -fill gray -tags cursor

update4DLoc
draw4D
