#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

#
# Automatic Creation of nlinLS Fitting Script.

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "simAllPeaks.tcl: Utility Used by autoFit.tcl to Simulate All Peaks in a Table."
    exit 1
   }

set dimCount     2
set tabName      nlin.tab
set specName     test.ft2
set simAllDir    simAll
set simAllScript simAll.com
set simAllName   %04d.dat

getArgD $ARGV -tab  tabName
getArgD $ARGV -ndim dimCount
getArgD $ARGV -ft   specName

getArgD $ARGV -simAllDir    simAllDir
getArgD $ARGV -simAllName   simAllName
getArgD $ARGV -simAllScript simAllScript

if {![strcasecmp $simAllName Auto]} {set simAllName %04d.ft$dimCount}

set loc [flagLoc $ARGV -simArgs]

set simArgs [lrange $ARGV [expr $loc + 1] end]

catch {file delete -force $simAllDir}
catch {file mkdir  $simAllDir}

set error [catch {set outUnit [open $simAllScript w 0744]}]

set pkTab [gdbRead table -in $tabName]
set entry [gdbFirst $pkTab]

puts $outUnit "#!/bin/csh -f"
puts $outUnit ""

puts $outUnit "# $ARGV"
puts $outUnit ""

puts $outUnit "echo ''"
puts $outUnit "echo Simulating All Peaks. Peak Table: $tabName Output: $simAllDir/$simAllName"
puts $outUnit ""

while {$entry} \
   {
    set id      [gdbGet $entry INDEX]
    set ass     [gdbGet $entry ASS]

    set outName [format $simAllDir/$simAllName $id]

    puts $outUnit "nmrPipe -in $specName -out $outName -fn SET -c 0.0 -ov"
    puts $outUnit ""

    puts $outUnit "simSpecND -in $tabName -data $outName \\"
    puts $outUnit "          -first $id -last $id \\"
    puts $outUnit "          $simArgs"
    puts $outUnit ""

    set tStr [format %3d $id]

    if {$dimCount >= 1} {append tStr [format -%.2fppm [gdbGet $entry X_PPM]]}
    if {$dimCount >= 2} {append tStr [format -%.2fppm [gdbGet $entry Y_PPM]]}
    if {$dimCount >= 3} {append tStr [format -%.2fppm [gdbGet $entry Z_PPM]]}
    if {$dimCount >= 4} {append tStr [format -%.2fppm [gdbGet $entry A_PPM]]}

    append tStr -$ass

    puts $outUnit "sethdr $outName -title $tStr -ndim $dimCount"
    puts $outUnit ""

    set entry [gdbNext $entry]
   }

close $outUnit

if {![flagLoc $ARGV -noexec]} \
   {
    set g 90x12+15+15
    set error [catch {exec xterm -bg cyan -fg black -geom $g -e $scriptName}]
   }

exit 0
