#!/bin/csh -f

if (`flagLoc $argv -help`) then
   echo "Convert SPARTA Chemical Shift Prediction Output to NMRPipe Table:"
   echo " -in  inName  [sparta.txt]   Prediction Text, as Produced by SPARTA." 
   echo " -pdb pdbName [ref.pdb]      PDB File, as used for SPARTA Prediction."
   exit 0
endif

set inName  = (`getArgD $argv -in  sparta.txt`) 
set pdbName = (`getArgD $argv -pdb ref.pdb`) 

if (!(-e $inName)) then
   exit 0
endif

if (-e $pdbName) then
   getTabInfo.tcl -in $pdbName -seqText
endif

set nLoc       = 0
set haLoc      = 1
set cLoc       = 2
set caLoc      = 3
set cbLoc      = 4
set hnLoc      = 5
set resIDLoc   = 6
set resNameLoc = 7

set resName1List = (A   R   N   D   C   c   Q   E   G   H   h    I   \
                    L   K   M   F   P   S   T   W   Y   V)

set resName3List = (ALA ARG ASN ASP CYS cys GLN GLU GLY HIS his  ILE \
                    LEU LYS MET PHE PRO SER THR TRP TYR VAL)

set resIDList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $resIDLoc`) 

set resTypeList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $resNameLoc`)

set hnList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $hnLoc`)

set nList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $nLoc`)

set haList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $haLoc`)

set cList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $cLoc`)

set caList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $caLoc`)

set cbList = (`fgrep -vi read $inName | fgrep -vi run | fgrep -vi pred | fgrep -vi HA | fgrep . | getCols - - $cbLoc`)

set resNameList = ""

set n  = $#resIDList
set nR = $#resName1List
set i  = 1

while ($i <= $n)
  set j = 1
  set resName3 = "XXX"

  while ($j <= $nR)
    if ("$resName1List[$j]" == "$resTypeList[$i]") then
       set resName3 = $resName3List[$j]
       break 
    endif

    @ j++
  end  

  set resNameList = ($resNameList $resName3)
  @ i++
end

set i = 1

echo "VARS   RESID RESNAME ATOMNAME SHIFT"
echo "FORMAT %4d   %-4s    %-4s     %9.3f"
echo ""

set fmt = " %4d %-4s %-4s % 9.3f\n"

while ($i <= $n)
 set resID   = $resIDList[$i]
 set resName = $resNameList[$i]

 set hnPPM   = $hnList[$i]
 set nPPM    = $nList[$i]
 set haPPM   = $haList[$i]
 set caPPM   = $caList[$i]
 set cbPPM   = $cbList[$i]
 set cPPM    = $cList[$i]

 nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName HN $hnPPM | fgrep -v 999.
 nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName N  $nPPM  | fgrep -v 999.

 if ("$resName" == "G") then
  nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName HA2 $haPPM | fgrep -v 999.
  nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName HA3 $haPPM | fgrep -v 999.
 else
  nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName HA  $haPPM | fgrep -v 999.
 endif

 nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName CA $caPPM | fgrep -v 999.
 nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName CB $cbPPM | fgrep -v 999.
 nmrPrintf " %4d %-4s %-4s % 9.3f\n" $resID $resName C  $cPPM  | fgrep -v 999.
 
 @ i++   
end
