#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -backingstore -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Create a series of GIF images from 2D Spectral Planes:" 
    puts " -in           inName1 ...  \[test.ft2\]  List of Spectral Input Files."
    puts " -out          outName1 ... \[Auto\]      List of GIF Output Files, or Keyword Auto."
    puts " -hi           hi           \[Full\]      Contour Height, or Keyword Full."
    puts " -snapDir      snapDir      \[gif\]       Directory Name for Output GIF Files, or Keyword None."
    puts " -sleep        sleepTime    \[1000\]      Sleep Time Between Screen Captures, msec."
    puts " -xSize        xSizePix     \[512\]       X Size of the Spectral Drawing Region, Pixels."
    puts " -ySize        ySizePix     \[512\]       Y Size of Spectral Drawing Region, Pixels."
    puts " -leftMargin   leftMargin   \[0\]         Left Margin, Pixels."
    puts " -rightMargin  rightMargin  \[0\]         Right Margin, Pixels."
    puts " -bottomMargin bottomMargin \[0\]         Bottom Margin, Pixels."
    puts " -topMargin    topMargin    \[0\]         Top Margin, Pixels."
    puts " -bg           bgColor      \[black\]     Background Color for Spectral Graphic."
    puts "Drawing Options, should be last on spec2gif.tcl command line:"
    puts " -drawArgs drawArgs  \[-cont -nobox\]     Arguments for drawROI."
    exit 0
   }

set inList    test.ft2
set outList   Auto
set hi        Full
set snapDir   gif
set sleepTime 1000
set xSize     512
set ySize     512

set leftMargin   0
set rightMargin  0
set bottomMargin 0
set topMargin    0

set captureFlag  1

set bgColor   black
set drawArgs  "-cont -nobox"

set loc [flagLoc $ARGV -drawArgs]

if {$loc} \
   {
    set drawArgs [lrange $ARGV [expr $loc+1] end]
    set ARGV     [lrange $ARGV 0 [expr $loc-1]]
    set ARGC     [llength $ARGV]
   }

getArgD $ARGV -sleep   sleepTime
getArgD $ARGV -snapDir snapDir
getArgD $ARGV -hi      hi
getArgD $ARGV -xSize   xSize
getArgD $ARGV -ySize   ySize 
getArgD $ARGV -bg      bgColor

getArgD $ARGV -leftMargin   leftMargin
getArgD $ARGV -rightMargin  rightMargin
getArgD $ARGV -bottomMargin bottomMargin
getArgD $ARGV -topMargin    topMargin

getListArgD $ARGV -in  inList
getListArgD $ARGV -out outList

if {[flagLoc $ARGV -capture]}   {set captureFlag 1}
if {[flagLoc $ARGV -nocapture]} {set captureFlag 0}

foreach inName $inList {lappend outList Auto}

proc captureScreen { outName } \
{
   update

   set im [image create photo]
   winop snap .c $im

   $im write -format gif $outName

   image delete $im
}

if {![file exists $snapDir]} {catch "file mkdir $snapDir"}

if {![file isdirectory $snapDir]} \
   {
    puts "Error creating or accessing output directory $snapDir"
    exit 1
   }

set xSizeFull [expr $xSize + $leftMargin + $rightMargin]
set ySizeFull [expr $ySize + $topMargin  + $bottomMargin]

canvas .c -width $xSizeFull -height $ySizeFull -borderwidth 0
pack   .c -side top -expand 0 -fill none -padx 0 -pady 0 -ipadx 0 -ipady 0

update

.c create pixmap 0 0 -width $xSizeFull -height $ySizeFull -win 1

defineCell -cell 1 -win 1 -x $leftMargin [expr $leftMargin + $xSize - 1] -y $topMargin [expr $topMargin + $ySize - 1]

update

set i 0

foreach inName $inList \
   {
    set outName [lindex $outList $i]

    if {![strcasecmp $outName Auto]} {set outName $snapDir/[file root [file tail $inName]].gif}

    incr i

    if {![file exists $inName]} \
       {
        puts "Skipping missing file: $inName"
        continue
       }

    readROI -roi 1 -in $inName -ndim 2 -x X_AXIS -y Y_AXIS

    drawRec -roi 1 -cell 1 -win 1 -x 1 $xSizeFull -y 1 $ySizeFull -fg $bgColor

    if {$captureFlag} \
       {
        eval "drawROI $drawArgs -roi 1 -cell 1 -win 1 -hi $hi -bg None"
        captureScreen $outName    
       } \
    else \
       {
        eval "drawROI $drawArgs -roi 1 -cell 1 -win 1 -hi $hi -bg None -outPixmap $outName -buff"
       }

    if {$sleepTime > 0} {after $sleepTime}

    puts "$inName $outName"
   }

update

exit 0
