#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set baseList {blue darkGreen darkOrchid red orange magenta cyan violet pink firebrick brown gray black}

if {[flagLoc $ARGV -help]} \
   {
    puts "Generate a List of Colors for Spectral Drawing and Plotting:"
    puts " -n         rCount \[Auto\]    Number of Colors to Return."
    puts " -genCount  gCount \[Auto\]    Number of Colors to Generate."
    puts " -in        inList \[None\]    Optional List of Input Spectra."
    puts " -colorMode type   \[Number\]  Color Mode: Number File Title 1 2 3 ... Last."
    puts " -cycle                      Cyclically Repeat Colors As Needed (Default)."
    puts " -nocycle                    Do Not Cyclically Repeat Colors."
    puts " -hex                        List All Colors as Hex Codes."
    puts " -nohex                      Do Not Convert Named Colors to Hex Codes (Default)."
    puts " -rev                        Reverse the Order of Colors."
    puts "Color Schemes:"
    puts " -colors colorList           Use List of Named Colors (Default; See Below)."
    puts " -hsv    h1 s1 v1  hN sN vN  Use Hue Saturation Value (HSV) Series (0.0 to 1.0)."
    puts " -rgb    r1 g1 b1  rN gN bN  Use Red Green Blue (RGB) Series (0 to 255)."
    puts "Predefined Settings:"
    puts " -pos                        Same As: -hsv 0.5 1.0 0.8  0.70 1.0 1.0"
    puts " -neg                        Same As: -hsv 0.0 1.0 0.8  0.20 1.0 1.0"
    puts " -rainbow                    Same As: -hsv 0.0 1.0 1.0  0.75 1.0 1.0"
    puts "Notes:"
    puts " 1. Values for -colors can be color names (such as: red) or RGB hex"
    puts "    codes (such as: #ff0000)."
    puts " 2. Option -colorMode File will assign colors according to the file names,"
    puts "    (with paths and extensions stripped) as given via the -in option." 
    puts " 3. Option -colorMode Title will assign colors according to the titles"
    puts "    recorded in NMRPipe-format data specified via the -in option." 
    puts " 4. If the value for -colorMode is an integer N, color will be assigned"
    puts "    according to the Nth dash-delimited item in the title (Requires -in ...)."
    puts " 5. Option -colorMode Last will assign colors according to the last"
    puts "    dash-delimited item in the title (Requires -in ...)." 
    puts " 6. Colors generated via -hsv will vary continuously in the HSV color"
    puts "    space from a starting color Hue = h1 Saturation = s1 Value = v1"
    puts "    to a final color Hue = hN Saturation = vN Value = vN." 
    puts " 7. Colors generated via -rgb will vary continuously in the RGB color"
    puts "    space from a starting color Red = r1 Green = g1 Blue = b1 to a"
    puts "    final color Red = rN Green = gN Blue = bN." 
    puts " 8. Values for -hsv are floating point numbers from 0.0 to 1.0,"
    puts "    while values for -rgb are integers from 0 to 255."
    puts " 9. Default colors for -colors are:"
   
    set i 0
 
    foreach color $baseList \
       {
        if {!($i % 5)} {puts -nonewline "      "}
        incr i

        puts -nonewline [format " %-12s" $color]
        if {!($i % 5)} {puts ""}
       }

    puts ""
    exit 0
   }

set colorCount  Auto 
set genCount    Auto 
set inCount     0

set inList      None
set colorMode   Number
set colorScheme Name 
set cycleFlag   1
set revFlag     0
set hexFlag     0

set hue1      0.0
set sat1      1.0
set val1      1.0

set hueN      1.0
set satN      1.0
set valN      1.0

set red1      0
set green1    0
set blue1     0

set redN      255 
set greenN    255 
set blueN     255

getListArgD $ARGV -in     inList
getListArgD $ARGV -colors baseList

getArgD $ARGV -n          colorCount
getArgD $ARGV -genCount   genCount
getArgD $ARGV -colorMode  colorMode

if {[flagLoc $ARGV -rev]}     {set revFlag 1}
if {[flagLoc $ARGV -norev]}   {set revFlag 0}

if {[flagLoc $ARGV -hex]}     {set hexFlag 1}
if {[flagLoc $ARGV -nohex]}   {set hexFlag 0}

if {[flagLoc $ARGV -cycle]}   {set cycleFlag 1}
if {[flagLoc $ARGV -nocycle]} {set cycleFlag 0}

if {[strcasecmp $inList None]} {set inCount [llength $inList]} 

if {![strcasecmp $colorCount Auto]} {set colorCount $inCount}
if {![strcasecmp $genCount   Auto]} {set genCount   $inCount}

set loc [flagLoc $ARGV -hsv]

if {$loc} \
   {
    set colorScheme hsv
    set nextLoc [nextFlag $ARGV $loc]

    if {$nextLoc != $loc + 7} \
       {
        puts stderr "Color Error: -hsv Requires Six Numbers from 0.0 to 1.0."
        exit 1
       }

    set hue1 [lindex $ARGV [expr $loc + 1]]
    set sat1 [lindex $ARGV [expr $loc + 2]]
    set val1 [lindex $ARGV [expr $loc + 3]]

    set hueN [lindex $ARGV [expr $loc + 4]]
    set satN [lindex $ARGV [expr $loc + 5]]
    set valN [lindex $ARGV [expr $loc + 6]]
   }

set loc [flagLoc $ARGV -rgb]

if {$loc} \
   {
    set colorScheme rgb 
    set nextLoc [nextFlag $ARGV $loc]

    if {$nextLoc != $loc + 7} \
       {
        puts stderr "Color Error: -rgb Requires Six Integers from 0 to 255."
        exit 1
       }

    set red1   [lindex $ARGV [expr $loc + 1]]
    set green1 [lindex $ARGV [expr $loc + 2]]
    set blue1  [lindex $ARGV [expr $loc + 3]]

    set redN   [lindex $ARGV [expr $loc + 4]]
    set greenN [lindex $ARGV [expr $loc + 5]]
    set blueN  [lindex $ARGV [expr $loc + 6]]
   }

if {[flagLoc $ARGV -pos]} \
   {
    set colorScheme hsv

    set hue1      0.5
    set sat1      1.0
    set val1      0.8
    
    set hueN      0.7
    set satN      1.0
    set valN      1.0
   }

if {[flagLoc $ARGV -neg]} \
   {
    set colorScheme hsv

    set hue1      0.0
    set sat1      1.0
    set val1      0.8

    set hueN      0.2
    set satN      1.0
    set valN      1.0
   }

if {[flagLoc $ARGV -rainbow]} \
   {
    set colorScheme hsv

    set hue1      0.00
    set sat1      1.00
    set val1      1.00

    set hueN      0.75
    set satN      1.00
    set valN      1.00
   }

#
#-------------------------------------------------------------------------------

if {![llength $baseList]} {set baseList blue}

set baseCount [llength $baseList]
set colorList ""

if {$colorCount < 1} {set colorCount $baseCount}
if {$genCount < 1}   {set genCount   $colorCount}

if {![strcasecmp $colorScheme hsv]} \
   {
    set baseList "" 

    set n [expr $genCount - 1]
    if {$n < 1} {set n 1}
    
    for {set i 0} {$i < $genCount} {incr i} \
       {
        set thisH [expr $hue1 + $i*($hueN - $hue1)/$n]
        set thisS [expr $sat1 + $i*($satN - $sat1)/$n]
        set thisV [expr $val1 + $i*($valN - $val1)/$n]

        lappend baseList [hsv2rgb $thisH $thisS $thisV] 
       }
   } \
elseif {![strcasecmp $colorScheme rgb]} \
   {
    set baseList "" 

    set n [expr $genCount - 1]
    if {$n < 1} {set n 1}

    for {set i 0} {$i < $genCount} {incr i} \
       {
        set thisR [expr int(0.5 + $red1   + double($i)*($redN   - $red1)/$n)]
        set thisG [expr int(0.5 + $green1 + double($i)*($greenN - $green1)/$n)]
        set thisB [expr int(0.5 + $blue1  + double($i)*($blueN  - $blue1)/$n)]

        rClip thisR 0 255
        rClip thisG 0 255
        rClip thisB 0 255

        lappend baseList [format "#%02x%02x%02x" $thisR $thisG $thisB] 
       }
   }

if {![llength $baseList]} {set baseList blue}

set baseCount [llength $baseList]
set titList   "" 
set fdata     0

if {![strcasecmp $colorMode Number]} \
   {
    set titList [getIota 0 1 $colorCount] 
   } \
elseif {![strcasecmp $colorMode File]} \
   {
    foreach inName $inList {lappend titList [file root [file tail $inName]]} 
   } \
else \
   {
    set i 0

    foreach inName $inList \
       {
        if {![file exists $inName]} \
           {
            puts stderr "Color Error Finding Input File $inName"
            exit 1
           }

        set error [rdFDATA [format $inName 1 1] fdata]

        if {$error} \
           {
            puts stderr "Color Error Reading Header in $inName"
            exit 1
           }
        
        set t [getParmStr $fdata FDTITLE NULL_DIM]

        if {[string is integer $colorMode]} \
           {
            set t [lindex [split $t "-"] [expr $colorMode - 1]]
           } \
        elseif {![strcasecmp $colorMode Last]} \
           {
            set t [lindex [split $t "-"] end]
           }

        if {![llength $t]} {set t $i}
      
        lappend titList $t
   
        catch "sysFree $fdata"
        set fdata 0
        incr i
       }
   }

set locList   ""
set uniqList  ""
set nextLoc   0

foreach t $titList \
   {
    set loc [lsearch -exact $uniqList $t]

    if {$loc < 0} \
       {
        lappend uniqList $t 
        set loc $nextLoc
        incr nextLoc
       }

    lappend locList $loc
   }

if {![llength $baseList]} \
   {
    set baseList  blue
    set baseCount 1
   }

set maxLoc [getMax $locList]
if {$maxLoc == 0} {set maxLoc 1}

set colorList ""

for {set i 0} {$i < $colorCount} {incr i} \
   {
    set j [lindex $locList $i]

    if {![string is integer $j]} {set j 0}
    if {$j < 0} {set j 0}

    if {$cycleFlag} \
       {
        while {$j > $baseCount - 1} {set j [expr $j - $baseCount]}
        set color [lindex $baseList $j]
       } \
    else \
       {
        set loc   [expr int(0.5 + double($j)*($baseCount - 1)/$maxLoc)]
        set color [lindex $baseList $loc]
       }

    if {![llength $color]} {set color blue}

    lappend colorList $color
   }

if {$hexFlag} \
   {
    set rgbTab  [gdbRead table -in $env(NMRTXT)/rgb.tab]
    set newList ""

    foreach color $colorList \
       {
        set eList [gdbSelect -from $rgbTab -cond "!strcasecmp( COLOR, '$color' )"]

        if {[llength $eList]} \
           {
            set e     [lindex $eList 0]
            set color [format "#%02x%02x%02x" [gdbGet $e RED] [gdbGet $e GREEN] [gdbGet $e BLUE]]
           }

        lappend newList $color
       }

    set colorList $newList
   }

if {$revFlag} {set colorList [lreverse $colorList]}

puts $colorList

exit
