#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]
  
if {[flagLoc $ARGV -help]} \
   {
    puts "Report Statistics for NMRPipe-format Real or Complex Data:"
    puts " -in   inName \[test.ft2\]  Input Name or Template"
    puts " -mult cMult  \[1.0]        Scaling Value for Statistic."
    puts " -stat sList  \[vMin vMax]  Function Names for Statistics to Report."
    puts "Specification of Statistics to Report:"
    puts " -stat vList  \[vMin vMax vEstNoise size N ndim\]"
    puts "Functions for Combining Statistics from Multiple Files:"
    puts " -join jList  \[Auto ...]   getSum getAvg getMedian getMin getMax."
    puts "Statistics Vector Function Names:"
    puts " vSum vSumSq vSumPos vSumNeg vAvg vStdDev vRMS vVariance"
    puts " vMin vMax vMaxAbs vMinAbs vMedian vEstNoise istMaxRes"
    puts " N size ndim"
    puts "Notes:"
    puts "1. Values for N size ndim will not be multiplied by cMult."
    puts "2. Special Functions:"
    puts ""
    puts "   istMaxRes: Returns cMult*3.0*100.0*vNoiseEst/vMaxAbs."
    puts "   N:         Returns the number of points in the region."
    puts "   size:      Returns a list of the sizes in each dimension."
    puts "   ndim:      Returns the number of dimensions."

    exit 0
   }

set allFnList   "vSum   vSumSq vSumPos vSumNeg vAvg    vStdDev    vRMS    vMin   vMax   vMaxAbs   vMinAbs   vMedian   vEstNoise istMaxRes ndim   N"
set allJoinList "getSum getSum getSum  getSum  getNAvg getNStdDev getNRMS getMin getMax getMaxAbs getMinAbs getMedian getMedian getMedian getMax getISum"

set inName    test.ft2
set statList  "vMin vMax vEstNoise size N ndim"
set joinList  ""

set cMult     1.0
set briefFlag 0

getArgD $ARGV -in   inName
getArgD $ARGV -mult cMult

getListArgD $ARGV -stat statList
getListArgD $ARGV -join joinList

if {[flagLoc $ARGV -brief]}   {set briefFlag 1}
if {[flagLoc $ARGV -nobrief]} {set briefFlag 0}

foreach s $statList {lappend joinList Auto}

#
#----------------------------------------------------------------------

proc appendStatVals { thisName } \
{
    global statList valList sizeList maxAbsList ndim

    set fdata 0
    set rdata 0

    set error [catch {set inUnit [sysOpen $thisName r]}]

    if {$error} \
       {
        puts stderr "specStat.tcl Error Opening File $thisName"
        exit 1
       }

    set error [rdFDATAU $inUnit fdata]

    if {$error} \
       {
        puts stderr "specStat.tcl Error Reading Header in $thisName"
        exit 1
       }

    set thisInfo [getDataSize $fdata]
    set n        [lindex $thisInfo 0]

    set error [catch {set rdata [sysMalloc $n float]}]
  
    if {$error} \
       {
        puts stderr "specStat.tcl Error Allocating $n Points for File $thisName"
        exit 1
       }

    set error [catch {sysRead  $inUnit $rdata $n}]

    foreach s $statList \
       {
        if {![strcasecmp $s N] || ![strcasecmp $s size]} \
           {
            set val $n
           } \
        elseif {![strcasecmp $s ndim]} \
           {
            set val $ndim
           } \
        elseif {![strcasecmp $s istMaxRes]} \
           {
            set noiseVal [vEstNoise $rdata $n]
            set thisMax  [vMaxAbs   $rdata $n]

            lappend maxAbsList $thisMax
           } \
        else \
           {
            set val [expr [$s $rdata $n]]
           }

        lappend valList($s) $val
       }

    lappend sizeList $n
 
    catch "sysClose $inUnit"

    if {[strcmp $fdata 0]} {catch "sysFree $fdata"}
    if {[strcmp $rdata 0]} {catch "sysFree $rdata"}
}

proc getISum { thisList } \
{
   set n [llength $thisList]

   if {!$n} {return 0}

   set sum [lindex $thisList 0]

   for {set i 1} {$i < $n} {incr i} \
      {
       set val [lindex $thisList $i]

       if {[string is int $sum] && [string is int $val]} \
          {
           incr sum $val
          } \
       else \
          {
           set sum [expr $sum + $val]
          }
      }

    return $sum
}

proc getNAvg { vList } \
{
   global sizeList

   set count [llength $vList]

   set nSum [getISum $sizeList] 
   set vSum 0.0

   if {$nSum == 0 || $count == 0} {return 0.0}

   for {set i 0} {$i < $count} {incr i} \
      {
       set v [lindex $vList    $i]
       set n [lindex $sizeList $i]

       set vSum [expr $vSum + $v*(double($n)/$nSum)]
      }

   return $vSum 
}

proc getNStdDev { vList } \
{
   global sizeList

   set count [llength $vList]

   set nSum  [expr [getISum $sizeList] - $count]
   set vSum  0.0

   if {$nSum <= 0 || $count == 0} {return 0.0}

   for {set i 0} {$i < $count} {incr i} \
      {
       set v [lindex $vList    $i]
       set n [lindex $sizeList $i]

       set vSum [expr $vSum + $v*$v*(double($n - 1)/$nSum)]
      }

   return [expr sqrt($vSum)]
}

proc getNRMS { vList } \
{
   global sizeList

   set count [llength $vList]
   set nSum  [getISum $sizeList]
   set vSum  0.0

   if {$nSum == 0 || $count == 0} {return 0.0}

   for {set i 0} {$i < $count} {incr i} \
      {
       set v [lindex $vList    $i]
       set n [lindex $sizeList $i]

       set vSum [expr $vSum + $v*$v*(double($n)/$nSum)]
      }

   return [expr sqrt($vSum)]
}

proc getSameVal { v } \
{
   return $v
}

foreach s $statList {set valList($s) ""}

set maxAbsList ""
set sizeList   ""
	
set thisName [format $inName 1 1]

if {![file exists $thisName]} \
   {
    puts stderr "specStat.tcl Error Finding Input File $thisName"
    exit 1
   }

#
# n dimCount qSize xSize ySize zSize aSize"

set fullSizeInfo [getSpecSizeInfo $inName]
set fileSizeInfo [getSpecSizeInfo $thisName]
set fmtCount     [getFmtCount     $inName]

set dimCount     [lindex $fullSizeInfo 1]
set zCount       [lindex $fullSizeInfo 5]
set aCount       [lindex $fullSizeInfo 6]

if {$fmtCount} \
   {
    set thisInfo $fullSizeInfo
   } \
else \
   {
    set thisInfo $fileSizeInfo
   }

set ndim  [lindex $thisInfo 1]
set qSize [lindex $thisInfo 2]
set xSize [lindex $thisInfo 3]
set ySize [lindex $thisInfo 4]
set zSize [lindex $thisInfo 5]
set aSize [lindex $thisInfo 6]

if {$fmtCount == 2} \
   {
    for {set ia 1} {$ia <= $aCount} {incr ia} \
       {
        for {set iz 1} {$iz <= $zCount} {incr iz} \
           {
            appendStatVals [format $inName $ia $iz]
           }
       }
   } \
elseif {$fmtCount == 1} \
   {
    if {$dimCount == 4} \
       {
        set fileCount $aCount
       } \
    else \
       {
        set fileCount $zCount
       }

    for {set i 1} {$i <= $fileCount} {incr i} \
       {
        appendStatVals [format $inName $i]
       }
   } \
else \
   {
    appendStatVals $inName
   }

set statCount [llength $statList]

for {set i 0} {$i < $statCount} {incr i} \
   {
    set s   [lindex $statList $i]
    set jFn [lindex $joinList $i]

    if {![strcasecmp $jFn Auto]} \
       {
        set loc [lsearch -exact $allFnList $s]

        if {$loc < 0} \
           {
            set jFn getSum
           } \
        else \
           {
            set jFn [lindex $allJoinList $loc]
           }
       }

    if {[llength $valList($s)] == 1} {set jFn getSameVal}

    if {![strcasecmp $s ndim]} \
       {
        set val $ndim
       } \
    elseif {![strcasecmp $s N]} \
       {
        set val [$jFn $valList($s)]
       } \
    elseif {![strcasecmp $s size]} \
       {
        set val "$xSize $ySize $zSize $aSize $qSize"
       } \
    elseif {![strcasecmp $s istMaxRes]} \
       {
        set thisMaxVal [getMaxAbs $maxAbsList]

        if {$thisMaxVal == 0.0} \
           {
            set val 0.0
           } \
        else \
           {
            set val [$jFn $valList($s)]
            set val [expr $cMult*3.0*100.0*$val/$thisMaxVal]
           }
       } \
    else \
       {
        set val [expr $cMult*[$jFn $valList($s)]]
       }

    if {$i} {puts -nonewline " "} 

    if {$briefFlag} \
       { 
        puts -nonewline "$val"
       } \
    else \
       {
        puts -nonewline "$s $val"
       }
   }

puts ""

exit 0
