#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc showViewerHelp args \
{
   puts "Plot Geometry and Details:"
   puts " -sxw          sxw   \[700\]    Initial Plot X-Size, Pixels."
   puts " -syw          syw   \[700\]    Initial Plot Y-Size, Pixels."
   puts " -leftMargin   lPix  \[80\]     Left Margin Size, Pixels."
   puts " -rightMargin  rPix  \[20\]     Right Margin Size, Pixels."
   puts " -bottomMargin bPix  \[80\]     Bottom Margin Size, Pixels."
   puts " -topMargin    tPix  \[40\]     Top Margin Size, Pixels."
   puts " -square                      Make Plots Always Square (Default)."
   puts " -nosquare                    Set Plot Size According to Window Size."
   puts " -axis                        Draw Axis (Default)."
   puts " -noaxis                      No Axis."
   puts " -grid                        Draw Grid (Default)."
   puts " -nogrid                      No Grid."
   puts "Plot Colors:"
   puts " -bgColor      bg   \[black\]   Plot Background Color."
   puts " -winColor     wc   \[gray30\]  Window Background Color."
   puts " -gridColor    gc   \[dimGray\] Plot Grid Color."
   puts " -boxColor     bc   \[yellow\]  Plot Box Color."
   puts " -labelColor   lc   \[yellow\]  Label Color."
   puts " -axisColor    ac   \[yellow\]  Axis Color."
   puts " -shadowColor  sc   \[black\]   Axis Shadow Color."

   return 0
}

proc getViewerArgs { obj } \
{ 
   global objInfo ARGV

   set thisARGV $ARGV

   set objInfo($obj,canvas)     .canvas$obj
   set objInfo($obj,pixmap)     pixmap$obj 
   set objInfo($obj,toplevel)   .
   set objInfo($obj,win)        1

   set objInfo($obj,inList)      ""
   set objInfo($obj,roiList)     ""
   set objInfo($obj,vecList)     ""
   set objInfo($obj,vecSizeList) ""
   set objInfo($obj,inCount)     0

   set objInfo($obj,specX1)     0%
   set objInfo($obj,specXN)     100%

   set objInfo($obj,specY1)     0%
   set objInfo($obj,specYN)     100%

   set objInfo($obj,specZ1)     0%
   set objInfo($obj,specZN)     100%

   set objInfo($obj,specA1)     0%
   set objInfo($obj,specAN)     100%

   set objInfo($obj,gridFlag)   1
   set objInfo($obj,squareFlag) 1
   set objInfo($obj,axisFlag)   1
   set objInfo($obj,labelFlag)  1

   set objInfo($obj,sxw) 700
   set objInfo($obj,syw) 700

   set objInfo($obj,leftMargin)   80
   set objInfo($obj,rightMargin)  20
   set objInfo($obj,topMargin)    40
   set objInfo($obj,bottomMargin) 80

   set objInfo($obj,bgColor)      black
   set objInfo($obj,winColor)     gray30
   set objInfo($obj,gridColor)    dimGray
   set objInfo($obj,labelColor)   yellow
   set objInfo($obj,axisColor)    yellow
   set objInfo($obj,fgColor)      yellow
   set objInfo($obj,shadowColor)  black
   set objInfo($obj,colorList)    "blue magenta blue4 red pink gray darkOrchid"

   set objInfo($obj,thresh)       None
   set objInfo($obj,threshMode)   above

   getArgD $thisARGV -toplevel        objInfo($obj,toplevel)
   getArgD $thisARGV -canvas          objInfo($obj,canvas)
   getArgD $thisARGV -win             objInfo($obj,win)
   getArgD $thisARGV -pixmap          objInfo($obj,pixmap)

   getArgD $thisARGV -swx             objInfo($obj,swx)
   getArgD $thisARGV -swy             objInfo($obj,swy)

   getArgD $thisARGV -leftMargin      objInfo($obj,leftMargin)
   getArgD $thisARGV -rightMargin     objInfo($obj,rightMargin)
   getArgD $thisARGV -bottomMargin    objInfo($obj,bottomMargin)
   getArgD $thisARGV -topMargin       objInfo($obj,topMargin)

   getArgD $thisARGV -bgColor         objInfo($obj,bgColor)
   getArgD $thisARGV -winColor        objInfo($obj,winColor)
   getArgD $thisARGV -gridColor       objInfo($obj,gridColor)
   getArgD $thisARGV -boxColor        objInfo($obj,fgColor)
   getArgD $thisARGV -axisColor       objInfo($obj,axisColor)
   getArgD $thisARGV -labelColor      objInfo($obj,labelColor)
   getArgD $thisARGV -shadowColor     objInfo($obj,shadowColor)

   getArgD $thisARGV -thresh          objInfo($obj,thresh)

   if {[flagLoc $thisARGV -above]}    {set objInfo($obj,threshMode) above}
   if {[flagLoc $thisARGV -below]}    {set objInfo($obj,threshMode) below}

   if {[flagLoc $thisARGV -square]}   {set objInfo($obj,squareFlag) 1}
   if {[flagLoc $thisARGV -nosquare]} {set objInfo($obj,squareFlag) 0}
   
   if {[flagLoc $thisARGV -grid]}     {set objInfo($obj,gridFlag) 1}
   if {[flagLoc $thisARGV -nogrid]}   {set objInfo($obj,gridFlag) 0}
   
   if {[flagLoc $thisARGV -axis]}     {set objInfo($obj,axisFlag) 1}
   if {[flagLoc $thisARGV -noaxis]}   {set objInfo($obj,axisFlag) 0}

   getListArgD $thisARGV -in          objInfo($obj,inList)
   getListArgD $thisARGV -colorList   objInfo($obj,colorList)

   return 0
}

proc setViewerCanvas { obj } \
{
   global objInfo 

   update

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,toplevel)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,toplevel)]

   $objInfo($obj,canvas) configure -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize)

   update

   set objInfo($obj,sxw) [expr $objInfo($obj,winXSize) - ($objInfo($obj,leftMargin) + $objInfo($obj,rightMargin))]
   set objInfo($obj,syw) [expr $objInfo($obj,winYSize) - ($objInfo($obj,topMargin) + $objInfo($obj,bottomMargin))]

   set objInfo($obj,cx1) $objInfo($obj,leftMargin)
   set objInfo($obj,cxn) [expr $objInfo($obj,cx1) + $objInfo($obj,sxw)] 
   set objInfo($obj,cy1) $objInfo($obj,topMargin)
   set objInfo($obj,cyn) [expr $objInfo($obj,cy1) + $objInfo($obj,syw)]

   $objInfo($obj,canvas) delete pixmap -tags $objInfo($obj,pixmap)

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) \
      -win $objInfo($obj,win) -tags $objInfo($obj,pixmap)

   defineCell -x $objInfo($obj,cx1) $objInfo($obj,cxn) -y $objInfo($obj,cy1) $objInfo($obj,cyn) -cell 1 -win $objInfo($obj,win)
 
   update

   bind $objInfo($obj,canvas) <Configure> "resizeViewer $obj %w %h %T"

   return 0
}

proc resizeViewer args \
{
   global objInfo

   set obj [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   setViewerCanvas   $obj
   drawViewerGraphic $obj
}

proc getViewerData { obj } \
{
   global objInfo 

   set objInfo($obj,roiList)     ""
   set objInfo($obj,vecList)     ""
   set objInfo($obj,vecSizeList) ""
   set objInfo($obj,inCount)     0

   foreach inName $objInfo($obj,inList) \
      {
       set thisName [format $inName 1 1]

       if {![file exists $thisName]} \
          {
           puts stderr "Viewer Error Finding Input File $inName ($thisName)"
           return 1
          }

       incr objInfo($obj,inCount)
      }

   if {[expr $objInfo($obj,inCount) % 2]} \
      {
       puts stderr "Viewer Error: -in Argument Should Have an Even Number of Spectra."
       return 1
      }

#
# N dimCount qSize xSize ySize zSize aSize"

   set pairCount [expr $objInfo($obj,inCount)/2]

   for {set roiID 1} {$roiID <= $objInfo($obj,inCount)} {incr roiID} \
      {
       set fullID [expr $roiID + $objInfo($obj,inCount) + 100]
       set inName [lindex $objInfo($obj,inList) [expr $roiID - 1]]

       set objInfo($obj,noise,$roiID)      [estSpecNoise $inName]
       set objInfo($obj,sizeInfo,$roiID)   [getSpecSizeInfo $inName]
       set objInfo($obj,dimCount,$roiID)   [lindex $objInfo($obj,sizeInfo,$roiID) 1]
       set objInfo($obj,title,$roiID)      None
       set objInfo($obj,rdata,$roiID)      0
       set objInfo($obj,fdata,$roiID)      0
       set objInfo($obj,vec,$roiID)        0
       set objInfo($obj,roiSize,$roiID)    0
       set objInfo($obj,vecSize,$roiID)    0
       set objInfo($obj,specSize,$roiID)   0

       if {$objInfo($obj,dimCount,$roiID) == 1} \
          {
           readROI -ndim 1 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN)

           readROI -ndim 1 -roi $fullID -in $inName \
                   -x X_AXIS 0% 100%
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 2} \
          {
           readROI -ndim 2 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN)

           readROI -ndim 2 -roi $fullID -in $inName \
                   -x X_AXIS 0% 100% -y Y_AXIS 0% 100%
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 3} \
          {
           readROI -ndim 3 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN) \
                   -z Z_AXIS $objInfo($obj,specZ1) $objInfo($obj,specZN)

           readROI -ndim 3 -roi $fullID -in $inName \
                   -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100%
          } \
       elseif {$objInfo($obj,dimCount,$roiID) == 4} \
          {
           readROI -ndim 4 -roi $roiID -in $inName \
                   -x X_AXIS $objInfo($obj,specX1) $objInfo($obj,specXN) \
                   -y Y_AXIS $objInfo($obj,specY1) $objInfo($obj,specYN) \
                   -z Z_AXIS $objInfo($obj,specZ1) $objInfo($obj,specZN) \
                   -a A_AXIS $objInfo($obj,specA1) $objInfo($obj,specAN)

           readROI -ndim 4 -roi $fullID -in $inName \
                   -x X_AXIS 0% 100% -y Y_AXIS 0% 100% -z Z_AXIS 0% 100% -a A_AXIS 0% 100% 
          }

       set rInfo [getROIInfo -roi $roiID]
       set n     [getROISize $roiID]

       set objInfo($obj,rdata,$roiID) [lindex $rInfo 0]
       set objInfo($obj,fdata,$roiID) [lindex $rInfo 1]
       set objInfo($obj,title,$roiID) [getParmStr $objInfo($obj,fdata,$roiID) FDTITLE NULL_DIM]

       set objInfo($obj,roiSize,$roiID)  $n
       set objInfo($obj,specSize,$roiID) [getROISize $fullID] 

       set objInfo($obj,vec,$roiID)       [sysMalloc $n float]
       set objInfo($obj,vecPaired,$roiID) [sysMalloc $n float]
       set objInfo($obj,vecSize,$roiID)   [getThreshData $obj $objInfo($obj,vec,$roiID) $objInfo($obj,rdata,$roiID) $n]

       if {![string length $objInfo($obj,title,$roiID)]} {set objInfo($obj,title,$roiID) None}

       lappend objInfo($obj,roiList) $roiID
      }

   set pair 1

   for {set roiID 1} {$roiID < $objInfo($obj,inCount)} {incr roiID 2} \
      {
       set roiNext  [expr $roiID + 1]

       set fullID   [expr $roiID   + $objInfo($obj,inCount) + 100]
       set fullNext [expr $roiNext + $objInfo($obj,inCount) + 100]

       if {$objInfo($obj,dimCount,$roiID) != $objInfo($obj,dimCount,$roiNext)} \
          {
           puts stderr "Viewer Warning: Dimension Count Mismatch:"
           puts stderr " $objInfo($obj,dimCount,$roiID)D [lindex $objInfo($obj,inList) [expr $roiID   - 1]]"
           puts stderr " $objInfo($obj,dimCount,$roiNext)D [lindex $objInfo($obj,inList) [expr $roiNext - 1]]"
          }

       if {$objInfo($obj,roiSize,$roiID) != $objInfo($obj,roiSize,$roiNext)} \
          {
           puts stderr "Viewer Warning: Size Mismatch:"
           puts stderr " $objInfo($obj,roiSize,$roiID) Points [lindex $objInfo($obj,inList) [expr $roiID   - 1]]"
           puts stderr " $objInfo($obj,roiSize,$roiNext) Points [lindex $objInfo($obj,inList) [expr $roiNext - 1]]"
          }

       if {$objInfo($obj,roiSize,$roiID) > $objInfo($obj,roiSize,$roiNext)} \
          {
           set objInfo($obj,pairSize,$pair) $objInfo($obj,roiSize,$roiNext)
          } \
       else \
          {
           set objInfo($obj,pairSize,$pair) $objInfo($obj,roiSize,$roiID)
          }

       set objInfo($obj,difVec,$pair) [sysMalloc $objInfo($obj,pairSize,$pair) float]

       set vecA  [lindex [getROIInfo -roi $fullID]   0]
       set vecB  [lindex [getROIInfo -roi $fullNext] 0]

       if {$objInfo($obj,specSize,$roiID) > $objInfo($obj,specSize,$roiNext)} \
          {
           set n $objInfo($obj,specSize,$roiNext)
          } \
       else \
          {
           set n $objInfo($obj,specSize,$roiNext)
          }

       vvSub $vecA $vecB $n

       set objInfo($obj,pairNoise,$pair) [vEstNoise $vecA $n]

       set difVec  $objInfo($obj,difVec,$pair)
       set rdataA  $objInfo($obj,rdata,$roiID)
       set rdataB  $objInfo($obj,rdata,$roiNext)
       set vecA    $objInfo($obj,vecPaired,$roiID)
       set vecB    $objInfo($obj,vecPaired,$roiNext)
       set n       $objInfo($obj,pairSize,$pair)

       set objInfo($obj,difSize,$pair) [getThreshDif $obj $difVec $vecA $vecB $rdataA $rdataB $n]

       lappend objInfo($obj,vecList) $vecA
       lappend objInfo($obj,vecList) $vecB

       lappend objInfo($obj,vecSizeList) $objInfo($obj,difSize,$pair)
       lappend objInfo($obj,vecSizeList) $objInfo($obj,difSize,$pair)

       incr pair
      }

   return 0
}

proc showDataStats { obj } \
{
   global objInfo

   set usedList ""

   for {set roiID 1} {$roiID <= $objInfo($obj,inCount)} {incr roiID} \
      {
       set inName [lindex $objInfo($obj,inList) [expr $roiID - 1]]
       if {[lsearch -exact $usedList $inName] >= 0} continue

       lappend usedList $inName

       if {![strcasecmp $objInfo($obj,title,$roiID) None]} \
          {
           set str "$inName"
          } \
       else \
          {
           set str "$inName = $objInfo($obj,title,$roiID)"
          }

       set n $objInfo($obj,vecSize,$roiID)

       puts [format "%s %s Quad %s" $str [lrange $objInfo($obj,sizeInfo,$roiID) 3 end] [lindex $objInfo($obj,sizeInfo,$roiID) 2]]
       puts [format "%s N:        %11d"  $str $objInfo($obj,roiSize,$roiID)]
       puts [format "%s Selected: %11d"  $str $n]
       puts [format "%s Min:      % .4e" $str [vMin      $objInfo($obj,vec,$roiID) $n]]
       puts [format "%s Max:      % .4e" $str [vMax      $objInfo($obj,vec,$roiID) $n]]
       puts [format "%s Average:  % .4e" $str [vAvg      $objInfo($obj,vec,$roiID) $n]]
       puts [format "%s RMS:      % .4e" $str [vRMS      $objInfo($obj,vec,$roiID) $n]]
       puts [format "%s Median    % .4e" $str [vMedian   $objInfo($obj,vec,$roiID) $n]]
       puts [format "%s Noise:    % .4e" $str $objInfo($obj,noise,$roiID)]

       puts ""
      }

   set pair 1

   for {set roiID 1} {$roiID < $objInfo($obj,inCount)} {incr roiID 2} \
      {
       if {$roiID > 1} {append objInfo($obj,vsTitle) ","}

       set roiNext [expr $roiID + 1]

       set inNameA [lindex $objInfo($obj,inList) [expr $roiID   - 1]]
       set inNameB [lindex $objInfo($obj,inList) [expr $roiNext - 1]]

       set vecA    $objInfo($obj,vecPaired,$roiID)
       set vecB    $objInfo($obj,vecPaired,$roiNext)
       set n       $objInfo($obj,difSize,$pair)

       if {![strcasecmp $objInfo($obj,title,$roiID) None] || ![strcasecmp $objInfo($obj,title,$roiNext) None]} \
          {
           set str " $inNameA vs $inNameB"
           append objInfo($obj,vsTitle) $str
          } \
       else \
          {
           set str " $objInfo($obj,title,$roiID) vs $objInfo($obj,title,$roiNext)"
           append objInfo($obj,vsTitle) $str
          } 

       set sumSq  [vSumSq $objInfo($obj,difVec,$pair) $n]
       set rms    [vRMS   $objInfo($obj,difVec,$pair) $n]
       set rP     [vvPearsonR $vecA $vecB $n]

       puts [format "%s N:          %11d"   $str $n]
       puts [format "%s Min:        % .4e"  $str [vMin    $objInfo($obj,difVec,$pair) $n]]
       puts [format "%s Max:        % .4e"  $str [vMax    $objInfo($obj,difVec,$pair) $n]]
       puts [format "%s Average:    % .4e"  $str [vAvg    $objInfo($obj,difVec,$pair) $n]]
       puts [format "%s SumSq:      % .4e"  $str $sumSq]
       puts [format "%s RMS:        % .4e"  $str $rms]
       puts [format "%s Median:     % .4e"  $str [vMedian $objInfo($obj,difVec,$pair) $n]]
       puts [format "%s Noise:      % .4e"  $str $objInfo($obj,pairNoise,$pair)]
       puts [format "%s RMS/Noise:  %11.4f" $str [safeDiv $rms $objInfo($obj,pairNoise,$pair)]]
#      puts [format "%s Chi2P:      %11.4f" $str [chi2Test $sumSq $objInfo($obj,pairNoise,$pair) $n]]
       puts [format "%s PearsonR:   %11.4f" $str $rP]

       append objInfo($obj,vsTitle) " R: [format %.3f $rP]"

       puts ""

       incr pair
      }
}

proc getOtherArgs { obj } \
{
   global objInfo ARGV

   set thisARGV $ARGV

   set objInfo($obj,pntSize) 3
   set objInfo($obj,hMin)    Auto
   set objInfo($obj,hMax)    Auto
   set objInfo($obj,vMin)    Auto
   set objInfo($obj,vMax)    Auto

   getArgD $thisARGV -pntSize objInfo($obj,pntSize)

   getArgD $thisARGV -hMin objInfo($obj,hMin)
   getArgD $thisARGV -hMax objInfo($obj,hMax)

   getArgD $thisARGV -vMin objInfo($obj,vMin)
   getArgD $thisARGV -vMax objInfo($obj,vMax)

   getArgD $thisARGV -x1 objInfo($obj,specX1)
   getArgD $thisARGV -xn objInfo($obj,specXN)

   getArgD $thisARGV -y1 objInfo($obj,specY1)
   getArgD $thisARGV -yn objInfo($obj,specYN)

   getArgD $thisARGV -z1 objInfo($obj,specZ1)
   getArgD $thisARGV -zn objInfo($obj,specZN)

   getArgD $thisARGV -a1 objInfo($obj,specA1)
   getArgD $thisARGV -an objInfo($obj,specAN)

   return 0
}

proc drawViewerGraphic { obj } \
{
   global objInfo

   if {$objInfo($obj,axisFlag)} \
      {
       set axisStr "-axis Bottom Left -colorA $objInfo($obj,axisColor) -colorS $objInfo($obj,shadowColor)"
      } \
   else \
      {
       set axisStr ""
      }

   if {$objInfo($obj,gridFlag)} \
      {
       set gridStr "-xGrid -yGrid -colorG $objInfo($obj,gridColor)"
      } \
   else \
      {
       set gridStr "-noxGrid -noyGrid"
      }

   drawRec -fg $objInfo($obj,winColor) -x 0 $objInfo($obj,winXSize) -y 0 $objInfo($obj,winYSize) -win $objInfo($obj,win)

   set sInfo [eval drawROIScatter \
                    -pntSize $objInfo($obj,pntSize) -cell 1 \
                    -roiList $objInfo($obj,roiList) -vec $objInfo($obj,vecList) -n $objInfo($obj,vecSizeList) \
                    -colorList $objInfo($obj,colorList) -bg $objInfo($obj,bgColor) -fg $objInfo($obj,fgColor) \
                    -hMin $objInfo($obj,hMin) -hMax $objInfo($obj,hMax) -vMin $objInfo($obj,vMin) -vMax $objInfo($obj,vMax) \
                    $axisStr $gridStr -scat]

   if {[llength $objInfo($obj,roiList)] <= 6} \
      {
       drawText -x $objInfo($obj,cx1) -y [expr $objInfo($obj,cy1) - 4] -bg $objInfo($obj,bgColor) -fg $objInfo($obj,labelColor) -text $objInfo($obj,vsTitle)
      }
}

proc getThreshData { obj dest src n } \
{
   global objInfo

   if {![strcasecmp $objInfo($obj,thresh) None]} \
      {
       vvMove $dest $src $n
       return $n
      }

   set j 0

   if {![strcasecmp $objInfo($obj,threshMode) above]} \
      {
       for {set i 0} {$i < $n} {incr i} \
          {
           set v [getPtrVal $src $i]

           if {$v > $objInfo($obj,thresh) || $v < -$objInfo($obj,thresh)} \
              {
               setPtrVal $dest $j $v
               incr j
              }
          } 
      } \
   else \
      {
       for {set i 0} {$i < $n} {incr i} \
          {
           set v [getPtrVal $src $i]

           if {$v <= $objInfo($obj,thresh) && $v >= -$objInfo($obj,thresh)} \
              {
               setPtrVal $dest $j $v
               incr j
              }
          }
      }
    
   return $j 
}

proc getThreshDif { obj destDif destA destB srcA srcB n } \
{
   global objInfo

   if {![strcasecmp $objInfo($obj,thresh) None]} \
      {
       vvSub $destDif $srcA $srcB $n
       vvMove $destA $srcA $n
       vvMove $destB $srcB $n
       return $n
      }

   set j 0

   if {![strcasecmp $objInfo($obj,threshMode) above]} \
      {
       for {set i 0} {$i < $n} {incr i} \
          {
           set vA [getPtrVal $srcA $i]
           set vB [getPtrVal $srcB $i]

           if {($vA > $objInfo($obj,thresh) || $vA < -$objInfo($obj,thresh)) || \
                ($vB > $objInfo($obj,thresh) || $vB < -$objInfo($obj,thresh))} \
              {
               setPtrVal $destDif $j [expr $vA - $vB] 
               setPtrVal $destA   $j $vA
               setPtrVal $destB   $j $vB 
               incr j
              }
          }
      } \
   else \
      {
       for {set i 0} {$i < $n} {incr i} \
          {
           set vA [getPtrVal $srcA $i]
           set vB [getPtrVal $srcB $i]

           if {($vA <= $objInfo($obj,thresh) && $vA >= -$objInfo($obj,thresh)) || \
                ($vB <= $objInfo($obj,thresh) && $vB >= -$objInfo($obj,thresh))} \
              {
               setPtrVal $destDif $j [expr $vA - $vB]
               setPtrVal $destA   $j $vA
               setPtrVal $destB   $j $vB 
               incr j
              }
          }
      }

   return $j
}

proc saveViewerSnapshot { obj } \
{
   global ARGC ARGV objInfo

   if {![flagLoc $ARGV -snap]} return

   if {![file exists snap]} {catch "file mkdir snap"}

   update

   set im [image create photo]

   winop snap $objInfo($obj,canvas) $im

   set n [llength [glob -nocomplain snap/*.gif]]

   set objInfo($obj,imageID) 0
 
   for {set i 1} {$i <= $n + 1} {incr i} \
      {
       set outName [format snap/im%03d.gif $i]
       if {[file exists $outName]} continue

       set objInfo($obj,imageID) $i
       break
      }

   set outName [format snap/im%03d.gif $objInfo($obj,imageID)]

   $im write -format gif $outName

   puts ""
   puts "Saved Image $outName"

   image delete $im

   if {[flagLoc $ARGV -exit]} {exit 0}   
}

#
#-----------------------------------------------------------------------

set obj specVsSpec1

if {[flagLoc $ARGV -help]} \
   {
    puts "Spectrum Vs Spectrum Scatterplots:"
    puts " -in        inList \[None\]     List of Input Spectrum Pairs."
    puts " -hMin      hMin   \[Auto\]     Minimum Value to Plot, X-Axis."
    puts " -hMax      hMax   \[Auto\]     Maximum Value to Plot, X-Axis."
    puts " -vMin      vMin   \[Auto\]     Minimum Value to Plot, Y-Axis."
    puts " -vMax      vMax   \[Auto\]     Maximum Value to Plot, Y-Axis."
    puts " -colorList cList  \[blue ...\] Colors for Scatter Plot Points."
    puts " -pntSize   pSize  \[3\]        Scatter Plot Point Size, Pixels."
    puts " -snap                        Save A Screen Capture."
    puts " -exit                        Exit After Screen Capture."
    puts "Restricting Analysis via Threshold:"
    puts " -thresh    t      \[None\]     Threshold Value." 
    puts " -above                       Use Points Above Threshold."
    puts " -below                       Use Points Equal or Below Threshold."
    puts "Spectral Region (-x -y -z -a):"
    puts " -x1        specX1   \[0%\]     Spectral Region Start."
    puts " -xn        specXN   \[100%\]   Spectral Region End."
    showViewerHelp $obj
    exit 0
   }

if {[getViewerArgs $obj]} {exit 1}
if {[getOtherArgs  $obj]} {exit 1}

if {[getViewerData $obj]} {exit 1}

if {$objInfo($obj,squareFlag)} \
   {
    if {$objInfo($obj,sxw) > $objInfo($obj,syw)} \
       {
        set objInfo($obj,sxw) $objInfo($obj,syw)
       } \
    else \
       {
        set objInfo($obj,syw) $objInfo($obj,sxw)
       }
   }

set objInfo($obj,winXSize) [expr $objInfo($obj,sxw) + $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]
set objInfo($obj,winYSize) [expr $objInfo($obj,syw) + $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

set objInfo($obj,vsTitle)  ""

canvas $objInfo($obj,canvas) -width $objInfo($obj,winXSize) -height $objInfo($obj,winYSize) -border 0
pack   $objInfo($obj,canvas) -side top -expand 1 -fill both

update

setViewerCanvas    $obj
showDataStats      $obj
drawViewerGraphic  $obj
saveViewerSnapshot $obj

