
#
# Return 1 if size is an integer power of two.

proc isPower2 { size } \
{
   if {$size < 0} {return 0}

   for {set n 1} {$size <= $n} {set n [expr 2*$n]} \
      {
       if {$n == $size} {return 1}
      }

    return 0
}

#
# Event handler for reading SPIFF parameter header and building
# an appropriate command script.

proc readParams_SPIFF {} \
{
   global inName inPath outName auxName outAuxName
   global progName specialArgs spectralType hdrSize pcFlag
   global Nz

   set hdrSize 1020
   setDimCount 3
   clearScript

   set progName    bin2pipe

   set outAuxPath "[file rootname [file tail $auxName]]"
   set outAuxRoot "[string tolower $spectralType]%03d.dat"

   if {[string compare $inName None]} \
      {
       getSPIFF3D $inName
      }

   if {$pcFlag} \
      {
       set specialArgs " -noswap -bo $hdrSize"
      } \
   else \
      {
       set specialArgs " -swap -bo $hdrSize"
      }

   if {[string compare $auxName None]} \
      {
       getSPIFF3D $auxName
      }

   showPoorEntries {LASTx LASTy FIRSTz LASTz}

   if {![isPower2 $Nz]} {showPoorEntries Nz}
}

#
# getSPIFF3D: get SPIFF acquistion parameters.

proc getSPIFF3D { spiffName } \
{
   global Nx Tx MODEx SWx FIRSTx LASTx OBSx CARx FTx LABx \
          Ny Ty MODEy SWy FIRSTy LASTy OBSy CARy FTy LABy \
          Nz Tz MODEz SWz FIRSTz LASTz OBSz CARz FTz LABz \
          Na Ta MODEa SWa FIRSTa LASTa OBSa CARa FTa LABa

   global aq2D dataSource hdrSize
   global osType pcFlag exeExt

   setOSType

   set dataSource    [getDataSource $spiffName]

   set spfHdr(error)      0
   set spfHdr(xsize)      0
   set spfHdr(ysize)      0
   set spfHdr(imag_size)  0
   set spfHdr(num_points) 0
   set spfHdr(first_wave) 0
   set spfHdr(last_wave)  0

   set fileBytes [getFileBytes $spiffName -estimate]

   if {[file exists spfHdr.out]} \
      {
       set error [catch "file delete spfHdr.out"]
      }

   switch $dataSource \
   {
    FILE \
      {
       catch "exec showSpiff$exeExt -var spfHdr -in $spiffName -out spfHdr.out"
      }
    ZCAT \
      {
       catch \
        "exec cat$exeExt $spiffName|zcat$exeExt|showSpiff$exeExt -var spfHdr -out spfHdr.out"
      }
     GZCAT \
       {
        catch \
         "exec gzcat$exeExt $spiffName|showSpiff$exeExt -var spfHdr -out spfHdr.out"
       }
     default \
       { 
        catch "exec showSpiff$exeExt -var spfHdr -in $spiffName -out spfHdr.out"
       }
   }

   set error [catch "source spfHdr.out"]

   if {$spfHdr(error) != 0} \
      {
       set txt "Error reading SPIFF header."
       tk_dialog .errDialog Error $txt error 0 Continue
       return $error
      }

   if {$spfHdr(identity) > 18289} \
      {
       set hdrSize 1020

       if {$spfHdr(identity) > 18290} {set hdrSize 1024}

       set Nx    $spfHdr(xsize)
       set Tx    $spfHdr(xsize)

       set Ny    $spfHdr(ysize)
       set Ty    $spfHdr(ysize)

       set Nz    $spfHdr(num_points) 
       set Tz    $spfHdr(num_points) 

       set extra [expr $fileBytes - $Nx*$Ny*$Nz*4]

       if {$extra == 1024} {set hdrSize 1024}
      } \
   else \
      {
       set hdrSize 128

       if {$spfHdr(imag_size) == 0} {set spfHdr(imag_size) 128}

       set Nx    $spfHdr(imag_size)
       set Tx    $spfHdr(imag_size)

       set Ny    $spfHdr(imag_size)
       set Ty    $spfHdr(imag_size)

       set Nz    $spfHdr(num_points)
       set Tz    $spfHdr(num_points)
      }

   if {$spfHdr(first_wave) > 1000.0 && $spfHdr(last_wave) < 10000.0} \
      {
       set FIRSTz  $spfHdr(last_wave)
       set LASTz   $spfHdr(first_wave)
      }

   set aq2D   Image

   set MODEx  Real
   set OBSx   1.0 
   set CARx   0.0
   set FTx    Freq
   set LABx   X 

   set MODEy  Real
   set OBSy   1.0
   set CARy   0.0
   set FTy    Freq
   set LABy   Y

   set MODEz Real 
   set OBSz  1.00
   set CARz  2962.5;
   set FTz   Time 
   set LABz  IR 
    
   return 0
}

#
# Bottom.
