#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"

setOSType

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set error     0
set inName    ""
set outName   ""
set maxBytes  1048576
set inBytes   0 
set execFlag  0

set inUnit    stdin
set outUnit   stdout
 
#
# Print help text if needed:

if {$ARGC == 1 || [flagLoc $ARGV "-HELP"]} \
   {
    puts stderr "$argv0: create a command to split a binary file into pieces:"
    puts stderr " -in   inName    \[stdin\]   Name of Input."
    puts stderr " -out  outName   \[stdout\]  Base Name of Output File."
    puts stderr " -bc   byteCount \[0\]       Total Bytes in Input."
    puts stderr "                            Use 0 For Auto-Mode."
    puts stderr " -mb   maxBytes  \[1048576\] Maximum Bytes Per Output File."
    puts stderr " -exec                       Execute the Split Commands."
    return 1
   }

#
# Extract the arguments;

getArgD $ARGV -in  inName
getArgD $ARGV -out outName
getArgD $ARGV -bc  inBytes
getArgD $ARGV -mb  maxBytes

if {[flagLoc $ARGV -exec]} {set execFlag 1}

if {![flagLoc $ARGV -in]} \
   {
    puts stderr "$argv0 Error: missing -in argument."
    return 1
   }

if {![flagLoc $ARGV -out]} \
   {
    puts stderr "$argv0 Error: missing -out argument."
    return 1
   }

#
# Find the input size if needed.

if {!$inBytes} \
   {
    set error [catch {set inBytes [file size $inName]}]

    if {$error} \
       {
        puts stderr "$argv0 Error finding size of input file $inName"
        return 1
       }
   }

#
# Create the I/O commands:

set n      1
set offset 0

for {set bytesLeft $inBytes} {$bytesLeft > 0} {incr bytesLeft -$bytesOut} \
   {
    if {$bytesLeft > $maxBytes} \
       {
        set bytesOut $maxBytes
       } \
    else \
       {
        set bytesOut $bytesLeft
       }

    set thisName [format "$outName.%03d" $n]

    puts stderr "byteAdjust$exeExt -in $inName -out $thisName \\"
    puts stderr "  -iws 1 -ows 1 -bo $offset -wc $bytesOut"

    if {$execFlag} \
       {
        exec byteAdjust$exeExt -in $inName -out $thisName \
             -iws 1 -ows 1 -bo $offset -wc $bytesOut
       }

    incr offset $bytesOut
    incr n
   }
