#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

#
# Split PDB according to segments.

set inName    ref.pdb
set outName   %s.pdb
set parmName  SEGID 
set nullName  NULL

if {[flagLoc $ARGV -help]} \
   {
    puts "Split PDB File According to SEGID etc:"
    puts " -in   pdbName     \[ref.pdb\]   Input PDB File."
    puts " -out  outTemplate \[%s.pdb\]    Output PDB Template."
    puts " -parm parmName  \[SEGID\]       LOCID CHAINID ICODE SEGID."
    puts " -null nullName  \[NULL\]        Value to Use for Empty SEGID."
    puts "Notes:"
    puts " Output files will be named according to parameter value." 
    exit 1
   }

getArgD $ARGV -in   inName
getArgD $ARGV -out  outName
getArgD $ARGV -parm parmName
getArgD $ARGV -null nullName

#
#----------------------------------------------------------------------------

if {![file exists $inName]} \
   {
    puts "Error Finding PDB Input $inName"
    exit 1
   }

set pdbTab [gdbRead table -in $inName -pdb]
set eList  [gdbGetInfo $pdbTab -parms entries]

set segList   ""
set nullCount 0

foreach e $eList \
   {
    set key     ""
    set segName [gdbGet $e $parmName]
    set status  [scan $segName %s key]

    if {![strcasecmp $segName "(Null)"]} {set segName $nullName}
    if {![strcasecmp $segName "{}"]}     {set segName $nullName}

    if {$status < 0} {set segName $nullName}

    if {![strcasecmp $segName $nullName]} \
       {
        incr nullCount
        continue
       }

    if {[lsearch -exact $segList $segName] < 0} \
       {
        lappend segList $segName
       }
   }

foreach segName $segList \
   {
    set c        "!strcasecmp( $parmName, '$segName' )"
    set thisName [format $outName $segName]
    set thisTab  [gdbSelect -from $pdbTab -new -cond $c]

    gdbWrite $thisTab $thisName -pdb
    gdbFree  $thisTab

    puts $thisName
   }

if {$nullCount} \
   {
    set c        "\[lsearch -exact $segList $parmName\] < 0"
    set thisName [format $outName $nullName]
    set thisTab  [gdbSelect -tcl -from $pdbTab -new -cond $c]

    gdbWrite $thisTab $thisName -pdb
    gdbFree  $thisTab

    puts $thisName
   }

exit
