#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

proc stdInitVars {} \
{
   global objInfo env

   set obj std

   if {[file exists $env(NMRBASE)/std/custom]} \
      {
       set objInfo($obj,comDir) $env(NMRBASE)/std/custom
      } \
   else \
      {
       set objInfo($obj,comDir) $env(NMRBASE)/std/com
      }

   set objInfo($obj,maskInfo) "None"
   set objInfo($obj,convInfo) "None"
   set objInfo($obj,winWidth) "0.002ppm"
   set objInfo($obj,dirList)  ""
   set objInfo($obj,expList)  ""
   set objInfo($obj,calcArgs) ""
}

proc stdInitArgs {} \
{
   global objInfo ARGC ARGV

   set obj std

   if {[flagLoc $ARGV -help]} \
      {
       puts "Auto-Process 1D and 1Dx2 STD/WG data:"
       puts " -in   dirList  \[*\]                Data Directories to Scan." 
       puts " -exp  expList  \[*\]                Experiments to Process." 
       puts " -com  comDir   \[NMRBASE/std/com\]  Location of STD Scripts."
       puts " -w    sW       \[0.002ppm\]         STD Scaling Window +/- Width."
       puts " -mask mInfo    \[None\]             Mask List Coord Pairs."
       puts " -conv convArgs \[None\]             Arguments for Conversion (Followed by --)."
       puts " -calc calcArgs \[None\]             Arguments for stdCalc.tcl"
       puts "Notes:"
       puts " 1. The default value for for STD '-com comDir' will be set to"
       puts "    NMRBASE/std/custom if that directory exists."
       puts " 2. Mask list is in pairs of coords for regions to exclude."
       puts "    Example: 0% 5%   47% 53%   4.1ppm 3.1ppm   95% 100%"
       puts " 3. Any optional arguments -conv for conversion must be followed"
       puts "    by -- (double dashes)."
       puts " 4. Any optional arguments -calc for stdCalc.tcl must be the last"
       puts "    options specified on the command line."
       puts ""
       puts "Example:"
       puts " stdCalc.tcl -w 0.02ppm -conv -xCAR 5.00 -- -calc -ax1 9.0ppm -axn 6.0ppm"
       exit 0
      }

   set ARGV [getListArgDT $ARGV -conv objInfo($obj,convInfo)]
   set ARGC [llength $ARGV]

   set loc [flagLoc $ARGV -calc]

   if {$loc} \
      {
       set objInfo($obj,calcArgs) [lrange $ARGV [expr $loc+1] end]
       set ARGV [lrange $ARGV 0 [expr $loc-1]]
       set ARGC [llength $ARGV]
      }

   getListArgD  $ARGV -in   objInfo($obj,dirList)
   getListArgD  $ARGV -exp  objInfo($obj,expList)
   getListArgD  $ARGV -mask objInfo($obj,maskInfo)

   getArgD     $ARGV -com objInfo($obj,comDir)
   getArgD     $ARGV -w   objInfo($obj,winWidth)
}

proc getStdFileExp { fName } \
{
   if {![file exists $fName]} \
      {
       return "0 0 0 none"
      }

   set error [rdFDATA $fName fdata]

   set eName    [getParmStr $fdata FDCOMMENT  NULL_DIM]
   set dimCount [getParmI   $fdata FDDIMCOUNT NULL_DIM]

   set xSize    [getParmI   $fdata NDSIZE CUR_XDIM]
   set ySize    [getParmI   $fdata NDSIZE CUR_YDIM]

   if {![llength $eName]} {set eName None}

   sysFree $fdata

   return "$dimCount $xSize $ySize $eName"
}

proc getStdArgExp { convArgs } \
{
   set eStr  None
   set eName ""

   set dimCount 0

   set xSize 1
   set ySize 1

   getListArgD $convArgs -comment eStr

   getArgD $convArgs -ndim dimCount
   getArgD $convArgs -xT   xSize
   getArgD $convArgs -yN   ySize 

   if {[strcasecmp $eStr None]} \
      {
       set eName [lindex $eStr 1]
      }

   if {![llength $eName]} {set eName None} 

   return "$dimCount $xSize $ySize $eName"
}

proc stdDirScan {} \
{
   global objInfo env

   set obj std

   set objInfo($obj,rawList)  ""
   set objInfo($obj,fidList)  ""
   set objInfo($obj,ftList)   ""
   set objInfo($obj,skipList) ""

   if {[llength $objInfo($obj,dirList)]} \
      {
       set fList $objInfo($obj,dirList) 
      } \
   else \
      {
       set fList [lsort -dictionary [glob *]]
      }
 
   foreach f $fList \
      {
       if {![file isdirectory $f]}  continue

       set inType None
       set bList  ""

       if {[file exists $f/acqus] && ([file exists $f/pulseprogram] || [file exists $f/pulseprogram.precomp])} \
          {
           set inType BRUKER_NIH
           set bList {fid ser fid.Z ser.Z fid.gz ser.gz}
          } \
       elseif {[file exists $f/procpar]} \
          {
           set inType VARIAN
           set bList {fid fid.Z fid.gz}
          } \
       elseif {[llength [glob -nocomplain $f/*.jdf $f/*.JDF]] != 0} \
          {
           set inType JEOL_DELTA
           set bList  [lsort -dictionary [glob -nocomplain $f/*.jdf $f/*.JDF]]
          } \
       else \
          {
           lappend objInfo($obj,skipList) "$f No supported parameter files."
           continue
          }
     
       puts stderr "Scanning Directory $f"

       set n 0

       foreach b $bList \
          {
           if {[file isdirectory $f/$b]} continue
           if {[file exists $f/$b]} {incr n}
          }

       if {$n == 0} \
          {
           lappend objInfo($obj,skipList) "$f No ser or fid File."
           continue
          }

       if {$n > 1} \
          {
           lappend objInfo($obj,skipList) "$f Multiple ser or fid Files."
           continue
          }

       set specName None 

       foreach b {test.ft1 test.ft2 test.dat test001.ft2 test001.ft3} \
          {
           if {[file exists $f/$b]}    {set specName $b}
           if {[file exists $f/ft/$b]} {set specName ft/$b}
          }

       if {[strcasecmp $specName None]} \
          {
           set eInfo [getStdFileExp $f/$specName]

           lappend objInfo($obj,ftList) "$f $eInfo"
           continue
          }

       set fidName None 

       foreach b {test.fid test001.fid} \
          {
           if {[file exists $f/$b]}      {set fidName $b}
           if {[file exists $f/data/$b]} {set fidName data/$b}
           if {[file exists $f/fid/$b]}  {set fidName fid/$b}
          }

       if {[strcasecmp $fidName None]} \
          {
           set eInfo [getStdFileExp $f/$fidName]

           lappend objInfo($obj,fidList) "$f $eInfo"
           continue
          }

       set convRequest ""

       append convRequest "nmrWish $env(TCLPATH)/conv.tcl"
       append convRequest " -- -inType $inType -nodmx -autoDim -info EXP"
       append convRequest " -script stdout -auto -noverb -nosleep -notk -cd $f"

       if {[strcasecmp $objInfo($obj,convInfo) None]} {append convRequest " -user $objInfo($obj,convInfo)"}

       set objInfo($obj,$f,convArgs) ""
     
       set error \
        [catch {set objInfo($obj,$f,convArgs) [eval "exec $convRequest"]}]

       set error 0

       if {$error} \
          {
           lappend objInfo($obj,skipList) "$f Error"
           continue
          }

       set eInfo [getStdArgExp $objInfo($obj,$f,convArgs)]
       set eName [lindex $eInfo end]

       if {[llength $objInfo($obj,expList)] && \
           [lsearch -exact $objInfo($obj,expList) $eName] < 0} \
          {
           lappend objInfo($obj,skipList) \
            "$f $eName is not a listed experiment."

           continue
          }

       lappend objInfo($obj,rawList) "$f $eInfo"
      }
}

proc stdConvert {} \
{
   global objInfo

   set obj std

   foreach eInfo $objInfo($obj,rawList) \
      {
       set f        [lindex $eInfo 0]
       set dimCount [lindex $eInfo 1]
       set xSize    [lindex $eInfo 2]
       set ySize    [lindex $eInfo 3]
       set eName    [lindex $eInfo 4]

       set fmt "Converting: %s X-Size: %5d Y-Size: %d Type: %s"

       puts [format $fmt $f $xSize $ySize $eName]

       setListArgD objInfo($obj,$f,convArgs) -comment $eName

       setArgD objInfo($obj,$f,convArgs) -xLAB 1H

       if {$dimCount == 2 && $ySize == 2} \
          {
           setArgD objInfo($obj,$f,convArgs) -yT    2
           setArgD objInfo($obj,$f,convArgs) -yMODE Real
           setArgD objInfo($obj,$f,convArgs) -yLAB  STDWG
           setArgD objInfo($obj,$f,convArgs) -aq2D  Real
          }

       set scriptName $f/fid.com

       set error [catch {set outUnit [open $scriptName w 0744]}]

       if {$error} \
          {
           puts stderr "Error $error Opening Conversion script $scriptName"
           continue
          }

       puts $outUnit "#!/bin/csh -f"
       puts $outUnit ""
       puts $outUnit "if (\$#argv == 1) then"
       puts $outUnit "   cd \$1"
       puts $outUnit "endif"
       puts $outUnit ""
       puts $outUnit "$objInfo($obj,$f,convArgs) >& fid.log"

       close $outUnit

       catch "file delete $f/test.fid"

       set error [catch "exec $scriptName [file normalize $f]"]

       if {![stdFileOk $f/test.fid]} \
          {
           puts stderr "Error Executing Conversion script $scriptName $f"
           continue
          }
      }
}

proc stdProcess {} \
{
   global objInfo

   set obj std

   puts "Using Processing Scripts in $objInfo($obj,comDir)"

   foreach eInfo [concat $objInfo($obj,rawList) $objInfo($obj,fidList)] \
      {
       set f        [lindex $eInfo 0]
       set dimCount [lindex $eInfo 1]
       set eName    [lindex $eInfo 4]

       puts [format "Processing: %s Type: %s" $f $eName]

       set scriptName $objInfo($obj,comDir)/$eName.com

       if {![file exists $scriptName]} \
          {
           set scriptName $objInfo($obj,comDir)/ft.com
          }

       catch "file delete $f/test.dat"

       puts [format "Command:    $scriptName $f $objInfo($obj,comDir)"]

       set error [catch "exec $scriptName $f $objInfo($obj,comDir)"]

       if {![stdFileOk $f/test.dat]} \
          {
           puts stderr "Error Executing processing script $scriptName $f"
           continue
          }
      }
}

proc stdScale {} \
{
   global objInfo

   set obj std

  
   foreach eInfo [concat $objInfo($obj,rawList) $objInfo($obj,fidList)] \
      {
       set f        [lindex $eInfo 0]
       set dimCount [lindex $eInfo 1]
       set xSize    [lindex $eInfo 2]
       set ySize    [lindex $eInfo 3]
       set eName    [lindex $eInfo 4]

       if {$ySize != 2} continue

       puts [format "Finding On/Off Difference: %s Type: %s" $f $eName]

       catch "file delete $f/on.dat $f/off.dat $f/w.dat $f/scaled.dat"

       set scriptName $objInfo($obj,comDir)/stdDif.com
       set difCmnd    "$scriptName $f $objInfo($obj,maskInfo)"

       puts [format "Difference Command:        $difCmnd"]

       set error [catch {eval "exec $difCmnd"}]

       if {![stdFileOk $f/on.dat] || ![stdFileOk $f/off.dat]} \
          {
           puts stderr "Error Executing STD Difference command: $difCmnd"
           continue
          }

       puts stderr "" 
       puts stderr "stdScale.tcl -in1 $f/dif.dat -in2 $f/off.dat -mask $f/mask.dat -out $f/w.dat -apply $f/scaled.dat -noverb -window $objInfo($obj,winWidth) -step 1 -def 1.0 -wMin 0.0 -wMax 1.0 -ndim 1"

       exec stdScale.tcl -in1 $f/dif.dat -in2 $f/off.dat -mask $f/mask.dat \
                         -out $f/w.dat -apply $f/scaled.dat -noverb \
                         -window $objInfo($obj,winWidth) -step 1 -def 1.0 \
                         -wMin 0.0 -wMax 1.0 -ndim 1

       if {![stdFileOk $f/w.dat] || ![stdFileOk $f/scaled.dat]} \
          {
           puts stderr "Error Executing STD Scaling."
           continue
          }

       set cmnd "stdCalc.tcl $objInfo($obj,calcArgs) -sx $objInfo($obj,winWidth) -dir $f"

       puts stderr "" 
       puts stderr $cmnd
       puts stderr "" 

       set pctInfo "" 

       catch {set pctInfo [eval "exec $cmnd"]}
       puts $pctInfo
      }
}

proc stdFileOk { f } \
{
#  puts "Testing file $f from [pwd]"

   if {![file exists $f]} \
      {
#      puts " File not found."
       return 0
      } 

   if {[file size $f] <= 2048} \
      {
#      puts " File too small."
       return 0
      }

   return 1
}

stdInitVars
stdInitArgs
stdDirScan
stdConvert
stdProcess
stdScale

exit

