#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts stderr "Find Scale Factor f to Minimize Spectrum1 - f*Spectrum2:"
    puts stderr " -in1  inName1  \[A.dat\]   Input Spectrum 1."
    puts stderr " -in2  inName2  \[B.dat\]   Input Spectrum 2." 
    puts stderr " -mask maskName \[None\]    Mask Spectrum."
    puts stderr " -ndim dimCount \[Auto\]    Dimension Count."
    puts stderr " -verb                      Verbose Mode ON (Default)."
    puts stderr " -noverb                    Verbose Mode OFF."
    puts stderr "Multiple Region Options:"
    puts stderr " -window        \[100%\]    +/- Width of Region." 
    puts stderr " -step          \[100%\]    Step Between Regions."
    puts stderr "Other Parameters:"
#   puts stderr " -thresh1 thresh1 \[0.0\]"
#   puts stderr " -thresh2 thresh2 \[0.0\]"
    puts stderr " -wMin    wMin   \[-1.0e+16] Minimum Allowed Scaling Value."
    puts stderr " -wMax    wMax   \[+1.0e+16] Maximum Allowed Scaling Value."
    puts stderr " -thresh  thresh \[0.0\]     Threshold for Finding Scale."
    puts stderr " -def     defVal \[0.0\]     Value Regions Below Threshold."
    puts stderr "Output:"
    puts stderr " -out   outName \[None\]    Scaling Vector Output."
    puts stderr " -apply sName   \[None\]    Scaled Spectrum Output."
    exit 1
   }

set verbose   1
set inName1   A.dat
set inName2   B.dat
set maskName  None
set outName   None
set applyName None
set dimCount  Auto
set window    100%
set step      100%
set defVal    0.0
set thresh1   0.0
set thresh2   0.0
set wMin      -1.0e+16
set wMax       1.0e+16

getArgD $ARGV -in1     inName1
getArgD $ARGV -in2     inName2
getArgD $ARGV -out     outName
getArgD $ARGV -apply   applyName
getArgD $ARGV -mask    maskName
getArgD $ARGV -ndim    dimCount
getArgD $ARGV -window  window
getArgD $ARGV -step    step
getArgD $ARGV -thresh  thresh1
getArgD $ARGV -thresh  thresh2
getArgD $ARGV -thresh1 thresh1
getArgD $ARGV -thresh2 thresh2
getArgD $ARGV -def     defVal
getArgD $ARGV -wMin    wMin
getArgD $ARGV -wMax    wMax

if {[flagLoc $ARGV -verb]}   {set verbose 1}
if {[flagLoc $ARGV -noverb]} {set verbose 0}

#
#----------------------------------------------------------------------

if {![file exists $inName1] || ![file exists $inName2]} \
   {
    puts stderr "Error Finding Input Files $inName1 or $inName2"
    exit 0
   }

set error [rdFDATA $inName1 fdata]

if {$error} \
   {
    puts stderr "Error Reading Header from $inName1"
    exit 1
   }

if {![strcasecmp $dimCount Auto]} \
   {
    set dimCount [getParmI $fdata FDDIMCOUNT NULL_DIM]
   }

if {$dimCount > 2} {set dimCount 2}

readROI -ndim $dimCount -roi 1 -in $inName1
readROI -ndim $dimCount -roi 2 -in $inName2
readROI -ndim $dimCount -roi 3 -in $inName1

set roiInfo  [getROIInfo -roi 1]
set rdataA   [lindex $roiInfo 0]
set sizeList [lindex $roiInfo 2]
set xSize    [lindex $sizeList 0]

set roiInfo  [getROIInfo -roi 2]
set rdataB   [lindex $roiInfo 0]

set roiInfo  [getROIInfo -roi 3]
set rdataO   [lindex $roiInfo 0]

set n 1

foreach size $sizeList {set n [expr $n*$size]}

if {![strcasecmp $window 100%]} \
   {
    set wx $xSize
   } \
else \
   {
    set wx [spec2pntW $fdata CUR_XDIM $window]
    set wx [expr int($wx)]
   }

if {![strcasecmp $step 100%]} \
   {
    set dx $xSize
   } \
elseif {![strcasecmp $step 1]} \
   {
    set dx 1
   } \
else \
   {
    set dx [spec2pntW $fdata CUR_XDIM $step]
    set dx [expr int($dx)]
   }

if {$wx > $xSize} {set wx $xSize}
if {$dx > $xSize} {set dx $xSize}

if {$wx < 1}  {set wx 1}
if {$dx < 1}  {set dx 1}

set wx2 [expr $wx/2]

if {$wx2 < 1} {set wx2 1}

if {[strcasecmp $maskName None]} \
   {
    readROI -ndim $dimCount -roi 4 -in $maskName

    set roiInfo  [getROIInfo -roi 4]
    set rdataM   [lindex $roiInfo 0]

    vvMult $rdataA $rdataM $n
    vvMult $rdataB $rdataM $n
   }

vsSet $rdataO 0.0 $n

set wn2 [expr 2*$wx2 + 1]
set wn  [expr 2*$wx  + 1]

if {$wn > $xSize} {set wn $xSize}

for {set ix 0} {$ix < $xSize} {incr ix $dx} \
   {
    set kx [expr $ix - $wx]

    rClip kx 0 [expr $xSize - $wn]

    set ptrA [incrPtr $rdataA $kx]
    set ptrB [incrPtr $rdataB $kx]
    set ptrO [incrPtr $rdataO $kx]

    set top  [vvDot $ptrA $ptrB $wn]
    set bot  [vvDot $ptrB $ptrB $wn]

    if {$bot == 0.0} \
       {
        set r $defVal 
       } \
    else \
       {
        set r [expr $top/$bot]
       }

    if {$thresh1 != 0.0 || $thresh2 != 0.0} \
       {
        set ptrA [incrPtr $ptrA $wx2]
        set ptrB [incrPtr $ptrB $wx2]

        set rmsA [vRMS $ptrA $wn2]
        set rmsB [vRMS $ptrB $wn2]

#       if {$rmsA < $thresh1 || $rmsB < $thresh2} {set r $defVal}
        if {$rmsB < $thresh2} {set r $defVal}
       }

    if {$r > $wMax} {set r $wMax}
    if {$r < $wMin} {set r $wMin}

    vsSet $ptrO $r $wn

    if {$verbose} {puts $r}
   }

if {[strcasecmp $outName None]} \
   {
    writeROI -roi 3 -out $outName -ov
   }

if {[strcasecmp $applyName None]} \
   {
    readROI -ndim $dimCount -roi 2 -in $inName2

    set roiInfo  [getROIInfo -roi 2]
    set rdataB   [lindex $roiInfo 0]

    vvMult $rdataB $rdataO $n

    writeROI -roi 2 -out $applyName -ov
   }

exit 0

