#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set inList        ft/test%03d.ft3
set hiList        Auto 

set xNameList     Auto
set yNameList     Auto
set zNameList     Auto

set z1List        0%
set znList        100%

set tabName       hn.tab
set mult          2.0
set plev          10 
set nlev          10 
set maxStrips     10
set boxFlag       1

set posColor      black
set negColor      red
set pageColor     lightGray 

set orient   portrait
set paper    A
set tray     0

set xw       0.12ppm
set yw       0.80ppm
set zw       1

set sdx      5.0
set sdy      85.0 
set sgap     1.0 
set ggap     1.5 

set xName    Auto 
set yName    Auto 
set zName    Auto 

set xVarName Auto 
set yVarName Auto

set xGrid    -xGrid
set yGrid    -yGrid

set labelVar None

set twoStripFlag  0
set rightAxisFlag 0
set leftAxisFlag  0

set drawArgs      ""

#
#---------------------------------------------------------------

wm iconify .
update

if {[flagLoc $ARGV -help]} \
   {
    puts "PostScript Strip plots of a single 3D spectrum."
    puts "  -in     inList  \[ft/test%03d.ft3\] List of 3D Input Spectra."
    puts "  -hi     hiList  \[Auto\]            Corresponding List of Contour Heights."
    puts "  -tab    tabName \[hn.tab\]          2D Table for strip locations."
    puts "  -plev   nPos    \[10\]              Number of Positive Contours."
    puts "  -nlev   nNeg    \[10\]              Number of Negative Contours."
    puts "  -mult   mult    \[2.0\]             Factor for Auto contour height."
    puts "Plot Format:"
    puts "  -paper     pSize   \[A\]            Paper Size (A-E, U, A1-A7)."
    puts "  -strips    sCount  \[10\]           Max Strips Per Page."
    puts "  -tray      trayID  \[0\]            Paper Tray Number."
    puts "  -portrait                           Plot in Portrait Mode." 
    puts "  -landscape                          Plot in Landscape Mode."
    puts "Strip Axis Names:"
    puts "  -xName  xName   \[Auto\]   First axis name of strip 2D coord."
    puts "  -yName  yName   \[Auto\]   Second axis name of strip 2D coord."
    puts "  -zName  zName   \[Auto\]   Axis name of strip dimension."
    puts "Spectral Region Widths:"
    puts "  -xw     xw      \[0.12ppm] X-Axis +/- Width"
    puts "  -yw     yw      \[0.80ppm] Y-Axis +/- Width (For Two-Strip Mode)."
    puts "  -zw     zw      \[1]       Z-Axis +/- Width (For Srip Projection)."
    puts "Spectral Region Limits, ppm, Hz, pts or %:"
    puts "  -z1     zStart  \[0%\]     Strip Starting Coord."
    puts "  -zn     zEnd    \[100%\]   Strip Ending Coord."
    puts "Strip Dimensions on Plot:"
    puts "  -sdx    sdx     \[5.0\]    Strip Width as Percent of Page Width."
    puts "  -sdy    sdy     \[85.0\]   Strip Height as Percent of Page Height."
    puts "  -sgap   sgap    \[1.0\]    Gap Between Strips as Percent of Page Width."
    puts "  -ggap   ggap    \[1.5\]    Gap Between Groups as Percent of Page Width."
    puts "Coordinate Table Names:"
    puts "  -xVar   xVar    \[Auto\]   Variable name of first 2D PPM coord."
    puts "  -yVar   yVar    \[Auto\]   Variable name of second 2D PPM coord."
    puts "Single/Pairwise Strip Plotting"
    puts "  -nopair         Plot One Strip Per 2D Coord (Default)."
    puts "  -pair           Plot Two Complementary Strips Per 2D Coord."
    puts "Strip Labels and Colors:"
    puts "  -posColor  pColor \[black\]     Color For Positive Contours, or Keyword multi."
    puts "  -negColor  nColor \[red\]       Color For Negative Contours, or Keyword multi."
    puts "  -pageColor pColor \[lightGray\] Page Background."
    puts "  -lab       lVar   \[None\]      Variable Name for Strip Label."
    puts "  -rightAxis                    Include Axis for Rightmost Strips."
    puts "  -leftAxis                     Include Axis for Leftmost Strips."
    puts "  -nobox                        Do Not Draw Boxes Around Strips."
    puts "Options for drawROI (Should be Last on Command Line):"
    puts "  -drawArgs args    \[None\]      Options for drawROI, or Keyword None."
    puts ""
    puts "Example:"
    puts ""
    puts " stripPlot.tcl \\"
    puts "  -tab test.tab -in ft/test%03d.ft3 -hi 9.0e4 \\"
    puts "  -posColor multi -negColor multi -drawArgs -lwPlus 1 -lwMinus 1 \\"
    puts "  -pHue1 0.7 -pHue2 0.5 -pSat1 1.0 -pSat2 1.0 -pVal1 0.8 -pVal2 1.0"
    exit
   }

set loc [flagLoc $ARGV -drawArgs]

if {$loc > 0} \
   {
    set drawArgs [lrange $ARGV [expr $loc + 1] end]
    set ARGV     [lrange $ARGV 0 [expr $loc - 1]]
   }

if {![strcasecmp $drawArgs None]} {set drawArgs ""}

getListArgD $ARGV -in inList
getListArgD $ARGV -hi hiList

getListArgD $ARGV -xName xNameList
getListArgD $ARGV -yName yNameList
getListArgD $ARGV -zName zNameList

getListArgD $ARGV -z1    z1 
getListArgD $ARGV -zn    zn 

set inCount [llength $inList]

set inName  [lindex $inList end] 
set hi      [lindex $hiList end] 

set xName   [lindex $xNameList end]
set yName   [lindex $yNameList end]
set zName   [lindex $zNameList end]

set z1      [lindex $z1List end]
set zn      [lindex $znList end]

foreach thisName $inList \
   {
    lappend hiList    $hi
    lappend xNameList $xName
    lappend yNameList $yName
    lappend zNameList $zName
   }

set hiList    [lrange $hiList    0 [expr $inCount - 1]]

set xNameList [lrange $xNameList 0 [expr $inCount - 1]]
set yNameList [lrange $yNameList 0 [expr $inCount - 1]]
set zNameList [lrange $zNameList 0 [expr $inCount - 1]]

getArgD $ARGV -tab        tabName
getArgD $ARGV -mult       mult 
getArgD $ARGV -plev       plev
getArgD $ARGV -nlev       nlev 
getArgD $ARGV -posColor   posColor
getArgD $ARGV -negColor   negColor 
getArgD $ARGV -pageColor  pageColor 

getArgD $ARGV -sdx     sdx
getArgD $ARGV -sdy     sdy
getArgD $ARGV -sgap    sgap
getArgD $ARGV -ggap    ggap

getArgD $ARGV -xw      xw
getArgD $ARGV -yw      yw
getArgD $ARGV -zw      zw

getArgD $ARGV -xVar    xVarName
getArgD $ARGV -yVar    yVarName
getArgD $ARGV -lab     labelVar

getArgD $ARGV -strips  maxStrips 
getArgD $ARGV -paper   paper
getArgD $ARGV -tray    tray

if {[flagLoc $ARGV -pair]}   {set twoStripFlag 1}
if {[flagLoc $ARGV -nopair]} {set twoStripFlag 0}

if {[flagLoc $ARGV -box]}       {set boxFlag 1}
if {[flagLoc $ARGV -nobox]}     {set boxFlag 0}

if {[flagLoc $ARGV -portrait]}  {set orient portrait}
if {[flagLoc $ARGV -landscape]} {set orient landscape}

if {[flagLoc $ARGV -xGrid]}     {set xGrid -xGrid}
if {[flagLoc $ARGV -noxGrid]}   {set xGrid -noxGrid}

if {[flagLoc $ARGV -yGrid]}     {set yGrid -yGrid}
if {[flagLoc $ARGV -noyGrid]}   {set yGrid -noyGrid}

if {[flagLoc $ARGV -rightAxis]} {set rightAxisFlag 1}

#
#---------------------------------------------------------------

if {$boxFlag} \
   {
    set boxArg box
   } \
else \
   {
    set boxArg nobox
   }

set hnTab [gdbRead table -in $tabName]

if {![strcasecmp $xVarName Auto]} \
   {
    set xVarName [getHNTabVar $hnTab]
    puts "Auto Mode HN Table Variable:  $xVarName"
   }

if {![strcasecmp $yVarName Auto]} \
   {
    set yVarName [get15NTabVar $hnTab]
    puts "Auto Mode 15N Table Variable: $yVarName"
   }

set fdata 0

set error [rdFDATA $inName fdata]

if {![strcasecmp $xName Auto]} \
   {
    set xName [getDirectDim $fdata]
    puts "Auto Mode Direct Dimension:   $xName"
   }

if {![strcasecmp $yName Auto]} \
   {
    set yName [get15NDim $fdata]
    puts "Auto Mode 15N Dimension:      $yName"
   }

if {![strcasecmp $zName Auto]} \
   {
    set zName [getIndirectDim $fdata]
    puts "Auto Mode Strip Dimension:    $zName"
   }

set hnTab [gdbRead table -in $tabName]

if {![strcasecmp $xVarName Auto]} \
   {
    set xVarName [getHNTabVar $hnTab]
    puts "Auto Mode HN Table Variable:  $xVarName"
   }

if {![strcasecmp $yVarName Auto]} \
   {
    set yVarName [get15NTabVar $hnTab]
    puts "Auto Mode 15N Table Variable: $yVarName"
   }

if {![strcasecmp $hi Auto]} \
   {
    set noise [estSpecNoise $inName]
    set hi    [expr $noise*$mult]

    if {$hi == 0.0} {set hi 1.0e+6}

    puts "Auto Mode Contour Height:     [format %.2e $hi]"
   }

#
#---------------------------------------------------------------

#
# For each 2D entry:
#   Extract the 2D coords.
#   Open a new plot if needed.
#   Extract and plot the strips.

set eList      [gdbGetInfo $hnTab -parms entries]
set entryCount [llength $eList]

set stripCount 0
set pageCount  0

set i  0

foreach entry $eList \
   {
    set  ii 0
    incr i

    set xVal [gdbGet $entry $xVarName]
    set yVal [gdbGet $entry $yVarName]
    set xPPM ${xVal}ppm
    set yPPM ${yVal}ppm
    
foreach inName $inList \
   {
    set  hi [lindex $hiList $ii]
    incr ii

    if {!$stripCount} \
       {
        set x1 7.0
        set xn [expr $x1 + $sdx]

        incr pageCount
        set  plotName [format roi%03d.ps $pageCount]
        puts "Page $pageCount:"

        plotOpen -color -plotName $plotName -paper $paper -$orient -tray $tray
              
        drawRec -x 0% 100% -y 0% 100% -fg $pageColor 
       }

    incr stripCount

    puts " [format %10s $xPPM] [format %10s $yPPM] ..."

    defineCell -x $x1% $xn% -y 5% [expr 5.0 + $sdy]% -cell $stripCount

    if {abs($xVal) < 100.0} \
       {
        set fmt %.3f
       } \
    else \
       {
        set fmt %.1f
       }

    if {$xVal < 999.0 && $yVal < 999.0} \
       {
        readROI -in $inName -roi $stripCount -w $xName -1 $yName -1 -ndim 2 \
                -dx $xName $xPPM $xw -y $zName $z1 $zn -dz $yName $yPPM $zw \
                -center X

        if {$stripCount == 1 && $leftAxisFlag} \
           {
            eval drawROI $drawArgs -roi $stripCount -cell $stripCount -bg white -hi $hi \
                         $xGrid $yGrid -tic Bottom,fmt=$fmt,lab=$xName -axis Left \
                         -color1 $posColor -color2 $negColor \
                         -plev $plev -nlev $nlev -pcc $plev -ncc $nlev -$boxArg
           } \
        else \
           {
            eval drawROI $drawArgs -roi $stripCount -cell $stripCount -bg white -hi $hi \
                         $xGrid $yGrid -tic Bottom,fmt=$fmt,lab=$xName \
                         -color1 $posColor -color2 $negColor \
                         -plev $plev -nlev $nlev -pcc $plev -ncc $nlev -$boxArg
           }

        if {!$boxFlag} \
           {
            drawLine -cell $stripCount -x 0% 100% -y 100% 100% -fg black 
           }
       }

    if {abs($yVal) < 100.0} \
       {
        set fmt %.3f
       } \
    else \
       {
        set fmt %.1f
       }

    if {!$twoStripFlag} \
       {
        drawText -cell $stripCount -x 10% -y 109% -fg black -text $yName [format %.2f $yVal]
       }

    if {[strcasecmp $labelVar None]} \
       {
        set str [gdbGet $entry $labelVar]

        drawText -cell $stripCount -x 10% -y -1% -fg black -text $str
       }

    if {$twoStripFlag} \
       {
        set x1 [expr $xn + $sgap]
        set xn [expr $x1 + $sdx]
        incr   stripCount

        defineCell -x $x1% $xn% -y 5% [expr 5.0 + $sdy]% -cell $stripCount

        if {$xVal < 999.0 && $yVal < 999.0} \
           {
            readROI -in $inName -roi $stripCount -w $xName -1 $yName -1 \
                    -ndim 2 -dz $xName $xPPM $zw -y $zName $z1 $zn \
                    -dx $yName $yPPM $yw -center X

            drawROI $drawArgs -roi $stripCount -cell $stripCount -bg white -hi $hi \
                    $xGrid $yGrid -tic Bottom,fmt=$fmt,lab=$yName \
                    -color1 $posColor -color2 $negColor \
                    -plev $plev -nlev $nlev  -pcc $plev -ncc $nlev -$boxArg

            if {!$boxFlag} \
               {
                drawLine -cell $stripCount -x 0% 100% -y 100% 100% -fg black 
               }
           }
       }

    set x1 [expr $xn + $sgap]
    set xn [expr $x1 + $sdx]

    if {$xn > 99.0 || ($i == $entryCount && $ii == [llength $inList])} \
       {
        if {$rightAxisFlag} \
           {
            drawROI -roi $stripCount -cell $stripCount -axis Right -axisOnly
           }

        plotClose

        set stripCount 0
       }
   }

    if {[llength $inList] > 1} \
       {
        set x1 [expr $x1 + $ggap]
        set xn [expr $xn + $ggap]
       }
   }

exit
