#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

set tabName   /spite1/frankie/valpha/hn/hn.tab
set inName1   /spite1/frankie/valpha/cbcanh/ft/test%03d.ft3
set inName2   /spite1/frankie/valpha/cbcaconh/ft/test%03d.ft3
set hi1       1.1e+5
set hi2       1.3e+5

set xName    HN
set yName    N
set zName    CACB
set z1PPM    72.295ppm
set znPPM    19.000ppm

set xVarName X_PPM
set yVarName Y_PPM

#
#-----------------------------------------------------------------------

if {[flagLoc $ARGV -help]} \
   {
    puts "Strip plots of two 3D spectra."
    puts "  -tab    tabName      2D Coordinate table for strip locations."
    puts "  -in1    inName1      Spectral data file #1 template."
    puts "  -in2    inName2      Spectral data file #2 template."
    puts "  -hi1    contHi1      First Contour height, Spectrum 1."
    puts "  -hi2    contHi2      First Contour height, Spectrum 2."
    puts "  -z1     z1PPM        Upper PPM loc for strip."
    puts "  -zn     znPPM        Lower PPM loc for strip."
    puts "Strip Axis Names:"
    puts "  -xName  xName   \[HN\]     First axis name of strip 2D coord."
    puts "  -yName  yName   \[N\]      Second axis name of strip 2D coord."
    puts "  -zName  zName   \[CACB\]   Axis name of strip dimension."
    puts "Coordinate Table Names:"
    puts "  -xVar   xVar    \[X_PPM\]  Variable name of first 2D PPM coord."
    puts "  -yVar   yVar    \[Y_PPM\]  Variable name of second 2D PPM coord."
    exit
   }

getArgD $ARGV -in1     inName1
getArgD $ARGV -in2     inName2
getArgD $ARGV -tab     tabName
getArgD $ARGV -hi1     hi1
getArgD $ARGV -hi2     hi2
getArgD $ARGV -z1      z1PPM
getArgD $ARGV -zn      znPPM
 
getArgD $ARGV -xName   xName 
getArgD $ARGV -yName   yName 
getArgD $ARGV -zName   zName 

getArgD $ARGV -xVar    xVarName 
getArgD $ARGV -yVar    yVarName 

#
# Initialize database engine.
# Read the 2D peak table.
 
gdbInit
gdbCreate dBase -name NMR

set stripCount 0
set pageCount  0

set hnTab [gdbRead table -in $tabName -name 2D -verb]
set entry [gdbGetInfo $hnTab -parms entry]

#
# For each 2D entry:
#   Extract the 2D coords.
#   Open a new plot if needed.
#   Extract and plot the strips.

while {$entry} \
   {
    set xPPM [gdbGet $entry $xVarName]ppm
    set yPPM [gdbGet $entry $yVarName]ppm

    if {!$stripCount} \
       {
        set x1  7.0
        set xn 12.5

        incr pageCount
        set  plotName [format roi%03d.ps $pageCount]
        puts "Page $pageCount:"

        plotOpen -color -landscape -plotName $plotName
        drawRec -x 0% 100% -y 0% 100% -fg lightGray 
       }

    incr stripCount

    puts " [format %10s $xPPM] [format %10s $yPPM] ..."

    defineCell -x $x1% $xn% -y 5% 90% -cell $stripCount

    readROI -in $inName1 -roi $stripCount -w $xName -1 $yName -1 -ndim 2 \
       -dx $xName $xPPM 0.12ppm -y $zName $z1PPM $znPPM                  \
       -dz $yName $yPPM 1 -center X

    if {$stripCount == 1} \
       {
        drawROI -roi $stripCount -cell $stripCount -bg white -hi $hi1 \
           -xGrid -yGrid -tic Bottom -axis Left -color1 black -color2 red
       } \
    else \
       {
        drawROI -roi $stripCount -cell $stripCount -bg white -hi $hi1 \
           -xGrid -yGrid -tic Bottom -color1 black -color2 red
       }

    set x1 [expr $x1 + 6.75]
    set xn [expr $xn + 6.75]
    incr   stripCount

    defineCell -x $x1% $xn% -y 5% 90% -cell $stripCount

    readROI -in $inName2 -roi $stripCount -w $xName -1 $yName -1 -ndim 2 \
       -dx $xName $xPPM 0.12ppm -y $zName $z1PPM $znPPM                  \
       -dz $yName $yPPM 1 -center X

    drawROI -roi $stripCount -cell $stripCount -bg white -hi $hi2 \
       -xGrid -yGrid -tic Bottom -color1 black -color2 red

    set x1 [expr $x1 + 9.0]
    set xn [expr $xn + 9.0]

    if {$stripCount > 10} \
       {
        plotClose
        set stripCount 0
       }

    set entry [gdbNext $entry]
   }

plotClose

exit
