#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

# stripView.tcl
#
#   3D Strip Viewer
#   (C) Frank Delaglio 1998
#-----------------------------------------------------------

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc openStrips {} \
{
   global objInfo

#
# Read the tables defining the screen contents:

   source stripInfo.tcl

   initPPMLimits

   initTab2D
   initTab3D

#
# Adjust drawing area sizes accordingly:

   set obj winST

   set qx  10
   set qy  10

   set wide [winfo width  $objInfo(winST,window)]
   set hi   [winfo height $objInfo(winST,window)]

   bind $objInfo($obj,canvas) <Configure> ""

   $objInfo($obj,canvas) delete pixmap -tags $obj

   setStripCount noDraw

   bind $objInfo($obj,canvas) <Configure> \
      "resizeCanvas $obj %w %h %T drawStrips"

   wm geometry $objInfo(winST,window) ${wide}x${hi}+$qx+$qy
 
   drawStrips
}

proc includeStrips {} \
{
}

proc nextStrips {} \
{
   global objInfo

   if {"$objInfo(tab2D,idList)" == ""} \
      {
       set entry 0
      } \
   else \
      {
       set id    [lindex $objInfo(tab2D,idList) 0]
       set entry [findEntryByID $id]
      }

   if {$entry}  {set entry [gdbNext $entry]}
   if {!$entry} {set entry [gdbFirst $objInfo(tab2D,tab)]}

   set objInfo(tab2D,idList) [gdbGet $entry INDEX]

   drawStrips
}

proc prevStrips {} \
{
   global objInfo

   if {"$objInfo(tab2D,idList)" == ""} \
      {
       set entry 0
      } \
   else \
      {
       set id    [lindex $objInfo(tab2D,idList) 0]
       set entry [findEntryByID $id]
      }

   if {$entry}  {set entry [gdbPrev $entry]}
   if {!$entry} {set entry [gdbLast $objInfo(tab2D,tab)]}

   set objInfo(tab2D,idList) [gdbGet $entry INDEX]

   drawStrips
}

proc findEntryByID { id } \
{
   global objInfo

   if {$id == 0} {return 0}
   if {![strcasecmp $id None]} {return 0}

   set cond  "INDEX == $id"
   set eList [gdbSelect -from $objInfo(tab2D,tab) -cond $cond]

   if {![llength $eList]} {return 0}

   set entry [lindex $eList 0]

   return $entry
}

proc zoomON { w xPix yPix } \
{
   global objInfo

   set objInfo(zoom2D,x1Pix) $xPix
   set objInfo(zoom2D,y1Pix) $yPix
   set objInfo(zoom2D,xnPix) $xPix
   set objInfo(zoom2D,ynPix) $yPix

   $objInfo($w,canvas) create rectangle \
      $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,y1Pix) \
      $objInfo(zoom2D,xnPix) $objInfo(zoom2D,ynPix) \
      -outline white -tags zoom2D
}

proc trackZoom { w xPix yPix } \
{
   global objInfo

   set objInfo(zoom2D,xnPix) $xPix
   set objInfo(zoom2D,ynPix) $yPix

   $objInfo($w,canvas) coords zoom2D \
      $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,y1Pix) \
      $objInfo(zoom2D,xnPix) $objInfo(zoom2D,ynPix)
}

proc zoom2D { xPix yPix } \
{
   global objInfo

   $objInfo(win2D,canvas) delete zoom2D

   if {$objInfo(zoom2D,x1Pix) > $objInfo(zoom2D,xnPix)} \
      {
       set itemp                  $objInfo(zoom2D,x1Pix)
       set $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,xnPix)
       set $objInfo(zoom2D,xnPix) $itemp
      }

   if {$objInfo(zoom2D,y1Pix) > $objInfo(zoom2D,ynPix)} \
      {
       set itemp                  $objInfo(zoom2D,y1Pix)
       set $objInfo(zoom2D,y1Pix) $objInfo(zoom2D,ynPix)
       set $objInfo(zoom2D,ynPix) $itemp
      }
}

proc zoomST { xPix yPix } \
{
   global objInfo

   $objInfo(winST,canvas) delete zoom2D

   if {$objInfo(zoom2D,x1Pix) > $objInfo(zoom2D,xnPix)} \
      {
       set itemp                  $objInfo(zoom2D,x1Pix)
       set $objInfo(zoom2D,x1Pix) $objInfo(zoom2D,xnPix)
       set $objInfo(zoom2D,xnPix) $itemp
      }

   if {$objInfo(zoom2D,y1Pix) > $objInfo(zoom2D,ynPix)} \
      {
       set itemp                  $objInfo(zoom2D,y1Pix)
       set $objInfo(zoom2D,y1Pix) $objInfo(zoom2D,ynPix)
       set $objInfo(zoom2D,ynPix) $itemp
      }

   set j 0

   puts ""

   foreach cellItem $objInfo(winST,cellList) \
      {
       set cellID [lindex $cellItem 0]
       set cx1    [lindex $cellItem 1]
       set cxn    [lindex $cellItem 2]

       if {$objInfo(zoom2D,xnPix) < $cx1} continue
       if {$objInfo(zoom2D,x1Pix) > $cxn} continue

       set  roiID  [lindex $objInfo(winST,roiList) $j]
       incr j

       puts "$j. Cell $cellID ROI $roiID"
      }
}

proc draw2DRegion {} \
{
   global objInfo

   newScreen -win $objInfo(win2D,pixmap)
   drawRec -x 0 100% -y 0 100% -fg gray20 -win $objInfo(win2D,pixmap)

   set i 1

   set id [lindex $objInfo(tab2D,idList) 0]

   if {"$id" == ""} return

   set entry [findEntryByID $id]

   if {!$entry} return

   set xPos   [gdbGet $entry $objInfo(tab2D,xVar)]
   set yPos   [gdbGet $entry $objInfo(tab2D,yVar)]
   set id     [gdbGet $entry INDEX]

   set cell   $objInfo(win2D,cell)
   set roi    $objInfo(win2D,roi)

   set fmt "%s: %.3f %s: %.3f"

   set str \
    [format $fmt $objInfo(data$i,xName) $xPos $objInfo(data$i,yName) $yPos] 

   readROI -roi $roi -ndim 2 -in $objInfo(data$i,name) \
           -dx $objInfo(data$i,xName) ${xPos}ppm $objInfo(data$i,xRegion)ppm \
           -dy $objInfo(data$i,yName) ${yPos}ppm $objInfo(data$i,yRegion)ppm 

   drawROI -roi $roi -cell $cell -bg black \
           -tic Bottom,lab=None,fmt=None Right,lab=None,fmt=None \
           -hi $objInfo(data$i,hi) -xGrid -yGrid

   draw2DPeaks $xPos $yPos

   drawText -roi $roi -cell $cell -x 1.5c1 -y 1.5c1 -fg yellow -text $str 
}

proc draw2DPeaks { xPos yPos } \
{
   global objInfo

   set i 1

   set    cond ""
   append cond "near( $objInfo(data$i,xVar), $xPos, $objInfo(data$i,xRegion) )"
   append cond " && "
   append cond "near( $objInfo(data$i,yVar), $yPos, $objInfo(data$i,yRegion) )"

   set eList [gdbSelect -from $objInfo(tab2D,tab) -cond $cond]

   set cell   $objInfo(win2D,cell)
   set roi    $objInfo(win2D,roi)

   foreach entry $eList \
      {
       set x  [gdbGet $entry $objInfo(data$i,xVar)]ppm
       set y  [gdbGet $entry $objInfo(data$i,yVar)]ppm
       set id [gdbGet $entry INDEX]

       drawText -roi $roi -cell $cell -x $x -y $y -fg white -text $id
      }
}

proc setStripCount { mode } \
{
   global objInfo

   set obj winST

   clearStrips

   bind $objInfo($obj,canvas) <Configure> ""

   $objInfo($obj,canvas) delete pixmap -tags $obj

   set n  $objInfo(tab2D,idCount)
   set nn 0
  
   if {$objInfo(winST,use1HStrip)}  {incr nn}
   if {$objInfo(winST,use15NStrip)} {incr nn}
 
   set dx  40
   set qx  10
 
   set objInfo($obj,xsSize) \
        [expr $n*$objInfo(data,expCount)*($nn*$dx + 2*$qx) + \
              $objInfo(winST,leftMargin) + $objInfo(winST,rightMargin)]

   if {$objInfo($obj,xsSize) < $objInfo($obj,xSize)} \
      {
       set objInfo($obj,xsSize) $objInfo($obj,xSize)
      }

   setCanvasSize $obj 

   bind $objInfo($obj,canvas) <Configure> \
      "resizeCanvas $obj %w %h %T drawStrips"

   if {![strcasecmp $mode Draw]} drawStrips
}

proc drawStrips {} \
{
   global objInfo

   draw2DRegion

   busy hold $objInfo(winST,window)
   update

   set dx 40 
   set qx 10

   set cx1 $dx 
   set cxn [expr $cx1 + $dx]
   set cxm [expr 0.5*($cx1 + $cxn)]

   set cy1 $objInfo(winST,y1) 
   set cyn $objInfo(winST,yn)

   set entryList ""
   set allFlag   1
   set cellID    1 
   set j         0

   if {$objInfo(winST,useXGrid)} \
      {
       set xGrid "-xGrid"
      } \
   else \
      {
       set xGrid "-noxGrid"
      }

   if {$objInfo(winST,useYGrid)} \
      {
       set yGrid "-yGrid"
      } \
   else \
      {
       set yGrid "-noyGrid"
      }

   clearStrips

   for {set i 0} {$i < $objInfo(tab2D,idCount)} {incr i} \
      {
       set id [lindex $objInfo(tab2D,idList) $i]

       if {"$id" != ""} {lappend entryList [findEntryByID $id]}
      }

   for {set i 1} {$i <= $objInfo(data,expCount)} {incr i} \
      {
       if {$objInfo(data$i,ndim) != 3} continue
      
       if {$objInfo(data$i,space)} \
          {
           set cx1 [expr $cx1 + $qx]
           set cxn [expr $cx1 + $dx]
           set cxm [expr 0.5*($cx1 + $cxn)]
          }

       set k 0

       foreach entry $entryList \
          {
           if {$entry} \
              {
               set xPos   [gdbGet $entry $objInfo(tab2D,xVar)]
               set yPos   [gdbGet $entry $objInfo(tab2D,yVar)]
               set id     [gdbGet $entry INDEX]
              } \
           else \
              {
               set xPos   0.0
               set yPos   0.0
               set id     0
              }

           set labelDone  0

           incr j
           incr k

           if {$j % 2} \
              {
               set xOff [expr -0.5*[string length $objInfo(data$i,type)]]
               set yOff -0.4
              } \
           else \
              {
               set xOff [expr -0.5*[string length $objInfo(data$i,type)]]
               set yOff -1.3
              }

           if {$objInfo(winST,use1HStrip) && $objInfo(winST,use15NStrip)} \
              {
               set xLoc [expr $cxn + $qx/2]
               set yLoc $cy1
              } \
           else \
              {
               set xLoc $cxm
               set yLoc $cy1
              }

           foreach iView {0 1} \
              {
               if {$iView == 0 && !$objInfo(winST,use1HStrip)}  continue
               if {$iView == 1 && !$objInfo(winST,use15NStrip)} continue

               set thisFG  [lindex $objInfo(color,stripFGList) [expr $k-1]]
               set roiID   [expr 64*$id + 2*$i + $iView]

               incr cellID

               drawSettings -reset

               defineCell -cell $cellID -win $objInfo(winST,pixmap) \
                  -x $cx1 $cxn -y $cy1 $cyn

               lappend objInfo(winST,cellList) "$cellID $cx1 $cxn $cy1 $cyn"
               lappend objInfo(winST,roiList)  $roiID

               set xPPM ${xPos}ppm
               set yPPM ${yPos}ppm

               set roiInfo [getROIInfo -roi $roiID]

               if {![llength $roiInfo]} \
                  {
                   if {$iView == 0} \
                      {
                       readROI -roi $roiID -ndim 2 -in $objInfo(data$i,name)  \
                        -dx $objInfo(data$i,xName) $xPPM $objInfo(data,dx)ppm \
                        -dz $objInfo(data$i,yName) $yPPM 1                    \
                        -y  $objInfo(data$i,zName) $objInfo(data$i,z1)        \
                            $objInfo(data$i,zn) -w $objInfo(data$i,xName) -1  \
                            $objInfo(data$i,yName) -1 -center X -edge Y
                      } \
                   else \
                      {
                       readROI -roi $roiID -ndim 2 -in $objInfo(data$i,name)  \
                        -dz $objInfo(data$i,xName) $xPPM 1                    \
                        -dx $objInfo(data$i,yName) $yPPM $objInfo(data,dy)ppm \
                        -y  $objInfo(data$i,zName) $objInfo(data$i,z1)        \
                            $objInfo(data$i,zn) -w $objInfo(data$i,xName) -1  \
                            $objInfo(data$i,yName) -1 -center X -edge Y
                      }
                  }

               if {$iView == 0} \
                  {
                   set axisName $objInfo(data$i,xName)
                   set axisFmt  %.3f
                  } \
               else \
                  {
                   set axisName $objInfo(data$i,yName)
                   set axisFmt  %.2f
                  }

               if {$allFlag && $entry} \
                  {
                   if {$objInfo(winST,shadeFolding)} \
                      {
                       drawRec -roiRef $roiID -cell $cellID \
                          -x  0% 100% -y 0% 100% -fg $objInfo(color,stripShade)

                       drawRec -roiRef $roiID -cell $cellID \
                          -x  0% 100%                     \
                          -y  $objInfo(data$i,zFirst)ppm  \
                              $objInfo(data$i,zLast)ppm   \
                          -fg $objInfo(color,stripBG)
                      } \
                   else \
                      {
                       drawRec -roiRef $roiID -cell $cellID \
                          -x  0% 100% -y 0% 100% -fg $objInfo(color,stripBG)
                      }

                   drawROI -roi $roiID -cell $cellID -hi $objInfo(data$i,hi) \
                      $xGrid $yGrid -tic Bottom,lab=$axisName,fmt=$axisFmt   \
                      -fg $thisFG -colorA $thisFG -bg None

                   if {!$labelDone} \
                      {
                       drawText -x $xLoc -y $yLoc -xOff $xOff -yOff $yOff \
                          -fg $thisFG -text $objInfo(data$i,type)

                       set labelDone 1
                      }

                   set cx1 [expr $cxn + $qx]
                   set cxn [expr $cx1 + $dx]
                   set cxm [expr 0.5*($cx1 + $cxn)]
                  }
              }
          }
      }

#
# Draw peak labels:

   set cellID    1 

   for {set i 1} {$i <= $objInfo(data,expCount)} {incr i} \
      {
       if {$objInfo(data$i,ndim) != 3} continue
         
       foreach entry $entryList \
          {
           if {$entry} \
              {
               set id [gdbGet $entry INDEX]

               foreach iView {0 1} \
                  {
                   if {$iView == 0 && !$objInfo(winST,use1HStrip)}  continue
                   if {$iView == 1 && !$objInfo(winST,use15NStrip)} continue

                   set  roiID   [expr 64*$id + 2*$i + $iView]
                   incr cellID

                   drawPkLabels winST $i $id $roiID $cellID
                  }
              }
          }
      }

   busy release $objInfo(winST,window)

   update
}

proc getShifts { parent } \
{
   global objInfo entryArray shiftArray assArray

   set id [gdbGet $parent INDEX]

   for {set i 1} {$i <= $objInfo(data,expCount)} {incr i} \
      {
       set xPos [gdbGet $parent $objInfo(tab2D,xVar)]
       set yPos [gdbGet $parent $objInfo(tab2D,yVar)]

       while {$yPos > $objInfo(data$i,yFirst)} \
          {
           set yPos [expr $yPos - $objInfo(data$i,ySW)]
          }

       while {$yPos < $objInfo(data$i,yLast)} \
          {
           set yPos [expr $yPos + $objInfo(data$i,ySW)]
          }

       set    cond ""
       append cond "near($objInfo(data$i,xVar), $xPos, $objInfo(data,xTol)) &&"
       append cond "near($objInfo(data$i,yVar), $yPos, $objInfo(data,yTol))"

       set eList [gdbSelect -from $objInfo(data$i,tab) -cond $cond]

       set entryArray($i,$id) "" 
       set shiftArray($i,$id) "" 
       set assArray($i,$id)   "" 

       foreach entry $eList \
          {
           set pkID [gdbGet $entry INDEX]
           set ass  [gdbGet $entry ASS]

           if {$pkID < 1} continue

           if {![strcasecmp $ass None]} \
              {
               set ass $objInfo(data$i,assInit)
               gdbSet $entry ASS $ass
              }

           lappend shiftArray($i,$id) [gdbGet $entry $objInfo(data$i,zVar)]
           lappend entryArray($i,$id) $entry
           lappend assArray($i,$id)   $ass
          }
      }
}

proc drawPkLabels { obj i id roiID cellID } \
{
   global objInfo entryArray shiftArray assArray stdfont env

   if {![info exists shiftArray($i,$id)]} \
      {
       set shiftArray($i,$id) 9999.0
       return
      }

   set n [llength $shiftArray($i,$id)]

   for {set j 0} {$j < $n} {incr j} \
      {
       set parent [findEntryByID $id]
       set entry  [lindex $entryArray($i,$id) $j]
       set zPos   [lindex $shiftArray($i,$id) $j]
       set ass    [lindex $assArray($i,$id) $j]
       set pkID   [gdbGet $entry INDEX]
       set zPPM   ${zPos}ppm

       if {$zPos > 999.0} continue
       if {$pkID < 1}     continue

       set cx1 [spec2pix -cell $cellID -roi $roiID -x 5%]
       set cy0 [spec2pix -cell $cellID -roi $roiID -y $zPPM]

       set tID \
           [$objInfo($obj,canvas) create text $cx1 $cy0 -anchor sw   \
              -fill $objInfo(color,peak) -tags peak -text $ass       \
              -font $stdfont -stipple @$env(NMRTXT)/clear.map]

       set cx1 [spec2pix -cell $cellID -roi $roiID -x 0%]
       set cx2 [spec2pix -cell $cellID -roi $roiID -x 25%]
       set cx3 [spec2pix -cell $cellID -roi $roiID -x 75%]
       set cxn [spec2pix -cell $cellID -roi $roiID -x 100%]
       set cy0 [spec2pix -cell $cellID -roi $roiID -y $zPPM]

       set lID \
           [$objInfo($obj,canvas) create line $cx1 $cy0 $cx2 $cy0 \
              -fill white -tags marker]

       set rID \
           [$objInfo($obj,canvas) create line $cx3 $cy0 $cxn $cy0 \
              -fill white -tags marker]

       $objInfo($obj,canvas) bind $lID <Any-Enter> \
          "showThisPeak   $obj $lID $rID $tID"

       $objInfo($obj,canvas) bind $lID <Any-Leave> \
          "unshowThisPeak $obj $lID $rID $tID"

       $objInfo($obj,canvas) bind $rID <Any-Enter> \
          "showThisPeak   $obj $lID $rID $tID"

       $objInfo($obj,canvas) bind $rID <Any-Leave> \
          "unshowThisPeak $obj $lID $rID $tID"

       $objInfo($obj,canvas) bind $rID <ButtonRelease-2> \
          "deleteThisPeak $obj $lID $rID $tID $parent $entry"

       $objInfo($obj,canvas) bind $lID <ButtonRelease-2> \
          "deleteThisPeak $obj $lID $rID $tID $parent $entry"
      }
}

proc showThisPeak { obj lID rID tID } \
{
   global objInfo env

   $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map

   $objInfo($obj,canvas) raise $rID
   $objInfo($obj,canvas) raise $lID

   update
}

proc unshowThisPeak { obj lID rID tID } \
{
   global objInfo env

   $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/clear.map

   $objInfo($obj,canvas) raise $rID
   $objInfo($obj,canvas) raise $lID
	
   update
}

proc deleteThisPeak { obj lID rID tID parent entry } \
{
   global objInfo env

   $objInfo($obj,canvas) delete $lID $rID $tID

   set pkID [gdbGet $entry INDEX]

   gdbSet $entry INDEX -$pkID

   getShifts $parent

   update
}

proc scorePair { id1 id2 scorePtr matchCountPtr maxPossiblePtr } \
{
   global objInfo shiftArray

   upvar $scorePtr       score
   upvar $matchCountPtr  matchCount
   upvar $maxPossiblePtr maxPossible

   set maxPossible   0
   set matchCount    0
   set score         0

   if {$id1 == $id2} return

   for {set i1 1} {$i1 <= $objInfo(data,expCount)} {incr i1} \
      {
       for {set i2 1} {$i2 <= $objInfo(data,expCount)} {incr i2} \
          {
           if {$objInfo(data$i1,zTol) > $objInfo(data$i2,zTol)} \
              {
               set tol $objInfo(data$i1,zTol)
              } \
           else \
              {
               set tol $objInfo(data$i2,zTol)
              }

           set jump      [expr $objInfo(data$i1,score)*$objInfo(data$i2,score)]
           set thisCount 0
           set thisScore 0

           foreach s1 $shiftArray($i1,$id1) \
              {
               if {![llength $s1]} continue

               set hasMatch 0

               foreach s2 $shiftArray($i2,$id2) \
                  {
                   if {![llength $s2]} continue

                   set maxPossible [expr $maxPossible + $jump]
 
                   if {abs($s1-$s2) <= $tol} \
                      {
                       incr matchCount
                       incr hasMatch
                       incr score     $jump
                       incr thisScore $jump
                      }
                  }

               if {$hasMatch} {incr thisCount}
              }

           if {$objInfo(data$i1,nmatch) != 0 && 
                $objInfo(data$i1,nmatch) <= $thisCount} \
              {
               incr score $thisScore
              }
          }       
      }
}

proc sortScores { i1 i2 } \
{
   set s1 [lindex $i1 1]
   set s2 [lindex $i2 1]

   if {$s1 > $s2} {return -1}
   if {$s1 < $s2} {return  1}

   return 0
}

proc allPairs {} \
{
   global objInfo

   set obj winST

   busy hold $objInfo($obj,window)
   update

   puts "VARS   INDEX_I INDEX_J SCORE MATCHCOUNT MAXCOUNT F"
   puts "FORMAT %4d     %4d     %4d   %4d        %4d      %7.4f"
   puts ""


   set eList [gdbGetInfo $objInfo(tab2D,tab) -parms entries]
   set n     [llength $eList]

   set fmt " %4d %4d   %4d %4d %4d  %7.4f"

   for {set i 0} {$i < $n - 1} {incr i} \
      {
       set entryI [lindex $eList $i]
       set idI    [gdbGet $entryI INDEX]

       wm title $objInfo($obj,window) "Finding matches for ID $idI" 
       update
 
       for {set j [expr $i + 1]} {$j < $n} {incr j} \
          {
           set entryJ [lindex $eList $j]
           set idJ    [gdbGet $entryJ INDEX]

           scorePair $idI $idJ score matchCount maxPossible

           if {$maxPossible} \
              {
               set f [expr (100.0*$score)/$maxPossible]
              } \
           else \
              {
               set f 0.0
              }

           puts [format $fmt $idI $idJ $score $matchCount $maxPossible $f]
          }
      }

   wm title $objInfo($obj,window) \
      "$objInfo(prog,name) $objInfo(prog,version) (c) F. Delaglio"

   busy release $objInfo($obj,window)
   update

   update
}

proc findNextRes {} \
{
   global objInfo

   busy hold $objInfo(winST,window)
   update

   set count     [gdbGetInfo $objInfo(tab2D,tab) -parms entryCount]
   set id1       [lindex $objInfo(tab2D,idList) 0]
   set entry1    [findEntryByID $id1]
   set entry2    $entry1
   set idList    $id1
   set scoreList ""

   for {set j 1} {$j <= $count} {incr j} \
      {
       set id2 [gdbGet $entry2 INDEX]

       scorePair $id1 $id2 score matchCount maxPossible 

       lappend scoreList "$id2 $score $matchCount"

       set entry2 [gdbNext $entry2]

       if {!$entry2} {set entry2 [gdbFirst $objInfo(tab2D,tab)]}
      }

    set scoreList [lsort -command sortScores $scoreList]
    set scoreInfo [lindex $scoreList 0]
    set bestID    [lindex $scoreInfo 0]

    foreach scoreInfo [lrange $scoreList 0 9] \
       {
        lappend idList [lindex $scoreInfo 0]      
        puts    $scoreInfo
       }

     busy release $objInfo(winST,window)
     update

     if {$objInfo(tab2D,idCount) == 1} \
        {
         set objInfo(tab2D,idCount) $objInfo(tab2D,defStripCount) 
         setStripCount noDraw
        } 

     set objInfo(tab2D,idList) $idList
 
     drawStrips 
}

proc findPrevRes {} \
{
}

proc placeRes {} \
{
}

proc clearAll {} \
{
   clearCursors
   clearStrips
}

proc clearStrips {} \
{
   global objInfo

   if {[llength $objInfo(winST,roiList)]} \
      {
       newScreen -win $objInfo(winST,pixmap)
      }

   set objInfo(winST,cellList) ""
   set objInfo(winST,roiList)  ""

   $objInfo(winST,canvas) delete peak marker

   drawRec -x 0 100% -y 0 100% -fg gray20 -win $objInfo(winST,pixmap)
}

proc trackST { xPix yPix } \
{
   global objInfo

   set obj winST

   set coordInfo [getCellInfo -x $xPix -y $yPix]

   set cellCount [lindex $coordInfo 1]
   set roiCount  [lindex $coordInfo 2]

   set str "X-Pixel: $xPix Y-Pixel: $yPix"

   if {$roiCount > 0} \
      {
       set count [llength $coordInfo]

       for {set j 3} {$j < $count} {incr j} \
          {
           set cellInfo [lindex $coordInfo $j]

           if {[llength $cellInfo] > 1} \
              {
               set cellID   [lindex $cellInfo  0]
               set roiID    [lindex $cellInfo  1]

               if {$cellID == $objInfo(win2D,cell)} continue

               set str \
                  [getCellInfo -x $xPix -y $yPix -cell $cellID -roi $roiID]

               break
              }
          }
      }

   wm title $objInfo($obj,window) $str

   update
}

proc track2D { xPix yPix } \
{
   global objInfo

   set obj win2D

   set cellID $objInfo($obj,cell)
   set roiID  $objInfo($obj,roi)
   set str    [getCellInfo -x $xPix -y $yPix -cell $cellID -roi $roiID]

   wm title $objInfo($obj,window) $str

   update
}

proc untrackST {} \
{
}

proc untrack2D {} \
{
}

proc addCursor {} \
{
   global objInfo

   set iy [expr $objInfo(winST,height)/(2+$objInfo(winST,cursorCount))]

   set id [$objInfo(winST,canvas) create line \
             0 $iy [expr $objInfo(winST,width)-1] $iy \
             -fill white -tags cursor]

   $objInfo(winST,canvas) bind $id <B1-Motion> "moveCursor $id %x %y"

   lappend objInfo(winST,cursorList) $id
   incr    objInfo(winST,cursorCount)
}

proc clearCursors {} \
{
   global objInfo

   $objInfo(winST,canvas) delete cursor

   set objInfo(winST,cursorCount) 0
   set objInfo(winST,cursorList)  ""
}

proc moveCursor { id ix iy} \
{
   global objInfo

   .c coords $id 0 $iy [expr $objInfo(winST,width)-1] $iy 
}

proc clearWin { obj } \
{
    global objInfo

    update
    newScreen -win $objInfo($obj,pixmap)
    drawRec -x 0 100% -y 0 100% -fg gray20 -win $objInfo($obj,pixmap)
    update
}

proc resizeCanvas { obj width height type rProc } \
{
   global objInfo

   bind $objInfo($obj,canvas) <Configure> ""

   $objInfo($obj,canvas) delete pixmap -tags $obj

   set dx [expr $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]
   set dy [expr $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

   set objInfo($obj,xSize) [expr $width  - $dx]
   set objInfo($obj,ySize) [expr $height - $dy]

   if {!$objInfo($obj,useHScroll)} \
      {
       set objInfo($obj,xsSize) $objInfo($obj,xSize)
      }

   if {!$objInfo($obj,useVScroll)} \
      {
       set objInfo($obj,ysSize) $objInfo($obj,ySize)
      }

   setCanvasSize $obj

   $rProc

   bind $objInfo($obj,canvas) <Configure> "resizeCanvas $obj %w %h %T $rProc"
}

proc printStrips {} \
{
}

proc quitStrips {} \
{
   okExit exit
}

proc helpStrips {} \
{
}

proc aboutStrips {} \
{
}

proc showSettings {} \
{
   global objInfo

   catch {destroy .set}

   toplevel .set
   wm title .set "Settings ..."

   checkbutton .set.xg -text "X-Axis Center Lines" -anchor w \
      -onvalue 1 -offvalue 0 -command drawStrips -width 22 -selectcolor green \
      -variable objInfo(winST,useXGrid)

   checkbutton .set.yg -text "Y-Axis Grid Lines" -anchor w \
      -onvalue 1 -offvalue 0 -command drawStrips -width 22 -selectcolor green \
      -variable objInfo(winST,useYGrid)

   checkbutton .set.sf -text "Shade Folded Regions" -anchor w \
      -onvalue 1 -offvalue 0 -command drawStrips -width 22 -selectcolor green \
      -variable objInfo(winST,shadeFolding)

   checkbutton .set.sh -text "Show 1H Strip View" -anchor w \
      -onvalue 1 -offvalue 0 -command drawStrips -width 22 -selectcolor green \
      -variable objInfo(winST,use1HStrip) 

   checkbutton .set.sn -text "Show 15N Strip View" -anchor w \
      -onvalue 1 -offvalue 0 -command drawStrips -width 22 -selectcolor green \
      -variable objInfo(winST,use15NStrip) 

   button .set.done -text Done -command "catch {destroy .set}"

   pack .set.xg .set.yg .set.sf .set.sh .set.sn .set.done \
      -side top -fill x -expand 1
}

proc setCanvasSize { obj } \
{
   global objInfo

   set objInfo($obj,x1) [expr $objInfo($obj,leftMargin) + 1]
   set objInfo($obj,y1) [expr $objInfo($obj,topMargin) + 1]

   set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,xsSize) - 1]
   set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,ysSize) - 1]

   set objInfo($obj,width)  [expr $objInfo($obj,xn)+$objInfo($obj,rightMargin)]
   set objInfo($obj,height) [expr $objInfo($obj,yn)+$objInfo($obj,bottomMargin)]

   set objInfo($obj,mouseX) [expr 0.5*($objInfo($obj,x1) + $objInfo($obj,xn))]
   set objInfo($obj,mouseY) [expr 0.5*($objInfo($obj,y1) + $objInfo($obj,yn))]

   $objInfo($obj,canvas) configure \
      -width $objInfo($obj,width) -height $objInfo($obj,height)

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,width) -height $objInfo($obj,height) \
      -win $objInfo($obj,pixmap) -tags $obj

   $objInfo($obj,canvas) configure \
      -scrollregion "0 0 $objInfo($obj,xsSize) $objInfo($obj,ysSize)"

   $objInfo($obj,canvas) lower $obj

   defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
      -x $objInfo($obj,x1) $objInfo($obj,xn) \
      -y $objInfo($obj,y1) $objInfo($obj,yn)

   clearWin $obj
}

proc initVars {} \
{
   global objInfo

   set objInfo(prog,name)    StripView
   set objInfo(prog,version) 1999.130.10.44

   setOSType
   setFonts

   gdbInit
   gdbCreate dBase

   set objInfo(mouse,state)        up
   set objInfo(win,count)          0

   set objInfo(winST,useXGrid)     1
   set objInfo(winST,useYGrid)     1
   set objInfo(winST,useHScroll)   1
   set objInfo(winST,useVScroll)   0
   set objInfo(winST,shadeFolding) 1
   set objInfo(winST,use1HStrip)   1
   set objInfo(winST,use15NStrip)  0
   set objInfo(winST,cursorCount)  0
   set objInfo(winST,cursorList)   ""
   set objInfo(winST,cellList)     ""
   set objInfo(winST,roiList)      ""

   set objInfo(win2D,useXGrid)     1
   set objInfo(win2D,useYGrid)     1
   set objInfo(win2D,useHScroll)   0
   set objInfo(win2D,useVScroll)   0
   set objInfo(win2D,shadeFolding) 0
   set objInfo(win2D,use1HStrip)   0
   set objInfo(win2D,use15NStrip)  0
   set objInfo(win2D,cursorCount)  0
   set objInfo(win2D,cursorList)   ""

   set objInfo(data,expCount)       0
   set objInfo(data,expList)        ""

   set objInfo(tab2D,idList)        ""
   set objInfo(tab2D,idCount)       0
   set objInfo(tab2D,defStripCount) 5

   set objInfo(color,peak)        white
   set objInfo(color,stripShade)  gray30
   set objInfo(color,stripShadow) black
   set objInfo(color,stripBG)     black
   set objInfo(color,stripFG)     yellow
   set objInfo(color,stripFGList) "yellow white green blue orange cyan"
}

proc initArgs {} \
{
   global ARGC ARGV objInfo

   getArgD $ARGV -stripCount objInfo(tab2D,defStripCount)
}

proc initStripWindow {} \
{
   global pcFlag objInfo

   set obj winST

   if {$pcFlag} \
      {
       set objInfo($obj,leftMargin)    10
       set objInfo($obj,rightMargin)   20
       set objInfo($obj,topMargin)     30
       set objInfo($obj,bottomMargin)  60
       set yBorder                     20
      } \
   else \
      {
       set objInfo($obj,leftMargin)    15 
       set objInfo($obj,rightMargin)   20 
       set objInfo($obj,topMargin)     40
       set objInfo($obj,bottomMargin)  70
       set yBorder                     20
      }

   set xMargin [expr $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]
   set yMargin [expr $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

   set yMargin [expr $yMargin + $yBorder]

   set objInfo($obj,cell)           1
   set objInfo($obj,pixmap)         1 
   set objInfo($obj,window)         . 
   set objInfo($obj,canvas)         .c
   set objInfo($obj,hScroll)        .sbh
   set objInfo($obj,vScroll)        .sbv

#
# Initial size and position:

   wm title $objInfo($obj,window) \
      "$objInfo(prog,name) $objInfo(prog,version) (c) F. Delaglio"

   update

   set sWide [winfo screenwidth  .]
   set sHi   [winfo screenheight .]

   set objInfo($obj,xSize)   [expr $sWide - 20]
   set objInfo($obj,ySize)   [expr $sHi   - 50]

   set objInfo($obj,xsSize)  [expr $objInfo($obj,xSize) - $xMargin]
   set objInfo($obj,ysSize)  [expr $objInfo($obj,ySize) - $yMargin]

   set qx 10
   set qy 10

   wm geometry $objInfo($obj,window) \
      $objInfo($obj,xSize)x$objInfo($obj,ySize)+$qx+$qy

#
# Command panel and menus:

  if {![winfo exists .cmndFrame]} \
     {
      frame  .cmndFrame

      menubutton .bFile -relief flat -menu .bFile.m -text "File"    -underline 0
      menubutton .bDraw -relief flat -menu .bDraw.m -text "Draw"    -underline 0
      menubutton .bProc -relief flat -menu .bProc.m -text "Special" -underline 0
      menubutton .bHelp -relief flat -menu .bProc.m -text "Help"    -underline 0

      label .lSpace1 -text "     "
      label .lID     -text "   ID List:"
      entry .eID     -width 40 -textvariable objInfo(tab2D,idList)

      label .lSpace2 -text "     "
      label .lCount  -text "Strip Count:"
      entry .eCount  -width 3 -textvariable objInfo(tab2D,idCount)

      bind .eID    <Return> {+ drawStrips}
      bind .eCount <Return> {+ setStripCount draw}

      menu       .bFile.m
      menu       .bDraw.m
      menu       .bProc.m
      menu       .bHelp.m

      .bFile.m add command -label "Open ..."          -command openStrips
      .bFile.m add command -label "Include ..."       -command includeStrips
      .bFile.m add command -label "Print ..."         -command printStrips
      .bFile.m add command -label "Exit"              -command quitStrips

      .bDraw.m add command -label "Next Position"     -command nextStrips
      .bDraw.m add command -label "Previous Position" -command prevStrips
      .bDraw.m add command -label "Redraw"            -command drawStrips
      .bDraw.m add command -label "Add Cursor"        -command addCursor
      .bDraw.m add command -label "Clear Cursors"     -command clearCursors
      .bDraw.m add command -label "Clear Screen"      -command clearAll
      .bDraw.m add command -label "Settings ..."      -command showSettings 
    
      .bProc.m   add command -label "Find Next Residue" -command findNextRes 
      .bProc.m   add command -label "Find Prev Residue" -command findPrevRes 
      .bProc.m   add command -label "Place in Sequence" -command placeRes 
      .bProc.m   add command -label "Find All Pairs"    -command allPairs 

      .bHelp.m   add command -label "Help ..."            -command helpStrips
      .bHelp.m   add command -label "About StripView ..." -command aboutStrips

      pack    .bFile .bDraw .bProc .lSpace1 .lID .eID \
              .lSpace2 .lCount .eCount -in .cmndFrame -side left

      pack    .bHelp -in .cmndFrame -side right
   
      pack    .cmndFrame -side top -fill x
     }

   update

   set yAdj                 [winfo height .cmndFrame]
   set objInfo($obj,ysSize) [expr $objInfo($obj,ysSize) - $yAdj]

#
# Drawing canvas:

   if {![winfo exists $objInfo($obj,canvas)]} \
      {
       scrollbar $objInfo($obj,hScroll) -orient horiz \
          -command "$objInfo($obj,canvas) xview"

       set yAdj                 [winfo height $objInfo($obj,hScroll)]
       set objInfo($obj,ysSize) [expr $objInfo($obj,ysSize) - $yAdj]

       canvas  $objInfo($obj,canvas) -border 0 \
          -scrollregion "0 0 $objInfo($obj,xsSize) $objInfo($obj,ysSize)" \
          -xscrollcommand "$objInfo($obj,hScroll) set" \
          -closeenough 20

       pack $objInfo($obj,hScroll) -side bottom -fill x    -expand 1
       pack $objInfo($obj,canvas)  -side top    -fill both -expand 1

       bind   $objInfo($obj,canvas)  <ButtonPress-3>    "zoomON     winST %x %y"
       bind   $objInfo($obj,canvas)  <ButtonRelease-3>  "zoomST     %x %y"
       bind   $objInfo($obj,canvas)  <B3-Motion>        "trackZoom  %x %y"
       bind   $objInfo($obj,canvas)  <Motion>           "trackST    %x %y"
       bind   $objInfo($obj,canvas)  <Leave>            "untrackST"
      }

   setCanvasSize $obj
}

proc init2DWindow {} \
{
   global pcFlag objInfo

   set obj win2D

   if {$pcFlag} \
      {
       set objInfo($obj,leftMargin)    10
       set objInfo($obj,rightMargin)   20
       set objInfo($obj,topMargin)     30
       set objInfo($obj,bottomMargin)  60

       set objInfo($obj,leftMargin)    10
       set objInfo($obj,rightMargin)   10
       set objInfo($obj,topMargin)     10
       set objInfo($obj,bottomMargin)  10

       set yBorder                     20
      } \
   else \
      {
       set objInfo($obj,leftMargin)    5 
       set objInfo($obj,rightMargin)   150 
       set objInfo($obj,topMargin)     5
       set objInfo($obj,bottomMargin)  70

       set objInfo($obj,leftMargin)    5 
       set objInfo($obj,rightMargin)   10 
       set objInfo($obj,topMargin)     5
       set objInfo($obj,bottomMargin)  10

       set yBorder                     0
      }

   set xMargin [expr $objInfo($obj,leftMargin) + $objInfo($obj,rightMargin)]
   set yMargin [expr $objInfo($obj,topMargin)  + $objInfo($obj,bottomMargin)]

   set yMargin [expr $yMargin + $yBorder]

   set w .win2D

   toplevel $w

   set objInfo($obj,cell)           666666 
   set objInfo($obj,roi)            666666 
   set objInfo($obj,pixmap)         2 
   set objInfo($obj,window)         $w 
   set objInfo($obj,canvas)         $w.c

#
# Initial size and position:

   wm title $objInfo($obj,window) "2D HSQC" 

   update

   set objInfo($obj,xSize)   200
   set objInfo($obj,ySize)   200 

   set objInfo($obj,xsSize)  [expr $objInfo($obj,xSize) - $xMargin]
   set objInfo($obj,ysSize)  [expr $objInfo($obj,ySize) - $yMargin]

   set qx 30
   set qy 80

   wm geometry $objInfo($obj,window) \
      $objInfo($obj,xSize)x$objInfo($obj,ySize)-$qx+$qy

   update

#
# Drawing canvas:

   if {![winfo exists $objInfo($obj,canvas)]} \
      {
       canvas  $objInfo($obj,canvas) -border 0 \
          -scrollregion "0 0 $objInfo($obj,xsSize) $objInfo($obj,ysSize)" \
          -closeenough 20

       pack $objInfo($obj,canvas)  -side top    -fill both -expand 1

#      bind   $objInfo($obj,canvas)  <ButtonPress-3>    "zoomON     win2D %x %y"
#      bind   $objInfo($obj,canvas)  <ButtonRelease-3>  "zoom2D     %x %y"
#      bind   $objInfo($obj,canvas)  <B3-Motion>        "trackZoom  win2D %x %y"
#      bind   $objInfo($obj,canvas)  <Motion>           "track2D    %x %y"
#      bind   $objInfo($obj,canvas)  <Leave>            "untrack2D"
      }

   setCanvasSize $obj
}

proc initResize {} \
{
   global objInfo

   update

   bind $objInfo(winST,canvas) <Configure> \
      "resizeCanvas winST %w %h %T drawStrips"

   bind $objInfo(win2D,canvas) <Configure> \
      "resizeCanvas win2D %w %h %T draw2DRegion"
}

proc initPPMLimits {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(data,expCount)} {incr i} \
      {
       set error [rdFDATA $objInfo(data$i,name) fdata]

       if {$error} \
          {
           showError StripView "Error reading header in $objInfo(data$i,name)"
          }

       if {$objInfo(data$i,ndim) > 0} \
          {
           set dim [getDimKeyword $objInfo(data$i,xName) $fdata]

           set objInfo(data$i,xSize)  [getParm $fdata NDSIZE $dim]
           set objInfo(data$i,xSWHz)  [getParm $fdata NDSW   $dim]
           set objInfo(data$i,xOBS)   [getParm $fdata NDOBS  $dim]
           set objInfo(data$i,xP0)    [getParm $fdata NDP0   $dim]
           set objInfo(data$i,xP1)    [getParm $fdata NDP1   $dim]

           set objInfo(data$i,xFirst) \
              [pnt2spec $fdata $dim 1 ppm]

           set objInfo(data$i,xLast)  \
              [pnt2spec $fdata $dim $objInfo(data$i,xSize) ppm]

           set objInfo(data$i,xSW)    \
              [expr $objInfo(data$i,xSWHz)/$objInfo(data$i,xOBS)]

           if {$objInfo(data$i,xP1) > 170.0 && $objInfo(data$i,xP1) < 190.0} \
              {
               set objInfo(data$i,xSign) 1
              } \
           else \
              {
               set objInfo(data$i,xSign) 0
              }
          }

       if {$objInfo(data$i,ndim) > 1} \
          {
           set dim [getDimKeyword $objInfo(data$i,yName) $fdata]

           set objInfo(data$i,ySize)  [getParm $fdata NDSIZE $dim]
           set objInfo(data$i,ySWHz)  [getParm $fdata NDSW   $dim]
           set objInfo(data$i,yOBS)   [getParm $fdata NDOBS  $dim]
           set objInfo(data$i,yP0)    [getParm $fdata NDP0   $dim]
           set objInfo(data$i,yP1)    [getParm $fdata NDP1   $dim]

           set objInfo(data$i,yFirst) \
              [pnt2spec $fdata $dim 1 ppm]

           set objInfo(data$i,yLast)  \
              [pnt2spec $fdata $dim $objInfo(data$i,ySize) ppm]

           set objInfo(data$i,ySW)    \
              [expr $objInfo(data$i,ySWHz)/$objInfo(data$i,yOBS)]

           if {$objInfo(data$i,yP1) > 170.0 && $objInfo(data$i,yP1) < 190.0} \
              {
               set objInfo(data$i,ySign) 1
              } \
           else \
              {
               set objInfo(data$i,ySign) 0
              }
          }

       if {$objInfo(data$i,ndim) > 2} \
          {
           set dim [getDimKeyword $objInfo(data$i,zName) $fdata]

           set objInfo(data$i,zSize)  [getParm $fdata NDSIZE $dim]
           set objInfo(data$i,zSWHz)  [getParm $fdata NDSW   $dim]
           set objInfo(data$i,zOBS)   [getParm $fdata NDOBS  $dim]
           set objInfo(data$i,zP0)    [getParm $fdata NDP0   $dim]
           set objInfo(data$i,zP1)    [getParm $fdata NDP1   $dim]

           set dz [expr 1.0/$objInfo(data$i,zSize)]

           set objInfo(data$i,zFirst) \
              [pnt2spec $fdata $dim [expr 1 + $dz] ppm]

           set objInfo(data$i,zLast)  \
              [pnt2spec $fdata $dim [expr $objInfo(data$i,zSize) - $dz] ppm]

           set objInfo(data$i,zSW)    \
              [expr $objInfo(data$i,zSWHz)/$objInfo(data$i,zOBS)]

           if {$objInfo(data$i,zP1) > 170.0 && $objInfo(data$i,zP1) < 190.0} \
              {
               set objInfo(data$i,zSign) 1
              } \
           else \
              {
               set objInfo(data$i,zSign) 0
              }
          }

       sysFree $fdata
      }
}

proc initTab2D {} \
{
   global objInfo

   set objInfo(tab2D,tab) \
      [gdbRead table -in $objInfo(tab2D,name) -name "Strip Positions"]

   set entry [gdbFirst $objInfo(tab2D,tab)]

   set objInfo(tab2D,idList)  [gdbGet $entry INDEX]
   set objInfo(tab2D,idCount) 1
}

proc initTab3D {} \
{
   global objInfo bigfont

   toplevel .msg
   wm title .msg "Peak Tables"

   message .msg.rd -width 30c -font $bigfont -relief raised \
           -text "Please Wait ..."

   pack .msg.rd
   update

   for {set i 1} {$i <= $objInfo(data,expCount)} {incr i} \
      {
       .msg.rd configure -text "Reading $objInfo(data$i,tabName) ..."
       update

       set objInfo(data$i,tab) \
          [gdbRead table -in $objInfo(data$i,tabName) \
                         -name $objInfo(data$i,type)]
      }

    set entry [gdbFirst $objInfo(tab2D,tab)] 

    while {$entry} \
       {
        getShifts $entry
        set entry [gdbNext $entry]
       }

   destroy .msg
   update
}

proc initStripView {} \
{
   global objInfo

   initStripWindow
   init2DWindow
   initResize
}

initVars
initArgs
initStripView

#
# Bottom.
