#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

#
# synth.tcl: synthesize a 1D dataset as a linear combination of others.

#
# Command line:

if {[flagLoc $ARGV -help]} \
   {
    puts "$argv0: synthesize a 1D dataset as a linear combination of others."
    puts " -inList inNameList  \[ft/*.ft1\]   List of Input Basis Names."
    puts " -cList  coefList    \[1.0 ...\]    List of Basis Coefficients."
    puts " -out    outName     \[synth.ft1\]  Output Name"
    exit 0
   }

set outName  synth.ft1
set inList   ""
set cList    ""

set inList [lsort -dictionary [glob -nocomplain ft/*.ft1]]

getListArgD $ARGV -inList  inList
getListArgD $ARGV -cList   cList
getArgD     $ARGV -out     outName

puts $inList
puts $cList

#
# Check the coefficient list:

if {![llength $inList]} {exit}

if {![llength $cList]} \
   {
    foreach inName $inList {lappend cList 1.0}
   }

if {[llength $cList] != [llength $inList]} \
   {
    puts "Error: Coefficient list mistmatch."
    exit
   }

#
# Read the basis set, form the sum:

set i 0

foreach bName $inList \
   {
    set c [lindex $cList $i]

    if {!$i} \
       {
        readROI -roi 1 -ndim 1 -in $bName

        set roiInfo [getROIInfo -roi 1]
        set rdata1  [lindex $roiInfo 0]
        set size1   [lindex $roiInfo 2] 

        vsMult $rdata1 $c $size1
       } \
    else \
       {
        readROI -roi 2 -ndim 1 -in $bName

        set roiInfo [getROIInfo -roi 2]
        set rdata2  [lindex $roiInfo 0]
        set size2   [lindex $roiInfo 2]

        vsMult $rdata2 $c $size2

        if {$size1 != $size2} \
           {
            puts "Size mismatch for $bName"
           } \
        else \
           {
            vvAdd $rdata1 $rdata2 $size1
           }
       }

    incr i
   }

#
# Save the results.

writeROI -roi 1 -out $outName -ov

exit
