#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Convert Object Coordinates from a Table into Density Surfaces:"
    puts " -in       inName    \[pcaCoords.tab\] Input Table." 
    puts " -outDir   outDir    \[surf\]          Output Directory."
    puts " -var      varList   \[PCA1 PCA2\]     Coordinate Variables."
    puts " -classVar classVar  \[CLASS\]         Variable with Class Information."
    puts " -valVar   valVar    \[None\]          Variable with Value Information."
    puts " -item     item      \[All\]           Integer Field to Use, or Keyword: All Last."
    puts " -pow      power     \[1.0\]           Raise Final Surface to this Power."
    puts " -zMode    zMode     \[Self\]          Class Variance: Self Max Avg SqrtSumSq"
    puts " -normMode mode      \[thresh\]        Normalization Mode: thresh max sum" 
    puts " -thresh   pctThresh \[95.0\]          Threshold for Normalization, Pct."
    puts "Widths of Surface Function for Individual Objects:"
    puts " -xsd      xsd       \[1.0\]           X-Axis Standard Deviation Multiplier."
    puts " -ysd      ysd       \[1.0\]           Y-Axis Standard Deviation Multiplier."
#   puts " -zsd      zsd       \[1.0\]           Z-Axis Standard Deviation Multiplier."
    puts "Surface Limits:"
    puts " -xMin     xMin      \[Auto\]          X-Axis Start Coord of Distribution."
    puts " -xMax     xMax      \[Auto\]          X-Axis End Coord of Distribution."
    puts " -yMin     yMin      \[Auto\]          Y-Axis Start Coord of Distribution."
    puts " -yMax     yMax      \[Auto\]          Y-Axis End Coord of Distribution."
    puts " -xm       xm        \[0.2\]           X-Axis Margin Size as a Fraction Range."
    puts " -ym       ym        \[0.2\]           Y-Axis Margin Size as a Fraction Range."
#   puts " -zm       zm        \[0.2\]           Z-Axis Margin Size as a Fraction Range."
    puts " -square                             Use the Same Limits for the X-Axis and Y-Axis."
    puts "Output Surface Sizes, Points:"
    puts " -xSize    xSize     \[256\]           X-Axis Size of Surface."
    puts " -ySize    ySize     \[256\]           Y-Axis Size of Surface."
#   puts " -zSize    zSize     \[256\]           Z-Axis Size of Surface (3D)."
    exit 0
   }

set inName     pcaCoords.tab
set outDir     surf
set allDir     None

set varList    "PCA1 PCA2"
set classVar   CLASS 
set valVar     None
set classItem  All 
set normMode   thresh
set zMode      Self
set dimCount   0
set squareFlag 0

set xSize     256 
set ySize     256 
set zSize     256 

set xsd       1.0
set ysd       1.0
set zsd       1.0

set xMin      Auto
set xMax      Auto
set yMin      Auto
set yMax      Auto

set xm        0.2
set ym        0.2
set zm        0.2

set sPow      1.0

set bsCount   0
set bsFrac    0.9
set iseed     7923613

set thresh    95.0 

getArgD $ARGV -in        inName
getArgD $ARGV -outDir    outDir
getArgD $ARGV -allDir    allDir
getArgD $ARGV -classVar  classVar 
getArgD $ARGV -valVar    valVar 
getArgD $ARGV -item      classItem
getArgD $ARGV -normMode  normMode
getArgD $ARGV -zMode     zMode

getArgD $ARGV -xSize     xSize
getArgD $ARGV -ySize     ySize
getArgD $ARGV -zSize     zSize

getArgD $ARGV -xsd       xsd
getArgD $ARGV -ysd       ysd
getArgD $ARGV -zsd       zsd

getArgD $ARGV -xMin      xMin 
getArgD $ARGV -xMax      xMax 
getArgD $ARGV -yMin      yMin 
getArgD $ARGV -yMax      yMax 

getArgD $ARGV -xm        xm
getArgD $ARGV -ym        ym
getArgD $ARGV -zm        zm

getArgD $ARGV -pow       sPow

getArgD $ARGV -bsCount   bsCount
getArgD $ARGV -bsFrac    fsFrac
getArgD $ARGV -iseed     iseed

getArgD $ARGV -thresh    thresh

getListArgD $ARGV -var   varList

if {[flagLoc $ARGV -square]}   {set squareFlag 1}
if {[flagLoc $ARGV -nosquare]} {set squareFlag 0}

#
#------------------------------------------------------------------------------------------------------------

proc normSurf { surfBuff workBuff n } \
{
   global normMode thresh

   if {![strcasecmp $normMode thresh]} \
      {
       vvMove $workBuff $surfBuff $n
       vSort  $workBuff $n
       vRev   $workBuff $n

       set sum    [vSum $workBuff $n]
       set target [expr 0.01*$thresh*$sum]

       set val 0.0
       set a   0.0

       for {set i 0} {$i < $n} {incr i} \
          {
           set hi [getPtrVal $workBuff $i]
           set a  [expr $a + $hi]
    
           if {$a >= $target} \
              {
               set val $hi
               break
              } 
          }

       if {$val == 0.0} return

       vsMult $surfBuff [expr (100.0 - $thresh)/$val] $n
      } \
   elseif {![strcasecmp $normMode max]} \
      {
       set val [vMax $surfBuff $n]

       if {$val == 0.0} return

       vsMult $surfBuff [expr 1.0/$val] $n
      } \
   else \
      {
       set val [vSum $surfBuff $n]

       if {$val == 0.0} return

       vsMult $surfBuff [expr 1.0/$val] $n
      }
}

proc getGaussProf { x1 xn x0 xsd xfw xSize } \
{
   set gProf ""
   set nProf ""

   for {set kx 0} {$kx < $xSize} {incr kx} \
      {
       set x  [expr ($x1 + $kx*($xn - $x1)/double($xSize - 1)) - $x0]
       set dx [safeDiv $x [expr $xsd*$xfw]]
       set r  [expr -0.5*$dx*$dx]

       if {$r < -14.0} \
          {
           lappend gProf 0.0
           } \
       else \
          {
           lappend gProf [expr exp($r)]
          }
      }

   return $gProf
}

if {![file exists $inName]} \
   {
    puts "Error Finding Input Table $inName"
    exit 1
   }

set t        [gdbRead table -in $inName]
set varNames [gdbGetInfo $t -parms varNames]
set eList    [gdbGetInfo $t -parms entries]
set valList  ""

foreach v "$varList $classVar $valVar" \
   {
    if {![strcasecmp $v None]} continue

    if {[lsearch -exact $varNames $v] < 0} \
       {
        puts "Error Finding Variable $v in Table $inName"
        exit 1
       }
   }

srand $iseed

if {$bsCount > 0} \
   {
    set nInit [llength $eList]
    set nNew  [expr int($bsFrac*$nInit)]

    set newList ""

    for {set i 0} {$i < $bsCount} {incr i} \
       {
        for {set j 0} {$j < $nNew} {incr j} \
           {
            lappend newList [lindex $eList [expr int([rand]*$nInit)]]
           }
       }

    foreach e $newList {lappend eList $e}
   }

set dimCount  [llength $varList]
set classList ""

foreach e $eList \
   {
    if {![strcasecmp $classItem All]} \
       {
        set thisClass [gdbGet $e $classVar]
       } \
    else \
       {
        set cStr [split [gdbGet $e $classVar] "-"]

        if {![strcasecmp $classItem last] || ![strcasecmp $classItem end]} \
           {
            set thisClass [lindex $cStr end]
           } \
        else \
           {
            set thisClass [lindex $cStr [expr $classItem - 1]]
           }
       }

    if {[lsearch -exact $classList $thisClass] < 0} {lappend classList $thisClass}

    lappend classInfo($thisClass) $e
    lappend classInfo(All)        $e

    foreach v $varList \
       {
        lappend coordInfo($thisClass,$v) [gdbGet $e $v]
        lappend coordInfo(All,$v)        [gdbGet $e $v]
       }

    if {![strcasecmp $valVar None]} \
       {
        lappend coordInfo($thisClass,val) 1.0
        lappend coordInfo(All,val)        1.0
       } \
    else \
       {
        lappend coordInfo($thisClass,val) [gdbGet $e $valVar] 
        lappend coordInfo(All,val)        [gdbGet $e $valVar] 
       }
   }

foreach thisClass "$classList All" \
   {
    puts [format "Class: %8s Count: %3d" $thisClass [llength $classInfo($thisClass)]]

     foreach v $varList \
        {
         set minVal [getMin    $coordInfo($thisClass,$v)]
         set maxVal [getMax    $coordInfo($thisClass,$v)]
         set avg    [getAvg    $coordInfo($thisClass,$v)]
         set dev    [getStdDev $coordInfo($thisClass,$v)]

         set classInfo($thisClass,$v,range) [expr $maxVal - $minVal]
         set classInfo($thisClass,$v,min)   $minVal
         set classInfo($thisClass,$v,max)   $maxVal
         set classInfo($thisClass,$v,avg)   $avg
         set classInfo($thisClass,$v,dev)   $dev

         if {[strcasecmp $thisClass All]} {lappend devList($v) $dev}

         puts [format " %s Min: % .5e Max: % .5e Avg: % .5e StdDev: % .5e" $v $minVal $maxVal $avg $dev]
        }

    puts ""
   }

if {![file isdirectory $outDir]} {catch "file mkdir $outDir"}

if {[strcasecmp $allDir None]} \
   {
    if {![file isdirectory $allDir]} {catch "file mkdir $allDir"}
   }

set fdata [sysMalloc $FDATASIZE float]

set xVar   [lindex $varList 0]
set yVar   [lindex $varList 1]

set thisXMin $classInfo(All,$xVar,min)
set thisXMax $classInfo(All,$xVar,max)

set thisYMin $classInfo(All,$yVar,min)
set thisYMax $classInfo(All,$yVar,max)

if {$squareFlag} \
   {
    set thisXMin [getMin "$thisXMin $thisYMin"]
    set thisYMin $thisXMin

    set thisXMax [getMax "$thisXMax $thisYMax"]
    set thisYMax $thisXMax
   }

if {$dimCount == 2} \
   {
    set n     [expr $xSize*$ySize]
    set rdata [sysMalloc $n float]
    set sdata [sysMalloc $n float]
    set qdata [sysMalloc $n float]
    set adata 0

    if {[strcasecmp $allDir None]} {set adata [sysMalloc $n float]}

    set xVar   [lindex $varList 0]
    set yVar   [lindex $varList 1]

    set dx     [expr $xm*($thisXMax - $thisXMin)]
    set dy     [expr $ym*($thisYMax - $thisYMin)]

    if {![strcasecmp $xMin Auto]} \
       {
        set x1 [expr $thisXMin - $dx]
       } \
    else \
       {
        set x1 $xMin
       }

    if {![strcasecmp $xMax Auto]} \
       {
        set xn [expr $thisXMax + $dx]
       } \
    else \
       {
        set xn $xMax
       }

    if {![strcasecmp $yMin Auto]} \
       {
        set y1 [expr $thisYMin - $dy]
       } \
    else \
       {
        set y1 $yMin
       }

    if {![strcasecmp $yMax Auto]} \
       {
        set yn [expr $thisYMax + $dy]
       } \
    else \
       {
        set yn $yMax
       }

    set xRange [expr $xn - $x1]
    set yRange [expr $yn - $y1]

    if {$xRange == 0.0} {set xRange 1.0}
    if {$yRange == 0.0} {set yRange 1.0}

    set xMid  [expr 0.5*($x1 + $xn)]
    set yMid  [expr 0.5*($y1 + $yn)]

    buildHdr $fdata \
      -xN      $xSize  -yN      $ySize  \
      -xT      $xSize  -yT      $ySize  \
      -xMODE   Real    -yMODE   Real    \
      -xSW     $xRange -ySW     $yRange \
      -xOBS    1.0000  -yOBS    1.0000  \
      -xCAR    $xMid   -yCAR    $yMid   \
      -xORIG   $xn     -yORIG   $yn     \
      -xFT     Freq    -yFT     Freq    \
      -xLAB    $xVar   -yLAB    $yVar   \
      -ndim    2       -aq2D    Image

    vsSet $sdata 0.0 $n

    set allSurfCount 0

    puts ""

    foreach thisClass $classList \
       {
        puts -nonewline [format "Class: %8s Count: %3d Mode: %s" $thisClass [llength $classInfo($thisClass)] $zMode]

        if {![strcasecmp $zMode Max]} \
            {
             set xfw [getMax $devList($xVar)]
             set yfw [getMax $devList($yVar)]
            } \
         elseif {![strcasecmp $zMode Avg]} \
            {
             set xfw [getAvg $devList($xVar)]
             set yfw [getAvg $devList($yVar)]
            } \
         elseif {![strcasecmp $zMode Self]} \
            {
             set xfw $classInfo($thisClass,$xVar,dev)
             set yfw $classInfo($thisClass,$yVar,dev)
            } \
         else \
            {
             set xfw [getSqrtSumSq $devList($xVar)]
             set yfw [getSqrtSumSq $devList($yVar)]

             set thisN [llength $classInfo(All,$xVar,dev)]
             if {$thisN > 2} {set xfw [expr $xfw/double($thisN - 1)]}

             set thisN [llength $classInfo(All,$yVar,dev)]
             if {$thisN > 2} {set yfw [expr $yfw/double($thisN - 1)]}
            }


        puts [format " %s: %.5e %s: %.5e" $xVar $xfw $yVar $yfw]

        vsSet $rdata 0.0 $n

        setParmStr $fdata FDTITLE $thisClass NULL_DIM

        set outName $outDir/${thisClass}.dat
        set outUnit [sysOpen $outName w]

        set classSize [llength $classInfo($thisClass)]
        set j 0

        foreach e $classInfo($thisClass) \
           {
            incr j

            set x0 [gdbGet $e $xVar]
            set y0 [gdbGet $e $yVar]

            if {![strcasecmp $valVar None]} \
               {
                set val 1.0
               } \
            else \
               {
                set val [gdbGet $e $valVar]
               }

            set xProf [getGaussProf $x1 $xn $x0 $xsd $xfw $xSize]
            set yProf [getGaussProf $y1 $yn $y0 $ysd $yfw $ySize]

            set k 0
 
            foreach qy $yProf \
               {
                foreach qx $xProf \
                   {
                    set hi [expr $val*$qx*$qy]
                    set q  [getPtrVal $rdata $k]

                    setPtrVal $rdata $k [expr $hi + $q] 

                    incr k
                   }
               }

            incr allSurfCount

            if {[strcasecmp $allDir None]} \
               {
                set k 0

                foreach qy $yProf \
                   {
                    foreach qx $xProf \
                       {
                        set hi [expr $val*$qx*$qy]

                        setPtrVal $adata $k $hi

                        incr k
                       }
                   }

                set allName [format "%s/%03d.dat" $allDir $allSurfCount]
                set allUnit [sysOpen $allName w]

                setParmStr $fdata FDTITLE [format "%s-%03d" $thisClass $j] NULL_DIM

                sysWrite $allUnit $fdata $FDATASIZE
                sysWrite $allUnit $adata $n
                sysClose $allUnit
               }
           }

        vvAdd $sdata $rdata $n

        vsPower  $rdata $sPow  $n
        normSurf $rdata $qdata $n

        sysWrite $outUnit $fdata $FDATASIZE
        sysWrite $outUnit $rdata $n
        sysClose $outUnit
       }
   } \
else \
   {
    puts "Error: Unsupported Number of Dimensions: $dimCount"
    exit 1
   }

set thisClass All
set outName   $outDir/${thisClass}.dat

vsPower  $sdata $sPow  $n
normSurf $sdata $qdata $n

set outUnit [sysOpen $outName w]

setParmStr $fdata FDTITLE $thisClass NULL_DIM

sysWrite $outUnit $fdata $FDATASIZE
sysWrite $outUnit $sdata $n
sysClose $outUnit

exit 0 
