
proc rowTagProc { row } \
{
   return userRow$row
}

proc colTagProc { col } \
{
   return userCol$col
}

proc clearTableEntries { aPtr } \
{
   upvar 0 $aPtr a
   global $aPtr

   for {set row 1} {$row <= $a(eCount)} {incr row} \
      {
       set col 0

       foreach var $a(vList) \
          {
           incr col
           catch "unset a($row,$col)"
          }
      }
}

proc reloadTableEntries { aPtr } \
{
   upvar 0 $aPtr a
   global $aPtr

   set a(eList) [gdbGetInfo $a(tab) -parms entries]
   
   for {set row 1} {$row <= $a(eCount)} {incr row} \
      {
       set e   [lindex $a(eList) [expr $row - 1]]
       set col 0

       foreach var $a(vList) \
          {
           incr col

           set val [gdbGet $e $var]
           set a($row,$col) $val
          }
      }

   update
}

proc showTabHelp { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   set hInfo \
    {
     "This is the help text."
     "Use Mouse-B1 to Select Table Items."
     "Use CTRL-Mouse-B1 to Toggle the Selection."
     "Use SHIFT-B1 to Sort a Variable."
    }

  set h .helpWin

  catch "destroy $h"

  toplevel $h
  frame    $h.top

  wm title $h "Table Viewer Help"

  text $h.tHelpNote \
    -yscrollcommand "$h.sHelpNote set" -relief flat -borderwidth 3 \
    -height 24 -width 80 -bg pink -fg black

  scrollbar $h.sHelpNote -command "$h.tHelpNote yview" -orient vertical

  button $h.sHelpClose -text Close -command "destroy $h"

  pack $h.tHelpNote  -in $h.top -side left -fill both -expand 1 -padx 8 -pady 8
  pack $h.sHelpNote  -in $h.top -side left -fill y    -expand 1

  pack $h.top        -in $h -side top -fill both -expand 1
  pack $h.sHelpClose -in $h -side top -fill x    -expand 1

  foreach t $hInfo {$h.tHelpNote insert end "$t\n"}

  update
}

proc showTabStats { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr


  set ws .statWin

  catch "destroy $ws"

  toplevel $ws
  frame    $ws.top

  wm title $ws "Table Selection Statistics"

  text $ws.tStatNote \
    -yscrollcommand "$ws.sStatNote set" -relief flat -borderwidth 3 \
    -height 25 -width 80 -bg pink -fg black

  scrollbar $ws.sStatNote -command "$ws.tStatNote yview" -orient vertical

  button $ws.sStatClose -text Close -command "destroy $ws"

  pack $ws.tStatNote -in $ws.top -side left -fill both -expand 1 -padx 8 -pady 8
  pack $ws.sStatNote -in $ws.top -side left -fill y    -expand 1

  pack $ws.top        -in $ws -side top -fill both -expand 1
  pack $ws.sStatClose -in $ws -side top -fill x    -expand 1

   set sign ""
   set dec  ""
   set dot  ""
   set frac ""
   set e    ""

   set tList {row col int float angle num text all}

   foreach t $tList \
      {
       set valList($t) ""
       set n($t) 0
      }

   foreach sInfo [$w curselection] \
      {
       set sList [split  $sInfo ,]
       set row   [lindex $sList 0]
       set col   [lindex $sList 1]
       set t     [lindex $a(tList) [expr $col - 1]]

       lappend valList($t)   $a($row,$col)
       lappend valList($col) $a($row,$col)
       lappend valList(all)  $a($row,$col)

       if {$a(isAngle,$col)} {lappend valList(angle) $a($row,$col)}

       if {[lsearch -exact $valList(row) $row] < 0} {lappend valList(row) $row}
       if {[lsearch -exact $valList(col) $col] < 0} {lappend valList(col) $col}
      }

   set valList(num) [concat $valList(int) $valList(float)]

   set numFmt "% 11.4e"
   set nDMax  3

   foreach v $valList(num) \
    {
     parseNum $v sign dec dot frac e
     set nD [string length $frac]

     if {$nD > $nDMax} {set nDMax $nD}
    }

   foreach t $tList {set n($t) [llength $valList($t)]}

   set tInfoList \
      {
       {row   "Rows"}
       {col   "Columns"}
       {int   "Integers" }
       {float "Float Values"}
       {num   "All Numbers"}
       {text  "Text Values"}
       {all   "All Items"}
      }

   $ws.tStatNote insert end "\n"
   $ws.tStatNote insert end "Counts of Items in Selection:\n\n"

   foreach tInfo $tInfoList \
      {
       set t     [lindex $tInfo 0]
       set str   [lindex $tInfo 1]

       $ws.tStatNote insert end [format " %-13s %9d\n" "$str:" $n($t)]
      }

   set sList {Min Max SMinAbs SMaxAbs Range Sum Avg RMS StdDev Median}

   if {$n(num)} \
      {
       $ws.tStatNote insert end "\n"

       $ws.tStatNote insert end \
        "Statistics of All Numerical Items in Selection:\n\n"

       foreach s $sList {set sVal($s) [get$s $valList(num)]}

       set fmt " %-11s % 11d\n"

       $ws.tStatNote insert end [format $fmt Count: [llength $valList(num)]]

       if {$sVal(Range) < 1.0 || $sVal(Range) > 1.0e+5} \
          {
           set fmt    " %-11s % 11.4e\n"
           set numFmt "% 11.4e"
          } \
       else \
          {
           set fmt    " %-11s % 11.${nDMax}f\n"
           set numFmt "% 11.${nDMax}f"
          }
 
       foreach s $sList {$ws.tStatNote insert end [format $fmt $s: $sVal($s)]}
      }

   if {$n(angle)} \
      {
       $ws.tStatNote insert end "\n"

       $ws.tStatNote insert end \
        "Statistics of Selected Items Classified as Angles in Degrees:\n\n"

       $ws.tStatNote insert end \
        [format " %-17s % 9d\n" "Number of Angles:" $n(angle)]

       set fmt " %-17s % 9.3f\n"

       $ws.tStatNote insert end \
        [format $fmt "Angular Avg:" [getAvgAngle $valList(angle)]]

       $ws.tStatNote insert end \
        [format $fmt "Angular StdDev:" [getStdDevAngle $valList(angle)]]
      }

   if {$n(num)} \
      {
       $ws.tStatNote insert end "\n"

       $ws.tStatNote insert end \
        "Histogram of All Numerical Items in Selection:\n"

       set rList ""
       set uList ""
       set nList ""

       getHist $valList(num) rList nList 21

       set uList  [getFormat $rList $numFmt]
       set maxN   [getMax $nList]
       set maxLen [getMaxStrlen $uList]
       set wide   [expr 68 - $maxLen]

       if {$wide < 10} {set wide 10}

       set fmt    " %${maxLen}s | %4d | %s\n"
       set nuList ""
       set i      0

       $ws.tStatNote insert end "\n"

       foreach u $uList \
          {
           set thisN [lindex $nList $i]
           set count [expr int( 0.5 + double($wide)*$thisN/$maxN )]

           $ws.tStatNote insert end [format $fmt $u $thisN [repeatStr * $count]]
           lappend nuList "$thisN $u"
           incr i
          }
      }

   if {$n(text)} \
      {
       $ws.tStatNote insert end "\n"

       $ws.tStatNote insert end \
        "Histograms of All Text Items in Selection:\n"

       set sList [lsort -ascii $valList(text)]
       set uList [listUniq $sList]
       set nList [listCounts $uList $sList]

       set maxN   [getMax $nList] 
       set maxLen [getMaxStrlen $uList]
       set wide   [expr 68 - $maxLen]

       if {$wide < 10} {set wide 10}

       set fmt    " %${maxLen}s | %4d | %s\n"
       set nuList ""
       set i      0

       $ws.tStatNote insert end "\n"

       foreach u $uList \
          {
           set thisN [lindex $nList $i]
           set count [expr int( 0.5 + double($wide)*$thisN/$maxN )]
           
           $ws.tStatNote insert end [format $fmt $u $thisN [repeatStr * $count]]
           lappend nuList "$thisN $u"
           incr i
          }

       set qList [lsort -int -index 0 $nuList]

       $ws.tStatNote insert end "\n"

       foreach nu $qList \
          {
           set thisN [lindex $nu 0]
           set u     [lindex $nu 1]

           set count [expr int( 0.5 + double($wide)*$thisN/$maxN )]
           
           $ws.tStatNote insert end [format $fmt $u $thisN [repeatStr * $count]]
          }
      }

  update
}

proc updateTabViewTitle { aPtr wName } \
{
   upvar 0 $aPtr a
   global $aPtr

   wm title $wName \
    "$a(prefix): $a(inName) Showing Variables: $a(vCount) Entries: $a(eCount)"
}

proc reloadTabFile { aPtr wName } \
{
   upvar 0 $aPtr a
   global $aPtr

   if {![strcasecmp $a(inName) None]} return
   if {![file exists $a(inName)]} return

   catch "gdbFree $a(tab)"

   set pdbMode $a(pdbMode)
   set hetMode $a(hetMode)

   set a(tab) [gdbRead table -in $a(inName) -$pdbMode -$hetMode]
   set eCount [gdbGetInfo $a(tab) -parms entryCount]
   set vCount [gdbGetInfo $a(tab) -parms varCount]
   set str    "Varibles: $vCount Entries: $eCount"


   if {![strcasecmp $pdbMode pdb]} \
      {
       set hStr "Read PDB Table $a(inName) Mode $hetMode $str"
      } \
   else \
      {
       set hStr "Read NMRPipe-Format Table $a(inName) $str"
      }

   if {![strcmp $wName "."]} \
      {
       set w "" 
      } \
   else \
      {
       set w $wName 
      }

   clearTableEntries $aPtr

   set a(eCount)  $eCount
   set a(history) ""

   lappend a(history) $hStr

   reloadTableEntries $aPtr
   updateTabViewText  $aPtr $w
   updateTabViewTitle $aPtr $wName

   $w.tab configure -rows [expr $a(eCount) + 1]

   update
}

proc showTabPlot { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr
}

proc copyTabView { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   set str [getTabViewSelectText $aPtr $w]

   clipboard clear
   clipboard append -- $str
}

proc tabValidateEdit { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   return 0
}

proc tabViewSelect { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   set str [getTabViewSelectText $aPtr $w]

   clipboard clear
   clipboard append -- $str

   return $str
}

proc getTabViewSelectText { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   set prevRow -666
   set str "" 

   set n  0
   set nn 0

   if {[string first "," $a(colseparator)] < 0} \
      {
       if {[string first "\n" $a(rowseparator)] < 0} \
          {
           foreach sInfo [$w curselection] \
              {
               set sList [split  $sInfo ,]
               set row   [lindex $sList 0]
               set col   [lindex $sList 1]

               if {$n} \
                  {
                   append str " $a($row,$col)"
                  } \
               else \
                  {
                   append str "$a($row,$col)"
                  }

               incr n
              }
          } \
       else \
          {
           foreach sInfo [$w curselection] \
              {
               set sList [split  $sInfo ,]
               set row   [lindex $sList 0]
               set col   [lindex $sList 1]

               if {$n && $row != $prevRow} \
                  {
                   append str "\n"
                   set nn 0
                  }

               if {$nn} \
                  {
                   append str " $a($row,$col)"
                  } \
               else \
                  {
                   append str "$a($row,$col)"
                  }

               set prevRow $row

               incr nn
               incr n
              }
          }
      } \
   else \
      {
       if {[string first "\n" $a(rowseparator)] < 0} \
          {
           foreach sInfo [$w curselection] \
              {
               set sList [split  $sInfo ,]
               set row   [lindex $sList 0]
               set col   [lindex $sList 1]

               if {$n} \
                  {
                   append str ", $a($row,$col)"
                  } \
               else \
                  {
                   append str "$a($row,$col)"
                  }

               incr n
              }
          } \
       else \
          {
           foreach sInfo [$w curselection] \
              {
               set sList [split  $sInfo ,]
               set row   [lindex $sList 0]
               set col   [lindex $sList 1]
       
               if {$n && $row != $prevRow} \
                  {
                   append str "\n"
                   set nn 0
                  }
          
               if {$nn} \
                  {
                   append str ", $a($row,$col)"
                  } \
               else \
                  {
                   append str "$a($row,$col)"
                  }
          
               set prevRow $row

               incr nn
               incr n
              }
          }
      }

   return $str
}

proc tabViewQuery { eWin aPtr } \
{
   global $aPtr
   upvar 0 $aPtr a

   global tabQueryCount sendList
 
   if {![strcasecmp $a(query) None] || ![string length $a(query)]} return

   if {![info exists tabQueryCount]} {set tabQueryCount 1}

   incr tabQueryCount

   set tab [gdbSelect -new -from $a(tab) -cond $a(query)] 

   set  w .showTabWin$tabQueryCount

   lappend a(history) "Query: $a(query)"
 
   set a(query) ""

   showTable -pre Query -tab $tab -histInfo $a(history) -win $w -send $sendList
}

proc resizeTabView { aPtr wTab wNote wHist wName wide hi } \
{
   upvar 0 $aPtr a
   global $aPtr

   bind $wName <Configure> ""

   setTabViewSize $aPtr $wTab $wName

   $wTab  configure -maxheight $a(tabWinMaxHeight) -maxwidth $a(tabWinMaxWidth)
   $wNote configure -width $a(noteWinCharWidth)
   $wHist configure -width [expr 8 + $a(noteWinCharWidth)]

   update

   bind $wName <Configure> \
    "resizeTabView $aPtr $wTab $wNote $wHist $wName %w %h" 

   update 
}

proc setTabViewSize { aPtr wTab wName } \
{
   upvar 0 $aPtr a
   global $aPtr

   update

   set a(fullWinWidth)   [winfo width  $wName]
   set a(fullWinHeight)  [winfo height $wName]

   set a(tabWinWidth)    [winfo width  $wTab]
   set a(tabWinHeight)   [winfo height $wTab]

   set a(tabWinMaxWidth)  [expr $a(fullWinWidth)  - $a(winExtraWidth)]
   set a(tabWinMaxHeight) [expr $a(fullWinHeight) - $a(winExtraHeight)]

   set a(noteWinCharWidth)  \
   [expr int(0.5 + $a(notePix2Chars)*($a(fullWinWidth) - $a(noteExtraWidth)))]

   set a(noteWinCharHeight) \
   [expr int(0.5 + $a(notePix2Chars)*($a(fullWinHeight) - $a(noteExtraHeight)))]
}

proc sortTabView { col aPtr } \
{
   upvar 0 $aPtr a
   global $aPtr

   if {$col < 1 || $col > $a(vCount)} return

   set vName   [lindex $a(vList) [expr $col - 1]]
   set thisCol 1

   foreach var $a(vList) \
      {
       set a(0,$thisCol) $var
       incr thisCol  
      }

   if {$a($vName,sort) == 0} \
      { 
       set a(0,$col) "$vName (>)"
       set a($vName,sort) 1

       gdbSortTab $a(tab) $vName -up
      } \
   elseif {$a($vName,sort) == 1} \
      {
       set a(0,$col) "$vName (<)"
       set a($vName,sort) -1

       gdbSortTab $a(tab) $vName -down
      } \
   elseif {$a($vName,sort) == -1} \
      {
       set a(0,$col) "$vName"
       set a($vName,sort)  0

       gdbSortTab $a(tab) _gdbid_ -up
      }

   reloadTableEntries $aPtr
}

proc tabViewReloadRecent {} \
{
   global recentShowTabArray recentShowTabWName
 
   reloadTabFile $recentShowTabArray $recentShowTabWName
}

proc tabViewEvent { ev xPix yPix w aPtr } \
{
   upvar  0 $aPtr a
   global $aPtr

   set cellInfo [split [$w index @$xPix,$yPix] ","]
   if  {[llength $cellInfo] != 2} return
 
   set row [lindex $cellInfo 0]
   set col [lindex $cellInfo 1]

   if {![isInt $row] || ![isInt $col]} return

   if {![strcasecmp $ev "Shift-B1-Release"]} \
      {
       if {!$row} {sortTabView $col $aPtr}
      }

   if {![strcasecmp $ev "B1-Release"]} \
      {
       set val None

       if {[info exists a($row,$col)]} {set val $a($row,$col)}

       puts "event $ev $xPix $yPix $row $col $val"

       if {[info exists a(sendList)]} \
          {
           set var None

           if {$col > 0 && $col <= [llength $a(vList)]} \
              {
               set var [lindex $a(vList) [expr $col - 1]]
              }

           foreach i $a(sendList) \
              {
               if {![llength $i]} continue

               puts \
               "send $i tabEvent $a(inName) $ev $xPix $yPix $row $col $var $val"

               catch \
                {send $i "tabEvent $a(inName) $ev $xPix $yPix $row $col $var $val"}
              }
          }
      }
}

proc parseTabViewKey { kSym xPix yPix aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr
}

proc tabViewQueryParms { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr
}

proc updateTabViewText { aPtr w } \
{
   upvar 0 $aPtr a
   global $aPtr

   set text [gdbGetInfo $a(tab) -parms text]

   $w.note delete 1.0 end

   set nPrev 0

   while {$text} \
      {
       set str [string trim [gdbGetInfo $text -parms value]]
       set n   [string length $str]

       if {$n || $nPrev} {$w.note insert end "${str}\n"}

       set text [gdbNext $text]
       set nPrev $n
      }

   $w.hist delete 1.0 end

   foreach str $a(history) \
      {
       set s [string trim $str "{} "]
       $w.hist insert end "${str}\n"
      }
}


proc showTable args \
{
   set ARGV [concat showTable $args]

   global showTableCount recentShowTabArray recentShowTabWName sendList env

   if {![info exists sendList]} {set sendList ""}
   if {![info exists showTableCount]} {set showTableCount 0}

   if {[flagLoc $ARGV -help]} \
      {
       puts stderr "NMRPipe-Format Table Viewer:"
       puts stderr " -in  inName  \[test.tab\]   Input Table Name."
       puts stderr " -pre pText   \[Table\]      Title Prefix Text."
       puts stderr "Options for PDB Input:" 
       puts stderr " -pdb                      Input is PDB (Default for .pdb)."
       puts stderr " -hetatm                   Interpret HETATMs as ATOMs."
       puts stderr "Options for Use from Command Line:"
       puts stderr " -var       vList  \[*\]     List of Variables to Display."
       puts stderr " -angle     aList  \[\"\"\]    Variables Which Are Angles."
       puts stderr " -hist      hList  \[\"\"\]    Table History Info."
       puts stderr "Options for Use from TCL Scripts:"
       puts stderr " -tab       inTab  \[0\]     GDB Table Handle."
       puts stderr " -varInfo   vList  \[*\]     List of Variables to Display."
       puts stderr " -histInfo  hList  \[\"\"\]    Table History Info."
       puts stderr " -angleInfo aList  \[\"\"\]    Variables Which Are Angles."
       puts stderr " -win       wName  \[\"\"\]    TK Window Name."
       return 1
      }

   incr showTableCount

   set aPtr [format stInfo%05d $showTableCount] 

   global $aPtr
   upvar 0 $aPtr a

   set inName  test.tab
   set pText   Table
   set pdbMode nopdb
   set hetMode nohetatm
   set maxHi   242
   set maxWide 662
   set inTab   0

   set vList     "*"
   set angleList ""
   set hList     ""
   set w         ""

   getArgD $ARGV -in        inName
   getArgD $ARGV -pre       pText 
   getArgD $ARGV -tab       inTab
   getArgD $ARGV -win       w
   getArgD $ARGV -array     aPtr 
   getArgD $ARGV -width     maxWide
   getArgD $ARGV -height    maxHi
   getArgD $ARGV -histInfo  hList
   getArgD $ARGV -varInfo   vList
   getArgD $ARGV -angleInfo angleList

   getListArgD $ARGV -var   vList 
   getListArgD $ARGV -hist  hList 
   getListArgD $ARGV -angle angleList 
   getListArgD $ARGV -send  sendList 

   if {![strcasecmp [file extension $inName] .pdb]} {set pdbMode pdb}

   if {[flagLoc $ARGV -pdb]}       {set pdbMode pdb}
   if {[flagLoc $ARGV -nopdb]}     {set pdbMode nopdb}

   if {[flagLoc $ARGV -hetatm]}    {set hetMode hetatm}
   if {[flagLoc $ARGV -nohetatm]}  {set hetMode nohetatm}

   if {!$inTab} \
      {
       if {![file exists $inName]} \
          {
           puts stderr "Error Finding Table Input File $inName"
           return ""
          }
      } \
   else \
      {
       set inName None
      }

   if {![string length $w]} \
      {
       set wName .
      } \
   else \
      {
       set wName $w
      }

   if {![winfo exists $wName]} {toplevel $wName}

   set recentShowTabArray $aPtr
   set recentShowTabWName $wName

   update

   set a(valid)  $showTableCount 
   set a(tab)    0

   set a(vListAll)  ""
   set a(fListAll)  ""
   set a(tListAll)  ""
   set a(eList)     "" 

   set a(vList)     ""
   set a(fList)     ""
   set a(tList)     ""

   set a(eCount) 0
   set a(vCount) 0 

   set eCount    0
   set vCount    0

   set a(tabWinMaxWidth)  $maxWide
   set a(tabWinMaxHeight) $maxHi
   set a(inName)          $inName
   set a(sendList)        $sendList
   set a(pdbMode)         $pdbMode
   set a(hetMode)         $hetMode
   set a(prefix)          $pText

   set a(0,0) ""

   if {![info exists a(history)]} {set a(history) ""}

   foreach h $hList {lappend a(history) $h}

   if {$inTab} \
      {
       set a(tab) $inTab 
       set eCount [gdbGetInfo $a(tab) -parms entryCount]
       set vCount [gdbGetInfo $a(tab) -parms varCount]
      } \
   else \
      {
       set a(tab) [gdbRead table -in $inName -$pdbMode -$hetMode]
       set eCount [gdbGetInfo $a(tab) -parms entryCount]
       set vCount [gdbGetInfo $a(tab) -parms varCount]
       set str    "Varibles: $vCount Entries: $eCount"

       if {![strcasecmp $pdbMode pdb]} \
          {
           lappend a(history) "Read PDB Table $inName Mode $hetMode $str"
          } \
       else \
          {
           lappend a(history) "Read NMRPipe-Format Table $inName $str"
          }
      }

   set a(vListAll)  [gdbGetInfo $a(tab) -parms varNames]
   set a(fListAll)  [gdbGetInfo $a(tab) -parms varFmts]
   set a(tListAll)  [gdbGetInfo $a(tab) -parms varTypes]
   set a(eList)     [gdbGetInfo $a(tab) -parms entries]

   if {![strcmp $vList "*"]} \
      {
       set a(vList) $a(vListAll)
       set a(fList) $a(fListAll)
       set a(tList) $a(tListAll)
      } \
   else \
      {
       foreach var [join $vList] \
          {
           set loc [lsearch -exact $a(vListAll) $var]
           if {$loc < 0} continue
         
           lappend a(vList) $var
           lappend a(fList) [lindex $a(fListAll) $loc] 
           lappend a(tList) [lindex $a(tListAll) $loc] 
          }
      }

   set a(eCount) [llength $a(eList)]
   set a(vCount) [llength $a(vList)]

   if {$vCount != $a(vCount)} \
      {
       lappend a(history) "Showing a Subset of $a(vCount) Variables."
      }

   set a(0,0) ""

   for {set col 0} {$col <= $a(vCount)} {incr col} \
      {
       set a(col$col,width) 1
       set a(isAngle,$col)  0
      }

   set row 0
   set col 1
 
   foreach var $a(vList) \
      {
       set n [expr [string length $var] + 6]
       if {$n > $a(col$col,width)} {set a(col$col,width) $n}
       set a($row,$col) $var
       set a($var,sort) 0

       if {[lsearch -exact $angleList $var] >= 0} {set a(isAngle,$col) 1}

       incr col
      }

   for {set row 1} {$row <= $a(eCount)} {incr row} \
      {
       set e   [lindex $a(eList) [expr $row - 1]]
       set col 0

       set val "$row"
       set a($row,$col) $val

       set n [expr [string length $val] + 1]
       if {$n > $a(col$col,width)} {set a(col$col,width) $n}
 
       foreach var $a(vList) \
          {
           incr col

           set val [gdbGet $e $var]
           set a($row,$col) $val

           set n [string length $val]
           if {$n > $a(col$col,width)} {set a(col$col,width) $n}
          }
      }

   set textHigh 7
   set textHigh 5
   set textWide 92
   set tColor   lightBlue

   if {$showTableCount > 1} {set tColor lightPink} 
      
   text $w.note -yscrollcommand "$w.sn set" -relief sunken -borderwidth 3 \
                -width $textWide -height $textHigh -bg $tColor 

   set histHi 2

   text $w.hist -yscrollcommand "$w.sh set" -relief sunken -borderwidth 3 \
                -width [expr 8 + $textWide] -height $histHi -bg $tColor 

   scrollbar $w.sn -command "$w.note yview" -orient vertical
   scrollbar $w.sh -command "$w.hist yview" -orient vertical

   updateTabViewText $aPtr $w

   set a(selecttype)   col
   set a(colseparator) " "
   set a(rowseparator) "\n"
   set a(query)        ""

   tktable $w.tab -variable $aPtr \
    -validate 1 -vcmd "tabValidateEdit $aPtr $w.tab" \
    -exportselection 1 -selcmd "tabViewSelect $aPtr $w.tab"  \
    -rows [expr $a(eCount) + 1] -cols [expr $a(vCount) + 1]               \
    -yscrollcommand "$w.sy set" -xscrollcommand "$w.sx set"               \
    -titlerows 1 -titlecols 1 -sparsearray 0 -insertofftime 0 -ipadx 6    \
    -resizeborders none -selectmode extended -selecttype $a(selecttype)   \
    -autoclear 1 -cache 1 -rowstretchmode none -colstretchmode all        \
    -rowtagcommand rowTagProc -coltagcommand colTagProc                   \
    -colseparator $a(colseparator) -rowseparator $a(rowseparator)         \
    -maxheight $a(tabWinMaxHeight) -maxwidth $a(tabWinMaxWidth)

   for {set col 0} {$col <= $a(vCount)} {incr col} \
      {
       set j w 

       if {$col} \
          {
           set f [lindex $a(fList) [expr $col - 1]]
           set c [string index $f end]
           if {[strcasecmp $c s]} {set j e}
          }
 
       $w.tab width $col [expr $a(col$col,width) + 1]
       $w.tab tag configure userCol$col -anchor $j 
      }

   $w.tab tag configure title    -anchor c

   scrollbar $w.sx -command "$w.tab xview" -orient horizontal 
   scrollbar $w.sy -command "$w.tab yview" -orient vertical 

   frame $w.seltype

   label       $w.selPad  -text " "  -width 13
   label       $w.selLab  -text "Select Type:" 

   radiobutton $w.selCell -text "Cell" -selectcolor pink -anchor e \
               -command "$w.tab configure -selecttype cell" \
               -value cell -variable ${aPtr}(selecttype) 

   radiobutton $w.selRow  -text "Row" -selectcolor pink -anchor e \
               -command "$w.tab configure -selecttype row" \
               -value row  -variable ${aPtr}(selecttype)

   radiobutton $w.selCol  -text "Column" -selectcolor pink -anchor e \
               -command "$w.tab configure -selecttype col" \
               -value col  -variable ${aPtr}(selecttype)

   radiobutton $w.selBoth -text "Both" -selectcolor pink -anchor e \
               -command "$w.tab configure -selecttype both" \
               -value both -variable ${aPtr}(selecttype)

   label $w.paste -width 9 -text "Paste: " -anchor e

   checkbutton $w.csv -text "Use Commas" -variable ${aPtr}(colseparator) \
    -command "$w.tab configure -colseparator \$${aPtr}(colseparator)" \
    -onvalue ", " -offvalue " " -selectcolor cyan

   checkbutton $w.nl  -text "Use Newlines" -variable ${aPtr}(rowseparator) \
    -command "$w.tab configure -rowseparator \$${aPtr}(rowseparator)" \
    -onvalue "\n" -offvalue " " -selectcolor cyan

   pack $w.selPad $w.selLab  $w.selCell $w.selRow $w.selCol $w.selBoth \
        $w.paste $w.csv $w.nl -in  $w.seltype -side left -anchor center

   frame $w.qframe
 
   label $w.qpad  -text " " -width 4
   label $w.qlab  -text "Query:"
   entry $w.qent  -width 90 -textvariable ${aPtr}(query)

   button $w.qwin \
    -command "tabViewQueryParms $aPtr $w.tab" \
    -relief raised -bitmap @$env(NMRTXT)/caret.map -fg blue

   pack $w.qpad $w.qlab $w.qent $w.qwin -in $w.qframe -side left 

   frame $w.cmndframe

   button $w.reload  -text "Reload"         -command "reloadTabFile $aPtr $wName"
   button $w.copy    -text "Copy Selection" -command "copyTabView   $aPtr $w.tab"
   button $w.stats   -text "Statistics"     -command "showTabStats  $aPtr $w.tab"
   button $w.plot    -text "X/Y Plot"       -command "showTabPlot   $aPtr $w.tab"
   button $w.help    -text "Help"           -command "showTabHelp   $aPtr $w.tab"

   pack $w.copy $w.stats $w.reload $w.help \
        -in $w.cmndframe -side left -fill x -expand 1

   set ir 0

   grid $w.note      -row $ir -column 0 -padx 8 -pady 8
   grid $w.sn        -row $ir -column 1 -sticky ns
   incr ir

   grid $w.hist      -row $ir -column 0 -padx 8 -pady 8
   grid $w.sh        -row $ir -column 1 -sticky ns
   incr ir

   grid $w.tab       -row $ir -column 0 -sticky nsew
   grid $w.sy        -row $ir -column 1 -sticky ns -rowspan 2
   incr ir

   grid $w.sx        -row $ir -column 0 -sticky ew
   incr ir

   grid $w.qframe    -row $ir -column 0 -sticky ew -columnspan 2
   incr ir

   grid $w.seltype   -row $ir -column 0 -sticky ew -columnspan 2
   incr ir

   grid $w.cmndframe -row $ir -column 0 -sticky ew -columnspan 2
   incr ir

   update

   set a(fullWinWidth)   [winfo width  $wName]
   set a(fullWinHeight)  [winfo height $wName]

   set a(tabWinWidth)    [winfo width  $w.tab]
   set a(tabWinHeight)   [winfo height $w.tab]

   set a(noteWinWidth)   [winfo width  $w.note]
   set a(noteWinHeight)  [winfo height $w.note]

   set a(winExtraWidth)  [expr 1 + abs($a(fullWinWidth)  - $a(tabWinWidth))]
   set a(winExtraHeight) [expr 1 + abs($a(fullWinHeight) - $a(tabWinHeight))]

   set a(noteExtraWidth)  [expr 1 + abs($a(fullWinWidth)  - $a(noteWinWidth))]
   set a(noteExtraHeight) [expr 1 + abs($a(fullWinHeight) - $a(noteWinHeight))]
   set a(notePix2Chars)   [expr double($textWide)/abs($a(noteWinWidth))]

   update

   bind $wName <Configure> \
    "resizeTabView $aPtr $w.tab $w.note $w.hist $wName %w %h" 

   bind $w.tab  <ButtonRelease-1> \
    "+ tabViewEvent b1-release %x %y $w.tab $aPtr"

   bind $w.tab <Shift-ButtonRelease-1> \
    "+ tabViewEvent shift-b1-release %x %y $w.tab $aPtr"

   bind $w.qent <Return> "+ tabViewQuery $w.qent $aPtr"

   bind $w.tab  <KeyPress> "+ parseTabViewKey %K %x %y $aPtr $w.tab"

   $w.hist see "end - 2 lines"

   updateTabViewTitle $aPtr $wName

   update

   if {[flagLoc $ARGV -iconify]} \
      {
       wm iconify $wName
       update
      }

   return "$showTableCount $aPtr $wName $w.tab |$w|"
}

