#!/bin/csh

set thisVersion = "2019.364.11.49"

if (`flagLoc $argv -help`) then
   nmrPrintf -stderr "tdBaseline Version $thisVersion\n"
   nmrPrintf -stderr "Baseline Correction by Iterative Time-Domain Replacement:\n"
   nmrPrintf -stderr " -in     inName  [test.ft1] Input  Spectrum (1D-4D).\n"
   nmrPrintf -stderr " -out    outName [base.ft1] Output Spectrum (Can be Same as Input).\n"
   nmrPrintf -stderr " -n      n       [10]       Number of Points to Replace in Time-Domain.\n"
   nmrPrintf -stderr " -iter   iter    [64]       Maximum Number of Iterations.\n"
   nmrPrintf -stderr " -tMult  tMult   [0.33]     Initial Threshold Fraction.\n"
   nmrPrintf -stderr " -iMult  iMult   [0.90]     Iteration Multiplier for Reducing Threshold.\n"
   nmrPrintf -stderr " -verb                      Verbose Mode ON (Default).\n"
   nmrPrintf -stderr " -noverb                    Verbose Mode OFF.\n"
   nmrPrintf -stderr "Automated Termination of Iterations:\n"
   nmrPrintf -stderr " -noise  noise [Auto]       Noise Estimate.\n"
   nmrPrintf -stderr " -factor  f    [3.0]        Multiplier for Automated Termination.\n"
   nmrPrintf -stderr " -target  t    [Auto]       Target Threshold (Auto = f*noise, or Keyword None).\n"
   nmrPrintf -stderr "Temporary Files:\n"
   nmrPrintf -stderr ' -prefix  p    [tmp_base$$] Prefix for Temporary Files ($$ is Process ID).\n'
   nmrPrintf -stderr ' -noclean                   Do Not Delete Temporary Files After Autophase.\n'
   nmrPrintf -stderr "Dimension to Correct:\n"
   nmrPrintf -stderr " -x                         Correct X-Axis (Default).\n"
   nmrPrintf -stderr " -y                         Correct Y-Axis (2D-4D Only).\n"
   nmrPrintf -stderr " -z                         Correct Z-Axis (3D-4D Only).\n"
   nmrPrintf -stderr " -a                         Correct A-Axis (4D Only).\n"
   exit 0
endif

set inName  = (`getArgD $argv -in     test.ft1`)
set outName = (`getArgD $argv -out    base.ft1`)
set prefix  = (`getArgD $argv -prefix tmp_base$$`)

set n       = (`getArgD $argv -n      10`)
set maxIter = (`getArgD $argv -iter   64`)
set tMult   = (`getArgD $argv -tMult  0.33`)
set iMult   = (`getArgD $argv -iMult  0.90`)
set factor  = (`getArgD $argv -factor 3.00`)
set noise   = (`getArgD $argv -noise  Auto`)
set target  = (`getArgD $argv -target Auto`)

set dimArg  = x
set tpFn    = NULL

@ autoFlag  = 1
@ cleanFlag = 1
@ vFlag     = 1
@ saveFlag  = 0

if (`flagLoc $argv -x`) then
   set dimArg  = x
   set tpFn    = NULL
endif

if (`flagLoc $argv -y`) then
   set dimArg  = y
   set tpFn    = TP
endif

if (`flagLoc $argv -z`) then
   set dimArg  = z
   set tpFn    = NULL 
endif

if (`flagLoc $argv -a`) then
   set dimArg  = a
   set tpFn    = NULL 
endif

if (`flagLoc $argv -verb`) then
   @ vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   @ vFlag = 0
endif

if (`flagLoc $argv -noclean`) then
   @ cleanFlag = 0
endif

if (`flagLoc $argv -save`) then
   @ saveFlag = 1
endif

if (`flagLoc $argv -auto`) then
   @ autoFlag = 1
endif

if (`flagLoc $argv -noauto`) then
   @ autoFlag = 0
endif

if ($saveFlag) then
   /bin/rm -rf tsave fsave
   mkdir tsave fsave
endif

set inNameOrig = $inName

set tmpNameP   = ${prefix}P.dat
set tmpName0   = ${prefix}0.dat
set tmpName1   = ${prefix}1.dat
set tmpName2   = ${prefix}2.dat
set tmpName3   = ${prefix}3.dat
set tmpName4   = ${prefix}4.dat

if (`string csame $inName stdin`) then
   nmrPipe -tty -out $tmpNameP -ov
   set inName = $tmpNameP
endif

set thisName = (`nmrPrintf $inName 1 1`)

@ fmtCount = (`echo $inName | sed -e 's/\(.\)/\1\n/g' | grep % | wc -l`)

@ dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
@ pipeFlag = (`getParm -in $thisName -parm FDPIPEFLAG -fmt %.0f`)
@ cubeFlag = (`getParm -in $thisName -parm FDCUBEFLAG -fmt %.0f`)

@ xyzFlag = 0

if ($dimCount == 1) then
   @ xyzFlag = 0
else if ($dimCount == 2) then
   @ xyzFlag = 0
else if ($dimCount == 3) then
   if ($fmtCount == 1 || $pipeFlag != 0) then
      @ xyzFlag = 1
   endif
else
   if ($fmtCount == 1 || $pipeFlag != 0 || $cubeFlag != 0) then
      @ xyzFlag = 1
   endif
endif

if ($xyzFlag) then
   xyz2pipe -in $inName -$dimArg  -out $tmpName0
else
   nmrPipe  -in $inName -fn $tpFn -out $tmpName0 -ov
endif

set thisName = $tmpName0

if (`string csame $noise Auto`) then
   set sInfo   = (`specStat.com -brief -in $thisName -stat vMaxAbs vMin vEstNoise`)
   set maxVal  = $sInfo[1]
   set minVal  = $sInfo[2]
   set noise   = $sInfo[3]
else
   set sInfo   = (`specStat.com -brief -in $thisName -stat vMaxAbs vMin`)
   set maxVal  = $sInfo[1]
   set minVal  = $sInfo[2]
endif

set tMultOrig = $tMult
set thisMin   = $minVal
set thisMax   = $maxVal

if (`string csame $target Auto`) then
   set target = (`MATH "$factor*$noise"`)
endif

set dimName = (`getParm -in $thisName -parm NDLABEL -dim CUR_XDIM -text -fmt %s`)
set pct     = (`MATH "100.0*div($target,$maxVal)"`)

if ($vFlag) then
   nmrPrintf -stderr "\n"
   nmrPrintf -stderr "tdBaseline. Baseline Correction by Iterative Time-Domain Replacement.\n"
   nmrPrintf -stderr "tdBaseline. Version: $thisVersion\n"
   nmrPrintf -stderr "\n"

   nmrPrintf -stderr "tdBaseline. Input:                %s\n"       $inNameOrig
   nmrPrintf -stderr "tdBaseline. Output:               %s\n"       $outName
   nmrPrintf -stderr "tdBaseline. Dimension to Correct: %s (%s)\n"  `string toupper $dimArg` $dimName
   nmrPrintf -stderr "tdBaseline. Points to Replace:    %d\n"       $n
   nmrPrintf -stderr "tdBaseline. Max Iterations:       %d\n"       $maxIter
   nmrPrintf -stderr "tdBaseline. Maximum Value:        %.3e\n"     $maxVal
   nmrPrintf -stderr "tdBaseline. Noise Estimate:       %.3e\n"     $noise

   if (`string csame $target None`) then
      nmrPrintf -stderr "tdBaseline. Target Threshold:     %s\n"    $target
   else
      nmrPrintf -stderr "tdBaseline. Target Threshold:     %.3e (%.3f%%)\n"  $target $pct
   endif
endif

if ($vFlag) then
   nmrPrintf -stderr "\n"
endif

@ x1 = $n + 1
@ i  = 1

while ($i <= $maxIter)
   set clipVal = (`MATH "$tMult*$thisMax"`)
   set pct     = (`MATH "100.0*div($clipVal,$maxVal)"`)

   if ($vFlag) then
      nmrPrintf -stderr "Iteration: %3d Max: %.3e Min: %.3e Threshold: %.3e %6.3f%s\n" $i $thisMax $thisMin $clipVal $pct %
   endif

#
# tmpName1: inverse FT with points 1 to N set to zero.
# tmpName2: inverse FT of clipped data, with points N+1 on set to zero.
# tmpName3: FT of tmpName1 + tmpName2.

   nmrPipe -in $thisName \
   | nmrPipe -fn HT -auto -nozf -nowarn \
   | nmrPipe -fn FT -inv -nodf \
   | nmrPipe -fn SET -c 0.0 -xn $n \
     -out $tmpName1 -ov

   nmrPipe -in $thisName \
   | nmrPipe -fn CLIP -below -$clipVal -above $clipVal -inside 0.0 \
   | nmrPipe -fn HT -auto -nozf -nowarn \
   | nmrPipe -fn FT -inv -nodf \
   | nmrPipe -fn SET -c 0.0 -x1 $x1 \
     -out $tmpName2 -ov

   addNMR -in1 $tmpName1 -in2 $tmpName2 -out $tmpName3 -add

   nmrPipe -in $tmpName3 -fn FT -nodf -di -out $tmpName4 -ov

   if ($saveFlag) then
      /bin/cp $tmpName3 `nmrPrintf tsave/%03d.dat $i`
      /bin/cp $tmpName4 `nmrPrintf fsave/%03d.dat $i`
   endif

   if (`string csame $target None` == 0) then
      if (`IMATH "$clipVal < $target"`) then
         break
      endif
   endif

   set tMult    = (`MATH "$iMult*$tMult"`)
   set thisName = $tmpName4

   set sInfo    = (`specStat.com -brief -in $thisName -stat vMaxAbs vMin`)
   set thisMax  = $sInfo[1]
   set thisMin  = $sInfo[2]

   @ i++
end

if ($xyzFlag) then
   pipe2xyz -in $tmpName4 -out $tmpNameP -$dimArg
else
   nmrPipe  -in $tmpName4 -out $tmpNameP -fn $tpFn -ov
endif

if (`string csame $outName stdout`) then
   nmrPipe -in $tmpNameP -tty
else
   if ($xyzFlag) then
      xyz2pipe -in $tmpNameP | pipe2xyz -out $outName
   else
      nmrPipe -in $tmpNameP -out $outName -ov
   endif
endif

if ($cleanFlag) then
   /bin/rm -f $tmpNameP $tmpName0 $tmpName1 $tmpName2 $tmpName3 $tmpName4
endif

