#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Extract Decay Profiles from Dimensions of Time-Domain Data."
   exit 0
endif

set inName = (`getArgD $argv -in     test.fid`)
set outDir = (`getArgD $argv -outDir tmpmc`)

set xSOL     = (`getArgD $argv -xSOL     Auto`)
set xBASEARG = (`getArgD $argv -xBASEARG Auto`)

set xP0    = (`getArgD $argv -xP0    0.0`)
set xP1    = (`getArgD $argv -xP1    0.0`)
set xC     = (`getArgD $argv -xC1    0.5`)

set yP0    = (`getArgD $argv -yP0    0.0`)
set yP1    = (`getArgD $argv -yP1    0.0`)
set yC     = (`getArgD $argv -yC1    0.5`)
set yFTARG = (`getArgD $argv -yFTARG None`)

set zP0    = (`getArgD $argv -zP0    0.0`)
set zP1    = (`getArgD $argv -zP1    0.0`)
set zC     = (`getArgD $argv -zC1    0.5`)
set zFTARG = (`getArgD $argv -zFTARG None`)

set ext1 = (`getArgD $argv -xEXTX1 5%`)
set extN = (`getArgD $argv -xEXTXN 45%`)

set x1 = (`getArgD $argv -x1 25%`)
set xN = (`getArgD $argv -xN 75%`)

set y1 = (`getArgD $argv -y1 25%`)
set yN = (`getArgD $argv -yN 75%`)

set z1 = (`getArgD $argv -z1 25%`)
set zN = (`getArgD $argv -zN 75%`)

set cleanFlag = (`getLogArgD $argv -clean      -noclean   1 0`)
set vFlag     = (`getLogArgD $argv -verb       -noverb   verb noverb`)

#
#---------------------------------------------------------------------------

set thisName = (`nmrPrintf $inName 1 1`)
set outList  = ""

if (!(-e $thisName)) then
   nmrPrintf -stderr "%s\n" "testFID.com Notice: Missing Test Input $thisName"
   exit 1
endif

set dimCount = (`getParm -in $thisName -parm FDDIMCOUNT -fmt %.0f`)
set pipeFlag = (`getParm -in $thisName -parm FDPIPEFLAG -fmt %.0f`)
set cubeFlag = (`getParm -in $thisName -parm FDCUBEFLAG -fmt %.0f`)

set xLAB = (`getParm -in $thisName -txt -fmt %s -parm NDLABEL -dim CUR_XDIM`)

if (`string match "HN*" $xLAB`) then
   if (`string csame $xSOL Auto`) then
      set xSOL = SOL 
   endif

   if (`string csame $xBASEARG Auto`) then
      set xBASEARG = POLY,auto
   endif
else
   if (`string csame $xSOL Auto`) then
      set xSOL = NULL
   endif

   if (`string csame $xBASEARG Auto`) then
      set xBASEARG = NULL 
   endif
endif

if ($dimCount != 1) then
   if (-d $outDir) then
      /bin/rm -rf $outDir
   endif

   mkdir $outDir
endif

if ($dimCount == 1) then
   set outList = (xMC.dat)
else if ($dimCount == 2) then
   set outList = (xMC.dat yMC.dat)
else if ($dimCount == 3) then
   set outList = (xMC.dat yMC.dat zMC.dat)
else if ($dimCount == 4) then
   set outList = (xMC.dat yMC.dat zMC.dat aMC.dat)
endif

foreach f ($outList)
   if (-e $f) then
      /bin/rm -f $f
   endif
end

if ($dimCount == 1) then
   /bin/rm -f xMC.dat

   nmrPipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \   
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn HT -auto \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -dmx \
   | nmrPipe -fn SP -inv -hdr \
   | nmrPipe -fn MC
     -out xMC.dat -ov

else if ($dimCount == 2) then
   /bin/rm -f xMC.dat yMC.dat

   nmrPipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $yC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT `expandArg $yFTARG` \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn TP \
   | nmrPipe -fn HT -auto \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn ZF -inv \
   | nmrPipe -fn SP -inv -hdr \
   | nmrPipe -fn MC \
   | nmrPipe -fn TP \
     -out $outDir/x.ft1 -ov

   nmrPipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $yC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT `expandArg $yFTARG` \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn EXT -time -dmx \
   | nmrPipe -fn SP -inv -hdr \
   | nmrPipe -fn MC \
     -out $outDir/y.ft1 -ov
 
   readROI -in $outDir/y.ft1 -out xMC.dat -ov -ndim 1 -x X_AXIS 0% 100% -y Y_AXIS $y1 $yN -noverb -silent
   readROI -in $outDir/x.ft1 -out yMC.dat -ov -ndim 1 -x Y_AXIS 0% 100% -y X_AXIS $x1 $xN -noverb -silent

else if ($dimCount == 3) then
   /bin/rm -f xMC.dat yMC.dat zMC.dat

   xyz2pipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $yC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT `expandArg $yFTARG` \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn MC \
   | nmrPipe -fn ZTP \
   | pipe2xyz -out $outDir/xy%03d.ft2 -ov
   
   xyz2pipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $zC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT `expandArg $zFTARG` \
   | nmrPipe -fn PS -p0 $zP0 -p1 $zP1 -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn TP \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn MC \
   | nmrPipe -fn TP \
   | pipe2xyz -out $outDir/xz%03d.ft2 -ov

   xyz2pipe -in $inName \
   | nmrPipe -fn `expandArg0 $xSOL` \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $xC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $xP0 -p1 $xP1 -di \
   | nmrPipe -fn EXT -x1 $ext1 -xn $extN -sw \
   | nmrPipe -fn TP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $yC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT `expandArg $yFTARG` \
   | nmrPipe -fn PS -p0 $yP0 -p1 $yP1 -di \
   | nmrPipe -fn TP \
   | nmrPipe -fn `expandArg0 $xBASEARG` \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn SP -off 0.5 -end 0.95 -c $zC \
   | nmrPipe -fn ZF -auto \
   | nmrPipe -fn FT \
   | nmrPipe -fn PS -p0 $zP0 -p1 $zP1 -di \
   | nmrPipe -fn ZTP \
   | nmrPipe -fn HT -auto \
   | nmrPipe -fn PS -inv -hdr \
   | nmrPipe -fn FT -inv \
   | nmrPipe -fn ZF -inv \
   | nmrPipe -fn SP -inv -hdr \
   | nmrPipe -fn MC \
   | pipe2xyz -out $outDir/yz%03d.ft2 -ov

   readROI -in $outDir/yz%03d.ft2 -out xMC.dat -ov -ndim 1 \
           -x X_AXIS 0% 100% -y Y_AXIS $y1 $yN -z Z_AXIS $z1 $zN -noverb -silent

   readROI -in $outDir/xz%03d.ft2 -out yMC.dat -ov -ndim 1 \
           -x Y_AXIS 0% 100% -y X_AXIS $x1 $xN -z Z_AXIS $z1 $zN -noverb -silent

   readROI -in $outDir/xy%03d.ft2 -out zMC.dat -ov -ndim 1 \
           -x Z_AXIS 0% 100% -y X_AXIS $x1 $xN -z Y_AXIS $y1 $yN -noverb -silent

else if ($dimCount == 4) then
endif

if ($cleanFlag) then
   if (-d $outDir) then
      /bin/rm -rf $outDir
   endif
endif

exit 0
