#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "$argv0: Re-Arrange Varian fid file:"
 puts stderr " -in     inName  \[stdin\]   Input fid Name."
 puts stderr " -out    outName \[stdout\]  Output fid Name."
 puts stderr " -order  oList   \[1 ...\]   Desired Order of Vectors in Output."
 puts stderr " -hb     hByts   \[32\]      Initial Header Size of Input."
 puts stderr " -pb     pBytes  \[28\]      Internal Pad Size of Input." 
 puts stderr " -expand                   Expand Multi-Experiment Files."
 puts stderr " -verb                     Verbose Mode ON (Default)."
 puts stderr " -noverb                   Verbose Mode OFF."
 puts stderr "Byte-Swap Flags (Temporary, for interpreting Varian Header Only):"
 puts stderr " -swap             Do Temporary Four-Byte Swap."
 puts stderr " -noswap           Suppress Four-Byte Swap."
 puts stderr " -aswap            Do Four-Byte Swap."
 puts stderr "                   (Meaning is System-dependant)"
 puts stderr " -noaswap          Suppress Four-Byte Swap (Default)."
 puts stderr "                   (Meaning is System-dependant)"
 puts stderr "Notes:"
 puts stderr " 1. Varian data must be four-byte."
 exit 1
}

set inName     stdin 
set outName    stdout 
set orderList  1
set vHdrSize   32 
set vPadSize   28 
set vTotalHdr  60
set expandFlag 0
set verbose    1

if {$SYS_BYTESWAP} \
   {
    set swapFlag 1
   } \
else \
   {
    set swapFlag 0
   }

getArgD     $ARGV -in    inName
getArgD     $ARGV -out   outName
getArgD     $ARGV -hb    vHdrSize
getArgD     $ARGV -pb    vPadSize
getListArgD $ARGV -order orderList

if {[flagLoc $ARGV -expand]}   {set expandFlag 1}
if {[flagLoc $ARGV -noexpand]} {set expandFlag 0}

if {$SYS_BYTESWAP} \
   {
    if {[flagLoc $ARGV -aswap]}     {set swapFlag 0}
    if {[flagLoc $ARGV -noaswap]}   {set swapFlag 1}
   } \
else \
   {
    if {[flagLoc $ARGV -aswap]}     {set swapFlag 1}
    if {[flagLoc $ARGV -noaswap]}   {set swapFlag 0}
   }

if {[flagLoc $ARGV -swap]}     {set swapFlag 1}
if {[flagLoc $ARGV -noswap]}   {set swapFlag 0}

if {[flagLoc $ARGV -verb]}     {set verbose 1}
if {[flagLoc $ARGV -noverb]}   {set verbose 0}

#
#---------------------------------------------------------------

#
# Prepare files:

if {![strcmp $inName $outName]} \
   {
    puts stderr "Error: input and output names must differ."
    exit 1
   }

if {![strcasecmp $inName stdin]} \
   {
    set inUnit 0
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts stderr "Error finding input file $inName"
        exit 1
       }

    set inUnit [sysOpen $inName r]
    
    if {$inUnit < 0} \
       {
        puts stderr "Error opening input file $inName"
        exit 1
       }
   }

if {![strcasecmp $outName stdout]} \
   {
    set outUnit 1
   } \
else \
   {
    set outUnit [sysOpen $outName w]
    
    if {$outUnit < 0} \
       {
        puts stderr "Error opening output file $outName"
        exit 1
       }
   }

#
# Read and extract header:

set thisSize [expr $vHdrSize + $vPadSize]

if {$vTotalHdr < $thisSize} {set vHdrSize $thisSize}

set vHdr [sysMalloc $vTotalHdr char]

sysReadB  $inUnit  $vHdr $vHdrSize

if {$swapFlag} {vByteSwap $vHdr [expr $vTotalHdr/4] float}

set ptr [incrPtr $vHdr 0]
set v1  [getPtrVal *$ptr int]

set ptr [incrPtr $vHdr 4]
set v2  [getPtrVal *$ptr int]

set ptr [incrPtr $vHdr 8]
set v3  [getPtrVal *$ptr int]

set ptr [incrPtr $vHdr 12]
set v4  [getPtrVal *$ptr int]

set ptr [incrPtr $vHdr 16]
set v5  [getPtrStr *$ptr 10]

set ptr [incrPtr $vHdr 26]
set v6  [getPtrVal *$ptr short]

set ptr [incrPtr $vHdr 28]
set v7  [getPtrStr *$ptr 4]

#
# Adjust header if needed, write header:

if {$expandFlag} \
   {
    setPtrVal $vHdr 0 [expr $v1*$v2] int
    setPtrVal $vHdr 4 1 int
   }

if {$swapFlag} {vByteSwap $vHdr [expr $vTotalHdr/4] float}

sysWrite $outUnit $vHdr $vHdrSize

#
# Allocate memory for FID vectors:
 
set fidCount $v1
set expCount $v2
set fidSize  $v3
set wordSize $v4

if {$verbose} \
   {
    puts stderr "Fid Count: $v1 Exp Count: $v2 FidSize: $v3 Word Size: $v4"
    puts stderr ""
   }

set vBytes [expr $fidSize*$wordSize]
set n      [llength $orderList]

for {set j 1} {$j <= $n} {incr j} \
   {
    set vData($j) [sysMalloc $vBytes char]
   }

#
# Read and write FID vectors:

set fidsIn  0
set fidsOut 0 

while {$fidsOut < $fidCount} \
   {
    for {set j 1} {$j <= $n} {incr j} \
       {
        if {!($fidsIn % $expCount)} \
           {
            sysReadB  $inUnit  $vHdr $vPadSize
           }

        sysReadB $inUnit $vData($j) $vBytes
        incr fidsIn 
       }

    foreach j $orderList \
       {
        if {$expandFlag || !($fidsOut % $expCount)} \
           {
            sysWrite $outUnit $vHdr $vPadSize
           }

        sysWrite $outUnit $vData($j) $vBytes
        incr fidsOut 

        if {$fidsOut == 1 || !($fidsOut % 10) || $fidsOut == $fidCount} \
           {
            if {$verbose} \
               {
                puts -nonewline stderr \
                 "                  FID $fidsOut of $fidCount\015"
               }

            flush stderr
           }
       }
   }
  
if {$verbose} {puts stderr ""}
 
exit 0
