#!/bin/csh

set vdName   = (`getArgD $argv -in   vdlist`)
set keepStr  = (`getArgD $argv -keep 3u`)
set dimCount = (`getArgD $argv -ndim 3`)

set yT       = (`getArgD $argv -yT   128`)
set zT       = (`getArgD $argv -zT   64`)
set aT       = (`getArgD $argv -aT   32`)

set revFlag = 0

if (`flagLoc $argv -rev`) then
   set revFlag = 1
endif

if (`flagLoc $argv -help`) then
   echo "Create nuslist-format NUS schedule from old-style vdlist schedule:"
   echo " -in   vdName   [vdlist] Input vdlist schedule."
   echo " -ndim ndim     [3]      Dimension Count of Fully-Expanded Data."
   echo " -yT   yT       [128]    Number of Complex Increments in the Y-Axis."
   echo " -zT   zT       [64]     Number of Complex Increments in the Z-Axis."
   echo " -aT   aT       [32]     Number of Complex Increments in the A-Axis."
   echo " -keep keepText [3u]     Text in vdlist Specifying Increment to Keep."
   echo " -rev                    Reverse the Column Order of Increments in Output."
   exit 0
endif

if (!(-e $vdName)) then
   echo "vd2nuslist. Error finding vdlist input $vdName"
   exit 1
endif

set vList = (`cat $vdName`)

if ($dimCount == 2) then
   if ($#vList != $yT) then
      echo "vd2nuslist. Error: size mismatch ($#vList $yT)"
      exit 1
   endif

   @ i  = 1
   @ iy = 0

   while ($iy < $yT)
      if (`string csame $vList[$i] $keepStr`) then
         nmrPrintf " %3d\n" $iy
      endif

      @ i++
      @ iy++
   end
else if ($dimCount == 3) then
   set zN = (`IMATH "2*$zT"`)
   set n  = (`IMATH "$yT*$zN"`)

   if ($#vList != $n) then
      echo "vd2nuslist. Error: size mismatch ($#vList $n)"
      exit 1
   endif

   @ iz = 0
   @ i  = 1

   while ($iz < $zT)
      @ iy = 0
   
      while ($iy < $yT)
         if (`string csame $vList[$i] $keepStr`) then
            if ($revFlag) then
               nmrPrintf " %3d %3d\n" $iz $iy
            else
               nmrPrintf " %3d %3d\n" $iy $iz
            endif
         endif

         @ i++
         @ iy++
      end

#
# Schedule is repeated twice (plane Z = real, plane Z = imag), skip second copy:

      @ i = $i + $yT
      @ iz++
   end
else if ($dimCount == 4) then
   set zN   = (`IMATH "2*$zT"`)
   set aN   = (`IMATH "2*$aT"`)
   set n    = (`IMATH "$yT*$zN*$aN"`)
   set jump = (`IMATH "$yT*$zT"`)

   if ($#vList != $n) then
      echo "vd2nuslist. Error: size mismatch ($#vList $n)"
      exit 1
   endif

   @ i  = 1
   @ ia = 0

   while ($ia < $aT)
      @ iz = 0

      while ($iz < $zT)
         @ iy = 0
   
         while ($iy < $yT)
            if (`string csame $vList[$i] $keepStr`) then
               if ($revFlag) then
                  nmrPrintf " %3d %3d %3d\n" $ia $iz $iy
               else
                  nmrPrintf " %3d %3d %3d\n" $iy $iz $ia
               endif
            endif

            @ i++
            @ iy++
         end

         @ i = $i + $yT
         @ iz++
      end

      @ i = $i + $jump
      @ ia++
   end
else
   echo "vd2nuslist. Error: unsupported dimension count $dimCount"
   exit 1
endif
