#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

# view1D.tcl
#
#   1D Spectrum Viewer
#   (C) Frank Delaglio 2002
#-----------------------------------------------------------

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initProgVars {} \
{
   global objInfo

   setFonts

   set objInfo(prog,name)    "View1D"
   set objInfo(prog,version) "2008.132.01.23"

   set objInfo(prog,title)   \
       "$objInfo(prog,name) $objInfo(prog,version) (C) F. Delaglio 2008"

   set xw [winfo screenwidth  .]
   set yw [winfo screenheight .]

   set objInfo(prog,verbose)       1
   set objInfo(prog,reuseFlag)     0
   set objInfo(prog,freeFlag)      0
   set objInfo(prog,nextID)        1
   set objInfo(prog,winCount)      0 
   set objInfo(prog,winList)       ""
   set objInfo(prog,freeIDList)    ""
   set objInfo(prog,labelItemList) ""
   set objInfo(prog,defXSize)      [expr 5*$xw/6] 
   set objInfo(prog,defYSize)      [expr 3*$yw/4]
   set objInfo(prog,exportProc)    None
   set objInfo(prog,gridFlag)      0
   set objInfo(prog,labFlag)       1
   set objInfo(prog,autoStartFlag) 1
   set objInfo(prog,tabViewFlag)   0
   set objInfo(prog,pkFlag)        0
   set objInfo(prog,charX)         10
   set objInfo(prog,charY)         16
   set objInfo(prog,updateFlag)    1
   set objInfo(prog,mouseList)     ""
   set objInfo(prog,mouseON)       0
   set objInfo(prog,B1)            0 
   set objInfo(prog,B2)            0 
   set objInfo(prog,B3)            0 

   set objInfo(data,statList)      Diff

   set objInfo(data,specCount)     2 
   set objInfo(data,roiCount)      0 
   set objInfo(data,inList)        ""
   set objInfo(data,tauList)       ""
   set objInfo(data,titleList)     ""
   set objInfo(data,drawOrder)     ""
   set objInfo(data,factor)        1.1
   set objInfo(data,lw1d)          1

   set objInfo(data,cursorH)   None
   set objInfo(data,cursorV)   None
   set objInfo(data,window)    .
   set objInfo(data,valid)     0
   set objInfo(data,fdata)     0
   set objInfo(data,rdata)     0
   set objInfo(data,xSize)     0
   
   set objInfo(data,alignX)    None
   set objInfo(data,alignY)    None
   set objInfo(data,realFlag)  real

   set objInfo(data,specX1)       0
   set objInfo(data,specXN)       0
   set objInfo(data,specY1)       0
   set objInfo(data,specYN)       0

   set objInfo(data,activeSpec)   "1 (None)"
}

proc initDataArgs {} \
{
   global ARGC ARGV objInfo

   if {[flagLoc $ARGV -one]} {set objInfo(data,specCount) 1}
   if {[flagLoc $ARGV -two]} {set objInfo(data,specCount) 2}
 
   if {[flagLoc $ARGV -imag]} {set objInfo(data,realFlag) imag}
 
   getArgD $ARGV -specCount objInfo(data,specCount)

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,inName$i) "None"
      }

   set objInfo(data,inList)  ""
   set objInfo(data,inName1) "test.ft2"

   getListArgD $ARGV -in    objInfo(data,inList)

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -in$i  objInfo(data,inName$i)
       }

   set n [llength $objInfo(data,inList)]

   if {$n} \
      {
       if {$n > $objInfo(data,specCount)} {set objInfo(data,specCount) $n}

       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           set objInfo(data,inName$i) \
            [lindex $objInfo(data,inList) [expr $i - 1]] 
          }
      } \
   else \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           lappend objInfo(data,inList) $objInfo(data,inName$i)
          }
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        lappend objInfo(data,drawOrder) $i

        set objInfo(data,drawFlag$i) 1
       }

   if {[flagLoc $ARGV -useTitle]} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
# %%%
           rdFDATA $objInfo(data,inName$i) fdata
           set t [getParmStr $fdata FDTITLE NULL_DIM]
           sysFree $fdata

           if {![llength $t]} \
              {
               set str [file tail [file dirname $objInfo(data,inName$i)]]
               set t   $objInfo(data,inName$i)

               if {[string length $str]} \
                  {
                   set sList [split $str _]
                   set ns    [llength $sList]

                   if {$ns >= 1} \
                      {
                       set t [lrange $sList 0 [expr $ns - 1]]
                      }
                  }
              }

           lappend objInfo(data,titleList) $t
          }
      }

   getListArgD $ARGV -title objInfo(data,titleList)

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,fhi$i)    0.0
       set objInfo(data,lhi$i)    0.0
       set objInfo(data,noise$i)  0.0
       set objInfo(data,scale$i)  1.0
       set objInfo(data,roi$i)    0
      }
}

proc initPeakVars {} \
{
   global objInfo

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(peak,tab$i)      0
       set objInfo(peak,pkCount$i)  0
      }

   set objInfo(peak,inList)      test.tab
   set objInfo(peak,xName)       None
}

proc initProgColors {} \
{
   global objInfo

   set objInfo(color,bg)            black
   set objInfo(color,on)            green
   set objInfo(color,shadow)        black
   set objInfo(color,label)         white
   set objInfo(color,zoom)          green
   set objInfo(color,propose)       white
   set objInfo(color,accept)        green
   set objInfo(color,cursor)        gray30
   set objInfo(color,box)           white
   set objInfo(color,peak)          yellow 
   set objInfo(color,grid)          dimGray
   set objInfo(color,neutral)       dimGray
   set objInfo(color,winBG)         gray30
   set objInfo(color,stat)          white
   set objInfo(color,statFill)      dimGray 
}

proc initSpecColors {} \
{
   global ARGC ARGV objInfo

   set objInfo(color,colorList) \
    "Yellow Cyan Green Red Blue Violet Magenta LightGray White"

   set objInfo(color,fillColorList) None

   if {[flagLoc $ARGV -useFill]} \
      {
       set objInfo(color,fillColorList) \
        "Yellow Cyan Green Red Blue Violet Magenta LightGray White"
      }

   getListArgD $ARGV -colors     objInfo(color,colorList)
   getListArgD $ARGV -fillColors objInfo(color,fillColorList)

   set nc [llength $objInfo(color,colorList)]
   set nf [llength $objInfo(color,fillColorList)]

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set j [expr ($i - 1) % $nc]
       set k [expr ($i - 1) % $nf]

       set objInfo(color,color$i)     [lindex $objInfo(color,colorList) $j]
       set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $j]
      }
}

proc initProgArgs {} \
{
   global objInfo ARGC ARGV

   getArgD $ARGV -export objInfo(prog,exportProc)

   if {[flagLoc $ARGV -help]} \
      {
       puts "1D Spectral Viewer with Overlay and Zoom:"
       puts "With a single spectrum:"
       puts " -in   name1        Spectrum Name"
       puts "With two spectra:"
       puts " -in1   name1       Spectrum 1 Name"
       puts " -in2   name2       Spectrum 2 Name"
       puts "With a spectral series:"
       puts " -in    nameList    Spectrum Name List"
       puts " -fhi    hi         Min Height."
       puts " -lhi    hi         Max Height." 
       puts "General Arguments:"
       puts " -align             Align by PPM (Approximate)."
       puts " -one               Configure for One Spectrum"
       puts " -two               Configure for Two Spectra (Default)"
       puts " -grid              Use Grid Lines."
       puts " -nogrid            No Grid Lines. (Default)"
       puts " -pts               Zoom Region Coords in Points"
       puts " -ppm               Zoom Region Coords in PPM"
       puts " -tab     tabName   Peak Table Name or List"
       puts " -xSize   xPix      Default X-Axis Window Size"
       puts " -ySize   yPix      Default Y-Axis Window Size"
       puts " -export  eName     Special Procedure for Region Export"
       puts "Spectrum Color Arguments:"
       puts " -colors       colorList  Color List for Spectra 1 2 ..."
       puts " -color1       color1     Color for Spectrum 1."
       puts " -color2       color2     Color for Spectrum 2 ..."
       puts " -fillColors   colorList  List of Colors for Optional 1D Fill."
       puts " -useFill                 Use 1D Fill Drawing Mode."
       puts "Other Color Arguments:"
       puts " -peakColor    pkColor    Peak Label Color"
       puts " -bgColor      bgColor    1D Graphic Background Color"
       puts " -winColor     winColor   Window Background Color"
       puts " -labelColor   lColor     Label Color"
       puts " -shadowColor  sColor     Label Shadow Color"
       puts " -zoomColor    zColor     Zoom Box Color"
       puts " -cursorColor  cColor     Cursor Color"
       puts " -gridColor    gColor     Grid Color"
       puts " -boxColor     bColor     Outline Box Color"
       puts " -proposeColor pColor     Selected Region Box Color"
       puts " -acceptColor  aColor     Accepted Region Box Color"
       
       if {"$objInfo(prog,exportProc)" != "None"} \
          {
           $objInfo(prog,exportProc) -- -help
          }

       exit 1
      }

   if {[flagLoc $ARGV -verb]}   {set objInfo(prog,verbose) 1}
   if {[flagLoc $ARGV -noverb]} {set objInfo(prog,verbose) 0}

   if {[flagLoc $ARGV -grid]}   {set objInfo(prog,gridFlag) 1}
   if {[flagLoc $ARGV -nogrid]} {set objInfo(prog,gridFlag) 0}
   if {[flagLoc $ARGV -lab]}    {set objInfo(prog,labFlag)  1}
   if {[flagLoc $ARGV -nolab]}  {set objInfo(prog,labFlag)  0}

   if {[flagLoc $ARGV -free]}   {set objInfo(prog,freeFlag) 1}
   if {[flagLoc $ARGV -nofree]} {set objInfo(prog,freeFlag) 0}

   if {[flagLoc $ARGV -start]}   {set objInfo(prog,autoStartFlag) 1}
   if {[flagLoc $ARGV -nostart]} {set objInfo(prog,autoStartFlag) 0}

   if {[flagLoc $ARGV -peak]}   {set objInfo(prog,pkFlag) 1}
   if {[flagLoc $ARGV -nopeak]} {set objInfo(prog,pkFlag) 0}

   if {[flagLoc $ARGV -tabView]}   {set objInfo(prog,tabViewFlag) 1}
   if {[flagLoc $ARGV -notabView]} {set objInfo(prog,tabViewFlag) 0}

   if {[flagLoc $ARGV -align]} \
      {
       set objInfo(data,alignX) X
       set objInfo(data,alignY) Y
      }

   getListArgD $ARGV -tab objInfo(peak,inList)

   getArgD $ARGV -xSize  objInfo(prog,defXSize)
   getArgD $ARGV -ySize  objInfo(prog,defYSize)

   initSpecColors

   getArgD $ARGV -fhi  objInfo(data,fhi1)
   getArgD $ARGV -lhi  objInfo(data,lhi1)
   getArgD $ARGV -lw1d objInfo(data,lw1d)

   getArgD $ARGV -color1 objInfo(color,color1)
   getArgD $ARGV -color2 objInfo(color,color2)

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -color$i objInfo(color,color$i)
        getArgD $ARGV -color$i objInfo(color,color$i)

        set objInfo(data,fhi$i) $objInfo(data,fhi1)
        set objInfo(data,lhi$i) $objInfo(data,lhi1)

        getArgD $ARGV -fhi$i objInfo(data,fhi$i)
        getArgD $ARGV -lhi$i objInfo(data,lhi$i)
       }

   getArgD $ARGV -peakColor     objInfo(color,peak)
   getArgD $ARGV -bgColor       objInfo(color,bg)
   getArgD $ARGV -winColor      objInfo(color,winBG)
   getArgD $ARGV -onColor       objInfo(color,on)
   getArgD $ARGV -shadowColor   objInfo(color,shadow)
   getArgD $ARGV -labelColor    objInfo(color,label)
   getArgD $ARGV -zoomColor     objInfo(color,zoom)
   getArgD $ARGV -cursorColor   objInfo(color,cursor)
   getArgD $ARGV -gridColor     objInfo(color,grid)
   getArgD $ARGV -boxColor      objInfo(color,box)
   getArgD $ARGV -proposeColor  objInfo(color,propose)
   getArgD $ARGV -acceptColor   objInfo(color,accept)
   getArgD $ARGV -statColor     objInfo(color,stat)
   getArgD $ARGV -statFillColor objInfo(color,statFill)

   if {"$objInfo(prog,exportProc)" != "None"} \
      {
       append objInfo(prog,name) " ($objInfo(prog,exportProc))"

       set objInfo(prog,title)   \
        "$objInfo(prog,name) $objInfo(prog,version) (C) F. Delaglio 2002"
      }
}

proc freeThisID { idList } \
{
   global objInfo

   foreach id $idList \
      {
       lappend objInfo(prog,freeIDList) $id
      }
}

proc getNextID {} \
{
   global objInfo

   if {$objInfo(prog,reuseFlag) && [llength $objInfo(prog,freeIDList)]} \
      {
       set id [lindex $objInfo(prog,freeIDList) 0]
       set objInfo(prog,freeIDList) [lrange $objInfo(prog,freeIDList) 1 end]

       showVerb "Reusing ID $id."

       return $id
      }

   set  id $objInfo(prog,nextID)
   incr objInfo(prog,nextID)

   return $id
}

proc initPeakTable {} \
{
   global objInfo

   if {!$objInfo(prog,tabViewFlag)} return

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      { 
       set tabName [lindex $objInfo(peak,inList) [expr $i - 1]]

       if {$objInfo(peak,tab$i)} continue
       if {"$tabName" == "None"} continue
       if {"$tabName" == ""}     continue

       if {![file exists $tabName]} continue 

       showVerb "Reading table $tabName ..."

       set error \
          [catch {set objInfo(peak,tab$i) \
               [gdbRead table -in $tabName -name "Peak table $i"]}]

       set objInfo(peak,pkCount$i) \
           [gdbGetInfo $objInfo(peak,tab$i) -parms entryCount]
      }
}

proc initSpectralData {} \
{
   global objInfo

   set obj data

   set objInfo($obj,valid)    0
   set objInfo($obj,fdata)    0
   set objInfo($obj,rdata)    0
   set objInfo($obj,xSize)    0

   set objInfo($obj,roi1)     0
   set objInfo($obj,roi2)     0

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo($obj,roi$i) 0
      }

   if {![strcasecmp $objInfo($obj,inName1) None]} {return -1} 
   if {![file exists $objInfo($obj,inName1)]}     {return -1} 
   if {[file isdirectory $objInfo($obj,inName1)]} {return -1} 

#
# Read data header.
# Extract size parameters:

   set error [rdFDATA $objInfo($obj,inName1) fdata]

   if {$error} \
      {
       showError view1D.tcl "Error reading file $objInfo($obj,inName1)"
       exit
      }

   set objInfo($obj,fdata)  $fdata
   set objInfo($obj,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]

   set objInfo($obj,hDim)  CUR_XDIM

   set objInfo(peak,xName) X_PPM
   set objInfo(peak,xAxis) X_AXIS

   set objInfo($obj,xAxis) [getParmStr $fdata NDLABEL $objInfo($obj,hDim)]

   set objInfo($obj,xSize) \
       [expr int([getParm $fdata NDSIZE $objInfo($obj,hDim)])]

   set objInfo($obj,ix)    [expr int($objInfo($obj,xSize)/2)]

   setSpecZoom $obj $fdata 1 $objInfo($obj,xSize) 0.0 0.0

#
# Extract noise levels:

   readView $obj 

   busyView hold .

   set zeroCount 0

   for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
      {
       if {$objInfo($obj,roi$i) != 0 && \
             $objInfo($obj,fhi$i) == 0.0 && \
               $objInfo($obj,lhi$i) == 0.0} \
          {
           showVerb \
            "Setting drawing heights for ROI $objInfo($obj,roi$i)."

           set roiInfo [getROIInfo -roi $objInfo($obj,roi$i)]
           set xSize   [lindex [lindex $roiInfo 2] 0]
           set n       [expr $xSize/5]
           set nn      $xSize

           incr zeroCount

           if {"$obj" != "data"} {set objInfo($obj,fdata)  [lindex $roiInfo 1]}

           set objInfo($obj,rdata)    [lindex $roiInfo 0] 
           set objInfo($obj,noise$i)  [vEstNoise $objInfo($obj,rdata) $n]

           set objInfo($obj,minVal$i) [vMin $objInfo($obj,rdata) $nn]
           set objInfo($obj,maxVal$i) [vMax $objInfo($obj,rdata) $nn]

           set r [expr 0.05*($objInfo($obj,maxVal$i) - $objInfo($obj,minVal$i))]

           if {$objInfo($obj,fhi$i) == 0.0 && $objInfo($obj,lhi$i) == 0.0} \
              {
               set objInfo($obj,fhi$i) [expr $objInfo($obj,minVal$i) - $r]
               set objInfo($obj,lhi$i) [expr $objInfo($obj,maxVal$i) + $r]

               set objInfo($obj,fhi$i) [format %e $objInfo($obj,fhi$i)]
               set objInfo($obj,lhi$i) [format %e $objInfo($obj,lhi$i)]
               }

           showVerb \
            "$objInfo($obj,inName$i) $objInfo($obj,fhi$i) $objInfo($obj,lhi$i)"
          }
      }

   setSpecZoom $obj $fdata \
    1 $objInfo($obj,xSize) $objInfo($obj,fhi1) $objInfo($obj,lhi1)

   if {$zeroCount == $objInfo(data,specCount)} \
      {
       for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
          {
           set objInfo($obj,fhi$i) $objInfo($obj,fhi1)
           set objInfo($obj,lhi$i) $objInfo($obj,lhi1)

           set fhi $objInfo($obj,fhi$i)
           set lhi $objInfo($obj,lhi$i)

           showVerb "Reset $objInfo($obj,inName$i) Hi: $fhi $lhi"
          }
      }

    busyView release .

    return 0
}

proc resizeView args \
{
   global objInfo

   set obj  [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   showVerb "Resize event $args"

   xupdate

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,window)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,window)]

   set objInfo($obj,cellXSize) \
       [expr $objInfo($obj,winXSize)     - \
             ($objInfo($obj,leftMargin)  + \
              $objInfo($obj,rightMargin) + \
              $objInfo($obj,winXAdj))]

   set objInfo($obj,cellYSize) \
       [expr $objInfo($obj,winYSize)      - \
             ($objInfo($obj,topMargin)    + \
              $objInfo($obj,bottomMargin) + \
              $objInfo($obj,winYAdj))]

   set objInfo($obj,xSize) \
       [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

   set objInfo($obj,ySize) \
       [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

   if {$objInfo($obj,xSize)     < 50} {set objInfo($obj,xSize) 50}
   if {$objInfo($obj,ySize)     < 50} {set objInfo($obj,ySize) 50}
   if {$objInfo($obj,cellXSize) < 50} {set objInfo($obj,cellXSize) 50}
   if {$objInfo($obj,cellYSize) < 50} {set objInfo($obj,cellYSize) 50}

   $objInfo($obj,canvas) delete pixmap -tags $obj

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
      -win $objInfo($obj,pixmap) -tags $obj

   set objInfo($obj,x1) $objInfo($obj,leftMargin)
   set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
   set objInfo($obj,y1) $objInfo($obj,topMargin)
   set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

   set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
   set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

   set objInfo($obj,x1Pix)        $xPix
   set objInfo($obj,y1Pix)        $yPix
   set objInfo($obj,xnPix)        $xPix
   set objInfo($obj,ynPix)        $yPix

   defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
              -x $objInfo($obj,x1) $objInfo($obj,xn) \
              -y $objInfo($obj,y1) $objInfo($obj,yn)

   $objInfo($obj,canvas) raise cursorH
   $objInfo($obj,canvas) raise cursorV

   $objInfo($obj,redrawProc) $obj resize 

   bind $objInfo($obj,canvas) <Configure> \
    "$objInfo($obj,resizeProc) $obj %w %h %T"
}

proc initMainWindow {} \
{
   global objInfo env

   wm title . "$objInfo(prog,name) \[Zoom\] \[Scale\] \[Offset\]"

    set colorList \
       {
        Multi
        Black        Red          Orange       Yellow
        Green        Blue         Violet       Cyan
        Magenta      LightGray    DimGray      White
       }

   frame .frameL
   frame .frameR
   frame .frameB
   
#
# Logo:

   catch {image delete pipeImage}

   image create photo pipeImage -file $env(NMRTXT)/frank.gif

   button .viewLogo -relief sunken -image pipeImage -background black \
          -command "showHelp logo"

   button .start    -text Start -command startView
   button .exit     -text Exit  -command quitAllView

   pack .viewLogo -in .frameL -side top -expand 1 -fill both 

#
# File Parameters:

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       frame .frame$i

       if {$objInfo(data,specCount) == 1} \
          {
           set str  "File:"
          } \
       else \
          {
           set str  "File $i:" 
          }

       label  .fileLab$i  -text $str -width 8 -anchor w
       entry  .fileName$i -width 32 -textvariable objInfo(data,inName$i)

       button .fileButt$i -bitmap @$env(NMRTXT)/caret.map \
                          -command "selectFile $i"

       bind   .fileName$i <Return> "+ setFile"

       label  .colorLab$i  -text "Color:"
       entry  .colorName$i -width 10 -textvariable objInfo(color,color$i)

       menubutton .colorButt$i -bitmap @$env(NMRTXT)/caret.map \
                  -relief raised -menu .colorButt$i.menu

#
# Color choice menus:

       menu .colorButt$i.menu

       foreach color $colorList \
          {
           .colorButt$i.menu add command -label $color \
              -command "set objInfo(color,color$i) $color"
          }

       pack .fileLab$i  .fileName$i  .fileButt$i       \
            .colorLab$i .colorName$i .colorButt$i      \
            -side left -padx 1m -pady 1m -in .frame$i

       pack .frame$i -in .frameR -side top -expand 1 -fill x 
      }

   pack .start .exit -in .frameB    -side left -expand 1 -fill x
   pack .frameB -in .frameR -side bottom -expand 1 -fill x
   pack .frameL .frameR -side left -fill both

   wm geom . +5-5
}

proc init1DWindow { parent sx1 sxn sy1 syn } \
{
    global objInfo

    set id  [getNextID]
    set obj win1D_$id
    set w   .win1D_$id

    showVerb "Initializing window $id"

    toplevel $w

    incr objInfo(prog,winCount)

    wm title $w "$objInfo(prog,title)  \[Zoom\] \[Scale\] \[Offset\]"

    lappend objInfo(prog,winList) $obj

    set objInfo($obj,parent)       $parent
    set objInfo($obj,winXSize)     $objInfo(prog,defXSize)
    set objInfo($obj,winYSize)     $objInfo(prog,defYSize) 

    if {$objInfo(prog,labFlag)} \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,rightMargin)   65 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin) 120
       } \
    else \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,rightMargin)   65 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin)  70
       }

    set objInfo($obj,winXAdj)        5 
    set objInfo($obj,winYAdj)       20 

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]
 
    set objInfo($obj,xSize) \
        [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

    set objInfo($obj,ySize) \
        [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

    set objInfo($obj,window)       $w
    set objInfo($obj,canvas)       $w.c
    set objInfo($obj,scrollH)      $w.sh
    set objInfo($obj,scrollV)      $w.sv
    set objInfo($obj,pixmap)       $id 
    set objInfo($obj,cell)         $id 

    set objInfo($obj,resizeProc)   resizeView
    set objInfo($obj,redrawProc)   showView

    set objInfo($obj,x1) $objInfo($obj,leftMargin)
    set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
    set objInfo($obj,y1) $objInfo($obj,topMargin)
    set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

    set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
    set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

    set objInfo($obj,x1Pix)        $xPix
    set objInfo($obj,y1Pix)        $yPix
    set objInfo($obj,xnPix)        $xPix
    set objInfo($obj,ynPix)        $yPix
    set objInfo($obj,x0Pix)        -1 
    set objInfo($obj,y0Pix)        -1

    set objInfo($obj,specCount)    $objInfo($parent,specCount)
    set objInfo($obj,inList)       $objInfo($parent,inList)

    set n $objInfo($parent,specCount)
 
    if {$n < 2} {set n 2}

    for {set i 1} {$i <= $n} {incr i} \
       {
        set objInfo($obj,fhi$i)     $objInfo($parent,fhi$i)
        set objInfo($obj,lhi$i)     $objInfo($parent,lhi$i)
        set objInfo($obj,inName$i)  $objInfo($parent,inName$i)

        set objInfo($obj,p0,$i)     0.0
        set objInfo($obj,p1,$i)     0.0
        set objInfo($obj,pivot,$i)  0.0
       }

    set objInfo($obj,xAxis)        $objInfo($parent,xAxis)

    set objInfo($obj,specX1)       $sx1
    set objInfo($obj,specXN)       $sxn
    set objInfo($obj,specY1)       $sy1
    set objInfo($obj,specYN)       $syn

    set objInfo($obj,fhi1)         $sy1
    set objInfo($obj,lhi1)         $syn

    set objInfo($obj,newWindow)    1

    wm title \
     $objInfo($obj,window) "$objInfo(prog,title) \[Zoom\] \[Scale\] \[Offset\]"

    frame $w.top
    frame $w.phase
    frame $w.bottom

    button $w.fullH  -text "Full Horiz" -command "fullView  $obj H"
    button $w.fullV  -text "Full Vert"  -command "fullView  $obj V"
    button $w.scaleV -text "Scale"      -command "scaleView $obj scale"

    if {"$objInfo(prog,exportProc)" != "None"} \
       {
        button $w.export -text "Export" -command "exportView $obj"
       }

    button $w.quit -text "Quit" -command "quitThisView  $obj"

    label  $w.mode -text "    Zoom:"

    radiobutton $w.new -text "New Window"   -value 1 \
       -variable objInfo($obj,newWindow) -anchor w

    radiobutton $w.same -text "Same Window" -value 0 \
       -variable objInfo($obj,newWindow) -anchor w

    label  $w.fhiLab   -text "First:"
    entry  $w.fhi      -textvariable objInfo($obj,fhi1) -width 12
    button $w.fhiIncr  -text "+" -padx 1 -pady 1 -command "incrHi $obj 1 fhi"
    button $w.fhiDecr  -text "-" -padx 1 -pady 1 -command "decrHi $obj 1 fhi"
    bind   $w.fhi      <Return> "+ setHi $obj 1"

    label  $w.lhiLab   -text "Last:"
    entry  $w.lhi      -textvariable objInfo($obj,lhi1) -width 12
    button $w.lhiIncr  -text "+" -padx 1 -pady 1 -command "incrHi $obj 1 lhi"
    button $w.lhiDecr  -text "-" -padx 1 -pady 1 -command "decrHi $obj 1 lhi"
    bind   $w.lhi      <Return> "+ setHi $obj 1"

    set color $objInfo(color,color1) 

    set objInfo($obj,activeSpec) "1 ($color)"

    label  $w.specLab  -text "Spectrum:"
    entry  $w.spec     -textvariable objInfo($obj,activeSpec) -width 10
    button $w.specIncr -text "+" -padx 1 -pady 1 -command "incrActiveSpec $obj"
    button $w.specDecr -text "-" -padx 1 -pady 1 -command "decrActiveSpec $obj"
    bind   $w.spec     <Return> "+ setActivSpec $obj"

    label $w.p0Lab -text Phase:
    entry $w.p0    -textvariable objInfo($obj,p0,1) -width 10

    scale $w.p0Scale \
     -resolution 1.0 -showvalue 0 -length 361 -from -180 -to 180 \
     -orient horiz -command "setPhase $obj p0" -variable objInfo($obj,p0,1) 

    if {"$objInfo(prog,exportProc)" != "None"} \
       {
        pack $w.fullH $w.fullV $w.export $w.quit $w.mode $w.new $w.same \
             -in $w.top -side left 
       } \
    else \
       { 
        pack $w.fullH $w.fullV $w.scaleV $w.quit $w.mode $w.new $w.same \
             -in $w.top -side left 
       }

    pack $w.lhiDecr $w.lhiIncr $w.lhi $w.lhiLab -in $w.top -side right
    pack $w.fhiDecr $w.fhiIncr $w.fhi $w.fhiLab -in $w.top -side right

    pack $w.p0Lab $w.p0 -in $w.phase -side left
    pack $w.p0Scale -in $w.phase -side left -expand 1 -fill x

    pack $w.specLab $w.spec $w.specIncr $w.specDecr \
         -in $w.phase -side left -anchor c

#
# Canvas and scrollbars:

    scrollbar $objInfo($obj,scrollV) \
       -orient vert -command "$objInfo($obj,canvas) yview"

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize)      \
       -xscrollcommand "$objInfo($obj,scrollH) set"                  \
       -yscrollcommand "$objInfo($obj,scrollV) set"                  \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)" \
       -closeenough 4 

    set dx [expr 5*$objInfo(prog,winCount)]
    set dy [expr 5*$objInfo(prog,winCount)]

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)+$dx+$dy

    xupdate

    pack $objInfo($obj,scrollV) -in $w.bottom -side right  -expand 0 -fill y 
    pack $objInfo($obj,scrollH) -in $w.bottom -side bottom -expand 0 -fill x
    pack $objInfo($obj,canvas)  -in $w.bottom -side left   -expand 1 -fill both

    pack $w.top    -side top -expand 0 -fill x
    pack $w.phase  -side top -expand 0 -fill x
    pack $w.bottom -side top -expand 1 -fill both

    xupdate

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1) $objInfo($obj,xn) \
               -y $objInfo($obj,y1) $objInfo($obj,yn)

    set qx [expr $objInfo($obj,x1) - 4] 
    set qy [expr $objInfo($obj,y1) - 4] 

    set idH [$objInfo($obj,canvas) create line     \
       $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy \
       -fill $objInfo(color,cursor) -tags cursorH]

    set idV [$objInfo($obj,canvas) create line     \
       $qx $objInfo($obj,y1) $qx $objInfo($obj,yn) \
       -fill $objInfo(color,cursor) -tags cursorV]

    readView $obj
    showView $obj init

    bind $objInfo($obj,canvas) <Configure> \
         "$objInfo($obj,resizeProc) $obj %w %h %T"

    bind $objInfo($obj,canvas) <ButtonPress-1>   "mouseON    $obj %x %y B1"
    bind $objInfo($obj,canvas) <ButtonRelease-1> "mouseDone  $obj %x %y B1"
    bind $objInfo($obj,canvas) <B1-Motion>       "trackMouse $obj %x %y B1"

    bind $objInfo($obj,canvas) <ButtonPress-2>   "mouseON    $obj %x %y B2"
    bind $objInfo($obj,canvas) <ButtonRelease-2> "mouseDone  $obj %x %y B2"
    bind $objInfo($obj,canvas) <B2-Motion>       "trackMouse $obj %x %y B2"

    bind $objInfo($obj,canvas) <ButtonPress-3>   "mouseON    $obj %x %y B3"
    bind $objInfo($obj,canvas) <ButtonRelease-3> "mouseDone  $obj %x %y B3"
    bind $objInfo($obj,canvas) <B3-Motion>       "trackMouse $obj %x %y B3"

#   bind $objInfo($obj,canvas) <B1-Motion>       "moveCursor $obj %x %y"
}

proc initPeakWindow {} \
{
   global objInfo

   set i 1

   if {!$objInfo(prog,tabViewFlag)} return

   if {!$objInfo(peak,tab$i)}    return
   if {[winfo exists .peakView]} return

   set objInfo(peak,viewer) .peakView
   set objInfo(peak,vList)  ""

   set firstList  "INDEX X_PPM HEIGHT"
   set nameList   $firstList

   set vNameList [gdbGetInfo $objInfo(peak,tab$i) -parms varNames]

   foreach vName $vNameList \
      {
       if {[lsearch -exact $firstList $vName] < 0} \
          {
           lappend nameList $vName
          }
      }

   foreach vName $nameList \
      {
       lappend objInfo(peak,vList) [gdbFindByName $objInfo(peak,tab$i) $vName]
      }

  showTable -tab $objInfo(peak,tab$i)  -var $nameList -win $objInfo(peak,viewer)

  wm geom $objInfo(peak,viewer) -5-5
}

proc setFile {} \
{
   global objInfo

   initSpecColors
}

proc selectFile { id } \
{
   global objInfo

   set obj data

    set fileTypes \
       {
        { {All Files}        *      }
        { {ft  Files}        {.ft}  }
        { {ft1 Files}        {.ft1} }
        { {ft2 Files}        {.ft2} }
        { {ft3 Files}        {.ft3} }
        { {fid Files}        {.fid} }
        { {dat Files}        {.dat} }
        { {DAT Files}        {.DAT} }
       }

   set objInfo($obj,inName$id) \
     [tk_getOpenFile -title "Spectrum $id" -filetypes $fileTypes]

   if {"$objInfo($obj,inName$id)" == ""} {set objInfo($obj,inName$id) "None"}

   set objInfo($obj,inList) ""

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       lappend objInfo($obj,inList) $objInfo($obj,inName$i)
      }

   initSpecColors
}

proc readView args \
{
   global objInfo

   set obj [lindex $args 0]
   set n   [llength $args]

   busyView hold $objInfo($obj,window)

   set nn $objInfo(data,specCount)

   if {$nn < 2} {set nn 2}

   for {set i 1} {$i <= $nn} {incr i} \
      {
       set objInfo($obj,roi$i) 0
       set roiID($i)           0
      }

   for {set i 1} {$i < $n} {incr i} \
      {
       set roiID($i) [lindex $args $i]
      }

   set hDim X_AXIS

   set objInfo($obj,tauList)  ""
   set objInfo(data,roiCount) 0

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {![strcasecmp $objInfo($obj,inName$i) None]} continue
       if {![file exists $objInfo($obj,inName$i)]} continue
       if {[file isdirectory $objInfo($obj,inName$i)]} continue
 
       if {!$roiID($i)} {set roiID($i) [getNextID]}

       showVerb "Reading ROI $roiID($i) $objInfo($obj,inName$i) ..."

       set objInfo($obj,roi$i) $roiID($i)

       if {$i == 1} \
          {
           readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) \
            -x $hDim $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm \
            -$objInfo(data,realFlag)
          } \
       else \
          {
           readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) \
            -x $hDim $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm \
            -center $objInfo(data,alignX) -$objInfo(data,realFlag)
          }

       incr objInfo(data,roiCount)

       set roiInfo [getROIInfo -roi $roiID($i)]
       set rdata   [lindex $roiInfo 0]
       set fdata   [lindex $roiInfo 1]
       set sList   [lindex $roiInfo 2]
       set size    [lindex $sList 0]

       vsMult $rdata $objInfo(data,scale$i) $size

       lappend objInfo($obj,tauList) [getParm $fdata FDTAU NULL_DIM]

       if {$i == 1 && [strcmp $obj data]} {set objInfo($obj,fdata) $fdata}
      }

   foreach stat $objInfo(data,statList) \
      {
       set objInfo($obj,roi$stat) [getNextID] 

       readROI -in $objInfo($obj,inName1) -ndim 1 -roi $objInfo($obj,roi$stat) \
        -x $hDim $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm \
        -$objInfo(data,realFlag)
      }

   showVerb "Done Reading ROIs."

   busyView release $objInfo($obj,window)
}

proc updateROIStats { obj } \
{
   global objInfo

   foreach stat $objInfo(data,statList) \
      {
       if {!$objInfo($obj,roi$stat)} continue

       switch $stat \
          {
           Diff {
                 set i [lindex $objInfo(data,drawOrder) 0]
                 set j [lindex $objInfo(data,drawOrder) 1]

                 if {!$objInfo($obj,roi$i) || !$objInfo($obj,roi$j)} continue

                 set rInfoI [getROIInfo -roi $objInfo($obj,roi$i)]
                 set rInfoJ [getROIInfo -roi $objInfo($obj,roi$j)]
                 set rInfoS [getROIInfo -roi $objInfo($obj,roi$stat)]

                 set rdataI [lindex $rInfoI]
                 set rdataJ [lindex $rInfoJ]
                 set rdataS [lindex $rInfoS]

                 set nI     [lindex [lindex $rInfoI 2] 0]
                 set nJ     [lindex [lindex $rInfoJ 2] 0]
                 set nS     [lindex [lindex $rInfoS 2] 0]

                 set n [getMin "$nI $nJ $nS"]

                 vvSub $rdataS $rdataI $rdataJ $n
                }
          }
      }
}

proc setActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 0} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc incrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   incr i

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc decrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   incr i -1

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc incrHi { obj mode v } \
{
   global objInfo

   if {"$mode" == "active"} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,$v$i) [expr $objInfo($obj,$v$i)*$objInfo($obj,factor)]
       set objInfo($obj,$v$i) [format %e $objInfo($obj,$v$i)]
      } \
   else \
      {
       set objInfo($obj,$v)   [expr $objInfo($obj,fhi)*$objInfo($obj,factor)]
       set objInfo($obj,$v)   [format %e $objInfo($obj,$v)]
      }

   showView $obj incrHi
}

proc decrHi { obj mode v } \
{
   global objInfo

   if {"$mode" == "active"} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,$v$i) [expr $objInfo($obj,$v$i)/$objInfo($obj,factor)]
       set objInfo($obj,$v$i) [format %e $objInfo($obj,$v$i)]
      } \
   else \
      {
       set objInfo($obj,$v)   [expr $objInfo($obj,fhi)/$objInfo($obj,factor)]
       set objInfo($obj,$v)   [format %e $objInfo($obj,$v)]
      }

   showView $obj decrHi
}

proc setHi { obj mode } \
{
   global objInfo

   showView $obj setHi
}

proc setPhase { obj mode val } \
{
   global objInfo

}

proc fullView { obj mode } \
{
   global objInfo


   if {![strcmp $mode H]} \
      {
       set sx1 1
       set sxn $objInfo(data,xSize)
       set fHi Same
       set lHi Same
      } \
   else \
      {
       set roiInfo [getROIInfo -roi $objInfo($obj,roi1)]
       set rdata   [lindex $roiInfo 0]
       set xSize   [lindex [lindex $roiInfo 2] 0]
 
       set maxVal [vMax $rdata $xSize]
       set minVal [vMin $rdata $xSize]

       set r      [expr 0.05*($maxVal - $minVal)]

       set sx1 Same
       set sxn Same

       set fHi [expr $minVal - $r]
       set lHi [expr $maxVal + $r]
      }

   setSpecZoom $obj $objInfo(data,fdata) $sx1 $sxn $fHi $lHi

   readView $obj
   showView $obj fullView 
}

proc scaleView { obj who } \
{
   global objInfo

   if {$objInfo(data,specCount) < 2} return

   set i [lindex $objInfo(data,drawOrder) 0]
   set j [lindex $objInfo(data,drawOrder) 1]

   set rInfoI [getROIInfo -roi $objInfo($obj,roi$i)]
   set rInfoJ [getROIInfo -roi $objInfo($obj,roi$j)]

   set rdataI [lindex $rInfoI]
   set rdataJ [lindex $rInfoJ]

   set nI     [lindex [lindex $rInfoI 2] 0]
   set nJ     [lindex [lindex $rInfoJ 2] 0]

   set n [getMin "$nI $nJ"]

   set objInfo(data,scale$j) [vvGetScale $rdataI $rdataJ $n]

showVerb "Scale $i $j: $objInfo(data,scale$j)"

   foreach obj $objInfo(prog,winList) \
      {
       readView $obj
       updateROIStats $obj
       showView2 $obj $who
      }
}

proc showViewAll { who } \
{
   global objInfo

   foreach obj $objInfo(prog,winList) \
      {
       updateROIStats $obj
       showView2 $obj $who
      } 
}

proc showView { obj who } \
{
   catch "showView2 $obj $who"
}

proc showView2 { obj who } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,window)]} return

   busyView hold $objInfo($obj,window)

   set objInfo(prog,updateFlag) 0

   clearView $obj

   if {$objInfo(prog,gridFlag)} \
      {
       set xGrid xGrid
       set yGrid yGrid
      } \
   else \
      {
       set xGrid noxGrid
       set yGrid noyGrid
      }

   if {$objInfo($obj,roi1)} \
      {
       set fHi $objInfo($obj,fhi1)
       set lHi $objInfo($obj,lhi1)

       drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -roi $objInfo($obj,roi1) -fhi $fHi -lhi $lHi         \
               -bg $objInfo(color,bg)                    \
               -fg $objInfo(color,color1)                \
               -fillColor $objInfo(color,fillColor1)     \
               -colorA $objInfo(color,label)             \
               -colorS $objInfo(color,shadow)            \
               -$xGrid -$yGrid -lw1d $objInfo(data,lw1d) \
               -axisOnly -axis Bottom -nobox
      }

   set j 1

   foreach i [lreverse $objInfo(data,drawOrder)] \
      {
       if {$objInfo($obj,roi$i) && $objInfo(data,drawFlag$i)} \
          {
           set fHi $objInfo($obj,fhi1)
           set lHi $objInfo($obj,lhi1)

           showVerb "Drawing ROI $objInfo($obj,roi$i) $fHi $lHi for $who"

           showVerb \
            "Mouse: $objInfo(prog,B1) $objInfo(prog,B2) $objInfo(prog,B3)"

           if {$j == 1} \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                       -roi $objInfo($obj,roi$i) -fhi $fHi -lhi $lHi        \
                       -bg $objInfo(color,bg)                 \
                       -fg $objInfo(color,color$i)            \
                       -fillColor $objInfo(color,fillColor$i) \
                       -colorA $objInfo(color,label)          \
                       -colorS $objInfo(color,shadow)         \
                       -$xGrid -$yGrid -lw1d $objInfo(data,lw1d) -nobox
              } \
           else \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap)   \
                       -roi $objInfo($obj,roi$i) -fhi $fHi -lhi $lHi -bg None \
                       -fg $objInfo(color,color$i)             \
                       -fillColor  $objInfo(color,fillColor$i) \
                       -colorA $objInfo(color,label)           \
                       -colorS $objInfo(color,shadow)          \
                       -$xGrid -$yGrid -lw1d $objInfo(data,lw1d) -nobox
              }
         }

       incr j
      }

   foreach stat $objInfo(data,statList) \
      {
       if {$objInfo($obj,roi$stat)} \
          {
           set fHi $objInfo($obj,fhi1)
           set lHi $objInfo($obj,lhi1)

           showVerb "Drawing ROI $objInfo($obj,roi$stat) $fHi $lHi for $who"

           drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                   -roi $objInfo($obj,roi$stat) -fhi $fHi -lhi $lHi -bg None \
                   -fg         $objInfo(color,stat)           \
                   -fillColor  $objInfo(color,statFill)       \
                   -colorA $objInfo(color,label)              \
                   -colorS $objInfo(color,shadow)             \
                   -$xGrid -$yGrid -lw1d $objInfo(data,lw1d) -nobox
          }
      }

   drawBox -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
           -x 0% 100% -y 0% 100% -fg $objInfo(color,label)

   busyView release $objInfo($obj,window)

   drawPeaks
   drawCursors
   drawLabelItems

   set objInfo(prog,updateFlag) 1

   xupdate
}

proc drawThisItem { item } \
{
   global objInfo stdfont

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue
     
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID  $objInfo($obj,pixmap)

       set cInfo [getCellInfo -cell $cellID -win $winID]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set kx1   [lindex $xInfo 0]
       set kxn   [lindex $xInfo 1]
       set ky1   [lindex $yInfo 0]
       set kyn   [lindex $yInfo 1]

       showVerb "Label Cell $cellID ROI $roiID: $kx1,$ky1 $kxn,$kyn"
 
       set type [lindex $item 0]
       set id   [lindex $item 1]

       set iID  None

       switch $type \
          {
           box  {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [lindex $item 7]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x1}ppm]

                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${xn}ppm]

                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y1}ppm]

                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${yn}ppm]

                 set iID \
                  [$objInfo($obj,canvas) create rectangle \
                    $ix1 $iy1 $ixn $iyn -outline $fg -width $w -tags labelItem]
                }

           line {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [lindex $item 7]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x1}ppm]

                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${xn}ppm]

                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y1}ppm]

                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${yn}ppm]

                 set iID \
                  [$objInfo($obj,canvas) create line \
                    $ix1 $iy1 $ixn $iyn -fill $fg -width $w -tags labelItem]
                }

           text {
                 set x0 [lindex $item 2]
                 set y0 [lindex $item 3]
                 set w  [lindex $item 4]
                 set a  [lindex $item 5]
                 set fg [lindex $item 6]
                 set s  [lrange $item 7 end]

                 if {$x0 > $objInfo(specX1)} continue
                 if {$x0 < $objInfo(specXN)} continue
                 if {$y0 > $objInfo(specY1)} continue
                 if {$y0 < $objInfo(specYN)} continue

                 set ix0 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x0}ppm]

                 set iy0 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y0}ppm]

                 set iID \
                  [$objInfo($obj,canvas) create text \
                    $ix0 $iy0 -fill $fg -width $w -anchor $a -text $s \
                    -font $stdfont -tags labelItem]
                }

           label {
                  set x0 [lindex $item 2]
                  set y0 [lindex $item 3]
                  set w  [lindex $item 4]
                  set a  [lindex $item 5]
                  set fg [lindex $item 6]
                  set s  [lrange $item 7 end]

                  set ix0 \
                   [spec2pix -roi $roiID -cell $cellID -win $winID -x $x0]

                  set iy0 \
                   [spec2pix -roi $roiID -cell $cellID -win $winID -y $y0]

                  set iID \
                   [$objInfo($obj,canvas) create text \
                    $ix0 $iy0 -fill $fg -width $w -anchor $a -text $s \
                    -font $stdfont -tags labelItem]
                 }
          }

       if {![strcasecmp $iID None]} continue

       $objInfo($obj,canvas) bind $iID <ButtonRelease-1> \
        "+ selLabelItem $id $iID 1"
      }

   xupdate
}

proc selLabelItem { id iID bID } \
{
   global objInfo

   showVerb "Selected $id Canvas Item $iID Button $bID"

   set sList [split $id _]
   set n     [llength $sList]
   set s     [lindex $sList 0]

   if {![strcmp $s title] && $n == 2} \
      {
       set i   [lindex $sList 1]
       set loc [lsearch -exact $objInfo(data,drawOrder) $i]
      
       if {$loc >= 0} \
          {
           set oList [lswap $objInfo(data,drawOrder) $loc [expr $loc - 1]]
           set objInfo(data,drawOrder) $oList

           showViewAll selLabel
           drawTitles
          }
      } 
}

proc addLabelItem { item } \
{
   global objInfo

   showVerb "Label item: $item"

   lappend objInfo(prog,labelItemList) $item

   drawThisItem $item
}

proc delLabelItem { idList } \
{
   global objInfo

   clearLabelItems

   set itemList ""

   foreach item $objInfo(prog,labelItemList) \
      {
       set thisID [lindex $item 1]

       if {[lsearch -exact $idList $thisID] < 0} \
          {
           showVerb "Delete Labels; Keeping item:  $item"
           lappend itemList $item
          } \
       else \
          {
           showVerb "Delete Labels; Deleting item: $item"
          } 
      }

   set objInfo(prog,labelItemList) $itemList

   drawLabelItems
}

proc drawLabelItems {} \
{
   global objInfo

   foreach item $objInfo(prog,labelItemList) \
      {
       drawThisItem $item
      }
}

proc clearLabelItems {} \
{
   global objInfo

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,canvas)]} continue
       $objInfo($obj,canvas) delete labelItem
      }

   xupdate
}

proc getSpecClipX { obj x0 } \
{
   global objInfo

   if {$x0 > $objInfo($obj,specX1)} \
      {
       set x0 $objInfo($obj,specX1)
      }

   if {$x0 < $objInfo($obj,specXN)} \
      {
       set x0 $objInfo($obj,specXN)
      }

   return $x0
}

proc getSpecClipY { obj y0 } \
{
   global objInfo

   if {$y0 > $objInfo($obj,specY1)} \
      {
       set y0 $objInfo($obj,specY1)
      }

   if {$y0 < $objInfo($obj,specYN)} \
      {
       set y0 $objInfo($obj,specYN)
      }

   return $y0
}

proc exportView { obj } \
{
   global objInfo

   set sx1 $objInfo($obj,specX1) 
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   showVerb "Exporting Current Region to $objInfo(prog,exportProc)"

   $objInfo(prog,exportProc) $obj $sx1 $sxn $sy1 $syn
}

proc showHelp {} \
{
}

proc showVerb args \
{
   global objInfo

   if {$objInfo(prog,verbose)} {puts $args}
}

proc quitThisView { obj } \
{
   global objInfo

   showVerb "Quiting View $obj, Cell $objInfo($obj,cell)"

   catch [destroy $objInfo($obj,window)]

   set   winList ""

   foreach w $objInfo(prog,winList) \
      {
       if {"$w" != "$obj"} {lappend winList $w}
      } 

   set objInfo(prog,winList) $winList

   freeViewROI $obj
}

proc freeViewROI { obj } \
{
   global objInfo

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       if {$objInfo($obj,roi$i)} \
          {
           showVerb "Free ID $objInfo($obj,roi$i)."
     
           freeThisID $objInfo($obj,roi$i)
 
           if {$objInfo(prog,freeFlag)} {freeROI -roi $objInfo($obj,roi$i)}
          }
     }
}

proc startView {} \
{
   global objInfo

   initPeakTable
   initPeakWindow
   
   set status [initSpectralData]

   if {$status} \
      {
       puts stderr "View1D: No First Input Data was Found."
       exit 1
      }

   freeViewROI  data

   init1DWindow data \
     $objInfo(data,specX1) $objInfo(data,specXN) \
     $objInfo(data,specY1) $objInfo(data,specYN)
}

proc busyView { mode w } \
{
   if {[strcmp $w .]} \
      {
       busy $mode .
       busy $mode $w
      } \
   else \
      {
       busy $mode $w
      }

   xupdate
}

proc quitAllView {} \
{
   global objInfo

   if {"$objInfo(prog,exportProc)" != "None"} \
      {
       $objInfo(prog,exportProc) -- -quit
      }

   okExit exit
}

proc clearView { obj } \
{
   global objInfo

   showVerb "Clearing window $obj ..."

   $objInfo($obj,canvas) delete peak labelItem

   newScreen -win $objInfo($obj,pixmap)

   drawRec -x 0% 100% -y 0% 100% \
    -fg $objInfo(color,winBG) -win $objInfo($obj,pixmap)
}

proc setSpecZoom { obj fdata zx1 zxn zy1 zyn } \
{
   global objInfo

   if {[strcasecmp $zx1 Same]} \
      {
       set objInfo($obj,specX1) [pnt2spec $fdata $objInfo(data,hDim) $zx1 ppm]
      }

   if {[strcasecmp $zxn Same]} \
      {
       set objInfo($obj,specXN) [pnt2spec $fdata $objInfo(data,hDim) $zxn ppm]
      }

   if {[strcasecmp $zy1 Same]} \
      {
       set objInfo($obj,specY1) $zy1
       set objInfo($obj,fhi1)   $zy1
      }

   if {[strcasecmp $zyn Same]} \
      {
       set objInfo($obj,specYN) $zyn
       set objInfo($obj,lhi1)   $zyn
      }

   showVerb "Zoom Limits X: $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm"
   showVerb "Zoom Limits Y: $objInfo($obj,specY1) $objInfo($obj,specYN)"
}

proc mouseON { obj xPix yPix bName } \
{
   global objInfo

   set objInfo(prog,mouseON) 0

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

   set wf [focus]

   if {[strcmp .$obj $wf]} \
      {
       showVerb "Mouse Focus Change in $obj Focus: $wf"
       return
      }

   set objInfo(prog,mouseON) 1

   set objInfo($obj,x1Pix) $xPix
   set objInfo($obj,y1Pix) $yPix
   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   if {[llength $objInfo(prog,mouseList)]} \
      {
       showVerb "Reseting Mouse List."
      }

   set objInfo(prog,mouseList) ""

   showVerb "Mouse ON in $obj Focus: [focus]"

   switch $bName \
      {
       B1 {
           set objInfo(prog,B1) 1

           $objInfo($obj,canvas) create rectangle \
              $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
              $objInfo($obj,xnPix) $objInfo($obj,ynPix) \
              -outline $objInfo(color,zoom) -tags zoom

           xupdate
          }

       B2 {set objInfo(prog,B2) 1}
       B3 {set objInfo(prog,B3) 1}
     }

}

proc trackMouse { obj xPix yPix bName } \
{
   global objInfo

   if {!$objInfo(prog,mouseON)} return

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

   lappend objInfo(prog,mouseList) "$obj $xPix $yPix $bName"

   showVerb "Tracking Mouse Events: $objInfo(prog,mouseList)"
 
   switch $bName \
      {
       B3 {
           set r [expr 0.01*($objInfo($obj,lhi1) - $objInfo($obj,fhi1))]

           if {$yPix > $objInfo($obj,ynPix)} \
              {
               set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1) + $r]
               set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1) + $r]
              } \
           else \
              {
               set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1) - $r]
               set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1) - $r]
              }

           showView $obj B3
          }

       B2 {
           if {$yPix > $objInfo($obj,ynPix)} \
              {
               set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1)/0.95]
               set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1)/0.95]
              } \
           else \
              {
               set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1)*0.95]
               set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1)*0.95]
              }

           showView $obj B2
          }

       B1 {
           $objInfo($obj,canvas) coords zoom \
              $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) $xPix $yPix

           xupdate
          }
     }

   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix
}

proc mouseDone { obj xPix yPix bName } \
{
   global objInfo

   if {!$objInfo(prog,mouseON)} return

   showVerb "Done with Mouse Events."

   switch $bName \
      {
       B3 {
           set objInfo(prog,B3) 0
          }

       B2 {
           set objInfo(prog,B2) 0
          }

       B1 {
           set objInfo(prog,B1) 0

           if {$objInfo($obj,x1Pix) == $objInfo($obj,xnPix) &&
               $objInfo($obj,y1Pix) == $objInfo($obj,ynPix)} \
              {
               catch    "$objInfo($obj,canvas) delete zoom"
               showVerb "Empty 1D Region Rejected in View $obj"
               return
              }

           showVerb "1D Region Selected in View $obj"

           $objInfo($obj,canvas) delete zoom

           if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
              {
               set itemp               $objInfo($obj,x1Pix)
               set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
               set objInfo($obj,xnPix) $itemp
              }

           if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
              {
               set itemp               $objInfo($obj,y1Pix)
               set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
               set objInfo($obj,ynPix) $itemp
              }

           set roiID  $objInfo($obj,roi1)
           set cellID $objInfo($obj,cell)
           set winID  $objInfo($obj,pixmap)

           set fHi $objInfo($obj,fhi1)
           set lHi $objInfo($obj,lhi1)

           set sx1 \
            [pix2spec -cell $cellID -win $winID -roi $roiID \
                      -x $objInfo($obj,x1Pix) -units ppm]
        
           set sxn \
            [pix2spec -cell $cellID -win $winID -roi $roiID \
                      -x $objInfo($obj,xnPix) -units ppm]
           set sy1 \
            [pix2hi -cell $cellID -win $winID -y $objInfo($obj,ynPix) \
                    -fhi $fHi -lhi $lHi]

           set syn \
            [pix2hi -cell $cellID -win $winID -y $objInfo($obj,y1Pix) \
                    -fhi $fHi -lhi $lHi]

           showVerb "1D Region in View $obj is $sx1 $sxn $sy1 $syn"

           if {$objInfo($obj,newWindow)} \
              {
               init1DWindow $obj $sx1 $sxn $sy1 $syn
              } \
           else \
              {
               set objInfo($obj,specX1) $sx1
               set objInfo($obj,specXN) $sxn
               set objInfo($obj,specY1) $sy1
               set objInfo($obj,specYN) $syn

               readView $obj
               showView $obj zoom
              }
          }
     }

   xupdate
}

proc moveCursor  { thisObj thisXPix thisYPix } \
{
   global objInfo

   set obj    $thisObj
   set xPix   $thisXPix
   set yPix   $thisYPix
   set inFlag 1

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} {set inFlag 0} 
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} {set inFlag 0} 

   set objInfo($obj,x0Pix) $xPix
   set objInfo($obj,y0Pix) $yPix

   if {$inFlag} \
      {
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID  $objInfo($obj,pixmap)

       set fhi   $objInfo($obj,fhi1)
       set lhi   $objInfo($obj,lhi1)

       set cursorH \
        [pix2hi -cell $cellID -win $winID -y $yPix -fhi $fhi -lhi $lhi]

       set objInfo(data,cursorV) \
        [pix2spec -cell $cellID -win $winID -roi $roiID -x $xPix -units ppm]
      } \
   else \
      {
       set objInfo(data,cursorH) None
       set objInfo(data,cursorV) None
      }

   drawCursors
}

proc drawCursors {} \
{
   global objInfo

   set inFlag 1

   if {"$objInfo(data,cursorH)" == "None"} {set inFlag 0}
   if {"$objInfo(data,cursorV)" == "None"} {set inFlag 0}

   set fmt "%s: %.3fppm %.0fpts   Hi: %.4e"

   if {$inFlag} \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue

           set roiID  $objInfo($obj,roi1)
           set cellID $objInfo($obj,cell)
           set winID  $objInfo($obj,pixmap)

           set range [expr $objInfo($obj,lhi1) - $objInfo($obj,fhi1)]

           set xv \
            [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) \
               $objInfo(data,cursorV)ppm]

           set yv $objInfo(data,cursorH)
 
           set qx \
            [spec2pix -roi $roiID -cell $cellID -win $winID \
                      -x $objInfo(data,cursorV)ppm]
           set cursorH \
            [expr 100.0*($objInfo(data,cursorH) - $objInfo($obj,fhi1))/$range]

           if {$cursorH <   0.0} {set cursorH   0.0}
           if {$cursorH > 100.0} {set cursorH 100.0}

           set qy \
            [spec2pix -roi $roiID -cell $cellID -win $winID -y $cursorH%]
 
           set str \
              [format $fmt \
                      $objInfo(data,xAxis) $objInfo(data,cursorV) $xv $yv]

           wm title $objInfo($obj,window) "$str \[Zoom\] \[Scale\] \[Offset\]"

           if {$objInfo(data,cursorV) > $objInfo($obj,specX1) || \
                $objInfo(data,cursorV) < $objInfo($obj,specXN)} \
              {
               set qx [expr $objInfo($obj,x1) - 4]
              }

           if {$objInfo(data,cursorH) > $objInfo($obj,specY1) || \
                $objInfo(data,cursorH) < $objInfo($obj,specYN)} \
              {
               set qy [expr $objInfo($obj,y1) - 4]
              }

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)
   
           $objInfo($obj,canvas) raise cursorH
           $objInfo($obj,canvas) raise cursorV
          }
      } \
   else \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue

           wm title $objInfo($obj,window) \
            "$objInfo(prog,title) \[Zoom\] \[Scale\] \[Offset\]"

           set qx [expr $objInfo($obj,x1) - 4]
           set qy [expr $objInfo($obj,y1) - 4]

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)

           $objInfo($obj,canvas) raise cursorH
           $objInfo($obj,canvas) raise cursorV

# %%%

           $objInfo($obj,canvas) lower cursorH
           $objInfo($obj,canvas) lower cursorV
          }
      }

   xupdate
}

proc showThisPeak { obj tID pk } \
{
   global objInfo env

   set iStr "Peak [gdbGet $pk INDEX]"
   set xStr "$objInfo(data,xAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set yStr "$objInfo(data,yAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set hStr "Height=[gdbGet $pk HEIGHT]"

   wm title $objInfo($obj,window) "$iStr $xStr $yStr $hStr"

   $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map
   $objInfo($obj,canvas) raise         $tID

   xupdate
}

proc unshowThisPeak { obj tID pk } \
{
   global objInfo env

   wm title $objInfo($obj,window) $objInfo(prog,title)

   $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/clear.map
   $objInfo($obj,canvas) raise         $tID

   xupdate
}

proc getViewPeakList { obj tab } \
{
   global objInfo

   set pkList ""

   if {!$tab} {return $pkList}

   set    cond ""
   append cond "$objInfo(peak,xName) >= $objInfo($obj,specXN)"
   append cond " && $objInfo(peak,xName) <= $objInfo($obj,specX1)"

   set pkList [gdbSelect -from $tab -cond $cond]

   return $pkList
}

proc drawTitles {} \
{
   global objInfo

   if {![llength $objInfo(data,titleList)]} return

   set dList ""
   set j 1

   foreach i $objInfo(data,drawOrder) {lappend dList title_$i}

   delLabelItem $dList 

   foreach i $objInfo(data,drawOrder) \
      {
       set t  [lindex $objInfo(data,titleList) [expr $i - 1]]

       set ix [expr 4*$objInfo(prog,charX)]
       set iy [expr (2 + $j)*$objInfo(prog,charY)]


       addLabelItem \
          "label title_$i $ix $iy 250 w  $objInfo(color,color$i) $t"

       incr j
      }
}

proc drawPeaks {} \
{
   global objInfo env stdfont

   if {!$objInfo(prog,pkFlag)} return

   showVerb "Drawing peaks ..."

   set i 1

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       set pkList [getViewPeakList $obj $objInfo(peak,tab$i)]

       foreach pk $pkList \
          {
           set xPPM [gdbGet $pk $objInfo(peak,xName)]ppm
           set yHi  [gdbGet $pk HEIGHT]
           set str  [gdbGet $pk INDEX]

           set tx0 \
           [spec2pix -roi  $objInfo($obj,roi1) \
                     -cell $objInfo($obj,cell) \
                     -win  $objInfo($obj,pixmap) \
                     -x $xPPM]

           set ty0 \
           [spec2pix -roi  $objInfo($obj,roi1)   \
                     -cell $objInfo($obj,cell)   \
                     -win  $objInfo($obj,pixmap) \
                     -y $yHi]

           set tID [$objInfo($obj,canvas) create text $tx0 $ty0 -anchor sw   \
                      -fill $objInfo(color,peak) -tags peak -text $str       \
                      -font $stdfont -stipple @$env(NMRTXT)/clear.map]

           $objInfo($obj,canvas) bind $tID <Any-Enter> \
              "+ showThisPeak   $obj $tID $pk"

           $objInfo($obj,canvas) bind $tID <Any-Leave> \
              "+ unshowThisPeak $obj $tID $pk"
          }
      }
}

proc pix2hi args \
{
   set ARGV  [concat pix2hi $args]
   set ARGC  [llength $ARGV]

   set hiList "" 
   set cellID 1
   set winID  1

   set xPix   None
   set yPix   None

   set fHi    0.0
   set lHi    0.0

   getArgD $ARGV -cell cellID
   getArgD $ARGV -win  winID
   getArgD $ARGV -x    xPix
   getArgD $ARGV -y    yPix
   getArgD $ARGV -fhi  fHi
   getArgD $ARGV -lhi  lHi

   set cInfo [getCellInfo -cell $cellID -win $winID]

   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set ix1   [lindex $xInfo 0]
   set ixn   [lindex $xInfo 1]
   set iy1   [lindex $yInfo 0]
   set iyn   [lindex $yInfo 1]

   if {[strcasecmp $xPix None]} \
      {
       set range [expr $lHi - $fHi]       
       set w     [expr $ixn - $ix1]

       if {$w == 0} {set w 1.0}

       set hi [expr $lHi - $range*($xPix - $ix1)/$w]

       lappend hiList $hi
      }

   if {[strcasecmp $yPix None]} \
      {
       set range [expr $lHi - $fHi]
       set w     [expr $iyn - $iy1]

       if {$w == 0} {set w 1.0}

       set hi [expr $fHi + $range*($iyn - $yPix)/$w]

       lappend hiList $hi
      }

   return $hiList
}

proc xupdate args \
{
   global objInfo

   if {$objInfo(prog,updateFlag)} \
      {
       showVerb "Performing screen update."
       update
      } \
   else \
      {
       showVerb "Skipping screen update."
      }
}

setOSType

gdbInit
gdbCreate dBase

initProgVars
initDataArgs
initPeakVars

initProgColors
initSpecColors
initProgArgs

initMainWindow
drawTitles

if {$objInfo(prog,autoStartFlag)} \
   {
    if {[file exists $objInfo(data,inName1)]} startView
   }

#
# Bottom
