#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

# view2D.tcl
#
#   2D Spectrum Viewer
#   (C) Frank Delaglio 1999-2012
#-----------------------------------------------------------

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

proc initProgVars {} \
{
   global objInfo

   setFonts

   set objInfo(prog,name)    "View2D"
   set objInfo(prog,version) "2012.320.15.01"

   set objInfo(prog,title)   \
       "$objInfo(prog,name) $objInfo(prog,version) (C) F. Delaglio 2012"

   set objInfo(prog,verbose)       1
   set objInfo(prog,autoFlag)      0
   set objInfo(prog,reuseFlag)     0
   set objInfo(prog,freeFlag)      1
   set objInfo(prog,plev)          20
   set objInfo(prog,nlev)          20 
   set objInfo(prog,nextID)        1
   set objInfo(prog,drawType)      "cont"
   set objInfo(prog,winCount)      0 
   set objInfo(prog,winList)       ""
   set objInfo(prog,freeIDList)    ""
   set objInfo(prog,labelItemList) ""
   set objInfo(prog,defXSize)      980 
   set objInfo(prog,defYSize)      800 
   set objInfo(prog,exportProc)    None
   set objInfo(prog,gridFlag)      1
   set objInfo(prog,labFlag)       1
   set objInfo(prog,tabViewFlag)   0
   set objInfo(prog,pkSelectFlag)  0
   set objInfo(prog,pkSelectList)  ""

   set objInfo(data,specCount)     2 
   set objInfo(data,inList)        ""
   set objInfo(data,tauList)       ""
   set objInfo(data,user1List)     ""
   set objInfo(data,user2List)     ""
   
   set objInfo(data,factor)    1.3
   set objInfo(data,mult)      6.0
   set objInfo(data,cursorH)   None
   set objInfo(data,cursorV)   None
   set objInfo(data,window)    .
   set objInfo(data,valid)     0
   set objInfo(data,fdata)     0
   set objInfo(data,rdata)     0
   set objInfo(data,xSize)     0
   set objInfo(data,ySize)     0

   set objInfo(data,alignX)    None
   set objInfo(data,alignY)    None

   set objInfo(data,specX1)       0
   set objInfo(data,specXN)       0
   set objInfo(data,specY1)       0
   set objInfo(data,specYN)       0

   set objInfo(data,activeSpec)   "1 (None)"
}

proc initDataArgs {} \
{
   global ARGC ARGV objInfo

   if {[flagLoc $ARGV -one]} {set objInfo(data,specCount) 1}
   if {[flagLoc $ARGV -two]} {set objInfo(data,specCount) 2}
  
   getArgD $ARGV -specCount objInfo(data,specCount)

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,inName$i) "None"
      }

   set objInfo(data,inList)  ""
   set objInfo(data,inName1) "test.ft2"
   set objInfo(data,inName2) None

   getListArgD $ARGV -in objInfo(data,inList)

   if {[llength $objInfo(data,inList)]} \
      {
       set objInfo(data,specCount) [llength $objInfo(data,inList)]
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -in$i  objInfo(data,inName$i)
       }

   set n [llength $objInfo(data,inList)]

   if {$n} \
      {
       if {$n > $objInfo(data,specCount)} {set objInfo(data,specCount) $n}

       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           set objInfo(data,inName$i) \
            [lindex $objInfo(data,inList) [expr $i - 1]] 
          }
      } \
   else \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           lappend objInfo(data,inList) $objInfo(data,inName$i)
          }
      }

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,hi$i)     0.0
       set objInfo(data,noise$i)  0.0
       set objInfo(data,roi$i)    0
      }
}

proc initPeakVars {} \
{
   global objInfo

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(peak,tab$i)      0
       set objInfo(peak,pkCount$i)  0
      }

   set objInfo(peak,inList)  test.tab
   set objInfo(peak,xName)   None
   set objInfo(peak,yName)   None 
   set objInfo(peak,inUse)   ""
}

proc initProgColors {} \
{
   global objInfo

   set objInfo(color,bg)          black
   set objInfo(color,win)         gray10
   set objInfo(color,on)          green
   set objInfo(color,shadow)      black
   set objInfo(color,label)       white
   set objInfo(color,zoom)        green
   set objInfo(color,propose)     white
   set objInfo(color,accept)      green
   set objInfo(color,cursor)      green
   set objInfo(color,box)         white
   set objInfo(color,peak)        gray30
   set objInfo(color,peakH)       white
   set objInfo(color,peakS)       black
   set objInfo(color,select)      green
   set objInfo(color,selectH)     palegreen
   set objInfo(color,selectS)     black
   set objInfo(color,grid)        dimGray
   set objInfo(color,neutral)     dimGray
   set objInfo(color,colorScheme) bicolor 
}

proc initSpecColors {} \
{
   global ARGC ARGV objInfo

   switch $objInfo(color,colorScheme) \
      {
       bicolor \
          {
           set objInfo(color,posColorList) \
            "Cyan Red     Green     Yellow Violet"

           set objInfo(color,negColorList) \
            "Blue Magenta LightGray Orange White"
          }

       monocolor \
          {
           set objInfo(color,posColorList) \
            "Red Orange Yellow Green Blue Cyan Violet Magenta LightGray White"

           set objInfo(color,negColorList) $objInfo(color,posColorList) 
          }

       default \
          {
           set objInfo(color,posColorList) "Multi"
           set objInfo(color,negColorList) "Multi"
          }
      }

   getListArgD $ARGV -posColors objInfo(color,posColorList)
   getListArgD $ARGV -negColors objInfo(color,negColorList)

   set n [llength $objInfo(color,posColorList)]

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set j [expr ($i - 1) % $n]

       set objInfo(color,posColor$i) [lindex $objInfo(color,posColorList) $j]
       set objInfo(color,negColor$i) [lindex $objInfo(color,negColorList) $j]
      }

   if {![strcasecmp $objInfo(data,inName2) None]} \
      {
       set objInfo(color,posColor1)  Multi 
       set objInfo(color,negColor1)  Multi 
      }
}

proc initProgArgs {} \
{
   global objInfo ARGC ARGV

   getArgD $ARGV -export objInfo(prog,exportProc)

   if {[flagLoc $ARGV -help]} \
      {
       puts "2D Spectral Viewer with Overlay and Zoom:"
       puts "With a single spectrum:"
       puts " -in   name1        Spectrum Name"
       puts "With two spectra:"
       puts " -in1   name1       Spectrum 1 Name"
       puts " -in2   name2       Spectrum 2 Name"
       puts "With a spectral series:"
       puts " -in    nameList    Spectrum Name List"
       puts "Contour Drawing:"
       puts " -plev   pCount     Positive Contour Count"
       puts " -nlev   nCount     Negative Contour Count"
       puts " -mult   c          Factor for Auto-Estimate of First Contour"
       puts " -factor f          Factor for Contour Spacing"
       puts " -hi     hi         First Contour Height"
       puts " -hi1    hi1        First Contour Height, Spectrum 1"
       puts " -hi2    hi2        First Contour Height, Spectrum 2"
       puts "General Arguments:"
       puts " -align             Align by PPM (Approximate)."
       puts " -one               Configure for One Spectrum"
       puts " -two               Configure for Two Spectra (Default)"
       puts " -grid              Use Grid Lines for Contours (Default)."
       puts " -nogrid            No Grid Lines for Contours."
       puts " -cont              Contour Displays (Default)"
       puts " -pix               Pixmap Displays"
       puts " -pts               Zoom Region Coords in Points"
       puts " -ppm               Zoom Region Coords in PPM"
       puts " -xSize   xPix      Default X-Axis Window Size"
       puts " -ySize   yPix      Default Y-Axis Window Size"
       puts " -export  eName     Special Procedure for Region Export"
       puts " -tab     tabName   Peak Table Name or List"
       puts " -tabView           Enable Peak Table Viewing."
       puts " -notabView         Disable Peak Table Viewing (Default)."
       puts " -pkSelect          Enable Peak Selection Mode."
       puts "Color Arguments:"
       puts " -posColor1    posColor   Positive Contour Drawing Color"
       puts " -negColor1    negColor   Negative Contour Drawing Color"
       puts " -posColor2    posColor   Positive Contour Drawing Color"
       puts " -negColor2    negColor   Negative Contour Drawing Color"
       puts " -posColors    posColors  Positive Contour Drawing Color List"
       puts " -negColors    negColors  Negative Contour Drawing Color List"
       puts " -peakColor    pkColor    Peak Label Color"
       puts " -peakColor2   pkColor2   Peak Label Color (Hilighted)"
       puts " -selColor     selColor   Peak Label Color (Selected)"
       puts " -selColor2    selColor2  Peak Label Color (Selected, Hilighted)"
       puts " -winColor     winColor   Window Background Color"
       puts " -bgColor      bgColor    Contour Background Color"
       puts " -labelColor   lColor     Label Color"
       puts " -shadowColor  sColor     Label Shadow Color"
       puts " -zoomColor    zColor     Zoom Box Color"
       puts " -cursorColor  cColor     Cursor Color"
       puts " -gridColor    gColor     Grid Color"
       puts " -boxColor     bColor     Outline Box Color"
       puts " -proposeColor pColor     Selected Region Box Color"
       puts " -acceptColor  aColor     Accepted Region Box Color"
       
       if {[strcasecmp $objInfo(prog,exportProc) None]} \
          {
           $objInfo(prog,exportProc) -- -help
          }

       exit 1
      }

   if {[flagLoc $ARGV -verb]}   {set objInfo(prog,verbose) 1}
   if {[flagLoc $ARGV -noverb]} {set objInfo(prog,verbose) 0}

   if {[flagLoc $ARGV -auto]}   {set objInfo(prog,autoFlag) 1}
   if {[flagLoc $ARGV -noauto]} {set objInfo(prog,autoFlag) 0}

   if {[flagLoc $ARGV -grid]}   {set objInfo(prog,gridFlag) 1}
   if {[flagLoc $ARGV -nogrid]} {set objInfo(prog,gridFlag) 0}
   if {[flagLoc $ARGV -lab]}    {set objInfo(prog,labFlag)  1}
   if {[flagLoc $ARGV -nolab]}  {set objInfo(prog,labFlag)  0}

   if {[flagLoc $ARGV -free]}   {set objInfo(prog,freeFlag) 1}
   if {[flagLoc $ARGV -nofree]} {set objInfo(prog,freeFlag) 0}

   if {[flagLoc $ARGV -tabView]}   {set objInfo(prog,tabViewFlag) 1}
   if {[flagLoc $ARGV -notabView]} {set objInfo(prog,tabViewFlag) 0}

   if {[flagLoc $ARGV -pkSelect]}   {set objInfo(prog,pkSelectFlag) 1}
   if {[flagLoc $ARGV -nopkSelect]} {set objInfo(prog,pkSelectFlag) 0}

   if {[flagLoc $ARGV -pix]}    {set objInfo(prog,drawType) pix}
   if {[flagLoc $ARGV -cont]}   {set objInfo(prog,drawType) cont}

   if {[flagLoc $ARGV -align]} \
      {
       set objInfo(data,alignX) X
       set objInfo(data,alignY) Y
      }

   getListArgD $ARGV -tab objInfo(peak,inList)

   getArgD $ARGV -mult   objInfo(data,mult)
   getArgD $ARGV -factor objInfo(data,factor)

   getArgD $ARGV -plev   objInfo(prog,plev)
   getArgD $ARGV -nlev   objInfo(prog,nlev)

   getArgD $ARGV -xSize  objInfo(prog,defXSize)
   getArgD $ARGV -ySize  objInfo(prog,defYSize)

   if {[flagLoc $ARGV -monocolor]}  {set objInfo(color,colorScheme) monocolor}
   if {[flagLoc $ARGV -bicolor]}    {set objInfo(color,colorScheme) bicolor}

   initSpecColors

   getArgD $ARGV -hi objInfo(data,hi1)

   getArgD $ARGV -posColor objInfo(color,posColor1)
   getArgD $ARGV -negColor objInfo(color,negColor1)

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -posColor$i objInfo(color,posColor$i)
        getArgD $ARGV -negColor$i objInfo(color,negColor$i)

        set objInfo(data,hi$i) $objInfo(data,hi1)

        getArgD $ARGV -hi$i       objInfo(data,hi$i)
       }

   getArgD $ARGV -peakColor    objInfo(color,peak)
   getArgD $ARGV -peakColorH   objInfo(color,peakH)
   getArgD $ARGV -peakColorS   objInfo(color,peakS)
   getArgD $ARGV -selColor     objInfo(color,select)
   getArgD $ARGV -selColorH    objInfo(color,selectH)
   getArgD $ARGV -selColorS    objInfo(color,selectS)
   getArgD $ARGV -winColor     objInfo(color,win)
   getArgD $ARGV -bgColor      objInfo(color,bg)
   getArgD $ARGV -onColor      objInfo(color,on)
   getArgD $ARGV -shadowColor  objInfo(color,shadow)
   getArgD $ARGV -labelColor   objInfo(color,label)
   getArgD $ARGV -zoomColor    objInfo(color,zoom)
   getArgD $ARGV -cursorColor  objInfo(color,cursor)
   getArgD $ARGV -gridColor    objInfo(color,grid)
   getArgD $ARGV -boxColor     objInfo(color,box)
   getArgD $ARGV -proposeColor objInfo(color,propose)
   getArgD $ARGV -acceptColor  objInfo(color,accept)

   if {[strcasecmp $objInfo(prog,exportProc) None]} \
      {
       append objInfo(prog,name) " ($objInfo(prog,exportProc))"

       set objInfo(prog,title)   \
        "$objInfo(prog,name) $objInfo(prog,version) (C) F. Delaglio 2012"

       append objInfo(prog,title) \
          "    \[Cursors\] \[2D Export\] \[2D Zoom\]"
      }
}

proc freeThisID { idList } \
{
   global objInfo

   foreach id $idList \
      {
       lappend objInfo(prog,freeIDList) $id
      }
}

proc getNextID {} \
{
   global objInfo

   if {$objInfo(prog,reuseFlag) && [llength $objInfo(prog,freeIDList)]} \
      {
       set id [lindex $objInfo(prog,freeIDList) 0]
       set objInfo(prog,freeIDList) [lrange $objInfo(prog,freeIDList) 1 end]

       showVerb "Reusing ID $id."

       return $id
      }

   set  id $objInfo(prog,nextID)
   incr objInfo(prog,nextID)

   return $id
}

proc initPeakTable {} \
{
   global objInfo

   set i 1

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      { 
       set tabName [lindex $objInfo(peak,inList) [expr $i - 1]]

       if {$objInfo(peak,tab$i)} continue
       if {"$tabName" == "None"} continue
       if {"$tabName" == ""}     continue

       if {![file exists $tabName]} continue 

       showVerb "Reading table $tabName ..."

       set error \
          [catch {set objInfo(peak,tab$i) \
               [gdbRead table -in $tabName -name "Peak table $i"]}]

       set objInfo(peak,pkCount$i) \
           [gdbGetInfo $objInfo(peak,tab$i) -parms entryCount]
      }
}

proc initSpectralData {} \
{
   global objInfo

   set obj data

   set objInfo($obj,valid)    0
   set objInfo($obj,fdata)    0
   set objInfo($obj,rdata)    0
   set objInfo($obj,xSize)    0
   set objInfo($obj,ySize)    0

   set objInfo($obj,roi1)     0
   set objInfo($obj,roi2)     0

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo($obj,roi$i) 0
      }

   if {"$objInfo($obj,inName1)" == "None"} {return}

#
# Read data header.
# Extract size parameters:

   set error [rdFDATA $objInfo($obj,inName1) fdata]

   if {$error} \
      {
       showError view2D.tcl "Error reading file $objInfo($obj,inName1)"
       exit
      }

   set objInfo($obj,fdata)  $fdata
   set objInfo($obj,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]

   if {$objInfo($obj,tpFlag)} \
      {
       set objInfo($obj,hDim)  CUR_YDIM
       set objInfo($obj,vDim)  CUR_XDIM

       set objInfo(peak,xName) Y_PPM
       set objInfo(peak,yName) X_PPM
       set objInfo(peak,xAxis) Y_AXIS
       set objInfo(peak,yAxis) X_AXIS
      } \
   else \
      {
       set objInfo($obj,hDim)  CUR_XDIM
       set objInfo($obj,vDim)  CUR_YDIM

       set objInfo(peak,xName) X_PPM
       set objInfo(peak,yName) Y_PPM
       set objInfo(peak,xAxis) X_AXIS
       set objInfo(peak,yAxis) Y_AXIS
      }

   set objInfo($obj,xAxis) [getParmStr $fdata NDLABEL $objInfo($obj,hDim)]
   set objInfo($obj,yAxis) [getParmStr $fdata NDLABEL $objInfo($obj,vDim)]

   set objInfo($obj,xSize) \
       [expr int([getParm $fdata NDSIZE $objInfo($obj,hDim)])]

   set objInfo($obj,ySize) \
       [expr int([getParm $fdata NDSIZE $objInfo($obj,vDim)])]

   set objInfo($obj,ix)    [expr int($objInfo($obj,xSize)/2)]
   set objInfo($obj,iy)    [expr int($objInfo($obj,ySize)/2)]

   setSpecZoom $obj $fdata 1 $objInfo($obj,xSize)  1 $objInfo($obj,ySize)

#
# Extract noise levels:

   readView $obj 

   busyView hold .

   set c         $objInfo(data,mult)
   set zeroCount 0

   for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
      {
       if {$objInfo($obj,roi$i) != 0 && $objInfo($obj,hi$i) == 0.0} \
          {
           showVerb \
            "Estimating first contour height for ROI $objInfo($obj,roi$i)."

           set roiInfo [getROIInfo -roi $objInfo($obj,roi$i)]
           set xSize   [lindex [lindex $roiInfo 2] 0]
           set ySize   [lindex [lindex $roiInfo 2] 1]
           set nn      [expr $xSize*$ySize]
           set n       [expr $xSize*$ySize/5]

           incr zeroCount

           if {"$obj" != "data"} {set objInfo($obj,fdata)  [lindex $roiInfo 1]}

           set objInfo($obj,rdata)    [lindex $roiInfo 0] 
           set objInfo($obj,noise$i)  [vEstNoise $objInfo($obj,rdata) $n]
           set objInfo($obj,minVal$i) [vMin $objInfo($obj,rdata) $nn]
           set objInfo($obj,maxVal$i) [vMax $objInfo($obj,rdata) $nn]

           if {$c != 0.0} \
              {
               set objInfo($obj,hi$i) [expr $c*$objInfo($obj,noise$i)]
              } \
           else \
              {
               set objInfo($obj,hi$i) \
                [getFirstLevel 10 10 \
                               $objInfo($obj,minVal$i) $objInfo($obj,maxVal$i) \
                               $objInfo(data,factor)]
              }

           showVerb "$objInfo($obj,inName$i) Hi: $objInfo($obj,hi$i)"

           if {$objInfo($obj,hi$i) == 0.0} {set objInfo($obj,hi$i) 1.0e+6}
          }
      }

   if {$zeroCount == $objInfo(data,specCount)} \
      {
       for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
          {
           set objInfo($obj,hi$i) $objInfo($obj,hi1)

           showVerb "Reset $objInfo($obj,inName$i) Hi: $objInfo($obj,hi$i)"
          }
      }

    busyView release .
}

proc resizeView args \
{
   global objInfo

   set obj  [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   showVerb "Resize event $args"

   update

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,window)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,window)]

   set objInfo($obj,cellXSize) \
       [expr $objInfo($obj,winXSize)     - \
             ($objInfo($obj,leftMargin)  + \
              $objInfo($obj,rightMargin) + \
              $objInfo($obj,winXAdj))]

   set objInfo($obj,cellYSize) \
       [expr $objInfo($obj,winYSize)      - \
             ($objInfo($obj,topMargin)    + \
              $objInfo($obj,bottomMargin) + \
              $objInfo($obj,winYAdj))]

   set objInfo($obj,xSize) \
       [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

   set objInfo($obj,ySize) \
       [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

   if {$objInfo($obj,xSize)     < 50} {set objInfo($obj,xSize) 50}
   if {$objInfo($obj,ySize)     < 50} {set objInfo($obj,ySize) 50}
   if {$objInfo($obj,cellXSize) < 50} {set objInfo($obj,cellXSize) 50}
   if {$objInfo($obj,cellYSize) < 50} {set objInfo($obj,cellYSize) 50}

   $objInfo($obj,canvas) delete pixmap -tags $obj

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
      -win $objInfo($obj,pixmap) -tags $obj

   set objInfo($obj,x1) $objInfo($obj,leftMargin)
   set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
   set objInfo($obj,y1) $objInfo($obj,topMargin)
   set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

   set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
   set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

   set objInfo($obj,x1Pix)        $xPix
   set objInfo($obj,y1Pix)        $yPix
   set objInfo($obj,xnPix)        $xPix
   set objInfo($obj,ynPix)        $yPix

   defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
              -x $objInfo($obj,x1) $objInfo($obj,xn) \
              -y $objInfo($obj,y1) $objInfo($obj,yn)

   $objInfo($obj,canvas) raise cursorH
   $objInfo($obj,canvas) raise cursorV

   $objInfo($obj,redrawProc) $obj

   bind $objInfo($obj,canvas) <Configure> \
    "$objInfo($obj,resizeProc) $obj %w %h %T"
}

proc initMainWindow {} \
{
   global objInfo env

   if {$objInfo(prog,autoFlag)} \
      {
       update
#      wm iconify .
      }

   wm title . $objInfo(prog,name)

    set colorList \
       {
        Multi
        Black        Red          Orange       Yellow
        Green        Blue         Violet       Cyan
        Magenta      LightGray    DimGray      White
       }

   frame .frameL
   frame .frameR
   frame .frameB
   
#
# Logo:

   catch {image delete pipeImage}

   image create photo pipeImage -file $env(NMRTXT)/frank.gif

   button .viewLogo -relief sunken -image pipeImage -background black \
          -command "showHelp logo"

   button .start    -text Start -command startView
   button .exit     -text Exit  -command quitAllView

   pack .viewLogo -in .frameL -side top -expand 1 -fill both 

#
# File Parameters:

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       frame .frame$i

       if {$objInfo(data,specCount) == 1} \
          {
           set str  "File:"
          } \
       else \
          {
           set str  "File $i:" 
          }

       label  .fileLab$i  -text $str -width 8 -anchor w
       entry  .fileName$i -width 32 -textvariable objInfo(data,inName$i)

       button .fileButt$i -bitmap @$env(NMRTXT)/caret.map \
                          -command "selectFile $i"

       bind   .fileName$i <Return> "+ setFile"

       label  .posColorLab$i -text "Pos:"
       entry  .posColorName$i -width 10 -textvariable objInfo(color,posColor$i)

       menubutton .posColorButt$i -bitmap @$env(NMRTXT)/caret.map \
                  -relief raised -menu .posColorButt$i.menu

       label  .negColorLab$i -text "Neg:"
       entry  .negColorName$i -width 10 -textvariable objInfo(color,negColor$i)

       menubutton .negColorButt$i -bitmap @$env(NMRTXT)/caret.map \
                  -relief raised -menu .negColorButt$i.menu

#
# Color choice menus:

       menu .posColorButt$i.menu
       menu .negColorButt$i.menu

       foreach color $colorList \
          {
           .posColorButt$i.menu add command -label $color \
              -command "set objInfo(color,posColor$i) $color"

           .negColorButt$i.menu add command -label $color \
              -command "set objInfo(color,negColor$i) $color"
          }

       pack .fileLab$i .fileName$i .fileButt$i              \
            .posColorLab$i .posColorName$i .posColorButt$i  \
            .negColorLab$i .negColorName$i .negColorButt$i  \
            -side left -padx 1m -pady 1m -in .frame$i

       pack .frame$i -in .frameR -side top -expand 1 -fill x 
      }

   pack .start .exit -in .frameB    -side left -expand 1 -fill x
   pack .frameB -in .frameR -side bottom -expand 1 -fill x
   pack .frameL .frameR -side left -fill both

   wm geom . +5-5
}

proc init2DWindow { parent sx1 sxn sy1 syn } \
{
    global objInfo

    set id  [getNextID]
    set obj win2D_$id
    set w   .win2D_$id

    showVerb "Initializing window $id"

    toplevel $w

    incr objInfo(prog,winCount)

    wm title $w $objInfo(prog,title)

    lappend objInfo(prog,winList) $obj

    set objInfo($obj,parent)       $parent
    set objInfo($obj,winXSize)     $objInfo(prog,defXSize)
    set objInfo($obj,winYSize)     $objInfo(prog,defYSize) 

    if {$objInfo(prog,labFlag)} \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,rightMargin)  120 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin) 100
       } \
    else \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,rightMargin)   65 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin)  70
       }

    set objInfo($obj,winXAdj)        5 
    set objInfo($obj,winYAdj)       20 

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]
 
    set objInfo($obj,xSize) \
        [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]

    set objInfo($obj,ySize) \
        [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

    set objInfo($obj,window)       $w
    set objInfo($obj,canvas)       $w.c
    set objInfo($obj,scrollH)      $w.sh
    set objInfo($obj,scrollV)      $w.sv
    set objInfo($obj,pixmap)       $id 
    set objInfo($obj,cell)         $id 

    set objInfo($obj,resizeProc)   resizeView
    set objInfo($obj,redrawProc)   showView

    set objInfo($obj,x1) $objInfo($obj,leftMargin)
    set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
    set objInfo($obj,y1) $objInfo($obj,topMargin)
    set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

    set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
    set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

    set objInfo($obj,x1Pix)        $xPix
    set objInfo($obj,y1Pix)        $yPix
    set objInfo($obj,xnPix)        $xPix
    set objInfo($obj,ynPix)        $yPix
    set objInfo($obj,x0Pix)        -1 
    set objInfo($obj,y0Pix)        -1

    set objInfo($obj,specCount)    $objInfo($parent,specCount)
    set objInfo($obj,inList)       $objInfo($parent,inList)

    set n $objInfo($parent,specCount)
 
    if {$n < 2} {set n 2}

    for {set i 1} {$i <= $n} {incr i} \
       {
        set objInfo($obj,hi$i)     $objInfo($parent,hi$i)
        set objInfo($obj,inName$i) $objInfo($parent,inName$i)
       }

    set objInfo($obj,factor)       $objInfo($parent,factor)
    set objInfo($obj,xAxis)        $objInfo($parent,xAxis)
    set objInfo($obj,yAxis)        $objInfo($parent,yAxis)
    set objInfo($obj,hDim)         $objInfo($parent,hDim)
    set objInfo($obj,vDim)         $objInfo($parent,vDim)

    set objInfo($obj,specX1)       $sx1
    set objInfo($obj,specXN)       $sxn
    set objInfo($obj,specY1)       $sy1
    set objInfo($obj,specYN)       $syn

    set objInfo($obj,newWindow)    1

    wm title $objInfo($obj,window) $objInfo(prog,title)

    frame $w.top
    frame $w.bottom

    button $w.draw -text "Draw" -command "showView $obj"
    button $w.full -text "Full" -command "fullView $obj"

    if {[strcasecmp $objInfo(prog,exportProc) None]} \
       {
        button $w.exportReg -text "Export Region" -command "exportView  $obj"
        button $w.exportSel -text "Export Peaks"  -command "exportPeaks $obj"
       }

    button $w.quit -text "Quit" -command "quitThisView  $obj"

    label  $w.mode -text "    Zoom:"

    radiobutton $w.new -text "New Window"   -value 1 \
       -variable objInfo($obj,newWindow) -anchor w

    radiobutton $w.same -text "Same Window" -value 0 \
       -variable objInfo($obj,newWindow) -anchor w

   if {$objInfo(data,specCount) == 1} \
      {
       label  $w.lab1   -text "First:"
       entry  $w.first1 -textvariable objInfo($obj,hi1) -width 12 
       button $w.incr1  -text "+" -padx 1 -pady 1 -command "incrHi $obj 1"
       button $w.decr1  -text "-" -padx 1 -pady 1 -command "decrHi $obj 1"
       bind   $w.first1 <Return> "+ setHi $obj 1"
      } \
   elseif {$objInfo(data,specCount) == 2} \
      {
       label  $w.lab1   -text "First:"
       entry  $w.first1 -textvariable objInfo($obj,hi1) -width 12
       button $w.incr1  -text "+" -padx 1 -pady 1 -command "incrHi $obj 1"
       button $w.decr1  -text "-" -padx 1 -pady 1 -command "decrHi $obj 1"
       bind   $w.first1 <Return> "+ setHi $obj 1"

       label  $w.lab2   -text "First:"
       entry  $w.first2 -textvariable objInfo($obj,hi2) -width 12
       button $w.incr2  -text "+" -padx 1 -pady 1 -command "incrHi $obj 2"
       button $w.decr2  -text "-" -padx 1 -pady 1 -command "decrHi $obj 2"
       bind   $w.first2 <Return> "+ setHi $obj 2"
      } \
   else \
      {
       label  $w.lab1   -text "First:"
       entry  $w.first1 -textvariable objInfo($obj,hi1) -width 12
       button $w.incr1  -text "+" -padx 1 -pady 1 -command "incrHi $obj active"
       button $w.decr1  -text "-" -padx 1 -pady 1 -command "decrHi $obj active"
       bind   $w.first1 <Return> "+ setHi $obj active"

       set color $objInfo(color,posColor1) 

       set objInfo($obj,activeSpec) "1 ($color)"

       label  $w.lab2   -text "Spectrum:"
       entry  $w.first2 -textvariable objInfo($obj,activeSpec) -width 10

       button $w.incr2  -text "+" -padx 1 -pady 1 -command "incrActiveSpec $obj"
       button $w.decr2  -text "-" -padx 1 -pady 1 -command "decrActiveSpec $obj"
       bind   $w.first2 <Return> "+ setActivSpec $obj"
      }

   if {[strcasecmp $objInfo(prog,exportProc) None]} \
      {
       pack $w.draw $w.full $w.exportReg $w.exportSel $w.quit $w.mode $w.new $w.same \
            -in $w.top -side left 
      } \
   else \
      { 
       pack $w.draw $w.full $w.quit $w.mode $w.new $w.same \
            -in $w.top -side left 
      }

   if {$objInfo(data,specCount) > 1} \
      {
       pack $w.decr2 $w.incr2 $w.first2 $w.lab2 -in $w.top -side right
      }

   if {$objInfo(data,specCount) > 0} \
      {
       pack $w.decr1 $w.incr1 $w.first1 $w.lab1 -in $w.top -side right
      }

#
# Canvas and scrollbars:

    scrollbar $objInfo($obj,scrollV) \
       -orient vert -command "$objInfo($obj,canvas) yview"

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize)      \
       -xscrollcommand "$objInfo($obj,scrollH) set"                  \
       -yscrollcommand "$objInfo($obj,scrollV) set"                  \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)" \
       -closeenough 10

    set dx [expr 5*$objInfo(prog,winCount)]
    set dy [expr 5*$objInfo(prog,winCount)]

    wm geom  $objInfo($obj,window) \
       =$objInfo($obj,winXSize)x$objInfo($obj,winYSize)+$dx+$dy

    update

    pack $objInfo($obj,scrollV) -in $w.bottom -side right  -expand 0 -fill y 
    pack $objInfo($obj,scrollH) -in $w.bottom -side bottom -expand 0 -fill x
    pack $objInfo($obj,canvas)  -in $w.bottom -side left   -expand 1 -fill both

    pack $w.top    -side top -expand 0 -fill x
    pack $w.bottom -side top -expand 1 -fill both

    update

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1) $objInfo($obj,xn) \
               -y $objInfo($obj,y1) $objInfo($obj,yn)

    set qx [expr $objInfo($obj,x1) - 4] 
    set qy [expr $objInfo($obj,y1) - 4] 

    set idH [$objInfo($obj,canvas) create line     \
       $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy \
       -fill $objInfo(color,cursor) -tags cursorH]

    set idV [$objInfo($obj,canvas) create line     \
       $qx $objInfo($obj,y1) $qx $objInfo($obj,yn) \
       -fill $objInfo(color,cursor) -tags cursorV]

    readView $obj
    showView $obj

    bind $objInfo($obj,canvas) <Configure> \
         "$objInfo($obj,resizeProc) $obj %w %h %T"

    bind $objInfo($obj,canvas) <ButtonPress-2>   "zoomON     $obj %x %y B2"
    bind $objInfo($obj,canvas) <ButtonPress-2>   "zoomON     $obj %x %y B2"

    bind $objInfo($obj,canvas) <ButtonPress-2>   "zoomON     $obj %x %y B2"
    bind $objInfo($obj,canvas) <ButtonRelease-2> "zoom2D     $obj %x %y B2"
    bind $objInfo($obj,canvas) <B2-Motion>       "trackZoom  $obj %x %y B2"

    bind $objInfo($obj,canvas) <ButtonPress-3>   "zoomON     $obj %x %y B3"
    bind $objInfo($obj,canvas) <ButtonRelease-3> "zoom2D     $obj %x %y B3"
    bind $objInfo($obj,canvas) <B3-Motion>       "trackZoom  $obj %x %y B3"

    bind $objInfo($obj,canvas) <B1-Motion>       "moveCursor $obj %x %y"

    bind $objInfo($obj,canvas) <Shift-ButtonPress-1>    ""
    bind $objInfo($obj,canvas) <Shift-ButtonPress-2>    ""
    bind $objInfo($obj,canvas) <Shift-ButtonPress-3>    ""

    bind $objInfo($obj,canvas) <Shift-B1-Motion>        ""
    bind $objInfo($obj,canvas) <Shift-B2-Motion>        ""
    bind $objInfo($obj,canvas) <Shift-B3-Motion>        ""

    bind $objInfo($obj,canvas) <Control-B1-Motion>      ""
    bind $objInfo($obj,canvas) <Control-B2-Motion>      ""
    bind $objInfo($obj,canvas) <Control-B3-Motion>      ""

    bind $objInfo($obj,canvas) <Control-ButtonPress-1>  ""
    bind $objInfo($obj,canvas) <Control-ButtonPress-2>  ""
    bind $objInfo($obj,canvas) <Control-ButtonPress-2>  ""
}

proc initPeakWindow {} \
{
   global objInfo 

   set i 1

   if {!$objInfo(prog,tabViewFlag)} return
   if {!$objInfo(peak,tab$i)} return

   set vFirstList  "INDEX X_PPM Y_PPM HEIGHT VOL ASS"

   set vNameList [gdbGetInfo $objInfo(peak,tab$i) -parms varNames]
   set vList ""

   foreach vName [concat $vFirstList $vNameList] \
      {
       if {[lsearch -exact $vList $vName] < 0} \
          {
           lappend vList $vName
          }
      }

   set tabName [lindex $objInfo(peak,inList) 0]

   toplevel .peakView

   showTable -in $tabName -var $vList -win .peakView

   wm geom .peakView -5-5
   update
}

proc setFile {} \
{
   global objInfo

   initSpecColors
}

proc selectFile { id } \
{
   global objInfo

   set obj data

    set fileTypes \
       {
        { {All Files}        *      }
        { {ft  Files}        {.ft}  }
        { {ft1 Files}        {.ft1} }
        { {ft2 Files}        {.ft2} }
        { {ft3 Files}        {.ft3} }
        { {fid Files}        {.fid} }
        { {dat Files}        {.dat} }
        { {DAT Files}        {.DAT} }
       }

   set objInfo($obj,inName$id) \
     [tk_getOpenFile -title "Spectrum $id" -filetypes $fileTypes]

   if {"$objInfo($obj,inName$id)" == ""} {set objInfo($obj,inName$id) "None"}

   set objInfo($obj,inList) ""

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       lappend objInfo($obj,inList) $objInfo($obj,inName$i)
      }

   initSpecColors
}

proc readView args \
{
   global objInfo

   set obj [lindex $args 0]
   set n   [llength $args]

   busyView hold $objInfo($obj,window)

   set nn $objInfo(data,specCount)

   if {$nn < 2} {set nn 2}

   for {set i 1} {$i <= $nn} {incr i} \
      {
       set objInfo($obj,roi$i) 0
       set roiID($i)           0
      }

   for {set i 1} {$i < $n} {incr i} \
      {
       set roiID($i) [lindex $args $i]
      }

   if {$objInfo(data,tpFlag)} \
      {
       set hDim Y_AXIS
       set vDim X_AXIS
      } \
   else \
      {
       set hDim X_AXIS
       set vDim Y_AXIS
      }

   set objInfo($obj,tauList)   ""
   set objInfo($obj,user1List) ""
   set objInfo($obj,user2List) ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {"$objInfo($obj,inName$i)" != "None"} \
          {
           if {!$roiID($i)} {set roiID($i) [getNextID]}

           showVerb "Reading ROI $roiID($i) ..."

           set objInfo($obj,roi$i) $roiID($i)

           if {$i == 1} \
              {
               readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) \
                -x $hDim $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm \
                -y $vDim $objInfo($obj,specY1)ppm $objInfo($obj,specYN)ppm
              } \
           else \
              {
               readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) \
                -x $hDim $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm \
                -y $vDim $objInfo($obj,specY1)ppm $objInfo($obj,specYN)ppm \
                -center $objInfo(data,alignX) $objInfo(data,alignY)
              }

           set roiInfo [getROIInfo -roi $roiID($i)]
           set fdata   [lindex $roiInfo 1]

           lappend objInfo($obj,tauList)   [getParm $fdata FDTAU   NULL_DIM]
           lappend objInfo($obj,user1List) [getParm $fdata FDUSER1 NULL_DIM]
           lappend objInfo($obj,user2List) [getParm $fdata FDUSER2 NULL_DIM]

           if {$i == 1 && "$obj" != "data"} {set objInfo($obj,fdata) $fdata}
          }
      }

   showVerb "Done Reading ROIs."

   busyView release $objInfo($obj,window)
}

proc incrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   incr i

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,posColor$i))"
 
   $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc decrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0] 

   incr i -1

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,posColor$i))"

   $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc setActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0] 

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 0} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,posColor$i))"

   $w.first1 configure -textvariable objInfo($obj,hi$i)
}

proc incrHi { obj mode } \
{
   global objInfo

   if {"$mode" == "active"} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,hi$i) [expr $objInfo($obj,hi$i)*$objInfo($obj,factor)]
      } \
   else \
      {
       set objInfo($obj,hi)   [expr $objInfo($obj,hi)*$objInfo($obj,factor)]
      }
}

proc decrHi { obj mode } \
{
   global objInfo

   if {"$mode" == "active"} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,hi$i) [expr $objInfo($obj,hi$i)/$objInfo($obj,factor)]
      } \
   else \
      {
       set objInfo($obj,hi)   [expr $objInfo($obj,hi)/$objInfo($obj,factor)]
      }
}

proc setHi { obj mode } \
{
   global objInfo

   showView $obj
}

proc fullView { obj } \
{
   global objInfo

   setSpecZoom \
    $obj $objInfo(data,fdata) 1 $objInfo(data,xSize) 1 $objInfo(data,ySize)

   readView $obj
   showView $obj
}

proc showView { obj } \
{
   global objInfo

   busyView hold $objInfo($obj,window)

   clearView $obj

   if {$objInfo(prog,gridFlag)} \
      {
       set xGrid xGrid
       set yGrid yGrid
      } \
   else \
      {
       set xGrid noxGrid
       set yGrid noyGrid
      }

   drawRec -fg $objInfo(color,win) -win $objInfo($obj,pixmap) \
           -x 1 $objInfo($obj,xSize) -y 1 $objInfo($obj,ySize) 

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$objInfo($obj,roi$i)} \
          {
           showVerb "Drawing ROI $objInfo($obj,roi$i) $objInfo($obj,hi$i) ..."

           if {$i == 1} \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                       -roi $objInfo($obj,roi$i) -hi $objInfo($obj,hi$i)   \
                       -bg $objInfo(color,bg) -axis Bottom Right           \
                       -color1 $objInfo(color,posColor$i)                  \
                       -color2 $objInfo(color,negColor$i)                  \
                       -colorS $objInfo(color,shadow)                      \
                       -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev) \
                       -mult $objInfo(data,factor)                         \
                       -$objInfo(prog,drawType) -$xGrid -$yGrid
              } \
           else \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                       -roi $objInfo($obj,roi$i) -hi $objInfo($obj,hi$i)    \
                       -bg None \
                       -color1 $objInfo(color,posColor$i)                  \
                       -color2 $objInfo(color,negColor$i)                  \
                       -colorS $objInfo(color,shadow)                      \
                       -plev $objInfo(prog,plev) -nlev $objInfo(prog,nlev) \
                       -mult $objInfo(data,factor)                         \
                       -$objInfo(prog,drawType)
              }

          update
         }
      }

   busyView release $objInfo($obj,window)

   drawPeaks
   drawCursors
   drawLabelItems

   update
}

proc drawThisItem { item } \
{
   global objInfo stdfont boldfont 

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue
     
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID  $objInfo($obj,pixmap)

       set cInfo [getCellInfo -cell $cellID -win $winID]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set kx1   [lindex $xInfo 0]
       set kxn   [lindex $xInfo 1]
       set ky1   [lindex $yInfo 0]
       set kyn   [lindex $yInfo 1]

       showVerb "Label Cell $cellID ROI $roiID: $kx1,$ky1 $kxn,$kyn"
 
       set type [lindex $item 0]
       set id   [lindex $item 1]

       switch $type \
          {
           box  {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [lindex $item 7]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x1}ppm]

                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${xn}ppm]

                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y1}ppm]

                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${yn}ppm]

                 $objInfo($obj,canvas) create rectangle \
                    $ix1 $iy1 $ixn $iyn -outline $fg -width $w -tags labelItem
                }

           line {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [lindex $item 7]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x1}ppm]

                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${xn}ppm]

                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y1}ppm]

                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${yn}ppm]

                 $objInfo($obj,canvas) create line \
                    $ix1 $iy1 $ixn $iyn -fill $fg -width $w -tags labelItem
                }

           text {
                 set x0 [lindex $item 2]
                 set y0 [lindex $item 3]
                 set w  [lindex $item 4]
                 set a  [lindex $item 5]
                 set fg [lindex $item 6]
                 set s  [lrange $item 7 end]

                 if {$x0 > $objInfo(specX1)} continue
                 if {$x0 < $objInfo(specXN)} continue
                 if {$y0 > $objInfo(specY1)} continue
                 if {$y0 < $objInfo(specYN)} continue

                 set ix0 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -x ${x0}ppm]

                 set iy0 [spec2pix -roi $roiID -cell $cellID -win $winID \
                     -y ${y0}ppm]

                 $objInfo($obj,canvas) create text \
                    $ix0 $iy0 -fill $fg -width $w -anchor $a -text $s \
                    -font $stdfont -tags labelItem
                }
          }
      }

   update
}

proc addLabelItem { item } \
{
   global objInfo

   showVerb "Label item: $item"

   lappend objInfo(prog,labelItemList) $item

   drawThisItem $item
}

proc delLabelItem { id } \
{
   global objInfo

   showVerb "Delete Label Item $id"

   clearLabelItems

   set itemList ""

   foreach item $objInfo(prog,labelItemList) \
      {
       set thisID [lindex $item 1]

       if {"$thisID" == "$id"} \
          {
           showVerb "Deleting item: $item"
          } \
       else \
          {
           showVerb "Keeping item: $item"
           lappend itemList $item
          } 
      }

   set objInfo(prog,labelItemList) $itemList

   drawLabelItems
}

proc drawLabelItems {} \
{
   global objInfo

   foreach item $objInfo(prog,labelItemList) \
      {
       drawThisItem $item
      }
}

proc clearLabelItems {} \
{
   global objInfo

   foreach obj $objInfo(prog,winList) \
      {
       $objInfo($obj,canvas) delete labelItem
       update
      }
}

proc getSpecClipX { obj x0 } \
{
   global objInfo

   if {$x0 > $objInfo($obj,specX1)} \
      {
       set x0 $objInfo($obj,specX1)
      }

   if {$x0 < $objInfo($obj,specXN)} \
      {
       set x0 $objInfo($obj,specXN)
      }

   return $x0
}

proc getSpecClipY { obj y0 } \
{
   global objInfo

   if {$y0 > $objInfo($obj,specY1)} \
      {
       set y0 $objInfo($obj,specY1)
      }

   if {$y0 < $objInfo($obj,specYN)} \
      {
       set y0 $objInfo($obj,specYN)
      }

   return $y0
}

proc exportView { obj } \
{
   global objInfo

   set sx1 $objInfo($obj,specX1) 
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   showVerb "Exporting Current Region to $objInfo(prog,exportProc)"

   $objInfo(prog,exportProc) $obj $sx1 $sxn $sy1 $syn
}

proc exportPeaks { obj } \
{
   global objInfo

   set sx1 $objInfo($obj,specX1)
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   if {![llength $objInfo(prog,pkSelectList)]} return

   showVerb "Exporting Current Peaks to $objInfo(prog,exportProc)"

   $objInfo(prog,exportProc) $obj $sx1 $sxn $sy1 $syn $objInfo(prog,pkSelectList)
}

proc showHelp {} \
{
}

proc showVerb args \
{
   global objInfo

   if {$objInfo(prog,verbose)} {puts $args}
}

proc quitThisView { obj } \
{
   global objInfo

   showVerb "Quiting View $obj, Cell $objInfo($obj,cell)"

   catch [destroy $objInfo($obj,window)]

   set   winList ""

   foreach w $objInfo(prog,winList) \
      {
       if {"$w" != "$obj"} {lappend winList $w}
      } 

   set objInfo(prog,winList) $winList

   freeViewROI $obj
}

proc freeViewROI { obj } \
{
   global objInfo

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       if {$objInfo($obj,roi$i)} \
          {
           showVerb "Free ID $objInfo($obj,roi$i)."
     
           freeThisID $objInfo($obj,roi$i)
 
           if {$objInfo(prog,freeFlag)} {freeROI -roi $objInfo($obj,roi$i)}
          }
     }
}

proc startView {} \
{
   global objInfo

   initPeakTable
   initPeakWindow
   initSpectralData

   freeViewROI  data

   init2DWindow data \
     $objInfo(data,specX1) $objInfo(data,specXN) \
     $objInfo(data,specY1) $objInfo(data,specYN)
}

proc busyView { mode w } \
{
   if {[strcmp $w .]} \
      {
       busy $mode .
       busy $mode $w
      } \
   else \
      {
       busy $mode $w
      }

   update
}

proc quitAllView {} \
{
   global objInfo

   if {[strcasecmp $objInfo(prog,exportProc) None]} \
      {
       $objInfo(prog,exportProc) -- -quit
      }

   okExit exit
}

proc clearView { obj } \
{
   global objInfo

   showVerb "Clearing window $obj ..."

   $objInfo($obj,canvas) delete peak labelItem

   newScreen -win $objInfo($obj,pixmap)
}

proc setSpecZoom { obj fdata zx1 zxn zy1 zyn } \
{
   global objInfo

   set objInfo($obj,specX1) [pnt2spec $fdata $objInfo($obj,hDim) $zx1 ppm]
   set objInfo($obj,specXN) [pnt2spec $fdata $objInfo($obj,hDim) $zxn ppm]
   set objInfo($obj,specY1) [pnt2spec $fdata $objInfo($obj,vDim) $zy1 ppm]
   set objInfo($obj,specYN) [pnt2spec $fdata $objInfo($obj,vDim) $zyn ppm]

   showVerb "Zoom Limits X: $objInfo($obj,specX1)ppm $objInfo($obj,specXN)ppm"
   showVerb "Zoom Limits Y: $objInfo($obj,specY1)ppm $objInfo($obj,specYN)ppm"
}

proc zoomON { obj xPix yPix bName } \
{
   global objInfo

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

   set objInfo($obj,x1Pix) $xPix
   set objInfo($obj,y1Pix) $yPix
   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) create rectangle \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix) \
      -outline $objInfo(color,zoom) -tags zoom

   update
}

proc trackZoom { obj xPix yPix bName } \
{
   global objInfo

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   $objInfo($obj,canvas) coords zoom \
      $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
      $objInfo($obj,xnPix) $objInfo($obj,ynPix)

   update
}

proc zoom2D { obj xPix yPix bName } \
{
   global objInfo

   if {$objInfo($obj,x1Pix) == $objInfo($obj,xnPix) &&
       $objInfo($obj,y1Pix) == $objInfo($obj,ynPix)} \
      {
       showVerb "Empty 2D Region Rejected in View $obj"
       return
      }

   showVerb "2D Region Selected in View $obj"

   $objInfo($obj,canvas) delete zoom

   if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
      {
       set itemp               $objInfo($obj,x1Pix)
       set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
       set objInfo($obj,xnPix) $itemp
      }

   if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
      {
       set itemp               $objInfo($obj,y1Pix)
       set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
       set objInfo($obj,ynPix) $itemp
      }

   set roiID  $objInfo($obj,roi1)
   set cellID $objInfo($obj,cell)
   set winID  $objInfo($obj,pixmap)

   set sx1 \
    [pix2spec -cell $cellID -win $winID -roi $roiID \
              -x $objInfo($obj,x1Pix) -units ppm]

   set sxn \
    [pix2spec -cell $cellID -win $winID -roi $roiID \
              -x $objInfo($obj,xnPix) -units ppm]

   set sy1 \
    [pix2spec -cell $cellID -win $winID -roi $roiID \
              -y $objInfo($obj,ynPix) -units ppm]

   set syn \
    [pix2spec -cell $cellID -win $winID -roi $roiID \
              -y $objInfo($obj,y1Pix) -units ppm]

   if {"$bName" == "B2" && "$objInfo(prog,exportProc)" != "None"} \
      {
       showVerb "Exporting Selected Region to $objInfo(prog,exportProc)"
       $objInfo(prog,exportProc) $obj $sx1 $sxn $sy1 $syn
       return
      }

   showVerb "2D Region in View $obj is $sx1 $sxn $sy1 $syn"

   if {$objInfo($obj,newWindow)} \
      {
       init2DWindow $obj $sx1 $sxn $sy1 $syn
      } \
   else \
      {
       set objInfo($obj,specX1) $sx1
       set objInfo($obj,specXN) $sxn
       set objInfo($obj,specY1) $sy1
       set objInfo($obj,specYN) $syn

       readView $obj
       showView $obj
      }

   update
}

proc moveCursor  { thisObj thisXPix thisYPix } \
{
   global objInfo

   set obj    $thisObj
   set xPix   $thisXPix
   set yPix   $thisYPix
   set inFlag 1

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} {set inFlag 0} 
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} {set inFlag 0} 

   set objInfo($obj,x0Pix) $xPix
   set objInfo($obj,y0Pix) $yPix

   if {$inFlag} \
      {
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID $objInfo($obj,pixmap)

       set objInfo(data,cursorV) \
        [pix2spec -cell $cellID -win $winID -roi $roiID -x $xPix -units ppm]

       set objInfo(data,cursorH) \
        [pix2spec -cell $cellID -win $winID -roi $roiID -y $yPix -units ppm]
      } \
   else \
      {
       set objInfo(data,cursorH) None
       set objInfo(data,cursorV) None
      }

   drawCursors
}

proc drawCursors {} \
{
   global objInfo

   set inFlag 1

   if {"$objInfo(data,cursorH)" == "None"} {set inFlag 0}
   if {"$objInfo(data,cursorV)" == "None"} {set inFlag 0}

   set fmt "%s: %.3fppm %.0fpts   %s: %.3fppm %.0fpts"

   if {$inFlag} \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue

           set roiID  $objInfo($obj,roi1)
           set cellID $objInfo($obj,cell)
           set winID  $objInfo($obj,pixmap)

           set xv \
            [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) \
               $objInfo(data,cursorV)ppm]
 
           set yv \
            [spec2pnt $objInfo(data,fdata) $objInfo(data,vDim) \
               $objInfo(data,cursorH)ppm]
 
           set qx \
            [spec2pix -roi $roiID -cell $cellID -win $winID \
                      -x $objInfo(data,cursorV)ppm]

           set qy \
            [spec2pix -roi $roiID -cell $cellID -win $winID \
                      -y $objInfo(data,cursorH)ppm]
 
           set str \
              [format $fmt \
                      $objInfo(data,xAxis) $objInfo(data,cursorV) $xv \
                      $objInfo(data,yAxis) $objInfo(data,cursorH) $yv]

           $objInfo($obj,canvas) bind cursorH <Shift-ButtonRelease-1> \
             "insertThisPeak $objInfo(data,cursorV) $objInfo(data,cursorH)"

           $objInfo($obj,canvas) bind cursorV <Shift-ButtonRelease-1> \
             "insertThisPeak $objInfo(data,cursorV) $objInfo(data,cursorH)"

           wm title $objInfo($obj,window) \
            "$str   \[SHIFT Insert Peak at Cursor\] \[None\] \[None\]"

           if {$objInfo(data,cursorV) > $objInfo($obj,specX1) || \
                $objInfo(data,cursorV) < $objInfo($obj,specXN)} \
              {
               set qx [expr $objInfo($obj,x1) - 4]
              }

           if {$objInfo(data,cursorH) > $objInfo($obj,specY1) || \
                $objInfo(data,cursorH) < $objInfo($obj,specYN)} \
              {
               set qy [expr $objInfo($obj,y1) - 4]
              }

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)
   
           $objInfo($obj,canvas) raise cursorH
           $objInfo($obj,canvas) raise cursorV
          }
      } \
   else \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue
       
           wm title $objInfo($obj,window) $objInfo(prog,title)

           set qx [expr $objInfo($obj,x1) - 4]
           set qy [expr $objInfo($obj,y1) - 4]

           $objInfo($obj,canvas) bind cursorH <Shift-ButtonRelease-1> "" 
           $objInfo($obj,canvas) bind cursorV <Shift-ButtonRelease-1> "" 

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)

           $objInfo($obj,canvas) raise cursorH
           $objInfo($obj,canvas) raise cursorV
          }
      }

   update
}

proc selectThisPeak { obj tID pk } \
{
   global objInfo

   set loc [lsearch -exact $objInfo(prog,pkSelectList) $pk]
  
   if {$loc < 0} \
      {
       lappend objInfo(prog,pkSelectList) $pk
      } \
   else \
      {
       set objInfo(prog,pkSelectList) [lreplace $objInfo(prog,pkSelectList) $loc $loc]
      }

#  showThisPeak $obj $tID $pk

   drawPeaks
}

proc modifyThisPeak { obj tID pk } \
{
   global objInfo

   if {[lsearch -exact $objInfo(peak,inUse) $pk] >= 0} \
      {
       showError View2D "Error Modifying: This peak is currently in use."
       return
      }

   set txt "Modify Peak [gdbGet $pk INDEX]?"

   $objInfo($obj,canvas) bind $tID <Any-Leave> "showThisPeak $obj $tID $pk"

   showThisPeak $obj $tID $pk

   set w .pkmDialogKeepWin

   catch "destroy $w"

   set objInfo($obj,modifyStr) [gdbGet $pk ASS]

   toplevel $w -class Dialog
   entry    $w.modEntry -textvariable objInfo($obj,modifyStr)

   pack $w.modEntry -in $w -side bottom -expand 1 -fill x

   if {[tk_dialog $w Modify $txt info 0 Cancel Modify]} \
      {
       gdbSet $pk ASS $objInfo($obj,modifyStr)
      }

   $objInfo($obj,canvas) bind $tID <Any-Leave> "unshowThisPeak $obj $tID $pk"

   unshowThisPeak $obj $tID $pk

   update
}

proc insertThisPeak { sx sy } \
{
   global objInfo

   if {!$objInfo(peak,tab1)} \
      {
       showError View2D "Error: Peak Insert Requires Existing Peak Table."
       return
      }

   set id [getTabMax $objInfo(peak,tab1) INDEX]

   incr id

   showVerb "Peak Insert: $id ${sx}ppm ${sy}ppm"

   set pk [gdbCreate entry -parent $objInfo(peak,tab1)]

   gdbSet $pk INDEX  $id
   gdbSet $pk ASS    None
   gdbSet $pk HEIGHT 0.0

   if {$objInfo(data,tpFlag)} \
      {
       gdbSet $pk X_PPM $sy
       gdbSet $pk Y_PPM $sx
      } \
   else \
      {
       gdbSet $pk X_PPM $sx
       gdbSet $pk Y_PPM $sy
      }

   drawPeaks
}

proc deleteThisPeak { obj tID pk } \
{
   global objInfo

   if {[lsearch -exact $objInfo(peak,inUse) $pk] >= 0} \
      {
       showError View2D "Error Deleting: This peak is currently in use."
       return
      }

   set txt "Delete Peak [gdbGet $pk INDEX]?"
 
   $objInfo($obj,canvas) bind $tID <Any-Leave> "showThisPeak $obj $tID $pk"
 
   showThisPeak $obj $tID $pk

   if {![tk_dialog .pkdDialog Delete $txt info 0 Cancel Delete]} \
      {
       $objInfo($obj,canvas) bind $tID <Any-Leave> "unshowThisPeak $obj $tID $pk"
       unshowThisPeak $obj $tID $pk
       return
      }

   $objInfo($obj,canvas) delete $tID

   gdbFree $pk

   update
}

proc showThisPeak { obj tID pk } \
{
   global objInfo env

   set iStr "Peak [gdbGet $pk INDEX]"
   set xStr "$objInfo(data,xAxis)=[gdbGet $pk $objInfo(peak,xName)]ppm"
   set yStr "$objInfo(data,yAxis)=[gdbGet $pk $objInfo(peak,yName)]ppm"
   set hStr "Ass=[gdbGet $pk ASS] Height=[gdbGet $pk HEIGHT]"
   set mStr " Ctrl: \[Select\] \[Ass\] \[Delete\]"

   wm title $objInfo($obj,window) "$iStr $xStr $yStr $hStr $mStr"

   if {$objInfo(prog,pkSelectFlag)} \
      {
       if {[lsearch -exact $objInfo(prog,pkSelectList) $pk] >= 0} \
          {
           set thisColor $objInfo(color,selectH)
          } \
       else \
          {
           set thisColor $objInfo(color,peakH)
          }

       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map
      }

   $objInfo($obj,canvas) raise $tID

   update
}

proc unshowThisPeak { obj tID pk } \
{
   global objInfo env

   wm title $objInfo($obj,window) $objInfo(prog,title)

   if {$objInfo(prog,pkSelectFlag)} \
      {
       if {[lsearch -exact $objInfo(prog,pkSelectList) $pk] >= 0} \
          {
           set thisColor $objInfo(color,select)
          } \
       else \
          {
           set thisColor $objInfo(color,peak)
          }

       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/clear.map
      }

   $objInfo($obj,canvas) raise $tID

   update
}

proc getViewPeakList { obj tab } \
{
   global objInfo

   set pkList ""

   if {!$tab} {return $pkList}

   set    cond ""
   append cond "$objInfo(peak,xName) >= $objInfo($obj,specXN)"
   append cond " && $objInfo(peak,xName) <= $objInfo($obj,specX1)"
   append cond " && $objInfo(peak,yName) >= $objInfo($obj,specYN)"
   append cond " && $objInfo(peak,yName) <= $objInfo($obj,specY1)"
  
   set pkList [gdbSelect -from $tab -cond $cond]

   return $pkList
}

proc drawPeaks {} \
{
   global objInfo env stdfont bigfont

   showVerb "Drawing peaks ..."

   set i 1

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       $objInfo($obj,canvas) delete peak

       set pkList [getViewPeakList $obj $objInfo(peak,tab$i)]

       foreach pk $pkList \
          {
           set xPPM [gdbGet $pk $objInfo(peak,xName)]ppm
           set yPPM [gdbGet $pk $objInfo(peak,yName)]ppm
           set str  [gdbGet $pk INDEX]

           set tx0 \
           [spec2pix -roi  $objInfo($obj,roi1) \
                     -cell $objInfo($obj,cell) \
                     -win  $objInfo($obj,pixmap) \
                     -x $xPPM]

           set ty0 \
           [spec2pix -roi  $objInfo($obj,roi1)   \
                     -cell $objInfo($obj,cell)   \
                     -win  $objInfo($obj,pixmap) \
                     -y $yPPM]

           if {$objInfo(prog,pkSelectFlag)} \
              {
               if {[lsearch -exact $objInfo(prog,pkSelectList) $pk] >= 0} \
                  {
                   set thisColor  $objInfo(color,select)
                   set thisShadow $objInfo(color,selectS)
                  } \
               else \
                  {
                   set thisColor  $objInfo(color,peak)
                   set thisShadow $objInfo(color,peakS)
                  }

               set sID [$objInfo($obj,canvas) create text [expr int($tx0) - 1] [expr int($ty0) - 1] \
                       -anchor sw  -fill $thisShadow -tags peak -text $str \
                       -font $bigfont -stipple @$env(NMRTXT)/solid.map]

               set tID [$objInfo($obj,canvas) create text [expr int($tx0)] [expr int($ty0)] \
                       -anchor sw  -fill $thisColor -tags peak -text $str \
                       -font $bigfont -stipple @$env(NMRTXT)/solid.map]
              } \
           else \
              {
               set tID [$objInfo($obj,canvas) create text $tx0 $ty0 -anchor sw   \
                       -fill $objInfo(color,peak) -tags peak -text $str          \
                       -font $bigfont -stipple @$env(NMRTXT)/clear.map]
              }

# %%% +

           $objInfo($obj,canvas) bind $tID <Any-Enter> "showThisPeak   $obj $tID $pk"
           $objInfo($obj,canvas) bind $tID <Any-Leave> "unshowThisPeak $obj $tID $pk"

           $objInfo($obj,canvas) bind $tID <Shift-ButtonRelease-1> "" 
           $objInfo($obj,canvas) bind $tID <Shift-ButtonRelease-2> "" 
           $objInfo($obj,canvas) bind $tID <Shift-ButtonRelease-3> "" 

           $objInfo($obj,canvas) bind $tID <Control-ButtonRelease-1> \
              "selectThisPeak $obj $tID $pk"

           $objInfo($obj,canvas) bind $tID <Control-ButtonRelease-2> \
              "modifyThisPeak $obj $tID $pk"

           $objInfo($obj,canvas) bind $tID <Control-ButtonRelease-3> \
              "deleteThisPeak $obj $tID $pk"
          }
      }
}

setOSType

gdbInit
gdbCreate dBase

initProgVars
initDataArgs
initPeakVars

initProgColors
initSpecColors

initProgArgs

initMainWindow

if {$objInfo(prog,autoFlag)} \
   {
    startView
#   wm iconify .
   }

#
# Bottom
