#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "View Results of Lineshape Fitting:"
   echo " -ft  ftName   [Auto]  Measured Data."
   echo " -sim simName  [Auto]  Model Data Produced by autoFit.tcl utility."
   echo " -dif difName  [Auto]  Residual Data Produced by autoFit.tcl utility."
   echo " -hi  hi       [Auto]  Contour Height."
   exit 0
endif

set ftName  = (`getArgD $argv -ft  Auto`)
set simName = (`getArgD $argv -sim Auto`)
set difName = (`getArgD $argv -dif Auto`)
set hi      = (`getArgD $argv -hi  Auto`)

set simList  = (sim/test%04d.ft4 sim/test%03d.ft4 sim/test%04d.ft3 sim/test%03d.ft3 sim/test%03d.ft2 sim.ft2 sim.ft1)
set difList  = (dif/test%04d.ft4 dif/test%03d.ft4 dif/test%04d.ft3 dif/test%03d.ft3 dif/test%03d.ft2 dif.ft2 dif.ft1)

set ftList   = (smile/test%04d.ft4,hms_ist/test%04d.ft4,ist/test%04d.ft4,ft/test%04d.ft4 \
                smile/test%03d.ft4,hms_ist/test%03d.ft4,ist/test%03d.ft4,ft/test%03d.ft4 \
                smile/test%04d.ft3,hms_ist/test%04d.ft3,ist/test%04d.ft3,ft/test%04d.ft3 \
                smile/test%03d.ft3,hms_ist/test%03d.ft3,ist/test%03d.ft3,ft/test%03d.ft3 \
                smile/test%03d.ft2,hms_ist/test%03d.ft2,ist/test%03d.ft2,ft/test%03d.ft2 \
                smile.ft2,hms_ist.ft2,ist.ft2,spec.ft2,test.ft2 \
                smile.ft1,hms_ist.ft1,ist.ft1,spec.ft1,test.ft1)

set testList = None

if (`string csame $simName Auto`) then
   @ i = 0

   foreach s ($simList)
      @ i++

      set thisName = (`nmrPrintf $s 1 1`)

      if (-e $thisName) then
         set simName = $s

         echo "simName Auto Mode: $simName"

         set testList = (`string split $ftList[$i] ","`)

         if (`string csame $difName Auto`) then
            set thisName = (`nmrPrintf $difList[$i] 1 1`)

            if (-e $thisName) then
               set difName = $difList[$i]

               echo "difName Auto Mode: $difName"
            endif
         endif

         break
      endif
   end
endif

if (`string csame $simName Auto`) then
   echo "Error finding simName in Auto Mode."
   exit 1
endif

if (`string csame $difName Auto`) then
   @ i = 0

   foreach d ($difList)
      @ i++

      set thisName = (`nmrPrintf $d 1 1`)

      if (-e $thisName) then
         set difName = $d

         echo "difName Auto Mode: $difName"
         break
      endif
   end
endif

if (`string csame $difName Auto`) then
   echo "Error finding difName in Auto Mode."
   exit 1
endif

if (`string csame $ftName Auto`) then
   if (`string csame $testList None`) then
      echo "Error finding ftName in Auto Mode."
      exit 1
   endif

   foreach f ($testList)
      set thisName = (`nmrPrintf $f 1 1`)

      if (-e $thisName) then
         set ftName = $f

         echo "ftName Auto Mode: $ftName"
         break
      endif
   end
endif

if (`string csame $ftName Auto`) then
   echo "Error finding ftName in Auto Mode."
   exit 1
endif

set thisName = (`nmrPrintf $simName 1 1`)
   
if (!(-e $thisName)) then
   echo "Error Finding simName $thisName"
   exit 1
endif

set thisName = (`nmrPrintf $difName 1 1`)

if (!(-e $thisName)) then
   echo "Error Finding difName $thisName"
   exit 1
endif

set thisName = (`nmrPrintf $ftName 1 1`)

if (!(-e $thisName)) then
   echo "Error Finding ftName $thisName"
   exit 1
endif

set hiArg = (`getSpecHi.tcl -in $ftName -hi $hi -brief`)

echo ""
echo "Spectrum: $ftName Simulation: $simName Residual: $difName Hi: $hiArg"

nmrDraw -in $ftName  -hi $hiArg -broadcast `getGeom -left`  -nophase -group 1 &
sleep 3

nmrDraw -in $simName -hi $hiArg -broadcast `getGeom -mid`   -nophase -group 1 &
sleep 3

nmrDraw -in $difName -hi $hiArg -broadcast `getGeom -right` -nophase -group 1 &

exit 0

