#!/bin/csh

if (`flagLoc $argv -help`) then
   echo "Delete data and scripts produced vj2pipe.tcl:"
   echo " -cd      cdDir \[.\]  Default Working Directory."
   echo " -pipeDir pDir  \[.\]  Directory for Output Scripts and Files."
   echo " -com                Delete .com files as well as data."
   echo " -nocom              Delete data only (Default)."
   echo " -safe               Clean only if file timestamp.vj2pipe exists (Default)."
   echo " -nosafe             Clean without checking if file timestamp.vj2pipe exists."
   echo " -verb               Verbose Mode ON (Default)."
   echo " -noverb             Verbose Mode OFF."
   exit 0
endif

set cdDir    = (`getArgD $argv -cd      .`)
set pipeDir  = (`getArgD $argv -pipeDir .`)
set origDir  = (`pwd`)
set dirList  = (.)
set comFlag  = 0
set safeFlag = 1
set vFlag    = 1
set n        = 0

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (`flagLoc $argv -com`) then
   set comFlag = 1
endif

if (`flagLoc $argv -nocom`) then
   set comFlag = 0
endif

if (`flagLoc $argv -safe`) then
   set safeFlag = 1
endif

if (`flagLoc $argv -nosafe`) then
   set safeFlag = 0
endif

if (($cdDir != ".") && ($cdDir != "./")) then
   if ($vFlag) then
      echo "vj_clean Changing Working Directory:"
      echo " $cdDir"
      echo ""
   endif

   cd $cdDir

   if ($vFlag) then
      echo "vj_clean Directory Status:"
      echo " `pwd`"
      echo ""
   endif
endif

if (($pipeDir != ".") && ($pipeDir != "./")) then
   set dirList = ($dirList $pipeDir)
endif

set prevDir = (`pwd`)

foreach dd ($dirList)
   if ($safeFlag) then
      if (!(-e $dd/timestamp.vj2pipe)) then
         if ($vFlag) then
            echo "Skipping Directory $dd ... No timestamp.vj2pipe file found."
         endif

         continue
      endif
   endif

   if ($vFlag) then
      echo "Cleaning Directory $dd ..."
   endif

   if (-d $dd) then
      cd $dd
   endif

   foreach d (data nusmask ft lp lps pcalp mem ml mlsp cln mdd ist istmult istspec xy xz yz yx zx zy)
      if (-d $d) then
         if ($vFlag) then
            echo " $d"
         endif

         /bin/rm -rf $d
         @ n++
      endif
   end

   set fList = (spec_nus.fid spec_mask.fid spec.fid spec.ft1 spec.ft2 xz001.fid xa001.fid \
                spec_xy.fid spec_xz.fid spec_xa.fid spec_xy.ft2 spec_xz.ft2 spec_xa.ft2 \
                spec_ist.ft2 ist.ft1 ist.ft2 ist_res.ft2 ist_tmp.ft1 ist_tmp.ft2 ist_mask.ft1 ist_inv.ft1 ist_sum.ft2 ist.ft3 ist_res.ft3 ist_sum.ft3 \
                test.fid test.ft1 test.dat test.ft2)

   foreach f ($fList)
      if (-e $f) then
         if ($vFlag) then
            echo " $f"
         endif

         /bin/rm -f $f
         @ n++
      endif
   end

   set fInfo = (`/bin/ls | fgrep .dat | wc -l` 0)

   if ($fInfo[1] >= 1) then
      foreach f (*.dat)
         if (-e $f) then
            if ($vFlag) then
               echo " $f"
            endif

            /bin/rm -f $f
            @ n++
         endif
      end
   endif
 
   if ($comFlag == 1) then
      set fList = (vj_fid.com vj_ft1_user.com \
                   vj_ft2_user.com vj_ft2_ist_user.com \
                   vj_ft2mc_user.com vj_ft2mc_ist_user.com \
                   vj_ft2av_user.com vj_ft2av_ist_user.com \
                   vj_xy_user.com vj_xz_user.com vj_xa_user.com  \
                   vj_xyz_user.com vj_xyz_ist_user.com vj_xyza_user.com \
                   vj_ft1_expert.com vj_ft2_expert.com vj_ft2_ist_expert.com \
                   vj_ft2mc_expert.com vj_ft2mc_ist_expert.com \
                   vj_xy_expert.com vj_xz_expert.com vj_xa_expert.com  \
                   vj_xyz_expert.com vj_xyz_ist_expert.com vj_xyza_expert.com \
                   vj_scroll_user.com vj_scroll_expert.com vj_tmp.com \
                   vj_tmp_user.com vj_tmp_expert.com proj.tab test.tab pk.tcl \
                   timestamp.vj2pipe)

      foreach f ($fList)
         if (-e $f) then
            if ($vFlag) then
               echo " $f"
            endif

            /bin/rm -f $f
            @ n++
         endif 
      end
   endif

   cd $prevDir
end

cd $origDir

if ($n != 0) then
   if ($vFlag) then
      echo ""
   endif
endif

exit 0
