#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path    "[split $env(TCLPATH) :] $auto_path"
set ARGV         [concat $argv0 $argv]
set ARGC         [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "Align MFR Fragment Library with DC Alignment Tensors:"
    puts "Input:"
    puts " -pdb    pdbList   \[frag/*.pdb\]       Input PDB Files."
    puts " -out    outList   \[pdbList\]          Output PDB Files."
    puts " -dc     dcList    \[dObs\[A-Z\].tab\]  Input DC Tables."
    puts " -seg    segName   \[Default\]          Segment Name, or 'None'."
    puts "Optional Specification of Fixed Tensor Parameters:"
    puts " -da     daList    \[None\]             Da Values."
    puts " -dr     drList    \[None\]             Dr Values."
    exit 1
   }

set dcNameList  [glob -nocomplain dObs\[A-Z\].tab]
set pdbNameList [glob -nocomplain frag/*.pdb]
set segName     Default 
set outNameList ""

set daList      "" 
set drList      ""
set dadrFlag    0

getListArgD $ARGV -pdb pdbNameList
getListArgD $ARGV -out outNameList
getListArgD $ARGV -dc  dcNameList
getListArgD $ARGV -da  daList
getListArgD $ARGV -dr  drList

getArgD $ARGV -seg segName

#
#-----------------------------------------------------------------------

proc addThisFrame { pdbSrc outName segName id } \
{
   set pInfo     [dynGetInfo -src $pdbSrc -limits]
   set atomCount [lindex $pInfo 2]

   set pInfo [dynGetInfo -src $pdbSrc -extent -all]
   set i 0

   foreach v {x1 xn y1 yn z1 zn x0 y0 z0} \
      {
       set $v [lindex $pInfo $i]
       incr i
      }

   set x0 [expr $xn + 50.0 + $id*10.0]

   set sx [dynGetPDBStr -atomID [expr $atomCount + 1] \
                        -resID  [expr $id + 500] -seg $segName  \
                        -resName ANI -atomName X \
                        -x [expr $x0 + 3.0] -y 0.0 -z 0.0]

   set sy [dynGetPDBStr -atomID [expr $atomCount + 2] \
                        -resID  [expr $id + 500] -seg $segName  \
                        -resName ANI -atomName Y  \
                        -x $x0 -y 3.0 -z 0.0]

   set sz [dynGetPDBStr -atomID [expr $atomCount + 3] \
                        -resID  [expr $id + 500] -seg $segName  \
                        -resName ANI -atomName Z  \
                        -x $x0 -y 0.0 -z 3.0]


   set s0 [dynGetPDBStr -atomID [expr $atomCount + 4] \
                        -resID  [expr $id + 500] -seg $segName  \
                        -resName ANI -atomName OO  \
                        -x $x0 -y 0.0 -z 0.0]

   exec echo $sx >> $outName
   exec echo $sy >> $outName
   exec echo $sz >> $outName
   exec echo $s0 >> $outName
   exec echo END >> $outName
}

#
#-----------------------------------------------------------------------

if {![llength $outNameList]} {set outNameList $pdbNameList}

if {![strcasecmp $daList None]} {set daList ""}
if {![strcasecmp $drList None]} {set drList ""}

if {![strcasecmp $segName None]} {set segName ""}

if {[llength $pdbNameList] != [llength $outNameList]} \
   {
    puts stderr "$argv0 Error: Input/Output PDB Count Mismatch."
    exit 1
   }

if {[llength $daList]} \
   {
    set dadrFlag 1

    if {[llength $daList] != [llength $drList]} \
       {
        puts stderr "$argv0 Error: Da/Dr Count Mismatch."
        exit 1
       }

    if {[llength $daList] != [llength $dcNameList]} \
       {
        puts stderr "$argv0 Error: Tensor/DC Table Count Mismatch."
        exit 1
       }
   }

set dcSrcList ""

foreach dcName $dcNameList \
   {
    if {![file exists $dcName]} \
       {
        puts stderr "$argv0 Error Finding DC Input $dcName"
        exit 1
       }

    lappend dcSrcList [dynRead -dc -in $dcName]
   }

#
#-----------------------------------------------------------------------

set j 0

foreach pdbName $pdbNameList \
   {
    set outName [lindex $outNameList $j]

    if {![file exists $pdbName]} \
       {
        puts "$argv0 Skipping Input PDB $pdbName"
       }

    set thisSeg $segName
    set inName  $pdbName
    set i 0

    foreach dcName $dcNameList \
       {
        set pdbSrc [dynRead -pdb -in $inName]
        set dcSrc  [lindex $dcSrcList $i]

        if {!$i} \
           {
            if {![strcasecmp $thisSeg Default]} \
               {
                set thisSeg [dynGetInfo -src $pdbSrc -segName]
               } \
            elseif {![strcasecmp $thisSeg None]} \
               {
                set thisSeg ""
               }
           }

        if {$dadrFlag} \
           {
            set da [lindex $daList $i]
            set dr [lindex $drList $i]

            set dInfo \
             [dynSimulate -dc $dcSrc -src $pdbSrc \
                          -dadr -da $da -dr $dr -retry 16]
           } \
        else \
           {
            set dInfo [dynSimulate -dc $dcSrc -src $pdbSrc -svd] 
           }

        set rx [lindex $dInfo 6]
        set ry [lindex $dInfo 7]
        set rz [lindex $dInfo 8]

        dynRotate -src $pdbSrc -all -rx $rx -ry $ry -rz $rz
        dynWrite  -src $pdbSrc -pdb -out $outName -renumber -noend

        addThisFrame $pdbSrc $outName $thisSeg $i
        dynFree $pdbSrc

        set inName $outName

        incr i        
       }

    incr j
   }

exit

