#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set auto_path [linsert $auto_path 0 $env(NMRBASE)/dynamo/tcl]

set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
   {
    puts "MFR Fragment Dipolar Coupling Refinement by XPLOR:"
    puts " -in    inPDBName   \[frag.pdb\]     Input Fragment PDB Name."
    puts " -psf   psfName     \[ref.psf\]      Input Parent PSF Name."
    puts " -out   outPDBName  \[sa.pdb\]       Output PDB Name."
    puts " -tmp   tmpPDBName  \[xplor.pdb\]    Temporary PDB Output Name."
    puts " -xplor xplorName   \[xplor\]        XPLOR Executable Name."
    puts "DC Parameters, Required:"
    puts " -dc   dcNames  \[dObs\[A-Z\].tab\]   DYNAMO Format DC Input Tables."
    puts " -da   daList   \[None\]            Corresponding Tensor Da Values."
    puts " -dr   drList   \[None\]            Corresponding Tensor Dr Values."
    puts "Optional Restraint Input:"
    puts " -noe  noeTabName   \[None\]     DYNAMO Format NOE Table."
    puts " -tor  torTabName   \[None\]     DYNAMO Format Torsion Table."
    puts " -vean veanInpName  \[None\]     XPLOR Format Vector Angles."
    exit 1
   }

set dcNameList  [glob -nocomplain dObs\[A-Z\].tab]
set xplorName   xplor
set psfName     ref.psf
set pdbName     frag.pdb
set tmpName     xplor.pdb
set outName     sa.pdb

set noeInName   None
set torInName   None
set noeOutName  None
set torOutName  None
set veanName    None

set daList      None 
set drList      None 

getListArgD $ARGV -dc  dcNameList
getListArgD $ARGV -da  daList
getListArgD $ARGV -dr  drList

getArgD $ARGV -xplor   xplorName
getArgD $ARGV -psf     psfName 
getArgD $ARGV -in      pdbName
getArgD $ARGV -out     outName
getArgD $ARGV -tmp     tmpName

getArgD $ARGV -noe     noeInName
getArgD $ARGV -tor     torInName
getArgD $ARGV -vean    veanName

#
#--------------------------------------------------------------------------

if {[strcasecmp $noeInName None]} {set noeOutName noes.xdcn}
if {[strcasecmp $torInName None]} {set torOutName torsions.xdih}

if {![strcasecmp $daList None] || ![strcasecmp $drList None]} \
   {
    puts stderr "Error: Missing -da or -dr Arguments.
    exit 1
   }

set  cmndArgs "-dc $dcNameList -da $daList -dr $drList"
eval "exec xAddFrame.tcl -pdb $pdbName -out $tmpName $cmndArgs"

eval "exec xRestraints.tcl -pdb $pdbName -noe $noeInName -tor $torInName"
eval "exec xDCSetup.tcl -da $daList -dr $drList"

set  cmndArgs "-script sa.inp -noe $noeOutName -tor $torOutName -vean $veanName"
eval "exec xScript.tcl -in $tmpName -out $outName $cmndArgs"

eval "exec $xplorName < sa.inp"

catch "file delete $tmpName"

exit
