#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts "Convert xlsx-Format Table of 96-well Plate IDs to NMRPipe Table:"
 puts " -in    inDir   \[xlsx\]      Directory of xlsx-Format Input Files."
 puts " -tmp   tmpDir  \[xlsx_tmp\]  Directory for Temporary Conversion Output."
 puts " -out   outDir  \[id\]        Directory for NMRPipe-Format Output Tables."
 puts " -var   varList \[None\]      Optional Variable Names for Columns."
 puts " -fmt   fmtList \[None\]      Optional Formats for Columns."
 puts " -plate                     Input is a List of 96-well Plate IDs."
 puts " -lab                       Use Sheet Tab Names in Output Name."
 puts " -nohdr                     No VARS/FORMAT Header Output."
 puts "Notes:"
 exit 0
}

set inDir       xlsx
set tmpDir      xlsx_tmp
set outDir      id
set hdrFlag     1
set plateFlag   0
set labFlag     0

set userVarList None
set userFmtList None

getArgD $ARGV -in  inDir
getArgD $ARGV -tmp tmpDir
getArgD $ARGV -out outDir

if {[flagLoc $ARGV -hdr]}     {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}   {set hdrFlag 0}

if {[flagLoc $ARGV -lab]}     {set labFlag 1}
if {[flagLoc $ARGV -nolab]}   {set labFlag 0}

if {[flagLoc $ARGV -plate]}   {set plateFlag 1}
if {[flagLoc $ARGV -noplate]} {set plateFlag 0}

if {$plateFlag} \
   {
    set userVarList "EXPERIMENT CMPD_ID"
    set userFmtList "%12s %12s"
   }

getListArgD $ARGV -var userVarList
getListArgD $ARGV -fmt userFmtList

if {![strcasecmp $userVarList None]} {set userVarList ""}
if {![strcasecmp $userFmtList None]} {set userFmtList ""}

set userVarList [string trim $userVarList "{}"]
set userFmtList [string trim $userFmtList "{}"]

set nUserVar [llength $userVarList]
set nUserFmt [llength $userFmtList]

if {$nUserFmt < $nUserVar} \
   {
    set n [expr $nUserVar - $nUserFmt]
    for {set i 0} {$i < $n} {incr i} {lappend userFmtList "%12s"}
    set nUserFmt [llength $userFmtList]
   }

if {$nUserVar != $nUserFmt} \
   {
    puts "XLS2Tab Error: VARS/FORMAT Mismatch."
    puts " VARS:    $userVarList"
    puts " FORMAT:  $userFmtList"
    exit 1
   }

#
#------------------------------------------------------------------------------

proc getXMLRels { thisName } \
{
   global file2ID

   catch "unset relInfo"
   catch "unset file2ID"

   set relInfo(tagCount) 0
   set file2ID(idList)   ""
   set file2ID(fileList) ""

   if {![file exists $thisName]} \
      {
       puts "Skipping xmlx _rels $thisName"
       return 0
      }

   set error [catch {set inUnit [open $thisName r]}]

   if {$error} \
      {
       puts "Error opening xmlx _rels $thisName"
       return 2
      }

   set buff  [read $inUnit]
   catch "close $inUnit"

   set n [parseTags $buff relInfo "\"'"]

   for {set i 1} {$i <= $n} {incr i} \
      {
       set key [lindex $relInfo($i) 0]
       if {[strcasecmp $key "<Relationship"]} continue

       set id [getTagData $relInfo($i) Id]
       if {![llength $id]} {set id None}

       set pathName [getTagData $relInfo($i) Target]
       if {![llength $pathName]} {set pathName None}

       set fileName [file tail $pathName]

       lappend file2ID(idList)    $id
       lappend file2ID(fileList)  $pathName
       lappend file2ID($fileName) $id
      }

   return 0
}

proc getXMLWorkbook { thisName } \
{
   global id2Name

   catch "unset workbookInfo"
   catch "unset id2Name"

   set workbookInfo(tagCount) 0

   set id2Name(nameList) ""
   set id2Name(idList)   ""

   if {![file exists $thisName]} \
      {
       puts "Skipping xmlx workbook $thisName"
       return 0
      }

   set error [catch {set inUnit [open $thisName r]}]

   if {$error} \
      {
       puts "Error opening xmlx workbook $thisName"
       return 2
      }

   set buff  [read $inUnit]
   catch "close $inUnit"

   set n [parseTags $buff workbookInfo "\"'"]

   for {set i 1} {$i <= $n} {incr i} \
      {
       set key [lindex $workbookInfo($i) 0]
       if {[strcasecmp $key "<sheet"]} continue

       set id [getTagData $workbookInfo($i) r:id]
       if {![llength $id]} {set id None}

       set name [getTagData $workbookInfo($i) name]
       if {![llength $name]} {set name None}

       lappend id2Name(idList)   $id
       lappend id2Name(nameList) $name
       lappend id2Name($id)      $name
      }

   return 0
}

proc getXMLStrings { thisName } \
{
   global xmlInfo

   set xmlInfo(strList) ""

   set s ""
   set t ""

   if {![file exists $thisName]} \
      {
       puts "Error finding xmlx component $thisName"
       return 1
      }

   set error [catch {set inUnit [open $thisName r]}]

   if {$error} \
      {
       puts "Error opening xmlx component $thisName"
       return 2 
      }

   set buff  [read $inUnit]
   catch "close $inUnit"

   set count [string length $buff]

   set pFlag 0

   for {set i 0} {$i < $count} {incr i} \
      {
       set c [string index $buff $i]
   
       if {![strcmp $c "<"]} \
          {
           set pFlag 1
           append t $c
          } \
       elseif {![strcmp $c ">"]} \
          {
           set pFlag 0
           append t $c
          } \
       else \
          {
           if {$pFlag} \
              {
               if {![string is space $c]} {append t $c}
              } \
           else \
              {
               append t $c
              }
          }
      }

   set tt   [string tolower $t]
   set loc1 0
   set locN 0

   while {$loc1 >= 0} \
      {
       set loc1 [string first "<t>" $tt $loc1]
       if {$loc1 < 0} break

       incr loc1 3

       set locN [string first "</t>" $tt $loc1]
       if {$locN < 0} {set locN [string first $tt "<" $loc1]}
       if {$locN < 0} break 
     
       incr locN -1

       if {$locN < $loc1} break
 
       lappend s [string range $t $loc1 $locN] 
      }
  
   set xmlInfo(strList) $s
 
   return 0 
}

proc getXMLData { thisName } \
{
   global xmlInfo

   if {![file exists $thisName]} \
      {
       puts "Error Finding xmlx Component $thisName"
       return 1
      }

   set error [catch {set inUnit [open $thisName r]}]

   if {$error} \
      {
       puts "Error Opening xmlx Component $thisName"
       return 2
      }

   set buff  [read $inUnit]
   catch "close $inUnit"

   set strList ""
   set s ""
   set t ""

   if {[info exists xmlInfo(strList)]} {set strList $xmlInfo(strList)}

   set count [string length $buff]
   set nS    [llength $strList]
   set pFlag 0

   for {set i 0} {$i < $count} {incr i} \
      {
       set c [string index $buff $i]
   
       if {![strcmp $c "<"]} \
          {
           set pFlag 1
           append t $c
          } \
       elseif {![strcmp $c ">"]} \
          {
           set pFlag 0
           append t $c
          } \
       elseif {![strcmp $c \"]} \
          {
           set c "'"
          } \
       else \
          {
           if {$pFlag} \
              {
               if {[string is space $c]} \
                  {
                   if {$pFlag > 1} {append t $c}
                  } \
               else \
                  {
                   append t $c
                   incr pFlag
                  }
              } \
           else \
              {
               append t $c
              }
          }
      }

   set tt    [string tolower $t]
   set count [string length $tt]

   set loc1 0
   set locN 0

   while {$loc1 >= 0} \
      {
       if {$loc1 >= $count} break

       set cellInfo None

       set loc1 [string first "<c " $tt $loc1]
       if {$loc1 < 0} break

       if {$loc1 >= 0} \
          {
           incr loc1 3
           set locN [string first ">" $tt $loc1]

           if {$locN >= 0} \
              {
               incr locN -1
 
               if {$locN >= $loc1} \
                  {
                   set cellInfo [string range $tt $loc1 $locN]
                   set loc1 [expr $locN + 1]
                  }
              }
          }
 
       parseCellInfo $cellInfo row col type

       set loc1 [string first "<v>" $tt $loc1]
       if {$loc1 < 0} break

       incr loc1 3

       set locN [string first "</v>" $tt $loc1]
       if {$locN < 0} {set locN [string first $tt "<" $loc1]}
       if {$locN < 0} break 
     
       incr locN -1

       if {$locN < $loc1} break

       set valID [string range $t $loc1 $locN]
       set val   $valID

       if {![strcasecmp $type string]} \
          {
           if {[string is int $valID]} \
              {
               if {$valID >= 0 && $valID < $nS} \
                  {
                   set val [lindex $strList $valID]
                  }
              }
          }

       set nV [expr 1 + [string length $val]]

       if {![info exists xmlInfo($row,size)]} {set xmlInfo($row,size) 1}
       if {![info exists xmlInfo($col,size)]} {set xmlInfo($col,size) 1}

       if {$nV > $xmlInfo($row,size)} {set xmlInfo($row,size) $nV}
       if {$nV > $xmlInfo($col,size)} {set xmlInfo($col,size) $nV}

       set xmlInfo($row,$col,valID) $valID
       set xmlInfo($row,$col,type)  $type
       set xmlInfo($row,$col,val)   $val

       set loc1 [expr $locN + 1]
      }
   
   return 0
}

proc parseCellInfo { cellInfo rowPtr colPtr typePtr } \
{
   upvar $rowPtr  row
   upvar $colPtr  col
   upvar $typePtr type

   global xmlInfo

# r=b8 s=1 t=s

   set row  0
   set col  None
   set type None

   foreach s $cellInfo \
      {
       set sInfo [split $s "="]
       if {[llength $sInfo] != 2} continue

       set key [lindex $sInfo 0]

       if {![strcasecmp $key r]} \
          {
           set rc [lindex $sInfo 1]
           set n  [string length $rc]

           if {$n > 1} \
              {
               set col [string index $rc 0]
               set row [string range $rc 1 end]

               if {![string is int $row]} {set row -1}
              } 
          } \
       elseif {![strcasecmp $key t]} \
          {
           set type [lindex $sInfo 1]
           if {![strcasecmp $type "s"]} {set type string}
          }
      }

   if {![strcasecmp $row None]} {set type data}

   if {[lsearch -exact $xmlInfo(rowList) $row] < 0} \
      {
       lappend xmlInfo(rowList) $row
       set xmlInfo($row,size) 1
      } 

   if {[lsearch -exact $xmlInfo(colList) $col] < 0} \
      {
       lappend xmlInfo(colList) $col
       set xmlInfo($col,size) 1
      } 
}

proc saveXMLData { outName outLabel } \
{
   global xmlInfo nUserVar nUserFmt userVarList userFmtList plateFlag hdrFlag

   if {![strcasecmp outName None]} {return 0}

   if {![llength $xmlInfo(rowList)]} \
      {
       puts "Error: No input rows were found."
       return 1
      }

   if {![llength $xmlInfo(colList)]} \
      {
       puts "Error: No input columns were found."
       return 2
      }

   if {![strcasecmp $outName stdout]} \
      {
       set outUnit stdout
      } \
   elseif {![strcasecmp $outName stderr]} \
      {
       set outUnit stderr
      } \
   else \
      {
       set error [catch {set outUnit [open $outName w]}]

       if {$error} \
          {
           puts "Error Creating output file $outName"
           return 3
          }
      }

   set rowList [lsort -integer $xmlInfo(rowList)] 
   set colList [lsort -ascii   $xmlInfo(colList)]

   if {$hdrFlag} \
      {
       puts $outUnit "DATA LABEL $outLabel"
       puts $outUnit ""

       if {$nUserVar} \
          {
           set firstDataRow 0

           puts $outUnit "VARS   $userVarList"
           puts $outUnit "FORMAT $userFmtList"
           puts $outUnit ""
          } \
       else \
          {
           set row [lindex $rowList 0]
           set firstDataRow 1

           puts -nonewline $outUnit "VARS   "

           foreach col $colList \
              {
               if {[info exists xmlInfo($row,$col,val)]} \
                  {
                   puts -nonewline $outUnit " [string toupper [join $xmlInfo($row,$col,val) _]]"
                  } \
               else \
                  {
                   puts -nonewline $outUnit " [string toupper COL_[join $col _]]"
                  }
              }

           puts $outUnit ""
           puts -nonewline $outUnit "FORMAT "

           foreach col $colList \
              {
               set fmt " %12s"

               if {[info exists xmlInfo($col,size)]} \
                  {
                   set fmt " %$xmlInfo($col,size)s"
                  }

               puts -nonewline $outUnit $fmt
              }

           puts $outUnit ""
           puts $outUnit ""
          }
      }

   set nCol  [llength $colList]
   set rowID 0

   foreach row [lrange $rowList $firstDataRow end] \
      {
       incr rowID

       if {$plateFlag && $nCol == 1} {puts -nonewline $outUnit [format " %12s" $rowID]} 
          
       foreach col $colList \
          {
           set fmt " %12s"

           if {[info exists xmlInfo($col,size)]} \
              {
               set fmt " %$xmlInfo($col,size)s"
              }

           if {[info exists xmlInfo($row,$col,val)]} \
              {
               puts -nonewline $outUnit [format $fmt $xmlInfo($row,$col,val)]
              } \
           else \
              {
               puts -nonewline $outUnit [format $fmt 1]
              }
          }

       puts $outUnit ""
      }

    if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
       {
        catch "close $outUnit"
       }

    return 0
}

proc xmlInit {} \
{
   global xmlInfo

   catch "unset xmlInfo"

   set xmlInfo(rowList) ""
   set xmlInfo(colList) ""
   set xmlInfo(strList) ""

   return 0
}

set xList [lsort -dictionary [glob -nocomplain $inDir/*.xlsx]]

if {![llength $xList]} \
   {
    puts "No .xlsx Files Found in directory $inDir"
    exit 0
   }

foreach xName $xList \
   {

    set baseName [string tolower [file root [file tail $xName]]]
    set rootName ""
    set strList  ""

    foreach i $baseName {append rootName $i}

    if {[file exists $tmpDir]} {catch "file delete -force $tmpDir"}

    catch "file mkdir $tmpDir"

    file copy $xName $tmpDir/$rootName.zip
    exec unzip $tmpDir/$rootName.zip -d $tmpDir

    if {![file exists $tmpDir/xl/sharedStrings.xml]} \
       {
        puts "Error Finding xl/sharedStrings.xml for $xName"
        continue
       }

    xmlInit

    set error [getXMLStrings $tmpDir/xl/sharedStrings.xml]

    if {$error} \
       {
        puts "Error Extracting string data for $xName"
        continue
       }

    set error [getXMLWorkbook $tmpDir/xl/workbook.xml]
    set error [getXMLRels     $tmpDir/xl/_rels/workbook.xml.rels]

    set sheetList [lsort -dictionary [glob -nocomplain $tmpDir/xl/worksheets/sheet*.xml]]
    set nSheet    [llength $sheetList]
    set i 0

    foreach thisName $sheetList \
       {
        incr i

        set tailName   [file tail $thisName]
        set sheetLabel None

        if {[info exists file2ID($tailName)]} \
           {
            set sheetID [lindex $file2ID($tailName) 0]
            if {[info exists id2Name($sheetID)]} {set sheetLabel [lindex $id2Name($sheetID) 0]}
           }

        set error [getXMLData $thisName]

        if {$error} \
           {
            puts "Error Extracting worksheet $i for $xName"
            continue
           }

        if {![file exists $outDir]} \
           {
            catch "file mkdir $outDir"
           }

        if {$labFlag} \
           {
            if {$nSheet == 1} \
               {
                set outName $outDir/${rootName}_$sheetLabel.tab
               } \
            else \
               {
                set outName [format $outDir/${rootName}_%03d_$sheetLabel.tab $i]
               }
           } \
        else \
           {
            if {$nSheet == 1} \
               {
                set outName $outDir/$rootName.tab
               } \
            else \
               {
                set outName [format $outDir/${rootName}_%03d.tab $i]
               }
           }

        set error [saveXMLData $outName $sheetLabel]

        if {$error} \
           {
            catch "file delete -force $outName"
            continue
           }

        if {![file exists $outName]} \
           {
            puts "Error Finding output file $outName"
            continue
           }

        set xTab    [gdbRead table -in $outName]
        set varList [gdbGetInfo $xTab -parms varNames]
        set eCount  [gdbGetInfo $xTab -parms entryCount]

        set fmt "Output: %-16s Label: $sheetLabel Vars: %s Entries: %3s"

        puts [format $fmt $outName  $varList $eCount]

        gdbFree $xTab
       }

    if {[file exists $tmpDir]} {catch "file delete -force $tmpDir"}
    puts ""
   }

exit 0

