#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -notk -- "$@"

set auto_path "[split $env(TCLPATH) :] $auto_path"
set ARGV      [concat $argv0 $argv]
set ARGC      [llength $ARGV]

if {[flagLoc $ARGV -help]} \
{
 puts stderr "Convert XML Input to an NMRPipe Table:"
 puts stderr " -in    inName  \[stdin\]     XML Input."
 puts stderr " -out   outName \[stdout\]    Name for NMRPipe-Format Output Table."
 puts stderr " -var   varList \[None\]      Optional Variable Names for Columns."
 puts stderr " -fmt   fmtList \[None\]      Optional Formats for Columns."
 puts stderr " -nohdr                     No VARS/FORMAT Header Output."
 puts stderr " -csv                       Output in Comma Separated Values Format."
 puts stderr " -verb                      Verbose Mode ON."
 exit 0
}

set inName      stdin
set outName     stdout
set eName       entry

set verbFlag    0
set hdrFlag     1
set csvFlag     0

set userVarList None
set userFmtList None

getArgD $ARGV -in    inName
getArgD $ARGV -out   outName
getArgD $ARGV -eName eName 

if {[flagLoc $ARGV -verb]}    {set verbFlag 1}
if {[flagLoc $ARGV -noverb]}  {set verbFlag 0}

if {[flagLoc $ARGV -hdr]}     {set hdrFlag 1}
if {[flagLoc $ARGV -nohdr]}   {set hdrFlag 0}

if {[flagLoc $ARGV -csv]}     {set csvFlag 1}
if {[flagLoc $ARGV -nocsv]}   {set csvFlag 0}

getListArgD $ARGV -var userVarList
getListArgD $ARGV -fmt userFmtList

if {![strcasecmp $userVarList None]} {set userVarList ""}
if {![strcasecmp $userFmtList None]} {set userFmtList ""}

set userVarList [string trim $userVarList "{}"]
set userFmtList [string trim $userFmtList "{}"]

set nUserVar [llength $userVarList]
set nUserFmt [llength $userFmtList]

if {$nUserFmt < $nUserVar} \
   {
    set n [expr $nUserVar - $nUserFmt]
    for {set i 0} {$i < $n} {incr i} {lappend userFmtList "%12s"}
    set nUserFmt [llength $userFmtList]
   }

if {$nUserVar != $nUserFmt} \
   {
    puts "XML2Tab Error: VARS/FORMAT Mismatch."
    puts " VARS:    $userVarList"
    puts " FORMAT:  $userFmtList"
    exit 1
   }

#
#------------------------------------------------------------------------------

set inUnit  stdin
set outUnit stdout
set buff    ""

if {![strcasecmp $inName stdin]} \
   {
    set buff [read $inUnit]
   } \
else \
   {
    if {![file exists $inName]} \
       {
        puts stderr "XML2Tab Error Finding XML Input $inName"
        exit 1
       }

    set error [catch {set inUnit [open $inName r]}]

    if {$error} \
       {
        puts stderr "XML2Tab Error Opening XML Input $inName"
        exit 1
       }

    set buff  [read $inUnit]
    catch "close $inUnit"
   }

if {![strcasecmp $outName stdout] || ![strcasecmp $outName stderr]} \
   {
    set outUnit $outName
   } \
else \
   {
    set error [catch {set outUnit [open $outName w]}]

    if {$error} \
       {
        puts stderr "XML2Tab Error Opening Output $outName"
        exit 1
       }
   }

set n [parseTags $buff tagInfo "\n"]

for {set i 1} {$i <= $n} {incr i} \
   {
    set str [string trim $tagInfo($i) "</> "]
    set key [lindex $str 0]

    puts stderr [format " %4d. Key: %10s Entry: %s" $i $key $tagInfo($i)]
    if {![strcmp $key $eName]} continue
   }


if {[strcasecmp $outUnit stdout] && [strcasecmp $outUnit stderr]} \
   {
    catch "close $outUnit"
   }
 
exit 0
