#
# XPLOR Format conversion utilities.

proc getXPLORPDBInfo { pdbName } \
{
   global seqInfo segInfo atomInfo segName

   set pdbTab [gdbRead table -in $pdbName -pdb]
   set entry  [gdbFirst $pdbTab]

   while {$entry} \
      {
       set resID    [gdbGet $entry RESID]
       set atomName [gdbGet $entry ATOMNAME]
       set resName  [gdbGet $entry RESNAME]
       set seg      [gdbGet $entry SEGID]

       if {"$seg" == "" || ![strcasecmp $seg (Null)]} \
          {
           lappend atomInfo($resID,$segName) $atomName
           set seqInfo($resID,$segName)      $resName
           set segInfo($resID) ""
          } \
       else \
          {
           lappend atomInfo($resID,$seg) $atomName
           set seqInfo($resID,$seg)      $resName
           set segInfo($resID) $seg 
          }

       set entry [gdbNext $entry]
      }
}

proc parseXPLORAss2 { line } \
{
   global segName seqInfo

   set str [string trim $line "() "]
   set n   [llength $str]

   set seg     $segName
   set assInfo ""

   for {set i 0} {$i < $n} {incr i} \
      {
       set w0 [string trim [lindex $str $i] "() "]

       switch $w0 \
          {
           segid {
                  set  w1   [string trim [lindex $str [expr $i + 1]] "() "]
                  set  seg  $w1
                  incr i
                 }

           resid {
                  set  w1    [string trim [lindex $str [expr $i + 1]] "() "]
                  set  resID $w1
                  incr i
                 }

           name  {
                  set w1    [string trim [lindex $str [expr $i + 1]] "() "]
                  set atom  $w1
                  incr i

                  append assInfo "$resID~$atom~$seg|"

                  if {![info exists seqInfo($resID,$seg)]} \
                     {
                      puts stderr \
                       "DYNAMO Convert Warning: No ResID $resID Segment $seg."

                      set seqInfo($resID,$seg) NoneAtAll
                     }
                 }

           default {
                   }
          }
      }

   return [string trim $assInfo "|"]
}

proc parseXPLORAss { assID line itemCount assListPtr tailPtr } \
{
   upvar $assListPtr assList
   upvar $tailPtr    tail

   set assList  ""
   set tail     ""

   set n [string length $line]
   set i -1

   for {set j 0} {$j < $itemCount} {incr j} \
      {
       set assStr  ""
       set lpCount 0
       set rpCount 0

       for {set i [expr $i+1]} {$i < $n} {incr i} \
          {
           set c [string index $line $i]

           if {"$c" == "("} {incr lpCount}
           if {"$c" == ")"} {incr rpCount}

           if {$lpCount} {append assStr $c}

           if {$lpCount > 0 && $lpCount == $rpCount} break;
          }

       lappend assList [parseXPLORAss2 $assStr] 
      }

   set tail [string range $line [expr $i+1] end]
}

proc expandXPLORAtom { resID atom seg } \
{
   global atomInfo

   set loc [string first "#" $atom]

   if {$loc >= 0} \
      {
       set pattern  [string range $atom 0 [expr $loc - 1]]\[0-9\]
       set atomList ""

       if {[info exists atomInfo($resID,$seg)]} \
          {
           foreach str $atomInfo($resID,$seg) \
              {
               if {[string match $pattern $str]} {lappend atomList $str} 
              }
          }

       return $atomList
      }

   set loc [string first "*" $atom]

   if {$loc >= 0} \
      {
       set pattern  $atom
       set atomList ""

       if {[info exists atomInfo($resID,$seg)]} \
          {
           foreach str $atomInfo($resID,$seg) \
              {
               if {[string match $pattern $str]} {lappend atomList $str} 
              }
          }

       return $atomList
      }

   return $atom
}

proc stripXPLORComment { line } \
{
   set loc [string first "!" $line]

   if {$loc > 0} \
      {
       return [string range $line 0 [expr $loc - 1]]
      }

   return $line
}
