#!/bin/csh login file for XPLOR running on a CONVEX / UNIX
#
# ===============
# Axel T. Brunger
# ===============
#
# 'logical' assignments for directories
setenv XPLOR  /net/gmc5/clore/xplor_40
setenv COMS 	$XPLOR/sgi 
setenv SOURCE  	$XPLOR/source 
setenv TEST  	$XPLOR/test 
setenv TOPPAR  	$XPLOR/toppar 
setenv OBJ 	$XPLOR/object_library_sgi_R10000
setenv FFT 	$XPLOR/sgi/fft
setenv RIBBONS 	/infix2/nico/ribbons/bin

#
# 'alias' definitions 
alias ribbons 	$RIBBONS/ribbons
alias pdb-range-ss $RIBBONS/pdb-range-ss
alias pdb-pro-ss $RIBBONS/pdb-pro-ss
alias pdb-model $RIBBONS/pdb-model
alias grasp /ink1/people/clore/grasp/grasp
#
alias xplor 	$OBJ/xplor.exe  
alias xplor_anis $OBJ/xplor_anis.exe
alias avsxplor 	$XPLOR/avsxplor/sgi-R4000/exec/avs-xplor
alias install 	$COMS/install.com
alias put 	$COMS/put.com  
alias put_o2 	$COMS/put_o2.com  
alias link 	$COMS/link.com 
alias precompile $COMS/precompile.com  
alias precompile_o2 $COMS/precompile_o2.com  
alias compile 	$COMS/compile.com  
alias runtests 	$COMS/runtests.com
alias difftests $COMS/difftests.com

# Default Optimization Level: -O
set OPTFLAG = '-O'
# Default Mips flag: -mips1
set MIPSFLAG = '-mips1'
set LFASTM = '-lfastm'
# Default Blas library: -lblas
set BLASLIB = '-lblas'
# Default pre-processor: -sopt
set SOPTPFA = '-sopt,-r=2,-sy=V'
# Default MPFLAG: [blank]
set MPFLAG =
# Default Olimit: 800
set OLIMIT = 800
# Extra Flags
set EXTRAFLAG = -Nc100

# some Info:

# OS Version:
set OSVERS = `uname -r`
set OSMAJOR = ${OSVERS:r}
set OSMAJOR = ${OSMAJOR:r}

if ( ! $?XPLOR_OS ) then
   setenv XPLOR_OS $OSMAJOR
   endif

# Number of Processors
set NPROC = `mpadmin -n | tail -1`
@ NPROC += 1

if ( $NPROC > 1 ) then
   if ( ! ( -e /usr/lib/pfa ) ) then
      set NPROC = 1
   else
      setenv NUM_THREADS 2
      setenv MP_SET_NUMTHREADS 2
      endif
   endif

if ( ! $?XPLOR_NPROC ) then
    setenv XPLOR_NPROC $NPROC
    endif

# Do we have more than 1 processor, do we have pfa?
if ( $XPLOR_NPROC > 1 ) then
   set MPFLAG = "-mp"
   set SOPTPFA  = "-pfa -WK,-minconcurr=600,-r=2,-sy=V"
   if ( $XPLOR_OS > 4 ) then
      set BLASLIB = "-lblas_mp"
      endif
   endif

# Memory:
set MEM = ` hinv -t memory | awk ' { print $(NF-1) } ' - `
@ MEM += 0

if ( ! $?XPLOR_MEM ) then
   setenv XPLOR_MEM $MEM
   endif

# Do we have enough memory   
if ( $XPLOR_MEM > 64 ) then
   set OLIMIT = 2000
else if (  $MEM > 32 ) then
   set OLIMIT = 1000
   endif

# Processor:
set WHICHIP = `uname -m | sed s/IP//`
@ WHICHIP += 0

if ( ! $?XPLOR_IP ) then
   setenv XPLOR_IP $WHICHIP
   endif

# Is this an r4x00 based machine?
if ( $XPLOR_IP >= 17 ) then
   set MIPSFLAG = '-mips2'
   set LFASTM = 
   endif

if ( ! $?XPLOR_MIPS) then
   set XPLOR_MIPS = $MIPSFLAG
   endif


# Set the arguments for 1. Makefile
#                       2. Compile
#                       3. Link

setenv XPLOR_MAKE "$MPFLAG $XPLOR_MIPS"
setenv XPLOR_COMP "-Olimit $OLIMIT $OPTFLAG $XPLOR_MIPS $SOPTPFA $EXTRAFLAG"
setenv XPLOR_LINK "$XPLOR_MIPS $BLASLIB $LFASTM $MPFLAG"
