
/***/
/* Convert NMRPipe-format to grayscale PNG image.
/***/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <png.h>

#include "fdatap.h"
#include "dimloc.h"
#include "cmndargs.h"
#include "memory.h"
#include "vutil.h"
#include "prec.h"
#include "atof.h"
#include "readnmr.h"

#define FABS(X) (((X) < 0.0) ? -(X) : (X))
#define FPR     (void)fprintf

#define GRAYSCALE_DIRECT   1
#define GRAYSCALE_MID_ZERO 2
#define GRAYSCALE_FULL     3
 
int main( int argc, char **argv )
{
    char   inName[NAMELEN+1], outName[NAMELEN+1];
    int    dimCount, bitDepth, mode, qSize, sizeList[MAXDIM], qSizeList[MAXDIM], verbose, error;
    float  *rdata, fdata[FDATASIZE];

    NMR_INT totalSize;

    error     = 0;
    rdata     = (float *)NULL;
    totalSize = (NMR_INT)0;
    qSize     = 0;
    dimCount  = 0;
    bitDepth  = 16;
    mode      = GRAYSCALE_FULL;
    verbose   = 1;

    (void) strcpy( inName,  "test.ft2" );
    (void) strcpy( outName, "test.png" );

    (void) iSet( sizeList, MAXDIM, 0 );
  
    if ((error = getparms( argc, argv, inName, outName, &bitDepth, &mode, &verbose ))) return( 0 );

    if ((error = readNMR( inName, fdata, &rdata, sizeList, qSizeList, &totalSize, &qSize, &dimCount )))
       {
        FPR( stderr, "NMR2PNG Error %d Reading Input %s\n", error, inName );
        return( error );
       }

    if ((error = rdata2png( rdata, sizeList, bitDepth, mode, outName, verbose )))
       {
        FPR( stderr, "NMR2PNG Error %d Creating Output %s\n", error, outName );
        return( error );
       }

    return( 0 );
}

int rdata2png( float *rdata, int sizeList[MAXDIM], int bitDepth, int mode, char *outName, int verbose )
{
    png_structp png_ptr = NULL;
    png_infop   info_ptr = NULL;
    png_bytepp  row_pointers;
    png_bytep   row;

    int   ix, iy, maxPix, totalSize, bytesPerPix;
    float *rPtr, minVal, maxVal, maxAbs, range, val;
    FILE *outUnit;

    unsigned short color16;
    unsigned char  color8;

    outUnit     = (FILE *)NULL;
    png_ptr     = (png_structp)NULL;
    info_ptr    = (png_infop)NULL;

    bytesPerPix = bitDepth == 8 ? 1 : 2;
    maxPix      = bitDepth == 8 ? 255 : 65535;

    if (!(outUnit = fopen( outName, "w" ))) return( 1 );

    if (!(png_ptr = png_create_write_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL)))
       {
        (void) fclose( outUnit );
        return( 2 );
       }

    if (!(info_ptr = png_create_info_struct( png_ptr))) 
       {
        (void) fclose( outUnit );
        (void) png_destroy_write_struct(&png_ptr, NULL);
        return( 3 );
       }

    if (setjmp(png_jmpbuf(png_ptr))) 
       {
        (void) fclose( outUnit );
        (void) png_destroy_write_struct( &png_ptr, &info_ptr );
        return( 4 );
       }

    totalSize = sizeList[XLOC]*sizeList[YLOC];

    minVal = vMin( rdata, totalSize );
    maxVal = vMax( rdata, totalSize );

    range  = minVal == maxVal ? 1.0 : maxVal - minVal;
    maxAbs = FABS(minVal) > FABS(maxVal) ? FABS(minVal) : FABS(maxVal);
    maxAbs = maxAbs == 0.0 ? 1.0 : maxAbs;

    if (verbose)
       {
        FPR( stderr, "NMR2PNG. X-Size: %4d Y-Size: %4d Depth: %2d Grayscale Mode: ", sizeList[XLOC], sizeList[YLOC], bitDepth );

        if (mode == GRAYSCALE_FULL)
           FPR( stderr, "Full" );
        else if (mode == GRAYSCALE_DIRECT)
           FPR( stderr, "Direct" );
        else
           FPR( stderr, "Zero-Gray50" );
        
        FPR( stderr, " Output: %s\n", outName );
       }
 
    (void) png_set_IHDR( png_ptr, info_ptr,
                         (size_t)sizeList[XLOC], (size_t)sizeList[YLOC], bitDepth,
                         PNG_COLOR_TYPE_GRAY, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT );

    row_pointers = (png_bytepp) png_malloc( png_ptr, sizeList[YLOC]*sizeof(png_bytep) );

    for( iy = 0; iy < sizeList[YLOC]; iy++ ) row_pointers[iy] = (png_bytep)NULL;

    for( iy = 0; iy < sizeList[YLOC]; iy++ )
       {
        row_pointers[iy] = (png_bytep)png_malloc( png_ptr, sizeList[XLOC]*bytesPerPix );

        if (!row_pointers[iy])
           {
            (void) fclose( outUnit );
            (void) png_destroy_write_struct( &png_ptr, &info_ptr );
            return( 5 );
           }
       }

    rPtr = rdata;

    for( iy = sizeList[YLOC] - 1; iy >= 0; iy-- )
       {
        row = row_pointers[iy];

        for( ix = 0; ix < sizeList[XLOC]; ix++ )
           {
            val = *rPtr;

            if (mode == GRAYSCALE_MID_ZERO)
               val = 0.5*maxPix*(1.0 + val/maxAbs);
            else if (mode == GRAYSCALE_FULL)
               val = maxPix*(val - minVal)/range;

            if (val < 0.0)    val = 0.0;
            if (val > maxPix) val = maxPix;

            if (bitDepth == 16)
               {            
                color16 = (unsigned short)val;
                *row++  = (png_byte)(color16 >> 8);
                *row++  = (png_byte)(color16 & 0xFF);
               } 
            else
               {
                color8 = (unsigned short)val;
                *row++ = (png_byte)(color8);
               }

            rPtr++;
           }
       }

    (void) png_init_io( png_ptr, outUnit );
    (void) png_set_rows( png_ptr, info_ptr, row_pointers );
    (void) png_write_png( png_ptr, info_ptr, PNG_TRANSFORM_IDENTITY, NULL );
    (void) png_write_image( png_ptr, row_pointers );

    for( iy = 0; iy < sizeList[YLOC]; iy++ )
       {
        (void) png_free( png_ptr, row_pointers[iy] );
        row_pointers[iy] = (png_bytep)NULL;
       }

    (void) png_free( png_ptr, row_pointers );
    (void) png_destroy_write_struct( &png_ptr, &info_ptr );
    (void) fclose( outUnit );

    return( 0 );
}

int getparms( int argc, char **argv, char *inName, char *outName, int *bitDepth, int *mode, int *verbose )
{
    if (flagLoc( argc, argv, "-help" ))
       {
        FPR( stderr, "Convert NMRPipe-format to a PNG Grayscale Image:\n" );
        FPR( stderr, " -in    inName  [test.ft2]  Input Spectrum (1D-2D Single File Format).\n" );
        FPR( stderr, " -out   outName [test.png]  Output Image.\n" );
        FPR( stderr, " -depth depth   [16]        Grayscale Bit Depth (8 or 16 Only).\n" );
        FPR( stderr, " -verb                      Verbose Mode ON (Default).\n" );
        FPR( stderr, " -noverb                    Verbose Mode OFF.\n" );
        FPR( stderr, "Intensity to Grayscale Conversion Mode (Use Only One):\n" );
        FPR( stderr, " -full      Map Min Intensity to 0, Max Intensity to 2^depth - 1. (Default)\n" );
        FPR( stderr, " -mid       Map Zero Intensity to 50%% Gray.\n" );
        FPR( stderr, " -direct    Use Intensity Directly, Clipped to Range 0 to 2^depth - 1.\n" );
        return( 1 );
       }

    (void) strArgD( argc, argv, "-in",    inName );
    (void) strArgD( argc, argv, "-out",   outName );
    (void) strArgD( argc, argv, "-depth", bitDepth );

    if (*bitDepth != 8 && *bitDepth != 16) *bitDepth = 8;

    if (flagLoc( argc, argv, "-full" ))   *mode = GRAYSCALE_FULL;
    if (flagLoc( argc, argv, "-mid" ))    *mode = GRAYSCALE_MID_ZERO;
    if (flagLoc( argc, argv, "-direct" )) *mode = GRAYSCALE_DIRECT;

    if (flagLoc( argc, argv, "-verb" ))   *verbose = 1;
    if (flagLoc( argc, argv, "-noverb" )) *verbose = 0;

    return( 0 );
}
