/******************************************************************************/
/*                                                                            */
/*                   ---- NIH NMR Software System ----                        */
/*                        Copyright 1992 and 1993                             */
/*                             Frank Delaglio                                 */
/*                   NIH Laboratory of Chemical Physics                       */
/*                                                                            */
/*                 Augmented by Dennis Benjamin for 4D Data.                  */
/*                                                                            */
/*               This software is not for distribution without                */
/*                  the written permission of the author.                     */
/*                                                                            */
/******************************************************************************/

/***/
/* var2pipe: conversions from Varian format to nmrPipe stream.
/***/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "version.h"
#include "cmndargs.h"
#include "testsize.h"
#include "dataio.h"
#include "inquire.h"
#include "minmax.h"
#include "memory.h"
#include "fdatap.h"
#include "prec.h"
#include "vutil.h"

#include "varian.h"

#define SWAP2( B ) { c=B[0];B[0]=B[1];B[1]=c; } c=(char)0
#define SWAP4( B ) { c=B[0];B[0]=B[3];B[3]=c; c=B[1];B[1]=B[2];B[2]=c; } c=(char)0

#define FPR        (void)fprintf
#define SPR        (void)sprintf
#define NMRTXT     "NMRTXT"

static int two2Four(), swapVHdr();

#ifdef CRAY
float  getVFloat();
int    getVInt();
short  getVShort();
#endif

int main( argc, argv )

   int  argc;
   char **argv;
{
    int   xSize, ySize, zSize, aSize, xMid, yMid, zMid, aMid,
          planeCount, xFreqSize, yFreqSize, zFreqSize, aFreqSize, varSize,
          matSize, plane, zPlane, aPlane, dimCount, dim, quadType, checkFlag,
          xyzFlag, ovFlag, swapFlag, i2rFlag, shortFlag, thisSize, thisQuad, 
          xQuadState, yQuadState, zQuadState, aQuadState, aqOrder, 
          yQuadFlag, zQuadFlag, aQuadFlag, firstPlane,
          lastPlane, iz, ia, xTD, yTD, zTD, aTD, izQuad, iaQuad,
          dirFlag, vHdrFlag, macCount, cmndCount, fmtCount, wordSize,
          infoFlag, tclFlag, signFlag, neFlag, shufFlag, verbose, 
          status, error;

    int     vWordSize, vFidSize, vCount1, vCount2, vQuadFlag;

    NMR_INT bytesIn, ptsOut, length;

    char  *varMat, inName[NAMELEN+1], outName[NAMELEN+1], outFmt[NAMELEN+1], outFmtOrig[NAMELEN+1],
          cmndName[NAMELEN+1], serverListName[NAMELEN+1], macNameY[NAMELEN+1],
          macNameZ[NAMELEN+1], macNameA[NAMELEN+1], infoName[NAMELEN+1], 
          ctemp[NAMELEN+1], *vPtr;

    float *dataMat, xSW, ySW, zSW, aSW, xObs, yObs, zObs, aObs, 
          xCar, yCar, zCar, aCar, xOrig, yOrig, zOrig, aOrig,
          maxVal, minVal, value, fdata[FDATASIZE];

    static char hdrMat[VHDRSIZE];

    struct VHdrBuff vHdrInfo;

    UNIT   inUnit, outUnit;
    FILE   *infoUnit;

/***/
/* Initialize:
/***/

    error     = 0;
    matSize   = 0;
    varSize   = 0;

    minVal    =  LARGENUM;
    maxVal    = -LARGENUM;
    
    inUnit    = UNIT_NULL;
    outUnit   = UNIT_NULL;
    infoUnit  = stdout;
 
    dataMat   = (float *)NULL;
    varMat    = (char *)NULL;

    cmndName[0] = '\0';
    macNameY[0] = '\0';
    macNameZ[0] = '\0';
    macNameA[0] = '\0';
    infoName[0] = '\0';

    inName[0]     = '\0';
    outFmt[0]     = '\0';
    outFmtOrig[0] = '\0';

    macCount  = 0;
    cmndCount = 0;

    (void) initArgs( argc, argv, CHECK_ENV );
    (void) setHdrNull( fdata );

/***/
/* Extract command parameters:
/***/

    error = getparams( argc, argv,
                       inName, outFmt, infoName, serverListName,
                       fdata, &firstPlane, &lastPlane, &xyzFlag,
                       &aqOrder, &swapFlag, &i2rFlag, &shortFlag,
                       &checkFlag, &ovFlag, &dirFlag, &vHdrFlag,
                       &infoFlag, &tclFlag, &signFlag, &neFlag, 
                       &shufFlag, &verbose );

    if (error)
       {
        FPR( stderr, "Var2Pipe Error extracting arguments.\n" );
        goto shutdown;
       }

/***/
/* Special Varian implementation:
/*  4D data must have explicit input file, because conversion requires random access to the input data.
/*  4D output is handled via pipeline to pipe2xyz. 
/***/

    (void) strcpy( outFmtOrig, outFmt );
    (void) fixfdata( fdata );

    dimCount = getParm( fdata, FDDIMCOUNT, 0 );

    if (dimCount > 3 && !*inName)
       {
        FPR( stderr, "Var2Pipe Error: 4D Data requires -in argument.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Handle special indirect acquisition modes:
/***/

    yQuadFlag = getParm( fdata, NDQUADFLAG, CUR_YDIM );
    zQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ZDIM );
    aQuadFlag = getParm( fdata, NDQUADFLAG, CUR_ADIM );

    switch( yQuadFlag )
       {
        case FD_SE:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );

           if (shufFlag)
              {
               SPR( macNameY, "%s/%s -all", getenv( NMRTXT ), "var_seY.M" );
               macCount++;
              }

           break;
        case FD_GRAD:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_YDIM );
 
           if (shufFlag)
              {
               SPR( macNameY, "%s/%s", getenv( NMRTXT ), "var_ranceY.M" );
               macCount++;
              }

           break;
        default:
           break;
       }

    switch( zQuadFlag )
       {
        case FD_SE:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );

           if (shufFlag)
              {
               SPR( macNameZ, "%s/%s -all", getenv( NMRTXT ), "var_seZ.M" );
               macCount++;
              }

           break;
        case FD_GRAD:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ZDIM );

           if (shufFlag)
              {
               SPR( macNameZ, "%s/%s", getenv( NMRTXT ), "var_ranceZ.M" );
               macCount++;
              }

           break;
        default:
           break;
       }

    switch( aQuadFlag )
       {
        case FD_SE:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );

           if (shufFlag)
              {
               SPR( macNameA, "%s/%s -all", getenv( NMRTXT ), "var_seA.M" );
               macCount++;
              }

           break;
        case FD_GRAD:
           (void) setParm( fdata, NDQUADFLAG, (float)FD_COMPLEX, CUR_ADIM );

           if (shufFlag)
              {
               SPR( macNameA, "%s/%s", getenv( NMRTXT ), "var_ranceA.M" );
               macCount++;
              }

           break;
        default:
           break;
       }

/*
    if (macCount > 1)
       {
        FPR( stderr, 
             "Var2Pipe Warning: %d special acquisition modes;\n",
             macCount );

        FPR( stderr, " Only one special mode is allowed.\n" );
       }
*/

    fmtCount = getFmtCount( outFmt );
 
    if (macCount)
       {
        *cmndName = '\0';

        if (*macNameY)
           {
            (void) sprintf( ctemp, 
                            "nmrPipe -fn MAC -macro %s -noRd -noWr", 
                            macNameY );

            (void) strcat( cmndName, ctemp );
            cmndCount++;
           }

        if (*macNameZ)
           {
            (void) sprintf( ctemp, 
                            "%s nmrPipe -fn MAC -macro %s -noRd -noWr", 
                            cmndCount ? " | " : " ", macNameZ );

            (void) strcat( cmndName, ctemp );
            cmndCount++;
           }

        if (*macNameA)
           {
            (void) sprintf( ctemp, 
                            "%s nmrPipe -fn MAC -macro %s -noRd -noWr",
                            cmndCount ? " | " : " ", macNameA );

            (void) strcat( cmndName, ctemp );
            cmndCount++;
           }


        if (*outFmt)
           {
            if (fmtCount == 0)
               {
                (void) sprintf( ctemp, " -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }
            else
               {
                (void) sprintf( ctemp, " | pipe2xyz -out %s", outFmt );
                (void) strcat( cmndName, ctemp );
               }

            if (ovFlag) (void) strcat( cmndName, " -ov" );

            *outFmt  = '\0';
            fmtCount = 0;
            xyzFlag  = 0;
           }
       }
    else
       {
        if (dimCount == 4 && fmtCount >= 1)
           {
            (void) sprintf( ctemp, "%s pipe2xyz -out %s", cmndCount ? " | " : " ", outFmt );
            (void) strcat( cmndName, ctemp );
            cmndCount++;

            if (ovFlag) (void) strcat( cmndName, " -ov" );

            *outFmt  = '\0';
            fmtCount = 0;
            xyzFlag  = 0;
           }
       }

/***/
/* Set redundant parameters if they have not been explicitly set:
/***/

    if (flagLoc( argc, argv, "-xMODE" ) && !flagLoc( argc, argv, "-aqMODE" ))
       {
        value = getParm( fdata, NDQUADFLAG, CUR_XDIM );
        (void) setParm( fdata, FDQUADFLAG, value, 0 );
       }

    if (flagLoc( argc, argv, "-xT" ) && !flagLoc( argc, argv, "-aqN" ))
       {
        value = getParm( fdata, NDAPOD, CUR_XDIM );
        (void) setParm( fdata, FDREALSIZE, value, 0 );
       }

    if (flagLoc( argc, argv, "-yMODE" ) && !flagLoc( argc, argv, "-aq2D" ))
       {
        yQuadState = dimCount < 2 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_YDIM );

        if (yQuadState == 1)
           (void) setParm( fdata, FD2DPHASE, (float)FD_TPPI,   NULL_DIM );
        else
           (void) setParm( fdata, FD2DPHASE, (float)FD_STATES, NULL_DIM );
       }

/***/
/* Extract sizes and quad states; adjust sizes and states for output.
/*   XSIZE, YSIZE, etc begin as count of real+imag points for each axis.
/*   XSIZE is reduced to number of complex points.
/***/

    wordSize = shortFlag ? 2 : 4;

    quadType = getParm( fdata, FDQUADFLAG, 0 );

    xSize      = dimCount < 1 ? 1 : getParm( fdata, NDSIZE, CUR_XDIM );
    ySize      = dimCount < 2 ? 1 : getParm( fdata, NDSIZE, CUR_YDIM );
    zSize      = dimCount < 3 ? 1 : getParm( fdata, NDSIZE, CUR_ZDIM ); 
    aSize      = dimCount < 4 ? 1 : getParm( fdata, NDSIZE, CUR_ADIM ); 

    yQuadState = dimCount < 2 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_YDIM );
    zQuadState = dimCount < 3 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_ZDIM );
    aQuadState = dimCount < 4 ? 1 : getQuad( fdata, NDQUADFLAG, CUR_ADIM );

    switch( quadType )
       {
        case 0:
           xQuadState  = 2;
           xSize      /= xQuadState;
           xMid        = 1 + xSize/2;
           xFreqSize   = xSize;
           (void) setParm( fdata, NDSIZE, (float)xSize, CUR_XDIM );
           break;
        case 1:
           xQuadState = 1;
           xMid       = 1 + xSize/4;
           xFreqSize  = xSize/2; 
           break;
        default:
           FPR( stderr, "Var2Pipe Error: bad quad type %d.\n", quadType );

           error = 1;
           goto shutdown;
           break;
       }

    planeCount = zSize*aSize;

    if (dimCount <= 2)
       {
        (void) setParm( fdata, FDFILECOUNT, (float)1, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)0, 0 );
        (void) setParm( fdata, FDCUBEFLAG,  (float)0, 0 );
       }
    else if (fmtCount == 2)
       {
        (void) setParm( fdata, FDFILECOUNT, (float)planeCount, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)0,          0 );
        (void) setParm( fdata, FDCUBEFLAG,  (float)0,          0 );
       }
    else if (fmtCount == 1)
       {
        if (dimCount == 4)
           {
            (void) setParm( fdata, FDFILECOUNT, (float)aSize, 0 );
            (void) setParm( fdata, FDPIPEFLAG,  (float)0,     0 );
            (void) setParm( fdata, FDCUBEFLAG,  (float)1,     0 );
           }
        else
           {
            (void) setParm( fdata, FDFILECOUNT, (float)planeCount, 0 );
            (void) setParm( fdata, FDPIPEFLAG,  (float)0,          0 );
            (void) setParm( fdata, FDCUBEFLAG,  (float)0,          0 );
           }
       }
    else
       {
        (void) setParm( fdata, FDFILECOUNT, (float)1, 0 );
        (void) setParm( fdata, FDPIPEFLAG,  (float)1, 0 );
        (void) setParm( fdata, FDCUBEFLAG,  (float)0, 0 );
       }

/***/
/* Center (Zero Freq) Locations are set according to complex data sizes.
/* Axis origins in hertz are set according to carrier at point freqSize/2 + 1.
/* Don't replace explicit definitions of Final Freq-Domain Size or Center:
/***/

    yFreqSize = ySize/2 ? ySize/2 : 1;
    zFreqSize = zSize/2 ? zSize/2 : 1;
    aFreqSize = aSize/2 ? aSize/2 : 1;

    if (flagLoc( argc, argv, "-xFD" ))
       {
        xFreqSize = intArg( argc, argv, "-xFD" );
        xMid      = xFreqSize/2 + 1;
       }

    if (flagLoc( argc, argv, "-yFD" )) yFreqSize = intArg( argc, argv, "-yFD" );
    if (flagLoc( argc, argv, "-zFD" )) zFreqSize = intArg( argc, argv, "-zFD" );
    if (flagLoc( argc, argv, "-aFD" )) aFreqSize = intArg( argc, argv, "-aFD" );

    yMid = 1 + yFreqSize/2;
    zMid = 1 + zFreqSize/2;
    aMid = 1 + aFreqSize/2; 

    if (flagLoc( argc, argv, "-xMID" )) xMid = intArg( argc, argv, "-xMID" );
    if (flagLoc( argc, argv, "-yMID" )) yMid = intArg( argc, argv, "-yMID" );
    if (flagLoc( argc, argv, "-zMID" )) zMid = intArg( argc, argv, "-zMID" );
    if (flagLoc( argc, argv, "-aMID" )) aMid = intArg( argc, argv, "-aMID" );

    (void) setParm( fdata, NDCENTER, (float)xMid, CUR_XDIM );
    (void) setParm( fdata, NDCENTER, (float)yMid, CUR_YDIM );
    (void) setParm( fdata, NDCENTER, (float)zMid, CUR_ZDIM );
    (void) setParm( fdata, NDCENTER, (float)aMid, CUR_ADIM );

    xObs  = getParm( fdata, NDOBS, CUR_XDIM );
    yObs  = getParm( fdata, NDOBS, CUR_YDIM );
    zObs  = getParm( fdata, NDOBS, CUR_ZDIM );
    aObs  = getParm( fdata, NDOBS, CUR_ADIM );

    xSW   = getParm( fdata, NDSW,  CUR_XDIM );
    ySW   = getParm( fdata, NDSW,  CUR_YDIM );
    zSW   = getParm( fdata, NDSW,  CUR_ZDIM );
    aSW   = getParm( fdata, NDSW,  CUR_ADIM );

    xCar  = getParm( fdata, NDCAR, CUR_XDIM );
    yCar  = getParm( fdata, NDCAR, CUR_YDIM );
    zCar  = getParm( fdata, NDCAR, CUR_ZDIM );
    aCar  = getParm( fdata, NDCAR, CUR_ADIM );

/***/
/* Don't replace explicit definitions of Axis Origin:
/***/

    if (flagLoc( argc, argv, "-xORIG" ))
       xOrig = fltArg( argc, argv, "-xORIG" );
    else
       xOrig = xObs*xCar - xSW*(xFreqSize - xMid)/xFreqSize;

    if (flagLoc( argc, argv, "-yORIG" ))
       yOrig = fltArg( argc, argv, "-yORIG" );
    else
       yOrig = yObs*yCar - ySW*(yFreqSize - yMid)/yFreqSize;

    if (flagLoc( argc, argv, "-zORIG" ))
       zOrig = fltArg( argc, argv, "-zORIG" );
    else
       zOrig = zObs*zCar - zSW*(zFreqSize - zMid)/zFreqSize;

    if (flagLoc( argc, argv, "-aORIG" ))
       aOrig = fltArg( argc, argv, "-aORIG" );
    else
       aOrig = aObs*aCar - aSW*(aFreqSize - aMid)/aFreqSize;

    (void) setParm( fdata, NDORIG, xOrig, CUR_XDIM );
    (void) setParm( fdata, NDORIG, yOrig, CUR_YDIM );
    (void) setParm( fdata, NDORIG, zOrig, CUR_ZDIM );
    (void) setParm( fdata, NDORIG, aOrig, CUR_ADIM );

/***/
/* Don't replace explicit definitions of Original Time-Domain Size:
/***/

    if (flagLoc( argc, argv, "-xTD" ))
       xTD = intArg( argc, argv, "-xTD" );
    else
       xTD = getParm( fdata, NDAPOD, CUR_XDIM );

    if (flagLoc( argc, argv, "-yTD" ))
       yTD = intArg( argc, argv, "-yTD" );
    else
       yTD = getParm( fdata, NDAPOD, CUR_YDIM );

    if (flagLoc( argc, argv, "-zTD" ))
       zTD = intArg( argc, argv, "-zTD" );
    else
       zTD = getParm( fdata, NDAPOD, CUR_ZDIM );

    if (flagLoc( argc, argv, "-aTD" ))
       aTD = intArg( argc, argv, "-aTD" );
    else
       aTD = getParm( fdata, NDAPOD, CUR_ADIM );

    (void) setParm( fdata, NDTDSIZE, (float)xTD, CUR_XDIM );
    (void) setParm( fdata, NDTDSIZE, (float)yTD, CUR_YDIM );
    (void) setParm( fdata, NDTDSIZE, (float)zTD, CUR_ZDIM );
    (void) setParm( fdata, NDTDSIZE, (float)aTD, CUR_ADIM );

/***/
/* Adjust parameters for aquisition order.
/* Set the date and time for the conversion.
/***/

    switch( aqOrder )
       {
        case 1:
           yQuadState  = 1;
           break;
        default:
           break;
       }

    (void) setHdrTime( fdata );

/***/
/* Extract explicit user adjustments to header data.
/* Check use of command-line options.
/***/

    if ((error = parseHdr2( argc, argv, fdata )))
       {
        FPR( stderr, "Var2Pipe Error setting User Header.\n" );
        goto shutdown;
       }

    (void) checkArgs( argc, argv );

/***/
/* Open a dummy output file for testing purposes.
/* Test for sufficient space as needed.
/* Delete the dummy file.
/* Open the Varian input file.
/***/

    if (*outFmt && checkFlag)
       {
        error = fidOpen( outFmt, outName, cmndName,
                         &outUnit, 1, fdata, ovFlag,
                         xyzFlag, dirFlag, verbose );

        if (error)
           {
            FPR( stderr, "Var2Pipe Error Testing %s\n", outName );
            goto shutdown;
           }

        error = fidClose( outFmt, &outUnit, minVal, maxVal, fdata, xyzFlag );

        if (error)
           {
            FPR( stderr, "Var2Pipe Error Updating %s\n", outName );
            goto shutdown;
           }

        switch( testNMRout( outName, fdata, xyzFlag, 1 ))
            {
             case FILE_SIZE_FAIL:
                FPR( stderr, "\n" );
                FPR( stderr, "Var2Pipe Warning:" );
                FPR( stderr, " can't check size for file" );
                FPR( stderr, " %s\n", outName );
                break;
             case  FILE_SIZE_TOO_LARGE:
                FPR( stderr, "\n" );
                FPR( stderr, "Var2Pipe Error:" );
                FPR( stderr, " insufficient space for file" );
                FPR( stderr, " %s\n", outName );
                error = 1;
                goto shutdown;
                break;
             default:
                break;
            }

        (void) delFile( outName );
       }

    if ((error = varOpen( inName, &inUnit )))
       {
        FPR( stderr, "Var2Pipe Error Opening Input.\n" );
        goto shutdown;
       }

/***/
/* Read the Varian header.
/* Adjust settings according to header if needed.
/***/

    length = (NMR_INT)VHDRSIZE;
    vPtr   = hdrMat;

#ifndef CRAY
    if (vHdrFlag)
       {
        vPtr   = (char *) &vHdrInfo;
        length = (NMR_INT)sizeof(struct VHdrBuff);

        if (length != (NMR_INT)sizeof(int)*VHDR_BEGIN)
           {
            FPR( stderr, "Var2Pipe Notice: bad internal header size.\n" );
           }
       }
#endif

    if (vHdrFlag)
       {
        if (*inName)
           {
            if ((error = dataReadC( inUnit, vPtr, length )))
               {
                FPR( stderr, "Var2Pipe Error Reading Input File.\n" );
                goto shutdown;
               }
           }
        else
           {
            if ((error = dataReadCB( inUnit, vPtr, length, -1, 0 )))
               {
                FPR( stderr, "Var2Pipe Error Reading Input Stream.\n"  );
                goto shutdown;
               }
           }
       }

   if (vHdrFlag)
      {
       if (swapFlag) (void) swapVHdr( vPtr );

#ifdef CRAY
       (void) expandVHdr( &vHdrInfo, hdrMat, swapFlag );
#endif

       vQuadFlag   = (vHdrInfo.v6 & 64) ? 1 : 0;

       vFidSize    = vHdrInfo.v3;
       vCount1     = vHdrInfo.v1;
       vCount2     = vHdrInfo.v2;
       vWordSize   = vHdrInfo.v4;

       shortFlag   = (vHdrInfo.v6 &  4) ? 0 : 1;
       i2rFlag     = (vHdrInfo.v6 &  8) ? 0 : 1;
       wordSize    = vHdrInfo.v4;    
      
/***/
/* Info output must be two-word keyword/value pairs to stdout:
/***/
 
       if (verbose)
          {
           if (infoFlag)
              {
               if (*infoName)
                  {
                   if (!(infoUnit = fopen( infoName, "w" )))
                      {
                       FPR( stderr,
                            "Var2Pipe Error opening info output %s\n",
                            infoName );

                       infoUnit = stdout;
                      }
                  }

               if (tclFlag)
                  {
                   FPR( infoUnit, 
                        "set LengthType %s\n", shortFlag ? "short" : "long" );

                   FPR( infoUnit, 
                        "set WordType   %s\n", i2rFlag   ? "int" : "float" );

                   FPR( infoUnit, 
                        "set QuadType   %s\n", vQuadFlag ? "complex" : "real" );

                   FPR( infoUnit, "set WordSize   %d\n", vWordSize );
                   FPR( infoUnit, "set FIDSize    %d\n", vFidSize );
                   FPR( infoUnit, "set FIDCount   %d\n", vCount1 );
                   FPR( infoUnit, "set ExpCount   %d\n", vCount2 );
                  }
               else
                  {
                   FPR( infoUnit, 
                        "LengthType: %s\n", shortFlag ? "short" : "long" );

                   FPR( infoUnit, 
                        "WordType:   %s\n", i2rFlag   ? "int" : "float" );

                   FPR( infoUnit, 
                        "QuadType:   %s\n", vQuadFlag ? "complex" : "real" );

                   FPR( infoUnit, "WordSize:   %d\n", vWordSize );
                   FPR( infoUnit, "FIDSize:    %d\n", vFidSize );
                   FPR( infoUnit, "FIDCount:   %d\n", vCount1 );
                   FPR( infoUnit, "ExpCount:   %d\n", vCount2 );
                  }

               if (infoUnit != stdout)
                  {
                   (void) fclose( infoUnit );

                   infoUnit = (FILE *)NULL;
                  } 
              }
           else
              {
               FPR( stderr, "\n" );

    
               FPR( stderr,
                    "Varian Format: %s %s %s, %d Bytes per Point.\n",
                    shortFlag ? "Short"         : "Long",
                    i2rFlag   ? "Integer"       : "Floating-Point",
                    vQuadFlag ? "Complex Data"  : "Real Data",
                    vWordSize );

               FPR( stderr,
                    "Varian Sizes:  %d Total Points per FID, %d FIDs, %d %s.\n",
                    vFidSize, vCount1, vCount2,
                    vCount2 == 1 ? "Experiment" : "Experiments." );
              }

           (void) fflush( stderr );
          }
      }

    if (infoFlag) goto shutdown;

/*
    if (neFlag) 
       {
        thisSize  = getParm( fdata, NDSIZE, CUR_YDIM );

        xSize    *= vCount2;
        thisSize *= vCount2;

        (void) setParm( fdata, NDSIZE,  (float)thisSize, CUR_YDIM );
        (void) setParm( fdata, FDUSER1, (float)vCount2,  NULL );
       }
*/

/***/
/* Compute sizes and allocate memory.
/*  Minimum memory is header size or greater to be on the safe side.
/***/

    bytesIn = (NMR_INT)(xSize*xQuadState*wordSize + (NMR_INT)VHDR_MID*VINTSIZE)*ySize*zQuadState*aQuadState;
    ptsOut  = (NMR_INT)xSize*ySize*xQuadState;
    varSize = 4*VHDR_BEGIN + (xSize*xQuadState + VHDR_MID)*ySize*zQuadState*aQuadState;
    matSize = FDATASIZE  + xSize*ySize*xQuadState;

    if (!(varMat = charAlloc( "var", VINTSIZE*varSize )))
       {
        FPR( stderr, "Var2Pipe Error allocating input memory.\n" );
        error = 1;
        goto shutdown;
       }
 
    if (!(dataMat = fltAlloc( "var", matSize )))
       {
        FPR( stderr, "Var2Pipe Error allocating output memory.\n" );
        error = 1;
        goto shutdown;
       }

/***/
/* Display initial status:
/***/

    if (verbose)
       {
        FPR( stderr, "\n" );

        FPR( stderr,
             "Varian %s %s Format --> NMRPipe Conversion.\n",
             wordSize == 2 ? "Two-Byte" : "Four-Byte",
             i2rFlag ? "Integer" : "Float" );

        if (*inName)
           FPR( stderr, " Input File:      %s\n", inName );            
        else
           FPR( stderr, " Input Data:      Standard Input.\n" );

        if (*outFmtOrig)
           {
            if (getFmtCount( outFmtOrig ))
               FPR( stderr, " Output %s %s\n", fmtCount ? "Template:" : "Pipeline:", outFmtOrig );
            else
               FPR( stderr, " Output %s %s\n", *outFmt ?  "File:    " : "Pipeline:", outFmtOrig );
           }
        else
           {
            FPR( stderr, " Output Data:     Standard Output.\n" );
           }

        if (macCount)
           {
            FPR( stderr, " Output Macros:  " );

            if (*macNameY) FPR( stderr, " %s", macNameY );
            if (*macNameZ) FPR( stderr, " %s", macNameZ );
            if (*macNameA) FPR( stderr, " %s", macNameA );

            FPR( stderr, "\n" );
           }

        FPR( stderr, " %dD Sizes:        ", dimCount );

        for( dim = 1; dim <= dimCount; dim++ )
           {
            thisSize = getParm( fdata, NDSIZE,     dim );
            thisQuad = getParm( fdata, NDQUADFLAG, dim );

            if (!thisQuad && dim == 1) thisSize *= 2;

            if (thisQuad)
               FPR( stderr, "(%d Real)", thisSize );
            else
               FPR( stderr, "(%d Real+Imag)", thisSize );
           }

        FPR( stderr, "\n\n" );
        (void) fflush( stderr );
       }

/***/
/* For each 2D plane:
/*   Read the plane, which may contain interleaved Z-complex vectors.
/*
/*   For each of the Z-real/imag part of plane, if any:
/*     Open the output plane as needed.
/*     Convert the slices.
/*     Write the slices as a single output plane.
/*     Close the output plane as needed.
/***/

    plane  = 1;
    aPlane = 0;

    for( ia = 1; ia <= aSize/aQuadState; ia++ )
       {
        for( iaQuad = 1; iaQuad <= aQuadState; iaQuad++ )
           {
            aPlane++;

            zPlane = 0;
 
            for( iz = 1; iz <= zSize/zQuadState; iz++ )
               {
                error = varRead( inName, inUnit, iz, ia, iaQuad,
                                 xSize, ySize, zSize, aSize, dimCount,
                                 xQuadState, yQuadState, zQuadState, aQuadState,
                                 aqOrder, varMat, wordSize, bytesIn );

                if (error)
                   {
                    FPR( stderr, "Var2Pipe Error Reading Plane.\n" );
                    goto shutdown; 
                   }
 
                for( izQuad = 1; izQuad <= zQuadState; izQuad++ )
                   { 
                    zPlane++;

                    error = fidOpen( outFmt, outName, cmndName,
                                     &outUnit, plane, fdata, ovFlag,
                                     xyzFlag, dirFlag, verbose );

                    if (error)
                       {
                        FPR( stderr, "Var2Pipe Error Opening Output.\n" );
                        goto shutdown;
                       }

                    (void) var2fid( varMat, dataMat,
                                    xSize, ySize, zSize, aSize, izQuad, iaQuad,
                                    xQuadState, yQuadState,
                                    zQuadState, aQuadState,
                                    aqOrder, swapFlag, i2rFlag, signFlag,
                                    &minVal, &maxVal );
    
                    if (dataWrite( outUnit, dataMat, sizeof(float)*ptsOut ))
                       {
                        FPR( stderr, "Var2Pipe Writing Output.\n" );
                        goto shutdown; 
                       }

                    if ((error = fidClose( outFmt, &outUnit, minVal, maxVal, fdata, xyzFlag )))
                       {
                        FPR( stderr, "Var2Pipe Error Updating Output.\n" );
                        goto shutdown;
                       }

                    if (verbose)
                       {
                        FPR( stderr, "Plane %d of %d\015", plane, planeCount );
                        (void) fflush( stderr );
                       }

                    plane++;
                   }
               }
           }
       }

    if (verbose)
       {
        FPR( stderr, "\n" );
        (void) fflush( stderr );
       }

    if (*cmndName)
       {
        (void) wait( &status ); 
       }

/***/
/* Exit point:
/***/

shutdown:

    if (inUnit  != FB_STDIN)  (void) dataClose( inUnit );
    if (outUnit != FB_STDOUT) (void) dataClose( outUnit );

    (void) deAlloc( "var", varMat,  VINTSIZE*varSize );
    (void) deAlloc( "var", dataMat, sizeof(float)*matSize );

    return( error );
}

/***/
/* varOpen: open Varian format file for input.
/***/

int varOpen( inName, inUnitPtr )

   char *inName;
   UNIT *inUnitPtr;
{
    if (*inName)
       {
        if (dataOpen( inName, inUnitPtr, FB_READ )) return( 1 );
       }
    else
       {
        *inUnitPtr = FB_STDIN;
       }

    return( 0 );
}

/***/
/* varRead: read a plane from Varian format input.
/***/

int varRead( inName, inUnit, iz, ia, iaQuad,
             xSize, ySize, zSize, aSize, dimCount,
             xQuadState, yQuadState, zQuadState, aQuadState,
             aqOrder, varMat, wordSize, bytesIn )

    int iz, ia, iaQuad, xSize, ySize, zSize, aSize, dimCount,
        xQuadState, yQuadState, zQuadState, aQuadState, aqOrder, wordSize;

    NMR_INT bytesIn;

    char *varMat, *inName;
    UNIT inUnit;
{
    NMR_INT bytes, block, length, jump, loc;
    int     iy, j, error;
    char    *vPtr;

    vPtr = varMat;

    if ((dimCount == 4) && (wordSize == 2))
       {
        if (aqOrder == 0)  /* dim=4  */      /* Here, one length = 4 fids  */
           {
            length  = (NMR_INT)xSize*xQuadState*yQuadState*zQuadState*wordSize;
            length += (NMR_INT)VHDR_MID*yQuadState*zQuadState*VINTSIZE;
            jump    = length*aQuadState;

            /* Step up the appropriate number of "planes".                  */
            /* Next, step up the to appropriate "cube".                     */
            /* And finally, step up to the "a" imaginary vector, if needed. */

            loc  = (NMR_INT)VHDR_BEGIN*VINTSIZE + (NMR_INT)(iz - 1)*aQuadState*(ySize/yQuadState)*length;
            loc += (NMR_INT)(ia - 1)*aQuadState*(ySize/yQuadState)*(zSize/zQuadState)*length;
            loc += (NMR_INT)(iaQuad - 1)*length;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               { 
                if ((error = dataPos( inUnit, loc )))   /* 4d only */
                   {
                    FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                    return( error );
                   }

                if (*inName)
                   { 
                    if ((error = dataReadC( inUnit, vPtr, length )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
                else
                   {
                    if ((error = dataReadCB( inUnit, vPtr, length, -1, 0 )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
 
                vPtr += length;
                loc  += jump;
               } 
           }
        else if (aqOrder == 1)   /* ndim == 4 */  /* Here, one length = 1 fid */
           {
            length  = (NMR_INT)xSize*xQuadState*wordSize;
            length += (NMR_INT)VHDR_MID*VINTSIZE;
            block   = length*yQuadState*zQuadState*aQuadState;

            /* Step 1 fid at a time to extract the components.            */
            /* Step up the appropriate number of "planes".                */
            /* Next, step up the to appropriate "cube".                   */
            /* Finally, step up to the "a" imaginary vector, if needed.   */

            loc  = (NMR_INT)VHDR_BEGIN*VINTSIZE + (NMR_INT)(iz - 1)*(ySize/yQuadState)*block;
            loc += (NMR_INT)(ia - 1)*(ySize/yQuadState)*(zSize/zQuadState)*block;
            loc += (NMR_INT)(iaQuad - 1)*length;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               { 
                /*
                 * This is ugly, but for (phase,phase2,phase3) data
                 * we have to read 1 fid, skip the next, read a fid,
                 * skip one, etc.
                 */

                for( j = 0; j < yQuadState*zQuadState; j++)
                   {
                    if ((error = dataPos( inUnit, loc )))
                       {
                        FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                        return( error );
                       }

                    if (*inName)
                       {
                        if ((error = dataReadC( inUnit, vPtr, length )))
                           {
                            FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                            return( error );
                           }
                       }
                    else
                       {
                        if ((error = dataReadCB( inUnit, vPtr, length, -1 , 0 )))
                           {
                            FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                            return( error );
                           }
                       }

                    vPtr  += length;
                    loc   += 2*length;
                   }
               }     
           }
        else if (aqOrder == 2)   /* ndim == 4 */
           {
            length  = (NMR_INT)xSize*xQuadState*yQuadState*zQuadState*wordSize;
            length += (NMR_INT)VHDR_MID*yQuadState*zQuadState*VINTSIZE;
            jump    = length*aSize;

            loc     = (NMR_INT)VHDR_BEGIN*VINTSIZE + (NMR_INT)((ia - 1)*aQuadState + iaQuad - 1)*length;
            loc    += (NMR_INT)(iz - 1)*length*ySize*aSize/yQuadState;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               {
                if ((error = dataPos( inUnit, loc )))
                   {
                    FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                    return( error );
                   }

                if (*inName)
                   {
                    if ((error = dataReadC( inUnit, vPtr, length )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
                else
                   {
                    if ((error = dataReadCB( inUnit, vPtr, length, -1, 0 )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }

                vPtr  += length;
                loc   += jump;
               }
           }
       }
    else if (dimCount == 4)
       {
        if (aqOrder == 0)  /* dim=4  */      /* Here, one length = 4 fids  */
           {
            length = (NMR_INT)(xSize*xQuadState + VHDR_MID)*yQuadState*zQuadState;
            bytes  = (NMR_INT)VINTSIZE*length;
            jump   = length*aQuadState;

            /* Step up the appropriate number of "planes".                  */
            /* Next, step up the to appropriate "cube".                     */
            /* And finally, step up to the "a" imaginary vector, if needed. */

            loc  = (NMR_INT)VHDR_BEGIN + (NMR_INT)(iz - 1)*aQuadState*(ySize/yQuadState)*length;
            loc += (NMR_INT)(ia - 1)*aQuadState*(ySize/yQuadState)*(zSize/zQuadState)*length;
            loc += (iaQuad - 1)*length;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               { 
                if ((error = dataPos( inUnit, VINTSIZE*loc )))   /* 4d only */
                   {
                    FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                    return( error );
                   }

                vPtr = varMat;

                if (*inName)
                   { 
                    if ((error = dataReadC( inUnit, vPtr, bytes )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
                else
                   {
                    if ((error = dataReadCB( inUnit, vPtr, bytes, -1, 0 )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
 
                varMat += VINTSIZE*length;
                loc    += jump;
               } 
           }
        else if (aqOrder == 1)   /* dim=4 */  /* Here, one length = 1 fid */
           {
            length = (NMR_INT)(xSize*xQuadState + VHDR_MID);
            block  = length*yQuadState*zQuadState*aQuadState;
            bytes  = (NMR_INT)VINTSIZE*length;

            /* Step 1 fid at a time to extract the components.            */
            /* Step up the appropriate number of "planes".                */
            /* Next, step up the to appropriate "cube".                   */
            /* Finally, step up to the "a" imaginary vector, if needed.   */

            loc  = (NMR_INT)VHDR_BEGIN + (NMR_INT)(iz - 1)*(ySize/yQuadState)*block;
            loc += (NMR_INT)(ia - 1)*(ySize/yQuadState)*(zSize/zQuadState)*block;
            loc += (iaQuad - 1)*length;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               { 
                /*
                 * This is ugly, but for (phase,phase2,phase3) data
                 * we have to read 1 fid, skip the next, read a fid,
                 * skip one, etc.
                 */

                for( j = 0; j < yQuadState*zQuadState; j++)
                   {
                    if ((error = dataPos( inUnit, VINTSIZE*loc )))
                       {
                        FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                        return( error );
                       }

                    vPtr = varMat;

                    if (*inName)
                       {
                        if ((error = dataReadC( inUnit, vPtr, bytes )))
                           {
                            FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                            return( error );
                           }
                       }
                    else
                       {
                        if ((error = dataReadCB( inUnit, vPtr, bytes, -1 , 0 )))
                           {
                            FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                            return( error );
                           }
                       }

                    varMat += VINTSIZE*length;
                    loc +=  2*length;
                   }
               }     
           }
        else if (aqOrder == 2)   /* dim=4 */
           {
            length = (NMR_INT)(xSize*xQuadState + VHDR_MID)*yQuadState*zQuadState;
            bytes  = (NMR_INT)VINTSIZE*length;
            jump   = length*aSize;

            loc    = (NMR_INT)VHDR_BEGIN + (NMR_INT)((ia - 1)*aQuadState + iaQuad - 1)*length;
            loc   += (NMR_INT)(iz - 1)*length*ySize*aSize/yQuadState;

            for( iy = 0; iy < ySize/yQuadState; iy++ )
               {
                if ((error = dataPos( inUnit, VINTSIZE*loc )))
                   {
                    FPR( stderr, "Var2Pipe Error Positioning Input.\n" );
                    return( error );
                   }

                vPtr = varMat;

                if (*inName)
                   {
                    if ((error = dataReadC( inUnit, vPtr, bytes )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }
                else
                   {
                    if ((error = dataReadCB( inUnit, vPtr, bytes, -1, 0 )))
                       {
                        FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                        return( error );
                       }
                   }

                varMat += VINTSIZE*length;
                loc    += jump;
               }
           }
       }
    else
       {
        vPtr  = varMat;

        if (*inName)
           {
            if ((error = dataReadC( inUnit, vPtr, bytesIn )))
               {
                FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                return( error );
               }
           }
        else
           {
            if ((error = dataReadCB( inUnit, vPtr, bytesIn, -1, 0 )))
               {
                FPR( stderr, "Var2Pipe Error Reading Input.\n" );
                return( error );
               }
           }

        if (wordSize == 2)
           {
            (void) two2Four( vPtr,
                             xSize*xQuadState,
                             ySize*zQuadState*aQuadState,
                             (int)(VHDR_MID*VINTSIZE) ); 
           }
       }

    return( error );
}

/***/
/* fidOpen: open a pipe format output file, write an initial header.
/***/

int fidOpen( outFmt, outName, cmndName, outUnitPtr, plane, fdata, ovFlag, xyzFlag, dirFlag, verbose )

   UNIT  *outUnitPtr;
   float fdata[FDATASIZE];
   char  *outFmt, *outName, *cmndName;
   int   plane, ovFlag, xyzFlag, dirFlag, verbose;
{
    char   dirName[NAMELEN+1];

    static int dirCreated = 0;

    if (dirCreated) dirFlag = 0;

    (void) sprintf( outName, outFmt, plane );

    if (*outFmt)
       {
        if (dirFlag && plane == 1)
           {
            if (!dirExists( outName, dirName ))
               {
                if (verbose)
                   {
                    FPR( stderr,
                         "Var2Pipe Notice: Creating directory %s\n",
                         dirName );
                   }
 
                if (dirCreate( dirName ))
                   {
                    FPR( stderr,
                         "Var2Pipe Warning: Error creating directory %s\n",
                         dirName );
                   }

                dirCreated = 1;
               }
           }

        if (!ovFlag && fileExists( outName ))
           {
            FPR( stderr,
                 "Var2Pipe Error: output file %s exists.\n",
                 outName );

            return( 1 );
           }

        if (xyzFlag || plane == 1)
           {
            if (dataOpen( outName, outUnitPtr, FB_CREATE )) return( 1 );
           }
       }
    else if (*cmndName)
       {
        if (plane == 1)
           {
            if (dataPOpen( cmndName, outUnitPtr, FB_WRITE )) return( 1 );
           }
       }
    else
       {
        *outUnitPtr = FB_STDOUT;
       }

/***/
/* Write a header if this is a separate plane:
/***/

    if (xyzFlag || plane == 1)
       {
        if (dataWrite( *outUnitPtr, fdata, sizeof(float)*FDATASIZE ))
           {
            return( 1 );
           }
       }

    return( 0 );
}

/***/
/* fidClose: update header, then close pipe format output.
/***/

int fidClose( outFmt, outUnitPtr, minVal, maxVal, fdata, xyzFlag )

   int   xyzFlag;
   char  *outFmt;
   UNIT  *outUnitPtr;
   float minVal, maxVal, fdata[FDATASIZE];
{
    if (!*outFmt) return( 0 );
    if (!xyzFlag) return( 0 );

    (void) setParm( fdata, FDSCALEFLAG, (float)1.0, 0 );

    (void) setParm( fdata, FDMAX,     maxVal, 0 );
    (void) setParm( fdata, FDMIN,     minVal, 0 );
    (void) setParm( fdata, FDDISPMAX, maxVal, 0 );
    (void) setParm( fdata, FDDISPMIN, minVal, 0 );

    if (dataPos( *outUnitPtr, 0 )) return( 1 );
    if (dataWrite( *outUnitPtr, fdata, sizeof(float)*FDATASIZE )) return( 1 );

    (void) dataClose( *outUnitPtr );

    *outUnitPtr = UNIT_NULL;

    return( 0 );
}

/***/
/* getVFloat: extract 4 byte float from Varian header or data.
/***/

#ifdef CRAY
float getVFloat( buff )

   char *buff;
{
    int   error, type, num, bitoff, stride, natlen, forlen, nativech;
    float native, *foreign;
 
    foreign  = (float *)buff;
    type     = 3;
    num      = 1;
    bitoff   = 0;
    stride   = 1;
    natlen   = 64;
    forlen   = 32;
    nativech = 0;

    error = CRI2CRY( &type, &num, foreign, &bitoff, native,
                     &stride, &natlen, &forlen, &nativech );


   return( native );
}
#endif

/***/
/* getVInt: extract 4 byte int from Varian header or data.
/***/

#ifdef CRAY
int getVInt( buff )

   char *buff;
{
    union r2i { int i; char s[sizeof(int)]; } out;

    out.i = 0;

    out.s[0] = *buff++;
    out.s[1] = *buff++;
    out.s[2] = *buff++;
    out.s[3] = *buff++;

    out.i /= ISCALE;

    return( out.i );
}
#endif

/***/
/* getVShort: extract 2 byte short from Varian header or data.
/***/

#ifdef CRAY
short getVShort( buff )

   char *buff;
{
    union r2i { int i; char s[sizeof(int)]; } out;

    out.i = 0;

    out.s[0] = *buff++;
    out.s[1] = *buff++;

    out.i /= ISCALE64K;

    return( out.i );
}
#endif

/***/
/* expandVHdr: extract Varian Header from byte array.
/***/

#ifdef CRAY
int expandVHdr( vInfo, hdrBuff, swapFlag )

   struct VHdrBuff *vInfo;
   char   *hdrBuff;
   int    swapFlag;
{
    char *sPtr;
    int  i;

    sPtr = hdrBuff;

    vInfo->v1 = getVInt( sPtr ); sPtr += VINTSIZE;
    vInfo->v2 = getVInt( sPtr ); sPtr += VINTSIZE;
    vInfo->v3 = getVInt( sPtr ); sPtr += VINTSIZE;
    vInfo->v4 = getVInt( sPtr ); sPtr += VINTSIZE;

    for( i = 0; i < 10; i++ )
       {
        vInfo->v5[i] = *sPtr++;
       }

    vInfo->v6 = getVShort( sPtr ); sPtr += VSHORTSIZE;

    for( i = 0; i < 4; i++ )
       {
        vInfo->v7[i] = *sPtr++;
       }

    return( 0 );
}
#endif

/***/
/* var2fid: convert Varian format 2D plane to nmrPipe format plane.
/***/

int var2fid( varMat, dataMat,
             xSize, ySize, zSize, aSize, izQuad, iaQuad,
             xQuadState, yQuadState, zQuadState, aQuadState,
             aqOrder, swapFlag, i2rFlag, signFlag, minVal, maxVal )

   int   xSize, ySize, zSize, aSize, izQuad, iaQuad,
         xQuadState, yQuadState, zQuadState, aQuadState, aqOrder,
         swapFlag, i2rFlag, signFlag;

   float *dataMat, *minVal, *maxVal;

   char  *varMat;
{
    int error;

/***/
/* Yamazaki's suggested fix 7/21/94: 
/*
/*   if (aqOrder == 1 || yQuadState == 1 || aqOrder == 2)
/***/

    if (aqOrder == 1 || yQuadState == 1)
       {
        error = 
         var2fid1( varMat, dataMat,
                   xSize, ySize, zSize, aSize, izQuad, iaQuad, 
                   xQuadState, yQuadState, zQuadState, aQuadState,
                   aqOrder, swapFlag, i2rFlag, signFlag, minVal, maxVal );
       }
    else
       {
        error = 
         var2fid2( varMat, dataMat,
                   xSize, ySize, zSize, aSize, izQuad, iaQuad,
                   xQuadState, yQuadState, zQuadState, aQuadState,
                   aqOrder, swapFlag, i2rFlag, signFlag, minVal, maxVal );
       }

    return( error );
}

/***/
/* var2fid1: convert Varian format Y-real plane to nmrPipe format plane.
/***/

int var2fid1( varMatV, dataMat,
              xSize, ySize, zSize, aSize, izQuad, iaQuad,
              xQuadState, yQuadState, zQuadState, aQuadState,
              aqOrder, swapFlag, i2rFlag, signFlag, minVal, maxVal )
  
   int   xSize, ySize, zSize, aSize, izQuad, iaQuad,
         xQuadState, yQuadState, zQuadState, aQuadState, aqOrder,
         swapFlag, i2rFlag, signFlag;

   float *dataMat, *minVal, *maxVal;
   char  *varMatV;
{
    VMATTYPE *varMat;

    int   ix, iSign, vJump;
    float *matPtrR, *matPtrI;
    char  c;

/***/
/* 3D Varian input:
/*   InternalHeader RIRIRI... for Y=real Z=real 1D Vector.
/*   InternalHeader RIRIRI... for Y=real Z=imag 1D Vector. 
/*   etc...
/*
/* Z-complex data means skip alternate vectors.
/*
/* Value iz=1 means extract odd vectors.
/* Value iz=2 means extract even ones.
/***/

    varMat = (VMATTYPE *)varMatV;

    if (zQuadState == 1)
       vJump = VHDR_MID;
    else
       vJump = VHDR_MID + (VHDR_MID + xSize*xQuadState);

    if (izQuad == 1)
       {
        VARJUMP( VHDR_MID );
        iSign   = 1;
       }
    else
       {
        VARJUMP( vJump );
        iSign   = -1;
       }

    if (iaQuad == 2)
       {
        iSign   = -iSign;
       }

/***/
/* For X-real data, extract all points.
/* For X-complex data, separate the interleaved R/I points.
/* Find max and min values along the way.
/***/

    if (xQuadState == 1)
       {
        matPtrR = dataMat;

        while( ySize-- )
           {
            for( ix = 0; ix < xSize; ix++ )
               {
                if (swapFlag) VARSWAP4;

                *matPtrR = VARDATA(iSign); VARINCR;

                if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                if (*matPtrR < *minVal) *minVal = *matPtrR;

                matPtrR++;
               }

            VARJUMP( vJump );
           }
       }
    else
       {
        matPtrR = dataMat;
        matPtrI = dataMat + xSize;

        if (aqOrder == 1)
           {
            while( ySize-- )
               {
                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (swapFlag) VARSWAP4;
                    *matPtrR = VARDATA(iSign);  VARINCR;

                    if (swapFlag) VARSWAP4;
                    *matPtrI = VARDATA(-iSign); VARINCR;

                    if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                    if (*matPtrR < *minVal) *minVal = *matPtrR;

                    matPtrR++;
                    matPtrI++;
                   }

                matPtrR += xSize;
                matPtrI += xSize;

                VARJUMP( vJump );

                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (swapFlag) VARSWAP4;
                    *matPtrR = VARDATA(-iSign); VARINCR;

                    if (swapFlag) VARSWAP4;
                    *matPtrI = VARDATA(iSign);  VARINCR;

                    if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                    if (*matPtrR < *minVal) *minVal = *matPtrR;

                    matPtrR++;
                    matPtrI++;
                   }

                matPtrR += xSize;
                matPtrI += xSize;

                VARJUMP( vJump );

                ySize--;
               }
           } 
        else
           {
            while( ySize-- )
               {
                for( ix = 0; ix < xSize; ix++ )
                   {
                    if (swapFlag) VARSWAP4;
                    *matPtrR = VARDATA(iSign);  VARINCR;

                    if (swapFlag) VARSWAP4;
                    *matPtrI = VARDATA(-iSign); VARINCR;

                    if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                    if (*matPtrR < *minVal) *minVal = *matPtrR;

                    matPtrR++;
                    matPtrI++;
                   }

                matPtrR += xSize;
                matPtrI += xSize;

                VARJUMP( vJump );
               }
           }
       }

    return( 0 );
} 

/***/
/* var2fid2: convert Varian format Y-complex plane to nmrPipe format plane.
/***/

int var2fid2( varMatV, dataMat,
              xSize, ySize, zSize, aSize, izQuad, iaQuad,
              xQuadState, yQuadState, zQuadState, aQuadState,
              aqOrder, swapFlag, i2rFlag, signFlag, minVal, maxVal )
  
   int   xSize, ySize, zSize, aSize, izQuad, iaQuad,
         xQuadState, yQuadState, zQuadState, aQuadState, aqOrder, 
         swapFlag, i2rFlag, signFlag;

   float *dataMat, *minVal, *maxVal;
   char  *varMatV;
{
    VMATTYPE *varMat;

    int   iSign, ix, vJump;
    float *matPtrR, *matPtrI;
    char  c;

/***/
/* Typical 3D Varian input:
/*
/*   InternalHeader RIRIRI... for Y=real Z=real 1D Vector.
/*   InternalHeader RIRIRI... for Y=imag Z=real 1D Vector.
/*
/*   InternalHeader RIRIRI... for Y=real Z=imag 1D Vector. 
/*   InternalHeader RIRIRI... for Y=imag Z=imag 1D Vector. 
/*
/*   etc...
/*
/* Z-complex data means skip alternate vector pairs.
/*
/* Value iz=1 means extract odd vector pairs.
/* Value iz=2 means extract even ones.
/***/

    varMat = (VMATTYPE *)varMatV;

    if (zQuadState == 1)
       vJump = VHDR_MID;
    else
       vJump = VHDR_MID + 2*(VHDR_MID + xSize*xQuadState);

    if (izQuad == 1)
       {
        VARJUMP( VHDR_MID );
        iSign   = 1;
       }
    else
       {
        VARJUMP( vJump );
        iSign   = -1;
       }

    if (iaQuad == 2)
       {
        iSign = -iSign;
       }

/***/
/* For X-real data, extract all points.
/* For X-complex data, separate the interleaved R/I points.
/* Find max and min values along the way.
/***/

    if (xQuadState == 1)
       {
        matPtrR = dataMat;

        while( ySize-- )
           {
            for( ix = 0; ix < xSize; ix++ )
               {
                if (swapFlag) VARSWAP4;
                *matPtrR = VARDATA(iSign); VARINCR;

                if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                if (*matPtrR < *minVal) *minVal = *matPtrR;

                matPtrR++;
               }

            iSign   = -iSign;
            VARJUMP( VHDR_MID );
            ySize--;

            for( ix = 0; ix < xSize; ix++ )
               {
                if (swapFlag) VARSWAP4;
                *matPtrR = VARDATA(iSign); VARINCR;

                if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                if (*matPtrR < *minVal) *minVal = *matPtrR;

                matPtrR++;
               }

            iSign   = -iSign;

            VARJUMP( vJump );
           }
       }
    else
       {
        matPtrR = dataMat;
        matPtrI = dataMat + xSize;

        while( ySize-- )
           {
            for( ix = 0; ix < xSize; ix++ )
               {
                if (swapFlag) VARSWAP4;
                *matPtrR = VARDATA(iSign);  VARINCR;

                if (swapFlag) VARSWAP4;
                *matPtrI = VARDATA(-iSign); VARINCR;

                if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                if (*matPtrR < *minVal) *minVal = *matPtrR;

                matPtrR++;
                matPtrI++;
               }

            iSign    = -iSign;
            matPtrR += xSize;
            matPtrI += xSize;

            VARJUMP( VHDR_MID ); 

            ySize--;

            for( ix = 0; ix < xSize; ix++ )
               {
                if (swapFlag) VARSWAP4;
                *matPtrR = VARDATA(iSign);  VARINCR;

                if (swapFlag) VARSWAP4;
                *matPtrI = VARDATA(-iSign); VARINCR;

                if (*matPtrR > *maxVal) *maxVal = *matPtrR;
                if (*matPtrR < *minVal) *minVal = *matPtrR;

                matPtrR++;
                matPtrI++;
               }

            iSign    = -iSign;
            matPtrR += xSize;
            matPtrI += xSize;

            VARJUMP( vJump );
           }
       }

    return( 0 );
} 

/***/
/* getparams: extract command arguments for var2pipe.
/***/

int getparams( argc, argv,
               inName, outFmt, infoName, serverListName,
               fdata, firstPlane, lastPlane, xyzFlag, aqOrder,
               swapFlag, i2rFlag, shortFlag, checkFlag, ovFlag,
               dirFlag, vHdrFlag, infoFlag, tclFlag, signFlag, neFlag, 
               shufFlag, verbose )

   int   argc, *swapFlag, *i2rFlag, *shortFlag, *checkFlag, *firstPlane,
         *lastPlane, *xyzFlag, *ovFlag, *aqOrder, *dirFlag, *vHdrFlag,
         *infoFlag, *tclFlag, *signFlag, *neFlag, *shufFlag, *verbose;

   char  **argv, *inName, *outFmt, *infoName, *serverListName;
   float fdata[FDATASIZE];
{
    int loc, nextLoc;

/***/
/* Initialize:
/***/

#ifdef SYS_BYTESWAP
    *swapFlag       = 1;
#else
    *swapFlag       = 0;
#endif

    *aqOrder        = 0;
    *checkFlag      = 1;
    *firstPlane     = 0;
    *lastPlane      = 0;
    *verbose        = 0;
    *xyzFlag        = 0;
    *ovFlag         = 1;
    *i2rFlag        = 1;
    *shortFlag      = 0;
    *dirFlag        = 1;
    *infoFlag       = 0;
    *tclFlag        = 0;
    *signFlag       = 1;
    *neFlag         = 0;
    *shufFlag       = 1;
    *vHdrFlag       = 1;

    *serverListName = '\0';
    *inName         = '\0';
    *outFmt         = '\0';
    *infoName       = '\0';

    if (argc == 1 || flagLoc( argc, argv, "-help" ))
       {
        FPR_VERSION;
        FPR( stderr, "Var2Pipe: Varian to NMRPipe Conversion.\n" );
        FPR( stderr, "%s hdrFlags < varData > pipeFile\n", argv[0] );
        FPR( stderr, "General Flags:\n" );
        FPR( stderr, " -in  inName       Varian Input (Required for 4D Data).\n" );
        FPR( stderr, " -out outTemplate  NMRPipe Output File or Template.\n" );
        FPR( stderr, "                   Template Required for 3D and 4D.\n" );
        FPR( stderr, " -nofs             No File Size Check.\n" );
        FPR( stderr, " -nodir            Suppress Creation of Directory.\n" );
        FPR( stderr, " -shuf             Use Shuffling Macros (Default).\n" );
        FPR( stderr, " -noshuf           Suppress Shuffling Macros.\n" );
        FPR( stderr, " -verb             Verbose Mode On.\n" );
        FPR( stderr, "Byte-Swap Options:\n" );
#ifdef SYS_BYTESWAP
        FPR( stderr, " -swap             Do Four-Byte Swap (Default).\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap.\n");
        FPR( stderr, " -aswap            Suppress Four-Byte Swap.\n");
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
        FPR( stderr, " -noaswap          Do Four-Byte Swap.\n" );
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
#else
        FPR( stderr, " -swap             Do Four-Byte Swap.\n" );
        FPR( stderr, " -noswap           Suppress Four-Byte Swap (Default).\n");
        FPR( stderr, " -aswap            Do Four-Byte Swap.\n" );
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
        FPR( stderr, " -noaswap          Suppress Four-Byte Swap.\n");
        FPR( stderr, "                   (Meaning is System-dependant)\n" );
#endif
        FPR( stderr, "Varian Header Flags:\n" );
        FPR( stderr, " -hdr              Use Varian Header Data (Default).\n" );
        FPR( stderr, " -nohdr            Don't Use Varian Header.\n" );
        FPR( stderr, " -hw    hSize [8]  Header Size in Four-Byte Words.\n" ); 
        FPR( stderr, " -pw    pSize [7]  Pad Size in Four-Byte Words.\n" ); 
        FPR( stderr, " -info             Show Header Info on stdout,\n" );
        FPR( stderr, "                   Don't Convert Data.\n" );
        FPR( stderr, " -tcl              Show Header Info in TCL Format.\n" );
        FPR( stderr, "Options Set Via Varian Header:\n" );
        FPR( stderr, " -i2r              Convert Integer to Real.\n" );
        FPR( stderr, " -noi2r            No Integer to Real Conversion.\n" );
        FPR( stderr, " -short            Input is Two-Byte Data.\n" );
        FPR( stderr, " -long             Input is Four-Byte Data.\n" );
        FPR( stderr, " -dsp              Input is floating-point DSP data;\n" );
        FPR( stderr, "                    (Same as -noi2r).\n" );
/*
        FPR( stderr, "Parallel Processing (Not Yet Implemented):\n" );
        FPR( stderr, " -par   sListName  Server List File Name.\n" );
        FPR( stderr, " -first firstPlane First Plane to process.\n");
        FPR( stderr, " -last  lastPlane  Last Plane to process.\n");
        FPR( stderr, "\n" );
*/

        (void) parseHdr( argc, argv, fdata );

        FPR( stderr, "Special Varian Format Flags:\n" );
/*
        FPR( stderr, " -ne               Use Experiment Count.\n" );
*/
        FPR( stderr, " -sign              Use Sign Adjustment (Default).\n" );
        FPR( stderr, " -nosign            Suppress Sign Adjustment.\n" );
        FPR( stderr, " -aqORD  aqOrd  [0] Acquisition Order Code:\n" );
        FPR( stderr, "     0 = 2D d2,phase\n" );
        FPR( stderr, "     0 = 3D d3,d2,phase2,phase\n" );
        FPR( stderr, "     0 = 4D d4,d3,d2,phase3,phase2,phase\n" );
        FPR( stderr, "     1 = 3D d3,d2,phase,phase2\n" );
        FPR( stderr, "     1 = 4D d4,d3,d2,phase,phase2,phase3\n" );
        FPR( stderr, "     2 = 4D d3,d2,d4,phase3,phase2,phase\n" );
        FPR( stderr, "\n" );
        FPR( stderr, "Notes:\n" );
        FPR( stderr, " 1. Output names for 3D data can be either single\n" );
        FPR( stderr, "    files (such as test.fid) or 2D series templates\n" );
        FPR( stderr, "    (such as fid/test%%03d.fid).\n" );
        FPR( stderr, "\n" );
        FPR( stderr, " 2. Output names for 4D data can be either single\n" );
        FPR( stderr, "    files (such as test.fid) or 3D cube templates\n" );
        FPR( stderr, "    (such as fid/test%%03d.fid) or 2D series templates\n" );
        FPR( stderr, "    (such as fid/test%%03d%%03d.fid).\n" );
        FPR( stderr, " 3. In the case of 4D data, an explicit input file must\n" );
        FPR( stderr, "    be specified, input cannot be from a pipe.\n" );
        FPR( stderr, "\n" );

        return( 1 );
       }

/***/
/* Extract command-specific parameters:
/***/

    if (flagLoc( argc, argv, "-ne" ))     *neFlag   = 1;

    if (flagLoc( argc, argv, "-tcl" ))    *tclFlag = 1;
    if (flagLoc( argc, argv, "-notcl" ))  *tclFlag = 0;

    if (flagLoc( argc, argv, "-sign" ))   *signFlag = 1;
    if (flagLoc( argc, argv, "-nosign" )) *signFlag = 0;

    if (flagLoc( argc, argv, "-shuf" ))   *shufFlag = 1;
    if (flagLoc( argc, argv, "-noshuf" )) *shufFlag = 0;

    (void) strArgD( argc, argv, "-in",    inName );
    (void) strArgD( argc, argv, "-out",   outFmt );
    (void) strArgD( argc, argv, "-par",   serverListName );
    (void) intArgD( argc, argv, "-first", firstPlane );
    (void) intArgD( argc, argv, "-last",  lastPlane );
    (void) intArgD( argc, argv, "-aqORD", aqOrder );

    (void) intArgD( argc, argv, "-hw", &VHDR_BEGIN );
    (void) intArgD( argc, argv, "-pw", &VHDR_MID );

    if (VHDR_BEGIN == 0) *vHdrFlag = 0;

    if ((loc = flagLoc( argc, argv, "-info" )))
       {
        *infoFlag = 1;
        *verbose  = 1;

        nextLoc = nextFlag( argc, argv, loc );

        if (nextLoc > loc + 1)
           {
            (void) strArgD( argc, argv, "-info", infoName );
           } 
       }

#ifdef SYS_BYTESWAP
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 0;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 1;
#else
    if (flagLoc( argc, argv, "-aswap" ))   *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noaswap" )) *swapFlag     = 0;
#endif

    if (flagLoc( argc, argv, "-swap" ))    *swapFlag     = 1;
    if (flagLoc( argc, argv, "-noswap" ))  *swapFlag     = 0;

    if (flagLoc( argc, argv, "-hdr" ))    *vHdrFlag  = 1;
    if (flagLoc( argc, argv, "-nohdr" ))  *vHdrFlag  = 0;
    if (flagLoc( argc, argv, "-dsp" ))    *i2rFlag   = 0;
    if (flagLoc( argc, argv, "-i2r" ))    *i2rFlag   = 1;
    if (flagLoc( argc, argv, "-noi2r" ))  *i2rFlag   = 0;
    if (flagLoc( argc, argv, "-short" ))  *shortFlag = 1;
    if (flagLoc( argc, argv, "-long" ))   *shortFlag = 0;
    if (flagLoc( argc, argv, "-nofs" ))   *checkFlag = 0;
    if (flagLoc( argc, argv, "-nodir" ))  *dirFlag   = 0;
    if (flagLoc( argc, argv, "-ov" ))     *ovFlag    = 1;
    if (flagLoc( argc, argv, "-noov" ))   *ovFlag    = 0;
    if (flagLoc( argc, argv, "-verb" ))   *verbose   = 1;
    if (flagLoc( argc, argv, "-noverb" )) *verbose   = 0;

/***/
/* Test for required output format with xyz style output:
/***/

    if (isZFmt( outFmt ) || isAZFmt( outFmt )) *xyzFlag = 1;

/***/
/* Set some default parameters:
/***/

    (void) txt2Flt( "X", fdata + FDF2LABEL, SIZE_F2LABEL );
    (void) txt2Flt( "Y", fdata + FDF1LABEL, SIZE_F1LABEL );
    (void) txt2Flt( "Z", fdata + FDF3LABEL, SIZE_F3LABEL );
    (void) txt2Flt( "A", fdata + FDF4LABEL, SIZE_F4LABEL );

/***/
/* Parse header parameters:
/***/

    if (parseHdr( argc, argv, fdata )) return( 1 );

    return( 0 );
}

/***/
/* swapVHdr: byte-swap elements of Varian header.
/***/

static int swapVHdr( vPtr )

   char *vPtr;
{
    char *sPtr, c;

    sPtr = vPtr;
    SWAP4( sPtr );

    sPtr += VINTSIZE;
    SWAP4( sPtr );

    sPtr += VINTSIZE;
    SWAP4( sPtr );

    sPtr += VINTSIZE;
    SWAP4( sPtr );

    sPtr += VINTSIZE;
    sPtr += 10;

    SWAP2( sPtr );

    return( 0 );
}

/***/
/* two2Four: expand two-byte words into four-byte words.
/***/

static int two2Four( cVec, xSize, ySize, jump )

   char *cVec;
   int  xSize, ySize, jump;
{
    char  *cDest, *cSrc;
    int   *dest, i;
    short *src;
 
    cDest = cVec + xSize*ySize*sizeof(int) + ySize*jump - sizeof(int);
    cSrc  = cVec + xSize*ySize*VSHORTSIZE  + ySize*jump - VSHORTSIZE;

    while( ySize-- )
       {
        dest   = (int *)cDest;
        src    = (short *)cSrc;
 
        for( i = 0; i < xSize; i++ ) *dest-- = (int) (*src--);

        cDest -= jump + xSize*sizeof(int);
        cSrc  -= jump + xSize*sizeof(short);
       }

    return( 0 ); 
}

/***/
/* Bottom.
/***/
