/***/
/* Synthetic images by K-Matrix relative values.
/*
/* planeName: name of a 2D plane in the 3D image series input.
/* refName:   template for 1D reference spectra.
/* refCount:  number of 1D reference spectra.
/***/

FDATASIZE = 512;

/***/
/* Prepare the design matrix from the reference spectra:
/***/

if (sliceCode == 1)
   {
    float specBuff[xSize], simBuff[zSize*ySize*refCount], hdr[FDATASIZE];
    float xdata[refCount*xSize], coef[refCount];
    char  thisName[80];

    first  = integer(0.2*xSize);
    last   = integer(0.8*xSize);
    length = 1 + last - first;

    for( i = 0; i < refCount; i++ )
       {
        (void) sprintf( thisName, refName, i+1 );

        fd  = dOpen( thisName, "r" );
 
        (void) dReadHdr( fd, hdr, 0 );
        (void) dRead( fd, specBuff, wordLen*size );
        (void) dClose( fd );

        k = i;

        for( j = first; j <= last ; j++ )
           {
            xdata[k] = specBuff[j];
            k += refCount;
           }
       }
   };

/***/
/* Perform and store the results for a given unknown 1D:
/***/

(void) vLinLS( xdata, rdata, coef, refCount, length );

sum = vSum( coef, refCount );

if (sum == 0.0)
   {
    sum = 1.0;
   };

for( i = 0; i < refCount; i++ )
   {
    loc = (sliceCode - 1) + i*zSize*ySize;
    simBuff[loc] = 100.0*coef[i]/sum;
   }

/***/
/* Save the results as a 3D output stream.
/***/

if (sliceCode == ySize*zSize)
   {
    FDPIPEFLAG = 57
    NDAPOD     = 1000+2;

    fd  = dOpen( planeName, "r" );

    (void) dReadHdr( fd, hdr, 0 );
    (void) dClose( fd );

    (void) setParm( hdr, FDPIPEFLAG, CUR_XDIM, 0 );
    (void) setParm( hdr, NDSIZE,     refCount, CUR_ZDIM );
    (void) setParm( hdr, NDAPOD,     refCount, CUR_ZDIM );

    (void) dWrite( outUnit, hdr, wordLen*FDATASIZE );
    (void) dWrite( outUnit, simBuff, wordLen*zSize*ySize*refCount );
   };
