#!/bin/csh

# filename: jres.com
#
# J-resolved processing.
# Finds "slope" (pts/pts) according to initial tilt in J-resolved data.
# Finds "mid"   (pts) according to zero fill in F1, and F1 acq parameters.

nmrPipe -in test.fid \
| nmrPipe -fn SOL                       \
| nmrPipe -fn SP -pow 1 -off 0.5 -verb  \
| nmrPipe -fn ZF -size 2048             \
| nmrPipe -fn FT -auto                  \
| nmrPipe -fn TP                        \
| nmrPipe -fn SP -pow 2                 \
| nmrPipe -fn ZF -size 128              \
| nmrPipe -fn FT -verb                  \
| nmrPipe -fn MC                        \
| nmrPipe -fn TP                        \
   -out test.ft2 -ov -verb

#
# Tilt and symmetrize:

set ySize = `getParm -in test.ft2 -parm NDSIZE -dim CUR_YDIM -fmt %.0f`
set xSW   = `getParm -in test.ft2 -parm NDSW   -dim CUR_XDIM -fmt %f`
set ySW   = `getParm -in test.ft2 -parm NDSW   -dim CUR_YDIM -fmt %f`

set mid   = `MATH "integer($ySize/2) + 1"`
set slope = `MATH "$ySW/$xSW"`

echo "J-Tilt. Midpoint: $mid Slope: $slope"

nmrPipe -in test.ft2 \
| nmrPipe -fn MAC -macro $NMRTXT/jTilt.M -var slope $slope mid $mid \
   -out tilt.ft2 -ov -verb

nmrPipe -in tilt.ft2 \
| nmrPipe -fn TP     \
| nmrPipe -fn MAC -macro sym.M -var mid $mid -verb \
| nmrPipe -fn TP     \
   -out sym.ft2 -ov
