/***/
/* sed.M: channel selection for Spin-Echo difference spectra.
/*
/*        Options:
/*          -ref  Selects 1 2 of 1234.
/*          -att  Selects 3 4 of 1234.
/*          -dif  Selects 1 2 - 3 4 of 1234.
/*          -sum  Selects 1 2 + 3 4 of 1234.
/***/

/***/
/* Initialization steps:
/*  ARGS: extract mode.
/*  INIT: update header to reflect reduced number of 1D vectors.
/***/

if (sliceCode == CODE_ARGS)
   {
    RefMode = 1;
    AttMode = 2;
    DifMode = 3;
    SumMode = 4;
    mode    = RefMode;

    if (argTest( "-ref" ))
       {
        mode = RefMode;
       };

    if (argTest( "-att" ))
       {
        (void) printf( "Selecting 3 4 of 1234.\n" );
        mode = AttMode;
       };

    if (argTest( "-diff" ))
       {
        (void) printf( "Selecting  1 2 - 3 4 of 1234.\n" );
        mode = DifMode;
       };

    if (argTest( "-sum" ))
       {
        (void) printf( "Selecting  1 2 + 3 4 of 1234.\n" );
        mode = SumMode;
       };

    if (mode == RefMode)
       {
        (void) printf( "Selecting 1 2 of 1234.\n" );
       };
   };

if (sliceCode == CODE_INIT)
   {
    NDAPOD = 1000 + 2;

    (void) setParm( fdata, NDSIZE, integer( specnum/2 ), CUR_YDIM ); 
    (void) setParm( fdata, NDAPOD, integer( specnum/4 ), CUR_YDIM ); 
   };

if (sliceCode < 1)
   {
    exit( 0 ); 
   };

/***/
/* Process only every group of 4 complex 1D slices:
/***/

if (sliceCode % 4)
   {
    exit( 0 );
   };

float f12[size*4], f34[size*4];

(void) dReadB( inUnit, f12, wordLen*size*4 );
(void) dReadB( inUnit, f34, wordLen*size*4 );

if (mode == DifMode)
   {
    (void) vvSub( f12, f34, size*4 );
   };

if (mode == SumMode)
   {
    (void) vvAdd( f12, f34, size*4 );
   };

if (mode == AttMode)
   {
    (void) dWrite( outUnit, f34, wordLen*size*4 );
   }
else
   {
    (void) dWrite( outUnit, f12, wordLen*size*4 );
   };
