#!/bin/csh

#
# 3D Processing with Linear Prediction: Parallal Version.
#  Invoked with CPU-ID number as argument 1 ($1).

#
# Initialize: delete any existing synchronization files:

waitFor -par pipe.net -ext part1 -init
waitFor -par pipe.net -ext part2 -init
waitFor -par pipe.net -ext part3 -init

#
# Part 1: Process the X-Axis.
# Wait for all computers to finish part 1.

xyz2pipe -in fid/test%03d.fid -x -verb -par pipe.net -cpu $1 \
| nmrPipe  -fn SOL                                  \
| nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 2 -c 0.5  \
| nmrPipe  -fn ZF -auto                             \
| nmrPipe  -fn FT                                   \
| nmrPipe  -fn PS -p0 43  -p1 0.0 -di               \
| nmrPipe  -fn EXT -left -sw                        \
| pipe2xyz -out ft/test%03d.dat -x

waitFor -par pipe.net -ext part1 -cpu $1

#
# Part 2: Process the Z-Axis.
# Wait for all computers to finish part 2.

xyz2pipe -in ft/test%03d.dat -z -verb -par pipe.net -cpu $1 \
| nmrPipe  -fn SP -off 0.5 -end 0.95 -pow 1 -c 0.5  \
| nmrPipe  -fn ZF -auto                             \
| nmrPipe  -fn FT                                   \
| nmrPipe  -fn PS -p0 0.0 -p1 0.0 -di               \
| pipe2xyz -out ft/test%03d.ft3 -z

waitFor -par pipe.net -ext part2 -cpu $1

#
# Part 3: Predict and Process the Y-Axis:
# Wait for all computers to finish part 3.

xyz2pipe -in ft/test%03d.ft3 -y -verb -par pipe.net -cpu $1 \
| nmrPipe  -fn LP -fb -ord 10                       \
| nmrPipe  -fn SP -off 0.5 -end 0.98 -pow 1 -c 1.0  \
| nmrPipe  -fn ZF -auto                             \
| nmrPipe  -fn FT                                   \
| nmrPipe  -fn PS -p0 -135 -p1 180 -di              \
| pipe2xyz -out ft/test%03d.dat -y

waitFor -par pipe.net -ext part3 -cpu $1

#
# Part 4: Inverse Process, Predict, Re-Process Z-Axis:

xyz2pipe -in lp/test%03d.dat -z -verb -par pipe.net -cpu $1 \
| nmrPipe  -fn HT  -auto                            \
| nmrPipe  -fn PS  -inv -hdr                        \
| nmrPipe  -fn FT  -inv                             \
| nmrPipe  -fn ZF  -inv                             \
| nmrPipe  -fn SP  -inv -hdr                        \
| nmrPipe  -fn LP  -fb                              \
| nmrPipe  -fn SP  -off 0.5 -end 0.98 -pow 1 -c 0.5 \
| nmrPipe  -fn ZF  -auto                            \
| nmrPipe  -fn FT                                   \
| nmrPipe  -fn PS  -hdr -di                         \
| pipe2xyz -out lp/test%03d.ft3 -z
