#!/bin/csh

#
# 4D 13C NOE Processing with re-ordering of the data dimensions. 
# This version includes LP in both 13C dimensions.
#
#   Original order: X=acq1H Y=indir1H Z=13Cz    A=13Ca.
#   Final order:    X=acq1H Y=13Cz    Z=indir1H A=13Ca.
#   The scheme processes (acq1H,indir1H), then 13Cz, then 13Ca.

#
# Process acq1H and indir1H, re-order data on output:

xyz2pipe -in fid/c13c13noe%02d%03d.DAT -x \
| nmrPipe -fn SP  -off 0.50 -end 0.95 -pow 2 -c 0.5 \
| nmrPipe -fn ZF  -auto                             \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 0.0 -p1 0.0 -di               \
| nmrPipe -fn TP                                    \
| nmrPipe -fn SP  -off 0.50 -end 0.95 -pow 1 -c 0.5 \
| nmrPipe -fn ZF  -auto                             \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 0.0 -p1 0.0 -di               \
| nmrPipe -fn TP                                    \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -z -ov

#
# Process 13Cz, which is now in the Y-axis:

xyz2pipe -in ft/c13c13noe%02d%03d.DAT -y -verb      \
| nmrPipe -fn SP -off 0.5 -end 0.95 -pow 1  -c 1.0  \
| nmrPipe -fn ZF -auto                              \
| nmrPipe -fn FT                                    \
| nmrPipe -fn PS  -p0 -90  -p1 180  -di             \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -y -ov -inPlace

#
# LP and Process 13Ca, which is still in the A-axis:

xyz2pipe -in ft/c13c13noe%02d%03d.DAT -a -verb     \
| nmrPipe -fn LP -fb                               \
| nmrPipe -fn SP -off 0.5 -end 0.95 -pow 1  -c 1.0 \
| nmrPipe -fn ZF -auto                             \
| nmrPipe -fn FT                                   \
| nmrPipe -fn PS  -p0 -90  -p1 180  -di            \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -a -ov -inPlace

#
# Inverse process, LP, and re-process 13Cz, which is in the Y-axis as before:

xyz2pipe -in ft/c13c13noe%02d%03d.DAT -y -verb     \
| nmrPipe -fn HT -auto                             \
| nmrPipe -fn PS -inv -hdr                         \
| nmrPipe -fn FT -inv                              \
| nmrPipe -fn ZF -inv                              \
| nmrPipe -fn SP -inv -hdr                         \
| nmrPipe -fn LP -fb                               \
| nmrPipe -fn SP -hdr                              \
| nmrPipe -fn ZF -auto                             \
| nmrPipe -fn FT                                   \
| nmrPipe -fn PS -hdr -di                          \
| pipe2xyz -out ft/c13c13noe%02d%03d.DAT -y -ov -inPlace
