#!/bin/csh -f

set varList = (dirName  outName   listName  logName         initX1  initXN  extX1  extXN  noiseX1  noiseXN  psX1     psX2     psX3     psX4      refX1   refXN    ref0PPM normX1   normXN  lbHz)
set defList = (.        test.ft1  in.list   aph2Proc1D.log  0%      100%    0%     100%   12.0ppm  11.0ppm  12.0ppm  11.0ppm  -1.5ppm  -2.5ppm   0.2ppm  -0.2ppm  0.0     9.0ppm   6.0ppm  1.0)

if (`flagLoc $argv -help`) then
   echo "Example Script to Convert, Process, and Auto-Phase 1D Data:"

   set i = 1
   
   foreach vv ($varList)
      set defVal = $defList[$i]
      nmrPrintf " %-12s %-12s\n" -$vv \[$defVal\]
      @ i++
   end

   exit 0
endif

set i = 1

foreach vv ($varList)
   set defVal = $defList[$i]
   set $vv = (`getArgD -$vv $defVal`)
   @ i++
end

set valList = ($dirName $outName  $listName $logName        $initX1 $initXN $extX1 $extXN $noiseX1 $noiseXN $psX1    $psX2    $psX3    $psX4     $refX1  $refXN   $ref0PPM $normX1 $normXN $lbHz)

cd $dirName

foreach f ($outName fid.com test.fid axt.ft1 refPPM.tab $logName)
   if (-e $f) then
      /bin/rm -rf $f
   endif
end

echo "Convert, Process, and Auto-Phase 1D Data. Settings:" >> $logName

set i = 1

foreach vv ($varList)
   set thisVal = $valList[$i]
   nmrPrintf " %-12s %-12s\n" $vv \[$thisVal\] >> $logName
   @ i++
end

echo "" >> $logName

echo $dirName/$outName >> $listName 

echo "" >> $logName
echo "Format Conversion:" >> $logName

if ((-e fid) && (-e acqus)) then
   bruker -notk -amx -auto -exit -nosleep -noverb >>& $logName
else if ((-e fid) && (-e procpar)) then
   varian -notk -auto -exit -nosleep -noverb >>& $logName
else
   set fList = (`find ./ -iname \*.jdf -maxdepth 0`)

   if ($#fList) then
      delta -notk -auto -exit -nosleep >>& $logName
   endif 
endif

if (!(-e fid.com)) then
   echo "Error creating conversion script in directory $dirName" >> $logName
   exit 1
endif

fid.com >>& $logName

if (!(-e test.fid)) then
   echo "Error converting data in directory $dirName" >> $logName
   exit 1
endif

if (-e title) then
   sethdr test.fid -title `cat title`
endif

echo "" >> $logName
echo "Processing:" >> $logName

nmrPipe -in test.fid \
| nmrPipe -fn EM -lb $lbHz -c 0.5 \
| nmrPipe -fn ZF -zf 1 -auto \
| nmrPipe -fn FT \
| nmrPipe -fn EXT -x1 $initX1 -xn $initXN -sw \
| nmrPipe  -fn MAC -macro $NMRTXT/aph1DRMS2.M -all \
           -reg -str psX1 $psX1 psX2 $psX2 psX3 $psX3 psX4 $psX4 -var xP1 0 p0Step 1.0 \
           -out axt.ft1 -ov >>& $logName

echo "Iteration 1 P0: `getParm -in axt.ft1 -parm NDP0 -dim CUR_XDIM -fmt %.1f`" >> $logName

nmrPipe -in axt.ft1 \
| nmrPipe -fn POLY -ord 1 -nw 8 -nl $psX1 $psX2 $psX3 $psX4 \
| nmrPipe  -fn MAC -macro $NMRTXT/aph1DRMS2.M -all \
           -reg -str psX1 $psX1 psX2 $psX2 psX3 $psX3 psX4 $psX4 -var xP1 0 p0Step 0.1 \
  -out axt.ft1 -inPlace -ov >>& $logName

echo "Iteration 2 P0: `getParm -in axt.ft1 -parm NDP0 -dim CUR_XDIM -fmt %.1f`" >> $logName

nmrPipe -in axt.ft1 \
| nmrPipe -fn POLY -ord 1 -nw 8 -nl $psX1 $psX2 $psX3 $psX4 \
  -out axt.ft1 -inPlace -ov >>& $logName

ref1D.tcl -noskip -in axt.ft1 -tab refPPM.tab -ref $ref0PPM -x1 $refX1 -xn $refXN -max >>& $logName

set nInfo = (`readROI -ndim 1 -in axt.ft1 -out None -silent -x X_AXIS $noiseX1 $noiseXN -vec vRMS`    1.0)
set sInfo = (`readROI -ndim 1 -in axt.ft1 -out None -silent -x X_AXIS $normX1  $normXN  -vec vMaxAbs` 1.0)

nmrPipe -in axt.ft1 \
| nmrPipe -fn EXT -x1 $extX1 -xn $extXN -sw \
  -out $outName -ov

nmrPipe -in axt.ft1 \
| nmrPipe -fn MULT -c 100.0 \
| nmrPipe -fn MULT -c $sInfo[1] -inv \
  -out axt.ft1 -ov -inPlace

nmrPipe -in $outName \
| nmrPipe -fn MULT -c 100.0 \
| nmrPipe -fn MULT -c $sInfo[1] -inv \
  -out $outName -ov -inPlace

echo "" >> $logName
nmrPrintf "Noise RMS:     %.3e\n" $nInfo[1] >> $logName
nmrPrintf "Normalization: %.3e\n" $sInfo[1] >> $logName
echo "" >> $logName

showhdr -verb $outName >> $logName

echo "" >> $logName
