#!/bin/csh

set inName    = (`getArgDD $argv -in   _inName_  spec.fid`)
set outName   = (`getArgDD $argv -out  _outName_ spec.ft2`)
set specType  = (`getArgDD $argv -type _type_    NMR`)
set hi        = (`getArgDD $argv -hi   _hi_      2.5@`)

set graphFlag = (`getLogArgDD $argv -graph     -nograph     _graphFlag_ 1 0`)
set drawPlane = (`getLogArgDD $argv -drawPlane -nodrawPlane _drawPlane_ 1 0`)

if (`string csame $specType HN`) then
   set xSOL = (`getArgDD $argv -xSOL _xSOL_ SOL`)
else
   set xSOL = (`getArgDD $argv -xSOL _xSOL_ None`)
endif

if (`string csame $xSOL None`) then
   set xSOL = NULL
endif
 
set xAPOD = (`getArgDD $argv -xAPOD _xAPOD_   SP`)
set xQ1   = (`getArgDD $argv -xQ1   _xQ1_   0.05`)
set xQ2   = (`getArgDD $argv -xQ2   _xQ2_   0.95`)
set xQ3   = (`getArgDD $argv -xQ3   _xQ3_   2`)
set xELB  = (`getArgDD $argv -xELB  _xELB_  0.0`)
set xGLB  = (`getArgDD $argv -xGLB  _xGLB_  0.0`)
set xGOFF = (`getArgDD $argv -xGOFF _xGOFF_ 0.0`)
set xC1   = (`getArgDD $argv -xC1   _xC1_   1.0`)
set xP0   = 0.0
set xP1   = 0.0

set xZFARG   = (`getArgDD $argv -xZFARG   _xZFARG_   zf=1,auto`)
set xEXTX1   = (`getArgDD $argv -xEXTX1   _xEXTX1_   0%`)
set xEXTXN   = (`getArgDD $argv -xEXTXN   _xEXTXN_   100%`)
set xBASEARG = (`getArgDD $argv -xBASEARG _xBASEARG_ POLY,auto`)

if (`string csame $xBASEARG Auto`) then
   set xBASEARG = POLY,auto
endif

if (`string csame $xC1 Auto`) then
   set v = (`printf "%f" $xP1`)
   set i = (`IMATH "abs($v) < 10.0"`)

   if ($i == 0) then
      set xC1 = 1.0
   else
      set xC1 = 0.5
   endif

   set xC1 = 1.0
endif

set yAPOD = (`getArgDD $argv -yAPOD _yAPOD_   SP`)
set yQ1   = (`getArgDD $argv -yQ1   _yQ1_   0.05`)
set yQ2   = (`getArgDD $argv -yQ2   _yQ2_   0.95`)
set yQ3   = (`getArgDD $argv -yQ3   _yQ3_   2`)
set yELB  = (`getArgDD $argv -yELB  _yELB_  0.0`)
set yGLB  = (`getArgDD $argv -yGLB  _yGLB_  0.0`)
set yGOFF = (`getArgDD $argv -yGOFF _yGOFF_ 0.0`)
set yC1   = (`getArgDD $argv -yC1   _yC1_   1.0`)
set yP0   = 0.0
set yP1   = 0.0

set yZFARG   = (`getArgDD $argv -yZFARG   _yZFARG_   zf=1,auto`)
set yFTARG   = (`getArgDD $argv -yFTARG   _yFTARG_   None`)
set yBASEARG = (`getArgDD $argv -yBASEARG _yBASEARG_ NULL`)

if (`string csame $yC1 Auto`) then
   set v = (`printf "%f" $yP1`)
   set i = (`IMATH "abs($v) < 10.0"`)

   if ($i == 0) then
      set yC1 = 1.0
   else
      set yC1 = 0.5
   endif

   set yC1 = 1.0
endif

if (`string csame $yBASEARG Auto`) then
   set v = (`printf "%f" $yP1`)
   set i = (`IMATH "abs($v) > 10.0 && abs($v-180.0) > 10.0"`)

   if ($i == 0) then
      set yBASEARG = NULL
   else
      set yBASEARG = POLY,auto,ord=0
   endif
endif

set xSOLInfo  = (`expandArg0 $xSOL`)
set xZFInfo   = (`expandArg  $xZFARG`)
set xBaseInfo = (`expandArg0 $xBASEARG`)

set yZFInfo   = (`expandArg  $yZFARG`)
set yFTInfo   = (`expandArg  $yFTARG`)
set yBaseInfo = (`expandArg0 $yBASEARG`)

set xP0F = (`nmrPrintf %.2f $xP0`)
set xP1F = (`nmrPrintf %.2f $xP1`)
set yP0F = (`nmrPrintf %.2f $yP0`)
set yP1F = (`nmrPrintf %.2f $yP1`)

nmrPrintf "%s" "#\!/bin/csh\n"
nmrPrintf "%s" "\n"

nmrPrintf "%s" "nmrPipe -in $inName \\\n"
nmrPrintf "%s" "| nmrPipe -fn $xSOLInfo \\\n"
nmrPrintf "%s" "| nmrPipe -fn APOD -qName $xAPOD -q1 $xQ1 -q2 $xQ2 -q3 $xQ3 -elb $xELB -glb $xGLB -goff $xGOFF -c $xC1 \\\n"
nmrPrintf "%s" "| nmrPipe -fn ZF $xZFInfo \\\n"
nmrPrintf "%s" "| nmrPipe -fn FT -verb \\\n"
nmrPrintf "%s" "| nmrPipe -fn EXT -x1 $xEXTX1 -xn $xEXTXN -sw \\\n"
nmrPrintf "%s" "| nmrPipe -fn TP \\\n"
nmrPrintf "%s" "| nmrPipe -fn APOD -qName $yAPOD -q1 $yQ1 -q2 $yQ2 -q3 $yQ3 -elb $yELB -glb $yGLB -goff $yGOFF -c $yC1 \\\n"
nmrPrintf "%s" "| nmrPipe -fn ZF $yZFInfo \\\n"
nmrPrintf "%s" "| nmrPipe -fn FT $yFTInfo -verb \\\n"
nmrPrintf "%s" "| nmrPipe -fn MC \\\n"
nmrPrintf "%s" "| nmrPipe -fn $yBaseInfo \\\n"
nmrPrintf "%s" "| nmrPipe -fn TP \\\n"
nmrPrintf "%s" "| nmrPipe -fn $xBaseInfo -verb \\\n"
nmrPrintf "%s" "  -out $outName -ov\n"

nmrPrintf "%s" "\n"
nmrPrintf "%s" "# proj2D.tcl -in $outName -axis -sum\n"
nmrPrintf "%s" "#\n"
nmrPrintf "%s" "# set hiInfo = ("'`'"getSpecHi.tcl -in $outName $* -hi $hi"'`'")\n"
nmrPrintf "%s" "# nmrDraw -in $outName "'$'"hiInfo "'`'"getGeom -west -xAdj 100 -yAdj 200"'`'" &\n"
nmrPrintf "%s" "# sleep 3\n"
