#!/bin/csh

#
# Wrapper script for executing vj2pipe.tcl Delaglio May 19 - July 18 2011
#  Handles setting of working directory.
#  Handles use of output subdirectory such as "pipe".
#  Handles execution of scripts created by vj2pipe.tcl

#
# ET 27 JUL 2011 to handle -noexec flag
# FD  1 NOV 2012 add ist
# FD 21 DEC 2021 cd "$origDir" etc

if (`flagLoc $argv -help`) then
   vj2pipe.tcl -wrapper $*
   exit 0
endif

if (`flagLoc $argv -info`) then
   vj2pipe.tcl -wrapper $*
   exit 0
endif

if (`flagLoc $argv -sinfo`) then
   vj2pipe.tcl -wrapper $*
   exit 0
endif

set cdDir       = (`getArgD $argv -cd      .`) 
set pipeDir     = (`getArgD $argv -pipeDir .`)
set fidName     = (`getArgD $argv -vjfid   fid`)
set procparName = (`getArgD $argv -procpar procpar`)
set argList     = (`getListArgDN $argv -args -- None`)

set fidNameOrig     = $fidName
set procparNameOrig = $procparName

set origDir  = (`pwd`)
set fullDir  = "$origDir"
set vjPath   = "."
set pArgs    = ""
set cdCount  = 0
set vFlag    = 1

if (`flagLoc $argv -verb`) then
   set vFlag = 1
endif

if (`flagLoc $argv -noverb`) then
   set vFlag = 0
endif

if (($cdDir != ".") && ($cdDir != "./")) then
   if ($vFlag) then
      echo "vj2pipe Changing Working Directory:"
      echo " $cdDir"
      echo ""
   endif

   cd $cdDir
   set fullDir = (`pwd`) 
   @ cdCount++

   set fidName     = (`appendDir $fullDir $fidName`) 
   set procparName = (`appendDir $fullDir $procparName`)
   set pArgs       = "-vjfid $fidName -procpar $procparName"

   if ($vFlag) then
      echo "vj2pipe Data and Working Directory Status:"
      echo " $fullDir"
      echo ""
   endif

   set vjPath = $fullDir
endif

if (($pipeDir != ".") && ($pipeDir != "./")) then
   if (!(-e $pipeDir)) then
      if ($vFlag) then
         echo "vj2pipe Creating Directory:"
         echo " $pipeDir"
         echo ""
      endif

      mkdir $pipeDir
      chmod a+rwx $pipeDir
   endif

   if (!(-d $pipeDir)) then
      echo "Error Finding Directory $pipeDir ... Exiting."
      cd "$origDir"
      exit 1 
   endif

   if ($cdCount == 0) then
      set fullDir = (`pwd`)

      set fidName     = (`appendDir $fullDir $fidName`)
      set procparName = (`appendDir $fullDir $procparName`)
      set pArgs       = "-vjfid $fidName -procpar $procparName"
   endif

   if ($vFlag) then
      echo "vj2pipe Changing Working Subdirectory:"
      echo " $pipeDir"
      echo ""
   endif

   cd $pipeDir
   @ cdCount++

   if ($vFlag) then
      echo "vj2pipe Working Directory Status:"
      echo " `pwd`"
    endif
endif

if ("$fidName" != "$fidNameOrig") then
   if ($vFlag) then
      echo "vj2pipe Setting fid File Name:"
      echo " $fidName"
      echo ""
   endif
endif

if (!(-e $fidName)) then
   if (-e ${fidName}.Z) then
      set fidName = ${fidName}.Z
      set pArgs   = "-vjfid $fidName -procpar $procparName"
      echo "vj2pipe Updating fid File Name:"
      echo " $fidName"
      echo ""
   else if (-e ${fidName}.gz) then
      set fidName = ${fidName}.gz
      set pArgs   = "-vjfid $fidName -procpar $procparName"
      echo "vj2pipe Updating fid File Name:"
      echo " $fidName"
      echo ""
   endif
endif

if ("$procparName" != "procparNameOrig") then
   if ($vFlag) then
      echo "vj2pipe Setting procpar File Name:"
      echo " $procparName"
      echo ""
   endif
endif

if ($cdCount != 0) then
   if ($vFlag) then
      echo "vj2pipe Directory Status:"
      echo " `pwd`"
      echo ""
   endif
endif

if (`flagLoc $argv -cleanOnly`) then
   vj_clean.com -com
   exit 0
endif

if (`flagLoc $argv -clean`) then
   vj_clean.com -com
endif

set graphFlag = 1
set progFlag  = 1
set execFlag  = 1
set graphArg  = graph
set progArg   = progress

if (`flagLoc $argv -exec`) then
   set execFlag = 1
endif

if (`flagLoc $argv -noexec`) then
   set execFlag = 0
endif

if (`flagLoc $argv -graph`) then
   set graphFlag = 1
   set graphArg  = graph
endif

if (`flagLoc $argv -nograph`) then
   set graphFlag = 0
   set graphArg  = nograph
endif

if (`flagLoc $argv -progress`) then
   set progFlag = 1
   set progArg  = progress
endif

if (`flagLoc $argv -noprogress`) then
   set progFlag = 0
   set progArg  = noprogress
endif

#
# Arguments in pArgs for -vjfid and -procpar will override later ones:

date > timestamp.vj2pipe
vj2pipe.tcl -wrapper $pArgs $* -vjPath $vjPath

if ($status != 0) then
   echo "vj2pipe Error while executing vj2pipe.tcl ... Exiting."
   cd "$origDir"
   exit 1
endif

set cList = (vj_fid.com \
             vj_ft1_user.com \
             vj_ft2_user.com vj_ft2_ist_user.com \
             vj_ft2av_user.com vj_ft2av_ist_user.com \
             vj_ft2mc_user.com vj_ft2mc_ist_user.com \
             vj_xy_user.com vj_xz_user.com vj_xa_user.com \
             vj_xyz_user.com vj_xyza_user.com vj_scroll_user.com \
             vj_xyz_ist_user.com)

foreach c ($cList)
   if (-e $c) then
      if ($vFlag) then
         echo $c -$graphArg -$progArg $argList
      endif

      if ($execFlag != 0) then
         $c -$graphArg -$progArg $argList      

         if ($status != 0) then
            echo "vj2pipe Error while executing $c ... Exiting."
            cd "$origDir"
            exit 1
         endif
      endif
   endif
end

cd "$origDir"
