#!/bin/sh
# The next line restarts using nmrWish \
exec nmrWish "$0" -- "$@"

set auto_path   "[split $env(TCLPATH) :] $auto_path"
set ARGV        [concat $argv0 $argv]
set ARGC        [llength $ARGV]

#
# specView.tcl MultiSpectrum Viewer frank.delaglio@nist.gov
#----------------------------------------------------------

proc initProgVars {} \
{
   global objInfo stdfont ARGC ARGV env xUnits yUnits xFmt yFmt

   setFonts

   set objInfo(prog,name)    "SpecView"
   set objInfo(prog,version) "2024.029.19.15"
   set objInfo(prog,title)   "$objInfo(prog,name) $objInfo(prog,version) (C) F. Delaglio 2008-2024"

   set objInfo(prog,viewerLogo)  $env(NMRTXT)/specView.gif 
   set objInfo(prog,pcaLogo)     $env(NMRTXT)/specPCA.gif
   set objInfo(prog,viewerLogo)  $env(NMRTXT)/nmrpipe_logo_blackbg.gif
   set objInfo(prog,pcaLogo)     $env(NMRTXT)/nmrpipe_logo_blackbg.gif
   set objInfo(prog,pcaFlag)     0
   set objInfo(prog,autoPCAFlag) 0
   set objInfo(prog,altPCAFlag)  0

   set xw [winfo screenwidth  .]
   set yw [winfo screenheight .]

   set objInfo(prog,screenwidth)  $xw
   set objInfo(prog,screenheight) $yw

   set xUnits  ppm
   set xFmt    %.3f

   set yUnits  ppm
   set yFmt    %.3f

   set objInfo(prog,verbose)        0
   set objInfo(prog,screenWidth)    $xw
   set objInfo(prog,screenHeight)   $yw
   set objInfo(prog,reuseFlag)      0
   set objInfo(prog,freeFlag)       0
   set objInfo(prog,pkFlag)         0
   set objInfo(prog,pkBoxFlag)      0
   set objInfo(prog,assBoxFlag)     0
   set objInfo(prog,assFlag)        0
   set objInfo(prog,textboxFlag)    0
   set objInfo(prog,nextID)         1
   set objInfo(prog,imageID)        0
   set objInfo(prog,winCount)       0 
   set objInfo(prog,winList)        ""
   set objInfo(prog,freeIDList)     ""
   set objInfo(prog,labelItemList)  ""
   set objInfo(prog,initialZoomID)  0

   set objInfo(prog,pcaViewer)         specViewPCA.tcl
   set objInfo(prog,pcaComponentModes) {PCA L0 L1 L2 LInf Chi2 KL EMD}
   set objInfo(prog,pcaSignModes)      {"None" "Absolute Value" "Keep Positive Only" "Keep Negative Only"}
   set objInfo(prog,pcaScaleModes)     {"None" "Log" "Square" "Square Root"}
   set objInfo(prog,pcaNormModes)      {"None" "Max" "Sum" "Range" "Variance" "RMS" "Noise"}
   set objInfo(prog,pcaCenterModes)    {"None" "Subtract Average"}
   set objInfo(prog,pcaWeightModes)    {"Gauss Nearest" "Gauss All" "Linear"}

   set objInfo(prog,pcaComponentMode)  0
   set objInfo(prog,pcaSignMode)       0
   set objInfo(prog,pcaNormMode)       1
   set objInfo(prog,pcaWeightMode)     0
   set objInfo(prog,pcaScaleMode)      0
   set objInfo(prog,pcaCenterFlag)     0
   set objInfo(prog,pcaClipMin)        None
   set objInfo(prog,pcaClipMax)        None 
   set objInfo(prog,pcaThresh)         1.0
   set objInfo(prog,pcaWeightFactor)   1.0
   set objInfo(prog,pcaBinX)           1
   set objInfo(prog,pcaBinY)           1

   set objInfo(prog,pcaOtherOpensions) ""

   set objInfo(prog,pcaProc)           $objInfo(prog,pcaViewer)
   set objInfo(prog,pcaInterpMode)     bi
   set objInfo(prog,usePCA)            1
   set objInfo(prog,pcaArgs)           ""
   set objInfo(prog,pcaFirstMode)      avg
   set objInfo(prog,pcaNC)             6
   set objInfo(prog,pcaItMin)          8
   set objInfo(prog,pcaTol)            1.0e-6
   set objInfo(prog,clustFlag)         1

   set objInfo(prog,draw1DArgs)     ""
   set objInfo(prog,draw2DArgs)     ""
   set objInfo(prog,pcaTab)         0
   set objInfo(prog,pcaCoordsTab)   0
   set objInfo(prog,pcaID)          0
   set objInfo(prog,defXSize)       [expr 5*$xw/6] 
   set objInfo(prog,defYSize)       [expr 3*$yw/4]
   set objInfo(prog,listboxWidth)   Auto
   set objInfo(prog,listboxHeight)  Auto 
   set objInfo(prog,defXSizeUser)   None
   set objInfo(prog,defYSizeUser)   None
   set objInfo(prog,exportProc)     None
   set objInfo(prog,autoStartFlag)  1

   set objInfo(prog,scaleByMax)     0
   set objInfo(prog,tabViewFlag)    0
   set objInfo(prog,extendedMode)   0
   set objInfo(prog,integMode)      1
   set objInfo(prog,graphItemMode)  1
   set objInfo(prog,zoomRegMode)    1
   set objInfo(prog,charX)          10
   set objInfo(prog,charY)          16
   set objInfo(prog,titleCharX)     10
   set objInfo(prog,titleCharY)     16
   set objInfo(prog,axisCharX)      10
   set objInfo(prog,axisCharY)      16
   set objInfo(prog,maxTitleLength) 12
   set objInfo(prog,lwdraw)         1
   set objInfo(prog,lwgrid)         1
   set objInfo(prog,titleFont)      $stdfont
   set objInfo(prog,pcaFont)        $stdfont
   set objInfo(prog,canvasSelXW)    12
   set objInfo(prog,colorWindow)    None
   set objInfo(prog,updateFlag)     1
   set objInfo(prog,freezeSpectra)  0
   set objInfo(prog,freezeGraphic)  0
   set objInfo(prog,fullNameFlag)   0
   set objInfo(prog,showSizeFlag)   1
   set objInfo(prog,mouseList)      ""
   set objInfo(prog,mouseON)        0
   set objInfo(prog,B1)             0 
   set objInfo(prog,B2)             0 
   set objInfo(prog,B3)             0 
   set objInfo(prog,Shift)          0 
   set objInfo(prog,Ctrl)           0 
   set objInfo(prog,pcaTitle)       None
   set objInfo(prog,noteName)       None 
   set objInfo(prog,noteSaveNeeded) 0

   set objInfo(prog,xAxisFlag)      1
   set objInfo(prog,yAxisFlag)      1
   set objInfo(prog,xGridFlag)      0
   set objInfo(prog,yGridFlag)      0

   set objInfo(prog,button,relief) raised 

   set objInfo(prog,scoreName)     score.tab
   set objInfo(data,zoom1)         "4%  45%"
   set objInfo(data,zoom2)         "55% 95%"
   set objInfo(data,zoom3)         "25% 75%"
   set objInfo(data,zoom4)         "33% 66%"

   set objInfo(data,specCount)     1 
   set objInfo(data,pcaExtraCount) 0 
   set objInfo(data,roiCount)      0 
   set objInfo(data,inList)        ""
   set objInfo(data,refList)       ""
   set objInfo(data,tauList)       ""
   set objInfo(data,user1List)     ""
   set objInfo(data,user2List)     ""
   set objInfo(data,oldDrawList)   ""
   set objInfo(data,drawList)      ""
   set objInfo(data,prevSpecList)  ""
   set objInfo(data,prevRefList)   ""
   set objInfo(data,graphicList)   ""
   set objInfo(data,moveList)      ""
   set objInfo(data,factor)        1.1
   set objInfo(data,lw1d)          0
   set objInfo(data,lw2d)          0
   set objInfo(data,scaleID)       0
   set objInfo(data,shiftNDFlag)   0
   set objInfo(data,roiCopy)       0

   set objInfo(data,refListInit)   1 
   set objInfo(data,drawListInit)  2 

   set objInfo(data,cursorH)   None
   set objInfo(data,cursorV)   None
   set objInfo(data,window)    .
   set objInfo(data,valid)     0
   set objInfo(data,fdata)     0
   set objInfo(data,rdata)     0

   set n 1024

   if {$n < $ARGC} {set n $ARGC}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,roi$i)        0
       set objInfo(data,$i,fdata)     0
       set objInfo(data,$i,dimCount)  0
       set objInfo(data,$i,xSize)     0
       set objInfo(data,$i,ySize)     0
       set objInfo(data,$i,zSize)     0
       set objInfo(data,$i,aSize)     0
      }
 
   foreach dim {1 2 3 4 5 6 7 8} {set objInfo(data,dimList$dim) ""}
 
   set objInfo(data,hDim)      CUR_XDIM
   set objInfo(data,vDim)      CUR_YDIM
   set objInfo(data,zDim)      CUR_ZDIM
   set objInfo(data,aDim)      CUR_ADIM
   set objInfo(data,hAxis)     X_AXIS
   set objInfo(data,vAxis)     Y_AXIS 

   set objInfo(data,alignX)    None
   set objInfo(data,alignY)    None
   set objInfo(data,realFlag)  real

   set objInfo(data,specX1)       0
   set objInfo(data,specXN)       0
   set objInfo(data,specY1)       0
   set objInfo(data,specYN)       0

   set objInfo(data,specX1Val)    0
   set objInfo(data,specXNVal)    0
   set objInfo(data,specY1Val)    0
   set objInfo(data,specYNVal)    0

   set objInfo(data,zoomList)     ""
   set objInfo(data,zoomLoc)      -1

   set objInfo(data,activeSpec)   "1 (None)"

   set objInfo(data,currentSpec)      None
   set objInfo(data,currentID)        1
   set objInfo(data,currentPCAFlag)   1
   set objInfo(data,currentScale)     1.0
   set objInfo(data,currentXShift)    0
   set objInfo(data,currentYShift)    0
   set objInfo(data,currentHi)        1.0
   set objInfo(data,currentColor)     None
   set objInfo(data,trackFlag)        0
   set objInfo(data,clustCount)       0

   set objInfo(data,currentlw1d)      0 
   set objInfo(data,currentlw2d)      0 
   set objInfo(data,currentIntegFlag) 0
   set objInfo(data,currentZeroFlag)  0
   set objInfo(data,currentScore)     0 

   set objInfo(data,colorMode)        Title 
   set objInfo(data,drawMode)         cont
   set objInfo(data,listByTitle)      1
   set objInfo(data,showPeakLabel)    1
   set objInfo(data,showAssLabel)     1
   set objInfo(data,useTitleLabel)    1
   set objInfo(data,useFileLabel)     0
   set objInfo(data,useAxis)          1
   set objInfo(data,drawSpectra)      1
   set objInfo(data,useFileLabel)     0
   set objInfo(data,newWindow)        0
   set objInfo(data,difMode)          0
   set objInfo(data,pcaScaleFlag)     1

   set objInfo(data,eventX)           0
   set objInfo(data,eventY)           0
   set objInfo(main,eventX)           0
   set objInfo(main,eventY)           0
}

proc initDataArgs {} \
{
   global ARGC ARGV objInfo xUnits yUnits xFmt yFmt

    set difMode None

    getArgD $ARGV -difMode difMode

    getArgD $ARGV -xUnits xUnits
    getArgD $ARGV -yUnits yUnits

    if {![strcasecmp $xUnits pts]} {set xFmt %.0f}
    if {![strcasecmp $yUnits pts]} {set yFmt %.0f}

    getArgD $ARGV -xUnitFmt xFmt
    getArgD $ARGV -yUnitFmt yFmt

    if {![strcasecmp $difMode None]}     {set objInfo(data,difMode) 0}
    if {![strcasecmp $difMode Internal]} {set objInfo(data,difMode) 1}
    if {![strcasecmp $difMode Shaded]}   {set objInfo(data,difMode) 2}
    if {![strcasecmp $difMode Vector]}   {set objInfo(data,difMode) 3}

   if {[flagLoc $ARGV -imag]} {set objInfo(data,realFlag) imag}

   if {[flagLoc $ARGV -shift1D]}   {set objInfo(data,shiftNDFlag) 1}
   if {[flagLoc $ARGV -noshift1D]} {set objInfo(data,shiftNDFlag) 0}

   if {[flagLoc $ARGV -shift2D]}   {set objInfo(data,shiftNDFlag) 1}
   if {[flagLoc $ARGV -noshift1D]} {set objInfo(data,shiftNDFlag) 0}

   if {[flagLoc $ARGV -shiftND]}   {set objInfo(data,shiftNDFlag) 1}
   if {[flagLoc $ARGV -noshift1D]} {set objInfo(data,shiftNDFlag) 0}

#
# Input list:

   set extraList ""
   set thisDir   None

   if {[flagLoc $ARGV -pcaInit]} \
      {
       getArgD $ARGV -pcaInit thisDir
       if {[file isdirectory $thisDir/ft]} {set objInfo(data,inList) [lsort [glob -nocomplain $thisDir/ft/*]]}
      } \
   elseif {[file isdirectory save/ft]} \
      {
       foreach eStr {ft3 ft2 ft1 dat} \
          {
           set objInfo(data,inList) [lsort [glob -nocomplain save/ft/*.$eStr]]
           if {[llength $objInfo(data,inList)]} break
          }
      } \
   elseif {[file isdirectory ft]} \
      {
       foreach eStr {ft3 ft2 ft1 dat} \
          {
           set objInfo(data,inList) [lsort [glob -nocomplain ft/*.$eStr]]
           if {[llength $objInfo(data,inList)]} break
          }
      }

   getListArgD $ARGV -in objInfo(data,inList)

   if {![strcasecmp $objInfo(data,inList) None]} {set objInfo(data,inList) ""}
   if {![strcasecmp $extraList            None]} {set extraList            ""}

   if {![llength $objInfo(data,inList)]} {set objInfo(data,inList) test.ft2}

   set objInfo(data,specCount) [llength $objInfo(data,inList)]

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(data,inName$i) "None"
      }

   set objInfo(data,inName1) [lindex $objInfo(data,inList) 0]

   foreach i $extraList {lappend objInfo(data,inList) $i}

   if {[llength $objInfo(data,inList)] > 1} \
      {
       set objInfo(data,drawListInit) "1 2" 
       set objInfo(data,refListInit)  1
      } \
   else \
      {
       set objInfo(data,drawListInit) 1 
       set objInfo(data,refListInit)  "" 
      }

   getArgD $ARGV -useZoom objInfo(prog,initialZoomID)

   getListArgD $ARGV -ref  objInfo(data,refListInit)
   getListArgD $ARGV -spec objInfo(data,drawListInit)

   if {![strcasecmp $objInfo(data,refListInit)  None]} {set objInfo(data,refListInit)  ""}
   if {![strcasecmp $objInfo(data,drawListInit) None]} {set objInfo(data,drawListInit) ""}

   if {![strcasecmp $objInfo(data,refListInit)  All]}  {set objInfo(data,refListInit)  [getIota 1 1 $objInfo(data,specCount)]}
   if {![strcasecmp $objInfo(data,drawListInit) All]}  {set objInfo(data,drawListInit) [getIota 1 1 $objInfo(data,specCount)]}

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -in$i  objInfo(data,inName$i)
       }

   set n [llength $objInfo(data,inList)]

   if {$n} \
      {
       if {$n > $objInfo(data,specCount)} {set objInfo(data,specCount) $n}

       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           set objInfo(data,inName$i) [lindex $objInfo(data,inList) [expr $i - 1]] 
          }
      } \
   else \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           lappend objInfo(data,inList) $objInfo(data,inName$i)
          }
      }

#
# Initial draw list:

   set nR [llength $objInfo(data,drawListInit)]
   set nS [llength $objInfo(data,refListInit)]

   set objInfo(data,drawList) ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        set objInfo(data,drawFlag$i) 0
       }

   if {$nR || $nS} \
      {
       foreach i $objInfo(data,refListInit) \
          {
           if {$i < 1 || $i > $objInfo(data,specCount)} continue

           if {[lsearch -exact $objInfo(data,refList) $i] < 0} \
              {
               lappend objInfo(data,refList) $i
              }

           if {[lsearch -exact $objInfo(data,drawList) $i] < 0} \
              {
               lappend objInfo(data,drawList) $i
               set objInfo(data,drawFlag$i) 1
              }
          }

       foreach i $objInfo(data,drawListInit) \
          {
           if {$i < 1 || $i > $objInfo(data,specCount)} continue

           if {[lsearch -exact $objInfo(data,drawList) $i] < 0} \
              {
               lappend objInfo(data,drawList) $i
               set objInfo(data,drawFlag$i) 1
              }
          }
      }

   if {![llength $objInfo(data,drawList)]} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
           {
            if {$i > 2} \
               {
                set objInfo(data,drawFlag$i) 0
               } \
            else \
               {
                set objInfo(data,drawFlag$i) 1
                lappend objInfo(data,drawList) $i
               }
           }
       }

#
# Extract titles, and simple acq params:

   set titleList ""
   set sizeList  ""
   set labList   ""
   set swList    ""
   set x1List    ""
   set xnList    ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {![file exists $objInfo(data,inName$i)]} \
          {
           puts "Error Finding Input File $i: $objInfo(data,inName$i)"
           exit 1
          }

       rdFDATA $objInfo(data,inName$i) objInfo(data,$i,fdata) 

       set fdata $objInfo(data,$i,fdata)

       set t    [string toupper [string trim [fixTCLString [getParmStr $fdata FDTITLE NULL_DIM]] "\"' "]]
       set lab  [string toupper [string trim [fixTCLString [getParmStr $fdata NDLABEL CUR_XDIM]] "\"' "]]

       set objInfo(data,$i,dimCount) [getParmI $fdata FDDIMCOUNT NULL_DIM]
       
       set size [getParm $fdata NDSIZE  CUR_XDIM]

       if {$objInfo(data,$i,dimCount) > 1} \
          {
           set specnum [getParm $fdata NDSIZE  CUR_YDIM]
           if {$specnum == 1} {set objInfo(data,$i,dimCount) 1}
          }

       lappend sizeList $size
       lappend labList  $lab
       lappend swList   [getParm $fdata NDSW CUR_XDIM]
       lappend x1List   [pnt2spec $fdata X_AXIS 1     $xUnits]
       lappend xnList   [pnt2spec $fdata X_AXIS $size $xUnits]

       if {![llength $t]} \
          {
           set str [file tail [file dirname $objInfo(data,inName$i)]]
           set t   $objInfo(data,inName$i)

           if {[string length $str]} \
              {
               set sList [split $str _]
               set ns    [llength $sList]

               if {$ns >= 1} \
                  {
                   set t [lrange $sList 0 [expr $ns - 1]]
                  }
              }
          }

       if {$objInfo(data,pcaExtraCount) > 0} \
          {
           if {$i > $objInfo(data,specCount) - $objInfo(data,pcaExtraCount)} \
              { 
               append t "-New"
              } \
           else \
              {
               append t "-Old"
              }
          }

       lappend titleList $t
      }

   if {$objInfo(data,1,dimCount) == 1} \
      {
       set objInfo(data,zoom1) "4%       45%"
       set objInfo(data,zoom2) "9.1ppm   6.6ppm"
       set objInfo(data,zoom3) "4.6ppm   3.2ppm"
       set objInfo(data,zoom4) "2.6ppm   0.4ppm"
      } \
   else \
      {
       set objInfo(data,zoom1) "0%   50%  0%  50%"
       set objInfo(data,zoom2) "0%   50% 50% 100%"
       set objInfo(data,zoom3) "50% 100%  0% 100%"
       set objInfo(data,zoom4) "50% 100% 50% 100%"
      }

   getListArgD $ARGV -title titleList

   if {[llength $titleList] != $objInfo(data,specCount)} \
      {
       puts "Error: title list does not match spectrum count."
       exit 1
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set t [lindex $titleList [expr $i - 1]]
       set s [lindex $labList   [expr $i - 1]]
       set objInfo(data,title$i) $t
       set objInfo(data,axisLabel$i) $s
       set objInfo(data,note$i)  ""

       set titSize [string length $t]
       if {$titSize > $objInfo(prog,maxTitleLength)} {set objInfo(prog,maxTitleLength) $titSize}
      }

#
# Test for uniform data, required for PCA:

   set size [lindex $sizeList 0]
   set sw   [lindex $swList   0]
   set x1   [lindex $x1List   0]
   set xn   [lindex $x1List   0]

   if {$size == 0} {set size 1} 

   set hzPerPt [expr $sw/$size]
   if {$hzPerPt == 0.0} {set hzPerPt 1.0}

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set objInfo(data,pcaFlag$i)   1
       set objInfo(data,clusterID$i) 0

       set thisLab  [string toupper [lindex $labList  [expr $i - 1]]]
       set thisSize [lindex $sizeList [expr $i - 1]]
       set thisSW   [lindex $swList   [expr $i - 1]]
   
       if {$thisSize == 0} {set thisSize 1} 

       set thisHz [expr $thisSW/$thisSize]
       set r      [expr $thisHz/$hzPerPt]

#      if {$r < 0.99 || $r > 1.01}   {set objInfo(data,pcaFlag$i) 0}
#      if {![strmatch $thisLab *H*]} {set objInfo(data,pcaFlag$i) 0}
      }

#
# Initial params:

   getArgD $ARGV -lw2d objInfo(data,lw2d)

   set objInfo(data,lw1d) $objInfo(data,lw2d)

   getArgD $ARGV -lw1d objInfo(data,lw1d)

   getArgD $ARGV -lwdraw objInfo(prog,lwdraw)
   getArgD $ARGV -lwgrid objInfo(prog,lwgrid)

   set objInfo(data,currentlw1d) $objInfo(data,lw1d)
   set objInfo(data,currentlw2d) $objInfo(data,lw2d)

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   set iList ""

   for {set i 1} {$i <= $n} {incr i} {lappend iList $i}

   lappend iList stat

   foreach i $iList \
      {
       if {$i == 1} \
          {
           set objInfo(data,hi$i) Auto
          } \
       else \
          {
           set objInfo(data,hi$i) None  
          }

       set objInfo(data,fhi$i)    0.0
       set objInfo(data,lhi$i)    0.0
       set objInfo(data,noise$i)  0.0
       set objInfo(data,scale$i)  1.0
       set objInfo(data,roi$i)    0

       set objInfo(data,integFlag$i) 0
       set objInfo(data,zeroFlag$i)  0
       set objInfo(data,axisFlag$i)  0
       set objInfo(data,score$i)     0
       set objInfo(data,lw1d$i)      $objInfo(data,lw1d) 
       set objInfo(data,lw2d$i)      $objInfo(data,lw2d) 
      }

   readNotes
}

proc initPeakVars {} \
{
   global objInfo

   set n $objInfo(data,specCount)

   if {$n < 2} {set n 2}

   for {set i 1} {$i <= $n} {incr i} \
      {
       set objInfo(peak,tab$i)      0
       set objInfo(peak,pkCount$i)  0
      }

   set objInfo(peak,inList)       test.tab
   set objInfo(peak,smileLogName) None
   set objInfo(peak,xName)        None
   set objInfo(peak,yName)        None
   set objInfo(peak,zName)        None
   set objInfo(peak,assName)      None
   set objInfo(peak,outName)      None
   set objInfo(peak,assVar)       ASS
   set objInfo(peak,idVar)        INDEX
   set objInfo(peak,labVarList)   INDEX
   set objInfo(peak,xAssVar)      X_PPM
   set objInfo(peak,yAssVar)      Y_PPM 
   set objInfo(peak,zAssVar)      Z_PPM 
   set objInfo(peak,assTab)       0
   set objInfo(peak,assDX)        0
   set objInfo(peak,assDY)        0

   set objInfo(peak,currentID)        None
   set objInfo(peak,currentAss)       None
   set objInfo(peak,selectedAss)      None
   set objInfo(peak,currentPeakEntry) None
   set objInfo(peak,currentAssEntry)  None
   set objInfo(peak,changeCount)      0
}

proc initProgColors {} \
{
   global objInfo

   set objInfo(color,bg)            black
   set objInfo(color,grayed)        gray10
   set objInfo(color,on)            green
   set objInfo(color,shadow)        black
   set objInfo(color,label)         white
   set objInfo(color,zoom)          green
   set objInfo(color,propose)       white
   set objInfo(color,accept)        green
   set objInfo(color,cursor)        gray30
   set objInfo(color,box)           white
   set objInfo(color,arrow)         gray
   set objInfo(color,hilite)        pink 
   set objInfo(color,ass)           green
   set objInfo(color,selectedAss)   red
   set objInfo(color,peak)          gray 
   set objInfo(color,selectedPeak)  darkOrchid 
   set objInfo(color,grid)          dimGray
   set objInfo(color,neutral)       dimGray
   set objInfo(color,winBG)         gray30
   set objInfo(color,stat)          white
   set objInfo(color,statFill1)     dimGray 
   set objInfo(color,statFill2)     violet
   set objInfo(color,multiFlag)     0
}

proc initSpecColors {} \
{
   global ARGC ARGV objInfo

   if {[flagLoc $ARGV -multiColor]}   {set objInfo(color,multiFlag) 1}
   if {[flagLoc $ARGV -nomultiColor]} {set objInfo(color,multiFlag) 0}

   getArgD $ARGV -bgColor objInfo(color,bg)

   if {![strcasecmp $objInfo(color,bg) black]} \
      {
       set objInfo(color,colorList) \
        {cyan        yellow     red         darkOrchid   green     blue \
         orange      magenta   forestGreen violet pink   firebrick brown}

       set objInfo(color,grayed) gray10
      } \
   else \
      {
       set objInfo(color,colorList) \
        {blue        yellow    red         darkOrchid    darkgreen darkblue \
         orange      magenta   forestGreen violet pink   firebrick brown}

       set objInfo(color,winBG)  white
       set objInfo(color,label)  black
       set objInfo(color,shadow) None

       set objInfo(color,grayed) gray70
      }

   getArgD $ARGV -grayedColor    objInfo(color,grayed)
   getArgD $ARGV -peakColor      objInfo(color,peak)
   getArgD $ARGV -assColor       objInfo(color,ass)
   getArgD $ARGV -hiliteColor    objInfo(color,hilite)
   getArgD $ARGV -arrowColor     objInfo(color,arrow)
   getArgD $ARGV -winColor       objInfo(color,winBG)
   getArgD $ARGV -onColor        objInfo(color,on) 
   getArgD $ARGV -shadowColor    objInfo(color,shadow)
   getArgD $ARGV -labelColor     objInfo(color,label) 
   getArgD $ARGV -zoomColor      objInfo(color,zoom)
   getArgD $ARGV -cursorColor    objInfo(color,cursor)
   getArgD $ARGV -gridColor      objInfo(color,grid)
   getArgD $ARGV -boxColor       objInfo(color,box)
   getArgD $ARGV -proposeColor   objInfo(color,propose)
   getArgD $ARGV -acceptColor    objInfo(color,accept)
   getArgD $ARGV -statColor      objInfo(color,stat)
   getArgD $ARGV -statFillColor1 objInfo(color,statFill1)
   getArgD $ARGV -statFillColor2 objInfo(color,statFill2)
      
   if {![strcasecmp $objInfo(color,bg) black]} \
      { 
       set objInfo(color,colorList) \
        {cyan        yellow    red         darkOrchid    green     blue \
         orange      magenta   forestGreen violet pink   firebrick brown}
      } \
   else \
      {
       set objInfo(color,colorList) \
        {blue        yellow    red         darkOrchid    darkgreen darkblue \
         orange      magenta   forestGreen violet pink   firebrick brown}
      }

   set objInfo(color,fillColorList) None

   if {[flagLoc $ARGV -useFill]} \
      {
       set objInfo(color,fillColorList) \
        {blue  paleGreen firebrick   violet   forestGreen  cyan \
         red   orange    green       darkOrchid    magenta   yellow    pink}
      }

   getListArgD $ARGV -colors     objInfo(color,colorList)
   getListArgD $ARGV -fillColors objInfo(color,fillColorList)

   if {![llength $objInfo(color,colorList)]}     {set objInfo(color,colorList)     gray} 
   if {![llength $objInfo(color,fillColorList)]} {set objInfo(color,fillColorList) dimGray} 

   setTitleColors
}

proc setColorMode { mode } \
{
   global objInfo

   if {[strcasecmp $mode Current]} \
      {
       set objInfo(data,colorMode) $mode
      }

   setTitleColors
   updateTitleColors

   showViewAll setColorMode
   drawTitles
}

proc updateTitleColors {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$objInfo(data,pcaFlag$i)} \
          {
           .tlb itemconfigure [expr $i - 1] \
                -foreground       [safeColor $objInfo(color,color$i)] \
                -background       $objInfo(color,bg) \
                -selectforeground [safeColor $objInfo(color,color$i)] \
                -selectbackground $objInfo(color,grayed)

           .rlb itemconfigure [expr $i - 1] \
                -foreground       [safeColor $objInfo(color,color$i)] \
                -background       $objInfo(color,bg)      \
                -selectforeground [safeColor $objInfo(color,color$i)] \
                -selectbackground $objInfo(color,grayed)
          } \
       else \
          {
           .tlb itemconfigure [expr $i - 1] \
                -foreground       gray50                  \
                -background       gray30                  \
                -selectforeground white                   \
                -selectbackground gray10

           .rlb itemconfigure [expr $i - 1] \
                -foreground       gray50                  \
                -background       gray30                  \
                -selectforeground white                   \
                -selectbackground gray10
          }
     }

   foreach i $objInfo(data,drawList) {.tlb selection set [expr $i - 1]}
}

proc setTitleColors {} \
{
   global objInfo

   set nc [llength $objInfo(color,colorList)]
   set nf [llength $objInfo(color,fillColorList)]

   set mode    [string tolower $objInfo(data,colorMode)]
   set modeVal $mode

   if {[string is int $mode]} {set mode int}

   switch -exact $mode \
      {
       title -
       int   -
       last {
             set sList ""
      
             if {![strcmp $mode last]} \
                {
                 set item end
                } \
             elseif {![strcmp $mode int]} \
                {
                 set mode $modeVal
                } \
             else \
                {
                 set item $mode
                }

             for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
                {
                 set t [string toupper $objInfo(data,title$i)]

                 if {![strcmp $mode last]} \
                    {
                     set s [lindex [split $t -] end]
                    } \
                 elseif {![strcmp $mode title]} \
                    {
                     set s $t
                    } \
                 else \
                    {
                     set s [lindex [split $t -] [expr $mode - 1]]
                    }

                 set loc [lsearch -exact $sList $s]

                 if {$loc < 0} \
                    {
                     set loc [llength $sList]
                     lappend sList $s
                    }

                 set j [expr $loc % $nc]
                 set k [expr $loc % $nf]
 
                 set objInfo(color,color$i)     [lindex $objInfo(color,colorList) $j]
                 set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $k]

                 set objInfo(color,pcaColor$i) $objInfo(color,color$i)
                }
            }

       file {
             for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
                {
                 set s [file root [file tail $objInfo(data,inName$i)]]
                 set f [lindex [split $s _] end]
                 set j [hashCode $f $nc]
                 set k [hashCode $f $nf]

                 set objInfo(color,color$i)     [lindex $objInfo(color,colorList) $j]
                 set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $k]

                 set objInfo(color,pcaColor$i) $objInfo(color,color$i)
                }
            }

       number {
               for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
                   {
                    set j [expr ($i - 1) % $nc]
                    set k [expr ($i - 1) % $nf] 

                    set objInfo(color,color$i)     [lindex $objInfo(color,colorList)     $j]
                    set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $k]

                    set objInfo(color,pcaColor$i) $objInfo(color,color$i)
                   }
              }

       cluster {
                for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
                    {
                     if {[info exists objInfo(data,clusterID$i)]} \
                        {
                         set cID $objInfo(data,clusterID$i)
                        } \
                     else \
                        {
                         set cID 0
                        }

                     set j [expr ($cID - 1) % $nc]
                     set k [expr ($cID - 1) % $nf]

                     set objInfo(color,color$i)     [lindex $objInfo(color,colorList)     $j]
                     set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $k]
   
                     set objInfo(color,pcaColor$i) $objInfo(color,color$i)
                    }
               }

       default {
                for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
                    {
                     set j [expr ($i - 1) % $nc]
                     set k [expr ($i - 1) % $nf]

                     set objInfo(color,color$i)     [lindex $objInfo(color,colorList)     $j]
                     set objInfo(color,fillColor$i) [lindex $objInfo(color,fillColorList) $k]

                     set objInfo(color,pcaColor$i) $objInfo(color,color$i)
                    }
               }
      }
}

proc showArgHelp {} \
{
   global ARGC ARGV

   if {![flagLoc $ARGV -help]} return

   set usePCA 1
   set exportProc None

   if {[flagLoc $ARGV -altPCA]}   {set usePCA 1}

   if {[flagLoc $ARGV -wampas]}   {set usePCA 1}
   if {[flagLoc $ARGV -nowampas]} {set usePCA 0}

   if {[flagLoc $ARGV -usePCA]}   {set usePCA 1}
   if {[flagLoc $ARGV -nousePCA]} {set usePCA 0}

   if {!$usePCA} {set objInfo(prog,exportProc) None}

   getArgD $ARGV -export exportProc

   puts "MultiSpectrum Viewer:"
   puts " -in      nameList  Spectrum Name List (All 1D or All 2D)."
   puts " -title   tList     Corresponding List of Titles."
   puts " -fhi     hi        1D Min Height."
   puts " -lhi     hi        1D Max Height." 
   puts " -hi      hi        2D First Contour Height."
   puts " -xUnits  xUnits    X-Axis Units (ppm % hz pts)."
   puts " -yUnits  yUnits    Y-Axis Units (2D Data, ppm % hz pts)."
   puts " -export  eName     Region Export Proc (Usually: -export titrView)."
   puts " -verb              Verbose Mode ON."
   puts " -noverb            Verbose Mode OFF (Default)."
   puts " -useFullName       Use File File Name for Title Labels."
   puts " -extended          Extended Mode (Annotation, Scoring, Etc.)"
   puts " -captureAll        Snapshot Captures Images for All Spectra."
   puts " -autoCapture       Capture Images for All Spectra and Exit."
   puts "Initial View:"
   puts " -spec    sIDList   Spectrum Indices for Initial Display  (All None)."
   puts " -ref     rIDList   Reference Indices for Initial Display (All None)."
   puts " -useZoom zoomID    Initial Zoom Region (1 2 3 4) As Defined by -zoom1 ..."
   puts " -difMode mode      Keyword: Shaded Internal Vector None."
   puts "Peak Table Opions:"
   puts " -peak              Peak Mode ON (Label Peaks)."
   puts " -inTab   tabList   List of Peak Table Files for  Spectra (Also -tab)."
   puts " -outTab  tabName   Peak Table Output for Assignment Mode."
   puts " -pkLab   varList   Peak Table Variables for Labels (Default: INDEX)."
#  puts " -pkBox             Draw Peak Limit Boxes."
   puts " -ass     assName   Assignment Table."
   puts " -assVar  assVar    Assignment Table Variable with Assignment Info."
   puts " -assBox            Draw Boxes Around Assignment Labels."
   puts " -xAssVar xVar      Assignment Table Variable Corresponding to X-Axis." 
   puts " -yAssVar yVar      Assignment Table Variable Corresponding to Y-Axis." 
   puts " -assDX   assDX     Horizontal Offset for Assignment Label, Pixels."
   puts " -assDY   assDY     Vertical Offset for Assignment Label, Pixels."
#  puts " -zAssVar zVar      Assignment Table Variable Corresponding to Z-Axis." 
   puts " -smile   sName     SMILE Verbose-Mode Log (For Displaying SMILE Signals)."
   puts "Scaling and Shifting:"
   puts " -scaleByMax        Scale Function Scales by Maximum."
   puts " -noscaleByMax      Scale Function Scales by RMSD (Default)."
   puts " -shiftND           Use Spectral Shifting Options."
   puts " -noshiftND         No Spectral Shifting Options (Default)."
   puts "Listing Mode:"
   puts " -listByTitle       List by Spectrum Title (Default)."
   puts " -listByFile        List by Spectrum File Name."
   puts " -showSize          Show Spectrum Size in Title (Default)."
   puts " -noshowSize        Do Not Show Spectrum Size in Title."
   puts "Pre-Defined Zoom Regions:"
   puts " -zoom1  x1 xn ...  Zoom Region Limits for Region 1."
   puts " -zoom2  x1 xn ...  Zoom Region Limits for Region 2."
   puts " -zoom3  x1 xn ...  Zoom Region Limits for Region 3."
   puts " -zoom4  x1 xn ...  Zoom Region Limits for Region 4."
   puts "Principal Component Analysis (PCA) Options:"
   puts " -usePCA            Enable PCA Options (Default)."
   puts " -noPCA             No PCA Options."
   puts " -pcaFirst fMode    PCA Starting Point: avg maxvec maxvar first."
   puts " -pcaItMin itMin    Minimum Allowed PCA Iterations (Default 8)."
   puts " -pcaTol   tol      PCA Convergence Tolerance (Default 1.0e-6)."
   puts " -pcaNC    nc       Number of Principal Components to Calculate."
   puts " -scale             Scale PCA Data by Variance (Default)."
   puts " -noscale           No Data Scaling."
   puts " -ftInterp          Interpolation by Fourier Transform."
   puts " -liInterp          Linear Interpolation (Default)."
   puts " -pcaInit  pcaDir   Start with Existing PCA Result (pca001 etc.)"
   puts " -pcaArgs  args --  Arguments for specViewPCA.tcl (Terminate by --)."
   puts " -autoPCA           Calculate PCA Results and Exit."
   puts "Alternative Component Analysis Options:"
   puts " -altPCA            Enable Alternative Component Options."
   puts " -noaltPCA          No Alternative Component Analysis Options."
   puts "Spectral Graphics Arguments:"
   puts " -drawMode mode     2D Drawing Mode Keyword: cont pix dot."
   puts " -lw1d     lw1d     Linewidth for 1D Drawing, Pixels."
   puts " -lw2d     lw2d     Linewidth for 2D Drawing, Pixels."
   puts " -lwdraw   lwdraw   Linewidth for Box, Etc. Drawing, Pixels."
   puts " -lwgrid   lwgrid   Linewidth for Grid/Zero-Line Drawing, Pixels."
   puts " -xSize    xPix     Default X-Axis Window Size"
   puts " -ySize    yPix     Default Y-Axis Window Size"
   puts "Axis Drawing:"
   puts " -useXAxis          Draw X-Axis Labels (Default)."
   puts " -nouseXAxis        Do Not Draw X-Axis Labels."
   puts " -useYAxis          Draw Y-Axis Labels (Default for 2D)."
   puts " -nouseYAxis        Do Not Draw Y-Axis Labels."
   puts " -useAxis           Same as: -useXAxis -useYAxis."
   puts " -nouseAxis         Same as: -nouseXAxis -nouseYAxis."
   puts "Grid Drawing:"
   puts " -noxGrid           Do Not Draw X-Axis Grid (Default)."
   puts " -xGrid             Draw X-Axis Grid."
   puts " -noyGrid           Do Not Draw Y-Axis Labels. (Default)"
   puts " -yGrid             Draw Y-Axis Grid."
   puts " -nogrid            Same as: -noxGrid -noyGrid."
   puts " -grid              Same as: -xGrid -yGrid."
   puts "Arguments for drawROI (See drawROI -help in nmrWish):"
   puts " -draw1DArgs  args --     Arguments for 1D Drawing (Terminate by --)."
   puts " -draw2DArgs  args --     Arguments for 2D Drawing (Terminate by --)."
   puts "Font and Graphics Sizes:"
   puts " -charX        cxPix      Button Character Width, Pixels."
   puts " -charY        cyPix      Button Character Height, Pixels."
   puts " -titleFont    fontName   Title Font Name."
   puts " -pcaFont      fontName   PCA Scatterplot Font Name."
   puts " -titleCharX   cxPix      Title Character Width, Pixels."
   puts " -titleCharY   cyPix      Title Character Height, Pixels."
   puts " -axisCharX    cxPix      Axis Character Width, Pixels."
   puts " -axisCharY    cyPix      Axis Character Height, Pixels."
   puts " -lbw          lbChars    Listbox Width, Characters."
   puts " -lbh          lbLines    Listbox Height, Lines."
   puts " -leftMargin   lPix       Left Margin Size, Pixels."
   puts " -rightMargin  rPix       Right Margin Size, Pixels."
   puts " -bottomMargin bPix       Bottom Margin Size, Pixels."
   puts " -topMargin    tPix       Top Margin Size, Pixels."
   puts " -winXAdj      xAdj       Adjustment to Window Width, Pixels."
   puts " -winYAdj      yAdj       Adjustment to Window Height, Pixels."
   puts "Spectrum Color Arguments:"
   puts " -colors       colorList  Color List for Spectra 1 2 ..."
   puts " -color1       color1     Color for Spectrum 1."
   puts " -color2       color2     Color for Spectrum 2 ..."
   puts " -fillColors   colorList  List of Colors for Optional 1D Fill."
   puts " -useFill                 Use 1D Fill Drawing Mode."
   puts " -multiColor              Draw 2D Data with Multi-Color Contours."
   puts " -colorMode    cMode      Mode Keyword: Title File Number 1 2 ... Last."
   puts "Other Color Arguments:"
   puts " -peakColor    pkColor    Peak Label Color"
   puts " -assColor     assColor   Assignment Label Color"
   puts " -bgColor      bgColor    1D Graphic Background Color"
   puts " -grayedColor  gColor     1D Graphic Shading Color (with bgColor)."
   puts " -winColor     winColor   Window Background Color"
   puts " -labelColor   lColor     Label Color"
   puts " -shadowColor  sColor     Label Shadow Color"
   puts " -zoomColor    zColor     Zoom Box Color"
   puts " -cursorColor  cColor     Cursor Color"
   puts " -gridColor    gColor     Grid Color"
   puts " -boxColor     bColor     Outline Box Color"
   puts " -proposeColor pColor     Selected Region Box Color"
   puts " -acceptColor  aColor     Accepted Region Box Color"
   puts "Example Environment Variables for Spectral Axis Drawing:"
   puts " setenv NMRPIPE_BIGFONT   \"-adobe-helvetica-bold-r-*-*-*-240-*-*-*-*-*-*\""
   puts " setenv NMRPIPE_SMALLFONT \"-adobe-helvetica-bold-r-*-*-*-240-*-*-*-*-*-*\""
   puts " setenv NMRPIPE_AXISFONT  \"-adobe-helvetica-bold-r-*-*-*-240-*-*-*-*-*-*\""

   if {[strcasecmp $exportProc None]} \
      {
       catch "$exportProc -- -help"
      }

   exit 1
}

proc initArgArgs {} \
{
   global objInfo ARGC ARGV

   set ARGV [getListArgDT $ARGV -pcaArgs    objInfo(prog,pcaArgs)]
   set ARGV [getListArgDT $ARGV -draw1DArgs objInfo(prog,draw1DArgs)]
   set ARGV [getListArgDT $ARGV -draw2DArgs objInfo(prog,draw2DArgs)]
   set ARGC [llength $ARGV]
}

proc initProgArgs {} \
{
   global objInfo ARGC ARGV

   if {[flagLoc $ARGV -wampas]}   {set objInfo(prog,usePCA) 1}
   if {[flagLoc $ARGV -nowampas]} {set objInfo(prog,usePCA) 0}

   if {[flagLoc $ARGV -usePCA]}   {set objInfo(prog,usePCA) 1}
   if {[flagLoc $ARGV -nousePCA]} {set objInfo(prog,usePCA) 0}

   if {[flagLoc $ARGV -altPCA]}   {set objInfo(prog,altPCAFlag) 1}
   if {[flagLoc $ARGV -noaltPCA]} {set objInfo(prog,altPCAFlag) 0}

   if {[flagLoc $ARGV -autoPCA]}   {set objInfo(prog,autoPCAFlag) 1}
   if {[flagLoc $ARGV -noautoPCA]} {set objInfo(prog,autoPCAFlag) 0}

   if {[flagLoc $ARGV -clust]}    {set objInfo(prog,clustFlag) 1}
   if {[flagLoc $ARGV -noclust]}  {set objInfo(prog,clustFlag) 0}

   if {[flagLoc ARGV -liInterp]}  {set objInfo(prog,pcaInterpMode) bi}
   if {[flagLoc ARGV -ftInterp]}  {set objInfo(prog,pcaInterpMode) ft}

   if {!$objInfo(prog,usePCA)}    {set objInfo(prog,exportProc) None}

   getArgD $ARGV -export   objInfo(prog,exportProc)
   getArgD $ARGV -pcaFirst objInfo(prog,pcaFirstMode)
   getArgD $ARGV -pcaItMin objInfo(prog,pcaItMin)
   getArgD $ARGV -pcaNC    objInfo(prog,pcaNC)
   getArgD $ARGV -pcaTol   objInfo(prog,pcaTol)

   if {[flagLoc $ARGV -verb]}   {set objInfo(prog,verbose) 1}
   if {[flagLoc $ARGV -noverb]} {set objInfo(prog,verbose) 0}

   if {[flagLoc $ARGV -scaleByMax]}   {set objInfo(prog,scaleByMax) 1}
   if {[flagLoc $ARGV -noscaleByMax]} {set objInfo(prog,scaleByMax) 0}

   if {[flagLoc $ARGV -showSize]}     {set objInfo(prog,showSizeFlag) 1}
   if {[flagLoc $ARGV -noshowSize]}   {set objInfo(prog,showSizeFlag) 0}

   if {[flagLoc $ARGV -textbox]}   {set objInfo(prog,textboxFlag) 1}
   if {[flagLoc $ARGV -notextbox]} {set objInfo(prog,textboxFlag) 0}

   if {[flagLoc $ARGV -nouseAxis]} \
      {
       objInfo(prog,xAxisFlag) 0
       objInfo(prog,yAxisFlag) 0
      }

   if {[flagLoc $ARGV -useAxis]} \
      {
       objInfo(prog,xAxisFlag) 1
       objInfo(prog,yAxisFlag) 1
      }

   if {[flagLoc $ARGV -useXAxis]}   {set objInfo(prog,xAxisFlag) 1}
   if {[flagLoc $ARGV -nouseXAxis]} {set objInfo(prog,xAxisFlag) 0}

   if {[flagLoc $ARGV -useYAxis]}   {set objInfo(prog,yAxisFlag) 1}
   if {[flagLoc $ARGV -nouseYAxis]} {set objInfo(prog,yAxisFlag) 0}

   if {[flagLoc $ARGV -grid]} \
      {
       objInfo(prog,xGridFlag) 1
       objInfo(prog,yGridFlag) 1
      }

   if {[flagLoc $ARGV -nogrid]} \
      {
       objInfo(prog,xGridFlag) 0
       objInfo(prog,yGridFlag) 0
      }

   if {[flagLoc $ARGV -xGrid]}   {set objInfo(prog,xGridFlag) 1}
   if {[flagLoc $ARGV -noxGrid]} {set objInfo(prog,xGridFlag) 0}

   if {[flagLoc $ARGV -yGrid]}   {set objInfo(prog,yGridFlag) 1}
   if {[flagLoc $ARGV -noyGrid]} {set objInfo(prog,yGridFlag) 0}

   if {[flagLoc $ARGV -free]}   {set objInfo(prog,freeFlag) 1}
   if {[flagLoc $ARGV -nofree]} {set objInfo(prog,freeFlag) 0}

   if {[flagLoc $ARGV -start]}   {set objInfo(prog,autoStartFlag) 1}
   if {[flagLoc $ARGV -nostart]} {set objInfo(prog,autoStartFlag) 0}

   if {[flagLoc $ARGV -peak]}   {set objInfo(prog,pkFlag) 1}
   if {[flagLoc $ARGV -nopeak]} {set objInfo(prog,pkFlag) 0}

   if {[flagLoc $ARGV -pkBox]}    {set objInfo(prog,pkBoxFlag) 1}
   if {[flagLoc $ARGV -nopkBox]}  {set objInfo(prog,pkBoxFlag) 0}

   if {[flagLoc $ARGV -assBox]}   {set objInfo(prog,assBoxFlag) 1}
   if {[flagLoc $ARGV -noassBox]} {set objInfo(prog,assBoxFlag) 0}

   if {[flagLoc $ARGV -useFullName]}   {set objInfo(prog,fullNameFlag) 1}
   if {[flagLoc $ARGV -nouseFullName]} {set objInfo(prog,fullNameFlag) 0}

   if {[flagLoc $ARGV -extended]}   {set objInfo(prog,extendedMode) 1}
   if {[flagLoc $ARGV -noextended]} {set objInfo(prog,extendedMode) 0}

   if {[flagLoc $ARGV -tabView]}    {set objInfo(prog,tabViewFlag) 1}
   if {[flagLoc $ARGV -notabView]}  {set objInfo(prog,tabViewFlag) 0}

   if {[flagLoc $ARGV -scale]}   {set objInfo(data,pcaScaleFlag) 1}
   if {[flagLoc $ARGV -noscale]} {set objInfo(data,pcaScaleFlag) 0}

   if {[flagLoc $ARGV -listByTitle]} \
      {
       set objInfo(data,listByTitle)  1
      }

   if {[flagLoc $ARGV -listByFile]} \
      {
       set objInfo(data,listByTitle)   0
       set objInfo(data,useFileLabel)  1
      }

   if {[flagLoc $ARGV -align]} \
      {
       set objInfo(data,alignX) X
       set objInfo(data,alignY) Y
      }

   getListArgD $ARGV -tab   objInfo(peak,inList)
   getListArgD $ARGV -inTab objInfo(peak,inList)
   getListArgD $ARGV -pkLab objInfo(peak,labVarList)

   getArgD $ARGV -ass     objInfo(peak,assName)
   getArgD $ARGV -outTab  objInfo(peak,outName)
   getArgD $ARGV -assVar  objInfo(peak,assVar)
   getArgD $ARGV -assDX   objInfo(peak,assDX)
   getArgD $ARGV -assDY   objInfo(peak,assDY)
   getArgD $ARGV -xPkVar  objInfo(peak,xName)
   getArgD $ARGV -yPkVar  objInfo(peak,yName)
   getArgD $ARGV -zPkVar  objInfo(peak,zName)
   getArgD $ARGV -xAssVar objInfo(peak,xAssVar)
   getArgD $ARGV -yAssVar objInfo(peak,yAssVar)
   getArgD $ARGV -zAssVar objInfo(peak,zAssVar)
   getArgD $ARGV -smile   objInfo(peak,smileLogName)

   getArgD $ARGV -xSize   objInfo(prog,defXSizeUser)
   getArgD $ARGV -ySize   objInfo(prog,defYSizeUser)

   initSpecColors

   getArgD $ARGV -charX objInfo(prog,charX)
   getArgD $ARGV -charY objInfo(prog,charY)

   getArgD $ARGV -titleCharX objInfo(prog,titleCharX)
   getArgD $ARGV -titleCharY objInfo(prog,titleCharY)

   getArgD $ARGV -titleFont objInfo(prog,titleFont)
   getArgD $ARGV -pcaFont   objInfo(prog,pcaFont)

   getArgD $ARGV -axisCharX objInfo(prog,axisCharX)
   getArgD $ARGV -axisCharY objInfo(prog,axisCharY)

   getArgD $ARGV -lbw    objInfo(prog,listboxWidth)
   getArgD $ARGV -lbh    objInfo(prog,listboxHeight)

   if {![strcasecmp $objInfo(prog,listboxWidth) Auto]} {set objInfo(prog,listboxWidth) 30}

   if {![strcasecmp $objInfo(prog,listboxHeight) Auto]} \
      {
       set nMax [expr int($objInfo(prog,screenheight)/$objInfo(prog,charY)) - 35]

       if {$nMax < 4}  {set nMax 4}
       if {$nMax > 24} {set nMax 24}

       set objInfo(prog,listboxHeight) $nMax 
      }

   getListArgD $ARGV -zoom1 objInfo(data,zoom1)
   getListArgD $ARGV -zoom2 objInfo(data,zoom2)
   getListArgD $ARGV -zoom3 objInfo(data,zoom3)
   getListArgD $ARGV -zoom4 objInfo(data,zoom4)

   getArgD $ARGV -drawMode  objInfo(data,drawMode)
   getArgD $ARGV -colorMode objInfo(data,colorMode)
   getArgD $ARGV -color1    objInfo(color,color1)
   getArgD $ARGV -color2    objInfo(color,color2)

   set hiList ""

   getListArgD $ARGV -hi hiList

   getArgD $ARGV -hi   objInfo(data,hi1)
   getArgD $ARGV -fhi  objInfo(data,fhi1)
   getArgD $ARGV -lhi  objInfo(data,lhi1)

   if {![strcasecmp objInfo(data,hi1) AutoAll]} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           set objInfo(data,hi$i) Auto
          } 
      }

   if {[llength $hiList] == $objInfo(data,specCount)} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           set objInfo(data,hi$i) [lindex $hiList [expr $i - 1]]
          } 
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        getArgD $ARGV -color$i objInfo(color,color$i)
        getArgD $ARGV -color$i objInfo(color,color$i)

        set objInfo(data,fhi$i)  $objInfo(data,fhi1)
        set objInfo(data,lhi$i)  $objInfo(data,lhi1)

        getArgD $ARGV -hi$i  objInfo(data,hi$i)
        getArgD $ARGV -fhi$i objInfo(data,fhi$i)
        getArgD $ARGV -lhi$i objInfo(data,lhi$i)
       }

   if {[strcasecmp $objInfo(prog,exportProc) None]} \
      {
       append objInfo(prog,name) " ($objInfo(prog,exportProc))"
       set objInfo(prog,title)   "$objInfo(prog,name) $objInfo(prog,version)"
      }
}

proc freeThisID { idList } \
{
   global objInfo

   foreach id $idList \
      {
       lappend objInfo(prog,freeIDList) $id
      }
}

proc getNextID {} \
{
   global objInfo

   if {$objInfo(prog,reuseFlag) && [llength $objInfo(prog,freeIDList)]} \
      {
       set id [lindex $objInfo(prog,freeIDList) 0]
       set objInfo(prog,freeIDList) [lrange $objInfo(prog,freeIDList) 1 end]

       showVerb "Reusing ID $id."

       return $id
      }

   set  id $objInfo(prog,nextID)
   incr objInfo(prog,nextID)

   return $id
}

proc initPeakTable {} \
{
   global objInfo

   if {!$objInfo(prog,pkFlag)} return

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      { 
       set tabName [lindex $objInfo(peak,inList) [expr $i - 1]]

       if {$objInfo(peak,tab$i)} continue

       if {![strcasecmp $tabName None]} continue
       if {![string length $tabName]}   continue

       if {![file exists $tabName]} continue 

       showVerb "Reading Peak Table $tabName ..."

       set error [catch {set objInfo(peak,tab$i) [gdbRead table -in $tabName -name "Peak table $i"]}]
       if {![info exists objInfo(peak,tab$i)]} {set objInfo(peak,tab$i) 0}

       if {$error || !$objInfo(peak,tab$i)} \
          {
           showError $objInfo(prog,name) "Error Reading Peak Table $tabName"
           set objInfo(peak,pkCount$i) 0
           continue
          }
       
       set objInfo(peak,pkCount$i) [gdbGetInfo $objInfo(peak,tab$i) -parms entryCount]
      }
}

proc initAssTable {} \
{
   global objInfo

   set objInfo(prog,assFlag) 0

   if {![strcasecmp $objInfo(peak,assName) None]} return

   if {![file exists $objInfo(peak,assName)]} \
      {
       showError $objInfo(prog,name) "Error Opening Assignment Table $objInfo(peak,assName)"
       return
      }

   showVerb "Reading Assignment Table $objInfo(peak,assName) ..."

   set error [catch {set objInfo(peak,assTab) [gdbRead table -in $objInfo(peak,assName)]}]
   if {![info exists objInfo(peak,assTab)]} {set objInfo(peak,assTab) 0}

   if {$error || !$objInfo(peak,assTab)} \
      {
       showError $objInfo(prog,name) "Error Reading Assignment Table $objInfo(peak,assName)"
       return
      }

   set objInfo(prog,assFlag) 1
}

proc initSpecData {} \
{
   global objInfo

   set obj data

   set i 1

   if {![strcasecmp $objInfo($obj,inName$i) None]} {return -1} 
   if {![file exists $objInfo($obj,inName$i)]}     {return -1} 
   if {[file isdirectory $objInfo($obj,inName$i)]} {return -1} 

#
# Read data header.
# Extract size parameters:

   initSpecSizes

   set fdata $objInfo($obj,$i,fdata)

   set objInfo($obj,fdata)  $fdata
   set objInfo($obj,tpFlag) [getParm $fdata FDTRANSPOSED NULL_DIM]

   if {$objInfo($obj,tpFlag)} \
      {
       set objInfo($obj,hDim)  CUR_YDIM
       set objInfo($obj,vDim)  CUR_XDIM
       set objInfo($obj,hAxis) Y_AXIS
       set objInfo($obj,vAxis) X_AXIS 

       if {![strcasecmp $objInfo(peak,xName) None]} {set objInfo(peak,xName) Y_PPM}
       if {![strcasecmp $objInfo(peak,yName) None]} {set objInfo(peak,yName) X_PPM}
      } \
   else \
      {
       set objInfo($obj,hDim)  CUR_XDIM
       set objInfo($obj,vDim)  CUR_YDIM
       set objInfo($obj,hAxis) X_AXIS
       set objInfo($obj,vAxis) Y_AXIS 

       if {![strcasecmp $objInfo(peak,xName) None]} {set objInfo(peak,xName) X_PPM}
       if {![strcasecmp $objInfo(peak,yName) None]} {set objInfo(peak,yName) Y_PPM}
      }

   set objInfo($obj,zDim) CUR_ZDIM
   set objInfo($obj,aDim) CUR_ADIM

   set objInfo($obj,xAxis) [getParmStr $fdata NDLABEL $objInfo($obj,hDim)]
   set objInfo($obj,yAxis) [getParmStr $fdata NDLABEL $objInfo($obj,vDim)]
   set objInfo($obj,zAxis) [getParmStr $fdata NDLABEL $objInfo($obj,zDim)]
   set objInfo($obj,aAxis) [getParmStr $fdata NDLABEL $objInfo($obj,aDim)]

   set objInfo($obj,ix) [expr int($objInfo($obj,$i,xSize)/2)]
   set objInfo($obj,iy) [expr int($objInfo($obj,$i,ySize)/2)]

   if {$objInfo(data,1,dimCount) == 1} \
      {
       setSpecZoom $obj $fdata 1 $objInfo($obj,$i,xSize) $objInfo(data,fhi1) $objInfo(data,lhi1) 
      } \
   else \
      {
       setSpecZoom $obj $fdata 1 $objInfo($obj,$i,xSize) 1 $objInfo($obj,$i,ySize) 
      }

#
# Extract noise levels:

   readView $obj 

   busyView hold .

   set zeroCount 0

   for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
      {
       if {$objInfo($obj,fhi$i) == 0.0 && $objInfo($obj,lhi$i) == 0.0} \
          {
           set objInfo($obj,fhi$i) $objInfo(data,fhi$i)
           set objInfo($obj,lhi$i) $objInfo(data,lhi$i)
          }

       if {$objInfo($obj,roi$i) != 0 && \
             $objInfo($obj,fhi$i) == 0.0 && \
               $objInfo($obj,lhi$i) == 0.0} \
          {
           showVerb "Finding max, min, noise for ROI $objInfo($obj,roi$i)."

           set roiInfo [getROIInfo -roi $objInfo($obj,roi$i)]
           set size    [getROISize $objInfo($obj,roi$i)]
           set n       [expr $size/5]
           set nn      $size

           incr zeroCount

           if {[strcasecmp $obj data]} \
              {
               set objInfo($obj,fdata) [lindex $roiInfo 1]
              }

           set objInfo($obj,rdata)    [lindex $roiInfo 0] 

           if {$size >= 32} \
              {
               set objInfo($obj,noise$i) [vEstNoise $objInfo($obj,rdata) $n]
              } \
           else \
              {
               set objInfo($obj,noise$i) 1.0
              }

           set objInfo($obj,minVal$i) [vMin $objInfo($obj,rdata) $nn]
           set objInfo($obj,maxVal$i) [vMax $objInfo($obj,rdata) $nn]

           set r [expr 0.05*($objInfo($obj,maxVal$i) - $objInfo($obj,minVal$i))]

           if {$objInfo($obj,fhi$i) == 0.0 && $objInfo($obj,lhi$i) == 0.0} \
              {
               set objInfo($obj,fhi$i) [expr $objInfo($obj,minVal$i) - $r]
               set objInfo($obj,lhi$i) [expr $objInfo($obj,maxVal$i) + $r]

               set objInfo($obj,fhi$i) [format %e $objInfo($obj,fhi$i)]
               set objInfo($obj,lhi$i) [format %e $objInfo($obj,lhi$i)]
              }

           if {$objInfo(data,$i,dimCount) == 1} \
              {
               showVerb "$objInfo($obj,inName$i) $objInfo($obj,fhi$i) $objInfo($obj,lhi$i)"
              }
          }
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {!$objInfo($obj,roi$i)} continue

       if {![strcasecmp $objInfo(data,hi$i) Auto]} \
          {
           set objInfo(data,hi$i) [format %e [expr 7.0*$objInfo($obj,noise$i)]]

           if {$objInfo(data,$i,dimCount) != 1} \
              {
               showVerb "$objInfo($obj,inName$i) Auto Mode Contour Height: $objInfo(data,hi$i)"
              }
          }

       if {![strcasecmp $objInfo(data,hi$i) None]} \
          {
           set objInfo(data,hi$i) $objInfo(data,hi1)

           if {$objInfo(data,$i,dimCount) != 1} \
              {
               showVerb "$objInfo($obj,inName$i) Contour Height: $objInfo(data,hi$i)"
              }
          }
      }

   set objInfo(prog,pcaThresh) $objInfo(data,hi1)
   set objInfo(data,currentHi) $objInfo(data,hi1)
   set objInfo(data,histat)    $objInfo(data,hi1)
   set i 1

   if {$objInfo(data,$i,dimCount) == 1} \
      {
       setSpecZoom $obj $fdata 1 $objInfo($obj,$i,xSize) $objInfo($obj,fhi1) $objInfo($obj,lhi1)
      } \
   else \
      {
       setSpecZoom $obj $fdata 1 $objInfo($obj,$i,xSize) 1 $objInfo($obj,$i,ySize)
      }

   if {$zeroCount == $objInfo(data,specCount)} \
      {
       for {set i $objInfo(data,specCount)} {$i >= 1} {incr i -1} \
          {
           set objInfo($obj,fhi$i) $objInfo($obj,fhi1)
           set objInfo($obj,lhi$i) $objInfo($obj,lhi1)

           set fhi $objInfo($obj,fhi$i)
           set lhi $objInfo($obj,lhi$i)

           showVerb "Reset $objInfo($obj,inName$i) Hi: $fhi $lhi"
          }
      }

    busyView release .

    return 0
}

proc initSpecSizes {} \
{
   global objInfo xUnits yUnits

   set obj data
   set n   0

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {![strcmp $objInfo($obj,$i,fdata) 0]} \
          {
           showVerb "No FDATA header for spectrum $i."
           continue
          }

       incr n

       set fdata $objInfo($obj,$i,fdata)

       set objInfo($obj,$i,tpFlag) [getParmI $fdata FDTRANSPOSED NULL_DIM]

       if {$objInfo($obj,$i,tpFlag)} \
          {
           set objInfo($obj,$i,hDim)  CUR_YDIM
           set objInfo($obj,$i,vDim)  CUR_XDIM
          } \
       else \
          {
           set objInfo($obj,$i,hDim)  CUR_XDIM
           set objInfo($obj,$i,vDim)  CUR_YDIM
          }

       set objInfo($obj,$i,zDim) CUR_ZDIM
       set objInfo($obj,$i,aDim) CUR_ADIM

       set objInfo($obj,$i,dimCount) [getParmI $fdata FDDIMCOUNT NULL_DIM]
    
       set objInfo($obj,$i,xSize) [getParmI $fdata NDSIZE $objInfo($obj,$i,hDim)]
       set objInfo($obj,$i,ySize) [getParmI $fdata NDSIZE $objInfo($obj,$i,vDim)]
       set objInfo($obj,$i,zSize) [getParmI $fdata NDSIZE $objInfo($obj,$i,zDim)]
       set objInfo($obj,$i,aSize) [getParmI $fdata NDSIZE $objInfo($obj,$i,aDim)]
     
       if {$objInfo(data,$i,dimCount) > 1} \
          {
           if {$objInfo($obj,$i,ySize) == 1} {set objInfo(data,$i,dimCount) 1}
          }

       if {$objInfo(data,$i,dimCount) > 2} {set objInfo(data,$i,dimCount) 2} 
 
       if {$objInfo($obj,$i,dimCount) < 4} {set objInfo($obj,$i,aSize) 1}
       if {$objInfo($obj,$i,dimCount) < 3} {set objInfo($obj,$i,zSize) 1}
       if {$objInfo($obj,$i,dimCount) < 2} {set objInfo($obj,$i,ySize) 1}

       set objInfo($obj,$i,xAxis)  [getParmStr $fdata NDLABEL $objInfo($obj,$i,hDim)]
       set objInfo($obj,$i,yAxis)  [getParmStr $fdata NDLABEL $objInfo($obj,$i,vDim)]
       set objInfo($obj,$i,zAxis)  [getParmStr $fdata NDLABEL $objInfo($obj,$i,zDim)]
       set objInfo($obj,$i,aAxis)  [getParmStr $fdata NDLABEL $objInfo($obj,$i,aDim)]

       set objInfo($obj,$i,xFirst) [pnt2spec $fdata CUR_XDIM 1 $xUnits] 
       set objInfo($obj,$i,xLast)  [pnt2spec $fdata CUR_XDIM $objInfo($obj,$i,xSize) $xUnits] 

       set objInfo($obj,$i,yFirst) [pnt2spec $fdata CUR_YDIM 1 $yUnits] 
       set objInfo($obj,$i,yLast)  [pnt2spec $fdata CUR_YDIM $objInfo($obj,$i,ySize) $yUnits] 
      }

    return $n
}

proc resizeView args \
{
   global objInfo

   set obj  [lindex $args 0]

   bind $objInfo($obj,canvas) <Configure> ""

   showVerb "Resize event $args"

   xupdate

   set objInfo($obj,winXSize) [winfo width  $objInfo($obj,window)]
   set objInfo($obj,winYSize) [winfo height $objInfo($obj,window)]

   set objInfo($obj,cellXSize) \
       [expr $objInfo($obj,winXSize)     - \
             ($objInfo($obj,leftMargin)  + \
              $objInfo($obj,rightMargin) + \
              $objInfo($obj,winXAdj))]

   set objInfo($obj,cellYSize) \
       [expr $objInfo($obj,winYSize)      - \
             ($objInfo($obj,topMargin)    + \
              $objInfo($obj,bottomMargin) + \
              $objInfo($obj,winYAdj))]

   set objInfo($obj,xSize) [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]
   set objInfo($obj,ySize) [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

   if {$objInfo($obj,xSize)     < 50} {set objInfo($obj,xSize) 50}
   if {$objInfo($obj,ySize)     < 50} {set objInfo($obj,ySize) 50}
   if {$objInfo($obj,cellXSize) < 50} {set objInfo($obj,cellXSize) 50}
   if {$objInfo($obj,cellYSize) < 50} {set objInfo($obj,cellYSize) 50}

   $objInfo($obj,canvas) delete pixmap -tags $obj

   $objInfo($obj,canvas) create pixmap 0 0 \
      -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
      -win $objInfo($obj,pixmap) -tags $obj

   set objInfo($obj,x1) $objInfo($obj,leftMargin)
   set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
   set objInfo($obj,y1) $objInfo($obj,topMargin)
   set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

   set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
   set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

   set objInfo($obj,x1Pix)        $xPix
   set objInfo($obj,y1Pix)        $yPix
   set objInfo($obj,xnPix)        $xPix
   set objInfo($obj,ynPix)        $yPix

   set objInfo($obj,eventX)       $xPix
   set objInfo($obj,eventY)       $yPix

   defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
              -x $objInfo($obj,x1) $objInfo($obj,xn) \
              -y $objInfo($obj,y1) $objInfo($obj,yn)

   $objInfo($obj,canvas) raise cursorH
   $objInfo($obj,canvas) raise cursorV

   $objInfo($obj,redrawProc) $obj resize 

   bind $objInfo($obj,canvas) <Configure> "$objInfo($obj,resizeProc) $obj %w %h %T"
}

proc updateAssWindow {} \
{
   global objInfo 

   return

   if {$objInfo(prog,assFlag) && [winfo exists .assWin]} \
      {
       raise .assWin
      }
}

proc initAssWindow {} \
{
   global objInfo fixedfont

   return

   if {$objInfo(prog,assFlag)} \
      {
       toplevel .assWin

       raise .assWin

       wm geom .assWin -60-120
 
       scrollbar .assWin.sb -orient vert -command ".assWin.lb yview"

       listbox .assWin.lb -font $fixedfont -selectmode extended -exportselection 0 \
                          -width $objInfo(prog,listboxWidth) -height 20 -yscrollcommand ".assWin.sb set"

       pack .assWin.sb .assWin.lb  -in .assWin -side right -expand 1 -fill y
      }
}

proc initMainWindow {} \
{
   global objInfo env fixedfont

    set colorList \
       {
        Black        Red          Orange       Yellow
        Green        Blue         Violet       Cyan
        Magenta      LightGray    DimGray      White
       }

#
# Logo:

   if {1 || !$objInfo(prog,extendedMode)} \
      {
       frame .logo -background black 
 
       catch {image delete specViewImage}
       catch {image delete specPCAImage}

       image create photo specViewImage -file $objInfo(prog,viewerLogo)
       image create photo specPCAImage   -file $objInfo(prog,pcaLogo)

       label .pwLogo  -relief flat -image specViewImage -background black -border 0
#      label .pwLabel -relief flat -foreground white -background black -text $objInfo(prog,name) -border 0

       pack .pwLogo -in .logo -side top -expand 1 -fill both -pady 4
       pack .logo -in . -side top -expand 1 -fill both
      }

#
# File info:

   frame .info -relief groove -border 4

   frame .fileFrame

   if {$objInfo(prog,extendedMode)} \
      {
       label .fileEnt -width 32 -textvariable objInfo(data,currentSpec)
      } \
   else \
      {
       label .fileEnt -width 21 -textvariable objInfo(data,currentSpec)
      }

   pack  .fileEnt -in .fileFrame -side left
   pack  .fileFrame -in .info -side top -anchor c

   if {$objInfo(prog,extendedMode)} \
      {
       frame .scoreFrame
       label .scoreLab -text "Score:"

       radiobutton .score0    -text "0" -value 0 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       radiobutton .score1    -text "1"    -value 1 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       radiobutton .score2    -text "2"    -value 2 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       radiobutton .score3    -text "3"    -value 3 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       radiobutton .score4    -text "4"    -value 4 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       radiobutton .score5    -text "5"    -value 5 \
          -variable objInfo(data,currentScore) -command setCurrentScore

       pack .scoreLab .score0 .score1 .score2 .score3 .score4 .score5 \
            -in .scoreFrame -side left

       pack .scoreFrame -in .info -side top -anchor c
      }

   frame .numFrame
   label .numLab -text "ID: "  -width 6 -anchor e

   spinbox .numEnt -width 19 -textvariable objInfo(data,currentID)  \
           -increment 0 -format %.0f -command "incrSelectID %d"

   bind  .numEnt <Return> "+ setCurrentID"

   pack  .numLab .numEnt -in .numFrame -side left
   pack  .numFrame -in .info -side top -anchor c

   frame .scaleFrame
   label .scaleLab -text "Scale: "  -width 6 -anchor e

   spinbox .scaleEnt -width 19 -textvariable objInfo(data,currentScale) \
           -increment 0 -format %.5f -command "incrCurrentScale %d"

   bind  .scaleEnt <Return> "+ setCurrentScale"

   pack  .scaleLab .scaleEnt -in .scaleFrame -side left
   pack  .scaleFrame -in .info -side top -anchor c

   if {$objInfo(data,1,dimCount) > 1} \
      {
       frame .hiFrame
       label .hiLab -text "Hi: "  -width 6 -anchor e

       spinbox .hiEnt -width 19 -textvariable objInfo(data,currentHi) \
               -increment 0 -command "incrCurrentHi %d"

       bind  .hiEnt <Return> "+ setCurrentHi"

       pack  .hiLab .hiEnt -in .hiFrame -side left
       pack  .hiFrame -in .info -side top -anchor c
      }

   frame .colorFrame
   label .colorLab -text "Color: "  -width 6 -anchor e
   entry .colorEnt -width 18 -textvariable objInfo(data,currentColor)
   bind  .colorEnt <Return> "+ setCurrentColor"

   button .colorButton -bitmap @$env(NMRTXT)/caret.map -relief raised \
           -width $objInfo(prog,charX) -height $objInfo(prog,charY)   \
           -fg blue -command "specColorChoiceON data,currentColor"

   pack   .colorLab .colorEnt .colorButton -in .colorFrame -side left
   pack   .colorFrame -in .info -side top -anchor c

   frame .drFrame

   label .drAxisLab -text "Axis X:" -anchor e

   checkbutton .drXAxisButt -text "" \
               -variable objInfo(prog,xAxisFlag) \
               -selectcolor white -command setCurrentAxisFlag

   checkbutton .drYAxisButt -text " Y:" \
               -variable objInfo(prog,yAxisFlag) \
               -selectcolor white -command setCurrentAxisFlag

   label .drZeroLab -text "Zero:" -anchor e

   checkbutton .drZeroButt -text "" \
               -variable objInfo(data,currentZeroFlag) \
               -selectcolor white -command setCurrentZeroFlag

   pack .drAxisLab .drXAxisButt .drYAxisButt .drZeroLab .drZeroButt -in .drFrame -side left 

   if {$objInfo(prog,integMode)} \
      {
       label .drIntegLab -text "Integral:" -anchor e
   
       checkbutton .drIntegButt -text "" \
                   -variable objInfo(data,currentIntegFlag) \
                   -selectcolor white -command setCurrentIntegFlag

       pack .drIntegLab .drIntegButt -in .drFrame -side left 
      }

#  pack .drFrame -in .info -side top -anchor c

#
# Line width in pixels.
# Color mode.

   frame .modeFrame 

   label .lw1dLab -text "Width: "

   spinbox .lw1dEnt -width 2 -textvariable objInfo(data,currentlw1d) \
           -from 0 -to $objInfo(prog,screenWidth) -increment 1 -format %.0f \
           -command "setCurrentlw1d 0"

   bind  .lw1dEnt <Return> "+ setCurrentlw1d 0"

   label .modeLab -text "Color By: "

   entry .modeEntry -width 7 -textvariable objInfo(data,colorMode)
   bind  .modeEntry <Return> "+ setColorMode Current"

   menubutton .modeButton -bitmap @$env(NMRTXT)/caret.map \
              -relief raised -menu .modeButton.menu       \
              -width $objInfo(prog,charX) -height $objInfo(prog,charY) 

   menu .modeButton.menu

   foreach i {Title File Number} \
      {
       .modeButton.menu add command -label $i -command "setColorMode $i"
      }

   foreach i {1 2 3 4 5 6 7 8 9 10 11 12} \
      {
       .modeButton.menu add command -label "Item $i" -command "setColorMode $i"
      }

   .modeButton.menu add command -label "Last Item"  -command "setColorMode Last"
   .modeButton.menu add command -label "Cluster"    -command "setColorMode Cluster"

   pack .lw1dLab .lw1dEnt .modeLab .modeEntry .modeButton -in .modeFrame -side left
   pack .modeFrame -in .info -side top

#
# Use for PCA:

   if {$objInfo(prog,usePCA)} \
      {
       frame .pcaFrame
    
       checkbutton .pcaBut -text "Use for PCA" \
                   -variable objInfo(data,currentPCAFlag) \
                   -selectcolor white -command setCurrentPCA

       pack .pcaBut -in .pcaFrame -side left
       pack .pcaFrame -in .info -side top
      }

   pack .info -in . -side top -pady 2 -expand 1 -fill x

#
# General window params:

   if {$objInfo(prog,extendedMode)} \
      {
       frame .wgen -relief groove -border 4

       frame .lwdrawFrame
       label .lwdrawLab -text "Box Width: "  -anchor e

       spinbox .lwdrawEnt -width 3 -textvariable objInfo(prog,lwdraw) \
               -increment 1 -from 0 -to 4096 -format %.0f \
               -command "showViewAll lwdraw"

       bind  .scaleEnt <Return> "+ showViewAll lwdraw"

       pack  .lwdrawLab .lwdrawEnt -in .lwdrawFrame -side left
       pack  .lwdrawFrame -in .wgen -side top -anchor c

       label .lwgridLab -text "Grid Width: "  -anchor e

       spinbox .lwgridEnt -width 3 -textvariable objInfo(prog,lwgrid) \
               -increment 1 -from 0 -to 4096 -format %.0f \
               -command "showViewAll lwgrid"
               
       bind  .scaleEnt <Return> "+ showViewAll lwgrid"
 
       pack  .lwgridLab .lwgridEnt -in .lwdrawFrame -side left

       frame .fgcolorFrame
       label .fgcolorLab -text "Box: " -width 6 -anchor e
       entry .fgcolorEnt -width 17 -textvariable objInfo(color,label)
       bind  .fgcolorEnt <Return> "+ showViewAll color,label"

       button .fgcolorButton -bitmap @$env(NMRTXT)/caret.map -relief raised \
               -width $objInfo(prog,charX) -height $objInfo(prog,charY)   \
               -fg blue -command "progColorChoiceON color,label"

       pack .fgcolorLab .fgcolorEnt .fgcolorButton -in .fgcolorFrame -side left
       pack .fgcolorFrame -in .wgen -side top -anchor c

       frame .gridcolorFrame
       label .gridcolorLab -text "Grid: " -width 6 -anchor e 
       entry .gridcolorEnt -width 17 -textvariable objInfo(color,grid)
       bind  .gridcolorEnt <Return> "+ showViewAll color,grid"
                   
       button .gridcolorButton -bitmap @$env(NMRTXT)/caret.map -relief raised \
               -width $objInfo(prog,charX) -height $objInfo(prog,charY)   \
               -fg blue -command "progColorChoiceON color,grid"
                   
       pack .gridcolorLab .gridcolorEnt .gridcolorButton -in .gridcolorFrame -side left
       pack .gridcolorFrame -in .wgen -side top -anchor c

       frame .bgcolorFrame
       label .bgcolorLab -text "Background: " -anchor e
       entry .bgcolorEnt -width 19 -textvariable objInfo(color,bg)
       bind  .bgcolorEnt <Return> "+ showViewAll color,bg"

       button .bgcolorButton -bitmap @$env(NMRTXT)/caret.map -relief raised \
               -width $objInfo(prog,charX) -height $objInfo(prog,charY)   \
               -fg blue -command "progColorChoiceON color,bg"

       pack .bgcolorLab .bgcolorEnt .bgcolorButton -in .bgcolorFrame -side left
       pack .bgcolorFrame -in .wgen -side top -anchor c

       pack .wgen -in . -side top -pady 2 -expand 1 -fill x
      }

#
# List title or file

    frame .lbModeFrame

    label .listLabel -text "List: "

    radiobutton .listTitle -text "Title" -value 1 \
       -variable objInfo(data,listByTitle) -command setListBoxMode

    radiobutton .listFile -text "File"   -value 0 \
       -variable objInfo(data,listByTitle) -command setListBoxMode

    pack .listLabel .listTitle .listFile -in .lbModeFrame -side left
    pack .lbModeFrame -in .info -side top

#
# Reference list:

   frame .refs

   scrollbar .sf -orient vert -command ".rlb yview"

   listbox .rlb -font $fixedfont -selectmode extended -exportselection 0 \
           -width $objInfo(prog,listboxWidth) -height 3 -yscrollcommand ".sf set"

   pack .sf  -in .refs -side right -expand 1 -fill y
   pack .rlb -in .refs -side left  -expand 1 -fill both

   pack .refs -in . -side top -expand 1 -fill both

#
# Pattern list:

   frame .patterns

   scrollbar .sp -orient vert -command ".plb yview"

   listbox .plb -font $fixedfont -selectmode single -exportselection 0 \
            -width $objInfo(prog,listboxWidth) -height 4 -yscrollcommand ".sp set"

   pack .sp  -in .patterns -side right -expand 1 -fill y
   pack .plb -in .patterns -side left  -expand 1 -fill both

   pack .patterns -in . -side top -expand 1 -fill x

#
# Title list:

   frame .titles

   scrollbar .st -orient vert -command ".tlb yview"

   listbox .tlb -font $fixedfont -selectmode extended -exportselection 0 \
           -width $objInfo(prog,listboxWidth) -height $objInfo(prog,listboxHeight) -yscrollcommand ".st set"

   pack .st  -in .titles -side right -expand 1 -fill y
   pack .tlb -in .titles -side left  -expand 1 -fill both

   pack .titles -in . -side top -expand 1 -fill both 

#
# Exit:

   button .exit -text Exit  -command quitAllView -relief $objInfo(prog,button,relief)
   pack .exit -in . -side top -expand 1 -fill x

   wm geom  . -20+40
   wm title . $objInfo(prog,name)

   update

   bind . <KeyPress> "+ parseKey main %K %x %y"

#
# Adjust size of future 1D windows:

   set sw [winfo screenwidth  .]
   set tw [winfo width .]

   set objInfo(prog,defXSize) [expr $sw - ($tw + 60)]

   if {$objInfo(prog,defXSize) < 200} {set objInfo(prog,defXSize) 200}
}


proc getGraphItemFont { obj id } \
{
   global objInfo

   if {[info exists objInfo($obj,graphItem$id,fontWindow)]} \
      { 
       set w $objInfo($obj,graphItem$id,fontWindow)

       if {[strcasecmp $w None] && [winfo exists $w]} \
          {
           catch "raise $w"
           return
          }
      }

   set objInfo($obj,graphItem$id,fontWindow) \
    [getFontChoice -font $objInfo($obj,graphItem$id,font) \
                   -command "graphItemFontProc $obj $id"]
}
       
proc getGraphItemColor { obj id mode } \
{
   global objInfo

   if {[info exists objInfo($obj,graphItem$id,colorWindow)]} \
      {
       set w $objInfo($obj,graphItem$id,colorWindow)

       if {[strcasecmp $w None] && [winfo exists $w]} \
          {
           catch "raise $w"
           return 
          }
      }

   set objInfo($obj,graphItem$id,colorWindow) \
    [getColorChoice -color $objInfo($obj,graphItem$id,color,$mode) \
                    -command "graphItemColorProc $obj $id $mode"]
}

proc resizeGraphItem { obj id xPix yPix initFlag } \
{
   global objInfo

   if {!$id || ![info exists objInfo($obj,graphItem$id,gID)]} return
   if {![strcasecmp $objInfo($obj,graphItem$id,type) text]} return

   set xf [getGraphItemXF $obj $id $xPix]
   set yf [getGraphItemYF $obj $id $yPix]

   set d    1.0e+16
   set mode 1

   if {$initFlag} \
      {
       set objInfo($obj,graphItem$id,resizeMode) 1

       if {![strcasecmp $objInfo($obj,graphItem$id,type) rectangle]
            || ![strcasecmp $objInfo($obj,graphItem$id,type) spectrum]} \
          {
           set xMin [getMin "$objInfo($obj,graphItem$id,x1) $objInfo($obj,graphItem$id,x2)"]
           set xMax [getMax "$objInfo($obj,graphItem$id,x1) $objInfo($obj,graphItem$id,x2)"]
           set yMin [getMin "$objInfo($obj,graphItem$id,y1) $objInfo($obj,graphItem$id,y2)"]
           set yMax [getMax "$objInfo($obj,graphItem$id,y1) $objInfo($obj,graphItem$id,y2)"]

           set objInfo($obj,graphItem$id,x1) $xMin
           set objInfo($obj,graphItem$id,x2) $xMax
           set objInfo($obj,graphItem$id,y1) $yMin
           set objInfo($obj,graphItem$id,y2) $yMax

           set xList "$xMin $xMin $xMax $xMax"
           set yList "$yMin $yMax $yMax $yMin"

           for {set i 1} {$i <= 4} {incr i} \
              {
               set dx    [expr $xf - [lindex $xList [expr $i - 1]]]
               set dy    [expr $yf - [lindex $yList [expr $i - 1]]]
               set thisD [expr $dx*$dx + $dy*$dy]
 
               if {$thisD <= $d} \
                  {
                   set d $thisD
                   set mode $i
                  }
              }
          } \
       else \
          {
           for {set i 1} {$i <= $objInfo($obj,graphItem$id,nCoords)} {incr i} \
              {
               set dx    [expr $xf - $objInfo($obj,graphItem$id,x$i)]
               set dy    [expr $yf - $objInfo($obj,graphItem$id,y$i)]
               set thisD [expr $dx*$dx + $dy*$dy]

               if {$thisD <= $d} \
                  {
                   set d $thisD
                   set mode $i
                  }
              }
          }

       set objInfo($obj,graphItem$id,resizeMode) $mode

       return
      }

   if {![strcasecmp $objInfo($obj,graphItem$id,type) rectangle] \
        || ![strcasecmp $objInfo($obj,graphItem$id,type) spectrum]} \
      {
       set xMin [getMin "$objInfo($obj,graphItem$id,x1) $objInfo($obj,graphItem$id,x2)"]
       set xMax [getMax "$objInfo($obj,graphItem$id,x1) $objInfo($obj,graphItem$id,x2)"]
       set yMin [getMin "$objInfo($obj,graphItem$id,y1) $objInfo($obj,graphItem$id,y2)"]
       set yMax [getMax "$objInfo($obj,graphItem$id,y1) $objInfo($obj,graphItem$id,y2)"]

       set mode $objInfo($obj,graphItem$id,resizeMode)

       if {$mode == 1 || $mode == 2} \
          {
           set xMin $xf
          } \
       else \
          {
           set xMax $xf
          }

       if {$mode == 1 || $mode == 4} \
          {
           set yMin $yf
          } \
       else \
          {
           set yMax $yf
          }

       set objInfo($obj,graphItem$id,x1) $xMin
       set objInfo($obj,graphItem$id,x2) $xMax
       set objInfo($obj,graphItem$id,y1) $yMin
       set objInfo($obj,graphItem$id,y2) $yMax
      } \
   else \
      {
       set mode $objInfo($obj,graphItem$id,resizeMode)

       set objInfo($obj,graphItem$id,x$mode) $xf
       set objInfo($obj,graphItem$id,y$mode) $yf
      }

   configGraphItemCoords $obj $id 
}
 
proc dragGraphItem { obj id idx idy } \
{
   global objInfo

   if {!$id || ![info exists objInfo($obj,graphItem$id,gID)]} return
   $objInfo($obj,canvas) move $objInfo($obj,graphItem$id,gID) $idx $idy
 
   set cList [$objInfo($obj,canvas) coords $objInfo($obj,graphItem$id,gID)]
   set n     [llength $cList]

   if {$n >= 2} \
      {
       set objInfo($obj,graphItem$id,x1) \
        [format %.4f [getGraphItemXF $obj $id [lindex $cList 0]]]

       set objInfo($obj,graphItem$id,y1) \
        [format %.4f [getGraphItemYF $obj $id [lindex $cList 1]]]
      } 

   if {$n >= 4} \
      {
       set objInfo($obj,graphItem$id,x2) \
        [format %.4f [getGraphItemXF $obj $id [lindex $cList 2]]]

       set objInfo($obj,graphItem$id,y2) \
        [format %.4f [getGraphItemYF $obj $id [lindex $cList 3]]]
      }
}

proc configGraphItemFont { obj id } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,canvas)]} return

   $objInfo($obj,canvas) itemconfigure $objInfo($obj,graphItem$id,gID) \
    -font $objInfo($obj,graphItem$id,font)

   update
}

proc graphItemFontProc args \
{
   global objInfo

   set obj [lindex $args 0]
   set id  [lindex $args 1]
   set f   [lrange $args 2 end]

   if {![winfo exists $objInfo($obj,canvas)]} return
      
   set objInfo($obj,graphItem$id,font) $f

   $objInfo($obj,canvas) itemconfigure $objInfo($obj,graphItem$id,gID) -font $f
 
   update
}

proc configGraphItemWidth { obj id } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,canvas)]} return

   $objInfo($obj,canvas) itemconfigure $objInfo($obj,graphItem$id,gID) \
    -width $objInfo($obj,graphItem$id,width)
       
   update
}       

proc configGraphItemColor { obj id mode color } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,canvas)]} return

   if {[strcasecmp $color current]} \
      {
       set objInfo($obj,graphItem$id,color,$mode) $color
      }

   set thisColor [parseColor $objInfo($obj,graphItem$id,color,$mode)]

   $objInfo($obj,canvas) itemconfigure $objInfo($obj,graphItem$id,gID) -$mode $thisColor

   update
}

proc graphItemColorProc { obj id mode color } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,canvas)]} return

   set objInfo($obj,graphItem$id,color,$mode) $color

   $objInfo($obj,canvas) itemconfigure $objInfo($obj,graphItem$id,gID) -$mode [parseColor $color]

   update
}

proc progColorChoiceProc { i color } \
{
   global objInfo

   set objInfo($i) $color
   showViewAll $i
}

proc progColorChoiceON { i } \
{
   global objInfo
      
   if {[info exists objInfo(prog,colorWindow,$i)]} \
      { 
       if {[winfo exists $objInfo(prog,colorWindow,$i)]} \
          {
           catch "raise $objInfo(prog,colorWindow,$i)"
           return
          }
      }

   set objInfo(prog,colorWindow,$i) \
    [getColorChoice -color $objInfo($i) -command "progColorChoiceProc $i"]
}

proc specColorChoiceON { i } \
{
   global objInfo

   if {[info exists objInfo(prog,colorWindow)]} \
      {
       if {[winfo exists $objInfo(prog,colorWindow)]} \
          {
           catch "raise $objInfo(prog,colorWindow)"
           return
          }
      }

   set objInfo(prog,colorWindow) \
    [getColorChoice -color $objInfo($i) -command specColorChoiceProc]
}

proc specColorChoiceProc { color } \
{
   global objInfo

   set objInfo(data,currentColor) $color
   setCurrentColor
}

proc setRefListBox {} \
{
   global objInfo
   
   .rlb delete 0 end

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$objInfo(data,listByTitle)} \
          {  
           set str $objInfo(data,title$i)

           if {[strcmp $objInfo(data,axisLabel1) $objInfo(data,axisLabel$i)]} \
              {
               append str " $objInfo(data,axisLabel$i)"
              }
          } \
       else \
          {
           if {$objInfo(prog,fullNameFlag)} \
              {
               set str [file root [file tail $objInfo(data,inName$i)]]
              } \
           else \
              {
               set s   [file root [file tail $objInfo(data,inName$i)]]
               set str [lindex [split $s _] end]
              }
          }

       .rlb insert end $str
      }
}

proc setListBoxMode {} \
{
   global objInfo

   .tlb delete 0 end

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$objInfo(data,listByTitle)} \
          {
           set str $objInfo(data,title$i)

           if {[strcmp $objInfo(data,axisLabel1) $objInfo(data,axisLabel$i)]} \
              {
               append str " $objInfo(data,axisLabel$i)"
              }
          } \
       else \
          {
           if {$objInfo(prog,fullNameFlag)} \
              {
               set str [file root [file tail $objInfo(data,inName$i)]]
              } \
           else \
              {
               set s   [file root [file tail $objInfo(data,inName$i)]]
               set str [lindex [split $s _] end]
              }
          }
   
       .tlb insert end $str
      }

   setRefListBox

   setColorMode Current
}

proc setCurrentPCA {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,pcaFlag$i) $objInfo(data,currentPCAFlag)

   if {$objInfo(data,pcaFlag$i)} \
      {
       .tlb itemconfigure [expr $i - 1] \
            -foreground       [safeColor $objInfo(color,color$i)] \
            -background       $objInfo(color,bg) \
            -selectforeground [safeColor $objInfo(color,color$i)] \
            -selectbackground $objInfo(color,grayed)
      } \
   else \
      {
       .tlb itemconfigure [expr $i - 1] \
            -foreground       gray50                  \
            -background       gray30                  \
            -selectforeground white                   \
            -selectbackground gray10
      }

   showViewAll setCurrentPCA
   drawTitles
}

proc setCurrentColor args \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   if {[llength $args]} {set objInfo(data,currentColor) [lindex $args 0]}

   set objInfo(color,color$i) $objInfo(data,currentColor)

   .tlb itemconfigure [expr $i - 1] \
        -foreground       [safeColor $objInfo(color,color$i)] \
        -background       $objInfo(color,bg) \
        -selectforeground [safeColor $objInfo(color,color$i)] \
        -selectbackground $objInfo(color,grayed)

   .rlb itemconfigure [expr $i - 1] \
        -foreground       [safeColor $objInfo(color,color$i)] \
        -background       $objInfo(color,bg)      \
        -selectforeground [safeColor $objInfo(color,color$i)] \
        -selectbackground $selColor 

   showViewAll setCurrentColor
   drawTitles
}

proc setCurrentID {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1} {set i 1}
   if {$i > $objInfo(data,specCount)} {set i $objInfo(data,specCount)}

   .tlb selection clear 0 end
   .tlb selection set [expr $i - 1]

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        set objInfo(data,drawFlag$i) 0
       }

   set objInfo(data,drawList)   $i 
   set objInfo(data,drawFlag$i) 1

   showViewAll setCurrentID
   drawTitles
}

proc getAxisFlagArg { i } \
{  
   global objInfo xUnits yUnits
               
   if {$i < 1 || $i > $objInfo(data,specCount)} {return None}

   if {$objInfo(prog,xAxisFlag)} {return Bottom,units=$xUnits}

   return None 
}

proc getZeroFlagArg { i } \
{
   global objInfo

   if {$i < 1 || $i > $objInfo(data,specCount)} {return "-nozero"}

   if {$objInfo(data,zeroFlag$i)} {return "-zero"}

   return "-nozero" 
}

proc setCurrentZeroFlag {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,zeroFlag$i) $objInfo(data,currentZeroFlag)

   foreach obj $objInfo(prog,winList) \
      {
       showView2 $obj setCurrentZeroFlag
      }

   drawTitles
}

proc setCurrentAxisFlag {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

#
#  Value of objInfo(prog,xAxisFlag) has been adjusted.

   foreach obj $objInfo(prog,winList) \
      {
       showView2 $obj setCurrentAxisFlag
      }

   drawTitles
}

proc setCurrentlw1d { lw } \
{
   global objInfo   

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   if {$lw} {set objInfo(data,currentlw1d) $lw}

   set objInfo(data,lw1d$i) $objInfo(data,currentlw1d)
   set objInfo(data,lw2d$i) $objInfo(data,currentlw1d)

   foreach obj $objInfo(prog,winList) \
      {
       showView2 $obj setCurrentlw1d
      }

   drawTitles
}

proc setCurrentScore {} \
{
   global objInfo   

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,score$i) $objInfo(data,currentScore)

   saveScore
   drawTitles
}

proc saveScore {} \
{
   global objInfo   

   set error [catch {set outUnit [open $objInfo(prog,scoreName) w]}]

   if {$error} \
      {
       showError $objInfo(prog,name) "Error Opening Score Output File $objInfo(prog,scoreName)"
       return 
      }

   puts $outUnit "VARS   SCORE TITLE SPEC_NAME"
   puts $outUnit "FORMAT %d    %s    %s"
   puts $outUnit ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set fmt " %d %32s %s"

       set t $objInfo(data,title$i)

       if {[llength $t] > 1} {set j [join $t "-"]}

       puts $outUnit [format $fmt $objInfo(data,score$i) $t $objInfo(data,inName$i)] 
      }

   close $outUnit
}

proc readNotes {} \
{
    global objInfo ARGC ARGV

    getArgD $ARGV -note objInfo(prog,noteName)

    if {![strcasecmp $objInfo(prog,noteName) None]} return
    if {![file exists $objInfo(prog,noteName)]} return

    puts "Reading notes in file $objInfo(prog,noteName)"

    set t     [gdbRead table -in $objInfo(prog,noteName)]
    set eList [gdbGetInfo $t -parms entries]
    set n     [llength $eList]

    if {$n != $objInfo(data,specCount)} \
       {
        showError $objInfo(prog,name) "Error: Mismatch in Note File $objInfo(prog,noteName) Entries: $n Spectra: $objInfo(data,specCount)"
        return
       }

    for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
       {
        set e [lindex $eList [expr $i - 1]]
        set f [gdbGet $e SPEC_NAME]
        set t [gdbGet $e NOTE]
      
        if {![strcasecmp $t None]} {set t ""}
 
        set thisName [file normalize $f]
        set specName [file normalize $objInfo(data,inName$i)]

        if {[strcmp $thisName $specName]} \
           {
            showError $objInfo(prog,name) "Error: Mismatch in Note File Entry $i: $f $objInfo(data,inName$i)"
            return
           }

        if {[llength $t]} {showVerb "Note $i $objInfo(data,inName$i): $t"}

        set objInfo(data,note$i) $t
       }
}

proc saveNotes {} \
{
   global objInfo

   if {![strcasecmp $objInfo(prog,noteName) None] || !$objInfo(prog,noteSaveNeeded)} return

   set objInfo(prog,noteSaveNeeded) 0

   set error [catch {set outUnit [open $objInfo(prog,noteName) w]}]

   if {$error} \
      {
       showError $objInfo(prog,name) "Error Opening Note Output File $objInfo(prog,noteName)"
       return
      }

   showVerb "Saving notes in file $objInfo(prog,noteName)"

   puts $outUnit "VARS   INDEX SPEC_NAME NOTE"
   puts $outUnit "FORMAT %4d   %32s      %s"
   puts $outUnit ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set t [string trim $objInfo(data,note$i)]
       if {![llength $t]} {set t None}

       if {[llength $t] > 1} \
          {
           set fmt " %4d %32s \"%s\""
          } \
       else \
          {
           set fmt " %4d %32s %s"
          }

       puts $outUnit [format $fmt $i $objInfo(data,inName$i) $t]
      }

   close $outUnit
}

proc incrCurrentHi { mode } \
{
   global objInfo

   if {![strcasecmp [string index $mode 0] d]} \
      {
       set objInfo(data,currentHi) [decrHiVal $objInfo(data,currentHi) 1.3]
      } \
   else \
      {
       set objInfo(data,currentHi) [incrHiVal $objInfo(data,currentHi) 1.3]
      }

   setCurrentHi
}

proc setCurrentHi {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,currentHi) [safeFormat %.4e $objInfo(data,currentHi)]

   set objInfo(data,trackFlag) 1
   set objInfo(data,hi$i)      $objInfo(data,currentHi)

   foreach obj $objInfo(prog,winList) \
      {
       if {$objInfo($obj,roi$i) < 1} continue 
       showView2 $obj setCurrentHi
      }

   drawTitles

   set objInfo(data,trackFlag) 0
}

proc incrCurrentScale { mode } \
{
   global objInfo

   if {![strcasecmp [string index $mode 0] d]} \
      {
       set objInfo(data,currentScale) [expr $objInfo(data,currentScale)*0.99]
      } \
   else \
      {
       set objInfo(data,currentScale) [expr $objInfo(data,currentScale)/0.99]
      }

   setCurrentScale
}

proc setCurrentScale {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,currentScale) [safeFormat %.5f $objInfo(data,currentScale)]

   set objInfo(data,trackFlag) 1
   set objInfo(data,scale$i) $objInfo(data,currentScale)

   foreach obj $objInfo(prog,winList) \
      {
       if {$objInfo($obj,roi$i) < 1} continue 

       set rInfo  [getROIInfo -roi $objInfo($obj,roi$i) -n -aux]

       set rdata  [lindex $rInfo 0]
       set n      [lindex $rInfo 3]
       set auxPtr [lindex [lindex $rInfo 4] 0]

       vsMult $rdata $auxPtr $objInfo(data,currentScale) $n

       updateROIStats $obj

       showView2 $obj setCurrentScale 
      }

   drawTitles

   set objInfo(data,trackFlag) 0
}

proc applyShiftX { obj id } \
{
   global objInfo

   set i $objInfo(data,currentID)

   set rInfo  [getROIInfo -roi $objInfo($obj,roi$i) -n -aux]

   set rdata  [lindex $rInfo 0]
   set n      [lindex $rInfo 3]
   set auxPtr [lindex [lindex $rInfo 4] 0]

   vCShift $rdata $auxPtr $objInfo($obj,shiftX,currentID) $n

   set objInfo($obj,shiftX,$i) $objInfo($obj,shiftX,currentID)

   showView2 $obj applyShiftX 

}

proc applyShiftY { obj id } \
{
   global objInfo

   set i $objInfo(data,currentID)

   set objInfo($obj,shiftY,$i) $objInfo($obj,shiftY,currentID)
}

proc setCurrentIntegFlag {} \
{
   global objInfo

   set i $objInfo(data,currentID)

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   set objInfo(data,integFlag$i) $objInfo(data,currentIntegFlag)

   foreach obj $objInfo(prog,winList) \
      {
       readView $obj
       showView2 $obj setCurrentIntegFlag
      }

   drawTitles
}

proc initPatterns {} \
{
   global objInfo

   set nMax 0

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set str $objInfo(data,title$i)

       if {[strcmp $objInfo(data,axisLabel1) $objInfo(data,axisLabel$i)]} \
          {
           append str " $objInfo(data,axisLabel$i)"
          }

       set tList [split $objInfo(data,title$i) -]
       set n     [llength $tList]

       if  {$n > $nMax} {set nMax $n}

       for {set nn 1} {$nn <= $n} {incr nn} \
          {
           set p [lindex $tList [expr $nn - 1]]

           if {![info exists iList($nn)]} {set iList($nn) ""}
           if {[lsearch -exact $iList($nn) $p] < 0} {lappend iList($nn) $p}
          }
      }

   setListBoxMode

   .plb delete 0 end
   .plb insert end *

   set nnList ""

   for {set nn 2} {$nn <= $nMax} {incr nn} {lappend nnList $nn}

   lappend nnList 1

   foreach nn $nnList \
      {
       set aList ""

       for {set nnn 1} {$nnn < $nn} {incr nnn} {lappend aList "*"}

       foreach i [lsort $iList($nn)] \
          {
           set str [join "$aList $i" "-"]
           if {$nn < $nMax} {append str "-*"} 
           .plb insert end $str
          }
      }

   foreach i $objInfo(data,drawList) {.tlb selection set [expr $i - 1]}

   updateTitleColors

   update

   bind .plb <ButtonRelease-1> "+ selectPattern %y"

   bind .tlb <ButtonRelease-1> "+ selectTitle   spec .tlb %y"
   bind .tlb <ButtonRelease-2> "+ selectCurrent spec .tlb %y"
   bind .tlb <ButtonRelease-3> "+ toggleTitle   spec .tlb %y"

   bind .rlb <ButtonRelease-1> "+ selectTitle   ref .rlb %y"
   bind .rlb <ButtonRelease-2> "+ selectCurrent ref .rlb %y"
   bind .rlb <ButtonRelease-3> "+ toggleTitle   ref .rlb %y"
}

proc selectPattern { yPix } \
{
   global objInfo

   set j [.plb nearest $yPix]
   set p [.plb get $j]

   showVerb "Search for Pattern: $p"

   set matchList ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {[strmatch $objInfo(data,title$i) $p]} {lappend matchList $i} 
      }

   set n [llength $matchList]

   showVerb "Matching Entries: $n"
 
   set objInfo(data,drawList)  ""

   .tlb selection clear 0 end
 
   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set objInfo(data,drawFlag$i) 0
      }

   set objInfo(data,drawList) $matchList

   foreach i $matchList \
      {
       set objInfo(data,drawFlag$i) 1
       .tlb selection set [expr $i - 1]
      }

   showViewAll selectPattern
   drawTitles

   updateSpecViewPCADrawList
}

proc updateCurrent { i } \
{
   global objInfo

   if {$i < 0 || $i > $objInfo(data,specCount)} return

   set objInfo(data,currentID)        $i
   set objInfo(data,currentSpec)      $objInfo(data,inName$i)
   set objInfo(data,currentColor)     $objInfo(color,color$i)
   set objInfo(data,currentScale)     $objInfo(data,scale$i)
   set objInfo(data,currentHi)        [safeFormat %.4e $objInfo(data,hi$i)]
   set objInfo(data,currentPCAFlag)   $objInfo(data,pcaFlag$i)

   set objInfo(data,currentlw1d)      $objInfo(data,lw1d$i)
   set objInfo(data,currentlw2d)      $objInfo(data,lw2d$i)
   set objInfo(data,currentScore)     $objInfo(data,score$i)
   set objInfo(data,currentIntegFlag) $objInfo(data,integFlag$i)
   set objInfo(data,currentZeroFlag)  $objInfo(data,zeroFlag$i)
 
   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       set objInfo($obj,shiftX,currentID) $objInfo($obj,shiftX,$i)
       set objInfo($obj,shiftY,currentID) $objInfo($obj,shiftY,$i) 
       
       if {[strcasecmp $objInfo(prog,noteName) None]} \
          {
           set objInfo($obj,currentNote) $objInfo(data,note$i) 
           $objInfo($obj,window).noteLabel configure -text "Note $i:"
          }
      }

   if {[strcasecmp $objInfo(prog,noteName) None]} {saveNotes}

   set j [expr $i - 1]

#  .tlb see $j
   .tlb activate $j 
}

proc applyCurrentNote { obj mode } \
{
   global objInfo

   if {![strcasecmp $objInfo(prog,noteName) None]} return
   
   set i $objInfo(data,currentID)
   if {$i < 1} return

   showVerb "Applying Note for Spectrum $i $objInfo(data,inName$i): $objInfo($obj,currentNote)" 

   set objInfo(prog,noteSaveNeeded) 1

   set objInfo(data,note$i) $objInfo($obj,currentNote)
  
   if {![strcasecmp $mode save]} {saveNotes}
}

proc selectCurrent { mode w yPix } \
{
   global objInfo

   set j [$w nearest $yPix]
   set i [expr $j + 1]

   selectCurrent2 $mode $i
   raiseSelMarkers
}
  
proc selectCurrent2 { mode i } \
{ 
   updateCurrent $i
}

proc selectTitle { mode w yPix } \
{
   global objInfo

   set jList [$w curselection]
   set iList ""

   foreach j $jList {lappend iList [expr $j + 1]}

   selectTitle2 $mode $iList
   raiseSelMarkers

   updateSpecViewPCADrawList
}

proc selectTitle2 { mode iList } \
{
   global objInfo

   if {![strcasecmp $mode ref]} \
      {
       set objInfo(data,prevRefList) ""

       clearRefSelection
       set objInfo(data,refList) $iList 
      } \
   else \
      {
       set objInfo(data,prevSpecList) ""
       set objInfo(data,drawList) $iList 
      }

   joinRefList
   updateTitleSelection

   showViewAll selectTitle
   drawTitles
}

proc toggleTitle { mode w yPix } \
{
   global objInfo

   set j [$w nearest $yPix]
   set i [expr $j + 1]

   toggleTitle2 $mode $i
   raiseSelMarkers

   updateSpecViewPCADrawList
}

proc toggleTitle2 { mode i } \
{
   global objInfo

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   if {![strcasecmp $mode ref]} \
      {
       set loc [lsearch -exact $objInfo(data,refList) $i]

       if {$loc < 0} \
          {
           lappend objInfo(data,refList) $i
          } \
       else \
          {
           set objInfo(data,refList) [lreplace $objInfo(data,refList) $loc $loc]

           set loc [lsearch -exact $objInfo(data,drawList) $i]

           if {$loc >= 0} \
              {
               set objInfo(data,drawList) \
                [lreplace $objInfo(data,drawList) $loc $loc]
              } 
          }
      } \
   else \
      {
       set loc [lsearch -exact $objInfo(data,drawList) $i]
    
       if {$objInfo(data,drawFlag$i)} \
          {
           set objInfo(data,drawFlag$i) 0

           if {$loc >= 0} \
              {
               set objInfo(data,drawList) \
                [lreplace $objInfo(data,drawList) $loc $loc]
              }
          } \
       else \
          {
           set objInfo(data,drawFlag$i) 1
    
           if {$loc < 0} {lappend objInfo(data,drawList) $i}
          }
      }

   joinRefList
   updateTitleSelection

   showViewAll toggleTitle
   drawTitles
}

proc clearRefSelection {} \
{
   global objInfo

   foreach i $objInfo(data,refList) \
      {
       set j [expr $i - 1]

       .rlb selection clear $j
       .tlb selection clear $j

       set objInfo(data,drawFlag$i) 0
      }

   set iList ""

   foreach i $objInfo(data,drawList) \
      {
       if {[lsearch -exact $objInfo(data,refList) $i] < 0} {lappend iList $i} 
      }

   set objInfo(data,drawList) $iList
}

proc joinRefList {} \
{
   global objInfo

   set n     [llength $objInfo(data,refList)]
   set iList ""

   if {$n} {lappend iList [lindex $objInfo(data,refList) 0]}

   foreach i $objInfo(data,drawList) \
      {
       if {[lsearch -exact $iList $i] < 0} {lappend iList $i} 
      }

   for {set j 1} {$j < $n} {incr j} \
      {
       set i [lindex $objInfo(data,refList) $j]
       if {[lsearch -exact $iList $i] < 0} {lappend iList $i} 
      }

   set objInfo(data,drawList) $iList
}

proc incrSelectID { mode } \
{
   global objInfo

   set oldID $objInfo(data,currentID)
   set iList ""

   if {![strcasecmp [string index $mode 0] d]} \
      {
       incr objInfo(data,currentID) -1
      } \
   else \
      {
       incr objInfo(data,currentID)  1
      }

   if {$objInfo(data,currentID) < 1} {set objInfo(data,currentID) $objInfo(data,specCount)}
   if {$objInfo(data,currentID) > $objInfo(data,currentID)} {set objInfo(data,currentID) 1}

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$i == $oldID} continue

       if {$i == $objInfo(data,currentID) \
           || [lsearch -exact $objInfo(data,drawList) $i] >= 0} \
          {
           lappend iList $i
          }
      }

   set objInfo(data,drawList) $iList

   updateTitleSelection

   showViewAll selectTitle
   drawTitles
}

proc incrSelection { mode step } \
{
   global objInfo

   if {![strcasecmp $mode spec]} \
      {
       set w .tlb
      } \
   else \
      {
       set w .rlb
      }

   set jList [$w curselection]
   if {![llength $jList]} return

   set j [lindex $jList end]
   set k [expr $j + $step + 1]

   if {$k < 1} {set k $objInfo(data,specCount)}
   if {$k > $objInfo(data,specCount)} {set k 1}

   set jList [lrange $jList 0 end-1]
   set iList ""
 
   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$i == $k || [lsearch -exact $jList [expr $i - 1]] >= 0} \
          {
           lappend iList $i
          }
      }

   selectTitle2 $mode $iList
   updateSpecViewPCADrawList
}

proc updateTitleSelection {} \
{
   global objInfo

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set objInfo(data,drawFlag$i) 0
      }

   foreach i $objInfo(data,drawList) \
      {
       set objInfo(data,drawFlag$i) 1
      }

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set loc [lsearch -exact $objInfo(data,refList) $i]
       set j   [expr $i - 1]

       if {$loc >= 0} \
          {
           .rlb selection set   $j
           .tlb selection clear $j
          } \
       else \
          {
           .rlb selection clear $j

           if {$objInfo(data,drawFlag$i)} \
              {
               .tlb selection set   $j
              } \
           else \
              {
               .tlb selection clear $j
              }
          }
      }
}

proc setNormRegion { obj sx1 sxn sy1 syn } \
{
   global objInfo xUnits yUnits

   showVerb "Selecting Normalize Region $sx1 $sxn for window $obj"

   if {!$objInfo(data,scaleID)} {set objInfo(data,scaleID) [getNextID]}

   foreach i $objInfo(data,drawList) \
      {
       set objInfo(data,scale$i) 1.0

       if {$objInfo(data,$i,dimCount) == 1} \
          {
           readROI -roi $objInfo(data,scaleID) \
                   -inROI $objInfo($obj,roi$i) -ndim 1 \
                   -x X_AXIS ${sx1}$xUnits ${sxn}$xUnits
          } \
       else \
          {
           readROI -roi $objInfo(data,scaleID)           \
                   -inROI $objInfo($obj,roi$i) -ndim 2   \
                   -x X_AXIS ${sx1}$xUnits ${sxn}$xUnits \
                   -y Y_AXIS ${sy1}$yUnits ${syn}$yUnits
           }

       set sum    [getROIStat $objInfo(data,scaleID) vSum]
       set maxVal [getROIStat $objInfo(data,scaleID) vMax]
       set n      [getROISize $objInfo(data,scaleID)]

       if {$maxVal != 0.0} {set objInfo(data,scale$i) [expr 1.0e+7/$maxVal]}

       showVerb "Scale $i Sum: sum Max: $maxVal N: $n Limits: $sx1 $sxn"

       set fmt " %4d %.4e %.4e %4d %8.3f %8.3f %s"

       puts [format $fmt $i $sum $maxVal $n $sx1 $sxn [split $objInfo($obj,inName$i) "_"]]
      }

   foreach obj $objInfo(prog,winList) \
      {
       readView $obj
       showView2 $obj scaleView
      }

   drawTitles
}

proc setSpecWindowTitle { w } \
{
    global objInfo

    wm title $w "$objInfo(prog,title)  \[Zoom\] \[Scale\] \[Offset\] Right-Click on Title to Adjust Order"
    update
}

proc initSpecWindow { parent sx1 sxn sy1 syn } \
{
    global objInfo env ARGV
    global glbWinXAdj glbWinYAdj

    getWinAdj

    set id  [getNextID]
    set obj winND_$id
    set w   .winND_$id

    showVerb "Initializing window $id"

    toplevel $w

    incr objInfo(prog,winCount)

    setSpecWindowTitle $w

    lappend objInfo(prog,winList) $obj

    set objInfo($obj,parent) $parent

    if {![strcasecmp $objInfo(prog,defXSizeUser) None]} \
       {
        set objInfo($obj,winXSize) $objInfo(prog,defXSize)
       } \
    else \
       {
        set objInfo($obj,winXSize) $objInfo(prog,defXSizeUser)
       }

    if {![strcasecmp $objInfo(prog,defYSizeUser) None]} \
       {
        set objInfo($obj,winYSize) $objInfo(prog,defYSize) 
       } \
    else \
       {
        set objInfo($obj,winYSize) $objInfo(prog,defYSizeUser) 
       }

    if {$objInfo(data,1,dimCount) == 1} \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin)  [expr 6*$objInfo(prog,axisCharY)] 
        set objInfo($obj,rightMargin)   [expr 4*$objInfo(prog,axisCharX)] 
       } \
    else \
       {
        set objInfo($obj,leftMargin)    20 
        set objInfo($obj,topMargin)     20
        set objInfo($obj,bottomMargin)  [expr 6*$objInfo(prog,axisCharY)] 
        set objInfo($obj,rightMargin)   [expr 12*$objInfo(prog,axisCharX)] 
       }

    set objInfo($obj,winXAdj)    $glbWinXAdj 
    set objInfo($obj,winYAdj)    $glbWinYAdj 

    getArgD $ARGV -leftMargin   objInfo($obj,leftMargin)
    getArgD $ARGV -rightMargin  objInfo($obj,rightMargin)
    getArgD $ARGV -topMargin    objInfo($obj,topMargin)
    getArgD $ARGV -bottomMargin objInfo($obj,bottomMargin)

    getArgD $ARGV -winXAdj      objInfo($obj,winXAdj)
    getArgD $ARGV -winYAdj      objInfo($obj,winYAdj)

    set objInfo($obj,cellXSize) \
        [expr $objInfo($obj,winXSize)     - \
              ($objInfo($obj,leftMargin)  + \
               $objInfo($obj,rightMargin) + \
               $objInfo($obj,winXAdj))]

    set objInfo($obj,cellYSize) \
        [expr $objInfo($obj,winYSize)      - \
              ($objInfo($obj,topMargin)    + \
               $objInfo($obj,bottomMargin) + \
               $objInfo($obj,winYAdj))]
 
    set objInfo($obj,xSize) [expr $objInfo($obj,winXSize) - $objInfo($obj,winXAdj)]
    set objInfo($obj,ySize) [expr $objInfo($obj,winYSize) - $objInfo($obj,winYAdj)]

    set objInfo($obj,window)       $w
    set objInfo($obj,canvas)       $w.c
    set objInfo($obj,canvasSelV)   $w.cSelV
    set objInfo($obj,canvasSelH)   $w.cSelH
    set objInfo($obj,scrollH)      $w.sh
    set objInfo($obj,scrollV)      $w.sv
    set objInfo($obj,pixmap)       $id 
    set objInfo($obj,cell)         $id 
    set objInfo($obj,graphicList)  ""
    set objInfo($obj,moveList)     ""
    set objInfo($obj,currentNote)  ""

    set objInfo($obj,resizeProc)   resizeView
    set objInfo($obj,redrawProc)   showView

    set objInfo($obj,x1) $objInfo($obj,leftMargin)
    set objInfo($obj,xn) [expr $objInfo($obj,x1) + $objInfo($obj,cellXSize)]
    set objInfo($obj,y1) $objInfo($obj,topMargin)
    set objInfo($obj,yn) [expr $objInfo($obj,y1) + $objInfo($obj,cellYSize)]

    set xPix [expr ($objInfo($obj,x1) + $objInfo($obj,xn))/2]
    set yPix [expr ($objInfo($obj,y1) + $objInfo($obj,yn))/2]

    set objInfo($obj,x1Pix)        $xPix
    set objInfo($obj,y1Pix)        $yPix
    set objInfo($obj,xnPix)        $xPix
    set objInfo($obj,ynPix)        $yPix
    set objInfo($obj,x0Pix)        -1 
    set objInfo($obj,y0Pix)        -1

    set objInfo($obj,specCount)    $objInfo($parent,specCount)
    set objInfo($obj,inList)       $objInfo($parent,inList)

    set n $objInfo($parent,specCount)
 
    if {$n < 2} {set n 2}

    set objInfo($obj,shiftX,currentID) 0
    set objInfo($obj,shiftY,currentID) 0

    for {set i 1} {$i <= $n} {incr i} \
       {
        set objInfo($obj,hi$i)      $objInfo($parent,hi$i)
        set objInfo($obj,fhi$i)     $objInfo($parent,fhi$i)
        set objInfo($obj,lhi$i)     $objInfo($parent,lhi$i)
        set objInfo($obj,inName$i)  $objInfo($parent,inName$i)

        set objInfo($obj,roiCopy,$i) 0
        set objInfo($obj,shiftX,$i)  0
        set objInfo($obj,shiftY,$i)  0
        set objInfo($obj,p0,$i)      0.0
        set objInfo($obj,p1,$i)      0.0
        set objInfo($obj,pivot,$i)   0.0
       }

    set objInfo($obj,xAxis)        $objInfo($parent,xAxis)

    set objInfo($obj,specX1)       $sx1
    set objInfo($obj,specXN)       $sxn
    set objInfo($obj,specY1)       $sy1
    set objInfo($obj,specYN)       $syn

    updateSpecZoomVal $obj

    set objInfo($obj,fhi1)         $sy1
    set objInfo($obj,lhi1)         $syn

    set objInfo($obj,showPeakLabel) $objInfo($parent,showPeakLabel) 
    set objInfo($obj,showAssLabel)  $objInfo($parent,showAssLabel) 
    set objInfo($obj,useTitleLabel) $objInfo($parent,useTitleLabel) 
    set objInfo($obj,useFileLabel)  $objInfo($parent,useFileLabel) 
    set objInfo($obj,newWindow)     $objInfo($parent,newWindow) 
    set objInfo($obj,difMode)       $objInfo($parent,difMode) 
    set objInfo($obj,useAxis)       $objInfo($parent,useAxis)
    set objInfo($obj,drawSpectra)   $objInfo($parent,drawSpectra)
    set objInfo($obj,pcaFlag)       0

    wm title $objInfo($obj,window) "$objInfo(prog,title) \[Zoom\] \[Scale\] \[Offset\] Right-Click on Title to Adjust Order"

    frame $w.top1
    frame $w.top2
    frame $w.topLeft
    frame $w.topMid
    frame $w.topMidTop
    frame $w.topMidBot
    frame $w.topRight
    frame $w.topRightTop
    frame $w.topRightBot
    frame $w.topCmnd
    frame $w.topCmndTop
    frame $w.topCmndBot
    frame $w.topAll
    frame $w.middle
    frame $w.bottom

    catch {image delete gearImage}

    image create photo gearImage -file $env(NMRTXT)/gear22.gif

    if {[flagLoc $ARGV -captureAll]} \
       {
        button $w.snap      -bitmap @$env(NMRTXT)/cameraIcon.xbm    -command "captureAllSpectra" -relief $objInfo(prog,button,relief) 
       } \
    else \
       {
        button $w.snap      -bitmap @$env(NMRTXT)/cameraIcon.xbm    -command "captureScreen  $obj" -relief $objInfo(prog,button,relief) 
       }

    button $w.settings -image gearImage -command "showSpecViewSettings $obj" -relief $objInfo(prog,button,relief)

    label  $w.snapSpace -width 1 -text " " 

    button $w.full2D   -text "Full 2D"       -command "fullView  $obj 2D"    -relief $objInfo(prog,button,relief) 
    button $w.fullH    -text "Full Horiz"    -command "fullView  $obj H"     -relief $objInfo(prog,button,relief) 
    button $w.fullV    -text "Full Vert"     -command "fullView  $obj V"     -relief $objInfo(prog,button,relief) 
    button $w.sortDown -text "Sort Down"     -command "sortView  $obj down"  -relief $objInfo(prog,button,relief) 
    button $w.sortUp   -text "Sort Up"       -command "sortView  $obj up"    -relief $objInfo(prog,button,relief) 
    button $w.scale    -text "Scale"         -command "scaleView $obj one"   -relief $objInfo(prog,button,relief) 
    button $w.scaleAll -text "Scale All"     -command "scaleView $obj all"   -relief $objInfo(prog,button,relief) 
    button $w.stack    -text "Stack"         -command "scaleView $obj stack" -relief $objInfo(prog,button,relief) 
    button $w.reset    -text "Reset"         -command "scaleView $obj reset" -relief $objInfo(prog,button,relief) 

    button $w.nmrDraw  -text "NMRDraw"       -command "startNMRDraw $obj"    -relief $objInfo(prog,button,relief) 

    button $w.zoom1    -text "Z1"  -command "setUserZoom $obj 1"    -relief $objInfo(prog,button,relief) 
    button $w.zoom2    -text "Z2"  -command "setUserZoom $obj 2"    -relief $objInfo(prog,button,relief) 
    button $w.zoom3    -text "Z3"  -command "setUserZoom $obj 3"    -relief $objInfo(prog,button,relief) 
    button $w.zoom4    -text "Z4"  -command "setUserZoom $obj 4"    -relief $objInfo(prog,button,relief) 
    button $w.zoomN    -text "ZN"  -command "setUserZoom $obj Next" -relief $objInfo(prog,button,relief) 
    button $w.zoomP    -text "ZP"  -command "setUserZoom $obj Prev" -relief $objInfo(prog,button,relief) 

    if {[strcasecmp $objInfo(prog,noteName) None]} \
       {
        label $w.noteLabel -text "Note 0:"  -width 9 -justify right -anchor e
        label $w.noteSpace -text " " -width 1

        entry $w.note -width 24 -textvariable objInfo($obj,currentNote)

        bind  $w.note <KeyRelease> "+ applyCurrentNote $obj nosave"
        bind  $w.note <Return>     "+ applyCurrentNote $obj save"
       }

    if {$objInfo(prog,zoomRegMode)} \
       {
        label $w.zoomXSpace -width 1 -text " " 
        label $w.zoomYSpace -width 1 -text " "

        entry $w.zoomX1 -width 11 -textvariable objInfo($obj,specX1Val) \
             -validate key -validatecommand "rejectEntryWhitespace %S"

        entry $w.zoomXN -width 11 -textvariable objInfo($obj,specXNVal) \
             -validate key -validatecommand "rejectEntryWhitespace %S"

        if {$objInfo(data,1,dimCount) == 1} \
           {
            label $w.zoomY1 -width 1 -text " "
            label $w.zoomYN -width 1 -text " "
           } \
        else \
           {
            entry $w.zoomY1 -width 11 -textvariable objInfo($obj,specY1Val) 
            entry $w.zoomYN -width 11 -textvariable objInfo($obj,specYNVal)

            bind $w.zoomY1 <ButtonRelease-3> "+ incrZoom $obj Y -1"
            bind $w.zoomYN <ButtonRelease-3> "+ incrZoom $obj Y +1"
           }
       }

    if {[strcmp $objInfo(peak,tab1) 0]} \
       {
        label $w.peakSpace1  -text " " -width 1
        label $w.peakIDLabel -text "Peak ID:"
        entry $w.peakIDEntry -textvariable objInfo(peak,currentID) -width 8

        label $w.peakSpace2   -text " " -width 1
        label $w.peakAssLabel -text "Peak Assignment:"
        entry $w.peakAssEntry -textvariable objInfo(peak,currentAss) -width 10 \
                              -validate key -validatecommand "rejectEntryWhitespace %S"

        label $w.peakSpace3  -text " " -width 11 
        label $w.assAssLabel -text "Selected Assignment: "

        label $w.assAssEntry -textvariable objInfo(peak,selectedAss) -foreground $objInfo(color,selectedAss)

        bind $w.peakIDEntry  <Return>     "+ setCurrentPeakID"
        bind $w.peakAssEntry <KeyRelease> "+ setCurrentAss %K"
       }

    if {$objInfo(data,shiftNDFlag) && $objInfo(data,1,dimCount) == 1} \
       {
        label $w.shiftXSpace -width 1 -text " " 

        set size 131072
 
        spinbox $w.shiftX -from -$size -to $size \
               -textvariable objInfo($obj,shiftX,currentID) \
               -command  "applyShiftX $obj $id"    \
               -increment 1 -format %.0f -width 6

        bind  $w.shiftX <Return> "+ applyShiftX $obj $id"
       } \
    elseif {$objInfo(data,shiftNDFlag) && $objInfo(data,1,dimCount) >= 2} \
       {
        label $w.shiftXSpace -width 1 -text " "

        set size 8192 

        spinbox $w.shiftX -from -$size -to $size \
               -textvariable objInfo($obj,shiftX,currentID) \
               -command  "applyShiftX $obj $id"    \
               -increment 1 -format %.0f -width 6

        spinbox $w.shiftY -from -$size -to $size \
               -textvariable objInfo($obj,shiftY,currentID) \
               -command  "applyShiftY $obj $id"    \
               -increment 1 -format %.0f -width 6

        bind  $w.shiftX <Return> "+ applyShiftX $obj $id"
        bind  $w.shiftY <Return> "+ applyShiftY $obj $id"
       }

    if {$objInfo(prog,graphItemMode)} \
       {
        button $w.instext -bitmap @$env(NMRTXT)/shapeT.xbm     -fg darkblue -command "insertGraphItem  text      $obj" -relief $objInfo(prog,button,relief) 
        button $w.insrect -bitmap @$env(NMRTXT)/shapeRect.xbm  -fg darkblue -command "insertGraphItem  rectangle $obj" -relief $objInfo(prog,button,relief) 
        button $w.instri  -bitmap @$env(NMRTXT)/shapeTri.xbm   -fg darkblue -command "insertGraphItem  triangle  $obj" -relief $objInfo(prog,button,relief) 
        button $w.insvec  -bitmap @$env(NMRTXT)/shapeVec.xbm   -fg darkblue -command "insertGraphItem  vector    $obj" -relief $objInfo(prog,button,relief) 
        button $w.insspec -bitmap @$env(NMRTXT)/shapeSpec.xbm  -fg darkblue -command "insertGraphItem  spectrum  $obj" -relief $objInfo(prog,button,relief) 
       }

    if {$objInfo(prog,usePCA)} \
       {
        button $w.pca      -text "PCA"   -command "specViewPCA   $obj" -relief $objInfo(prog,button,relief) 
        label  $w.pcaSpace -text " "  -width 1
       }

    if {$objInfo(data,shiftNDFlag)} \
       {
        button $w.shift -text "Shift" -command "specViewShift $obj" -relief $objInfo(prog,button,relief) 
       }

    button $w.help -text "Help" -command "showHelp $obj" -relief $objInfo(prog,button,relief) 

    if {[strcasecmp $objInfo(prog,exportProc) None]} \
       {
        button $w.export -text "Export" -command "exportView $obj" -relief $objInfo(prog,button,relief) 
       }

    label  $w.qSpace -width 1 -text " " 
    button $w.clear  -text "Clear" -command "clearThisView  $obj main" -relief $objInfo(prog,button,relief) 
    button $w.quit   -text "Quit"  -command "quitThisView   $obj main" -relief $objInfo(prog,button,relief) 

    label  $w.dif -text "    Difference:"

    radiobutton $w.condif -text "Vector"   -value 3 -variable objInfo($obj,difMode) -anchor w -command "setDifMode $obj"
    radiobutton $w.shdif  -text "Shaded"   -value 2 -variable objInfo($obj,difMode) -anchor w -command "setDifMode $obj"
    radiobutton $w.intdif -text "Internal" -value 1 -variable objInfo($obj,difMode) -anchor w -command "setDifMode $obj"
    radiobutton $w.nodif  -text "None"     -value 0 -variable objInfo($obj,difMode) -anchor w -command "setDifMode $obj"

    label  $w.zmode -text "    Zoom Window:"

    radiobutton $w.znew  -text "New"  -value 1 \
       -variable objInfo($obj,newWindow) -anchor w

    radiobutton $w.zsame -text "Same" -value 0 \
       -variable objInfo($obj,newWindow) -anchor w

    label  $w.lmode -text "  Label:"

    checkbutton $w.ltitle -text "Title" \
     -variable objInfo($obj,useTitleLabel) -anchor w -command "setLabelMode $obj title"

    checkbutton $w.lfile -text "File"   \
     -variable objInfo($obj,useFileLabel) -anchor  w -command "setLabelMode $obj file"

    if {$objInfo(prog,pkFlag)} \
       {
        checkbutton $w.lpeak -text "Peaks"   \
         -variable objInfo($obj,showPeakLabel) -anchor  w -command "setLabelMode $obj peak"
       }

    if {$objInfo(prog,assFlag)} \
       {
        checkbutton $w.lass -text "Assignments"   \
         -variable objInfo($obj,showAssLabel)  -anchor  w -command "setLabelMode $obj ass"
       }

    if {$objInfo(prog,extendedMode)} \
       {
        label  $w.luspec -text "  Spectra:"

        checkbutton $w.uspec -text "" -selectcolor white \
         -variable objInfo($obj,drawSpectra) -anchor w   \
         -command "setDrawSpectraMode $obj"

        set uAxisStr "Axis:"

        label $w.lfreeze -text "  Freeze:"

        checkbutton $w.freezeSpec -text "Spectra" \
         -variable objInfo(prog,freezeSpectra) -anchor w \
         -selectcolor red

        checkbutton $w.freezeGraph -text "Graphics" \
         -variable objInfo(prog,freezeGraphic) -anchor w \
         -selectcolor red
       } \
    else \
       {
        set uAxisStr "  Axis:"
       }

    label  $w.luaxis -text $uAxisStr

    checkbutton $w.uaxis -text "" -selectcolor white \
     -variable objInfo($obj,useAxis) -anchor w -command "setAxisMode $obj"

    set color $objInfo(color,color1) 

    set objInfo($obj,activeSpec) "1 ($color)"

    if {$objInfo(data,1,dimCount) == 1} \
       {
        pack \
         $w.snap $w.snapSpace $w.clear $w.fullH $w.fullV $w.sortDown $w.sortUp $w.scale \
         $w.scaleAll $w.reset $w.stack $w.zoom1 $w.zoom2 $w.zoom3 $w.zoom4 \
         $w.zoomN $w.zoomP -in $w.top1 -side left 
       } \
    else \
       {
        pack \
         $w.snap $w.snapSpace $w.clear $w.full2D \
         $w.reset $w.zoom1 $w.zoom2 $w.zoom3 $w.zoom4 \
         $w.zoomN $w.zoomP -in $w.top1 -side left 
       }

    if {$objInfo(prog,zoomRegMode)} \
       {
        pack $w.zoomXSpace $w.zoomX1 $w.zoomXN -in $w.topMidTop -side left
        pack $w.zoomYSpace $w.zoomY1 $w.zoomYN -in $w.topMidBot -side left
       }

    if {[strcmp $objInfo(peak,tab1) 0]} \
       {
        pack $w.peakSpace1  $w.peakIDLabel  $w.peakIDEntry  \
             $w.peakSpace2  $w.peakAssLabel $w.peakAssEntry -in $w.topMidTop -side left

        pack $w.peakSpace3 $w.assAssLabel $w.assAssEntry -in $w.topMidBot -side left 
       }

    if {$objInfo(data,shiftNDFlag) && $objInfo(data,1,dimCount) == 1} \
       {
        pack $w.shiftXSpace $w.shiftX -in $w.topMidTop -side left
       } \
    elseif {$objInfo(data,shiftNDFlag) && $objInfo(data,1,dimCount) >= 2} \
       {
        pack $w.shiftXSpace $w.shiftX $w.shiftY -in $w.topMidTop -side left
       }

    if {$objInfo(prog,graphItemMode)} \
       {
        pack $w.instext $w.insrect $w.instri $w.insvec -in $w.topRightTop -side left 
       }

    pack $w.nmrDraw -in $w.top1 -side left

    if {$objInfo(prog,usePCA)} \
       {
        if {$objInfo(prog,altPCAFlag)} \
           {
            pack $w.pcaSpace $w.pca $w.settings -in $w.top1 -side left
           } \
        else \
           {
            pack $w.pca -in $w.top1 -side left
           }
       }

    if {$objInfo(data,shiftNDFlag)} {pack $w.shift -in $w.top1 -side left}

    if {[strcasecmp $objInfo(prog,exportProc) None]} \
       {
        pack $w.export -in $w.topRightTop -side left 
       }

    if {[strcasecmp $objInfo(prog,noteName) None]} \
       {
        pack $w.noteLabel $w.note $w.noteSpace -in $w.top2 -side left
       }

    if {$objInfo(data,1,dimCount) == 1} \
       { 
        pack $w.dif $w.condif $w.shdif $w.intdif $w.nodif -in $w.top2 -side left
       }

    if {$objInfo(prog,extendedMode)} \
       {
        pack \
         $w.zmode $w.znew $w.zsame $w.luspec $w.uspec $w.luaxis $w.uaxis     \
         $w.lmode $w.ltitle $w.lfile -in $w.top2 -side left 

        if {$objInfo(prog,pkFlag)}  {pack $w.lpeak -in $w.top2 -side left}
        if {$objInfo(prog,assFlag)} {pack $w.lass  -in $w.top2 -side left}

        pack $w.lfreeze $w.freezeSpec $w.freezeGraph -in $w.topRightBot -side left 
       } \
    else \
       {
        pack \
         $w.zmode $w.znew $w.zsame $w.luaxis $w.uaxis \
         $w.lmode $w.ltitle $w.lfile -in $w.top2 -side left 

        if {$objInfo(prog,pkFlag)}  {pack $w.lpeak -in $w.top2 -side left}
        if {$objInfo(prog,assFlag)} {pack $w.lass  -in $w.top2 -side left}
       }

   pack $w.help $w.quit -in $w.topCmndTop -side right 

   pack $w.qSpace -in $w.topCmndBot -side left
   pack $w.topCmndTop $w.topCmndBot -in $w.topCmnd -side top
   pack $w.topCmnd -in $w.topRight  -side right 

#
# Canvas and scrollbars:

    scrollbar $objInfo($obj,scrollH) \
       -orient horiz -command "$objInfo($obj,canvas) xview"

    canvas $objInfo($obj,canvas) -border 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize)      \
       -xscrollcommand "$objInfo($obj,scrollH) set"                  \
       -scrollregion "0 0 $objInfo($obj,xSize) $objInfo($obj,ySize)" \
       -closeenough 12 

    if {$objInfo(data,shiftNDFlag)} \
       {
        frame $w.canvasSelHFrame

        canvas $w.canvasSelSpace -border 0 \
           -width  [expr 3 + $objInfo(prog,canvasSelXW)] \
           -height $objInfo(prog,canvasSelXW)

        canvas $objInfo($obj,canvasSelH) -border 0 \
           -height $objInfo(prog,canvasSelXW) -width $objInfo($obj,xSize) \
           -background $objInfo(color,winBG) -closeenough 6

        pack $w.canvasSelSpace -in $w.canvasSelHFrame -side left 
        pack $objInfo($obj,canvasSelH) -in $w.canvasSelHFrame -expand 1 -fill x
       }

    canvas $objInfo($obj,canvasSelV) -border 0 \
       -width $objInfo(prog,canvasSelXW) -height $objInfo($obj,ySize) \
       -background $objInfo(color,winBG) -closeenough 6

    set dx [expr 5*$objInfo(prog,winCount)]
    set dy [expr 5*$objInfo(prog,winCount)]

    wm geom  $objInfo($obj,window) +$dx+$dy

    xupdate

#   pack $objInfo($obj,scrollH) \
#    -in $w.bottom -side bottom -expand 0 -fill x

    pack $objInfo($obj,canvasSelV) \
     -in $w.middle -side left   -expand 0 -fill y -padx 2

    pack $objInfo($obj,canvas) \
     -in $w.middle -side left -expand 1 -fill both

    pack $w.topMidTop $w.topMidBot -in $w.topMid -side top -expand 1 -fill x
    pack $w.topRightTop $w.topRightBot -in $w.topRight -side top
    pack $w.top1 $w.top2 -in $w.topLeft -side top -expand 0 -fill x
    pack $w.topLeft $w.topMid -in $w.topAll -side left
    pack $w.topRight -in $w.topAll -side left -expand 1 -fill x
    pack $w.topAll -side top -expand 0 -fill x

    if {$objInfo(data,shiftNDFlag)} \
       {
        pack $w.canvasSelHFrame -side top -expand 1 -fill x
       }

    pack $w.middle -in $w.bottom -side bottom -expand 1 -fill both
    pack $w.bottom -in $w        -side top    -expand 1 -fill both

    xupdate

    $objInfo($obj,canvas) create pixmap 0 0 \
       -width $objInfo($obj,xSize) -height $objInfo($obj,ySize) \
       -win $objInfo($obj,pixmap) -tags $obj

    defineCell -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x $objInfo($obj,x1) $objInfo($obj,xn) \
               -y $objInfo($obj,y1) $objInfo($obj,yn)

    set qx [expr $objInfo($obj,x1) - 4] 
    set qy [expr $objInfo($obj,y1) - 4] 

    set idH [$objInfo($obj,canvas) create line     \
       $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy \
       -fill $objInfo(color,cursor) -tags cursorH]

    set idV [$objInfo($obj,canvas) create line     \
       $qx $objInfo($obj,y1) $qx $objInfo($obj,yn) \
       -fill $objInfo(color,cursor) -tags cursorV]

    readView $obj
    showView $obj init

    if {$objInfo(prog,zoomRegMode)} \
       {
        if {$objInfo(data,1,dimCount) >= 1} \
           {
            bind $w.zoomX1 <Return> "+ setUserSpecZoom $obj specX1"
            bind $w.zoomXN <Return> "+ setUserSpecZoom $obj specXN"

            bind $w.zoomX1 <ButtonRelease-3> "+ incrZoom $obj X -1"
            bind $w.zoomXN <ButtonRelease-3> "+ incrZoom $obj X +1"
           }

        if {$objInfo(data,1,dimCount) >= 2} \
           {
            bind $w.zoomY1 <Return> "+ setUserSpecZoom $obj specY1"
            bind $w.zoomYN <Return> "+ setUserSpecZoom $obj specYN"

            bind $w.zoomY1 <ButtonRelease-3> "+ incrZoom $obj Y -1"
            bind $w.zoomYN <ButtonRelease-3> "+ incrZoom $obj Y +1"
           }
       }

 bind $objInfo($obj,canvas) <Configure> "$objInfo($obj,resizeProc) $obj %w %h %T"

 bind $objInfo($obj,canvas) <Shift-ButtonPress-1>   "mouseON    $obj %x %y B1 1 0"
 bind $objInfo($obj,canvas) <Shift-ButtonRelease-1> "mouseDone  $obj %x %y B1 1 0"
 bind $objInfo($obj,canvas) <Shift-B1-Motion>       "trackMouse $obj %x %y B1 1 0"

 bind $objInfo($obj,canvas) <Shift-ButtonPress-2>   "mouseON    $obj %x %y B2 1 0"
 bind $objInfo($obj,canvas) <Shift-ButtonRelease-2> "mouseDone  $obj %x %y B2 1 0"
 bind $objInfo($obj,canvas) <Shift-B2-Motion>       "trackMouse $obj %x %y B2 1 0"

 bind $objInfo($obj,canvas) <Shift-ButtonPress-3>   "mouseON    $obj %x %y B3 1 0"
 bind $objInfo($obj,canvas) <Shift-ButtonRelease-3> "mouseDone  $obj %x %y B3 1 0"
 bind $objInfo($obj,canvas) <Shift-B3-Motion>       "trackMouse $obj %x %y B3 1 0"

 bind $objInfo($obj,canvas) <Control-ButtonPress-3>   "mouseON    $obj %x %y B3 0 1"
 bind $objInfo($obj,canvas) <Control-ButtonRelease-3> "mouseDone  $obj %x %y B3 0 1"
 bind $objInfo($obj,canvas) <Control-B3-Motion>       "trackMouse $obj %x %y B3 0 1"

 bind $objInfo($obj,canvas) <ButtonPress-1>         "mouseON    $obj %x %y B1 0 0"
 bind $objInfo($obj,canvas) <ButtonRelease-1>       "mouseDone  $obj %x %y B1 0 0"
 bind $objInfo($obj,canvas) <B1-Motion>             "trackMouse $obj %x %y B1 0 0"

 bind $objInfo($obj,canvas) <ButtonPress-2>         "mouseON    $obj %x %y B2 0 0"
 bind $objInfo($obj,canvas) <ButtonRelease-2>       "mouseDone  $obj %x %y B2 0 0"
 bind $objInfo($obj,canvas) <B2-Motion>             "trackMouse $obj %x %y B2 0 0"

 bind $objInfo($obj,canvas) <ButtonPress-3>         "mouseON    $obj %x %y B3 0 0"
 bind $objInfo($obj,canvas) <ButtonRelease-3>       "mouseDone  $obj %x %y B3 0 0"
 bind $objInfo($obj,canvas) <B3-Motion>             "trackMouse $obj %x %y B3 0 0"
 bind $objInfo($obj,canvas) <Button-4>              "mouseON $obj %x %y B4 0 0"
 bind $objInfo($obj,canvas) <Button-5>              "mouseON $obj %x %y B5 0 0"

 bind $objInfo($obj,canvas) <KeyPress>              "+ parseKey $obj %K %x %y"
 bind $objInfo($obj,canvas) <Any-Enter>             "+ setSpecWindowTitle $objInfo($obj,window)"

# bind $objInfo($obj,canvas) <B1-Motion> "moveCursor $obj %x %y"

#
# X adjustment only needed when window is established:

  set objInfo($obj,winXAdj) 0
# set objInfo($obj,winYAdj) 0 

 resizeView $obj

 updateAssWindow
}

proc rejectEntryWhitespace s \
{
  if {![strcmp $s " "]} {return 0}
  return 1
}

proc parseKey { objOrig kStr xPix yPix } \
{
   global keyInfo objInfo

   set keyInfo(score0)   0
   set keyInfo(score1)   1
   set keyInfo(score2)   2
   set keyInfo(score3)   3
   set keyInfo(score4)   4
   set keyInfo(score5)   5

   set keyInfo(nextSpec) Up
   set keyInfo(prevSpec) Down

   set obj $objOrig

   if {![strcasecmp $objOrig main]} \
      {
       set objInfo($objOrig,eventX) $xPix
       set objInfo($objOrig,eventY) $yPix

       foreach thisObj [listReverse $objInfo(prog,winList)] \
          {
           if {![winfo exists $objInfo($thisObj,window)]} continue
           set obj $thisObj
           break
          } 
      }

   showVerb "parseKey  $objOrig $obj |$kStr| $xPix $yPix"

   if {![info  exists objInfo($obj,window)]}  return
   if {![winfo exists $objInfo($obj,window)]} return

   set objInfo($obj,eventX) $xPix
   set objInfo($obj,eventY) $yPix

   switch -exact $kStr \
      {
       0 {
          set objInfo(data,currentScore) 0
          setCurrentScore
         }
 
       1 {
          set objInfo(data,currentScore) 1
          setCurrentScore
         }

       2 {
          set objInfo(data,currentScore) 2
          setCurrentScore
         }

       3 {
          set objInfo(data,currentScore) 3
          setCurrentScore
         }

       4 {
          set objInfo(data,currentScore) 4
          setCurrentScore
         }

       5 {
          set objInfo(data,currentScore) 5
          setCurrentScore
         }

       Down  {incrSelection spec  1}
       Up    {incrSelection spec -1}
       Left  {incrSelection ref   1}
       Right {incrSelection ref  -1}

       a   {scaleView $obj all}
       b   {}
       c   {clearDisplay $obj}
       d   {sortView     $obj down}
       e   {}
       f   {fullView    $obj 2D}
       g   {scaleView   $obj reset}
       h   {fullView    $obj H}
       i   {toggleInteg $obj}
       j   {}
       k   {scaleView $obj stack}
       l   {}
       m   {}
       n   {setUserZoom $obj Next}
       o   {}
       p   {setUserZoom $obj Prev}
       q   {}
       r   {toggleRef      $obj}
       s   {scaleView $obj one}
       t   {toggleSpectrum $obj}
       u   {sortView $obj up}
       v   {fullView $obj V}
       w   {}
       x   {}
       y   {}
       z   {}

       H   {showHelp $obj}
       Q   {quitAllView}

       F1  {setUserZoom $obj 1}
       F2  {setUserZoom $obj 2}
       F3  {setUserZoom $obj 3}
       F4  {setUserZoom $obj 4}

       F5  {captureScreen  $obj}
       F6  {insertGraphItem text      $obj}
       F7  {insertGraphItem rectangle $obj}
       F8  {insertGraphItem triangle  $obj}
       F9  {insertGraphItem vector    $obj}
       F10 {insertGraphItem spectrum  $obj}

       F9  {}
       F10 {}
       F11 {}
       F12 {}
      }
}

proc toggleSpectrum { obj } \
{
   global objInfo

   set newList ""

   if {[llength $objInfo(data,prevSpecList)]} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           if {[lsearch -exact $objInfo(data,refList) $i] >= 0 \
                || [lsearch -exact $objInfo(data,drawList) $i] >= 0 \
                 || [lsearch -exact $objInfo(data,prevSpecList) $i] >= 0} \
              {
               lappend newList $i
              }
          }

       set objInfo(data,drawList) $newList
       set objInfo(data,prevSpecList) ""
      } \
   else \
      {
       set objInfo(data,prevSpecList) ""

       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           if {[lsearch -exact $objInfo(data,refList) $i] >= 0} \
              {
               lappend newList $i
              } \
           elseif {[lsearch -exact $objInfo(data,drawList) $i] >= 0} \
              {
               lappend objInfo(data,prevSpecList) $i
              }
          }

       set objInfo(data,drawList) $newList
      }

   updateTitleSelection

   showViewAll toggleSpectrum 
   drawTitles
}

proc toggleRef { obj } \
{
   global objInfo

   set newDrawList ""
   set newRefList ""

   if {[llength $objInfo(data,prevRefList)]} \
      {
       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           if {[lsearch -exact $objInfo(data,prevRefList) $i] >= 0} \
              {
               lappend newRefList  $i
               lappend newDrawList $i
               continue
              }

           if {[lsearch -exact $objInfo(data,drawList) $i] >= 0} \
              {
               lappend newDrawList $i
              }
          }

       set objInfo(data,drawList) $newDrawList
       set objInfo(data,refList)  $newRefList

       set objInfo(data,prevRefList) ""
      } \
   else \
      {
       set objInfo(data,prevRefList) ""

       for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
          {
           if {[lsearch -exact $objInfo(data,refList) $i] >= 0} \
              {
               lappend objInfo(data,prevRefList) $i
              } \
           elseif {[lsearch -exact $objInfo(data,drawList) $i] >= 0} \
              {
               lappend newDrawList $i
              }
          }

       set objInfo(data,drawList) $newDrawList
       set objInfo(data,refList)  ""
      }

   updateTitleSelection

   showViewAll toggleRef
   drawTitles
}

proc toggleInteg { obj } \
{
   global objInfo
}

proc clearDisplay { obj } \
{
}

proc updateSpecZoomVal { obj } \
{
   global objInfo xUnits yUnits xFmt yFmt

   if {![info exists objInfo($obj,specX1)]} return

   set objInfo($obj,specX1Val) [format $xFmt $objInfo($obj,specX1)]$xUnits
   set objInfo($obj,specXNVal) [format $xFmt $objInfo($obj,specXN)]$xUnits
   set objInfo($obj,specY1Val) [format $yFmt $objInfo($obj,specY1)]$yUnits
   set objInfo($obj,specYNVal) [format $yFmt $objInfo($obj,specYN)]$yUnits
}

proc startNMRDraw { obj } \
{
   global objInfo

   if {![llength $objInfo(data,drawList)]} return

   set i [lindex $objInfo(data,drawList) 0]

   if {$objInfo(data,$i,dimCount) == 1} \
      {
       showVerb "nmrDraw -in $objInfo(data,inName$i) -useTitle -rephase"
       catch "exec nmrDraw -in $objInfo(data,inName$i) -useTitle -rephase &"
      } \
   else \
      {
       showVerb "nmrDraw -in $objInfo(data,inName$i) -hi $objInfo(data,hi$i) -useTitle"
       catch "exec nmrDraw -in $objInfo(data,inName$i) -hi $objInfo(data,hi$i) -useTitle &"
      }
}

proc setUserZoom { obj i } \
{
   global objInfo xUnits yUnits

   set iStep 0

   if {![strcasecmp $i Next]} {set iStep  1}
   if {![strcasecmp $i Prev]} {set iStep -1}

   if {$iStep} \
      {
       set n [expr [llength $objInfo(data,zoomList)] - 1]
       if {$n < 0} return

       if {$objInfo(data,zoomLoc) < 0} {set objInfo(data,zoomLoc) $n}

       incr objInfo(data,zoomLoc) $iStep

       if {$objInfo(data,zoomLoc) <  0} {set objInfo(data,zoomLoc) $n}
       if {$objInfo(data,zoomLoc) > $n} {set objInfo(data,zoomLoc)  0}

       set zInfo [lindex $objInfo(data,zoomList) $objInfo(data,zoomLoc)]

       if {$objInfo(data,1,dimCount) == 1} \
          {
           set objInfo($obj,specX1) [lindex $zInfo 0]
           set objInfo($obj,specXN) [lindex $zInfo 1]
          } \
       else \
          {
           set objInfo($obj,specX1) [lindex $zInfo 0]
           set objInfo($obj,specXN) [lindex $zInfo 1]
           set objInfo($obj,specY1) [lindex $zInfo 2]
           set objInfo($obj,specYN) [lindex $zInfo 3]
          }

       updateSpecZoomVal $obj
      } \
   else \
      {
       if {$objInfo(data,1,dimCount) == 1} \
          {
           set x1 [lindex $objInfo(data,zoom$i) 0]
           set xn [lindex $objInfo(data,zoom$i) 1]

           set zx1 [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) $x1] 
           set zxn [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) $xn] 

           set zy1 Same
           set zyn Same

           if {$zx1 > $zxn} \
              {
               set iz  $zx1
               set zx1 $zxn
               set zxn $iz
              }

           set sx1 [pnt2spec $objInfo(data,fdata) $objInfo(data,hDim) $zx1 $xUnits]
           set sxn [pnt2spec $objInfo(data,fdata) $objInfo(data,hDim) $zxn $xUnits]
          } \
       else \
          {
           set x1 [lindex $objInfo(data,zoom$i) 0]
           set xn [lindex $objInfo(data,zoom$i) 1]
           set y1 [lindex $objInfo(data,zoom$i) 2]
           set yn [lindex $objInfo(data,zoom$i) 3]

           set zx1 [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) $x1]
           set zxn [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) $xn]
           set zy1 [spec2pnt $objInfo(data,fdata) $objInfo(data,vDim) $y1]
           set zyn [spec2pnt $objInfo(data,fdata) $objInfo(data,vDim) $yn]

           if {$zx1 > $zxn} \
              {
               set iz  $zx1
               set zx1 $zxn
               set zxn $iz
              }

           if {$zy1 > $zyn} \
              {
               set iz  $zy1
               set zy1 $zyn
               set zyn $iz
              }

           set sx1 [pnt2spec $objInfo(data,fdata) $objInfo(data,hDim) $zx1 $xUnits]
           set sxn [pnt2spec $objInfo(data,fdata) $objInfo(data,hDim) $zxn $xUnits]
           set sy1 [pnt2spec $objInfo(data,fdata) $objInfo(data,vDim) $zy1 $yUnits]
           set syn [pnt2spec $objInfo(data,fdata) $objInfo(data,vDim) $zyn $yUnits]
          }

       setSpecZoom $obj $objInfo(data,fdata) $zx1 $zxn $zy1 $zyn 
      }

   readView $obj
   showView $obj setUserZoom
}

proc pushZoomInfo { obj } \
{
   global objInfo

   if {$objInfo(data,1,dimCount) == 1} \
      {
       set thisInfo "$objInfo($obj,specX1) $objInfo($obj,specXN)"
      } \
   else \
      {
       set thisInfo "$objInfo($obj,specX1) $objInfo($obj,specXN) $objInfo($obj,specY1) $objInfo($obj,specYN)"
      }

   foreach zInfo $objInfo(data,zoomList) \
      {
       if {![strcasecmp $zInfo $thisInfo]} return
      }

   lappend objInfo(data,zoomList) $thisInfo

   set objInfo(data,zoomLoc) [expr [llength $objInfo(data,zoomList)] - 1]

   set n   [llength $objInfo(data,zoomList)]
   set loc $objInfo(data,zoomLoc)
 
   showVerb "Zoom List: $objInfo(data,zoomList) Length: $n Loc: $loc"
}

proc setDifMode { obj } \
{
   global objInfo

   if {![info exists objInfo($obj,difMode)]} {set objInfo($obj,difMode) 0}

   updateROIStats $obj
   showView $obj difMode
   drawTitles
}

proc setLabelMode { obj mode } \
{
   if {![strcasecmp $mode peak]} \
      {
       togglePeakLabels $obj
      } \
   elseif {![strcasecmp $mode ass]} \
      {
       toggleAssLabels $obj
      } \
   else \
      {
       drawTitles
      }
}

proc setDrawSpectraMode { obj } \
{
   global objInfo

   if {$objInfo($obj,drawSpectra)} \
      {
       drawTitles
      } \
   else \
      {
       clearTitles
      }

   showView $obj setDrawSpectraMode
}

proc setAxisMode { obj } \
{
   showView $obj setAxisMode
}

proc initPeakWindow {} \
{
   global objInfo

   set i 1

   if {!$objInfo(prog,tabViewFlag)} return

   if {!$objInfo(peak,tab$i)}    return
   if {[winfo exists .peakView]} return

   set objInfo(peak,viewer) .peakView
   set objInfo(peak,vList)  ""

   set firstList  "INDEX X_PPM HEIGHT"
   set nameList   $firstList

   set vNameList [gdbGetInfo $objInfo(peak,tab$i) -parms varNames]

   foreach vName $vNameList \
      {
       if {[lsearch -exact $firstList $vName] < 0} \
          {
           lappend nameList $vName
          }
      }

   foreach vName $nameList \
      {
       lappend objInfo(peak,vList) [gdbFindByName $objInfo(peak,tab$i) $vName]
      }

   showTable -tab $objInfo(peak,tab$i) -var $nameList -win $objInfo(peak,viewer)

   wm geom $objInfo(peak,viewer) -5-5
}

proc setFile {} \
{
   global objInfo

   initSpecColors
}

proc selectFile { id } \
{
   global objInfo

   set obj data

    set fileTypes \
       {
        { {All Files}        *      }
        { {ft  Files}        {.ft}  }
        { {ft1 Files}        {.ft1} }
        { {ft2 Files}        {.ft2} }
        { {ft3 Files}        {.ft3} }
        { {fid Files}        {.fid} }
        { {dat Files}        {.dat} }
        { {DAT Files}        {.DAT} }
       }

   set objInfo($obj,inName$id) \
     [tk_getOpenFile -title "Spectrum $id" -filetypes $fileTypes]

   if {![string length $objInfo($obj,inName$id)]} \
      {
       set objInfo($obj,inName$id) "None"
      }

   set objInfo($obj,inList) ""

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       lappend objInfo($obj,inList) $objInfo($obj,inName$i)
      }

   initSpecColors
}

proc readView args \
{
   global objInfo xUnits yUnits

   set obj [lindex $args 0]
   set n   [llength $args]

   busyView hold $objInfo($obj,window)

   set nn $objInfo(data,specCount)

   if {$nn < 2} {set nn 2}

   for {set i 1} {$i <= $nn} {incr i} \
      {
       set objInfo($obj,roi$i) 0
       set roiID($i)           0
      }

   set objInfo($obj,roistat) 0

   for {set i 1} {$i < $n} {incr i} \
      {
       set roiID($i) [lindex $args $i]
      }

   set hAxis $objInfo(data,hAxis)
   set vAxis $objInfo(data,vAxis)

   set objInfo($obj,tauList)   ""
   set objInfo($obj,user1List) ""
   set objInfo($obj,user2List) ""
   set objInfo(data,roiCount)  0

   pushZoomInfo $obj

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {![strcasecmp $objInfo($obj,inName$i) None]} continue
       if {![file exists $objInfo($obj,inName$i)]} continue
       if {[file isdirectory $objInfo($obj,inName$i)]} continue
 
       if {!$roiID($i)} {set roiID($i) [getNextID]}

       showVerb "Reading ROI $roiID($i) $objInfo($obj,inName$i) ..."

       set objInfo($obj,roi$i) $roiID($i)

       set naux 1

       if {$objInfo(data,$i,dimCount) == 1} \
          {
           if {$i == 1} \
              {
               showVerb "readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits -$objInfo(data,realFlag) -naux $naux"

               readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) \
                -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits \
                -$objInfo(data,realFlag) -naux $naux 

              } \
           else \
              {
               showVerb "readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits -center $objInfo(data,alignX) -$objInfo(data,realFlag) -naux $naux" 

               readROI -in $objInfo($obj,inName$i) -ndim 1 -roi $roiID($i) \
                -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits \
                -center $objInfo(data,alignX) -$objInfo(data,realFlag) -naux $naux 
              }
          } \
       else \
          {
           if {$i == 1} \
              {
               showVerb "readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits -y $vAxis $objInfo($obj,specY1)$yUnits $objInfo($obj,specYN)$yUnits -$objInfo(data,realFlag) -naux $naux"

               readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) \
                -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits \
                -y $vAxis $objInfo($obj,specY1)$yUnits $objInfo($obj,specYN)$yUnits \
                -$objInfo(data,realFlag) -naux $naux
              } \
           else \
              {
               showVerb "readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits -y $vAxis $objInfo($obj,specY1)$yUnits $objInfo($obj,specYN)$yUnits -$objInfo(data,realFlag) -naux $naux"

               readROI -in $objInfo($obj,inName$i) -ndim 2 -roi $roiID($i) \
                -x $hAxis $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits \
                -y $vAxis $objInfo($obj,specY1)$yUnits $objInfo($obj,specYN)$yUnits \
                -$objInfo(data,realFlag) -naux $naux
              }
          }

       incr objInfo(data,roiCount)

       set roiInfo [getROIInfo -roi $roiID($i)]
       set rdata   [lindex $roiInfo 0]
       set fdata   [lindex $roiInfo 1]
       set sList   [lindex $roiInfo 2]
       set size    [lindex $sList 0]

       vsMult $rdata $objInfo(data,scale$i) $size

       lappend objInfo($obj,tauList)   [getParm $fdata FDTAU   NULL_DIM]
       lappend objInfo($obj,user1List) [getParm $fdata FDUSER1 NULL_DIM]
       lappend objInfo($obj,user2List) [getParm $fdata FDUSER2 NULL_DIM]

       if {$i == 1 && [strcmp $obj data]} {set objInfo($obj,fdata) $fdata}
      }

    updateROIStats $obj

    showVerb "Status: [sysInfo]"
    showVerb "Done Reading ROIs."

    busyView release $objInfo($obj,window)
}

proc updateROIStats { obj } \
{
   global objInfo xUnits yUnits

   if {!$objInfo($obj,roistat)} {set objInfo($obj,roistat) [getNextID]}
   if {![info exists objInfo($obj,difMode)]} {set objInfo($obj,difMode) 0}

   if {[llength $objInfo(data,drawList)]} \
      {
       set i [lindex $objInfo(data,drawList) 0]
      } \
   else \
      {
       set i 1
      }
  
   if {$objInfo(data,$i,dimCount) == 1} \
      { 
       readROI \
         -inROI $objInfo($obj,roi$i) -ndim 1 \
         -roi $objInfo($obj,roistat) -x X_AXIS 
      } \
   else \
      {
       readROI \
        -inROI $objInfo($obj,roi$i) -ndim 2 -roi $objInfo($obj,roistat) \
        -x X_AXIS -y Y_AXIS
      }

   set rInfoS [getROIInfo -roi $objInfo($obj,roistat)]
   set rdataS [lindex $rInfoS 0]
   set nS     [lindex [lindex $rInfoS 2] 0]

   vsSet $rdataS 0.0 $nS

   if {[llength $objInfo(data,drawList)] < 2} return 

   if {$objInfo(data,1,dimCount) == 2} return
   if {$objInfo($obj,difMode) == 0 || $objInfo($obj,difMode) == 1} return

   set i [lindex $objInfo(data,drawList) 0]
   set j [lindex $objInfo(data,drawList) 1]

   if {!$objInfo($obj,roi$i) || !$objInfo($obj,roi$j)} continue

   set rInfoI [getROIInfo -roi $objInfo($obj,roi$i)]
   set rInfoJ [getROIInfo -roi $objInfo($obj,roi$j)]

   set rdataI [lindex $rInfoI 0]
   set rdataJ [lindex $rInfoJ 0]

   set fdataI [lindex $rInfoI 1]
   set fdataJ [lindex $rInfoJ 1]

   set nI     [lindex [lindex $rInfoI 2] 0]
   set nJ     [lindex [lindex $rInfoJ 2] 0]

   set nMin   [getMin "$nI $nJ $nS"]
   set nMax   [getMax "$nI $nJ $nS"]

   set optimizeShift 0

   if {abs($nI - $nJ) < 2} \
      { 
       showVerb "Finding Simple Difference $i - $j"
       vvSub $rdataS $rdataI $rdataJ $nMin
      } \
   else \
      {
       showVerb "Finding Interpolated Difference $i - $j, S: $nS I: $nI J: $nJ"

       set qdataI [sysMalloc $nMax float] 
       set qdataJ [sysMalloc $nMax float] 

       set midI   [expr 1 + $nI/2]
       set midJ   [expr 1 + $nJ/2]

       set pI [pnt2spec $fdataI CUR_XDIM $midI $xUnits]
       set pJ [pnt2spec $fdataJ CUR_XDIM $midJ $xUnits]
       set pR [expr abs($objInfo($obj,specX1) - $objInfo($obj,specXN))]

       set s 0.0

       if {$pR != 0} {set s [expr ($pJ - $pI)*$nJ/$pR]}

       vResize -src $rdataI -dest $qdataI -xSize $nI -xZF $nS -ndim 1
       vResize -src $rdataJ -dest $qdataJ -xSize $nJ -xZF $nS -ndim 1 -xLS $s

       if {$optimizeShift} \
          {
           set bestRMS    [expr [vRMS $qdataI $nS] + [vRMS $qdataJ $nS]]
           set bestOffset 0.0

           foreach offset {-0.5 -0.4 -0.3 -0.2 -0.1 0.0 0.1 0.2 0.3 0.4 0.5} \
              {
               vResize -src $rdataJ -dest $qdataJ -xSize $nJ -xZF $nS -ndim 1 \
                       -xLS [expr $s + $offset]
          
               vvSub $rdataS $qdataI $qdataJ $nS

               set thisRMS [vRMS $rdataS $nS] 

               if {$thisRMS < $bestRMS} \
                  {
                   set bestRMS $thisRMS
                   set bestOffset $offset
                  }
              }

           vResize -src $rdataJ -dest $qdataJ -xSize $nJ -xZF $nS -ndim 1 \
                   -xLS [expr $s + $bestOffset]
         }

       vvSub $rdataS $qdataI $qdataJ $nS

       sysFree $qdataI
       sysFree $qdataJ
      }
}

proc setActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo(data,hi$i)
}

proc incrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   incr i

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo(data,hi$i)
}

proc decrActiveSpec { obj } \
{
   global objInfo

   set w $objInfo($obj,window)
   set i [lindex $objInfo($obj,activeSpec) 0]

   incr i -1

   if {$i > $objInfo(data,specCount)} {set i 1}
   if {$i < 1} {set i $objInfo(data,specCount)}

   set objInfo($obj,activeSpec) "$i ($objInfo(color,color$i))"

#  $w.first1 configure -textvariable objInfo(data,hi$i)
}

proc incrHi { obj mode v } \
{
   global objInfo

   if {![strcasecmp $mode active]} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,$v$i) [incrHiVal $objInfo($obj,$v$i) $objInfo($obj,factor)]
      } \
   else \
      {
       set objInfo($obj,$v)   [incrHiVal $objInfo($obj,fhi) $objInfo($obj,factor)]
      }

   showView $obj incrHi
}

proc decrHi { obj mode v } \
{
   global objInfo

   if {![strcasecmp $mode active]} \
      {
       set i [lindex $objInfo($obj,activeSpec) 0]
      } \
   else \
      { 
       set i $mode
      }

   if {$i} \
      {
       set objInfo($obj,$v$i) [decrHiVal $objInfo($obj,$v$i) $objInfo($obj,factor)]
      } \
   else \
      {
       set objInfo($obj,$v)   [decrHiVal $objInfo($obj,fhi) $objInfo($obj,factor)]
      }

   showView $obj decrHi
}

proc setHi { obj mode } \
{
   global objInfo

   showView $obj setHi
}

proc setPhase { obj mode val } \
{
   global objInfo

}

proc fullView { obj mode } \
{
   global objInfo

   if {$objInfo(data,1,dimCount) == 1} \
      {
       if {![strcmp $mode H]} \
          {
           set sx1 1
           set sxn $objInfo(data,1,xSize)
           set fHi Same
           set lHi Same
          } \
       else \
          {
           set maxList ""
           set minList ""

           foreach i $objInfo(data,drawList) \
              {
               lappend minList [getROIStat $objInfo($obj,roi$i) vMin]
               lappend maxList [getROIStat $objInfo($obj,roi$i) vMax]
              }

           if {$objInfo($obj,difMode) == 2 || $objInfo($obj,difMode) == 3} \
              {
               if {$objInfo($obj,roistat)} \
                  {
                   lappend minList [getROIStat $objInfo($obj,roistat) vMin]
                   lappend maxList [getROIStat $objInfo($obj,roistat) vMax]
                  }
              }

           set maxVal [getMax $maxList]
           set minVal [getMin $minList]

           set r      [expr 0.05*($maxVal - $minVal)]

           set sx1 Same
           set sxn Same

           set fHi [expr $minVal - $r]
           set lHi [expr $maxVal + $r]
          }

       setSpecZoom $obj $objInfo(data,fdata) $sx1 $sxn $fHi $lHi
      } \
   else \
      {
       set sx1 Same
       set sxn Same
       set sy1 Same
       set syn Same

       if {![strcmp $mode H] || ![strcmp $mode 2D]} \
          {
           set sx1 1
           set sxn $objInfo(data,1,xSize)
          }

       if {![strcmp $mode V] || ![strcmp $mode 2D]} \
          {
           set sy1 1
           set syn $objInfo(data,1,ySize)
          }

       setSpecZoom $obj $objInfo(data,fdata) $sx1 $sxn $sy1 $syn
      }

   readView $obj
   showView $obj fullView 
}

proc pSort { s1 s2 } \
{
   set v1 [lindex $s1 0] 
   set v2 [lindex $s2 0] 

   if {$v1 > $v2} {return  1}
   if {$v1 < $v2} {return -1}

   return 0
}

proc sortView { obj mode } \
{
   global objInfo

   if {[llength $objInfo(data,drawList)] < 2} return

   set rList ""

   foreach i $objInfo(data,drawList) \
      {
       set rms [getROIStat $objInfo($obj,roi$i) vRMS]
       if {![strcasecmp $mode Down]} {set rms [expr -1.0*$rms]}

       lappend rList "$rms $i"
      }

   set sList [lsort -command pSort $rList]

   set objInfo(data,drawList) ""

   foreach s $sList {lappend objInfo(data,drawList) [lindex $s 1]}   

   showViewAll sortView
   drawTitles
}

proc scaleView { obj mode } \
{
   global objInfo

   set n [llength $objInfo(data,drawList)]
   if {$n < 2} return

   set i [lindex $objInfo(data,drawList) 0]

   if {![strcasecmp $mode All]} \
      {
       set jList [lrange $objInfo(data,drawList) 1 end]
      } \
   else \
      {
       set jList [lindex $objInfo(data,drawList) 1]
      }

   if {![strcasecmp $mode Reset]} \
      {
       foreach i $objInfo(data,drawList) \
          {
           set objInfo(data,scale$i) 1.0
           set objInfo($obj,xOff$i)  0.0
           set objInfo($obj,yOff$i)  0.0
          }

       set objInfo($obj,yOffstat)  0.0
       set objInfo($obj,yOffdiff)  0.0
      } \
   elseif {![strcasecmp $mode stack]} \
      {
#      set range [expr $objInfo($obj,lhi1) - $objInfo($obj,fhi1)]
#      set objInfo($obj,fhi1) [expr ($n + 1)*$objInfo($obj,fhi1)]

       set j 0

       foreach i $objInfo(data,drawList) \
          {
           set objInfo($obj,yOff$i) [expr -0.75*$j/$n]
           incr j
          }
      } \
   else \
      {
       set rInfoI [getROIInfo -roi $objInfo($obj,roi$i)]
       set rdataI [lindex $rInfoI]
       set nI     [lindex [lindex $rInfoI 2] 0]

       foreach j $jList \
          {
           set rInfoJ [getROIInfo -roi $objInfo($obj,roi$j)]
           set rdataJ [lindex $rInfoJ]
           set nJ     [lindex [lindex $rInfoJ 2] 0]

           set n [getMin "$nI $nJ"]
   
           if {$objInfo(prog,scaleByMax)} \
              { 
               set maxValI [vMax $rdataI $nI]
               set maxValJ [vMax $rdataJ $nJ]

               if {$maxValJ == 0.0} \
                  {
                   set objInfo(data,scale$j) 0.0
                  } \
               else \
                  {
                   set objInfo(data,scale$j) \
                    [format %.5f [expr $objInfo(data,scale$j)*$maxValI/$maxValJ]]
                  }           
              } \
           else \
              {
               set objInfo(data,scale$j) \
                [format %.5f [expr $objInfo(data,scale$j)*[vvGetScale $rdataI $rdataJ $n]]]
              }
    
           showVerb "Scale $i $j: $objInfo(data,scale$j)"
          }
      }

   foreach obj $objInfo(prog,winList) \
      {
       readView $obj
       showView2 $obj scaleView 
      }

   drawTitles
}

proc showViewAll { who } \
{
   global objInfo

   foreach obj $objInfo(prog,winList) \
      {
       updateROIStats $obj
       showView2 $obj $who
      } 
}

proc showView { obj who } \
{
   showView2 $obj $who
}

proc showView2 { obj who } \
{
   global objInfo xUnits yUnits

   if {![winfo exists $objInfo($obj,window)]} return

   busyView hold $objInfo($obj,window)

   set objInfo(prog,updateFlag) 0

   clearView $obj

   if {![strcasecmp $who adjSelMarkerH] || ![strcasecmp $who adjSelMarkerV]} \
      {
       set adjMarkerFlag 1
      } \
   else \
      {
       set adjMarkerFlag 0
       clearMarkers $obj
      }

   if {$objInfo(prog,xGridFlag)} \
      {
       set xGrid xGrid
      } \
   else \
      {
       set xGrid noxGrid
      }

   if {$objInfo(prog,yGridFlag)} \
      {
       set yGrid yGrid
      } \
   else \
      {
       set yGrid noyGrid
      }

   set i 1

   if {$objInfo(data,$i,dimCount) == 1} {set yGrid noyGrid}

   if {!$objInfo(prog,lwgrid)} \
      {
       set xGrid noxGrid
       set yGrid noyGrid
      }

#
# Axis:

   drawRec -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
           -x 0% 100% -y 0% 100% -fg $objInfo(color,bg)

   if {$objInfo($obj,roi1)} \
      {
       set fHi $objInfo($obj,fhi1)
       set lHi $objInfo($obj,lhi1)

       if {$objInfo($obj,useAxis)} \
          {
           if {$objInfo(data,1,dimCount) == 1} \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                       -roi $objInfo($obj,roi1) -fhi $fHi -lhi $lHi         \
                       -bg None -fg $objInfo(color,color1)       \
                       -fillColor $objInfo(color,fillColor1)     \
                       -colorA $objInfo(color,label)             \
                       -colorS $objInfo(color,shadow)            \
                       -colorG $objInfo(color,grid)              \
                       -lwdraw $objInfo(prog,lwdraw)             \
                       -lwgrid $objInfo(prog,lwgrid)             \
                       -lw1d   $objInfo(data,lw1d)               \
                       -lw2d   $objInfo(data,lw2d)               \
                       -$xGrid -$yGrid -axisOnly -axis Bottom,units=$xUnits -nobox -big

               draw1DYGrid $obj
              } \
           else \
              {
               drawROI -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                       -roi $objInfo($obj,roi1) -hi $objInfo(data,hi1) \
                       -bg None -fg $objInfo(color,color1)       \
                       -fillColor $objInfo(color,fillColor1)     \
                       -colorA $objInfo(color,label)             \
                       -colorS $objInfo(color,shadow)            \
                       -colorG $objInfo(color,grid)              \
                       -lwdraw $objInfo(prog,lwdraw)             \
                       -lwgrid $objInfo(prog,lwgrid)             \
                       -lw1d   $objInfo(data,lw1d)               \
                       -lw2d   $objInfo(data,lw2d)               \
                       -$xGrid -$yGrid -axisOnly -axis Bottom,units=$xUnits Right,units=$yUnits -nobox -big
              }
          }
      }

   if {![llength $objInfo(data,drawList)]} \
      {
       if {$objInfo(prog,lwdraw)} \
          {
           drawBox -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                   -x 0% 100% -y 0% 100% -fg $objInfo(color,box)        \
                   -lwdraw $objInfo(prog,lwdraw)
          }

       busyView release $objInfo($obj,window)
       set objInfo(prog,updateFlag) 1

       xupdate
       return
      }

#
# Adjust colors and draw lists for difference drawing:

   set drawList $objInfo(data,drawList)
   set n [llength $drawList]

   if {$n > 1} \
      {
       set i [lindex $drawList 0]
       set k [lindex $drawList 1]

       set thisColor1 [getDataColor $obj $i]
       set thisColor2 [getDataColor $obj $k]
      } \
   else \
      {
       set i [lindex $drawList 0]
       set k $i

       set thisColor1 [getDataColor $obj $i]
       set thisColor2 $thisColor1
      }

   if {$n} {updateCurrent [lindex $objInfo(data,drawList) end]}

#
# Draw differences as needed:

   set fHi $objInfo($obj,fhi1)
   set lHi $objInfo($obj,lhi1)

   set fillColor1 $objInfo(color,statFill1)
   set fillColor2 $objInfo(color,statFill2)

   if {$objInfo(data,1,dimCount) == 1} \
      {
       if {$objInfo($obj,difMode) == 0 || $objInfo($obj,difMode) == 3} \
          {
           set fillColor1 None
           set fillColor2 None
          }
      }

   if {$n > 1 && $objInfo($obj,difMode) == 1} \
      {
       set drawList [lrange $drawList 2 end]

       if {$objInfo($obj,drawSpectra)} \
          {
           showVerb "Drawing Internal Difference ROI $objInfo($obj,roi$i)  $objInfo($obj,roi$k) $thisColor1 $thisColor2 $fHi $lHi for $who"

           eval drawROI $objInfo(prog,draw1DArgs) \
                 -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap)  \
                 -roi1 $objInfo($obj,roi$i) -roi2 $objInfo($obj,roi$k) \
                 -fhi $fHi -lhi $lHi                        \
                 -bg  None                                  \
                 -fg          $objInfo(color,stat)          \
                 -color1      $thisColor1                   \
                 -color2      $thisColor2                   \
                 -fillColor1  $fillColor1                   \
                 -fillColor2  $fillColor2                   \
                 -colorA $objInfo(color,label)              \
                 -colorS $objInfo(color,shadow)             \
                 -lw1d   $objInfo(data,lw1d$i) -nobox       \
                 -lwgrid $objInfo(prog,lwgrid)              \
                 -lwdraw $objInfo(prog,lwdraw)              \
                 -yOffSpec [getSpecYOff $obj diff] [getZeroFlagArg $i]

           setSelMarkers $obj diff \
            "$objInfo($obj,roi$i) $objInfo($obj,roi$k)" \
            "$thisColor1 $thisColor2"  $adjMarkerFlag
          }

      } \
   elseif {$objInfo($obj,difMode) == 2 || $objInfo($obj,difMode) == 3} \
      {
       if {$n > 1 && $objInfo($obj,roistat)} \
          {
           if {$objInfo($obj,drawSpectra)} \
              {
               showVerb \
                "Drawing Shaded Difference ROI $objInfo($obj,roi$i) $thisColor1 $fHi $lHi for $who"

               eval drawROI $objInfo(prog,draw1DArgs) \
                     -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
                     -roi $objInfo($obj,roistat) -fhi $fHi -lhi $lHi      \
                     -bg None                                   \
                     -fg         $objInfo(color,stat)           \
                     -fillColor1 $fillColor1                    \
                     -fillColor2 $fillColor2                    \
                     -colorA $objInfo(color,label)              \
                     -colorS $objInfo(color,shadow)             \
                     -lw1d $objInfo(data,lw1d$i) -nobox         \
                     -lwgrid $objInfo(prog,lwgrid)              \
                     -lwdraw $objInfo(prog,lwdraw)              \
                     -yOffSpec [getSpecYOff $obj stat] [getZeroFlagArg $i] 

              setSelMarkers $obj stat \
               $objInfo($obj,roistat) $objInfo(color,stat) $adjMarkerFlag
             }
         }
      }

#
# Draw data:

   set i 1

   if {$objInfo(data,$i,dimCount) == 1} \
      {
       set iList [lreverse $drawList] \
      } \
   else \
      {
       set iList $drawList
      }

   foreach i $iList \
      {
       if {$objInfo($obj,roi$i) && $objInfo(data,drawFlag$i)} \
          {
           set fHi $objInfo($obj,fhi1)
           set lHi $objInfo($obj,lhi1)

           set thisColor1 [getDataColor $obj $i]

           if {$objInfo($obj,drawSpectra)} \
              {
               showVerb \
               "Drawing ROI $objInfo($obj,roi$i) $thisColor1 $fHi $lHi for $who"

               showVerb \
                "Mouse: $objInfo(prog,B1) $objInfo(prog,B2) $objInfo(prog,B3)"

               drawExtraAxis $obj $i

               if {$objInfo(data,$i,dimCount) == 1} \
                  {
                   eval drawROI $objInfo(prog,draw1DArgs) \
                         -axis [getAxisFlagArg $i]   \
                         -cell $objInfo($obj,cell)   \
                         -win  $objInfo($obj,pixmap) \
                         -roi  $objInfo($obj,roi$i)  \
                         -fhi  $fHi -lhi $lHi        \
                         -bg None -fg $thisColor1 -nobox       \
                         -fillColor $objInfo(color,fillColor$i) \
                         -colorA $objInfo(color,label)          \
                         -colorS $objInfo(color,shadow)         \
                         -lw1d   $objInfo(data,lw1d$i)          \
                         -lwgrid $objInfo(prog,lwgrid)          \
                         -lwdraw $objInfo(prog,lwdraw)          \
                         -yOffSpec [getSpecYOff $obj $i] [getZeroFlagArg $i]
                  } \
               else \
                  {
                   if {$objInfo(color,multiFlag)} \
                      {
                       set pColor multi
                       set nColor multi
                      } \
                   else \
                      {
                       set pColor $thisColor1
                       set nColor $thisColor1 
                      }

# %%% -x11GX

                   eval drawROI $objInfo(prog,draw2DArgs) \
                         -$objInfo(data,drawMode)         \
                         -axis [getAxisFlagArg $i]        \
                         -cell $objInfo($obj,cell)        \
                         -win  $objInfo($obj,pixmap)      \
                         -roi  $objInfo($obj,roi$i)       \
                         -hi   $objInfo(data,hi$i) -nobox \
                         -bg None -color1 $pColor -color2 $nColor \
                         -colorA $objInfo(color,label)            \
                         -colorS $objInfo(color,shadow)           \
                         -lw2d   $objInfo(data,lw2d$i)            \
                         -lwdraw $objInfo(prog,lwdraw)            \
                         -xOffSpec [getSpecXOff $obj $i]          \
                         -yOffSpec [getSpecYOff $obj $i] 
                  }

              setSelMarkers $obj $i \
               $objInfo($obj,roi$i) $thisColor1 $adjMarkerFlag
             }
         }
      }

#
# Exit:

   if {$objInfo(prog,lwdraw)} \
      {
       drawBox -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) \
               -x 0% 100% -y 0% 100% -fg $objInfo(color,box)        \
               -lwdraw $objInfo(prog,lwdraw)
      }

   busyView release $objInfo($obj,window)

   drawPeaks
   drawAss
   drawSMILE
   drawCursors
   drawLabelItems

   set objInfo(prog,updateFlag) 1

   xupdate
}

proc drawExtraAxis { obj i } \
{
   global objInfo

   if {$i < 1 || $i > $objInfo(data,specCount)} return

   if {!$objInfo(prog,xAxisFlag) && !$objInfo(prog,yAxisFlag)} return
}

proc draw1DYGrid { obj } \
{
   global objInfo

   set i 1

   if {!$objInfo(prog,lwgrid) || !$objInfo(prog,yGridFlag)} return
   if {$objInfo(data,$i,dimCount) != 1} return

   set dy [expr 1 + $objInfo($obj,yn) - $objInfo($obj,y1)]
   set n  10 

   for {set i 1} {$i < $n} {incr i} \
      {
       set yLoc [expr $objInfo($obj,y1) + int(double($dy*$i)/$n)]

       drawLine -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap)     \
                -x    $objInfo($obj,x1) $objInfo($obj,xn) -y $yLoc $yLoc \
                -fg   $objInfo(color,grid) -lwdraw $objInfo(prog,lwgrid)
      }
}

proc raiseSelMarkers {} \
{
   global objInfo

   if {![info exists objInfo(data,currentID)]} return
   if {!$objInfo(data,shiftNDFlag)} return

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue
       $objInfo($obj,canvasSelH) raise selMarkerH$objInfo(data,currentID)
       $objInfo($obj,canvasSelV) raise selMarkerV$objInfo(data,currentID)
      }

   update
}

proc setSelMarkers { obj id roiList colorList adjFlag } \
{
   global objInfo

   if {![llength $roiList]} return
   if {![llength $objInfo(data,drawList)]} return
   if {![llength $colorList]} {set colorList white}

   if {![info exists objInfo($obj,yOff$id)]} {set objInfo($obj,yOff$id) 0.0}
   if {![info exists objInfo($obj,xOff$id)]} {set objInfo($obj,xOff$id) 0.0}

   set roi   [lindex $roiList   0]
   set color [safeColor [lindex $colorList 0]]
   set hi    [getROINthVal $roi 1]

   if {[llength $roiList] > 1} \
      {
       set tagH selMarkerHStat
       set tagV selMarkerHStat
      } \
   else \
      {
       set tagH selMarkerH$roi
       set tagV selMarkerH$roi
      }

   set tagH selMarkerH$id
   set tagV selMarkerV$id

   set range [expr $objInfo($obj,lhi1) - $objInfo($obj,fhi1)]
   set yw    [expr $objInfo($obj,yn) - $objInfo($obj,y1)]

   if {$hi < $objInfo($obj,fhi1)} {set hi $objInfo($obj,fhi1)}
   if {$hi > $objInfo($obj,lhi1)} {set hi $objInfo($obj,lhi1)}

   if {$range == 0.0 || $objInfo(data,1,dimCount) > 1} \
      {
       set y0 [expr int(0.5*($objInfo($obj,y1) + $objInfo($obj,yn)))]
      } \
   else \
      {
       set f  [expr ($hi - $objInfo($obj,fhi1))/$range]
       set y0 [expr int($objInfo($obj,yn) - $f*$yw)]
      }

   set y0 [expr $y0 + int($objInfo($obj,yOff$id)*$yw)]
   set y1 [expr $y0 - 3]
   set yn [expr $y0 + 4]
 
   set x1 0
   set xn $objInfo(prog,canvasSelXW)

   if {!$adjFlag} \
      {
       set selID [$objInfo($obj,canvasSelV) create rectangle \
                   $x1 $y1 $xn $yn -outline $color -fill $color \
                   -tags "selMarkerV $tagV"]

        $objInfo($obj,canvasSelV) bind $selID <B1-Motion> \
         "+ adjSelMarkerV $obj $id $selID $tagV $roi %x %y"

        $objInfo($obj,canvasSelV) bind $selID <ButtonRelease-1> \
         "+ showView $obj adjSelMarkerDone"
       }

   if {!$objInfo(data,shiftNDFlag)} return

   set xw [expr $objInfo($obj,xn) - $objInfo($obj,x1)]
   set x0 [expr int(0.5*($objInfo($obj,x1) + $objInfo($obj,xn)))]

   set x0 [expr $x0 + int($objInfo($obj,xOff$id)*$xw)]
   set x1 [expr $x0 - 3]
   set xn [expr $x0 + 4]

   set y1 0
   set yn $objInfo(prog,canvasSelXW)

   if {!$adjFlag} \
      {
       set selID [$objInfo($obj,canvasSelH) create rectangle \
                   $x1 $y1 $xn $yn -outline $color -fill $color \
                   -tags "selMarkerH $tagH"]

        $objInfo($obj,canvasSelH) bind $selID <B1-Motion> \
         "+ adjSelMarkerH $obj $id $selID $tagH $roi %x %y"

        $objInfo($obj,canvasSelH) bind $selID <ButtonRelease-1> \
         "+ showView $obj adjSelMarkerDone"
       }
}

proc getSpecXOff { obj id } \
{
   global objInfo
                           
   if {![info exists objInfo($obj,xOff$id)]} \
      {
       set objInfo($obj,xOff$id) 0.0
       return 0    
      }
   
   set xw [expr $objInfo($obj,xn) - $objInfo($obj,x1)]
   set dx [expr int(double($objInfo($obj,xOff$id))*$xw)]

   return $dx
}

proc getSpecYOff { obj id } \
{
   global objInfo

   if {![info exists objInfo($obj,yOff$id)]} \
      {
       set objInfo($obj,yOff$id) 0.0
       return 0
      }

   set yw [expr $objInfo($obj,yn) - $objInfo($obj,y1)]
   set dy [expr int(double($objInfo($obj,yOff$id))*$yw)]

   return $dy
}

proc adjSelMarkerV { obj id selID tag roi x0 y0 } \
{
   global objInfo

   set yw [expr $objInfo($obj,yn) - $objInfo($obj,y1)]

   if {$objInfo(data,1,dimCount) == 1} \
      {
       set hi    [getROINthVal $roi 1]
       set range [expr $objInfo($obj,lhi1) - $objInfo($obj,fhi1)]

       if {$range == 0.0 || $yw == 0} return

       if {$y0 < $objInfo($obj,y1)} {set y0 $objInfo($obj,y1)}
       if {$y0 > $objInfo($obj,yn)} {set y0 $objInfo($obj,yn)}

       if {$hi < $objInfo($obj,fhi1)} {set hi $objInfo($obj,fhi1)}
       if {$hi > $objInfo($obj,lhi1)} {set hi $objInfo($obj,lhi1)}

       set f      [expr ($hi - $objInfo($obj,fhi1))/$range]
       set y0Orig [expr int($objInfo($obj,yn) - $f*$yw)]
      } \
   else \
      {
       set y0Orig [expr int(0.5*($objInfo($obj,y1) + $objInfo($obj,yn)))]
      }

   set dy [expr $y0 - $y0Orig]

   set objInfo($obj,yOff$id) [expr double($dy)/$yw]

   set x1 0
   set xn $objInfo(prog,canvasSelXW)

   set y1 [expr $y0 - 3]
   set yn [expr $y0 + 4]

   $objInfo($obj,canvasSelV) coords $selID $x1 $y1 $xn $yn

   showView $obj adjSelMarkerV
}

proc adjSelMarkerH { obj id selID tag roi x0 y0 } \
{
   global objInfo

   set xw     [expr $objInfo($obj,yn) - $objInfo($obj,y1)]
   set x0Orig [expr int(0.5*($objInfo($obj,x1) + $objInfo($obj,xn)))]
   set dx     [expr $x0 - $x0Orig]

   set y1 0
   set yn $objInfo(prog,canvasSelXW)

   set x1 [expr $x0 - 3]
   set xn [expr $x0 + 4]

   if {$objInfo(data,1,dimCount) == 1} \
      {
       set objInfo($obj,xOff$id) [expr double($dx)/$xw]
      } \
   else \
      {
       set objInfo($obj,xOff$id) [expr double($dx)/$xw]
      }

   $objInfo($obj,canvasSelH) coords $selID $x1 $y1 $xn $yn

   showView $obj adjSelMarkerH
}

proc getDataColor { obj i } \
{
   global objInfo

   set thisColor gray30

   if {$i < 1 || $i > $objInfo(data,specCount)} {return $thisColor}

   if {$objInfo(data,pcaFlag$i)} \
      {
       if {$objInfo($obj,pcaFlag)} \
          {
           set thisColor $objInfo(color,pcaColor$i)
          } \
       else \
          {
           set thisColor $objInfo(color,color$i)
          }
      }

   return $thisColor
}

proc drawThisItem { item } \
{
   global objInfo stdfont xUnits yUnits

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue
     
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID  $objInfo($obj,pixmap)

       set cInfo [getCellInfo -cell $cellID -win $winID]
       set xInfo [lindex $cInfo 1]
       set yInfo [lindex $cInfo 2]

       set kx1   [lindex $xInfo 0]
       set kxn   [lindex $xInfo 1]
       set ky1   [lindex $yInfo 0]
       set kyn   [lindex $yInfo 1]

       showVerb "Label Cell $cellID ROI $roiID: $kx1,$ky1 $kxn,$kyn"
 
       set type [lindex $item 0]
       set id   [lindex $item 1]

       set iID  None

       switch $type \
          {
           box  {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [safeColor [lindex $item 7]]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID -x ${x1}$xUnits]
                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID -x ${xn}$xUnits]
                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID -y ${y1}$yUnits]
                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID -y ${yn}$yUnits]

                 set iID \
                  [$objInfo($obj,canvas) create rectangle \
                    $ix1 $iy1 $ixn $iyn -outline $fg -width $w -tags labelItem]
                }

           line {
                 set x1 [getSpecClipX $obj [lindex $item 2]]
                 set xn [getSpecClipX $obj [lindex $item 3]]
                 set y1 [getSpecClipY $obj [lindex $item 4]]
                 set yn [getSpecClipY $obj [lindex $item 5]]
                 set w  [lindex $item 6]
                 set fg [safeColor [lindex $item 7]]

                 if {$x1 == $xn} continue
                 if {$y1 == $yn} continue

                 set ix1 [spec2pix -roi $roiID -cell $cellID -win $winID -x ${x1}$xUnits]
                 set ixn [spec2pix -roi $roiID -cell $cellID -win $winID -x ${xn}$xUnits]
                 set iy1 [spec2pix -roi $roiID -cell $cellID -win $winID -y ${y1}$yUnits]
                 set iyn [spec2pix -roi $roiID -cell $cellID -win $winID -y ${yn}$yUnits]

                 set iID \
                  [$objInfo($obj,canvas) create line \
                    $ix1 $iy1 $ixn $iyn -fill $fg -width $w -tags labelItem]
                }

           text {
                 set x0 [lindex $item 2]
                 set y0 [lindex $item 3]
                 set w  [lindex $item 4]
                 set a  [lindex $item 5]
                 set fg [safeColor [lindex $item 6]]
                 set s  [lrange $item 7 end]

                 if {$x0 > $objInfo(specX1)} continue
                 if {$x0 < $objInfo(specXN)} continue
                 if {$y0 > $objInfo(specY1)} continue
                 if {$y0 < $objInfo(specYN)} continue

                 set ix0 [spec2pix -roi $roiID -cell $cellID -win $winID -x ${x0}$xUnits]
                 set iy0 [spec2pix -roi $roiID -cell $cellID -win $winID -y ${y0}$yUnits]

                 set iID \
                  [$objInfo($obj,canvas) create text \
                    $ix0 $iy0 -fill $fg -width $w -anchor $a -text $s \
                    -font $stdfont -tags labelItem]
                 }

           label {
                  set x0 [lindex $item 2]
                  set y0 [lindex $item 3]
                  set w  [lindex $item 4]
                  set a  [lindex $item 5]
                  set fg [safeColor [lindex $item 6]]
                  set s  [lrange $item 7 end]

                  set ix0 \
                   [spec2pix -roi $roiID -cell $cellID -win $winID -x $x0]

                  set iy0 \
                   [spec2pix -roi $roiID -cell $cellID -win $winID -y $y0]

                  set iID \
                   [$objInfo($obj,canvas) create text \
                    $ix0 $iy0 -fill $fg -width $w -anchor $a -text $s \
                    -font $stdfont -tags labelItem]
                 }

           title {
                  set x0  [lindex $item 2]
                  set y0  [lindex $item 3]
                  set w   [lindex $item 4]
                  set a   [lindex $item 5]
                  set i   [lindex $item 6]
                  set j   [lindex $item 7]
                  set f   [lindex $item 8]
                  set s   [lrange $item 9 end]

                  set yMin [expr int($objInfo($obj,y1) + 1.5*$objInfo(prog,titleCharY))]
                  set t    "" 

                  while {$y0 > ($objInfo($obj,yn) - 1.6*$objInfo(prog,titleCharY))} \
                     {
                      set titPerCol [expr int(($objInfo($obj,yn) - $objInfo($obj,y1))/(1.5*$objInfo(prog,titleCharY)))]
                      set y0        [expr $yMin + int($y0 - $titPerCol*1.5*$objInfo(prog,titleCharY))]
                      set x0        [expr int($x0 + $objInfo(prog,titleCharX)*$objInfo(prog,maxTitleLength))]
                     }

                  if {$objInfo($obj,useFileLabel)} \
                     {
                      set t $f

                      if {$objInfo($obj,useTitleLabel)} \
                         {
                          if {[strcasecmp $s Difference]} {append t " $s"}
                         } 
                     } \
                  else \
                     {
                      if {$objInfo($obj,useTitleLabel)} {set t $s} 
                     }
                  
                  if {![strcasecmp $id title_stat]} \
                     {
                      if {!$objInfo($obj,difMode)} continue
                      set fg $objInfo(color,stat)
                     } \
                  else \
                     {
                      if {$objInfo($obj,pcaFlag)} \
                         {
                          set fg $objInfo(color,pcaColor$i)
                         } \
                      else \
                         {
                          set fg $objInfo(color,color$i)
                         }
                     }

                  if {[string length $t]} \
                     {
                      set sizeInfo [lindex [getROIInfo -roi $objInfo($obj,roi$i)] 2]

                      if {$objInfo(prog,showSizeFlag)} \
                         {
                          if {$objInfo(data,1,dimCount) == 1} \
                             {
                              append t " [lindex $sizeInfo 0] pts"
                             } \
                          else \
                             {
                              append t " [lindex $sizeInfo 0] x [lindex $sizeInfo 1] pts"
                             }
                         }

                      if {$objInfo(prog,extendedMode)} \
                         {
                          if {[strcasecmp $i stat]} \
                             {
                              if {[isRefSpec $i]} \
                                 {
                                  append t " Reference"
                                 } \
                              elseif {!$objInfo(data,score$i)} \
                                 {
                                  append t " Score: None"
                                 } \
                              else \
                                 {
                                  append t " Score: $objInfo(data,score$i)"
                                 }
                             }
                         }

                      if {$objInfo(data,clustCount) > 1} \
                         {
                          if {[info exists objInfo(data,clusterID$i)]} \
                             {
                              set cID $objInfo(data,clusterID$i)
                             } \
                          else \
                             {
                              set cID 0
                             }

                          set t "$cID=$t"
                         }

                      set ix0 [spec2pix -roi $roiID -cell $cellID -win $winID -x $x0]
                      set iy0 [spec2pix -roi $roiID -cell $cellID -win $winID -y $y0]

                      set iID \
                       [$objInfo($obj,canvas) create text \
                        $ix0 $iy0 -fill [safeColor $fg] -anchor $a -text $t \
                        -font $objInfo(prog,titleFont) -tags labelItem]

                      if {$objInfo(prog,textboxFlag)} {set boxID [boxThisText $objInfo($obj,canvas) $iID dimGray black textbox]}
                     }
                 }
          }

       if {![strcasecmp $iID None]} continue

       $objInfo($obj,canvas) bind $iID <ButtonRelease-3> \
        "+ selLabelItem $id $iID 1"
      }

   xupdate
}

proc insertGraphItem { type obj } \
{
   global objInfo

   set id [getNextID]

   set xf1 0.3
   set yf1 0.3

   if {[info exists objInfo($obj,eventX)]} \
      {
       set xf1 [format %.4f [getGraphItemXF $obj $id $objInfo($obj,eventX)]]
      }

   if {[info exists objInfo($obj,eventY)]} \
      {
       set yf1 [format %.4f [getGraphItemYF $obj $id $objInfo($obj,eventY)]]
      }

   set xf2 [expr $xf1 + 0.3] 
   set xf3 [expr $xf1 + 0.3] 

   set yf2 [expr $yf1 + 0.3] 
   set yf3 [expr $yf1 - 0.3] 

   set xc1 [getGraphItemX $obj $id $xf1]
   set xc2 [getGraphItemX $obj $id $xf2]
   set xc3 [getGraphItemX $obj $id $xf3]

   set yc1 [getGraphItemY $obj $id $yf1]
   set yc2 [getGraphItemY $obj $id $yf2]
   set yc3 [getGraphItemY $obj $id $yf3]

   set objInfo($obj,graphItem$id,type)          $type 
   set objInfo($obj,graphItem$id,text)          text 
   set objInfo($obj,graphItem$id,anchor)        center
   set objInfo($obj,graphItem$id,resizeMode)    1

   set objInfo($obj,graphItem$id,x1)            $xf1
   set objInfo($obj,graphItem$id,x2)            $xf2
   set objInfo($obj,graphItem$id,x3)            $xf3

   set objInfo($obj,graphItem$id,y1)            $yf1
   set objInfo($obj,graphItem$id,y2)            $yf2
   set objInfo($obj,graphItem$id,y3)            $yf3

   set objInfo($obj,graphItem$id,width)         1
   set objInfo($obj,graphItem$id,useOutline)    1
   set objInfo($obj,graphItem$id,nCoords)       1
   set objInfo($obj,graphItem$id,font)          $objInfo(prog,titleFont) 

   set objInfo($obj,graphItem$id,color,fill)    white
   set objInfo($obj,graphItem$id,color,outline) None
   set objInfo($obj,graphItem$id,color,stipple) None
   set objInfo($obj,graphItem$id,colorWindow)   None

   if {![strcasecmp $type text]} \
      {
       set objInfo($obj,graphItem$id,nCoords)     1
       set objInfo($obj,graphItem$id,useOutline)  0

       set gID \
        [$objInfo($obj,canvas) create text $xc1 $yc1    \
          -font $objInfo($obj,graphItem$id,font)        \
          -fill    [parseColor $objInfo($obj,graphItem$id,color,fill)]     \
          -stipple [parseColor $objInfo($obj,graphItem$id,color,stipple)]  \
          -text $objInfo($obj,graphItem$id,text)]
      } \
   elseif {![strcasecmp $type rectangle]} \
      {
       set objInfo($obj,graphItem$id,nCoords)     2
       set objInfo($obj,graphItem$id,useOutline)  1

       set objInfo($obj,graphItem$id,color,fill)    gray 
       set objInfo($obj,graphItem$id,color,outline) yellow 
       set objInfo($obj,graphItem$id,color,stipple) gray12 

       set gID \
        [$objInfo($obj,canvas) create rectangle $xc1 $yc1 $xc2 $yc2 \
          -fill    [parseColor $objInfo($obj,graphItem$id,color,fill)]     \
          -outline [parseColor $objInfo($obj,graphItem$id,color,outline)]  \
          -stipple [parseColor $objInfo($obj,graphItem$id,color,stipple)]  \
          -width   $objInfo($obj,graphItem$id,width)]
      } \
   elseif {![strcasecmp $type triangle]} \
      {
       set objInfo($obj,graphItem$id,nCoords)     3
       set objInfo($obj,graphItem$id,useOutline)  1

       set objInfo($obj,graphItem$id,color,fill)    pink 
       set objInfo($obj,graphItem$id,color,outline) blue 
       set objInfo($obj,graphItem$id,color,stipple) gray12

       set gID \
        [$objInfo($obj,canvas) create polygon $xc1 $yc1 $xc2 $yc2 $xc3 $yc3 \
          -fill    [parseColor $objInfo($obj,graphItem$id,color,fill)]      \
          -outline [parseColor $objInfo($obj,graphItem$id,color,outline)]   \
          -stipple [parseColor $objInfo($obj,graphItem$id,color,stipple)]   \
          -width   $objInfo($obj,graphItem$id,width)]
      } \
   elseif {![strcasecmp $type vector]} \
      {
       set objInfo($obj,graphItem$id,nCoords)     2
       set objInfo($obj,graphItem$id,width)       $objInfo(data,lw1d) 
       set objInfo($obj,graphItem$id,useOutline)  0

       set objInfo($obj,graphItem$id,color,fill)    white 
       set objInfo($obj,graphItem$id,color,stipple) None 

       set gID \
        [$objInfo($obj,canvas) create line $xc1 $yc1 $xc2 $yc2 \
          -fill    [parseColor $objInfo($obj,graphItem$id,color,fill)]     \
          -stipple [parseColor $objInfo($obj,graphItem$id,color,stipple)]  \
          -width   $objInfo($obj,graphItem$id,width)]
      } \
   elseif {![strcasecmp $type spectrum]} \
      {
       set objInfo($obj,graphItem$id,nCoords)     2
       set objInfo($obj,graphItem$id,useOutline)  1

       set objInfo($obj,graphItem$id,color,fill)    blue 
       set objInfo($obj,graphItem$id,color,outline) green 
       set objInfo($obj,graphItem$id,color,stipple) gray12

       set gID \
        [$objInfo($obj,canvas) create rectangle $xc1 $yc1 $xc2 $yc2 \
          -fill    [parseColor $objInfo($obj,graphItem$id,color,fill)]     \
          -outline [parseColor $objInfo($obj,graphItem$id,color,outline)]  \
          -stipple [parseColor $objInfo($obj,graphItem$id,color,stipple)]  \
          -width   $objInfo($obj,graphItem$id,width)]
      }

   set objInfo($obj,graphItem$id,gID) $gID
   set objInfo($obj,gID$gID,id)       $id

   lappend objInfo($obj,graphicList)  $gID

   adjGraphItemProps $type $obj $id
}

proc getGraphItemID { obj gID } \
{
   global objInfo

   if {[lsearch -exact $objInfo($obj,graphicList) $gID] < 0} {return 0} 
   if {![info exists objInfo($obj,gID$gID,id)]} {return 0}

   return $objInfo($obj,gID$gID,id)
}

proc boxThisText { w tID fgColor bgColor tag } \
{
   set oList [$w bbox $tID]

   set ox1   [lindex $oList 0]
   set oy1   [lindex $oList 1]
   set oxn   [lindex $oList 2]
   set oyn   [lindex $oList 3]

   if {$ox1 > $oxn} \
      {
       set itemp $ox1
       set ox1   $oxn
       set oxn   $itemp
      }

   if {$oy1 > $oyn} \
      {
       set itemp $oy1
       set oy1   $oyn
       set oyn   $itemp
      }

   set ox1 [expr $ox1 - 2]
   set oxn [expr $oxn + 2]
   set oy1 [expr $oy1 - 2]
   set oyn [expr $oyn + 2]

   set rID [$w create rectangle $ox1 $oy1 $oxn $oyn -width 1 -outline $fgColor -fill $bgColor -tags $tag]

   $w raise $tID
}

proc safeColor { c } \
{
   if {![strcasecmp $c multi]} {return cyan}
   if {![strcasecmp $c None]}  {return white}

   return $c
}

proc parseColor { c } \
{
   if {![strcasecmp $c None]}  {return ""}
   if {![strcasecmp $c solid]} {return ""}
   if {![strcasecmp $c multi]} {return cyan}

   return $c
}

proc adjGraphItemProps { type obj id } \
{
   global objInfo env

   set w  .insGraphItem$id

   if {![winfo exists $w]} {toplevel $w}

   raise $w

   wm title $w "$type ID $id"

   set nCoords $objInfo($obj,graphItem$id,nCoords)

   if {![strcasecmp $type text]} \
      {
       frame $w.alltext -relief groove -border 2
       pack  $w.alltext -in $w -side top -padx 2 -pady 3

       frame $w.ftext
       label $w.ltext -text "Text: "
       entry $w.text  -width 44 -textvariable objInfo($obj,graphItem$id,text)
       pack  $w.ltext $w.text -in $w.ftext -side left
       pack  $w.text -in $w.ftext -side left -anchor c
       pack  $w.ftext -in $w.alltext -side top -anchor c

       frame $w.ffont
       label $w.lfont -text "      Font: "
       entry $w.font  -width 39 -textvariable objInfo($obj,graphItem$id,font)
   
       button $w.bfont -bitmap @$env(NMRTXT)/caret.map -relief raised  \
              -width $objInfo(prog,charX) -height $objInfo(prog,charY) \
              -fg firebrick -command "getGraphItemFont $obj $id"

       pack  $w.lfont $w.font $w.bfont -in $w.ffont -side left -anchor c
       pack  $w.ffont -in $w.alltext -side top 

       frame $w.fanc

       foreach i {center n s e w ne nw sw se} \
          {
           radiobutton $w.banc$i -text $i -selectcolor firebrick -anchor e \
            -value $i -variable objInfo($obj,graphItem$id,anchor)          \
            -command "configGraphItemAnchor $obj $id $i"

           pack $w.banc$i -in $w.fanc -side left -anchor c
          }

       pack  $w.ffont -in $w.alltext -side top -expand 1 -fill x
       pack  $w.fanc  -in $w.alltext -side top -expand 1 -fill x

       bind $w.font <Return>             "+ configGraphItemFont $obj $id"
       bind $w.text <KeyRelease>         "+ configGraphItemText NoCtrl %K $obj $id" 
       bind $w.text <Control-KeyRelease> "+ configGraphItemText Ctrl   %K $obj $id" 
      }

   if {$nCoords == 1} \
      {
       frame $w.fxy

       set xFrame $w.fxy
       set yFrame $w.fxy
      } \
   else \
      {
       frame $w.fx
       frame $w.fy

       set xFrame $w.fx
       set yFrame $w.fy
      }

   for {set i 1} {$i <= $nCoords} {incr i} \
      {
       set j $i

       if {$i == $nCoords} {set j N}
       if {$nCoords == 1}  {set j ""}

       label $w.lx$i -text " X$j: "
       label $w.ly$i -text " Y$j: "

       entry $w.x$i  -width 8  -textvariable objInfo($obj,graphItem$id,x$i)
       entry $w.y$i  -width 8  -textvariable objInfo($obj,graphItem$id,y$i)

       pack  $w.lx$i $w.x$i -in $xFrame -side left -anchor c
       pack  $w.ly$i $w.y$i -in $yFrame -side left -anchor c

       bind  $w.x$i <Return> "+ configGraphItemCoords $obj $id"
       bind  $w.y$i <Return> "+ configGraphItemCoords $obj $id"
      }

   if {$nCoords == 1} \
      {
       pack $w.fxy -in $w -side top -anchor c
      } \
   else \
      {
       pack $w.fx $w.fy -in $w -side top -anchor c
      }

   frame $w.fcolors
   label $w.lfcolor -text "Fill: "
   entry $w.fcolor  -width 12 -textvariable objInfo($obj,graphItem$id,color,fill)

   button $w.bfcolor -bitmap @$env(NMRTXT)/caret.map -relief raised \
           -width $objInfo(prog,charX) -height $objInfo(prog,charY) \
           -fg blue -command "getGraphItemColor $obj $id fill"

   pack $w.lfcolor $w.fcolor $w.bfcolor -in $w.fcolors -side left 
   bind $w.fcolor <Return> "+ configGraphItemColor $obj $id fill current"

   label $w.lstip -text " Stipple: "
   entry $w.stip  -width 8  -textvariable objInfo($obj,graphItem$id,color,stipple)

   menubutton $w.bstip -bitmap @$env(NMRTXT)/caret.map     \
              -relief raised -menu $w.bstip.menu -fg blue  \
              -width  [expr $objInfo(prog,charX) + 3]      \
              -height [expr $objInfo(prog,charY) + 2]

   menu $w.bstip.menu

   foreach i {None gray12 gray50 gray75} \
      {
       $w.bstip.menu add command -label $i \
        -command "configGraphItemColor $obj $id stipple $i"
      }

   pack $w.lstip $w.stip $w.bstip -in $w.fcolors -side left
   pack $w.fcolors -in $w -side top -anchor c

   bind $w.stip <Return> "+ configGraphItemColor $obj $id stipple current"

   if {$objInfo($obj,graphItem$id,useOutline)} \
      {
       frame $w.fout

       label $w.locolor -text " Outline: "
       entry $w.ocolor -width 12 -textvariable objInfo($obj,graphItem$id,color,outline)

       button $w.bocolor -bitmap @$env(NMRTXT)/caret.map -relief raised \
               -width $objInfo(prog,charX) -height $objInfo(prog,charY) \
               -fg blue -command "getGraphItemColor $obj $id outline"

       spinbox $w.owide -from 0 -to $objInfo(prog,screenWidth) \
               -textvariable objInfo($obj,graphItem$id,width)  \
               -command  "configGraphItemWidth $obj $id"       \
               -increment 1 -format %.0f -width 6 

       pack $w.locolor $w.ocolor $w.bocolor $w.owide -in $w.fout -side left
       pack $w.fout -in $w -side top -anchor c

       bind $w.ocolor <Return> "+ configGraphItemColor $obj $id outline current"
       bind $w.owide  <Return> "+ configGraphItemWidth $obj $id"
      }

   button $w.quit -text Done -command "insertGraphItemDone $type $w $obj $id"
   pack   $w.quit -in $w -side top -expand 1 -fill x
}

proc configGraphItemAnchor { obj id mode } \
{
   global objInfo

   if {![info exists objInfo($obj,graphItem$id,gID)]} return
   if {![winfo exists $objInfo($obj,canvas)]} return

   if {![strcasecmp $mode user]} \
      {
       set objInfo($obj,graphItem$id,anchor) [string tolower $objInfo($obj,graphItem$id,anchor)]

       set aList {center n s e w ne nw sw se}

       if {[lsearch -exact $aList objInfo($obj,graphItem$id,anchor)] < 0} \
          {
           set objInfo($obj,graphItem$id,anchor) center 
          } 
      } \
   else \
      {
       set objInfo($obj,graphItem$id,anchor) $mode
      }

   $objInfo($obj,canvas) itemconfig $objInfo($obj,graphItem$id,gID) -anchor $objInfo($obj,graphItem$id,anchor)

   update 
}

proc configGraphItemCoords { obj id } \
{
   global objInfo

   if {![info exists objInfo($obj,graphItem$id,gID)]} return
   if {![winfo exists $objInfo($obj,canvas)]} return

   if {$objInfo($obj,graphItem$id,nCoords) == 1} \
      {
       $objInfo($obj,canvas) coords $objInfo($obj,graphItem$id,gID) \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x1)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y1)]
      } \
   elseif {$objInfo($obj,graphItem$id,nCoords) == 2} \
      {
       $objInfo($obj,canvas) coords $objInfo($obj,graphItem$id,gID) \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x1)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y1)]   \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x2)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y2)]
      } \
   elseif {$objInfo($obj,graphItem$id,nCoords) == 3} \
      {
       $objInfo($obj,canvas) coords $objInfo($obj,graphItem$id,gID) \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x1)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y1)]   \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x2)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y2)]   \
        [getGraphItemX $obj $id $objInfo($obj,graphItem$id,x3)]   \
        [getGraphItemY $obj $id $objInfo($obj,graphItem$id,y3)]
      }

   update
}

proc configGraphItemXY { obj id cID xPix yPix } \
{
   global objInfo

   if {![winfo exists $objInfo($obj,canvas)]} return

   $objInfo($obj,canvas) itemconfigure -coords $xPix $yPix
}

proc configGraphItemText { ctrlSym ksym obj id } \
{
   global objInfo

   if {![strcasecmp $ctrlSym Ctrl] && ![strcasecmp $ctrlSym u]} \
      {
       set objInfo($obj,graphItem$id,text) ""
      }

   if {![info exists objInfo($obj,graphItem$id,gID)]} return
   if {![winfo exists $objInfo($obj,canvas)]} return

   $objInfo($obj,canvas) itemconfigure \
    $objInfo($obj,graphItem$id,gID) -text $objInfo($obj,graphItem$id,text)

   update
}

proc getGraphItemX { obj id x } \
{
   global objInfo

   set xPix \
    [expr int($objInfo($obj,x1) + $x*($objInfo($obj,xn) - $objInfo($obj,x1)))]

   if {$xPix < $objInfo($obj,x1)} {set xPix $objInfo($obj,x1)}
   if {$xPix > $objInfo($obj,xn)} {set xPix $objInfo($obj,xn)}

   return $xPix
}

proc getGraphItemY { obj id y } \
{
   global objInfo

   set yPix \
    [expr int($objInfo($obj,yn) - $y*($objInfo($obj,yn) - $objInfo($obj,y1)))]

   if {$yPix < $objInfo($obj,y1)} {set yPix $objInfo($obj,y1)}
   if {$yPix > $objInfo($obj,yn)} {set yPix $objInfo($obj,yn)}

   return $yPix
}

proc getGraphItemXF { obj id xPix } \
{
   global objInfo

   set dx [expr $objInfo($obj,xn) - $objInfo($obj,x1)]
   if {$dx == 0} {set dx 1}

   set xf [expr double($xPix - $objInfo($obj,x1))/$dx]

   if {$xf < 0.0} {set xf 0.0}
   if {$xf > 1.0} {set xf 1.0}

   return $xf
}

proc getGraphItemYF { obj id yPix } \
{
   global objInfo

   set dy [expr $objInfo($obj,yn) - $objInfo($obj,y1)]
   if {$dy == 0} {set dy 1}

   set yf [expr double($objInfo($obj,yn) - $yPix)/$dy]

   if {$yf < 0.0} {set yf 0.0}
   if {$yf > 1.0} {set yf 1.0}

   return $yf
}

proc insertGraphItemDone { type w obj id } \
{
   catch "destroy $w"
}

proc isRefSpec { i } \
{
    global objInfo

    if {![strcasecmp $i stat]} {return 0}
    if {[lsearch -exact $objInfo(data,refList) $i] >= 0} {return 1}
    return 0
}

proc selLabelItem { id iID bID } \
{
   global objInfo

   if {$objInfo(prog,Shift) || $objInfo(prog,Ctrl)} return

   showVerb "Selected $id Canvas Item $iID Button $bID"

   set sList [split $id _]
   set n     [llength $sList]
   set s     [lindex $sList 0]

   if {![strcmp $s title] && $n == 2} \
      {
       set i   [lindex $sList 1]
       set loc [lsearch -exact $objInfo(data,drawList) $i]

       if {$loc >= 0} \
          {
           showVerb "Adjusting Draw Order at Location $i"

           set oList [lswap $objInfo(data,drawList) $loc [expr $loc - 1]]
           set objInfo(data,drawList) $oList

           showViewAll selLabel
           drawTitles
          }
      } 
}

proc addLabelItem { item } \
{
   global objInfo

   showVerb "Label item: $item"

   lappend objInfo(prog,labelItemList) $item

   drawThisItem $item
}

proc delLabelItem { idList } \
{
   global objInfo

   clearLabelItems

   set itemList ""

   foreach item $objInfo(prog,labelItemList) \
      {
       set thisID [lindex $item 1]

       if {[lsearch -exact $idList $thisID] < 0} \
          {
           showVerb "Delete Labels; Keeping item:  $item"
           lappend itemList $item
          } \
       else \
          {
           showVerb "Delete Labels; Deleting item: $item"
          } 
      }

   set objInfo(prog,labelItemList) $itemList

   drawLabelItems
}

proc drawLabelItems {} \
{
   global objInfo

   foreach item $objInfo(prog,labelItemList) \
      {
       drawThisItem $item
      }
}

proc clearLabelItems {} \
{
   global objInfo

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,canvas)]} continue
       $objInfo($obj,canvas) delete labelItem textbox
      }

   xupdate
}

proc getSpecClipX { obj x0 } \
{
   global objInfo

   if {$x0 > $objInfo($obj,specX1)} \
      {
       set x0 $objInfo($obj,specX1)
      }

   if {$x0 < $objInfo($obj,specXN)} \
      {
       set x0 $objInfo($obj,specXN)
      }

   return $x0
}

proc getSpecClipY { obj y0 } \
{
   global objInfo

   if {$y0 > $objInfo($obj,specY1)} \
      {
       set y0 $objInfo($obj,specY1)
      }

   if {$y0 < $objInfo($obj,specYN)} \
      {
       set y0 $objInfo($obj,specYN)
      }

   return $y0
}

proc exportView { obj } \
{
   global objInfo

   set sx1 $objInfo($obj,specX1) 
   set sxn $objInfo($obj,specXN)
   set sy1 $objInfo($obj,specY1)
   set syn $objInfo($obj,specYN)

   showVerb "Exporting Current Region to $objInfo(prog,exportProc)"

   $objInfo(prog,exportProc) $obj $sx1 $sxn $sy1 $syn
}

proc showVerb args \
{
   global objInfo

   if {$objInfo(prog,verbose)} {puts "specView. $args"}
}

proc quitThisView { obj who } \
{
   global objInfo

   showVerb "Quiting View for $who $obj, Cell $objInfo($obj,cell)"

   catch "destroy $objInfo($obj,window)"

   set   winList ""

   foreach w $objInfo(prog,winList) \
      {
       if {[strcasecmp $w $obj]} {lappend winList $w}
      } 

   set objInfo(prog,winList) $winList

   freeViewROI $obj
}

proc freeViewROI { obj } \
{
   global objInfo

   if {$objInfo(prog,verbose)} \
      {
       set vArg verb
      } \
   else \
      {
       set vArg noverb
      }

   for {set i 1} {$i <= $objInfo($obj,specCount)} {incr i} \
      {
       if {$objInfo($obj,roi$i)} \
          {
           showVerb "Free ID $objInfo($obj,roi$i)."
     
           freeThisID $objInfo($obj,roi$i)

           if {$objInfo(prog,freeFlag)} {freeROI -roi $objInfo($obj,roi$i) -$vArg}
          }
     }
}

proc startView {} \
{
   global objInfo

   initPeakTable
   initAssTable

   initPeakWindow
   initAssWindow
   
   set status [initSpecData]

   if {$status} \
      {
       puts stderr "SpecView: No First Input Data was Found."
       exit 1
      }

   freeViewROI  data

   initSpecWindow data \
     $objInfo(data,specX1) $objInfo(data,specXN) \
     $objInfo(data,specY1) $objInfo(data,specYN)
}

proc busyView { mode w } \
{
   if {[strcmp $w .]} \
      {
       busy $mode .
       busy $mode $w
      } \
   else \
      {
       busy $mode $w
      }

   xupdate
}

proc quitAllView {} \
{
   global objInfo

   saveNotes
   saveAss 1

   if {[strcasecmp $objInfo(prog,exportProc) None]} \
      {
       $objInfo(prog,exportProc) -- -quit
      }

   okExit exit
}

proc clearThisView { obj who } \
{
   global objInfo
           
   showVerb "Clearing window $obj for $who ..."
    
   $objInfo($obj,canvas) delete peak ass labelItem

   newScreen -win $objInfo($obj,pixmap)

   drawRec -x 0% 100% -y 0% 100% \
    -fg $objInfo(color,winBG) -win $objInfo($obj,pixmap)

   set objInfo(data,drawList) "" 

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set objInfo(data,drawFlag$i) 0
      } 

   .tlb selection clear 0 end 

   showViewAll clearThisView

   drawTitles

   update
}

proc clearMarkers { obj } \
{
   global objInfo

   showVerb "Clearing selection markers for window $obj ..."
   catch "$objInfo($obj,canvasSelH) delete selMarkerH"
   catch "$objInfo($obj,canvasSelV) delete selMarkerV"
}

proc clearView { obj } \
{
   global objInfo

   showVerb "Clearing window $obj ..."

   $objInfo($obj,canvas) delete peak labelItem 

   newScreen -win $objInfo($obj,pixmap)

   drawRec -x 0% 100% -y 0% 100% \
    -fg $objInfo(color,winBG) -win $objInfo($obj,pixmap)
}

proc incrZoom { obj dim jumpSign } \
{
   global objInfo xUnits yUnits

   set i 1

   if {![info exists objInfo($obj,roi$i)]} return
   if {$objInfo($obj,roi1) < 1} return

   set fdata $objInfo(data,$i,fdata)

   if {![strcasecmp $dim X]} \
      {
       set fullSize $objInfo(data,$i,xSize)
       set loc1     [spec2pnt $fdata $objInfo(data,hDim) $objInfo($obj,specX1)$xUnits] 
       set locN     [spec2pnt $fdata $objInfo(data,hDim) $objInfo($obj,specXN)$xUnits] 
      } \
   elseif {![strcasecmp $dim Y]} \
     {
       set fullSize $objInfo(data,$i,ySize)
       set loc1     [spec2pnt $fdata $objInfo(data,vDim) $objInfo($obj,specY1)$yUnits] 
       set locN     [spec2pnt $fdata $objInfo(data,vDim) $objInfo($obj,specYN)$yUnits] 
     }

   set loc1    [expr int($loc1 + 0.5)]
   set locN    [expr int($locN + 0.5)]
   set regSize [expr 1 + $locN - $loc1]

   set jump [expr int(0.25*$regSize)]
   if {$jump < 1} {set jump 1} 

   set jump [expr $jumpSign*$jump]

   set loc1 [expr $loc1 + $jump] 
   if {$loc1 < 1} {set loc1 1}

   set locN [expr $loc1 + $regSize - 1] 

   if {$locN >= $fullSize} \
      {
       set locN $fullSize
       set loc1 [expr 1 + $locN - $regSize]
      }

   if {![strcasecmp $dim X]} \
      {
       set objInfo($obj,specX1) [pnt2spec $fdata $objInfo(data,hDim) $loc1 $xUnits]
       set objInfo($obj,specXN) [pnt2spec $fdata $objInfo(data,hDim) $locN $xUnits]
      } \
   elseif {![strcasecmp $dim Y]} \
      {
       set objInfo($obj,specY1) [pnt2spec $fdata $objInfo(data,vDim) $loc1 $yUnits]
       set objInfo($obj,specYN) [pnt2spec $fdata $objInfo(data,vDim) $locN $yUnits]
      }

   updateSpecZoomVal $obj

   readView $obj
   showView $obj zoom
}

proc setUserSpecZoom { obj mode } \
{
   global objInfo xUnits yUnits

   set i 1

   if {![info exists objInfo($obj,$mode)]} return
   if {![info exists objInfo($obj,roi$i)]} return
   if {$objInfo($obj,roi1) < 1} return

   set fdata   $objInfo(data,$i,fdata)
   set modeVal ${mode}Val

   if {![strcasecmp $mode specX1] || ![strcasecmp $mode specXN]} \
      {
       set loc [spec2pnt $fdata $objInfo(data,hDim) $objInfo($obj,$modeVal)]
       set objInfo($obj,$mode) [pnt2spec $fdata $objInfo(data,hDim) $loc $xUnits]
      } \
   else \
      {
       set loc [spec2pnt $fdata $objInfo(data,vDim) $objInfo($obj,$modeVal)]
       set objInfo($obj,$mode) [pnt2spec $fdata $objInfo(data,vDim) $loc $yUnits]
      }

   updateSpecZoomVal $obj

   if {$objInfo($obj,newWindow)} \
      {
       initSpecWindow $obj \
         $objInfo($obj,specX1) $objInfo($obj,specXN) \
         $objInfo($obj,specY1) $objInfo($obj,specYN)
      } \
   else \
      {
       readView $obj
       showView $obj zoom
      }
}

proc setSpecZoom { obj fdata zx1 zxn zy1 zyn } \
{
   global objInfo xUnits yUnits

   if {[strcasecmp $zx1 Same]} \
      {
       set objInfo($obj,specX1) [pnt2spec $fdata $objInfo(data,hDim) $zx1 $xUnits]
      }

   if {[strcasecmp $zxn Same]} \
      {
       set objInfo($obj,specXN) [pnt2spec $fdata $objInfo(data,hDim) $zxn $xUnits]
      }

   if {$objInfo(data,1,dimCount) == 1} \
      {
       if {[strcasecmp $zy1 Same]} \
          {
           set objInfo($obj,specY1) $zy1
           set objInfo($obj,fhi1)   $zy1
          }

       if {[strcasecmp $zyn Same]} \
          {
           set objInfo($obj,specYN) $zyn
           set objInfo($obj,lhi1)   $zyn
          }

       showVerb "Zoom X: $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits"
       showVerb "Zoom Y: $objInfo($obj,specY1)     $objInfo($obj,specYN)"
      } \
   else \
      {
       if {[strcasecmp $zy1 Same]} \
          {
           set objInfo($obj,specY1) [pnt2spec $fdata $objInfo(data,vDim) $zy1 $yUnits]
          }
    
       if {[strcasecmp $zyn Same]} \
          {
           set objInfo($obj,specYN) [pnt2spec $fdata $objInfo(data,vDim) $zyn $yUnits]
          }

       showVerb "Zoom X: $objInfo($obj,specX1)$xUnits $objInfo($obj,specXN)$xUnits"
       showVerb "Zoom Y: $objInfo($obj,specY1)$yUnits $objInfo($obj,specYN)$yUnits"
      }

   updateSpecZoomVal $obj

}

proc mouseON { obj xPix yPix bName shiftFlag ctrlFlag } \
{
   global objInfo

   set objInfo(prog,mouseON) 0

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

   set wf [focus]

   if {[strcmp .$obj $wf] && [string first winND $wf] < 0} \
      {
       showVerb "Mouse Focus Change in $obj Focus: $wf"
       return
      }

   set objInfo(prog,mouseON) 1

   set objInfo($obj,x1Pix) $xPix
   set objInfo($obj,y1Pix) $yPix
   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix

   if {[llength $objInfo(prog,mouseList)]} \
      {
       showVerb "Reseting Mouse List."
      }

   set objInfo(prog,mouseList) ""

#  showVerb "Mouse ON in $obj Focus: [focus]"

   if {$ctrlFlag}  {set bName C$bName}
   if {$shiftFlag} {set bName S$bName}

   set objInfo(prog,Shift) $shiftFlag
   set objInfo(prog,Ctrl)  $ctrlFlag

   switch $bName \
      {
       CB1 {set objInfo(prog,B1) 1} 
       SB1 {set objInfo(prog,B1) 1} 

       B1  {
            set objInfo(prog,B1) 1

            set nW $objInfo(prog,lwgrid)
            if {$nW < 1} {set nW 1}

            if {!$objInfo(prog,freezeSpectra)} \
               {
                $objInfo($obj,canvas) create rectangle \
                   $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) \
                   $objInfo($obj,xnPix) $objInfo($obj,ynPix) \
                   -outline $objInfo(color,zoom) -width $nW -tags zoom 

                xupdate
               }
           }

       CB2 { 
            if {!$objInfo(prog,freezeGraphic)} \
               {
                if {[llength $objInfo($obj,graphicList)]} \
                   {
                    set objInfo($obj,moveList) \
                     [getClosestGraphicItem $obj $xPix $yPix]
                   }

                if {[llength $objInfo($obj,moveList)]} \
                   {
                    set gID [lindex $objInfo($obj,moveList) 0]
                    set id  [getGraphItemID $obj $gID]

                    if {[info exists $objInfo($obj,graphItem$id,type)]} \
                       {
                        set type $objInfo($obj,graphItem$id,type)
#                       adjGraphItemProps $type $obj $id
                       }
                   }
               }
           }

       SB2 {set objInfo(prog,B2) 1} 
       B2  {set objInfo(prog,B2) 1}

       CB3 {
            set objInfo(prog,B3) 1

            set objInfo($obj,moveList) ""

            if {!$objInfo(prog,freezeGraphic)} \
               {
                if {[llength $objInfo($obj,graphicList)]} \
                   {
                    set objInfo($obj,moveList) \
                     [getClosestGraphicItem $obj $xPix $yPix]
                   }

                if {[llength $objInfo($obj,moveList)]} \
                   {
                    set gID [lindex $objInfo($obj,moveList) 0]
                    set id  [getGraphItemID $obj $gID]

                    resizeGraphItem $obj $id $xPix $yPix 1
                   }
               }
           }
 
       SB3 {
            set objInfo(prog,B3) 1

            set objInfo($obj,moveList) ""
 
            if {!$objInfo(prog,freezeGraphic)} \
               {
                if {[llength $objInfo($obj,graphicList)]} \
                   {
                    set objInfo($obj,moveList) \
                     [getClosestGraphicItem $obj $xPix $yPix]
                   }
               }
           }

       B3  {set objInfo(prog,B3) 1}

       B4  {
            set objInfo(prog,B4) 1

            incrCurrentHi u

            set objInfo(prog,B4)      0
            set objInfo(prog,mouseON) 0
           }

       B5  {
            set objInfo(prog,B5) 1
   
            incrCurrentHi d

            set objInfo(prog,B5)      0
            set objInfo(prog,mouseON) 0
           }
      }
}

proc getClosestGraphicItem { obj xPix yPix } \
{
   global objInfo

# $objInfo($obj,canvas) find closest $xPix $yPix

   set gList [$objInfo($obj,canvas) find overlap  \
               [expr $xPix - 5] [expr $yPix - 5]  \
               [expr $xPix + 5] [expr $yPix + 5]]

   foreach grID $gList \
      {
       if {[lsearch -exact $objInfo($obj,graphicList) $grID] >= 0} \
          {
           return $grID
          }
      }

   return ""
}

proc trackMouse { obj xPix yPix bName shiftFlag ctrlFlag } \
{
   global objInfo

   if {!$objInfo(prog,mouseON)} return

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} return
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} return

#  lappend objInfo(prog,mouseList) "$obj $xPix $yPix $bName"

   if {$ctrlFlag}  {set bName C$bName}
   if {$shiftFlag} {set bName S$bName}

   showVerb "Tracking Mouse Events: $bName $objInfo(prog,mouseList) move $objInfo($obj,moveList)"
 
   switch $bName \
      {
       CB3 {
            if {!$objInfo(prog,freezeGraphic)} \
               {
                if {[llength $objInfo($obj,moveList)]} \
                   {
                    set gID [lindex $objInfo($obj,moveList) 0]
                    set id  [getGraphItemID $obj $gID]
  
                    resizeGraphItem $obj $id $xPix $yPix 0
                   }
               }
           }

       SB3 {
            if {!$objInfo(prog,freezeGraphic)} \
               {
                if {[llength $objInfo($obj,moveList)]} \
                   {
                    set idx [expr $xPix - $objInfo($obj,xnPix)]
                    set idy [expr $yPix - $objInfo($obj,ynPix)]

                    set gID [lindex $objInfo($obj,moveList) 0]
                    set id  [getGraphItemID $obj $gID]
 
                    dragGraphItem $obj $id $idx $idy
                   }
               }
           }

       B3 {
           if {!$objInfo(prog,freezeSpectra)} \
              {
               set r [expr 0.01*($objInfo($obj,lhi1) - $objInfo($obj,fhi1))]

               if {$yPix > $objInfo($obj,ynPix)} \
                  {
                   set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1) + $r]
                   set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1) + $r]
                  } \
               else \
                  {
                   set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1) - $r]
                   set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1) - $r]
                  }
    
               showView $obj B3
              }
          }

       CB2 {}

       SB2 {
            if {!$objInfo(prog,freezeSpectra)} \
               {
                if {$yPix > $objInfo($obj,ynPix)} \
                   {
                    set objInfo(data,currentScale) \
                     [expr $objInfo(data,currentScale)*0.99]
                   } \
                else \
                   {
                    set objInfo(data,currentScale) \
                     [expr $objInfo(data,currentScale)/0.99]
                   }

                if {$objInfo(data,trackFlag)} return

                setCurrentScale
               }
           }  
       B2 {
           if {!$objInfo(prog,freezeSpectra)} \
              {
               if {$yPix > $objInfo($obj,ynPix)} \
                  {
                   set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1)/0.95]
                   set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1)/0.95]
                  } \
               else \
                  {
                   set objInfo($obj,fhi1) [expr $objInfo($obj,fhi1)*0.95]
                   set objInfo($obj,lhi1) [expr $objInfo($obj,lhi1)*0.95]
                  }

               showView $obj B2
              }
          }

       CB1 {}
       SB1 {}

       B1  {
           if {!$objInfo(prog,freezeSpectra)} \
              {
               $objInfo($obj,canvas) coords zoom \
                $objInfo($obj,x1Pix) $objInfo($obj,y1Pix) $xPix $yPix

               xupdate
              }
           }
     }

   set objInfo($obj,xnPix) $xPix
   set objInfo($obj,ynPix) $yPix
}

proc mouseDone { obj xPix yPix bName shiftFlag ctrlFlag } \
{
   global objInfo xUnits yUnits

   set delFlag 0

   if {!$objInfo(prog,mouseON)} \
      {
       catch "$objInfo($obj,canvas) delete zoom"
       return
      }

   if {$ctrlFlag}  {set bName C$bName}
   if {$shiftFlag} {set bName S$bName}

   showVerb "Done with Mouse Events."

   set objInfo($obj,moveList) ""
 
   switch $bName \
      {
       CB3 {set objInfo(prog,B3) 0} 
       SB3 {set objInfo(prog,B3) 0} 
       B3  {set objInfo(prog,B3) 0}

       CB2 {set objInfo(prog,B2) 0}
       B2  {set objInfo(prog,B2) 0}

       SB2 {
            set objInfo(prog,B2) 0

            if {!$objInfo(prog,freezeSpectra)} {setCurrentScale}
           }
       
       CB1 {set objInfo(prog,B1) 0}
       SB1 {set objInfo(prog,B1) 0}

       B1 {
           set objInfo(prog,B1) 0

           if {$objInfo(prog,freezeSpectra)} \
              {
               catch "$objInfo($obj,canvas) delete zoom"
               return
              }

           if {$objInfo($obj,x1Pix) == $objInfo($obj,xnPix) &&
               $objInfo($obj,y1Pix) == $objInfo($obj,ynPix)} \
              {
               catch    "$objInfo($obj,canvas) delete zoom"
               showVerb "Empty 1D Region Rejected in View $obj"
               return
              }

           showVerb "1D Region Selected in View $obj"

           $objInfo($obj,canvas) delete zoom

           set delFlag 1

           if {$objInfo($obj,x1Pix) > $objInfo($obj,xnPix)} \
              {
               set itemp               $objInfo($obj,x1Pix)
               set objInfo($obj,x1Pix) $objInfo($obj,xnPix)
               set objInfo($obj,xnPix) $itemp
              }

           if {$objInfo($obj,y1Pix) > $objInfo($obj,ynPix)} \
              {
               set itemp               $objInfo($obj,y1Pix)
               set objInfo($obj,y1Pix) $objInfo($obj,ynPix)
               set objInfo($obj,ynPix) $itemp
              }

           set roiID  $objInfo($obj,roi1)
           set cellID $objInfo($obj,cell)
           set winID  $objInfo($obj,pixmap)

           set fHi $objInfo($obj,fhi1)
           set lHi $objInfo($obj,lhi1)

           set sx1 [pix2spec -cell $cellID -win $winID -roi $roiID -x $objInfo($obj,x1Pix) -units $xUnits]
           set sxn [pix2spec -cell $cellID -win $winID -roi $roiID -x $objInfo($obj,xnPix) -units $xUnits]

           if {![strcasecmp $xUnits pts]} \
              {
               set sx1 [expr int($sx1 + 0.5)]
               set sxn [expr int($sxn + 0.5)]

               iClip sx1 1 $objInfo(data,1,xSize)
               iClip sxn 1 $objInfo(data,1,xSize)
              }

           if {$objInfo(data,1,dimCount) == 1} \
              {
               set sy1 [pix2hi -cell $cellID -win $winID -y $objInfo($obj,ynPix) -fhi $fHi -lhi $lHi]
               set syn [pix2hi -cell $cellID -win $winID -y $objInfo($obj,y1Pix) -fhi $fHi -lhi $lHi]

               set objInfo($obj,fhi1) $sy1
               set objInfo($obj,lhi1) $syn
              } \
           else \
              {
               set sy1 [pix2spec -cell $cellID -win $winID -roi $roiID -y $objInfo($obj,ynPix) -units $yUnits]
               set syn [pix2spec -cell $cellID -win $winID -roi $roiID -y $objInfo($obj,y1Pix) -units $yUnits]

               if {![strcasecmp $yUnits pts]} \
                   {
                    set sy1 [expr int($sy1 + 0.5)]
                    set syn [expr int($syn + 0.5)]

                    iClip sy1 1 $objInfo(data,1,ySize)
                    iClip syn 1 $objInfo(data,1,ySize)
                   }
              }

           if {[llength $objInfo(data,drawList)]} \
              {
               set thisID    [lindex $objInfo(data,drawList) 0]
               set thisName  $objInfo(data,inName$thisID)
               set thisTitle $objInfo(data,title$thisID)
              } \
           else \
              {
               set thisName  None
               set thisTitle None
              }

           showVerb "Region in View $obj is $sx1 $sxn $sy1 $syn $thisName $thisTitle"

           if {$shiftFlag} \
              {
               setNormRegion $obj $sx1 $sxn $sy1 $syn
              } \
           elseif {$objInfo($obj,newWindow)} \
              {
               initSpecWindow $obj $sx1 $sxn $sy1 $syn
              } \
           else \
              {
               set objInfo($obj,specX1) $sx1
               set objInfo($obj,specXN) $sxn
               set objInfo($obj,specY1) $sy1
               set objInfo($obj,specYN) $syn

               updateSpecZoomVal $obj

               readView $obj
               showView $obj zoom
              }
          }
     }

   if {!$delFlag} {catch "$objInfo($obj,canvas) delete zoom"}

   xupdate
}

proc moveCursor  { thisObj thisXPix thisYPix } \
{
   global objInfo xUnits yUnits

   set obj    $thisObj
   set xPix   $thisXPix
   set yPix   $thisYPix
   set inFlag 1

   if {$xPix < $objInfo($obj,x1) || $xPix > $objInfo($obj,xn)} {set inFlag 0} 
   if {$yPix < $objInfo($obj,y1) || $yPix > $objInfo($obj,yn)} {set inFlag 0} 

   set objInfo($obj,x0Pix) $xPix
   set objInfo($obj,y0Pix) $yPix

   if {$inFlag} \
      {
       set roiID  $objInfo($obj,roi1)
       set cellID $objInfo($obj,cell)
       set winID  $objInfo($obj,pixmap)

       set fhi   $objInfo($obj,fhi1)
       set lhi   $objInfo($obj,lhi1)

       set cursorH [pix2hi -cell $cellID -win $winID -y $yPix -fhi $fhi -lhi $lhi]

       set objInfo(data,cursorV) [pix2spec -cell $cellID -win $winID -roi $roiID -x $xPix -units $xUnits]
      } \
   else \
      {
       set objInfo(data,cursorH) None
       set objInfo(data,cursorV) None
      }

   drawCursors
}

proc drawCursors {} \
{
   global objInfo xUnits yUnits

   set inFlag 1

   if {![strcasecmp $objInfo(data,cursorH) None]} {set inFlag 0}
   if {![strcasecmp $objInfo(data,cursorV) None]} {set inFlag 0}

   set fmt "%s: %.3f$xUnits %.0fpts   Hi: %.4e"

   if {$inFlag} \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue

           set roiID  $objInfo($obj,roi1)
           set cellID $objInfo($obj,cell)
           set winID  $objInfo($obj,pixmap)

           set range [expr $objInfo($obj,lhi1) - $objInfo($obj,fhi1)]
           if {$range == 0.0} {set range 1.0}

           set xv [spec2pnt $objInfo(data,fdata) $objInfo(data,hDim) $objInfo(data,cursorV)$xUnits]

           set yv $objInfo(data,cursorH)
 
           set qx [spec2pix -roi $roiID -cell $cellID -win $winID -x $objInfo(data,cursorV)$xUnits]

           set cursorH [expr 100.0*($objInfo(data,cursorH) - $objInfo($obj,fhi1))/$range]

           if {$cursorH <   0.0} {set cursorH   0.0}
           if {$cursorH > 100.0} {set cursorH 100.0}

           set qy  [spec2pix -roi $roiID -cell $cellID -win $winID -y $cursorH%]
           set str [format $fmt $objInfo(data,xAxis) $objInfo(data,cursorV) $xv $yv]

           wm title $objInfo($obj,window) \
            "$str \[Zoom\] \[Scale\] \[Offset\] Right-Click Title to Adjust Order"

           if {$objInfo(data,cursorV) > $objInfo($obj,specX1) || \
                $objInfo(data,cursorV) < $objInfo($obj,specXN)} \
              {
               set qx [expr $objInfo($obj,x1) - 4]
              }

           if {$objInfo(data,cursorH) > $objInfo($obj,specY1) || \
                $objInfo(data,cursorH) < $objInfo($obj,specYN)} \
              {
               set qy [expr $objInfo($obj,y1) - 4]
              }

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)
   
           $objInfo($obj,canvas) raise cursorH
           $objInfo($obj,canvas) raise cursorV
          }
      } \
   else \
      {
       foreach obj $objInfo(prog,winList) \
          {
           if {![winfo exists $objInfo($obj,window)]} continue

           wm title $objInfo($obj,window) \
            "$objInfo(prog,title) \[Zoom\] \[Scale\] \[Offset\] Right-Click on Title to Adjust Order"

           set qx [expr $objInfo($obj,x1) - 4]
           set qy [expr $objInfo($obj,y1) - 4]

           $objInfo($obj,canvas) coords cursorH \
               $objInfo($obj,x1) $qy $objInfo($obj,xn) $qy

           $objInfo($obj,canvas) coords cursorV \
               $qx $objInfo($obj,y1) $qx $objInfo($obj,yn)

#          $objInfo($obj,canvas) raise cursorH
#          $objInfo($obj,canvas) raise cursorV

           $objInfo($obj,canvas) lower cursorH
           $objInfo($obj,canvas) lower cursorV
          }
      }

   xupdate
}

proc selectPeakAss { obj tID pk } \
{
   global objInfo

   set objInfo(peak,currentPeakEntry) $pk
   set objInfo(peak,currentID)        [gdbGet $pk INDEX]
   set objInfo(peak,currentAss)       $objInfo(peak,selectedAss)

   catch "gdbSet $pk ASS $objInfo(peak,selectedAss)"

   incr objInfo(peak,changeCount)

   drawPeaks
   drawAss
   saveAss 0

   update
}

proc selectThisPeak { obj tID pk } \
{
   global objInfo

   set objInfo(peak,currentPeakEntry) $pk

   set objInfo(peak,currentID)  [gdbGet $pk INDEX]
   set objInfo(peak,currentAss) [gdbGet $pk ASS]

   drawPeaks
   drawAss

   update
}

proc setCurrentAss { key } \
{
   global objInfo

   if {[strcasecmp $objInfo(peak,currentPeakEntry) None]} \
      { 
       catch "gdbSet $objInfo(peak,currentPeakEntry) ASS $objInfo(peak,currentAss)"
      }

   incr objInfo(peak,changeCount)

   if {![strcasecmp $key Return]} \
      {
       saveAss 0
       drawPeaks
       drawAss
      }
}

proc setCurrentPeakID {} \
{
   global objInfo

   if {![strcmp $objInfo(peak,tab1) 0]} return

   set eList [gdbGetInfo $objInfo(peak,tab1) -parms entries]

   foreach e $eList \
      {
       set id [gdbGet $e INDEX]

       if {![strcmp $objInfo(peak,currentID) $id]} \
          {
           set objInfo(peak,currentPeakEntry) $e
           set objInfo(peak,currentPeakID)    $id
           set objInfo(peak,currentAss)       [gdbGet $e ASS]

           drawPeaks
           drawAss
           return
          }
      }

   set objInfo(peak,currentPeakEntry) None
   set objInfo(peak,currentID)        None 

   drawPeaks
   drawAss
}

proc showThisPeak { obj tID pk } \
{
   global objInfo env xUnits yUnits

   set id [gdbGet $pk INDEX]

   set iStr "Peak $id [getEntryValsFmt $pk $objInfo(peak,labVarList)]"
   set xStr "$objInfo(data,xAxis)=[gdbGet $pk $objInfo(peak,xName)]$xUnits"
   set yStr "$objInfo(data,yAxis)=[gdbGet $pk $objInfo(peak,yName)]$yUnits"
   set hStr "Height=[gdbGet $pk HEIGHT]"
   set aStr "Ass=[gdbGet $pk ASS]"

   if {[strcasecmp $objInfo(peak,currentID) None]} \
      {
       if {![strcasecmp $id $objInfo(peak,currentID)]} {set thisColor $objInfo(color,selectedPeak)}
      }

   if {![strcasecmp $aStr Ass=None]} \
      { 
       wm title $objInfo($obj,window) "$iStr $xStr $yStr $hStr  Left-Click to Select Peak, Shift-Left-Click to Set Assignment" 
      } \
   else \
      {
       wm title $objInfo($obj,window) "$iStr $xStr $yStr $hStr $aStr Left-Click to Select Peak, Shift-Left-Click to Set Assignment" 
      }

   if {$objInfo($obj,showPeakLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $objInfo(color,hilite)
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      }

   $objInfo($obj,canvas) raise $tID

   xupdate
}

proc selectThisAss { obj tID eAss } \
{
   global objInfo 

   set objInfo(peak,currentAssEntry) $eAss

   set objInfo(peak,selectedAss) [gdbGet $eAss $objInfo(peak,assVar)]

   drawPeaks
   drawAss

   update
}

proc showThisAss { obj tID ass } \
{
   global objInfo env xUnits yUnits

   set str  [gdbGet $ass $objInfo(peak,assVar)]

   set iStr "Assignment $str"
   set xStr "$objInfo(data,xAxis)=[gdbGet $ass $objInfo(peak,xAssVar)]$xUnits"
   set yStr "$objInfo(data,yAxis)=[gdbGet $ass $objInfo(peak,yAssVar)]$yUnits"

   wm title $objInfo($obj,window) "$iStr $xStr $yStr   Left-Click to Select Assignment"

   set thisColor $objInfo(color,ass)

   if {[strcasecmp $str None] && [strcasecmp $objInfo(peak,selectedAss) None]} \
      {
       if {![strcasecmp $str $objInfo(peak,selectedAss)]} {set thisColor $objInfo(color,selectedAss)}
      }

   if {$objInfo($obj,showAssLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $objInfo(color,hilite)
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      }

   $objInfo($obj,canvas) raise $tID

   xupdate
}

proc unshowThisPeak { obj tID pk } \
{
   global objInfo env

   wm title $objInfo($obj,window) $objInfo(prog,title)

   set thisColor $objInfo(color,peak)

   if {[strcasecmp $objInfo(peak,currentID) None]} \
      {
       if {![strcasecmp [gdbGet $pk INDEX] $objInfo(peak,currentID)]} {set thisColor $objInfo(color,selectedPeak)}
      }

   if {$objInfo($obj,showPeakLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/clear.map -fill $thisColor
      }

   $objInfo($obj,canvas) raise $tID

   xupdate
}

proc unshowThisAss { obj tID ass } \
{
   global objInfo env

   wm title $objInfo($obj,window) $objInfo(prog,title)

   set str [gdbGet $ass $objInfo(peak,assVar)]

   set thisColor $objInfo(color,ass)

   if {[strcasecmp $str None] && [strcasecmp $objInfo(peak,selectedAss) None]} \
      {
       if {![strcasecmp $str $objInfo(peak,selectedAss)]} {set thisColor $objInfo(color,selectedAss)}
      }

   if {$objInfo($obj,showAssLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/solid.map -fill $thisColor
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure $tID -stipple @$env(NMRTXT)/clear.map -fill $thisColor
      }

   $objInfo($obj,canvas) raise $tID

   xupdate
}

proc togglePeakLabels { obj } \
{
   global objInfo env

   if {$objInfo($obj,showPeakLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure peak -stipple @$env(NMRTXT)/solid.map -fill $objInfo(color,peak)
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure peak -stipple @$env(NMRTXT)/clear.map -fill $objInfo(color,peak)
      }
}

proc toggleAssLabels { obj } \
{
   global objInfo env

   if {$objInfo($obj,showAssLabel)} \
      {
       $objInfo($obj,canvas) itemconfigure ass -stipple @$env(NMRTXT)/solid.map -fill $objInfo(color,ass)
      } \
   else \
      {
       $objInfo($obj,canvas) itemconfigure ass -stipple @$env(NMRTXT)/clear.map -fill $objInfo(color,ass)
      }
}

proc saveAss { finalFlag } \
{
   global objInfo

   if {$finalFlag} {set objInfo(peak,changeCount) 1}

   if {![strcmp $objInfo(peak,tab1) 0] || !$objInfo(peak,changeCount)} return

   set w .[lindex $objInfo(prog,winList) 0]

   if {![winfo exists $w]} return

   if {![strcasecmp $objInfo(peak,outName) None]} \
      {
       set t [format %s-%s-%s [sysTime %Y] [sysTime %m] [sysTime %d]]
       set objInfo(peak,outName) peak_${t}.tab
      }

   gdbWrite $objInfo(peak,tab1) $objInfo(peak,outName)

   set str "Saved [gdbGetInfo $objInfo(peak,tab1) -parms entryCount] Peaks in File $objInfo(peak,outName)"

   wm title $w $str

   if {$finalFlag} \
      {
       puts $str
      } \
   else \
      {
       showVerb $str
      }

   set objInfo(peak,changeCount) 0
}

proc getViewPeakList { obj tab } \
{
   global objInfo

   set pkList ""

   if {!$tab} {return $pkList}

   set    cond ""
   append cond "$objInfo(peak,xName) >= $objInfo($obj,specXN)"
   append cond " && $objInfo(peak,xName) <= $objInfo($obj,specX1)"

   if {$objInfo(data,1,dimCount) >= 2} \
      {
       append cond " && $objInfo(peak,yName) >= $objInfo($obj,specYN)"
       append cond " && $objInfo(peak,yName) <= $objInfo($obj,specY1)"
      }

   set pkList [gdbSelect -from $tab -cond $cond]

   return $pkList
}

proc getViewAssList { obj tab } \
{
   global objInfo

   set assList ""

   if {!$tab} {return $assList}

   set    cond ""
   append cond "$objInfo(peak,xAssVar) >= $objInfo($obj,specXN)"
   append cond " && $objInfo(peak,xAssVar) <= $objInfo($obj,specX1)"

   if {$objInfo(data,1,dimCount) >= 2} \
      {
       append cond " && $objInfo(peak,yAssVar) >= $objInfo($obj,specYN)"
       append cond " && $objInfo(peak,yAssVar) <= $objInfo($obj,specY1)"
      }

   set assList [gdbSelect -from $tab -cond $cond]

   return $assList
}

proc clearTitles {} \
{
   global objInfo

   set dList title_stat

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       lappend dList title_$i
      }

   delLabelItem $dList
}

proc drawTitles {} \
{
   global objInfo

   clearTitles

   set j 1

   if {[llength $objInfo(data,drawList)] > 1} \
      {
       set ix [expr 4*$objInfo(prog,titleCharX)]
       set iy [expr int((1.5 + $j)*$objInfo(prog,titleCharY))]

       set t "Difference"

       addLabelItem "title title_stat $ix $iy 300 w stat $t $t"
      }

   incr j

   foreach i $objInfo(data,drawList) \
      {
       set t  $objInfo(data,title$i)

       if {$objInfo(prog,fullNameFlag)} \
          {
           set f [file root [file tail $objInfo(data,inName$i)]]
          } \
       else \
          {
           set s [file root [file tail $objInfo(data,inName$i)]]
           set f [lindex [split $s _] end]
          }

       if {[strcmp $objInfo(data,axisLabel1) $objInfo(data,axisLabel$i)]} \
          {
           append t " $objInfo(data,axisLabel$i)"
          }

       if {$objInfo(data,scale$i) != 1.0} \
          {
           append t [format " x %.3f" $objInfo(data,scale$i)]
          }

       set ix [expr 4*$objInfo(prog,titleCharX)]
       set iy [expr int((1.5 + $j)*$objInfo(prog,titleCharY))]

       set titSize [string length $t]
       if {$titSize > $objInfo(prog,maxTitleLength)} {set objInfo(prog,maxTitleLength) $titSize}

       addLabelItem "title title_$i $ix $iy 300 w $i $j $f $t"

       incr j
      }
}

proc drawPeaks {} \
{
   global objInfo env stdfont xUnits yUnits

   if {!$objInfo(prog,pkFlag)} return

   showVerb "Drawing Peaks ..."

   set i 1

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       $objInfo($obj,canvas) delete peak 

       set pkList [getViewPeakList $obj $objInfo(peak,tab$i)]

       foreach pk $pkList \
          {
           set xPPM [gdbGet $pk $objInfo(peak,xName)]$xUnits

           set tx0 \
           [spec2pix -roi  $objInfo($obj,roi1) \
                     -cell $objInfo($obj,cell) \
                     -win  $objInfo($obj,pixmap) \
                     -x    $xPPM]

           if {$objInfo(data,1,dimCount) == 1} \
              {
               set str "[getEntryValsFmt $pk $objInfo(peak,labVarList)]\n$xPPM"
               set yHi  [gdbGet $pk HEIGHT]

               set ty0 [spec2pix -roi  $objInfo($obj,roi1)   \
                                 -cell $objInfo($obj,cell)   \
                                 -win  $objInfo($obj,pixmap) \
                                 -y $yHi]
# %%%
               set ty0 100
              } \
           else \
              {
               set str  [getEntryValsFmt $pk $objInfo(peak,labVarList)]
               set yPPM [gdbGet $pk $objInfo(peak,yName)]$yUnits

               set ty0 \
               [spec2pix -roi  $objInfo($obj,roi1) \
                         -cell $objInfo($obj,cell) \
                         -win  $objInfo($obj,pixmap) \
                         -y    $yPPM]
              }

           if {$objInfo($obj,showPeakLabel)} \
              {
               set thisMap $env(NMRTXT)/solid.map
              } \
           else \
              {
               set thisMap $env(NMRTXT)/clear.map
              }

           set tID [$objInfo($obj,canvas) create text $tx0 $ty0 -anchor sw   \
                      -fill $objInfo(color,peak) -tags peak -text $str       \
                      -font $stdfont -stipple @$env(NMRTXT)/solid.map]

           $objInfo($obj,canvas) bind $tID <Any-Enter>             "+ showThisPeak   $obj $tID $pk"
           $objInfo($obj,canvas) bind $tID <Any-Leave>             "+ unshowThisPeak $obj $tID $pk"
           $objInfo($obj,canvas) bind $tID <ButtonRelease-1>       "+ selectThisPeak $obj $tID $pk"
           $objInfo($obj,canvas) bind $tID <Shift-ButtonRelease-1> "+ selectPeakAss  $obj $tID $pk"

           if {$objInfo(data,1,dimCount) == 2 && $objInfo(prog,pkBoxFlag)} \
              {
               set ibx1 [expr [gdbGet $pk X1] - 0.5]pts
               set ibxn [expr [gdbGet $pk X3] + 0.5]pts
               set iby1 [expr [gdbGet $pk Y1] - 0.5]pts
               set ibyn [expr [gdbGet $pk Y3] + 0.5]pts

               set bx1 [spec2pix -roi  $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -x $ibx1 -clip]
               set bxn [spec2pix -roi  $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -x $ibxn -clip]
               set by1 [spec2pix -roi  $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -y $iby1 -clip]
               set byn [spec2pix -roi  $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -y $ibyn -clip]

               set tID [$objInfo($obj,canvas) create rect $bx1 $by1 $bxn $byn -outline $objInfo(color,grid) -width 1 -tags peak]
              }
          }
      }
}

proc drawSMILE {} \
{
   global objInfo env stdfont

   if {![strcasecmp $objInfo(peak,smileLogName) None]} return
   if {![file exists $objInfo(peak,smileLogName)]} return

   showVerb "Drawing SMILE Peaks ..."

   set inUnit [open $objInfo(peak,smileLogName) r]

   set maxSpecVal  [getMax "[expr abs($objInfo(data,minVal1))] [expr abs($objInfo(data,maxVal1))]"]
   set maxPeakVal  0.0
   set pkCount     0
   set threshCount 0

   while {[gets $inUnit line] >= 0} \
      {
       if {![lineBegins "SMILE SIGNAL INFO INDEX" $line]} continue

       set thisHi [expr abs([getNamedItem HEIGHT $line])]
       if {$thisHi > $maxPeakVal} {set maxPeakVal $thisHi}
      }

   catch "close $inUnit"

   set i 1

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       $objInfo($obj,canvas) delete smile 

       foreach layer "bottom top" \
          {
           set inUnit [open $objInfo(peak,smileLogName) r]

           while {[gets $inUnit line] >= 0} \
              {
               if {![lineBegins "SMILE SIGNAL INFO INDEX" $line]} continue

               set xPPM   [getNamedItem Y_PPM  $line]
               set yPPM   [getNamedItem X_PPM  $line]
               set thisHi [expr $maxSpecVal*[getNamedItem HEIGHT $line]/$maxPeakVal]

               if {$xPPM < $objInfo($obj,specXN) || $xPPM > $objInfo($obj,specX1)} continue
               if {$yPPM < $objInfo($obj,specYN) || $yPPM > $objInfo($obj,specY1)} continue

               incr pkCount

               set tx0 [expr int(0.5 + [spec2pix -roi $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -x ${xPPM}ppm])]
               set ty0 [expr int(0.5 + [spec2pix -roi $objInfo($obj,roi1) -cell $objInfo($obj,cell) -win $objInfo($obj,pixmap) -y ${yPPM}ppm])]

               if {abs($thisHi) > $objInfo(data,hi1)} \
                  {
                   set dx 8
                   set dy 8

                   if {![strcmp $layer top]} \
                      {
                       drawCircle -cell $objInfo($obj,cell) -win  $objInfo($obj,pixmap) -fg red \
                                  -x [expr $tx0 - $dx] [expr $tx0 + $dx] -y [expr $ty0 + $dy] [expr $ty0 - $dy]
                      }

                   incr threshCount
                  } \
               else \
                  {
                   set dx 2
                   set dy 2

                   if {![strcmp $layer bottom]} \
                      {    
                       drawXMark -cell $objInfo($obj,cell) -win  $objInfo($obj,pixmap) -fg gray30 \
                                 -x [expr $tx0 - $dx] [expr $tx0 + $dx] -y [expr $ty0 + $dy] [expr $ty0 - $dy]
                      }
                  }
              }

           catch "close $inUnit"

           showVerb [format "SMILE Total Signals: %d Above Threshold: %d" $pkCount $threshCount]
          }
      }
}

proc drawAss {} \
{
   global objInfo env stdfont boldfont xUnits yUnits

   if {!$objInfo(prog,assFlag)} return

   showVerb "Drawing Assignments ..."

   set i 1

   foreach obj $objInfo(prog,winList) \
      {
       if {![winfo exists $objInfo($obj,window)]} continue

       $objInfo($obj,canvas) delete ass 

       set assList [getViewAssList $obj $objInfo(peak,assTab)]

       foreach ass $assList \
          {
           set str  [gdbGet $ass $objInfo(peak,assVar)]
           set xPPM [gdbGet $ass $objInfo(peak,xAssVar)]$xUnits

           set tx0 \
           [spec2pix -roi  $objInfo($obj,roi1) \
                     -cell $objInfo($obj,cell) \
                     -win  $objInfo($obj,pixmap) \
                     -x    $xPPM]

           if {$objInfo(data,1,dimCount) == 1} \
              {
               set ty0 [spec2pix -roi  $objInfo($obj,roi1)   \
                                 -cell $objInfo($obj,cell)   \
                                 -win  $objInfo($obj,pixmap) \
                                 -y $yHi]
              } \
           else \
              {
               set yPPM [gdbGet $ass $objInfo(peak,yAssVar)]$yUnits

               set ty0 \
               [spec2pix -roi  $objInfo($obj,roi1) \
                         -cell $objInfo($obj,cell) \
                         -win  $objInfo($obj,pixmap) \
                         -y    $yPPM]
              }

           if {$objInfo($obj,showAssLabel)} \
              {
               set thisMap $env(NMRTXT)/solid.map
              } \
           else \
              {
               set thisMap $env(NMRTXT)/clear.map
              }

          set thisColor $objInfo(color,ass)

          if {[strcasecmp $str None] && [strcasecmp $objInfo(peak,selectedAss) None]} \
             {
              if {![strcasecmp $str $objInfo(peak,selectedAss)]} {set thisColor orchid}
             }

          set tx0 [expr $tx0 + $objInfo(peak,assDX)]
          set ty0 [expr $ty0 + $objInfo(peak,assDY)]
 
          set tID [$objInfo($obj,canvas) create text $tx0 $ty0 -anchor sw \
                      -fill $thisColor -tags ass -text ($str)             \
                      -font $boldfont -stipple @$env(NMRTXT)/solid.map]

          if {$objInfo(prog,assBoxFlag)} {set boxID [boxThisText $objInfo($obj,canvas) $tID dimGray "" ass]}

          $objInfo($obj,canvas) bind $tID <Any-Enter>       "+ showThisAss   $obj $tID $ass"
          $objInfo($obj,canvas) bind $tID <Any-Leave>       "+ unshowThisAss $obj $tID $ass"
          $objInfo($obj,canvas) bind $tID <ButtonRelease-1> "+ selectThisAss $obj $tID $ass"
         }
     }
}

proc pix2hi args \
{
   set ARGV  [concat pix2hi $args]
   set ARGC  [llength $ARGV]

   set hiList "" 
   set cellID 1
   set winID  1

   set xPix   None
   set yPix   None

   set fHi    0.0
   set lHi    0.0

   getArgD $ARGV -cell cellID
   getArgD $ARGV -win  winID
   getArgD $ARGV -x    xPix
   getArgD $ARGV -y    yPix
   getArgD $ARGV -fhi  fHi
   getArgD $ARGV -lhi  lHi

   set cInfo [getCellInfo -cell $cellID -win $winID]

   set xInfo [lindex $cInfo 1]
   set yInfo [lindex $cInfo 2]

   set ix1   [lindex $xInfo 0]
   set ixn   [lindex $xInfo 1]
   set iy1   [lindex $yInfo 0]
   set iyn   [lindex $yInfo 1]

   if {[strcasecmp $xPix None]} \
      {
       set range [expr $lHi - $fHi]       
       set w     [expr $ixn - $ix1]

       if {$w == 0} {set w 1.0}

       set hi [expr $lHi - $range*($xPix - $ix1)/$w]

       lappend hiList $hi
      }

   if {[strcasecmp $yPix None]} \
      {
       set range [expr $lHi - $fHi]
       set w     [expr $iyn - $iy1]

       if {$w == 0} {set w 1.0}

       set hi [expr $fHi + $range*($iyn - $yPix)/$w]

       lappend hiList $hi
      }

   return $hiList
}

proc updateSpecViewSettings args \
{
   global objInfo

   set obj prog

   set w .wPCASettings

   if {![winfo exists $w]} return

   set sStr "Bin Size, Pts:   ("
   set wStr "("

   if {$objInfo(data,1,dimCount) >= 1} \
      {
       set thisPPM [pnt2specW $objInfo(data,1,fdata) CUR_XDIM $objInfo($obj,pcaBinX) ppm]
       set thisHz  [pnt2specW $objInfo(data,1,fdata) CUR_XDIM $objInfo($obj,pcaBinX) Hz]

       set thisLab  [getParmStr $objInfo(data,1,fdata) NDLABEL CUR_XDIM]
       set thisSize [getParmI   $objInfo(data,1,fdata) NDSIZE  CUR_XDIM]

       if {$objInfo($obj,pcaBinX) != 0} \
          {
           set thisSize [expr $thisSize/$objInfo($obj,pcaBinX)]
          }

       append sStr [format "Final Size %s: %d" $thisLab $thisSize]
       append wStr [format "%s: %.3f ppm, %.2f Hz" $thisLab $thisPPM $thisHz]
      }

   if {$objInfo(data,1,dimCount) >= 2} \
      {
       set thisPPM [pnt2specW $objInfo(data,1,fdata) CUR_YDIM $objInfo($obj,pcaBinY) ppm]
       set thisHz  [pnt2specW $objInfo(data,1,fdata) CUR_YDIM $objInfo($obj,pcaBinY) Hz]

       set thisLab  [getParmStr $objInfo(data,1,fdata) NDLABEL CUR_YDIM]
       set thisSize [getParmI   $objInfo(data,1,fdata) NDSIZE  CUR_YDIM]

       if {$objInfo($obj,pcaBinY) != 0} \
          {
           set thisSize [expr $thisSize/$objInfo($obj,pcaBinY)]
          }

       append sStr [format " %s: %d" $thisLab $thisSize]
       append wStr [format " %s: %.3f ppm, %.2f Hz" $thisLab $thisPPM $thisHz]
      }

   append sStr ")"
   append wStr ")"

   $w.pcaBinWidthLabel configure -text $wStr
   $w.pcaBinLabel      configure -text $sStr
}

proc showSpecViewSettings args \
{
   global objInfo

   set obj prog

   set w .wPCASettings

   if {[winfo exists $w]} \
      {
       updateSpecViewSettings $obj $w
       return
      }

   toplevel $w

   raise $w

   wm title $w "PCA Settings"

   frame  $w.topSpacerFrame -height 8
   frame  $w.spacerFrame1   -height 8
   frame  $w.spacerFrame2   -height 8
   frame  $w.spacerFrame3   -height 8
   frame  $w.spacerFrame4   -height 8 
   frame  $w.spacerFrame5   -height 8 
   frame  $w.spacerFrame6   -height 8 
   frame  $w.spacerFrame7   -height 10 
   frame  $w.spacerFrame8   -height 10 
   frame  $w.spacerFrame9   -height 14 
   frame  $w.spacerFrame10  -height 4 
   frame  $w.botSpacerFrame -height 8 

   frame  $w.pcaModeFrame
   frame  $w.pcaScaleFrame
   frame  $w.pcaClipLimitsFrame
   frame  $w.pcaSignModeFrame
   frame  $w.pcaWeightFrame
   frame  $w.pcaCenterFrame
   frame  $w.pcaNormFrame
   frame  $w.pcaBinFrame
   frame  $w.pcaOtherFrame
   frame  $w.pcaNCFrame

   label  $w.pcaModeLabel -text "Component Mode:"
   label  $w.modeSpace1   -text " " -width 2
   label  $w.modeSpace2   -text " " -width 2

   set k 0

   pack $w.modeSpace1 -in $w.pcaModeFrame -side left

   foreach mode $objInfo(prog,pcaComponentModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.pcaMode$v -text "$mode  " -value $k -variable objInfo($obj,pcaComponentMode) -anchor w -command "setComponentParms $obj pcaComponentMode $mode $objInfo(prog,pcaComponentModes)"
       pack $w.pcaMode$v -in $w.pcaModeFrame -side left
       incr k
      }

   pack $w.modeSpace2  -in $w.pcaModeFrame -side left

   label  $w.pcaWeightLabel -text "Component Weighting:"
   label  $w.weightSpace1   -text " " -width 2
   label  $w.weightSpace2   -text " " -width 2
   label  $w.weightSpace3   -text " " -width 2

   set k 0

   pack $w.weightSpace1 -in $w.pcaWeightFrame -side left

   foreach mode $objInfo(prog,pcaWeightModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.pcaWeight$v -text "$mode  " -value $k -variable objInfo($obj,pcaWeightMode) -anchor w -command "setComponentParms $obj pcaWeightMode $mode $objInfo(prog,pcaWeightModes)"
       pack $w.pcaWeight$v -in $w.pcaWeightFrame -side left
       incr k
      }

   label $w.pcaWeightFactorLabel -text "Gauss Factor:"
   entry $w.pcaWeightFactorEntry -textvariable objInfo($obj,pcaWeightFactor) -width 7 

   pack $w.weightSpace2 $w.pcaWeightFactorLabel $w.pcaWeightFactorEntry $w.weightSpace3  -in $w.pcaWeightFrame -side left

   label  $w.pcaScaleLabel -text "Scaling:"
   label  $w.scaleSpace1   -text " " -width 2
   label  $w.scaleSpace2   -text " " -width 2

   set k 0

   pack $w.scaleSpace1 -in $w.pcaScaleFrame -side left

   foreach mode $objInfo(prog,pcaScaleModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.pcaScale$v -text "$mode   " -value $k -variable objInfo($obj,pcaScaleMode) -anchor w -command "setComponentParms $obj pcaScaleMode $mode $objInfo(prog,pcaScaleModes)"
       pack $w.pcaScale$v -in $w.pcaScaleFrame -side left
       incr k
      }

   pack $w.scaleSpace2  -in $w.pcaScaleFrame -side left

   label  $w.pcaCenterLabel -text "Centering:"
   label  $w.centerSpace1   -text " " -width 2
   label  $w.centerSpace2   -text " " -width 2

   set k 0

   pack $w.centerSpace1 -in $w.pcaCenterFrame -side left

   foreach mode $objInfo($obj,pcaCenterModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.pcaCenter$v -text "$mode  " -value $k -variable objInfo($obj,pcaCenterFlag) -anchor w -command "setComponentParms $obj pcaCenterFlag $mode $objInfo($obj,pcaCenterModes)"
       pack $w.pcaCenter$v -in $w.pcaCenterFrame -side left
       incr k
      }

   pack $w.centerSpace2  -in $w.pcaCenterFrame -side left

   label  $w.pcaClipLabel -text "Sign Adjustment:"
   label  $w.clipSpace1   -text " " -width 2
   label  $w.clipSpace2   -text " " -width 2

   set k 0

   pack $w.clipSpace1 -in $w.pcaSignModeFrame -side left

   foreach mode $objInfo($obj,pcaSignModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.clip$v -text "$mode  " -value $k -variable objInfo($obj,pcaSignMode) -anchor w -command "setComponentParms $obj pcaSignMode $mode $objInfo($obj,pcaSignModes)"
       pack $w.clip$v -in $w.pcaSignModeFrame -side left
       incr k
      }

   pack $w.clipSpace2 -in $w.pcaSignModeFrame -side left

   label  $w.pcaLimitsLabel -text "Clipping Limits and Threshold for Metrics:" -pady 2
   label  $w.limitsSpace1   -text " " -width 1
   label  $w.limitsSpace2   -text " " -width 4
   label  $w.limitsSpace3   -text " " -width 2

   label $w.minLab      -text "Clip Min: "   -anchor e
   label $w.maxLab      -text "Clip Max: "   -anchor e
   label $w.threshLab   -text "Threshold: "  -anchor e

   label $w.maxSpaceMid -text " " -width 2

   entry $w.clipMin -width 8 -textvariable objInfo($obj,pcaClipMin)
   entry $w.clipMax -width 8 -textvariable objInfo($obj,pcaClipMax)
   entry $w.thresh  -width 8 -textvariable objInfo($obj,pcaThresh)

   pack $w.limitsSpace1 $w.minLab $w.clipMin $w.maxSpaceMid $w.maxLab $w.clipMax $w.limitsSpace2 $w.threshLab $w.thresh $w.limitsSpace3 -in $w.pcaClipLimitsFrame -side left
  
   label  $w.pcaNormLabel -text "Normalization:"
   label  $w.normSpace1   -text " " -width 2
   label  $w.normSpace2   -text " " -width 2

   set k 0

   pack $w.normSpace1 -in $w.pcaNormFrame -side left

   foreach mode $objInfo($obj,pcaNormModes) \
      {
       set v [stripWhitespace $mode]
       radiobutton $w.norm$v -text "$mode  " -value $k -variable objInfo($obj,pcaNormMode) -anchor w -command "setComponentParms $obj pcaNormMode $mode $objInfo($obj,pcaNormModes)"
       pack $w.norm$v -in $w.pcaNormFrame -side left
       incr k
      }

   pack $w.normSpace2 -in $w.pcaNormFrame -side left

   label  $w.pcaBinLabel -text "Bin Size, Pts:" -pady 2 
   label  $w.binSpace1   -text " " -width 2
   label  $w.binSpace2   -text " " -width 1
   label  $w.binSpace3   -text " " -width 1
   label  $w.binSpaceMid -text " " -width 1

   label   $w.pcaBinXLabel     -text "X:"
   label   $w.pcaBinYLabel     -text "Y:"
   label   $w.pcaBinWidthLabel -text " "

   spinbox $w.pcaBinX -width 3 -textvariable objInfo($obj,pcaBinX) -increment 1 -from 1 -to 999999 -command "setComponentParms $obj pcaBinX pcaBinX None"
   spinbox $w.pcaBinY -width 3 -textvariable objInfo($obj,pcaBinY) -increment 1 -from 1 -to 999999 -command "setComponentParms $obj pcaBinY pcaBinY None"

   if {$objInfo(data,1,dimCount) == 1} \
      {
       pack $w.binSpace1 $w.pcaBinXLabel $w.pcaBinX $w.binSpace2 $w.pcaBinWidthLabel $w.binSpace3 -in $w.pcaBinFrame -side left
      } \
   else \
      {
       pack $w.binSpace1 $w.pcaBinXLabel $w.pcaBinX $w.binSpaceMid $w.pcaBinYLabel $w.pcaBinY $w.binSpace2 $w.pcaBinWidthLabel $w.binSpace3 -in $w.pcaBinFrame -side left
      }

   pack $w.pcaBinLabel $w.pcaBinFrame -in $w -side top -anchor w

   label  $w.pcaNCLabel  -text "Component Count:" -anchor w -width 20
   label  $w.pcaNCSpace1 -text " " -width 2
   label  $w.pcaNCSpace2 -text " " -width 2

   spinbox $w.pcaNC -width 3 -textvariable objInfo(prog,pcaNC) -increment 1 -from 1 -to 999999 -command "setComponentParms $obj pcaNC pcaNC None"

   pack $w.pcaNCSpace1 $w.pcaNC $w.pcaNCSpace2 -in $w.pcaNCFrame -side left

   label $w.pcaOtherSpace1 -text " " -width 2
   label $w.pcaOtherSpace2 -text " " -width 2

   label $w.pcaOtherLabel -text "Other Options:" -anchor w
   entry $w.pcaOtherEntry -width 60 -textvariable objInfo($obj,pcaOtherOptions)

   pack $w.pcaOtherSpace1 $w.pcaOtherEntry $w.pcaOtherSpace2 -in $w.pcaOtherFrame -side left


   pack $w.topSpacerFrame \
        $w.pcaModeLabel   $w.pcaModeFrame       $w.spacerFrame1  \
        $w.pcaWeightLabel $w.pcaWeightFrame     $w.spacerFrame2  \
        $w.pcaClipLabel   $w.pcaSignModeFrame   $w.spacerFrame3  \
        $w.pcaLimitsLabel $w.pcaClipLimitsFrame $w.spacerFrame7  \
        $w.pcaNormLabel   $w.pcaNormFrame       $w.spacerFrame4  \
        $w.pcaScaleLabel  $w.pcaScaleFrame      $w.spacerFrame5  \
        $w.pcaCenterLabel $w.pcaCenterFrame     $w.spacerFrame6  \
        $w.pcaBinLabel    $w.pcaBinFrame        $w.spacerFrame8  \
        $w.pcaOtherLabel  $w.pcaOtherFrame      $w.spacerFrame9  \
        $w.pcaNCLabel     $w.pcaNCFrame         $w.spacerFrame10 \
        $w.botSpacerFrame -in $w -side top -anchor w
  
   button $w.done -text Done -command "Done" -command "setComponentParmsDone $obj $w"

   pack $w.done -in $w -side top -fill x
 
   bind  $w.pcaWeightFactorEntry <Return> "+ setComponentParms $obj pcaWeightFactor pcaWeightFactor"
   bind  $w.clipMin              <Return> "+ setComponentParms $obj pcaClipMin      pcaClipMin"
   bind  $w.clipMax              <Return> "+ setComponentParms $obj pcaClipMax      pcaClipMax"
   bind  $w.pcaBinX              <Return> "+ setComponentParms $obj pcaBinX         pcaBinX"
   bind  $w.pcaBinY              <Return> "+ setComponentParms $obj pcaBinY         pcaBinY"

   set objInfo($obj,pcaBinWidthLabel) $w.pcaBinWidthLabel

   updateSpecViewSettings $obj $w
}

proc getComponentParm { listName parmVal } \
{
   global objInfo

   set obj prog

   if {![info exists objInfo($obj,$listName)] || ![info exists objInfo($obj,$parmVal)]} {return Unknown}
   if {![string is integer $objInfo($obj,$parmVal)]} {return Unknown}

   set v [lindex $objInfo($obj,$listName) $objInfo($obj,$parmVal)]

   if {![llength $v]} {return Unknown}

   return $v
}

proc setComponentParmsDone { obj w } \
{
   global objInfo

   catch "destroy $w"

   set objInfo($obj,pcaBinWidthLabel) None
}

proc setComponentParms args \
{
   global objInfo

   set w .wPCASettings

   set obj   [lindex $args 0]
   set mode  [lindex $args 1]
   set val   [lindex $args 2]
   set vList [lindex $args 3]

   if {[string is integer $val]} \
      {
       set item [lindex $vList $val]
      } \
   else \
      {
       set item Unknown
      }

   if {[info exists objInfo($obj,$mode)]} \
      {
       showVerb "setComponentParms obj $obj mode $mode val $val objInfo $objInfo($obj,$mode) item $item"
      } \
   else \
      {
       showVerb "setComponentParms obj $obj mode $mode val $val objInfo Unknown item $item"
      }

   if {![strcasecmp $val pcaBinX] || ![strcasecmp $val pcaBinY]} {updateSpecViewSettings $obj $w}
}

proc captureAllSpectra {} \
{
   global objInfo ARGV

   set obj [lindex $objInfo(prog,winList) 0]
 
   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       set objInfo(data,drawList) $i
       set objInfo(data,refList)  ""

       clearTitles
       updateTitleSelection

       showView $obj startCapture

       drawTitles
 
       captureScreen $obj
      }
}

proc captureScreen { obj } \
{
   global objInfo ARGV

   update

   if {![file exists snap]} {catch "file mkdir snap"}

   set im [image create photo]
   winop snap $objInfo($obj,canvas) $im

   set n [llength [glob -nocomplain snap/*.gif]]

   for {set i 1} {$i <= $n + 1} {incr i} \
      {
       set outName [format snap/im%03d.gif $i]
       if {[file exists $outName]} continue

       set objInfo(prog,imageID) $i
       break
      }

   set outName [format snap/im%03d.gif $objInfo(prog,imageID)]

   $im write -format gif $outName

   image delete $im

   wm title $objInfo($obj,window) "Saved Snapshot $outName" 
}

proc xupdate args \
{
   global objInfo

   if {$objInfo(prog,updateFlag)} \
      {
       showVerb "Performing screen update."
       update
      } \
   else \
      {
       showVerb "Skipping screen update."
      }
}

proc specViewShift { obj } \
{
   global objInfo

   if {$objInfo(data,1,dimCount) != 1} return

   set n [llength $objInfo(data,drawList)]
   if {$n < 2} return

   set i     [lindex $objInfo(data,drawList) 0]
   set iList [lrange $objInfo(data,drawList) 1 end]

   set refName   shift_ref_tmp.dat
   set regName   shift_reg_tmp.dat
   set shiftName shift_tmp.dat

   writeROI -roi $objInfo($obj,roi$i) -out $refName -ov

   foreach i $iList \
      {
       catch "file delete $shiftName"

       writeROI -roi $objInfo($obj,roi$i) -out $regName -ov
      
       showVerb "autoShift.com -ref $refName -in $regName -out $shiftName -del"
 
       set error [catch {set sInfo [exec autoShift.com -ref $refName -in $regName -out $shiftName -del]}]
       if {$error} continue

       if {![file exists $shiftName]} continue

       readROI -in $shiftName -ndim 1 -x X_AXIS 0% 100% -naux 1 -roi $objInfo($obj,roi$i) 
      }

   catch "file delete $shiftName $regName $refName"

   updateROIStats $obj
   showView $obj difMode
   drawTitles
}

proc initPCADir {} \
{
   global ARGC ARGV objInfo xUnits yUnits xFmt yFmt

   set origDirName None
   set thisFDATA   0

   getArgD $ARGV -pcaInit origDirName

   set dirName $origDirName

   if {![strcasecmp $dirName None]} {return 0}
   if {![file exists $objInfo(data,inName1)]} {return 0}

   set interpName $dirName/ft/test00001.dat

   if {![file exists $interpName]} \
      {
       showError $objInfo(prog,name) "Error Finding Existing PCA Input $interpName"
       exit 1
      }

   if {[file exists $dirName/pca.tab]} \
      {
       set objInfo(prog,pcaTab) [gdbRead table -in $dirName/pca.tab] 

       getKeyData $objInfo(prog,pcaTab) pcaInfo

       if {[info exists pcaInfo(SCALE)]} \
          {
           showVerb "Auto Set PCA Scaling from $objInfo(data,pcaScaleFlag) to [lindex $pcaInfo(SCALE) 0]"
           set objInfo(data,pcaScaleFlag) [lindex $pcaInfo(SCALE) 0]
          }
      }

   if {[file exists $dirName/pcaCoords.tab]} \
      {
       set objInfo(prog,pcaCoordsTab) [gdbRead table -in $dirName/pcaCoords.tab]

       getKeyData $objInfo(prog,pcaCoordsTab) pcaCoordsInfo

       if {[info exists pcaCoordsInfo(PCA_EXTRA)]} \
          {
           set objInfo(data,pcaExtraCount) [lindex $pcaCoordsInfo(PCA_EXTRA) 0]
          }
      }

   set obj [lindex $objInfo(prog,winList) end]

   if {![info  exists objInfo($obj,window)]}  {return 1}
   if {![winfo exists $objInfo($obj,window)]} {return 1}

   rdFDATA $interpName thisFDATA

   set xSize [getParmI $thisFDATA NDSIZE CUR_XDIM]
   set ySize [getParmI $thisFDATA NDSIZE CUR_YDIM]

   sysFree $thisFDATA

   set dimCount 1

   if {$ySize > 1} {incr dimCount}

   if {$dimCount >= 1} \
      {
       set objInfo($obj,specX1) [pnt2spec $thisFDATA X_AXIS 1      $xUnits]
       set objInfo($obj,specXN) [pnt2spec $thisFDATA X_AXIS $xSize $xUnits]
       showVerb [format "Setting PCA X Limits:  % 6.3f$xUnits to % 6.3f$xUnits" $objInfo($obj,specX1) $objInfo($obj,specXN)]
      }

   if {$dimCount >= 2} \
      {
       set objInfo($obj,specY1) [pnt2spec $thisFDATA Y_AXIS 1      $yUnits]
       set objInfo($obj,specYN) [pnt2spec $thisFDATA Y_AXIS $ySize $yUnits]
       showVerb [format "Setting PCA Y Limits:  % 6.3f$yUnits to % 6.3f$yUnits" $objInfo($obj,specY1) $objInfo($obj,specYN)]
      }

   set objInfo(prog,pcaTitle) ""

   if {$dimCount == 1} \
      {
       append objInfo(prog,pcaTitle) [format " $xFmt$xUnits to $xFmt$xUnits" $objInfo($obj,specX1) $objInfo($obj,specXN)]
      } \
   else \
      {
       append objInfo(prog,pcaTitle) [format " X: $xFmt$xUnits to $yFmt$xUnits" $objInfo($obj,specX1) $objInfo($obj,specXN)]
       append objInfo(prog,pcaTitle) [format " Y: $yFmt$yUnits to $yFmt$yUnits" $objInfo($obj,specY1) $objInfo($obj,specYN)]
      }

   set thisMode [getComponentParm pcaComponentModes pcaComponentMode]

   append objInfo(prog,pcaTitle) " Mode: $thisMode"

   updateSpecZoomVal $obj

   readView $obj
   showView $obj setUserZoom

   set objInfo(prog,pcaFlag) 1

   startPCAViewer $origDirName $dirName $obj
}

proc specViewPCA { obj } \
{
   global objInfo

   if {$objInfo(data,1,dimCount) == 1} \
      {
       specViewPCA1D $obj
      } \
   else \
      {
       specViewPCA2D $obj
      }
}

proc specViewPCA1D { obj } \
{
   global objInfo xUnits yUnits xFmt yFmt

   set iList ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {!$objInfo(data,pcaFlag$i)} continue
       lappend iList $i 
      }

   if {[llength $iList] < 2} {return 1}

    wm title . SpecViewPCA 

   if {[winfo exists .pwLogo]} \
      { 
       .pwLogo  configure -image specPCAImage 
#      .pwLabel configure -text PCA 
      }

   if {[winfo exists $objInfo($obj,window).pca]} \
      {
       $objInfo($obj,window).pca config -state disabled
      }

   set objInfo(prog,pcaFlag) 1 
   set objInfo($obj,pcaFlag) 1 
   set objInfo($obj,difMode) 0 

   busyView hold $objInfo($obj,window)

   if {!$objInfo(prog,pcaID)} {set objInfo(prog,pcaID) [getNextID]}

   for {set pcaID 1} {$pcaID <= 1000} {incr pcaID} \
      {
       set dirName [format pca%03d $pcaID]

       if {![file exists $dirName]} break 
      }

   if {$pcaID > 999} \
      {
       showError $objInfo(prog,name) "Error: Too Many PCA Directories. Delete some and try again." 
       return 1
      }

   catch "exec /bin/rm -rf $dirName"

   catch "file mkdir $dirName"
   catch "file mkdir $dirName/ft"
   catch "file mkdir $dirName/pca"

   if {![file exists $dirName/pca]} \
      {
       busyView release $objInfo($obj,window)
       showError $objInfo(prog,name) "Error creating directories in $dirName"
       quitSpecViewPCA
       return 1
      }

   set error [catch {set outUnit [open $dirName/pca.list w]}]

   if {$error} \
      {
       busyView release $objInfo($obj,window)
       showError $objInfo(prog,name) "Error creating PCA list in $dirName"
       quitSpecViewPCA
       return 1
      }

   set j 1

   set refName [format $dirName/ft/test%05d.dat $j]
   set tmpName tmp_interp[pid].dat

   foreach i $iList \
      {
       set outName [format $dirName/ft/test%05d.dat $j]

       set s  [file root [file tail $objInfo(data,inName$i)]]
       set f  [lindex [split $s _] end]

       readROI -ndim 1 -roi $objInfo(prog,pcaID) -inROI $objInfo($obj,roi$i) -x X_AXIS 0% 100% 

       set fdata [lindex [getROIInfo -roi $objInfo(prog,pcaID)] 1]

       setParm    $fdata FDID      $i NULL_DIM
       setParmStr $fdata FDCOMMENT $f NULL_DIM

       catch "file delete $tmpName"

       if {$j == 1} \
          {
           writeROI -roi $objInfo(prog,pcaID) -out $outName -ov
          } \
       else \
          {
           writeROI -roi $objInfo(prog,pcaID) -out $tmpName -ov

           puts $outUnit $objInfo(data,inName$i)

           showVerb "interpNMR -in $tmpName -out $outName -ref $refName"
    
           catch "exec interpNMR -in $tmpName -out $outName -ref $refName"
          }

       if {![file exists $outName]} \
          {
           showError $objInfo(prog,name) "Error Interpolating $objInfo(data,inName$i) to generate $outName"
           busyView release $objInfo($obj,window)
           quitSpecViewPCA

           catch "close $outUnit"
           return 1
          }

       catch "file delete $tmpName"

       incr j
      }

   catch "close $outUnit"

   set objInfo(prog,pcaTitle) ""

   append objInfo(prog,pcaTitle) [format " $xFmt$xUnits to $xFmt$xUnits" $objInfo($obj,specX1) $objInfo($obj,specXN)]

   set thisMode [getComponentParm pcaComponentModes pcaComponentMode]

   append objInfo(prog,pcaTitle) " Mode: $thisMode"
   append objInfo(prog,pcaTitle) " Interpolated"

  if {$objInfo(data,pcaScaleFlag)} \
      {
       set pcaScaleMode scale
       append objInfo(prog,pcaTitle) ", Variance Scaled"
      } \
   else \
      {
       set pcaScaleMode noscale
       append objInfo(prog,pcaTitle) ", Unscaled"
      }

   showVerb "alignPPM.tcl -in $dirName/ft/test%05d.dat -noverb"

   catch "exec alignPPM.tcl -in $dirName/ft/test%05d.dat -noverb"

   if {$objInfo(prog,altPCAFlag)} \
      {
       set thisMetric [lindex $objInfo(prog,pcaComponentModes) $objInfo(prog,pcaComponentMode)]

       showVerb "specComponent.tcl -pcaTab $dirName/pca.tab -coordTab $dirName/pcaCoords.tab -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score001.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -title $objInfo(prog,pcaTitle) -metric $thisMetric"

       catch "exec specComponent.tcl -pcaTab $dirName/pca.tab -coordTab $dirName/pcaCoords.tab -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score001.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -title $objInfo(prog,pcaTitle) metric $thisMetric"
      } \
   else \
      {
       showVerb "pcaNMR.com -pcaTab $dirName/pca.tab -pcaArgs -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score001.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -$pcaScaleMode -$objInfo(prog,pcaFirstMode) -itmin $objInfo(prog,pcaItMin) -conv $objInfo(prog,pcaTol)"

       catch "exec pcaNMR.com -pcaTab $dirName/pca.tab -pcaArgs -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score001.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -$pcaScaleMode -$objInfo(prog,pcaFirstMode) -itmin $objInfo(prog,pcaItMin) -conv $objInfo(prog,pcaTol)"
      }

   if {![file exists $dirName/load.dat]} \
      {
       showError $objInfo(prog,name) "Error Executing pcaNMR Command for $dirName"
       busyView release $objInfo($obj,window)
       quitSpecViewPCA
       return 1
      }

   if {!$objInfo(prog,altPCAFlag)} \
      {
       showVerb "pcaCoords.tcl -in $dirName/ft/test%05d.dat -pcaCoords $dirName/pcaCoords.tab -load $dirName/load.dat -score $dirName/score001.dat $objInfo(prog,pcaArgs) -title $objInfo(prog,pcaTitle)"  

       catch "exec pcaCoords.tcl -in $dirName/ft/test%05d.dat -pcaCoords $dirName/pcaCoords.tab -load $dirName/load.dat -score $dirName/score001.dat $objInfo(prog,pcaArgs) -title $objInfo(prog,pcaTitle)"  
      }

   if {$objInfo(prog,autoPCAFlag)} {exit 0}

   startPCAViewer $dirName $dirName $obj

   busyView release $objInfo($obj,window)
}

proc specViewPCA2D { obj } \
{
   global objInfo xUnits yUnits xFmt yFmt 

   set iList ""

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {!$objInfo(data,pcaFlag$i)} continue
       lappend iList $i 
      }

   if {[llength $iList] < 2} {return 1}

   wm title . SpecViewPCA 

   if {[winfo exists .pwLogo]} \
      { 
       .pwLogo  configure -image specPCAImage 
#      .pwLabel configure -text PCA 
      }

   if {[winfo exists $objInfo($obj,window).pca]} \
      {
       $objInfo($obj,window).pca config -state disabled
      }

   set objInfo(prog,pcaFlag) 1 
   set objInfo($obj,pcaFlag) 1 
   set objInfo($obj,difMode) 0 

   busyView hold $objInfo($obj,window)

   if {!$objInfo(prog,pcaID)} {set objInfo(prog,pcaID) [getNextID]}

   for {set pcaID 1} {$pcaID <= 1000} {incr pcaID} \
      {
       set dirName [format pca%03d $pcaID]

       if {![file exists $dirName]} break 
      }

   if {$pcaID > 999} \
      {
       showError $objInfo(prog,name) "Error: Too Many PCA Directories. Delete some and try again." 
       return 1
      }

   catch "exec /bin/rm -rf $dirName"

   catch "file mkdir $dirName"
   catch "file mkdir $dirName/ft"
   catch "file mkdir $dirName/pca"

   if {![file exists $dirName/pca]} \
      {
       busyView release $objInfo($obj,window)
       showError $objInfo(prog,name) "Error creating directories in $dirName"
       quitSpecViewPCA
       return 1
      }

   showVerb "Using 2D PCA with interpolation."

   set error [catch {set outUnit [open $dirName/pca.list w]}]

   if {$error} \
      {
       busyView release $objInfo($obj,window)
       showError $objInfo(prog,name) "Error creating PCA list in $dirName"
       quitSpecViewPCA
       return 1
      }

   set j 1

   set refName [format $dirName/ft/test%05d.dat $j]
   set tmpName tmp_interp[pid].dat

   foreach i $iList \
      {
       set outName [format $dirName/ft/test%05d.dat $j]

       set s  [file root [file tail $objInfo(data,inName$i)]]
       set f  [lindex [split $s _] end]

       readROI -ndim 2 -roi $objInfo(prog,pcaID) -inROI $objInfo($obj,roi$i) -x X_AXIS 0% 100% -y Y_AXIS 0% 100%

       set fdata [lindex [getROIInfo -roi $objInfo(prog,pcaID)] 1]

       setParm    $fdata FDID      $i NULL_DIM
       setParmStr $fdata FDCOMMENT $f NULL_DIM

       catch "file delete $tmpName"

       if {$j == 1} \
          {
           writeROI -roi $objInfo(prog,pcaID) -out $outName -ov
          } \
       else \
          {
           writeROI -roi $objInfo(prog,pcaID) -out $tmpName -ov

           puts $outUnit $objInfo(data,inName$i)

           showVerb "interpNMR -in $tmpName -out $outName -ref $refName"
    
           catch "exec interpNMR -in $tmpName -out $outName -ref $refName"
          }

       if {![file exists $outName]} \
          {
           showError $objInfo(prog,name) "Error Interpolating $objInfo(data,inName$i) to generate $outName"
           busyView release $objInfo($obj,window)
           quitSpecViewPCA

           catch "close $outUnit"
           return 1
          }

       catch "file delete $tmpName"

       incr j
      }

   catch "close $outUnit"

   set objInfo(prog,pcaTitle) ""
   
   append objInfo(prog,pcaTitle) [format " X: $xFmt$xUnits to $xFmt$xUnits" $objInfo($obj,specX1) $objInfo($obj,specXN)]
   append objInfo(prog,pcaTitle) [format " Y: $yFmt$yUnits to $yFmt$yUnits" $objInfo($obj,specY1) $objInfo($obj,specYN)]

   set thisMode [getComponentParm pcaComponentModes pcaComponentMode]

   append objInfo(prog,pcaTitle) " Mode: $thisMode"

   if {$objInfo(data,pcaScaleFlag)} \
      {
       set pcaScaleMode scale
       append objInfo(prog,pcaTitle) " Variance Scaled" 
      } \
   else \
      {
       set pcaScaleMode noscale
       append objInfo(prog,pcaTitle) " Unscaled" 
      }

   if {$objInfo(prog,altPCAFlag)} \
      {
      set thisMetric [lindex $objInfo(prog,pcaComponentModes) $objInfo(prog,pcaComponentMode)]

       showVerb "specComponent.tcl -pcaTab $dirName/pca.tab -coordTab $dirName/pcaCoords.tab -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score%03d.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -metric $thisMetric"

       catch "exec specComponent.tcl -pcaTab $dirName/pca.tab -coordTab $dirName/pcaCoords.tab -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score%03d.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -metric $thisMetric" 
      } \
   else \
      {
       showVerb "pcaNMR.com -pcaTab $dirName/pca.tab -pcaArgs -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score%03d.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -$pcaScaleMode -$objInfo(prog,pcaFirstMode) -itmin $objInfo(prog,pcaItMin) -conv $objInfo(prog,pcaTol)"

       catch "exec pcaNMR.com -pcaTab $dirName/pca.tab -pcaArgs -in $dirName/ft/test%05d.dat -nc $objInfo(prog,pcaNC) -score $dirName/score%03d.dat -load $dirName/load.dat -pca $dirName/pca/pca%03d.dat -$pcaScaleMode -$objInfo(prog,pcaFirstMode) -itmin $objInfo(prog,pcaItMin) -conv $objInfo(prog,pcaTol)"
      }

   if {![file exists $dirName/load.dat]} \
      {
       showError $objInfo(prog,name) "Error Executing pcaNMR Command for $dirName"
       busyView release $objInfo($obj,window)
       quitSpecViewPCA
       return 1
      }

   if {!$objInfo(prog,altPCAFlag)} \
      {
       showVerb "pcaCoords.tcl -in $dirName/ft/test%05d.dat -pcaCoords $dirName/pcaCoords.tab -load $dirName/load.dat -score $dirName/score%03d.dat $objInfo(prog,pcaArgs) -title $objInfo(prog,pcaTitle)"

       catch "exec pcaCoords.tcl -in $dirName/ft/test%05d.dat -pcaCoords $dirName/pcaCoords.tab -load $dirName/load.dat -score $dirName/score%03d.dat $objInfo(prog,pcaArgs) -title $objInfo(prog,pcaTitle)"
      }

   startPCAViewer $dirName $dirName $obj

   busyView release $objInfo($obj,window)
}

proc startPCAViewer { origDirName dirName obj } \
{
   global objInfo 

#  clearThisView $obj specViewPCA

   if {$objInfo(prog,clustFlag)} \
      {
       set clustArg "-clust"
      } \
   else \
      {
       set clustArg "-noclust"
      }
 
   showVerb "$objInfo(prog,pcaViewer) -in $origDirName/ft/test%05d.dat -load $dirName/load.dat -dir $dirName -obj $obj -stdfont [replaceStr $objInfo(prog,pcaFont) * _star_] -colorMode $objInfo(data,colorMode) -pcaTab $dirName/pca.tab -pcaExtra $objInfo(data,pcaExtraCount) -title $objInfo(prog,pcaTitle) -- $objInfo(prog,pcaArgs) $clustArg -send [winfo name .] --"

   catch {eval exec $objInfo(prog,pcaViewer) -in $origDirName/ft/test%05d.dat -load $dirName/load.dat -dir $dirName -obj $obj -stdfont [replaceStr $objInfo(prog,pcaFont) * _star_] -colorMode $objInfo(data,colorMode) -pcaTab $dirName/pca.tab -pcaExtra $objInfo(data,pcaExtraCount) -title $objInfo(prog,pcaTitle) -- $objInfo(prog,pcaArgs) $clustArg -send [winfo name .] -- &}
}

#
# registerSpecViewPCA w sendProc

proc registerSpecViewPCA args \
{
   global objInfo

   showVerb "Register Message from PCA Viewer: $args"

   if {[llength $args]} \
      {
       set objInfo(prog,pcaProc) [lrange $args 1 end]
       showVerb "Setting PCA Process: $objInfo(prog,pcaProc)"
      }

   return $objInfo(data,drawList)
}

#
# toggleSpecViewPCA obj specViewID specViewPCAID title color sendProc

proc toggleSpecViewPCA args \
{
   global objInfo

   showVerb "Toggle Message from PCA Viewer: $args"

   set obj   [lindex $args 0]
   set i     [lindex $args 1]
   set id    [lindex $args 2]
   set t     [lindex $args 3]
   set color [lindex $args 4]
   set bID   [lindex $args 5]
   set wProc [lindex $args 6]
   
   set objInfo(color,pcaColor$i) $color
   set objInfo(prog,pcaProc)     $wProc

   if {$bID == 1} \
      {
       selectTitle2 spec $i
      } \
   elseif {$bID == 2} \
      {
       selectCurrent2 spec $i
      } \
   else \
      {
       toggleTitle2 spec $i
      }

   return $objInfo(data,drawList)
}

proc readSpecViewClusters args \
{
   global objInfo

   showVerb "Received readSpecViewClusters $args"

   set inName [lindex $args end]

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} {set objInfo(data,clusterID$i) 0}

   set objInfo(data,clustCount) 0
 
   if {![file exists $inName]} return

   showVerb "Reading Cluster Table $inName"

   set t     [gdbRead table -in $inName]
   set eList [gdbGetInfo $t -parms entries]

   set cList ""

   set k 0
   set j 0

   for {set i 1} {$i <= $objInfo(data,specCount)} {incr i} \
      {
       if {$objInfo(data,pcaFlag$i)} \
          {
           set cID [gdbGet [lindex $eList $j] CLUSTID]

           if {[lsearch -exact $cList $cID] < 0} {lappend cList $cID}

           set objInfo(data,clusterID$i) $cID
           incr j
          } \
       else \
          {
           incr k
          }
      }

   set objInfo(data,clustCount) [llength $cList]

   showVerb "Cluster Count: $objInfo(data,clustCount) Spectra Skipped: $k"

   catch "gdbFree $t"
}

proc quitSpecViewPCA args \
{
   global objInfo

   showVerb "Quit Message from PCA Viewer: $args"

   set obj None
   set w   None

   if {[llength $args] >= 1} {set obj [lindex $args 0]}
   if {[llength $args] >= 2} {set w   [lindex $args 1]}

   if {[info exists objInfo($obj,window)]} \
      {
       if {[winfo exists $objInfo($obj,window).pca]} \
          {
           $objInfo($obj,window).pca config -state normal
          }
      }

   wm title . $objInfo(prog,name)

   if {[winfo exists .pwLogo]} \
      { 
       .pwLogo  configure -image specViewImage
#      .pwLabel configure -text $objInfo(prog,name)
      }

   set objInfo(prog,pcaFlag) 0

   if {[info exists objInfo($obj,pcaFlag)]} {set objInfo($obj,pcaFlag) 0}

   showViewAll quitSpecViewPCA
   drawTitles
}

proc updateSpecViewPCADrawList args \
{
   global objInfo

   if {!$objInfo(prog,pcaFlag)} return

   showVerb "Sending Draw List to $objInfo(prog,pcaProc)"
   showVerb "send \{$objInfo(prog,pcaProc)\} updateSpecViewPCADrawList $objInfo(data,drawList)"

   catch "send \{$objInfo(prog,pcaProc)\} updateSpecViewPCADrawList $objInfo(data,drawList)" 
}

proc showSpecViewPCAMsg args \
{
   puts "Message Display from SpecViewPCA: $args"
}

proc showHelp { obj } \
{
 set tInfo \
  {
   "Command Keys:"
   ""
   " Advancing Spectra:        Scoring:"
   "  Down  - Next Spectrum     0 - Set Score to None"
   "  Up    - Prev Spectrum     1 - Set Score to 1"
   "  Right - Next Reference    2 - Set Score to 2"
   "  Left  - Prev Reference    3 - Set Score to 3"
   "  t     - Toggle Spectrum   4 - Set Score to 4"
   "  r     - Toggle Reference  5 - Set Score to 5"
   ""
   " Command Panel:"
   "  c - Clear Display         n   - Next Zoom Region"
   "  v - Full 2D               p   - Prev Zoom Region"
   "  h - Full Horiz    (1D)    F1  - Zoom Region 1"
   "  v - Full Vert     (1D)    F2  - Zoom Region 2"
   "  d - Sort Down     (1D)    F3  - Zoom Region 3"
   "  u - Sort Up       (1D)    F4  - Zoom Region 4"
   "  s - Scale         (1D)    F5  - Snapshot"
   "  a - Scale All     (1D)    F6  - Insert Text"
   "  g - Reset Scaling (1D)    F7  - Insert Rectangle"
   "  k - Stack (1D)            F8  - Insert Triangle"
   "                            F9  - Insert Vector"
   ""
   "Mouse Motion in Spectral Viewing Area:"
   " B1 - Drag and release to select and expand Zoom Region"
   " B2 - Drag to adjust Vertical Display Scale"
   " B3 - Drag to adjust Vertical Display Offset"
   ""
   " Shift-B2 - Adjust Spectrum Scale Multiplier"
   " Shift-B3 - Move Graphic Item"
   " Ctrl-B3  - Resize Graphic Item"
   ""
   "Mouse Scroll Wheel: Adjust Contour Height (2D)"
   ""
   "Mouse Motion in Vertical Placement Band to Left of Display:"
   " B1 - Move Vertical Placement Slider for a Given Spectrum"
   ""
   "Spectrum List Box Window:"
   "  The TOP list box selects the current reference spectrum."
   "  The MIDDLE list box selects a pattern of spectrum names to draw." 
   "  The BOTTOM list box selects one or more spectrum names to draw."
   ""
   "  Click with LEFT   MOUSE to Select and Draw a Single Entry."
   "  Click with RIGHT  MOUSE to Toggle Drawing of an Entry."
   "  Click with MIDDLE MOUSE to Select an Entry for Adjustment."
  }   

   catch "destroy .helpWin"

   set h .helpWin

   toplevel $h

   wm title $h "1D Viewer Help"

   frame    $h.top

   text $h.tHelpNote \
    -yscrollcommand "$h.sHelpNote set" -relief flat -borderwidth 3 \
    -height 28 -width 80 -bg cyan -fg black

   scrollbar $h.sHelpNote -command "$h.tHelpNote yview" -orient vertical

   button $h.sHelpClose -text Close -command "destroy $h"

   pack $h.tHelpNote  -in $h.top -side left -fill both -expand 1 -padx 8 -pady 8
   pack $h.sHelpNote  -in $h.top -side left -fill y    -expand 1

   pack $h.top        -in $h -side top -fill both -expand 1
   pack $h.sHelpClose -in $h -side top -fill x    -expand 1

   foreach t $tInfo {$h.tHelpNote insert end "$t\n"}

   update
}

showArgHelp

setOSType

gdbInit
gdbCreate dBase

initProgVars

initArgArgs
initDataArgs
initPeakVars

initProgColors
initSpecColors
initProgArgs

initMainWindow
initPatterns
drawTitles

if {$objInfo(prog,autoStartFlag)} \
   {
    if {[file exists $objInfo(data,inName1)]} startView

    initPCADir 
    updateTitleSelection
   }

raiseSelMarkers

if {$objInfo(prog,initialZoomID) != 0} {setUserZoom [lindex $objInfo(prog,winList) 0] $objInfo(prog,initialZoomID)}

if {$objInfo(prog,autoPCAFlag)} \
   {
    if {![file exists $objInfo(data,inName1)]} {exit 0}

    startView
    specViewPCA [lindex $objInfo(prog,winList) 0]
   }

if {[flagLoc $ARGV -autoCapture]} \
   {
    captureAllSpectra
    exit 0
   }

#
# Bottom
